/***------------------------------------------------------------------------
* NAME:     bintree_train.c
* PURPOSE:  driver for training bintree classifier
*
* SEE ALSO:     bintree_pack.c
*
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/bintree/bintree_train.c,v 11.4 2001/08/23 20:11:26 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/bintree/bintree_train.c,v $
* CREATED: Jul-93 deniz                             MIT LINCOLN LABORATORY
-------------------------------------------------------------------------**/

/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */


/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include "bintree_pack.h"
#include "bintree.h"

#define MAXSTR (256)

static char prog_name[MAXSTR];  /* static variables */

void bintree_train(int nraw, int ninputs, int *features, int noutputs,
           int linear, int full_tree, int leaf_min_npatterns,
           int normalize, int npatterns, int debg, int verbosity,
           int random_flag, char *data_path, char *data_fname,
           char *norm_fname, char *exp_path, char *param_fname,
           char *comment, int adjust_priors, int priors_npatterns,
           float *priors_list)
{
    int model, data_model;
    int normalization=0;
    int desired_class;
    float *inputs;
    char *tmp_path;
    FILE *param_file;

    strcpy(prog_name, get_program_name());

    /* allocate and initialize model structure */
    model = bintree_build(ninputs, noutputs, verbosity);
    bintree_init(model, linear, full_tree, leaf_min_npatterns, features);
    /* normalization */
    if (normalize)
      normalization = read_norm_file(path_build(data_path, norm_fname));
    else
      norm_setup_none(nraw, noutputs);

    display_model_params(model, ninputs, normalization, features, verbosity);

    /* store data in tree structure */
    if(adjust_priors){
    /* read data and initialize data handling routines */
    data_model =
      class_data_read(data_path, data_fname, npatterns, nraw, ninputs,
              noutputs, features, normalization, verbosity);
    if(random_flag) class_data_random_init(data_model);
    class_data_priors_init(data_model, priors_list, priors_npatterns);
    } else {
    data_model = class_data_single_init(data_path, data_fname, npatterns,
                        nraw, ninputs, noutputs, features,
                        normalization, verbosity);
    }
    while((inputs = class_data_get_next(data_model, &desired_class)) != NULL){
    bintree_store_pattern(model, desired_class, inputs);
    }
    class_data_exit(data_model);

    /* organize data into tree */
    bintree_init_tree(model); 
    if(verbosity > OVERALL_VERBOSITY)
      printf("\n\nGROWING THE TREE: \n");
    bintree_grow_tree(model);
    /* prune tree */
    if(verbosity > OVERALL_VERBOSITY)
      printf("\n\nPRUNING THE TREE: \n");
    bintree_prune(model);

    /* store model in param file */
    tmp_path = path_build(exp_path, param_fname);
    param_file = nn_fopen(tmp_path, "w", "for write of model file");
    fprintf(param_file, "%s\n", comment);
    fprintf(param_file, "normalization data\n");
    norm_write_paramfile(param_file);
    bintree_write_model(model, param_file);
    nn_fclose(param_file, tmp_path, "for write of model file");
    if ( verbosity > 0)
      printf("\nFinished -- model saved in \"%s\"\n", tmp_path);

    /* free space and close files */
    norm_free();
    bintree_free(model);
}
