/* (c) Copyright 1999 M.I.T. Lincoln Laboratory 

   Permission is hereby granted, without written agreement or royalty fee, 
   to use, copy, modify, and distribute this software and its 
   documentation for any purpose, provided that the above copyright notice 
   and the following three paragraphs appear in all copies of this 
   software. 

     IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR 
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF 
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN 
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  M.I.T. LINCOLN 
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT. 

        THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN 
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, 
ENHANCEMENTS, OR MODIFICATIONS. 
*/

/* cknn2c.c */
/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#define GEN2CVARS
#include <gen_2c_args.h>
#include "cknn_pack.h"

/* cknn specific variables */
static int k, accum_trials, *classes;
static float **centers;
static char ncenters_name[64];
static char l2dist_name[64];
static char k_name[64];

/* function declerations */
int read_model_file(char **argv);
void free_model(int model);

void get_params(int model);
void get_general_params(int model);
void get_specific_params(int model);

void print_comments(void);
void print_declarations(void);
void print_macros(void);
void print_classify_code(int model);
void print_l2dist_code(void);
void clear_macros(void);

int
main(int argc, char **argv)
{
    int model;

    argc = nnargs(argc, argv, argtab2c, TABSIZE2C);

    /* check arguments */
    if(strlen(class_suff) > 53){
    fatal_error("suffix (%s) is too long.  use at most 50 characters",
            class_suff);
    }

    /* set up subroutine names and macro names */
    strcpy(classify_name, "classify");
    strcpy(normalize_name, "normalize");
    strcpy(l2dist_name, "l2dist");
    strcpy(nraw_name, "NRAW");
    strcpy(ninputs_name, "NINPUTS");
    strcpy(noutputs_name, "NCLASSES");
    strcpy(ncenters_name, "NCENTERS");
    strcpy(k_name, "K");
    if(class_suff[0] != '\0'){
        strcpy(class_exten, "_");
    strcat(class_exten, class_suff);
    strcat(classify_name, class_exten);
    strcat(normalize_name, class_exten);
    strcat(l2dist_name, class_exten);
    strcat(nraw_name, class_exten);
    strcat(ninputs_name, class_exten);
    strcat(noutputs_name, class_exten);
    strcat(ncenters_name, class_exten);
    strcat(k_name, class_exten);
    } else {
      strcpy(class_exten, "");
    }

    /* read in parameters */
    model = read_model_file(argv);
    get_params(model);

    if(adjust_test)
      warning("%s: This algorithm does not adjust test priors", argv[0]);

    print_comments();
    print_declarations();
    print_macros();
    print_classify_code(model);
    print_normalize_code(normalize_name);
    print_l2dist_code();
    clear_macros();

    free_model(model);
    exit(0);
}


int
read_model_file(char **argv)
{
    int   model, norm_model, normalization, accum_trials;
    FILE *param_file;

    param_file = nn_fopen(param_fname, "r", "for read of model file");
    norm_model = norm_read_paramfile(param_file, &normalization);
    if (norm_model < 0)
      fatal_error("%s: read error %d %s %s", argv[0], norm_model,
          "on normalization parameter file", param_fname);

    model = cknn_read_model(param_file, &accum_trials);
    if (model < 0)
      fatal_error("%s: read error %d on model file %s",
          argv[0], model, param_fname);
    nn_fclose(param_file, param_fname, "for read of model file");

    return model;
}

void
free_model(int model)
{
  norm_free();
  cknn_free(model);
}

void
get_params(int model)
{
    get_specific_params(model);
    get_general_params(model);
    get_norm_params(model, ninputs, features);
}

void
get_specific_params(int model)
{
    accum_trials = *(int *)cknn_param(model, "accum_trials");
    centers = *(float ***)cknn_param(model, "exemplars");
    classes = *(int **)cknn_param(model, "classes");
    k = *(int *)cknn_param(model, "k");
}

void
get_general_params(int model)
{
    ninputs = *(int *)cknn_param(model, "ninputs");
    noutputs = *(int *)cknn_param(model, "nclasses");
    features = *(int **)cknn_param(model, "features");
}

void
print_comments(void)
{
    printf("/* %s */\n", param_fname);
    printf("/* %d Nearest Neighbors */\n", k);
    print_norm_comments();
}

void
print_declarations(void)
{
    printf("\n");
    printf("#include <stdio.h>\n");
    printf("#include <math.h>\n");
    printf("#include <values.h>\n");
    printf("#ifndef SQR(x)\n");
    printf("#define SQR(x) ((x) * (x))\n");
    printf("#endif\n");
    printf("\n");
    printf("\n /* function declarations */\n");
    printf("extern int %s(/* float *,float * */);\n", classify_name);
    printf("extern int %s(/* float * */);\n", normalize_name);
    printf("static float %s(/* int, float *, float * */);\n", l2dist_name);
    printf("\n\n");
}

void
print_macros(void)
{
    print_norm_macros(nraw_name, ninputs_name);
    printf("#define %s %d\n", noutputs_name, noutputs);
    printf("#define %s %d\n", ncenters_name, accum_trials);
    printf("#define %s %d\n", k_name, k);
    printf("\n");
}    

void
clear_macros(void)
{
    clear_norm_macros();
    printf("#undef %s\n", noutputs_name);
    printf("#undef %s\n", ncenters_name);
    printf("#undef %s\n", k_name);
    printf("\n");
}
 
void    
print_classify_code(int model)
{
    printf("int %s (inputs, outputs)\n", classify_name);
    printf("   float *inputs,*outputs;\n");
    printf("{\n");
    printf("  int n, j, best;\n");
    print_matrix_initialization("centers", centers, accum_trials, ninputs,
                ncenters_name, ninputs_name);
    print_ivector_initialization("classes", classes, accum_trials,
                 ncenters_name);
    printf("  float dist, dists[%s];\n", k_name);
    printf("  int class_vote[%s], near_class[%s];\n", noutputs_name, k_name);

    /* copy and normlize inputs */
    print_copy_inputs();
    printf("\n  /* normalize loaded data */\n");
    printf("  %s(x);\n", normalize_name);

    printf("  /* find %d nearest neighbors */\n", k);
    printf("  for(n = 0; n < %s; n++){\n", k_name);
    printf("    dists[n] = MAXFLOAT;\n");
    printf("    near_class[n] = -1;\n");
    printf("  }\n");
    printf("  for(n = 0; n < %s; n++){\n", ncenters_name);
    printf("    dist = %s(%s, x, centers[n]);\n", l2dist_name, ninputs_name);
    printf("    if(dist < dists[%s-1]){\n", k_name);
    printf("      for(j = %s-1; j > 0 && dist < dists[j-1]; j--){\n", k_name);
    printf("        dists[j] = dists[j-1];\n");
    printf("        near_class[j] = near_class[j-1];\n");
    printf("      }\n");
    printf("      dists[j] = dist;\n");
    printf("      near_class[j] = classes[n];\n");
    printf("    }\n");
    printf("  }\n");
    printf("  /* take class vote among %d neighbors*/\n", k);
    printf("  for(n = 0; n < %s; n++) class_vote[n]=0;\n", noutputs_name);
    printf("  for(n = 0; n < %s; n++) class_vote[near_class[n]]++;\n", k_name);
    printf("\n  /* copy outputs */\n");
    printf("  for(n = 0; n < %s; n++)\n", noutputs_name);
    printf("    outputs[n] = class_vote[n]/(float)%s;\n", k_name);
    printf("\n  /* find most common class */\n");
    printf("  for(best = n = 0; n < %s; n++)\n", noutputs_name);
    printf("    if(class_vote[best] < class_vote[n]) best = n;\n");
    printf("  return(best);\n");
    printf("}\n");
}

void
print_l2dist_code(void)
{
    printf("\n\n");
    printf("static float %s(n, x1, x2)\n", l2dist_name);
    printf("   register int n;\n");
    printf("   register float *x1, *x2;\n");
    printf("{\n");
    printf("  float dist = 0.;\n");
    printf("  for(--n; n >= 0; n--)\n");
    printf("    dist += SQR(x1[n]-x2[n]);\n");
    printf("  return(dist);\n");
    printf("}\n");
}
