/***--------------------------------------------------------------------------
* NAME:     committee_plot_bound.c
* PURPOSE:  Quick and dirty plot routine for use with committee C files
* USAGE:
*
* DESCRIPTION:  Reads in a trained model file and plots the decision
*               region and/or scatter plot in 2 selected dimensions
*       This program requires 2 classifier dependent subroutines
*           (read_model and classify) and the software they call 
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/committee/committee_plot_bound.c,v 11.4 2001/08/23 20:11:32 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/committee/committee_plot_bound.c,v $
* CREATED: 5-APR-91 dan                             MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
                /* Include necessary header files */
#include <math.h>
#include <stdio.h>
#include <nnlib.h>
#include <xlippic.h>
#define PLOT_BOUND
#include "plot_bound_args.h"
#include <plot_colors.h>
#define FALSE 0
#define TRUE 1
#define SQUARE 0
#define SOLID_BLOCK 6
#define CIRCLE 7
#define FILLED_CIRCLE 8
#define NSYMBOLS 12

void mac_scatter();

                /*** GLOBALS ***/ 
static float *inputs, *slice_inputs, *outputs, *inputs_copy, *inputs_copy2;
static float **graph_y, *graph_x;
static int ubot, vbot, utop, vtop, uwidth, vwidth;
static int vbot_whole,vtop_whole;
static float xwidth, ywidth;
static int ninputs, noutputs;
static int normalization;
static int *nn_features;
static int norm_temp;
static int *features_temp;
static int ninputs_temp;
static int legend_x, legend_y;
static float xlimits[2], ylimits[2];
static char plot_heading[100];
static char xlabel[50], ylabel[50];

int find_class();
static char **plot_labels = NULL; 
static char **plot_input_labels = NULL;
                /*** MAIN DRIVER ROUTINE ***/
main(argc,argv)
     register int argc;
     register char **argv;
{
    void plot_boundary_in_window(), plot_profile_in_window(), read_model();
    int i;
    int find_class();
    float *get_outputs();
    char *path_build(), print_filename[100];
    int db_type;
    double total;
    DEFAULTS_FILE_STRUCT df_args;

                /* Process the command line arguments */
    init_nnsubs(argc, argv, rcsid);
    argc = nnargs(argc, argv, bound_argtab, BOUND_TABSIZE);

    /* read defaults file to get labels if defaults file is set */
    if(strcmp(scatter_desc_fullname, "") != 0){
    /* it would be better to use read_defaults, then we'd have
       the input labels, too */
    read_defaults_file(scatter_desc_fullname, &df_args);
    plot_labels = df_args.output_labels;
    plot_n_inputs = df_args.ninputs;
    plot_n_outputs = df_args.noutputs;
    plot_input_labels = df_args.input_labels;
    db_type = df_args.db_type;
    }

    /* allocate space for feature list */
    nn_features = allocate_ivector(plot_n_inputs+1);

    /* initialize plot_priors for priors adjustment in testing */
    if(plot_uniform_priors){
    plot_priors_list[0] = (float)plot_n_outputs;
    for(i = 1; i <= plot_n_outputs; i++)
      plot_priors_list[i] = 1./(float)plot_n_outputs;
    }
    if(plot_adjust_priors){
    if(plot_priors_list[0] != (float)plot_n_outputs)
      warning("plot_bound: priors_list length is %.0f, should be %d=\n%s",
          plot_priors_list[0], plot_n_outputs,
          "will not adjust prior probabilities of classes");
    else
      /* normalize priors to sum to 1 */
      if(!plot_uniform_priors){
          total = 0.;
          for(i = 1; i <= plot_n_outputs; i++)
        total += plot_priors_list[i];
          for(i = 1; i <= plot_n_outputs; i++)
        plot_priors_list[i] /= total;
      }
    }

    ninputs = plot_n_inputs;
    noutputs = plot_n_outputs;
    outputs = allocate_vector(plot_n_outputs);

    /* allocate and initialize inputs arrays */
    inputs = allocate_vector(plot_n_inputs);
    slice_inputs = allocate_vector(plot_n_inputs);
    if((int)other_vars[0] == plot_n_inputs){
    for(i = 0; i < plot_n_inputs; i++){
        slice_inputs[i] = other_vars[i+1];
    }
    }
    inputs_copy = allocate_vector(plot_n_inputs);
    inputs_copy2 = allocate_vector(plot_n_inputs);

    /* we never do no norm, that is handled in the C routines */
    normalization = 0;
    nn_features = allocate_ivector(1);
    nn_features[0] = 0;
    ninputs = plot_n_inputs;

    /* insure that plot dimensions are within bounds */
    if (bound_dim_1 >= ninputs) bound_dim_1 = ninputs - 1;
    if (bound_dim_1 < 0) bound_dim_1 = 0;
    if (bound_dim_2 >= ninputs) bound_dim_2 = ninputs - 1;
    if (bound_dim_2 < 0) bound_dim_2 = 0;

    if (region_plot || scatter_plot){
    /* plot two D plots */
    strcpy (print_filename,
        path_build(plot_exp_path, bound_plot_fname));
    color_table[plot_n_outputs % N_PLOT_COLORS] = "gray";
    plot_boundary_in_window(plot_error_fname, TRUE, print_filename,
                bound_plot_label);
    } else if (profile_plot || histogram_plot) {
    strcpy (print_filename, path_build(plot_exp_path, profile_plot_fname));
    color_table[plot_n_outputs % N_PLOT_COLORS] = "black";
    bound_ymin = profile_ymin;
    bound_ymax = profile_ymax;
    bound_ystep = profile_ystep;
    plot_profile_in_window (plot_error_fname, FALSE, print_filename,
                profile_plot_label);
    }

    deallocate_vector(inputs);
    deallocate_vector(slice_inputs);
    deallocate_vector(inputs_copy);
    deallocate_vector(inputs_copy2);
    deallocate_ivector(nn_features);
    if(!(region_plot || profile_plot))
      deallocate_vector(outputs);

    exit_nnsubs();
    exit(0);
}               /* END main() */


void
plot_boundary_in_window(error_filename, two_d, print_filename, plot_label)
    char *error_filename, *print_filename, *plot_label;
    int two_d;
{
    float axis_xmin, axis_xmax, axis_xstep;
    float axis_ymin, axis_ymax, axis_ystep;
    void plot_bound();
    void over_plot(), calc_bound_autoscale();
    float plot_scatter(), percent_error;
    float height_fract = 1., width_fract = 1.;
    float xoffset_fract = 0.1, yoffset_fract = 0.;
    char percent_string[20];
    
    FILE *data_file;
    float stddev;
    
    if(no_graphics){
    ginit("no graphics");
    } else {
    ginit("-fg black -bg white -geometry 510x450");
    }
    ghardcopy(1);
    gwtitle("Decision Region/Scatter Plot");

    if(mac_flag && scatter_plot){
    data_file = nn_fopen(scatter_data_fullname, "r",
                 "for read of scatter data for macintosh files");
    mac_scatter(data_file);
    nn_fclose(data_file, scatter_data_fullname,
          "for read of scatter data for macintosh files");
    }
    if (bound_autoscale) {
    data_file = nn_fopen (scatter_data_fullname, "r",
                  "for read of train or test data for autoscale");
    calc_bound_autoscale(data_file, two_d);
    nn_fclose(data_file,scatter_data_fullname,
          "for read of train or test data for autoscale");
    }
    /* calculate the scale for this graph */
    xygraph_scale (xlimits, ylimits, (bound_autoscale ? 2 : 0),
           0, 0, 0, 10,/*symbols */
           1, 2, "",/* grid and plot label */
           0, 0.0, 0.0,/* autoabscissas*/
           height_fract, width_fract, xoffset_fract, yoffset_fract,
           0, bound_autoscale, bound_xmin, bound_xmax, bound_xstep,
           "xlabel",
           0, bound_autoscale, bound_ymin, bound_ymax, bound_ystep,
           "ylabel", "black",
           &ubot, &vbot, &utop, &vtop,
           &axis_xmin, &axis_xmax, &axis_xstep,
           &axis_ymin, &axis_ymax, &axis_ystep);
                /* set up for x/y-u/v conversion function */
    bound_xmin = axis_xmin;
    bound_xmax = axis_xmax;
    bound_xstep = axis_xstep;
    bound_ymin = axis_ymin;
    bound_ymax = axis_ymax;
    bound_ystep = axis_ystep;

    uwidth = utop - ubot;
    vwidth = vtop - vbot;
    xwidth = bound_xmax - bound_xmin;
    ywidth = bound_ymax - bound_ymin;

    plot_legend(two_d);

    /* setup plot_heading */
    strcpy (plot_heading, "");
    if (region_plot) {
    strcat (plot_heading, error_filename);
    if (scatter_plot) {
        strcat (plot_heading, "  ");
    }
    }
    if (scatter_plot) {
    strcat (plot_heading, scatter_data_fullname);
    }

    /* draw x, y, and plot labels */
    gcolor("black");
    /* set the ylabel */
    sprintf(ylabel, "X%d ", bound_dim_2);
    /* set xlabel */
    if(plot_input_labels == NULL){
    sprintf(xlabel, "X%d ", bound_dim_1);
    } else {
    /* include input labels if available */
    /* get_input_label might return a static, so we must
       do bound_1_label and bound_2_label separately */
    sprintf(xlabel, "X%d (%s ", bound_dim_1,
        get_input_label(bound_dim_1, plot_input_labels,
                nn_features, normalization));
    sprintf(xlabel + strlen(xlabel), "vs %s)",
        get_input_label(bound_dim_2, plot_input_labels,
                nn_features, normalization));
    }
    uvprintf(512,vbot/5,2,plot_label);

    /* do decision region plot */
    if (region_plot) plot_bound ();
    
    /* plot grid */
    xygraph(xlimits, ylimits, 0,
        0, 0, 0, 10,
        1, 1, plot_heading,
        0, 0.0, 0.0,
        height_fract, width_fract, xoffset_fract, yoffset_fract,
        0, 0, bound_xmin, bound_xmax, bound_xstep, xlabel,
        0, 0, bound_ymin, bound_ymax, bound_ystep, ylabel, "black");

    /* do scatter plot */
    if (scatter_plot) {
    data_file = nn_fopen (scatter_data_fullname, "r",
        "for read of train or test data for scatter plot");

    percent_error = plot_scatter (data_file, scatter_n_patterns,
                      &stddev);
    if(region_plot){
        gcolor("black");
        uvprintf (legend_x, vbot + 60, 1, "Percent");
        uvprintf (legend_x, vbot + 30, 1, " Error");
        sprintf(percent_string, "%6.2f", percent_error * 100);
        uvprintf (legend_x, vbot, 1, percent_string);
        sprintf(percent_string, "+/-%4.1f", stddev * 100);
        uvprintf (legend_x, vbot - 30, 1, percent_string);
    }
    nn_fclose(data_file,scatter_data_fullname,
          "for read of train or test data for scatter plot");
    }
    /* finish graphics */
    gsave (print_filename, 0);
    gexit(0);
}

void
plot_bound ()
{
    register int old_color_index, current_color_index;
    register float yval, xval, del_x, del_y;
    float old_x, x, y;
    int ix, iy, left_ix;
    int *class_save;
    FILE *macdots_file;
    FILE *macwireframe_file;

    /* Calculate the increments */
    del_x = xwidth / (float) bound_npoints;
    del_y = ywidth / (float) bound_npoints;

    /* Open file to store macintosh data */
    if(mac_flag)
      macdots_file = nn_fopen(macdots_fname, "w",
                "for writing Macintosh decision region dots");
    if(mac_wireframe_flag)
      macwireframe_file = nn_fopen(macwireframe_fname, "w",
        "for writing Macintosh 3D wireframe discrim function data");

        /* Start generating boundary points - looping thru x and y */
    yval = bound_ymax;

    class_save = allocate_ivector (bound_npoints);
    if (!show_color) {
    xval = bound_xmin;
    for (ix = 0; ix < bound_npoints; ix++) {
        xval += del_x;
        /* y first in case xdim and ydim are equal */
        slice_inputs[bound_dim_2]=yval;
        slice_inputs[bound_dim_1]=xval;
        class_save[ix] = find_class (slice_inputs);
    }
    }
    for (iy = 0; iy < bound_npoints; iy++) {
    yval -= del_y;
    slice_inputs[bound_dim_2]=yval;
    slice_inputs[bound_dim_1]=bound_xmin;
    old_color_index = find_class (slice_inputs);
    if(old_color_index < 0) old_color_index = 0;
    if (show_color)
        old_x = bound_xmin;
    xval = bound_xmin;
    for (left_ix = ix = 0; ix < bound_npoints; ix++) {
        if(ix > 0) left_ix = ix - 1;
        xval += del_x;
        /* find the class of the current point */
        slice_inputs[bound_dim_2]=yval;/* y first in case they are = */
        slice_inputs[bound_dim_1]=xval;
        current_color_index = find_class (slice_inputs);
        if(current_color_index < 0) 
          current_color_index = 0;
        if ( mac_wireframe_flag ) {
        if ( mac_wireframe_output < 0)
          fprintf(macwireframe_file, "%.3f ",
              outputs[current_color_index]);
        else
          fprintf(macwireframe_file, "%.3f ",
              outputs[mac_wireframe_output]);
        }
           
        if (current_color_index != old_color_index ||
        current_color_index != class_save[ix] ||
        current_color_index != class_save[left_ix]) {

        /* output rectangle */
        x = xval - (del_x * 0.5);
        y = yval - (del_y * 0.5);
        if(mac_flag)
          fprintf(macdots_file, "%.3f %.3f\n", x, y);
        if(show_color){
            /* only track horizontal changes for color plot */
            if(current_color_index != old_color_index){
            uvblock( color_table[old_color_index % N_PLOT_COLORS],
                x2u(old_x), y2v(yval+del_y),
                x2u(xval-del_x), y2v(yval));
            old_x = xval-del_x;
            }
        }else{
/*
            uvdot("black", x2u(xval-del_x/2.), y2v(yval+del_y/2.),
              (uwidth/bound_npoints));
*/
            uvblock( "black",
            x2u(xval-del_x), y2v(yval+del_y),
            x2u(xval), y2v(yval));
        }
        old_color_index = current_color_index;
        class_save[ix] = current_color_index;
        }
    }
    if (show_color)
      uvblock( color_table[old_color_index % N_PLOT_COLORS], x2u(old_x),
          y2v(yval+del_y), x2u(xval), y2v(yval));
    if(mac_wireframe_flag)
      fprintf(macwireframe_file, "\n");
    }
    deallocate_ivector (class_save);
    if(mac_flag)
      nn_fclose(macdots_file, macdots_fname,
        "for writing Macintosh decision region dots");
    if(mac_wireframe_flag)
      nn_fclose(macwireframe_file, macwireframe_fname, 
        "for writing Macintosh 3D wireframe discrim function data");
}

float
plot_scatter (data_file, n_patterns, stddev)
    FILE *data_file;
    int n_patterns;
    float *stddev;
{
    int desired_class, class, rv;
    int i, i1, u1, v1;
    int symbol_radius = 6;
    int trial_cnt, error_cnt;
    float percent_error, *desired_outputs;
    int relevant;

    rv = 0;
    if(plot_map) show_color = 0;
    else desired_class = trial_cnt = error_cnt = percent_error = 0;
    for(i = 0; i < n_patterns; i++){
    if(plot_map){
        desired_outputs = allocate_vector(plot_n_outputs);
        rv = get_map_line(data_file, scatter_data_fullname, inputs,
                  desired_outputs, ninputs, ninputs,
                  plot_n_outputs, normalization, nn_features);
        if(rv == EOF) break;
    } else {
        gcolor ("black");
        desired_class =
          get_class_line (data_file, scatter_data_fullname, inputs,
                  plot_n_inputs, normalization, nn_features,
                  ninputs);
        if (desired_class == EOF) break;
        if(region_plot){
        trial_cnt++;
        class = find_class(inputs);
        if (class != desired_class)error_cnt++;
        }
    }
    /* check to see if we should show this pattern */
    relevant = 1;
    if (!show_all) {
        for (i1 = 0; i1 < ninputs; i1++) {
        /* check that the pattern is withing bounds */
        if ((i1 != bound_dim_1) && (i1 != bound_dim_2)) {
            if (inputs[i1] < other_vars[i1+1] - distance_limit || 
            inputs[i1] > other_vars[i1+1] + distance_limit)
              relevant = 0;
        }
        }
    }
    u1 = x2u (inputs[bound_dim_1]);
    v1 = y2v (inputs[bound_dim_2]);
    if (relevant && 
        u1 >= ubot && u1 <= utop && v1 >= vbot && v1 <= vtop) {
        uvmove(u1, v1);
        if (show_misses) {
        if (show_color) {
            if (class == desired_class)
              gcolor(color_table[desired_class % N_PLOT_COLORS]);
            else
              gcolor("gray");
            gsymbol (SOLID_BLOCK, symbol_radius);
            gcolor("background");
            gsymbol (SQUARE, symbol_radius);
        } else {
            if(plot_map) uvpoint(u1, v1);
            else {
            if (class == desired_class) uvpoint(u1, v1);
            else if(internals_level > 1)
              gputc (1, 'A' + desired_class);
            else gsymbol(desired_class%NSYMBOLS, symbol_radius);
            }
        }
        } else {
        if (show_color) {
            gcolor (color_table[desired_class % N_PLOT_COLORS]);
            gsymbol (SOLID_BLOCK, symbol_radius);
            gcolor("background");
            gsymbol (SQUARE, symbol_radius);
        } else {
            if(plot_map)
              gsymbol(SQUARE, symbol_radius);
            else {
            if(internals_level > 1)
              gputc (1, 'A' + desired_class);
            else gsymbol(desired_class%NSYMBOLS, symbol_radius);
            }
        }
        }
    }
    }
    if(!plot_map){
    if (trial_cnt > 0) {
        percent_error = (float) error_cnt / (float) trial_cnt;
        *stddev = sqrt ((percent_error * (1 - percent_error)) 
            / (float) trial_cnt);
    }
    return (percent_error);
    } else return 0;
}


void
plot_profile_in_window(error_filename, two_d, print_filename, plot_label)
    char *error_filename, *print_filename, *plot_label;
    int two_d;
{
    float axis_xmin, axis_xmax, axis_xstep;
    float axis_ymin, axis_ymax, axis_ystep;
    void plot_profile();
    void calc_bound_autoscale();
    float plot_histogram(), percent_error;
    float height_fract = 1., width_fract = 1.;
    float xoffset_fract = 0.1, yoffset_fract = 0.;
    char percent_string[20];
    
    FILE *data_file;
    float stddev;
    
    if(no_graphics){
    ginit("no graphics");
    } else {
    ginit("-fg black -bg white -geometry 510x450");
    }
    ghardcopy(1);
    if(profile_plot && histogram_plot)
      gwtitle("Profile/Histogram Plot");
    else if(profile_plot)
      gwtitle("Profile Plot");
    else if (histogram_plot)
      gwtitle("Histogram Plot");

    /* get autoscale limits for x axis */
    if (bound_autoscale) {
    data_file = nn_fopen (scatter_data_fullname, "r",
                  "for read of train or test data for autoscale");
    calc_bound_autoscale(data_file, two_d);
    nn_fclose(data_file,scatter_data_fullname,
          "for read of train or test data for autoscale");
    }
    xygraph_scale (xlimits, ylimits, (bound_autoscale ? 2: 0),
    0, 0, 0, 10,
    1, 2, "",
    0, 0.0, 0.0,
    height_fract, width_fract, xoffset_fract, yoffset_fract,
    0, bound_autoscale, bound_xmin, bound_xmax, bound_xstep, "xlabel",
    0, 0, bound_ymin, bound_ymax, bound_ystep, "ylabel", "black",
    &ubot, &vbot, &utop, &vtop,
    &axis_xmin, &axis_xmax, &axis_xstep,
    &axis_ymin, &axis_ymax, &axis_ystep);

    bound_xmin = axis_xmin;
    bound_xmax = axis_xmax;
    bound_xstep = axis_xstep;
                /* set up for x/y-u/v conversion function */
    vbot_whole = vbot;
    vtop_whole = vtop;
    uwidth = utop - ubot;
    vwidth = vtop - vbot;
    xwidth = bound_xmax - bound_xmin;

    plot_legend(two_d);

    /* set up plot_heading */
    strcpy (plot_heading, "");
    if (profile_plot){
    strcat (plot_heading, error_filename);
    if (histogram_plot) {
        strcat (plot_heading, "  ");
    }
    }
    if (histogram_plot) {
    strcat (plot_heading, scatter_data_fullname);
    }

    /* display x, y, and plot labels */
    gcolor("black");
    /* profile and histogram plots */
    sprintf(ylabel,"Y"); /* for histogram, we want "N"? */
    if(plot_input_labels == NULL){
    sprintf(xlabel, "X%d ", bound_dim_1);
    } else {
    sprintf(xlabel, "X%d (%s)", bound_dim_1,
        get_input_label(bound_dim_1, plot_input_labels, 
                nn_features, normalization));
    }
    uvprintf(512,vbot/5,2,plot_label);

    if (profile_plot) {
    plot_profile ();
    }

    if (histogram_plot) {
    data_file =
      nn_fopen (scatter_data_fullname, "r", "for histogram data");
    percent_error =
      plot_histogram (data_file, scatter_n_patterns, &stddev);

    if(profile_plot){
        gcolor("black");
        uvprintf (legend_x, vbot + 60, 1, "Percent");
        uvprintf (legend_x, vbot + 30, 1, " Error");
        sprintf(percent_string, "%6.2f", percent_error * 100);
        uvprintf (legend_x, vbot, 1, percent_string);
        sprintf(percent_string, "+/-%4.1f", stddev * 100);
        uvprintf (legend_x, vbot - 30, 1, percent_string);
    }
    nn_fclose(data_file,scatter_data_fullname,
          "for read of train or test data for scatter plot");
    }
    gsave (print_filename, 0);
    gexit(0);
}

/* y1 and y2 are the y coordinates for the class bar */
void
plot_profile ()
{
    register int i;
    register float xval,del_x;
    int u2, v1, v2;
    float old_x, x;
    int indx, x2u(),y2v();
    int *graph_class;
    FILE *maclines_file;
    float height_fract = 1., width_fract = 1.;
    float xoffset_fract = 0.1, yoffset_fract = 0.;
    float axis_xmin, axis_xmax, axis_xstep;
    float axis_ymin, axis_ymax, axis_ystep;

    graph_x = allocate_vector (bound_npoints+1);
    graph_y = allocate_matrix (plot_n_outputs+1, bound_npoints+1);
    graph_class = allocate_ivector(bound_npoints+1);

    /* Open file to store delta graph data */
    if(mac_flag)
      maclines_file = nn_fopen(maclines_fname, "w",
                   "for writing Macintosh profile plots");

                /* Calculate the increments */
    del_x = xwidth / (float) bound_npoints;
    slice_inputs[bound_dim_1]=bound_xmin;
    graph_class[0] = find_class(slice_inputs);
    xval = bound_xmin;

    /* Start generating boundary points - looping thru x values */
    ylimits[0] = ylimits[1] = 0.;
    for (indx = 0; indx <= bound_npoints; indx++) {
    graph_x [indx] = xval;
    if (indx > 0) {     /* after the first point */
        /* get the class for the point */
        slice_inputs[bound_dim_1]=xval;
        graph_class[indx] = find_class(slice_inputs);
    }
    if(mac_flag) fprintf(maclines_file, "%.3f", xval);
    /* copy the outputs for the point */
    for (i=0; i < plot_n_outputs; i++) {
        graph_y[i][indx] = outputs[i];
        /* check lower and upper limits */
        if(ylimits[0] > outputs[i]) ylimits[0] = outputs[i];
        if(ylimits[1] < outputs[i]) ylimits[1] = outputs[i];
        graph_y[plot_n_outputs][indx] += outputs[i];
        if(mac_flag) fprintf(maclines_file, " %.3f", outputs[i]);
    }
    /* check limits for sum, as well */
    if(ylimits[0] > graph_y[plot_n_outputs][indx])
      ylimits[0] = graph_y[plot_n_outputs][indx];
    if(ylimits[1] < graph_y[plot_n_outputs][indx])
      ylimits[1] = graph_y[plot_n_outputs][indx];

    if(mac_flag) fprintf(maclines_file, "\n");
    xval += del_x;
    }
    if(mac_flag)
      nn_fclose(maclines_file, maclines_fname,
        "for writing Macintosh profile lines");
    
    /* get the scale for the y axis */
    if(histogram_plot){
    height_fract = .5;
    yoffset_fract = .5;
    } else {
    height_fract = .95;
    yoffset_fract = .05;
    }
    xygraph_scale (xlimits, ylimits, (bound_autoscale ? 2: 0),
           0, 0, 0, 10,
           1, 2, "",
           0, 0.0, 0.0,
           height_fract, width_fract, xoffset_fract, yoffset_fract,
           0, 0, bound_xmin, bound_xmax, bound_xstep, "xlabel",
           0, bound_autoscale, bound_ymin, bound_ymax, bound_ystep,
           "ylabel", "black",
           &ubot, &vbot, &utop, &vtop,
           &axis_xmin, &axis_xmax, &axis_xstep,
           &axis_ymin, &axis_ymax, &axis_ystep);
    bound_ymin = axis_ymin;
    bound_ymax = axis_ymax;
    bound_ystep = axis_ystep;
    ywidth = bound_ymax - bound_ymin;
    vwidth = vtop - vbot;

    /* plot grid */
    xygraph_advanced(xlimits, ylimits, 0,
             0, 0, 0, 10,/* symbol args*/
             1, 1, plot_heading,/* line_type, grid_type, label*/
             0, 0.0, 0.0,/* auto_abscissas */
             height_fract, width_fract, xoffset_fract, yoffset_fract,
             0, 0, bound_xmin, bound_xmax, bound_xstep, "",
             0, 0, bound_ymin, bound_ymax, bound_ystep, ylabel,
             "black",
             1,0,1, /*NoXBotTic,NoXTopTic,NoXLabel,*/
             0,0,0, /* NoYBotTic,NoYTopTic,NoYLabel,*/
             0,0,0,0/* NoXBotNum,NoXTopNum,NoYBotNum,NoYTopNum*/);

    /* plot profile plot lines, including total line */
    for (i=0; i<plot_n_outputs+1; i++) {
    xygraph(graph_x, graph_y[i], bound_npoints+1,
        0, 0, 0, 10,
        1, 0, "",
        0, 0.0, 0.0,
        height_fract, width_fract, xoffset_fract, yoffset_fract,
        0, 0, bound_xmin, bound_xmax, bound_xstep, "",
        0, 0, bound_ymin, bound_ymax, bound_ystep, "",
        color_table[i % N_PLOT_COLORS]);
    }

    if(!histogram_plot){
    /* we want to draw a small plot with just the class bars in it */
    height_fract = .05;
    yoffset_fract = 0.;
    xygraph_scale(xlimits,ylimits,0,
              0, 0, 0, 10,
              1, 0, "",
              0, 0.0, 0.0,
              height_fract, width_fract, xoffset_fract, yoffset_fract,
              0, 0, bound_xmin, bound_xmax, bound_xstep, "",
              0, 0, bound_ymin, bound_ymax, bound_ystep, "",
              "black",
              &ubot, &vbot, &utop, &vtop,
              &axis_xmin, &axis_xmax, &axis_xstep,
              &axis_ymin, &axis_ymax, &axis_ystep);
    xygraph_advanced(xlimits, ylimits, 0,
             0, 0, 0, 10,/* symbol args*/
             1, 1, "",/* line_type, grid_type, label*/
             0, 0.0, 0.0,/* auto_abscissas */
             height_fract, width_fract,
             xoffset_fract, yoffset_fract,
             0, 0, bound_xmin, bound_xmax, bound_xstep,
             (histogram_plot ? "":xlabel),
             0, 0, bound_ymin, bound_ymax, bound_ystep, ylabel,
             "black",
             0,1,0,
             /*NoXBotTic,NoXTopTic,NoXLabel,*/
             1,1,1, /* NoYBotTic,NoYTopTic,NoYLabel,*/
             0,0,0,0/* NoXBotNum,NoXTopNum,NoYBotNum,NoYTopNum*/);
    }
    /* draw the decision region "bar" */
    if(histogram_plot) v1 = vbot;
    else v1 = vtop;
    v1 -= 11; /* !!! height of bar on original standard graph */
    v2 = v1 - 11;
    old_x = xval = bound_xmin;
    for (indx = 1; indx <= bound_npoints; indx++) {
    if (graph_class[indx] != graph_class[indx-1]) {
        /* output class band rectangles */
        x = xval - del_x/2.;
        u2 = x2u (x);
        uvblock( color_table[graph_class[indx-1] % N_PLOT_COLORS],
            x2u (old_x), v1, u2, v2);
        old_x = x;
        /* output gray lines to separate classes */
        gcolor("gray");
        uvline (u2, vbot_whole, u2, vtop_whole);
    }
    xval += del_x;
    }
    uvblock( color_table[graph_class[indx-1] % N_PLOT_COLORS],
        x2u(old_x), v1, utop, v2);

    gcolor("black");
    deallocate_vector(graph_x);
    deallocate_matrix(graph_y);
    deallocate_ivector(graph_class);
}

float
plot_histogram (data_file, n_patterns, stddev)
     FILE *data_file;
     int n_patterns;
     float *stddev;
{
    int desired_class,  class;
    int  i0, i1, indx, u1, v1, u2, v2;
    float x1, del_x, cur_y;
    int *correct_class, *incorrect_class;
    int **correct_matrix, **incorrect_matrix;
    int trial_cnt, error_cnt;
    float percent_error;
    int relevant = 1;
    float axis_xmin, axis_xmax, axis_xstep;
    float axis_ymin, axis_ymax, axis_ystep;
    float height_fract = 1., width_fract = 1.;
    float xoffset_fract = 0.1, yoffset_fract = 0.;
    float y0 = 0.;

    correct_class = allocate_ivector(bound_npoints);
    correct_matrix = allocate_imatrix(bound_npoints, plot_n_outputs);
    incorrect_class = allocate_ivector(bound_npoints);
    incorrect_matrix = allocate_imatrix(bound_npoints, plot_n_outputs);
    desired_class = trial_cnt = error_cnt = 0;
    del_x = xwidth / (float) bound_npoints;

    /* run through the patterns once to get our scale */
    for (i0=0; i0<n_patterns; i0++) {
    /* should this be combined in histo and scatter? */
    desired_class =
      get_class_line (data_file, scatter_data_fullname, inputs,
              plot_n_inputs, normalization, nn_features, ninputs);
    if (desired_class == EOF) break;
    if (!show_all) {
        relevant = 1;
        for (i1=0; i1<ninputs; i1++) {
        if (i1 != bound_dim_1) {
            if (inputs[i1] < other_vars[i1+1] - distance_limit || 
            inputs[i1] > other_vars[i1+1] + distance_limit)
              relevant = 0;
        }
        }
    }
    if(profile_plot){
        trial_cnt++;
        class = find_class(inputs);
        if (class != desired_class) error_cnt++;
    } else {
        /* if just histogram, plot all on top */
        class = desired_class;
    }
    x1 = inputs[bound_dim_1];
    if (relevant && x1 >= bound_xmin && x1 <= bound_xmax) {
        /* find the correct bin for this pattern and update it */
        indx = (x1 - bound_xmin) / del_x;
        if(0 <= indx && indx < bound_npoints){
        if (class == desired_class) {
            correct_class[indx]++;
            correct_matrix[indx][desired_class]++;
        } else {
            incorrect_class[indx]++;
            incorrect_matrix[indx][desired_class]++;
        }
        }
    }
    }
    ylimits[0] = ylimits[1] = 0.;
    for(indx = 0; indx < bound_npoints; indx++){
    if(correct_class[indx] > ylimits[1])
      ylimits[1] = correct_class[indx];
    if(incorrect_class[indx] > ylimits[0])
      ylimits[0] = incorrect_class[indx];
    }
    ylimits[0] *= -1;
    if(profile_plot) height_fract = .5;
    /* always autoscale the histogram plot */
    xygraph_scale (xlimits, ylimits, 2,
           0, 0, 0, 10,
           1, 2, "",
           0, 0.0, 0.0,
           height_fract, width_fract, xoffset_fract, yoffset_fract,
           0, 0, bound_xmin, bound_xmax, bound_xstep, "xlabel",
           0, 1, bound_ymin, bound_ymax, bound_ystep,
           "ylabel", "black",
           &ubot, &vbot, &utop, &vtop,
           &axis_xmin, &axis_xmax, &axis_xstep,
           &axis_ymin, &axis_ymax, &axis_ystep);
    bound_ymin = axis_ymin;
    bound_ymax = axis_ymax;
    bound_ystep = axis_ystep;
    vwidth = vtop - vbot;
    ywidth = bound_ymax - bound_ymin;

    v1 = y2v (y0);
    uvline (ubot, v1, utop, v1);

    /* go through correct_matrix and incorrect_matrix and draw the blocks
       given there */
    for(indx = 0; indx < bound_npoints; indx++){
    u1 = x2u (bound_xmin + indx * del_x);
    u2 = x2u (bound_xmin + (indx + 1) * del_x);
    cur_y = y0;
    for(class = 0; class < plot_n_outputs; class++){
        /* skip classes that have no data in this bin */
        if(correct_matrix[indx][class] == 0) continue;
        v1 = y2v(cur_y);
        cur_y +=  correct_matrix[indx][class];
        v2 = y2v(cur_y);
        uvblock( color_table[class % N_PLOT_COLORS], u1, v1, u2, v2);
    }
    cur_y = y0;
    for(class = 0; class < plot_n_outputs; class++){
        /* skip classes that have no data in this bin */
        if(incorrect_matrix[indx][class] == 0) continue;
        v1 = y2v(cur_y);
        cur_y -=  incorrect_matrix[indx][class];
        v2 = y2v(cur_y);
        uvblock( color_table[class % N_PLOT_COLORS], u1, v1, u2, v2);
    }
    }
    deallocate_ivector(correct_class);
    deallocate_imatrix(correct_matrix);
    deallocate_ivector(incorrect_class);
    deallocate_imatrix(incorrect_matrix);

    /* show histogram grid */
    xygraph_advanced(xlimits, ylimits, 0,
             0, 0, 0, 10,
             1, 1, (profile_plot ? "":plot_heading),
             0, 0.0, 0.0,
             height_fract, width_fract, xoffset_fract, yoffset_fract,
             0, 0, bound_xmin, bound_xmax, bound_xstep, xlabel,
             0, 0, bound_ymin, bound_ymax, bound_ystep, ylabel,
             "black",
             0,(profile_plot ? 1 : 0),0,
             /*NoXBotTic,NoXTopTic,NoXLabel,*/
             0,0,0, /* NoYBotTic,NoYTopTic,NoYLabel,*/
             0,0,0,(profile_plot ? 1 : 0)
             /* NoXBotNum,NoXTopNum,NoYBotNum,NoYTopNum*/);

    if (trial_cnt > 0) {
    percent_error = (float) error_cnt / (float) trial_cnt;
    *stddev = sqrt ((percent_error * (1 - percent_error))
            / (float) trial_cnt);
    }
    return (percent_error);
}

int
find_class(inputs)
    register float *inputs;
{
    int i;

    return classify_COMMITTEE(inputs, outputs);
}

int
x2u (raw_x)
    float raw_x;
{
    return (ubot + (raw_x - bound_xmin) * uwidth / xwidth);
}

int
y2v (raw_y)
    float raw_y;
{
    return (vbot + (raw_y - bound_ymin) * vwidth / ywidth);
}

int
xlength2u (raw_x)
    float raw_x;
{
    return (raw_x * uwidth / xwidth);
}

int
ylength2v (raw_y)
    float raw_y;
{
    return (raw_y * vwidth / ywidth);
}

void
draw_ellipse (scale, ucenter, vcenter, uwidth, vwidth)
    float scale;
    int ucenter, vcenter, uwidth, vwidth;
{
    float c, s, delt;
    int u, v, unew, vnew;
    int i, nseg=100;

    uwidth /= scale;
    vwidth /= scale;
    delt = 2 * 3.141596 / (float) nseg;
    c = cos(0);
    s = sin(0);
    u = (int)(ucenter + uwidth * c);
    v = (int)(vcenter + vwidth * s);
    uvmove (u, v);
    for (i=1; i<=nseg; i++) {
    c = cos (i * delt);
    s = sin (i * delt);
    unew = (int)(ucenter + uwidth * c);
    vnew = (int)(vcenter + vwidth * s);
    uvline (u, v, unew, vnew);
    u = unew;
    v = vnew;
    }
    gflush();
}

void
draw_angle_ellipse (scale, ucenter, vcenter, uwidth, vwidth, theta)
     float scale;
     int ucenter, vcenter, uwidth, vwidth;
     double theta;
{
    float c, s, delt, ct, st, u, v;
    int urot, vrot, urot_new, vrot_new;
    int i, nseg=100;

    ct = cos(theta);
    st = sin(theta);

    uwidth *= scale;
    vwidth *= scale;
    delt = 2 * 3.141596 / (float) nseg;
    c = cos(0);
    s = sin(0);
    u = uwidth * c;
    v = vwidth * s;
    urot = (int)(ucenter + u*ct + v*st);
    vrot = (int)(vcenter - u*st + v*ct);
    uvmove (urot, vrot);
    for (i=1; i<=nseg; i++) {
    c = cos (i * delt);
    s = sin (i * delt);
    u = uwidth * c;
    v = vwidth * s;
    urot_new = (int)(ucenter + u*ct + v*st);
    vrot_new = (int)(vcenter - u*st + v*ct);
    uvline (urot, vrot, urot_new, vrot_new);
    urot = urot_new;
    vrot = vrot_new;
    }
    gflush();
}

/* Figures out eigenvalues and eigenvectors for a diagonal 2x2 matrix,
 then uses them to return the major and minor axes and the angle of rotation
 for the ellipse described by the matrix */
void
get_ellipse_rotation(a, b, c, px, py, ptheta)
     float a, b, c, *px, *py;
     double *ptheta;
{
    float b2, c2, l1, l2;

    if(a == b && b == -c){
    *px = *py = a; *ptheta = 0;
    return;}
    b2 = (a + b)/2;
    c2 = a*b - c*c;
    if(b2*b2 < c2)
      warning("bad covariance matrix in get_ellipse_rotation\n");
    l1 = b2 + sqrt(fabs(b2*b2 - c2));
    l2 = b2 - sqrt(fabs(b2*b2 - c2));
    if(l1 < 0 || l2 < 0){
    warning("negative axis length produced by get_ellipse_rotation\n");
    l1 = fabs(l1);
    l2 = fabs(l2);
    }
    if(l1 == 0){
    warning("0 axis length produced by get_ellipse_rotation\n");
    l1 = 1.;
    }
    if(l2 == 0){
    warning("0 axis length produced by get_ellipse_rotation\n");
    l2 = 1.;
    }
    *ptheta = atan2(c, (l1 - a));
    *px = sqrt(1./l1);
    *py = sqrt(1./l2);
}

void
calc_bound_autoscale(data_file, two_d)
    FILE *data_file;
    int two_d;
{
    int desired_class;
    float tmp;

    desired_class = 0;
    desired_class =
      get_class_line (data_file, scatter_data_fullname, inputs,
              plot_n_inputs, normalization, nn_features, ninputs);
    if (desired_class != EOF) {
    xlimits[0] = xlimits[1] = inputs[bound_dim_1];
    if (two_d) ylimits[0] = ylimits[1] = inputs[bound_dim_2];
    else ylimits[0] = ylimits[1] = 0.;
    while (1) {
        desired_class =
          get_class_line (data_file, scatter_data_fullname, inputs,
                  plot_n_inputs, normalization, nn_features,
                  ninputs);
        if (desired_class == EOF) break;
        tmp = inputs[bound_dim_1];
        if (tmp < xlimits[0]) xlimits[0] = tmp;
        if (tmp > xlimits[1]) xlimits[1] = tmp;
        if (two_d) {
        tmp = inputs[bound_dim_2];
        if (tmp < ylimits[0]) ylimits[0] = tmp;
        if (tmp > ylimits[1]) ylimits[1] = tmp;
        }
    }
            /* add some on each side to make it understandable */
    /*
    bound_xmin = xlimits[0];
    bound_xmax = xlimits[1];
    if(two_d){
        bound_ymin = ylimits[0];
        bound_ymax = ylimits[1];
    }

    bound_xmin = floor((double) bound_xmin);
    bound_xmax = ceil((double) bound_xmax);
    if (bound_xmin == bound_xmax) {
        bound_xmin = bound_xmin - 1;
        bound_xmax = bound_xmax + 1;
    }
    bound_xstep = (bound_xmax - bound_xmin) / 10.;
    if (two_d) {
        bound_ymin = floor((double) bound_ymin);
        bound_ymax = ceil((double) bound_ymax);
        if (bound_ymin == bound_ymax) {
        bound_ymin = bound_ymin - 1;
        bound_ymax = bound_ymax + 1;
        }
        bound_ystep = (bound_ymax - bound_ymin) / 10.;
    }
    */
    }
}

/*--------------------------------------------------------------
* write out an ascii spreadsheet  file to plot a scatter 
* plot on the macintosh 
--------------------------------------------------------------*/
void mac_scatter(data_file)
     FILE *data_file;
{
    int i,class, n;
    int nclasses;
    int *patterns_per_class;
    int max_patterns_per_class;
    int desired_class;
    FILE *macscatter_file;
    struct pattern {
    int n;
    float *x;
    float *y;
      } *class_patterns;
    
    /* Open file to store delta graph data */
    macscatter_file = nn_fopen(macscatter_fname, "w",
                   "for writing Macintosh scatter plot data");
    
    nclasses = plot_n_outputs;
    if ( nclasses <= 0)
      fatal_error("plot_bound: number of output classes is 0");
    patterns_per_class = allocate_ivector(nclasses);
    
    
    /* first scan to find the maximum number of patterns per class 
       -------------------------------------------------------------*/
    for (i=0; i<scatter_n_patterns; i++) {
    desired_class = get_class_line (data_file, scatter_data_fullname, 
                    inputs, plot_n_inputs, normalization, 
                    nn_features, ninputs);
    if (desired_class == EOF) 
      break;
      
    /* increment number of patterns per class */
    if ( desired_class < nclasses)
      patterns_per_class[desired_class] += 1;
    else
      warning("plot_bound: %s %d of file %s %s %d", 
          "desired class in line",  i+1, scatter_data_fullname,
          "is greater than specified number of outputs =", 
          nclasses);
    }
    
    /* allocate memory for patterns and find max npatterns per class
     -----------------------------------------------------------------------*/
    class_patterns = (struct pattern *)
      allocate_structvector(nclasses, sizeof(struct pattern));
    max_patterns_per_class=0;
    for(class=0; class<nclasses; class++) {
    n = patterns_per_class[class];
    if ( n > max_patterns_per_class)
      max_patterns_per_class = n;
    class_patterns[class].n=0;
    class_patterns[class].x=allocate_vector(n);
    class_patterns[class].y=allocate_vector(n);
    }
    
    rewind(data_file);
    
    /* store patterns 
       -------------------------------------------------------------*/
    for (i=0; i<scatter_n_patterns; i++) {
    desired_class = get_class_line (data_file, scatter_data_fullname, 
                    inputs, plot_n_inputs, normalization, 
                    nn_features, ninputs);
    if (desired_class == EOF) 
      break;
      
    /* increment number of patterns per class */
    if ( desired_class < nclasses) {
        n = class_patterns[desired_class].n;
        class_patterns[desired_class].x[n] = inputs[0];
        class_patterns[desired_class].y[n] = inputs[1];
        class_patterns[desired_class].n += 1;
    } else
      warning("plot_bound: %s %d of file %s %s %d", 
          "desired class in line",  i+1, scatter_data_fullname,
          "is greater than specified number of outputs =", 
          nclasses);
    }
    
    /* write patterns to paired xy file 
       -------------------------------------------------------------*/
    for(class=0; class< nclasses; class++)
      {
      if(plot_labels != NULL && atoi(plot_labels[0]) > class)
        fprintf(macscatter_file, "%s\t%s\t", 
            plot_labels[class+1], plot_labels[class+1]); 
      }
    fprintf(macscatter_file, "\n"); 
    
    for (n=0; n<max_patterns_per_class; n++) {
    for(class=0; class< nclasses; class++) {
        if ( class_patterns[class].n > n)
          fprintf(macscatter_file, "%.3f\t%.3f\t", 
              class_patterns[class].x[n],
              class_patterns[class].y[n]);
        else
          fprintf(macscatter_file, " \t \t"); 
    }
    fprintf(macscatter_file, "\n");
    }
    
    nn_fclose(macscatter_file, macscatter_fname,
          "for writing Macintosh scatter plot data");
    deallocate_ivector(patterns_per_class);
    for(class = 0; class < nclasses; class++){
    deallocate_vector(class_patterns[class].x);
    deallocate_vector(class_patterns[class].y);
    }
    deallocate_structvector(class_patterns);
}

plot_legend(two_d)
{
    int i;
    int symbol_radius = 20;
    char *label_str;

    legend_x = utop + 40;
    uvprintf (legend_x, vtop - 40, 1, "%s", "LEGEND");
    gcolor("black");
    for (i = 0; i < plot_n_outputs; i++) {
    legend_y = vtop - 80 - i * 40;
    uvmove(legend_x, legend_y);
    if (show_color) {
        gcolor(color_table[i % N_PLOT_COLORS]);
        gsymbol(SOLID_BLOCK,symbol_radius);
        gcolor("black");
    } else {
        if(internals_level > 1)
          gputc (1, 'A' + i);
        else
          gsymbol(i%NSYMBOLS, symbol_radius/2);
    }
    if(plot_labels != NULL && atoi(plot_labels[0]) > i)
      uvprintf (legend_x + 40, legend_y, 1, "%s", plot_labels[i+1]);
    }
    if (show_misses || !two_d) {
    legend_y = vtop - 100 - plot_n_outputs * 40;
    if (show_color) {
        uvmove(legend_x, legend_y);
        gcolor(color_table[i % N_PLOT_COLORS]);
        gsymbol(SOLID_BLOCK,symbol_radius);
        gcolor("black");
    } else {
        uvpoint(legend_x, legend_y);
    }
    if (show_misses) {
        if (show_color) label_str = "Errors";
        else        label_str = "Correct";
    } else      label_str = "Total";
    uvprintf (legend_x + 40, legend_y, 1, "%s", label_str);
    }
}
