/***--------------------------------------------------------------------------
* NAME:     err2data_main.c
* PURPOSE:  create a committee data base from a set of long error files
* USAGE:
*       err2data -<flags>
* EXAMPLE:  err2data --                * shows all available flags
*
* SEE ALSO: 
*
* GLOBALS:  <flags> defined in err2data_args.h
*
* DESCRIPTION:
*   Reads a series of error files and combines matching lines
*   to create a data file with the outputs from each pattern as its inputs
*   The program checks that the line numbers match and that the class
*   labels match for each pattern read
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/committee/err2data.c,v 11.5 2001/08/23 20:11:32 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/committee/err2data.c,v $
* CREATED 15-Mar-94 lck             MIT LINCOLN LABORATORY
*--------------------------------------------------------------------------**/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/committee/err2data.c,v 11.5 2001/08/23 20:11:32 craig Exp $";

/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#include "err2data_args.h"          /* parameter definitions */

/* real definition is in get_subs.c? */
#define OUTPUTS 2

int
main(int argc, char **argv)
{
    FILE **fp_list, *new_fp;
    ERROR_LINE *err_line_list;
    float *inputs;
    int nfiles, ninputs, i, j, k, first_npatterns, npatterns, rv, trial_no, class_no;

    init_nnsubs(argc, argv, rcsid);
    /* read arguments */
    argc = nnargs(argc, argv, err2data_tab, ERR2DATA_TABSIZE);
    
    /* check the arguments */
    if(file_list == NULL)
      fatal_error("%s: A list of input files must be specified", argv[0]);
    if(output_file == NULL)
      fatal_error("%s: An output data file must be specified", argv[0]);
    if(noutputs < 0)
      fatal_error("%s: noutputs must be greater than 0", argv[0]);

    /* set variables, allocate space, and open files */
    nfiles = atoi(file_list[0]);
    if(nfiles == 0)
      fatal_error("%s: A list of input files must be specified", argv[0]);

    ninputs = noutputs*nfiles;
    inputs = allocate_vector(ninputs);
    fp_list = (FILE **)allocate_pvector(nfiles);
    /* this is a convience routine defined in get_subs.c */
    err_line_list =
      reallocate_error_lines(NULL, 0, nfiles, ninputs, noutputs);

    new_fp = nn_fopen(path_build(exp_path, output_file), "w",
                   "for new committee data base file");
    for(i = 0; i < nfiles; i++){
    fp_list[i] = nn_fopen(path_build(exp_path, file_list[i+1]), "r",
                  "for committee input data file");
    }

    /* check file lengths are equal */
    first_npatterns = 0;
    check_file_length(path_build(exp_path, file_list[1]), &first_npatterns);
    for(i = 1; i < nfiles; i++){
    npatterns = 0;
    check_file_length(path_build(exp_path, file_list[i+1]), &npatterns);
    if(npatterns != first_npatterns)
      fatal_error("%s: Files are different lengths (%s and %s)",
              argv[0], file_list[1], file_list[i+1]);
    }
    
    /* read the outputs from all the files and write the committee file */
    while(1){
    /* read the error lines */
    for(i = 0; i < nfiles; i++){
        rv = get_err_line(fp_list[i], file_list[i+1], &err_line_list[i],
                  OUTPUTS);
        if(rv == EOF) break;
    }
    if(rv == EOF) break;
    /* check trial numbers */
    trial_no = err_line_list[0].trial;
    for(i = 1; i < nfiles; i++){
        if(err_line_list[i].trial != trial_no)
          /* I'm not sure that all of the classifiers are
         providing the correct sort of trial number these days,
         so this is just a warning */
          warning("%s: trial numbers do not match in input data files",
              argv[0]);
    }
    /* check class numbers */
    class_no = err_line_list[0].desired_class;
    for(i = 1; i < nfiles; i++){
        if(err_line_list[i].desired_class != class_no)
          fatal_error("%s: class numbers do not match in input data files",
              argv[0]);
    }
    /* copy the outputs into the big inputs array */
    for(i = j = 0; i < nfiles; i++){
        for(k = 0; k < noutputs; k++, j++){
        inputs[j] = err_line_list[i].outputs[k];
        }
    }
    /* print out the class all outputs, in order */
    write_class_line(new_fp, output_file, inputs, class_no, ninputs);
    }
    /* close files and deallocate space */
    nn_fclose(new_fp, output_file, "for new committee data base file");
    for(i = 0; i < nfiles; i++)
      nn_fclose(fp_list[i], file_list[i+1], "for committee input data file");
    deallocate_structvector(err_line_list);
    deallocate_pvector((void **)fp_list);
    deallocate_vector(inputs);

    exit(0);
}
