/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* em_clus2c.c */
/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#define GEN_NO_OUTPUTS
#define GEN2CVARS
#include <gen_2c_args.h>
#include "em_clus_pack.h"

/* em_clus specific variables */
int ncenters;
float **centers;
float **inv_var;
float *log_det_covar;
char ncenters_name[64];
char get_ncenters_name[64];
char l2dist_name[64];

/* function declerations */
int read_model_file(char **argv);
void free_model(int model);

void get_params(int model);
void get_general_params(int model);
void get_specific_params(int model);

void print_comments(void);
void print_declarations(void);
void print_macros(void);
void print_classify_code(int model);
void print_l2dist_code(void);
void print_get_ncenters_code(void);
void clear_macros(void);

int
main(int argc, char **argv)
{
    int model;

    /* process arguments */
    argc = nnargs(argc, argv, argtab2c, TABSIZE2C);

    /* check arguments */
    if(strlen(class_suff) > 50){
    fatal_error("suffix (%s) is too long.  use at most 50 characters",
            class_suff);
    }

    /* set up subroutine names and macro names */
    strcpy(classify_name, "classify");
    strcpy(normalize_name, "normalize");
    strcpy(l2dist_name, "l2dist");
    strcpy(nraw_name, "NRAW");
    strcpy(ninputs_name, "NINPUTS");
    strcpy(ncenters_name, "NCENTERS");
    strcpy(get_ncenters_name, "get_ncenters");
    if(class_suff[0] != '\0'){
        strcpy(class_exten, "_");
    strcat(class_exten, class_suff);
    strcat(classify_name, class_exten);
    strcat(normalize_name, class_exten);
    strcat(l2dist_name, class_exten);
    strcat(nraw_name, class_exten);
    strcat(ninputs_name, class_exten);
    strcat(ncenters_name, class_exten);
    strcat(get_ncenters_name, class_exten);
    } else {
      strcpy(class_exten, "");
    }


    /* read in parameters */
    model = read_model_file(argv);
    get_params(model);
    if(adjust_test)
      warning("%s: This algorithm does not adjust test priors", argv[0]);

    print_comments();
    print_declarations();
    print_macros();
    print_classify_code(model);
    print_normalize_code(normalize_name);
    print_get_ncenters_code();
    print_l2dist_code();
    clear_macros();

    free_model(model);
    exit(0);
}


int
read_model_file(char **argv)
{
    int   model, norm_model, normalization;
    FILE *param_file;

    param_file = nn_fopen(param_fname, "r", "for read of model file");
    norm_model = norm_read_paramfile(param_file, &normalization);
    if (norm_model < 0)
      fatal_error("%s: read error %d %s %s", argv[0], norm_model,
          "on normalization parameter file", param_fname);

    model = em_clus_read_model(param_file);
    if (model < 0)
      fatal_error("%s: read error %d on model file %s",
          argv[0], model, param_fname);
    nn_fclose(param_file, param_fname, "for read of model file");

    return model;
}

void
free_model(int model)
{
  norm_free();
  em_clus_free(model);
}
void
get_params(int model)
{
    get_specific_params(model);
    get_general_params(model);
    get_norm_params(model, ninputs, features);
}

void
get_specific_params(int model)
{
    centers = *(float ***)em_clus_param(model, "means");
    inv_var = *(float ***)em_clus_param(model, "inv_var");
    log_det_covar = *(float **)em_clus_param(model, "log_det_covar");
    ncenters = *(int *)em_clus_param(model, "ncenters");
}

void
get_general_params(int model)
{
    ninputs = *(int *)em_clus_param(model, "ninputs");
    ncenters = *(int *)em_clus_param(model, "ncenters");
    features = *(int **)em_clus_param(model, "features");
}

void
print_comments(void)
{
    printf("/* %s */\n", param_fname);
    print_norm_comments();
}

void
print_declarations(void)
{
    printf("\n");
    printf("#include <stdio.h>\n");
    printf("#include <math.h>\n");
    printf("#include <values.h>\n");
    printf("#ifndef SQR(x)\n");
    printf("#define SQR(x) ((x) * (x))\n");
    printf("#endif\n");
    printf("#ifndef M_LOG10E\n");
    printf("#define M_LOG10E 0.43429448190325182765\n");
    printf("#endif\n");
    printf("\n");
    printf("#ifndef M_LN10\n");
    printf("#define M_LN10 2.30258509299404568402\n");
    printf("#endif\n");
    printf("\n");
    printf("#ifndef LOG_2PI\n");
    printf("#define LOG_2PI 0.798179868358\n");
    printf("#endif\n");
    printf("\n");
    printf("\n /* function declarations */\n");
    printf("extern int %s(/* float *,float * */);\n", classify_name);
    printf("extern int %s(/* float * */);\n", normalize_name);
    printf("static float %s(/* int, float *, float * */);\n", l2dist_name);
    printf("\n\n");
}

void
print_macros(void)
{
    print_norm_macros(nraw_name, ninputs_name);
    printf("#define %s %d\n", ncenters_name, ncenters);
    printf("\n");
}

void
clear_macros(void)
{
    clear_norm_macros();
    printf("#undef %s\n", ncenters_name);
    printf("\n");
}

void
print_get_ncenters_code(void)
{
    printf("int %s ()\n", get_ncenters_name);
    printf("{\n");
    printf("return %s;\n", ncenters_name);
    printf("}\n");
}

void    
print_classify_code(int model)
{
    printf("int %s (inputs, outputs)\n", classify_name);
    printf("   float *inputs,*outputs;\n");
    printf("{\n");
    printf("  int n, j, max_n;\n");
    printf("  float max_dist;\n");
    print_matrix_initialization("means", centers, ncenters, ninputs,
                ncenters_name, ninputs_name);
    print_matrix_initialization("inv_var", inv_var, ncenters, ninputs,
                ncenters_name, ninputs_name);
    print_vector_initialization("log_determinant", log_det_covar,
                ncenters, ncenters_name);

    /* copy and normlize inputs */
    print_copy_inputs();
    printf("\n  /* normalize loaded data */\n");
    printf("  %s(x);\n", normalize_name);
    printf("\n");
    printf("  /* find the nearest cluster */\n");
    printf("  max_dist = -MAXFLOAT;\n");
    printf("  max_n = -1;\n");
    printf("  for(n = 0; n < %s; n++){\n", ncenters_name);
    printf("    outputs[n] = 0.;\n");
    printf("    for(j = 0; j < %s; j++){\n", ninputs_name);
    printf("      outputs[n] += SQR(x[j]-means[n][j]) * inv_var[n][j];\n");
    printf("    }\n");
    printf("    outputs[n] *= M_LOG10E;\n");
    printf("    outputs[n] += log_determinant[n] + %s * LOG_2PI;\n", ninputs_name);
    printf("    outputs[n] *= -0.5;\n");
    printf("    if(outputs[n] > max_dist){\n");
    printf("      max_dist = outputs[n];\n");
    printf("      max_n = n;\n");
    printf("    }\n");
    printf("  }\n");
    printf("\n  /* copy outputs and make linear */\n");
    printf("  for(n = 0; n < %s; n++)\n", ncenters_name);
    printf("    outputs[n] = exp(M_LN10 * (double)outputs[n]);\n");
    printf("  return(max_n);\n");
    printf("}\n");
}

void
print_l2dist_code(void)
{
    printf("\n\n");
    printf("static float %s(n, x1, x2)\n", l2dist_name);
    printf("   register int n;\n");
    printf("   register float *x1, *x2;\n");
    printf("{\n");
    printf("  float dist = 0.;\n");
    printf("  for(--n; n >= 0; n--)\n");
    printf("    dist += SQR(x1[n]-x2[n]);\n");
    printf("  return(dist);\n");
    printf("}\n");
}
