/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* gauss2c.c */
/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#define GEN2CVARS
#include <gen_2c_args.h>
#include "gauss_pack.h"

/* decleration of static (global) variables to be used in most functions */
/* they are initialized by function call get_param() */

/* gauss specific variables */
static int per_class;
static int full_covar;                             /* from gauss model */

/* function declerations */
int read_model_file(char **argv);
void free_model(int model);

void get_params(int model);
void get_general_params(int model);
void get_specific_params(int model);

void set_priors(int model);

void print_comments(void);
void print_declarations(void);
void print_macros(void);
void print_classify_code(int model);
void clear_macros(void);

int
main(int argc, char **argv)
{
    int model;

    /* process arguments */
    argc = nnargs(argc, argv, argtab2c, TABSIZE2C);

    /* check arguments */
    if(strlen(class_suff) > 53){
    fatal_error("suffix (%s) is too long.  use at most 50 characters",
            class_suff);
    }

    /* set up subroutine names and macro names */
    strcpy(classify_name, "classify");
    strcpy(normalize_name, "normalize");
    strcpy(nraw_name, "NRAW");
    strcpy(ninputs_name, "NINPUTS");
    strcpy(noutputs_name, "NCLASSES");
    if(class_suff[0] != '\0'){
    strcpy(class_exten, "_");
    strcat(class_exten, class_suff);
    strcat(classify_name, class_exten);
    strcat(normalize_name, class_exten);
    strcat(nraw_name, class_exten);
    strcat(ninputs_name, class_exten);
    strcat(noutputs_name, class_exten);
    } else {
      strcpy(class_exten, "");
    }

    /* read in parameters */
    model = read_model_file(argv);
    get_params(model);

    set_priors(model);

    /* print the source file */
    print_comments();
    print_declarations();
    print_macros();
    print_classify_code(model);
    print_normalize_code(normalize_name);
    clear_macros();

    free_model(model);
    exit(0);
}

int
read_model_file(char **argv)
{
    int   model, norm_model, normalization;
    FILE *param_file;

    param_file = nn_fopen(param_fname, "r", "for read of model file");
    norm_model = norm_read_paramfile(param_file, &normalization);
    if (norm_model < 0)
      fatal_error("%s: read error %d %s %s", argv[0], norm_model,
          "on normalization parameter file", param_fname);
    
    model = gauss_read_model(param_file);
    if (model < 0)
      fatal_error("%s: read error %d on model file %s",
          argv[0], model, param_fname);
    nn_fclose(param_file, param_fname, "for read of model file");

    return((int) model);
}

void
free_model(int model)
{
  norm_free();
  gauss_free(model);
}


void
get_params(int model)
{
    get_specific_params(model);
    get_general_params(model);
    get_norm_params(model, ninputs, features);
}

void
get_specific_params(int model)
{
    full_covar = *(int *)gauss_param(model, "full_covar", 0);
    per_class = *(int *)gauss_param(model, "per_class", 0);
}

void 
get_general_params(int model)
{
    ninputs = *(int *)gauss_param(model, "ninputs", 0);
    noutputs = *(int *)gauss_param(model, "noutputs", 0);
    features = *(int **)gauss_param(model, "features", 0);
}

void
set_priors(int model)
{
    int i;
    float total;
    float *test_priors_store;

    if(adjust_test){
        test_priors_store = allocate_vector(noutputs);
    if(test_uniform_priors){
        if(test_priors_list != NULL)
          deallocate_vector(test_priors_list);
        test_priors_list = allocate_vector(noutputs+1);
        test_priors_list[0] = noutputs;
        for(i = 1; i <= noutputs; i++){
        test_priors_list[i] = 1./(float)noutputs;
        test_priors_list[i] = log10(test_priors_list[i]);
        test_priors_store[i-1] = test_priors_list[i];
        }
    } else {
        if(test_priors_list[0] != (float)noutputs)
          fatal_error("%s: %s %.0f, should be %d",
              "test_priors_list length is", test_priors_list[0],
              noutputs);
        total = 0.;
        for(i = 1; i <= noutputs; i++) total += test_priors_list[i];
        for(i = 1; i <= noutputs; i++){
        test_priors_list[i] /= total;
        test_priors_list[i] = log10(test_priors_list[i]);
        test_priors_store[i-1] = test_priors_list[i];
        }
    }
    deallocate_vector(test_priors_list);
    test_priors_list = test_priors_store;
    } else {
        if (test_priors_list != NULL) deallocate_vector(test_priors_list);
    test_priors_list = allocate_vector(noutputs);
    total = *(int *)gauss_param(model, "total_trials", 0);
    for(i = 0; i < noutputs; i++){
        test_priors_list[i] = (float)*(int *)
          gauss_param(model, "class_trials", i);
        test_priors_list[i] /= total;
        test_priors_list[i] = log10(test_priors_list[i]);
    }
    }
}

void
print_comments(void)
{
    printf("/* %s */\n", param_fname);
    printf("/* ");
    if(per_class) printf("Per Class, ");
    else printf("Grand, ");
    if(full_covar) printf("Full */\n");
    else printf("Diagonal */\n");
    print_norm_comments();
}

void
print_declarations(void)
{
    printf("\n");
    printf("#include <math.h>\n");
    printf("\n");
    printf("\n /* macro definitions */\n");
    printf("#ifndef SQR(x)\n");
    printf("#define SQR(x) ((x) * (x))\n");
    printf("#endif\n");
    printf("\n");
    printf("#ifndef M_LOG10E\n");
    printf("#define M_LOG10E 0.43429448190325182765\n");
    printf("#endif\n");
    printf("\n");
    printf("#ifndef M_LN10\n");
    printf("#define M_LN10 2.30258509299404568402\n");
    printf("#endif\n");
    printf("\n");
    printf("#ifndef LOG_2PI\n");
    printf("#define LOG_2PI 0.798179868358\n");
    printf("#endif\n");
    printf("\n");

    printf("\n /* function declarations */\n");
    printf("extern int %s(/* float *,float * */);\n", classify_name);
    printf("extern int %s(/* float * */);\n", normalize_name);
    printf("\n\n");
}

void
print_macros(void)
{
    print_norm_macros(nraw_name, ninputs_name);
    printf("#define %s %d\n", noutputs_name, noutputs);
    printf("\n");
}    

void
clear_macros(void)
{
    clear_norm_macros();
    printf("#undef %s\n", noutputs_name);
    printf("\n");
}    

void
print_classify_code(int model)
{
    int   i, j;
    float *means, **covar, *var, log_det;

    printf("int %s (inputs, outputs)\n", classify_name);
    printf("   float *inputs,*outputs;\n");
    printf("{\n");
    printf("  int n, best;\n");
    printf("  int j");
    if( full_covar) printf(",k");
    printf(";\n");
    printf("  float y[%s]", noutputs_name);
    if(full_covar) printf(",p");
    printf(";\n");
    /* print means */
    printf("  static float means[%s][%s] = {\n", noutputs_name, ninputs_name);
    for(i = 0; i < noutputs; i++){
    printf("\t{");
    means = *(float **)gauss_param(model, "mean", i);
    print_vector(means, ninputs);
    printf("},\n");
    }
    printf("};\n");
    /* print variances */
    if(per_class){
    if(full_covar){
        printf("  static float inv_var[%s][%s][%s] = {\n",
           noutputs_name, ninputs_name, ninputs_name);
        for(i = 0; i < noutputs; i++){
        printf("\t/* class %d */{\n", i);
        covar = *(float ***)gauss_param(model,
                        "per_class_covariance", i);
        for(j = 0; j < ninputs; j++){
            printf("\t  {");
            print_vector(covar[j], ninputs);
            printf("},\n");
        }
        printf("},\n");
        }
        printf("};\n");
    } else {
        printf("  static float inv_var[%s][%s] = {\n",
           noutputs_name, ninputs_name);
        for(i = 0; i < noutputs; i++){
        var = *(float **)gauss_param(model, "per_class_variance", i);
        printf("\t {");
        print_vector(var, ninputs);
        printf("},\n");
        }
        printf("};\n");
    }
    } else {
    if(full_covar){
        covar = *(float ***)gauss_param(model, "covariance", 0);
        print_matrix_initialization("inv_var", covar, ninputs, ninputs,
                    ninputs_name, ninputs_name);
    } else {
        var = *(float **)gauss_param(model, "variance", 0);
        print_vector_initialization("inv_var", var, ninputs, ninputs_name);
    }
    }
    /* print determinant */
    if(per_class){
    printf("  static float log_determinant[%s] = {\n", noutputs_name);
    for(i = 0; i < noutputs; i++){
        log_det = *(float *)gauss_param(model, "per_class_determinant",
                         i);
        if(i && !(i % NFLOATS_PER_LINE)) printf("\n ");
        printf("%14.6e, ", log_det);
    }
    printf("};\n");
    } else {
    log_det = *(float *)gauss_param(model, "determinant", 0);
    printf("  float log_determinant = %14.6e;\n", log_det);
    }
    print_vector_initialization("log_class_priors", test_priors_list,
                noutputs, noutputs_name);

    /* copy and normlize inputs */
    print_copy_inputs();
    printf("\n  /* normalize loaded data */\n");
    printf("  %s(x);\n", normalize_name);

    printf("/* calculate outputs for each class */\n");
    printf("  for(n = 0; n < %s; n++){\n", noutputs_name);
    printf("    y[n] = 0;\n");
    printf("    for(j = 0; j < %s; j++){\n", ninputs_name);
    if(full_covar){
    printf("      p = 0.;\n");
    printf("      for(k = 0; k < %s; k++){\n", ninputs_name);
    if(per_class)
      printf("        p += (x[k]-means[n][k])*inv_var[n][j][k];\n");
    else
      printf("        p += (x[k]-means[n][k])*inv_var[j][k];\n");
    printf("      }\n");
    printf("      y[n] += (x[j] - means[n][j])*p;\n");
    printf("    }\n");
    } else {
    if(per_class)
      printf("      y[n] += SQR(x[j]-means[n][j]) * inv_var[n][j];\n");
    else
      printf("      y[n] += SQR(x[j]-means[n][j]) * inv_var[j];\n");
    printf("    }\n");
    }
    printf("    y[n] *= 0.5 * M_LOG10E;\n");
    if(per_class)
      printf("    y[n] += log_determinant[n] * 0.5 + %s*0.5*LOG_2PI;\n",
         ninputs_name);
    else
      printf("    y[n] += log_determinant * 0.5 + %s*0.5*LOG_2PI;\n", ninputs_name);
    printf("    y[n] = -y[n];\n");
    printf("    y[n] += log_class_priors[n];\n");
    printf("  }\n");
    printf("\n  /* copy outputs and make linear */\n");
    printf("  for(n = 0; n < %s; n++)\n", noutputs_name);
    printf("    outputs[n] = exp(M_LN10 * (double)y[n]);\n");
    printf("\n  /* find highest output */\n");
    printf("  for(best = n = 0; n < %s; n++)\n", noutputs_name);
    printf("    if(y[best] < y[n]) best = n;\n");
    printf("  return(best);\n");
    printf("}\n");
}
