/***-------------------------------------------------------------------------
 * NAME:        gauss_seq_pack.c
 * PURPOSE: Functions specific to gauss_seq
 *          
 * SEE ALSO:    gauss_seq_train.c gauss_seq_test.c gauss_seq_main.c
 *
 *
 * DESCRIPTION:
 *    Functions in this file
 *
 * ALGORITHM:
 * REFERENCE:
 *
 * BUGS AND LIMITATIONS:
 *
 * $Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/gauss/gauss_seq_pack.c,v 11.3 2001/08/23 20:11:43 craig Exp $
 * $Source: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/gauss/gauss_seq_pack.c,v $
 * CREATED: 7-May-93 lck            MIT LINCOLN LABORATORY
 ------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#include "gauss_seq_pack.h"
#include "gauss_pack.h"

/* constant definitions */
#define PEQ(y) (0==strcmp(param_name,y))
#define SMALLEST 1.e-30 /* variance is not allowed to be less than SMALLEST */

typedef struct {
    int noutputs;
    float *seq_sums;
    int gauss_model;
} GAUSS_SEQ_PARAMS;

/* Internal Globals */
static GAUSS_SEQ_PARAMS **gausspp;/* array of parameters for each model */
static int max_models = 0;  /* next new model number to be assigned */


/************************************************************************
 * NAME: gauss_seq_build(ninputs, noutputs, per_class, full)
 * DESCRIPTION:
 *  Allocates space for a gauss model 
 *
 *************************************************************************/
int gauss_seq_build(int noutputs)
{
    register int    new_model;
    GAUSS_SEQ_PARAMS *gpp;  /* temporary pointer to genpp[] */
    
    new_model = model_get_number((void ***)&gausspp, &max_models,
                 sizeof(GAUSS_SEQ_PARAMS));
    gpp = gausspp[new_model];
    
    /* set fields which are array sizes */
    gpp->noutputs = noutputs;
    
    /* allocate space for vectors */
    gpp->seq_sums = allocate_vector(noutputs);
    return new_model;
}

/*****************************************************************************
 * NAME: gauss_seq_init ()
 * DESCRIPTION:     
 *      Initialize gaussian classifier variables
 *
 *****************************************************************************/
void gauss_seq_init (int model, int gauss_model)
{
    int i;
    GAUSS_SEQ_PARAMS *gpp;  /* temporary pointer to genpp[] */
    
    gpp = gausspp[model];
    gpp->gauss_model = gauss_model;
    for(i = 0; i < gpp->noutputs; i++) gpp->seq_sums[i] = 0.;
}      

/************************************************************************
 * NAME: gauss_seq_free()
 * DESCRIPTION:
 *      Deallocates space used in a gauss model 
 *
 *************************************************************************/
void gauss_seq_free(int model)
{
    GAUSS_SEQ_PARAMS *gpp;  /* temporary pointer to genpp[] */
    
    gpp = gausspp[model];
    deallocate_vector(gpp->seq_sums);
    nn_free((char *)gpp, "gauss model");
    gausspp[model] = NULL;
}

/************************************************************************
 * NAME: gauss_seq_param(param_name)
 * DESCRIPTION:
 *  takes parameter name.
 *  returns a floating point pointer to the parameter.
 *  single value parameters are read only.
 *  function really not necessary if not interested in accessing
 *  parameters (reading) from main()
 *************************************************************************/
void * gauss_seq_param(int model, char *param_name)
{
    void *param_address;
    GAUSS_SEQ_PARAMS *gpp;  /* temporary pointer to genpp[] */
    
    gpp = gausspp[model];
    if (PEQ("seq_sums"))
      param_address = &gpp->seq_sums;
    else {
    warning("gauss_seq_param: parameter %s not defined", param_name);
    param_address = NULL;
    }
    return param_address;
} 


void
gauss_seq_batch_classify(int model, float *inputs)
{
    int i;
    GAUSS_SEQ_PARAMS *gpp;
    float *outputs;

    gpp = gausspp[model];
    gauss_eval(gpp->gauss_model, inputs);
    outputs = *(float **)gauss_param(gpp->gauss_model, "outputs", 0);
    /* assumes log outputs. must be set by driver currently */
    for(i = 0; i < gpp->noutputs; i++){
    gpp->seq_sums[i] += outputs[i];
    }
}

int
gauss_seq_update_classify(int model)
{
    GAUSS_SEQ_PARAMS *gpp;
    gpp = gausspp[model];
    return(max_pick(gpp->seq_sums, gpp->noutputs));
}
