/***---------------------------------------------------------------------------
* NAME:     gauss_seq_train.c 
* PURPOSE:  training driver for Gaussian sequence classifier
*
* SEE ALSO: gauss_seq_main.c gauss_seq_pack.c
*
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/gauss/gauss_seq_train.c,v 11.4 2001/08/23 20:11:44 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/gauss/gauss_seq_train.c,v $
* CREATED: 7-May-93 lck         MIT LINCOLN LABORATORY
****************************************************************************/

/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <nnlib.h>
#include "gauss_seq.h"
#include "gauss_seq_pack.h"
#include "gauss_pack.h"

#define MAXSTR (256)

static char     prog_name[MAXSTR];  /* static variables */

void
gauss_seq_train(int nraw, int ninputs, int *features, int nwindow_frames,
        int *window_frames, int noutputs, int normalize,
        int nsequences, int debg, int verbosity, char *data_path,
        char *data_fname, char *norm_fname, char *exp_path,
        char *param_fname, char **labels, char *comment,
        float minimum_variance, int covariance_type,
        int variance_type, float max_ratio)
{
    int     model, data_model, normalization = 0;
    int     n_frames, n_files, model_ninputs;
    int     desired_class;
    float   *inputs;
    char    *tmp_path;
    FILE    *param_file;

    strcpy(prog_name, get_program_name());

    /* allocate and initialize model structure */
    /* there is no retraining for gauss models */
    model_ninputs = ninputs*nwindow_frames;

    model = gauss_build(model_ninputs, noutputs,
            variance_type, covariance_type);
    gauss_init(model, features, minimum_variance);
    if (normalize)
      normalization = read_norm_file(path_build(data_path, norm_fname));
    else norm_setup_none(ninputs, noutputs);

    /* read data */
    data_model = seq_data_read(data_path, data_fname, nsequences, nraw,
                   ninputs, noutputs, features, normalization,
                   labels, verbosity, nwindow_frames, window_frames);
    nsequences = seq_data_get_nfiles(data_model);

    for(n_files = 0; n_files < nsequences; n_files++){
    if(seq_data_get_next_file(data_model) == -1) break;
    desired_class = seq_data_get_class(data_model);
    for(n_frames = 0;; n_frames++){
        inputs = seq_data_get_next_frame(data_model);
        if(inputs == NULL) break;
        gauss_batch_ctrain(model, inputs, desired_class);
    }
    }
    gauss_batch_update (model, max_ratio);

    /****************** store model in param file ************************/
    tmp_path = path_build(exp_path, param_fname);
    param_file = nn_fopen(tmp_path, "w", "for write of model file");
    fprintf(param_file, "%s\n", comment);
    fprintf(param_file, "normalization data\n");
    norm_write_paramfile(param_file);
    fprintf(param_file, "feature selection\n");
    feature_write_paramfile(param_file, features, nraw, ninputs);
    fprintf(param_file, "windowing parameters\n");
    window_write_paramfile(param_file, window_frames, nwindow_frames);
    fprintf(param_file, "gauss model\n");
    gauss_write_model(model, param_file);
    nn_fclose(param_file, tmp_path, "for write of model file");
    if ( verbosity > 0)
      printf("\nFinished -- gauss model saved in \"%s\"\n", tmp_path);
    
    /* free space */
    gauss_free (model);
    norm_free();
    seq_data_exit(data_model);
}
