/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* generic2c.c */
/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#define GEN2CVARS
#include <gen_2c_args.h>
#include "generic_pack.h"

/* decleration of static (global) variables to be used in most functions */
/* they are initialized by function call get_param() */

/* variables specific to generic */
static float *train_priors_list;

/* function declerations */
int read_model_file(char** argv);
void free_model(int model );

void get_params(int model);
void get_general_params(int model);
void get_specific_params(int model);

void set_priors(int model);

void print_comments(void);
void print_declarations(void);
void print_macros(void);
void print_classify_code(int model);
void clear_macros(void);

void *generic_param(int model, char *param_name);


int
main(int argc, char **argv)
{
    int model;

    /* process arguments */
    argc = nnargs(argc, argv, argtab2c, TABSIZE2C);

    /* check arguments */
    if(strlen(class_suff) > 53){
    fatal_error("suffix (%s) is too long.  use at most 50 characters",
            class_suff);
    }

    /* set up subroutine names */
    strcpy(classify_name, "classify");
    strcpy(normalize_name, "normalize");
    strcpy(nraw_name, "NRAW");
    strcpy(ninputs_name, "NINPUTS");
    strcpy(noutputs_name, "NCLASSES");
    if(class_suff[0] != '\0'){
        strcpy(class_exten, "_");
    strcat(class_exten, class_suff);
    strcat(classify_name, class_exten);
    strcat(normalize_name, class_exten);
    strcat(nraw_name, class_exten);
    strcat(ninputs_name, class_exten);
    strcat(noutputs_name, class_exten);
    } else {
    strcpy(class_exten, "");
    }

    /* read in parameters */
    model = read_model_file(argv);
    get_params(model);

    set_priors(model);

    /* print the source file */
    print_comments();
    print_declarations();
    print_macros();
    print_classify_code(model);
    print_normalize_code(normalize_name);
    clear_macros();

    free_model(model);
    exit(0);
}

int
read_model_file(char **argv)
{
    int   model, norm_model, normalization;
    FILE *param_file;

    param_file = nn_fopen(param_fname, "r", "for read of model file");
    norm_model = norm_read_paramfile(param_file, &normalization);
    if (norm_model < 0)
      fatal_error("%s: read error %d %s %s", argv[0], norm_model,
          "on normalization parameter file", param_fname);
    
    model = generic_read_model(param_file);
    if (model < 0)
      fatal_error("%s: read error %d on model file %s",
          argv[0], model, param_fname);
    nn_fclose(param_file, param_fname, "for read of model file");

    return((int) model);
}

void
free_model(int model)
{
  norm_free();
  generic_free(model);
}


void
get_params(int model)
{
    get_specific_params(model);
    get_general_params(model);
    get_norm_params(model, ninputs, features);
}


void
get_specific_params(int model)
{
    /* get generic specific parameters */

    /* (1) get parameters from the arguments list of generic */
    /* (2) get parameters from generic model */

}


void
get_general_params(int model)
{
    /* get general parameters */

    ninputs = *(int *)generic_param(model, "ninputs");
    noutputs = *(int *)generic_param(model, "nclasses");
    features = *(int **)generic_param(model, "features");
}

void
set_priors(int model)
{
    int i;
    float total;

    /* now that we know noutputs, we can check the priors settings */
    if(adjust_test){
    if(test_uniform_priors){
        if(test_priors_list != NULL)
          deallocate_vector(test_priors_list);
        test_priors_list = allocate_vector(noutputs+1);
        test_priors_list[0] = noutputs;
        for(i = 1; i <= noutputs; i++)
          test_priors_list[i] = 1./(float)noutputs;
    } else {
        if(test_priors_list[0] != (float)noutputs)
          fatal_error("%s: %s %.0f, should be %d",
              "test_priors_list length is", test_priors_list[0],
              noutputs);
        total = 0.;
        for(i = 1; i <= noutputs; i++) total += test_priors_list[i];
        for(i = 1; i <= noutputs; i++) test_priors_list[i] /= total;
    }
    train_priors_list = *(float **)generic_param(model, "class_cnt");
    total = *(float *)generic_param(model, "total_npatterns");
    for(i = 0; i < noutputs; i++) train_priors_list[i] /= total;
    }
}

void
print_comments(void)
{
    printf("/* %s */\n", param_fname);
    print_norm_comments();
}


void
print_declarations(void)
{
    printf("\n");
    printf("#include <stdio.h>\n");
    printf("#include <math.h>\n");
    printf("\n");
    printf("\n /* macro definitions */\n");
    printf("#ifndef SQR(x)\n");
    printf("#define SQR(x) ((x) * (x))\n");
    printf("#endif\n");
    printf("\n");
    printf("#ifndef M_LOG10E\n");
    printf("#define M_LOG10E 0.43429448190325182765\n");
    printf("#endif\n");
    printf("\n");
    printf("#ifndef LOG_2PI\n");
    printf("#define LOG_2PI 0.798179868358\n");
    printf("#endif\n");
    printf("\n");

    printf("\n /* function declarations */\n");
    printf("extern int %s(/* float *,float * */);\n", classify_name);
    printf("extern int %s(/* float * */);\n", normalize_name);

    printf("\n\n");
}

void
print_macros(void)
{
    print_norm_macros(nraw_name, ninputs_name);
    printf("\n");
}

void
clear_macros(void)
{
    clear_norm_macros();
    printf("\n");
}

void
print_classify_code(int model)
{
    printf("int %s (inputs, outputs)\n", classify_name);
    printf("   float *inputs,*outputs;\n");
    printf("{\n");
    printf("  int i, j, best;\n");
    if(adjust_test){
    print_vector_initialization("new_priors_list", &test_priors_list[1],
                    noutputs, noutputs_name);
    print_vector_initialization("old_priors_list", train_priors_list,
                    noutputs, noutputs_name);
    printf("  float total;\n");
    }
    
    printf("  float y[%s];\n", noutputs_name);
    /* copy and normlize inputs */
    print_copy_inputs();

    printf("\n  /* normalize loaded data */\n");
    printf("  %s(x);\n", normalize_name);

    printf("\n /* calculate final outputs */\n");
    printf("  for(i = 0; i < %s; i++){\n", noutputs_name);
    printf("    y[i] = (float) rand();\n");
    printf("  } /* for i */\n");

    printf("\n  /* copy outputs */\n");
    printf("  for(i = 0; i < %s; i++)\n", noutputs_name);
    printf("    outputs[i] = (float) y[i];\n");

    if(adjust_test){
    printf("\n  /* scale outputs by priors */\n");
    printf("  for(i = 0; i < %s; i++){\n", noutputs_name);
    printf("    outputs[i] *= new_priors_list[i];\n");
    printf("    outputs[i] /= old_priors_list[i];\n");
    printf("  }\n");
    printf("  /* rescale the new outputs to sum to 1 again */\n");
    printf("  total = 0;\n");
    printf("  for(i = 0; i < %s; i++) total += outputs[i];\n",
           noutputs_name);
    printf("  for(i = 0; i < %s; i++) outputs[i] /= total;\n",
           noutputs_name);
    }
    printf("\n  /* find highest output */\n");
    printf("  for(best = n = 0; n < %s; n++)\n", noutputs_name);
    printf("    if(y[best] < y[n]) best = n;\n");
    printf("  return(best);\n");
    printf("}\n");    
}
