/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
                                /* gmix2c.c */
/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#define GEN2CVARS
#include <gen_2c_args.h>
#include "gmix_pack.h"

/* decleration of static (global) variables to be used in most functions */
/* they are initialized by function call get_param() */

/* gmix specific variables */
static int full_flag, grand_flag, tied_flag;
static int ngauss, *ngauss_list;
static char ngauss_name[64];
static char logadd_name[64];

/* function declerations */
int read_model_file(char **argv);
void free_model(int model);

void get_params(int model);
void get_general_params(int model);
void get_specific_params(int model);

void set_priors(int model);

void print_comments(void);
void print_declarations(void);
void print_macros(void);
void print_classify_code(int model);
void print_logadd_code(void);
void clear_macros(void);

int
main(int argc, char **argv)
{
    int model;

    /* process arguments */
    argc = nnargs(argc, argv, argtab2c, TABSIZE2C);

    /* check arguments */
    if(strlen(class_suff) > 53){
    fatal_error("suffix (%s) is too long.  use at most 50 characters",
            class_suff);
    }

    /* set up subroutine names and macro names */
    strcpy(classify_name, "classify");
    strcpy(normalize_name, "normalize");
    strcpy(logadd_name, "logadd");
    strcpy(nraw_name, "NRAW");
    strcpy(ninputs_name, "NINPUTS");
    strcpy(noutputs_name, "NCLASSES");
    strcpy(ngauss_name, "NGAUSS");
    if(class_suff[0] != '\0'){
        strcpy(class_exten, "_");
    strcat(class_exten, class_suff);
    strcat(classify_name, class_exten);
    strcat(normalize_name, class_exten);
    strcat(logadd_name, class_exten);
    strcat(nraw_name, class_exten);
    strcat(ninputs_name, class_exten);
    strcat(noutputs_name, class_exten);
    strcat(ngauss_name, class_exten);
    } else {
      strcpy(class_exten, "");
    }

    /* read in parameters */
    model = read_model_file(argv);
    get_params(model);

    set_priors(model);

    /* print the source file */
    print_comments();
    print_declarations();
    print_macros();
    print_classify_code(model);
    print_normalize_code(normalize_name);
    print_logadd_code();
    clear_macros();

    free_model(model);
    exit(0);
}

int
read_model_file(char **argv)
{
    int   model, norm_model, normalization;
    FILE *param_file;

    param_file = nn_fopen(param_fname, "r", "for read of model file");
    norm_model = norm_read_paramfile(param_file, &normalization);
    if (norm_model < 0)
      fatal_error("%s: read error %d %s %s", argv[0], norm_model,
          "on normalization parameter file", param_fname);
    
    model = gmix_read_model(param_file);
    if (model < 0)
      fatal_error("%s: read error %d on model file %s",
          argv[0], model, param_fname);
    nn_fclose(param_file, param_fname, "for read of model file");

    return((int) model);
}


void
free_model(int model)
{
  norm_free();
  gmix_free(model);
}

void
get_params(int model)
{
    get_general_params(model);
    get_norm_params(model, ninputs, features);
    get_specific_params(model);
}

void
get_specific_params(int model)
{
    int i;

    full_flag = *(int *)gmix_param(model, "full_flag", 0, 0);
    grand_flag = *(int *)gmix_param(model, "grand_flag", 0, 0);
    tied_flag = *(int *)gmix_param(model, "tied_flag", 0, 0);

    ngauss_list = allocate_ivector(noutputs);
    if(!tied_flag){
    for(i = 0; i < noutputs; i++)
      ngauss_list[i] = *(int *)gmix_param(model, "class_ngauss", i, 0);
    ngauss = ngauss_list[0];
    /* find the most gaussians in a class */
    for(i = 1; i < noutputs; i++)
      if(ngauss < ngauss_list[i]) ngauss = ngauss_list[i];
    } else {
    ngauss = *(int *)gmix_param(model, "ngauss", 0, 0);
    }
}

void get_general_params(int model)
{
    ninputs = *(int *)gmix_param(model, "ninputs", 0, 0);
    noutputs = *(int *)gmix_param(model, "noutputs", 0, 0);
    features = *(int **)gmix_param(model, "features", 0, 0);
}

void
set_priors(int model)
{
    int i;
    float total;
    float *test_priors_store;

    if(adjust_test){
        test_priors_store = allocate_vector(noutputs);
    if(test_uniform_priors){
        if(test_priors_list != NULL)
          deallocate_vector(test_priors_list);
        test_priors_list = allocate_vector(noutputs+1);
        test_priors_list[0] = noutputs;
        for(i = 1; i <= noutputs; i++){
        test_priors_list[i] = 1./(float)noutputs;
        test_priors_list[i] = log10(test_priors_list[i]);
        test_priors_store[i-1] = test_priors_list[i];
        }
    } else {
        if(test_priors_list[0] != (float)noutputs)
          fatal_error("%s: %s %.0f, should be %d",
              "test_priors_list length is", test_priors_list[0],
              noutputs);
        total = 0.;
        for(i = 1; i <= noutputs; i++) total += test_priors_list[i];
        for(i = 1; i <= noutputs; i++){
        test_priors_list[i] /= total;
        test_priors_list[i] = log10(test_priors_list[i]);
        test_priors_store[i-1] = test_priors_list[i];
        }
    }
    deallocate_vector(test_priors_list);
    test_priors_list = test_priors_store;
    } else {
        if (test_priors_list != NULL) deallocate_vector(test_priors_list);
    test_priors_list = allocate_vector(noutputs);
    total = *(float *)gmix_param(model, "total_trials", 0, 0);
    for(i = 0; i < noutputs; i++){
        test_priors_list[i] =
          *(float *)gmix_param(model, "class_trials", i, 0);
        test_priors_list[i] /= total;
        test_priors_list[i] = log10(test_priors_list[i]);
    }
    }
}

void
print_comments(void)
{
    printf("/* %s */\n", param_fname);
    if(tied_flag) printf("/* Tied Mixtures */\n");
    else printf("/* One Mixture Per Class */\n");
    printf("/* ");
    if(grand_flag == 2) printf("Grand Overall ");
    else if (grand_flag) printf("Grand Per Class ");
    else printf("Per Mixture ");
    if(full_flag) printf("Full Covariance");
    else printf("Diagonal Variance");
    printf("*/\n");
    print_norm_comments();
}

void
print_declarations(void)
{
    printf("\n");
    printf("#include <math.h>\n");
    printf("#include <values.h>\n");
    printf("\n");
    printf("\n /* macro definitions */\n");
    printf("#ifndef SQR(x)\n");
    printf("#define SQR(x) ((x) * (x))\n");
    printf("#endif\n");
    printf("\n");
    printf("#ifndef M_LOG10E\n");
    printf("#define M_LOG10E 0.43429448190325182765\n");
    printf("#endif\n");
    printf("\n");
    printf("#ifndef M_LN10\n");
    printf("#define M_LN10 2.30258509299404568402\n");
    printf("#endif\n");
    printf("\n");
    printf("#ifndef LOG_2PI\n");
    printf("#define LOG_2PI 0.798179868358\n");
    printf("#endif\n");
    printf("\n");

    printf("\n /* function declarations */\n");
    printf("extern int %s(/* float *,float * */);\n", classify_name);
    printf("extern int %s(/* float * */);\n", normalize_name);
    printf("static float %s(/* float,float */);\n", logadd_name);
    printf("\n\n");
}

void
print_macros(void)
{
    print_norm_macros(nraw_name, ninputs_name);
    printf("#define %s %d\n", noutputs_name, noutputs);
    printf("#define %s %d\n", ngauss_name, ngauss);
    printf("\n");
}    

void
clear_macros(void)
{
    clear_norm_macros();
    printf("#undef %s\n", noutputs_name);
    printf("#undef %s\n", ngauss_name);
    printf("\n");
}    

void
print_classify_code(int model)
{
    int   i, j, k;
    float *means, **covar, *var, log_det, log_weight;

    printf("int %s (inputs, outputs)\n", classify_name);
    printf("   float *inputs,*outputs;\n");
    printf("{\n");
    printf("  int best, class, gauss");
    if(full_flag) printf(",j");
    printf(";\n");
    printf("  float y[%s], q", noutputs_name);
    if(full_flag) printf(",p");
    printf(";\n");
    
    if(tied_flag){
    /* tied means */
    printf("  static float means[%s][%s] = {\n", ngauss_name, ninputs_name);
    for(i = 0; i < ngauss; i++){
        means = *(float **)gmix_param(model, "tied_means", 0, i);
        printf("{");
        print_vector(means, ninputs);
        printf("},\n");
    }
    printf("};\n");

    /* tied variances */
    if(grand_flag){ /* tied grand */
        if(full_flag){
        covar = *(float ***)gmix_param(model, "full_var", 0, 0);
        print_matrix_initialization("inv_var", covar, ninputs, ninputs,
                        ninputs_name, ninputs_name);
        } else {
        var = *(float **)gmix_param(model, "diag_var", 0, 0);
        print_vector_initialization("inv_var", var, ninputs, ninputs_name);
        }
    } else { /* tied, not grand */
        if(full_flag){
        printf("  static float inv_var[%s][%s][%s] = {\n",
               ngauss_name, ninputs_name, ninputs_name);
        for(i = 0; i < ngauss; i++){
            printf("\t/* gaussian %d */{\n", i);
            covar = *(float ***)gmix_param(model, "tied_full_var",
                          0, i);
            for(j = 0; j < ninputs; j++){
            printf("\t {");
            print_vector(covar[j], ninputs);
            printf("},\n");
            }
            printf("},\n");
        }
        printf("};\n");
        } else {
        printf("  static float inv_var[%s][%s] = {\n", ngauss_name, ninputs_name);
        for(i = 0; i < ngauss; i++){
            var = *(float **)gmix_param(model, "tied_diag_var",
                        0, i);
            printf("\t{");
            print_vector(var, ninputs);
            printf("},\n");
        }
        printf("};\n");
        }
    }
    /* tied determinants */
    if(grand_flag){
        log_det = *(float *)gmix_param(model, "determinant", 0, 0);
        printf("  float log_determinant = %14.6e;\n", log_det);
    } else {
        printf("  static float log_determinant[%s] = {\n", ngauss_name);
        for(i = 0; i < ngauss; i++){
        log_det = *(float *)gmix_param(model, "tied_determinant",
                           0, i);
        if(i && !(i%NFLOATS_PER_LINE))printf("\n  ");
        printf("%14.6e, ", log_det);
        }
        printf("};\n");
    }
    /* tied weights */
    printf("  static float weights[%s][%s] = {\n", noutputs_name, ngauss_name);
    for(i = 0; i < noutputs; i++){
        printf("{");
        for(j = 0; j < ngauss; j++){
        log_weight = *(float *)gmix_param(model, "log_weight", i, j);
        if(j && !(j%NFLOATS_PER_LINE))printf("\n  ");
        printf("%14.6e, ", log_weight);
        }
        printf("},\n");
    }
    printf("};\n");
    } else { /* untied */
    printf("/* leave space for the maximum number of gaussians per class */\n");
    printf("  struct gmix {\n");
    printf("    int ngauss;\n");
    printf("    float means[%s][%s];\n", ngauss_name, ninputs_name);
    printf("    float weights[%s];\n", ngauss_name);
    switch(grand_flag){
      case 2:
        break;
      case 0:
        if(full_flag) printf("    float inv_vars[%s][%s][%s];\n",
                 ngauss_name,ninputs_name, ninputs_name);
        else printf("    float inv_vars[%s][%s];\n", ngauss_name, ninputs_name);
        printf("    float log_determinant[%s];\n",ngauss_name);
        break;
      default:
        if(full_flag){
        printf("    float inv_vars[%s][%s];\n", ninputs_name, ninputs_name);
        } else {
        printf("    float inv_vars[%s];\n", ninputs_name);
        }
        printf("    float log_determinant;\n");
        break;
    }
    printf("};\n");
    printf("  static struct gmix class_gmix[%s] = {\n", noutputs_name);
    for(i = 0; i < noutputs; i++){
        printf("{/* class %d */ %d,\n", i, ngauss_list[i]);
        /* print means */
        printf("/* means (class %d) */{\n", i);
        for(j = 0; j < ngauss_list[i]; j++){
        printf("{");
        means = *(float **)gmix_param(model, "class_means", i, j);
        print_vector(means, ninputs);
        printf("},\n");
        }
        printf("},\n");
        /* print weights */
        printf("/* weights (class %d) */{", i);
        for(j = 0; j < ngauss_list[i]; j++){
        log_weight = *(float *)gmix_param(model, "log_weight", i, j);
        if(j && !(j % NFLOATS_PER_LINE)) printf("\n  ");
        printf("%14.6e, ", log_weight);
        }
        printf("},\n");
        /* print variances and determinants */
        switch(grand_flag){
          case 2:
        break;
          case 0:
        if(full_flag){
            printf("/* inverse variance (class %d) */{", i);
            for(j = 0; j < ngauss_list[i]; j++){
            printf("/* gaussian %d*/{\n", j);
            covar =
              *(float ***)gmix_param(model, "per_class_full_var",
                         i, j);
            for(k = 0; k < ninputs; k++){
                printf("\t{");
                print_vector(covar[k], ninputs);
                printf("},\n");
            }
            printf("},\n");
            }
            printf("},\n");
        } else {
            printf("/* inverse variance (class %d) */{\n", i);
            for(j = 0; j < ngauss_list[i]; j++){
            var =
              *(float **)gmix_param(model, "per_class_diag_var",
                        i, j);
            printf("\t{");
            print_vector(var, ninputs);
            printf("},\n");
            }
            printf("},\n");
        }
        /* per component determinant */
        printf("/* log_determinant (class %d) */{", i);
        for(j = 0; j < ngauss_list[i]; j++){
            log_det =
              *(float *)gmix_param(model, "per_class_determinant",
                       i, j);
            if(j && !(j % NFLOATS_PER_LINE)) printf("\n  ");
            printf("%14.6e, ", log_det);
        }
        printf("},\n");
        break;
          default:
        if(full_flag){
            covar = *(float ***)gmix_param(model, "class_full_var",
                           i, 0);
            printf("/* inverse variance (class %d) */{", i);
            for(j = 0; j < ninputs; j++){
            printf("{");
            print_vector(covar[j], ninputs);
            printf("},\n");
            }
            printf("},\n");
        } else {
            var = *(float **)gmix_param(model, "class_diag_var", i, 0);
            printf("/* inverse variance (class %d) */{", i);
            print_vector(var, ninputs);
            printf("},\n");
        }
        printf("/* log_determinant (class %d) */", i);
        log_det =
          *(float *)gmix_param(model, "class_determinant", i, 0);
        printf("%14.6e", log_det);
        break;
        }
        printf("},/* end of class %d */\n", i);
    }
    printf("};\n");
    if(grand_flag == 2){ /* grand overall */
        if(full_flag){
        covar = *(float ***)gmix_param(model, "full_var", 0, 0);
        print_matrix_initialization("inv_var", covar, ninputs, ninputs,
                        ninputs_name, ninputs_name);
        } else {
        var = *(float **)gmix_param(model, "diag_var", 0, 0);
        print_vector_initialization("inv_var", var, ninputs, ninputs_name);
        }
        log_det = *(float *)gmix_param(model, "determinant", 0, 0);
        printf("  float log_determinant = %14.6e;\n", log_det);
    }
    }
    printf("\n");
    /* class priors */
    print_vector_initialization("log_class_priors", test_priors_list,
                noutputs, noutputs_name);
    
    /* copy and normlize inputs */
    print_copy_inputs();
    printf("\n  /* normalize loaded data */\n");
    printf("  %s(x);\n", normalize_name);

    printf("  /* calculate outputs for each class */\n");
    printf("  for(class = 0; class < %s; class++){\n", noutputs_name);
    printf("    y[class] = -(MAXFLOAT);\n");
    printf("    /* calculate outputs for each gaussian */\n");
    printf("    for(gauss = 0; gauss <");
    if(tied_flag)
      printf(" %s;", ngauss_name);
    else
      printf(" class_gmix[class].ngauss;");
    printf(" gauss++){\n");
    printf("      /* (x-mean)T * Inverse_Variance * (x-mean) */\n");
    printf("      q = 0.;\n");
    if(full_flag){
    printf("      for(i = 0; i < %s; i++){\n", ninputs_name);
    printf("        p = 0.;\n");
    printf("        for(j = 0; j < %s; j++){\n", ninputs_name);
    if(tied_flag)
      printf("          p += (x[j] - means[gauss][j])*");
    else
      printf("          p += (x[j] - class_gmix[class].means[gauss][j])*");
    if(tied_flag) {
      if(grand_flag) printf("inv_var[i][j]");
      else printf("inv_var[gauss][i][j]");
    } else {
      if(grand_flag == 2) printf("inv_var[i][j]");
      else if(grand_flag) printf("class_gmix[class].inv_vars[i][j]");
      else printf("class_gmix[class].inv_vars[gauss][i][j]");
    }
    printf(";\n");
    printf("        }\n");
    if(tied_flag) printf("        q += (x[i] - means[gauss][i])");
    else printf("        q += (x[i] - class_gmix[class].means[gauss][i])");
    printf(" *p;\n");
    printf("      }\n");
    } else { /* diagonal covariance */
    printf("      for(i = 0; i < %s; i++){\n", ninputs_name);
    if(tied_flag) printf("        q += SQR(x[i] - means[gauss][i])*\n");
    else printf("        q += SQR(x[i] - class_gmix[class].means[gauss][i])*\n");
    if(tied_flag) {
      if(grand_flag) printf("          inv_var[i]");
      else printf("          inv_var[gauss][i]");
    } else {
      if(grand_flag == 2) printf("          inv_var[i]");
      else if(grand_flag) printf("          class_gmix[class].inv_vars[i]");
      else printf("          class_gmix[class].inv_vars[gauss][i]");
    }
    printf(";\n");
    printf("      }\n");
    }
    printf("      /* finish calculation of gaussian output */\n");
    printf("      q *= 0.5 * M_LOG10E;\n");
    if(tied_flag) {
      if(grand_flag) printf("      q += log_determinant");
      else printf("      q += log_determinant[gauss]");
    } else {
      if(grand_flag == 2) printf("      q += log_determinant");
      else if(grand_flag) printf("      q += class_gmix[class].log_determinant");
      else printf("      q += class_gmix[class].log_determinant[gauss]");
    }
    printf("*0.5 + %s * 0.5 * LOG_2PI;\n", ninputs_name);
    printf("      q = -q;\n");
    printf("      /* add log weight and take output sum */\n");
    if(tied_flag) {
      printf("      q += weights[class][gauss];\n");
    } else {
      printf("      q += class_gmix[class].weights[gauss];\n");
    }
    printf("      y[class] = %s(y[class], q);\n", logadd_name);
    printf("    }\n");
    printf("    y[class] += log_class_priors[class];\n");
    printf("  }\n");
#define LOG_OUTPUTS
#undef LOG_OUTPUTS
#ifdef LOG_OUTPUTS
    printf("\n  /* copy log outputs  */\n");
    printf("  for(class = 0; class < %s; class++)\n", noutputs_name);
    printf("    outputs[class] = y[class];\n");
#else
    printf("\n  /* copy outputs and make linear */\n");
    printf("  for(class = 0; class < %s; class++)\n", noutputs_name);
    printf("    outputs[class] = exp(M_LN10 * (double)y[class]);\n");
#endif
    printf("\n  /* find highest output */\n");
    printf("  for(best = class = 0; class < %s; class++)\n", noutputs_name);
    printf("    if(y[best] < y[class]) best = class;\n");
    printf("  return(best);\n");
    printf("}\n");
}

void
print_logadd_code(void)
{
    printf("\n\n");
    printf("static float %s(x, y)\n", logadd_name);
    printf("   register float x, y;\n");
    printf("{\n");
    printf("  register double b;\n");
    printf("  if(x < y)    /*** exchange if x < y ***/\n");
    printf("    b = (double) x, x = (double) y, y = (double) b;\n");
    printf("  b = (double) x - y;\n");
    printf("  if(b > 16.0) /*** keep up to 16 decimal digits precision ***/\n");
    printf("    return(x);\n");
    printf("  else\n");
    printf("    return(y + (float)log10(1.0 + exp(M_LN10*b)));\n");
    printf("}\n");
}
