/***--------------------------------------------------------------------------
* NAME:         gmix_pack.c
* PURPOSE:      Functions specific to gmix.c
*
* SEE ALSO: gmix.c gmix_main.c
*
* GLOBALS:
*
* DESCRIPTION:
*    Functions in this file
*
* ALGORITHM:
* REFERENCE:
*
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/gmix/gmix_seq_pack.c,v 11.3 2001/08/23 20:11:53 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/gmix/gmix_seq_pack.c,v $
* CREATED: 12-Dec-90 lck                             MIT LINCOLN LABORATORY
*--------------------------------------------------------------------------*/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#include <math.h>
#include "gmix_seq_pack.h"
#include "gmix_pack.h"

                /* constant definitions */
#define MODEL_STORE_SIZE 10
#define PEQ(y) (0==strcmp(param_name,y))

typedef struct {
    int    noutputs;    /* number of elements in patterns */
    float *seq_sums;
    int    gmix_model;
} GMIX_SEQ_PARAMS;

                /* Internal Globals */
static GMIX_SEQ_PARAMS **gmixpp;/* array of parameters for each model */
static int      max_models = 0; /* current length of gmixpp */

#define SMALLEST 1.e-30

/*****************************************************************************
* NAME: gmix_seq_build(noutputs)
*
* DESCRIPTION:
*       Allocate storage for model parameters.
*
*****************************************************************************/
int gmix_seq_build(int noutputs)
{
    GMIX_SEQ_PARAMS *gmpp;  /* temporary pointer to gmixpp[] */
    register int    new_model;

    new_model = model_get_number((void ***)&gmixpp,
                 &max_models, sizeof(GMIX_SEQ_PARAMS));

    gmpp = gmixpp[new_model];

    /* set mode flags */
    gmpp->noutputs = noutputs;

    /* allocate space for vectors */
    gmpp->seq_sums = allocate_vector(noutputs);

    return new_model;
}
/*****************************************************************************
* NAME: gmix_seq_init(model)
* DESCRIPTION:
*       Initialize model parameters not set in build
*
* EXAMPLE:
*
*****************************************************************************/
void gmix_seq_init(int model, int gmix_model)
{
    int             i;
    GMIX_SEQ_PARAMS *gmpp;  /* temporary pointer to gmixpp[] */

    gmpp = gmixpp[model];

    gmpp->gmix_model = gmix_model;
    for(i = 0; i < gmpp->noutputs; i++) gmpp->seq_sums[i] = 0.;
}

/*****************************************************************************
* NAME: gmix_seq_free()
* DESCRIPTION:
*       Deallocate storage used in a model
*
* EXAMPLE:
*
*****************************************************************************/
void
gmix_seq_free(int model)
{
    GMIX_SEQ_PARAMS *gmpp;  /* temporary pointer to gmixpp[] */

    gmpp = gmixpp[model];
    /* free vectors and matrices */
    deallocate_vector(gmpp->seq_sums);

    /* free structure itself */
    nn_free((char *) gmpp, "for gmix parameter structure");
    gmixpp[model] = NULL;
}

/************************************************************************
* NAME: gmix_seq_param(model, param_name)
* DESCRIPTION:
*       Takes model number and parameter name.
*       Returns a pointer to the named parameter.
*
*************************************************************************/

void * gmix_seq_param(int model, char *param_name)
{
    void           *param_address;
    GMIX_SEQ_PARAMS *gmpp;  /* temporary pointer to gmixpp[] */

    gmpp = gmixpp[model];
    if (PEQ("seq_sums"))
      param_address = &gmpp->seq_sums;
    else {
    warning("gmix_seq_param: Parameter %s not defined", param_name);
    param_address = NULL;
    }
    return param_address;
}

void
gmix_seq_batch_classify(int model, float *inputs)
{
    int i;
    GMIX_SEQ_PARAMS *gmpp;  /* temporary pointer to gmixpp[] */
    float *outputs;

    gmpp = gmixpp[model];
    gmix_eval(gmpp->gmix_model, inputs);
    outputs = *(float **)gmix_param(gmpp->gmix_model, "outputs", 0, 0);
    /* clip_floor should be in here somewhere */
    /* assumes log outputs. must be set by driver currently */
    for(i = 0; i < gmpp->noutputs; i++){
    gmpp->seq_sums[i] += outputs[i];
    }
}

int
gmix_seq_update_classify(int model)
{
    GMIX_SEQ_PARAMS *gmpp;
    gmpp = gmixpp[model];
    return(max_pick(gmpp->seq_sums, gmpp->noutputs));
}
    
