/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
                                /* histogram2c.c */
/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <math.h>
#define histogram_pack_h_internal_types
#include "histogram_pack.h"
#define GEN2CVARS
#include <gen_2c_args.h>

static int use_gauss;

static ARG argtab2[] = 
{
    {"use_gauss", FLAG, &use_gauss,
       "Use Gaussian outputs for outlyers, default: allow unknown class"},
};
#define TABSIZE2 (sizeof(argtab2)/sizeof(ARG))

/* variables specific to histogram */
static char *nbins_name ="NBINS";
static int nbins_max = -1;
static int *nbins;
static int grand_bins;

/* function declerations */
int read_model_file(char **argv);
void free_model(int model);

void get_params(int model);
void get_general_params(int model);
void get_specific_params(int model);

void set_priors(int model);

void print_comments(void);
void print_declarations(void);
void print_macros(void);
void print_classify_code(int model);
void clear_macros(void);

int
main(int argc, char **argv)
{
    int model;
    ARG *argtab;
    int tabsize;

    /* process arguments */
    nnargs_combine_argtabs(argtab2c, TABSIZE2C, argtab2, TABSIZE2,
			   &argtab, &tabsize);
    argc = nnargs(argc, argv, argtab, tabsize);

    /* check arguments */
    if(strlen(class_suff) > 53){
	fatal_error("suffix (%s) is too long.  use at most 50 characters",
		    class_suff);
    }

    /* set up subroutine names */
    strcpy(classify_name, "classify");
    strcpy(normalize_name, "normalize");
    strcpy(nraw_name, "NRAW");
    strcpy(ninputs_name, "NINPUTS");
    strcpy(noutputs_name, "NCLASSES");
    if(class_suff[0] != '\0'){
        strcpy(class_exten, "_");
	strcat(class_exten, class_suff);
	strcat(classify_name, class_exten);
	strcat(normalize_name, class_exten);
	strcat(nraw_name, class_exten);
	strcat(ninputs_name, class_exten);
	strcat(noutputs_name, class_exten);
    } else {
      strcpy(class_exten, "");
    }


    /* read in parameters */
    model = read_model_file(argv);
    get_params(model);

    set_priors(model);

    /* print the source file */
    print_comments();
    print_declarations();
    print_macros();
    print_classify_code(model);
    print_normalize_code(normalize_name);
    clear_macros();

    nnargs_free_argtab(argtab, tabsize);
    free_model(model);
    exit(0);
}

int
read_model_file(char **argv)
{
    int   model, rv, normalization;
    int	  nraw;
    FILE *param_file;

    param_file = nn_fopen(param_fname, "r", "for read of model file");
    rv = norm_read_paramfile(param_file, &normalization);
    if (rv < 0)
      fatal_error("%s: read error %d %s %s", argv[0], rv,
		  "on normalization parameter file", param_fname);
    rv = feature_read_paramfile(param_file, &features, &nraw, &ninputs);
    if (rv < 0)
      fatal_error("%s: read error %d %s %s", argv[0], rv,
		  "on feature selection parameters in file", param_fname);
    
    model = histogram_read_model(param_file);
    if (model < 0)
      fatal_error("%s: read error %d on model file %s",
		  argv[0], model, param_fname);
    nn_fclose(param_file, param_fname, "for read of model file");

    return((int) model);
}


void
free_model(int model)
{
  norm_free();
  histogram_free(model);
}

void
get_params(int model)
{
    get_general_params(model);
    get_norm_params(model, ninputs, features);
    get_specific_params(model);
}


void
get_specific_params(int model)
{
    int c;

    /* get histogram specific parameters */
    nbins = *(int **)histogram_param(model, "nbins", 0, 0);
    for(c = 0; c < noutputs; c++){
	nbins_max = MAX(nbins_max, nbins[c]);
    }
    grand_bins = *(int *)histogram_param(model, "grand", 0, 0);
}


void
get_general_params(int model)
{
    /* get general parameters */
    noutputs = *(int *)histogram_param(model, "nclasses", 0, 0);
}

void
set_priors(int model)
{
    int i;
    float total;
    float *test_priors_store;

    if(adjust_test){
        test_priors_store = allocate_vector(noutputs);
	if(test_uniform_priors){
	    if(test_priors_list != NULL)
	      deallocate_vector(test_priors_list);
	    test_priors_list = allocate_vector(noutputs+1);
	    test_priors_list[0] = noutputs;
	    for(i = 1; i <= noutputs; i++){
		test_priors_list[i] = 1./(float)noutputs;
		test_priors_list[i] = log(test_priors_list[i]);
		test_priors_store[i-1] = test_priors_list[i];
	    }
	} else {
	    if(test_priors_list[0] != (float)noutputs)
	      fatal_error("%s: %s %.0f, should be %d",
			  "test_priors_list length is", test_priors_list[0],
			  noutputs);
	    total = 0.;
	    for(i = 1; i <= noutputs; i++) total += test_priors_list[i];
	    for(i = 1; i <= noutputs; i++){
		test_priors_list[i] /= total;
		test_priors_list[i] = log(test_priors_list[i]);
		test_priors_store[i-1] = test_priors_list[i];
	    }
	}
	deallocate_vector(test_priors_list);
	test_priors_list = test_priors_store;
    } else {
        if (test_priors_list != NULL) deallocate_vector(test_priors_list);
	test_priors_store = allocate_vector(noutputs);
	test_priors_list = *(float **)histogram_param(model, "class_cnt", 0, 0);
	total = *(float *)histogram_param(model, "total_npatterns", 0, 0);
	for(i = 0; i < noutputs; i++){
	    test_priors_store[i] = test_priors_list[i];
	    test_priors_store[i] /= total;
	    test_priors_store[i] = log(test_priors_store[i]);
	}
	test_priors_list = test_priors_store;
    }
}

void
print_comments(void)
{
    printf("/* %s */\n", param_fname);
    switch(grand_bins){
      case PER_CLASS:
	printf("/* Different Bins for Classes and Inputs */\n");
	break;
      case GRAND_INPUT:
	printf("/* Bins for each Input, shared over classes */\n");
	break;
      case USER_BIN:
	printf("/* User Supplied Bins shared over inputs and classes */\n");
	break;
    }
    if(use_gauss)
      printf("/* Use Gauss outputs to decide class when outside ALL bins */\n");
    else
      printf("/* Return UNKNOWN_CLASS when input is outside ALL bins */\n");
    print_norm_comments();
}


void
print_declarations(void)
{
    printf("\n");
    printf("#include <stdio.h>\n");
    printf("#include <math.h>\n");
    printf("\n");
    printf("\n /* macro definitions */\n");
    printf("#ifndef LN_2PI\n");
    printf("#define LN_2PI 1.83787706640935\n");
    printf("#endif\n");
    printf("\n");

    printf("\n /* function declarations */\n");
    printf("extern int %s(/* float *,float * */);\n", classify_name);
    printf("extern int %s(/* float * */);\n", normalize_name);

    printf("\n\n");
}

void
print_macros(void)
{
    print_norm_macros(nraw_name, ninputs_name);
    printf("#define %s %d\n", noutputs_name, noutputs);
    printf("#define %s %d\n", nbins_name, nbins_max);
    printf("#define UNKNOWN_CLASS %d\n", UNKNOWN_CLASS);
    printf("\n");
}

void
clear_macros(void)
{
    clear_norm_macros();
    printf("#undef %s\n", noutputs_name);
    printf("#undef %s\n", nbins_name);
    printf("#undef UNKNOWN_CLASS\n");
    printf("\n");
}

void
print_classify_code(int model)
{
    int c, b, i;
    HISTOGRAM_BIN *bin;
    HISTOGRAM_PROB *prob;
    float *temp_vector;

    printf("int %s (inputs, outputs)\n", classify_name);
    printf("   float *inputs,*outputs;\n");
    printf("{\n");
    printf("  int c, b, best, in_bin = 0, missed_bin, nb;\n");
    print_ivector_initialization("nbins", nbins, noutputs, noutputs_name);
    switch(grand_bins){
      case PER_CLASS:
	printf("  static float bin_ends[%s][%s][%s+1] ={\n", noutputs_name,
		ninputs_name, nbins_name);
	for(c = 0; c < noutputs; c++){
	    printf("\t{");
	    b = nbins[c];
	    for(i = 0; i < ninputs; i++){
		printf("\t{");
		bin = (HISTOGRAM_BIN *) histogram_param(model, "bin", c, i);
		print_vector(bin->bin_ends, b+1);
		printf("},\n");
	    }
	    printf("\t},\n");
	}
	printf(" };\n");
	printf("  float *bins;\n");
	break;
      case GRAND_INPUT:
	printf("  static float bin_ends[%s][%s+1] = {\n", ninputs_name, nbins_name);
	b = nbins[0];
	for(i = 0; i < ninputs; i++){
	    printf("\t{");
	    bin = (HISTOGRAM_BIN *) histogram_param(model, "bin", 0, i);
	    print_vector(bin->bin_ends, b+1);
	    printf("},\n");
	}
	printf("};\n");
	printf("  float *bins;\n");
	break;

      case USER_BIN:
	b = nbins[0];
	bin = (HISTOGRAM_BIN *) histogram_param(model, "bin", 0, 0);
	printf("  static float %s[%s+1] = {\n\t", "bins", nbins_name);
	print_vector(bin->bin_ends, b+1);
	printf("  };\n");
	break;
    }
    printf("  static float log_probs[%s][%s][%s] ={\n",
	   noutputs_name, ninputs_name, nbins_name);
    temp_vector = allocate_vector(nbins_max);
    for(c = 0; c < noutputs; c++){
	printf("\t{");
	for(i = 0; i < ninputs; i++){
	    printf("\t{");
	    prob = (HISTOGRAM_PROB *) histogram_param(model, "prob", c, i);
	    for(b = 0; b < nbins[c]; b++){
		temp_vector[b] = log(prob->bin_prob[b]);
	    }
	    print_vector(temp_vector, nbins[c]);
	    printf("},\n");
	}
	printf("\t},\n");
    }
    printf("\t};\n");
    if(use_gauss){
	temp_vector = reallocate_vector(temp_vector, nbins_max, ninputs);
	printf("  static float mean[%s][%s] = {\n", noutputs_name, ninputs_name);
	for(c = 0; c < noutputs; c++){
	    printf("\t{");
	    for(i = 0; i < ninputs; i++){
		prob = (HISTOGRAM_PROB *) histogram_param(model, "prob", c, i);
		temp_vector[i] = prob->mean;
	    }
	    print_vector(temp_vector, ninputs);
	    printf("},\n");
	}
	printf("};\n");
	printf("  static float var[%s][%s] = {\n", noutputs_name, ninputs_name);
	for(c = 0; c < noutputs; c++){
	    printf("\t{");
	    for(i = 0; i < ninputs; i++){
		prob = (HISTOGRAM_PROB *) histogram_param(model, "prob", c, i);
		temp_vector[i] = prob->var;
	    }
	    print_vector(temp_vector, ninputs);
	    printf("},\n");
	}
	printf("};\n");
	printf("  static float lnvar[%s][%s] = {\n", noutputs_name, ninputs_name);
	for(c = 0; c < noutputs; c++){
	    printf("\t{");
	    for(i = 0; i < ninputs; i++){
		prob = (HISTOGRAM_PROB *) histogram_param(model, "prob", c, i);
		temp_vector[i] = log(prob->var);
	    }
	    print_vector(temp_vector, ninputs);
	    printf("},\n");
	}
	printf("};\n");
    }
    printf("  float y[%s];\n", noutputs_name);
    if(use_gauss) printf("  float gy[%s], gauss;\n", noutputs_name);
    print_vector_initialization("ln_class_priors", test_priors_list, noutputs,
				noutputs_name);
    /* copy and normlize inputs */
    print_copy_inputs();

    printf("\n  /* normalize loaded data */\n");
    printf("  %s(x);\n", normalize_name);

    if(grand_bins != PER_CLASS){
	printf("  nb = %s;\n", nbins_name);
    }
    printf("  /* find outputs for each class */\n");
    printf("  for(c = 0; c < %s; c++){\n", noutputs_name);
    printf("    y[c] = 0.;\n");
    if(use_gauss) printf("    gy[c] = 0.;\n");
    if(grand_bins == PER_CLASS) printf("    nb = nbins[c];\n");
    printf("    missed_bin = 0;\n");
    printf("    for(i = 0; i < %s; i++){\n", ninputs_name);
    if(grand_bins == PER_CLASS)
      printf("      bins = &bin_ends[c][i][0];\n");
    else if(grand_bins == GRAND_INPUT)
      printf("      bins = &bin_ends[i][0];\n");
    printf("      /* find the bin this input feature is in */\n");
    printf("      if(!missed_bin){\n");
    printf("        for(b = 0; b <= nb && x[i] > bins[b]; b++);\n");
    printf("        b--;\n");
    printf("        if( 0 <= b && b < nb){\n");
    printf("          y[c] += log_probs[c][i][b];\n");
    printf("        } else {\n");
    printf("          y[c] = 0;\n");
    printf("          missed_bin = 1;\n");
    printf("        }\n");
    printf("      }\n");
    if(use_gauss){
	printf("      /* calculate the Gaussian output if we need it */\n");
	printf("      if(!in_bin){\n");
	printf("        gauss = x[i] - mean[c][i];\n");
	printf("        gauss *= gauss;\n");
	printf("        gauss /= var[c][i];\n");
	printf("        gauss += lnvar[c][i] + LN_2PI;\n");
	printf("        gauss *= -.5;\n");
	printf("        gy[c] += gauss;\n");
	printf("      }\n");
    }
    printf("    }\n");
    printf("    if(!missed_bin) in_bin = 1;\n");
    printf("  }\n");
    if (use_gauss){
	printf("\n  /* if the pattern was in no bin, use the gauss outputs instead */\n");
	printf("  if(!in_bin){\n");
	printf("    for(c = 0; c < %s; c++){\n", noutputs_name);
	printf("      y[c] = gy[c];\n");
	printf("    }\n");
	printf("  }\n");
    } else {
	printf("\n  /* if the pattern was in no bin, set outputs to 0 and return UNKNOWN */\n");
	printf("  if(!in_bin){\n");
	printf("    for(c = 0; c < %s; c++)\n", noutputs_name);
	printf("      outputs[c] = 0;\n");
	printf("    return UNKNOWN_CLASS;\n");
	printf("  }\n");
    }
    printf("\n  /* make outputs linear and add in class priors */\n");
    printf("\n  /* and find highest output */\n");
    printf("  for(best = c = 0; c < %s; c++){\n", noutputs_name);
    printf("    if(y[c] != 0){\n");
    printf("      y[c] += ln_class_priors[c];\n");
    printf("      y[c] = exp(y[c]);\n");
    printf("    }\n");
    printf("    if(y[best] < y[c]) best = c;\n");
    printf("  }\n");
    printf("\n  /* copy outputs */\n");
    printf("  for(c = 0; c < %s; c++)\n", noutputs_name);
    printf("    outputs[c] = (float) y[c];\n");

    printf("  return(best);\n");
    printf("}\n");    

    /* clean up */
    deallocate_vector(temp_vector);
}
