/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* hyper2c.c */
/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#define GEN2CVARS
#include <gen_2c_args.h>
#include "hyper_pack.h"


/* declaration of static (global) variables to be used in most functions */
/* they are initialized by function call get_param() */

/* hyper specific variables */
static int force;
static char ncenters_name[64];
static int nexemplars;
static char l2dist_name[64];

/* function declerations */
int read_model_file(char **argv);
void free_model(int model);

void get_params(int model);
void get_general_params(int model);
void get_specific_params(int model);

void print_comments(void);
void print_declarations(void);
void print_macros(void);
void print_classify_code(int model);
void print_l2dist_code(void);
void clear_macros(void);

int
main(int argc, char **argv)
{
    int model;

    /* process arguments */
    argc = nnargs(argc, argv, argtab2c, TABSIZE2C);

    /* check arguments */
    if(strlen(class_suff) > 53){
    fatal_error("suffix (%s) is too long.  use at most 50 characters",
            class_suff);
    }

    /* set up subroutine names and macro names */
    strcpy(classify_name, "classify");
    strcpy(normalize_name, "normalize");
    strcpy(l2dist_name, "l2dist");
    strcpy(nraw_name, "NRAW");
    strcpy(ninputs_name, "NINPUTS");
    strcpy(noutputs_name, "NCLASSES");
    strcpy(ncenters_name, "NCENTERS");
    if(class_suff[0] != '\0'){
        strcpy(class_exten, "_");
    strcat(class_exten, class_suff);
    strcat(classify_name, class_exten);
    strcat(normalize_name, class_exten);
    strcat(l2dist_name, class_exten);
    strcat(nraw_name, class_exten);
    strcat(ninputs_name, class_exten);
    strcat(noutputs_name, class_exten);
    strcat(ncenters_name, class_exten);
    } else {
      strcpy(class_exten, "");
    }

    /* read in parameters */
    model = read_model_file(argv);
    get_params(model);

    if(adjust_test)
      warning("%s: This algorithm does not adjust test priors", argv[0]);

    /* print the source file */
    print_comments();
    print_declarations();
    print_macros();
    print_classify_code(model);
    print_normalize_code(normalize_name);
    print_l2dist_code();
    clear_macros();

    free_model(model);
    exit(0);
}

int
read_model_file(char **argv)
{
    int   model, norm_model, normalization, i;
    FILE *param_file;

    param_file = nn_fopen(param_fname, "r", "for read of model file");
    norm_model = norm_read_paramfile(param_file, &normalization);
    if (norm_model < 0)
      fatal_error("%s: read error %d %s %s", argv[0], norm_model,
          "on normalization parameter file", param_fname);
    
    model = hyper_read_model(param_file, &i);
    if (model < 0)
      fatal_error("%s: read error %d on model file %s",
          argv[0], model, param_fname);
    nn_fclose(param_file, param_fname, "for read of model file");

    return((int) model);
}


void
free_model(int model)
{
  norm_free();
  hyper_free(model);
}

void
get_params(int model)
{
    get_specific_params(model);
    get_general_params(model);
    get_norm_params(model, ninputs, features);
}

void
get_specific_params(int model)
{
    force = *(int *)hyper_param(model, "force");
    nexemplars = *(int *)hyper_param(model, "accum_trials");
}

void
get_general_params(int model)
{
    ninputs = *(int *)hyper_param(model, "ninputs");
    noutputs = *(int *)hyper_param(model, "nclasses");
    features = *(int **)hyper_param(model, "features");
}

void
print_comments(void)
{
    printf("/* %s */\n", param_fname);
    if(force)
      printf("/* force classification using nearest neighbor*/\n");
    else printf("/* allow unknown as output class */\n");
    print_norm_comments();
}

void
print_declarations(void)
{
    printf("\n");
    printf("#include <math.h>\n");
    printf("#include <values.h>\n");
    printf("\n");
    printf("\n /* macro definitions */\n");
    printf("#ifndef SQR(x)\n");
    printf("#define SQR(x) ((x) * (x))\n");
    printf("#endif\n");
    printf("\n");

    printf("\n /* function declarations */\n");
    printf("extern int %s(/* float *,float * */);\n", classify_name);
    printf("extern int %s(/* float * */);\n", normalize_name);
    printf("static float %s(/* int, float *, float * */);\n", l2dist_name);
    printf("\n\n");
}

void
print_macros(void)
{
    print_norm_macros(nraw_name, ninputs_name);
    printf("#define %s %d\n", noutputs_name, noutputs);
    printf("#define %s %d\n", ncenters_name, nexemplars);
    printf("\n");
}    

void
clear_macros(void)
{
    clear_norm_macros();
    printf("#undef %s\n", noutputs_name);
    printf("#undef %s\n", ncenters_name);
    printf("\n");
}    

void
print_classify_code(int model)
{
    int *classes;
    float **centers, *radii;

    printf("int %s (inputs, outputs)\n", classify_name);
    printf("   float *inputs,*outputs;\n");
    printf("{\n");
    printf("  int class, sphere;\n");
    classes = *(int **)hyper_param(model, "classes");
    print_ivector_initialization("classes", classes, nexemplars,
                 ncenters_name);
    centers = *(float ***)hyper_param(model, "centroids");
    print_matrix_initialization("centers", centers, nexemplars, ninputs,
                ncenters_name, ninputs_name);
    radii = *(float **)hyper_param(model, "radii");
    print_vector_initialization("radii", radii, nexemplars, ncenters_name);
    printf("  float dist, min_dist = MAXFLOAT;\n");

    /* copy and normlize inputs */
    print_copy_inputs();
    printf("\n  /* normalize loaded data */\n");
    printf("  %s(x);\n", normalize_name);

    printf("  class = -1;\n");
    printf("  for(sphere = 0; sphere < %s; sphere++){\n", ncenters_name);
    printf("    dist = %s(%s, x, centers[sphere]);\n", l2dist_name, ninputs_name);
    printf("    if(dist > radii[sphere]) dist += 10000.; /* x is outside this sphere */\n");
    if(!force){
    printf("    else if(class != -1 && class != classes[sphere])\n");
    printf("      /* x is inside two spheres of differing class */\n");
    printf("      return -1;\n");
    }
    printf("    if(dist < min_dist){\n");
    if(force){
    printf("      class = classes[sphere];\n");
    } else {
    printf("      if(dist < radii[sphere]) class = classes[sphere];\n");
    }
    printf("      min_dist = dist;\n");
    printf("    }\n");
    printf("  }\n");
    printf("  return class;\n");
    printf("}\n");
}

void
print_l2dist_code(void)
{
    printf("\n\n");
    printf("static float %s(n, x1, x2)\n", l2dist_name);
    printf("   register int n;\n");
    printf("   register float *x1, *x2;\n");
    printf("{\n");
    printf("  float dist = 0.;\n");
    printf("  for(--n; n >= 0; n--)\n");
    printf("    dist += SQR(x1[n]-x2[n]);\n");
    printf("  return(dist);\n");
    printf("}\n");
}
