/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
                                  /* irbf2c.c */
/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#define GEN2CVARS
#include <gen_2c_args.h>
#include "irbf_pack.h"

float exhspread = 1;
static ARG argtab2[] = 
{
    {"exhspread", FLOAT, (int *)&exhspread,
       "extra factor for basis function scaling in testing"},
};
#define TABSIZE2 (sizeof(argtab2)/sizeof(ARG))

/* irbf specific variables */
int nhidden, bias, norm_outputs, grand;
float hspread;
float **irbf_means, **irbf_inv_vars, **irbf_weights, *irbf_grand_inv_vars;
char nhidden_name[64];
char bias_name[64];
char logadd_name[64];
static float *train_priors_list;

/* function declerations */
int read_model_file(char **argv);
void free_model(int model);

void get_params(int model);
void get_general_params(int model);
void get_specific_params(int model);

void set_priors(int model);

void print_comments(void);
void print_declarations(void);
void print_macros(void);
void print_classify_code(int model);
void print_logadd_code(void);
void clear_macros(void);

int main(int argc, char **argv)
{
    int model;
    ARG *argtab;
    int tabsize;

    /* process arguments */
    nnargs_combine_argtabs(argtab2c, TABSIZE2C, argtab2, TABSIZE2,
               &argtab, &tabsize);
    argc = nnargs(argc, argv, argtab, tabsize);

    /* check arguments */
    if(strlen(class_suff) > 53){
    fatal_error("suffix (%s) is too long.  use at most 50 characters",
            class_suff);
    }

    /* set up subroutine names and macro names */
    strcpy(classify_name, "classify");
    strcpy(normalize_name, "normalize");
    strcpy(logadd_name, "logadd");
    strcpy(nraw_name, "NRAW");
    strcpy(ninputs_name, "NINPUTS");
    strcpy(noutputs_name, "NCLASSES");
    strcpy(nhidden_name, "NHIDDEN");
    if(class_suff[0] != '\0'){
        strcpy(class_exten, "_");
    strcat(class_exten, class_suff);
    strcat(classify_name, class_exten);
    strcat(normalize_name, class_exten);
    strcat(logadd_name, class_exten);
    strcat(nraw_name, class_exten);
    strcat(ninputs_name, class_exten);
    strcat(noutputs_name, class_exten);
    strcat(nhidden_name, class_exten);
    } else {
      strcpy(class_exten, "");
    }

    /* read in parameters */
    model = read_model_file(argv);
    get_params(model);

    set_priors(model);

    print_comments();
    print_declarations();
    print_macros();
    print_classify_code(model);
    print_normalize_code(normalize_name);
    print_logadd_code();
    clear_macros();

    nnargs_free_argtab(argtab, tabsize);
    free_model(model);
    exit(0);
}

int
read_model_file(char **argv)
{
    int   model, norm_model, normalization;
    FILE *param_file;

    param_file = nn_fopen(param_fname, "r", "for read of model file");
    norm_model = norm_read_paramfile(param_file, &normalization);
    if (norm_model < 0)
      fatal_error("%s: read error %d %s %s", argv[0], norm_model,
          "on normalization parameter file", param_fname);

    model = irbf_read_model(param_file, 0., 0., 0., 0.);
    if (model < 0)
      fatal_error("%s: read error %d on model file %s",
          argv[0], model, param_fname);
    nn_fclose(param_file, param_fname, "for read of model file");

    return model;
}

void
free_model(int model)
{
  norm_free();
  irbf_free(model);
}
void
get_params(int model)
{
    get_specific_params(model);
    get_general_params(model);
    get_norm_params(model, ninputs, features);
}

void
get_specific_params(int model)
{
    nhidden = *(int *)irbf_param(model, "ngauss_hidden");
    bias = *(int *)irbf_param(model, "bias");
    hspread = *(float *)irbf_param(model, "hspread");
    norm_outputs = *(int *)irbf_param(model, "norm_outputs");
    irbf_means = *(float ***)irbf_param(model, "means");
    irbf_inv_vars = *(float ***)irbf_param(model, "inv_var_m");
    irbf_grand_inv_vars = *(float **)irbf_param(model, "inv_var");
    irbf_weights = *(float ***)irbf_param(model, "weights");
    grand = *(int *)irbf_param(model, "grandvar");
}

void
get_general_params(int model)
{
    ninputs = *(int *)irbf_param(model, "ninputs");
    noutputs = *(int *)irbf_param(model, "nclasses");
    features = *(int **)irbf_param(model, "features");
}

void
set_priors(int model)
{
    int i;
    float total;

    /* now that we know noutputs, we can check the priors settings */
    if(adjust_test){
    if(test_uniform_priors){
        if(test_priors_list != NULL)
          deallocate_vector(test_priors_list);
        test_priors_list = allocate_vector(noutputs+1);
        test_priors_list[0] = noutputs;
        for(i = 1; i <= noutputs; i++)
          test_priors_list[i] = 1./(float)noutputs;
    } else {
        if(test_priors_list[0] != (float)noutputs)
          fatal_error("%s: %s %.0f, should be %d",
              "test_priors_list length is", test_priors_list[0],
              noutputs);
        total = 0.;
        for(i = 1; i <= noutputs; i++) total += test_priors_list[i];
        for(i = 1; i <= noutputs; i++) test_priors_list[i] /= total;
    }
    train_priors_list = *(float **)irbf_param(model, "class_cnt");
    total = *(float *)irbf_param(model, "total_npatterns");
    for(i = 0; i < noutputs; i++) train_priors_list[i] /= total;
    }
}

void
print_comments(void)
{
    printf("/* %s */\n", param_fname);
    printf("/* hspread:%.2f exhspread:%.2f */\n", hspread, exhspread);
    if(bias){
    printf("/* bias node included */\n");
    }
    print_norm_comments();
}

void
print_declarations(void)
{
    printf("\n");
    printf("#include <stdio.h>\n");
    printf("#include <math.h>\n");
    printf("#ifndef SQR(x)\n");
    printf("#define SQR(x) ((x) * (x))\n");
    printf("#endif\n");
    printf("#ifndef M_LOG10E\n");
    printf("#define M_LOG10E 0.43429448190325182765\n");
    printf("#endif\n");
    printf("\n");
    printf("#ifndef M_LN10\n");
    printf("#define M_LN10 2.30258509299404568402\n");
    printf("#endif\n");
    printf("\n");
    printf("\n /* function declarations */\n");
    printf("extern int %s(/* float *,float * */);\n", classify_name);
    printf("extern int %s(/* float * */);\n", normalize_name);
    printf("static float %s(/* float,float */);\n", logadd_name);
    printf("\n\n");
}

void
print_macros(void)
{
    print_norm_macros(nraw_name, ninputs_name);
    printf("#define %s %d\n", noutputs_name, noutputs);
    printf("#define %s %d\n", nhidden_name, nhidden);
    printf("\n");
}    

void
clear_macros(void)
{
    clear_norm_macros();
    printf("#undef %s\n", noutputs_name);
    printf("#undef %s\n", nhidden_name);
    printf("\n");
}    

void    
print_classify_code(int model)
{
    printf("int %s (inputs, outputs)\n", classify_name);
    printf("   float *inputs,*outputs;\n");
    printf("{\n");
    printf("  int n, best, j;\n");
    if(adjust_test){
    print_vector_initialization("new_priors_list", &test_priors_list[1],
                    noutputs, noutputs_name);
    print_vector_initialization("old_priors_list", train_priors_list,
                    noutputs, noutputs_name);
    printf("  float total;\n");
    }
    printf("  float f;\n");
    printf("  float h[%s];\n", nhidden_name);
    printf("  float y[%s];\n", noutputs_name);
    print_matrix_initialization("means", irbf_means, nhidden, ninputs,
                nhidden_name, ninputs_name);
    if(grand)
      print_vector_initialization("inv_vars", irbf_grand_inv_vars, ninputs, ninputs_name);
    else
      print_matrix_initialization("inv_vars", irbf_inv_vars, nhidden, ninputs,
                  nhidden_name, ninputs_name);
    if(bias){
    strcpy(bias_name, nhidden_name);
    strcat(bias_name, "+1");
    print_matrix_initialization("weights", irbf_weights, nhidden+1,
                    noutputs, bias_name, noutputs_name);
    } else
      print_matrix_initialization("weights", irbf_weights, nhidden, noutputs,
                  nhidden_name, noutputs_name);
    if(norm_outputs)
      printf("  float hsum;\n");

    /* copy and normlize inputs */
    print_copy_inputs();

    printf("\n  /* normalize loaded data */\n");
    printf("  %s(x);\n", normalize_name);

    printf("\n /* calculate basis function outputs */\n");
    printf("\n /* scale everything by hspread and exhspread*/\n");
    printf("  f = -M_LOG10E / (2.0 * %f * %f);\n", hspread, exhspread);
    printf("  for(n = 0; n < %s; n++){\n", nhidden_name);
    printf("    h[n] = 0.;\n");
    printf("    for(i = 0; i < %s; i++)\n", ninputs_name);
    if(grand)
      printf("      h[n] += SQR(x[i] - means[n][i]) * inv_vars[i];\n");
    else
      printf("      h[n] += SQR(x[i] - means[n][i]) * inv_vars[n][i];\n");
    printf("    h[n] *= f;\n");
    printf("  }\n");

    if(norm_outputs){
    printf("\n  /* normalize basis function outputs */\n");
    printf("  hsum = h[0];\n");
    printf("  for(n = 1; n < %s; n++)\n", nhidden_name);
    printf("    hsum = %s(hsum, h[n]);\n", logadd_name);
    printf("  for(n = 0; n < %s; n++)\n", nhidden_name);
    printf("    h[n] = exp(M_LN10*(double) (h[n] - hsum));\n");
    } else{
    printf("\n  /* make basis function outputs linear */\n");
    printf("  for(n = 0; n < %s; n++)\n", nhidden_name);
    printf("    h[n] = exp(M_LN10*(double) h[n]);\n");
    }   

    printf("\n  /* calculate final outputs */\n");
    printf("  for(j = 0; j < %s; j++){\n", noutputs_name);
    printf("    y[j] = 0.;\n");
    printf("    for(n = 0; n < %s; n++){\n", nhidden_name);
    printf("       y[j] += weights[n][j] * h[n];\n");
    printf("    }\n");
    if(bias)
      printf("    y[j] += weights[%s][j];  /* bias node */\n", nhidden_name);
    printf("  }\n");

    printf("\n  /* copy outputs */\n");
    printf("  for(n = 0; n < %s; n++)\n", noutputs_name);
    printf("    outputs[n] = y[n];\n");

    if(adjust_test){
    printf("\n  /* scale outputs by priors */\n");
    printf("  for(i = 0; i < %s; i++){\n", noutputs_name);
    printf("    outputs[i] *= new_priors_list[i];\n");
    printf("    outputs[i] /= old_priors_list[i];\n");
    printf("  }\n");
    printf("  /* rescale the new outputs to sum to 1 again */\n");
    printf("  total = 0;\n");
    printf("  for(i = 0; i < %s; i++) total += outputs[i];\n",
           noutputs_name);
    printf("  for(i = 0; i < %s; i++) outputs[i] /= total;\n",
           noutputs_name);
    }
    printf("\n  /* find highest output */\n");
    printf("  for(best = n = 0; n < %s; n++)\n", noutputs_name);
    printf("    if(outputs[best] < outputs[n]) best = n;\n");
    printf("  return(best);\n");
    printf("}\n");
}

void
print_logadd_code(void)
{
    printf("\n\n");
    printf("static float %s(x, y)\n", logadd_name);
    printf("   register float x, y;\n");
    printf("{\n");
    printf("  register double b;\n");
    printf("  if(x < y)    /*** exchange if x < y ***/\n");
    printf("    b = (double) x, x = (double) y, y = (double) b;\n");
    printf("  b = (double) x - y;\n");
    printf("  if(b > 16.0) /*** keep up to 16 decimal digits precision ***/\n");
    printf("    return(x);\n");
    printf("  else\n");
    printf("    return(y + (float)log10(1.0 + exp(M_LN10*b)));\n");
    printf("}\n");
}
