/***--------------------------------------------------------------------------
* NAME:     irbf_main.c
* PURPOSE:  Entry routine for Incrementally Trained Radial Basis
*        Function classifier
* USAGE:
*       irbf -<flags>
* EXAMPLE:  irbf --     * shows all available flags
*
* SEE ALSO: irbf_train.c irbf_cv.c irbf_test.c irbf_pack.c
*
* GLOBALS:  <flags> defined in irbf_args.h and <gen_class_args.h>
*
* DESCRIPTION:
*       sets up arguments for and call one of irbf_train(),
*       irbf_test(), irbf_cross_valid();
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/irbf/irbf_main.c,v 11.5 2001/08/23 20:12:05 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/irbf/irbf_main.c,v $
* CREATED 27-11-92 lck              MIT LINCOLN LABORATORY
*--------------------------------------------------------------------------**/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char     rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/irbf/irbf_main.c,v 11.5 2001/08/23 20:12:05 craig Exp $";



/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#define NO_CV_TABLE
#include <cv_args.h>
#include <gen_class_args.h>
#include "irbf_args.h"
#include "irbf.h"

int
main(int argc, char **argv)
{
    char *comment;
    int use_splits, norm_outputs;
    /* arguments for reading defaults file */
    DEFAULTS_FILE_STRUCT *df_args;
    /* argument table with all entries */
    ARG *argtab;
    int tabsize;
    int roc_cost = 0, roc_target = 0;
    float roc_lower = 0, roc_upper = 1;

    init_nnsubs(argc, argv, rcsid);
    /* get arguments */
    nnargs_combine_argtabs(gen_tab, GEN_TABSIZE, irbf_tab, IRBF_TABSIZE,
               &argtab, &tabsize);
    argc = nnargs(argc, argv, argtab, tabsize);

    print_log_header();

            
    /************* read defaults and set input data parameters ********/
    /* read the defaults file */
    if(strcmp(desc_fname, "") == 0){ /* desc_fname is unset */
    fatal_error("%s: -fdescribe must be set", argv[0]);
    }
    /* set the number of inputs and outputs and read the labels,
       allocating the space */
    df_args = (DEFAULTS_FILE_STRUCT *)
      nn_malloc(sizeof(DEFAULTS_FILE_STRUCT), "for the defaults file struct");
    read_defaults_file(path_build(data_path, desc_fname), df_args);
    if(df_args->noutputs == 0)
      fatal_error("noutputs in %s must be greater than 0", desc_fname);
    if(df_args->ninputs == 0)
      fatal_error("ninputs in %s must be greater than 0", desc_fname);

    /* set the number of patterns to no more than the number of lines
       in the data file */
    if(strcmp(data_fname, "") == 0){ /* data_fname is unset */
    fatal_error("%s: -finput must be set", argv[0]);
    }
    check_file_length(path_build(data_path, data_fname), &npatterns);
    if(npatterns == 0)
      fatal_error("%s: Empty data file %s", argv[0],
          path_build(data_path, data_fname));
    /* check the data base type */
    if (df_args->db_type != CLASS_DATA)
      fatal_error("class data required, check defaults file");

    /***************** check feature selection flags *******************/
    if(verbosity > 3){
    if(ninputs != 0)
      warning("-ninputs flag has no effect in %s.\n%s",
        argv[0], "Value in -fclparam is used");
    if(features != NULL && features[0] != 0)
      warning("-features flag has no effect in %s.\n%s",
        argv[0], "Value in -fclparam is used");
    if(normalize != 0)
      warning("-normalize flag has no effect in %s.\n%s",
        argv[0], "Value in -fclparam is used");
    }
    if (ninputs > df_args->ninputs)
       ninputs = df_args->ninputs;
    ninputs = fix_features(df_args->ninputs, ninputs, features);

    /*************** set up the cross validation arguments ***********/
    if(train_flag && cross_valid > 2){
    warning("%s\n%s\n","-train and -cross_valid are both set.",
        " -cross_valid will be ignored");
    cross_valid = 0;
    }
    use_splits =
      fix_cross_valid_args(&cross_valid, &cv_nfolds, &cv_nsplits, cv_splits,
               cv_train_mask, cv_test_mask, cross_valid_from_file,
               path_build(data_path, cross_valid_fname));

    /********* fix up priors adjustment arguments *********************/
    fix_priors(adjust_train, train_uniform_priors, &train_priors_list,
           adjust_test, test_uniform_priors, &test_priors_list,
           train_flag, cross_valid, df_args->noutputs);

    /********** check IRBF args ***************/
    if (nepochs < 1) fatal_error("epochs must be greater than 0");

    /* set norm outputs to the opposite of no_norm_outputs */
    norm_outputs = (no_norm_outputs == 0) ? 1 : 0;

    if(cost_func != 0 && cost_func != 4)
      fatal_error("IRBF: Only the cost functions %s are available for IRBF",
          "squared error and top2diff");
    if(hspread <= 0.)
      fatal_error("irbf_main: hspread must be greater than 0");
    if(extra_hspread <= 0.)
      fatal_error("irbf_main: exhspread must be greater than 0");

    /************** make and print header comment *******************/
    comment = make_comment_string(nnargs_get_flags(argtab, tabsize));
    if (verbosity > FLAGS_VERBOSITY) {
    printf("%s", comment);
    fflush(stdout);
    }

    srandom(seed);      /* initialize random numbers */

    if(train_flag){
    irbf_train(create_flag, df_args->ninputs, ninputs, features, df_args->noutputs,
           npatterns, debug, verbosity, verror, random_flag,
           data_path, data_fname, exp_path, error_fname,
           param_fname, df_args->output_labels, comment, adjust_train,
           priors_npatterns, &train_priors_list[1],
           log_outputs, nepochs, bias, grand_flag, hspread,
           max_magnitude, cluster_param_fname, minimum_variance,
           weight_eta, mean_eta, var_eta, cost_func, cost_param,
           norm_outputs);
    } else if(cv_nfolds > 1){
    irbf_cross_valid(df_args->db_type, df_args->ninputs, ninputs, features, df_args->noutputs,
             npatterns, debug, verbosity, verror, random_flag,
             cv_nfolds, cv_nsplits, use_splits, cv_splits,
             &cv_train_mask[1], &cv_test_mask[1], data_path,
             data_fname, exp_path, error_fname, param_fname,
             df_args->output_labels, comment, adjust_train, adjust_test,
             priors_npatterns, &train_priors_list[1],
             &test_priors_list[1], cv_random, log_outputs,
             (roc_cost?ROC_COST:cost_func),
             roc_target, roc_lower, roc_upper,
             nepochs, bias, grand_flag, extra_hspread,
             hspread, max_magnitude, cluster_param_fname,
             minimum_variance, weight_eta, mean_eta, var_eta,
             cost_func, cost_param, norm_outputs);
    } else {
    irbf_test(create_flag, df_args->db_type, df_args->ninputs, ninputs, df_args->noutputs, npatterns,
          debug, verbosity, verror, data_path, data_fname, exp_path,
          error_fname, param_fname, df_args->output_labels, adjust_test,
          &test_priors_list[1], log_outputs, extra_hspread, cost_func,
          cost_param);
    }
    print_log_footer();

    nnargs_free_argtab(argtab, tabsize);
    /* free the vectors allocated when the defaults file was read */
    free_defaults_file_struct(df_args);

    exit_nnsubs();
    exit(0);
}
