/***--------------------------------------------------------------------------
* NAME:         kmeans_main.c
* PURPOSE:      Entry routine for kmeans clustering algorithm
* USAGE:
*           kmeans -<flags>
* EXAMPLE:  kmeans --                * shows all available flags
*
* SEE ALSO:     kmeans.c kmeans_pack.c kmeans.run
*
* GLOBALS:  <flags> defined in kmeans_args.h
*
* DESCRIPTION:
*       sets up arguments for the routine kmeans()
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/kmeans/kmeans_seq_main.c,v 11.4 2001/08/23 20:12:15 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/kmeans/kmeans_seq_main.c,v $
* CREATED 16-Aug-93 lck             MIT LINCOLN LABORATORY
*--------------------------------------------------------------------------**/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/kmeans/kmeans_seq_main.c,v 11.4 2001/08/23 20:12:15 craig Exp $";



/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#define NO_CV_TABLE
#include <cv_args.h>
#include "gen_seq_args.h"
#include "kmeans_seq_args.h"            /* parameter definitions */
#include "kmeans_seq.h"

int
main(int argc, char **argv)
{
    char    *comment;
    int     use_splits, i, nwindow_frames;
    /* arguments for reading defaults file */
    DEFAULTS_FILE_STRUCT *df_args;
    ARG *argtab;
    int tabsize;

    init_nnsubs(argc, argv, rcsid);

    /* build an argument table with all available arguments */
    /* and use it to parse the command line */
    nnargs_combine_argtabs(gen_tab, GEN_TABSIZE, kmeans_tab, KMEANS_TABSIZE,
               &argtab, &tabsize);
    argc = nnargs(argc, argv, argtab, tabsize);

    print_log_header();

    /* read the defaults file */
    if(strcmp(desc_fname, "") == 0){ /* desc_fname is unset */
    fatal_error("%s: -fdescribe must be set", argv[0]);
    }
    df_args = (DEFAULTS_FILE_STRUCT *)
      nn_malloc(sizeof(DEFAULTS_FILE_STRUCT), "for the defaults file struct");
    read_defaults_file(path_build(data_path, desc_fname), df_args);
    if(df_args->noutputs == 0)
      fatal_error("noutputs in %s must be greater than 0", desc_fname);
    if(df_args->ninputs == 0)
      fatal_error("ninputs in %s must be greater than 0", desc_fname);

    /* set the number of sequences to no more than are in the file */
    if(strcmp(data_fname, "") == 0){ /* data_fname is unset */
    fatal_error("%s: -flist must be set", argv[0]);
    }    
    check_file_length(path_build(data_path, data_fname), &nsequences);
    if(nsequences == 0)
      fatal_error("%s: Empty data file %s", argv[0],
          path_build(data_path, data_fname));
    /* check the data base type */
    if (df_args->db_type != SEQ_DATA)
      fatal_error("sequence data required, check defaults file");

    /* check the feature selection arguments */
    if (ninputs > df_args->ninputs)
    ninputs = df_args->ninputs;
    if(features == NULL){
    /* no feature selection is specified */
    features = allocate_ivector(1);
    features[0] = 0;
    }
    ninputs = fix_features(df_args->ninputs, ninputs, features);
    nwindow_frames = fix_windows(window_width, window_frames);

    /* check any relevent flags to see that we can do what is requested */
    if (!train_flag && cross_valid <= 0) {
    fatal_error("%s\n%s",
            "Test mode not implemented for kmeans clustering.",
            "    Use -train flag.");
    }
    if (!create_flag && cross_valid <= 0) {
    fatal_error("%s\n%s",
            "Continue mode not implemented for kmeans clustering.",
            "    Use -create flag.");
    }
    if(train_flag && cross_valid > 2)
      warning("%s\n%s\n","-train and -cross_valid are both set.",
          " -cross_valid will be ignored");

    /* set up the cross validation arguments */
    use_splits =
      fix_cross_valid_args(&cross_valid, &cv_nfolds, &cv_nsplits, cv_splits,
               cv_train_mask, cv_test_mask, cross_valid_from_file,
               path_build(data_path, cross_valid_fname));

    /* check arguments on generic list that are ignored here */
    if(verbosity > 3){
    if(random_flag)
      warning("-random flag has no effect in %s", argv[0]);
    if(verror != 0)
      warning("-verror has no effect in %s", argv[0]);
    if(strcmp(error_fname, "gen.err") != 0)
      warning("-ferror has no effect in %s", argv[0]);
    }

    /* fix ncenters_list to tell the number of centers needed for each
     clustering run */
    if(cluster_by_class){
    /* if the user has not specified the numbers of centers for each
       class, use ncenters for all */
    if(ncenters_list == NULL || ncenters_list[0] == 0){
        if(ncenters_list != NULL) deallocate_ivector(ncenters_list);
        ncenters_list = allocate_ivector(df_args->noutputs+1);
        ncenters_list[0] = df_args->noutputs;
        for(i = 1; i <= df_args->noutputs; i++){
        ncenters_list[i] = ncenters;
        }
    }
    if(ncenters_list[0] != df_args->noutputs){
        fatal_error("%s: %s. Is %d long, should be %d long",
            "kmeans_main.c", "ncenters list is the wrong length",
            ncenters_list[0], df_args->noutputs);
    }
    } else {
    if(ncenters_list != NULL) deallocate_ivector(ncenters_list);
    ncenters_list = allocate_ivector(2);
    ncenters_list[0] = 1;
    ncenters_list[1] = ncenters;
    }

    /* make and print header comment */
    comment = make_comment_string(nnargs_get_flags(argtab, tabsize));
    if (verbosity > 1) {
    printf("%s", comment);
    fflush(stdout);
    }

    /* initialize the random number generator */
    srandom(seed);

    /* call the appropriate driver */
    if(train_flag){
    kmeans_seq_train(df_args->ninputs, ninputs, features, df_args->noutputs, nwindow_frames,
             window_frames, normalize, nsequences, debug,
             verbosity, data_path, data_fname, norm_fname,
             exp_path, param_fname, df_args->output_labels, comment,
             max_iteration, split_percentage,
             stop_percentage, cluster_by_class,
             add_random_offset, reduce_step,
             &ncenters_list[1]);
    } else {
    kmeans_seq_cross_valid(df_args->ninputs, ninputs, features, df_args->noutputs,
                   nwindow_frames, window_frames,
                   normalize, nsequences, debug,
                   verbosity, cv_nfolds, cv_nsplits,
                   use_splits, cv_splits,
                   &cv_train_mask[1], cv_random,
                   data_path, data_fname, norm_fname,
                   exp_path, param_fname, df_args->output_labels, comment,
                   max_iteration, split_percentage,
                   stop_percentage, cluster_by_class,
                   add_random_offset, reduce_step,
                   &ncenters_list[1]);
    }
    /* finish up */
    print_log_footer();

    nnargs_free_argtab(argtab, tabsize);
    /* free the vectors allocated when the defaults file was read */
    free_defaults_file_struct(df_args);

    exit_nnsubs();
    exit(0);
}
