/***--------------------------------------------------------------------------
 * NAME:		knn.c
 * PURPOSE:	driver for knn training
 *
 * SEE ALSO: knn_pack.c
 *
 * DESCRIPTION: 
 *	knn_train()
 * 
 * BUGS AND LIMITATIONS:
 *
 * $Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/knn/knn_train.c,v 11.4 2001/08/23 20:12:19 craig Exp $
 * $Source: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/knn/knn_train.c,v $
 * CREATED: 12-Dec-90 lck			MIT LINCOLN LABORATORY
 **************************************************************************/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */



/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include "knn.h"
#include "knn_pack.h"

#define MAXSTR (256)

static char prog_name[MAXSTR];	/* static variables */

void
knn_train(int create_flag, int nraw, int ninputs, int *features, int noutputs,
	  int normalize, int npatterns, int debg, int verbosity,
	  int random_flag, char *data_path, char *data_fname,
	  char *norm_fname, char *exp_path, char *param_fname, char *comment,
	  int adjust_priors, int priors_npatterns, float *priors_list, int k)
{
    int model, data_model, rv, normalization = 0;
    int accum_trials, desired_class;
    float *inputs;
    char *tmp_path;
    FILE *param_file;
    
    strcpy(prog_name, get_program_name());
    
    if(create_flag){
	/* allocate and initialize model structure */
	accum_trials = 0;
	model = knn_build(ninputs, noutputs, npatterns, k);
	knn_init(model, accum_trials, features);
	
	if (normalize) 
	  normalization = read_norm_file(path_build(data_path, norm_fname));
	else
	  norm_setup_none(nraw, noutputs);
    } else {			/* continue training */
	tmp_path = path_build(exp_path, param_fname);
	param_file = nn_fopen(tmp_path, "r", "for read of knn model file");
	rv = norm_read_paramfile(param_file, &normalization);
	if (rv < 0)
	  fatal_error ("%s: read error %d %s %s", prog_name, rv,
		       "on normalization parameter file", tmp_path);
	model = knn_read_model(param_file, &accum_trials);
	if(model < 0)
	  fatal_error("%s: error reading model file %s", prog_name, tmp_path);
	nn_fclose(param_file, tmp_path, "for read of knn model file");
	
	verify_model_params (model,
			     ninputs, *(int *) knn_param(model, "ninputs"),
			     features, *(int **) knn_param(model, "features"));
    }
    /* use the features from the model */
    features = *(int **) knn_param(model, "features");
    ninputs = *(int *) knn_param(model, "ninputs");
    
    if(adjust_priors){
	data_model = class_data_read(data_path, data_fname, npatterns,
				     nraw, ninputs, noutputs, features,
				     normalization, verbosity);
	if(random_flag) class_data_random_init(data_model);
	class_data_priors_init(data_model, priors_list, priors_npatterns);
    } else {
	data_model = class_data_single_init(data_path, data_fname, npatterns,
					    nraw, ninputs, noutputs, features,
					    normalization, verbosity);
    }
    while((inputs = class_data_get_next(data_model, &desired_class)) != NULL){
	knn_inc_ctrain(model, inputs, desired_class);
    }
    class_data_exit(data_model);
    
    /* write training data to model file */
    tmp_path = path_build(exp_path, param_fname);
    param_file = nn_fopen(tmp_path, "w", "for write of knn model file");
    /* write comment */
    fprintf(param_file, "%s", comment);
    fprintf(param_file, "normalization data\n");
    norm_write_paramfile(param_file);
    fprintf(param_file, "knn model\n");
    knn_write_model(model, param_file);
    nn_fclose(param_file, tmp_path, "for write of knn model file");
    
    if ( verbosity > 0)
      printf("\nFinished -- knn model saved in \"%s\"\n", tmp_path);
    
    /*free space and close files, for neatness sake */
    norm_free();
    knn_free(model);
}
