/***--------------------------------------------------------------------------
* NAME:         lead_clus_main.c
* PURPOSE:      Entry routine for lead_clus clustering algorithm
* USAGE:
*           lead_clus -<flags>
* EXAMPLE:  lead_clus --                * shows all available flags
*
* SEE ALSO:     lead_clus_train.c lead_clus_cv.c lead_clus_pack.c
*
* GLOBALS:  <flags> defined in lead_clus_args.h and <gen_class_args.h>
*
* DESCRIPTION:
*       sets up arguments for the routine lead_clus()
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/lead_clus/lead_clus_main.c,v 11.4 2001/08/23 20:12:22 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/lead_clus/lead_clus_main.c,v $
* CREATED 6-Apr-92 lck              MIT LINCOLN LABORATORY
*--------------------------------------------------------------------------**/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/lead_clus/lead_clus_main.c,v 11.4 2001/08/23 20:12:22 craig Exp $";



/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#define NO_CV_TABLE
#include <cv_args.h>
#include <gen_class_args.h>
#include "lead_clus.h"
#include "lead_clus_args.h"         /* parameter definitions */

int
main(int argc, char **argv)
{
    char    *comment;
    int     use_splits;

    /* arguments for reading defaults file */
    DEFAULTS_FILE_STRUCT *df_args;
    /* argument table with all entries */
    ARG *argtab;
    int tabsize;

    init_nnsubs(argc, argv, rcsid);
    /******************* read argument list **********************/
    nnargs_combine_argtabs(gen_tab, GEN_TABSIZE,
               lead_clus_tab, LEAD_CLUS_TABSIZE,
               &argtab, &tabsize);
    argc = nnargs(argc, argv, argtab, tabsize);

    print_log_header();

    /************* read defaults and set input data parameters ********/
    if(strcmp(desc_fname, "") == 0){ /* desc_fname is unset */
    fatal_error("%s: -fdescribe must be set", argv[0]);
    }
    /* set the number of inputs and outputs and read the labels,
       allocating the space */
    df_args = (DEFAULTS_FILE_STRUCT *)
      nn_malloc(sizeof(DEFAULTS_FILE_STRUCT), "for the defaults file struct");
    read_defaults_file(path_build(data_path, desc_fname), df_args);
    if(df_args->noutputs == 0)
      fatal_error("noutputs in %s must be greater than 0", desc_fname);
    if(df_args->ninputs == 0)
      fatal_error("ninputs in %s must be greater than 0", desc_fname);

    /* set the number of patterns to no more than the number of lines
       in the data file */
    if(strcmp(data_fname, "") == 0){ /* data_fname is unset */
    fatal_error("%s: -finput must be set", argv[0]);
    }
    check_file_length(path_build(data_path, data_fname), &npatterns);
    if(npatterns == 0)
      fatal_error("%s: Empty data file %s", argv[0],
          path_build(data_path, data_fname));
    /* check the data base type */
    if (df_args->db_type != CLASS_DATA)
      fatal_error("class data required, check defaults file");

    /***************** check feature selection flags *******************/
    if (ninputs > df_args->ninputs)
      ninputs = df_args->ninputs;
    ninputs = fix_features(df_args->ninputs, ninputs, features);

    /*************** set up the cross validation arguments ***********/
    if(train_flag && cross_valid > 2){
    warning("%s\n%s\n","-train and -cross_valid are both set.",
        " -cross_valid will be ignored");
    cross_valid = 0;
    }
    use_splits =
      fix_cross_valid_args(&cross_valid, &cv_nfolds, &cv_nsplits, cv_splits,
               cv_train_mask,cv_test_mask,cross_valid_from_file,
               path_build(data_path, cross_valid_fname));
        
    /********* fix up priors adjustment arguments *********************/
    if(adjust_test){
    /*warning("%s cannot adjust testing priors", argv[0]);*/
    adjust_test = 0;
    }
    fix_priors(adjust_train, train_uniform_priors, &train_priors_list,
           adjust_test, test_uniform_priors, &test_priors_list,
           train_flag, cross_valid, df_args->noutputs);

    /********** check lead_clus args *****************/

    if(cluster_by_class){
    /* if the user has not specified a delta for each class,
       use "delta" for all */
    if(delta_list == NULL || delta_list[0] == 0.){
        int i;
        if(delta_list != NULL) deallocate_vector(delta_list);
        delta_list = allocate_vector(df_args->noutputs+1);
        delta_list[0] = df_args->noutputs;
        for(i = 1; i <= df_args->noutputs; i++){
        delta_list[i] = delta;
        }
    }
    if(delta_list[0] != df_args->noutputs){
        fatal_error("%s: %s. Is %.0f long, should be %d long",
            "lead_clus_main", "delta_list is the wrong length",
            delta_list[0], df_args->noutputs);
    }
    } else {
    if(delta_list != NULL) deallocate_vector(delta_list);
    delta_list = allocate_vector(2);
    delta_list[0] = 1.;
    delta_list[1] = delta;
    }

    /****************** check for unnecessary arguments ****************/
    if(!create_flag){
    if(verbosity > 3)
      warning("flag -create is always set for %s", argv[0]);
    create_flag = 1;
    }
    if(log_outputs){
    warning("%s does not support log_outputs", argv[0]);
    log_outputs = 0;
    }

    srandom(seed);      /* initialize random numbers */
    
    /************** make and print header comment *******************/
    comment = make_comment_string(nnargs_get_flags(argtab, tabsize));
    if (verbosity > FLAGS_VERBOSITY) {
    printf("%s", comment);
    fflush(stdout);
    }

    /************* call the selected driver ***************/
    if(train_flag){
    lead_clus_train(df_args->ninputs, ninputs, features,
            df_args->noutputs, normalize, npatterns, debug, verbosity,
            random_flag, data_path, data_fname,
            norm_fname, exp_path, param_fname, comment,
            adjust_train, priors_npatterns, &train_priors_list[1],
            &delta_list[1], cluster_by_class);
    } else if(cv_nfolds > 1){
    lead_clus_cross_valid(df_args->ninputs, ninputs,
                  features, df_args->noutputs, normalize, npatterns,
                  debug, verbosity, random_flag, cv_nfolds,
                  cv_nsplits, use_splits, cv_splits,
                  &cv_train_mask[1], &cv_test_mask[1],
                  data_path, data_fname,
                  norm_fname, exp_path, param_fname, comment,
                  adjust_train, priors_npatterns,
                  &train_priors_list[1], cv_random,
                  &delta_list[1], cluster_by_class);
    } else {
    lead_clus_test(create_flag, df_args->ninputs, ninputs, features, df_args->noutputs,
               npatterns, verbosity, verror, data_path, data_fname,
               exp_path, error_fname, param_fname, df_args->output_labels);
    }

    print_log_footer();

    nnargs_free_argtab(argtab, tabsize);
    /* free the vectors allocated when the defaults file was read */
    free_defaults_file_struct(df_args);

    exit_nnsubs();
    exit(0);
}
