/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
                                  /* lvq2c.c */
/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>

char *param_fname = "";
char *class_suff = "";
char *norm_suff = "";
static ARG argtab[] = 
{
    {"model_file", STRING, (int *)&param_fname, "mlp model to be translated"},
    {"classify_", STRING, (int *)&class_suff, "to name classify_<suffix>"},
    {"normalize_", STRING, (int *)&norm_suff, "to name normalize_<suffix>"},
};
#define TABSIZE (sizeof(argtab)/sizeof(ARG))

void main(int argc, char **argv)
{
    FILE *param_file;
    int model, norm_model;
    int normalization;
    int mas_flag;

    argc = nnargs(argc, argv, argtab, TABSIZE);
    param_file = nn_fopen(param_fname, "r", "for read of model file");
    norm_model = norm_read_paramfile(param_file, &normalization);
    if (norm_model < 0)
      fatal_error("%s: read error %d %s %s", argv[0], norm_model,
		  "on normalization parameter file", param_fname);
    
    model = lvq_read_model(param_file);
    if (model < 0)
      fatal_error("%s: read error %d on model file %s",
		  argv[0], model, param_fname);
    nn_fclose(param_file, param_fname, "for read of model file");

    printf("/* %s */\n", param_fname);
    print_c_model(model);
    print_norm_model(model);
    exit(0);
}

print_c_model(int model)
{
    int i, j, best;
    int *features, ninputs, ncenters, nclasses, *classes, **centers;
    int norm_type, norm_ninputs;
    float *means, *sdevs, **transform_matrix;
    float distance, min_distance;
    void *lvq_param(int model, char *param_name);

    /* get general parameters */
    features = *(int **)lvq_param(model, "features");
    ninputs = *(int *)lvq_param(model, "ninputs");
    nclasses = *(int *)lvq_param(model, "nclasses");
    ncenters = *(int *)lvq_param(model, "ncenters");
    classes = *(int **)lvq_param(model, "classes");
    centers = *(int ***)lvq_param(model, "centers");

    /* get normalization parameters */
    norm_type = *(int *)norm_param("normalization");
    norm_ninputs = *(int *)norm_param("ninputs");
    if(norm_ninputs == 0) norm_ninputs = ninputs;
    means = *(float **)norm_param("means");
    sdevs = *(float **)norm_param("sdevs");
    transform_matrix = *(float ***)norm_param("transform");

    /* print description of variables */
    switch (norm_type){
      case 0:
	printf("/* norm: NONE */\n");
	break;
      case 1:
	printf("/* norm: SIMPLE */\n");
	break;
      case 2:
	printf("/* norm: PCA */\n");
	break;
      case 3:
	printf("/* norm: LDA */\n");
	break;
    }
    if(ninputs != norm_ninputs){
	printf("/* %d features", ninputs);
	if(features[0] != 0){
	    printf(": ");
	    for(i = 1; i <= features[0]; i++) printf("%d ", features[i]);
	}
	printf("*/\n");
    } else {
	printf("/* %d (all) features */\n", ninputs);
    }
    
    printf("\n");
    printf("#include <math.h>\n");
    printf("\n");
    printf("#ifndef SQR(x)\n");
    printf("#define SQR(x) ((x) * (x))\n");
    printf("#endif\n");
    printf("\n");
    printf("#ifndef MAXFLOAT\n");
    printf("#define MAXFLOAT ((float) 3.40282346638528860e+38)\n");
    printf("#endif\n");
    printf("\n");
    printf("\n");
    printf("\n  /* declaring the vector holding the classes */\n");
    printf("  static int classes[%d] = {\n   ", ncenters);
    for(i = 0; i < ncenters; i++){
	    if((i+1)%8 == 1 && i) printf("\n   ");
	    printf("%3d, ", classes[i]);
	}
    printf("\n   };\n");
    printf("\n  /* declaring the matrix holding the center vectors */\n");
    printf("  static float center_vectors[%d][%d] = {\n", ncenters, ninputs);
    for(i = 0; i < ncenters; i++){
	printf("    {");
	for(j = 0; j < ninputs; j++){
	    if((j+1)%5 == 1 && j) printf("\n     ");
	    printf("%13.5e", centers[i][j]);
	    if(j < ninputs-1) printf(", ");
	}
	printf("},\n");
    }
    printf("   };\n");
    if(class_suff[0] != '\0')
      printf("\nclassify_%s(inputs, outputs)\n", class_suff);
    else
      printf("\nclassify(inputs, outputs)\n");
    printf("   float *inputs, *outputs;\n");
    printf("{\n");
    printf("  int i, j, n, best;\n");
    printf("  float x[%d];\n", ninputs);
    printf("  float distance, min_distance;\n");
    printf("\n  /* load inputs */\n");
    printf("  for(n = 0; n < %d; n++)\n", ninputs);
    printf("    x[n] = inputs[n];\n");
    printf("\n  /* normalize loaded data */\n");
    if(norm_suff[0] != '\0')
      printf("  normalize_%s(x);\n", norm_suff);
    else
      printf("  normalize(x);\n");
    printf("\n  /* calculate and compare distances to find the nearest */\n");
    printf("  min_distance = MAXFLOAT;\n");
    printf("  best = 0;\n");
    printf("  for(i = 0; i < %d; i++){\n", ncenters);
    printf("    distance = 0;\n");
    printf("    for(j = 0; j < %d; j++){\n", ninputs);
    printf("      distance += SQR(center_vectors[i][j] - x[j]);\n");
    printf("    }\n");
    printf("    if(distance < min_distance){\n");
    printf("      min_distance = distance;\n");
    printf("      best = i;\n");
    printf("    }\n");
    printf("  }\n");
    printf("\n  /* copy the output */\n");
    printf("  outputs[0] = classes[best];\n");
    printf("  return(classes[best]);\n");
    printf("  }\n");
}
	   

print_norm_model(int model)
{
    int ninputs, *features;
    int m, n, i;
    float *means, *sdevs, **transform_matrix;
    int norm_type, norm_ninputs, sample = 0;
    void *lvq_param(int model, char *param_name);

    /* get general parameters */
    ninputs = *(int *)lvq_param(model, "ninputs");
    features = *(int **)lvq_param(model, "features");
    norm_type = *(int *)norm_param("normalization");
    norm_ninputs = *(int *)norm_param("ninputs");
    if(norm_ninputs == 0) norm_ninputs = ninputs;
    means = *(float **)norm_param("means");
    sdevs = *(float **)norm_param("sdevs");
    transform_matrix = *(float ***)norm_param("transform");

    if(ninputs != norm_ninputs && features[0] != 0){
	sample = 1;
    }

    printf("\n\n");
    if(norm_suff[0] != '\0')
      printf("\nint normalize_%s(inputs)\n", norm_suff);
    else
      printf("\nint normalize(inputs)\n");
    printf("   float *inputs;\n");
    printf("{\n");
    if(norm_type == 0){
	printf("  return %d;\n", norm_ninputs);
	printf("}\n");
	return;
    }
    printf("  int n;\n");
    printf("  float new_inputs[%d];\n", ninputs);
    printf("\n");
    for(i = 0; i < ninputs; i++){
	if(sample) n = features[i];
	else n = i;
	switch(norm_type){
	  case 0:
	    printf("  new_inputs[%d] = inputs[%d];\n", i, n);
	    break;
	  case 1:
	    printf("  new_inputs[%d] = (inputs[%d] - %f) / %f;\n",
		   i, n , means[n], sdevs[n]);
	    break;
	  case 2:
	  case 3: /* !!! */
	    printf("\n");
	    printf("  new_inputs[%d] = 0.;\n", i);
	    for(m = 0; m < norm_ninputs; m++){
		if((int) (1.0e+06 * transform_matrix[n][m] + 0.5) != 0)
		  printf("  new_inputs[%d] += %f * inputs[%d];\n",
			 i, transform_matrix[n][m], m);
	    }
	    break;
	}
    }
    printf("\n");
    printf("  for(n = 0; n < %d; n++)\n", ninputs);
    printf("    inputs[n] = new_inputs[n];\n");
    printf("\n");
    printf("  return(%d);\n", ninputs);
    printf("}\n");
}

