/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
                                  /* mlp2c.c */
/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#if HAVE_STRINGS_H
#include <strings.h>
#endif
#include <string.h>
#include <nnlib.h>
#define GEN2CVARS
#include <gen_2c_args.h>
#include "mlp_pack.h"

/* mlp specific variables */
int  nlayers, *layers, cost_func, sigm_type;
static float *train_priors_list;

/* internal function declarations */
int read_model_file(char **argv);
void free_model(int model);

void get_params(int model);
void get_general_params(int model);
void get_specific_params(int model);

void set_priors(int model);

void print_comments(void);
void print_declarations(void);
void print_macros(void);
void print_classify_code(int model);
void clear_macros(void);

int
main(int argc, char **argv)
{
    int model;

    /* process arguments */
    argc = nnargs(argc, argv, argtab2c, TABSIZE2C);

    /* check arguments */
    if(strlen(class_suff) > 53){
    fatal_error("suffix (%s) is too long.  use at most 50 characters",
            class_suff);
    }

    /* set up subroutine names */
    strcpy(classify_name, "classify");
    strcpy(normalize_name, "normalize");
    strcpy(nraw_name, "NRAW");
    strcpy(ninputs_name, "NINPUTS");
    strcpy(noutputs_name, "NCLASSES");
    if(class_suff[0] != '\0'){
        strcpy(class_exten, "_");
    strcat(class_exten, class_suff);
    strcat(classify_name, class_exten);
    strcat(normalize_name, class_exten);
    strcat(nraw_name, class_exten);
    strcat(ninputs_name, class_exten);
    strcat(noutputs_name, class_exten);
    } else {
      strcpy(class_exten, "");
    }

    /* read in parameters */
    model = read_model_file(argv);
    get_params(model);

    set_priors(model);

    print_comments();
    print_declarations();
    print_macros();
    print_classify_code(model);
    print_normalize_code(normalize_name);
    clear_macros();

    free_model(model);
    exit(0);
}

int
read_model_file(char **argv)
{
    int   model, norm_model, normalization;
    FILE *param_file;

    param_file = nn_fopen(param_fname, "r", "for read of model file");
    norm_model = norm_read_paramfile(param_file, &normalization);
    if (norm_model < 0)
      fatal_error("%s: read error %d %s %s", argv[0], norm_model,
          "on normalization parameter file", param_fname);
    
    model = mlp_read_model(param_file);
    if (model < 0)
      fatal_error("%s: read error %d on model file %s",
          argv[0], model, param_fname);
    nn_fclose(param_file, param_fname, "for read of model file");

    return((int) model);
}

void
free_model(int model)
{
  norm_free();
  mlp_free(model);
}
void
get_params(int model)
{
    get_specific_params(model);
    get_general_params(model);
    get_norm_params(model, ninputs, features);
}

void
get_specific_params(int model)
{
    nlayers = *(int *)mlp_param(model, "nlayers", 0);
    layers = *(int **)mlp_param(model, "layers", 0);
    cost_func = *(int *)mlp_param(model, "cost_func", 0);
}

void
get_general_params(int model)
{
    /* get general parameters */
    ninputs = *(int *)mlp_param(model, "ninputs", 0);
    noutputs = *(int *)mlp_param(model, "noutputs", 0);
    features = *(int **)mlp_param(model, "features", 0);
}

void
set_priors(int model)
{
    int i;
    float total;

    /* now that we know noutputs, we can check the priors settings */
    if(adjust_test){
    if(test_uniform_priors){
        if(test_priors_list != NULL)
          deallocate_vector(test_priors_list);
        test_priors_list = allocate_vector(noutputs+1);
        test_priors_list[0] = noutputs;
        for(i = 1; i <= noutputs; i++)
          test_priors_list[i] = 1./(float)noutputs;
    } else {
        if(test_priors_list[0] != (float)noutputs)
          fatal_error("%s: %s %.0f, should be %d",
              "test_priors_list length is", test_priors_list[0],
              noutputs);
        total = 0.;
        for(i = 1; i <= noutputs; i++) total += test_priors_list[i];
        for(i = 1; i <= noutputs; i++) test_priors_list[i] /= total;
    }
    train_priors_list = *(float **)mlp_param(model, "class_cnt", 0);
    total = *(float *)mlp_param(model, "total_npatterns", 0);
    for(i = 0; i < noutputs; i++) train_priors_list[i] /= total;
    }
}

void
print_comments(void)
{
    int i;

    printf("/* %s */\n", param_fname);
    print_norm_comments();

    switch(cost_func){
      case SQ_ERR:
    printf("/* cost_func: SQ_ERR */\n");
    break;
      case CROSS_ENTROPY:
    printf("/* cost_func: CROSS_ENTROPY */\n");
    break;
      case MAX_LIKE:
    printf("/* cost_func: MAX_LIKE */\n");
    break;
      case PC_PROC:
    printf("/* cost_func: PC_PROC */\n");
    break;
      case CFOM:
    printf("/* cost_func: CFOM */\n");
    break;
    }
    switch(sigm_type){
      case SIGM:
    printf("/* sigm: SIGM */\n");
    break;
      case SYM_SIGM:
    printf("/* sigm: SYM_SIGM */\n");
    break;
      case STEP:
    printf("/* sigm: STEP */\n");
    break;
      case LINEAR:
    printf("/* sigm: LINEAR */\n");
    break;
    }
    printf("/* net: %d", layers[0]-1);
    for(i = 1; i < nlayers; i++) printf(",%d", layers[i]-1);
    printf(" */\n");
}

void
print_declarations(void)
{
    printf("\n");
    printf("#include <stdio.h>\n");
    printf("#include <math.h>\n");
    printf("\n");

    printf("\n /* function declarations */\n");
    printf("extern int %s(/* float *,float * */);\n", classify_name);
    printf("extern int %s(/* float * */);\n", normalize_name);
    printf("\n\n");
}

void
print_macros(void)
{
    int l;

    print_norm_macros(nraw_name, ninputs_name);
    for(l = 1; l < nlayers-1; l++){
    printf("#define NH%d%s %d\n", l, class_exten, layers[l]-1);
    }
    printf("#define %s %d\n", noutputs_name, noutputs);
    printf("\n");
}    

void
clear_macros(void)
{
    int l;

    clear_norm_macros();
    for(l = 1; l < nlayers-1; l++){
    printf("#undef NH%d%s\n", l, class_exten);
    }
    printf("#undef %s\n", noutputs_name);
    printf("\n");
}    

void
print_classify_code(int model)
{
    int l;
    float **weights, sigm_param;
    char left_arg[8], right_arg[8], matrix_name[16];
    char left_name[64], right_name[64];

    printf("int %s (inputs, outputs)\n", classify_name);
    printf("   float *inputs,*outputs;\n");
    printf("{\n");

    printf("  int n, best, j, k;\n");
    for(l = 1; l < nlayers-1; l++){
    printf("  float h%d[NH%d%s];\n", l, l, class_exten);
    }

    printf("  float y[%s];\n", noutputs_name);
    if(cost_func == MAX_LIKE) printf("  float out_sum;\n");
    if(adjust_test){
    print_vector_initialization("new_priors_list", &test_priors_list[1],
                    noutputs, noutputs_name);
    print_vector_initialization("old_priors_list", train_priors_list,
                    noutputs, noutputs_name);
    printf("  float total;\n");
    }
    
    for(l = 1; l < nlayers; l++){
    if(l == 1){
        sprintf(right_arg, "x");
        strcpy(right_name, ninputs_name);
    } else {
        sprintf(right_arg, "h%d", l-1);
        sprintf(right_name, "NH%d%s", l-1, class_exten);
    }
    strcat(right_name, "+1");
    if(l == nlayers-1){
        sprintf(left_arg, "y");
        strcpy(left_name, noutputs_name);
    } else {
        sprintf(left_arg, "h%d", l);
        sprintf(left_name, "NH%d%s", l, class_exten);
    }
    strcat(left_name, "+1");
    sprintf(matrix_name, "%s_%s", right_arg, left_arg);
    
    weights = *(float ***)mlp_param(model, "weights", l);
    print_matrix_initialization(matrix_name, weights, layers[l],
                    layers[l-1], left_name, right_name);
    }

    /* copy and normlize inputs */
    print_copy_inputs();

    printf("\n  /* normalize loaded data */\n");
    printf("  %s(x);\n", normalize_name);

    /* write out classification */
    for(l = 1; l < nlayers; l++){
    if(l == 1){
        printf("\n  /* calculate hidden layer outputs */\n");
        strcpy(right_arg, "x");
        strcpy(right_name, ninputs_name);
    } else {
        sprintf(right_arg, "h%d", l-1);
        sprintf(right_name, "NH%d%s", l-1, class_exten);
    }
    if(l == nlayers-1){
        printf("\n  /* calculate outputs */\n");
        strcpy(left_arg, "y");
        strcpy(left_name, noutputs_name);
    } else {
        sprintf(left_arg, "h%d", l);
        sprintf(left_name, "NH%d%s", l, class_exten);
    }

    /* get weights and sigmoids for this layer */
    sigm_type = *(int *)mlp_param(model, "sigm_type", l);
    sigm_param = *(float *)mlp_param(model, "sigm_param", l);

    printf("  for(j = 0; j < %s; j++){\n", left_name);
    printf("    %s[j] = %s_%s[j][%s];\n",
           left_arg, right_arg, left_arg, right_name);
    printf("    for(k = 0; k < %s; k++){\n", right_name);
    printf("      %s[j] += %s[k] * %s_%s[j][k];\n",
           left_arg, right_arg, right_arg, left_arg);
    printf("    }\n  }\n");

    if(l == nlayers-1)
      printf("  /* take output sigmoid */\n");
    else
      printf("  /* take sigmoids */\n");
    printf("  for(j = 0; j < %s; j++){\n", left_name);
    if(l == nlayers-1 && cost_func == MAX_LIKE){
        printf("    %s[j] = exp(%f * %s[j]);\n",
           left_arg, sigm_param, left_arg);
    } else {
        switch(sigm_type){
          case SIGM:
        printf("    %s[j] = 1. / (1. + exp(- %f * %s[j]));\n",
               left_arg, sigm_param, left_arg);
        break;
          case SYM_SIGM:
        printf("    %s[j] = 2. / (1. + exp(- %f * %s[j])) - 1.;\n",
               left_arg, sigm_param, left_arg);
        break;
          case STEP:
        printf("    %s[j] = (%s[j] > 0.) ? 1. : 0.;\n",
               left_arg, left_arg);
        break;
          case LINEAR:
        printf("    /* linear sigmoid does not change %s */\n",
               left_arg);
        break;
        }
    }
    printf("  }\n");

    if(l == nlayers -1 && cost_func == MAX_LIKE){
        printf("  /* normalize outputs */\n");
        printf("  out_sum = y[0];\n");
        printf("  for(n = 1; n < %s; n++)\n", noutputs_name);
        printf("    out_sum += y[n];\n");
        printf("  for(n = 0; n < %s; n++)\n", noutputs_name);
        printf("    y[n] /= out_sum;\n");
    }
    }

    printf("\n  /* copy outputs */\n");
    printf("  for(n = 0; n < %s; n++)\n", noutputs_name);
    printf("    outputs[n] = y[n];\n");
    if(adjust_test){
    printf("\n  /* scale outputs by priors */\n");
    printf("  for(i = 0; i < %s; i++){\n", noutputs_name);
    printf("    outputs[i] *= new_priors_list[i];\n");
    printf("    outputs[i] /= old_priors_list[i];\n");
    printf("  }\n");
    printf("  /* rescale the new outputs to sum to 1 again */\n");
    printf("  total = 0;\n");
    printf("  for(i = 0; i < %s; i++) total += outputs[i];\n",
           noutputs_name);
    printf("  for(i = 0; i < %s; i++) outputs[i] /= total;\n",
           noutputs_name);
    }
    printf("\n  /* find highest output */\n");
    printf("  for(best = n = 0; n < %s; n++)\n", noutputs_name);
    printf("    if(outputs[best] < outputs[n]) best = n;\n");
    printf("  return(best);\n");
    printf("}\n");
}
