/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#include "mlp_pack.h"
char *param_fname = "";
static ARG argtab[] = 
{
    {"model_file", STRING, (int *)&param_fname, "mlp model to be translated"},
};
#define TABSIZE (sizeof(argtab)/sizeof(ARG))

char *i2A(int i, char *A);
void print_delta_model(int model, int normalization);

int
main(int argc, char **argv)
{
    FILE *param_file;
    int model, norm_model;
    int normalization;

    argc = nnargs(argc, argv, argtab, TABSIZE);

    param_file = nn_fopen(param_fname, "r", "for read of model file");
    norm_model = norm_read_paramfile(param_file, &normalization);
    if (model < 0)
      fatal_error("%s: read error %d %s %s", argv[0], model,
          "on normalization parameter file", param_fname);
    
    model = mlp_read_model(param_file);
    if (model < 0)
      fatal_error("%s: read error %d on model file %s",
          argv[0], model, param_fname);
    nn_fclose(param_file, param_fname, "for read of model file");

    printf("/* %s */\n", param_fname);
    print_delta_model(model, normalization);

    /************* free space *************/
    norm_free();
    mlp_free(model);

    exit(0);
}

void
print_delta_model(int model, int normalization)
{
    int ninputs, noutputs, nlayers, *layers, cost_func, *features;
    int l, h, prev_n, cur_n, m, n, i;
    float **weights, sigm_param, *means, *sdevs, **transform_matrix;
    int sigm_type, norm_type, norm_ninputs, sample = 0;
    char left_arg[8], right_arg[8];
    int starti, startj;

    /* get general parameters */
    ninputs = *(int *)mlp_param(model, "ninputs", 0);
    noutputs = *(int *)mlp_param(model, "noutputs", 0);
    nlayers = *(int *)mlp_param(model, "nlayers", 0);
    layers = *(int **)mlp_param(model, "layers", 0);
    cost_func = *(int *)mlp_param(model, "cost_func", 0);
    features = *(int **)mlp_param(model, "features", 0);
    norm_type = *(int *)norm_param("normalization");
    norm_ninputs = *(int *)norm_param("ninputs");
    if(norm_ninputs == 0) norm_ninputs = ninputs;
    means = *(float **)norm_param("means");
    sdevs = *(float **)norm_param("sdevs");
    transform_matrix = *(float ***)norm_param("transform");

    /* print description of variables */
    switch (norm_type){
      case 0:
    printf("/* norm: NONE */\n");
    break;
      case 1:
    printf("/* norm: SIMPLE */\n");
    break;
      case 2:
    printf("/* norm: PCA */\n");
    break;
      case 3:
    printf("/* norm: LDA */\n");
    break;
    }
    if(ninputs != norm_ninputs){
    printf("/* %d features", ninputs);
    if(features[0] != 0){
        printf(": ");
        for(i = 1; i <= features[0]; i++) printf("%d ", features[i]);
    }
    printf("*/\n");
    } else {
    printf("/* %d (all) features */\n", ninputs);
    }
    switch(cost_func){
      case SQ_ERR:
    printf("/* cost_func: SQ_ERR */\n");
    break;
      case CROSS_ENTROPY:
    printf("/* cost_func: CROSS_ENTROPY */\n");
    break;
      case MAX_LIKE:
    printf("/* cost_func: MAX_LIKE */\n");
    break;
      case PC_PROC:
    printf("/* cost_func: PC_PROC */\n");
    break;
      case CFOM:
    printf("/* cost_func: CFOM */\n");
    break;
    }
    switch(sigm_type){
      case SIGM:
    printf("/* sigm: SIGM */\n");
    break;
      case SYM_SIGM:
    printf("/* sigm: SYM_SIGM */\n");
    break;
      case STEP:
    printf("/* sigm: STEP */\n");
    break;
      case LINEAR:
    printf("/* sigm: LINEAR */\n");
    break;
    }
    printf("/* net: %d", layers[0]-1);
    for(i = 1; i < nlayers; i++) printf(",%d", layers[i]-1);
    printf(" */\n");

    if(ninputs != norm_ninputs && features[0] != 0){
    sample = 1;
    }
    
    printf("\n  /* raw inputs in");
    for(i = 0; i < ninputs; i++){
    printf(" %s", i2A(i, left_arg));
    }
    printf(" */\n");
    printf("\n  /* load inputs */\n");
    starti = norm_ninputs + noutputs;
    for(i = 0; i < ninputs; i++){
    if(sample) n = features[i];
    else n = i;
    switch(norm_type){
      case 0:
        printf("%s = %s\n", i2A(i+starti, left_arg), i2A(n, right_arg));
        break;
      case 1:
        printf("%s = (%s - %.3f) / %.3f\n", i2A(i+starti, left_arg),
           i2A(n, right_arg), means[n], sdevs[n]);
        break;
      case 2:
      case 3:
        printf("%s = (%.3f * %s)\n", i2A(i+starti, left_arg),
           transform_matrix[n][0], i2A(0, right_arg));
        for(m  =  1; m < norm_ninputs; m++){
        printf("%s = %s + (%.3f * %s)\n",
               i2A(i+starti, left_arg), i2A(i+starti, left_arg),
               transform_matrix[n][m], i2A(m, right_arg));
        }
        printf("\n");
        break;
    }
    }
    for(l = 1; l < nlayers; l++){
    if(l == nlayers-1){
        printf("\n  /* calculate outputs */\n");
    } else if(l == 1){
        printf("\n  /* calculate hidden layer outputs */\n");
    }
    weights = *(float ***)mlp_param(model, "weights", l);
    sigm_type = *(int *)mlp_param(model, "sigm_type", l);
    sigm_param = *(float *)mlp_param(model, "sigm_param", l);
    prev_n = layers[l-1] -1;
    cur_n = layers[l] - 1;
    startj = starti;
    starti += prev_n;
    for(h = 0; h < cur_n; h++){
        printf("%s = %.3f\n", i2A(h+starti, left_arg), weights[h][prev_n]);
        for(n = 0; n < prev_n; n++){
        printf("%s = %s + (%s * %.3f)\n",
               i2A(h+starti, left_arg), i2A(h+starti, left_arg),
               i2A(n + startj, right_arg), weights[h][n]);
        }
        printf("\n");
    }
    startj = starti;
    if(l != nlayers - 1){
        printf("\n  /* take sigmoids */\n");
        starti += cur_n;
        for(h = 0; h < cur_n; h++){
        switch(sigm_type){
          case SIGM:
            printf("%s = 1. / (1. + Exp(- %.3f * %s))\n",
               i2A(h+starti, left_arg), sigm_param,
               i2A(h+startj, right_arg));
            break;
          case SYM_SIGM:
            printf("%s = (2. / (1 + Exp(- %.3f * %s))) -1.\n",
               i2A(h+starti, left_arg), sigm_param,
               i2A(h+startj, right_arg));
            break;
          case STEP:
            printf("%s = (%s > 0.)\n",
               i2A(h+starti, left_arg), i2A(h+startj, right_arg));
            break;
          case LINEAR:
            printf("%s = %s\n",
               i2A(h+starti, left_arg), i2A(h+startj, right_arg));
        }
        }
    }
    }
    l = nlayers - 1;
    cur_n = layers[l] - 1;
    weights = *(float ***)mlp_param(model, "weights", l);
    sigm_type = *(int *)mlp_param(model, "sigm_type", l);
    sigm_param = *(float *)mlp_param(model, "sigm_param", l);
    startj = starti;
    if(cost_func == MAX_LIKE){
    starti += cur_n;
    } else {
    starti = norm_ninputs;
    }
    for(h = 0; h < cur_n; h++){
    if(cost_func != MAX_LIKE){
        switch(sigm_type){
          case SIGM:
        printf("%s = 1. / (1. + Exp(- %.3f * %s))\n",
               i2A(h+starti, left_arg), sigm_param, 
               i2A(h+startj, right_arg));
        break;
          case SYM_SIGM:
        printf("%s = (2. / (1 + Exp(- %.3f * %s))) -1.\n",
               i2A(h+starti, left_arg), sigm_param, 
               i2A(h+startj, right_arg));
        break;
          case STEP:
        printf("%s = (%s > 0.)\n",
               i2A(h+starti, left_arg), i2A(h+startj, right_arg));
        break;
          case LINEAR:
        printf("%s = %s\n",
               i2A(h+starti, left_arg), i2A(h+startj, right_arg));
        }
    } else {
        printf("%s = Exp(%.3f * %s)\n",
           i2A(h+starti, left_arg), sigm_param, 
           i2A(h+startj, right_arg));
    }
    }
    if(cost_func == MAX_LIKE){
    startj = starti;
    starti += cur_n;
    printf("\n  /* normalize outputs */\n");
    printf("%s = %s\n", i2A(starti, left_arg), i2A(startj, right_arg));
    for(n = 1; n < noutputs; n++){
        printf("%s = %s + %s\n",
           i2A(starti, left_arg), i2A(starti, left_arg),
           i2A(n + startj, right_arg));
    }
    printf("\n");
    starti++;
    for(n = 0; n < noutputs; n++){
        printf("%s = %s / %s\n",
           i2A(n+starti, left_arg), i2A(n+startj, right_arg),
           i2A(noutputs+startj, right_arg));
    }
    }
}

char *i2A(int i, char *A)
{
    if(i > 254){
    fatal_error("Too many nodes for delta graph (%d)\n%s",
            i, "Only 127 total nodes can be calculated");
    }
    if(i < 26){
    A[0] = 'A' + i;
    A[1] = '\0';
    } else {
    A[0] = 'A' + i/26 - 1;
    A[1] = 'A' + i%26;
    A[2] = '\0';
    }
    return A;
}
