/***--------------------------------------------------------------------------
* NAME:         mlp_main.c
* PURPOSE:      Entry routine for MultiLayer Perceptron
* USAGE:
*           mlp -<flags>
* EXAMPLE:  mlp --                * shows all available flags
*
* SEE ALSO:     mlp.c mlp_pack.c mlp.run
*
* GLOBALS:  <flags> defined in mlp_args.h
*
* DESCRIPTION:
*       sets up arguments for the routine mlp()
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/mlp/mlp_main.c,v 11.5 2001/08/23 20:12:31 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/mlp/mlp_main.c,v $
* CREATED 10-Jan-91 dan             MIT LINCOLN LABORATORY
*--------------------------------------------------------------------------**/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char     rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/mlp/mlp_main.c,v 11.5 2001/08/23 20:12:31 craig Exp $";



/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#define NO_CV_TABLE
#include <cv_args.h>
#include <gen_class_args.h>
#include "mlp_args.h"       /* parameter definitions */
#include "mlp.h"

int
main(int argc, char **argv)
{
    char    *comment;
    int     use_splits;
    int     log_outputs = 0;
    int i, roc_cost = 0, roc_target = 0;
    float roc_lower = 0, roc_upper = 1;
    /* arguments for reading defaults file */
    DEFAULTS_FILE_STRUCT *df_args;
    /* argument table with all entries */
    ARG *argtab;
    int tabsize;

    
    /******************* read argument list **********************/
    init_nnsubs(argc, argv, rcsid);
    /* get arguments */
    nnargs_combine_argtabs(gen_tab, GEN_TABSIZE, mlp_tab, MLP_TABSIZE,
               &argtab, &tabsize);
    argc = nnargs(argc, argv, argtab, tabsize);
    
    print_log_header();
            
    /************* read defaults and set input data parameters ********/
    /* read the defaults file */
    if(strcmp(desc_fname, "") == 0){ /* desc_fname is unset */
    fatal_error("%s: -fdescribe must be set", argv[0]);
    }
    /* set the number of inputs and outputs and read the labels,
       allocating the space */
    df_args = (DEFAULTS_FILE_STRUCT *)
      nn_malloc(sizeof(DEFAULTS_FILE_STRUCT), "for the defaults file struct");
    read_defaults_file(path_build(data_path, desc_fname), df_args);
    if(df_args->noutputs == 0)
      fatal_error("noutputs in %s must be greater than 0", desc_fname);
    if(df_args->ninputs == 0)
      fatal_error("ninputs in %s must be greater than 0", desc_fname);

    /* set the number of patterns to no more than the number of lines
       in the data file */
    if(strcmp(data_fname, "") == 0){ /* data_fname is unset */
    fatal_error("%s: -finput must be set", argv[0]);
    }
    check_file_length(path_build(data_path, data_fname), &npatterns);
    if(npatterns == 0)
      fatal_error("%s: Empty data file %s", argv[0],
          path_build(data_path, data_fname));
    /* check the data base type */
    if (df_args->db_type != CLASS_DATA)
      fatal_error("class data required, check defaults file");
    

    /***************** check feature selection flags *******************/
    if(verbosity > 3){
    if(ninputs != 0)
      warning("-ninputs flag has no effect in %s. -nodes flag is used",
          argv[0]);
    }
    ninputs = nnodes[1];
    if (ninputs > df_args->ninputs)
       ninputs = df_args->ninputs;
    ninputs = fix_features(df_args->ninputs, ninputs, features);
    nnodes[1] = ninputs;    /* update nnodes in case values changed */

    if(df_args->noutputs != nnodes[nnodes[0]]){
    warning("%s (%d) %s (%d)\n%s",
        "Number of output nodes specified", nnodes[nnodes[0]], 
        "not equal to number of classes in data base",df_args->noutputs,
        "Number of output nodes set to number of classes");
    nnodes[nnodes[0]] = df_args->noutputs;
    }

    /*************** set up the cross validation arguments ***********/
    if(train_flag && cross_valid > 2){
    warning("%s\n%s\n","-train and -cross_valid are both set.",
        " -cross_valid will be ignored");
    cross_valid = 0;
    }
    use_splits =
      fix_cross_valid_args(&cross_valid, &cv_nfolds, &cv_nsplits, cv_splits,
               cv_train_mask, cv_test_mask, cross_valid_from_file,
               path_build(data_path, cross_valid_fname));

    /********* fix up priors adjustment arguments *********************/
    fix_priors(adjust_train, train_uniform_priors, &train_priors_list,
           adjust_test, test_uniform_priors, &test_priors_list,
           train_flag, cross_valid, df_args->noutputs);

    /********** check MLP args *****************/
    if (nepochs < 1) fatal_error("epochs must be greater than 0");

    /* check the ofunction and cost_func flags */
    if(cost_func == MAX_LIKE || cost_func == CROSS_ENTROPY)
      if(ofunction != 0){
      warning("%s %d %s %d.\n%s %d",
          "mlp_main: sigmoid function type", ofunction,
          "is not allowed with cost_func", cost_func,
          "changing the output sigmoid to type", SQ_ERR);
      ofunction = SQ_ERR;
      }
    /* a bit of a kludge to make the initialization work */
    if(cost_func == ROC_COST){
    cost_param = df_args->noutputs;
    }
    
    /* check the nnodes for any zeros, that is, a hidden layer
       with no inputs or no outputs */
    for(i = 2; i < nnodes[0]; i++){/* don't check the input or output layer */
    if(nnodes[i] == 0)
      fatal_error("mlp_main: -nnodes is bad. 0 nodes in a hidden layer");
    }

    /* work out the etta_list flags.  by the end, etta_list should be
       filled either with what the user asked for or with etta on each level
       */
    /* I would prefer to do this without a goto,
       but this seemed a little cleaner for now */
    if(etta_list != NULL){
    if(use_etta_list){
        if(((int)etta_list[0]) < nnodes[0]-1){
        warning("etta_list is too short. Using etta for all layers");
        } else goto etta_list_ok;  /* etta list is all right */
    }
    /* we need to free the old etta_list so we can build the new one */
    deallocate_vector(etta_list);
    } else {
    if(use_etta_list)
      warning("etta_list has not been set.  Using etta for all layers");
    }
    /* if etta list is ok, we should have jumped ahead a little bit */
    etta_list = allocate_vector(nnodes[0]);
    for(i = 1; i < nnodes[0]; i++) etta_list[i] = etta;
    etta_list[0] = (float)nnodes[0]-1;

  etta_list_ok:

    /* work out the sigmoid_list flags.  by the end, sigmoid list should be
       filled either with what the user asked for or with sigmoid_param
       on each level */
    if(sigmoid_param_list != NULL){
    if(use_sigmoid_param_list){
        if((int)sigmoid_param_list[0] < nnodes[0]-1){
        warning("sigmoid_param_list is too short.  Using sigmoid_param for all layers");
        } else goto sigmoid_list_ok;
    }
    /* free the old list so we can build a new one */
    deallocate_vector(sigmoid_param_list);
    } else {
    if(use_sigmoid_param_list)
      warning("sigmoid_param_list has not been set.  Using sigmoid_param for all layers");
    }
    sigmoid_param_list = allocate_vector(nnodes[0]);
    for(i = 1; i < nnodes[0]; i++) sigmoid_param_list[i] = sigmoid_param;
    sigmoid_param_list[0] = (float)nnodes[0]-1;

  sigmoid_list_ok:

    /************** make and print header comment *******************/
    comment = make_comment_string(nnargs_get_flags(argtab, tabsize));
    if (verbosity > FLAGS_VERBOSITY) {
    printf("%s", comment);
    fflush(stdout);
    }
        
    srandom(seed);      /* initialize random numbers */

    /************* call the selected driver ***************/
    if(train_flag)
      mlp_train(create_flag, df_args->db_type, df_args->ninputs, ninputs,
        features, df_args->noutputs,
        normalize, npatterns, verbosity, verror, nepochs,
        random_flag, data_path, data_fname, norm_fname, exp_path,
        error_fname, param_fname, df_args->output_labels, comment,
        adjust_train, priors_npatterns, &train_priors_list[1],
        etta_list, alpha, tolerance,
        decay, kappa, epsilon, sigmoid_param_list, nnodes, batch,
        cost_func, cost_param, step_change, hfunction, ofunction,
        max_magnitude, log_outputs, etta_nepochs);
    else if (cv_nfolds > 1)
      mlp_cross_valid(df_args->db_type, df_args->ninputs, ninputs, features,
              df_args->noutputs,
              normalize, npatterns, verbosity, verror, nepochs,
              random_flag, cv_nfolds, cv_nsplits, use_splits,
              cv_splits, &cv_train_mask[1], &cv_test_mask[1],
              data_path, data_fname, norm_fname, exp_path, error_fname,
              param_fname, df_args->output_labels, comment,
              (roc_cost? ROC_COST : cost_func), roc_target,
              roc_lower, roc_upper, adjust_train,
              priors_npatterns, &train_priors_list[1], adjust_test,
              &test_priors_list[1], cv_random, etta_list, alpha,
              tolerance, decay, kappa, epsilon, sigmoid_param_list,
              nnodes,
              batch, cost_func,  cost_param, step_change, hfunction,
              ofunction, max_magnitude, log_outputs, etta_nepochs);
    else
      mlp_test(create_flag, df_args->db_type, df_args->ninputs, ninputs,
           features, df_args->noutputs,
           npatterns, verbosity, verror, data_path, data_fname,
           exp_path, error_fname, param_fname, df_args->output_labels,
           adjust_test, &test_priors_list[1], nnodes, cost_func,
           cost_param, log_outputs);

    print_log_footer();

    nnargs_free_argtab(argtab, tabsize);
    /* free the vectors allocated when the defaults file was read */
    free_defaults_file_struct(df_args);

    exit_nnsubs();
    exit(0);
}
