/***--------------------------------------------------------------------------
* NAME:         mlp_pack.c
* PURPOSE:      Functions specific to mlp.c
*
* SEE ALSO: mlp.c mlp_main.c
*
* GLOBALS:
*
* DESCRIPTION:
*    Functions in this file
*
* ALGORITHM:
* REFERENCE:
*
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/mlp/mlp_seq_pack.c,v 11.3 2001/08/23 20:12:32 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/mlp/mlp_seq_pack.c,v $
* CREATED: 12-Dec-90 lck                             MIT LINCOLN LABORATORY
*--------------------------------------------------------------------------*/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */



/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#include <math.h>
#include "mlp_pack.h"

#define mlp_seq_pack_h_internal_types
#include "mlp_seq_pack.h"

                /* constant definitions */
#define MODEL_STORE_SIZE 10
#define PEQ(y) (0==strcmp(param_name,y))

                /* Internal Globals */
static MLP_SEQ_PARAMS **mlpp;/* array of parameters for each model */
static int      max_models = 0; /* current length of mlpp */

#define SMALLEST 1.e-30

/*************************************************************************
* NAME: mlp_seq_build(noutputs)
*
* DESCRIPTION:
*       Allocate storage for model parameters.
*
*************************************************************************/
int
mlp_seq_build(int noutputs)
{
    MLP_SEQ_PARAMS *mpp;    /* temporary pointer to mlpp[] */
    register int    new_model;

    new_model =
      model_get_number((void ***)&mlpp, &max_models, sizeof(MLP_SEQ_PARAMS));

    mpp = mlpp[new_model];

    /* set mode flags */
    mpp->noutputs = noutputs;

    /* allocate space for vectors */
    mpp->seq_sums = allocate_vector(noutputs);

    return new_model;
}
/*************************************************************************
* NAME: mlp_seq_init(model)
* DESCRIPTION:
*       Initialize model parameters not set in build
*
* EXAMPLE:
*
*************************************************************************/
void
mlp_seq_init(int model, int mlp_model, float clip_floor)
{
    int             i;
    MLP_SEQ_PARAMS *mpp;    /* temporary pointer to mlpp[] */

    mpp = mlpp[model];

    mpp->mlp_model = mlp_model;
    mpp->clip_floor = clip_floor;
    for(i = 0; i < mpp->noutputs; i++) mpp->seq_sums[i] = 0.;
}

/*************************************************************************
* NAME: mlp_seq_free()
* DESCRIPTION:
*       Deallocate storage used in a model
*
* EXAMPLE:
*
*************************************************************************/
void
mlp_seq_free(int model)
{
    MLP_SEQ_PARAMS *mpp;    /* temporary pointer to mlpp[] */

    mpp = mlpp[model];
    /* free vectors and matrices */
    deallocate_vector(mpp->seq_sums);

    /* free structure itself */
    nn_free((char *) mpp, "for mlp parameter structure");
    mlpp[model] = NULL;
}

/************************************************************************
* NAME: mlp_seq_param(model, param_name)
* DESCRIPTION:
*       Takes model number and parameter name.
*       Returns a pointer to the named parameter.
*
*************************************************************************/
void *
mlp_seq_param(int model, char *param_name)
{
    void           *param_address;
    MLP_SEQ_PARAMS *mpp;    /* temporary pointer to mlpp[] */

    mpp = mlpp[model];
    if (PEQ("seq_sums"))
      param_address = &mpp->seq_sums;
    else {
    warning("mlp_seq_param: Parameter %s not defined", param_name);
    param_address = NULL;
    }
    return param_address;
}

void
mlp_seq_batch_classify(int model, float *inputs)
{
    int i;
    MLP_SEQ_PARAMS *mpp;    /* temporary pointer to mlpp[] */
    float *outputs;

    mpp = mlpp[model];
    mlp_eval(mpp->mlp_model, inputs);
    outputs = *(float **)mlp_param(mpp->mlp_model, "output", 0);

    /* mlp cannot give log outputs yet */
    for(i = 0; i < mpp->noutputs; i++){
    /* clip the outputs */
    if(outputs[i] < mpp->clip_floor) outputs[i] = mpp->clip_floor;
    if(outputs[i] > 1.) outputs[i] = 1.;
    mpp->seq_sums[i] += log10((double)outputs[i]);
    }
}

int
mlp_seq_update_classify(int model)
{
    MLP_SEQ_PARAMS *mpp;
    mpp = mlpp[model];
    return(max_pick(mpp->seq_sums, mpp->noutputs));
}
    
