/***--------------------------------------------------------------------------
* NAME:         mlpm_main.c
* PURPOSE:      Entry routine for MultiLayer Perceptron which does mapping
* USAGE:
*       	mlp_map -<flags>
* EXAMPLE: 	mlp_map --                * shows all available flags
*
* SEE ALSO: 	mlp.c mlp_pack.c mlp.run
*
* GLOBALS: 	<flags> defined in mlp_args.h
*
* DESCRIPTION:
*		sets up arguments for the routine mlp()
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/mlp/mlpm_main.c,v 11.4 2001/08/23 20:12:33 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/mlp/mlpm_main.c,v $
* CREATED 13-Jul-92 lck				MIT LINCOLN LABORATORY
*--------------------------------------------------------------------------**/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char     rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/mlp/mlpm_main.c,v 11.4 2001/08/23 20:12:33 craig Exp $";



/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <nnlib.h>
#include <cv_args.h>
/* we need to make gen_map_args.h !!! */
#include "mlpm_args.h"		/* parameter definitions */
#include "mlpm.h"

int
main(int argc, char **argv)
{
    char	*comment, *tmp_path;
    FILE	*cv_file;
    int		nargc, i, use_splits;
    int		ninputs, noutputs, map_flag = 1;
    char **vlabels;
    int *vfeatures;

    init_nnsubs(argc, argv, rcsid);
			/* get arguments */
    argc = nnargs_part(argc, argv, length_tab, LEN_TABSIZE);
    ninputs = nnodes[1];
    noutputs = nnodes[nnodes[0]];
    if(nraw > NNARGS_MAX_LABELS){
	vfeatures = allocate_ivector(nraw+1);
	nnargs_set_variable("features", vfeatures, argtab, TABSIZE);
	nnargs_set_list_length("features", nraw+1, argtab, TABSIZE);
    } else {
	vfeatures = &features[0];
    }
    if(noutputs > NNARGS_MAX_LABELS){
	vlabels = (char **)allocate_pvector(noutputs+1);
	nnargs_set_variable("labels", (int *)vlabels, argtab, TABSIZE);
	nnargs_set_list_length("labels", noutputs+1, argtab, TABSIZE);
    } else {
	vlabels = &labels[0];
    }
    argc = nnargs(argc, argv, argtab, TABSIZE);
    
    ninputs = nnodes[1];
    noutputs = nnodes[nnodes[0]];

    print_log_header();

    check_desc_args(path_build(data_path, desc_fname), vlabels, &nraw, 
		    &noutputs, &map_flag, &npatterns);
    if(noutputs == 0) fatal_error("noutputs in %s must be greater than 0",
				  desc_fname);
    if(nraw == 0) fatal_error("ninputs in %s must be greater than 0",
			      desc_fname);

    if (ninputs > nraw)
	ninputs = nraw;
    if (cross_valid < 1) cross_valid = 0;	/* also takes care of < 0 */
    if(cross_valid_from_file){
	tmp_path = path_build(data_path, cross_valid_fname);
	if((cv_file = fopen(tmp_path, "r")) == NULL){
	    warning("cross_valid_file not found: %s", tmp_path);
	} else {
	    nargc = nnargs_from_file(cv_file, tmp_path, cross_valid_argtab,
				     CROSS_VALID_TABSIZE);
	    nn_fclose(cv_file, tmp_path, "for reading cross_valid splits");
	}
	/* do some argument checking */
	cross_valid_check_args(&cv_nfolds, &cv_nsplits, cv_splits,
			       cv_train_mask, cv_test_mask);
	cross_valid = cv_nfolds;
	use_splits = 1;
    } else {
	use_splits = 0;
	cv_nfolds = cv_nsplits = cross_valid;
	build_cross_valid_masks(cross_valid, cv_train_mask, cv_test_mask);
    }

    ninputs = fix_features(nraw, ninputs, vfeatures);

    /********** check MLP args *****************/
    /* work out the etta_list flags.  by the end, etta_list should be
       filled either with what the user asked for or with etta on each level
       */
    /* I would prefer to do this without a goto,
       but this seemed a little cleaner for now */
    if(etta_list != NULL){
	if(use_etta_list){
	    if(((int)etta_list[0]) < nnodes[0]-1){
		warning("etta_list is too short. Using etta for all layers");
	    } else goto etta_list_ok;  /* etta list is all right */
	}
	/* we need to free the old etta_list so we can build the new one */
	deallocate_vector(etta_list);
    } else {
	if(use_etta_list)
	  warning("etta_list has not been set.  Using etta for all layers");
    }
    /* if etta list is ok, we should have jumped ahead a little bit */
    etta_list = allocate_vector(nnodes[0]);
    for(i = 1; i < nnodes[0]; i++) etta_list[i] = etta;
    etta_list[0] = (float)nnodes[0]-1;

  etta_list_ok:

    /* work out the sigmoid_list flags.  by the end, sigmoid list should be
       filled either with what the user asked for or with sigmoid_param
       on each level */
    if(sigmoid_param_list != NULL){
	if(use_sigmoid_param_list){
	    if((int)sigmoid_param_list[0] < nnodes[0]-1){
		warning("sigmoid_param_list is too short.  Using sigmoid_param for all layers");
	    } else goto sigmoid_list_ok;
	}
	/* free the old list so we can build a new one */
	deallocate_vector(sigmoid_param_list);
    } else {
	if(use_sigmoid_param_list)
	  warning("sigmoid_param_list has not been set.  Using sigmoid_param for all layers");
    }
    sigmoid_param_list = allocate_vector(nnodes[0]);
    for(i = 1; i < nnodes[0]; i++) sigmoid_param_list[i] = sigmoid_param;
    sigmoid_param_list[0] = (float)nnodes[0]-1;

  sigmoid_list_ok:

    /************** make and print header comment *******************/
    comment = make_comment_string(nnargs_get_flags(argtab, TABSIZE));

    if (verbosity > FLAGS_VERBOSITY) {
	printf("%s", comment);
	fflush(stdout);
    }
		
    nnodes[1] = ninputs;	    /* update nnodes in case values changed */
    nnodes[nnodes[0]] = noutputs;

    srandom(seed);		/* initialize random numbers */

    /************* call the selected driver ***************/
    if(train_flag){
	printf("training");
	mlpm_train(create_flag, map_flag, nraw, ninputs, vfeatures, noutputs,
		   normalize, npatterns, verbosity, verror, nepochs,
		   random_flag, data_path, data_fname, norm_fname, exp_path,
		   error_fname, param_fname, vlabels, comment, etta_list,
		   alpha, tolerance, decay, kappa, epsilon,
		   sigmoid_param_list, nnodes,
		   batch, etta_type, hfunction, ofunction, max_magnitude,
		   etta_nepochs);
    } else if (cv_nfolds > 1){
	printf("cross valid");
	mlpm_cross_valid(map_flag, nraw, ninputs, vfeatures, noutputs,
			 normalize, npatterns, verbosity, verror, nepochs,
			 random_flag, cv_nfolds, cv_nsplits, use_splits,
			 cv_splits, &cv_train_mask[1], &cv_test_mask[1],
			 data_path, data_fname, norm_fname, exp_path,
			 error_fname, param_fname, vlabels, comment, etta_list,
			 alpha, tolerance, decay, kappa, epsilon,
			 sigmoid_param_list, nnodes, batch, etta_type,
			 hfunction, ofunction, max_magnitude, etta_nepochs);
    } else{
	printf("testing");
	mlpm_test(create_flag, map_flag, nraw, ninputs, vfeatures,
		  noutputs, npatterns, verbosity, verror, data_path,
		  data_fname, exp_path, error_fname, param_fname,
		  vlabels, nnodes);
    }
    print_log_footer();

    exit_nnsubs();
    exit(0);
}
