/***--------------------------------------------------------------------------
 * NAME:		mlpm_test.c
 * PURPOSE:	testing driver for mapping MultiLayer Perceptron
 *
 * SEE ALSO: 	mlp/mlp_pack.c
 *
 * BUGS AND LIMITATIONS:
 *
 * $Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/mlp/mlpm_test.c,v 11.4 2001/08/23 20:12:33 craig Exp $
 * $Source: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/mlp/mlpm_test.c,v $
 * CREATED: 13-Jul-92 lck			MIT LINCOLN LABORATORY
 ****************************************************************************/

/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */



/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include "mlp_pack.h"

#define MAXSTR (256)

static char     prog_name[MAXSTR];	/* static variables */

void
mlpm_test(int create_flag, int map_flag, int nraw, int ninputs,
	  int *features, int noutputs, int npatterns, int verbosity,
	  int verror, char *data_path, char *data_fname, char *exp_path,
	  char *error_fname, char *param_fname, char **labels,
	  int *nnodes)
{
    /* Local variables */
    int		model, err_model, rv, normalization=0, log_outputs = 0;
    int		post_sum_to_one = 0;
    int		n_read, i;
    float	*inputs, *outputs, *desired_outputs;
    char	data_fullname[MAXSTR], *tmp_path;
    FILE	*data_file, *error_file, *param_file;
    
    int		nlayers;	/* number of layers */
    int		layers[20];	/* number of nodes in each layer */
    
    strcpy(prog_name, get_program_name());
    
    /* put together the full path to the data file*/
    strcpy(data_fullname, path_build(data_path, data_fname));
    
    nlayers = nnodes[0];	/* Put the number of nodes in each layer in
				 * nlayers */
    for (i = 0; i < nlayers; i++)
      layers[i] = nnodes[i + 1];
    
    tmp_path = path_build(exp_path, param_fname);
    param_file = nn_fopen(tmp_path, "r", "for read of model file");
    rv = norm_read_paramfile(param_file, &normalization);
    if (rv < 0)
      fatal_error("%s: read error %d %s %s", prog_name, rv,
		  "on normalization parameter file", tmp_path);
    
    model = mlp_read_model(param_file);
    if (model < 0)
      fatal_error("%s: read error %d on model file %s",
		  prog_name, model, tmp_path);
    nn_fclose(param_file, tmp_path, "for read of model file");
    
    /* use features from the model */
    features = *(int **) mlp_param(model, "features", 0);
    ninputs = *(int *) mlp_param(model, "ninputs", 0);
    outputs = *(float **) mlp_param(model, "output", 0);
    
    display_model_params (model, ninputs, normalization, features, verbosity);
    
    if (verror > 0) {	/* open file for mapping errors */
	if(create_flag) 
	  error_file = nn_fopen(path_build(exp_path, error_fname),
				"w", "for mapping errors");
	else
	  error_file = nn_fopen(path_build(exp_path, error_fname),
				"a", "for mapping errors");
    }
    /* do remaining initializations and allocations */
    err_model = report_build(verbosity, verror, noutputs);
    report_init(err_model, map_flag, ninputs, SQ_ERR, log_outputs, post_sum_to_one);
    init_cost(SQ_ERR, 1.);

    /* open data file */
    data_file = nn_fopen(data_fullname, "r", "for test data");
    if (verbosity > 1)		
      printf("\nReading %s\n", data_fullname);
    
    inputs = allocate_vector(ninputs);
    desired_outputs = allocate_vector(noutputs);
    
    /* read and map input from data file */
    for (n_read = 0; n_read < npatterns; n_read++) {
	/* read input vector */
	if(get_map_line(data_file, data_fullname, inputs, desired_outputs,
			nraw, ninputs, noutputs, normalization, features)
	   == EOF) break;

	/* classify pattern and report errors */
	mlp_eval(model, inputs);
	report_map_trial(err_model, error_file,
			 inputs, outputs, desired_outputs, n_read);
    }
    deallocate_vector(inputs);
    
    /* finish error reporting and close error file */
    report_overall(err_model, stdout, error_fname, labels);
    report_free(err_model);
    
    if (verror > 0) {
	nn_fclose(error_file, path_build(exp_path, error_fname),
		  "for error file");
    }
    nn_fclose(data_file, data_fullname, "for test data");
    /* free space and close files */
    mlp_free(model);
    norm_free();
}
