/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
                                /* nbayes_augment.c */
/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <math.h>
#define nbayes_pack_h_internal_types
#include "nbayes_pack.h"

static char *param_fname = "";
static char *count_fname = "";
static float weight = -1.;
int ignore_unseen = 0;
int do_unseen = 0;
int unity_laplace = 0;

static ARG argtab[] = 
{
    {"original_model", STRING, (int *)&param_fname, "model to be converted"},
    {"count_model", STRING, (int *)&count_fname, "model with counts"},
    {"weight", FLOAT, (int *)&weight, "counts are weight*orig"},
    {"ignore_unseen", FLAG, &ignore_unseen, "Ignore test inputs with unknown values. Default is use laplace correction"},
    {"unity_laplace", FLAG, &unity_laplace, "Laplace correction is 1. Default is 1/Npat"},
};
#define TABSIZE (sizeof(argtab)/sizeof(ARG))

struct shared_params {
  int normalization;
  int *features;
  int nraw;
  int ninputs;
  int noutputs;
};

void calc_counts(int orig_model);
int read_model_file(char *param_fname, char *prog_name);
void write_model_file(char *orig_name, int model);

int
main(int argc, char **argv)
{
  int orig_model;

    /* process arguments */
    argc = nnargs(argc, argv, argtab, TABSIZE);

    do_unseen = ignore_unseen ? 0 : 1;

    /* read in parameters */
    orig_model = read_model_file(param_fname, argv[0]);

    /* figure out combined likelihoods */
    calc_counts(orig_model);

    /* write model with counts for probs */
    write_model_file(count_fname, orig_model);

    exit(0);
}

int
read_model_file(char *param_fname, char *prog_name)
{
    int   model, rv, normalization;
    int	  nraw;
    FILE *param_file;
    int *features, ninputs;

    param_file = nn_fopen(param_fname, "r", "for read of model file");
    rv = norm_read_paramfile(param_file, &normalization);
    if (rv < 0)
      fatal_error("%s: read error %d %s %s", prog_name, rv,
		  "on normalization parameter file", param_fname);

    rv = feature_read_paramfile(param_file, &features, &nraw, &ninputs);
    if (rv < 0)
      fatal_error("%s: read error %d %s %s", prog_name, rv,
		  "on feature selection parameters in file", param_fname);
    
    model = nbayes_read_model(param_file);
    if (model < 0)
      fatal_error("%s: read error %d on model file %s",
		  prog_name, model, param_fname);
    nn_fclose(param_file, param_fname, "for read of model file");

    return((int) model);
}

void
write_model_file(char *aug_fname, int model)
{
  FILE *param_file;

  param_file = nn_fopen(aug_fname, "w", "for write of augmented model");
  fprintf(param_file, "nbayes model\n");
  nbayes_write_model(model, param_file);
  nn_fclose(param_file, aug_fname, "for write of model file");
}

void calc_counts(int orig_model)
{
  /* retrieve count from models */
  /*  get N, Nc, nbins, and probs.
      undo algebra to get "count". This might not be an integer if
      we have applied weights during a previous augmentation */
  /* p = (Ncij + 1/N)/(Nc + (NBINSi+1)/N) */
  /* Ncij = p*(Nc + (NBINSi+1)/N) - 1/N */
  int i, j, c;
  float *n_class_orig, n_total_orig;
  int *orig_bins;
  float *orig_probs;
  int ninputs, noutputs;
  
  ninputs = *(int *)nbayes_param(orig_model, "ninputs", 0, 0);
  noutputs = *(int *)nbayes_param(orig_model, "noutputs", 0, 0);

  orig_bins = *(int **)nbayes_param(orig_model, "nbins", 0, 0);

  n_class_orig = *(float **)nbayes_param(orig_model, "class_cnt", 0, 0);
  n_total_orig = *(float *)nbayes_param(orig_model, "total_npatterns", 0, 0);

  for(i = 0; i < ninputs; i++){
    for(j = 0; j <= orig_bins[i]; j++){
      orig_probs = *(float **)nbayes_param(orig_model, "prob", i, j);

      for(c = 0; c < noutputs; c++){
	orig_probs[c] = nn_exp10(orig_probs[c]);
	if(unity_laplace){
	  orig_probs[c] = orig_probs[c]*(n_class_orig[c]
					 + orig_bins[i]+do_unseen) - 1.;
	} else {
	  orig_probs[c] = orig_probs[c]*(n_class_orig[c] +
					 (orig_bins[i]+do_unseen)/n_total_orig)
	  - 1./n_total_orig;
	}
	if(weight >= 0){
	  orig_probs[c] *= weight;
	}
      }
    }
  }
}
