/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* nc_class2c.c */
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#define GEN2CVARS
#include <gen_2c_args.h>
#include "nc_class_pack.h"

/* nc_class specific variables */
int dist_type, accum_trials, *classes;
float **exemplars, **inv_vars, *log_det_vars;
char l2dist_name[64];
char gauss_out_name[64];
char ncenters_name[64];

/* function declarations */
int read_model_file(char **argv);
void free_model(int model);
void get_params(int model);
void get_specific_params(int model);
void get_general_params(int model);
void print_comments(void);
void print_declarations(void);
void print_macros(void);
void clear_macros(void);
void print_classify_code(int model);
void print_gauss_out_code(void);
void print_l2dist_code(void);

int
main(int argc, char **argv)
{
    int model;

    argc = nnargs(argc, argv, argtab2c, TABSIZE2C);

    /* check arguments */
    if(strlen(class_suff) > 53){
	fatal_error("suffix (%s) is too long.  use at most 50 characters",
		    class_suff);
    }

    /* set up subroutine and macro names */
    strcpy(classify_name, "classify");
    strcpy(normalize_name, "normalize");
    strcpy(l2dist_name, "l2dist");
    strcpy(gauss_out_name, "gauss_out");
    strcpy(nraw_name, "NRAW");
    strcpy(ninputs_name, "NINPUTS");
    strcpy(noutputs_name, "NCLASSES");
    strcpy(ncenters_name, "NCENTERS");
    if(class_suff[0] != '\0'){
        strcpy(class_exten, "_");
	strcat(class_exten, class_suff);
	strcat(classify_name, class_exten);
	strcat(normalize_name, class_exten);
	strcat(l2dist_name, class_exten);
	strcat(gauss_out_name, class_exten);
	strcat(nraw_name, class_exten);
	strcat(ninputs_name, class_exten);
	strcat(noutputs_name, class_exten);
	strcat(ncenters_name, class_exten);
    } else {
      strcpy(class_exten, "");
    }


    /* read in parameters */
    model = read_model_file(argv);
    get_params(model);

    if(adjust_test)
      warning("%s: This algorithm does not adjust test priors",
	      argv[0]);

    print_comments();
    print_declarations();
    print_macros();
    print_classify_code(model);
    print_normalize_code(normalize_name);
    if(dist_type)
      print_gauss_out_code();
    else
      print_l2dist_code();
    clear_macros();
    free_model(model);
    exit(0);
}

int
read_model_file(char **argv)
{
    int   model, norm_model, normalization;
    FILE *param_file;

    param_file = nn_fopen(param_fname, "r", "for read of model file");
    norm_model = norm_read_paramfile(param_file, &normalization);
    if (norm_model < 0)
      fatal_error("%s: read error %d %s %s", argv[0], norm_model,
		  "on normalization parameter file", param_fname);

    model = nc_class_read_model(param_file);
    if (model < 0)
      fatal_error("%s: read error %d on model file %s",
		  argv[0], model, param_fname);
    nn_fclose(param_file, param_fname, "for read of model file");

    return model;
}

void
free_model(int model)
{
  norm_free();
  nc_class_free(model);
}
void
get_params(int model)
{
    get_specific_params(model);
    get_general_params(model);
    get_norm_params(model, ninputs, features);
}

void
get_specific_params(int model)
{
    accum_trials = *(int *)nc_class_param(model, "ncenters");
    exemplars = *(float ***)nc_class_param(model, "centers");
    classes = *(int **)nc_class_param(model, "classes");
    dist_type = *(int *)nc_class_param(model, "dist_type");
    inv_vars = *(float ***)nc_class_param(model, "inv_variances");
    log_det_vars = *(float **)nc_class_param(model, "log_det_var");
}

void
get_general_params(int model)
{
    ninputs = *(int *)nc_class_param(model, "ninputs");
    noutputs = *(int *)nc_class_param(model, "nclasses");
    features = *(int **)nc_class_param(model, "features");
}

void
print_comments(void)
{
    printf("/* %s */\n", param_fname);
    if(dist_type)
      printf("/* Mahalanobis (Gaussian) Distances */\n");
    else
      printf("/* Euclidian Distances */\n");
    print_norm_comments();
}

void
print_declarations(void)
{
    printf("\n");
    printf("#include <stdio.h>\n");
    printf("#include <math.h>\n");
    printf("#include <values.h>\n");
    printf("#ifndef SQR(x)\n");
    printf("#define SQR(x) ((x) * (x))\n");
    printf("#endif\n");
    if(dist_type){
	printf("\n");
	printf("#ifndef M_LOG10E\n");
	printf("#define M_LOG10E 0.43429448190325182765\n");
	printf("#endif\n");
	printf("\n");
	printf("#ifndef LOG_2PI\n");
	printf("#define LOG_2PI 0.798179868358\n");
	printf("#endif\n");
    }
    printf("\n /* function declarations */\n");
    printf("extern int %s(/* float *, float * */);\n", classify_name);
    printf("extern int %s(/* float * */);\n", normalize_name);
    if(dist_type)
      printf("static float %s(/* int,float *,float *,float *,float */);\n",
	     gauss_out_name);
    else
      printf("static float %s(/* int,float *,float * */);\n", l2dist_name);
    printf("\n\n");
}

void
print_macros(void)
{
    print_norm_macros(nraw_name, ninputs_name);
    printf("#define %s %d\n", ncenters_name, accum_trials);
    printf("#define %s %d\n", noutputs_name, noutputs);
    printf("\n");
}

void
clear_macros(void)
{
    clear_norm_macros();
    printf("#undef %s\n", ncenters_name);
    printf("#undef %s\n", noutputs_name);
    printf("\n");
}

void
print_classify_code(int model)
{
    printf("int %s (inputs, outputs)\n", classify_name);
    printf("   float *inputs,*outputs;\n");
    printf("{\n");
    printf("  int n, best_n;\n");
    print_matrix_initialization("centers", exemplars, accum_trials, ninputs,
				ncenters_name, ninputs_name);
    print_ivector_initialization("classes", classes, accum_trials,
				 ncenters_name);
    if(dist_type){
	print_matrix_initialization("inv_vars", inv_vars, accum_trials,
				    ninputs, ncenters_name, ninputs_name);
	print_vector_initialization("log_det_vars", log_det_vars,
				    accum_trials, ncenters_name);
    }
    printf("  float dist, best_dist;\n");

    /* copy and normlize inputs */
    print_copy_inputs();
    printf("\n  /* normalize loaded data */\n");
    printf("  %s(x);\n", normalize_name);

    if(dist_type){
	printf("  /* find gaussian with the highest output */\n");
	printf("  best_dist = -MAXFLOAT;\n");
	printf("  best_n = -1;\n");
	printf("  for(n = 0; n < %s; n++){\n", ncenters_name);
	printf("    dist = %s(%s, x, centers[n], inv_vars[n], log_det_vars[n]);\n",
	       gauss_out_name, ninputs_name);
	printf("    if(dist > best_dist){\n");
	printf("      best_dist = dist;\n");
	printf("      best_n = n;\n");
	printf("    }\n");
	printf("  }\n");
    } else {
	printf("  /* find nearest neighbor */\n");
	printf("  best_dist = MAXFLOAT;\n");
	printf("  best_n = -1;\n");
	printf("  for(n = 0; n < %s; n++){\n", ncenters_name);
	printf("    dist = %s(%s, x, centers[n]);\n", l2dist_name, ninputs_name);
	printf("    if(dist < best_dist){\n");
	printf("      best_dist = dist;\n");
	printf("      best_n = n;\n");
	printf("    }\n");
	printf("  }\n");
    }
    printf("\n  /* return class of nearest neighbor */\n");
    printf("  return(classes[best_n]);\n");
    printf("}\n");
}

void
print_gauss_out_code(void)
{
    printf("\n");
    printf("static float %s(n, x, m, C, det)\n", gauss_out_name);
    printf("  register int n;\n");
    printf("  register float *x, *m, *C, det;\n");
    printf("{\n");
    printf("  float dist = 0.;\n");
    printf("  for(--n; n >= 0; n--)\n");
    printf("    dist += SQR(x[n] - m[n]) * C[n];\n");
    printf("  dist *= 0.5 * M_LOG10E;\n");
    printf("  dist += det * 0.5 + %s * 0.5 * LOG_2PI;\n", ninputs_name);
    printf("  dist = -dist;\n");
    printf("  return dist;\n");
    printf("}\n");
}
void
print_l2dist_code(void)
{
    printf("\n\n");
    printf("static float %s(n, x1, x2)\n", l2dist_name);
    printf("   register int n;\n");
    printf("   register float *x1, *x2;\n");
    printf("{\n");
    printf("  float dist = 0.;\n");
    printf("  for(--n; n >= 0; n--)\n");
    printf("    dist += SQR(x1[n]-x2[n]);\n");
    printf("  return(dist);\n");
    printf("}\n");
}
