/***--------------------------------------------------------------------------
 * NAME:		nc_class_test.c
 * PURPOSE:	testing driver for nearest cluster classifier
 *
 * SEE ALSO:	nc_class_pack.c
 *
 * FUNCTIONS:	void nc_class_test();
 *
 * BUGS AND LIMITATIONS:
 *
 * $Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/nc_class/nc_class_test.c,v 11.4 2001/08/23 20:12:40 craig Exp $
 * $Source: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/nc_class/nc_class_test.c,v $
 * CREATED: 15-Apr-92 lck			     MIT LINCOLN LABORATORY
 */

/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */



/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include "nc_class.h"
#include "nc_class_pack.h"

#define MAXSTR (256)

static char	prog_name[MAXSTR];	/* static variables */

void
nc_class_test(int create, int map_flag, int nraw, int ninputs, int noutputs,
	      int npatterns, int debg, int verbosity, int verror,
	      char *data_path, char *data_fname, char *exp_path,
	      char *error_fname, char *param_fname, char **labels)
{
    /* local variables */
    int		model, err_model, rv, normalization = 0, data_model;
    int		desired_class, class;
    float	*inputs, *outputs;
    int		post_sum_to_one = 0;
    char	*tmp_path;
    FILE	*error_file, *param_file;
    int		*features;
    
    strcpy(prog_name, get_program_name());
    
    tmp_path = path_build(exp_path, param_fname);
    param_file = nn_fopen(tmp_path, "r", "for read of model file");
    if (verbosity > 1)
      printf("\nReading nc_class parameters from \"%s\"\n", tmp_path);
    rv = norm_read_paramfile(param_file, &normalization);
    if (rv < 0)
      fatal_error("%s: read error %d %s %s",
		  prog_name, rv, "on normalization parameter file",
		  tmp_path);
    
    model = nc_class_read_model(param_file);
    if (model < 0)
      fatal_error("nc_class: read error %d on model file %s", model,
		  tmp_path);
    nn_fclose(param_file, tmp_path, "for read of nc_class parameters");
    
    /* use features from the model */
    features = *(int **) nc_class_param(model, "features");
    ninputs = *(int *) nc_class_param(model, "ninputs");
    
    /* print various debugging and information messages before starting
     * actual training or testing
     */
    display_model_params (model, ninputs, normalization, features, verbosity);
    
    if (verror > 0) {		/* open file for classification errors */
	if(create)
	  error_file = nn_fopen(path_build(exp_path, error_fname),
				"w", "for classification errors");
	else
	  error_file = nn_fopen(path_build(exp_path, error_fname),
				"a", "for classification errors");
    }
    /* do remaining initializations and allocations */
    err_model = report_build (verbosity, verror, noutputs);
    report_init (err_model, map_flag, ninputs, NO_COST, 0, post_sum_to_one);
    data_model =
      class_data_single_init(data_path, data_fname, npatterns,
			     nraw, ninputs, noutputs, features,
			     normalization, verbosity);


    outputs = allocate_vector(noutputs);
    
    /* read and classify input from data file */
    while((inputs = class_data_get_next(data_model, &desired_class)) != NULL){
	/* classify pattern and report errors */
	class = nc_class_classify(model, inputs);
	
	report_class_trial(err_model, error_file,
			   inputs, outputs, desired_class, class,
			   class_data_get_input_num(data_model));
    }
    /* free space and close files */
    deallocate_vector(outputs);
    
    /* finish error reporting and close error file */
    report_overall(err_model, stdout, error_fname, labels);
    report_test_result(err_model, stdout, data_fname);
    report_free(err_model);
    if (verror > 0) {
	nn_fclose(error_file, path_build(exp_path, error_fname),
		  "for error file");
    }
    class_data_exit(data_model);
    nc_class_free(model);
    norm_free();
}
