/***--------------------------------------------------------------------------
 * NAME:         norm_apply.c
 * PURPOSE:      Program to read in un-normalized data, normalize it,
 *		and write it out
 * USAGE:
 *       	norm_apply -<flags>
 * EXAMPLE: 	norm_apply --                * shows all available flags
 *
 * GLOBALS: 	<flags> defined in norm_apply_args.h
 *
 * DESCRIPTION: Reads a normalization file.  Opens a data file and applies
 *  the requested normalization and feature selection to it and writes out
 * the processed data to a new file.
 *
 * BUGS AND LIMITATIONS:
 *
 * $Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/norm/norm_apply.c,v 11.3 2001/08/23 20:12:42 craig Exp $
 * $Source: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/norm/norm_apply.c,v $
 * CREATED 18-June-91 lck				MIT LINCOLN LABORATORY
 *-----------------------------------------------------------------------**/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char     rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/norm/norm_apply.c,v 11.3 2001/08/23 20:12:42 craig Exp $";



/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#include "norm_apply_args.h"

int
main(int argc, char **argv)
{
    /* arguments for reading defaults file */
    int nraw, noutputs, db_type;
    char **labels;
    /* other arguments */
    int normalization, n_read, class;
    FILE *data_file, *data_out_file;
    float *inputs;
    
    /******************* read argument list **********************/
    init_nnsubs(argc, argv, rcsid);
    argc = nnargs(argc, argv, norm_tab, NORM_TABSIZE);
    
    /************* read defaults and set input data parameters ********/
    /* read the defaults file */
    if(strcmp(desc_fname, "") == 0){ /* desc_fname is unset */
	fatal_error("%s: -fdescribe must be set", argv[0]);
    }
    /* set the number of inputs and outputs and read the labels,
       allocating the space */
    set_desc_args(path_build(data_path, desc_fname), &labels, &nraw,
		  &noutputs, &db_type);
    
    /* set the number of patterns to no more than the number of lines
       in the data file */
    if(strcmp(data_fname, "") == 0){ /* data_fname is unset */
	fatal_error("%s: -finput must be set", argv[0]);
    }
    check_file_length(path_build(data_path, data_fname), &npatterns);
    if(npatterns == 0)
      fatal_error("%s: Empty data file %s", argv[0],
		  path_build(data_path, data_fname));
    /* check the data base type */
    if (db_type != CLASS_DATA)
      fatal_error("class data required, check defaults file");
    
    /***************** check feature selection flags *******************/
    if (ninputs > nraw)
      ninputs = nraw;
    ninputs = fix_features(nraw, ninputs, features);
    
    /************* read normalization ***********/
    if (normalize)
      normalization = read_norm_file(path_build(data_path, norm_fname));
    
    /* open the data file to read it line by line */
    data_file = nn_fopen(path_build(data_path, data_fname),
			 "r", "for un-normalized input data");
    /* open output data file to write it line by line */
    if(!strcmp(data_out_fname, "")){
	/* no output file was specified, use stdout */
	data_out_file = stdout;
    } else
      data_out_file = nn_fopen(path_build(data_path, data_out_fname),
			       "w", "for normalized output data");
    
    inputs = allocate_vector(ninputs);
    for (n_read = 0; n_read < npatterns; n_read++) {
	/* read input vector */
	class =
	  get_class_line(data_file, data_fname, inputs, nraw,
			 normalization, features, ninputs);
	if (class == EOF) break;
	
	/* write out data line */
	write_class_line(data_out_file, data_out_fname, inputs, class,
			 ninputs);
    }
    deallocate_vector(inputs);
    nn_fclose(data_file, data_fname, "for un-normalized input data");
    if(data_out_file != stdout)
      nn_fclose(data_out_file, data_out_fname, "for normalized output data");

    if(strcmp(desc_out_fname, "")){
	write_desc_args(path_build(data_path, desc_out_fname),
			labels, ninputs, noutputs, db_type);
    }
    if (normalize) norm_free();

    return(0);
}
