/***---------------------------------------------------------------------------
* NAME:		norm_map.c
* PURPOSE:	driver for normalization program
*
* SEE ALSO: 	norm_map_main.c (norm_subs.c in nnlib)
*
* FUNCTIONS:
*	driver for normalization program
*
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/norm/norm_map.c,v 11.3 2001/08/23 20:12:42 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/norm/norm_map.c,v $
* CREATED: 20-Sep-93 lck			MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */



/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#include "norm.h"

#ifndef FALSE
#define FALSE 0
#endif

#define MAXSTR (256)

static char     prog_name[MAXSTR];	/* static variables */

void
norm_map(int ninputs, int noutputs, int normalization, int debg,
	 int verbosity, char *data_path, char *data_fname, char *norm_fname,
	 char *comment)
{
    char *tmp_path;
    int rv;
    FILE *data_file,*param_file;
    char data_fullname[MAXSTR];
    float *inputs, *outputs;
    
    strcpy(prog_name, get_program_name());
    
    norm_build (ninputs, noutputs, normalization);
    norm_init (0);

    /* open data file */
    strcpy(data_fullname, path_build(data_path, data_fname));
    data_file = nn_fopen(data_fullname,"r", "for training or test data");
    if(verbosity > 1) printf("\nReading %s\n", data_fullname);

    /* allocate space for data */
    inputs = allocate_vector(ninputs);
    outputs = allocate_vector(noutputs);

    /* read all data and calculate norm parameters */
    rv = 0;
    while (rv != EOF) {
	/* read in data, don't normalize, don't select features */
	rv = get_map_line(data_file, data_fullname, inputs, outputs,
			  ninputs, ninputs, noutputs, FALSE, NULL);
	if (normalization && rv != EOF) norm_inc(inputs,rv);
    }
    norm_update();

    /* free space and close files */
    deallocate_vector(inputs);
    nn_fclose(data_file,data_fullname, "for training or test data");

    /* store norm parameters */
    tmp_path = path_build(data_path,norm_fname);
    param_file = nn_fopen(tmp_path,"w",
			  "for write of normalization information");
    fprintf(param_file,"%s\n",comment);
    rv = norm_write_paramfile (param_file);
    nn_fclose (param_file, tmp_path, "for write of normalization information");
    if ( verbosity > 0)
      printf("\nFinished -- normalization saved in \"%s\"\n", tmp_path); 

    /* free norm parameters */
    norm_free();
}
