/***---------------------------------------------------------------------------
* NAME:         norm_seq_main.c
* PURPOSE:      Entry routine for normalization program
* USAGE:
*       	norm_seq -<flags>
* EXAMPLE: 	norm_seq --                * shows all available flags
*
* SEE ALSO: 	norm_seq.c
*
* GLOBALS: 	<flags> defined in norm_args.h
*
* DESCRIPTION:
*		sets up arguments for the routine norm_seq()
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/norm/norm_seq_main.c,v 11.3 2001/08/23 20:12:43 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/norm/norm_seq_main.c,v $
* CREATED 18-June-91 lck				MIT LINCOLN LABORATORY
*-------------------------------------------------------------------------**/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char     rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/norm/norm_seq_main.c,v 11.3 2001/08/23 20:12:43 craig Exp $";



/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#include "norm_args.h"
#include "norm.h"

int
main(int argc, char **argv)
{
    char *comment;
    /* arguments for reading defaults file */
    DEFAULTS_FILE_STRUCT *df_args;

    init_nnsubs(argc, argv, rcsid);
    /* get arguments */
    argc = nnargs(argc, argv, argtab, TABSIZE);

    print_log_header();

    /* read the defaults file */
    if(strcmp(desc_fname, "") == 0){ /* desc_fname is unset */
	fatal_error("%s: -fdescribe must be set", argv[0]);
    }
    /* set the number of inputs and outputs and read the labels,
       allocating the space */
    df_args = (DEFAULTS_FILE_STRUCT *)
      nn_malloc(sizeof(DEFAULTS_FILE_STRUCT), "for the defaults file struct");
    read_defaults_file(path_build(data_path, desc_fname), df_args);
    if(df_args->noutputs == 0)
      fatal_error("noutputs in %s must be greater than 0", desc_fname);
    if(df_args->ninputs == 0)
      fatal_error("ninputs in %s must be greater than 0", desc_fname);

    /* check the data base type */
    if (df_args->db_type != SEQ_DATA)
      fatal_error("class data required, check defaults file");

    /* make and print header comment */
    comment = make_comment_string(nnargs_get_flags(argtab, TABSIZE));
    if (verbosity > 1) {
	printf("%s", comment);
	fflush(stdout);
    }


    norm_seq(df_args->ninputs, df_args->noutputs, normalization, debug, verbosity,
	     data_path, data_fname, norm_fname, comment, df_args->output_labels);

    print_log_footer();

    /* free the vectors allocated when the defaults file was read */
    free_defaults_file_struct(df_args);

    exit_nnsubs();
    exit(0);
}
