/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/parzen/parzen2c.c,v 11.5 2001/08/23 20:12:45 craig Exp $*/
/* $Source: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/parzen/parzen2c.c,v $*/

/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <math.h>
#define GEN2CVARS
#include <gen_2c_args.h>
#define parzen_pack_h_internal_types
#include "parzen_pack.h"

/* parzen specific variables */
static int per_class = 0;
static int per_input = 0;
static int mixed = 0;
static int circ = 0;
static int npatterns;
static int *input_types = NULL;
static float *input_type_values = NULL;

char logadd_name[64];
char npatterns_name[64];
char get_radians_name[64];
char make_0_2PI_name[64];
char inside_circ_name[64];
char vonmises_name[64];
char bessi0_name[64];


/* command line arguments */
static int linear = 0;
static float spread = 1.;

static ARG argtab2[] = 
{
    {"spread", FLOAT, (int *)&spread,
       "factor for scaling kernel functions for testing"},
    {"linear", BOOLEAN, &linear,
       "Use step functions for kernels. Default: Gaussian kernels"},
};
#define TABSIZE2 (sizeof(argtab2)/sizeof(ARG))

/* function declerations */
int read_model_file(char **argv);
void free_model(int model);

void get_params(int model);
void get_general_params(int model);
void get_specific_params(int model);

void set_priors(int model);

void print_comments(void);
void print_declarations(void);
void print_macros(void);
void print_classify_code(int model);
void print_logadd_code(void);
void clear_macros(void);
void print_get_radians_code(void);
void print_make_0_2PI_code(void);
void print_inside_circ_code(void);
void print_vonmises_code(void);
void print_bessi0_code(void);


int
main(int argc, char **argv)
{
    int model;
    ARG *argtab;
    int tabsize;

    /* process arguments */
    nnargs_combine_argtabs(argtab2c, TABSIZE2C, argtab2, TABSIZE2,
               &argtab, &tabsize);
    argc = nnargs(argc, argv, argtab, tabsize);

    /* check arguments */
    if(strlen(class_suff) > 53){
    fatal_error("suffix (%s) is too long.  use at most 50 characters",
            class_suff);
    }

    /* set up subroutine names and macro names */
    strcpy(classify_name, "classify");
    strcpy(normalize_name, "normalize");
    strcpy(nraw_name, "NRAW");
    strcpy(ninputs_name, "NINPUTS");
    strcpy(noutputs_name, "NCLASSES");

    strcpy(logadd_name, "logadd");
    strcpy(npatterns_name, "NPATTERNS");
    strcpy(get_radians_name, "get_radians");
    strcpy(make_0_2PI_name, "make_0_2PI");
    strcpy(inside_circ_name, "inside_circ");
    strcpy(vonmises_name, "vonmises");
    strcpy(bessi0_name, "bessi0");
    if(class_suff[0] != '\0'){
    strcpy(class_exten, "_");
    strcat(class_exten, class_suff);

    strcat(classify_name, class_exten);
    strcat(normalize_name, class_exten);
    strcat(nraw_name, class_exten);
    strcat(ninputs_name, class_exten);
    strcat(noutputs_name, class_exten);

    strcat(logadd_name, class_exten);
    strcat(npatterns_name, class_exten);
    strcat(get_radians_name, class_exten);
    strcat(make_0_2PI_name, class_exten);
    strcat(inside_circ_name, class_exten);
    strcat(vonmises_name, class_exten);
    strcat(bessi0_name, class_exten);
    } else {
      strcpy(class_exten, "");
    }

    /* read in parameters */
    model = read_model_file(argv);
    get_params(model);

    set_priors(model);

    /* print the source file */
    print_comments();
    print_declarations();
    print_macros();
    print_classify_code(model);
    print_normalize_code(normalize_name);
    if(!linear) print_logadd_code();
    if(mixed || circ){
    print_get_radians_code();
    if(linear){
        print_make_0_2PI_code();
        print_inside_circ_code();
    } else {
        print_vonmises_code();
        print_bessi0_code();
    }
    }
    clear_macros();

    nnargs_free_argtab(argtab, tabsize);
    free_model(model);
    exit(0);
}

int
read_model_file(char **argv)
{
    int   model, norm_model, normalization, rv;
    int nraw;
    FILE *param_file;

    param_file = nn_fopen(param_fname, "r", "for read of model file");
    norm_model = norm_read_paramfile(param_file, &normalization);
    if (norm_model < 0)
      fatal_error("%s: read error %d %s %s", argv[0], norm_model,
          "on normalization parameter file", param_fname);
    rv = feature_read_paramfile(param_file, &features, &nraw, &ninputs);
    if (rv < 0)
      fatal_error("%s: read error %d %s %s", argv[0], rv,
          "on feature selection parameters in file", param_fname);
    
    model = parzen_read_model(param_file);
    if (model < 0)
      fatal_error("%s: read error %d on model file %s",
          argv[0], model, param_fname);
    nn_fclose(param_file, param_fname, "for read of model file");

    return((int) model);
}


void
free_model(int model)
{
  norm_free();
  parzen_free(model);
}

void
get_params(int model)
{
    get_specific_params(model);
    get_general_params(model);
    get_norm_params(model, ninputs, features);
}

void
get_specific_params(int model)
{
    int i;

    per_input = *(int *)parzen_param(model, "per_input");
    per_class = *(int *)parzen_param(model, "per_class");
    npatterns = *(int *)parzen_param(model, "total_trials");
    input_types = *(int **)parzen_param(model, "input_types");
    input_type_values = *(float **)parzen_param(model, "input_type_values");
    mixed = 0;
    circ = (input_types[0] == CIRC);
    for(i = 1; i < ninputs; i++)
      if( circ != (input_types[i] == CIRC)) mixed = 1;
    
}

void 
get_general_params(int model)
{
    ninputs = *(int *)parzen_param(model, "ninputs");
    noutputs = *(int *)parzen_param(model, "noutputs");
}

void
set_priors(int model)
{
    int i;
    float total;
    int *class_trials;
    float *test_priors_store;

    if(adjust_test){
        test_priors_store = allocate_vector(noutputs);
    if(test_uniform_priors){
        /* set up uniform priors */
        if(test_priors_list != NULL)
          deallocate_vector(test_priors_list);
        test_priors_list = allocate_vector(noutputs+1);
        test_priors_list[0] = noutputs;
        for(i = 1; i <= noutputs; i++){
        test_priors_list[i] = 1./(float)noutputs;
        test_priors_store[i-1] = test_priors_list[i];
        }
    } else {
        /* normalize priors to sum to one */
        if(test_priors_list[0] != (float)noutputs)
          fatal_error("%s: %s %.0f, should be %d",
              "test_priors_list length is", test_priors_list[0],
              noutputs);
        total = 0.;
        for(i = 1; i <= noutputs; i++) total += test_priors_list[i];
        for(i = 1; i <= noutputs; i++){
        test_priors_list[i] /= total;
        test_priors_store[i-1] = test_priors_list[i];
        }
    }
    deallocate_vector(test_priors_list);
    test_priors_list = test_priors_store;
    } else {
        if (test_priors_list != NULL) deallocate_vector(test_priors_list);
    test_priors_list = allocate_vector(noutputs);
#ifdef notdef
    /* in parzen_pack.c, I don't put in the class_trials/total_trials */
    total = *(int *)parzen_param(model, "total_trials");
    class_trials =  *(int **) parzen_param(model, "class_trials");
    for(i = 0; i < noutputs; i++){
        test_priors_list[i] = (float)class_trials[i];
        test_priors_list[i] /= total;
    }
#else
    for(i = 0; i < noutputs; i++) test_priors_list[i] = 1.;
#endif
    }
    class_trials =  *(int **) parzen_param(model, "class_trials");
    for(i = 0; i < noutputs; i++){
    test_priors_list[i] /= class_trials[i];
    if(!linear)
      test_priors_list[i] = log10(test_priors_list[i]);
    }
}

void
print_comments(void)
{
    printf("/* %s */\n", param_fname);
    printf("/* ");
    if(per_class) printf("Per Class, ");
    else printf("Grand, ");
    if(per_input) printf("Elliptical */\n");
    else printf("Circular */\n");
    if(linear)
      printf("/* Rectangular Kernels */\n");
    else
      printf("/* Gaussian Kernels */\n");
    print_norm_comments();
}

void
print_declarations(void)
{
    printf("\n");
    printf("#include <math.h>\n");
    printf("\n");
    printf("\n /* macro definitions */\n");
    printf("#ifndef SQR(x)\n");
    printf("#define SQR(x) ((x) * (x))\n");
    printf("#endif\n");
    printf("\n");
    printf("#ifndef M_LOG10E\n");
    printf("#define M_LOG10E 0.43429448190325182765\n");
    printf("#endif\n");
    printf("\n");
    printf("#ifndef M_LN10\n");
    printf("#define M_LN10 2.30258509299404568402\n");
    printf("#endif\n");
    printf("\n");
    printf("#ifndef LOG_2PI\n");
    printf("#define LOG_2PI 0.798179868358\n");
    printf("#endif\n");
    printf("#ifndef M_2PI\n");
    printf("#define M_2PI 6.28318530717958647692\n");
    printf("#endif\n");
    printf("\n");
    if(mixed){
    printf("#ifndef CIRC\n");
    printf("#define CIRC %d\n", CIRC);
    printf("#endif\n");
    }

    printf("\n /* function declarations */\n");
    printf("extern int %s(/* float * inputs,float * outputs*/);\n",
       classify_name);
    printf("extern int %s(/* float * inputs*/);\n", normalize_name);
    if(!linear)
      printf("static float %s(/* float,float */);\n", logadd_name);
    if(mixed || circ){
    printf("static float %s(/*float x,float range*/);\n",
           get_radians_name);
    if(linear){
        printf("static float %s(/*float r*/);\n", make_0_2PI_name);
        printf("static int %s(/*float x, float m, float edge, float range */);\n",
           inside_circ_name);
    } else {
        printf("static float %s(/*float x, float m, float range, float inv_var*/);\n",
           vonmises_name);
        printf("static float %s(/*float x*/);\n", bessi0_name);
    }
    }
    printf("\n\n");
}

void
print_macros(void)
{
    print_norm_macros(nraw_name, ninputs_name);
    printf("#define %s %d\n", noutputs_name, noutputs);
    printf("#define %s %d\n", npatterns_name, npatterns);
    if(linear)
      printf("#define UNKNOWN_CLASS %d\n", UNKNOWN_CLASS);
    printf("\n");
}    

void
clear_macros(void)
{
    clear_norm_macros();
    printf("#undef %s\n", noutputs_name);
    printf("#undef %s\n", npatterns_name);
    if(linear)
      printf("#undef UNKNOWN_CLASS\n");
    printf("\n");
}    

void
print_classify_code(int model)
{
    int   i, j;
    float **centers;
    int *classes;
    INVVAR *inv_var;
    LOGDET *log_det;

    printf("int %s (inputs, outputs)\n", classify_name);
    printf("   float *inputs,*outputs;\n");
    printf("{\n");
    printf("  int n, best;\n");
    printf("  int j");
    if( per_input) printf(",k");
    printf(";\n");
    printf("  float y[%s];\n", noutputs_name);
    printf("  int p,c;\n");
    if(mixed){
    printf("  static int input_types[%s] = {\n", ninputs_name);
    print_ivector(input_types, ninputs);
    printf("  };\n");
    }
    if( mixed || circ ){
    printf("  static float input_type_values[%s] = {\n", ninputs_name);
    print_vector(input_type_values, ninputs);
    printf("  };\n");
    }
    printf("  float dist;\n");
    /* print centers */
    printf("  static float centers[%s][%s] = {\n",
       npatterns_name, ninputs_name);
    centers = *(float ***)parzen_param(model, "centers");
    for(i = 0; i < npatterns; i++){
    printf("\t{");
    print_vector(&centers[i][0], ninputs);
    printf("},\n");
    }
    printf("};\n");
    /* print classes */
    printf("  static int classes[%s] = {\n", npatterns_name);
    classes = *(int **)parzen_param(model, "classes");
    print_ivector(classes, npatterns);
    printf("  };\n");

    if(linear){
    double temp;
    /* print edges, converting 1/var to 2*stddev (2/sqrt(inv_var)) */
    inv_var = (INVVAR *)parzen_param(model, "inv_var");
    if(per_class){
        if(per_input){
        for(i = 0; i < noutputs; i++){
            for(j = 0; j < ninputs; j++){
            temp = inv_var->per_class_input[i][j];
            temp /= spread;
            temp = 2*(1./sqrt(temp));
            inv_var->per_class_input[i][j] = temp;
            }
        }
        printf("  static float edges[%s][%s] = {\n",
               noutputs_name, ninputs_name);
        for(i = 0; i < noutputs; i++){
            printf("\t/* class %d */{\n", i);
            print_vector(inv_var->per_class_input[i], ninputs);
            printf("},\n");
        }
        printf("};\n");
        } else {
        for(i = 0; i < noutputs; i++){
            temp = inv_var->per_class[i];
            temp /= spread;
            temp = 2.*(1./sqrt(temp));
            inv_var->per_class[i] = temp;
        }
        printf("  static float edges[%s] = {\n", noutputs_name);
        print_vector(inv_var->per_class, noutputs);
        printf("};\n");
        }
    } else {
        if(per_input){
        for(j = 0; j < ninputs; j++){
            temp = inv_var->per_input[j];
            temp /= spread;
            temp = 2.*(1./sqrt(temp));
            inv_var->per_input[j] = temp;
        }
        printf("  static float edges[%s] = {\n", ninputs_name);
        print_vector(inv_var->per_input, ninputs);
        printf("};\n");
        } else {
        temp = inv_var->grand;
        temp /= spread;
        temp = 2.*(1./sqrt(temp));
        inv_var->grand = temp;
        printf("  static float edges = %14.6e;\n", inv_var->grand);
        }
    }
    printf("  /* priors = N_per_class/(N_patterns * N_per_class) */\n");
    printf("  /* second N_per_class is to scale sum of Kernel outputs*/\n");
    print_vector_initialization("class_priors", test_priors_list,
                    noutputs, noutputs_name);
    printf("  float t;\n");
    
    /* copy and normlize inputs */
    print_copy_inputs();
    printf("\n  /* normalize loaded data */\n");
    printf("  %s(x);\n", normalize_name);

    printf("  /* calculate outputs for each class */\n");
    printf("  /* clear outputs array */\n");
    printf("  for(n = 0; n < %s; n++) y[n] = 0;\n", noutputs_name);
    printf("  /* find Kernel output for each center */\n");
    printf("  for(p = 0; p < %s; p++){\n", npatterns_name);
    printf("    c = classes[p];\n");
    printf("    t = 1;\n");
    printf("    for(j = 0; j < %s && t > 0; j++){\n", ninputs_name);
    if(mixed){
        printf("      if(input_types[j] == CIRC){\n");
    }
    if(mixed || circ){
        if(mixed) printf("  ");
        printf("      t = %s(x[j], centers[p][j],", inside_circ_name);
        if(per_input){
        if(per_class) printf(" edges[c][j],");
        else printf(" edges[j],");
        } else {
        if(per_class) printf(" edges[c],");
        else printf(" edges,");
        }
        printf(" input_type_values[j]);\n");
    }
    if(mixed) printf("      } else {\n");
    if(mixed || !circ){
        if(mixed) printf("  ");
        printf("      dist = x[j]-centers[p][j];\n");
        if(mixed) printf("  ");
        printf("      if(dist < 0) dist = -dist;\n");
        if(mixed) printf("  ");
        if(per_class){
        if(per_input)
          printf("      if(dist > edges[c][j]) t = 0;\n");
        else
          printf("      if(dist > edges[c]) t = 0;\n");
        } else {
        if(per_input)
          printf("      if(dist > edges[j]) t = 0;\n");
        else
          printf("      if(dist > edges) t = 0;\n");
        }
    }
    if(mixed) printf("      }\n");
    printf("    }\n");
    printf("    /* sum up outputs for each class */\n");
    printf("    y[c] += t;\n");
    printf("  }\n");
    printf("  /* scale by the priors and number of patterns per class */\n");
    printf("  for(n = 0; n < %s; n++)\n", noutputs_name);
    printf("    y[n] *= class_priors[n];\n");
    printf("\n  /* copy outputs */\n");
    printf("  for(n = 0; n < %s; n++)\n", noutputs_name);
    printf("    outputs[n] = y[n];\n");
    if(linear){
        printf("  /* check that there were outputs */\n");
        printf("  t = 0;\n");
        printf("  for(n = 0; n < %s; n++)\n", noutputs_name);
        printf("    if(y[n] > 0) t = 1;\n");
        printf("  if (!t) return UNKNOWN_CLASS;\n");
    }
    printf("\n  /* find highest output */\n");
    printf("  for(best = n = 0; n < %s; n++)\n", noutputs_name);
    printf("    if(y[best] < y[n]) best = n;\n");
    printf("  return(best);\n");
    printf("}\n");


    } else {/* Gaussian kernels */
    /* print variances */
    inv_var = (INVVAR *)parzen_param(model, "inv_var");
    if(per_class){
        if(per_input){
        printf("  static float inv_var[%s][%s] = {\n",
               noutputs_name, ninputs_name);
        for(i = 0; i < noutputs; i++){
            printf("\t/* class %d */{\n", i);
            for(j = 0; j < ninputs; j++){
            inv_var->per_class_input[i][j] /= spread;
            }
            print_vector(inv_var->per_class_input[i], ninputs);
            printf("},\n");
        }
        printf("};\n");
        } else {
        for(i = 0; i < noutputs; i++) inv_var->per_class[i] /= spread;
        printf("  static float inv_var[%s] = {\n", noutputs_name);
        print_vector(inv_var->per_class, noutputs);
        printf("};\n");
        }
    } else {
        if(per_input){
        for(i = 0; i < ninputs; i++) inv_var->per_input[i] /= spread;
        printf("  static float inv_var[%s] = {\n", ninputs_name);
        print_vector(inv_var->per_input, ninputs);
        printf("};\n");
        } else {
        inv_var->grand /= spread;
        printf("  static float inv_var = %14.6e;\n", inv_var->grand);
        }
    }
    /* print determinant */
    log_det = (LOGDET *)parzen_param(model, "log_det");
    if(per_class){
        printf("  static float log_determinant[%s] = {\n", noutputs_name);
        print_vector(log_det->per_class, noutputs);
        printf("};\n");
    } else {
        printf("  float log_determinant = %14.6e;\n",
           log_det->grand_class);
    }

    printf("  /* N_per_class/(N_patterns * N_per_class) */");
    printf("  /* second N_per_class is to scale sum of Kernel outputs*/\n");
    print_vector_initialization("log_class_priors", test_priors_list,
                    noutputs, noutputs_name);
    printf("  float t;\n");
    
    /* copy and normlize inputs */
    print_copy_inputs();
    printf("\n  /* normalize loaded data */\n");
    printf("  %s(x);\n", normalize_name);
    
    printf("  /* calculate outputs for each class */\n");
    printf("  /* clear outputs array */\n");
    printf("  for(n = 0; n < %s; n++) y[n] = -1e30;\n", noutputs_name);
    printf("  /* find Kernel output for each center */\n");
    printf("  for(p = 0; p < %s; p++){\n", npatterns_name);
    printf("    c = classes[p];\n");
    printf("    t = 0.;\n");
    printf("    for(j = 0; j < %s; j++){\n", ninputs_name);
    if(mixed) printf("      if(input_types[j] == CIRC){\n");
    if(mixed || circ){
        if(mixed) printf("  ");
        printf("      dist = %s(x[j],centers[p][j],input_type_values[j],",
           vonmises_name);
        if(per_input){
        if(per_class) printf("inv_var[c][j]");
        else printf("inv_var[j]");
        } else {
        if(per_class) printf("inv_var[c]");
        else printf("inv_var");
        }
        printf(");\n");
    }
    if(mixed) printf("      } else {\n");
    if(mixed || !circ){
        if(mixed) printf("  ");
        printf("      dist = SQR(x[j]-centers[p][j]) * ");
        if(per_input){
        if(per_class) printf("inv_var[c][j]");
        else printf("inv_var[j]");
        } else {
        if(per_class) printf("inv_var[c]");
        else printf("inv_var");
        }
        printf(";\n");
        if(mixed) printf("  ");
        printf("      dist *= M_LOG10E;\n");
        if(mixed) printf("  ");
        if(per_input){
        if(per_class) printf("      dist -= log10(inv_var[c][j]);\n");
        else printf("      dist -= log10(inv_var[j]);\n");
        } else {
        if(per_class) printf("      dist -= log10(inv_var[c]);\n");
        else printf("      dist -= log10(inv_var);\n");
        }
        if(mixed) printf("  ");
        printf("      dist += LOG_2PI;\n");
        if(mixed) printf("  ");
        printf("      dist *= -0.5;\n");
        if(mixed) printf("      }\n");
    }
    printf("      t += dist;\n");
    printf("    }\n");
    printf("    /* sum up outputs for each class */\n");
    printf("    y[c] = %s(y[c], t);\n",logadd_name);
    printf("  }\n");
    printf("  /* scale by the priors and number of patterns per class */\n");
    printf("  for(n = 0; n < %s; n++)\n", noutputs_name);
    printf("    y[n] += log_class_priors[n];\n");
    printf("\n  /* copy outputs and make linear */\n");
    printf("  for(n = 0; n < %s; n++)\n", noutputs_name);
    printf("    outputs[n] = exp(M_LN10 * (double)y[n]);\n");
    printf("\n  /* find highest output */\n");
    printf("  for(best = n = 0; n < %s; n++)\n", noutputs_name);
    printf("    if(y[best] < y[n]) best = n;\n");
    printf("  return(best);\n");
    printf("}\n");
    }
}

void
print_logadd_code(void)
{
    printf("\n\n");
    printf("static float %s(x, y)\n", logadd_name);
    printf("   register float x, y;\n");
    printf("{\n");
    printf("  register double b;\n");
    printf("  if(x < y)    /*** exchange if x < y ***/\n");
    printf("    b = (double) x, x = (double) y, y = (double) b;\n");
    printf("  b = (double) x - y;\n");
    printf("  if(b > 16.0) /*** keep up to 16 decimal digits precision ***/\n");
    printf("    return(x);\n");
    printf("  else\n");
    printf("    return(y + (float)log10(1.0 + exp(M_LN10*b)));\n");
    printf("}\n");
}

void
print_get_radians_code(void)
{
    printf("/* take a variable and convert it to radians for\n");
    printf("   a circle with values [0,range] */\n");
    printf("static float\n");
    printf("%s(x, range)\n",get_radians_name);
    printf("     float x, range;\n");
    printf("{\n");
    printf("    return (x * M_2PI)/range;\n");
    printf("}\n");
}

void
print_make_0_2PI_code(void)
{
    printf("/* take a value in radians and force it to be in the range [0,2PI] */\n");
    printf("static float\n");
    printf("make_0_2PI(x)\n");
    printf("     float x;\n");
    printf("{\n");
    printf("    while (x < 0) x += M_2PI;\n");
    printf("    while (x >= M_2PI) x -= M_2PI;\n");
    printf("    return x;\n");
    printf("}\n");
}
void
print_inside_circ_code(void)
{
    printf("static int\n");
    printf("inside_circ(x, mean, edge, range)\n");
    printf("     float x, mean, edge, range;\n");
    printf("{\n");
    printf("    float rad_mean, radian, lower, upper;\n");
    printf("\n");
    printf("    if(edge >= M_2PI) return 1;\n");
    printf("    rad_mean = %s(mean, range);\n",get_radians_name);
    printf("    lower = rad_mean - edge;\n");
    printf("    lower = %s(lower);\n",make_0_2PI_name);
    printf("    upper = rad_mean + edge;\n");
    printf("    upper = %s(upper);\n",make_0_2PI_name);
    printf("    radian = %s(x, range);\n",get_radians_name);
    printf("    radian = %s(radian);\n",make_0_2PI_name);
    printf("    if(lower <= upper){\n");
    printf("    if( radian < lower || radian > upper ) return 0;\n");
    printf("    else return 1;\n");
    printf("    /* if lower == upper, radian must be == also to pass */\n");
    printf("    } else { /* lower > upper, range spans the break */\n");
    printf("    if(radian < lower && radian > upper) return 0;\n");
    printf("    else return 1;\n");
    printf("    }\n");
    printf("}\n");
}

void
print_vonmises_code(void)
{
    printf("/* calculate pdf of vonmises function */\n");
    printf("static float\n");
    printf("%s(x, mean, range, inv_var)\n", vonmises_name);
    printf("     float x, mean, range, inv_var;\n");
    printf("{\n");
    printf("    double constant;\n");
    printf("    float pdf;\n");
    printf("    float radian, rad_mean;\n");
    printf("\n");
    printf("    radian = get_radians(x,range);\n");
    printf("    rad_mean = get_radians(mean,range);\n");
    printf("    /* modified bessel function is ok if k is small */\n");
    printf("    if ( inv_var < 50.0) {\n");
    printf("        constant = 1.0/(M_2PI*%s(inv_var));\n", bessi0_name);
    printf("        pdf = log10(constant) + M_LOG10E*inv_var*cos(radian-rad_mean);\n");
    printf("    } else {\n");
    printf("    /* when inv_var is large, approximate bessel function by exp(inv_var)/sqrt(2PI*inv_var) \n");
    printf("       and fold this into the exponential in the pdf numerator */\n");
    printf("        constant = 1.0/(M_2PI*sqrt(M_2PI*inv_var));\n");
    printf("        pdf = log10(constant) + M_LOG10E*inv_var*(cos(radian-rad_mean)-1.0);\n");
    printf("    }\n");
    printf("    return(pdf);\n");
    printf("}\n");
}

void
print_bessi0_code(void)
{
    printf("/* modified bessel function good if x < 50.0 \n");
    printf("---------------------------------------------------*/\n");
    printf("static float\n");
    printf("bessi0(x)\n");
    printf("     float x;\n");
    printf("{\n");
    printf("    float ax,ans;\n");
    printf("    double y;\n");
    printf("\n");
    printf("    if ((ax=fabs(x)) < 3.75) {\n");
    printf("        y=x/3.75;\n");
    printf("        y*=y;\n");
    printf("        ans=1.0+y*(3.5156229+y*(3.0899424+y*(1.2067492\n");
    printf("            +y*(0.2659732+y*(0.360768e-1+y*0.45813e-2)))));\n");
    printf("    } else {\n");
    printf("        y=3.75/ax;\n");
    printf("        ans=(exp(ax)/sqrt(ax))*(0.39894228+y*(0.1328592e-1\n");
    printf("            +y*(0.225319e-2+y*(-0.157565e-2+y*(0.916281e-2\n");
    printf("            +y*(-0.2057706e-1+y*(0.2635537e-1+y*(-0.1647633e-1\n");
    printf("            +y*0.392377e-2))))))));\n");
    printf("    }\n");
    printf("    return ans;\n");
    printf("}\n");
}
