/***-------------------------------------------------------------------------
* NAME:		parzen_feat_sel.c
* PURPOSE:	driver to run a feature search using parzen_cross_valid
*		and the feat_sel feature list building routines
*
* SEE ALSO: parzen_cv.c
*
* DESCRIPTION:
*	does a feature search using the feat_sel routines to build lists
*	of features and passing them to the parzen cross validation driver
*	to do the actual classification tests
*
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/parzen/parzen_feat_sel.c,v 11.4 2001/08/23 20:12:45 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/parzen/parzen_feat_sel.c,v $
* CREATED: 05-Oct-94 lck			MIT LINCOLN LABORATORY
----------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */



/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#include "parzen.h"
#include "parzen_feat_sel.h"

void
parzen_feat_sel(int search_type, int search_n, int search_verbosity,
		char *search_param_fname,
		int roc_cost, int roc_target, float roc_lower, float roc_upper,
		/* cross valid arguments */
		DEFAULTS_FILE_STRUCT *df_args, int *features, int normalize,
		int npatterns, int verbosity, int verror, int random_flag,
		int cv_nfolds, int cv_nsplits, int use_cv_splits,
		int *cv_splits, char **cv_train_mask, char **cv_test_mask,
		char *data_path, char *data_fname, char *norm_fname,
		char *exp_path, char *error_fname, char *param_fname,
		char *comment, int adjust_train, int adjust_test,
		int priors_npatterns, float *train_priors_list,
		float *test_priors_list, int cv_random, int log_outputs,
		float minimum_variance, int per_input, int per_class,
		float spread, int linear)
{
    float percent_error;

    cat_sel_build(df_args->ninputs, search_type, search_n, search_verbosity,
		  df_args->categories);
    feat_sel_print_head();

    /* main loop */
    while(!feat_sel_search_done()){
	/* search over all available features for the best to
	 * add or eliminate for this number of features
	 */
	feat_sel_init_nfeatures();
	while((features = feat_sel_next_features()) != NULL){
	    /* call parzen_cross_valid with the current set of features */
	    percent_error =
	      parzen_cross_valid(df_args, features[0], features,
				 normalize, npatterns, verbosity, verror,
				 random_flag, cv_nfolds, cv_nsplits, use_cv_splits,
				 cv_splits, cv_train_mask, cv_test_mask,
				 data_path, data_fname, norm_fname, exp_path,
				 error_fname, param_fname, comment,
				 adjust_train, adjust_test, priors_npatterns,
				 train_priors_list, test_priors_list, cv_random,
				 (roc_cost?ROC_COST:(linear?SQ_ERR:LOG_LIKE)),
				 roc_target, roc_lower, roc_upper,
				 log_outputs, minimum_variance,
				 per_input, per_class, spread, linear);
	    /* keep track of feature that gives minimum error */
	    feat_sel_check_best(percent_error);
	}
	/* add the best feature to the list */
	feat_sel_pick_best();
    }
    feat_sel_print_final_result();
    
    feat_sel_save_list(path_build(exp_path, search_param_fname),
		       comment);
    feat_sel_free();
}
