/***--------------------------------------------------------------------------
 * NAME:		parzen_test.c 
 * PURPOSE:	testing driver for Parzen window classifier 
 *
 * SEE ALSO:	parzen_main.c parzen_pack.c
 *
 *
 * BUGS AND LIMITATIONS:
 *
 *
 * $Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/parzen/parzen_test.c,v 11.4 2001/08/23 20:12:46 craig Exp $
 * $Source: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/parzen/parzen_test.c,v $
 * CREATED: 5-Oct-94 lck			MIT LINCOLN LABORATORY
 ********************************************************************/

/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */



/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include "parzen_pack.h"
#include "parzen.h"

#define MAXSTR (256)

static char     prog_name[MAXSTR];	/* static variables */

void
parzen_test(DEFAULTS_FILE_STRUCT *df_args, int create_flag, int ninputs,
	    int *features, int npatterns, int verbosity, int verror,
	    char *data_path, char *data_fname, char *exp_path,
	    char *error_fname, char *param_fname, int adjust_priors,
	    float *priors_list, int log_outputs, float spread, int linear)
{
    int		model, err_model, rv, normalization=0;
    int		post_sum_to_one = 0;
    int		desired_class, class;
    int		data_model;
    float	*inputs, *outputs;
    char	*tmp_path;
    FILE	*error_file = NULL, *param_file;
    
    strcpy(prog_name, get_program_name());
    
    tmp_path = path_build(exp_path, param_fname);
    param_file = nn_fopen(tmp_path, "r", "for read of model file");
    rv = norm_read_paramfile(param_file, &normalization);
    if (rv < 0)
      fatal_error("%s: read error %d %s %s", prog_name, rv,
		  "on normalization parameter file", tmp_path);
    rv = feature_read_paramfile(param_file, &features, &df_args->ninputs, &ninputs);
    if (rv < 0)
      fatal_error("%s: read error %d %s %s", prog_name, rv,
		  "on feature selection parameters in file", tmp_path);
	
    model = parzen_read_model(param_file);
    nn_fclose(param_file, tmp_path, "for read of model file");
    
    if (verror > 0) {	/* open file for classification errors */
	if(create_flag)
	  error_file = nn_fopen(path_build(exp_path, error_fname),
				"w", "for classification errors");
	else
	  error_file = nn_fopen(path_build(exp_path, error_fname),
				"a", "for classification errors");
    }
    /* prepare error reporting */
    err_model = report_build (verbosity, verror, df_args->noutputs);
    if(linear)
      report_init (err_model, df_args->db_type, ninputs, SQ_ERR, log_outputs, post_sum_to_one);
    else
      report_init (err_model, df_args->db_type, ninputs, LOG_LIKE, log_outputs, post_sum_to_one);
    
    data_model =
      class_data_single_init(data_path, data_fname, npatterns, df_args->ninputs,
			     ninputs, df_args->noutputs, features,
			     normalization, verbosity);
    /* prepare parzen model for testing */
    parzen_test_init (model, adjust_priors, priors_list, spread, linear);
    outputs = *(float **) parzen_param(model, "outputs");

    /* read and classify input from data file */
    while((inputs = class_data_get_next(data_model, &desired_class)) != NULL){
	/* classify pattern and report errors */
	class = parzen_classify(model, inputs);
	report_class_trial(err_model, error_file,
			   inputs, outputs, desired_class, class,
			   class_data_get_input_num(data_model));
    }

    /* finish error reporting and close error file */
    report_overall(err_model, stdout, error_fname, df_args->output_labels);
    report_test_result(err_model, stdout, data_fname);
    report_free(err_model);    
    if (verror > 0) {
	nn_fclose(error_file, path_build(exp_path, error_fname),
		  "for error file");
    }

    /* free space and close files */
    class_data_exit(data_model);
    parzen_free (model);
    norm_free();
}
