/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#include "rbf_pack.h"
char *param_fname = "";
static ARG argtab[] = 
{
    {"model_file", STRING, (int *)&param_fname, "rbf model to be translated"},
};
#define TABSIZE (sizeof(argtab)/sizeof(ARG))

int debug;

char *i2A(int i, char *A);
void print_delta_model(int model, int normalization);

int
main(int argc, char **argv)
{
    FILE *param_file;
    int model, norm_model;
    int normalization;

    argc = nnargs(argc, argv, argtab, TABSIZE);

    param_file = nn_fopen(param_fname, "r", "for read of model file");
    norm_model = norm_read_paramfile(param_file, &normalization);
    if (norm_model < 0)
      fatal_error("%s: read error %d %s %s", argv[0], norm_model,
          "on normalization parameter file", param_fname);
    
    model = rbf_read_model(param_file);
    if (model < 0)
      fatal_error("%s: read error %d on model file %s",
          argv[0], model, param_fname);
    nn_fclose(param_file, param_fname, "for read of model file");

    printf("/* %s */\n", param_fname);
    print_delta_model(model, normalization);
    /******* free space ***********/
    norm_free();
    rbf_free(model);

    exit(0);
}

void
print_delta_model(int model, int normalization)
{
    int ninputs, noutputs, nhidden, *features, bias, hspread, norm_outputs;
    float **rbf_means, **rbf_sdevs, **rbf_weights, f;
    int norm_type, norm_ninputs, sample = 0;
    float *means, *sdevs, **transform_matrix;
    int n, m, h, j, i;

    char left_arg[8], right_arg[8];
    int starti, starth, starty;

    /* get general parameters */
    ninputs = *(int *)rbf_param(model, "ninputs");
    noutputs = *(int *)rbf_param(model, "nclasses");
    nhidden = *(int *)rbf_param(model, "ngauss_hidden");
    features = *(int **)rbf_param(model, "features");
    bias = *(int *)rbf_param(model, "bias");
    hspread = *(float *)rbf_param(model, "hspread");
    norm_outputs = *(int *)rbf_param(model, "norm_outputs");
    rbf_means = *(float ***)rbf_param(model, "centroids");
    rbf_sdevs = *(float ***)rbf_param(model, "spreads");
    rbf_weights = *(float ***)rbf_param(model, "weights");

    norm_type = *(int *)norm_param("normalization");
    norm_ninputs = *(int *)norm_param("ninputs");
    if(norm_ninputs == 0) norm_ninputs = ninputs;
    means = *(float **)norm_param("means");
    sdevs = *(float **)norm_param("sdevs");
    transform_matrix = *(float ***)norm_param("transform");

    /* print description of variables */
    switch (norm_type){
      case 0:
    printf("/* norm: NONE */\n");
    break;
      case 1:
    printf("/* norm: SIMPLE */\n");
    break;
      case 2:
    printf("/* norm: PCA */\n");
    break;
      case 3:
    printf("/* norm: LDA */\n");
    break;
    }
    if(ninputs != norm_ninputs){
    printf("/* %d features", ninputs);
    if(features[0] != 0){
        printf(": ");
        for(i = 1; i <= features[0]; i++) printf("%d ", features[i]);
    }
    printf("*/\n");
    } else {
    printf("/* %d (all) features */\n", ninputs);
    }
    if(bias){
    printf("/* bias node included */\n");
    }

    if(ninputs != norm_ninputs && features[0] != 0){
    sample = 1;
    }
    
    printf("\n  /* raw inputs in");
    for(i = 0; i < ninputs; i++){
    printf(" %s", i2A(i, left_arg));
    }
    printf(" */\n\n");
    printf("\n  /* load inputs */\n");
    starti = norm_ninputs + noutputs;
    for(i = 0; i < ninputs; i++){
    if(sample) n = features[i];
    else n = i;
    switch(norm_type){
      case 0:
        printf("%s = %s\n", i2A(i+starti, left_arg), i2A(n, right_arg));
        break;
      case 1:
        printf("%s = %s - %.3f) / %.3f;\n", i2A(i+starti, left_arg),
           i2A(n, right_arg), means[n], sdevs[n]);
        break;
      case 2:
      case 3:
        printf("%s = %.3f * %s\n", i2A(i, left_arg),
           transform_matrix[n][0], i2A(0, right_arg));
        for(m = 1; m < norm_ninputs; m++){
        printf("%s = %s + %.3f * %s\n",
               i2A(i, left_arg), i2A(i, left_arg),
               transform_matrix[n][m], i2A(m, right_arg));
        }
        printf("\n");
        break;
    }
    }

    printf("\n  /* calculate basis function outputs */\n");
    f = -0.5/(hspread);
    starth = starti + ninputs;
    for(h = 0; h < nhidden; h++){
    printf("%s = (%s - %.3f) * (%s - %.3f) / %.3f\n",
           i2A(starth+h, left_arg),
           i2A(starti, right_arg), rbf_means[h][0],
           i2A(starti, right_arg), rbf_means[h][0],
           rbf_sdevs[h][0]);
    for(i = 1; i < ninputs; i++){
        printf("%s = %s + (%s - %.3f) * (%s - %.3f) / %.3f\n",
           i2A(starth+h, left_arg), i2A(starth+h, left_arg),
           i2A(starti+i, right_arg), rbf_means[h][i],
           i2A(starti+i, right_arg), rbf_means[h][i],
           rbf_sdevs[h][i]);
    }
    printf("%s = %s * %.3f\n",
           i2A(starth+h, left_arg), i2A(starth+h, right_arg), f);
    printf("%s = Exp(%s)\n",
           i2A(starth+h, left_arg), i2A(starth+h, right_arg));
    printf("\n");
    }

    if(norm_outputs){
    printf("\n  /* normalize basis function outputs */\n");
    printf("%s = %s\n",
           i2A(starth+nhidden, left_arg), i2A(starth, right_arg));
    for(h = 1; h < nhidden; h++){
        printf("%s = %s + %s\n", i2A(starth+nhidden, left_arg),
           i2A(starth+nhidden, left_arg),
           i2A(starth+h, right_arg));
    }
    printf("\n");
    printf("%s = %s / %d\n", i2A(starth+nhidden, left_arg),
           i2A(starth+nhidden, right_arg), nhidden);
    for(h = 1; h < nhidden; h++){
        printf("%s = %s / %s\n", i2A(starth+h, left_arg),
           i2A(starth+h, left_arg), i2A(starth+nhidden, right_arg));
    }
    }

    printf("\n  /* calculate final outputs */\n");
    starty = starti /* ? */;
    for(j = 0; j < noutputs; j++){
    printf("%s = %.3f * %s\n", i2A(starty+j, left_arg),
           rbf_weights[0][j], i2A(starth, right_arg));         
    for(h = 1; h < nhidden; h++){
        printf("%s = %s + %.3f * %s\n",
           i2A(starty+j, left_arg), i2A(starty+j, left_arg),
           rbf_weights[h][j], i2A(starth+h, right_arg));
    }
    printf("\n");
    if(bias){
        printf("%s = %s + %.3f   /* bias node */\n",
           i2A(starty+j, left_arg), i2A(starty+j, right_arg),
           rbf_weights[nhidden][j]);
    }
    }
    printf("\n");
}


char *i2A(int i, char *A)
{
    if(i > 254){
    fatal_error("Too many nodes for delta graph (%d)\n%s",
            i, "Only 127 total nodes can be calculated");
    }
    if(i < 26){
    A[0] = 'A' + i;
    A[1] = '\0';
    } else {
    A[0] = 'A' + i/26 - 1;
    A[1] = 'A' + i%26;
    A[2] = '\0';
    }
    return A;
}
