/***--------------------------------------------------------------------------
* NAME:		rbf_main.c
* PURPOSE:	Entry routine for Radial Basis Function classifier
* USAGE:
*		rbfm -<flags>
* EXAMPLE:	rbfm --                * shows all available flags
*
* SEE ALSO:	rbfm_train.c rbfm_test.c
*
* GLOBALS: 	<flags> defined in rbfm_args.h
*
* DESCRIPTION:
*		sets up arguments for the routine rbfm_train and rbfm_test
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/rbf/rbfm_main.c,v 11.4 2001/08/23 20:12:53 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/rbf/rbfm_main.c,v $
* CREATED 15-Oct-92 lck				MIT LINCOLN LABORATORY
*--------------------------------------------------------------------------**/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char     rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/rbf/rbfm_main.c,v 11.4 2001/08/23 20:12:53 craig Exp $";



/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#include "rbfm.h"
#include "rbfm_args.h"

int
main(int argc, char **argv)
{
    char *comment;
    int map_flag = 1;
    char **vlabels;
    int *vfeatures;

    init_nnsubs(argc, argv, rcsid);
			/* get arguments */
    argc = nnargs_part(argc, argv, length_tab, LEN_TABSIZE);
    if(nraw > NNARGS_MAX_LABELS){
	vfeatures = allocate_ivector(nraw+1);
	nnargs_set_variable("features", vfeatures, argtab, TABSIZE);
	nnargs_set_list_length("features", nraw+1, argtab, TABSIZE);
    } else {
	vfeatures = &features[0];
    }
    if(noutputs > NNARGS_MAX_LABELS){
	vlabels = (char **)allocate_pvector(noutputs+1);
	nnargs_set_variable("labels", (int *)vlabels, argtab, TABSIZE);
	nnargs_set_list_length("labels", noutputs+1, argtab, TABSIZE);
    } else {
	vlabels = &labels[0];
    }
    argc = nnargs(argc, argv, argtab, TABSIZE);

    print_log_header();

    check_desc_args(path_build(data_path, desc_fname), vlabels, &nraw, 
		    &noutputs, &map_flag, &npatterns);
    if(noutputs == 0) fatal_error("noutputs in %s must be greater than 0",
				  desc_fname);
    if(nraw == 0) fatal_error("ninputs in %s must be greater than 0",
			      desc_fname);

    ninputs = fix_features(nraw, ninputs, vfeatures);

    comment = make_comment_string(nnargs_get_flags(argtab, TABSIZE));

    if (verbosity > 1) {
	printf("%s", comment);
	fflush(stdout);
    }

    srandom(seed);		/* initialize random numbers */

    /* remember to strip off the first entry in features, which is a length
       which is duplicated in ninputs */
    if(train_flag){
	rbfm_train(create_flag, nraw, ninputs, vfeatures, noutputs,
		  npatterns, debug, verbosity, data_path,
		  data_fname, exp_path, param_fname, comment,
		  bias, hspread, max_ratio,
		  cluster_param_fname, minimum_variance);
    } else {
	rbfm_test(create_flag, map_flag, nraw, ninputs, noutputs, npatterns,
		  debug, verbosity, verror, data_path, data_fname, exp_path,
		  error_fname, param_fname, vlabels, extra_hspread);
    }

    print_log_footer();

    exit_nnsubs();
    exit(0);
}
