/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
                                  /* svm2c.c */
/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#if HAVE_STRINGS_H
#include <strings.h>
#endif
#include <nnlib.h>
#define GEN2CVARS
#include <gen_2c_args.h>
#include "svm_pack.h"

/* svm specific variables */
int  nlayers, *layers, cost_func, sigm_type;
static float *train_priors_list;

char nsvm_name[64];
char kernel_name[64];
char output_name[64];
char struct_name[64];

int kernel_type;
int sigmoid_fit;
int nsvm;
int multi_type;

/* internal function declarations */
int read_model_file(char **argv);
void free_model(int model);

void get_params(int model);
void get_general_params(int model);
void get_specific_params(int model);

void set_priors(int model);

void print_comments(void);
void print_declarations(void);
void print_macros(void);
void print_classify_code(int model);
void clear_macros(void);

static ARG argtab2[] = 
{
  {"sigmoid_fit", BOOLEAN, &sigmoid_fit, 
        "fit sigmoid to output to approximate posterior prob"},
};
#define TABSIZE2 (sizeof(argtab2)/sizeof(ARG))

int
main(int argc, char **argv)
{
    int model;
    ARG *argtab;
    int tabsize;

    /* process arguments */
    nnargs_combine_argtabs(argtab2c, TABSIZE2C, argtab2, TABSIZE2,
			   &argtab, &tabsize);
    argc = nnargs(argc, argv, argtab, tabsize);

    /* check arguments */
    if(strlen(class_suff) > 53){
	fatal_error("suffix (%s) is too long.  use at most 50 characters",
		    class_suff);
    }

    /* set up subroutine names */
    strcpy(classify_name, "classify");
    strcpy(normalize_name, "normalize");
    strcpy(nraw_name, "NRAW");
    strcpy(ninputs_name, "NINPUTS");
    strcpy(noutputs_name, "NCLASSES");
    strcpy(nsvm_name, "NSVM");
    strcpy(kernel_name, "svm_kernel");
    strcpy(output_name, "svm_output");
    strcpy(struct_name, "svm_struct");
    if(class_suff[0] != '\0'){
        strcpy(class_exten, "_");
	strcat(class_exten, class_suff);
	strcat(classify_name, class_exten);
	strcat(normalize_name, class_exten);
	strcat(nraw_name, class_exten);
	strcat(ninputs_name, class_exten);
	strcat(noutputs_name, class_exten);
	strcat(nsvm_name, class_exten);
	strcat(kernel_name, class_exten);
	strcat(output_name, class_exten);
	strcat(struct_name, class_exten);
    } else {
      strcpy(class_exten, "");
    }

    /* read in parameters */
    model = read_model_file(argv);
    get_params(model);

    set_priors(model);

    print_comments();
    print_macros();
    print_declarations();
    print_classify_code(model);
    print_normalize_code(normalize_name);
    clear_macros();

    free_model(model);
    exit(0);
}

int
read_model_file(char **argv)
{
    int   model, norm_model, normalization;
    FILE *param_file;
    int nraw, rv;
    int verbosity = 0;

    param_file = nn_fopen(param_fname, "r", "for read of model file");
    norm_model = norm_read_paramfile(param_file, &normalization);
    if (norm_model < 0)
      fatal_error("%s: read error %d %s %s", argv[0], norm_model,
		  "on normalization parameter file", param_fname);
    rv = feature_read_paramfile(param_file, &features, &nraw, &ninputs);
    if (rv < 0)
      fatal_error("%s: read error %d %s", argv[0], rv,
		  "on feature selection parameters");
   
    model = svm_read_model_multi(param_file, verbosity);
    if (model < 0)
      fatal_error("%s: read error %d on model file %s",
		  argv[0], model, param_fname);
    nn_fclose(param_file, param_fname, "for read of model file");

    return((int) model);
}

void
free_model(int model)
{
  norm_free();
  svm_free(model);
}
void
get_params(int model)
{
    get_specific_params(model);
    get_general_params(model);
    get_norm_params(model, ninputs, features);
}

void
get_specific_params(int model)
{
  kernel_type = *(int *)svm_param(0, "kernel_type");
  if(!sigmoid_fit){
    sigmoid_fit = *(int *)svm_param(0, "sigmoid_fit");
  }
  nsvm = *(int *)svm_param_multi(model, "n");
  multi_type = *(int *)svm_param_multi(model, "type");
}

void
get_general_params(int model)
{
    /* get general parameters */
    ninputs = *(int *)svm_param_multi(model, "ninputs");
    noutputs = *(int *)svm_param_multi(model, "noutputs");
}

void
set_priors(int model)
{
    int i;
    float total;

    /* now that we know noutputs, we can check the priors settings */
    if(adjust_test){
	if(test_uniform_priors){
	    if(test_priors_list != NULL)
	      deallocate_vector(test_priors_list);
	    test_priors_list = allocate_vector(noutputs+1);
	    test_priors_list[0] = noutputs;
	    for(i = 1; i <= noutputs; i++)
	      test_priors_list[i] = 1./(float)noutputs;
	} else {
	    if(test_priors_list[0] != (float)noutputs)
	      fatal_error("%s: %s %.0f, should be %d",
			  "test_priors_list length is", test_priors_list[0],
			  noutputs);
	    total = 0.;
	    for(i = 1; i <= noutputs; i++) total += test_priors_list[i];
	    for(i = 1; i <= noutputs; i++) test_priors_list[i] /= total;
	}
	train_priors_list = *(float **)svm_param_multi(model, "class_cnt");
	total = *(float *)svm_param_multi(model, "total_npatterns");
	for(i = 0; i < noutputs; i++) train_priors_list[i] /= total;
    }
}

void
print_comments(void)
{
    printf("/* %s */\n", param_fname);
    print_norm_comments();
    switch (kernel_type) {
    case SMO_LINEAR:
      printf("/* Linear kernel */\n");
      break;
    case SMO_LINEAR_USE_SVS:
      printf("/* Linear kernel, using support vectors */\n");
      break;
    case SMO_GAUSSIAN:
      printf("/* Gaussian kernel */\n");
      break;
    case SMO_POLYNOMIAL:
      printf("/* Polynomial kernel */\n");
      break;
    case SMO_INHOMOGENEOUS_POLYNOMIAL:
      printf("/* Inhomogeneous Polynomial kernel */\n");
    }
    if(sigmoid_fit){
      printf("/* Fitting outputs to a sigmoid */\n");
    } else {
      printf("/* Raw outputs */\n");
    }
    if(multi_type == 0){
      printf("/* Single SVM per class, class vs all others */\n");
    } else {
      printf("/* SVM for each pair of classes */\n");
    }
    printf("\n");
}

void
print_declarations(void)
{
    printf("\n");
    printf("#include <stdio.h>\n");
    printf("#include <math.h>\n");
    printf("\n");

    printf(" /* structure declarations */\n");
    printf("struct %s {\n", struct_name);
    if(kernel_type == SMO_LINEAR){
      printf("  float weight[%s];\n", ninputs_name);
    } else {
      printf("  int sv_n;\n");
      printf("  float **support_vectors;\n");
      printf("  float *alpha;\n");
      printf("  int *sv_y;\n");
      if(kernel_type == SMO_GAUSSIAN){
	printf("  float gauss_var_times_two;\n");
      } else if(kernel_type == SMO_POLYNOMIAL ||
		kernel_type == SMO_INHOMOGENEOUS_POLYNOMIAL){
	printf("  int poly_order;\n");
	printf("  float inv_poly_divisor;\n");
      }
    }
    printf("  float threshold;\n");
    if(sigmoid_fit){
      printf("  float posterior_aexp;\n");
    }
    printf("  int class_a;\n");
    if(multi_type != 0 || nsvm == 1){
      printf("  int class_b;\n");
    }
    printf("};\n");

    printf("\n /* function declarations */\n");
    printf("extern int %s(float *inputs,float * outputs);\n",
	   classify_name);
    printf("extern int %s(float * inputs);\n", normalize_name);
    printf("static float %s(struct %s *svmP, float *inputs);\n",
	   output_name, struct_name);
    printf("static float %s(struct %s *svmP, float *p1, float *p2);\n",
	   kernel_name, struct_name);
    printf("\n\n");

}

void
print_macros(void)
{
    print_norm_macros(nraw_name, ninputs_name);
    printf("#define %s %d\n", nsvm_name, nsvm);
    printf("#define %s %d\n", noutputs_name, noutputs);
    printf("\n");
}    

void
clear_macros(void)
{
    clear_norm_macros();
    printf("#undef %s\n", nsvm_name);
    printf("#undef %s\n", noutputs_name);
    printf("\n");
}    

void
print_classify_code(int model)
{
  int i, j;
  float *weights, threshold, aexp;
  int class_a, class_b;
  char svec_name[16];
  float *alpha, **svectors, tempF;
  int *sv_y;
  int sv_n, tempI;

  printf("int %s (inputs, outputs)\n", classify_name);
  printf("   float *inputs,*outputs;\n");
  printf("{\n");

  if(kernel_type != SMO_LINEAR){
    for(i = 0; i < nsvm; i++){
      sv_n = *(int *)svm_param(i, "sv_n");
      svectors = *(float ***)svm_param(i, "support_vectors");
      for(j = 0; j < sv_n; j++){
	sprintf(svec_name, "svec%d_%d", i, j);
	print_vector_initialization(svec_name, svectors[j], ninputs,
				    ninputs_name);
      }
      sprintf(svec_name, "svec%d", i);
      printf("static float *svec%d[%d] = {", i, sv_n);
      for(j = 0; j < sv_n; j++){
	printf("svec%d_%d, ", i, j);
      }
      printf("};\n");
      alpha = *(float **)svm_param(i, "sv_alpha");
      sprintf(svec_name, "alpha%d", i);
      print_vector_initialization(svec_name, alpha, sv_n, NULL);
      sv_y = *(int **)svm_param(i, "sv_y");
      sprintf(svec_name, "sv_y%d", i);
      print_ivector_initialization(svec_name, sv_y, sv_n, NULL);
    }
  }
  printf(" struct %s svm_array[%s] = {\n", struct_name, nsvm_name);
  for(i = 0; i < nsvm; i++){
      printf("  {\n");
      if(kernel_type == SMO_LINEAR){
	printf("    {\n");
	weights = *(float **)svm_param(i, "weight");
	print_vector(weights, ninputs);
	printf("    },\n");
      } else {
	sv_n = *(int *)svm_param(i, "sv_n");
	sprintf(svec_name, "svec%d", i);
	printf("    %d, svec%d, alpha%d, sv_y%d,", sv_n, i, i, i);
	if(kernel_type == SMO_GAUSSIAN){
	  tempF = *(float *)svm_param(i, "gaussian_sigma");
	  tempF *= tempF;
	  tempF *= 2.0;
	  printf("%f, ", tempF);
	} else if(kernel_type == SMO_POLYNOMIAL ||
		  kernel_type == SMO_INHOMOGENEOUS_POLYNOMIAL){
	  tempF = *(float *)svm_param(i, "polynomial_divisor");
	  tempF = 1./tempF;
	  tempI = *(int *)svm_param(i, "polynomial_order");
	  printf("%d, %f,", tempI, tempF);
	}
      }
      threshold = *(float *)svm_param(i, "threshold");
      class_a = *(int *)svm_param(i, "multi_class_a");
      if(multi_type != 0){
	class_b = *(int *)svm_param(i, "multi_class_b");
      }
      aexp = *(float *)svm_param(i, "posterior_aexp");
      printf("    %f, ", threshold);
      if(sigmoid_fit){
	printf("%f, ", aexp);
      }
      if(multi_type != 0){
	printf("%d, %d\n", class_a, class_b);
      } else {
	if(nsvm == 1){
	  if(class_a) printf("1, 0 \n");
	  else printf("0, 1\n");
	} else {
	  printf("%d \n", class_a);
	}
      }
      printf("  },\n");
  }
  printf("  };\n");
  printf("  int n, best;\n");
  printf("  float y[%s];\n", noutputs_name);
  printf("  float ya;\n");
  if(multi_type != 0){
    printf("  float yb;\n");
    printf("  int ia, ib;\n");
  }
  if(adjust_test){
    print_vector_initialization("new_priors_list", &test_priors_list[1],
				noutputs, noutputs_name);
    print_vector_initialization("old_priors_list", train_priors_list,
				noutputs, noutputs_name);
    printf("  float total;\n");
  }
  
  /* copy and normlize inputs */
  print_copy_inputs();
  
  printf("\n  /* normalize loaded data */\n");
  printf("  %s(x);\n", normalize_name);
  
  /* write out classification */
  printf("\n  /* calculate raw outputs */\n");
  printf("  for(n = 0; n < %s; n++)\n", noutputs_name);
  if(multi_type == 0){
    printf("    y[n] = 0.;\n");
  } else {
    printf("    y[n] = MAXFLOAT;\n");
  }
  printf("  for(n = 0; n < %s; n++){\n", nsvm_name);
  printf("    ya = %s(&svm_array[n], x);\n", output_name);
  if(multi_type == 0){
    printf("    y[svm_array[n].class_a] = ya;\n");
    if(nsvm == 1){
      if(sigmoid_fit){
	printf("    y[svm_array[n].class_b] = 1-ya;\n");
      } else {
	printf("    y[svm_array[n].class_b] = -1.0*ya;\n");
      }
    }
  } else {
    if(sigmoid_fit){
      printf("    yb = 1-ya;\n");
    } else {
      printf("    yb = -1.0*ya;\n");
    }
    printf("    ia = svm_array[n].class_a;\n");
    printf("    ib = svm_array[n].class_b;\n");
    printf("    if(ya < y[ia]) y[ia] = ya;\n");
    printf("    if(yb < y[ib]) y[ib] = yb;\n");
  }
  printf("  }\n");
  printf("\n  /* copy outputs */\n");
  printf("  for(n = 0; n < %s; n++)\n", noutputs_name);
  printf("    outputs[n] = y[n];\n");

    if(adjust_test){
	printf("\n  /* scale outputs by priors */\n");
	printf("  for(i = 0; i < %s; i++){\n", noutputs_name);
	printf("    outputs[i] *= new_priors_list[i];\n");
	printf("    outputs[i] /= old_priors_list[i];\n");
	printf("  }\n");
	printf("  /* rescale the new outputs to sum to 1 again */\n");
	printf("  total = 0;\n");
	printf("  for(i = 0; i < %s; i++) total += outputs[i];\n",
	       noutputs_name);
	printf("  for(i = 0; i < %s; i++) outputs[i] /= total;\n",
	       noutputs_name);
    }
  printf("\n  /* find highest output */\n");
  printf("  for(best = n = 0; n < %s; n++)\n", noutputs_name);
  printf("    if(outputs[best] < outputs[n]) best = n;\n");
  printf("  return(best);\n");
  printf("}\n");

  printf(" /* subroutines */\n");
  printf("static float %s(struct %s *svmP, float *x){\n",
	 output_name, struct_name);
  printf("  float total = 0.;\n");
  if(kernel_type == SMO_LINEAR){
    printf("  int j;\n");
    printf("  for(j = 0; j < %s; j++)\n", ninputs_name);
    printf("    total += svmP->weight[j] * x[j];\n");
    printf("  total -= svmP->threshold;\n");
  } else {
    printf("  int i;\n");
    printf("  float xkernel;\n");
    printf("  for(i = 0; i < svmP->sv_n; i++){\n");
    printf("    xkernel = %s(svmP, x, svmP->support_vectors[i]);\n", kernel_name);
    printf("    total += (svmP->sv_y[i]*svmP->alpha[i]*xkernel);\n");
    printf("  }\n");
    printf("  total -= svmP->threshold;\n");
  }
  if(sigmoid_fit){
    printf("  total = 1.0/(1+exp(svmP->posterior_aexp *total));\n");
  }
  printf("  return total;\n");
  printf("}\n");
  
  if(kernel_type == SMO_LINEAR) return;
  
  printf("static float %s(struct %s *svmP, float *p1, float *p2){\n",
	 kernel_name, struct_name);
  printf("  int j;\n");
  if(kernel_type == SMO_GAUSSIAN){
    printf("  double sum_squared = 0.;\n");
  } else {
    printf("  double dot_product = 0.;\n");
  }
  if(kernel_type != SMO_LINEAR_USE_SVS){
    printf("  float total, diff;\n");
  }
  if(kernel_type == SMO_GAUSSIAN){
    printf("  for(j = 0; j < %s; j++){\n", ninputs_name);
    printf("    diff = p1[j] - p2[j];\n");
    printf("    sum_squared -= diff*diff;\n");
    printf("  }\n");
    printf("  sum_squared /= svmP->gauss_var_times_two;\n");
    printf("  total = exp(sum_squared);\n");
    printf("  return total;\n");
  } else {
    printf("  for(j = 0; j < %s; j++){\n", ninputs_name);
    printf("    dot_product += p1[j]*p2[j];\n");
    printf("  }\n");
    if(kernel_type == SMO_LINEAR_USE_SVS){
      printf("  return dot_product;\n");
    } else {
      if(kernel_type == SMO_POLYNOMIAL){
	printf("  total = dot_product*(svmP->inv_poly_divisor);\n");
      } else {
	printf("  dot_product = (1.0 + dot_product)*(svmP->inv_poly_divisor);\n");
	printf("  total = dot_product;\n");
      }
      printf("  for(j = 1; j < svmP->poly_order; j++)\n");
      printf("    total *= dot_product;\n");
      printf(" return total;\n");
    }
  }
  printf("}\n");
}
