/***--------------------------------------------------------------------------
* NAME:		svm_main.c
* PURPOSE:	Entry routine for svm program 
*
* USAGE: 	svm -<flags>
* EXAMPLE:	svm --		 * shows all available flags
* 
* SEE ALSO:	svm_train.c svm_cv.c svm_test.c svm_pack.c
*
* GLOBALS:	<flags> defined in svm_args.h <gen_class_args.h>
*
* DESCRIPTION:	sets up arguments for the routine svm()
*
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/svm/svm_main.c,v 1.2 2001/08/23 20:12:56 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/svm/svm_main.c,v $
* CREATED: 1/4/2001 rpl                             MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/algorithm/svm/svm_main.c,v 1.2 2001/08/23 20:12:56 craig Exp $";



/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#define NO_CV_TABLE
#include <cv_args.h>
#include <gen_class_args.h>
#include "svm_args.h"
#include "svm.h"
#include "svm_pack.h"

int
main(int argc, char **argv)
{
    char	*comment;
    int use_splits;
    DEFAULTS_FILE_STRUCT *df_args;
    /* argument table with all entries */
    ARG *argtab;
    int tabsize;
    int roc_cost = 0, roc_target = 0;
    float roc_lower = 0, roc_upper = 1;

    /******************* read argument list **********************/
    init_nnsubs(argc, argv, rcsid);

    /* get arguments */
    nnargs_combine_argtabs(gen_tab, GEN_TABSIZE, svm_tab, SVM_TABSIZE,
			   &argtab, &tabsize);
    argc = nnargs(argc, argv, argtab, tabsize);
    
    print_log_header();

    /************* read defaults and set input data parameters ********/
    /* read the defaults file */
    if(strcmp(desc_fname, "") == 0){ /* desc_fname is unset */
	fatal_error("%s: -fdescribe must be set", argv[0]);
    }
    /* set the number of inputs and outputs and read the labels,
       allocating the space */
    df_args = (DEFAULTS_FILE_STRUCT *)
      nn_malloc(sizeof(DEFAULTS_FILE_STRUCT), "for the defaults file struct");
    read_defaults_file(path_build(data_path, desc_fname), df_args);
    if(df_args->noutputs == 0)
      fatal_error("noutputs in %s must be greater than 0", desc_fname);
    if(df_args->ninputs == 0)
      fatal_error("ninputs in %s must be greater than 0", desc_fname);

    /* set the number of patterns to no more than the number of lines
       in the data file */
    if(strcmp(data_fname, "") == 0){ /* data_fname is unset */
	fatal_error("%s: -finput must be set", argv[0]);
    }
    check_file_length(path_build(data_path, data_fname), &npatterns);
    if(npatterns == 0)
      fatal_error("%s: Empty data file %s", argv[0],
		  path_build(data_path, data_fname));
    /* check the data base type */
    if (df_args->db_type != CLASS_DATA)
      fatal_error("class data required, check defaults file");

    /* check the feature selection arguments */
    if (ninputs > df_args->ninputs)
      ninputs = df_args->ninputs;
    ninputs = fix_features(df_args->ninputs, ninputs, features);

    /* set up the cross validation arguments */
    if(train_flag && cross_valid > 2){
	warning("%s\n%s\n","-train and -cross_valid are both set.",
		" -cross_valid will be ignored");
	cross_valid = 0;
    }
    use_splits =
      fix_cross_valid_args(&cross_valid, &cv_nfolds, &cv_nsplits, cv_splits,
			   cv_train_mask,cv_test_mask,cross_valid_from_file,
			   path_build(data_path, cross_valid_fname));
        
    /********* fix up priors adjustment arguments *********************/
    if(adjust_test )
           {
           sigmoid_fit=1;
	   }
    fix_priors(adjust_train, train_uniform_priors, &train_priors_list,
 	       adjust_test, test_uniform_priors, &test_priors_list,
 	       train_flag, cross_valid, df_args->noutputs);
    
    /****************** check for unnecessary arguments ****************/
    if(log_outputs){
	warning("%s does not support log_outputs", argv[0]);
	log_outputs = 0;
    }
    /************* check SVM args  ****************/
    if ( kernel_type != SMO_LINEAR && 
	 kernel_type != SMO_GAUSSIAN &&
         kernel_type != SMO_POLYNOMIAL &&
         kernel_type != SMO_INHOMOGENEOUS_POLYNOMIAL &&
         kernel_type != SMO_LINEAR_USE_SVS)
	{
	fatal_error("invalid kernel type %d\n", kernel_type);
	}
    if ( cbound <= 0.0)
	{
        fatal_error("cbound must be greater than 0");
	}
    if ( kernel_type == SMO_GAUSSIAN && kernel_sigma <= 0.0)
	{
        fatal_error("kernel_sigma must be greater than 0");
	}
    if ( (kernel_type == SMO_POLYNOMIAL || 
          kernel_type == SMO_INHOMOGENEOUS_POLYNOMIAL) && kernel_order <= 0)
	{
        fatal_error("kernel_order must be greater than 0");
	}
    if ( (kernel_type == SMO_POLYNOMIAL || 
          kernel_type == SMO_INHOMOGENEOUS_POLYNOMIAL) &&  
          kernel_poly_divisor <= 0.0)
	{
        fatal_error("kernel_poly_divisor must be greater than 0");
	}
    if ( kkt_tolerance <= 0.0)
	{
        fatal_error("kkt_tolerance must be greater than 0");
	}
    if ( alpha_tolerance <= 0.0)
	{
        fatal_error("alpha_tolerance must be greater than 0");
	}

    /************** make and print header comment *******************/
    comment = make_comment_string(nnargs_get_flags(argtab, tabsize));
    if (verbosity > FLAGS_VERBOSITY) {
	printf("%s", comment);
	fflush(stdout);
    }
    
    srandom(seed);		/* initialize random numbers */

    /************* call the selected driver ***************/
    if(train_flag)
		{
	        svm_train(df_args->ninputs, ninputs, features, df_args->noutputs,
		          normalize, npatterns, verbosity, 
		          data_path, data_fname, norm_fname, 
                          exp_path, param_fname, comment, df_args->output_labels, 
                          cbound, kernel_type, kernel_sigma,
                          kernel_order, kernel_poly_divisor,
                          sigmoid_fit,
                          kkt_tolerance, alpha_tolerance,
                          multi_class_type);
		  
		}
    else if (cross_valid > 1) 
		{
		svm_cross_valid(df_args->db_type, df_args->ninputs, ninputs, 
                        features, df_args->noutputs, normalize,
			npatterns, verbosity, verror, 
			cv_nfolds, cv_nsplits, use_splits, 
                        cv_splits, &cv_train_mask[1], 
                        &cv_test_mask[1], data_path,
			data_fname, norm_fname, 
                        exp_path, error_fname,
			param_fname, df_args->output_labels, comment, 
                        adjust_test, &test_priors_list[1], 
                        cv_random, log_outputs,
			(roc_cost?ROC_COST:SQ_ERR), roc_target, roc_lower, 
                        roc_upper,
 			cbound, kernel_type, kernel_sigma,
                        kernel_order, kernel_poly_divisor,
                        sigmoid_fit,
                        kkt_tolerance, alpha_tolerance,
                        multi_class_type);

                  } 
     else 
		 {
	         svm_test(create_flag, df_args->db_type, df_args->ninputs, ninputs, 
                 features, df_args->noutputs, npatterns, verbosity, 
                 verror, data_path, data_fname, 
                 exp_path, error_fname, param_fname, 
                 df_args->output_labels, adjust_test, &test_priors_list[1], 
		 sigmoid_fit);
		 }

    print_log_footer();

    /* free the vectors allocated when argtab was built */
    nnargs_free_argtab(argtab, tabsize);

    /* free the vectors allocated when the defaults file was read */
    free_defaults_file_struct(df_args);
    exit_nnsubs();
    exit(0);
}
