/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <math.h>
#include <nnlib.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif

static int npatterns = 200;
static int seed = 1;
static float xscale = 10;
static float yscale = 1;
static float separation = 1;
static float angle = 45;
static ARG argtab[] =
{
  { "npatterns", INT, &npatterns, "Number of pairs of points to generate"},
  { "seed", INT,&seed,"Random number seed"},
  { "xscale", FLOAT, (int *)&xscale,"Scale factor to stretch x coordinate"},
  { "yscale", FLOAT, (int *)&yscale,"Scale factor to stretch y coordinate"},
  { "separation", FLOAT, (int *)&separation,"Separation between the clusters (from center, normal to angle"},
  { "angle", FLOAT, (int *)&angle,"Angle of offset from horizontal (0 - 90)"},
};
#define TABSIZE (sizeof(argtab)/sizeof(ARG))

float pi = 3.1415926;

int
main(argc,argv)
     int argc;
     char **argv;
{
  float random_uniform(),x,y;
  float cos_a, sin_a;
  int i;
  double cluster_angle;
  void make_cluster_point();

  argc = nnargs(argc,argv,argtab,TABSIZE);
	
  srandom(seed);
  cluster_angle = angle / 180. * pi;
  cos_a = cos(cluster_angle);
  sin_a = sin(cluster_angle);

  for (i=0; i<npatterns/2; i++) {

	make_cluster_point (&x, &y, xscale, yscale, cos_a, sin_a, separation);
    
	printf("0 %.3g %.3g\n",x,y);

	make_cluster_point (&x, &y, xscale, yscale, cos_a, sin_a, -separation);

	printf("1 %.3g %.3g\n",x,y);
  } 
  return 0;
}

void
make_cluster_point (x, y, xscale, yscale, cos_a, sin_a, separation)
float *x, *y, xscale, yscale, separation, cos_a, sin_a;
{
    float r, xtmp, ytmp;
    double theta;

    r = random_uniform();
    theta = 2 * pi * random_uniform();

    /* calculate point position and scale */
    xtmp = r * cos(theta) * xscale;
    ytmp = r * sin(theta) * yscale + separation;
    					/* rotate point */
    *x = xtmp * cos_a - ytmp * sin_a;
    *y = xtmp * sin_a + ytmp * cos_a;
}

