/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/***---------------------------------------------------------------------------
* NAME:		star
* PURPOSE:	generate data for classes in a star pattern to test lnknet
*
* SEE ALSO: angle, cross 
*
* DESCRIPTION:
* 	There are three groups of classes. One group (0,3,6,9) contains data
* in only one cluster with std dev sigma[0]. The second group (1,5,7,11) 
* contains data in two clusters with std dev sigma[1]. The third group
* (2,4,8,11) contains data in three clusters with std dev sigma[2].
* Clusters are arranged as spokes around a desired center.
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/datagen/daisy.c,v 1.6 2001/08/23 20:13:00 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/datagen/daisy.c,v $
* CREATED: 11-Jan-92 rpl                             MIT LINCOLN LABORATORY
*---------------------------------------------------------------------------
* Copyright 1990 MIT Lincoln Laboratory.
* This software is not for release and should neither be copied
* nor distributed without explicit permission of MIT Lincoln Laboratory.
--------------------------------------------------------------------------***/
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <math.h>
#include <nnlib.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif

static int npatterns = 500;
static int seed = 0;
static float m_scale = 1.0;
static float sig_scale = 1.0;
static int x_offset = 0.0;
static int y_offset = 0.0;
static int nclasses = 12;
static float xstd_dev[101] = 
	    {12, 2.0, 1.0, 0.5, 2.0, 0.5, 1.0, 2.0, 1.0, 0.5, 2.0, 0.5, 1.0};
static float ystd_dev[101] = 
	    {12, 0.2, 0.2, 0.2, 0.2, 0.2, 0.2, 0.2, 0.2, 0.2, 0.2, 0.2, 0.2};
static float mean_diff[101] =
	    {12, 6.0, 4.0, 3.0, 6.0, 3.0, 4.0, 6.0, 4.0, 3.0, 6.0, 3.0, 4.0};
static int nclusters[101] = 
	    {12, 1,     2,   3,   1,   3,   2,   1,   2,   3,   1,   3,   2};
		 

static ARG argtab[] =
{
 {"nclasses", INT, &nclasses, "Number of classes or spokes in pattern"},
 {"nclusters", INT_LIST, (int *)&nclusters[0], "num clusters array per class"},
 {"mean_dev", FLOAT_LIST, (int *)&mean_diff[0], "mean diff array per class"},
 {"xstd_dev", FLOAT_LIST, (int *)&xstd_dev[0], "x std dev array per class"},
 {"ystd_dev", FLOAT_LIST, (int *)&ystd_dev[0], "y std dev array per class"},
 {"npatterns", INT, &npatterns, "Number of patterns to generate per class"},
 {"seed", INT,&seed,"Random number seed"},
 {"x_offset", INT,&x_offset,"Global x offset for center of star"},
 {"y_offset", INT,&y_offset,"Global y offset for center of star"},
 {"m_scale", FLOAT, (int *)&m_scale,"Global scale factor for cluster centers"},
 {"sig_scale", FLOAT, (int *)&sig_scale,"Global scale factor for cluster standard deviations"},
};
#define TABSIZE (sizeof(argtab)/sizeof(ARG))


int
main(argc,argv)
     int argc;
     char **argv;
{
  int icluster;
  int i,j;
  int class;
  float x,y, xcluster, ycluster;
  float prob_array[100];
  float smallest_angle, class_angle;
  float pi = 3.1415926;
  void generate_pattern();

  argc = nnargs(argc,argv,argtab,TABSIZE);
	
  srandom(seed);
  smallest_angle = 2*pi/nclasses;

  for( class=1; class<=nclasses; class++)
	{
	/* angle from origin for all clusters in this class */
	class_angle = smallest_angle*(class-1);

	/* each cluster is equally likely */
	for(j=0; j<nclusters[class]; j++)
		prob_array[j] = 1.0/nclusters[class];

	for (i=0; i<npatterns; i++) 
	    {
	    /* select cluster in this class and calculate x and y offset */
	    icluster = random_select(prob_array, nclusters[class]);
	    xcluster = (icluster+1)*mean_diff[class]*m_scale; 
	    ycluster = 0;
	    generate_pattern(&x, &y, xcluster, ycluster, 
			     xstd_dev[class]*sig_scale, 
			     ystd_dev[class]*sig_scale, 
			     class_angle);
	    x += x_offset;
	    y += y_offset;

	    printf("%d %.3g %.3g\n",class-1,x,y);
	    } 
        }
  return 0;
}

/* sample a point from a gaussian cluster with given center 
   and x and y variances and rotated about the point 0,0  counter clockwise
   from the positive x axis 
   ----------------------------------------------------------------*/
void
generate_pattern(x, y, xmean, ymean, xsigma, ysigma, angle)
float *x, *y;
float xmean, ymean, xsigma, ysigma;
float angle;
{
    float xtemp, ytemp, cos_a, sin_a;
    double theta;

    theta = angle;
    cos_a = cos(theta);
    sin_a = sin(theta);

    /* sample from unrotated pattern position */ 
    xtemp = xmean + random_gauss(xmean, xsigma);
    ytemp = ymean + random_gauss(ymean, ysigma);

    /* rotate */
    *x = xtemp * cos_a - ytemp * sin_a;
    *y = xtemp * sin_a + ytemp * cos_a;
}
