/***---------------------------------------------------------------------------
* NAME:		gauss_map.c 
* PURPOSE:	Generate mapping data for a gaussian function
* DESCRIPTION:  
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/datagen/gauss.c,v 1.7 2001/08/23 20:13:00 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/datagen/gauss.c,v $
* CREATED: 13-Jul-92 lck			MIT Lincoln Laboratory
--------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/datagen/gauss.c,v 1.7 2001/08/23 20:13:00 craig Exp $";
/* header files 
-----------------*/
/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <math.h>
#if STDC_HEADERS
#include <string.h>
#include <stdlib.h>
#endif
#include <nnlib.h>

/* definitions  
----------------*/
#define TABSIZE (sizeof(argtab)/sizeof(ARG))

/* Variables set from command line flags 
-----------------------------------------*/
static int debug = 0;
static float sdev = .5;
static float mean = 0.;
static float ymax = 1.;
static float xmin = -2.;
static float xmax = 2.;
static int npatterns = 500;
static int verbose = 1;
static int seed = 0;

/*** Argument Table ***/
static ARG argtab[]=
{
    {"debug",  INT, &debug, "higher values for more printout"},
    {"seed", INT, &seed, "Random number seed"},
    {"sdev", FLOAT, (int *)&sdev, "standard dev of Gaussian"},
    {"mean", FLOAT, (int *)&mean, "mean of Gaussian"},
    {"ymax", FLOAT, (int *)&ymax, "scaling factor = Gaussian value at mean"},
    {"xmin", FLOAT, (int *)&xmin, "lowest x value"},
    {"xmax", FLOAT, (int *)&xmax, "highest x value"},
    {"npatterns", INT, &npatterns, "number of vectors on each iteration"},
    {"verbose", INT, &verbose, "verbosity"},
};


int
main(argc, argv)
    register int argc;
    register char **argv;
{
    int n;
    float x, y;

    init_nnsubs(argc, argv, rcsid);
    argc = nnargs(argc, argv, argtab, TABSIZE);
    srandom(seed);
    			/* Print out the input flag configuration used */
    if ( verbose > 2)
      nnargs_print_flags(argtab, TABSIZE, stdout);

    if ( verbose > 1)
	printf("generating training data\n");

    for(n = 0; n < npatterns; n++) {
	x = xmin + (random_uniform() * (xmax - xmin));
	y = random_gauss_height(mean, sdev, x);
	printf("%g %g\n", x, y);
    }
  return 0;
}
