/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <math.h>
#include <nnlib.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif

static int npatterns = 200;
static int seed = 1;
static int nclasses = 2;
static int ncenters = 3;
static float center_weights[NNARGS_MAX_LABELS] = {0};
static float center_sdevs[NNARGS_MAX_LABELS] = {0};
static float center_xdev[NNARGS_MAX_LABELS] = {0};
static float center_ydev[NNARGS_MAX_LABELS] = {0};
static float class_xdev[NNARGS_MAX_LABELS] = {0};
static float class_ydev[NNARGS_MAX_LABELS] = {0};
static ARG argtab[] =
{
  { "npatterns", INT, &npatterns,
      "Number of points to generate for each class"},
  { "seed", INT,&seed,"Random number seed"},
  {"nclasses", INT, &nclasses, "number of classes"},
  {"ncenters", INT, &ncenters, "number of centers per class"},
  {"class_xdev", FLOAT_LIST, (int *)class_xdev, "x deviation for each class"},
  {"class_ydev", FLOAT_LIST, (int *)class_ydev, "y deviation for each class"},
  {"center_xdev", FLOAT_LIST, (int *)center_xdev,
     "x deviation for each center in a class"},
  {"center_ydev", FLOAT_LIST, (int *)center_ydev,
     "y deviation for each center in a class"},
  {"weights", FLOAT_LIST, (int *)center_weights,
     "weights for each center in a class"},
  {"sdevs", FLOAT_LIST, (int *)center_sdevs,
     "standard deviations for each center in a class"},
};
#define TABSIZE (sizeof(argtab)/sizeof(ARG))

float pi = 3.1415926;

int
main(argc,argv)
     int argc;
     char **argv;
{
  float x,y;
  int i, class_num, gnum;
  void make_cluster_point();

  argc = nnargs(argc,argv,argtab,TABSIZE);

  /* check list lengths */
  if(class_xdev[0] != (float)nclasses)
    fatal_error("wrong number of floats in class_xdev. need %d. have %f",
		nclasses, class_xdev[0]);
  if(class_ydev[0] != (float)nclasses)
    fatal_error("wrong number of floats in class_ydev");
  if(center_xdev[0] != (float)ncenters)
    fatal_error("wrong number of floats in center_xdev, need %d, have %f",
		ncenters, center_xdev[0]);
  if(center_ydev[0] != (float)ncenters)
    fatal_error("wrong number of floats in center_ydev");
  if(center_weights[0] != (float)ncenters)
    fatal_error("wrong number of floats in weights");
  if(center_sdevs[0] != (float)ncenters)
    fatal_error("wrong number of floats in sdevs");

  srandom(seed);

  for (i=0; i<npatterns; i++) {
    for(class_num = 0; class_num < nclasses; class_num++){
      gnum = random_select(&center_weights[1], ncenters);
      make_cluster_point (&x, &y,
			  center_xdev[gnum+1] + class_xdev[class_num+1],
			  center_ydev[gnum+1] + class_ydev[class_num+1],
			  center_sdevs[gnum+1]);
      printf("%d %.3g %.3g\n",class_num, x,y);
    }
  } 
  return 0;
}

void
make_cluster_point (x, y, xdev, ydev, sdev)
float *x, *y, xdev, ydev, sdev;
{
    /* calculate point position and scale */
    *x = random_gauss(xdev, sdev);
    *y = random_gauss(ydev, sdev);

    /* rotate point */
/*
    *x = xtmp * cos_a - ytmp * sin_a;
    *y = xtmp * sin_a + ytmp * cos_a;
*/
}


