/***---------------------------------------------------------------------------
* NAME:		gnoise.c 
* PURPOSE:	Generate data with gaussian noise 
* DESCRIPTION:  Creates vectors by adding some gaussian noise to n vectors
*     with elements equal to 0,0,0,0   1,1,1,1   2,2,2,2    3,3,3,3
*     ect.
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/datagen/gnoise.c,v 1.7 2001/08/23 20:13:00 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/datagen/gnoise.c,v $
* CREATED: 23-AUGUST-90 rpl                           MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/datagen/gnoise.c,v 1.7 2001/08/23 20:13:00 craig Exp $";
/* header files 
-----------------*/
/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <math.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <nnlib.h>

/* definitions  
----------------*/
#define TABSIZE (sizeof(argtab)/sizeof(ARG))

/* Variables set from command line flags 
-----------------------------------------*/
static int debug=0;
static int noutputs=16;
static int ninputs =12;
static float noise_sdev=.5;
static int npatterns = 500;
static int verbose =1;
static int seed = 0;

/*** Argument Table ***/
static ARG argtab[]=
{
  {"debug", INT,&debug,"higher values for more printout"},
  { "seed", INT,&seed,"Random number seed"},
  {"noutputs", INT,&noutputs,"number of desired centers or clusters"},
  {"ninputs", INT,&ninputs,"number of inputs"},
  {"noise_sdev", FLOAT, (int *)&noise_sdev, "standard dev of additive noise"},
  {"npatterns", INT,&npatterns, "number of vectors on each iteration"},
  {"verbose", INT, &verbose, "verbosity"}
};


int
main(argc,argv)
    register int argc;
    register char **argv;
{
  				/* local variables
  				----------------------*/
    int i, j, n;
    float **pgenerating_centers;	/* centers used to generate data */
    float *pnorm_variance;	/* normalizing variance */
    float data_item;


    init_nnsubs(argc, argv, rcsid);
    argc=nnargs(argc,argv,argtab,TABSIZE);
    srandom(seed);
    			/* Print out the input flag configuration used */
    if ( verbose > 2)
	nnargs_print_flags(argtab, TABSIZE, stdout);


  		/* set up means and variances for underlying clusters
    		 on a high dimensional diagonal so they are equally spaced
  		----------------------------------------------------*/
    pgenerating_centers = allocate_matrix( noutputs, ninputs);
    pnorm_variance = allocate_vector(ninputs);
    for(i=0; i<noutputs; i++) {
	for(j=0; j<ninputs; j++)
	    pgenerating_centers[i][j] = i;
    }
    for(j=0; j<ninputs; j++)
	pnorm_variance[j] = noise_sdev*noise_sdev;

  				/* generate all training vectors
  				------------------------------*/
    if ( verbose > 1)
	printf("generating training data\n");

    for(n=0; n<npatterns; n++) { 
	i = n%noutputs;
	printf("%d",i);
	for(j=0; j<ninputs; j++) {
	    data_item = pgenerating_centers[i][j]+ random_gauss(0.0,noise_sdev);
	    printf(" %g",data_item);
	}
	printf("\n");
    }
  return 0;
}
