/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <math.h>
#include <nnlib.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif

static int npatterns = 400;
static int seed = 1;
static float box_start = -1.;
static float box_end = 1.;
static float box_prob = .9;
static float tail_start = -50.;
static float tail_end = 50.;
static float gauss_mean = 5.;
static float gauss_sdev = 1.;

static ARG argtab[] =
{
  { "npatterns", INT, &npatterns, "Number of pairs of points to generate"},
  { "seed", INT,&seed,"Random number seed"},
  { "box_start", FLOAT, (int *)&box_start, "Start of dense region of class 0"},
  { "box_end", FLOAT, (int *)&box_end, "End of dense region of class 0"},
  { "box_prob", FLOAT, (int *)&box_prob,
      "Probability of class 0 point being in dense region"},
  {"tail_start", FLOAT, (int *)&tail_start, "Start of class 0 tail"},
  {"tail_end", FLOAT, (int *)&tail_end, "End of class 0 tail"},
  {"gauss_mean", FLOAT, (int *)&gauss_mean, "Mean of class 1"},
  {"gauss_sdev", FLOAT, (int *)&gauss_sdev, "Standard Deviation of class 1"},
};
#define TABSIZE (sizeof(argtab)/sizeof(ARG))

int
main(argc,argv)
     int argc;
     char **argv;
{
  float random_uniform(),x;
  int i;
  float box_width, tail_width;

  argc = nnargs(argc,argv,argtab,TABSIZE);

  box_width = box_end - box_start;
  tail_width = tail_end - tail_start;

  srandom(seed);

  for (i=0; i<npatterns/2; i++) {
      x = random_uniform();
      if(random_event(box_prob))
	x = (x * box_width) + box_start;
      else
	x = (x * tail_width) + tail_start;
      printf("0 %.3g\n", x);

      x = random_gauss(gauss_mean, gauss_sdev);
      printf("1 %.3g\n",x);
  } 
  return 0;
}
