/***--------------------------------------------------------------------------
* NAME:		gnoise.c 
* PURPOSE:	Generate data with gaussian noise 
* DESCRIPTION:  Creates vectors by adding some gaussian noise to n vectors
*     with elements equal to 0,0,0,0   1,1,1,1   2,2,2,2    3,3,3,3
*     ect.
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/datagen/mixedCirc.c,v 1.4 2001/08/23 20:13:01 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/datagen/mixedCirc.c,v $
* CREATED: 23-AUGUST-90 rpl                           MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/datagen/mixedCirc.c,v 1.4 2001/08/23 20:13:01 craig Exp $";
/* header files 
-----------------*/
/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <math.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <nnlib.h>

/* definitions  
----------------*/
#define TABSIZE (sizeof(argtab)/sizeof(ARG))

/* Variables set from command line flags 
-----------------------------------------*/
static int noutputs=16;
static int ninputs =12;
static float * mean_list=NULL;
static float * sdev_list=NULL;
static char ** input_types= NULL;
static float *input_type_values=NULL;
static float * offset_list = NULL;
static int npatterns = 500;
static int seed = 0;

/*** Argument Table ***/
static ARG argtab[]=
{
  {"seed", INT,&seed,"Random number seed"},
  {"noutputs", INT,&noutputs,"number of desired centers or clusters"},
  {"ninputs", INT,&ninputs,"number of inputs"},
  {"means", FLOAT_ALLOC_LIST, (int *)&mean_list,
     "means for each input feature"},
  {"sdevs", FLOAT_ALLOC_LIST, (int *)&sdev_list,
     "standard deviations for each input feature"},
  {"input_types", STRING_ALLOC_LIST, (int *)&input_types,
     "CONT,CIRC,BIN,CAT for each input feature"},
  {"input_type_values", FLOAT_ALLOC_LIST, (int *)&input_type_values,
     "0 for CONT or BIN features, range of CIRC features, number of CATagories"},
  {"offsets", FLOAT_ALLOC_LIST, (int *)&offset_list,
     "amount to add to each input mean to find means for higher classes"},
  {"npatterns", INT,&npatterns, "number of vectors on each iteration"},
};

#define MAX_RANDOM	2147483647.0     /* maximum value of random number */
float local_random_gauss( mean, sdev)
     float mean, sdev;
{
	double u1,u2,x,chi,angle;

	u1=random()/(MAX_RANDOM);
	u2=random()/(MAX_RANDOM);
	chi=sqrt(-2*log(u1));
	angle=2*3.141593*u2;
	x=chi*cos(angle);
	x = mean + x*sdev;
	return((float)x);
}

main(argc,argv)
    register int argc;
    register char **argv;
{
  				/* local variables
  				----------------------*/
    int i, j, n;
    float data_item;
    float mean, sdev;

    init_nnsubs(argc, argv, rcsid);
    argc=nnargs(argc,argv,argtab,TABSIZE);
    srandom(seed);

    /* check arguments */
    if(mean_list == NULL || mean_list[0] != ninputs){
	fatal_error("list of means is the wrong length");
    }
    if(sdev_list == NULL || sdev_list[0] != ninputs){
	fatal_error("list of sdevs is the wrong length");
    }
    if(offset_list == NULL || offset_list[0] != ninputs){
	fatal_error("list of offsets is the wrong length");
    }
    if(input_types == NULL || atoi(input_types[0]) != ninputs){
	fatal_error("list of input_types is the wrong length");
    }
    if(input_type_values == NULL || input_type_values[0] != ninputs){
	fatal_error("list of input_type_values is the wrong length");
    }
    /* generate all training vectors
       ------------------------------*/
    for(n=0; n<npatterns; n++) { 
	i = n%noutputs;
	printf("%d",i);
	for(j=0; j<ninputs; j++) {
	    mean = mean_list[j+1]+i*offset_list[j+1];
	    sdev = sdev_list[j+1];
	    data_item = local_random_gauss(mean,sdev);
	    if(strcmp(input_types[j+1],"CIRC") == 0){
		while(data_item < 0) data_item += input_type_values[j+1];
		while(data_item >= input_type_values[j+1])
		  data_item -= input_type_values[j+1];
	    }
	    printf(" %g",data_item);
	}
	printf("\n");
    }
}
