/***---------------------------------------------------------------------------
* NAME:		test_gmix.c 
* PURPOSE:	test gaussian mixture subroutines 
* DESCRIPTION:  Creates vectors by adding some gaussian noise to n vectors
*     with elements equal to 0,0,0,0   1,1,1,1   2,2,2,2    3,3,3,3
*     ect. Then clusters these n vectors using the gaussian mixture
*     subroutines.
*     Adapted from /u1/rpl/nn/src/kmeans/test_kmeans.c, v1.2
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/datagen/test_gmix.c,v 1.4 2001/08/23 20:13:01 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/datagen/test_gmix.c,v $
* CREATED: 25-Mar-91 es					MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/datagen/test_gmix.c,v 1.4 2001/08/23 20:13:01 craig Exp $";
/* header files 
-----------------*/
/* config.h must be first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <math.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <nnlib.h>

/* definitions  
----------------*/
#define TABSIZE (sizeof(argtab)/sizeof(ARG))

/* Variables set from command line flags 
-----------------------------------------*/
static int debug=0;
static int ncenters=16;
static int initial_nmixtures=0;
static int ninputs =12;
static float noise_sdev=.5;
static float variance_scale = 4.0;
static int nvectors = 500;
static int plotdata_flag=0;
static int verbose =1;
static float stop_percentage = 1.0;
static int max_iteration = 10;
static float split_percentage = 1.0;

/*** Argument Table ***/
static ARG argtab[]=
{
  {"debug", INT,&debug,"higher values for more printout"},
  {"max_iteration", INT,&max_iteration,"maximum iterations after each split"},
  {"ncenters", INT,&ncenters,"number of desired centers or clusters"},
  {"initial_nmixtures", INT,&initial_nmixtures,"initial number of clusters"},
  {"ninputs", INT,&ninputs,"number of inputs"},
  {"noise_sdev", FLOAT, (int *)&noise_sdev, "standard dev of additive noise"},
  {"variance_scale", FLOAT, (int *)&variance_scale,
		"factor for variance scaling after split"},
  {"nvectors", INT,&nvectors, "number of vectors on each iteration"},
  {"plotdata_flag", INT,&plotdata_flag, "generate file gmix.data for plot"},
  {"stop_percentage", FLOAT, (int *)&stop_percentage,
		"stop when ave sqrd error percent change less than this"},
  {"split_percentage", FLOAT, (int *)&split_percentage,
		"add +/- this percentage stdev on each split"},
  {"verbose", INT, &verbose, "verbosity"}
};


main(argc,argv)
register int argc;
register char **argv;
 {
  /* local variables
  ----------------------*/
  int i, j, n;
  float *pweights;		/* weights returned */
  float **pcenters;		/* centers returned */
  float **pvariance;		/* variance returned */
  float **pgenerating_centers;	/* centers used to generate data */
  float *pnorm_variance;	/* normalizing variance */
  float **all_training_data;	/* holds all training vectors */
  int report_actual_ncenters;
  int report_iterations;
  int report_nvectors;
  float report_avg_logl;
  int stop_flag;
  int actual_ncenters;


  init_nnsubs(argc, argv, rcsid);
  argc=nnargs(argc,argv,argtab,TABSIZE);
  /* Print out the input flag configuration used */
  if ( verbose >= 2)
	nnargs_print_flags(argtab, TABSIZE, stdout);


  /* set up means and variances for underlying clusters
     on a high dimensional diagonal so they are equally spaced
  ----------------------------------------------------*/
  pweights = allocate_vector(ncenters);
  pcenters = allocate_matrix( ncenters, ninputs);
  pvariance = allocate_matrix( ncenters, ninputs);
  pgenerating_centers = allocate_matrix( ncenters, ninputs);
  pnorm_variance = allocate_vector(ninputs);
  for(i=0; i<ncenters; i++)
	{
	for(j=0; j<ninputs; j++)
		pgenerating_centers[i][j] = i;
	}
  for(j=0; j<ninputs; j++)
	pnorm_variance[j] = noise_sdev*noise_sdev;

  /* generate all training vectors
  ------------------------------*/
  if ( verbose >= 1)
      printf("generating training data\n");
  all_training_data= allocate_matrix( nvectors, ninputs);
  for(n=0; n<nvectors; n++)
	{ 
	i = n%ncenters;
	for(j=0; j<ninputs; j++) {
		all_training_data[n][j] = pgenerating_centers[i][j] + 
					      random_gauss(0.0, noise_sdev);
	}
	}

  /* initialize gaussian mixture clustering 
  ------------------------------*/
  gmix_init(ninputs, ncenters, initial_nmixtures, pweights, pcenters, pvariance,
	    variance_scale,
	    stop_percentage, split_percentage, max_iteration, verbose);

  /* loop over data as often as necessary 
  ---------------------------------------*/
  stop_flag=0;
  while ( stop_flag <= 0)
      {
      for(n=0; n<nvectors; n++)
	gmix_batch_train( all_training_data[n] );
	stop_flag = gmix_batch_update( &report_iterations, &report_actual_ncenters,
				 &report_avg_logl, &report_nvectors);
	printf("ITER %d  NCENTERS %d  AVG_LOGL %g  VECTORS %d\n\n",
		report_iterations, report_actual_ncenters, report_avg_logl,
		report_nvectors);
      }

  /* report results 
  ------------------*/
  actual_ncenters = stop_flag;
  printf("DONE  ncenters = %d\n", actual_ncenters);
  if ( verbose >= 2)
	{
      for(i=0; i< actual_ncenters; i++)
	  for(j=0; j< ninputs; j++)
	    printf("%3d %3d %6.3f %6.3f\n", 
			i, j, pcenters[i][j], pvariance[i][j]);
	}

}
