/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <math.h>
#include <nnlib.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif

static int npatterns_list[NNARGS_MAX_LABELS] = {0};
static int seed = 0;
static float offset = .707;
static ARG argtab[] =
{
  { "npatterns_list", INT_LIST, npatterns_list,
      "Number of points to generate per class"},
  { "seed", INT,&seed,"Random number seed"},
  { "offset", FLOAT, (int *)&offset,"offset of centers from axes"},
};
#define TABSIZE (sizeof(argtab)/sizeof(ARG))

float pi = 3.1415926;

int
main(argc,argv)
     int argc;
     char **argv;
{
    float x,y;
    int i,npatterns, class;
    void make_cluster_point();
    
    argc = nnargs(argc,argv,argtab,TABSIZE);
    
    srandom(seed);
    
    if(npatterns_list[0] != 2)
      fatal_error("npatterns_list must be 2 integers long");
    npatterns = npatterns_list[1] + npatterns_list[2];

    /* print out the points alternating, while both classes last */
    for (i=0, class = 1; i<npatterns; i++) {
	if(npatterns_list[class] > 0){
	    npatterns_list[class] -= 1;
	    if(class == 1){
		make_cluster_point (&x, &y, offset);
	    } else {
		make_cluster_point (&x, &y, -offset);
	    }
	    printf("%d %.3g %.3g\n", class-1, x, y);
	    class = (class == 1)? 2: 1;	/* switch classes */
	} else {
	    class = (class == 1)? 2: 1;	/* switch classes again */
	    break;
	}
    }
    /* print out the remainder of the points */
    for(; i < npatterns; i++){
	if(class == 1)
	  make_cluster_point (&x, &y, offset);
	else
	  make_cluster_point (&x, &y, -offset);
	printf("%d %.3g %.3g\n",class - 1, x, y);
    }
  return 0;
}

void
make_cluster_point (x, y, offset)
float *x, *y, offset;
{
    float xtmp, ytmp;

    /* calculate point position */
    xtmp = random_gauss(0.,1.) + offset;
    ytmp = random_gauss(0.,1.) + offset;
    *y = ytmp;
    *x = xtmp;
}

