/***---------------------------------------------------------------------------
* NAME:     vmpdf
* PURPOSE:  print pdf of 
*           a vonmises circular distribution where
*           p(theta) = C*exp(kappa*cos(theta-alpha))
*                       theta   = circular input variable ranging from 0 to pi
*                                 (like x, 0 is 3 o'clock,
*                                  increases counter clockwise)
*                       kappa   = spread parameter 
*                                 (like 1/variance)
*                       alpha   = location parameter ranging from 0 to 2pi
*                                 (like mean)
*                       C       = 1/[2pi*I0(kappa)], where I0(k) is the
*                                 modified Bessel function 
*
*                       
* USAGE: 
*    vmpdf  -verbosity  n  -circle_max n -alpha n -k n -rad_step n
* RETURN VALUE:
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/datagen/vmpdf.c,v 1.3 2001/08/23 20:13:01 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/datagen/vmpdf.c,v $
* CREATED: 30-Jul-86 rpl                             MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/datagen/vmpdf.c,v 1.3 2001/08/23 20:13:01 craig Exp $";

#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <math.h>
#include "/u/kukolich/lnknet/src/include/nnlib.h"
#include "vonmises_lib.h"

static float circle_max=24.0;
static float rad_step=.05;
static float rad_mean=0.0;
static float kappa=1.0;
static int verbosity;

static ARG      argtab[]= 
{
  {"verbosity", INT,        &verbosity,  "control amount of printout"},
  {"circle_max", FLOAT, (int *)&circle_max, "maximum value of cicular input"},
  {"alpha", FLOAT, (int *)&rad_mean, "mean (0 to circle_max)"},
  {"k", FLOAT, (int *)&kappa, "spread (similar to 1/variance)"},
  {"rad_step", FLOAT, (int *)&rad_step, "stepsize to sample pdf"},
};
#define TABSIZE (sizeof(argtab)/sizeof(ARG))

main (argc, argv)
int argc;
char **argv;
{
    float x, radian;


    init_nnsubs(argc, argv, rcsid);
    argc = nnargs(argc, argv, argtab, TABSIZE);

    rad_mean = rad_mean*2.0*M_PI/circle_max;
    kappa = kappa*circle_max/(2.0*M_PI);
    for (x=0.0; x<=circle_max; x+=rad_step) {
	radian = x*2.0*M_PI/circle_max;
	printf("%6.4f %g\n", 
	       x, pdf_vonmises(rad_mean, kappa, radian));
        }
}

