/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <nnlib.h>
#include "LNKnet_subs.h"

#include "gnn/gnn_ui.h"
#include "gnn/nn_params.h"
gnn_main_window_objects	*Gnn_main_window;

#define MAXSTR 256
static char chartmp[MAXSTR];

/* general routines for use by LNKnet windows */

void
command_file_close (shell_file, filename, message)
     char *filename, *message;
     FILE *shell_file;
{
    nn_fclose (shell_file, filename, message);
    
    sprintf (chartmp, "chmod +x %s", filename);
    system (chartmp);
}

int
execute_command_file(filename)
     char *filename;
{
    int cur_com_pgrp;

    sprintf (chartmp, "csh -ef %s&", filename);
    cur_com_pgrp = nn_system (chartmp);
    return(cur_com_pgrp);
}

void
stop_command_file(cur_com_pgrp)
     int cur_com_pgrp;
{
    if(cur_com_pgrp != 0){
	killpg (cur_com_pgrp, SIGKILL);
	printf("\nKilled the most recently started run file\n");
	cur_com_pgrp = 0;
    }
}

int
chdir_and_execute_command(shell_fname)
     char *shell_fname;
{
    int pgrp;

    if(chdir(nn_params.exp_path))
      warning("couldn't change directory to %s", nn_params.exp_path);
    pgrp = execute_command_file(shell_fname);
    if(chdir(nn_params.working_directory))
      warning("couldn't change directory to %s", nn_params.working_directory);
    return pgrp;
}

FILE *
command_file_open (filename, message)
     char *filename, *message;
{
    FILE *shell_file;

    shell_file = fopen(filename, "w", message);
    if(shell_file == NULL){
	notice_prompt(Gnn_main_window->main_window, NULL,
		      NOTICE_MESSAGE_STRINGS,
		      "Cannot create file for shell script",
		      filename,
		      NULL,
		      NOTICE_BUTTON_YES,	"OK",
		      NULL);
	return NULL;
    }
    fprintf (shell_file, "#!/bin/csh -ef\n# %s\n", filename);
    
    return shell_file;
}

int
file_exists (filename)
     char *filename;
{
    FILE *file;
    int return_val;
    
    if ((file = fopen(filename, "r")) == NULL) {
    	return_val = FALSE;
    } else {
    	return_val = TRUE;
	nn_fclose(file, filename, "for file existence check");
    }
    return return_val;
}

char *
make_net_string (array)
     int array[];
{
    static char dummy[256];
    int i;
    
    *dummy = NULL;
    if(array == NULL) return dummy;
    if (array[0] > 0) {
	sprintf (dummy, "%d", array[1]);
    }
    if (array[0] > 1) {
	for(i=2; i<= array[0] && i <= NNARGS_MAX_NET; i++)
	  sprintf (dummy + strlen(dummy), ",%d", array[i]);
    }
    return dummy;
}

char *
make_float_string (array)
     float array[];
{
    static char dummy[256];
    int i;
    
    *dummy = NULL;
    if(array == NULL) return dummy;
    if (array[0] > 0) {
	sprintf (dummy, "%g", array[1]);
    }
    if (array[0] > 1) {
	for(i=2; i<= array[0] && i <= NNARGS_MAX_LABELS; i++)
	  sprintf (dummy + strlen(dummy), ",%g", array[i]);
    }
    return dummy;
}

char *
make_labels_string (array)
     char *array[];
{
    char *dummy;
    int i, array_len, total_length;
    
    if(array == NULL) array_len = 0;
    else array_len = atoi(array[0]);
    /* add up the length of the string to be created */
    total_length = 0;
    for(i = 1; i <= array_len; i++)
      total_length += strlen(array[i]);
    /* remember the commas and the NIL at the end */
    total_length += array_len + 1;

    /* allocate space for the string */
    dummy = nn_malloc(total_length, "for make_labels_string");
    dummy[0] = '\0';

    if(array_len > 0){
	for(i = 1; i <= array_len; i++){
	    strcat(dummy, array[i]);
	    strcat(dummy, ",");
	}
    }
    /* take off the comma at the end */
    total_length = strlen(dummy);
    if(total_length > 1) dummy[total_length-1] = '\0';

    return dummy;
}

int
overwrite_ok (filename)
     char *filename;
{
    int result, return_value;
    
    return_value = TRUE;
    if (file_exists(filename)) {
	result = notice_prompt(Gnn_main_window->main_window, NULL,
			       NOTICE_MESSAGE_STRINGS,
			       "Shell file exists: OK to overwrite?",
			       NULL,
			       NOTICE_BUTTON_YES,	"Overwrite",
			       NOTICE_BUTTON_NO,	"Cancel",
			       NULL);
	if(result != NOTICE_YES) return_value = FALSE;
    }
    return return_value;
}

/********************************************************/
/* routines for displaying text and numbers on a window */
/********************************************************/
void
show_int_value(item, value)
     unsigned int item;
     int value;
{
    xv_set(item, PANEL_VALUE, value, NULL);
}

void
show_int_text (item, value)
     unsigned int item;
     int value;
{
    char *str[10];
    
    sprintf (str, "%d", value);
    xv_set (item, PANEL_VALUE, str, NULL);
}

void
show_int_label (item, value)
     unsigned int item;
     int value;
{
    char *str[10];
    
    sprintf (str, "%d", value);
    xv_set (item, PANEL_LABEL_STRING, str, NULL);
}

void 
show_float_text (item, value)
     unsigned int item;
     float value;
{
    char *str[10];
    
    sprintf (str, "%5.2g", value);
    xv_set (item, PANEL_VALUE, str, NULL);
}

void 
show_float_label (item, value)
     unsigned int item;
     float value;
{
    char *str[10];
    
    sprintf (str, "%5.2g", value);
    xv_set (item, PANEL_LABEL_STRING, str, NULL);
}

void
show_string_text(item, string)
     unsigned int item;
     char *string;
{
    xv_set(item, PANEL_VALUE, string, NULL);
}

void
show_string_label(item, string)
     unsigned int item;
     char *string;
{
    xv_set(item, PANEL_LABEL_STRING, string, NULL);
}

void
show_image_label(item, glyph)
     unsigned int item;
     unsigned int glyph;
{
    xv_set(item, PANEL_LABEL_IMAGE, glyph, NULL);
}

/* should be get_atof_value */
float
set_atof_value(str, name, item, value)
     char *str, *name;
     unsigned int item;
     float value;
{
    float new_val;
    
    if(sscanf (str, "%f", &new_val) == 1){
	return new_val;
    } else {
	warning("set_atof_value -- unable to set %s to %s",
		name, str);
	show_float_text(item, value);
	return value;
    }
}

/* should be get_atoi_value */
int
set_atoi_value(str, name, item, value)
     char *str, *name;
     unsigned int item;
     int value;
{
    int new_val;
    
    if(sscanf (str, "%d", &new_val) == 1){
	return new_val;
    } else {
	warning("set_atoi_value -- unable to set %s to %s",
		name, str);
	show_int_text(item, value);
	return value;
    }
}

/* should be get_atof_range_value */
float
set_atof_range_value(str, name, item, value, low, high)
     char *str, *name;
     unsigned int item;
     float value, low, high;
{
    float new_val;
    
    if(sscanf (str, "%f", &new_val) == 1){
	if(new_val >= low && new_val <= high)
	  return new_val;
	else {
	    warning("set_atof_range_value -- new %s (%f) out of range (%f,%f)",
		    name, new_val, low, high);
	    show_float_text(item, value);
	    return value;
	}
    } else {
	warning("set_atof_range_value -- unable to set %s to %s",
		name, str);
	show_float_text(item, value);
	return value;
    }
}

/* should be get_atof_low_value */
float
set_atof_low_value(str, name, item, value, low)
     char *str, *name;
     unsigned int item;
     float value, low;
{
    float new_val;
    
    if(sscanf (str, "%f", &new_val) == 1){
	if(new_val >= low)
	  return new_val;
	else {
	    warning("set_atof_low_value -- new %s (%f) < (%f)",
		    name, new_val, low);
	    show_float_text(item, value);
	    return value;
	}
    } else {
	warning("set_atof_low_value -- unable to set %s to %s",
		name, str);
	show_float_text(item, value);
	return value;
    }
}

/* should be get_atoi_low_value */
int
set_atoi_low_value(str, name, item, value, low)
     char *str, *name;
     unsigned int item;
     int value, low;
{
    int new_val;
    
    if(sscanf (str, "%d", &new_val) == 1){
	if(new_val >= low)
	  return new_val;
	else {
	    warning("set_atoi_low_value -- new %s (%d) < (%d)",
		    name, new_val, low);
	    show_int_text(item, value);
	    return value;
	}
    } else {
	warning("set_atoi_low_value -- unable to set %s to %s",
		name, str);
	show_int_text(item, value);
	return value;
    }
}

/* should be get_atoi_range_value */
int
set_atoi_range_value(str, name, item, value, low, high)
     char *str, *name;
     unsigned int item;
     int value, low, high;
{
    int new_val;
    
    if(sscanf (str, "%d", &new_val) == 1){
	if(new_val >= low && new_val <= high)
	  return new_val;
	else {
	    warning("set_atoi_range_value -- new %s (%d) out of range (%d,%d)",
		    name, new_val, low, high);
	    show_int_text(item, value);
	    return value;
	}
    } else {
	warning("set_atoi_range_value -- unable to set %s to %s",
		name, str);
	show_int_text(item, value);
	return value;
    }
}

/* should be get_string_not_empty */
char *
set_string_not_empty(str, name, item, value)
     char *str, *name, *value;
     unsigned int item;
{
    if(strcmp(str, "") == 0){
	warning("set_string_value -- cannot set %s to an empty string",
		name);
	show_string_text(item, value);
	return value;
    } else {
	return str;
    }
}

/* these routines are used when setting string arguments for
   saving and restoring experiment screens */

/* if string1 is NULL or and empty string, return NULL
   otherwise return string1 */
char *
null_if_empty_string(string1)
     char *string1;
{
    if(string1 == NULL) return NULL;
    if(strcmp(string1, "") == 0) /* string is empty */ return NULL;
    return string1; /* string is not empty */
}

/* is string2 is null, make string1 the empty string,
   otherwise copy string2 into string 1 */
char *
strcpy_empty_if_null(string1, string2)
     char *string1, *string2;
{
    if(string2 == NULL) strcpy(string1, "");
    else strcpy(string1, string2);
    return string1;
}

void
close_popup(item)
     unsigned int item;
{
    xv_set(item, FRAME_CMD_PUSHPIN_IN, FALSE, NULL);
    xv_set(item, XV_SHOW, FALSE, NULL);
}

void
open_popup(item)
     unsigned int item;
{
    xv_set(item, FRAME_CMD_PUSHPIN_IN, TRUE, NULL);
    xv_set(item, XV_SHOW, TRUE, NULL);
}

void
gray_out_item(item, direction)
     unsigned int item;
     int direction;
{
    xv_set(item, PANEL_INACTIVE, direction, NULL);
}

void
show_item(item, direction)
     unsigned int item;
     int direction;
{
    xv_set(item, XV_SHOW, direction, NULL);
}

void
setup_character_watch(item)
     unsigned int item;
{
    xv_set(item,
	   PANEL_NOTIFY_LEVEL, PANEL_SPECIFIED,
	   PANEL_NOTIFY_STRING, ALLCHARS,
	   NULL);
}

int
check_stringNwidget(stored, item, warning)
     char *stored;
     unsigned int item;
     unsigned int warning;
{
    if (strcmp(stored, xv_get(item, PANEL_VALUE))) {
	show_item(warning, TRUE);
	return 1;
    } else
      return 0;
}

int
check_intNwidget(stored, item, warning)
     int stored;
     unsigned int item;
     unsigned int warning;
{
    if (stored != xv_get(item, PANEL_VALUE)) {
	show_item(warning, TRUE);
	return 1;
    } else
      return 0;
}

void
update_stringNwidget(string, item, warning)
     char *string;
     unsigned int item, warning;
{
    char *cvalue;
    cvalue = (char *)xv_get(item, PANEL_VALUE);
    strcpy(string, cvalue);
    show_item(warning, FALSE);
}

int
set_string2intNcheck(item, warning, ivalue, name, event, saved)
     unsigned int item, warning;
     Event *event;
     int *ivalue;
     char *name, *saved;
{
    char *cvalue;

    cvalue = (char *)xv_get(item, PANEL_VALUE);

    if (EVENT_IS_RTN(event)){
	*ivalue = set_atoi_value(cvalue, name, item, *ivalue);
	update_stringNwidget(saved, item, warning);
	return 1;
    } else {
	show_item(warning, TRUE);
	return 0;
    }
}

int
set_string2floatNcheck(item, warning, ivalue, name, event, saved)
     unsigned int item, warning;
     Event *event;
     float *ivalue;
     char *name, *saved;
{
    char *cvalue;

    cvalue = (char *)xv_get(item, PANEL_VALUE);

    if (EVENT_IS_RTN(event)){
	*ivalue = set_atof_value(cvalue, name, item, *ivalue);
	update_stringNwidget(saved, item, warning);
	return 1;
    } else {
	show_item(warning, TRUE);
	return 0;
    }
}
