/**-------------------------------------------------------------------------
* NAME:		gartmap.c
* PURPOSE:	parameter popup for artmap
*
* USAGE:	gnn	(from command line - produces control panel window)
*
* SEE ALSO:	artmap.c artmap_pack.c artmap_subs.c gartmap_subs.c c_subs.c
*
* DESCRIPTION: 
*	gartmap_display_params
*	gartmap_init
*	gartmap_get_plot_label
*
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/gclass/artmap/gartmap.c,v 10.4 2001/08/23 20:13:06 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/gclass/artmap/gartmap.c,v $
* created 23-Jan-91 lck
-----------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gclass/artmap/gartmap.c,v 10.4 2001/08/23 20:13:06 craig Exp $";




#define TRUE 1
#define FALSE 0

#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>

#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <sys/ioctl.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/icon_load.h>
#include <xview/panel.h>
#include <xview/scrollbar.h>
#include <xview/svrimage.h>
#include <xview/termsw.h>
#include <xview/text.h>
#include <xview/tty.h>
#include <xview/xv_xrect.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <gcm.h>
#include <gdd.h>
#include <nnlib.h>
#include "gartmap_ui.h"
#include <artmap_args.h>	/* statics for artmap args */
#include "../LNKnet_subs.h"
#include "../gnn/nn_params.h"

/* Needed window pointers */
extern gartmap_param_pop_objects	*Gartmap_param_pop;

/* statics for file name parts */
#define MAXSTR 256

/* routines found in this file */
void gartmap_display_params();
void gartmap_init ();

void gartmap_display_params()
{
    int val;
    
    val = 2*input_sum_to_one;
    if(!no_complement_code) val += 1;
    show_int_value (Gartmap_param_pop->set_norm, val);
    show_float_text(Gartmap_param_pop->beta, beta);
    show_float_text(Gartmap_param_pop->alpha, alpha);
    show_float_text(Gartmap_param_pop->arta_train, ro_a_train);
    show_float_text(Gartmap_param_pop->arta_test, ro_a_test); 
    show_float_text(Gartmap_param_pop->artb_train_test, ro_b);
    show_int_value(Gartmap_param_pop->epochs, nepochs);
    show_int_value(Gartmap_param_pop->train_cycle, train_cycle);
    if(prune_threshold >= 0){
	show_int_value(Gartmap_param_pop->prune_thresh, prune_threshold);
	show_int_value(Gartmap_param_pop->prune, 1);
    } else {
	show_int_value(Gartmap_param_pop->prune, 0);
	gray_out_item(Gartmap_param_pop->prune_thresh, TRUE);
    }
}

void gartmap_init ()
{
    no_complement_code = 0;
    input_sum_to_one = 0;
    beta = 1.;
    alpha = .001;
    ro_a_train = .95;
    ro_a_test = .90;
    ro_b = .999;
    nepochs = 5;
    train_cycle = 0;
    prune_threshold = -1;
    gartmap_display_params();
}

char * gartmap_get_plot_label(string)
     char *string;
{
    return string;
}

/*
 * Notify callback function for `set_norm'.
 */
void
gartmap_set_norm_handler(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;/*ARGSUSED*/
{
	if(value & 0x1) no_complement_code = 0;
	else no_complement_code = 1;
	input_sum_to_one = value & 0x2;
}

/*
 * Notify callback function for `alpha'.
 */
Panel_setting gartmap_alpha_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);

    alpha = set_atof_range_value(value, "alpha", Gartmap_param_pop->alpha,
				 alpha, 0., 1.);
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `beta'.
 */
Panel_setting gartmap_beta_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);

    beta = set_atof_range_value(value, "beta", Gartmap_param_pop->beta,
				beta, 0., 1.);
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `arta_train'.
 */
Panel_setting gartmap_arta_train_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);

    ro_a_train = set_atof_range_value(value, "arta_train",
				      Gartmap_param_pop->arta_train,
				      ro_a_train, 0., 1.);
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `arta_test'.
 */
Panel_setting gartmap_arta_test_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);

    ro_a_test = set_atof_range_value(value, "arta_test",
				     Gartmap_param_pop->arta_test,
				     ro_a_test, 0., 1.);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `artb_train_test'.
 */
Panel_setting gartmap_artb_train_test_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);

    ro_b = set_atof_range_value(value, "artb_train_test",
				Gartmap_param_pop->artb_train_test,
				ro_b, 0., 1.);
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `epochs'.
 */
Panel_setting
gartmap_epochs_handler(item, event)
	Panel_item	item;
	Event		*event;
{
	int	value = (int) xv_get(item, PANEL_VALUE);
	
	nepochs = value;
	return panel_text_notify(item, event);
}

/*
 * Notify callback function for `train_cycle'.
 */
void
gartmap_train_cycle_handler(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;/*ARGSUSED*/
{
    train_cycle = value;
}


/*
 * Notify callback function for `prune'.
 */
void
gartmap_prune_handler(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;/*ARGSUSED*/  
{
    if(value){
	gray_out_item(Gartmap_param_pop->prune_thresh, FALSE);
	prune_threshold = xv_get(Gartmap_param_pop->prune_thresh,
				 PANEL_VALUE, NULL);
    } else {
	gray_out_item(Gartmap_param_pop->prune_thresh, TRUE);
	prune_threshold = -1;
    }
}


/*
 * Notify callback function for `prune_thresh'.
 */
Panel_setting
gartmap_prune_thresh_handler(item, event)
	Panel_item	item;
	Event		*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);

    prune_threshold = value;
    return panel_text_notify(item, event);
}

char *
gartmap_get_args ()
{
    return (nnargs_get_part_flags(artmap_tab, ARTMAP_TABSIZE));
}

char *
gartmap_get_important_args ()
{
    return (nnargs_get_flags_one_line(artmap_tab, ARTMAP_TABSIZE));
}

void
write_artmap_defaults(default_file)
     FILE *default_file;
{
    if(default_file != NULL){
	fprintf (default_file, "artmap %s",
		 nnargs_get_flags(artmap_tab, ARTMAP_TABSIZE));
    }
}

void
read_artmap_defaults(argc, argv)
     int argc;
     char **argv;
{
    if(argc > 0)
      argc = nnargs(argc, argv, artmap_tab, ARTMAP_TABSIZE);
    else
      nepochs = 5;

    /* algorithm specific flags are now set.  generic flags are taken
       from the argument list for the main screen */
}
