/***------------------------------------------------------------------------
 * NAME:        gbintree.c
 * PURPOSE:     CART graphics program
 *
 * BUGS AND LIMITATIONS:
 *
 -----------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header:";




#define TRUE 1
#define FALSE 0

#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>

#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <sys/ioctl.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/icon_load.h>
#include <xview/panel.h>
#include <xview/scrollbar.h>
#include <xview/svrimage.h>
#include <xview/termsw.h>
#include <xview/text.h>
#include <xview/tty.h>
#include <xview/xv_xrect.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <gcm.h>
#include <gdd.h>
#include "gbintree_ui.h"
#include <nnlib.h>
#include <bintree_args.h>	/* statics for bintree args */
#include "../LNKnet_subs.h"
#include "../gnn/nn_params.h"

/* Needed window pointers */
extern gbintree_param_pop_objects	*Gbintree_param_pop;

/* statics for file name parts */
#define MAXSTR 256

/* routines found in this file */
void gbintree_display_params();
void gbintree_init ();
void gbintree_set_n_epochs ();

void gbintree_display_params()
{
    show_int_value(Gbintree_param_pop->linear, linear);
    show_int_value(Gbintree_param_pop->train_full, stop_train_early);
    gbintree_set_train_full(stop_train_early);
    show_int_text(Gbintree_param_pop->leaf_min_npatterns, leaf_min_npatterns);
    show_int_value(Gbintree_param_pop->test_full, prune_tree);
    gbintree_set_test_full(prune_tree);
    show_int_text(Gbintree_param_pop->max_test_nodes, max_nodes);
}

void gbintree_init ()
{
    max_nodes = 100;
    gbintree_display_params();
}

char *
gbintree_get_plot_label(string)
     char *string;
{
    sprintf(string, "%s %s", string, linear ? "Linear" : "");
    if(prune_tree)
      sprintf(string, "%s  Test nodes:%d", string, max_nodes);
    else
      sprintf(string, "%s Test nodes:all", string);
    return string;
}

/*
 * Notify callback function for `train_full'.
 */
void
gbintree_train_full_handler(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;/*ARGSUSED*//*ARGSUSED*/
{
    gbintree_set_train_full(value);
}

gbintree_set_train_full(value)
     int value;
{
    if(value){
	stop_train_early = 1;
	gray_out_item(Gbintree_param_pop->leaf_min_npatterns, FALSE);
	gray_out_item(Gbintree_param_pop->stop_message, FALSE);
    } else {
	stop_train_early = 0;
	gray_out_item(Gbintree_param_pop->leaf_min_npatterns, TRUE);
	gray_out_item(Gbintree_param_pop->stop_message, TRUE);
    }
}

/*
 * Notify callback function for `leaf_min_npatterns'.
 */
Panel_setting
gbintree_leaf_min_npatterns_handler(item, event)
	Panel_item	item;
	Event		*event;
{
	char *	value = (char *) xv_get(item, PANEL_VALUE);
	

	leaf_min_npatterns = set_atoi_value(value, "leaf_min_npatterns",
					    item, leaf_min_npatterns);

	return panel_text_notify(item, event);
}

/*
 * Notify callback function for `test_full'.
 */
void
gbintree_test_full_handler(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;/*ARGSUSED*/
{
    gbintree_set_test_full(value);
}

gbintree_set_test_full(value)
     int value;
{
    prune_tree = value;
    if(value){
	gray_out_item(Gbintree_param_pop->max_test_nodes, FALSE);
    } else {
	gray_out_item(Gbintree_param_pop->max_test_nodes, TRUE);
    }
    set_plot_labels();

}
/*
 * Notify callback function for `max_test_nodes'.
 */
Panel_setting
gbintree_max_test_nodes_handler(item, event)
	Panel_item	item;
	Event		*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
	
    max_nodes = set_atoi_value(value, "max_nodes", item, max_nodes);
    set_plot_labels();

    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `linear'.
 */
void
gbintree_linear_handler(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;/*ARGSUSED*/
{
    linear = value;
    set_plot_labels();
}

void
gbintree_store_plot_command(shell_file)
     FILE *shell_file;
{
    if(prune_tree)
      fprintf(shell_file, " -prune_tree");
    fprintf(shell_file, " -max_nodes %d", max_nodes);
}


char *
gbintree_get_args ()
{
    return (nnargs_get_part_flags(bintree_tab, BINTREE_TABSIZE));
}

char *
gbintree_get_important_args ()
{
    return (nnargs_get_flags_one_line(bintree_tab, BINTREE_TABSIZE));
}

void
write_bintree_defaults(default_file)
     FILE *default_file;
{
    if(default_file != NULL){
	fprintf (default_file, "bintree %s",
		 nnargs_get_flags(bintree_tab, BINTREE_TABSIZE));
    }
}

void
read_bintree_defaults(argc, argv)
     int argc;
     char **argv;
{
    stop_train_early = 0;
    linear = 0;
    prune_tree = 0;
    if(argc > 0){
	argc = nnargs(argc, argv, bintree_tab, BINTREE_TABSIZE);
    } else {
	max_nodes = 100;
    }
}
