/***--------------------------------------------------------------------------
 * NAME:        gcknn.c
 * PURPOSE:     Condensed K Nearest Neighbor main program driver
 *			(DEVGUIDE version)
 *
 *		Executes cknn after setting command line parameters
 *
 * USAGE:	gnn, then select CKNN from classifier menu
 *
 * SEE ALSO:	cknn.c cknn_pack.c cknn_subs.c gcknn_subs.c c_subs.c
 *
 * DESCRIPTION: 
 *	gcknn_display_params
 *	gcknn_init
 *	gcknn_get_plot_label
 *	gcknn_set_common_flags
 *	gcknn_set_description_values
 *	gcknn_set_n_epochs
 *	gcknn_store_run_command
 *
 * BUGS AND LIMITATIONS:
 *
 *
 ------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header:";




#define TRUE 1
#define FALSE 0

#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>

#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <sys/ioctl.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/icon_load.h>
#include <xview/panel.h>
#include <xview/scrollbar.h>
#include <xview/svrimage.h>
#include <xview/termsw.h>
#include <xview/text.h>
#include <xview/tty.h>
#include <xview/xv_xrect.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <gcm.h>
#include <gdd.h>
#include "gcknn_ui.h"
#include <nnlib.h>
#include <cknn_args.h>	/* statics for cknn args */
#include "../LNKnet_subs.h"
#include "../gnn/nn_params.h"

/* Needed window pointers */
extern gcknn_param_pop_objects	*Gcknn_param_pop;

/* statics for file name parts */
#define MAXSTR 256

/* routines found in this file */
void gcknn_display_params();
void gcknn_init ();
void gcknn_set_n_epochs ();
void gcknn_store_bound_plot_command();
void gcknn_store_run_command ();
void gcknn_set_common_flags();
void gcknn_set_description_values();

void
gcknn_display_params()
{
    show_int_value (Gcknn_param_pop->n_epochs, nepochs);
}

void
gcknn_init ()
{
    k = 1;	/* can only produce call for cnn classifier from here */
    gcknn_display_params();
}

char *
gcknn_get_plot_label(string)
     char *string;
{
    sprintf(string, "%s Epochs:%d", string, nepochs);
    
    return string;
}

/*
 * Notify callback function for `n_epochs'.
 */
Panel_setting
gcknn_nepoch_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);
    
    nepochs = value;
    set_plot_labels();
    return panel_text_notify(item, event);
}

int
gcknn_get_epochs()
{
    return nepochs;
}

void
gcknn_set_epochs(e)
     int e;
{
    /* popup will not reflect the set number of epochs */
    nepochs = e;
}

char *
gcknn_get_args ()
{
    return (nnargs_get_part_flags(cknn_tab, CKNN_TABSIZE));
}

char *
gcknn_get_important_args ()
{
    return (nnargs_get_flags_one_line(cknn_tab, CKNN_TABSIZE));
}


void
write_cknn_defaults(default_file)
     FILE *default_file;
{
    if(default_file != NULL)
      fprintf (default_file, "cknn %s", nnargs_get_flags(cknn_tab, CKNN_TABSIZE));
    
}

void
read_cknn_defaults(argc, argv)
     int argc;
     char **argv;
{
    if(argc > 0)
      argc = nnargs(argc, argv, cknn_tab, CKNN_TABSIZE);
    else
      nepochs = 5;
    
    /* algorithm specific flags are now set.  generic flags are taken
       from the argument list for the main screen */
}
