/***--------------------------------------------------------------------------
 * NAME:        ggauss.c
 * PURPOSE:     Gaussian Classifier GUIDE handler functions
 *		Executes gauss functions after setting command line parameters
 *
 * USAGE:	gnn	(from command line - produces control panel window)
 *
 * SEE ALSO:	gnn_main.c gnn.c gnn_subs.c gauss.c gauss_pack.c ggauss_subs.c
 *
 * DESCRIPTION:
 *
 *	ggauss_display_params
 *	ggauss_get_plot_label
 *	ggauss_init
 *		
 * BUGS AND LIMITATIONS:
 *
 *
 ------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gclass/gauss/ggauss.c,v 10.4 2001/08/23 20:13:18 craig Exp $";



#define TRUE 1
#define FALSE 0
#define MAXSTR 256
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <unistd.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <gdd.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <nnlib.h>
#include "ggauss_ui.h"
#include <gauss_args.h>			/* nnargs definitions */
#include "../gnn/nn_params.h"
#include "../LNKnet_subs.h"

extern ggauss_popup1_objects	*Ggauss_popup1;

void set_plot_descriptions();
void ggauss_set_common_flags();

void ggauss_display_params()
{
    show_int_value(Ggauss_popup1->full, covariance_type);
    if(variance_type)
      show_int_value(Ggauss_popup1->grand, 0);
    else
      show_int_value(Ggauss_popup1->grand, 1);
    show_float_text (Ggauss_popup1->minimum_variance, minimum_variance);
}

char * ggauss_get_plot_label (string)
     char *string;
{
    if (covariance_type==0) strcat (string," Diagonal"); 
    else 		    strcat(string," Full");
    if (variance_type==0) strcat (string," Grand"); 
    else 		  strcat(string," Per Class");
    return string;
}

void ggauss_init ()
{
    ggauss_display_params();
}

/*
 * Notify callback function for `covariance_type'.
 */
void
ggauss_select_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;/*ARGSUSED*/
{
    void set_plot_labels();
    ggauss_popup1_objects *ip =
      (ggauss_popup1_objects *)xv_get(item, XV_KEY_DATA, INSTANCE);

    if(item == ip->full){
	covariance_type = value;
	set_plot_labels();
    } else if (item == ip->grand){
	if(value)
	  variance_type = 0;
	else
	  variance_type = 1;
	set_plot_labels();
    } else {
	warning("ggauss_select_handler: unhandled interrupt");
    }
}

/*
 * Notify callback function for `minimum_variance'.
 */
Panel_setting ggauss_minimum_variance_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    minimum_variance =
      set_atof_low_value(value, "minimum_variance",
			 Ggauss_popup1->minimum_variance, minimum_variance, 0.);
    return panel_text_notify(item, event);
}

char *
ggauss_get_args ()
{
    return (nnargs_get_part_flags(gauss_tab, GAUSS_TABSIZE));
}

char *
ggauss_get_important_args ()
{
    return (nnargs_get_flags_one_line(gauss_tab, GAUSS_TABSIZE));
}


static int variance_arg = 0;
static int covariance_arg = 0;
static ARG argtab2[]=
{
    {"variance_arg", INT, &variance_arg, "batch type"},
    {"covariance_arg", INT, &covariance_arg, "batch type"},
};
#define TABSIZE2 (sizeof(argtab2)/sizeof(ARG))

void
write_gauss_defaults(default_file)
     FILE *default_file;
{
    covariance_arg = covariance_type;
    variance_arg = variance_type;
    if(default_file != NULL){
	fprintf (default_file, "gauss %s",
		 nnargs_get_part_flags(gauss_tab, GAUSS_TABSIZE));
	fprintf(default_file,"%s",nnargs_get_flags(argtab2, TABSIZE2));
    }
}

void
read_gauss_defaults(argc, argv)
     int argc;
     char **argv;
{
    if(argc > 0){
	argc = nnargs_part(argc, argv, gauss_tab, GAUSS_TABSIZE);
	argc = nnargs(argc, argv, argtab2, TABSIZE2);
    }
    variance_type = variance_arg;
    covariance_type = covariance_arg;
    
    /* algorithm specific flags are now set.  generic flags are taken
       from the argument list for the main screen */
}
