/***--------------------------------------------------------------------------
* NAME:        ggmix.c
* PURPOSE:     Gaussian Mixture main program driver (DEVGUIDE version)
*		Executes gmix after setting command line parameters
*
* USAGE:	LNKnet	(from command line - produces control panel window)
*		then select GMIX from Algorithm Menu
*
* SEE ALSO:	gmix.c gmix_pack.c gmix_subs.c ggmix_subs.c c_subs.c
*
* DESCRIPTION: 
*	ggmix_display_params
*	ggmix_init
*	ggmix_get_plot_label
*	ggmix_set_common_flags
*	ggmix_set_description_values
*	ggmix_store_run_command
*
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/gclass/gmix/ggmix.c,v 10.4 2001/08/23 20:13:20 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/gclass/gmix/ggmix.c,v $
* created 23-Jan-91 lck
-------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gclass/gmix/ggmix.c,v 10.4 2001/08/23 20:13:20 craig Exp $";




#define TRUE 1
#define FALSE 0

#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>

#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <sys/ioctl.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/icon_load.h>
#include <xview/panel.h>
#include <xview/scrollbar.h>
#include <xview/svrimage.h>
#include <xview/termsw.h>
#include <xview/text.h>
#include <xview/tty.h>
#include <xview/xv_xrect.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <gcm.h>
#include <gdd.h>
#include <nnlib.h>
#include "ggmix_ui.h"
#include <gmix_args.h>	/* statics for gmix args */
#include "../gnn/nn_params.h"
#include "../gnn/galgs.h"
#include "../LNKnet_subs.h"

#define MAXSTR 256

			/* static GLOBALS internal to this file */
static char *clustering_algorithm_name = "kmeans";
static char param_ftmp[MAXSTR];

static int clustering_active_flag;
static int clustering_algorithm = KMEANS;

/* Needed window pointers */
extern ggmix_param_pop_objects	*Ggmix_param_pop;

/* routines found in this file */
void ggmix_display_params();
void ggmix_init ();
void ggmix_store_run_command ();
void ggmix_set_common_flags();
void ggmix_set_description_values();
void ggmix_set_cluster_algorithm();
void ggmix_show_clustering_algorithm();
void ggmix_set_cluster_param_fname();

void
ggmix_display_params()
{
    show_int_value (Ggmix_param_pop->clustering_active,
		    clustering_active_flag);
    ggmix_show_clustering_algorithm();
    show_string_label(Ggmix_param_pop->cluster_param_fname,
		      cluster_param_fname);
    ggmix_set_cluster_algorithm(clustering_algorithm);
    ggmix_set_cluster_param_fname();
    show_int_value (Ggmix_param_pop->cov_type, full_flag);
    show_int_value (Ggmix_param_pop->grand_type, grand_flag);
    show_int_value (Ggmix_param_pop->mix_type, tied_flag);
    show_int_value (Ggmix_param_pop->epochs, nepochs);
    show_float_text (Ggmix_param_pop->min_var, minimum_variance);
    show_float_text (Ggmix_param_pop->max_ratio, max_ratio);
    show_float_text (Ggmix_param_pop->var_spread, var_spread);
    show_float_text (Ggmix_param_pop->stop_percentage,
		      gmix_stop_percentage);
}

void ggmix_init ()
{
    ggmix_display_params();
}

char *
ggmix_get_plot_label(string)
     char *string;
{
    if(tied_flag)
      sprintf(string, "%s Tied %s", string,
	      (grand_flag) ? "Grand" : "per Component");
    else
      sprintf(string, "%s Untied %s", string,
	      (grand_flag > 1 ) ? "Grand" :
	      (grand_flag == 1) ? "Grand per Class" : "per Component");
    sprintf(string, "%s %s", string, full_flag ? "Full" : "Diagonal");
    switch(clustering_algorithm){
      case KMEANS:
	string = gkmeans_get_plot_label(string);
	break;
      case EM_CLUS:
	string = gem_clus_get_plot_label(string);
	break;
      case LEAD_CLUS:
	string = glead_clus_get_plot_label(string);
	break;
      case RAN_CLUS:
	string = gran_clus_get_plot_label(string);
	break;
    }
    return string;
}
/*
 * Notify callback function for gmix parameter window
 */
void
ggmix_callback(item, event)
     Panel_item	item;
     Event	*event;/*ARGSUSED*/
{
    ggmix_param_pop_objects *ip =
      (ggmix_param_pop_objects *)xv_get(item, XV_KEY_DATA, INSTANCE);
    int ivalue;
    char *svalue;

    if(item == ip->clustering_active){
	clustering_active_flag = (int)xv_get(item, PANEL_VALUE);
    } else if (item == ip->clustering_algorithm){
	int temp;
	ivalue = (int)xv_get(item, PANEL_VALUE);
	switch (ivalue){
	  case 0:
	    ggmix_set_cluster_algorithm(KMEANS);
	    break;
	  case 1:
	    ggmix_set_cluster_algorithm(EM_CLUS);
	    break;
	  case 2:
	    ggmix_set_cluster_algorithm(LEAD_CLUS);
	    break;
	  case 3:
	    ggmix_set_cluster_algorithm(RAN_CLUS);
	    break;
	}
	temp = clustering_active_flag;
	clustering_active_flag = 1;
	ggmix_set_cluster_param_fname();
	clustering_active_flag = temp;
    } else if(item == ip->clustering_parameters){
	void gkmeans_show_menu(), gem_clus_show_menu();
	void glead_clus_show_menu(), gran_clus_show_menu();

	switch(clustering_algorithm){
	  case KMEANS:
	    gkmeans_show_menu(); break;
	  case EM_CLUS:
	    gem_clus_show_menu(); break;
	  case LEAD_CLUS:
	    glead_clus_show_menu(); break;
	  case RAN_CLUS:
	    gran_clus_show_menu(); break;
	}
    } else if (item == ip->mix_type){
	tied_flag = (int)xv_get(item, PANEL_VALUE);
	set_plot_labels();
    } else if (item == ip->grand_type){
	grand_flag = (int)xv_get(item, PANEL_VALUE);
	set_plot_labels();
    } else if (item == ip->cov_type){
	full_flag = (int)xv_get(item, PANEL_VALUE);
	set_plot_labels();
    } else if (item == ip->epochs){
	nepochs = (int) xv_get(item, PANEL_VALUE);
    } else if (item == ip->stop_percentage){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	gmix_stop_percentage =
	  set_atof_low_value(svalue, "stop_percentage",
			     Ggmix_param_pop->stop_percentage,
			     gmix_stop_percentage, 0.);
    } else if (item == ip->min_var){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	minimum_variance =
	  set_atof_low_value(svalue, "minimum_variance",
			     Ggmix_param_pop->min_var, minimum_variance, 0.);
    } else if (item == ip->max_ratio){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	max_ratio =
	  set_atof_low_value(svalue, "max_ratio",
			     Ggmix_param_pop->max_ratio, max_ratio, 1.);
    } else if (item == ip->var_spread){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	var_spread = set_atof_low_value(svalue, "var_spread",
					Ggmix_param_pop->var_spread,
					var_spread, 0.);
    } else {
	warning("ggmix_callback: unhandled interrupt");
    }
}

/*
 * Notify callback function for selection widgets
 */
void
ggmix_select_handler(item, value, event)
	Panel_item	item;
	int		value;/*ARGSUSED*/
	Event		*event;
{
    ggmix_callback(item, event);
}

/*
 * Notify callback function for button widgets
 */
void
ggmix_btn_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    ggmix_callback(item, event);
}

/*
 * Notify callback function for text widgets
 */
Panel_setting
ggmix_text_handler(item, event)
	Panel_item	item;
	Event		*event;
{
    ggmix_callback(item, event);
    return panel_text_notify(item, event);
}

void
ggmix_show_clustering_algorithm()
{
    switch(clustering_algorithm){
      case KMEANS:
	show_int_value(Ggmix_param_pop->clustering_algorithm, 0);
	break;
      case EM_CLUS:
	show_int_value(Ggmix_param_pop->clustering_algorithm, 1);
	break;
      case LEAD_CLUS:
	show_int_value(Ggmix_param_pop->clustering_algorithm, 2);
	break;
      case RAN_CLUS:
	show_int_value(Ggmix_param_pop->clustering_algorithm, 3);
	break;
    }
}

void
ggmix_set_cluster_algorithm(value)
     int value;
{
    clustering_algorithm = value;
    switch(value){
      case KMEANS:
	clustering_algorithm_name = "kmeans";
	break;
      case EM_CLUS:
	clustering_algorithm_name = "em_clus";
	break;
      case LEAD_CLUS:
	clustering_algorithm_name = "lead_clus";
	break;
      case RAN_CLUS:
	clustering_algorithm_name = "ran_clus";
	break;
      default:
	break;
    }
    /* cluster_param_fname still needs to be set */
    set_plot_labels();
}

void
ggmix_set_cluster_param_fname()
{
    if(clustering_active_flag){
	strcpy(param_ftmp, nn_params.exp_prefix);
	strcat(param_ftmp, clustering_algorithm_name);
	cluster_param_fname = strcat(param_ftmp,".param");
	show_string_label(Ggmix_param_pop->cluster_param_fname,
			  cluster_param_fname);
    }
}

int ggmix_get_epochs()
{
    return nepochs;
}

void ggmix_set_epochs(e)
int e;
{
    /* popup will not reflect the set number of epochs */
    nepochs = e;
}

char *
ggmix_get_args ()
{
    return (nnargs_get_part_flags(gmix_tab, GMIX_TABSIZE));
}

char *
ggmix_get_important_args ()
{
    return (nnargs_get_flags_one_line(gmix_tab, GMIX_TABSIZE));
}

char *
ggmix_get_cluster_fname(){
    return cluster_param_fname;
}

int
ggmix_get_cluster_alg(){
    return clustering_algorithm;
}

int
ggmix_must_store_clustering(){
    return clustering_active_flag;
}

static int clustering_active_arg = 0;
static int tied_arg = 0;
static int full_arg = 0;
static ARG argtab2[]=
{
    {"clus_active_arg", INT, &clustering_active_arg,
       "write cluster calls on create"},
    {"clus_alg", INT, &clustering_algorithm, "clustering algorithm number"},
    {"tied_arg", INT, &tied_arg, "tied mixtures"},
    {"full_arg", INT, &full_arg, "full covariance"},
};
#define TABSIZE2 (sizeof(argtab2)/sizeof(ARG))

void
write_gmix_defaults(default_file)
     FILE *default_file;
{
    clustering_active_arg = clustering_active_flag;
    tied_arg = tied_flag;
    full_arg = full_flag;
    if(default_file != NULL){
	fprintf (default_file,"gmix %s",
		 nnargs_get_part_flags(gmix_tab,GMIX_TABSIZE));
	fprintf (default_file, "%s", nnargs_get_flags(argtab2, TABSIZE2));
    }
}

void
read_gmix_defaults(argc, argv)
     int argc;
     char **argv;
{
    if(argc > 0){
	argc = nnargs_part(argc, argv, gmix_tab, GMIX_TABSIZE);
	argc = nnargs(argc, argv, argtab2, TABSIZE2);
    } else {
	clustering_active_arg = 1;
	nepochs = 10;
    }
    clustering_active_flag = clustering_active_arg;
    strcpy(param_ftmp, cluster_param_fname);
    cluster_param_fname = param_ftmp;
    tied_flag = tied_arg;
    full_flag = full_arg;
    /* algorithm specific flags are now set.  generic flags are taken
       from the argument list for the main screen */
}
