/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/*
 * norm_stubs.c - Notify and event callback function stubs.
 * This file was generated by `gxv' from `norm.G'.
 */

#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <nnlib.h>
#include <norm_args.h>			/* nnargs definitions */
#define NORM_PROGRAM_NAME "norm"
#define PLOT_NORM "plot_norm"
#include <plot_norm_args.h>
#include "nn_params.h"
#include "norm_ui.h"
#include "../LNKnet_main_ui.h"
#include "../LNKnet_subs.h"

extern	norm_norm_pop_objects	*Norm_norm_pop;
extern	norm_norm_generate_pop_objects	*Norm_norm_generate_pop;
extern lnknet_glyph_objects *LNKnet_glyphs;


/* macros for normalization */
#define NONE 0
#define SIMPLE 1
#define PCA 2
#define LDA 3

#define EXP_AND_PLOT 0
#define PLOT_ONLY 1

#define MAXSTR 256
static char norm_ftmp[MAXSTR];	/* normalization parameter file */
static char data_ftmp[MAXSTR];	/* data file */
static char norm_plot_ftmp[MAXSTR];	/* plot file */
static char norm_plot_lbl_tmp[MAXSTR];	/* plot label */
static char norm_log_ftmp[MAXSTR];	/* log file */
static char norm_shell_ftmp[MAXSTR];	/* shell file */
static int norm_plot;

static int norm_pid = 0;

void g_set_normalize_filename();
void g_create_search_filename();
void set_select_n();
void g_set_norm_autoscale();
void gnorm_set_common_flags();
void norm_run();
void g_set_norm_plot_label();
void activate_norm_plots();
void norm_check_file();
void set_plot_labels();

/* call once, on initialization.  sets variables and initializes
   strings in nn_params */
void
g_init_norm()
{
    nn_params.normalize = 0;
    nn_params.norm_fname = norm_ftmp;
    norm_plot = 1;
    norm_plot_fname = norm_plot_ftmp;
    norm_fname = norm_ftmp;
}

void
g_display_norm()
{
    /* norm window */
    show_int_value (Norm_norm_pop->normalize, nn_params.normalize);
    show_string_label (Norm_norm_pop->norm_file, nn_params.norm_fname);
    norm_check_file();

    /* norm_generate window */
    show_string_label(Norm_norm_generate_pop->norm_type_msg,
		      norm_description(nn_params.normalize));
    show_string_label(Norm_norm_generate_pop->norm_file_msg,
		      nn_params.norm_fname);
    show_string_label(Norm_norm_generate_pop->norm_shell_script,
		      norm_shell_ftmp);
    show_string_label(Norm_norm_generate_pop->norm_log_file, norm_log_ftmp);
    show_int_value(Norm_norm_generate_pop->norm_do_plot, norm_plot);
    show_string_label(Norm_norm_generate_pop->norm_plot_file,
		      norm_plot_fname);
    show_int_value (Norm_norm_generate_pop->norm_autoscale,
		    norm_auto_scale_flag);
    g_set_norm_autoscale(norm_auto_scale_flag);
    show_float_text (Norm_norm_generate_pop->norm_xmin, norm_xmin);
    show_float_text (Norm_norm_generate_pop->norm_xmax, norm_xmax);
    show_float_text (Norm_norm_generate_pop->norm_ymin, norm_ymin);
    show_float_text (Norm_norm_generate_pop->norm_ymax, norm_ymax);
    show_float_text (Norm_norm_generate_pop->norm_xstep, norm_xstep);
    show_float_text (Norm_norm_generate_pop->norm_ystep, norm_ystep);
    show_string_label(Norm_norm_generate_pop->norm_plot_label,
		      norm_plot_label);
    activate_norm_plots(nn_params.normalize == PCA ||
			nn_params.normalize == LDA);

}

/*
 * Notify callback function for norm settings
 */
void
norm_3_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;/*ARGSUSED*/
{
    norm_norm_pop_objects *ip =
      (norm_norm_pop_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
    void select_set_plot_label();
    void region_set_input_label(), profile_set_input_label();
	
    if(item == ip->normalize){
	nn_params.normalize = value;
	/* if this is lda, we need to play with the feature selection */
	if(value == LDA) set_select_n(nn_params.ninputs);
	activate_norm_plots(value == PCA || value == LDA);
	g_set_normalize_filename();
	g_create_search_filename();
	show_string_label(Norm_norm_generate_pop->norm_type_msg,
			  norm_description(nn_params.normalize));

	/* if this is NONE, we should turn off the Start button on
	   the generate window */
	gray_out_item(Norm_norm_generate_pop->norm_run, (value == NONE));

	/* update plot labels */
	sprintf(norm_plot_lbl_tmp, "Norm:%s",
		norm_description(nn_params.normalize));
	norm_plot_label = norm_plot_lbl_tmp;
	show_string_label(Norm_norm_generate_pop->norm_plot_label,
			 norm_plot_label);

	select_set_plot_label();
	set_plot_labels();
	region_set_input_label();
	profile_set_input_label();
	/* tell notebook that something important changed */
	nn_params.major_note = 1;
    } else {
	fprintf(stderr,
		"LNKnet: norm_3_handler: unhandled setting: value: %u\n",
		value);
    }
}

/*
 * Notify callback function for `norm_generate'.
 */
void
norm_btn_handler(item, event)
     Panel_item	item;
     Event	*event;/*ARGSUSED*/
{
    norm_norm_pop_objects *ip =
      (norm_norm_pop_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
    if(item == ip->norm_generate){
	open_popup(Norm_norm_generate_pop->norm_generate_pop);
    } else {
	fputs("LNKnet: norm_btn_handler: unhandled button\n", stderr);
    }
}

/*
 * Event callback function for `norm_warning'.
 */
void
norm_norm_pop_norm_warning_event_callback(item, event)
     Panel_item	item;
     Event	*event;
{
    if (event_action(event) == ACTION_SELECT) {
	norm_check_file();
    }
    panel_default_handle_event(item, event);
}

/*
 * Event callback function for `norm_err_message'.
 */
void
norm_norm_pop_norm_err_message_event_callback(item, event)
	Panel_item	item;
	Event		*event;
{
    if (event_action(event) == ACTION_SELECT) {
	norm_check_file();
    }
    panel_default_handle_event(item, event);
}

/*
 * Notify callback function for norm generate buttons
 */
void
norm_gen_btn_handler(item, event)
	Panel_item	item;
	Event		*event;/*ARGSUSED*/
{
    norm_norm_generate_pop_objects *ip =
      (norm_norm_generate_pop_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
    if(item == ip->norm_cancel){
	stop_command_file(norm_pid);
	norm_pid = 0;
	close_popup(ip->norm_generate_pop);
    } else if(item == ip->norm_run){
	if(nn_params.normalize != NONE) norm_run(EXP_AND_PLOT);
    } else if(item == ip->norm_plot_only){
	norm_run(PLOT_ONLY);
    } else {
	fputs("LNKnet: norm_gen_btn_handler: unhandled button\n", stderr);
    }
}

/*
 * Notify callback function for norm generate text items
 */
Panel_setting
norm_gen_2_handler(item, event)
     Panel_item	item;
     Event	*event;/*ARGSUSED*/
{
    norm_norm_generate_pop_objects *ip =
      (norm_norm_generate_pop_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
    char *	value = (char *) xv_get(item, PANEL_VALUE);
	
    if(item == ip->norm_xmax){
	norm_xmax = set_atof_value(value, "norm_xmax",
				   ip->norm_xmax, norm_xmax);
    } else if(item == ip->norm_xmin){
	norm_xmin = set_atof_value(value, "norm_xmin",
				   ip->norm_xmin, norm_xmin);
    } else if(item == ip->norm_xstep){
	norm_xstep = set_atof_value(value, "norm_xstep",
				   ip->norm_xstep, norm_xstep);
    } else if(item == ip->norm_ymax){
	norm_ymax = set_atof_value(value, "norm_ymax",
				   ip->norm_ymax, norm_ymax);
    } else if(item == ip->norm_ymin){
	norm_ymin = set_atof_value(value, "norm_ymin",
				   ip->norm_ymin, norm_ymin);
    } else if(item == ip->norm_ystep){
	norm_ystep = set_atof_value(value, "norm_ystep",
				   ip->norm_ystep, norm_ystep);
    } else {
	fprintf(stderr,
		"LNKnet: norm_gen_2_handler: unhandled text: value: %s\n",
		value);
    }
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for norm generate settings
 */
void
norm_gen_3_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;/*ARGSUSED*/
{
    norm_norm_generate_pop_objects *ip =
      (norm_norm_generate_pop_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
    short	i;
	
    if(item == ip->norm_autoscale){
	g_set_norm_autoscale(value);
    } else if(item == ip->norm_do_plot){
	norm_plot = value;
    } else {
	fputs("LNKnet: norm_gen_3_handler: unhandled setting\n", stderr);
	for (i = 0; i < 1; i++) {
	    if (value & 01)
	      fprintf(stderr, "\t%dth item selected\n", i);
	    
	    value >>= 1;
	}
    }
}

void
g_set_normalize_filename()
{
    static char *norm_string[4] = {"none", "simple", "pca", "lda"};
    
    strcpy (norm_ftmp, nn_params.data_base);
    strcat (norm_ftmp, ".norm.");
    nn_params.norm_fname =
      strcat (norm_ftmp, norm_string[nn_params.normalize]);
    show_string_label (Norm_norm_pop->norm_file, norm_ftmp);
    g_set_norm_fname();
}

/* set strings that depend on norm_fname */
g_set_norm_fname()
{
    norm_check_file();
    strcpy(norm_shell_ftmp, norm_ftmp);
    strcat(norm_shell_ftmp, ".run");
    show_string_label(Norm_norm_generate_pop->norm_shell_script,
		      norm_shell_ftmp);
    strcpy(norm_log_ftmp, norm_ftmp);
    strcat(norm_log_ftmp, ".log");
    show_string_label(Norm_norm_generate_pop->norm_log_file, norm_log_ftmp);
    strcpy(norm_plot_ftmp, norm_ftmp);
    strcat(norm_plot_ftmp, ".plot");
    show_string_label(Norm_norm_generate_pop->norm_plot_file, norm_plot_ftmp);
    show_string_label(Norm_norm_generate_pop->norm_file_msg, norm_ftmp);
}

void
g_set_norm_autoscale (value)
     int value;
{
    int on_off;
    if(value) on_off = TRUE;
    else on_off = FALSE;
    norm_auto_scale_flag = on_off; 
    gray_out_item(Norm_norm_generate_pop->norm_axes_limits, on_off);
    gray_out_item(Norm_norm_generate_pop->norm_xmin, on_off);
    gray_out_item(Norm_norm_generate_pop->norm_ymin, on_off);
    gray_out_item(Norm_norm_generate_pop->norm_xmax, on_off);
    gray_out_item(Norm_norm_generate_pop->norm_ymax, on_off);
    gray_out_item(Norm_norm_generate_pop->norm_xstep, on_off);
    gray_out_item(Norm_norm_generate_pop->norm_ystep, on_off);
}

void
g_set_norm_plot_label (value)
     char *value;
{
    sprintf (norm_plot_lbl_tmp, "\"%s\"", value);
    norm_plot_label = norm_plot_lbl_tmp;
}

char *
gnorm_get_plot_label (string)
     char * string;
{
    return string;
}

void
gnorm_set_common_flags()
{
    data_path = nn_params.data_path;
    strcpy(data_ftmp, nn_params.data_base);
    strcat(data_ftmp, nn_params.ext_train);
    data_fname = data_ftmp;
    desc_fname = nn_params.desc_fname;
    norm_fname = nn_params.norm_fname;
    normalization = nn_params.normalize;
    verbosity = nn_params.verbosity;
    /* npatterns is not set because the whole file is read, regardless */
    plot_exp_path = nn_params.exp_path;
    plot_error_fname = norm_fname;
    plot_n_inputs = nn_params.nraw;
    plot_n_outputs = nn_params.noutputs;
    mac_normlines_fname = "norm.mac";
    /* labels */
    norm_data_path = nn_params.data_path;
}

/* defaults file arguments, table, and routines */
static char *norm_shell_arg = "";
static char *norm_log_arg = "";

static ARG normtab[] =
{
    {"normalize", INT, &normalization, ""},
    {"norm_fname", STRING, (int *)&norm_fname, ""},
    {"shell_script", STRING, (int *)&norm_shell_arg, ""},
    {"log_file", STRING, (int *)&norm_log_arg, ""},
    {"generate_plot", INT, &norm_plot, ""},
};
#define NORM_DEF_LEN (sizeof(normtab)/sizeof(ARG))

void
write_norm_defaults(default_file)
     FILE *default_file;
{
    norm_fname = null_if_empty_string(norm_ftmp);
    norm_shell_arg = null_if_empty_string(norm_shell_ftmp);
    norm_log_arg = null_if_empty_string(norm_log_ftmp);

    norm_plot_fname = null_if_empty_string(norm_plot_ftmp);
    norm_plot_label = null_if_empty_string(norm_plot_lbl_tmp);

    /* strings set in gnorm_set_common_flags */
    plot_exp_path = null_if_empty_string(plot_exp_path);
    plot_error_fname = null_if_empty_string(plot_error_fname);
    norm_data_path = null_if_empty_string(norm_data_path);
    mac_normlines_fname = null_if_empty_string(mac_normlines_fname);

    if(default_file != NULL){
	/* regular flags */
	fprintf (default_file, "norm %s",
		 nnargs_get_part_flags(normtab, NORM_DEF_LEN));
	/* plot flags */
	fprintf (default_file, "%s",
		 nnargs_get_flags(norm_argtab, NORM_TABSIZE));
    }
}

void
read_norm_defaults(argc, argv)
     int argc;
     char **argv;
{
    /* zero out flags that are not present when not set */
    norm_auto_scale_flag = 0;
    norm_fname = norm_shell_arg = norm_log_arg = NULL;
    norm_plot_label = norm_plot_fname = NULL;

    /* these flags are reset each time they are used */
    plot_exp_path = plot_error_fname = norm_data_path = NULL;
    mac_normlines_fname = NULL;

    if(argc > 0){
	argc = nnargs_part (argc, argv, normtab, NORM_DEF_LEN);
	argc = nnargs(argc, argv, norm_argtab, NORM_TABSIZE);
    } else {
	normalization = SIMPLE;
	norm_auto_scale_flag = 1;
    }

    norm_plot_fname = strcpy_empty_if_null(norm_plot_ftmp, norm_plot_fname);
    norm_plot_label =
      strcpy_empty_if_null(norm_plot_lbl_tmp, norm_plot_label);

    nn_params.normalize = normalization;
    norm_fname = nn_params.norm_fname =
      strcpy_empty_if_null(norm_ftmp, norm_fname);
    strcpy_empty_if_null(norm_shell_ftmp, norm_shell_arg);
    strcpy_empty_if_null(norm_log_ftmp, norm_log_arg);
}

void
norm_run(run_flag)
     int run_flag;
{
    char *shell_path;
    shell_path = path_build(nn_params.exp_path, norm_shell_ftmp);
    if(overwrite_ok(shell_path)){
	if(store_norm_commands(shell_path, run_flag)) return;
	if(!nn_params.store_only)
	  norm_pid = chdir_and_execute_command(norm_shell_ftmp);
    }
}

int
store_norm_commands(shell_path, run_flag)
     char *shell_path;
     int run_flag;
{
    FILE *shell_file;

    gnorm_set_common_flags();

    shell_file = command_file_open(shell_path, "for norm shell file");
    if(shell_file == NULL) return -1;

    if(run_flag != PLOT_ONLY){
	fprintf(shell_file, "\n#calculate normalization parameters\n");
	fprintf (shell_file, "%s %s", NORM_PROGRAM_NAME,
		 nnargs_get_part_flags(argtab, TABSIZE));
	if(nn_params.store_only)
	  fprintf(shell_file, ">& %s\n", norm_log_ftmp);
	else
	  fprintf(shell_file, "|& nn_tee -h %s\n", norm_log_ftmp);
    }

    if(norm_plot || run_flag == PLOT_ONLY){
	if(nn_params.normalize == PCA || nn_params.normalize == LDA){
	    extern char plot_command_line[];

	    fprintf(shell_file, "\n#plot normalization parameters\n");
	    if(nn_params.store_only) no_graphics = 1;
	    else no_graphics = 0;
	    if(nn_params.store_only)
	      fprintf(shell_file, "(time ");
	    sprintf(plot_command_line, "%s %s", PLOT_NORM,
		    nnargs_get_flags(norm_argtab, NORM_TABSIZE));
	    *strrchr(plot_command_line, '\n') = ' ';
	    fprintf(shell_file, "%s", plot_command_line);
	    if(nn_params.store_only)
	      fprintf(shell_file, "\\\n) >>& %s\n", norm_log_ftmp);
	    else fprintf(shell_file, "\n");
	}
    }
    command_file_close(shell_file, shell_path, "for norm shell file");
    return 0;
}

void
activate_norm_plots(activate_flag)
     int activate_flag;
{
    if(activate_flag){
	gray_out_item(Norm_norm_generate_pop->norm_do_plot, FALSE);
	gray_out_item(Norm_norm_generate_pop->norm_plot_file, FALSE);
	gray_out_item(Norm_norm_generate_pop->norm_plot_file1, FALSE);
	gray_out_item(Norm_norm_generate_pop->norm_autoscale, FALSE);
	g_set_norm_autoscale(norm_auto_scale_flag);
	gray_out_item(Norm_norm_generate_pop->norm_plot_label, FALSE);
	gray_out_item(Norm_norm_generate_pop->norm_plot_label1, FALSE);
    } else {
	gray_out_item(Norm_norm_generate_pop->norm_do_plot, TRUE);
	gray_out_item(Norm_norm_generate_pop->norm_plot_file, TRUE);
	gray_out_item(Norm_norm_generate_pop->norm_plot_file1, TRUE);
	gray_out_item(Norm_norm_generate_pop->norm_autoscale, TRUE);
	gray_out_item(Norm_norm_generate_pop->norm_axes_limits, TRUE);
	gray_out_item(Norm_norm_generate_pop->norm_xmin, TRUE);
	gray_out_item(Norm_norm_generate_pop->norm_ymin, TRUE);
	gray_out_item(Norm_norm_generate_pop->norm_xmax, TRUE);
	gray_out_item(Norm_norm_generate_pop->norm_ymax, TRUE);
	gray_out_item(Norm_norm_generate_pop->norm_xstep, TRUE);
	gray_out_item(Norm_norm_generate_pop->norm_ystep, TRUE);
	gray_out_item(Norm_norm_generate_pop->norm_plot_label, TRUE);
	gray_out_item(Norm_norm_generate_pop->norm_plot_label1, TRUE);
    }
}

void
norm_check_file()
{
    time_t train_time, norm_time;
    struct stat buf;
    char *train_path;

    if(nn_params.normalize == NONE)
      norm_clear_error();
    else {
	/* check if the file exists */
	if(file_exists(path_build(nn_params.data_path, nn_params.norm_fname))){
	    /* check that the training data file is older
	       than the normalization file */
	    train_path = path_build(nn_params.data_path, nn_params.data_base);
	    strcat(train_path, nn_params.ext_train);
	    if(stat(train_path, &buf) == 0) train_time = buf.st_mtime;
	    else train_time = 0;
	    if(stat(path_build(nn_params.data_path, nn_params.norm_fname), &buf)
	       == 0) norm_time = buf.st_mtime;
	    else norm_time = 1;
	    if(norm_time < train_time){
		norm_error("Warning: Normalization file is older than training file");
	    } else{
		norm_clear_error();
	    }
	} else {
	    norm_error("Error: Normalization file does not exist");
	}
    }
}
norm_error(string)
     char *string;
{
    void set_main_norm_warning();
    show_string_label(Norm_norm_pop->norm_err_message, string);
    show_image_label(Norm_norm_pop->norm_warning,
		     LNKnet_glyphs->LNKnet_warning_pic);
    set_main_norm_warning(LNKnet_glyphs->small_warning_pic);
}

norm_clear_error()
{
    void set_main_norm_warning();
    /* clear error message if we have made it this far */
    show_string_label(Norm_norm_pop->norm_err_message, "");
    show_image_label(Norm_norm_pop->norm_warning,
		     LNKnet_glyphs->LNKnet_blank_pic);
    set_main_norm_warning(LNKnet_glyphs->small_blank_pic);
}
