/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/*
 * reports_stubs.c - Notify and event callback function stubs.
 * This file was generated by `gxv' from `reports.G'.
 */

#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include "reports_ui.h"
#include "nn_params.h"
#include "../LNKnet_subs.h"
#include <nnlib.h>

extern	reports_file_pop_objects	*Reports_file_pop;

#define MAXSTR 256

/* space for file names */
static char working_directory[MAXSTR];
static char exp_path[MAXSTR] = "./";
static char param_ftmp[MAXSTR];
static char error_ftmp[MAXSTR];
static char shell_full_path[MAXSTR];
static char shell_ftmp[MAXSTR];
static char log_ftmp[MAXSTR];
static char screen_ftmp[MAXSTR];
static char screen_full_path[MAXSTR];
static char notebook_ftmp[MAXSTR] = "LNKnet.note";
static char notebook_full_path[MAXSTR];

/* call once, on initialization.  sets variables and initializes
   strings in nn_params */
void
g_init_reports()
{
    if(getwd(working_directory) == 0){
	warning("couldn't get directory name. %s", working_directory);
    }
    nn_params.working_directory = working_directory;
    nn_params.exp_path = exp_path;
    nn_params.param_fname = param_ftmp;
    nn_params.shell_fname = shell_ftmp;
    nn_params.shell_path = shell_full_path;
    nn_params.log_fname = log_ftmp;
    nn_params.screen_fname = screen_ftmp;
    nn_params.screen_path = screen_full_path;
    nn_params.error_fname = error_ftmp;
    nn_params.notebook_path = notebook_full_path;
    nn_params.notebook_fname = notebook_ftmp;
}

void
g_display_reports()
{
    show_string_text (Reports_file_pop->wdir_name,
		      nn_params.working_directory);
    show_string_text(Reports_file_pop->experiment_path, nn_params.exp_path);
    show_string_label(Reports_file_pop->shell_file, nn_params.shell_fname);
    show_string_label(Reports_file_pop->log_file, nn_params.log_fname);
    show_string_label(Reports_file_pop->param_file, nn_params.param_fname);
    show_string_label(Reports_file_pop->error_file, error_ftmp);
    show_string_label(Reports_file_pop->screen_file, nn_params.screen_fname);
    show_string_text(Reports_file_pop->notebook_file,
		     nn_params.notebook_fname);
    show_int_value(Reports_file_pop->verbosity, nn_params.verbosity);
    show_int_value(Reports_file_pop->verror, nn_params.verror);
    show_int_value(Reports_file_pop->no_train_verror,
		   nn_params.no_train_verror);
    show_int_value(Reports_file_pop->notebook_on, nn_params.notebook_on);
}

/*
 * Notify callback function for main window buttons
 */
void
reports_btn_handler(item, event)
     Panel_item	item;
     Event	*event;/*ARGSUSED*/
{
    reports_file_pop_objects *ip =
      (reports_file_pop_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
    FILE *default_file;
    void read_defaults();

    if (item == ip->restore_screen){
	if ((default_file =
	     fopen(nn_params.screen_path, "r")) == NULL) {
	    warning("Default parameter file %s not found.\n",
		    nn_params.screen_path);
	} else {
	    read_defaults(default_file, nn_params.screen_path);
	    nn_fclose(default_file, nn_params.screen_path,
		      "for reading LNKnet defaults");
	}
    } else {
	fputs("LNKnet: reports_btn_handler, unhandled button\n", stderr);
    }
}

/*
 * Notify callback function for reports text
 */
Panel_setting
reports_text_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    reports_file_pop_objects *ip =
      (reports_file_pop_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
    char *	value = (char *) xv_get(item, PANEL_VALUE);
	
#ifdef notdef
    if(item == ip->shell_file){
	value = set_string_not_empty(value, "shell file", item, shell_ftmp);
	nn_params.shell_fname = strcpy(shell_ftmp, value);
	nn_params.shell_path =
	  strcpy(shell_full_path, path_build(nn_params.exp_path, shell_ftmp));
    } else if(item == ip->error_file){
	value = set_string_not_empty(value, "error file", item, error_ftmp);
	nn_params.error_fname = strcpy(error_ftmp, value);
    } else if(item == ip->param_file){
	value = set_string_not_empty(value, "parameter file", item,
				 param_ftmp);
	nn_params.param_fname = strcpy(param_ftmp, value);
    } else if (item == ip->log_file){
	value = set_string_not_empty(value, "log file", item, log_ftmp);
	nn_params.log_fname = strcpy(log_ftmp, value);
	print_set_log_status();
    } else if(item == ip->screen_file){
	value = set_string_not_empty(value, "screen file", item, screen_ftmp);
	nn_params.screen_fname = strcpy(screen_ftmp, value);
	nn_params.screen_path =
	  strcpy(screen_full_path, path_build(nn_params.exp_path,
					      screen_ftmp));
    } /* else */
#endif
    if(item == ip->notebook_file){
	value = set_string_not_empty(value, "notebook file",
				     item, notebook_ftmp);
	nn_params.notebook_fname = strcpy(notebook_ftmp, value);
	nn_params.notebook_path = 
	  strcpy(notebook_full_path, path_build(nn_params.working_directory,
						notebook_ftmp));
	nn_params.major_note = 1;
    } else if(item == ip->wdir_name){
	show_string_text(item, nn_params.working_directory);
    } else if(item == ip->experiment_path){
	void committeeData_update_exp_path(), ccode_check_file();

	nn_params.exp_path = strcpy(exp_path, value);
	nn_params.shell_path =
	  strcpy(shell_full_path,
		 path_build(nn_params.exp_path, nn_params.shell_fname));
	nn_params.screen_path =
	  strcpy(screen_full_path, path_build(nn_params.exp_path,
					      screen_ftmp));
	ccode_check_file();
	committeeData_update_exp_path();
	print_set_all_status();
    } else {
	fprintf(stderr,
		"LNKnet: reports_2_handler: unhandled text: value: %s\n",
		value);
    }
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for report settings
 */
void
reports_select_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;/*ARGSUSED*/
{
    reports_file_pop_objects *ip =
      (reports_file_pop_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
    void display_verror();
	
    if(item == ip->verbosity){
	nn_params.verbosity = value;
    } else if(item == ip->verror){
	/* check that cost and perr are not selected */
	if(plots_verror_value_ok(value)){
	    nn_params.verror = value;
	} else
	  display_verror();
    } else if (item == ip->notebook_on){
	nn_params.notebook_on = value;
    } else if (item == ip->no_train_verror){
	/* switch the sense of the value to find the resulting verror
	   and ask permission to make the change from the plots */
	if(plots_train_verror_value_ok(value ? 0 : nn_params.verror))
	  nn_params.no_train_verror = value;
	else display_verror();
    } else {
	fprintf(stderr,
		"LNKnet: reports_3_handler: unhandled setting: value: %u\n",
		value);
    }
}

void
create_experiment_file_names()
{
    /* shell script name */
    strcpy(shell_ftmp, nn_params.exp_name);
    nn_params.shell_fname = strcat(shell_ftmp, ".run");
    show_string_label(Reports_file_pop->shell_file, shell_ftmp);
    nn_params.shell_path = 
      strcpy(shell_full_path,
	     path_build(nn_params.exp_path, nn_params.shell_fname));

    /* log file */
    strcpy(log_ftmp, nn_params.exp_name);
    nn_params.log_fname = strcat(log_ftmp, ".log");
    show_string_label(Reports_file_pop->log_file, log_ftmp);
    print_set_log_status();

    /* parameter file name */
    strcpy(param_ftmp, nn_params.exp_name);
    nn_params.param_fname = strcat(param_ftmp, ".param");
    show_string_label(Reports_file_pop->param_file, param_ftmp);
    
    /* error name prefix */
    strcpy(error_ftmp, nn_params.exp_name);
    nn_params.error_fname = strcat(error_ftmp, ".err");
    show_string_label(Reports_file_pop->error_file, error_ftmp);
    
    /* screen file */
    strcpy(screen_ftmp, nn_params.exp_name);
    nn_params.screen_fname = strcat(screen_ftmp, ".screen");
    show_string_label(Reports_file_pop->screen_file, screen_ftmp);
    nn_params.screen_path =
      strcpy(screen_full_path,
	     path_build(nn_params.exp_path, nn_params.screen_fname));
}

void
display_verror()
{
    show_int_value (Reports_file_pop->verror, nn_params.verror);
    show_int_value (Reports_file_pop->no_train_verror,
		    nn_params.no_train_verror);
}

/* defaults file arguments, table, and routines */
static char *exp_path_arg = "./";
static char *param_fname_arg = "";
static char *error_fname_arg = "";
static char *shell_fname_arg = "";
static char *log_fname_arg = "";
static char *screen_fname_arg = "";
static char *notebook_fname_arg = "";
static int notebook_on_arg = 0;
static int verbosity_arg = 3;
static int verror_arg = 0;
static int no_train_verror_arg = 0;

static ARG reportstab[] =
{
    {"exp_path", STRING, (int *)&exp_path_arg},
    {"param_fname", STRING, (int *)&param_fname_arg},
    {"error_fname", STRING, (int *)&error_fname_arg},
    {"shell_fname", STRING, (int *)&shell_fname_arg},
    {"log_fname", STRING, (int *)&log_fname_arg},
    {"screen_fname", STRING, (int *)&screen_fname_arg},
    {"notebook_fname", STRING, (int *)&notebook_fname_arg},
    {"notebook_on", FLAG, &notebook_on_arg},
    {"verbosity", INT, &verbosity_arg},
    {"verror", INT, &verror_arg},
    {"no_train_verror", FLAG, &no_train_verror_arg},
};
#define TABSIZE (sizeof(reportstab)/sizeof(ARG))

void
read_reports_defaults(argc, argv)
     int argc;
     char **argv;
{
    /* clear flags that are not present in the defaults file if they
       are not "set" */
    exp_path_arg = param_fname_arg = error_fname_arg = NULL;
    shell_fname_arg = log_fname_arg = screen_fname_arg = NULL;
    notebook_fname_arg = NULL;
    notebook_on_arg = 0;
    no_train_verror_arg = 0;

    if(argc > 0){
	argc = nnargs(argc, argv, reportstab, TABSIZE);
    } else {
	verbosity_arg = 3;
	verror_arg = 0;
	no_train_verror_arg = 0;
	exp_path_arg = "./";
	notebook_on_arg = 1;
	notebook_fname_arg = "LNKnet.note";
    }
    nn_params.exp_path = strcpy_empty_if_null(exp_path, exp_path_arg);
    nn_params.param_fname = strcpy_empty_if_null(param_ftmp, param_fname_arg);
    nn_params.error_fname = strcpy_empty_if_null(error_ftmp, error_fname_arg);
    nn_params.shell_fname = strcpy_empty_if_null(shell_ftmp, shell_fname_arg);
    nn_params.shell_path =
      strcpy_empty_if_null(shell_full_path, path_build(exp_path, shell_ftmp));
    nn_params.log_fname = strcpy_empty_if_null(log_ftmp, log_fname_arg);
    nn_params.screen_fname =
      strcpy_empty_if_null(screen_ftmp, screen_fname_arg);
    nn_params.screen_path =
      strcpy_empty_if_null(screen_full_path,
			   path_build(exp_path, screen_ftmp));
    nn_params.notebook_fname =
      strcpy_empty_if_null(notebook_ftmp, notebook_fname_arg);
    nn_params.notebook_path =
      strcpy_empty_if_null(notebook_full_path,
			   path_build(nn_params.working_directory,
				      notebook_ftmp));
    nn_params.notebook_on = notebook_on_arg;
    nn_params.verbosity = verbosity_arg;
    nn_params.verror = verror_arg;
    nn_params.no_train_verror = no_train_verror_arg;
}

void
write_reports_defaults(default_file)
     FILE *default_file;
{
    exp_path_arg = null_if_empty_string(exp_path);
    param_fname_arg = null_if_empty_string(param_ftmp);
    error_fname_arg = null_if_empty_string(error_ftmp);
    shell_fname_arg = null_if_empty_string(shell_ftmp);
    log_fname_arg = null_if_empty_string(log_ftmp);
    screen_fname_arg = null_if_empty_string(screen_ftmp);
    notebook_fname_arg = null_if_empty_string(notebook_ftmp);
    verbosity_arg = nn_params.verbosity;
    verror_arg = nn_params.verror;
    no_train_verror_arg = nn_params.no_train_verror;

    if(default_file != NULL){
	fprintf(default_file, "%s %s", "reports",
		nnargs_get_flags(reportstab, TABSIZE));
    }
}
