/***--------------------------------------------------------------------------
* NAME:        ghistogram.c
* PURPOSE:     Histogram classifier graphical argument setter
*
* BUGS AND LIMITATIONS:
*
*
-------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gclass/histogram/ghistogram.c,v 10.3 2001/08/23 20:13:28 craig Exp $";




#define TRUE 1
#define FALSE 0


#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>

#include "ghistogram_ui.h"
#include <nnlib.h>
#include <histogram_args.h>	/* statics for histogram args */
#include "../LNKnet_subs.h"
#include "../gnn/nn_params.h"

			/* static GLOBALS internal to this file */

/* Needed window pointers */
extern ghistogram_param_pop_objects	*Ghistogram_param_pop;

/* statics for file name parts */
#define MAXSTR 256

static int specify_nbins = 0;
static char tmp_str[MAXSTR];
static void ghistogram_show_nbins_choices();
static void ghistogram_show_grand_choices();
void ghistogram_set_total_nbins();

static int total_nbins;

void
ghistogram_display_params()
{
    show_int_value(Ghistogram_param_pop->grand_bins, grand_bins);
    ghistogram_show_grand_choices();
    show_int_value(Ghistogram_param_pop->bin_type, bin_type);
    show_int_value(Ghistogram_param_pop->specify_nbins, specify_nbins);
    ghistogram_show_nbins_choices();
    if(nBinsPerClass != NULL){
	show_string_text(Ghistogram_param_pop->nBinsPerClass,
			 make_net_string(nBinsPerClass));
    }
    show_int_value(Ghistogram_param_pop->use_gauss, use_gauss);
    if(use_gauss) gray_out_item(Ghistogram_param_pop->min_var, FALSE);
    else gray_out_item(Ghistogram_param_pop->min_var, TRUE);
    show_float_text(Ghistogram_param_pop->min_var, minimum_var);
    show_int_text(Ghistogram_param_pop->nbins, nbins);
    show_float_text(Ghistogram_param_pop->range_factor, range_factor);
    show_float_text(Ghistogram_param_pop->user_min, user_min);
    show_float_text(Ghistogram_param_pop->user_max, user_max);
}

void
ghistogram_init ()
{
    use_gauss = 1;
}

/*
 * callback for widgets on histogram parameter window
 */
void
ghistogram_callback(item, event)
     Panel_item item;
     Event	*event;
{
    ghistogram_param_pop_objects *ip =
      (ghistogram_param_pop_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
    char *	svalue;

    if (item == ip->specify_nbins){
	specify_nbins = (int)xv_get(item, PANEL_VALUE);
	ghistogram_show_nbins_choices();
    } else if(item == ip->nbins){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	nbins = set_atoi_value(svalue, "nbins", item, nbins);
	ghistogram_set_total_nbins();
	set_plot_labels();
    } else if(item == ip->nBinsPerClass){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	if(nBinsPerClass != NULL) deallocate_ivector(nBinsPerClass);
	nBinsPerClass = nnargs_get_int_alloc_list(strcpy(tmp_str, svalue));
	ghistogram_set_total_nbins();
	set_plot_labels();
    } else if(item == ip->grand_bins){
	grand_bins = (int)xv_get(item, PANEL_VALUE);
	ghistogram_show_grand_choices();
	ghistogram_set_total_nbins();
	set_plot_labels();
    } else if(item == ip->user_min){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	user_min = set_atof_value(svalue, "user_min", item, user_min);
    } else if(item == ip->user_max){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	user_max = set_atof_value(svalue, "user_max", item, user_max);
    } else if(item == ip->bin_type){
	bin_type = (int)xv_get(item, PANEL_VALUE);
    } else if(item == ip->range_factor){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	range_factor = set_atof_low_value(svalue, "range_factor", item,
					  range_factor, 0.);
    } else if (item == ip->use_gauss){
	use_gauss = (int)xv_get(item, PANEL_VALUE);
	if(use_gauss) gray_out_item(ip->min_var, FALSE);
	else gray_out_item(ip->min_var, TRUE);
	set_plot_labels();
    } else if(item == ip->min_var){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	minimum_var = set_atof_value(svalue, "min_var", item, minimum_var);
    } else {
	fprintf(stderr,
		"LNKnet: ghistogram_callback: unhandled interrupt\n");
    }
}
    
/*
 * Notify callback function for selection widgets
 */
void
ghistogram_select_handler(item, value, event)
     Panel_item	item;
     int	value;/*ARGSUSED*/
     Event	*event;
{
    ghistogram_callback(item, event);
}

/*
 * Notify callback function for text widgets
 */
Panel_setting
ghistogram_text_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    ghistogram_callback(item, event);
    return panel_text_notify(item, event);
}

static void
ghistogram_show_nbins_choices()
{
    if(specify_nbins){
	gray_out_item(Ghistogram_param_pop->nBinsPerClass, FALSE);
	gray_out_item(Ghistogram_param_pop->nbins, TRUE);
    } else {
	gray_out_item(Ghistogram_param_pop->nBinsPerClass, TRUE);
	gray_out_item(Ghistogram_param_pop->nbins, FALSE);
    }    
}

static void
ghistogram_show_grand_choices()
{
    if(grand_bins == 0){
	gray_out_item(Ghistogram_param_pop->specify_nbins, FALSE);
	gray_out_item(Ghistogram_param_pop->bin_type, FALSE);
	ghistogram_show_nbins_choices();
	gray_out_item(Ghistogram_param_pop->range_factor, FALSE);
	gray_out_item(Ghistogram_param_pop->grand_msg, TRUE);
	gray_out_item(Ghistogram_param_pop->user_min, TRUE);
	gray_out_item(Ghistogram_param_pop->user_max, TRUE);
    } else if(grand_bins == 1){
	gray_out_item(Ghistogram_param_pop->specify_nbins, TRUE);
	gray_out_item(Ghistogram_param_pop->bin_type, FALSE);
	gray_out_item(Ghistogram_param_pop->nBinsPerClass, TRUE);
	gray_out_item(Ghistogram_param_pop->nbins, FALSE);
	gray_out_item(Ghistogram_param_pop->range_factor, FALSE);
	gray_out_item(Ghistogram_param_pop->grand_msg, TRUE);
	gray_out_item(Ghistogram_param_pop->user_min, TRUE);
	gray_out_item(Ghistogram_param_pop->user_max, TRUE);
    } else {
	gray_out_item(Ghistogram_param_pop->specify_nbins, TRUE);
	gray_out_item(Ghistogram_param_pop->bin_type, TRUE);
	gray_out_item(Ghistogram_param_pop->nBinsPerClass, TRUE);
	gray_out_item(Ghistogram_param_pop->nbins, FALSE);
	gray_out_item(Ghistogram_param_pop->range_factor, TRUE);
	gray_out_item(Ghistogram_param_pop->grand_msg, FALSE);
	gray_out_item(Ghistogram_param_pop->user_min, FALSE);
	gray_out_item(Ghistogram_param_pop->user_max, FALSE);
    }
}

void
ghistogram_set_total_nbins()
{
    static char total_label[64];
    if(grand_bins == 0){
	if(specify_nbins && nBinsPerClass != NULL){
	    int i;
	    for(i = 1, total_nbins = 0; i <= nBinsPerClass[0]; i++){
		total_nbins += nBinsPerClass[i];
	    }
	} else {
	    total_nbins = nn_params.noutputs * nbins;
	}
    } else if (grand_bins){
	total_nbins = nbins;
    }
    sprintf(total_label, "This will create %d bins per dimension.",
	    total_nbins);
    show_string_label(Ghistogram_param_pop->total_nbins_msg, total_label);
}

char *
ghistogram_get_plot_label(string)
     char *string;
{
    sprintf(string, "%s NBins: %d %s", string,
	    total_nbins, use_gauss ? "Use Gauss" : "Allow Unknown");
    return string;
}

void
ghistogram_store_plot_command(shell_file)
     FILE *shell_file;
{
    if(use_gauss)
      fprintf(shell_file, "\\\n -use_gauss");
}

char *
ghistogram_get_args ()
{
    int tmp_nBins;
    char *ret_str;

    if(!specify_nbins && nBinsPerClass != NULL){
	tmp_nBins = nBinsPerClass[0];
	nBinsPerClass[0] = 0;
    }
    ret_str = nnargs_get_part_flags(histogram_tab, HISTOGRAM_TABSIZE);
    if(!specify_nbins && nBinsPerClass != NULL){
	nBinsPerClass[0] = tmp_nBins;
    }
    return (ret_str);
}

char *
ghistogram_get_important_args ()
{
    int tmp_nBins;
    char *ret_str;

    if(!specify_nbins && nBinsPerClass != NULL){
	tmp_nBins = nBinsPerClass[0];
	nBinsPerClass[0] = 0;
    }
    ret_str = nnargs_get_flags_one_line(histogram_tab, HISTOGRAM_TABSIZE);
    if(!specify_nbins && nBinsPerClass != NULL){
	nBinsPerClass[0] = tmp_nBins;
    }
    return (ret_str);
}

static ARG argtab2[]=
{
    {"specify_nbins", FLAG, &specify_nbins, ""},
};
#define TABSIZE2 1

void
write_histogram_defaults(default_file)
     FILE *default_file;
{
    if(default_file != NULL){
	if(specify_nbins){
	    fprintf (default_file, "histogram %s",
		     nnargs_get_part_flags(histogram_tab, HISTOGRAM_TABSIZE));
	    fprintf(default_file, "%s", nnargs_get_flags(argtab2, TABSIZE2));
	} else {
	    fprintf (default_file, "histogram %s",
		     nnargs_get_flags(histogram_tab, HISTOGRAM_TABSIZE));
	}
    }
}

void
read_histogram_defaults(argc, argv)
     int argc;
     char **argv;
{
    specify_nbins = 0;
    use_gauss = 0;
    if(argc > 0){
	argc = nnargs_part(argc, argv, argtab2, TABSIZE2);
	argc = nnargs(argc, argv, histogram_tab, HISTOGRAM_TABSIZE);
    } else {
	use_gauss = 1;
    }
}
