/***--------------------------------------------------------------------------
* NAME:        ghyper.c
* PURPOSE:     Hypersphere main program driver
*			(DEVGUIDE version)
*
*		Executes hyper after setting command line parameters
*
* USAGE:	gnn, then select HYPER from classifier menu
*
* SEE ALSO:	hyper.c hyper_pack.c hyper_subs.c ghyper_subs.c c_subs.c
*
* DESCRIPTION: 
*	ghyper_display_params
*	ghyper_init
*	ghyper_get_plot_label
*
* BUGS AND LIMITATIONS:
*
*
-------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header:";




#define TRUE 1
#define FALSE 0

#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>

#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <sys/ioctl.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/icon_load.h>
#include <xview/panel.h>
#include <xview/scrollbar.h>
#include <xview/svrimage.h>
#include <xview/termsw.h>
#include <xview/text.h>
#include <xview/tty.h>
#include <xview/xv_xrect.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <gcm.h>
#include <gdd.h>
#include "ghyper_ui.h"
#include <nnlib.h>
#include <hyper_args.h>	/* statics for hyper args */
#include "../LNKnet_subs.h"
#include "../gnn/nn_params.h"

			/* static GLOBALS internal to this file */

/* Needed window pointers */
extern ghyper_param_pop_objects	*Ghyper_param_pop;

/* statics for file name parts */
#define MAXSTR 256

/* routines found in this file */
void ghyper_display_params();
void ghyper_init ();
void ghyper_set_prune_graying();

void
ghyper_display_params()
{
    show_int_value (Ghyper_param_pop->add_epochs, nepochs);
    show_float_text (Ghyper_param_pop->r_init, r_init);

    show_int_value (Ghyper_param_pop->prune, prune_flag);
    show_int_value(Ghyper_param_pop->prune_type, correct_sort);
    show_int_value (Ghyper_param_pop->prune_n, prune_n);
    ghyper_set_prune_graying();

    show_int_value (Ghyper_param_pop->force_class, force_classification);
}

void
ghyper_init ()
{
    add_flag = 1;
    prune_flag = 1;
    force_classification = 1;
    prune_n = 100;
}

char *
ghyper_get_plot_label(string)
     char *string;
{
    sprintf(string, "%s epochs:%d radius:%.2g", string,
	    nepochs, r_init);
    if(prune_flag)
      sprintf(string, "%s Prune by %s", string,
	      correct_sort ? "size": "times used");
    sprintf(string, "%s %s", string,
	    force_classification ? "Use NN" : "Allow Unknown");
    return string;
}

void
ghyper_callback(item, event)
     Panel_item	item;
     Event	*event;/*ARGSUSED*/
{
    ghyper_param_pop_objects *ip =
      (ghyper_param_pop_objects *)xv_get(item, XV_KEY_DATA, INSTANCE);
    int ivalue;
    char *svalue;
    if (item == ip->add_epochs){
	ivalue = (int)xv_get(item, PANEL_VALUE);
	nepochs = ivalue;
	set_plot_labels();
    } else if (item == ip->r_init){
	svalue = (char *)xv_get(item, PANEL_VALUE);
	r_init = set_atof_low_value(svalue, "r_init", Ghyper_param_pop->r_init,
				    r_init, 0.);
	set_plot_labels();
    } else if (item == ip->prune){
	ivalue = (int)xv_get(item, PANEL_VALUE);
	prune_flag = ivalue;
	ghyper_set_prune_graying();
    } else if (item == ip->prune_type){
	ivalue = (int)xv_get(item, PANEL_VALUE);
	best_n = 1;
	correct_sort = ivalue;
	set_plot_labels();
    } else if (item == ip->prune_n){
	ivalue = (int) xv_get(item, PANEL_VALUE);
	prune_n = ivalue;
    } else if (item == ip->force_class){
	ivalue = (int)xv_get(item, PANEL_VALUE);
	force_classification = ivalue;
	set_plot_labels();
    } else {
	fprintf(stderr, "LNKnet: ghyper_callback: unhandled interrupt\n");
    }
}

/*
 * Notify callback function for settings.
 */
void
ghyper_select_handler(item, value, event)
     Panel_item item;
     int	value;
     Event	*event;/*ARGSUSED*/
{
    ghyper_callback(item, event);
}

/*
 * Notify callback function for `add_epochs'.
 */
Panel_setting
ghyper_text_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    ghyper_callback(item, event);
    return panel_text_notify(item, event);
}

void
ghyper_set_prune_graying()
{
    if(prune_flag){
	gray_out_item(Ghyper_param_pop->prune_type, FALSE);
	gray_out_item(Ghyper_param_pop->prune_n, FALSE);
    } else {
	gray_out_item(Ghyper_param_pop->prune_n, TRUE);
	gray_out_item(Ghyper_param_pop->prune_type, TRUE);
    }
}

int
ghyper_get_epochs()
{
    return nepochs;
}

void
ghyper_set_epochs(e)
     int e;
{
    /* popup will not reflect the set number of epochs */
    nepochs = e;
}

char *
ghyper_get_args ()
{
    return (nnargs_get_part_flags(hyper_tab, HYPER_TABSIZE));
}

char *
ghyper_get_important_args ()
{
    return (nnargs_get_flags_one_line(hyper_tab, HYPER_TABSIZE));
}

void
write_hyper_defaults(default_file)
     FILE *default_file;
{
    if(default_file != NULL){
	fprintf (default_file, "hyper %s",
		 nnargs_get_flags(hyper_tab, HYPER_TABSIZE));
    }
}

void
read_hyper_defaults(argc, argv)
     int argc;
     char **argv;
{
    add_flag = 0;
    prune_flag = 0;
    force_classification = 0;
    best_n = 0;
    correct_sort = 0;
    if(argc > 0){
	argc = nnargs(argc, argv, hyper_tab, HYPER_TABSIZE);
    } else {
	prune_flag = 1;
	best_n = 1;
	correct_sort = 0;
	force_classification = 1;
	prune_n = 100;
	nepochs = 5;
    }
    add_flag = 1;
}
