/***-------------------------------------------------------------------------
 * NAME:        girbf.c
 * PURPOSE:     Incremental Radial Basis Function Classifier parameters
 *		Executes irbf functions after setting command line parameters
 *
 ------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gclass/irbf/girbf.c,v 10.4 2001/08/23 20:13:32 craig Exp $";



#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <unistd.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <gdd.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <nnlib.h>
#include "girbf_ui.h"
#include <irbf_args.h>			/* nnargs definitions */
#include "../gnn/nn_params.h"
#include "../gnn/galgs.h"
#include "../LNKnet_subs.h"

#define TRUE 1
#define FALSE 0
#define MAXSTR 256

static char *clustering_algorithm_name = "kmeans";
static char param_ftmp[MAXSTR];

static int clustering_active_flag;
static int clustering_algorithm = KMEANS;

extern girbf_popup1_objects	*Girbf_popup1;

void girbf_set_clustering_algorithm();
void girbf_show_clustering_algorithm();
void girbf_set_cost_func();
void girbf_set_cluster_param_fname();

void girbf_display_params()
{
    show_int_value (Girbf_popup1->clustering_active, clustering_active_flag);
    girbf_show_clustering_algorithm();
    show_string_label(Girbf_popup1->cluster_param_fname, cluster_param_fname);
    girbf_set_clustering_algorithm(clustering_algorithm);
    girbf_set_cluster_param_fname();
    show_int_value (Girbf_popup1->bias, bias);
    show_int_value(Girbf_popup1->grandvar, grand_flag);
    show_float_text (Girbf_popup1->min_var, minimum_variance);
    show_float_text (Girbf_popup1->weight_eta, weight_eta);
    show_float_text (Girbf_popup1->mean_eta, mean_eta);
    show_float_text (Girbf_popup1->var_eta, var_eta);
    show_float_text (Girbf_popup1->hspread, hspread);
    show_float_text (Girbf_popup1->extra_hspread, extra_hspread);
    show_float_text (Girbf_popup1->max_magnitude, max_magnitude);
    show_int_value(Girbf_popup1->epochs, nepochs);
    if(cost_func == SQ_ERR)
      show_int_value(Girbf_popup1->cost_func, 0);
    else if(cost_func == CFOM)
      show_int_value(Girbf_popup1->cost_func, 1);
    girbf_set_cost_func(cost_func);
    show_float_text(Girbf_popup1->cost_param, cost_param);
}

char * girbf_get_plot_label (string)
     char *string;
{
    sprintf (string,"%s H:%g Extra H:%g", string, hspread, extra_hspread);
    if (bias) sprintf (string, "%s Bias", string);
    sprintf(string, "%s Weight step:%.2g Epochs:%d", string,
	    weight_eta, nepochs);
    switch(clustering_algorithm){
      case KMEANS:
	string = gkmeans_get_plot_label(string);
	break;
      case EM_CLUS:
	string = gem_clus_get_plot_label(string);
	break;
      case LEAD_CLUS:
	string = glead_clus_get_plot_label(string);
	break;
      case RAN_CLUS:
	string = gran_clus_get_plot_label(string);
	break;
    }
    
    return string;
}

void girbf_init ()
{
    girbf_display_params();
}

/*
 * Notify callback function for irbf parameter window
 */
void
girbf_callback(item, event)
     Panel_item	item;
     Event	*event;/*ARGSUSED*/
{
    girbf_popup1_objects *ip =
      (girbf_popup1_objects *)xv_get(item, XV_KEY_DATA, INSTANCE);
    int ivalue;
    char *svalue;
    
    if(item == ip->clustering_active){
	clustering_active_flag = (int)xv_get(item, PANEL_VALUE);
    } else if (item == ip->clustering_algorithm){
	int temp;
	ivalue = (int)xv_get(item, PANEL_VALUE);
	switch(ivalue){
	  case 0:
	    girbf_set_clustering_algorithm(KMEANS); break;
	  case 1:
	    girbf_set_clustering_algorithm(EM_CLUS); break;
	  case 2:
	    girbf_set_clustering_algorithm(LEAD_CLUS); break;
	  case 3:
	    girbf_set_clustering_algorithm(RAN_CLUS); break;
	}
	temp = clustering_active_flag;
	clustering_active_flag = 1;
	girbf_set_cluster_param_fname();
	clustering_active_flag = temp;
    } else if (item == ip->clustering_parameters){
	void gkmeans_show_menu(), gem_clus_show_menu();
	void glead_clus_show_menu(), gran_clus_show_menu();

	switch(clustering_algorithm){
	  case KMEANS:
	    gkmeans_show_menu(); break;
	  case EM_CLUS:
	    gem_clus_show_menu(); break;
	  case LEAD_CLUS:
	    glead_clus_show_menu(); break;
	  case RAN_CLUS:
	    gran_clus_show_menu(); break;
	}
    } else if (item == ip->epochs){
	nepochs = (int)xv_get(item, PANEL_VALUE);
	set_plot_labels();
    } else if (item == ip->weight_eta){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	weight_eta =
	  set_atof_low_value(svalue, "weight_eta", Girbf_popup1->weight_eta,
			     weight_eta, 0.);
	set_plot_labels();
    } else if (item == ip->mean_eta){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	mean_eta =
	  set_atof_low_value(svalue, "mean_eta", Girbf_popup1->mean_eta,
			     mean_eta, 0.);
    } else if (item == ip->var_eta){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	var_eta =
	  set_atof_low_value(svalue, "var_eta", Girbf_popup1->var_eta,
			     var_eta, 0.);
    } else if (item == ip->hspread){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	hspread =
	  set_atof_low_value(svalue, "hspread", Girbf_popup1->hspread,
			     hspread, 0.);
	set_plot_labels();
    } else if (item == ip->extra_hspread){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	extra_hspread =
	  set_atof_low_value(svalue, "extra_hspread",
			     Girbf_popup1->extra_hspread, extra_hspread, 0.);
	set_plot_labels();
    } else if (item == ip->max_magnitude){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	max_magnitude =
	  set_atof_low_value(svalue, "max_magnitude",
			     Girbf_popup1->max_magnitude, max_magnitude, 0.);
    } else if (item == ip->min_var){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	minimum_variance =
	  set_atof_low_value(svalue, "minimum_variance",
			     Girbf_popup1->min_var, minimum_variance, 0.);
    } else if (item == ip->bias){
	bias = (int)xv_get(item, PANEL_VALUE);
	set_plot_labels();
    } else if (item == ip->cost_func){
	ivalue = (int)xv_get(item, PANEL_VALUE);
	if(ivalue == 1)
	  girbf_set_cost_func(CFOM);
	else
	  girbf_set_cost_func(ivalue);
    gnn_set_cost_func(cost_func);
    } else if (item == ip->cost_param){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	cost_param = set_atof_value(svalue, "cost_param",
				    Girbf_popup1->cost_param, cost_param);
    } else if (item == ip->grandvar){
	grand_flag = (int)xv_get(item, PANEL_VALUE);
    } else {
	warning("girbf_callback: unhandled interrupt");
    }
}

/*
 * Notify callback function for `clustering_active'.
 */
void
girbf_select_handler(item, value, event)
     Panel_item	item;
     int	value;/*ARGSUSED*/
     Event	*event;
{
    girbf_callback(item, event);
}

/*
 * Notify callback function for `clustering_parameters'.
 */
void
girbf_btn_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    girbf_callback(item, event);
}

/*
 * Notify callback function for `extra_hspread'.
 */
Panel_setting
girbf_text_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    girbf_callback(item, event);
    return panel_text_notify(item, event);
}

void
girbf_show_clustering_algorithm()
{
    switch(clustering_algorithm){
      case KMEANS:
	show_int_value(Girbf_popup1->clustering_algorithm, 0);
	break;
      case EM_CLUS:
	show_int_value(Girbf_popup1->clustering_algorithm, 1);
	break;
      case LEAD_CLUS:
	show_int_value(Girbf_popup1->clustering_algorithm, 2);
	break;
      case RAN_CLUS:
	show_int_value(Girbf_popup1->clustering_algorithm, 3);
	break;
    }
}

/* needed by girbf_set_description_values */
void girbf_set_clustering_algorithm (value)
     int value;
{
    clustering_algorithm = value;
    switch(value){
      case KMEANS:
	clustering_algorithm_name = "kmeans"; break;
      case EM_CLUS:
	clustering_algorithm_name = "em_clus"; break;
      case LEAD_CLUS:
	clustering_algorithm_name = "lead_clus"; break;
      case RAN_CLUS:
	clustering_algorithm_name = "ran_clus"; break;
      default: break;
    }
    /* cluster_param_fname still needs to be set */
    set_plot_labels();
}

void
girbf_set_cluster_param_fname()
{
    if(clustering_active_flag){
	strcpy(param_ftmp,nn_params.exp_prefix);
	strcat(param_ftmp,clustering_algorithm_name);
	cluster_param_fname = strcat(param_ftmp,".param");
	show_string_label(Girbf_popup1->cluster_param_fname,
			  cluster_param_fname);
    }
}

void girbf_store_plot_command (shell_file)
     FILE *shell_file;
{
    fprintf (shell_file, " -exhspread %g", extra_hspread);
}


char *
girbf_get_args ()
{
    return (nnargs_get_part_flags(irbf_tab, IRBF_TABSIZE));
}

char *
girbf_get_important_args ()
{
    return (nnargs_get_flags_one_line(irbf_tab, IRBF_TABSIZE));
}

char *girbf_get_cluster_fname(){
    return cluster_param_fname;
}

int girbf_get_cluster_alg(){
    return clustering_algorithm;
}

int girbf_must_store_clustering(){
    return clustering_active_flag;
}

static int clustering_active_arg = 0;
static int bias_arg = 0;
static int grand_arg = 0;
static ARG argtab2[]=
{
    {"clus_active", INT, &clustering_active_arg,
       "write cluster calls on create"},
    {"clus_alg", INT, &clustering_algorithm, "clustering algorithm number"},
    {"bias_arg", INT, &bias_arg, "add a bias node"},
    {"grand_arg", INT, &grand_arg, "grand variances"},
};
#define TABSIZE2 (sizeof(argtab2)/sizeof(ARG))

void
write_irbf_defaults(default_file)
     FILE *default_file;
{
    clustering_active_arg = clustering_active_flag;
    bias_arg = bias;
    grand_arg = grand_flag;
    if(default_file != NULL){
	fprintf (default_file,"irbf %s",
		 nnargs_get_part_flags(irbf_tab,IRBF_TABSIZE));
	fprintf (default_file, "%s", nnargs_get_flags(argtab2, TABSIZE2));
    }
}

void
read_irbf_defaults(argc, argv)
     int argc;
     char **argv;
{
    if(argc > 0){
	argc = nnargs_part(argc, argv, irbf_tab, IRBF_TABSIZE);
	argc = nnargs(argc, argv, argtab2, TABSIZE2);
    } else {
	clustering_active_arg = 1;
	nepochs = 10;
    }
    clustering_active_flag = clustering_active_arg;
    strcpy(param_ftmp, cluster_param_fname);
    cluster_param_fname = param_ftmp;
    bias = bias_arg;
    grand_flag = grand_arg;
    /* algorithm specific flags are now set.  generic flags are taken
       from the argument list for the main screen */
}

int girbf_get_epochs()
{
    return nepochs;
}

void girbf_set_epochs(e)
{
    nepochs = e;
}

void
girbf_set_cost_func(value)
     int value;
{
    cost_func = value;
    switch(value){
      case CFOM:
	gray_out_item(Girbf_popup1->cost_param, FALSE);
	break;
      default:
	gray_out_item(Girbf_popup1->cost_param, TRUE);
    }
}

int
girbf_get_cost_func()
{
    return cost_func;
}
