/***--------------------------------------------------------------------------
* NAME:		gkmeans.c
* PURPOSE:	Executes kmeans after setting command line parameters
*
* SEE ALSO:	gkmeans_subs.c
*
* DESCRIPTION:
*	gkmeans_display_params
*	gkmeans_get_plot_label
*	gkmeans_init
*		
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/gclass/kmeans/gkmeans.c,v 10.4 2001/08/23 20:13:35 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/gclass/kmeans/gkmeans.c,v $
-------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gclass/kmeans/gkmeans.c,v 10.4 2001/08/23 20:13:35 craig Exp $";



#define TRUE 1
#define FALSE 0
#define MAXSTR 256
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <unistd.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <gdd.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <nnlib.h>
#include "gkmeans_ui.h"
#include <kmeans_args.h>		/* nnargs definitions */
#include "../LNKnet_subs.h"
#include "../gnn/nn_params.h"

static int specify_by_class;
static char tmp_str[MAXSTR];

extern gkmeans_popup1_objects	*Gkmeans_popup1;

void gkmeans_set_common_flags();
void gkmeans_show_ncenters_choices();
void gkmeans_set_total_ncenters();
static int total_ncenters;

void
gkmeans_display_params()
{
    show_int_value (Gkmeans_popup1->cluster_by_class, cluster_by_class);
    show_int_text (Gkmeans_popup1->ncenters, ncenters);
    if(ncenters_list != NULL){
	show_string_text (Gkmeans_popup1->ncenters_list,
			  make_net_string(ncenters_list));
    }

    show_int_value (Gkmeans_popup1->reduce_step, reduce_step);
    show_float_text (Gkmeans_popup1->stop_percentage, stop_percentage);
    show_int_value (Gkmeans_popup1->max_iteration,max_iteration);
    show_float_text (Gkmeans_popup1->split_percentage, split_percentage);
    show_int_value (Gkmeans_popup1->add_random_offset, add_random_offset);

    show_int_value(Gkmeans_popup1->equal_ncenters, specify_by_class);
    gkmeans_show_ncenters_choices();
}

char * gkmeans_get_plot_label (string)
     char * string;
{
    sprintf (string, "%s Centers:%d", string, total_ncenters);

    return string;
}

void gkmeans_init ()
{
    add_random_offset = 1;
    gkmeans_display_params();
}
void
gkmeans_callback(item, event)
     Panel_item	item;
     Event	*event;
{
    gkmeans_popup1_objects *ip =
      (gkmeans_popup1_objects *)xv_get(item, XV_KEY_DATA, INSTANCE);
    int ivalue;
    char *svalue;

    if (item == ip->cluster_by_class){
	ivalue = (int)xv_get(item, PANEL_VALUE);
	cluster_by_class = ivalue;
	gkmeans_show_ncenters_choices();
	gkmeans_set_total_ncenters();
	set_plot_labels();
    } else if (item == ip->equal_ncenters){
	ivalue = (int)xv_get(item, PANEL_VALUE);
	specify_by_class = ivalue;
	gkmeans_show_ncenters_choices();
	gkmeans_set_total_ncenters();
	set_plot_labels();
    } else if (item == ip->ncenters){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	ncenters = set_atoi_value(svalue, "ncenters", item, ncenters);
	gkmeans_set_total_ncenters();
	set_plot_labels();
    } else if (item == ip->ncenters_list){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	if(ncenters_list != NULL) deallocate_ivector(ncenters_list);
	ncenters_list = nnargs_get_int_alloc_list (strcpy(tmp_str, svalue));
	gkmeans_set_total_ncenters();
	set_plot_labels();
    } else if (item == ip->reduce_step){
	ivalue = (int)xv_get(item, PANEL_VALUE);
	reduce_step = ivalue;
    } else if (item == ip->stop_percentage){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	stop_percentage =
	  set_atof_low_value(svalue, "stop_percentage",
			     Gkmeans_popup1->stop_percentage,
			     stop_percentage, 0.);
    } else if (item == ip->max_iteration){
	ivalue = (int)xv_get(item, PANEL_VALUE);
	max_iteration = ivalue;
    } else if (item == ip->split_percentage){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	split_percentage =
	  set_atof_low_value(svalue, "split_percentage",
			     Gkmeans_popup1->split_percentage,
			     split_percentage, 0.);
    } else if (item == ip->add_random_offset){
	ivalue = (int)xv_get(item, PANEL_VALUE);
	add_random_offset = ivalue;
    } else {
	fprintf(stderr, "LNKnet: gkmeans_handler: unhandled interupt\n");
    }
}

Panel_setting
gkmeans_text_handler(item, event)
     Panel_item item;
     Event	*event;
{
    gkmeans_callback(item, event);
    return panel_text_notify(item, event);
}

void
gkmeans_select_handler(item, value, event)
     Panel_item	item;
     int	value;/*ARGSUSED*/
     Event	*event;
{
    gkmeans_callback(item, event);
}

void
gkmeans_show_ncenters_choices()
{
    if(cluster_by_class){
	gray_out_item(Gkmeans_popup1->equal_ncenters, FALSE);
	if(specify_by_class){
	    gray_out_item(Gkmeans_popup1->ncenters_list, FALSE);
	    gray_out_item(Gkmeans_popup1->ncenters, TRUE);
	} else {
	    gray_out_item(Gkmeans_popup1->ncenters_list, TRUE);
	    gray_out_item(Gkmeans_popup1->ncenters, FALSE);
	}
    } else {
	gray_out_item(Gkmeans_popup1->equal_ncenters, TRUE);
	gray_out_item(Gkmeans_popup1->ncenters_list, TRUE);
	gray_out_item(Gkmeans_popup1->ncenters, FALSE);
    }
}

void
gkmeans_set_total_ncenters()
{
    static char total_label[64];
    if(cluster_by_class){
	if(specify_by_class && ncenters_list != NULL){
	    int i;
	    for(i = 1, total_ncenters = 0; i <= ncenters_list[0]; i++){
		total_ncenters += ncenters_list[i];
	    }
	} else {
	    total_ncenters = nn_params.noutputs * ncenters;
	}
    } else {
	total_ncenters = ncenters;
    }
    sprintf(total_label, "This will create a total of %d centers.", total_ncenters);
    show_string_label(Gkmeans_popup1->total_ncenters_msg, total_label);
}

void
gkmeans_show_menu()
{
    open_popup (Gkmeans_popup1->popup1);
}

char *
gkmeans_get_args ()
{
    char * ret_str;
    int tmp_ncenters;

    /* ncenters_list will take precedence if it is there.
       making it appear to be length 0 will prevent it from
       being written out as an argument when it is not needed */
    if(!specify_by_class && ncenters_list != NULL){
	tmp_ncenters = ncenters_list[0];
	ncenters_list[0] = 0;
    }
    ret_str = nnargs_get_part_flags(kmeans_tab, KMEANS_TABSIZE);

    if(!specify_by_class && ncenters_list != NULL){
	ncenters_list[0] = tmp_ncenters;
    }
    return (ret_str);
}

char *
gkmeans_get_important_args ()
{
    char * ret_str;
    int tmp_ncenters;

    /* ncenters_list will take precedence if it is there.
       making it appear to be length 0 will prevent it from
       being written out as an argument when it is not needed */
    if(!specify_by_class && ncenters_list != NULL){
	tmp_ncenters = ncenters_list[0];
	ncenters_list[0] = 0;
    }
    ret_str = nnargs_get_flags_one_line(kmeans_tab, KMEANS_TABSIZE);

    if(!specify_by_class && ncenters_list != NULL){
	ncenters_list[0] = tmp_ncenters;
    }
    return (ret_str);
}

static int specify_by_class_arg = 0;
static int cluster_by_class_arg = 0;
static int add_offset_arg = 0;
static int *ncenters_list_arg = NULL;

static ARG argtab2[]=
{
    {"specify_by_class", INT, &specify_by_class_arg,
       "specify centers by class"},
    {"cluster_by_class_flag", INT, &cluster_by_class_arg,
       "cluster by class"},
    {"add_offset_arg", INT, &add_offset_arg,
       "dummy for add_random_offset"},
    {"ncenters_list_arg", INT_ALLOC_LIST, (int *)&ncenters_list_arg},
};
#define TABSIZE2 (sizeof(argtab2)/sizeof(ARG))

void
write_kmeans_defaults(default_file)
     FILE *default_file;
{
    specify_by_class_arg = specify_by_class;
    cluster_by_class_arg = cluster_by_class;
    add_offset_arg = add_random_offset;
    if(ncenters_list_arg != NULL) deallocate_ivector(ncenters_list_arg);
    strcpy(tmp_str,(char *)xv_get(Gkmeans_popup1->ncenters_list, PANEL_VALUE));
    ncenters_list_arg = nnargs_get_int_alloc_list(tmp_str);
    if(default_file != NULL){
	fprintf (default_file, "kmeans %s",
		 nnargs_get_part_flags(kmeans_tab, KMEANS_TABSIZE));
	fprintf (default_file, "%s", nnargs_get_flags(argtab2, TABSIZE2));
    }
}

void
read_kmeans_defaults(argc, argv)
     int argc;
     char **argv;
{
    int i;
    if(argc > 0){
	argc = nnargs_part(argc, argv, kmeans_tab, KMEANS_TABSIZE);
	argc = nnargs(argc, argv, argtab2, TABSIZE2);
    } else {
	add_offset_arg = 1;
    }
    specify_by_class = specify_by_class_arg;
    cluster_by_class = cluster_by_class_arg;
    add_random_offset = add_offset_arg;
    if(ncenters_list_arg != NULL){
	if(ncenters_list != NULL) deallocate_ivector(ncenters_list);
	ncenters_list = allocate_ivector(ncenters_list_arg[0]+1);
	for(i = 0; i <= ncenters_list_arg[0] && i < NNARGS_MAX_LABELS; i++)
	  ncenters_list[i] = ncenters_list_arg[i];
    }
    /* algorithm specific flags are now set.  generic flags are taken
       from the argument list for the main screen */
}
