/***--------------------------------------------------------------------------
* NAME:        gknn.c
* PURPOSE:     K Nearest Neighbor main program driver (DEVGUIDE version)
*		Executes knn after setting command line parameters
*
* USAGE:	gknn	(from command line - produces control panel window)
*
* SEE ALSO:	knn.c knn_pack.c knn_subs.c gknn_subs.c c_subs.c
*
* DESCRIPTION: 
*	gknn_display_params
*	gknn_init
*	gknn_get_plot_label
*	gknn_set_k
*	gknn_store_plot_command
*
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/gclass/knn/gknn.c,v 10.4 2001/08/23 20:13:37 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/gclass/knn/gknn.c,v $
* created 23-Jan-91 lck
-------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gclass/knn/gknn.c,v 10.4 2001/08/23 20:13:37 craig Exp $";



#define TRUE 1
#define FALSE 0

#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>

#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <sys/ioctl.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/icon_load.h>
#include <xview/panel.h>
#include <xview/scrollbar.h>
#include <xview/svrimage.h>
#include <xview/termsw.h>
#include <xview/text.h>
#include <xview/tty.h>
#include <xview/xv_xrect.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <gcm.h>
#include <gdd.h>
#include <nnlib.h>
#include "gknn_ui.h"
#include <knn_args.h>	/* statics for knn args */
#include "../LNKnet_subs.h"
#include "../gnn/nn_params.h"

/* Needed window pointers */
extern gknn_param_pop_objects	*Gknn_param_pop;

/* statics for file name parts */
#define MAXSTR 256

/* routines found in this file */
void gknn_display_params();
void gknn_init ();
void gknn_set_k ();
void gknn_store_plot_command();

void gknn_display_params()
{
    show_int_value (Gknn_param_pop->k, k);
    show_int_value (Gknn_param_pop->leave_1_out, cvtest_flag);
}

void gknn_init ()
{
    gknn_display_params();
}

char * gknn_get_plot_label(string)
     char * string;
{
    /* add to existing label */
    sprintf (string, "%s K:%d", string, k);

    return string;
}

/*
 * Notify callback function for `k'.
 */
Panel_setting
gknn_k_handler(item, event)
	Panel_item	item;
	Event		*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);
	
    k = value;
    set_plot_labels();
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `leave_1_out'.
 */
void
gknn_leave_1_out_handler(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;/*ARGSUSED*/
{
    if(value) cvtest_flag = 1;
    else cvtest_flag = 0;
}

void
gknn_store_plot_command (shell_file)
     FILE *shell_file;
{
    fprintf(shell_file, "\\\n -k %d", k);
}

char *
gknn_get_args()
{
    return (nnargs_get_part_flags(knn_tab, KNN_TABSIZE));
}

char *
gknn_get_important_args()
{
    return (nnargs_get_flags_one_line(knn_tab, KNN_TABSIZE));
}

static int cvtest_arg = 0;
static ARG argtab2[]=
{
    {"cvtest_arg", INT, &cvtest_arg, "dummy for cvtest_flag"},
};
#define TABSIZE2 (sizeof(argtab2)/sizeof(ARG))

void
write_knn_defaults(default_file)
     FILE *default_file;
{
    cvtest_arg = cvtest_flag;
    if(default_file != NULL){
	fprintf(default_file,"knn %s",
		nnargs_get_part_flags(knn_tab,KNN_TABSIZE));
	fprintf(default_file,"%s",nnargs_get_flags(argtab2, TABSIZE2));
    }
}

void
read_knn_defaults(argc, argv)
     int argc;
     char **argv;
{
    if(argc > 0){
	argc = nnargs_part(argc, argv, knn_tab, KNN_TABSIZE);
	argc = nnargs(argc, argv, argtab2, TABSIZE2);
    }
    cvtest_flag = cvtest_arg;

    /* algorithm specific flags are now set.  generic flags are taken
       from the argument list for the main screen */
}
