/***--------------------------------------------------------------------------
* NAME:		glead_clus.c
* PURPOSE:	Executes lead_clus after setting command line parameters
*
* SEE ALSO:	glead_clus_subs.c
*
* DESCRIPTION:
*	glead_clus_display_params
*	glead_clus_get_plot_label
*	glead_clus_init
*		
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/gclass/lead_clus/glead_clus.c,v 10.4 2001/08/23 20:13:39 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/gclass/lead_clus/glead_clus.c,v $
-------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gclass/lead_clus/glead_clus.c,v 10.4 2001/08/23 20:13:39 craig Exp $";



#define TRUE 1
#define FALSE 0
#define MAXSTR 256
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <unistd.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <gdd.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <nnlib.h>
#include "glead_clus_ui.h"
#include <lead_clus_args.h>		/* nnargs arguments */
#include "../gnn/nn_params.h"
#include "../LNKnet_subs.h"

static int specify_by_class;
static char tmp_str[MAXSTR];

extern glead_clus_popup1_objects	*Glead_clus_popup1;

void glead_clus_show_delta_choices();

void glead_clus_display_params()
{
    show_int_value (Glead_clus_popup1->cluster_by_class, cluster_by_class);
    show_float_text (Glead_clus_popup1->delta, delta);
    if(delta_list != NULL){
	show_string_text (Glead_clus_popup1->delta_list,
			  make_float_string(delta_list));
    }
    show_int_value(Glead_clus_popup1->equal_delta, specify_by_class);
    glead_clus_show_delta_choices();
}

char * glead_clus_get_plot_label (string)
     char *string;
{
    sprintf (string, "%s Centers Delta:%.2f", string, delta);

    return string;
}

void glead_clus_init ()
{
    glead_clus_display_params();
}

void
glead_clus_callback(item, event)
     Panel_item	item;
     Event	*event;
{
    glead_clus_popup1_objects *ip =
      (glead_clus_popup1_objects *)xv_get(item, XV_KEY_DATA, INSTANCE);
    int ivalue;
    char *svalue;

    if(item == ip->cluster_by_class){
	ivalue = (int) xv_get(item, PANEL_VALUE);
	cluster_by_class = ivalue;
	glead_clus_show_delta_choices();
    } else if(item == ip->equal_delta){
	ivalue = (int) xv_get(item, PANEL_VALUE);
	specify_by_class = ivalue;
	glead_clus_show_delta_choices();
    } else if (item == ip->delta){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	delta = set_atof_low_value(svalue, "delta", Glead_clus_popup1->delta,
				   delta, 0.);
	set_plot_labels();
    } else if (item == ip->delta_list){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	if(delta_list != NULL) deallocate_vector(delta_list);
	delta_list = nnargs_get_float_alloc_list(strcpy(tmp_str, svalue));
    } else {
	fprintf(stderr, "LNKnet: glead_clus_callback: unhandled item\n");
    }
}

/*
 * Notify callback function for `delta'.
 */
Panel_setting
glead_clus_text_handler(item, event)
	Panel_item	item;
	Event		*event;
{
    glead_clus_callback(item, event);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for choices
 */
void
glead_clus_select_handler(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;/*ARGSUSED*/
{
    glead_clus_callback(item, event);
}

void
glead_clus_show_delta_choices()
{
    if(cluster_by_class){
	gray_out_item(Glead_clus_popup1->equal_delta, FALSE);
	if(specify_by_class){
	    gray_out_item(Glead_clus_popup1->delta_list, FALSE);
	    gray_out_item(Glead_clus_popup1->delta, TRUE);
	} else {
	    gray_out_item(Glead_clus_popup1->delta_list, TRUE);
	    gray_out_item(Glead_clus_popup1->delta, FALSE);
	}
    } else {
	gray_out_item(Glead_clus_popup1->equal_delta, TRUE);
	gray_out_item(Glead_clus_popup1->delta_list, TRUE);
	gray_out_item(Glead_clus_popup1->delta, FALSE);
    }
}
	    
void
glead_clus_show_menu()
{
    open_popup (Glead_clus_popup1->popup1);
}

char *
glead_clus_get_args ()
{
    char * ret_str;
    int tmp_ndeltas;

    /* delta_list will take precedence if it is there.
       making it appear to be length 0 will prevent it from
       being written out as an argument when it is not needed */
    if(!specify_by_class && delta_list != NULL){
	tmp_ndeltas = delta_list[0];
	delta_list[0] = 0.;
    }

    ret_str = nnargs_get_part_flags(lead_clus_tab, LEAD_CLUS_TABSIZE);

    if(!specify_by_class && delta_list != NULL){
	delta_list[0] = tmp_ndeltas;
    }

    return (ret_str);
}

char *
glead_clus_get_important_args ()
{
    char * ret_str;
    int tmp_ndeltas;

    /* delta_list will take precedence if it is there.
       making it appear to be length 0 will prevent it from
       being written out as an argument when it is not needed */
    if(!specify_by_class && delta_list != NULL){
	tmp_ndeltas = delta_list[0];
	delta_list[0] = 0.;
    }

    ret_str = nnargs_get_flags_one_line(lead_clus_tab, LEAD_CLUS_TABSIZE);

    if(!specify_by_class && delta_list != NULL){
	delta_list[0] = tmp_ndeltas;
    }

    return (ret_str);
}

static int specify_by_class_arg = 0;
static int cluster_by_class_arg = 0;
static float *delta_list_arg = NULL;
static ARG argtab2[]=
{
    {"specify_by_class", INT, &specify_by_class_arg,
       "specify centers by class"},
    {"cluster_by_class_flag", INT, &cluster_by_class_arg,
       "cluster by class"},
    {"delta_list_arg", FLOAT_ALLOC_LIST, (int *)&delta_list_arg},
};
#define TABSIZE2 (sizeof(argtab2)/sizeof(ARG))

void write_lead_clus_defaults(default_file)
     FILE *default_file;
{
    specify_by_class_arg = specify_by_class;
    cluster_by_class_arg = cluster_by_class;
    if(delta_list_arg != NULL) deallocate_vector(delta_list_arg);
    strcpy(tmp_str, (char *)xv_get(Glead_clus_popup1->delta_list,
						 PANEL_VALUE));
    delta_list_arg = nnargs_get_float_alloc_list(tmp_str);
    if(default_file != NULL){
	fprintf (default_file, "lead_clus %s",
		 nnargs_get_part_flags(lead_clus_tab, LEAD_CLUS_TABSIZE));
	fprintf (default_file, "%s", nnargs_get_flags(argtab2, TABSIZE2));
    }
}

void
read_lead_clus_defaults(argc, argv)
     int argc;
     char **argv;
{
    int i;

    if(argc > 0){
	argc = nnargs_part(argc, argv, lead_clus_tab, LEAD_CLUS_TABSIZE);
	argc = nnargs(argc, argv, argtab2, TABSIZE2);
    }
    specify_by_class = specify_by_class_arg;
    cluster_by_class = cluster_by_class_arg;
    if(delta_list_arg != NULL){
	if(delta_list != NULL) deallocate_vector(delta_list);
	delta_list = allocate_vector((int)delta_list_arg[0]+1);
	for(i = 0; i <= delta_list_arg[0] && i < NNARGS_MAX_LABELS; i++)
	  delta_list[i] = delta_list_arg[i];
    }
    /* algorithm specific flags are now set.  generic flags are taken
       from the argument list for the main screen */
}
