/***--------------------------------------------------------------------------
* NAME:        glvq.c
* PURPOSE:     Parameter window for Learning Vector Quantizer
*
* USAGE:	gnn	(from command line - produces control panel window)
*
* SEE ALSO:	gnn_main.c gnn.c gnn_subs.c lvq_main.c lvq_pack.c
*
* DESCRIPTION:
*	
*	glvq_display_params
*	glvq_get_plot_label
*	glvq_init
*		
* BUGS AND LIMITATIONS:
*
*
-------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gclass/lvq/glvq.c,v 10.4 2001/08/23 20:13:41 craig Exp $";



#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <unistd.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <gdd.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <nnlib.h>
#include "glvq_ui.h"
#include <lvq_args.h>			/* nnargs definitions */
#include "../gnn/nn_params.h"
#include "../gnn/galgs.h"
#include "../LNKnet_subs.h"

#define TRUE 1
#define FALSE 0
#define MAXSTR 256

#define OLVQ 0
#define LVQ1 1
#define LVQ2 2
#define LVQ3 3

static char *clustering_algorithm_name = "kmeans";
static char param_ftmp[MAXSTR];

static int clustering_active_flag;
static int clustering_algorithm = KMEANS;

extern glvq_popup1_objects	*Glvq_popup1;

void glvq_set_clustering_algorithm();
void glvq_set_cluster_param_fname();
void glvq_set_lvq_type_graying();
void glvq_show_clustering_algorithm();
void glvq_set_help_text();

void
glvq_display_params()
{
    show_int_value (Glvq_popup1->clustering_active, clustering_active_flag);
    glvq_show_clustering_algorithm();
    show_string_label(Glvq_popup1->cluster_param_fname, cluster_param_fname);
    glvq_set_clustering_algorithm(clustering_algorithm);
    glvq_set_cluster_param_fname();
    show_int_value (Glvq_popup1->lvq_type, lvq_type);
    glvq_set_lvq_type_graying();
    glvq_set_help_text();
    show_float_text (Glvq_popup1->alpha, alpha);
    show_float_text (Glvq_popup1->window, window);
    show_float_text (Glvq_popup1->epsilon, epsilon);
    show_int_value (Glvq_popup1->n_epochs, nepochs);
}

char *
glvq_get_plot_label (string)
     char *string;
{
    sprintf(string, "%s %s Stepsize:%.2g Epochs:%d", string,
	    (lvq_type == OLVQ) ? "OLVQ" :
	    (lvq_type == LVQ1) ? "LVQ1" :
	    (lvq_type == LVQ2) ? "LVQ2" : "LVQ3",
	    alpha, nepochs);
    switch(clustering_algorithm){
      case KMEANS:
	string = gkmeans_get_plot_label(string);
	break;
      case EM_CLUS:
	string = gem_clus_get_plot_label(string);
	break;
      case LEAD_CLUS:
	string = glead_clus_get_plot_label(string);
	break;
      case RAN_CLUS:
	string = gran_clus_get_plot_label(string);
	break;
    }
    return string;
}

void
glvq_init ()
{
    glvq_display_params();
}

/*
 * Notify callback function for the lvq parameter window.
 */
void
glvq_callback(item, event)
     Panel_item	item;
     Event	*event;/*ARGSUSED*/
{
    glvq_popup1_objects *ip =
      (glvq_popup1_objects *)xv_get(item, XV_KEY_DATA, INSTANCE);
    int ivalue;
    char *svalue;

    if(item == ip->clustering_active){
	clustering_active_flag = (int)xv_get(item, PANEL_VALUE);
    } else if (item == ip->clustering_algorithm){
	int temp;
	ivalue = (int)xv_get(item, PANEL_VALUE);
	switch(ivalue){
	  case 0:
	    glvq_set_clustering_algorithm(KMEANS); break;
	  case 1:
	    glvq_set_clustering_algorithm(EM_CLUS); break;
	  case 2:
	    glvq_set_clustering_algorithm(LEAD_CLUS); break;
	  case 3:
	    glvq_set_clustering_algorithm(RAN_CLUS); break;
	}
	temp = clustering_active_flag;
	clustering_active_flag = 1;
	glvq_set_cluster_param_fname();
	clustering_active_flag = temp;
    } else if (item == ip->clustering_parameters){
	void gkmeans_show_menu(), gem_clus_show_menu();
	void glead_clus_show_menu(), gran_clus_show_menu();

	switch(clustering_algorithm){
	  case KMEANS:
	    gkmeans_show_menu(); break;
	  case EM_CLUS:
	    gem_clus_show_menu(); break;
	  case LEAD_CLUS:
	    glead_clus_show_menu(); break;
	  case RAN_CLUS:
	    gran_clus_show_menu(); break;
	}
    } else if (item == ip->lvq_type){
	lvq_type = (int)xv_get(item, PANEL_VALUE);
	glvq_set_lvq_type_graying();
	set_plot_labels();
	glvq_set_help_text();
    } else if (item == ip->alpha){
	char *	svalue = (char *) xv_get(item, PANEL_VALUE);
	alpha = set_atof_range_value(svalue, "alpha",
				     Glvq_popup1->alpha, alpha, 0., 1.);
	set_plot_labels();
    } else if (item == ip->n_epochs){
	nepochs = (int) xv_get(item, PANEL_VALUE);
	set_plot_labels();
    } else if (item == ip->window){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	window = set_atof_range_value(svalue, "window", Glvq_popup1->window,
				      window, 0., 1.);
    } else if (item == ip->epsilon){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	epsilon = set_atof_range_value(svalue, "epsilon", Glvq_popup1->epsilon,
				       epsilon, 0., 1.);
    } else {
	fprintf(stderr, "LNKnet: glvq_callback: unhandled interrupt\n");
    }
}


/*
 * Notify callback function for lvq settings (choices).
 */
void
glvq_select_handler(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;/*ARGSUSED*/
{
    glvq_callback(item, event);
}

/*
 * Notify callback function for lvq buttons.
 */
void
glvq_btn_handler(item, event)
     Panel_item	item;
     Event	*event;/*ARGSUSED*/
{
    glvq_callback(item, event);
}

/*
 * Notify callback function for text widgets
 */
Panel_setting
glvq_text_handler(item, event)
     Panel_item	item;
     Event		*event;
{
    glvq_callback(item, event);
    return panel_text_notify(item, event);
}

void
glvq_show_clustering_algorithm()
{
    switch(clustering_algorithm){
      case KMEANS:
	show_int_value(Glvq_popup1->clustering_algorithm, 0);
	break;
      case EM_CLUS:
	show_int_value(Glvq_popup1->clustering_algorithm, 1);
	break;
      case LEAD_CLUS:
	show_int_value(Glvq_popup1->clustering_algorithm, 2);
	break;
      case RAN_CLUS:
	show_int_value(Glvq_popup1->clustering_algorithm, 3);
	break;
    }
}

/* needed by glvq_set_description_values */
void
glvq_set_clustering_algorithm (value)
     int value;
{
    clustering_algorithm = value;
    switch(value){
      case KMEANS:
	clustering_algorithm_name = "kmeans";
	break;
      case EM_CLUS:
	clustering_algorithm_name = "em_clus";
	break;
      case LEAD_CLUS:
	clustering_algorithm_name = "lead_clus";
	break;
      case RAN_CLUS:
	clustering_algorithm_name = "ran_clus";
      default:
	break;
    }
    set_plot_labels();
}

void
glvq_set_cluster_param_fname()
{
    if(clustering_active_flag){
	strcpy(param_ftmp,nn_params.exp_prefix);
	strcat(param_ftmp,clustering_algorithm_name);
	cluster_param_fname = strcat(param_ftmp,".param");
	show_string_label(Glvq_popup1->cluster_param_fname,
			  cluster_param_fname);
    }
}

void
glvq_set_lvq_type_graying(){
    switch(lvq_type){
      case OLVQ:
      case LVQ1:
	gray_out_item(Glvq_popup1->window, TRUE);
	gray_out_item(Glvq_popup1->epsilon, TRUE);
	break;
      case LVQ2:
	gray_out_item(Glvq_popup1->window, FALSE);
	gray_out_item(Glvq_popup1->epsilon, TRUE);
	break;
      case LVQ3:
	gray_out_item(Glvq_popup1->window, FALSE);
	gray_out_item(Glvq_popup1->epsilon, FALSE);
	break;
    }
}
void
glvq_set_help_text()
{
    switch(lvq_type){
      case OLVQ:
	show_string_label(Glvq_popup1->alg_help1,
			  "Closest center moves toward pattern if classes match,");
	show_string_label(Glvq_popup1->alg_help2, "away from pattern otherwise.");
	show_string_label(Glvq_popup1->alg_help3, "Each center has its own stepsize");
	show_string_label(Glvq_popup1->alg_help4, "");
	break;
      case LVQ1:
	show_string_label(Glvq_popup1->alg_help1,
			  "Closest center moves toward pattern if classes match,");
	show_string_label(Glvq_popup1->alg_help2, "away from pattern otherwise.");
	show_string_label(Glvq_popup1->alg_help3, "Stepsize decreases to 0 over N epochs");
	show_string_label(Glvq_popup1->alg_help4, "");
	break;
      case LVQ2:
	show_string_label(Glvq_popup1->alg_help1,
			  "Move the two closest centers if one matches");
	show_string_label(Glvq_popup1->alg_help2,
			  "the pattern's class and the other doesn't");
	show_string_label(Glvq_popup1->alg_help3,
			  "and the distance ratio is inside the window.");
	show_string_label(Glvq_popup1->alg_help4,
			  "Larger windows let the distance ratio be larger.");
	break;
      case LVQ3:
	show_string_label(Glvq_popup1->alg_help1,
			  "Move the two closest centers if one matches");
	show_string_label(Glvq_popup1->alg_help2,
			  "the pattern's class and the other doesn't");
	show_string_label(Glvq_popup1->alg_help3,
			  "and the distance ratio is inside a window.");
	show_string_label(Glvq_popup1->alg_help4,
			  "If both match the class, move with scaled stepsize.");
	break;
    }
}

int
glvq_get_epochs()
{
    return nepochs;
}

void
glvq_set_epochs(e)
     int e;
{
    /* popup will not reflect the set number of epochs */
    nepochs = e;
}

char *
glvq_get_args ()
{
    return (nnargs_get_part_flags(lvq_tab, LVQ_TABSIZE));
}

char *
glvq_get_important_args ()
{
    return (nnargs_get_flags_one_line(lvq_tab, LVQ_TABSIZE));
}

char *
glvq_get_cluster_fname(){
    return cluster_param_fname;
}

int
glvq_get_cluster_alg(){
    return clustering_algorithm;
}

int
glvq_must_store_clustering(){
    return clustering_active_flag;
}

static int clustering_active_arg = 0;
static ARG argtab2[]=
{
    {"clus_active", INT, &clustering_active_arg,
       "write cluster calls on create"},
    {"clus_alg", INT, &clustering_algorithm, "clustering algorithm number"},
};
#define TABSIZE2 (sizeof(argtab2)/sizeof(ARG))

void
write_lvq_defaults(default_file)
     FILE *default_file;
{
    clustering_active_arg = clustering_active_flag;
    if(default_file != NULL){
	fprintf (default_file,"lvq %s",
		 nnargs_get_part_flags(lvq_tab,LVQ_TABSIZE));
	fprintf (default_file, "%s", nnargs_get_flags(argtab2, TABSIZE2));
    }
}

void
read_lvq_defaults(argc, argv)
     int argc;
     char **argv;
{
    if(argc > 0){
	argc = nnargs_part(argc, argv, lvq_tab, LVQ_TABSIZE);
	argc = nnargs(argc, argv, argtab2, TABSIZE2);
    } else {
	clustering_active_arg = 1;
	nepochs = 10;
    }
    clustering_active_flag = clustering_active_arg;
    strcpy(param_ftmp, cluster_param_fname);
    cluster_param_fname = param_ftmp;
    /* algorithm specific flags are now set.  generic flags are taken
       from the argument list for the main screen */
}
