/***------------------------------------------------------------------------
 * NAME:        gmlp.c
 * PURPOSE:     Multilayer Perceptron driver (DEVGUIDE version)
 *		Executes mlp functions after setting command line parameters
 *
 * USAGE:	gnn	(from command line - produces control panel window)
 *
 * SEE ALSO:	gmain.c g.c gmlp_subs.c
 *
 * DESCRIPTION:
 *	
 *	gmlp_display_params
 *	gmlp_get_plot_label
 *	gmlp_init
 *	gmlp_set_description_values
 *	gmlp_set_epochs
 *	gmlp_set_weight_update_mode
 *		
 * BUGS AND LIMITATIONS:
 *
 -----------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gclass/mlp/gmlp.c,v 10.4 2001/08/23 20:13:43 craig Exp $";



#define TRUE 1
#define FALSE 0
#define MAXSTR 256

#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <unistd.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <gdd.h>
#include <signal.h>
#include <xview/textsw.h>
#include <sys/ioctl.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <nnlib.h>
#include "gmlp_ui.h"
#include <mlp_args.h>
#include "../gnn/nn_params.h"
#include "../LNKnet_subs.h"

/* static GLOBALS internal to this file */
extern gmlp_popup1_objects	*Gmlp_popup1;
extern gmlp_weight_popup_objects *Gmlp_weight_popup;
extern gmlp_cost_popup_objects *Gmlp_cost_popup;
extern gmlp_node_popup_objects *Gmlp_node_popup;
static int weight_update_mode = 0;

void gmlp_display_params();
char *gmlp_get_plot_label();
void gmlp_init();
Panel_setting gmlp_main_text_handler();
void gmlp_main_btn_handler();
void gmlp_weight_select_handler();
Panel_setting gmlp_weight_text_handler();
void gmlp_node_select_handler();
Panel_setting gmlp_node_text_handler();
Panel_setting gmlp_cost_text_handler();
void gmlp_cost_select_handler();
Panel_setting gmlp_batch_handler(); /* not used!! */
void gmlp_set_description_values();
int gmlp_get_epochs();
void gmlp_set_epochs();
void gmlp_set_cost_func();
int gmlp_get_cost_func();
void gmlp_set_etta_change();
void gmlp_set_weight_update_mode();
void gmlp_set_ofunction();
char *gmlp_get_args();
void write_mlp_defaults();
void read_mlp_defaults();

void gmlp_display_params()
{
    char *make_net_string();
    
    show_int_text(Gmlp_popup1->epochs, nepochs);
    show_string_text (Gmlp_popup1->layers, make_net_string(nnodes));
    show_float_text (Gmlp_popup1->etta, etta);

    show_float_text (Gmlp_weight_popup->decay, decay);
    show_float_text (Gmlp_weight_popup->momentum, alpha);
    show_int_value (Gmlp_weight_popup->etta_change, step_change);
    gmlp_set_etta_change(step_change);
    show_float_text (Gmlp_weight_popup->etta_nepochs, etta_nepochs);
    show_float_text (Gmlp_weight_popup->kappa, kappa);
    show_float_text (Gmlp_weight_popup->epsilon, epsilon);
    show_int_value (Gmlp_weight_popup->use_etta_list, use_etta_list);
    if(use_etta_list)
      gray_out_item(Gmlp_weight_popup->etta_list, FALSE);
    else
      gray_out_item(Gmlp_weight_popup->etta_list, TRUE);
    show_string_text(Gmlp_weight_popup->etta_list,
		     make_float_string(etta_list));
    show_float_text (Gmlp_weight_popup->tolerance, tolerance);
    show_float_text(Gmlp_weight_popup->max_weight, max_magnitude);
    show_int_value(Gmlp_weight_popup->weight_update, weight_update_mode);

    switch(ofunction){
      case 0:
      case 1:
	show_int_value(Gmlp_node_popup->ofunction, ofunction);
	break;
      case LINEAR:
	show_int_value(Gmlp_node_popup->ofunction, 2);
	break;
      case STEP:
	/* do not display */
	break;
    }
    switch(hfunction){
      case 0:
      case 1:
	show_int_value(Gmlp_node_popup->hfunction, hfunction);
	break;
      case LINEAR:
      case STEP:
	/* do not display */
	break;
    }
    show_float_text(Gmlp_node_popup->sigmoid_param, sigmoid_param);
    show_int_value (Gmlp_node_popup->use_sigmoid_list, use_sigmoid_param_list);
    if(use_sigmoid_param_list)
      gray_out_item(Gmlp_node_popup->sigmoid_list, FALSE);
    else
      gray_out_item(Gmlp_node_popup->sigmoid_list, TRUE);
    show_string_text(Gmlp_node_popup->sigmoid_list,
		     make_float_string(sigmoid_param_list));

    show_int_value (Gmlp_cost_popup->cost_func, cost_func);
    gmlp_set_cost_func(cost_func);
    show_float_text(Gmlp_cost_popup->cost_param, cost_param);
}

char *
gmlp_get_plot_label (string)
     char *string;
{
    /* add to existing label */
    sprintf (string, "%s Net:%s Step:%g", string,
	     make_net_string(nnodes), etta);
    
    return string;
}

void
gmlp_init ()
{
    gmlp_display_params();
    gmlp_set_weight_update_mode (weight_update_mode);
}

Panel_setting
gmlp_main_text_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    gmlp_popup1_objects *ip =
      (gmlp_popup1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
    char *value = (char *)xv_get(item, PANEL_VALUE);

    if(item == ip->epochs){
	nepochs = set_atoi_value(value, "epochs", item, nepochs);
    } else if (item == ip->layers){
	char tmp[80];
    
	nnargs_get_int_list (strcpy(tmp,value), nnodes,NNODES_SIZE);
	if(nnodes[0] < 2) nnodes[0] = 2;
	nnodes[1] = nn_params.ninputs;;
	nnodes[nnodes[0]] = nn_params.noutputs;
	show_string_text (Gmlp_popup1->layers, make_net_string(nnodes));
	
	set_plot_labels();
    } else if (item == ip->etta){
	etta = set_atof_low_value(value, "etta", Gmlp_popup1->etta, etta, 0.);
	set_plot_labels();
    } else {
	fprintf(stderr,
		"LNKnet: gmlp_main_text_handler: unhandled text value: %s\n",
		value);
    }
    return panel_text_notify(item, event);
    
}

void
gmlp_main_btn_handler(item, event)
     Panel_item item;
     Event	*event;/*ARGSUSED*/
{
    gmlp_popup1_objects *ip =
      (gmlp_popup1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);

    if(item == ip->weight_button){
	open_popup(Gmlp_weight_popup->weight_popup);
    } else if(item == ip->node_button){
	open_popup(Gmlp_node_popup->node_popup);
    } else if(item == ip->cost_button){
	open_popup(Gmlp_cost_popup->cost_popup);
    } else {
	fprintf(stderr, "LNKnet: gmlp_main_btn_handler: unhandled btn\n");
    }
}

void
gmlp_weight_select_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;/*ARGSUSED*/
{
    gmlp_weight_popup_objects *ip =
      (gmlp_weight_popup_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);

    if(item == ip->etta_change){
	gmlp_set_etta_change(value);
    } else if (item == ip->use_etta_list){
	use_etta_list = value;
	if(value)
	  gray_out_item(Gmlp_weight_popup->etta_list, FALSE);
	else
	  gray_out_item(Gmlp_weight_popup->etta_list, TRUE);
    } else if (item == ip->weight_update){
	gmlp_set_weight_update_mode (value);
    } else {
	fprintf(stderr,
		"LNKnet: gmlp_weight_select_handler: unhandled: value: %u\n",
		value);
    }
}

Panel_setting
gmlp_weight_text_handler(item, event)
	Panel_item	item;
	Event		*event;
{
    gmlp_weight_popup_objects *ip =
      (gmlp_weight_popup_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	char *	value = (char *) xv_get(item, PANEL_VALUE);
	
    if(item == ip->etta_nepochs){
	etta_nepochs =
	  set_atof_value(value, "etta_nepochs",
			 Gmlp_weight_popup->etta_nepochs, etta_nepochs);
    } else if (item == ip->kappa){
	kappa = set_atof_low_value(value, "kappa", Gmlp_weight_popup->kappa,
				   kappa, 0.);
    } else if (item == ip->epsilon){
	epsilon = set_atof_range_value(value, "epsilon",
				       Gmlp_weight_popup->epsilon, epsilon,
				       0., 1.);
    } else if (item == ip->decay){
	decay = set_atof_range_value(value, "decay",
				     Gmlp_weight_popup->decay, decay, 0., 1.);
    } else if (item == ip->momentum){
	alpha = set_atof_range_value(value, "alpha",
				     Gmlp_weight_popup->momentum, alpha,
				     0., 1.);
    } else if (item == ip->tolerance){
	tolerance =
	  set_atof_low_value(value, "tolerance", Gmlp_weight_popup->tolerance,
			     tolerance, 0.);
    } else if (item == ip->etta_list){
	char tmp[NNMAX_LINE];
	
	if(etta_list != NULL) deallocate_vector(etta_list);
	etta_list = nnargs_get_float_alloc_list(strcpy(tmp, value));
    } else if (item == ip->max_weight){
	max_magnitude =
	  set_atof_value(value, "max_weight", Gmlp_weight_popup->max_weight,
			 max_magnitude);
    } else {
	fprintf(stderr,
		"LNKnet: gmlp_weight_text_handler: unhandled: value: %s\n",
		value);
    }
    return panel_text_notify(item, event);
}

void
gmlp_node_select_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;/*ARGSUSED*/
{
    gmlp_node_popup_objects *ip =
      (gmlp_node_popup_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
    
    if(item == ip->ofunction){
	gmlp_set_ofunction(value);
    } else if (item == ip->hfunction){
	switch(value){
	  case 0: /* standard sigmoid */
	  case 1: /* symetric sigmoid */
	    hfunction = value;
	    break;
	  case 2: /* linear */
	    hfunction = 4;
	}
    } else if (item == ip->use_sigmoid_list){
	use_sigmoid_param_list = value;
	if(value)
	  gray_out_item(Gmlp_node_popup->sigmoid_list, FALSE);
	else
	  gray_out_item(Gmlp_node_popup->sigmoid_list, TRUE);
    } else {
	fprintf(stderr,
		"LNKnet: gmlp_node_select_handler: unhandled: value: %u\n",
		value);
    }
}

Panel_setting
gmlp_node_text_handler(item, event)
     Panel_item item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    gmlp_node_popup_objects *ip =
      (gmlp_node_popup_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);

    if(item == ip->sigmoid_param){
	sigmoid_param =
	  set_atof_value(value, "sigmoid_param",
			 Gmlp_node_popup->sigmoid_param, sigmoid_param);
    } else if (item == ip->sigmoid_list){
	char tmp[NNMAX_LINE];
	
	if(sigmoid_param_list != NULL) deallocate_vector(sigmoid_param_list);
	sigmoid_param_list = nnargs_get_float_alloc_list(strcpy(tmp, value));
    } else {
	fprintf(stderr,
		"LNKnet: gmlp_node_text_handler: unhandled: value %s\n",
		value);
    }
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `cost_param'.
 */
Panel_setting  
gmlp_cost_text_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);

    cost_param = set_atof_value(value, "cost_param", Gmlp_cost_popup->cost_param,
				cost_param);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `cost_funct'
 */
void  
gmlp_cost_select_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;/*ARGSUSED*/
{
    gmlp_set_cost_func(value);
    gnn_set_cost_func(cost_func);
}

/*
 * Notify callback function for `batch'.
 */
Panel_setting 
gmlp_batch_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    char tmp[MAXSTR];

    nnargs_get_int_list (strcpy(tmp,value), batch, BATCH_SIZE);
    return panel_text_notify(item, event);
}

void
gmlp_set_description_values()
{
    nnodes[1] = nn_params.ninputs;
    nnodes[nnodes[0]] = nn_params.noutputs;

    show_string_text (Gmlp_popup1->layers, make_net_string (nnodes));
    set_plot_labels();
    if(weight_update_mode == 1){
	batch[1] = nn_params.npatterns_train;
	batch[2] = nn_params.npatterns_train;
    } else if(weight_update_mode == 2){
	/* leave ends alone, here */
	batch[2] = nn_params.npatterns_train;
    }
}

int
gmlp_get_epochs()
{
    return nepochs;
}

void
gmlp_set_epochs(e)
     int e;
{
    /* popup will not reflect the set number of epochs */
    nepochs = e;
}

void
gmlp_set_cost_func(value)
     int value;
{
    void gmlp_set_ofunction();
    char tmp[80];
    
    cost_func = value;
    if(cost_func == PC_PROC){
	/* go to a single layer perceptron and step function for ofunction */
	gmlp_set_ofunction(STEP);
	gray_out_item(Gmlp_node_popup->ofunction, TRUE);
	gray_out_item(Gmlp_popup1->layers, TRUE);
	nnodes[0] = 2;
	nnodes[1] = nn_params.ninputs;
	nnodes[2] = nn_params.noutputs;
	/* turn off momemtum */
	gray_out_item(Gmlp_weight_popup->momentum, TRUE);
	alpha = 0.;
	set_plot_labels();
    } else {
	/* set net layers up again */
	gray_out_item(Gmlp_popup1->layers, FALSE);
	nnargs_get_int_list(strcpy(tmp, xv_get(Gmlp_popup1->layers,
					       PANEL_VALUE)),
			    nnodes, NNODES_SIZE);
	nnodes[1] = nn_params.ninputs;
	nnodes[nnodes[0]] = nn_params.noutputs;
	show_string_text (Gmlp_popup1->layers, make_net_string(nnodes));
	set_plot_labels();
	
	/* turn momentum back on */
	alpha = set_atof_range_value((char *)xv_get(Gmlp_weight_popup->momentum,
						    PANEL_VALUE),
				     "alpha", Gmlp_weight_popup->momentum,
				     alpha, 0., 1.);
	gray_out_item(Gmlp_weight_popup->momentum, FALSE);
	
	set_plot_labels();
	
	/* set the output function */
	if(cost_func == SQ_ERR || cost_func == CFOM){
	    gray_out_item(Gmlp_node_popup->ofunction, FALSE);
	    gmlp_set_ofunction(xv_get(Gmlp_node_popup->ofunction, PANEL_VALUE,
				      NULL));
	} else {
	    gray_out_item(Gmlp_node_popup->ofunction, TRUE);
	    gmlp_set_ofunction(SIGM);
	}
    }
    if(cost_func == CFOM)
      gray_out_item(Gmlp_cost_popup->cost_param, FALSE);
    else
      gray_out_item(Gmlp_cost_popup->cost_param, TRUE);
}

int
gmlp_get_cost_func()
{
    return cost_func;
}

void
gmlp_set_etta_change(value)
     int value;
{
    step_change = value;
    if(value == 0){/* constant etta */
	gray_out_item(Gmlp_weight_popup->kappa, TRUE);
	gray_out_item(Gmlp_weight_popup->epsilon, TRUE);
	gray_out_item(Gmlp_weight_popup->etta_nepochs, TRUE);
	gray_out_item(Gmlp_weight_popup->weight_update, FALSE);
    } else if (value == 1){ /* decrease etta after tau trials */
	gray_out_item(Gmlp_weight_popup->kappa, TRUE);
	gray_out_item(Gmlp_weight_popup->epsilon, TRUE);
	gmlp_set_weight_update_mode(0);
	show_int_value(Gmlp_weight_popup->weight_update, weight_update_mode);
	gray_out_item(Gmlp_weight_popup->etta_nepochs, FALSE);
	gray_out_item(Gmlp_weight_popup->weight_update, TRUE);
    } else { /* multiple adaptive step size */
	gmlp_set_weight_update_mode(1);
	show_int_value(Gmlp_weight_popup->weight_update, weight_update_mode);
	gray_out_item(Gmlp_weight_popup->kappa, FALSE);
	gray_out_item(Gmlp_weight_popup->epsilon, FALSE);
	gray_out_item(Gmlp_weight_popup->etta_nepochs, TRUE);
	gray_out_item(Gmlp_weight_popup->weight_update, TRUE);
    }
}

void
gmlp_set_weight_update_mode(value)
     int value;
{
    char *make_net_string();
    
    switch (value) {
      case 0:
	batch[1]=1;
	batch[2]=1;
	batch[3]=0;
	break;
      case 1:
	batch[1]=nn_params.npatterns_train;
	batch[2]=nn_params.npatterns_train;
	batch[3]=0;
	break;
      case 2:
	batch[1]=1;
	batch[2]=nn_params.npatterns_train;
	batch[3]=1;
	break;
      default:
	batch[1]=1;
	batch[2]=1;
	batch[3]=0;
	break;
    }
    weight_update_mode = value;
}

void
gmlp_set_ofunction(value)
     int value;
{
    switch(value){
      case 0: /* standard sigmoid */
      case 1: /* symetric sigmoid */
	ofunction = value;
	break;
      case 2: /* linear */
	ofunction = LINEAR;
	break;
      case 3: /* step function */
	/* this case is used for pcp and is not displayed */
	ofunction = STEP;
	break;
    }
}

char *
gmlp_get_args ()
{
    return (nnargs_get_part_flags(mlp_tab, MLP_TABSIZE));
}

char *
gmlp_get_important_args ()
{
    return (nnargs_get_flags_one_line(mlp_tab, MLP_TABSIZE));
}

static ARG argtab2[]=
{
    {"weight_update", INT, &weight_update_mode, "batch type"},
};
#define TABSIZE2 (sizeof(argtab2)/sizeof(ARG))

void
write_mlp_defaults(default_file)
     FILE *default_file;
{
    if(default_file != NULL){
	fprintf(default_file, "mlp %s",
		nnargs_get_part_flags(mlp_tab, MLP_TABSIZE));
	fprintf (default_file, "%s", nnargs_get_flags(argtab2, TABSIZE2));
    }
}

void
read_mlp_defaults(argc, argv)
     int argc;
     char **argv;
{
    use_etta_list = use_sigmoid_param_list = 0;
    if(argc > 0){
	argc = nnargs_part(argc, argv, mlp_tab, MLP_TABSIZE);
	argc = nnargs(argc, argv, argtab2, TABSIZE2);
    } else {
	nepochs = 20;
    }
    /* algorithm specific flags are now set.  generic flags are taken
       from the argument list for the main screen */
}
