/***--------------------------------------------------------------------------
 * NAME:		gnc_class.c
 * PURPOSE:	parameter popup for nearest cluster classifier
 *
 * USAGE:	gnn	(from command line - produces control panel window)
 *
 * SEE ALSO:	gnn_main.c gnn.c gnn_subs.c nc_class.c nc_class_pack.c
 *
 * DESCRIPTION:
 *	
 *	gnc_class_display_params
 *	gnc_class_get_plot_label
 *	gnc_class_init
 *	gnc_class_set_clustering_active
 *	gnc_class_set_clustering_algorithm
 *	gnc_class_set_cluster_param_filename
 *	gnc_class_set_minimum_variance
 *		
 * BUGS AND LIMITATIONS:
 *
 *
 ------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gclass/nc_class/gnc_class.c,v 10.4 2001/08/23 20:13:45 craig Exp $";



#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <unistd.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <gdd.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <nnlib.h>
#include "gnc_class_ui.h"
#include <nc_class_args.h>			/* nnargs definitions */
#include "../gnn/nn_params.h"
#include "../gnn/galgs.h"
#include "../LNKnet_subs.h"

#define TRUE 1
#define FALSE 0
#define MAXSTR 256

#define EUCLID 0
#define MAHALANOBIS 1

static char *clustering_algorithm_name = "kmeans";
static char param_ftmp[MAXSTR];

static int clustering_active_flag;
static int clustering_algorithm = KMEANS;

extern gnc_class_popup1_objects	*Gnc_class_popup1;

void gnc_class_set_clustering_algorithm();
void gnc_class_show_clustering_algorithm();
void gnc_class_set_cluster_param_fname();

void
gnc_class_display_params()
{
    show_int_value (Gnc_class_popup1->clustering_active,
		    clustering_active_flag);
    gnc_class_show_clustering_algorithm();
    show_string_label(Gnc_class_popup1->cluster_param_fname,
		      cluster_param_fname);
    gnc_class_set_clustering_algorithm(clustering_algorithm);
    gnc_class_set_cluster_param_fname();
    show_float_text (Gnc_class_popup1->min_var, minimum_variance);
    show_int_value (Gnc_class_popup1->gauss_dist, dist_type);
}

char *
gnc_class_get_plot_label (string)
     char *string;
{
    sprintf(string, "%s %s", string,
	    dist_type ? "Mahalanobis" : "Euclidean");
    switch(clustering_algorithm){
      case KMEANS:
	string = gkmeans_get_plot_label(string);
	break;
      case EM_CLUS:
	string = gem_clus_get_plot_label(string);
	break;
      case LEAD_CLUS:
	string = glead_clus_get_plot_label(string);
	break;
      case RAN_CLUS:
	string = gran_clus_get_plot_label(string);
	break;
    }
    return string;
}

void
gnc_class_init ()
{
    gnc_class_display_params();
}

/*
 * Notify callback function for nc_class parameter window
 */
void
gnc_class_callback(item, event)
     Panel_item	item;
     Event	*event;/*ARGSUSED*/
{
    gnc_class_popup1_objects *ip =
      (gnc_class_popup1_objects *)xv_get(item, XV_KEY_DATA, INSTANCE);
    int ivalue;
    char *svalue;

    if(item == ip->clustering_active){
	clustering_active_flag = (int)xv_get(item, PANEL_VALUE);
    } else if (item == ip->clustering_algorithm){
	int temp;
	ivalue = (int)xv_get(item, PANEL_VALUE);
	switch(ivalue){
	  case 0:
	    gnc_class_set_clustering_algorithm(KMEANS);
	    break;
	  case 1:
	    gnc_class_set_clustering_algorithm(EM_CLUS);
	    break;
	  case 2:
	    gnc_class_set_clustering_algorithm(LEAD_CLUS);
	    break;
	  case 3:
	    gnc_class_set_clustering_algorithm(RAN_CLUS);
	    break;
	}
	temp = clustering_active_flag;
	clustering_active_flag = 1;
	gnc_class_set_cluster_param_fname();
	clustering_active_flag = temp;
    } else if (item == ip->clustering_parameters){
	void gkmeans_show_menu(), gem_clus_show_menu();
	void glead_clus_show_menu(), gran_clus_show_menu();

	switch(clustering_algorithm){
	  case KMEANS:
	    gkmeans_show_menu();
	    break;
	  case EM_CLUS:
	    gem_clus_show_menu();
	    break;
	  case LEAD_CLUS:
	    glead_clus_show_menu();
	    break;
	  case RAN_CLUS:
	    gran_clus_show_menu();
	    break;
	}
    } else if (item == ip->gauss_dist){
	dist_type = (int)xv_get(item, PANEL_VALUE);
	set_plot_labels();
    } else if (item == ip->min_var){
	svalue = (char *)xv_get(item, PANEL_VALUE);
	minimum_variance =
	  set_atof_low_value(svalue, "minimum_variance",
			     Gnc_class_popup1->min_var, minimum_variance, 0.);
    } else {
	warning("gnc_class_callback: unhandled interrupt");
    }
}

/*
 * Notify callback function for nc_class selection widgets
 */
void
gnc_class_select_handler(item, value, event)
     Panel_item	item;
     int	value;/*ARGSUSED*/
     Event	*event;
{
    gnc_class_callback(item, event);
}

/*
 * Notify callback function for `min_var'.
 */
Panel_setting
gnc_class_text_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    
    gnc_class_callback(item, event);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `clustering_parameters'.
 */
void
gnc_class_btn_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    gnc_class_callback(item, event);
}

void
gnc_class_show_clustering_algorithm()
{
    switch(clustering_algorithm){
      case KMEANS:
	show_int_value(Gnc_class_popup1->clustering_algorithm, 0);
	break;
      case EM_CLUS:
	show_int_value(Gnc_class_popup1->clustering_algorithm, 1);
	break;
      case LEAD_CLUS:
	show_int_value(Gnc_class_popup1->clustering_algorithm, 2);
	break;
      case RAN_CLUS:
	show_int_value(Gnc_class_popup1->clustering_algorithm, 3);
	break;
    }
}

void
gnc_class_set_clustering_algorithm (value)
     int value;
{
    clustering_algorithm = value;
    dist_type = EUCLID;
    switch(value){
      case KMEANS:
	clustering_algorithm_name = "kmeans";
	break;
      case EM_CLUS:
	clustering_algorithm_name = "em_clus";
	dist_type = MAHALANOBIS;
	break;
      case LEAD_CLUS:
	clustering_algorithm_name = "lead_clus";
	break;
      case RAN_CLUS:
	clustering_algorithm_name = "ran_clus";
	break;
      default:
	break;
    }
    show_int_value(Gnc_class_popup1->gauss_dist, dist_type);
    set_plot_labels();

    /* cluster_param_fname still needs to be set */
}

void
gnc_class_set_cluster_param_fname()
{
    if(clustering_active_flag){
	strcpy(param_ftmp,nn_params.exp_prefix);
	strcat(param_ftmp,clustering_algorithm_name);
	cluster_param_fname = strcat(param_ftmp,".param");
	show_string_label(Gnc_class_popup1->cluster_param_fname,
			 cluster_param_fname);
    }
}

char *
gnc_class_get_args ()
{
    return (nnargs_get_part_flags(nc_class_tab, NC_CLASS_TABSIZE));
}

char *
gnc_class_get_important_args ()
{
    return (nnargs_get_flags_one_line(nc_class_tab, NC_CLASS_TABSIZE));
}

char *gnc_class_get_cluster_fname(){
    return cluster_param_fname;
}

int gnc_class_get_cluster_alg(){
    return clustering_algorithm;
}

int gnc_class_must_store_clustering(){
    return clustering_active_flag;
}

static int clustering_active_arg = 0;
static int dist_type_arg = 0;
static ARG argtab2[]=
{
    {"clus_active", INT, &clustering_active_arg,
       "write cluster calls on create"},
    {"clus_alg", INT, &clustering_algorithm, "clustering algorithm number"},
    {"dist_type", INT, &dist_type_arg, "distance type"},
};
#define TABSIZE2 (sizeof(argtab2)/sizeof(ARG))

void
write_nc_class_defaults(default_file)
     FILE *default_file;
{
    clustering_active_arg = clustering_active_flag;
    dist_type_arg = dist_type;
    if(default_file != NULL){
	fprintf (default_file,"nc_class %s",
		 nnargs_get_part_flags(nc_class_tab,NC_CLASS_TABSIZE));
	fprintf (default_file, "%s", nnargs_get_flags(argtab2, TABSIZE2));
    }
}

void
read_nc_class_defaults(argc, argv)
     int argc;
     char **argv;
{
    if(argc > 0){
	argc = nnargs_part(argc, argv, nc_class_tab, NC_CLASS_TABSIZE);
	argc = nnargs(argc, argv, argtab2, TABSIZE2);
    } else {
	clustering_active_arg = 1;
    }
    clustering_active_flag = clustering_active_arg;
    dist_type = dist_type_arg;
    strcpy(param_ftmp, cluster_param_fname);
    cluster_param_fname = param_ftmp;
    /* algorithm specific flags are now set.  generic flags are taken
       from the argument list for the main screen */
}
