/***--------------------------------------------------------------------------
 * NAME:        gparzen.c
 * PURPOSE:     Parzen Window Classifier graphics
 *
 * SEE ALSO:
 *
 * DESCRIPTION:
 *		
 * BUGS AND LIMITATIONS:
 *
 ------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gclass/parzen/gparzen.c,v 10.4 2001/08/23 20:13:47 craig Exp $";



#define TRUE 1
#define FALSE 0
#define MAXSTR 256
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <unistd.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <gdd.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <nnlib.h>
#include "gparzen_ui.h"
#include <parzen_args.h>			/* nnargs definitions */
#include "../gnn/nn_params.h"
#include "../LNKnet_subs.h"

extern gparzen_popup1_objects	*Gparzen_popup1;

void set_plot_descriptions();
void gparzen_set_common_flags();

void gparzen_display_params()
{
    show_int_value(Gparzen_popup1->per_input, per_input);
    if(per_class)
      show_int_value(Gparzen_popup1->grand, 0);
    else
      show_int_value(Gparzen_popup1->grand, 1);
    show_int_value(Gparzen_popup1->linear, linear);
    show_float_text (Gparzen_popup1->minimum_variance, minimum_variance);
    show_float_text(Gparzen_popup1->spread, spread);
}

char *
gparzen_get_plot_label (string)
     char *string;
{
    sprintf(string, "%s %s %s %s",
	    string,
	    linear ? "Step Functions" : "Gaussian Functions",
	    per_input ? "Ellipse" : "Sphere",
	    per_class ? "Per Class" : "Grand");
    return string;
}
void
gparzen_store_plot_command(shell_file)
     FILE *shell_file;
{
    if(linear)
      fprintf(shell_file, "\\\n -linear -spread %f", spread);
    else
      fprintf(shell_file, "\\\n -spread %f", spread);
}

void
gparzen_init ()
{
    gparzen_display_params();
}

/*
 * Notify callback function for `per_input'.
 */
void
gparzen_select_handler(item, value, event)
Panel_item	item;
int		value;
Event		*event;/*ARGSUSED*/
{
    gparzen_popup1_objects *ip =
      (gparzen_popup1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);

    if(item == ip->per_input){
	per_input = value;
	set_plot_labels();
    } else if(item == ip->grand){
	if(value)
	  per_class = 0;
	else
	  per_class = 1;
	set_plot_labels();
    } else if(item == ip->linear){
	linear = value;
	set_plot_labels();
    } else {
	fprintf(stderr,
		"LNKnet: gparzen_select_handler: unhandled value: %u\n",
		value);
    }
}

/*
 * Notify callback function for `minimum_variance'.
 */
Panel_setting
gparzen_text_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    gparzen_popup1_objects *ip =
      (gparzen_popup1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    if(item == ip->minimum_variance){
	minimum_variance =
	  set_atof_low_value(value, "minimum_variance", item,
			     minimum_variance, 0.);
    } else if (item == ip->spread){
	spread = 
	  set_atof_value(value, "spread", item, spread);
    } else {
	fprintf(stderr,
		"LNKnet: gparzen_text_handler: unhandled value: %s\n",
		value);
    }

    return panel_text_notify(item, event);
}

char *
gparzen_get_args ()
{
    return (nnargs_get_part_flags(parzen_tab, PARZEN_TABSIZE));
}

char *
gparzen_get_important_args ()
{
    return (nnargs_get_flags_one_line(parzen_tab, PARZEN_TABSIZE));
}

void
write_parzen_defaults(default_file)
     FILE *default_file;
{
    if(default_file != NULL){
	fprintf (default_file, "parzen %s",
		 nnargs_get_flags(parzen_tab, PARZEN_TABSIZE));
    } else {
	per_input = 1;
	per_class = 1;
    }
}

void
read_parzen_defaults(argc, argv)
     int argc;
     char **argv;
{
    per_input = per_class = linear = 0;
    if(argc > 0){
	argc = nnargs(argc, argv, parzen_tab, PARZEN_TABSIZE);
    }
}
