/***--------------------------------------------------------------------------
* NAME:		gran_clus.c
* PURPOSE:	Executes ran_clus after setting command line parameters
*
* SEE ALSO:	gran_clus_subs.c
*
* DESCRIPTION:
*	gran_clus_display_params
*	gran_clus_get_plot_label
*	gran_clus_init
*		
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/gclass/ran_clus/gran_clus.c,v 10.4 2001/08/23 20:13:52 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/gclass/ran_clus/gran_clus.c,v $
-------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gclass/ran_clus/gran_clus.c,v 10.4 2001/08/23 20:13:52 craig Exp $";



#define TRUE 1
#define FALSE 0
#define MAXSTR 256
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <unistd.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <gdd.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <nnlib.h>
#include "gran_clus_ui.h"
#include <ran_clus_args.h>		/* nnargs arguments */
#include "../LNKnet_subs.h"
#include "../gnn/nn_params.h"

static int specify_by_class= 0;

extern gran_clus_popup1_objects	*Gran_clus_popup1;

void gran_clus_show_ncenters_choices();
static char tmp_str[MAXSTR];
void gran_clus_set_total_ncenters();
static int total_ncenters;


void
gran_clus_display_params()
{
    show_int_value (Gran_clus_popup1->cluster_by_class, cluster_by_class);
    show_int_value(Gran_clus_popup1->equal_ncenters, specify_by_class);
    show_int_value(Gran_clus_popup1->ncenters, ncenters);
    if(ncenters_list != NULL){
	show_string_text (Gran_clus_popup1->ncenters_list,
			  make_net_string(ncenters_list));
    }
    gran_clus_show_ncenters_choices();
}

char * gran_clus_get_plot_label (string)
     char *string;
{
    sprintf (string, "%s Centers:%d", string, total_ncenters);

    return string;
}

void gran_clus_init ()
{
    gran_clus_display_params();
}

/*
 * Notify callback function for `class_centers'.
 */
void gran_clus_cluster_by_class_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;/*ARGSUSED*/
{
    cluster_by_class = value;
    gran_clus_show_ncenters_choices();
    gran_clus_set_total_ncenters();
    set_plot_labels();
}

/*
 * Notify callback function for `equal_ncenters'.
 */
void gran_clus_equal_ncenters_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;/*ARGSUSED*/
{
    specify_by_class = value;
    gran_clus_show_ncenters_choices();
    gran_clus_set_total_ncenters();
    set_plot_labels();
}


/*
 * Notify callback function for `ncenters'.
 */
Panel_setting gran_clus_ncenters_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int value = (int) xv_get(item, PANEL_VALUE);

    ncenters = value;
    gran_clus_set_total_ncenters();
    set_plot_labels();
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `ncenters_list'.
 */
Panel_setting
gran_clus_ncenters_list_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);

    if(ncenters_list != NULL) deallocate_ivector(ncenters_list);
    ncenters_list = nnargs_get_int_alloc_list(strcpy(tmp_str, value));
    gran_clus_set_total_ncenters();
    set_plot_labels();
    return panel_text_notify(item, event);
}

void gran_clus_show_ncenters_choices()
{
    if(cluster_by_class){
	gray_out_item(Gran_clus_popup1->equal_ncenters, FALSE);
	if(specify_by_class){
	    gray_out_item(Gran_clus_popup1->ncenters_list, FALSE);
	    gray_out_item(Gran_clus_popup1->ncenters, TRUE);
	} else {
	    gray_out_item(Gran_clus_popup1->ncenters_list, TRUE);
	    gray_out_item(Gran_clus_popup1->ncenters, FALSE);
	}
    } else {
	gray_out_item(Gran_clus_popup1->equal_ncenters, TRUE);
	gray_out_item(Gran_clus_popup1->ncenters_list, TRUE);
	gray_out_item(Gran_clus_popup1->ncenters, FALSE);
    }
}

void
gran_clus_set_total_ncenters()
{
    static char total_label[64];
    if(cluster_by_class){
	if(specify_by_class && ncenters_list != NULL){
	    int i;
	    for(i = 1, total_ncenters = 0; i <= ncenters_list[0]; i++){
		total_ncenters += ncenters_list[i];
	    }
	} else {
	    total_ncenters = nn_params.noutputs * ncenters;
	}
    } else {
	total_ncenters = ncenters;
    }
    sprintf(total_label, "This will create a total of %d centers.", total_ncenters);
    show_string_label(Gran_clus_popup1->total_ncenters_msg, total_label);
}
	    
void gran_clus_show_menu()
{
    open_popup (Gran_clus_popup1->popup1);
}

char *
gran_clus_get_args ()
{
    char * ret_str;
    int tmp_ncenters;

    /* ncenters_list will take precedence if it is there.
       making it appear to be length 0 will prevent it from
       being written out as an argument when it is not needed */
    if(!specify_by_class && ncenters_list != NULL){
	tmp_ncenters = ncenters_list[0];
	ncenters_list[0] = 0;
    }
    ret_str = nnargs_get_part_flags(ran_clus_tab, RAN_CLUS_TABSIZE);

    if(!specify_by_class && ncenters_list != NULL){
	ncenters_list[0] = tmp_ncenters;
    }
    return (ret_str);
}

char *
gran_clus_get_important_args ()
{
    char * ret_str;
    int tmp_ncenters;

    /* ncenters_list will take precedence if it is there.
       making it appear to be length 0 will prevent it from
       being written out as an argument when it is not needed */
    if(!specify_by_class && ncenters_list != NULL){
	tmp_ncenters = ncenters_list[0];
	ncenters_list[0] = 0;
    }
    ret_str = nnargs_get_flags_one_line(ran_clus_tab, RAN_CLUS_TABSIZE);

    if(!specify_by_class && ncenters_list != NULL){
	ncenters_list[0] = tmp_ncenters;
    }
    return (ret_str);
}

static int specify_by_class_arg = 0;
static int cluster_by_class_arg = 0;
static int *ncenters_list_arg = NULL;
static ARG argtab2[]=
{
    {"specify_by_class", INT, &specify_by_class_arg,
       "specify centers by class"},
    {"cluster_by_class_flag", INT, &cluster_by_class_arg,
       "cluster by class"},
    {"ncenters_list_arg", INT_ALLOC_LIST, (int *)&ncenters_list_arg},
};
#define TABSIZE2 (sizeof(argtab2)/sizeof(ARG))

void
write_ran_clus_defaults(default_file)
     FILE *default_file;
{
    specify_by_class_arg = specify_by_class;
    cluster_by_class_arg = cluster_by_class;
    if(ncenters_list_arg != NULL) deallocate_ivector(ncenters_list_arg);
    strcpy(tmp_str,(char *)xv_get(Gran_clus_popup1->ncenters_list,
				  PANEL_VALUE));
    ncenters_list_arg =
      nnargs_get_int_alloc_list(tmp_str);
    if(default_file != NULL){
	fprintf (default_file, "ran_clus %s",
		 nnargs_get_part_flags(ran_clus_tab, RAN_CLUS_TABSIZE));
	fprintf (default_file, "%s", nnargs_get_flags(argtab2, TABSIZE2));
    }
}

void
read_ran_clus_defaults(argc, argv)
     int argc;
     char **argv;
{
    int i;

    if(argc > 0){
	argc = nnargs_part(argc, argv, ran_clus_tab, RAN_CLUS_TABSIZE);
	argc = nnargs(argc, argv, argtab2, TABSIZE2);
    }
    specify_by_class = specify_by_class_arg;
    cluster_by_class = cluster_by_class_arg;
    if(ncenters_list_arg != NULL){
	if(ncenters_list != NULL) deallocate_ivector(ncenters_list);
	ncenters_list = allocate_ivector(ncenters_list_arg[0]+1);
	for(i = 0; i <= ncenters_list_arg[0] && i < NNARGS_MAX_LABELS; i++)
	  ncenters_list[i] = ncenters_list_arg[i];
    }
    /* algorithm specific flags are now set.  generic flags are taken
       from the argument list for the main screen */
}
