/***--------------------------------------------------------------------------
 * NAME:        grbf.c
 * PURPOSE:     Radial Basis Function Classifier driver (DEVGUIDE version)
 *		Executes rbf functions after setting command line parameters
 *
 * USAGE:	gnn	(from command line - produces control panel window)
 *
 * SEE ALSO:	gnn_main.c gnn.c gnn_subs.c rbf.c rbf_pack.c grbf_subs.c
 *
 * DESCRIPTION:
 *	
 *	grbf_display_params
 *	grbf_get_plot_label
 *	grbf_init
 *	grbf_set_clustering_algorithm
 *	grbf_store_plot_command
 *		
 * BUGS AND LIMITATIONS:
 *
 *
 ------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gclass/rbf/grbf.c,v 10.4 2001/08/23 20:13:55 craig Exp $";



#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <unistd.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <gdd.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <nnlib.h>
#include "grbf_ui.h"
#include <rbf_args.h>			/* nnargs definitions */
#include "../gnn/nn_params.h"
#include "../gnn/galgs.h"
#include "../LNKnet_subs.h"

#define TRUE 1
#define FALSE 0
#define MAXSTR 256

static char *clustering_algorithm_name = "kmeans";
static char param_ftmp[MAXSTR];

static int clustering_active_flag;
static int clustering_algorithm = KMEANS;

extern grbf_popup1_objects	*Grbf_popup1;

void grbf_set_clustering_algorithm();
void grbf_show_clustering_algorithm();
void grbf_set_cluster_param_fname();

void grbf_display_params()
{
    show_int_value (Grbf_popup1->clustering_active, clustering_active_flag);
    grbf_show_clustering_algorithm();
    show_string_label(Grbf_popup1->cluster_param_fname, cluster_param_fname);
    grbf_set_clustering_algorithm(clustering_algorithm);
    grbf_set_cluster_param_fname();
    show_int_value (Grbf_popup1->bias, bias);
    show_int_value (Grbf_popup1->fast_train, fast_train);
    show_int_text (Grbf_popup1->fast_nhidden, fast_nhidden);
    if(fast_train) gray_out_item(Grbf_popup1->fast_nhidden, FALSE);
    else gray_out_item(Grbf_popup1->fast_nhidden, TRUE);
    show_float_text (Grbf_popup1->min_var, minimum_variance);
    show_float_text (Grbf_popup1->hspread, hspread);
    show_float_text (Grbf_popup1->extra_hspread, extra_hspread);
    show_float_text (Grbf_popup1->max_ratio, max_ratio);
}

char *
grbf_get_plot_label (string)
     char *string;
{
    sprintf (string,"%s H:%g Extra H:%g", string, hspread, extra_hspread);
    if (bias) strcat (string, " Bias");
    switch(clustering_algorithm){
      case KMEANS:
	string = gkmeans_get_plot_label(string);
	break;
      case EM_CLUS:
	string = gem_clus_get_plot_label(string);
	break;
      case LEAD_CLUS:
	string = glead_clus_get_plot_label(string);
	break;
      case RAN_CLUS:
	string = gran_clus_get_plot_label(string);
	break;
    }
    return string;
}

void
grbf_init ()
{
    grbf_display_params();
}

void
grbf_callback(item, event)
     Panel_item	item;
     Event	*event;/*ARGSUSED*/
{
    grbf_popup1_objects *ip =
      (grbf_popup1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
    char *svalue;
    int ivalue;

    if(item == ip->clustering_active){
	clustering_active_flag = (int)xv_get(item, PANEL_VALUE);
    } else if (item == ip->clustering_algorithm){
	int temp;
	ivalue = (int)xv_get(item, PANEL_VALUE);
	switch(ivalue){
	  case 0:
	    grbf_set_clustering_algorithm(KMEANS); break;
	  case 1:
	    grbf_set_clustering_algorithm(EM_CLUS); break;
	  case 2:
	    grbf_set_clustering_algorithm(LEAD_CLUS); break;
	  case 3:
	    grbf_set_clustering_algorithm(RAN_CLUS); break;
	}
	temp = clustering_active_flag;
	clustering_active_flag = 1;
	grbf_set_cluster_param_fname();
	clustering_active_flag = temp;
    } else if(item == ip->clustering_parameters){
	void gkmeans_show_menu(), gem_clus_show_menu();
	void glead_clus_show_menu(), gran_clus_show_menu();
	switch(clustering_algorithm){
	  case KMEANS:
	    gkmeans_show_menu(); break;
	  case EM_CLUS:
	    gem_clus_show_menu(); break;
	  case LEAD_CLUS:
	    glead_clus_show_menu(); break;
	  case RAN_CLUS:
	    gran_clus_show_menu(); break;
	}
    } else if (item == ip->hspread){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	hspread =
	  set_atof_low_value(svalue, "hspread", item, hspread, 0.);
	set_plot_labels();
    } else if (item == ip->extra_hspread){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	extra_hspread =
	  set_atof_low_value(svalue, "extra_hspread", item, extra_hspread, 0.);
	set_plot_labels();
    } else if (item == ip->max_ratio){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	max_ratio =
	  set_atof_low_value(svalue, "max_ratio", item, max_ratio, 1.);
    } else if (item == ip->min_var){
	svalue = (char *) xv_get(item, PANEL_VALUE);
	minimum_variance =
	  set_atof_low_value(svalue, "minimum_variance", item,
			     minimum_variance, 0.);
    } else if (item == ip->bias){
	bias = (int)xv_get(item, PANEL_VALUE);
	set_plot_labels();
    } else if (item == ip->fast_train){
	fast_train = (int)xv_get(item, PANEL_VALUE);
	if(fast_train) gray_out_item(ip->fast_nhidden, FALSE);
	else gray_out_item(ip->fast_nhidden, TRUE);
    } else if(item == ip->fast_nhidden){
	svalue = (char *)xv_get(item, PANEL_VALUE);
	fast_nhidden = set_atoi_value(svalue, "fast_nhidden", item,
				      fast_nhidden);
    } else {
	warning("rbf_callback: unhandled interrupt\n");
    }
}

/*
 * Notify callback function for grbf text fields.
 */
Panel_setting
grbf_text_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    grbf_callback(item, event);
    return panel_text_notify(item, event);
}
/*
 * Notify callback function for rbf window settings
 */
void
grbf_select_handler(item, value, event)
     Panel_item	item;
     int	value;/*ARGSUSED*/
     Event	*event;
{
    grbf_callback(item, event);
}

/*
 * Notify callback function for button widgets
 */
void
grbf_btn_handler(item, event)
     Panel_item	item;
     Event	*event;/*ARGSUSED*/
{
    grbf_callback(item, event);
}

void
grbf_show_clustering_algorithm()
{
    switch(clustering_algorithm){
      case KMEANS:
	show_int_value(Grbf_popup1->clustering_algorithm, 0);
	break;
      case EM_CLUS:
	show_int_value(Grbf_popup1->clustering_algorithm, 1);
	break;
      case LEAD_CLUS:
	show_int_value(Grbf_popup1->clustering_algorithm, 2);
	break;
      case RAN_CLUS:
	show_int_value(Grbf_popup1->clustering_algorithm, 3);
	break;
    }
}

/* needed by grbf_set_description_values */
void
grbf_set_clustering_algorithm (value)
     int value;
{
    clustering_algorithm = value;
    switch(value){
      case KMEANS:
	clustering_algorithm_name = "kmeans";
	break;
      case EM_CLUS:
	clustering_algorithm_name = "em_clus";
	break;
      case LEAD_CLUS:
	clustering_algorithm_name = "lead_clus";
	break;
      case RAN_CLUS:
	clustering_algorithm_name = "ran_clus";
	break;
      default:
	break;
    }
    /* cluster_param_fname still needs to be set */
    set_plot_labels();
}

void
grbf_set_cluster_param_fname()
{
    if(clustering_active_flag){
	strcpy(param_ftmp,nn_params.exp_prefix);
	strcat(param_ftmp,clustering_algorithm_name);
	cluster_param_fname = strcat(param_ftmp,".param");
	show_string_label(Grbf_popup1->cluster_param_fname,
			  cluster_param_fname);
    }
}

void
grbf_store_plot_command (shell_file)
     FILE *shell_file;
{
    fprintf (shell_file, " -exhspread %g", extra_hspread);
}

char *
grbf_get_args ()
{
    return (nnargs_get_part_flags(rbf_tab, RBF_TABSIZE));
}

char *
grbf_get_important_args ()
{
    return (nnargs_get_flags_one_line(rbf_tab, RBF_TABSIZE));
}

char *
grbf_get_cluster_fname(){
    return cluster_param_fname;
}

int
grbf_get_cluster_alg(){
    return clustering_algorithm;
}

int
grbf_must_store_clustering(){
    return clustering_active_flag;
}

static int clustering_active_arg = 0;
static int bias_arg = 0;
static ARG argtab2[]=
{
    {"clus_active", INT, &clustering_active_arg,
       "write cluster calls on create"},
    {"clus_alg", INT, &clustering_algorithm, "clustering algorithm number"},
    {"bias_arg", INT, &bias_arg, "add bias node"},
};
#define TABSIZE2 (sizeof(argtab2)/sizeof(ARG))

void
write_rbf_defaults(default_file)
     FILE *default_file;
{
    clustering_active_arg = clustering_active_flag;
    bias_arg = bias;
    if(default_file != NULL){
	fprintf (default_file,"rbf %s",
		 nnargs_get_part_flags(rbf_tab, RBF_TABSIZE));
	fprintf (default_file, "%s", nnargs_get_flags(argtab2, TABSIZE2));
    }
}

void
read_rbf_defaults(argc, argv)
     int argc;
     char **argv;
{
    fast_train = 0;
    if(argc > 0){
	argc = nnargs_part(argc, argv, rbf_tab, RBF_TABSIZE);
	argc = nnargs(argc, argv, argtab2, TABSIZE2);
    } else {
	clustering_active_arg = 1;
    }
    clustering_active_flag = clustering_active_arg;
    strcpy(param_ftmp, cluster_param_fname);
    cluster_param_fname = param_ftmp;
    bias = bias_arg;
    /* algorithm specific flags are now set.  generic flags are taken
       from the argument list for the main screen */
}
