/***--------------------------------------------------------------------------
 * NAME:        girbfm.c
 * PURPOSE:     Incremental Radial Basis Function Classifier parameters
 *		Executes irbfm functions after setting command line parameters
 *
 --------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gmap/girbfm/girbfm.c,v 10.4 2001/08/23 20:13:57 craig Exp $";



#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <unistd.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <gdd.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <nnlib.h>
#include "girbfm_ui.h"
#include <irbfm_args.h>			/* nnargs definitions */
#include <gmap_params.h>		/* common arguments */

#define TRUE 1
#define FALSE 0
#define MAXSTR 256

/* clustering algorithm integer designator */
/* this list must be the same as the one in girbfm_subs.c */
#define KMEANSM 0
#define EM_CLUSM 1
#define LEAD_CLUSM 2
#define RAN_CLUSM 3

static char *program_name="irbfm";
static char *run_time_system=NULL;
static char chartmp[80];
static char *clustering_algorithm_name = "kmeansm";
static char param_ftmp[MAXSTR];


static int clustering_algorithm = KMEANSM;
static int clustering_active_arg = 0;
static int clustering_active_flag = 0;
static int bias_arg = 0;
static int grand_arg = 0;
static ARG argtab2[]=
{
    {"clus_active", INT, &clustering_active_arg,
       "write cluster calls on create"},
    {"clus_alg", INT, &clustering_algorithm, "clustering algorithm number"},
    {"bias_arg", INT, &bias_arg, "add a bias node"},
    {"grand_arg", INT, &grand_arg, "grand variances"},
};
#define TABSIZE2 (sizeof(argtab2)/sizeof(ARG))

extern girbfm_popup1_objects	*girbfm_popup1;

void girbfm_set_common_flags();
void set_float_string();
void girbfm_set_cluster_param_fname();
void girbfm_set_clustering_algorithm();
float set_atof_value(), set_atof_range_value(), set_atof_low_value();

void girbfm_display_params()
{
    xv_set (girbfm_popup1->clustering_active, PANEL_VALUE,
	    clustering_active_flag, NULL);
    xv_set(girbfm_popup1->clustering_algorithm, PANEL_VALUE,
	   clustering_algorithm, NULL);
    xv_set(girbfm_popup1->cluster_param_fname, PANEL_VALUE,
	   cluster_param_fname, NULL);
    girbfm_set_clustering_algorithm(clustering_algorithm);
    girbfm_set_cluster_param_fname();
    xv_set (girbfm_popup1->bias, PANEL_VALUE, bias, NULL);
    xv_set(girbfm_popup1->grandvar, PANEL_VALUE, grand_flag, NULL);
    set_float_string (girbfm_popup1->min_var, "%g", minimum_variance);
    set_float_string (girbfm_popup1->weight_eta,"%5g",weight_eta);
    set_float_string (girbfm_popup1->mean_eta,"%5g",mean_eta);
    set_float_string (girbfm_popup1->var_eta,"%5g",var_eta);
    set_float_string (girbfm_popup1->hspread, "%5.2g", hspread);
    set_float_string (girbfm_popup1->extra_hspread, "%5.2g", extra_hspread);
    set_float_string (girbfm_popup1->max_magnitude, "%5.2g", max_magnitude);
    xv_set(girbfm_popup1->epochs, PANEL_VALUE, nepochs, NULL);
}

char * girbfm_get_plot_label ()
{
    static char string[MAXSTR] = "";
    
    girbfm_set_common_flags();
    
    sprintf (string,"H:%g Extra H:%g", hspread, extra_hspread);
    if (bias) strcat (string, " Bias");
    
    return string;
}

void girbfm_init ()
{
    girbfm_display_params();
}

void girbfm_set_common_flags()
{
    int i;
    
    create_flag = map_params.create_flag;
    data_fname = map_params.data_fname;
    data_path = map_params.data_path;
    debug = map_params.debug;
    desc_fname = map_params.desc_fname;
    error_fname = map_params.error_fname;
    exp_path = map_params.exp_path;
    for(i=0; i<= atoi(map_params.labels[0]) && i < NNARGS_MAX_LABELS; i++)
      labels[i] = map_params.labels[i];
    noutputs = map_params.noutputs;
    npatterns = map_params.n_patterns;
    nraw = map_params.nraw;
    param_fname = map_params.param_fname;
    run_time_system = map_params.run_time_system;
    seed = map_params.seed;
    train_flag = map_params.train_flag;
    verbosity = map_params.verbosity;
    verror = map_params.verror;
}

void girbfm_set_description_values()
{
    girbfm_set_common_flags();
}


/*
 * Notify callback function for `clustering_active'.
 */
void girbfm_create_clusters_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    clustering_active_flag = value;
}


/*
 * Notify callback function for `clustering_algorithm'.
 */
void girbfm_clustering_algorithm_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    switch(value){
      case 0:
	girbfm_set_clustering_algorithm(KMEANSM);
	break;
      case 1:
	girbfm_set_clustering_algorithm(EM_CLUSM);
	break;
      case 2:
	girbfm_set_clustering_algorithm(LEAD_CLUSM);
	break;
      case 3:
	girbfm_set_clustering_algorithm(RAN_CLUSM);
	break;
    }
}

/* needed by girbfm_set_description_values */
void girbfm_set_clustering_algorithm (value)
     int value;
{
    int temp;

    clustering_algorithm = value;
    switch(value){
      case KMEANSM:
	clustering_algorithm_name = "kmeansm";
	break;
      case EM_CLUSM:
	clustering_algorithm_name = "em_clusm";
	break;
      case LEAD_CLUSM:
	clustering_algorithm_name = "lead_clusm";
	break;
      case RAN_CLUSM:
	clustering_algorithm_name = "ran_clusm";
	break;
      default:
	break;
    }
    temp = clustering_active_flag;
    clustering_active_flag = 1;
    girbfm_set_cluster_param_fname();
    clustering_active_flag = temp;
}

void girbfm_set_cluster_param_fname()
{
    if(clustering_active_flag){
	strcpy(param_ftmp,map_params.exp_label);
	strcat(param_ftmp,clustering_algorithm_name);
	cluster_param_fname = strcat(param_ftmp,".param");
	xv_set(girbfm_popup1->cluster_param_fname,PANEL_VALUE,
	       cluster_param_fname, NULL);
    }
}

/*
 * Notify callback function for `clustering_parameters'.
 */
void girbfm_clustering_parameters_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    void gkmeansm_show_menu();

    switch(clustering_algorithm){
      case KMEANSM:
	gkmeansm_show_menu();
	break;
#if 0
      case EM_CLUSM:
	gem_clusm_show_menu();
	break;
      case LEAD_CLUSM:
	glead_clusm_show_menu();
	break;
      case RAN_CLUSM:
	gran_clusm_show_menu();
	break;
#endif
    }
    
}

/*
 * Notify callback function for `cluster_param_fname'.
 */
Panel_setting girbfm_cluster_param_fname_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    cluster_param_fname = value;
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `extra_hspread'.
 */
Panel_setting girbfm_extra_hspread_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    extra_hspread =
      set_atof_low_value(value, "extra_hspread",
			 girbfm_popup1->extra_hspread, extra_hspread, 0.);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `hspread'.
 */
Panel_setting girbfm_hspread_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    hspread =
      set_atof_low_value(value, "hspread", girbfm_popup1->hspread, hspread, 0.);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `weight_eta'.
 */
Panel_setting girbfm_weight_eta_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    weight_eta =
      set_atof_low_value(value, "weight_eta", girbfm_popup1->weight_eta, weight_eta, 0.);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `mean_eta'.
 */
Panel_setting girbfm_mean_eta_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    mean_eta =
      set_atof_low_value(value, "mean_eta", girbfm_popup1->mean_eta, mean_eta, 0.);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `var_eta'.
 */
Panel_setting girbfm_var_eta_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    var_eta =
      set_atof_low_value(value, "var_eta", girbfm_popup1->var_eta, var_eta, 0.);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `max_magnitude'.
 */
Panel_setting girbfm_max_magnitude_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    max_magnitude =
      set_atof_low_value(value, "max_magnitude", girbfm_popup1->max_magnitude,
			 max_magnitude, 0.);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `min_var'.
 */
Panel_setting girbfm_min_var_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    minimum_variance =
      set_atof_low_value(value, "minimum_variance",
			 girbfm_popup1->min_var, minimum_variance, 0.);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `bias'.
 */
void
  girbfm_bias_handler(item, value, event)
Panel_item	item;
int		value;
Event		*event;
{
    bias = value;
}

void girbfm_store_plot_command (shell_file)
     FILE *shell_file;
{
    girbfm_set_common_flags();
    
    fprintf (shell_file, " -exhspread %g", extra_hspread);
}

void girbfm_store_run_command (shell_file, log_filename)
     FILE *shell_file;
     char *log_filename;
{
    static char command_line[1024] = "";
    char tmp_buf[1024];
    char chartmp2[3];
    void gkmeansm_store_run_command();
    
    girbfm_set_common_flags();
    
    if(clustering_active_flag && train_flag && create_flag){
	strcpy(tmp_buf, map_params.param_fname);
	strcpy(map_params.param_fname, cluster_param_fname);
	switch(clustering_algorithm){
	  case KMEANSM:
	    gkmeansm_store_run_command(shell_file, log_filename);
	    break;
#if 0
	  case EM_CLUSM:
	    gem_clusm_store_run_command(shell_file, log_filename);
	    break;
	  case LEAD_CLUSM:
	    glead_clusm_store_run_command(shell_file, log_filename);
	    break;
	  case RAN_CLUSM:
	    gran_clusm_store_run_command(shell_file, log_filename);
	    break;
#endif
	}
	strcpy(map_params.param_fname, tmp_buf);
    }
    sprintf (tmp_buf,"%s",nnargs_get_flags(argtab, TABSIZE));
    *strrchr(tmp_buf,'\n') = ' ';	/* get rid of newline at end */
    
    if (run_time_system == NULL || strlen(run_time_system)<1)
      chartmp[0] = '\0';
    else sprintf (chartmp, "rsh -n %s ", run_time_system);
    
    if (!create_flag || (clustering_active_flag && train_flag))
      sprintf (chartmp2, "-a");
    else chartmp2[0] = '\0';
    
    if (debug>0 || verbosity>1) 
      sprintf (command_line, "(time %s%s \\\n%s) \\\n%s %s %s",
	       chartmp, program_name, tmp_buf,  
	       " |& nn_tee -h", chartmp2, log_filename);
    else
      sprintf (command_line, "%s%s \\\n%s \\\n%s %s %s",
	       chartmp, program_name, tmp_buf,  
	       " |& nn_tee -h", chartmp2, log_filename);
    
    if (debug>10) printf("g_run -- command line: %s\n",command_line);
    
    fprintf (shell_file, "%s\n", command_line);
}

void write_irbfm_defaults(default_file, default_fullname)
     FILE *default_file;
     char *default_fullname;
{
    clustering_active_arg = clustering_active_flag;
    bias_arg = bias;
    grand_arg = grand_flag;
    if(default_file != NULL){
	fprintf (default_file, "irbfm %s", nnargs_get_flags(argtab, TABSIZE));
	fprintf (default_file, "%s", nnargs_get_flags(argtab2, TABSIZE2));
  }
}

void read_irbfm_defaults(default_file, default_fullname)
     FILE *default_file;
     char *default_fullname;
{
    int nargc;
    
    if(default_file != NULL){
	nargc =
	  nnargs_from_file(default_file, default_fullname, argtab, TABSIZE);
	nargc =
	  nnargs_from_file(default_file, default_fullname, argtab2, TABSIZE2);
    } else {
	clustering_active_arg = 1;
	nepochs = 20;
    }
    clustering_active_flag = clustering_active_arg;
    strcpy(param_ftmp, cluster_param_fname);
    cluster_param_fname = param_ftmp;
    bias = bias_arg;
    grand_flag = grand_arg;
    /* algorithm specific flags are now set.  generic flags are taken
       from the argument list for the main screen */
}

/*
 * Notify callback function for `epochs'.
 */
Panel_setting girbfm_epochs_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);
    
    nepochs = value;
    return panel_text_notify(item, event);
}

int girbfm_get_epochs()
{
    return nepochs;
}

void girbfm_set_epochs(e)
{
    nepochs = e;
}

/*
 * Notify callback function for `grandvar'.
 */
void girbfm_grandvar_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    grand_flag = value;
}
