/***--------------------------------------------------------------------------
 * NAME:		gkmeansm.c
 *
 *
 * DESCRIPTION:
 *	gkmeansm_display_params
 *	gkmeansm_get_plot_label
 *	gkmeansm_init
 *	gkmeansm_set_common_flags
 *	gkmeansm_set_description_values
 *	gkmeansm_set_max_iteration
 *	gkmeansm_set_ncenters
 *	gkmeansm_set_split_percentage
 *	gkmeansm_set_stop_percentage
 *	gkmeansm_store_run_command
 *	gkmeansm_set_params
 *		
 * BUGS AND LIMITATIONS:
 *
 * $Header: /data/speech/kukolich/cvsroot/lnknet/src/gmap/gkmeansm/gkmeansm.c,v 10.4 2001/08/23 20:14:00 craig Exp $
 * $Source: /data/speech/kukolich/cvsroot/lnknet/src/gmap/gkmeansm/gkmeansm.c,v $
 --------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gmap/gkmeansm/gkmeansm.c,v 10.4 2001/08/23 20:14:00 craig Exp $";



#define TRUE 1
#define FALSE 0
#define MAXSTR 256
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <unistd.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <gdd.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <nnlib.h>
#include "gkmeansm_ui.h"
#include <kmeansm_args.h>		/* nnargs definitions */
#include <gmap_params.h>		/* common arguments */

static char *program_name="kmeansm";
static char *run_time_system=NULL;

extern gkmeansm_popup1_objects	*gkmeansm_popup1;

void gkmeansm_set_common_flags();
void set_float_string();
float set_atof_value(), set_atof_range_value(), set_atof_low_value();

void gkmeansm_display_params()
{
    char *make_net_string(), *value;
    
    xv_set (gkmeansm_popup1->ncenters, PANEL_VALUE, ncenters, NULL);
    xv_set (gkmeansm_popup1->max_iteration, PANEL_VALUE, max_iteration, NULL);
    set_float_string (gkmeansm_popup1->stop_percentage, "%5.2g",
		      stop_percentage);
    set_float_string (gkmeansm_popup1->split_percentage, "%5.2g",
		      split_percentage);
    value = make_net_string(&ncenters_list[0]);
    xv_set (gkmeansm_popup1->add_random_offset, PANEL_VALUE,
	    add_random_offset, NULL);
    xv_set (gkmeansm_popup1->reduce_step, PANEL_VALUE, reduce_step, NULL);
}

char * gkmeansm_get_plot_label ()
{
    static char string[MAXSTR] = "";
    
    gkmeansm_set_common_flags();
    
    sprintf (string, "Norm:%s K:%d", norm_description(normalize), ncenters);
    
    return string;
}

void gkmeansm_init ()
{
    ncenters = 8;
    
    gkmeansm_display_params();
}

void gkmeansm_set_common_flags()
{
    int i;
    
    data_fname = map_params.data_fname;
    data_path = map_params.data_path;
    debug = map_params.debug;
    desc_fname = map_params.desc_fname;
    error_fname = map_params.error_fname;
    exp_path = map_params.exp_path;
    for(i=0; i<= atoi(map_params.labels[0]) && i < NNARGS_MAX_LABELS; i++)
      labels[i] = map_params.labels[i];
    nraw = map_params.nraw;
    ninputs = map_params.ninputs;
    noutputs = map_params.noutputs;
    normalize = map_params.normalize;
    norm_fname = map_params.norm_fname;
    npatterns = map_params.n_patterns;
    param_fname = map_params.param_fname;
    run_time_system = map_params.run_time_system;
    seed = map_params.seed;
    verbosity = map_params.verbosity;
    verror = map_params.verror;
}

void gkmeansm_set_description_values()
{
    gkmeansm_set_common_flags();
}


/*
 * Notify callback function for `max_iteration'.
 */
Panel_setting gkmeansm_max_iteration_handler(item, event)
     Panel_item	item;
     Event		*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);
    
    max_iteration = value;
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `split_percentage'.
 */
Panel_setting gkmeansm_split_percentage_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    split_percentage =
      set_atof_low_value(value, "split_percentage",
			 gkmeansm_popup1->split_percentage, split_percentage, 0.);
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `stop_percentage'.
 */
Panel_setting gkmeansm_stop_percentage_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    stop_percentage =
      set_atof_low_value(value, "stop_percentage",
			 gkmeansm_popup1->stop_percentage, stop_percentage, 0.);
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `ncenters'.
 */
Panel_setting gkmeansm_ncenters_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);
    
    ncenters = value;
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `reduce_step'.
 */
Panel_setting gkmeansm_reduce_step_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);
    
    reduce_step = value;
    return panel_text_notify(item, event);
}



/*
 * Notify callback function for `add_random_offset'.
 */
void gkmeansm_add_random_offset_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    add_random_offset = value;
}

void gkmeansm_show_menu()
{
    xv_set (gkmeansm_popup1->popup1, XV_SHOW, TRUE, 0);
}

void gkmeansm_store_run_command (shell_file, log_filename)
     FILE *shell_file;
     char *log_filename;
{
    char tmp_buf[1024];
    static char command_line[1024] = "";
    char chartmp[80];
    char chartmp2[3];
    
    gkmeansm_set_common_flags();
    
    sprintf (tmp_buf,"%s",nnargs_get_flags(argtab, TABSIZE));
    *strrchr(tmp_buf,'\n') = ' ';	/* get rid of newline at end */
    
    if (run_time_system == NULL || strlen(run_time_system)<1)
      chartmp[0] = '\0';
    else sprintf (chartmp, "rsh -n %s ", run_time_system);
    
    *chartmp2 = NULL;
    
    if (debug>0 || verbosity>1) 
      sprintf (command_line, "(time %s%s \\\n%s) \\\n%s %s %s",
	       chartmp, program_name, tmp_buf,  
	       " |& nn_tee -h", chartmp2, log_filename);
    else
      sprintf (command_line, "%s%s \\\n%s \\\n%s %s %s",
	       chartmp, program_name, tmp_buf,  
	       " |& nn_tee -h", chartmp2, log_filename);
    
    if (debug>10) printf("g_run -- command line: %s\n",command_line);
    
    fprintf (shell_file, "%s\n", command_line);
}
static int add_offset_arg = 0;
static ARG argtab2[]=
{
    {"add_offset_arg", INT, &add_offset_arg,
       "dummy for add_random_offset"},
};
#define TABSIZE2 (sizeof(argtab2)/sizeof(ARG))

void write_kmeansm_defaults(default_file, default_fullname)
     FILE *default_file;
     char *default_fullname;
{
    add_offset_arg = add_random_offset;
    if(default_file != NULL){
	fprintf (default_file,"kmeansm %s",
		 nnargs_get_part_flags(argtab, TABSIZE));
	fprintf (default_file, "%s", nnargs_get_flags(argtab2, TABSIZE2));
    }
}

void read_kmeansm_defaults(default_file, default_fullname)
     FILE *default_file;
     char *default_fullname;
{
    int nargc;
    char *argv[256];
    
    if(default_file != NULL){
	nargc = fill_argv(argv, default_file, default_fullname);
	nargc = nnargs_part(nargc, argv, argtab, TABSIZE);
	nargc = nnargs(nargc, argv, argtab2, TABSIZE2);
    }
    add_random_offset = add_offset_arg;
    /* algorithm specific flags are now set.  generic flags are taken
       from the argument list for the main screen */
}
