/***--------------------------------------------------------------------------
 * NAME:        gmlpm.c
 * PURPOSE:     Multilayer Perceptron driver (DEVGUIDE version)
 *		Executes mlp functions after setting command line parameters
 *
 * USAGE:	gmap	(from command line - produces control panel window)
 *
 * SEE ALSO:	gmain.c g.c gmlpm_subs.c
 *
 * DESCRIPTION:
 *	
 *	gmlpm_display_params
 *	gmlpm_get_plot_label
 *	gmlpm_init
 *	gmlpm_set_alpha
 *	gmlpm_set_batch
 *	gmlpm_set_common_flags
 *	gmlpm_set_decay
 *	gmlpm_set_description_values
 *	gmlpm_set_epochs
 *	gmlpm_set_epsilon
 *	gmlpm_set_etta
 *	gmlpm_set_kappa
 *	gmlpm_set_layers
 *	gmlpm_set_mas
 *	gmlpm_set_stats_flag
 *	gmlpm_set_tolerance
 *	gmlpm_set_weight_update_mode
 *	gmlpm_store_run_command
 *		
 * BUGS AND LIMITATIONS:
 *
 *
 --------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gmap/gmlpm/gmlpm.c,v 10.4 2001/08/23 20:14:02 craig Exp $";



#define TRUE 1
#define FALSE 0
#define MAXSTR 256

#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <unistd.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <gdd.h>
#include <signal.h>
#include <xview/textsw.h>
#include <sys/ioctl.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <nnlib.h>
#include "gmlpm_ui.h"
#include <mlpm_args.h>
#include <gmap_params.h>

/* static GLOBALS internal to this file */
static char *run_time_system = NULL;
static char *program_name="mlpm";
static int weight_update_mode = 0;
static char chartmp[MAXSTR];

extern gmlpm_popup1_objects	*gmlpm_popup1;

void set_float_string();
void gmlpm_set_common_flags();
void gmlpm_set_weight_update_mode();
void set_plot_descriptions();
float set_atof_value(), set_atof_range_value(), set_atof_low_value();

void gmlpm_display_params()
{
    char *make_net_string();
    
    if (xv_set (gmlpm_popup1->epochs, PANEL_VALUE, nepochs, NULL) != XV_OK)
      warning("init_params -- unable to set epoch value %d",nepochs);
    
    xv_set (gmlpm_popup1->layers, PANEL_VALUE, make_net_string (nnodes), NULL);
    set_float_string (gmlpm_popup1->etta, "%5.2g", etta);
    set_float_string (gmlpm_popup1->momentum, "%5.2g", alpha);
    set_float_string (gmlpm_popup1->tolerance, "%5.2g", tolerance);
    set_float_string (gmlpm_popup1->decay, "%5.2g", decay);
    set_float_string (gmlpm_popup1->kappa, "%5.2g", kappa);
    set_float_string (gmlpm_popup1->epsilon, "%5.2g", epsilon);
    switch(ofunction){
      case 0:
      case 1:
	xv_set(gmlpm_popup1->ofunction, PANEL_VALUE, ofunction, NULL);
	break;
      case LINEAR:
	xv_set(gmlpm_popup1->ofunction, PANEL_VALUE, 2, NULL);
	break;
    }
    xv_set(gmlpm_popup1->batch, PANEL_VALUE, make_net_string(batch), NULL);
    xv_set(gmlpm_popup1->mas_flag, PANEL_VALUE, etta_type, NULL);
    xv_set(gmlpm_popup1->weight_update, PANEL_VALUE, weight_update_mode, NULL);
}

char * gmlpm_get_plot_label ()
{
    static char string[MAXSTR] = "";
    
    gmlpm_set_common_flags();
    
    sprintf (string, "Norm:%s Net:%s Step:%g %Momen:%g",
	     norm_description(normalize), make_net_string(nnodes), etta, alpha);
    
    return string;
}

void gmlpm_init ()
{
    nepochs = 15;
    verror = 1;
    
    gmlpm_display_params();
    gmlpm_set_weight_update_mode (weight_update_mode);
}


/*
 * Notify callback function for `momentum'.
 */
Panel_setting gmlpm_alpha_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    alpha = set_atof_range_value(value, "alpha", gmlpm_popup1->momentum, alpha, 0., 1.);
    set_plot_descriptions();
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `batch'.
 */
Panel_setting gmlpm_batch_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    char tmp[MAXSTR];
    
    nnargs_get_int_list (strcpy(tmp,value), batch, BATCH_SIZE);
    return panel_text_notify(item, event);
}

void gmlpm_set_common_flags()
{
    int i;
    
    create_flag = map_params.create_flag;
    cross_valid = map_params.cross_valid;
    cross_valid_fname = map_params.cv_fname;
    cross_valid_from_file = map_params.cv_from_file;
    data_fname = map_params.data_fname;
    data_path = map_params.data_path;
    debug = map_params.debug;
    desc_fname = map_params.desc_fname;
    error_fname = map_params.error_fname;
    exp_path = map_params.exp_path;
    for(i=0; i<= atoi(map_params.labels[0]) && i <= NNARGS_MAX_LABELS; i++)
      labels[i] = map_params.labels[i];
    nraw = map_params.nraw;
    nnodes[1] = map_params.ninputs;
    normalize = map_params.normalize;
    norm_fname = map_params.norm_fname;
    nnodes[nnodes[0]] = map_params.noutputs;
    npatterns = map_params.n_patterns;
    param_fname = map_params.param_fname;
    random_flag = map_params.random_flag;
    run_time_system = map_params.run_time_system;
    seed = map_params.seed;
    train_flag = map_params.train_flag;
    verbosity = map_params.verbosity;
    verror = map_params.verror;
}


/*
 * Notify callback function for `decay'.
 */
Panel_setting gmlpm_decay_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    decay = set_atof_range_value(value, "decay", gmlpm_popup1->decay, decay, 0., 1.);
    return panel_text_notify(item, event);
}

void gmlpm_set_description_values()
{
    gmlpm_set_common_flags();
    xv_set (gmlpm_popup1->layers, PANEL_VALUE, make_net_string (nnodes), NULL);
    gmlpm_set_weight_update_mode (weight_update_mode);
}


/*
 * Notify callback function for `epochs'.
 */
Panel_setting gmlpm_epochs_handler(item, event)
Panel_item	item;
Event		*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);
    
    nepochs = value;
    return panel_text_notify(item, event);
}

int gmlpm_get_epochs()
{
    return nepochs;
}

void gmlpm_set_epochs(e)
     int e;
{
    /* popup will not reflect the set number of epochs */
    nepochs = e;
}

/*
 * Notify callback function for `epsilon'.
 */
Panel_setting gmlpm_epsilon_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    epsilon = set_atof_range_value(value, "epsilon",
				   gmlpm_popup1->epsilon, epsilon, 0., 1.);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `etta'.
 */
Panel_setting gmlpm_etta_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    etta = set_atof_low_value(value, "etta", gmlpm_popup1->etta, etta, 0.);
    set_plot_descriptions();
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `kappa'.
 */
Panel_setting gmlpm_kappa_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    kappa = set_atof_low_value(value, "kappa", gmlpm_popup1->kappa, kappa, 0.);
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `layers'.
 */
Panel_setting gmlpm_layers_handler(item, event)
Panel_item	item;
Event		*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    char tmp[80];
    
    nnargs_get_int_list (strcpy(tmp,value), nnodes, NNODES_SIZE);
    if (nnodes[0] > 0) {
	map_params.ninputs = nnodes[1];
	map_params.noutputs = nnodes[nnodes[0]];
    }
    set_plot_descriptions();
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `algorithm_options2'.
 */
void gmlpm_mas_handler(item, value, event)
Panel_item	item;
int		value;
Event		*event;
{
    if (value & 1) etta_type = TRUE; else etta_type = FALSE;
}

/*
 * Notify callback function for `tolerance'.
 */
Panel_setting gmlpm_tolerance_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    tolerance =
      set_atof_low_value(value, "tolerance", gmlpm_popup1->tolerance,
			 tolerance, 0.);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `weight_update1'.
 */
void gmlpm_weight_update_handler(item, value, event)
Panel_item	item;
int		value;
Event		*event;
{
    gmlpm_set_weight_update_mode (value);
}

void
  gmlpm_set_weight_update_mode(value)
int value;
{
    char *make_net_string();
    
    switch (value) {
      case 0:
	batch[1]=1;
	batch[2]=1;
	batch[3]=0;
	break;
      case 1:
	batch[1]=map_params.n_patterns_train;
	batch[2]=map_params.n_patterns_train;
	batch[3]=0;
	break;
      case 2:
	batch[1]=1;
	batch[2]=map_params.n_patterns_train;
	batch[3]=1;
	break;
      default:
	batch[1]=1;
	batch[2]=1;
	batch[3]=0;
	break;
    }
    xv_set (gmlpm_popup1->batch, PANEL_VALUE, make_net_string (batch), NULL);
    weight_update_mode = value;
}


/*
 * Notify callback function for `hfunction'.
 */
void gmlpm_hfunction_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    switch(value){
      case 0: /* standard sigmoid */
      case 1: /* symetric sigmoid */
	hfunction = value;
	break;
      case 2: /* linear */
	hfunction = 4;
    }
}


/*
 * Notify callback function for `ofunction'.
 */
void gmlpm_ofunction_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    switch(value){
      case 0: /* standard sigmoid */
      case 1: /* symetric sigmoid */
	ofunction = value;
	break;
      case 2: /* linear */
	ofunction = 4;
    }
}

void gmlpm_store_run_command (shell_file, log_filename)
     FILE *shell_file;
     char *log_filename;
{
    static char command_line[1024] = "";
    char tmp_buf[1024];
    char chartmp2[3];
    
    gmlpm_set_common_flags();
    
    sprintf (tmp_buf,"%s",nnargs_get_flags(argtab, TABSIZE));
    *strrchr(tmp_buf,'\n') = ' ';	/* get rid of newline at end */
    
    if (run_time_system == NULL || strlen(run_time_system)<1)
      chartmp[0] = '\0';
    else sprintf (chartmp, "rsh -n %s ", run_time_system);
    
    if(create_flag) chartmp2[0] = '\0';
    else sprintf (chartmp2, "-a");
    
    if (debug>0 || verbosity>1) 
      sprintf (command_line, "(time %s%s \\\n%s) \\\n%s %s %s",
	       chartmp, program_name, tmp_buf,  
	       " |& nn_tee -h", chartmp2, log_filename);
    else
      sprintf (command_line, "%s%s \\\n%s \\\n%s %s %s",
	       chartmp, program_name, tmp_buf,  
	       " |& nn_tee -h", chartmp2, log_filename);
    
    if (debug>10) printf("g_run -- command line: %s\n",command_line);
    
    fprintf (shell_file, "%s\n", command_line);
}

static int mas_arg = 0;

static ARG argtab2[]=
{
    {"weight_update", INT, &weight_update_mode, "batch type"},
    {"mas_arg", INT, &mas_arg, "dummy for mas flag"},
};
#define TABSIZE2 (sizeof(argtab2)/sizeof(ARG))

void write_mlpm_defaults(default_file, dfname)
     FILE *default_file;
     char *dfname;
{
    mas_arg = etta_type;
    if(default_file != NULL){
	fprintf(default_file, "mlpm %s",
		nnargs_get_part_flags(argtab, TABSIZE));
	fprintf (default_file, "%s", nnargs_get_flags(argtab2, TABSIZE2));
    }
}

void read_mlpm_defaults(default_file, default_fullname)
     FILE *default_file;
     char *default_fullname;
{
    int nargc;
    char *argv[256];

    use_etta_list = 0;
    use_sigmoid_param_list = 0;
    if(default_file != NULL){
	nargc = fill_argv(argv, default_file, default_fullname);
	nargc = nnargs_part(nargc, argv, argtab, TABSIZE);
	nargc = nnargs(nargc, argv, argtab2, TABSIZE2);
    }
    etta_type = mas_arg;
    /* algorithm specific flags are now set.  generic flags are taken
       from the argument list for the main screen */
}
