/***--------------------------------------------------------------------------
 * NAME:        gmap.c
 * PURPOSE:     Neural Network Experiment main control panel driver
 *
 * USAGE:	LNKmap	(from command line - produces control panel window)
 *		Default arguments can be provided in .LNKmaprc in login dir.
 *
 * SEE ALSO:	 gmap_main.c gmap_subs.c gmap.G
 *
 * DESCRIPTION:
 *		
 * BUGS AND LIMITATIONS:
 *
 ------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gmap/lnkmap/gmap.c,v 10.4 2001/08/23 20:14:08 craig Exp $";



#define TRUE (1)
#define FALSE (0)
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <unistd.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <sys/param.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <gdd.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <nnlib.h>
#include "gmap_ui.h"
#include <gnn_args.h>
#include "gmap_params.h"
#include "lnkmap_args.h"
#include <norm_args.h>

/* run mode variables  */
#define CROSS_VALID (64)
#define TRAIN (1)		/* values for run_options */
#define CLASS_TRAIN (2)
#define CLASS_EVAL (4)
#define CLASS_TEST (8)
#define CLASS (16)	/* for store command, not for run_options */
/* run_options is a bit vector. (shown little endian)
   1		2	4		8
   TRAIN CLASS_ON_TRAIN CLASS_ON_EVAL CLASS_ON_TEST */

/* for plot commands, to allow them to not be shown */
#define SHOW_GRAPHICS 1
#define NO_GRAPHICS 0

static char *defaults_filename = ".lnkmaprc";

int debug;

#define MAXSTR (256)
#define MAXLABEL (1024)

static int enable_plotting_flag;
static int auto_inc;
static int cross_valid_on;
static int store_only;
static int epoch_plots;

/* static GLOBALS internal to this file */
static char *shell_fname="map.run", shell_ftmp[MAXSTR];
static char shell_full_path[MAXSTR];
static char *log_fname="map.log", log_ftmp[MAXSTR];
static char log_full_path[MAXSTR];
static char *screen_fname="map.screen", screen_ftmp[MAXSTR];
static char screen_full_path[MAXSTR];

static char run_time_system[MAXSTR]="";
static char ext_eval[MAXSTR] = ".eval";
static char ext_test[MAXSTR] = ".test";
static char ext_train[MAXSTR] = ".train";
static char ext_cv[MAXSTR] = ".cv";
static char norm_filename[MAXSTR];
static char cv_filename[MAXSTR];
#define EXT_DESC ".defaults"

static char command_line[1024] = "";
static char chartmp[MAXSTR];

static char data_fname_base[MAXSTR], error_fname_base[MAXSTR];
static char tmp_data_fname_space[MAXSTR], tmp_error_fname_space[MAXSTR];
static char tmp_desc_fname_space[MAXSTR];
static char base_ftmp[MAXSTR];
static char param_ftmp[MAXSTR], pid_ftmp[MAXSTR];
static char exp_path_val[MAXSTR];
static char data_path_val[MAXSTR];
static char data_path_full[MAXSTR];
static char exp_name_val[MAXSTR];
static char exp_label_val[MAXSTR];
static int cur_com_pgrp = 0;
static char working_directory[MAXPATHLEN];

/* statics for all windows */
extern gmap_window1_objects	*gmap_window1;
extern gmap_plot_pop_objects	*gmap_plot_pop;
extern gmap_map_plot_objects	*gmap_map_plot;
extern gmap_clus_plot_objects	*gmap_clus_plot;
extern gmap_norm_plot_objects	*gmap_norm_plot;
extern gmap_cost_plot_objects	*gmap_cost_plot;
extern gmap_file_pop_objects	*gmap_file_pop;
extern gmap_feature_pop_objects	*gmap_feature_pop;

/* Macros for storing only plot commands
   or whole experiment and all plot files
   Used as arguments to store_commands */
#define PLOT_ONLY 0
#define EXP_AND_PLOT 1

/* macros for feature selection window */
#define ALL 0
#define FIRST_N 1
#define MANUAL 2
#define FOR 3
#define BACK 4
#define FOR_BACK 5

/* Sub-routine declarations that we need at the top*/
char *make_net_string();
char *make_labels_string();
void g_set_cvtest_flag();
void g_set_cv_filename();
void set_data_file();
void set_data_filename();
void set_experiment_label();
void set_experiment_number();
void set_standard_data_set();
void set_description_values();
void store_norm_command();
void gnorm_store_run_command();
void g_defaults_write();
void set_select_type();
void g_set_normalize_filename();

void change_data_file_processing()
{
    xv_set (gmap_file_pop->data_file, PANEL_VALUE, data_fname_base, NULL);
    g_set_normalize_filename();
    g_set_cv_filename();
    set_description_values();
}


void command_file_close (shell_file, filename, message)
     char *filename, *message;
     FILE *shell_file;
{
    nn_fclose (shell_file, filename, message);
    
    sprintf (chartmp, "chmod +x %s", filename);
    system (chartmp);
}


FILE * command_file_open (filename, message)
     char *filename, *message;
{
    FILE *shell_file;
    int rv;
    
    shell_file = fopen(filename, "w", message);
    if(shell_file == NULL){
	rv = notice_prompt(gmap_window1->window1, NULL,
			   NOTICE_MESSAGE_STRINGS,
			   "Cannot create file for shell script",
			   filename, NULL,
			   NOTICE_BUTTON_YES, "OK", NULL);
	return NULL;
    }
    fprintf (shell_file, "#!/bin/csh -e\n# %s\n", filename);
    
    return shell_file;
}

void display_experiment_names()
{
    strcpy(base_ftmp, map_params.exp_label);
    strcat(base_ftmp, map_params.classifier_name);
    
    strcpy(param_ftmp, base_ftmp);
    map_params.param_fname = strcat(param_ftmp, ".param");
    xv_set(gmap_file_pop->model_file, PANEL_VALUE,
	   map_params.param_fname, NULL);
    
    strcpy(shell_ftmp, base_ftmp);
    shell_fname = strcat(shell_ftmp, ".run");
    xv_set(gmap_file_pop->shell_file, PANEL_VALUE, shell_fname, NULL);
    strcpy(shell_full_path, path_build(map_params.exp_path, shell_fname));
    
    strcpy(error_fname_base, base_ftmp);
    strcat(error_fname_base, ".err");
    xv_set(gmap_file_pop->error_file, PANEL_VALUE, error_fname_base, NULL);
    
    display_plot_exper_names(base_ftmp);

    strcpy(log_ftmp, base_ftmp);
    log_fname = strcat(log_ftmp, ".log");
    xv_set(gmap_file_pop->log_file, PANEL_VALUE, log_fname, NULL);
    strcpy(log_full_path, path_build(map_params.exp_path, log_fname));

    strcpy(screen_ftmp, base_ftmp);
    screen_fname = strcat(screen_ftmp, ".screen");
    xv_set(gmap_file_pop->screen_file, PANEL_VALUE, screen_fname, NULL);
    strcpy(screen_full_path, path_build(map_params.exp_path, screen_fname));
    
    strcpy(pid_ftmp, base_ftmp);
    map_params.pid_fname=strcat(pid_ftmp, ".pid");
}

void display_params()
{
    void display_plot_params();

    char *value;

    xv_set(gmap_window1->normalize, PANEL_VALUE, map_params.normalize, NULL);
    xv_set(gmap_window1->norm_file, PANEL_VALUE, map_params.norm_fname,NULL);
    xv_set(gmap_window1->experiment_name, PANEL_VALUE, exp_name_val, NULL);
    xv_set(gmap_window1->experiment_number, PANEL_VALUE, exp_number, NULL);
    xv_set (gmap_window1->auto_inc, PANEL_VALUE, auto_inc, NULL);
    xv_set (gmap_window1->random_flag, PANEL_VALUE,
	    map_params.random_flag, NULL);
    xv_set (gmap_window1->seed, PANEL_VALUE, map_params.seed, NULL);
    xv_set (gmap_window1->store_only, PANEL_VALUE, store_only, NULL);
    xv_set (gmap_window1->train_select1, PANEL_VALUE, run_options & 1, NULL);
    xv_set (gmap_window1->train_select2, PANEL_VALUE, (run_options & 2) >> 1,
	    NULL);
    xv_set (gmap_window1->train_select3, PANEL_VALUE, run_options >> 2, NULL);
    xv_set (gmap_window1->enable_plotting, PANEL_VALUE,
	    enable_plotting_flag, NULL);
    xv_set (gmap_window1->cv_from_file, PANEL_VALUE,
	    map_params.cv_from_file, NULL);
    xv_set (gmap_window1->ext_cv, PANEL_VALUE, ext_cv, NULL);
    xv_set (gmap_window1->cross_valid, PANEL_VALUE,
	    map_params.cross_valid, NULL);
    g_set_cvtest_flag(cross_valid_on);
    xv_set (gmap_window1->cvtest_flag, PANEL_VALUE, cross_valid_on, NULL);
    xv_set (gmap_window1->epoch_plots, PANEL_VALUE, epoch_plots, NULL);
    xv_set (gmap_window1->e_per_plot, PANEL_VALUE, e_per_plot, NULL);
    if(epoch_plots)
      xv_set(gmap_window1->e_per_plot, PANEL_INACTIVE, FALSE, NULL);
    else
      xv_set(gmap_window1->e_per_plot, PANEL_INACTIVE, TRUE, NULL);

    xv_set(gmap_file_pop->experiment_path, PANEL_VALUE,
	   map_params.exp_path, NULL);
    xv_set(gmap_file_pop->model_file, PANEL_VALUE,map_params.param_fname,NULL);
    xv_set(gmap_file_pop->error_file, PANEL_VALUE, error_fname_base, NULL);
    xv_set(gmap_file_pop->shell_file, PANEL_VALUE, shell_fname, NULL);
    xv_set(gmap_file_pop->log_file, PANEL_VALUE, log_fname, NULL);
    xv_set(gmap_file_pop->screen_file, PANEL_VALUE, screen_fname, NULL);
    xv_set(gmap_file_pop->debug_verbosity,PANEL_VALUE, map_params.debug, NULL);
    xv_set(gmap_file_pop->report_verbosity, PANEL_VALUE,
	   map_params.verbosity, NULL);
    xv_set(gmap_file_pop->verror, PANEL_VALUE, map_params.verror, NULL);
    xv_set(gmap_file_pop->run_time_system, PANEL_VALUE,
	   map_params.run_time_system, NULL);
    xv_set(gmap_file_pop->data_path, PANEL_VALUE, data_path_val, NULL);
    if(standard_data_set_value > 0){
	xv_set(gmap_file_pop->standard_data_list, PANEL_LIST_SELECT,
	       standard_data_set_value, TRUE, NULL);
	xv_set(gmap_file_pop->add_data, PANEL_INACTIVE, TRUE, NULL);
	xv_set(gmap_file_pop->delete_data, PANEL_INACTIVE, FALSE, NULL);
    } else {
	xv_set(gmap_file_pop->delete_data, PANEL_INACTIVE, TRUE, NULL);
	xv_set(gmap_file_pop->add_data, PANEL_INACTIVE, FALSE, NULL);
    }
    xv_set(gmap_file_pop->data_file, PANEL_VALUE, map_params.data_fname, NULL);
    xv_set(gmap_file_pop->ext_eval, PANEL_VALUE, ext_eval, NULL);
    xv_set(gmap_file_pop->ext_test, PANEL_VALUE, ext_test, NULL);
    xv_set(gmap_file_pop->ext_train, PANEL_VALUE, ext_train, NULL);
    xv_set(gmap_file_pop->npatterns_train, PANEL_VALUE, npatterns_train, NULL);
    xv_set(gmap_file_pop->npatterns_eval, PANEL_VALUE, npatterns_eval, NULL);
    xv_set(gmap_file_pop->npatterns_test, PANEL_VALUE, npatterns_test, NULL);
    xv_set(gmap_file_pop->n_inputs, PANEL_VALUE, map_params.nraw, NULL);
    xv_set(gmap_file_pop->n_outputs, PANEL_VALUE, map_params.noutputs, NULL);
    xv_set (gmap_file_pop->labels, PANEL_VALUE,
	    make_labels_string (map_params.labels), NULL);
    xv_set(gmap_feature_pop->select_type, PANEL_VALUE, select_type, NULL);
    xv_set(gmap_feature_pop->select_n, PANEL_VALUE, map_params.ninputs, NULL);
    value = make_net_string(&map_params.features[0]);
    xv_set(gmap_feature_pop->feature_list, PANEL_VALUE, value, NULL);
    set_select_type(select_type);

    display_plot_params();
}	

void execute_command_file(filename)
     char *filename;
{
    sprintf (chartmp, "csh %s&", filename);
    cur_com_pgrp = nn_system (chartmp);
}


int file_exists (filename)
     char *filename;
{
    FILE *file;
    int return_val;
    
    if ((file = fopen(filename, "r", "for file existence check")) == NULL) {
    	return_val = FALSE;
    } else {
    	return_val = TRUE;
	nn_fclose(file, filename, "for file existence check");
    }
    return return_val;
}

void g_init ()
{
    void display_params();
    void set_algorithm();

    g_init_plot();
    xv_set(gmap_window1->cvtest_flag, PANEL_VALUE, FALSE, NULL);
    g_set_cvtest_flag(FALSE);

    set_experiment_label();
    display_params();
    
    set_algorithm (algorithm); /* must be called after display_params */
    
    /* must be called after set_algorithm */
    if (standard_data_set_value > 0)
      set_standard_data_set (standard_data_set_value);
    else 
      set_data_file (data_fname_base);

    if(getwd(working_directory) == 0){
	warning("couldn't get directory name. %s", working_directory);
    }
    xv_set (gmap_file_pop->wdir_name, PANEL_VALUE, working_directory, NULL);
    select_type = ALL;
}

/*
 * Notify callback function for `plot_only'.
 */
void plot_only_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int tmp;
    
    if (overwrite_ok(shell_full_path)) {
	/* make sure plotting is enabled (not permanently) */
	tmp = enable_plotting_flag;
	enable_plotting_flag = TRUE;

	if(store_commands(shell_full_path, PLOT_ONLY)) return;

	enable_plotting_flag = tmp;
	if(!store_only){
	    if(chdir(map_params.exp_path))
	      warning("couldn't change directory to %s", map_params.exp_path);
	    execute_command_file(shell_fname);
	    if(chdir(working_directory))
	      warning("couldn't change directory to %s", working_directory);
	}
    }
}

/*
 * Menu handler for `quit_menu (CONFIRM)'.
 */
Menu_item confirm_handler(item, op)
     Menu_item	item;
     Menu_generate	op;
{
    if(op == MENU_NOTIFY){
	exit_nnsubs();			       
	exit(0);
    }
    return item;
}

void g_run ()
{
    if (overwrite_ok(shell_full_path)) {
	if(store_commands(shell_full_path, EXP_AND_PLOT)) return;
        
	if(!store_only){
	    if(chdir(map_params.exp_path))
	      warning("couldn't change directory to %s", map_params.exp_path);
	    execute_command_file(shell_fname);
	    if(chdir(working_directory))
	      warning("couldn't change directory to %s", working_directory);
	    /* save latest pid filename for stop later */
        }
	if (auto_inc) 
	  set_experiment_number (exp_number + 1);
    }
}


/*
 * Notify callback function for `run_store_mode'.
 */
void store_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    store_only = value;
}


/*
 * Notify callback function for `save_defaults'.
 */
void save_defaults_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char filename[MAXSTR];
    
    strcpy (filename, path_build(getenv("HOME"), defaults_filename));
    if (overwrite_ok (filename)) {
	g_defaults_write(filename);
	printf ("\nLNKmap default values saved in %s\n", filename);
    }
}

/*
 * Notify callback function for `auto_inc'.
 */
void auto_inc_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    auto_inc = value;
}

/*
 * Notify callback function for `shell_file'.
 */
Panel_setting shell_fname_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    shell_fname = value;
    strcpy(shell_full_path, path_build(map_params.exp_path, shell_fname));
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `cross_valid'.
 */
Panel_setting cross_valid_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);
    
    map_params.cross_valid = value;
    return panel_text_notify(item, event);
}

void g_set_cvtest_flag(value)
int value;
{
    /* because gmap_window1->cross_valid is >= 2
       and map_params.cross_valid is set
       whenever cvtest_flag is set, cross_valid > 1 is a valid way
       to test for cv_test selected.  This is confusing because cv_from_file
       being set should also trigger cvtest_flag is set reactions, but
       is not tested */
    cross_valid_on = value;
    if(value){	    
	map_params.cross_valid = xv_get (gmap_window1->cross_valid, PANEL_VALUE);
	xv_set(gmap_window1->cv_from_file, PANEL_INACTIVE, FALSE, NULL);
	if(map_params.cv_from_file){
	    xv_set(gmap_window1->ext_cv, PANEL_INACTIVE, FALSE, NULL);
	} else {
	    xv_set (gmap_window1->cross_valid, PANEL_INACTIVE, FALSE, NULL);
	}
    } else {
	map_params.cross_valid = 0;
	xv_set (gmap_window1->cross_valid, PANEL_INACTIVE, TRUE, NULL);
	xv_set (gmap_window1->cv_from_file, PANEL_INACTIVE, TRUE, NULL);
	xv_set (gmap_window1->ext_cv, PANEL_INACTIVE, TRUE, NULL);
    }
}

/*
 * Notify callback function for `cv_from_file'.
 */
void cv_from_file_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    
    if(value){
	map_params.cv_from_file = 1;
	xv_set (gmap_window1->ext_cv, PANEL_INACTIVE, FALSE, NULL);
	xv_set (gmap_window1->cross_valid, PANEL_INACTIVE, TRUE, NULL);
    } else {
	map_params.cv_from_file = 0;
	xv_set (gmap_window1->ext_cv, PANEL_INACTIVE, TRUE, NULL);
	xv_set (gmap_window1->cross_valid, PANEL_INACTIVE, FALSE, NULL);
    }
}

/*
 * Notify callback function for `ext_cv'.
 */
Panel_setting ext_cv_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    strcpy (ext_cv, value);
    xv_set (gmap_window1->ext_cv, PANEL_VALUE, ext_cv, NULL);
    g_set_cv_filename();
    return panel_text_notify(item, event);
}

void g_set_cv_filename()
{
    strcpy (cv_filename, data_fname_base);
    strcat (cv_filename, ext_train);
    strcat (cv_filename, ext_cv);
    map_params.cv_fname = cv_filename;
}

void set_data_file (value)
     char *value;
{
    xv_set(gmap_file_pop->standard_data_list, PANEL_LIST_SELECT,
	   standard_data_set_value, FALSE, NULL);
    standard_data_set_value = -1;
    xv_set(gmap_file_pop->delete_data, PANEL_INACTIVE, TRUE, NULL);
    xv_set(gmap_file_pop->add_data, PANEL_INACTIVE, FALSE, NULL);
    set_data_filename(value);
}

void set_data_filename (value)
     char *value;
{
    if (value == NULL) data_fname_base[0] = '\0';
    else strcpy (data_fname_base, value);
    map_params.data_fname = data_fname_base;
    change_data_file_processing();
}

/*
 * Notify callback function for `data_path'.
 */
Panel_setting dpath_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    strcpy(data_path_val, value);
    if(value[0] == '/')
      strcpy(data_path_full, value);
    else {
	chdir(value);
	getwd(data_path_full);
	chdir(working_directory);
    }
    map_params.data_path = data_path_full;
    change_data_file_processing();
    
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `debug_verbosity'.
 */
Panel_setting debug_verbosity_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);
    
    map_params.debug = value;
    debug = value;
    
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `error_file'.
 */
Panel_setting ename_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    strcpy(error_fname_base, value);
    return panel_text_notify(item, event);
}

void set_experiment_label ()
{
    void g_set_cluster_names();

    sprintf (exp_label_val, "%s%d", exp_name_val, exp_number);
    map_params.exp_label = exp_label_val;
    g_set_cluster_names();
}

/*
 * Notify callback function for `experiment_name'.
 */
Panel_setting experiment_name_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    strcpy(exp_name_val, value);
    set_experiment_label();
    display_experiment_names();
    xv_set (gmap_window1->experiment_name, PANEL_VALUE, exp_name_val, NULL);
    
    return panel_text_notify(item, event);
}

void set_experiment_number (value)
     int value;
{
    exp_number = value;
    set_experiment_label();
    display_experiment_names();
    xv_set (gmap_window1->experiment_number, PANEL_VALUE, exp_number, NULL);
}

/*
 * Notify callback function for `experiment_path'.
 */
Panel_setting experiment_path_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    map_params.exp_path = value;
    /* update the shell and log file paths */
    strcpy(shell_full_path, path_build(map_params.exp_path, shell_fname));
    strcpy(log_full_path, path_build(map_params.exp_path, log_fname));
    strcpy(screen_full_path, path_build(map_params.exp_path, screen_fname));
    
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `ext_eval'.
 */
Panel_setting ext_eval_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    strcpy (ext_eval, value);
    xv_set (gmap_file_pop->ext_eval, PANEL_VALUE, ext_eval, NULL);
    change_data_file_processing();
    
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `ext_test'.
 */
Panel_setting ext_test_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    strcpy (ext_test, value);
    xv_set (gmap_file_pop->ext_test, PANEL_VALUE, ext_test, NULL);
    change_data_file_processing();
    
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `ext_train'.
 */
Panel_setting ext_train_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    strcpy (ext_train, value);
    xv_set (gmap_file_pop->ext_train, PANEL_VALUE, ext_train, NULL);
    change_data_file_processing();
    
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `epoch_plots'.
 */
void epoch_plots_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    epoch_plots = value;
    if(value)
      xv_set(gmap_window1->e_per_plot, PANEL_INACTIVE, FALSE, NULL);
    else
      xv_set(gmap_window1->e_per_plot, PANEL_INACTIVE, TRUE, NULL);
}

/*
 * Notify callback function for `e_per_plot'.
 */
Panel_setting
  e_per_plot_handler(item, event)
Panel_item	item;
Event		*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);
    
    e_per_plot = value;
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `model_file'.
 */
Panel_setting param_fname_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    map_params.param_fname = value;
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `labels'.
 */
Panel_setting labels_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    static char tmp[NNMAX_LINE];
    
    nnargs_get_string_list (strcpy(tmp, value), gnn_labels, NNARGS_MAX_LABELS);
    map_params.labels = gnn_labels;
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `log_file'.
 */
Panel_setting log_file_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    log_fname = value;
    strcpy(log_full_path, path_build(map_params.exp_path, log_fname));
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `screen_file'.
 */
Panel_setting screen_file_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    screen_fname = value;
    strcpy(screen_full_path, path_build(map_params.exp_path, screen_fname));
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `n_inputs'.
 */
Panel_setting n_inputs_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);
    void g_set_description_values();

    map_params.nraw = value;
    set_select_type(select_type);
    g_set_description_values ();
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `n_outputs'.
 */
Panel_setting n_outputs_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);
    
    map_params.noutputs = value;
    g_set_description_values ();
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `npatterns_eval'.
 */
Panel_setting npatterns_eval_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);
    
    npatterns_eval = value;
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `npatterns_test'.
 */
Panel_setting npatterns_test_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);
    
    npatterns_test = value;
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `npatterns_train'.
 */
Panel_setting npatterns_train_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);
    
    npatterns_train = value;
    map_params.n_patterns_train = npatterns_train;
    return panel_text_notify(item, event);
}

void g_set_normalize_filename()
{
    void set_plot_descriptions();
    static char *norm_string[3] = {"none", "simple", "pca"};
    
    strcpy (norm_filename, data_fname_base);
    strcat (norm_filename, ".norm.");
    strcat (norm_filename, norm_string[map_params.normalize]);
    map_params.norm_fname = norm_filename;
    xv_set (gmap_window1->norm_file, PANEL_VALUE, map_params.norm_fname, NULL);
    set_plot_descriptions();
}

void set_normalize(value)
     int value;
{
    map_params.normalize = value;
    g_set_normalize_filename();
}


/*
 * Notify callback function for `norm_file'.
 */
Panel_setting norm_fname_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    strcpy(norm_filename, value);
    map_params.norm_fname = norm_filename;
    return panel_text_notify(item, event);
}
/*
 * Notify callback function for `random_flag'.
 */
void random_flag_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    map_params.random_flag = value; 
}

/*
 * Notify callback function for `run_time_system'.
 */
Panel_setting run_time_system_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    strcpy (run_time_system, value);
    map_params.run_time_system = run_time_system;
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `seed'.
 */
Panel_setting seed_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);
    
    map_params.seed = value;
    return panel_text_notify(item, event);
}

void set_standard_data_set (value)
     int value;
{
    standard_data_set_value = value;
    set_data_filename ((char *) xv_get(gmap_file_pop->standard_data_list,
				       PANEL_LIST_STRING, value, NULL));
}

void set_train(value)
     int value;
{
    run_options = value;
}

/*
 * Notify callback function for `train_select1'.
 */
void train1_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    /* turn off lowest bit: 2, OR with value */
    set_train((run_options & 14) | value);
}

/*
 * Notify callback function for `train_select2'.
 */
void train2_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    /* turn off second to lowest bit: 2, OR with value */
    set_train((run_options & 13) | value << 1);
}

/*
 * Notify callback function for `train_select3'.
 */
void train3_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    /* turn off top two bits: 4+8, OR with value */
    set_train((run_options & 3) | value << 2);
}

/*
 * Notify callback function for `report_verbosity'.
 */
Panel_setting report_verbosity_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);
    
    map_params.verbosity = value;
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `verror'.
 */
Panel_setting verror_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int value = (int) xv_get(item, PANEL_VALUE);

    map_params.verror = value;
    return panel_text_notify(item, event);
}

void display_verror(value)
     int value;
{
    xv_set (gmap_file_pop->verror, PANEL_VALUE, value, NULL);
}
/*
 * Notify callback function for `wdir_name'.
 * Do not allow chages in working directory
 */
Panel_setting gmap_wdir_name_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    xv_set(gmap_file_pop->wdir_name, PANEL_VALUE, working_directory, NULL);
    
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `start_run'.
 */
void start_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    map_params.create_flag = TRUE;
    g_run();
}

/*
 * Notify callback function for `Continue'.
 */
void continue_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    map_params.create_flag = FALSE;
    g_run();
}

/*
 * Notify callback function for `stop_run'.
 */
void stop_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    if(cur_com_pgrp != 0){
	killpg (cur_com_pgrp, SIGKILL);
	printf("\nKilled the most recently started run file\n");
	cur_com_pgrp = 0;
    }
}

char * make_net_string (array)
     int array[];
{
    static char dummy[256];
    int i;
    
    *dummy = NULL;
    if (array[0] > 0) {
	sprintf (dummy, "%d", array[1]);
    }
    if (array[0] > 1) {
	for(i=2; i<= array[0] && i <= NNARGS_MAX_NET; i++)
	  sprintf (dummy + strlen(dummy), ",%d", array[i]);
    }
    return dummy;
}

char * make_float_string (array)
     float array[];
{
    static char dummy[256];
    int i;
    
    *dummy = NULL;
    if (array[0] > 0) {
	sprintf (dummy, "%g", array[1]);
    }
    if (array[0] > 1) {
	for(i=2; i<= array[0] && i <= NNARGS_MAX_LABELS; i++)
	  sprintf (dummy + strlen(dummy), ",%g", array[i]);
    }
    return dummy;
}

char * make_labels_string (array)
     char *array[];
{
    static char dummy[256];
    int i;
    
    *dummy = NULL;
    if (atoi(array[0]) > 0) {
	sprintf (dummy, "%s", array[1]);
    }
    if (atoi(array[0]) > 1) {
	for(i=2; i<= atoi(array[0]) && i <= NNARGS_MAX_LABELS; i++)
	  sprintf (dummy + strlen(dummy), ",%s", array[i]);
    }
    return dummy;
}

int overwrite_ok (filename)
     char *filename;
{
    int result, return_value;
    
    return_value = TRUE;
    if (file_exists(filename)) {
	result = notice_prompt(gmap_window1->window1, NULL,
			       NOTICE_MESSAGE_STRINGS,
			       "Shell file exists: OK to overwrite?",
			       NULL,
			       NOTICE_BUTTON_YES,	"Overwrite",
			       NOTICE_BUTTON_NO,	"Cancel",
			       NULL);
	if(result != NOTICE_YES) return_value = FALSE;
    }
    return return_value;
}

void set_description_values ()
{
    /* read all description files when filename changes */
    npatterns_test = set_description_values_specific (ext_test);
    xv_set (gmap_file_pop->npatterns_test, PANEL_VALUE, npatterns_test, NULL);
    
    npatterns_eval = set_description_values_specific (ext_eval);
    xv_set (gmap_file_pop->npatterns_eval, PANEL_VALUE, npatterns_eval, NULL);
    
    npatterns_train = set_description_values_specific (ext_train);
    xv_set (gmap_file_pop->npatterns_train, PANEL_VALUE, npatterns_train, NULL);
    map_params.n_patterns_train = npatterns_train;
    
    set_select_type(select_type);
    g_set_description_values ();
}

int set_description_values_specific (exten)
     char *exten;
{
    char desc_fullname[MAXSTR], desc_filename[MAXSTR];
    int nargc, i;
    FILE *desc_file;
    char *new_str;
    
    strcpy(desc_filename, data_fname_base);
    strcat(desc_filename, exten);
    strcat(desc_filename, EXT_DESC);
    map_params.desc_fname = desc_filename;
    strcpy(desc_fullname, path_build(map_params.data_path, 
				     map_params.desc_fname));
    if ((desc_file = fopen(desc_fullname, "r")) == NULL) {
	gnn_n_patterns = 0;
    } else {
	nargc= nnargs_from_file(desc_file, desc_fullname,
				gnn_argtab, GNN_TABSIZE);
	nn_fclose(desc_file, desc_fullname, "for reading data description");
	map_params.nraw = gnn_ninputs;
	map_params.noutputs = gnn_noutputs;
	gnn_noutputs = atoi(gnn_labels[0]);
	map_params.labels = gnn_labels;
	/* copy strings, the ones in gnn_labels are in argv */
	/* this is memory that gets lost */
	for(i = 0; i <= gnn_noutputs; i++){
	    new_str =  nn_malloc(strlen(gnn_labels[i])+1, "for label list");
	    strcpy(new_str, gnn_labels[i]);
	    map_params.labels[i] = new_str;
	}
	xv_set (gmap_file_pop->labels, PANEL_VALUE,
		make_labels_string (gnn_labels), NULL);
	xv_set (gmap_file_pop->n_inputs, PANEL_VALUE, gnn_ninputs, NULL);
	xv_set (gmap_file_pop->n_outputs, PANEL_VALUE, gnn_noutputs, NULL);
	g_set_trial_average(gnn_n_patterns);
    }
    
    return (gnn_n_patterns);
}


void set_float_string (item, format, value)
     unsigned int item;
     char *format;
     float value;
{
    char *str[10];
    
    sprintf (str, format, value);
    xv_set (item, PANEL_VALUE, str, NULL);
}

float set_atof_value(str, name, item, value)
     char *str, *name;
     unsigned int item;
     float value;
{
    float new_val;
    
    if(sscanf (str, "%f", &new_val) == 1){
	return new_val;
    } else {
	warning("set_atof_value -- unable to set %s to %s",
		name, str);
	set_float_string(item, "%5.2g", value);
	return value;
    }
}

float set_atof_range_value(str, name, item, value, low, high)
     char *str, *name;
     unsigned int item;
     float value, low, high;
{
    float new_val;
    
    if(sscanf (str, "%f", &new_val) == 1){
	if(new_val >= low && new_val <= high)
	  return new_val;
	else {
	    warning("set_atof_range_value -- new %s (%f) out of range (%f,%f)",
		    name, new_val, low, high);
	    set_float_string(item, "%5.2g", value);
	    return value;
	}
    } else {
	warning("set_atof_range_value -- unable to set %s to %s",
		name, str);
	set_float_string(item, "%5.2g", value);
	return value;
    }
}

float set_atof_low_value(str, name, item, value, low)
     char *str, *name;
     unsigned int item;
     float value, low;
{
    float new_val;
    
    if(sscanf (str, "%f", &new_val) == 1){
	if(new_val >= low)
	  return new_val;
	else {
	    warning("set_atof_low_value -- new %s (%f) < (%f)",
		    name, new_val, low);
	    set_float_string(item, "%5.2g", value);
	    return value;
	}
    } else {
	warning("set_atof_low_value -- unable to set %s to %s",
		name, str);
	set_float_string(item, "%5.2g", value);
	return value;
    }
}

int store_commands (filename, run_flag)
     char *filename;
     int run_flag;/* stores only plot command if run_flag == PLOT_ONLY */
{
    FILE *shell_file;
    void set_plot_flags(), store_plot_commands();
    void store_commands_specific();
    char tmp_save;
    
    shell_file = command_file_open (filename, "for run shell file");
    if(shell_file == NULL) return -1;
    
    strcpy(exp_path_val, map_params.exp_path);
    map_params.exp_path = "$loc";
    
    if (enable_plotting_flag) set_plot_flags();
    
    /* print out variable necessary for running remotely */
    fprintf(shell_file, "set loc=`pwd`\n");
    if (run_flag != PLOT_ONLY) {
	/* generate normalization file if necessary */
	if (map_params.normalize > 0)
	  if (!file_exists (path_build(map_params.data_path,
				       map_params.norm_fname))) {
	      map_params.n_patterns = npatterns_train;
	      if(!alg_is_norm())
		store_norm_command (shell_file, ext_train);
	  }
    }
    tmp_save = map_params.cross_valid;
    map_params.cross_valid = 0;	/* temporarily disable cross_validation */
    
    if (run_options & TRAIN) {			/* train selected */
	map_params.n_patterns = npatterns_train;
	if(epoch_plots){
	    if(store_epochs_train(shell_file, run_flag, TRAIN, ext_train) == 0)
	      /* not enough epochs to do more than one training run */
	      store_commands_specific (shell_file, run_flag, TRAIN,
				       ext_train, SHOW_GRAPHICS);
	} else
	  store_commands_specific (shell_file, run_flag, TRAIN, ext_train,
				   SHOW_GRAPHICS);
    }
    if(run_options & CLASS_TRAIN) {
	map_params.n_patterns = npatterns_train;
	store_commands_specific (shell_file, run_flag, CLASS, ext_train,
				 SHOW_GRAPHICS);
    }
    if (run_options & CLASS_EVAL) {
	map_params.n_patterns = npatterns_eval;
	store_commands_specific (shell_file, run_flag, CLASS, ext_eval,
				 SHOW_GRAPHICS);
    }
    if (run_options & CLASS_TEST) {
	map_params.n_patterns = npatterns_test;
	store_commands_specific (shell_file, run_flag, CLASS, ext_test,
				 SHOW_GRAPHICS);
    }
    map_params.cross_valid = tmp_save;	/* restore cross validation flag */
    
    if (enable_plotting_flag) {
	store_plot_commands(shell_file, error_fname_base, ext_train);
    }
    if (map_params.cross_valid > 1) {		/* cross validate selected */
	map_params.n_patterns = npatterns_train;
	store_commands_specific (shell_file, run_flag, CROSS_VALID,
				 ext_train, SHOW_GRAPHICS);
    }
    
    map_params.exp_path = exp_path_val;
    
    /* tell the log file where we are */
    /* telling sooner risks the log file being overwritten */
    fprintf(shell_file, "echo \"current directory:\" >> %s\n", log_fname);
    fprintf(shell_file, "echo $loc >> %s\n", log_fname);
    
    command_file_close (shell_file, filename, "for run shell file");

    /* store screen file */
    g_defaults_write(screen_full_path);

    return 0;
}


void store_commands_specific (shell_file, run_flag, run_option, exten,
			      graphics)
     FILE *shell_file;
     int run_flag;/* stores only plot command if run_flag == PLOT_ONLY */
     int run_option;
     char *exten;
     int graphics;
{
    void g_store_run_command(), store_plot_commands(),
	store_alg_plot_commands();

    if (map_params.debug>0)
      printf("store_commands_specific: %d %d %d %s\n",
	     run_flag, run_option, PLOT_ONLY, exten);
    
    strcpy(tmp_data_fname_space, data_fname_base);
    strcat(tmp_data_fname_space, exten);
    map_params.data_fname = tmp_data_fname_space;
    
    strcpy(tmp_desc_fname_space, tmp_data_fname_space);
    strcat(tmp_desc_fname_space, EXT_DESC);
    map_params.desc_fname = tmp_desc_fname_space;
    
    strcpy(tmp_error_fname_space, error_fname_base);
    strcat(tmp_error_fname_space, exten);
    map_params.error_fname = tmp_error_fname_space;
    
    if (run_option == TRAIN) map_params.train_flag = TRUE;
    else 		     map_params.train_flag = FALSE;
    
    g_set_description_values();
    
    if(run_flag != PLOT_ONLY){
	g_store_run_command (shell_file, log_fname);
    }
    if (enable_plotting_flag && run_option != CROSS_VALID) {
	store_alg_plot_commands(shell_file, run_option, graphics);
    }
    map_params.create_flag = FALSE; /* ensure log file contains all of run */
    map_params.data_fname = data_fname_base;	/* reset data_fname */
}

void store_norm_command (shell_file, exten)
     FILE *shell_file;
     char *exten;
{
    /* temporarily change specific file name values */
    strcpy(tmp_data_fname_space, data_fname_base);
    strcat(tmp_data_fname_space, exten);
    map_params.data_fname = tmp_data_fname_space;
    
    strcpy(tmp_desc_fname_space, map_params.data_fname);
    strcat(tmp_desc_fname_space, EXT_DESC);
    map_params.desc_fname = tmp_desc_fname_space;
    
    gnorm_store_run_command (shell_file);

    map_params.data_fname = data_fname_base;
}

int store_epochs_train(shell_file, run_flag, run_option, exten)
     FILE *shell_file;
     int run_flag;
     int run_option;
     char *exten;
{
    int epochs;
    void setup_epoch_plot_calls(), finish_epoch_plot_calls();

    /* get the number of epochs we are supposed to do */
    epochs = g_get_epochs();
    if(epochs <= e_per_plot)
      return 0;  /* can only do one pass, do it normally */
    
    /* set up variables in the shell file */
    setup_epoch_plot_calls(shell_file);
    fprintf(shell_file, "set epochs_left = %d\n", epochs);
    
    /* set variables to do the first training run */
    g_set_epochs(e_per_plot);
    
    store_commands_specific(shell_file, run_flag, run_option, exten,
			    NO_GRAPHICS);
    
    /* the rest of the training will be done in continue mode */
    map_params.create_flag = 0;
    
    /* create a call to display the set of plots */
    fprintf(shell_file, "set plotnum_list = $plotnum\n");
    
    fprintf(shell_file, "@ epochs_left -= %d\n", e_per_plot);
    fprintf(shell_file, "@ plotnum++\n");
    
    /* loop through most of the training epochs */
    fprintf(shell_file, "while( $epochs_left > %d)\n", e_per_plot);
    fprintf(shell_file, "set pfname = \"$plotfile$plotnum\"\n");
    
    store_commands_specific(shell_file, run_flag, run_option, exten,
			    NO_GRAPHICS);
    
    fprintf(shell_file, "set plotnum_list = $plotnum_list,$plotnum\n");
    fprintf(shell_file, "@ epochs_left -= %d\n", e_per_plot);
    fprintf(shell_file, "@ plotnum++\n");
    fprintf(shell_file, "end\n");
    
    /* do the last training call, which may want fewer epochs */
    fprintf(shell_file, "set pfname = \"$plotfile$plotnum\"\n");
    
    if(epochs%e_per_plot != 0)
      g_set_epochs(epochs%e_per_plot);
    store_commands_specific(shell_file, run_flag, run_option, exten,
			    NO_GRAPHICS);
    
    fprintf(shell_file, "set plotnum_list = $plotnum_list,$plotnum\n");
    
    /* display the plots */
    finish_epoch_plot_calls(shell_file);
    g_set_epochs(epochs);
    
    return(1);
}

void activate_test_selects(activate_flag)
     int activate_flag;
{
    int temp, value;
    if(activate_flag){
	xv_set(gmap_window1->train_select2, PANEL_INACTIVE, FALSE, NULL);
	xv_set(gmap_window1->train_select3, PANEL_INACTIVE, FALSE, NULL);
	xv_set(gmap_file_pop->ext_eval, PANEL_INACTIVE, FALSE, NULL);
	xv_set(gmap_file_pop->npatterns_eval, PANEL_INACTIVE, FALSE, NULL);
	xv_set(gmap_file_pop->ext_test, PANEL_INACTIVE, FALSE, NULL);
	xv_set(gmap_file_pop->npatterns_test, PANEL_INACTIVE, FALSE, NULL);
	
	/* set run_options to values shown on screen */
	value = xv_get(gmap_window1->train_select1, PANEL_VALUE, NULL);
	temp = xv_get(gmap_window1->train_select2, PANEL_VALUE, NULL);
	value |= (temp << 1);
	temp = xv_get(gmap_window1->train_select3, PANEL_VALUE, NULL);
	value |= (temp << 2);
	set_train(value);
    } else {
	xv_set(gmap_window1->train_select2, PANEL_INACTIVE, TRUE, NULL);
	xv_set(gmap_window1->train_select3, PANEL_INACTIVE, TRUE, NULL);
	xv_set(gmap_file_pop->ext_eval, PANEL_INACTIVE, TRUE, NULL);
	xv_set(gmap_file_pop->npatterns_eval, PANEL_INACTIVE, TRUE, NULL);
	xv_set(gmap_file_pop->ext_test, PANEL_INACTIVE, TRUE, NULL);
	xv_set(gmap_file_pop->npatterns_test, PANEL_INACTIVE, TRUE, NULL);
	
	/* set run_options to train only */
	set_train(run_options & TRAIN);
    }
}

void activate_cross_valid(activate_flag)
     int activate_flag;
{
    if(activate_flag){
	xv_set(gmap_window1->cvtest_flag, PANEL_INACTIVE, FALSE, NULL);
	g_set_cvtest_flag(xv_get(gmap_window1->cvtest_flag, PANEL_VALUE, NULL));
    } else {
	xv_set(gmap_window1->cvtest_flag, PANEL_INACTIVE, TRUE, NULL);
	g_set_cvtest_flag(FALSE);
    }
}

void g_defaults_read (default_fullname)
     char *default_fullname;
{
    FILE *default_file;
    void read_defaults();
    
    default_file = fopen(default_fullname, "r");
    read_defaults(default_file, default_fullname);
    if(default_file != NULL)
      nn_fclose(default_file, default_fullname, "for reading LNKnet defaults");
}

void read_main_defaults(default_file, default_fullname)
     FILE *default_file;
     char *default_fullname;
{
    int nargc, i;
    char *new_str;
    
    if(default_file != NULL)
      nargc=nnargs_from_file(default_file, default_fullname,
			     LNKMAPargtab, LNKMAPTABSIZE);
    /* some of these args are set directly.  see lnkmap_args.h */

    /* algorithm */
    map_params.normalize = normalization_arg;
    strcpy(norm_filename, norm_fname_arg);
    map_params.norm_fname = norm_filename;
    map_params.random_flag = random_flag_arg;
    map_params.seed = seed_arg;
    store_only = store_only_arg;
    /* run_options */
    enable_plotting_flag = enable_plotting_flag_arg;
    cross_valid_on = cross_valid_on_arg;
    map_params.cv_from_file = cv_from_file;
    map_params.cross_valid = cv_nfolds;
    strcpy(ext_cv, ext_cv_arg);
    g_set_cv_filename();
    /* e_per_plot */
    epoch_plots = epoch_plots_arg;
    /* exp_number */
    strcpy(exp_name_val, exp_name_arg);
    auto_inc = auto_inc_arg;
    strcpy(exp_path_val, exp_path_arg);
    map_params.exp_path = exp_path_val;
    strcpy(param_ftmp, fparam_arg);
    map_params.param_fname = param_ftmp;
    strcpy(pid_ftmp, fpid_arg);
    map_params.pid_fname = pid_ftmp;
    strcpy(error_fname_base, ferror_arg);
    strcpy(shell_ftmp, frun_arg);
    shell_fname = shell_ftmp;
    strcpy(shell_full_path, path_build(map_params.exp_path, shell_fname));
    strcpy(log_ftmp, flog_arg);
    log_fname = log_ftmp;
    strcpy(log_full_path, path_build(map_params.exp_path, log_fname));
    strcpy(screen_ftmp, fscreen_arg);
    screen_fname = screen_ftmp;
    strcpy(screen_full_path, path_build(map_params.exp_path, screen_fname));
    map_params.verbosity = verbosity_arg;
    map_params.verror = verror_arg;
    map_params.debug = debug = debug_arg;
    strcpy(run_time_system, run_time_system_arg);
    map_params.run_time_system = run_time_system;
    strcpy(data_path_val, data_path_arg);
    map_params.data_path = data_path_full;
    if(data_path_arg[0] == '/'){
	strcpy(data_path_full, data_path_arg);
    } else {
	getwd(working_directory);
	chdir(data_path_arg);
	getwd(data_path_full);
	chdir(working_directory);
    }
    /* standard_data_set_value */
    copy_data_bases();
    setup_shown_list();
    strcpy(data_fname_base, data_fname_arg);
    map_params.data_fname = data_fname_base;
    strcpy(ext_train, ext_train_arg);
    strcpy(ext_test, ext_test_arg);
    strcpy(ext_eval, ext_eval_arg);
    /* npatterns_train, npatterns_eval, npatterns_test */
    map_params.nraw = nraw_arg;
    map_params.noutputs = noutputs_arg;
    map_params.labels = labels_arg;
    for(i = 0; i <= atoi(labels_arg[0]); i++){
	new_str = nn_malloc(strlen(labels_arg[i])+1, "for label list");
	strcpy(new_str, labels_arg[i]);
	map_params.labels[i] = new_str;
    }
    /*select_type */
    map_params.ninputs = select_n;
    for(i = 0; i <= feature_list[0]; i++)
      map_params.features[i] = feature_list[i];
}

void g_defaults_write(default_fullname)
     char *default_fullname;
{
    FILE *default_file;
    void write_defaults();
    
    if ((default_file = fopen(default_fullname, "w", "defaults file"))
	== NULL) {
	if (map_params.debug>0) 
	  warning("Open error for default parameter file %s.\n",
		  default_fullname);
    }
    write_defaults(default_file, default_fullname);
    nn_fclose(default_file, default_fullname, 
	      "for reading data description");

}

void write_main_defaults (default_file, dfname)
     FILE *default_file;
     char *dfname;
{
    int i;

    /* algorithm */
    normalization_arg = map_params.normalize;
    norm_fname_arg = map_params.norm_fname;
    random_flag_arg = map_params.random_flag;
    seed_arg = map_params.seed;
    store_only_arg = store_only;
    /* run_options */
    enable_plotting_flag_arg = enable_plotting_flag;
    cross_valid_on_arg = cross_valid_on;
    cv_from_file = map_params.cv_from_file;
    cv_nfolds = map_params.cross_valid;
    ext_cv_arg = ext_cv;
    epoch_plots_arg = epoch_plots;
    /* e_per_plot */
    exp_name_arg = exp_name_val;
    /* exp_number */
    auto_inc_arg = auto_inc;
    exp_path_arg = map_params.exp_path;
    fparam_arg = map_params.param_fname;
    fpid_arg = map_params.pid_fname;
    ferror_arg = error_fname_base;
    frun_arg = shell_fname;
    flog_arg = log_fname;
    fscreen_arg = screen_fname;
    verbosity_arg = map_params.verbosity;
    verror_arg = map_params.verror;
    debug_arg = map_params.debug;
    run_time_system_arg = map_params.run_time_system;
    data_path_arg = data_path_val;
    /* standard_data_set_value */
    setup_file_list_arg();
    data_fname_arg = map_params.data_fname;
    ext_train_arg = ext_train;
    ext_eval_arg = ext_eval;
    ext_test_arg = ext_test;
    /* npatterns_train, npatterns_eval, npatterns_test */
    nraw_arg = map_params.nraw;
    noutputs_arg = map_params.noutputs;
    for(i = 0; i <= atoi(map_params.labels[0]); i++)
      labels_arg[i] = map_params.labels[i];
    /* select_type */
    select_n = map_params.ninputs;
    for(i = 0; i <= map_params.features[0]; i++)
      feature_list[i] = map_params.features[i];

    if(default_file != NULL){
	fprintf (default_file, "%s %s", get_program_name(),
		 nnargs_get_flags(LNKMAPargtab, LNKMAPTABSIZE));
    }
}

setup_file_list_arg()
{
    int nrows, i;
    nrows = xv_get(gmap_file_pop->standard_data_list, PANEL_LIST_NROWS, NULL);
    for(i = 0; i < nrows; i++)
      data_bases[i+1] = (char *)
	xv_get(gmap_file_pop->standard_data_list, PANEL_LIST_STRING, i, NULL);
    sprintf(data_bases[0], "%d", nrows);
}

copy_data_bases()
{
    int i, nrows;
    char * new_str;

    nrows = atoi(data_bases[0]);
    for(i = nrows; i > 0; i--){
	new_str = nn_malloc(strlen(data_bases[i])+1, "for scroll list");
	strcpy(new_str, data_bases[i]);
	data_bases[i] = new_str;
    }
}

setup_shown_list()
{
    int i, nrows;

    xv_set(gmap_file_pop->standard_data_list, PANEL_INACTIVE, TRUE, NULL);
    nrows = atoi(data_bases[0]);
    if(nrows != 0){
	/* empty the current list */
	nrows = xv_get(gmap_file_pop->standard_data_list,
		       PANEL_LIST_NROWS, NULL);
	for(i = 0; i < nrows; i++)
	  xv_set(gmap_file_pop->standard_data_list, PANEL_LIST_DELETE, 0, NULL);
    }
    nrows = atoi(data_bases[0]);
    for(i = nrows; i > 0; i--){
	xv_set(gmap_file_pop->standard_data_list,
	       PANEL_LIST_INSERT, 0,
	       PANEL_LIST_STRING, 0, data_bases[i],
	       NULL);
    }
    xv_set(gmap_file_pop->standard_data_list, PANEL_INACTIVE, FALSE, NULL);
}

/*
 * Notify callback function for `enable_plotting'.
 */
void enable_plotting_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    enable_plotting_flag = value;
}

#ifdef SGI
/*
 * Notify callback function for `standard_data_list'.
 */
int standard_data_list_handler(item, string, client_data, op, event)
     Panel_item	item;
     char	*string;
     Xv_opaque	client_data;
     Panel_list_op	op;
     Event	*event;
{
     int	row, nrows;
     void set_standard_data_set();

    switch(op) {
      case PANEL_LIST_OP_SELECT:
	nrows = xv_get(item, PANEL_LIST_NROWS, NULL);
	for(row = 0; row < nrows; row++){
	    if(xv_get(item, PANEL_LIST_SELECTED, row) == TRUE) break;
	}
	if(row >= nrows) return XV_ERROR;
	set_standard_data_set(row);
	xv_set(gmap_file_pop->add_data, PANEL_INACTIVE, TRUE, NULL);
	xv_set(gmap_file_pop->delete_data, PANEL_INACTIVE, FALSE, NULL);
	break;
      case PANEL_LIST_OP_DELETE:
      case PANEL_LIST_OP_DESELECT:
	standard_data_set_value = -1;
	xv_set(gmap_file_pop->delete_data, PANEL_INACTIVE, TRUE, NULL);
	xv_set(gmap_file_pop->add_data, PANEL_INACTIVE, FALSE, NULL);
	break;
    }
    return XV_OK;
}
#else
/*
 * Notify callback function for `standard_data_list'.
 */
int standard_data_list_handler(item, string, client_data, op, event, row)
     Panel_item	item;
     char	*string;
     Xv_opaque	client_data;
     Panel_list_op	op;
     Event	*event;
     int	row;
{
	void set_standard_data_set();

    switch(op) {
      case PANEL_LIST_OP_SELECT:
	set_standard_data_set(row);
	xv_set(gmap_file_pop->add_data, PANEL_INACTIVE, TRUE, NULL);
	xv_set(gmap_file_pop->delete_data, PANEL_INACTIVE, FALSE, NULL);
	break;
      case PANEL_LIST_OP_DELETE:
      case PANEL_LIST_OP_DESELECT:
	standard_data_set_value = -1;
	xv_set(gmap_file_pop->delete_data, PANEL_INACTIVE, TRUE, NULL);
	xv_set(gmap_file_pop->add_data, PANEL_INACTIVE, FALSE, NULL);
	break;
    }
    return XV_OK;
}
#endif
/*
 * Notify callback function for `add_data'.
 */
void add_data_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    xv_set(gmap_file_pop->standard_data_list, PANEL_LIST_INSERT, 0,
	   PANEL_LIST_STRING, 0, map_params.data_fname, NULL);
}

/*
 * Notify callback function for `delete_data'.
 */
void delete_data_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    xv_set(gmap_file_pop->standard_data_list, PANEL_LIST_DELETE,
	   standard_data_set_value, NULL);
    standard_data_set_value = -1;
}

/* get defaults from call to LNKmap, then check .lnkmaprc startup file */
void get_defaults (argc, argv)
     int argc;
     char **argv;
{
    argc = nnargs(argc, argv, LNKMAPargtab, LNKMAPTABSIZE);
    
    map_params.labels = gnn_labels;
    map_params.features[0] = 0;
    map_params.map_flag = 0;	/* in the process of being taken out */
    map_params.feat_sel_type = 0;
    map_params.desc_fname = "";
    map_params.nraw = 1;
    map_params.ninputs = 1;
    map_params.noutputs = 1;
    map_params.cross_valid = 0;
    map_params.cv_from_file = 0;
    map_params.cv_fname = "";
    
    /* get arguments from default file if present */
    g_defaults_read (path_build(getenv("HOME"), defaults_filename));

    set_experiment_label();
}

/*
 * User-defined action for `restore_defaults'.
 */
void restore_defaults_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    void set_short_algorithm();
    FILE *default_file;
    
    if ((default_file = fopen(screen_full_path, "r")) == NULL) {
	warning("Default parameter file %s not found.\n", screen_full_path);
    } else {
	read_defaults(default_file, screen_full_path);
	nn_fclose(default_file, screen_full_path,
		  "for reading LNKnet defaults");
	g_display_params();
	set_short_algorithm(algorithm);
    }
}

/* select stuff */

void inactivate_features()
{
    /* turn off fancy selection and don't let the user chose them */
    set_select_type(ALL); /* ALL */
    xv_set (gmap_feature_pop->select_type ,PANEL_INACTIVE, TRUE, NULL);
}

void activate_features()
{
    /* allow user to select fancy stuff and return him to the one
    he was using before */
    xv_set (gmap_feature_pop->select_type ,PANEL_INACTIVE, FALSE, NULL);
    set_select_type(xv_get(gmap_feature_pop->select_type, PANEL_VALUE));
}

void set_feature_list(value)
     char *value;
{
    static char tmp[NNMAX_LINE];
    int size;

    size = sizeof(map_params.features)/sizeof(int);
    nnargs_get_int_list(strcpy(tmp,value), map_params.features, size);
    map_params.ninputs = map_params.features[0];
    g_set_description_values();
}

/*
 * Notify callback function for `feature_list'.
 */
Panel_setting feature_list_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
	
    set_feature_list(value);
    return panel_text_notify(item, event);
}

void set_select_type(value)
     int value;
{
    select_type = value;

    xv_set(gmap_feature_pop->feature_list, PANEL_INACTIVE, TRUE, NULL);
    map_params.features[0] = 0;
    xv_set(gmap_feature_pop->select_n, PANEL_INACTIVE, TRUE, NULL);
    map_params.ninputs = 0;
    map_params.feat_sel_type = FALSE;
    
    switch(value){
      case ALL: /* all */
	map_params.ninputs = map_params.nraw;
	map_params.features[0] = 0;
	break;
      case FIRST_N: /* first n */
	xv_set(gmap_feature_pop->select_n,
	       PANEL_INACTIVE, FALSE, NULL);
	map_params.ninputs = xv_get(gmap_feature_pop->select_n, PANEL_VALUE);
	g_set_description_values();
	break;
      case MANUAL: /* fill in */
	xv_set(gmap_feature_pop->feature_list,
	       PANEL_INACTIVE, FALSE, NULL);
	set_feature_list((char *)xv_get(gmap_feature_pop->feature_list,
					PANEL_VALUE));
	break;
      default: break;
    }
}

/*
 * Notify callback function for `feat_sel_type'.
 */
void select_type_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    set_select_type(value);
    g_set_description_values();
}

void
gnorm_store_run_command (shell_file)
    FILE *shell_file;
{
    if (run_time_system==NULL || strlen(run_time_system)<1) chartmp[0] = '\0';
    else sprintf (chartmp, "rsh -n %s ", run_time_system);

    /* set all the flags */
    data_path = map_params.data_path;
    strcpy(tmp_data_fname_space, data_fname_base);
    strcat(tmp_data_fname_space, ext_train);
    data_fname = tmp_data_fname_space;
    strcpy(tmp_desc_fname_space, tmp_data_fname_space);
    strcat(tmp_desc_fname_space, EXT_DESC);
    desc_fname = tmp_desc_fname_space;
    normalization = map_params.normalize;
    norm_fname = map_params.norm_fname;
    debug = map_params.debug;
    verbosity = map_params.verbosity;
    fprintf (shell_file, "%snorm_map %s",
	     chartmp, nnargs_get_flags(argtab, TABSIZE));
}

/*
 * Notify callback function for `norm_autoscale'.
 */
void norm_autoscale_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    void g_set_norm_autoscale();
    g_set_norm_autoscale(value);
}

void g_set_normalize(value)
int value;
{
    map_params.normalize = value;
    g_set_normalize_filename();
}

char * gnorm_get_plot_label ()
{
    static char string[MAXSTR] = "";

    sprintf (string, "Norm:%s", norm_description(map_params.normalize));
    return string;
}

