/***-------------------------------------------------------------------------
 * NAME:		gmap_main.c
 * PURPOSE:	Neural Network main program driver (DEVGUIDE version)
 *
 *
 * USAGE:	gmap
 *
 * SEE ALSO:	gmap.c gmap_subs.c gmap.G
 *
 * DESCRIPTION: 
 *		Processes command line parameters and calls algorithm program
 *	
 *	main
 *	activate_profile_plots
 *	activate_region_plots
 *	set_algorithm
 *	algorithm_parameters_handler
 *	g_get_plot_label
 *	g_set_description_values
 *	g_store_plot_command
 *	g_store_run_command
 *	
 * BUGS AND LIMITATIONS:
 *
 *
 ------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gmap/lnkmap/gmap_main.c,v 10.3 2001/08/23 20:14:08 craig Exp $";



#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <gdd.h>
#include <nnlib.h>
#include "gmap_ui.h"
#include "gmap_params.h"
#include "../makelib/gmlpm_ui.h"
#include "../makelib/grbfm_ui.h"
#include "../makelib/girbfm_ui.h"
#include "../makelib/gkmeansm_ui.h"

/* GLOBAL POPUP WINDOW DEFINIITIONS */
gmap_window1_objects	*gmap_window1;
gmap_plot_pop_objects	*gmap_plot_pop;
gmap_map_plot_objects	*gmap_map_plot;
gmap_clus_plot_objects	*gmap_clus_plot;
gmap_cost_plot_objects	*gmap_cost_plot;
gmap_norm_plot_objects	*gmap_norm_plot;
gmap_file_pop_objects	*gmap_file_pop;
gmap_feature_pop_objects	*gmap_feature_pop;

/* GLOBALS FOR PARAMETER POPUP WINDOWS */
gmlpm_popup1_objects	*gmlpm_popup1;
grbfm_popup1_objects	*grbfm_popup1;
girbfm_popup1_objects	*girbfm_popup1;
gkmeansm_popup1_objects	*gkmeansm_popup1;

/* algorithm integer designator */
/* this list must be the same as the one in gmap_subs.c */
#define MLPM 0
#define RBFM 1
#define IRBFM 2
#define KMEANSM 3
#define SIMPLE_A 4
#define PCA_A 5

/* globals */
int debug;
int algorithm = MLPM;

/* this list must be in the same order as the algorithm integer designator */
static char *classifier_strings[] = {
    "MLPM (Multi Layer Perceptron)",
    "RBFM (Radial Basis Function)",
    "IRBFM (Radial Basis Function)",
    "KMEANSM (Kmeans Clusterer)",
    "Simple Normalization",
    "Principle Components Analysis", 
};

/*
 * Instance XV_KEY_DATA key.  An instance is a set of related
 * user interface objects.  A pointer to an object's instance
 * is stored under this key in every object.  This must be a
 * global variable.
 */
Attr_attribute	INSTANCE;

/* routines in gmap.c */
void get_defaults();
void g_init();
void display_experiment_names();
void set_plot_descriptions();

void main(argc, argv)
     int	argc;
     char	**argv;
{
    void gmlpm_init(), grbfm_init(), girbfm_init(), gkmeansm_init();
    
    print_disclaimer();
    /*
     * Initialize XView.
     */
    xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, 0);
    INSTANCE = xv_unique_key();
    
    /*
     * Initialize user interface components.
     */
    gmap_window1 = gmap_window1_objects_initialize(NULL, NULL);
    gmap_plot_pop =
      gmap_plot_pop_objects_initialize(NULL, gmap_window1->window1);
    gmap_map_plot =
      gmap_map_plot_objects_initialize(NULL, gmap_window1->window1);
    gmap_clus_plot =
      gmap_clus_plot_objects_initialize(NULL, gmap_window1->window1);
    gmap_cost_plot =
      gmap_cost_plot_objects_initialize(NULL, gmap_window1->window1);
    gmap_norm_plot =  
      gmap_norm_plot_objects_initialize(NULL, gmap_window1->window1);
    gmap_feature_pop = 
      gmap_feature_pop_objects_initialize(NULL, gmap_window1->window1);
    gmap_file_pop = 
      gmap_file_pop_objects_initialize(NULL, gmap_window1->window1);

    gmlpm_popup1 =
      gmlpm_popup1_objects_initialize(NULL, gmap_window1->window1);
    grbfm_popup1 = 
      grbfm_popup1_objects_initialize(NULL, gmap_window1->window1);
    girbfm_popup1 = 
      girbfm_popup1_objects_initialize(NULL, gmap_window1->window1);
    gkmeansm_popup1 =
      gkmeansm_popup1_objects_initialize(NULL, gmap_window1->window1);
    
    init_nnsubs(argc, argv, rcsid);
    
    /* get default values from startup file */
    get_defaults(argc, argv);
    
    /* initialize parameter popups */
    gmlpm_init ();
    grbfm_init();
    girbfm_init();
    gkmeansm_init();
    
    /* initialize main popups */
    g_init ();
    
    /*
     * Turn control over to XView.
     */
    xv_main_loop(gmap_window1->window1);
    exit(0);
}

void activate_error_plots (activate_flag)
     int activate_flag;
{
    void g_set_cost_plots();
    
    if (activate_flag) {
	xv_set (gmap_plot_pop->cost_plot, PANEL_INACTIVE, FALSE, NULL);
	g_set_cost_plots(xv_get(gmap_plot_pop->cost_plot,
				 PANEL_VALUE));
	xv_set (gmap_plot_pop->cost_plot_parameters,
		PANEL_INACTIVE, FALSE, NULL);
    } else {
	xv_set (gmap_plot_pop->cost_plot, PANEL_INACTIVE, TRUE, NULL);
	g_set_cost_plots(0);
	xv_set (gmap_plot_pop->cost_plot_parameters,
		PANEL_INACTIVE, TRUE, NULL);
    }
}

void activate_region_plots (activate_flag)
     int activate_flag;
{
    void g_bound_select_train();
    void g_bound_select_test();
    
    if (activate_flag) {
	xv_set (gmap_plot_pop->clus_plot, PANEL_INACTIVE, FALSE, NULL);
	g_bound_select_train(xv_get(gmap_plot_pop->clus_plot,
				    PANEL_VALUE));
	xv_set (gmap_plot_pop->cluster_plot_parameters,
		PANEL_INACTIVE, FALSE, NULL);
    } else {
	xv_set (gmap_plot_pop->clus_plot, PANEL_INACTIVE, TRUE, NULL);
	g_bound_select_train (0);
	xv_set (gmap_plot_pop->cluster_plot_parameters,
		PANEL_INACTIVE, TRUE, NULL);
    }
}

/* set up variables and windows for selected algorithm */
void set_short_algorithm(value)
     int	value;
{
    void activate_error_plots(), activate_test_selects();
    void activate_cross_valid();
    void g_set_norm_plot();

    algorithm = value;
    activate_error_plots(TRUE);
    activate_region_plots(TRUE);
    xv_set (gmap_window1->normalize, PANEL_INACTIVE, FALSE, NULL);
    xv_set (gmap_window1->random_flag, PANEL_INACTIVE, FALSE, NULL);
    xv_set (gmap_window1->seed, PANEL_INACTIVE, FALSE, NULL);
    xv_set (gmap_window1->epoch_plots, PANEL_INACTIVE, FALSE, NULL);
    if(xv_get(gmap_window1->epoch_plots, PANEL_VALUE))
      xv_set (gmap_window1->e_per_plot, PANEL_INACTIVE, FALSE, NULL);
    activate_test_selects(TRUE);
    activate_cross_valid(TRUE);

    /* These fields are usually turned off */
    g_set_norm_plot(FALSE);
    xv_set (gmap_plot_pop->norm_plot, PANEL_INACTIVE, TRUE, NULL);
    xv_set (gmap_plot_pop->norm_plot_parameters, PANEL_INACTIVE, TRUE,
	    NULL);
    xv_set (gmap_plot_pop->norm_plot, PANEL_VALUE, FALSE, NULL);

    xv_set (gmap_plot_pop->map_plot_train, PANEL_INACTIVE, FALSE, NULL);
    xv_set (gmap_plot_pop->map_plot_test, PANEL_INACTIVE, FALSE, NULL);

    switch (algorithm) {
      case MLPM:
	map_params.classifier_name = "mlpm";
	break;
      case RBFM:
	map_params.classifier_name = "rbfm";
	xv_set (gmap_window1->epoch_plots, PANEL_INACTIVE, TRUE, NULL);
	xv_set (gmap_window1->e_per_plot, PANEL_INACTIVE, TRUE, NULL);
	activate_error_plots(FALSE);
	break;
      case IRBFM:
	map_params.classifier_name = "irbfm";
	break;
      case KMEANSM:
	map_params.classifier_name = "kmeansm";
	activate_test_selects(FALSE);
	activate_error_plots(FALSE);
	activate_region_plots(TRUE);
	xv_set (gmap_window1->epoch_plots, PANEL_INACTIVE, TRUE, NULL);
	xv_set (gmap_window1->e_per_plot, PANEL_INACTIVE, TRUE, NULL);
	xv_set (gmap_plot_pop->map_plot_train, PANEL_INACTIVE, TRUE, NULL);
	xv_set (gmap_plot_pop->map_plot_test, PANEL_INACTIVE, TRUE, NULL);
      default:
	break;
    }
    xv_set (gmap_window1->algorithm_sel, PANEL_LABEL_STRING,
	    classifier_strings[algorithm], NULL);
}

/* set up variables and windows for selected algorithm */
void set_algorithm(value)
     int	value;
{
    set_short_algorithm(value);
    display_experiment_names();
    set_plot_descriptions();
}

/* for contributed algorithms,
   for setting and displaying the classifier name */
#if 0
set_algorithm_name(name, label_string)
     char *name, *label_string;
{
    algorithm = CONTRIB;
    map_params.classifier_name = name;
    xv_set (gmap_window1->algorithm_sel, PANEL_LABEL_STRING,
	    label_string, NULL);
}
#endif

/*
 * Notify callback function for `algorithm_parameters'.
 * Display algorithm popup window
 */
void algorithm_parameters_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    switch (algorithm) {
      case MLPM:
	xv_set(gmlpm_popup1->popup1, XV_SHOW, TRUE, NULL);
	break;
      case RBFM:
	xv_set(grbfm_popup1->popup1, XV_SHOW, TRUE, NULL);
	break;
      case IRBFM:
	xv_set(girbfm_popup1->popup1, XV_SHOW, TRUE, NULL);
	break;
      case KMEANSM:
	xv_set(gkmeansm_popup1->popup1, XV_SHOW, TRUE, NULL);
	break;
      default:
	break;
    }
}

/* get algorithm specific plot label strings */
char * g_get_plot_label()
{
    char *gmlpm_get_plot_label();
    char *grbfm_get_plot_label();
    char *girbfm_get_plot_label();
    char *gkmeansm_get_plot_label();
    char *gnorm_get_plot_label();
    
    switch (algorithm) {
      case MLPM:
	return gmlpm_get_plot_label();
      case RBFM:
	return grbfm_get_plot_label();
      case IRBFM:
	return girbfm_get_plot_label();
      case KMEANSM:
	return gkmeansm_get_plot_label();
      case PCA_A:
	return gnorm_get_plot_label();
      default:
	return "";
    }
}

/* set nnargs variables for algorithms to those in map_params struct */
void g_set_description_values()
{
    void gmlpm_set_description_values(), grbfm_set_description_values();
    void girbfm_set_description_values(), gkmeansm_set_description_values();
    void gplot_set_description_values();

    gmlpm_set_description_values();
    grbfm_set_description_values();
    girbfm_set_description_values();
    gkmeansm_set_description_values();
    gplot_set_description_values();
}

/* store the algorithm specific mapping plot command */
void g_store_plot_command(shell_file)
     FILE *shell_file;
{
    void grbfm_store_plot_command(), girbfm_store_plot_command();

    switch (algorithm) {
      case RBFM:
	grbfm_store_plot_command(shell_file);
	break;
      case IRBFM:
	girbfm_store_plot_command(shell_file);
	break;
      default:
	break;
    }
}

/* store the call to the algorithm program */
void g_store_run_command(shell_file, log_filename)
     FILE *shell_file;
     char *log_filename;
{
    void gmlpm_store_run_command(), grbfm_store_run_command();
    void girbfm_store_run_command(), gkmeansm_store_run_command();

    switch (algorithm) {
      case MLPM:
	gmlpm_store_run_command (shell_file, log_filename);
	break;
      case RBFM:
	grbfm_store_run_command(shell_file, log_filename);
	break;
      case IRBFM:
	girbfm_store_run_command(shell_file, log_filename);
	break;
      case KMEANSM:
	gkmeansm_store_run_command(shell_file, log_filename);
	break;
      default:
	break;
    }
}

int alg_is_norm(){
    if(algorithm == SIMPLE_A || algorithm == PCA_A)
      return 1;
    else
      return 0;
}


/*
 * get the number of epochs from the current algorithm
 */
int g_get_epochs()
{
    switch (algorithm) {
      case MLPM:
	return gmlpm_get_epochs();
      case IRBFM:
	return girbfm_get_epochs();
      default:
	return 1;
    }
}

/*
 * set the number of epochs from the current algorithm
 */
int g_set_epochs(e)
     int e;
{
    void gmlpm_set_epochs(), girbfm_set_epochs();

    switch (algorithm) {
      case MLPM:
	gmlpm_set_epochs(e);
      case IRBFM:
	girbfm_set_epochs(e);
      default:
	break;
    }
}

/* read the defaults files for the main windows and for each algortihm */
void read_defaults(dfp, dfname)
     FILE *dfp;
     char *dfname;
{
    void read_main_defaults(), read_mlpm_defaults(), read_rbfm_defaults();
    void read_irbfm_defaults(), read_kmeansm_defaults();
    void read_plot_defaults();

    read_main_defaults(dfp, dfname);
    read_mlpm_defaults(dfp, dfname);
    read_rbfm_defaults(dfp, dfname);
    read_irbfm_defaults(dfp, dfname);
    read_kmeansm_defaults(dfp, dfname);
    read_plot_defaults(dfp, dfname);
}

void write_defaults(dfp, dfname)
     FILE *dfp;
     char *dfname;
{
    void write_main_defaults(), write_mlpm_defaults(), write_rbfm_defaults();
    void write_irbfm_defaults(), write_kmeansm_defaults();
    void write_plot_defaults();

    write_main_defaults(dfp, dfname);
    write_mlpm_defaults(dfp, dfname);
    write_rbfm_defaults(dfp, dfname);
    write_irbfm_defaults(dfp, dfname);
    write_kmeansm_defaults(dfp, dfname);
    write_plot_defaults(dfp, dfname);
}

int alg_is_clus()
{
    switch(algorithm){
      case KMEANSM:
	return 1;
      default:
	return 0;
    }
}

char * get_clus_plot_name()
{
    switch(algorithm){
      case KMEANSM:
	return "kmeans";
      default:
	return "";
    }
}
void g_set_cluster_names()
{
    void grbfm_set_cluster_param_fname(), girbfm_set_cluster_param_fname();

    grbfm_set_cluster_param_fname();
    girbfm_set_cluster_param_fname();
}

g_display_params()
{
    void gmlpm_display_params(), grbfm_display_params();
    void girbfm_display_params(), gkmeansm_display_params();
    void display_params();

    gmlpm_display_params();
    grbfm_display_params();
    girbfm_display_params();
    gkmeansm_display_params();
    display_params();
}

print_disclaimer()
{
    printf("*******************************************************\n");
    printf("LNKmap and the LNKmap input/output mapping programs are\n");
    printf("experimental and are not fully supported.\n");
    printf("*******************************************************\n");
}
