/***--------------------------------------------------------------------------
 * NAME:        gmap_plot.c
 * PURPOSE:     functions for plot window paramters
 *
 * SEE ALSO:	 gmap_main.c gmap_subs.c gmap.G
 *
 * DESCRIPTION:
 *		
 * BUGS AND LIMITATIONS:
 *
 *
 --------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gmap/lnkmap/gmap_plot.c,v 10.5 2001/08/23 20:14:08 craig Exp $";



#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <unistd.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <sys/param.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <gdd.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <nnlib.h>
#include "gmap_ui.h"
#include "gmap_params.h"
#include "lnkmap_args.h"

#define MAXSTR (256)
#define MAXLABEL (1024)

#define CROSS_VALID (64)
#define TRAIN (1)		/* values for run_options */
#define CLASS_TRAIN (2)
#define CLASS_EVAL (4)
#define CLASS_TEST (8)
#define CLASS (16)	/* for store command, not for run_options */


/* for plot commands, to allow them to not be shown */
#define SHOW_GRAPHICS 1
#define NO_GRAPHICS 0

/* include files for generating plot calls */
/* and variables needed for them */
static int plot_trial_average;
/* number of trials to average for stat display */
static char *plot_exp_path;
static char *plot_error_fname;
static int plot_n_inputs;
static int plot_n_outputs;
static char *plot_labels [NNARGS_MAX_LABELS];
static int no_graphics = 0;

#define DEFAULT_PLOT_PATH ""

/* include files for generating plot calls and variables needed for them */
#undef PLOT_MAP
#include <plot_map_args.h>
#define PLOT_MAP "_plot_map"
#undef PLOT_BOUND
#include <plot_bound_args.h>
#define PLOT_BOUND "_plot_bound"
#undef PLOT_COST
#include <plot_cost_args.h>
#define PLOT_COST "plot_cost"
#undef PLOT_NORM
#include <plot_norm_args.h>
#define PLOT_NORM "plot_norm"

static int cost_plot_flag = 0; 
static int norm_plot_flag = 0;
static int outputs_plot_train = 0;
static int outputs_plot_test = 0;
static int scatter_plot_train = 0;
static int scatter_plot_test = 0;
/* region_plot, scatter_plot, internals_plot */

static char scatter_plot_ftmp[MAXSTR];
static char cost_plot_ftmp[MAXSTR], norm_plot_ftmp[MAXSTR];
static char bound_plot_ftmp[MAXSTR];
static char map_plot_ftmp[MAXSTR];
static char scatter_plot_data_name[MAXSTR];
static char map_description[MAXLABEL];
static char bound_description[MAXLABEL];
static char norm_plot_lbl_tmp[MAXLABEL];
static char cost_description[MAXLABEL];
static int trial_average;
static char command_line[1024] = "";

extern gmap_window1_objects	*gmap_window1;
extern gmap_plot_pop_objects	*gmap_plot_pop;
extern gmap_map_plot_objects	*gmap_map_plot;
extern gmap_clus_plot_objects	*gmap_clus_plot;
extern gmap_norm_plot_objects	*gmap_norm_plot;
extern gmap_cost_plot_objects	*gmap_cost_plot;

void set_map_autoscale();
void g_set_show_all();
void g_set_cost_autoscale();
void g_set_norm_autoscale();
void g_set_clus_autoscale();
void store_cost_plot_command();
void store_map_plot_command();

/* functions in other files */
void set_float_string();
float set_atof_value();
float set_atof_low_value();
float set_atof_range_value();

void display_plot_params()
{
    int val;

    /* plot selection window */
    val = outputs_plot_train | (scatter_plot_train << 1);
    xv_set (gmap_plot_pop->map_plot_train, PANEL_VALUE, val, NULL);
    val = outputs_plot_test | (scatter_plot_test << 1);
    xv_set (gmap_plot_pop->map_plot_test, PANEL_VALUE, val, NULL);
    xv_set(gmap_plot_pop->cost_plot, PANEL_VALUE, cost_plot_flag, NULL);
    xv_set(gmap_plot_pop->trial_average, PANEL_VALUE, trial_average, NULL);
    xv_set(gmap_plot_pop->norm_plot, PANEL_VALUE, norm_plot_flag, NULL);
    val = region_plot | (scatter_plot << 1) | (internals_plot << 2);
    xv_set (gmap_plot_pop->clus_plot, PANEL_VALUE, val, NULL);

    /* set defaults for graph scales */
    xv_set(gmap_map_plot->map_plot_file, PANEL_VALUE,
	   plot_fname, NULL);
    xv_set (gmap_map_plot->map_in_dim, PANEL_VALUE, map_in_dim, NULL);
    xv_set (gmap_map_plot->map_out_dim, PANEL_VALUE, map_out_dim, NULL);
    xv_set (gmap_map_plot->map_npoints, PANEL_VALUE, map_npoints, NULL);
    xv_set (gmap_map_plot->map_autoscale, PANEL_VALUE, map_autoscale, NULL);
    set_map_autoscale(map_autoscale);
    set_float_string (gmap_map_plot->map_xmin, "%5.2g", map_xmin);
    set_float_string (gmap_map_plot->map_xmax, "%5.2g", map_xmax);
    set_float_string (gmap_map_plot->map_ymin, "%5.2g", map_ymin);
    set_float_string (gmap_map_plot->map_ymax, "%5.2g", map_ymax);
    set_float_string (gmap_map_plot->map_xstep, "%5.2g", map_xstep);
    set_float_string (gmap_map_plot->map_ystep, "%5.2g", map_ystep);
    xv_set(gmap_map_plot->map_plot_label, PANEL_VALUE, map_plot_label, NULL);

    xv_set(gmap_clus_plot->clus_plot_file, PANEL_VALUE, bound_plot_fname,NULL);
    xv_set (gmap_clus_plot->clus_dim_1, PANEL_VALUE, bound_dim_1, NULL);
    xv_set (gmap_clus_plot->clus_dim_2, PANEL_VALUE, bound_dim_2, NULL);
    xv_set (gmap_clus_plot->clus_npoints, PANEL_VALUE, bound_npoints, NULL);
    xv_set (gmap_clus_plot->clus_autoscale, PANEL_VALUE, bound_autoscale,NULL);
    g_set_clus_autoscale(bound_autoscale);
    set_float_string (gmap_clus_plot->clus_xmin, "%5.2g", bound_xmin);
    set_float_string (gmap_clus_plot->clus_xmax, "%5.2g", bound_xmax);
    set_float_string (gmap_clus_plot->clus_ymin, "%5.2g", bound_ymin);
    set_float_string (gmap_clus_plot->clus_ymax, "%5.2g", bound_ymax);
    set_float_string (gmap_clus_plot->clus_xstep, "%5.2g", bound_xstep);
    set_float_string (gmap_clus_plot->clus_ystep, "%5.2g", bound_ystep);
    xv_set(gmap_clus_plot->show_all_check_box, PANEL_VALUE,show_all,NULL);
    set_float_string(gmap_clus_plot->distance_limit,"%5.2g",
		     distance_limit);
    xv_set(gmap_clus_plot->internals_level, PANEL_VALUE, 
	    internals_level, NULL);
    set_float_string(gmap_clus_plot->internals_scale, "%5.2g",
		     internals_scale);
    xv_set(gmap_clus_plot->clus_plot_label, PANEL_VALUE,
	   bound_plot_label, NULL);

    xv_set(gmap_cost_plot->cost_plot_file, PANEL_VALUE, cost_plot_fname, NULL);
    xv_set (gmap_cost_plot->cost_autoscale, PANEL_VALUE,
	    cost_autoscale, NULL);
    g_set_cost_autoscale(cost_autoscale);
    set_float_string(gmap_cost_plot->cost_xmin, "%5.2g", cost_xmin);
    set_float_string(gmap_cost_plot->cost_xmax, "%5.2g", cost_xmax);
    set_float_string(gmap_cost_plot->cost_ymin, "%5.2g", cost_ymin);
    set_float_string(gmap_cost_plot->cost_ymax, "%5.2g", cost_ymax);
    set_float_string(gmap_cost_plot->cost_xstep, "%5.2g", cost_xstep);
    set_float_string(gmap_cost_plot->cost_ystep, "%5.2g", cost_ystep);
    xv_set(gmap_cost_plot->cost_plot_label, PANEL_VALUE,
	   cost_plot_label, NULL);

    xv_set(gmap_norm_plot->norm_plot_file, PANEL_VALUE, norm_plot_fname, NULL);
    xv_set (gmap_norm_plot->norm_autoscale, PANEL_VALUE,
	    norm_auto_scale_flag, NULL);
    g_set_norm_autoscale(norm_auto_scale_flag);
    set_float_string(gmap_norm_plot->norm_xmin, "%5.2g", norm_xmin);
    set_float_string(gmap_norm_plot->norm_xmax, "%5.2g", norm_xmax);
    set_float_string(gmap_norm_plot->norm_ymin, "%5.2g", norm_ymin);
    set_float_string(gmap_norm_plot->norm_ymax, "%5.2g", norm_ymax);
    set_float_string(gmap_norm_plot->norm_xstep, "%5.2g", norm_xstep);
    set_float_string(gmap_norm_plot->norm_ystep, "%5.2g", norm_ystep);
    xv_set(gmap_norm_plot->norm_plot_label, PANEL_VALUE,
	   norm_plot_label, NULL);
}

void g_bound_select_test (value)
     int value;
{
    if(value & 1){
	outputs_plot_test = 1;
    } else {
	outputs_plot_test = 0;
    }
    if(value & 2){
	scatter_plot_test = 1;
    } else {
	scatter_plot_test = 0;
    }
}

void g_bound_select_train (value)
     int value;
{
    if(value & 1){
	outputs_plot_train = 1;
    } else {
	outputs_plot_train = 0;
    }
    if(value & 2){
	scatter_plot_train = 1;
    } else {
	scatter_plot_train = 0;
    }
}

g_init_plot(){
    void set_experiment_label();

    g_set_show_all (show_all);
    g_set_norm_autoscale (TRUE);
    set_experiment_label();
}

g_set_trial_average(value)
     int value;
{
    trial_average = value;
    xv_set(gmap_plot_pop->trial_average, PANEL_VALUE, trial_average, NULL);
}
/*
 * Notify callback function for `trial_average'.
 */
Panel_setting trial_average_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);
    
    trial_average = value;
    return panel_text_notify(item, event);
}

void set_map_autoscale (value)
     int value;
{
    if (value) {
	map_autoscale = TRUE; 
	xv_set (gmap_map_plot->map_axes_limits, PANEL_INACTIVE, TRUE, NULL);
	xv_set (gmap_map_plot->map_xmin, PANEL_INACTIVE, TRUE, NULL);
	xv_set (gmap_map_plot->map_ymin, PANEL_INACTIVE, TRUE, NULL);
	xv_set (gmap_map_plot->map_xmax, PANEL_INACTIVE, TRUE, NULL);
	xv_set (gmap_map_plot->map_ymax, PANEL_INACTIVE, TRUE, NULL);
	xv_set (gmap_map_plot->map_xstep, PANEL_INACTIVE, TRUE, NULL);
	xv_set (gmap_map_plot->map_ystep, PANEL_INACTIVE, TRUE, NULL);
    } else {
	map_autoscale = FALSE;
	xv_set (gmap_map_plot->map_axes_limits, PANEL_INACTIVE,
		FALSE, NULL);
	xv_set (gmap_map_plot->map_xmin, PANEL_INACTIVE, FALSE, NULL);
	xv_set (gmap_map_plot->map_ymin, PANEL_INACTIVE, FALSE, NULL);
	xv_set (gmap_map_plot->map_xmax, PANEL_INACTIVE, FALSE, NULL);
	xv_set (gmap_map_plot->map_ymax, PANEL_INACTIVE, FALSE, NULL);
	xv_set (gmap_map_plot->map_xstep, PANEL_INACTIVE, FALSE, NULL);
	xv_set (gmap_map_plot->map_ystep, PANEL_INACTIVE, FALSE, NULL);
    }
}


/*
 * Notify callback function for `map_in_dim'.
 */
Panel_setting map_in_dim_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);
    
    if (value < 0) value = 0;
    if (value >= map_params.nraw) value = map_params.nraw -1;
    xv_set (gmap_map_plot->map_in_dim, PANEL_VALUE, value, NULL);
    map_in_dim = value;
    
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `map_out_dim'.
 */
Panel_setting map_out_dim_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);
    
    if (value < 0) value = 0;
    if (value >= map_params.noutputs) value = map_params.noutputs -1;
    xv_set (gmap_map_plot->map_out_dim, PANEL_VALUE, value, NULL);
    map_out_dim = value;
    
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `other_vars'.
 */
Panel_setting other_vars_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char tmp[NNMAX_LINE];
    char *	value = (char *) xv_get(item, PANEL_VALUE);
	
    nnargs_get_float_list(strcpy(tmp, value), map_other_vars,
			  NNARGS_MAX_LABELS);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `no_norm'.
 */
void no_norm_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    map_no_norm = value;
}

/*
 * Notify callback function for `map_npoints'.
 */
Panel_setting map_npoints_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);
    
    map_npoints = value;
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `map_plot_file'.
 */
Panel_setting map_plot_file_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    strncpy(map_plot_ftmp, value, MAXSTR);
    plot_fname = map_plot_ftmp;
    return panel_text_notify(item, event);
}

void set_map_plot_label(value)
     char *value;
{
    sprintf (map_description, "\"%s\"", value);
    map_plot_label = map_description;
}


/*
 * Notify callback function for `map_xmax'.
 */
Panel_setting map_xmax_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    map_xmax = set_atof_value(value, "map_xmax",
			      gmap_map_plot->map_xmax, map_xmax);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `map_xmin'.
 */
Panel_setting map_xmin_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    map_xmin = set_atof_value(value, "map_xmin",
			      gmap_map_plot->map_xmin, map_xmin);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `map_xstep'.
 */
Panel_setting map_xstep_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    map_xstep = set_atof_value(value, "map_xstep",
			       gmap_map_plot->map_xstep, map_xstep);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `map_ymax'.
 */
Panel_setting map_ymax_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    map_ymax = set_atof_value(value, "map_ymax",
			      gmap_map_plot->map_ymax, map_ymax);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `map_ymin'.
 */
Panel_setting map_ymin_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    map_ymin = set_atof_value(value, "map_ymin",
			      gmap_map_plot->map_ymin, map_ymin);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `map_ystep'.
 */
Panel_setting map_ystep_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    map_ystep = set_atof_value(value, "map_ystep",
			       gmap_map_plot->map_ystep, map_ystep);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `clus_plot_train'.
 */
void clus_plot_train_handler(item, value, event)
     Panel_item	item;
     int		value;
     Event		*event;
{
    region_plot = (value & 1) ? 1 : 0;
    scatter_plot = (value & 2) ? 1 : 0;
    internals_plot = (value & 4) ? 1 : 0;
}

/*
 * Notify callback function for `distance_limit'.
 */
Panel_setting distance_limit_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);

    distance_limit =
      set_atof_low_value(value, "distance_limit",
			 gmap_clus_plot->distance_limit, distance_limit, 0.);
    return panel_text_notify(item, event);
}

void
g_set_show_all (value)
	int value;
{
	if(value & 1){
		xv_set (gmap_clus_plot->distance_limit, PANEL_INACTIVE,
		    TRUE, NULL);
	} else { 
		xv_set (gmap_clus_plot->distance_limit, PANEL_INACTIVE,
		    FALSE, NULL);
	}
	show_all = value;
}

/*
 * Notify callback function for `clus_xmax'.
 */
Panel_setting clus_xmax_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);

    bound_xmax = set_atof_value(value, "clus_xmax",
				gmap_clus_plot->clus_xmax, bound_xmax);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `clus_xmin'.
 */
Panel_setting clus_xmin_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);

    bound_xmin = set_atof_value(value, "clus_xmin",
				gmap_clus_plot->clus_xmin, bound_xmin);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `clus_xstep'.
 */
Panel_setting clus_xstep_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);

    bound_xstep = set_atof_value(value, "clus_xstep",
				 gmap_clus_plot->clus_xstep, bound_xstep);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `clus_ymax'.
 */
Panel_setting clus_ymax_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);

    bound_ymax = set_atof_value(value, "clus_ymax",
				gmap_clus_plot->clus_ymax, bound_ymax);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `clus_ymin'.
 */
Panel_setting clus_ymin_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);

    bound_ymin = set_atof_value(value, "clus_ymin",
				gmap_clus_plot->clus_ymin, bound_ymin);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `clus_ystep'.
 */
Panel_setting clus_ystep_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);

    bound_ystep = set_atof_value(value, "clus_ystep",
				 gmap_clus_plot->clus_ystep, bound_ystep);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `internals_level'.
 */
Panel_setting internals_level_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);
    
    internals_level = value;
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `internals_scale'.
 */
Panel_setting internals_scale_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);

    internals_scale = set_atof_low_value(value, "internals_scale",
					 gmap_clus_plot->internals_scale,
					 internals_scale, 0.);
    return panel_text_notify(item, event);
}

void g_set_cost_plots(value)
     int value;
{
    void display_verror();

    if(value){
	cost_plot_flag = 1;
	map_params.verror = 1;
    } else {
	cost_plot_flag = 0;
    }
    display_verror(map_params.verror);
}

/*
 * Notify callback function for `clus_plot_file'.
 */
Panel_setting clus_plot_file_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    strncpy(bound_plot_ftmp, value, MAXSTR);
    bound_plot_fname = bound_plot_ftmp;
    return panel_text_notify(item, event);
}

void g_set_cost_autoscale (value)
     int value;
{
    if (value) {
	cost_autoscale = TRUE; 
	xv_set(gmap_cost_plot->cost_axes_limits,PANEL_INACTIVE,TRUE,
	       NULL);
	xv_set (gmap_cost_plot->cost_xmin, PANEL_INACTIVE, TRUE, NULL);
	xv_set (gmap_cost_plot->cost_ymin, PANEL_INACTIVE, TRUE, NULL);
	xv_set (gmap_cost_plot->cost_xmax, PANEL_INACTIVE, TRUE, NULL);
	xv_set (gmap_cost_plot->cost_ymax, PANEL_INACTIVE, TRUE, NULL);
	xv_set (gmap_cost_plot->cost_xstep, PANEL_INACTIVE, TRUE, NULL);
	xv_set (gmap_cost_plot->cost_ystep, PANEL_INACTIVE, TRUE, NULL);
    } else {
	cost_autoscale = FALSE;
	xv_set (gmap_cost_plot->cost_axes_limits,PANEL_INACTIVE,FALSE,
		NULL);
	xv_set (gmap_cost_plot->cost_xmin, PANEL_INACTIVE, FALSE, NULL);
	xv_set (gmap_cost_plot->cost_ymin, PANEL_INACTIVE, FALSE, NULL);
	xv_set (gmap_cost_plot->cost_xmax, PANEL_INACTIVE, FALSE, NULL);
	xv_set (gmap_cost_plot->cost_ymax, PANEL_INACTIVE, FALSE, NULL);
	xv_set (gmap_cost_plot->cost_xstep, PANEL_INACTIVE, FALSE,NULL);
	xv_set (gmap_cost_plot->cost_ystep, PANEL_INACTIVE, FALSE,NULL);
    }
}

/*
 * Notify callback function for `cost_plot_file'.
 */
Panel_setting cost_plot_file_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    cost_plot_fname = value;
    return panel_text_notify(item, event);
}

void g_set_cost_plot_label (value)
     char *value;
{
    sprintf (cost_description,"\"%s\"",value);
    cost_plot_label = cost_description;
}

/*
 * Notify callback function for `cost_xmax'.
 */
Panel_setting cost_xmax_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    cost_xmax = set_atof_value(value, "cost_xmax",
				 gmap_cost_plot->cost_xmax, cost_xmax);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `cost_xmin'.
 */
Panel_setting cost_xmin_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    cost_xmin = set_atof_value(value, "cost_xmin",
				 gmap_cost_plot->cost_xmin, cost_xmin);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `cost_xstep'.
 */
Panel_setting cost_xstep_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    cost_xstep = set_atof_value(value, "cost_xstep",
				  gmap_cost_plot->cost_xstep,
				  cost_xstep);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `cost_ymax'.
 */
Panel_setting cost_ymax_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    cost_ymax = set_atof_value(value, "cost_ymax",
				 gmap_cost_plot->cost_ymax, cost_ymax);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `cost_ymin'.
 */
Panel_setting cost_ymin_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    cost_ymin = set_atof_value(value, "cost_ymin",
				 gmap_cost_plot->cost_ymin, cost_ymin);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `cost_ystep'.
 */
Panel_setting cost_ystep_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    cost_ystep = set_atof_value(value, "cost_ystep",
				  gmap_cost_plot->cost_ystep,
				  cost_ystep);
    return panel_text_notify(item, event);
}

void g_set_norm_autoscale (value)
     int value;
{
    if (value) {
	norm_auto_scale_flag = TRUE; 
	xv_set(gmap_norm_plot->norm_axes_limits, PANEL_INACTIVE, TRUE,
	       NULL);
	xv_set (gmap_norm_plot->norm_xmin, PANEL_INACTIVE, TRUE, NULL);
	xv_set (gmap_norm_plot->norm_ymin, PANEL_INACTIVE, TRUE, NULL);
	xv_set (gmap_norm_plot->norm_xmax, PANEL_INACTIVE, TRUE, NULL);
	xv_set (gmap_norm_plot->norm_ymax, PANEL_INACTIVE, TRUE, NULL);
	xv_set (gmap_norm_plot->norm_xstep, PANEL_INACTIVE, TRUE, NULL);
	xv_set (gmap_norm_plot->norm_ystep, PANEL_INACTIVE, TRUE, NULL);
    } else {
	norm_auto_scale_flag = FALSE;
	xv_set (gmap_norm_plot->norm_axes_limits, PANEL_INACTIVE, FALSE,
		NULL);
	xv_set (gmap_norm_plot->norm_xmin, PANEL_INACTIVE, FALSE, NULL);
	xv_set (gmap_norm_plot->norm_ymin, PANEL_INACTIVE, FALSE, NULL);
	xv_set (gmap_norm_plot->norm_xmax, PANEL_INACTIVE, FALSE, NULL);
	xv_set (gmap_norm_plot->norm_ymax, PANEL_INACTIVE, FALSE, NULL);
	xv_set (gmap_norm_plot->norm_xstep, PANEL_INACTIVE, FALSE, NULL);
	xv_set (gmap_norm_plot->norm_ystep, PANEL_INACTIVE, FALSE, NULL);
    }
}

void g_set_norm_plot (value)
int value;
{
    norm_plot_flag = value;
}

/*
 * Notify callback function for `norm_plot_file'.
 */
Panel_setting norm_plot_file_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    norm_plot_fname = value;
    return panel_text_notify(item, event);
}

void g_set_norm_plot_label (value)
char *value;
{
    sprintf (norm_plot_lbl_tmp, "\"%s\"", value);
    norm_plot_label = norm_plot_lbl_tmp;
}

/*
 * Notify callback function for `norm_xmax'.
 */
Panel_setting norm_xmax_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    norm_xmax = set_atof_value(value, "norm_xmax",
			       gmap_norm_plot->norm_xmax, norm_xmax);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `norm_xmin'.
 */
Panel_setting norm_xmin_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    norm_xmin = set_atof_value(value, "norm_xmin",
			       gmap_norm_plot->norm_xmin, norm_xmin);
    
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `norm_xstep'.
 */
Panel_setting norm_xstep_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    norm_xstep = set_atof_value(value, "norm_xstep",
				gmap_norm_plot->norm_xstep, norm_xstep);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `norm_ymax'.
 */
Panel_setting norm_ymax_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    norm_ymax = set_atof_value(value, "norm_ymax",
			       gmap_norm_plot->norm_ymax, norm_ymax);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `norm_ymin'.
 */
Panel_setting norm_ymin_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    norm_ymin = set_atof_value(value, "norm_ymin",
			       gmap_norm_plot->norm_ymin, norm_ymin);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `norm_ystep'.
 */
Panel_setting norm_ystep_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    norm_ystep = set_atof_value(value, "norm_ystep",
				gmap_norm_plot->norm_ystep, norm_ystep);
    return panel_text_notify(item, event);
}

void g_set_clus_plot_label(value)
     char *value;
{
    sprintf (bound_description, "\"%s\"", value);
    bound_plot_label = bound_description;
}

void set_plot_descriptions()
{
    char *desc_text;
    char *g_get_plot_label();
    void set_map_plot_label();
    
    /* get label from current algorithm */
    desc_text = g_get_plot_label();
    set_map_plot_label(desc_text);
    xv_set(gmap_map_plot->map_plot_label, PANEL_VALUE, map_plot_label,NULL);
    g_set_clus_plot_label(desc_text);
    xv_set (gmap_clus_plot->clus_plot_label,PANEL_VALUE,bound_plot_label,NULL);
    g_set_cost_plot_label(desc_text);
    xv_set(gmap_cost_plot->cost_plot_label, PANEL_VALUE, cost_plot_label,NULL);
    g_set_norm_plot_label(desc_text);
    xv_set(gmap_norm_plot->norm_plot_label, PANEL_VALUE, norm_plot_label,NULL);
}

void set_plot_flags()
{
    int i;
    
    plot_trial_average = trial_average;
    plot_exp_path = map_params.exp_path;
    plot_n_inputs = map_params.nraw;
    plot_n_outputs = map_params.noutputs;
    for(i = 0;  i <= atoi(map_params.labels[0]) && i <= NNARGS_MAX_LABELS; i++)
      plot_labels[i] = map_params.labels[i];
    plot_error_fname = map_params.param_fname;
    strcpy(bound_plot_ftmp, path_build(map_params.data_path,
				       map_params.data_fname));
    scatter_data_fullname = scatter_plot_ftmp;
    norm_data_path = map_params.data_fname;
}

void gplot_set_description_values()
{
    set_plot_flags();
}


void store_alg_plot_commands(shell_file, run_option, show_plot)
     FILE *shell_file;
     int run_option, show_plot;
{
    if (show_plot){
	no_graphics = 0;
	no_map_graphics = 0;
    } else {
	no_graphics = 1;
	no_map_graphics = 1;
    }
    
    strcpy(scatter_plot_data_name, path_build(map_params.data_path,
					      map_params.data_fname));
    map_scat_data_fullname = scatter_plot_data_name;
    scatter_data_fullname = scatter_plot_data_name;
    map_scat_n_patterns = map_params.n_patterns;
    scatter_n_patterns = map_params.n_patterns;
    plot_map = 1;
    if (run_option == TRAIN) {
	outputs_plot = outputs_plot_train;
	map_scat_plot = scatter_plot_train;
    } else {
	outputs_plot = outputs_plot_test;
	map_scat_plot = scatter_plot_test;
    }
    if(alg_is_clus()){
	if(region_plot || scatter_plot || internals_plot)
	  store_map_plot_command(shell_file);
    } else {
	if(outputs_plot || map_scat_plot)
	  store_map_plot_command(shell_file);
    }
}

void store_cost_plot_command(shell_file)
     FILE *shell_file;
{
    map_data = 1;
    class_data = seq_data = 0;
    fprintf(shell_file, "%s %s", PLOT_COST,
	     nnargs_get_flags(cost_argtab, COST_TABSIZE));
}

void store_map_plot_command(shell_file)
     FILE *shell_file;
{
    char *get_clus_plot_name();
    void g_store_plot_command();

    if(alg_is_clus()){
	sprintf (command_line, "%s",
		 nnargs_get_flags(bound_argtab, BOUND_TABSIZE));
	*strrchr(command_line, '\n') = ' ';/* get rid of newline at end */
	fprintf(shell_file, "%s%s%s\\\n %s", DEFAULT_PLOT_PATH, 
		get_clus_plot_name(), PLOT_BOUND, command_line);
    } else {
	sprintf (command_line, "%s",
		 nnargs_get_flags(map_argtab, MAP_TABSIZE));
	*strrchr(command_line, '\n') = ' ';/* get rid of newline at end */
	fprintf(shell_file, "%s%s%s\\\n %s", DEFAULT_PLOT_PATH, 
		map_params.classifier_name, PLOT_MAP, command_line);
    }
    /* put in classifier dependent call if needed */
    g_store_plot_command (shell_file);
    fprintf(shell_file, "\n");
}

void store_norm_plot_command(shell_file)
     FILE *shell_file;
{
    norm_data_path = map_params.data_path;
    fprintf(shell_file, "%s %s", PLOT_NORM,
	    nnargs_get_flags(norm_argtab, NORM_TABSIZE));
}

display_plot_exper_names(base_ftmp)
     char *base_ftmp;
{
    strcpy(map_plot_ftmp, base_ftmp);
    strcat(map_plot_ftmp, ".plot");
    plot_fname = map_plot_ftmp;
    xv_set(gmap_map_plot->map_plot_file, PANEL_VALUE, plot_fname, NULL);

    strcpy(bound_plot_ftmp, map_plot_ftmp);
    bound_plot_fname = bound_plot_ftmp;
    xv_set(gmap_clus_plot->clus_plot_file, PANEL_VALUE, bound_plot_fname,
	   NULL);

    strcpy(cost_plot_ftmp, base_ftmp);
    cost_plot_fname = strcat(cost_plot_ftmp, ".cost.plot");
    xv_set(gmap_cost_plot->cost_plot_file, PANEL_VALUE, cost_plot_fname,
	   NULL);

    strcpy(norm_plot_ftmp, base_ftmp);
    norm_plot_fname = strcat(norm_plot_ftmp, ".norm.plot");
    xv_set(gmap_norm_plot->norm_plot_file, PANEL_VALUE, norm_plot_fname,
	   NULL);
}


void read_plot_defaults(default_file, default_fullname)
     FILE *default_file;
     char *default_fullname;
{
    int nargc;

    if(default_file != NULL)
      nargc = nnargs_from_file(default_file, default_fullname,
			       bound_argtab, BOUND_TABSIZE);
    strcpy(bound_plot_ftmp, bound_plot_fname);
    bound_plot_fname = bound_plot_ftmp;
    /* profile_plot_fname */
    /* bound_dim_1, bound_dim_2 */
    /* bound_xmin, bound_xmax, bound_xstep */
    /* bound_ymin, bound_ymax, bound_ystep */
    /* show_color */
    strcpy(bound_description, bound_plot_label);
    bound_plot_label = bound_description;
    strcpy(scatter_plot_ftmp, scatter_data_fullname);
    scatter_data_fullname = scatter_plot_ftmp;

    if(default_file != NULL)
      nargc = nnargs_from_file(default_file, default_fullname,
			       cost_argtab, COST_TABSIZE);
    strcpy(cost_plot_ftmp, cost_plot_fname);
    cost_plot_fname = cost_plot_ftmp;
    strcpy(cost_description, cost_plot_label);
    cost_plot_label = cost_description;

    if(default_file != NULL)
      nargc = nnargs_from_file(default_file, default_fullname,
			       norm_argtab, NORM_TABSIZE);
    strcpy(norm_plot_ftmp, norm_plot_fname);
    norm_plot_fname = norm_plot_ftmp;
    strcpy(norm_plot_lbl_tmp, norm_plot_label);
    norm_plot_label = norm_plot_lbl_tmp;

    if(default_file != NULL)
      nargc = nnargs_from_file(default_file, default_fullname, 
			       map_argtab, MAP_TABSIZE);
    strcpy(map_plot_ftmp, plot_fname);
    plot_fname = map_plot_ftmp;
    /* map_xmin, map_xmax, map_xstep */
    /* map_ymin, map_ymax, map_ystep */
    strcpy(map_description, map_plot_label);
    map_plot_label = map_description;
    /* map_in_dim, map_out_dim */
    /* map_npoints */
    /* map_scat_n_patterns */
    strcpy(scatter_plot_data_name, map_scat_data_fullname);
    map_scat_data_fullname = scatter_plot_data_name;
    /* no_map_graphics */
    map_macdots_fname = "map_dots.mac";
    map_macscatter_fname = "map_scat.mac";

    if(default_file != NULL){
	nargc = nnargs_from_file(default_file, default_fullname,
				 plot_argtab, PLOT_TABSIZE);
    } else {
	bound_autoscale_arg = 1;
	cost_autoscale_arg = 1;
	norm_autoscale_arg = 1;
	map_autoscale_arg = 1;
    }
    outputs_plot_train = outputs_train_arg;
    scatter_plot_train = scatter_train_arg;
    outputs_plot_test = outputs_test_arg;
    scatter_plot_test = scatter_test_arg;
    cost_plot_flag = cost_plot_arg;
    norm_plot_flag = norm_plot_arg;
    trial_average = trial_average_arg;
    region_plot = clus_region_arg;
    scatter_plot = clus_scat_arg;
    internals_plot = clus_internals_arg;
    bound_autoscale = bound_autoscale_arg;
    cost_autoscale = cost_autoscale_arg;
    norm_auto_scale_flag = norm_autoscale_arg;
    map_autoscale = map_autoscale_arg;
    /* map_mac_flag_arg */
    show_misses = show_misses_arg;
    show_all = all_data_arg;
    distance_limit = distance_limit_arg;
    internals_scale = internals_scale_arg;
    internals_level = internals_level_arg;
}
void write_plot_defaults(default_file, dfname)
     FILE *default_file;
     char *dfname;
{
    /* plot_fname */
    map_autoscale_arg = map_autoscale;
    /* map_xmin, map_xmax, map_xstep */
    /* map_ymin, map_ymax, map_ystep */
    /* map_plot_label */
    /* map_in_dim, map_out_dim */
    /* map_npoints */
    /* map_scat_n_patterns */
    /* map_scat_data_fullname */
    /* no_map_graphics */
    /* map_mac_flag_arg */
    map_macdots_fname = "map_dots.mac";
    map_macscatter_fname = "map_scat.mac";
    
    /* bound_plot_fname */
    profile_plot_fname = "profile_name";
    /* bound_dim_1, bound_dim_2 */
    /* bound_xmin, bound_xmax, bound_xstep */
    /* bound_ymin, bound_ymax, bound_ystep */
    /* bound_plot_label */
    bound_autoscale_arg = bound_autoscale;
    show_misses_arg = show_misses;
    all_data_arg = show_all;
    distance_limit_arg = distance_limit;
    internals_scale_arg = internals_scale;
    internals_level_arg = internals_level;
    macdots_fname = "boundary_dots.mac";
    macscatter_fname = "scatter.mac";
    maclines_fname = "profile_lines.mac";

    /* cost_plot_fname */
    cost_autoscale_arg = cost_autoscale;
    /* cost_xmin, cost_xmax, cost_xstep */
    /* cost_ymin, cost_ymax, cost_ystep */
    /* cost_plot_label */

    /* norm_plot_fname */
    norm_autoscale_arg = norm_auto_scale_flag;
    /* norm_xmin, norm_xmax, norm_xstep */
    /* norm_ymin, norm_ymax, norm_ystep */
    /* norm_plot_label */

    outputs_train_arg = outputs_plot_train;
    scatter_train_arg = scatter_plot_train;
    outputs_test_arg = outputs_plot_test;
    scatter_test_arg = scatter_plot_test;
    cost_plot_arg = cost_plot_flag;
    norm_plot_arg = norm_plot_flag;
    trial_average_arg = trial_average;
    clus_region_arg = region_plot;
    clus_scat_arg = scatter_plot;
    clus_internals_arg = internals_plot;
    if(default_file != NULL){
	fprintf(default_file, "plot_bound %s",
		nnargs_get_flags(bound_argtab, BOUND_TABSIZE));
	fprintf(default_file, "plot_cost %s",
		nnargs_get_flags(cost_argtab, COST_TABSIZE));
	fprintf(default_file, "plot_norm %s",
		nnargs_get_flags(norm_argtab, NORM_TABSIZE));
	fprintf(default_file, "plot_map %s",
		nnargs_get_flags(map_argtab, MAP_TABSIZE));
	fprintf(default_file, "plot %s",
		nnargs_get_flags(plot_argtab, PLOT_TABSIZE));
    }
}

void store_plot_commands(shell_file, error_fname_base, ext_train)
     FILE *shell_file;
     char *error_fname_base, *ext_train;
{
    char tmp_error_fname_space[MAXSTR];

    if (cost_plot_flag) {
	strcpy(tmp_error_fname_space, error_fname_base);
	strcat(tmp_error_fname_space, ext_train);
	plot_error_fname = tmp_error_fname_space;
	store_cost_plot_command(shell_file);
    }
    if(norm_plot_flag) {
	plot_error_fname = map_params.norm_fname;
	store_norm_plot_command(shell_file);
    }
}
static    char *map_temp;

void setup_epoch_plot_calls(shell_file)
     FILE *shell_file;
{
    fprintf(shell_file, "set plotfile = %s\n", plot_fname);
    fprintf(shell_file, "set plotnum = 0\n");
    fprintf(shell_file, "set pfname = $plotfile$plotnum\n");
    map_temp = plot_fname;
    plot_fname = "$pfname";
}

void finish_epoch_plot_calls(shell_file)
     FILE *shell_file;
{
    if(scatter_plot_train || outputs_plot_train)
      fprintf(shell_file, "olxplot $plotfile{$plotnum_list} &\n");
    plot_fname = map_temp;
}


/*
 * Notify callback function for `plot_train'.
 */
void map_plot_train_handler(item, value, event)
     Panel_item	item;
     int		value;
     Event		*event;
{
    outputs_plot_train = (value & 1) ? 1 : 0;
    scatter_plot_train = (value & 2) ? 1 : 0;
}

/*
 * Notify callback function for `plot_test'.
 */
void map_plot_test_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    outputs_plot_test = (value & 1) ? 1 : 0;
    scatter_plot_test = (value & 2) ? 1 : 0;
}

/*
 * Notify callback function for `clus_dim_1'.
 */
Panel_setting clus_dim_1_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);

    if (value < 1) value = 1;
    if (value > map_params.nraw) value = map_params.nraw;
    xv_set (gmap_clus_plot->clus_dim_1, PANEL_VALUE, value, NULL);
    bound_dim_1 = value;

    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `clus_dim_2'.
 */
Panel_setting clus_dim_2_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);

    if (value < 1) value = 1;
    if (value > map_params.nraw) value = map_params.nraw;
    xv_set (gmap_clus_plot->clus_dim_2, PANEL_VALUE, value, NULL);
    bound_dim_2 = value;

    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `clus_other_vars'.
 */
Panel_setting clus_other_vars_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char tmp[NNMAX_LINE];
    char *	value = (char *) xv_get(item, PANEL_VALUE);
	
    nnargs_get_float_list(strcpy(tmp, value), other_vars,
			  NNARGS_MAX_LABELS);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `clus_no_norm'.
 */
void clus_no_norm_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    no_norm = value;
}

void g_set_clus_autoscale (value)
     int value;
{
	if (value) {
	    bound_autoscale = TRUE; 
	    xv_set(gmap_clus_plot->clus_axes_limits, PANEL_INACTIVE,
		   TRUE, NULL);
	    xv_set (gmap_clus_plot->clus_xmin, PANEL_INACTIVE, TRUE, NULL);
	    xv_set (gmap_clus_plot->clus_ymin, PANEL_INACTIVE, TRUE, NULL);
	    xv_set (gmap_clus_plot->clus_xmax, PANEL_INACTIVE, TRUE, NULL);
	    xv_set (gmap_clus_plot->clus_ymax, PANEL_INACTIVE, TRUE, NULL);
	    xv_set (gmap_clus_plot->clus_xstep, PANEL_INACTIVE, TRUE, NULL);
	    xv_set (gmap_clus_plot->clus_ystep, PANEL_INACTIVE, TRUE, NULL);
	} else {
	    bound_autoscale = FALSE;
	    xv_set (gmap_clus_plot->clus_axes_limits, PANEL_INACTIVE,
		    FALSE, NULL);
	    xv_set (gmap_clus_plot->clus_xmin, PANEL_INACTIVE, FALSE, NULL);
	    xv_set (gmap_clus_plot->clus_ymin, PANEL_INACTIVE, FALSE, NULL);
	    xv_set (gmap_clus_plot->clus_xmax, PANEL_INACTIVE, FALSE, NULL);
	    xv_set (gmap_clus_plot->clus_ymax, PANEL_INACTIVE, FALSE, NULL);
	    xv_set (gmap_clus_plot->clus_xstep, PANEL_INACTIVE, FALSE, NULL);
	    xv_set (gmap_clus_plot->clus_ystep, PANEL_INACTIVE, FALSE, NULL);
	}
}

/*
 * Notify callback function for `clus_npoints'.
 */
Panel_setting clus_npoints_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);

    bound_npoints = value;
    return panel_text_notify(item, event);
}
