/***---------------------------------------------------------------------------
* NAME:		gmap_subs.c
* PURPOSE:	GUIDE Neural Network control panel handler functions
*
*
* USAGE:	gmap
*
* SEE ALSO:	gmap.c gmap_main.c gmap.G
*
* DESCRIPTION: 
*	
*
* BUGS AND LIMITATIONS:
*
*
--------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gmap/lnkmap/gmap_subs.c,v 10.3 2001/08/23 20:14:09 craig Exp $";



#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <gdd.h>
#include "gmap_ui.h"

/* GLOBAL POPUP WINDOW DEFINIITIONS */
extern gmap_plot_pop_objects	*gmap_plot_pop;
extern gmap_clus_plot_objects	*gmap_clus_plot;
extern gmap_feature_pop_objects	*gmap_feature_pop;
extern gmap_file_pop_objects	*gmap_file_pop;
extern gmap_map_plot_objects	*gmap_map_plot;
extern gmap_norm_plot_objects	*gmap_norm_plot;
extern gmap_cost_plot_objects	*gmap_cost_plot;

/* algorithm integer designator */
/* this list must be the same as the one in gmap_main.c */
#define MLPM 0
#define RBFM 1
#define IRBFM 2
#define KMEANSM 3

void set_algorithm();

/*
 * Menu handler for `(MLPM (Multi-Layer Perceptron))'.
 */
Menu_item alg_mlpm_handler(item, op)
     Menu_item		item;
     Menu_generate	op;
{	
    if(op == MENU_NOTIFY) set_algorithm(MLPM);
    return item;
}

/*
 * Menu handler for `(RBFM (Radial Basis Function))'.
 */
Menu_item alg_rbfm_handler(item, op)
     Menu_item		item;
     Menu_generate	op;
{	
    if(op == MENU_NOTIFY) set_algorithm(RBFM);
    return item;
}

/*
 * Menu handler for `(IRBFM (Incremental RBF))'.
 */
Menu_item alg_irbfm_handler(item, op)
     Menu_item		item;
     Menu_generate	op;
{	
    if(op == MENU_NOTIFY) set_algorithm(IRBFM);
    return item;
}

/*
 * Menu handler for `(KMEANSM (Kmeans Clusterer))'.
 */
Menu_item alg_kmeansm_handler(item, op)
     Menu_item		item;
     Menu_generate	op;
{	
    if(op == MENU_NOTIFY) set_algorithm(KMEANSM);
    return item;
}

/*
 * Notify callback function for `map_autoscale'.
 */
void map_autoscale_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    void set_map_autoscale();
    set_map_autoscale(value);
}

/*
 * Notify callback function for `map_plot_label'.
 */
Panel_setting map_plot_label_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    void set_map_plot_label();

    set_map_plot_label(value);
    return panel_text_notify(item, event);
}
#if 0
/*
 * Notify callback function for `map_select_test'.
 */
void map_select_test_handler(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
	g_map_select_test(value);
}

/*
 * Notify callback function for `map_select_train'.
 */
void map_select_train_handler(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
	g_map_select_train(value);
}
#endif
/*
 * Notify callback function for `data_file'.
 */
Panel_setting dname_handler(item, event)
	Panel_item	item;
	Event		*event;
{
	char *	value = (char *) xv_get(item, PANEL_VALUE);
	void set_data_file();
	
	set_data_file(value);
	return panel_text_notify(item, event);
}

/*
 * Notify callback function for `experiment_number'.
 */
Panel_setting experiment_number_handler(item, event)
	Panel_item	item;
	Event		*event;
{
	int	value = (int) xv_get(item, PANEL_VALUE);
	void set_experiment_number();
	
	set_experiment_number(value);
	return panel_text_notify(item, event);
}

/*
 * Notify callback function for `standard_data_set'.
 */
void standard_data_set_handler(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
	void set_standard_data_set();

	set_standard_data_set(value);
}

/*
 * Notify callback function for `show_all_check_box'.
 */
void show_all_handler(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
	void g_set_show_all();

	g_set_show_all(value);
}

/*
 * Notify callback function for `clus_autoscale'.
 */
void clus_autoscale_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    void g_set_clus_autoscale();

    g_set_clus_autoscale(value);
}

/*
 * Notify callback function for `clus_plot_label'.
 */
Panel_setting clus_plot_label_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    void g_set_clus_plot_label();

    g_set_clus_plot_label(value);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `cost_autoscale'.
 */
void cost_autoscale_handler(item, value, event)
     Panel_item	item;
     int		value;
     Event		*event;
{
    void g_set_cost_autoscale();
    g_set_cost_autoscale(value);
}

/*
 * Notify callback function for `cost_plot_label'.
 */
Panel_setting cost_plot_label_handler(item, event)
     Panel_item	item;
     Event		*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    void g_set_cost_plot_label();
    g_set_cost_plot_label(value);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `bound_select_test'.
 */
void bound_select_test_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    void g_bound_select_test();
    g_bound_select_test(value);
}

/*
 * Notify callback function for `bound_select_train'.
 */
void bound_select_train_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    void g_bound_select_train();
    g_bound_select_train(value);
}

/*
 * Notify callback function for `error_graph_select'.
 */
void error_graph_select_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    void g_set_cost_plots();
    g_set_cost_plots(value);
}

/*
 * Notify callback function for `norm_graph_select'.
 */
void norm_graph_select_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;
{
    void g_set_norm_plot();
    g_set_norm_plot(value);
}

/*
 * Notify callback function for `norm_plot_label'.
 */
Panel_setting norm_plot_label_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    void g_set_norm_plot_label();
    g_set_norm_plot_label(value);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `normalize'.
 */
void normalize_handler(item, value, event)
     Panel_item      item;
     int             value;
     Event           *event;
{
    void g_set_normalize();
    g_set_normalize(value);
}

/*
 * Notify callback function for `cvtest_flag'.
 */
void cvtest_handler(item, value, event)
     Panel_item      item;
     int             value;
     Event           *event;
{
    void g_set_cvtest_flag();
    g_set_cvtest_flag(value);
}

/*
 * Notify callback function for `feature_select'.
 */
void
gmap_window1_feature_select_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	xv_set(gmap_feature_pop->feature_pop, XV_SHOW, TRUE, NULL);
	
	/* gxv_end_connections */

}

/*
 * Notify callback function for `plots'.
 */
void
gmap_window1_plots_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	xv_set(gmap_plot_pop->plot_pop, FRAME_CMD_PUSHPIN_IN, TRUE, NULL);
	xv_set(gmap_plot_pop->plot_pop, XV_SHOW, TRUE, NULL);
	
	/* gxv_end_connections */

}


/*
 * Notify callback function for `file_button'.
 */
void
gmap_window1_file_button_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	xv_set(gmap_file_pop->file_pop, XV_SHOW, TRUE, NULL);
	
	/* gxv_end_connections */

}


/*
 * Notify callback function for `map_plot_parameters'.
 */
void
gmap_plot_pop_map_plot_parameters_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	xv_set(gmap_map_plot->map_plot, FRAME_CMD_PUSHPIN_IN, TRUE, NULL);
	xv_set(gmap_map_plot->map_plot, XV_SHOW, TRUE, NULL);
	
	/* gxv_end_connections */

}


/*
 * Notify callback function for `cost_plot_parameters'.
 */
void
gmap_plot_pop_cost_plot_parameters_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	xv_set(gmap_cost_plot->cost_plot, FRAME_CMD_PUSHPIN_IN, TRUE, NULL);
	xv_set(gmap_cost_plot->cost_plot, XV_SHOW, TRUE, NULL);
	
	/* gxv_end_connections */

}


/*
 * Notify callback function for `norm_plot_parameters'.
 */
void
gmap_plot_pop_norm_plot_parameters_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	xv_set(gmap_norm_plot->norm_plot, XV_SHOW, TRUE, NULL);
	
	/* gxv_end_connections */

}


/*
 * Notify callback function for `cluster_plot_parameters'.
 */
void
gmap_plot_pop_cluster_plot_parameters_notify_callback(item, event)
	Panel_item	item;
	Event		*event;
{
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	xv_set(gmap_clus_plot->clus_plot, FRAME_CMD_PUSHPIN_IN, TRUE, NULL);
	xv_set(gmap_clus_plot->clus_plot, XV_SHOW, TRUE, NULL);
	
	/* gxv_end_connections */

}
