/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <nnlib.h>

#include "gseq/gseq_ui.h"
#include "gseq/nn_params.h"
gseq_main_window_objects	*Gseq_main_window;

#define MAXSTR 256
static char chartmp[MAXSTR];

/* general routines for use by LNKseq windows */

void
command_file_close (shell_file, filename, message)
     char *filename, *message;
     FILE *shell_file;
{
    nn_fclose (shell_file, filename, message);
    
    sprintf (chartmp, "chmod +x %s", filename);
    system (chartmp);
}

int
execute_command_file(filename)
     char *filename;
{
    int cur_com_pgrp;
    sprintf (chartmp, "csh -ef %s&", filename);
    cur_com_pgrp = nn_system (chartmp);
    return(cur_com_pgrp);
}

void
stop_command_file(cur_com_pgrp)
     int cur_com_pgrp;
{
    if(cur_com_pgrp != 0){
	killpg (cur_com_pgrp, SIGKILL);
	printf("\nKilled the most recently started run file\n");
	cur_com_pgrp = 0;
    }
}

int
chdir_and_execute_command(shell_fname)
     char *shell_fname;
{
    int pgrp;
    if(chdir(nn_params.exp_path))
      warning("couldn't change directory to %s", nn_params.exp_path);
    pgrp = execute_command_file(shell_fname);
    if(chdir(nn_params.working_directory))
      warning("couldn't change directory to %s", nn_params.working_directory);
    return pgrp;
}

FILE *
command_file_open (filename, message)
     char *filename, *message;
{
    FILE *shell_file;
    /*SUPPRESS 594*/
    int rv;
    
    shell_file = fopen(filename, "w", message);
    if(shell_file == NULL){
	rv = notice_prompt(Gseq_main_window->main_window, NULL,
			   NOTICE_MESSAGE_STRINGS,
			   "Cannot create file for shell script",
			   filename,
			   NULL,
			   NOTICE_BUTTON_YES,	"OK",
			   NULL);
	return NULL;
    }
    fprintf (shell_file, "#!/bin/csh -ef\n# %s\n", filename);
    
    return shell_file;
}

int
file_exists (filename)
     char *filename;
{
    FILE *file;
    int return_val;
    
    if ((file = fopen(filename, "r")) == NULL) {
    	return_val = FALSE;
    } else {
    	return_val = TRUE;
	nn_fclose(file, filename, "for file existence check");
    }
    return return_val;
}

char *
make_net_string (array)
     int array[];
{
    static char dummy[256];
    int i;
    
    *dummy = NULL;
    if(array == NULL) return dummy;
    if (array[0] > 0) {
	sprintf (dummy, "%d", array[1]);
    }
    if (array[0] > 1) {
	for(i=2; i<= array[0] && i <= NNARGS_MAX_NET; i++)
	  sprintf (dummy + strlen(dummy), ",%d", array[i]);
    }
    return dummy;
}

char *
make_float_string (array)
     float array[];
{
    static char dummy[256];
    int i;
    
    *dummy = NULL;
    if(array == NULL) return dummy;
    if (array[0] > 0) {
	sprintf (dummy, "%g", array[1]);
    }
    if (array[0] > 1) {
	for(i=2; i<= array[0] && i <= NNARGS_MAX_LABELS; i++)
	  sprintf (dummy + strlen(dummy), ",%g", array[i]);
    }
    return dummy;
}

char *
make_labels_string (array)
     char *array[];
{
    char *dummy;
    int i, array_len, total_length;
    
    if(array == NULL) array_len = 0;
    else array_len = atoi(array[0]);
    /* add up the length of the string to be created */
    total_length = 0;
    for(i = 1; i <= array_len; i++)
      total_length += strlen(array[i]);
    /* remember the commas and the NIL at the end */
    total_length += array_len + 1;

    /* allocate space for the string */
    dummy = nn_malloc(total_length, "for make_labels_string");
    dummy[0] = '\0';

    if(array_len > 0){
	for(i = 1; i <= array_len; i++){
	    strcat(dummy, array[i]);
	    strcat(dummy, ",");
	}
    }
    /* take off the comma at the end */
    total_length = strlen(dummy);
    if(total_length > 1) dummy[total_length-1] = '\0';

    return dummy;
}

int
overwrite_ok (filename)
     char *filename;
{
    int result, return_value;
    
    return_value = TRUE;
    if (file_exists(filename)) {
	result = notice_prompt(Gseq_main_window->main_window, NULL,
			       NOTICE_MESSAGE_STRINGS,
			       "Shell file exists: OK to overwrite?",
			       NULL,
			       NOTICE_BUTTON_YES,	"Overwrite",
			       NOTICE_BUTTON_NO,	"Cancel",
			       NULL);
	if(result != NOTICE_YES) return_value = FALSE;
    }
    return return_value;
}
    
void 
set_float_string (item, format, value)
     unsigned int item;
     char *format;
     float value;
{
    char *str[10];
    
    sprintf (str, format, value);
    xv_set (item, PANEL_VALUE, str, NULL);
}

void
set_int_string (item, format, value)
     unsigned int item;
     char *format;
     int value;
{
    char *str[10];
    
    sprintf (str, format, value);
    xv_set (item, PANEL_VALUE, str, NULL);
}

void
set_int_label (item, format, value)
     unsigned int item;
     char *format;
     int value;
{
    char *str[10];
    
    sprintf (str, format, value);
    xv_set (item, PANEL_LABEL_STRING, str, NULL);
}

float
set_atof_value(str, name, item, value)
     char *str, *name;
     unsigned int item;
     float value;
{
    float new_val;
    
    if(sscanf (str, "%f", &new_val) == 1){
	return new_val;
    } else {
	warning("set_atof_value -- unable to set %s to %s",
		name, str);
	set_float_string(item, "%5.2g", value);
	return value;
    }
}

int
set_atoi_value(str, name, item, value)
     char *str, *name;
     unsigned int item;
     int value;
{
    int new_val;
    
    if(sscanf (str, "%d", &new_val) == 1){
	return new_val;
    } else {
	warning("set_atoi_value -- unable to set %s to %s",
		name, str);
	set_float_string(item, "%d", (float)value);
	return value;
    }
}

float
set_atof_range_value(str, name, item, value, low, high)
     char *str, *name;
     unsigned int item;
     float value, low, high;
{
    float new_val;
    
    if(sscanf (str, "%f", &new_val) == 1){
	if(new_val >= low && new_val <= high)
	  return new_val;
	else {
	    warning("set_atof_range_value -- new %s (%f) out of range (%f,%f)",
		    name, new_val, low, high);
	    set_float_string(item, "%5.2g", value);
	    return value;
	}
    } else {
	warning("set_atof_range_value -- unable to set %s to %s",
		name, str);
	set_float_string(item, "%5.2g", value);
	return value;
    }
}

float
set_atof_low_value(str, name, item, value, low)
     char *str, *name;
     unsigned int item;
     float value, low;
{
    float new_val;
    
    if(sscanf (str, "%f", &new_val) == 1){
	if(new_val >= low)
	  return new_val;
	else {
	    warning("set_atof_low_value -- new %s (%f) < (%f)",
		    name, new_val, low);
	    set_float_string(item, "%5.2g", value);
	    return value;
	}
    } else {
	warning("set_atof_low_value -- unable to set %s to %s",
		name, str);
	set_float_string(item, "%5.2g", value);
	return value;
    }
}

int
set_atoi_low_value(str, name, item, value, low)
     char *str, *name;
     unsigned int item;
     int value, low;
{
    int new_val;
    
    if(sscanf (str, "%d", &new_val) == 1){
	if(new_val >= low)
	  return new_val;
	else {
	    warning("set_atoi_low_value -- new %s (%d) < (%d)",
		    name, new_val, low);
	    set_int_string(item, "%5.2g", value);
	    return value;
	}
    } else {
	warning("set_atoi_low_value -- unable to set %s to %s",
		name, str);
	set_int_string(item, "%5.2g", value);
	return value;
    }
}

int
set_atoi_range_value(str, name, item, value, low, high)
     char *str, *name;
     unsigned int item;
     int value, low, high;
{
    int new_val;
    
    if(sscanf (str, "%d", &new_val) == 1){
	if(new_val >= low && new_val <= high)
	  return new_val;
	else {
	    warning("set_atoi_range_value -- new %s (%d) out of range (%d,%d)",
		    name, new_val, low, high);
	    set_int_string(item, "%d", value);
	    return value;
	}
    } else {
	warning("set_atoi_range_value -- unable to set %s to %s",
		name, str);
	set_int_string(item, "%d", value);
	return value;
    }
}

/* these routines are used when setting string arguments for
   saving and restoring experiment screens */

/* if string1 is NULL or and empty string, return NULL
   otherwise return string1 */
char *
null_if_empty_string(string1)
     char *string1;
{
    if(string1 == NULL) return NULL;
    if(strcmp(string1, "") == 0) /* string is empty */ return NULL;
    return string1; /* string is not empty */
}

/* is string2 is null, make string1 the empty string,
   otherwise copy string2 into string 1 */
char *
strcpy_empty_if_null(string1, string2)
     char *string1, *string2;
{
    if(string2 == NULL) strcpy(string1, "");
    else strcpy(string1, string2);
    return string1;
}
