/***--------------------------------------------------------------------------
 * NAME:        ggauss.c
 * PURPOSE:     Gaussian Classifier GUIDE handler functions
 *		Executes gauss functions after setting command line parameters
 *
 * USAGE:	gseq	(from command line - produces control panel window)
 *
 * SEE ALSO:	gseq_main.c gseq.c gseq_subs.c gauss.c gauss_pack.c ggauss_subs.c
 *
 * DESCRIPTION:
 *
 *	ggauss_display_params
 *	ggauss_get_plot_label
 *	ggauss_init
 *	ggauss_set_covariance_type
 *	ggauss_set_minimum_variance
 *	ggauss_set_variance_type
 *		
 * BUGS AND LIMITATIONS:
 *
 *
 ------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gseq/gauss/ggauss.c,v 10.4 2001/08/23 20:14:14 craig Exp $";



#define TRUE 1
#define FALSE 0
#define MAXSTR 256
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <unistd.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <gdd.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <nnlib.h>
#include "ggauss_ui.h"
#include <gauss_seq_args.h>			/* nnargs definitions */
#include "../gseq/nn_params.h"

extern ggauss_popup1_objects	*Ggauss_popup1;

void set_plot_descriptions();
void ggauss_set_common_flags();
void set_float_string();
float set_atof_value(), set_atof_range_value(), set_atof_low_value();

void ggauss_display_params()
{
    xv_set(Ggauss_popup1->covariance_type, PANEL_VALUE, covariance_type, NULL);
    xv_set(Ggauss_popup1->variance_type, PANEL_VALUE, variance_type, NULL);
    set_float_string (Ggauss_popup1->minimum_variance,
		      "%5.2g", minimum_variance);
}

char * ggauss_get_plot_label (string)
     char *string;
{
    if (covariance_type==0) strcat (string," Diagonal"); 
    else 		    strcat(string," Full");
    if (variance_type==0) strcat (string," Grand"); 
    else 		  strcat(string," Per Class");
    return string;
}

void ggauss_init ()
{
    ggauss_display_params();
}

/*
 * Notify callback function for `covariance_type'.
 */
void
  ggauss_covariance_type_handler(item, value, event)
Panel_item	item;
int		value;
Event		*event;/*ARGSUSED*/
{
    covariance_type = value;
}

/*
 * Notify callback function for `minimum_variance'.
 */
Panel_setting ggauss_minimum_variance_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    minimum_variance =
      set_atof_low_value(value, "minimum_variance",
			 Ggauss_popup1->minimum_variance, minimum_variance, 0.);
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `variance_type'.
 */
void ggauss_variance_type_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;/*ARGSUSED*/
{
    variance_type = value;
}

char * ggauss_get_args ()
{
    return (nnargs_get_part_flags(gauss_tab, GAUSS_TABSIZE));
}


static int variance_arg = 0;
static int covariance_arg = 0;
static ARG argtab2[]=
{
    {"variance_arg", INT, &variance_arg, "batch type"},
    {"covariance_arg", INT, &covariance_arg, "batch type"},
};
#define TABSIZE2 (sizeof(argtab2)/sizeof(ARG))

void write_gauss_defaults(default_file)
     FILE *default_file;
{
    covariance_arg = covariance_type;
    variance_arg = variance_type;
    if(default_file != NULL){
	fprintf (default_file, "gauss %s",
		 nnargs_get_part_flags(gauss_tab, GAUSS_TABSIZE));
	fprintf(default_file,"%s",nnargs_get_flags(argtab2, TABSIZE2));
    }
}

void read_gauss_defaults(default_file, default_fullname)
     FILE *default_file;
     char *default_fullname;
{
    int nargc;
    char *argv[256];

    if(default_file != NULL){
	nargc = fill_argv(argv, default_file, default_fullname);
	nargc = nnargs_part(nargc, argv, gauss_tab, GAUSS_TABSIZE);
	nargc = nnargs(nargc, argv, argtab2, TABSIZE2);
    }
    variance_type = variance_arg;
    covariance_type = covariance_arg;
    
    /* algorithm specific flags are now set.  generic flags are taken
       from the argument list for the main screen */
}
