/***--------------------------------------------------------------------------
* NAME:        ggmix.c
* PURPOSE:     Gaussian Mixture main program driver (DEVGUIDE version)
*		Executes gmix after setting command line parameters
*
* USAGE:	LNKseq	(from command line - produces control panel window)
*		then select GMIX from Algorithm Menu
*
* SEE ALSO:	gmix.c gmix_pack.c gmix_subs.c ggmix_subs.c c_subs.c
*
* DESCRIPTION: 
*	ggmix_display_params
*	ggmix_init
*	ggmix_get_plot_label
*	ggmix_set_common_flags
*	ggmix_set_description_values
*	ggmix_store_run_command
*
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/gseq/gmix/ggmix.c,v 10.4 2001/08/23 20:14:17 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/gseq/gmix/ggmix.c,v $
* created 23-Jan-91 lck
-------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gseq/gmix/ggmix.c,v 10.4 2001/08/23 20:14:17 craig Exp $";




#define TRUE 1
#define FALSE 0

#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>

#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <sys/ioctl.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/icon_load.h>
#include <xview/panel.h>
#include <xview/scrollbar.h>
#include <xview/svrimage.h>
#include <xview/termsw.h>
#include <xview/text.h>
#include <xview/tty.h>
#include <xview/xv_xrect.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <gcm.h>
#include <gdd.h>
#include <nnlib.h>
#include "ggmix_ui.h"
#include <gmix_seq_args.h>	/* statics for gmix args */
#include "../gseq/nn_params.h"
#include "../gseq/galgs.h"

#define MAXSTR 256

			/* static GLOBALS internal to this file */
static char *clustering_algorithm_name = "kmeans";
static char param_ftmp[MAXSTR];

static int clustering_active_flag;
static int clustering_algorithm = KMEANS;

/* Needed window pointers */
extern ggmix_param_pop_objects	*Ggmix_param_pop;

/* routines found in this file */
void ggmix_display_params();
void ggmix_init ();
void ggmix_store_run_command ();
void ggmix_set_common_flags();
void ggmix_set_description_values();
void ggmix_set_cluster_algorithm();
float set_atof_value(), set_atof_range_value(), set_atof_low_value();
void set_float_string();

void ggmix_display_params()
{
    xv_set (Ggmix_param_pop->clustering_active, PANEL_VALUE,
	    clustering_active_flag, NULL);
    xv_set(Ggmix_param_pop->clustering_algorithm, PANEL_VALUE,
	   clustering_algorithm, NULL);
    xv_set(Ggmix_param_pop->cluster_param_fname, PANEL_VALUE,
	   cluster_param_fname, NULL);
    ggmix_set_cluster_algorithm(clustering_algorithm);
    ggmix_set_cluster_param_fname();
    xv_set (Ggmix_param_pop->cov_type, PANEL_VALUE, full_flag, NULL);
    xv_set (Ggmix_param_pop->grand_type, PANEL_VALUE, grand_flag, NULL);
    xv_set (Ggmix_param_pop->mix_type, PANEL_VALUE, tied_flag, NULL);
    xv_set (Ggmix_param_pop->epochs, PANEL_VALUE, nepochs, NULL);
    set_float_string (Ggmix_param_pop->min_var, "%g", minimum_variance);
    set_float_string (Ggmix_param_pop->max_ratio, "%g", max_ratio);
    set_float_string (Ggmix_param_pop->var_spread, "%g", var_spread);
    set_float_string (Ggmix_param_pop->stop_percentage, "%g",
		      stop_percentage);
}

void ggmix_init ()
{
    ggmix_display_params();
}

char * ggmix_get_plot_label(string)
     char *string;
{
    if(tied_flag) strcpy (string, "Tied");
    else strcpy (string, "Untied");
    if(grand_flag) strcat (string, " Grand");
    else if(grand_flag == 1) strcat (string, " per Class");
    else strcat(string, "per Component");
    if(full_flag) strcat (string, " Full");
    else strcat(string, " Diagonal");

    return string;
}

/*
 * Notify callback function for `clustering_active'.
 */
void
ggmix_create_clusters_handler(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;/*ARGSUSED*/
{
    clustering_active_flag = value;
}


/*
 * Notify callback function for `clustering_algorithm'.
 */
void
ggmix_cluster_algorithm_handler(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;/*ARGSUSED*/
{
    int temp;
    switch (value){
      case 0:
	ggmix_set_cluster_algorithm(KMEANS);
	break;
    }
    temp = clustering_active_flag;
    clustering_active_flag = 1;
    ggmix_set_cluster_param_fname();
    clustering_active_flag = temp;
}

void
ggmix_set_cluster_algorithm(value)
     int value;
{
    clustering_algorithm = value;
    switch(value){
      case KMEANS:
	clustering_algorithm_name = "kmeans";
	break;
      default:
	break;
    }
    /* cluster_param_fname still needs to be set */
}
ggmix_set_cluster_param_fname()
{
    if(clustering_active_flag){
	strcpy(param_ftmp, nn_params.exp_prefix);
	strcat(param_ftmp, clustering_algorithm_name);
	cluster_param_fname = strcat(param_ftmp,".param");
	xv_set(Ggmix_param_pop->cluster_param_fname, PANEL_VALUE,
	       cluster_param_fname, NULL);
    }
}


/*
 * Notify callback function for `clustering_parameters'.
 */
void
ggmix_cluster_parameters_handler(item, event)
	Panel_item	item;
	Event		*event;/*ARGSUSED*/
{
    void gkmeans_show_menu();

    switch(clustering_algorithm){
      case KMEANS:
	gkmeans_show_menu();
	break;
    }
}


/*
 * Notify callback function for `cluster_param_fname'.
 */
Panel_setting
ggmix_cluster_param_fname_handler(item, event)
	Panel_item	item;
	Event		*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
	
    cluster_param_fname = value;
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `cov_type'.
 */
void
ggmix_cov_type_handler(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;/*ARGSUSED*/
{
    full_flag = value;
}


/*
 * Notify callback function for `mix_type'.
 */
void
ggmix_mix_type_handler(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;/*ARGSUSED*/
{
    tied_flag = value;
}


/*
 * Notify callback function for `grand_type'.
 */
void
ggmix_grand_type_handler(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;/*ARGSUSED*/
{
    grand_flag = value;
}

/*
 * Notify callback function for `epochs'.
 */
Panel_setting
ggmix_epochs_handler(item, event)
	Panel_item	item;
	Event		*event;
{
	int	value = (int) xv_get(item, PANEL_VALUE);
	
    nepochs = value;
	return panel_text_notify(item, event);
}

int ggmix_get_epochs()
{
    return nepochs;
}

void ggmix_set_epochs(e)
int e;
{
    /* popup will not reflect the set number of epochs */
    nepochs = e;
}

/*
 * Notify callback function for `min_var'.
 */
Panel_setting ggmix_min_var_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);

    minimum_variance =
      set_atof_low_value(value, "minimum_variance",
			 Ggmix_param_pop->min_var, minimum_variance, 0.);
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `max_ratio'.
 */
Panel_setting ggmix_max_ratio_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);

    max_ratio =
      set_atof_low_value(value, "max_ratio",
			 Ggmix_param_pop->max_ratio, max_ratio, 1.);
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `var_spread'.
 */
Panel_setting ggmix_var_spread_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);

    var_spread = set_atof_low_value(value, "var_spread",
				    Ggmix_param_pop->var_spread, var_spread, 0.);
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `stop_percentage'.
 */
Panel_setting ggmix_stop_percentage_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);

    stop_percentage =
      set_atof_low_value(value, "stop_percentage",
			 Ggmix_param_pop->stop_percentage,
			 stop_percentage, 0.);
    return panel_text_notify(item, event);
}

char * ggmix_get_args ()
{
    return (nnargs_get_part_flags(gmix_tab, GMIX_TABSIZE));
}

char *ggmix_get_cluster_fname(){
    return cluster_param_fname;
}

int ggmix_get_cluster_alg(){
    return clustering_algorithm;
}

int ggmix_must_store_clustering(){
    return clustering_active_flag;
}

static int clustering_active_arg = 0;
static int tied_arg = 0;
static int full_arg = 0;
static ARG argtab2[]=
{
    {"clus_active_arg", INT, &clustering_active_arg,
       "write cluster calls on create"},
    {"clus_alg", INT, &clustering_algorithm, "clustering algorithm number"},
    {"tied_arg", INT, &tied_arg, "tied mixtures"},
    {"full_arg", INT, &full_arg, "full covariance"},
};
#define TABSIZE2 (sizeof(argtab2)/sizeof(ARG))

void write_gmix_defaults(default_file)
     FILE *default_file;
{
    clustering_active_arg = clustering_active_flag;
    tied_arg = tied_flag;
    full_arg = full_flag;
    if(default_file != NULL){
	fprintf (default_file,"gmix %s",
		 nnargs_get_part_flags(gmix_tab,GMIX_TABSIZE));
	fprintf (default_file, "%s", nnargs_get_flags(argtab2, TABSIZE2));
    }
}

void read_gmix_defaults(default_file, default_fullname)
     FILE *default_file;
     char *default_fullname;
{
    int nargc;
    char *argv[256];

    if(default_file != NULL){
	nargc = fill_argv(argv, default_file, default_fullname);
	nargc = nnargs_part(nargc, argv, gmix_tab, GMIX_TABSIZE);
	nargc = nnargs(nargc, argv, argtab2, TABSIZE2);
    } else {
	clustering_active_arg = 1;
	nepochs = 20;
    }
    clustering_active_flag = clustering_active_arg;
    strcpy(param_ftmp, cluster_param_fname);
    cluster_param_fname = param_ftmp;
    tied_flag = tied_arg;
    full_flag = full_arg;
    /* algorithm specific flags are now set.  generic flags are taken
       from the argument list for the main screen */
}
