/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <nnlib.h>
#include "galgs.h"
#include "nn_params.h"

#include "gseq_ui.h"
#include "menus_ui.h"
#include "data_ui.h"
#include "norm_ui.h"
#include "select_ui.h"
#include "reports_ui.h"
#include "../gauss/ggauss_ui.h"
#include "../gmix/ggmix_ui.h"
#include "../rbf/grbf_ui.h"
#include "../mlp/gmlp_ui.h"
#include "../kmeans/gkmeans_ui.h"
#include "../irbf/girbf_ui.h"

extern	gseq_main_window_objects	*Gseq_main_window;
extern	data_data_pop_objects	*Data_data_pop;
extern	data_defaults_pop_objects	*Data_defaults_pop;
extern	norm_norm_pop_objects	*Norm_norm_pop;
extern	norm_norm_generate_pop_objects	*Norm_norm_generate_pop;
extern	select_select_pop_objects	*Select_select_pop;
extern	reports_file_pop_objects	*Reports_file_pop;
extern	ggauss_popup1_objects	*Ggauss_popup1;
extern	ggmix_param_pop_objects	*Ggmix_param_pop;
extern	grbf_popup1_objects	*Grbf_popup1;
extern	gmlp_popup1_objects	*Gmlp_popup1;
extern	gmlp_weight_popup_objects	*Gmlp_weight_popup;
extern	gmlp_cost_popup_objects	*Gmlp_cost_popup;
extern	gmlp_node_popup_objects	*Gmlp_node_popup;
extern	gkmeans_popup1_objects	*Gkmeans_popup1;
extern	girbf_popup1_objects	*Girbf_popup1;

algorithm_number algorithm = KMEANS;

void set_short_algorithm(value)
     int	value;
{
    void activate_test_selects();
    void g_set_norm_plot();
    void g_set_normalize();
    void g_set_train();

    algorithm = (algorithm_number) value;
    nn_params.algorithm = value;

    if(alg_is_clus()){
	/* KMEANS */
	activate_test_selects(FALSE);
    } else {
	activate_test_selects(TRUE);
    }
    
    nn_params.classifier_name = classifier_names[algorithm];
    xv_set (Gseq_main_window->algorithm_sel, PANEL_LABEL_STRING,
	    classifier_strings[algorithm], NULL);
}

int
alg_is_clus()
{
    switch(algorithm){
      case KMEANS:
	return 1;
    }
    return 0;
}

int
alg_is_epoch_class()
{
    switch(algorithm){
      case MLP:
      case IRBF:
      case GMIX:
	return 1;
    }
    return 0;
}

int
alg_is_continuous_outputs()
{
    switch(algorithm){
      case MLP:
      case GAUSS:
      case RBF:
      case IRBF:
      case GMIX:
	return 1;
      default:
	return 0;
    }
}

void
show_algorithm_popup()
{
    switch (algorithm) {
      case MLP:
	xv_set (Gmlp_popup1->popup1, XV_SHOW, TRUE, 0);
	break;
      case GAUSS:
	xv_set (Ggauss_popup1->popup1, XV_SHOW, TRUE, 0);
	break;
      case RBF:
	xv_set (Grbf_popup1->popup1, XV_SHOW, TRUE, 0);
	break;
      case IRBF:
	xv_set (Girbf_popup1->popup1, XV_SHOW, TRUE, 0);
	break;
      case GMIX:
	xv_set (Ggmix_param_pop->param_pop, XV_SHOW, TRUE, 0);
	break;
      case KMEANS:
	xv_set (Gkmeans_popup1->popup1, XV_SHOW, TRUE, 0);
	break;
      default:
	break;
	}
}

char * g_get_plot_label(string)
     char * string;
{
    char *norm_description();
    /* norm string is already there */
    char *gmlp_get_plot_label();
    char *grbf_get_plot_label();
    char *girbf_get_plot_label();
    char *gkmeans_get_plot_label();
    char *ggauss_get_plot_label();
    char *gnorm_get_plot_label();
    char *ggmix_get_plot_label();
    
    sprintf(string, "Norm:%s", norm_description(nn_params.normalize));
    switch (algorithm) {
      case MLP:
	return gmlp_get_plot_label(string);
      case GAUSS:
	return ggauss_get_plot_label(string);
      case RBF:
	return grbf_get_plot_label(string);
      case IRBF:
	return girbf_get_plot_label(string);
      case GMIX:
	return ggmix_get_plot_label(string);
      case KMEANS:
	return gkmeans_get_plot_label(string);
      default:
	return "";
    }
}

void g_update_ninputs()
{
    void gmlp_set_description_values();

    gmlp_set_description_values();
}

void g_get_test_args(shell_file)
     FILE *shell_file;
{
    void grbf_store_plot_command();
    void girbf_store_plot_command();
    switch (algorithm) {
      case RBF:
	grbf_store_plot_command (shell_file);
	break;
      case IRBF:
	girbf_store_plot_command (shell_file);
	break;
      default:
	break;
    }
}

char * g_get_alg_args()
{
    char * gmlp_get_args();
    char * ggauss_get_args(), *grbf_get_args();
    char * girbf_get_args(), *ggmix_get_args();
    char * gkmeans_get_args();
    char * gnorm_get_args();

    switch (algorithm) {
      case MLP:
	return gmlp_get_args ();
      case GAUSS:
	return ggauss_get_args ();
      case RBF:
	return grbf_get_args ();
      case IRBF:
	return girbf_get_args ();
      case GMIX:
	return ggmix_get_args ();
      case KMEANS:
	return gkmeans_get_args ();
      default:
	break;
    }
}

int must_store_clustering(){
    switch(algorithm){
      case RBF:
	return grbf_must_store_clustering();
      case IRBF:
	return girbf_must_store_clustering();
      case GMIX:
	return ggmix_must_store_clustering();
      default:
	return 0;
    }
}

char * g_get_cluster_fname(){
    char * grbf_get_cluster_fname(), *girbf_get_cluster_fname();
    char * ggmix_get_cluster_fname();
    switch(algorithm){
      case RBF:
	return grbf_get_cluster_fname();
      case IRBF:
	return girbf_get_cluster_fname();
      case GMIX:
	return ggmix_get_cluster_fname();
      default:
	return 0;
    }
}

int g_get_cluster_alg(){
    int cluster_alg;
    switch(algorithm){
      case RBF:
	cluster_alg = grbf_get_cluster_alg();
	break;
      case IRBF:
	cluster_alg = girbf_get_cluster_alg();
	break;
      case GMIX:
	cluster_alg = ggmix_get_cluster_alg();
	break;
      default:
	cluster_alg = 0;
	break;
    }
    return cluster_alg;
}

int alg_uses_clustering(){
    switch(algorithm){
      case RBF:
      case IRBF:
      case GMIX:
	return 1;
      default:
	return 0;
    }
}

/* these are intended for use when gseq is storing cluster commands,
   gseq must temporarily change the algorithm to the appropriate clustering
   algorithm */
int g_get_alg(){
    return algorithm;
}

int g_set_alg(alg){
    algorithm = alg;
    nn_params.classifier_name = classifier_names[alg];
}

/*
 * get the number of epochs from the current algorithm
 */
int g_get_epochs()
{
    switch (algorithm) {
      case GMIX:
	return ggmix_get_epochs();
      case MLP:
	return gmlp_get_epochs();
      case IRBF:
	return girbf_get_epochs();
      case GAUSS:
      case RBF:
      case KMEANS:
      default:
	return 1;
    }
}

/*
 * set the number of epochs from the current algorithm
 */
int g_set_epochs(e)
     int e;
{
    void gmlp_set_epochs();
    void ggmix_set_epochs(), girbf_set_epochs();
    switch (algorithm) {
      case MLP:
	gmlp_set_epochs(e);
	break;
      case GMIX:
	ggmix_set_epochs(e);
	break;
      case IRBF:
	girbf_set_epochs(e);
	break;
      case GAUSS:
      case RBF:
      case KMEANS:
      default:
	break;
    }
}

void
read_defaults(dfp, dfname)
     FILE *dfp;
     char *dfname;
{
    void read_main_defaults(), read_reports_defaults();
    void read_data_defaults(), read_norm_defaults();
    void read_select_defaults();
    void read_gauss_defaults(), read_gmix_defaults();
    void read_irbf_defaults();
    void read_kmeans_defaults();
    void read_mlp_defaults();
    void read_rbf_defaults();
    void set_algorithm(), data_set_data_base();

    read_main_defaults(dfp, dfname);
    read_data_defaults(dfp, dfname);
    read_norm_defaults(dfp, dfname);
    read_select_defaults(dfp, dfname);
    read_reports_defaults(dfp, dfname);
    read_gauss_defaults(dfp,dfname);
    read_gmix_defaults(dfp,dfname);
    read_irbf_defaults(dfp,dfname);
    read_kmeans_defaults(dfp,dfname);
    read_mlp_defaults(dfp, dfname);
    read_rbf_defaults(dfp,dfname);

    if(dfp == NULL){
	/* update algorithm derived file names */
	set_algorithm(nn_params.algorithm);
	/* update data base derived file names */
	data_set_data_base();
    }
    g_display();
    g_display_algs();
    /* write messages, like class priors for files, that are generated
       not stored */
/*
    set_description_values();
*/
}

write_defaults(dfp)
     FILE *dfp;
{
    void set_common_flags();
    void write_main_defaults();
    void write_data_defaults(), write_norm_defaults();
    void write_select_defaults();
    void write_reports_defaults();
    void write_gauss_defaults(), write_gmix_defaults();
    void write_irbf_defaults();
    void write_kmeans_defaults();
    void write_mlp_defaults();
    void write_rbf_defaults();

    set_common_flags(0);/* CLASS_TRAIN */

    write_main_defaults(dfp);
    write_data_defaults(dfp);
    write_norm_defaults(dfp);
    write_select_defaults(dfp);
    write_reports_defaults(dfp);
    write_gauss_defaults(dfp);
    write_gmix_defaults(dfp);
    write_irbf_defaults(dfp);
    write_kmeans_defaults(dfp);
    write_mlp_defaults(dfp);
    write_rbf_defaults(dfp);
}

void
g_set_cluster_names()
{
    grbf_set_cluster_param_fname();
    girbf_set_cluster_param_fname();
    ggmix_set_cluster_param_fname();
}

/* call once, on initialization.  sets variables and initializes
   strings in nn_params */
g_init_algs()
{
    void gmlp_init();
    void grbf_init();
    void gkmeans_init();
    void ggauss_init(), girbf_init();
    void ggmix_init();

    gmlp_init ();
    grbf_init ();
    gkmeans_init ();
    ggauss_init ();
    ggmix_init();
    girbf_init ();
}

g_display_algs()
{
    void gmlp_display_params();
    void grbf_display_params();
    void gkmeans_display_params();
    void ggauss_display_params(), ggmix_display_params();
    void girbf_display_params();

    gmlp_display_params ();
    grbf_display_params ();
    gkmeans_display_params ();
    ggauss_display_params ();
    ggmix_display_params();
    girbf_display_params ();
}

void
set_common_flags(which_data)
     int which_data;
{
    void gnorm_set_common_flags(), gseq_set_common_flags();

    gseq_set_common_flags(which_data);
    gnorm_set_common_flags();
}
