/***-------------------------------------------------------------------------
 * NAME:        girbf.c
 * PURPOSE:     Incremental Radial Basis Function Classifier parameters
 *		Executes irbf functions after setting command line parameters
 *
 ------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gseq/irbf/girbf.c,v 10.4 2001/08/23 20:14:22 craig Exp $";



#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <unistd.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <gdd.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <nnlib.h>
#include "girbf_ui.h"
#include <irbf_seq_args.h>			/* nnargs definitions */
#include "../gseq/nn_params.h"
#include "../gseq/galgs.h"

#define TRUE 1
#define FALSE 0
#define MAXSTR 256

static char *clustering_algorithm_name = "kmeans";
static char param_ftmp[MAXSTR];

static int clustering_active_flag;
static int clustering_algorithm = KMEANS;

extern girbf_popup1_objects	*Girbf_popup1;

void set_float_string();
void girbf_set_clustering_algorithm();
float set_atof_value(), set_atof_range_value(), set_atof_low_value();
void girbf_set_cost_func();

void girbf_display_params()
{
    xv_set (Girbf_popup1->clustering_active, PANEL_VALUE,
	    clustering_active_flag, NULL);
    xv_set(Girbf_popup1->clustering_algorithm, PANEL_VALUE,
	   clustering_algorithm, NULL);
    xv_set(Girbf_popup1->cluster_param_fname, PANEL_VALUE,
	   cluster_param_fname, NULL);
    girbf_set_clustering_algorithm(clustering_algorithm);
    girbf_set_cluster_param_fname();
    xv_set (Girbf_popup1->bias, PANEL_VALUE, bias, NULL);
    xv_set(Girbf_popup1->grandvar, PANEL_VALUE, grand_flag, NULL);
    set_float_string (Girbf_popup1->min_var, "%g", minimum_variance);
    set_float_string (Girbf_popup1->weight_eta,"%5g",weight_eta);
    set_float_string (Girbf_popup1->mean_eta,"%5g",mean_eta);
    set_float_string (Girbf_popup1->var_eta,"%5g",var_eta);
    set_float_string (Girbf_popup1->hspread, "%5.2g", hspread);
    set_float_string (Girbf_popup1->extra_hspread, "%5.2g", extra_hspread);
    set_float_string (Girbf_popup1->max_magnitude, "%5.2g", max_magnitude);
    xv_set(Girbf_popup1->epochs, PANEL_VALUE, nepochs, NULL);
    if(cost_func == SQ_ERR)
      xv_set(Girbf_popup1->cost_func, PANEL_VALUE, 0, NULL);
    else if(cost_func == CFOM)
      xv_set(Girbf_popup1->cost_func, PANEL_VALUE, 1, NULL);
    girbf_set_cost_func(cost_func);
    set_float_string(Girbf_popup1->cost_param, "%5.2g", cost_param);
}

char * girbf_get_plot_label (string)
     char *string;
{
    sprintf (string,"H:%g Extra H:%g", hspread, extra_hspread);
    if (bias) strcat (string, " Bias");
    
    return string;
}

void girbf_init ()
{
    girbf_display_params();
}

/*
 * Notify callback function for `clustering_active'.
 */
void girbf_create_clusters_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event; /*ARGSUSED*/
{
    clustering_active_flag = value;
}


/*
 * Notify callback function for `clustering_algorithm'.
 */
void girbf_clustering_algorithm_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;/*ARGSUSED*/
{
    int temp;
    switch(value){
      case 0:
	girbf_set_clustering_algorithm(KMEANS);
	break;
    }
    temp = clustering_active_flag;
    clustering_active_flag = 1;
    girbf_set_cluster_param_fname();
    clustering_active_flag = temp;
}

/* needed by girbf_set_description_values */
void girbf_set_clustering_algorithm (value)
     int value;
{
    clustering_algorithm = value;
    switch(value){
      case KMEANS:
	clustering_algorithm_name = "kmeans";
	break;
      default:
	break;
    }
    /* cluster_param_fname still needs to be set */
}
girbf_set_cluster_param_fname()
{
    if(clustering_active_flag){
	strcpy(param_ftmp,nn_params.exp_prefix);
	strcat(param_ftmp,clustering_algorithm_name);
	cluster_param_fname = strcat(param_ftmp,".param");
	xv_set(Girbf_popup1->cluster_param_fname,PANEL_VALUE,
	       cluster_param_fname, NULL);
    }
}

/*
 * Notify callback function for `clustering_parameters'.
 */
void girbf_clustering_parameters_handler(item, event)
     Panel_item	item;
     Event	*event;/*ARGSUSED*/
{
    void gkmeans_show_menu();

    switch(clustering_algorithm){
      case KMEANS:
	gkmeans_show_menu();
	break;
    }
    
}

/*
 * Notify callback function for `cluster_param_fname'.
 */
Panel_setting girbf_cluster_param_fname_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    cluster_param_fname = value;
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `extra_hspread'.
 */
Panel_setting girbf_extra_hspread_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    extra_hspread =
      set_atof_low_value(value, "extra_hspread",
			 Girbf_popup1->extra_hspread, extra_hspread, 0.);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `hspread'.
 */
Panel_setting girbf_hspread_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    hspread =
      set_atof_low_value(value, "hspread", Girbf_popup1->hspread, hspread, 0.);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `weight_eta'.
 */
Panel_setting girbf_weight_eta_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    weight_eta =
      set_atof_low_value(value, "weight_eta", Girbf_popup1->weight_eta, weight_eta, 0.);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `mean_eta'.
 */
Panel_setting girbf_mean_eta_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    mean_eta =
      set_atof_low_value(value, "mean_eta", Girbf_popup1->mean_eta, mean_eta, 0.);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `var_eta'.
 */
Panel_setting girbf_var_eta_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    var_eta =
      set_atof_low_value(value, "var_eta", Girbf_popup1->var_eta, var_eta, 0.);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `max_magnitude'.
 */
Panel_setting girbf_max_magnitude_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    max_magnitude =
      set_atof_low_value(value, "max_magnitude", Girbf_popup1->max_magnitude,
			 max_magnitude, 0.);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `min_var'.
 */
Panel_setting girbf_min_var_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    minimum_variance =
      set_atof_low_value(value, "minimum_variance",
			 Girbf_popup1->min_var, minimum_variance, 0.);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `bias'.
 */
void
  girbf_bias_handler(item, value, event)
Panel_item	item;
int		value;
Event		*event;/*ARGSUSED*/
{
    bias = value;
}

void girbf_store_plot_command (shell_file)
     FILE *shell_file;
{
    fprintf (shell_file, " -exhspread %g", extra_hspread);
}


char * girbf_get_args ()
{
    return (nnargs_get_part_flags(irbf_tab, IRBF_TABSIZE));
}

char *girbf_get_cluster_fname(){
    return cluster_param_fname;
}

int girbf_get_cluster_alg(){
    return clustering_algorithm;
}

int girbf_must_store_clustering(){
    return clustering_active_flag;
}

static int clustering_active_arg = 0;
static int bias_arg = 0;
static int grand_arg = 0;
static ARG argtab2[]=
{
    {"clus_active", INT, &clustering_active_arg,
       "write cluster calls on create"},
    {"clus_alg", INT, &clustering_algorithm, "clustering algorithm number"},
    {"bias_arg", INT, &bias_arg, "add a bias node"},
    {"grand_arg", INT, &grand_arg, "grand variances"},
};
#define TABSIZE2 (sizeof(argtab2)/sizeof(ARG))

void write_irbf_defaults(default_file)
     FILE *default_file;
{
    clustering_active_arg = clustering_active_flag;
    bias_arg = bias;
    grand_arg = grand_flag;
    if(default_file != NULL){
	fprintf (default_file,"irbf %s",
		 nnargs_get_part_flags(irbf_tab,IRBF_TABSIZE));
	fprintf (default_file, "%s", nnargs_get_flags(argtab2, TABSIZE2));
    }
}

void read_irbf_defaults(default_file, default_fullname)
     FILE *default_file;
     char *default_fullname;
{
    int nargc;
    char *argv[256];

    if(default_file != NULL){
	nargc = fill_argv(argv, default_file, default_fullname);
	nargc = nnargs_part(nargc, argv, irbf_tab, IRBF_TABSIZE);
	nargc = nnargs(nargc, argv, argtab2, TABSIZE2);
    } else {
	clustering_active_arg = 1;
	nepochs = 20;
    }
    clustering_active_flag = clustering_active_arg;
    strcpy(param_ftmp, cluster_param_fname);
    cluster_param_fname = param_ftmp;
    bias = bias_arg;
    grand_flag = grand_arg;
    /* algorithm specific flags are now set.  generic flags are taken
       from the argument list for the main screen */
}

/*
 * Notify callback function for `epochs'.
 */
Panel_setting girbf_epochs_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);
    
    nepochs = value;
    return panel_text_notify(item, event);
}

int girbf_get_epochs()
{
    return nepochs;
}

void girbf_set_epochs(e)
{
    nepochs = e;
}

/*
 * Notify callback function for `grandvar'.
 */
void girbf_grandvar_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;/*ARGSUSED*/
{
    grand_flag = value;
}
void girbf_set_cost_func(value)
     int value;
{
    cost_func = value;
    switch(value){
      case CFOM:
	xv_set(Girbf_popup1->cost_param, PANEL_INACTIVE, FALSE, NULL);
	break;
      default:
	xv_set(Girbf_popup1->cost_param, PANEL_INACTIVE, TRUE, NULL);
    }
}

/*
 * Notify callback function for `cost_func'.
 */
void girbf_cost_func_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;/*ARGSUSED*/
{
    if(value == 1)
      /* the real cost func 1 is cross entropy, which irbf doesn't support */
      girbf_set_cost_func(CFOM);
    else
      girbf_set_cost_func(value);
}


/*
 * Notify callback function for `cost_param'.
 */
Panel_setting girbf_cost_param_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);

    cost_param = set_atof_value(value, "cost_param", Girbf_popup1->cost_param,
				cost_param);
    return panel_text_notify(item, event);
}
