/***--------------------------------------------------------------------------
* NAME:		gkmeans.c
* PURPOSE:	Executes kmeans after setting command line parameters
*
* SEE ALSO:	gkmeans_subs.c
*
* DESCRIPTION:
*	gkmeans_display_params
*	gkmeans_get_plot_label
*	gkmeans_init
*	gkmeans_set_max_iteration
*	gkmeans_set_ncenters
*	gkmeans_set_split_percentage
*	gkmeans_set_stop_percentage
*	gkmeans_set_params
*		
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/gseq/kmeans/gkmeans.c,v 10.4 2001/08/23 20:14:25 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/gseq/kmeans/gkmeans.c,v $
-------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gseq/kmeans/gkmeans.c,v 10.4 2001/08/23 20:14:25 craig Exp $";



#define TRUE 1
#define FALSE 0
#define MAXSTR 256
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <unistd.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <gdd.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <nnlib.h>
#include "gkmeans_ui.h"
#include <kmeans_args.h>		/* nnargs definitions */
#include "../gseq/nn_params.h"

static int specify_by_class;

extern gkmeans_popup1_objects	*Gkmeans_popup1;

static char tmp_str[MAXSTR];
void gkmeans_set_common_flags();
void gkmeans_show_ncenters_choices();
void set_float_string();
float set_atof_value(), set_atof_range_value(), set_atof_low_value();

void gkmeans_display_params()
{
    char *make_net_string(), *value;

    xv_set (Gkmeans_popup1->cluster_by_class, PANEL_VALUE,
	    cluster_by_class, NULL);
    xv_set (Gkmeans_popup1->ncenters, PANEL_VALUE, ncenters, NULL);
    if(ncenters_list != NULL){
	value = make_net_string(ncenters_list);
	xv_set (Gkmeans_popup1->ncenters_list, PANEL_VALUE, value, NULL);
    }

    xv_set (Gkmeans_popup1->reduce_step, PANEL_VALUE, reduce_step, NULL);
    set_float_string (Gkmeans_popup1->stop_percentage, "%5.2g",
		      stop_percentage);
    xv_set (Gkmeans_popup1->max_iteration, PANEL_VALUE,max_iteration,NULL);
    set_float_string (Gkmeans_popup1->split_percentage, "%5.2g",
		      split_percentage);
    xv_set (Gkmeans_popup1->add_random_offset, PANEL_VALUE,
	    add_random_offset, NULL);

    xv_set(Gkmeans_popup1->equal_ncenters, PANEL_VALUE, specify_by_class,
	   NULL);
    gkmeans_show_ncenters_choices();
}

char * gkmeans_get_plot_label (string)
     char * string;
{
    sprintf (string, "%s K:%d", string, ncenters);

    return string;
}

void gkmeans_init ()
{
    gkmeans_display_params();
}

/*
 * Notify callback function for `max_iteration'.
 */
Panel_setting gkmeans_max_iteration_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);
	
    max_iteration = value;
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `split_percentage'.
 */
Panel_setting gkmeans_split_percentage_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);

    split_percentage =
      set_atof_low_value(value, "split_percentage",
			 Gkmeans_popup1->split_percentage, split_percentage, 0.);
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `stop_percentage'.
 */
Panel_setting gkmeans_stop_percentage_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);

    stop_percentage =
      set_atof_low_value(value, "stop_percentage",
			 Gkmeans_popup1->stop_percentage, stop_percentage, 0.);
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `ncenters'.
 */
Panel_setting
gkmeans_ncenters_handler(item, event)
	Panel_item	item;
	Event		*event;
{
	int	value = (int) xv_get(item, PANEL_VALUE);
	
	ncenters = value;
	return panel_text_notify(item, event);
}


/*
 * Notify callback function for `class_centers'.
 */
void
gkmeans_cluster_by_class_handler(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;/*ARGSUSED*/
{
    cluster_by_class = value;
    gkmeans_show_ncenters_choices();
}


/*
 * Notify callback function for `equal_ncenters'.
 */
void
gkmeans_equal_ncenters_handler(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;/*ARGSUSED*/
{
    specify_by_class = value;
    gkmeans_show_ncenters_choices();
}


/*
 * Notify callback function for `ncenters_list'.
 */
Panel_setting
gkmeans_ncenters_list_handler(item, event)
	Panel_item	item;
	Event		*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);

    if(ncenters_list != NULL) deallocate_ivector(ncenters_list);
    ncenters_list = nnargs_get_int_alloc_list (strcpy(tmp_str, value));
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `reduce_step'.
 */
Panel_setting
gkmeans_reduce_step_handler(item, event)
	Panel_item	item;
	Event		*event;
{
    int	value = (int) xv_get(item, PANEL_VALUE);
	
    reduce_step = value;
	return panel_text_notify(item, event);
}



/*
 * Notify callback function for `add_random_offset'.
 */
void
gkmeans_add_random_offset_handler(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;/*ARGSUSED*/
{
    add_random_offset = value;
}

void
  gkmeans_show_ncenters_choices()
{
    if(cluster_by_class){
	xv_set(Gkmeans_popup1->equal_ncenters, PANEL_INACTIVE, FALSE, NULL);
	if(specify_by_class){
	    xv_set(Gkmeans_popup1->ncenters_list, PANEL_INACTIVE, FALSE, NULL);
	    xv_set(Gkmeans_popup1->ncenters, PANEL_INACTIVE, TRUE, NULL);
	} else {
	    xv_set(Gkmeans_popup1->ncenters_list, PANEL_INACTIVE, TRUE, NULL);
	    xv_set(Gkmeans_popup1->ncenters, PANEL_INACTIVE, FALSE, NULL);
	}
    } else {
	xv_set(Gkmeans_popup1->equal_ncenters, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Gkmeans_popup1->ncenters_list, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Gkmeans_popup1->ncenters, PANEL_INACTIVE, FALSE, NULL);
    }
}

void
gkmeans_show_menu()
{
	xv_set (Gkmeans_popup1->popup1, XV_SHOW, TRUE, 0);
}

char * gkmeans_get_args ()
{
    char * ret_str;
    int tmp_ncenters;

    /* ncenters_list will take precedence if it is there.
       making it appear to be length 0 will prevent it from
       being written out as an argument when it is not needed */
    if(!specify_by_class && ncenters_list != NULL){
	tmp_ncenters = ncenters_list[0];
	ncenters_list[0] = 0;
    }
    ret_str = nnargs_get_part_flags(kmeans_tab, KMEANS_TABSIZE);

    if(!specify_by_class && ncenters_list != NULL){
	ncenters_list[0] = tmp_ncenters;
    }
    return (ret_str);
}

static int specify_by_class_arg = 0;
static int cluster_by_class_arg = 0;
static int add_offset_arg = 0;
static int *ncenters_list_arg = NULL;

static ARG argtab2[]=
{
    {"specify_by_class", INT, &specify_by_class_arg,
       "specify centers by class"},
    {"cluster_by_class_flag", INT, &cluster_by_class_arg,
       "cluster by class"},
    {"add_offset_arg", INT, &add_offset_arg,
       "dummy for add_random_offset"},
    {"ncenters_list_arg", INT_ALLOC_LIST, (int *)&ncenters_list_arg},
};
#define TABSIZE2 (sizeof(argtab2)/sizeof(ARG))

void write_kmeans_defaults(default_file)
     FILE *default_file;
{
    specify_by_class_arg = specify_by_class;
    cluster_by_class_arg = cluster_by_class;
    add_offset_arg = add_random_offset;
    if(ncenters_list_arg != NULL) deallocate_ivector(ncenters_list_arg);
    strcpy(tmp_str, (char *)xv_get(Gkmeans_popup1->ncenters_list,
				   PANEL_VALUE));
    ncenters_list_arg =
      nnargs_get_int_alloc_list(tmp_str);
    if(default_file != NULL){
	fprintf (default_file, "kmeans %s",
		 nnargs_get_part_flags(kmeans_tab, KMEANS_TABSIZE));
	fprintf (default_file, "%s", nnargs_get_flags(argtab2, TABSIZE2));
    }
}

void read_kmeans_defaults(default_file, default_fullname)
     FILE *default_file;
     char *default_fullname;
{
    int nargc, i;
    char *argv[256];

    if(default_file != NULL){
	nargc = fill_argv(argv, default_file, default_fullname);
	nargc = nnargs_part(nargc, argv, kmeans_tab, KMEANS_TABSIZE);
	nargc = nnargs(nargc, argv, argtab2, TABSIZE2);
    }
    specify_by_class = specify_by_class_arg;
    cluster_by_class = cluster_by_class_arg;
    add_random_offset = add_offset_arg;
    if(ncenters_list_arg != NULL){
	if(ncenters_list != NULL) deallocate_ivector(ncenters_list);
	ncenters_list = allocate_ivector(ncenters_list_arg[0]+1);
	for(i = 0; i <= ncenters_list_arg[0] && i < NNARGS_MAX_LABELS; i++)
	  ncenters_list[i] = ncenters_list_arg[i];
    }
    /* algorithm specific flags are now set.  generic flags are taken
       from the argument list for the main screen */
}
