/***------------------------------------------------------------------------
 * NAME:        gmlp.c
 * PURPOSE:     Multilayer Perceptron driver (DEVGUIDE version)
 *		Executes mlp functions after setting command line parameters
 *
 * USAGE:	gseq	(from command line - produces control panel window)
 *
 * SEE ALSO:	gmain.c g.c gmlp_subs.c
 *
 * DESCRIPTION:
 *	
 *	gmlp_display_params
 *	gmlp_get_plot_label
 *	gmlp_init
 *	gmlp_set_alpha
 *	gmlp_set_batch
 *	gmlp_set_decay
 *	gmlp_set_description_values
 *	gmlp_set_epochs
 *	gmlp_set_epsilon
 *	gmlp_set_error_function_type
 *	gmlp_set_etta
 *	gmlp_set_kappa
 *	gmlp_set_layers
 *	gmlp_set_mas
 *	gmlp_set_stats_flag
 *	gmlp_set_tolerance
 *	gmlp_set_weight_update_mode
 *		
 * BUGS AND LIMITATIONS:
 *
 *
 -----------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gseq/mlp/gmlp.c,v 10.4 2001/08/23 20:14:27 craig Exp $";



#define TRUE 1
#define FALSE 0
#define MAXSTR 256

#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <unistd.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <gdd.h>
#include <signal.h>
#include <xview/textsw.h>
#include <sys/ioctl.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <nnlib.h>
#include "gmlp_ui.h"
#include <mlp_seq_args.h>
#include "../gseq/nn_params.h"
#include "../LNKseq_subs.h"

/* static GLOBALS internal to this file */
extern gmlp_popup1_objects	*Gmlp_popup1;
extern gmlp_weight_popup_objects *Gmlp_weight_popup;
extern gmlp_cost_popup_objects *Gmlp_cost_popup;
extern gmlp_node_popup_objects *Gmlp_node_popup;
static int weight_update_mode = 0;

void set_float_string();
void gmlp_set_weight_update_mode();
void set_plot_descriptions();
float set_atof_value(), set_atof_range_value(), set_atof_low_value();

void gmlp_display_params()
{
    char *make_net_string();
    
    set_int_string(Gmlp_popup1->epochs, "%d", nepochs);
    xv_set (Gmlp_popup1->layers, PANEL_VALUE, make_net_string(nnodes), NULL);
    set_float_string (Gmlp_popup1->etta, "%5.2g", etta);

    set_float_string (Gmlp_weight_popup->decay, "%5.2g", decay);
    set_float_string (Gmlp_weight_popup->momentum, "%5.2g", alpha);
    xv_set (Gmlp_weight_popup->etta_change, PANEL_VALUE, etta_type, NULL);
    gmlp_set_etta_change(etta_type);
    set_float_string (Gmlp_weight_popup->etta_nepochs, "%5.2g", etta_nepochs);
    set_float_string (Gmlp_weight_popup->kappa, "%5.2g", kappa);
    set_float_string (Gmlp_weight_popup->epsilon, "%5.2g", epsilon);
    xv_set (Gmlp_weight_popup->use_etta_list, PANEL_VALUE,
	    use_etta_list, NULL);
    if(use_etta_list)
      xv_set(Gmlp_weight_popup->etta_list, PANEL_INACTIVE, FALSE, NULL);
    else
      xv_set(Gmlp_weight_popup->etta_list, PANEL_INACTIVE, TRUE, NULL);
    xv_set(Gmlp_weight_popup->etta_list, PANEL_VALUE,
	   make_float_string(etta_list), NULL);
    set_float_string (Gmlp_weight_popup->tolerance, "%5.2g", tolerance);
    set_float_string(Gmlp_weight_popup->max_weight, "%5.2g", max_magnitude);
    xv_set(Gmlp_weight_popup->weight_update, PANEL_VALUE,
	   weight_update_mode, NULL);

    switch(ofunction){
      case 0:
      case 1:
	xv_set(Gmlp_node_popup->ofunction, PANEL_VALUE, ofunction, NULL);
	break;
      case LINEAR:
	xv_set(Gmlp_node_popup->ofunction, PANEL_VALUE, 2, NULL);
	break;
      case STEP:
	/* do not display */
	break;
    }
    switch(hfunction){
      case 0:
      case 1:
	xv_set(Gmlp_node_popup->ofunction, PANEL_VALUE, ofunction, NULL);
	break;
      case LINEAR:
      case STEP:
	/* do not display */
	break;
    }
    set_float_string(Gmlp_node_popup->sigmoid_param, "%5.2g", sigmoid_param);
    xv_set (Gmlp_node_popup->use_sigmoid_list, PANEL_VALUE,
	    use_sigmoid_param_list, NULL);
    if(use_sigmoid_param_list)
      xv_set(Gmlp_node_popup->sigmoid_list, PANEL_INACTIVE, FALSE, NULL);
    else
      xv_set(Gmlp_node_popup->sigmoid_list, PANEL_INACTIVE, TRUE, NULL);
    xv_set(Gmlp_node_popup->sigmoid_list, PANEL_VALUE,
	   make_float_string(sigmoid_param_list), NULL);

    xv_set (Gmlp_cost_popup->cost_func, PANEL_VALUE,
	    cost_func, NULL);
    gmlp_set_cost_func(cost_func);
    set_float_string(Gmlp_cost_popup->cost_param, "%5.2g", cost_param);
}

char * gmlp_get_plot_label (string)
     char *string;
{
    /* add to existing label */
    sprintf (string, "%s Net:%s Step:%g %Momen:%g", string,
	     make_net_string(nnodes), etta, alpha);
    
    return string;
}

void gmlp_init ()
{
    gmlp_display_params();
    gmlp_set_weight_update_mode (weight_update_mode);
}


/*
 * Notify callback function for `momentum'.
 */
Panel_setting
gmlp_alpha_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    alpha = set_atof_range_value(value, "alpha",
				 Gmlp_weight_popup->momentum, alpha, 0., 1.);
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `batch'.
 */
Panel_setting 
gmlp_batch_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    char tmp[MAXSTR];

    nnargs_get_int_list (strcpy(tmp,value), batch, BATCH_SIZE);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `decay'.
 */
Panel_setting 
gmlp_decay_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    decay = set_atof_range_value(value, "decay",
				 Gmlp_weight_popup->decay, decay, 0., 1.);
    return panel_text_notify(item, event);
}

void gmlp_set_description_values()
{
    nnodes[1] = count_nwindow_frames(nn_params.window_width,
				     nn_params.window_frames);
    nnodes[1] *= nn_params.ninputs;
    nnodes[nnodes[0]] = nn_params.noutputs;

    xv_set (Gmlp_popup1->layers, PANEL_VALUE, make_net_string (nnodes), NULL);
    if(weight_update_mode == 1){
	batch[1] = nn_params.npatterns_train;
	batch[2] = nn_params.npatterns_train;
    } else if(weight_update_mode == 2){
	/* leave ends alone, here */
	batch[2] = nn_params.npatterns_train;
    }
}


/*
 * Notify callback function for `epochs'.
 */
Panel_setting  
gmlp_epochs_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *value = (char *)xv_get(item, PANEL_VALUE);
    
    nepochs = set_atoi_value(value, "epochs", item, nepochs);
    return panel_text_notify(item, event);
}

int gmlp_get_epochs()
{
    return nepochs;
}

void gmlp_set_epochs(e)
     int e;
{
    /* popup will not reflect the set number of epochs */
    nepochs = e;
}

/*
 * Notify callback function for `epsilon'.
 */
Panel_setting  
gmlp_epsilon_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    epsilon = set_atof_range_value(value, "epsilon", Gmlp_weight_popup->epsilon, epsilon,
				   0., 1.);
    return panel_text_notify(item, event);
}

gmlp_set_cost_func(value)
     int value;
{
    void gmlp_set_ofunction();
    char tmp[80];
    
    cost_func = value;
    if(cost_func == PC_PROC){
	/* go to a single layer perceptron and step function for ofunction */
	gmlp_set_ofunction(STEP);
	xv_set(Gmlp_node_popup->ofunction, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Gmlp_popup1->layers, PANEL_INACTIVE, TRUE, NULL);
	nnodes[0] = 2;
	nnodes[1] = nn_params.ninputs;
	nnodes[2] = nn_params.noutputs;
	/* turn off momemtum */
	xv_set(Gmlp_weight_popup->momentum, PANEL_INACTIVE, TRUE, NULL);
	alpha = 0.;
    } else {
	/* set net layers up again */
	xv_set(Gmlp_popup1->layers, PANEL_INACTIVE, FALSE, NULL);
	nnargs_get_int_list(strcpy(tmp, xv_get(Gmlp_popup1->layers,
					       PANEL_VALUE)),
			    nnodes, NNODES_SIZE);
	if (nnodes[0] > 0) {
	    nn_params.ninputs = nnodes[1];
	    nn_params.noutputs = nnodes[nnodes[0]];
	}
	/* turn momentum back on */
	alpha = set_atof_range_value((char *)xv_get(Gmlp_weight_popup->momentum,
						    PANEL_VALUE),
				     "alpha", Gmlp_weight_popup->momentum,
				     alpha, 0., 1.);
	xv_set(Gmlp_weight_popup->momentum, PANEL_INACTIVE, FALSE, NULL);
	
	/* set the output function */
	if(cost_func == SQ_ERR || cost_func == CFOM){
	    xv_set(Gmlp_node_popup->ofunction, PANEL_INACTIVE, FALSE, NULL);
	    gmlp_set_ofunction(xv_get(Gmlp_node_popup->ofunction, PANEL_VALUE,
				      NULL));
	} else {
	    xv_set(Gmlp_node_popup->ofunction, PANEL_INACTIVE, TRUE, NULL);
	    gmlp_set_ofunction(SIGM);
	}
    }
    if(cost_func == CFOM)
      xv_set(Gmlp_cost_popup->cost_param, PANEL_INACTIVE, FALSE, NULL);
    else
      xv_set(Gmlp_cost_popup->cost_param, PANEL_INACTIVE, TRUE, NULL);
}
/*
 * Notify callback function for `cost_funct'
 */
void  
gmlp_cost_func_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;/*ARGSUSED*/
{
    gmlp_set_cost_func(value);
}


/*
 * Notify callback function for `cost_param'.
 */
Panel_setting  
gmlp_cost_param_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);

    cost_param = set_atof_value(value, "cost_param", Gmlp_cost_popup->cost_param,
				cost_param);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `etta'.
 */
Panel_setting  
gmlp_etta_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    etta = set_atof_low_value(value, "etta", Gmlp_popup1->etta, etta, 0.);
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `etta_change'.
 */
void
gmlp_etta_change_handler(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;/*ARGSUSED*/
{
    gmlp_set_etta_change(value);
}

gmlp_set_etta_change(value)
     int value;
{
    etta_type = value;
    if(value == 0){/* constant etta */
	xv_set(Gmlp_weight_popup->kappa, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Gmlp_weight_popup->epsilon, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Gmlp_weight_popup->etta_nepochs, PANEL_INACTIVE, TRUE, NULL);
    } else if (value == 1){ /* decrease etta after tau trials */
	xv_set(Gmlp_weight_popup->kappa, PANEL_INACTIVE, TRUE, NULL);
	xv_set(Gmlp_weight_popup->epsilon, PANEL_INACTIVE, TRUE, NULL);
	if(!weight_update_mode){
	    xv_set(Gmlp_weight_popup->etta_nepochs, PANEL_INACTIVE,
		   FALSE, NULL);
	}
    } else { /* multiple adaptive step size */
	if(weight_update_mode){
	    xv_set(Gmlp_weight_popup->kappa, PANEL_INACTIVE, FALSE, NULL);
	    xv_set(Gmlp_weight_popup->epsilon, PANEL_INACTIVE, FALSE, NULL);
	}
	xv_set(Gmlp_weight_popup->etta_nepochs, PANEL_INACTIVE, TRUE, NULL);
    }
}

/*
 * Notify callback function for `etta_nepochs'.
 */
Panel_setting
gmlp_ntrials_handler(item, event)
	Panel_item	item;
	Event		*event;
{
	char *	value = (char *) xv_get(item, PANEL_VALUE);
	
	etta_nepochs =
	  set_atof_value(value, "etta_nepochs",
			 Gmlp_weight_popup->etta_nepochs, etta_nepochs);

	return panel_text_notify(item, event);
}

/*
 * Notify callback function for `kappa'.
 */
Panel_setting  
gmlp_kappa_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    kappa = set_atof_low_value(value, "kappa", Gmlp_weight_popup->kappa,
			       kappa, 0.);
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `layers'.
 */
Panel_setting
 
gmlp_layers_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *value = (char *) xv_get(item, PANEL_VALUE);
    char tmp[80];
    
    nnargs_get_int_list (strcpy(tmp,value), nnodes,NNODES_SIZE);
    if (nnodes[0] > 1) {
	nn_params.ninputs = nnodes[1];
	nn_params.noutputs = nnodes[nnodes[0]];
    } else {
	warning("There must be at least 2 items in the nodes list");
    }
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `tolerance'.
 */
Panel_setting  
gmlp_tolerance_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    tolerance =
      set_atof_low_value(value, "tolerance", Gmlp_weight_popup->tolerance,
			 tolerance, 0.);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `weight_update'.
 */
void  
gmlp_weight_update_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;/*ARGSUSED*/
{
    gmlp_set_weight_update_mode (value);
}

void
gmlp_set_weight_update_mode(value)
     int value;
{
    char *make_net_string();
    
    switch (value) {
      case 0:
	batch[1]=1;
	batch[2]=1;
	batch[3]=0;
	break;
      case 1:
	batch[1]=nn_params.npatterns_train;
	batch[2]=nn_params.npatterns_train;
	batch[3]=0;
	break;
      case 2:
	batch[1]=1;
	batch[2]=nn_params.npatterns_train;
	batch[3]=1;
	break;
      default:
	batch[1]=1;
	batch[2]=1;
	batch[3]=0;
	break;
    }
    weight_update_mode = value;
    /* re-do graying out for MAS and DECREASE_ETTA */
    gmlp_set_etta_change(etta_type);
}

/*
 * Notify callback function for `sigmoid_param'.
 */
Panel_setting
gmlp_sigmoid_param_handler(item, event)
	Panel_item	item;
	Event		*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    sigmoid_param =
      set_atof_value(value, "sigmoid_param", Gmlp_node_popup->sigmoid_param,
		     sigmoid_param);
    
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `use_sigmoid_list'.
 */
void
gmlp_use_sigmoid_list_handler(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;/*ARGSUSED*/
{
    use_sigmoid_param_list = value;
    if(value)
      xv_set(Gmlp_node_popup->sigmoid_list, PANEL_INACTIVE, FALSE, NULL);
    else
      xv_set(Gmlp_node_popup->sigmoid_list, PANEL_INACTIVE, TRUE, NULL);
}

/*
 * Notify callback function for `sigmoid_param_list'.
 */
Panel_setting
gmlp_sigmoid_param_list_handler(item, event)
	Panel_item	item;
	Event		*event;
{
    char tmp[NNMAX_LINE];
	char *	value = (char *) xv_get(item, PANEL_VALUE);
	
    if(sigmoid_param_list != NULL) deallocate_vector(sigmoid_param_list);
    sigmoid_param_list = nnargs_get_float_alloc_list(strcpy(tmp, value));
    
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `hfunction'.
 */
void
gmlp_hfunction_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;/*ARGSUSED*/
{
    switch(value){
      case 0: /* standard sigmoid */
      case 1: /* symetric sigmoid */
	hfunction = value;
	break;
      case 2: /* linear */
	hfunction = 4;
    }
}

/*
 * Notify callback function for `ofunction'.
 */
void  
gmlp_ofunction_handler(item, value, event)
     Panel_item	item;
     int	value;
     Event	*event;/*ARGSUSED*/
{
    gmlp_set_ofunction(value);
}

void
gmlp_set_ofunction(value)
     int value;
{
    switch(value){
      case 0: /* standard sigmoid */
      case 1: /* symetric sigmoid */
	ofunction = value;
	break;
      case 2: /* linear */
	ofunction = LINEAR;
	break;
      case 3: /* step function */
	/* this case is used for pcp and is not displayed */
	ofunction = STEP;
	break;
    }
}


/*
 * Notify callback function for `node_button'.
 */
void
gmlp_popup1_node_button_notify_callback(item, event)
	Panel_item	item;
	Event		*event;/*ARGSUSED*/
{
	xv_set(Gmlp_node_popup->node_popup, XV_SHOW, TRUE, NULL);
}

/*
 * Notify callback function for `weight_button'.
 */
void
gmlp_popup1_weight_button_notify_callback(item, event)
	Panel_item	item;
	Event		*event;/*ARGSUSED*/
{
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	xv_set(Gmlp_weight_popup->weight_popup, FRAME_CMD_PUSHPIN_IN, TRUE, NULL);
	xv_set(Gmlp_weight_popup->weight_popup, XV_SHOW, TRUE, NULL);
	
	/* gxv_end_connections */

}

/*
 * Notify callback function for `cost_button'.
 */
void
gmlp_popup1_cost_button_notify_callback(item, event)
	Panel_item	item;
	Event		*event;/*ARGSUSED*/
{
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	xv_set(Gmlp_cost_popup->cost_popup, XV_SHOW, TRUE, NULL);
	
	/* gxv_end_connections */

}

/*
 * Notify callback function for `use_etta_list'.
 */
void
gmlp_use_etta_list_handler(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;/*ARGSUSED*/
{
    use_etta_list = value;
    if(value)
      xv_set(Gmlp_weight_popup->etta_list, PANEL_INACTIVE, FALSE, NULL);
    else
      xv_set(Gmlp_weight_popup->etta_list, PANEL_INACTIVE, TRUE, NULL);
}

/*
 * Notify callback function for `etta_list'.
 */
Panel_setting
gmlp_etta_list_handler(item, event)
	Panel_item	item;
	Event		*event;
{
    char tmp[NNMAX_LINE];
	char *	value = (char *) xv_get(item, PANEL_VALUE);
	
    if(etta_list != NULL) deallocate_vector(etta_list);
    etta_list = nnargs_get_float_alloc_list(strcpy(tmp, value));
    
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `max_weight'.
 */
Panel_setting
gmlp_max_weight_handler(item, event)
	Panel_item	item;
	Event		*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    max_magnitude =
      set_atof_value(value, "max_weight", Gmlp_weight_popup->max_weight,
		     max_magnitude);
	
	return panel_text_notify(item, event);
}

char *
gmlp_get_args ()
{
    return (nnargs_get_part_flags(mlp_tab, MLP_TABSIZE));
}

static int use_etta_list_arg = 0;
static int use_sigmoid_list_arg = 0;

static ARG argtab2[]=
{
    {"weight_update", INT, &weight_update_mode, "batch type"},
    {"use_etta_list_arg", INT, &use_etta_list_arg, "dummy for use_etta_list",},
    {"use_sigmoid_list_arg", INT, &use_sigmoid_list_arg,
       "dummy for use_sig_param_list",},
};
#define TABSIZE2 (sizeof(argtab2)/sizeof(ARG))

void write_mlp_defaults(default_file)
     FILE *default_file;
{
    use_etta_list_arg = use_etta_list;
    use_sigmoid_list_arg = use_sigmoid_param_list;
    if(default_file != NULL){
	fprintf(default_file, "mlp %s",
		nnargs_get_part_flags(mlp_tab, MLP_TABSIZE));
	fprintf (default_file, "%s", nnargs_get_flags(argtab2, TABSIZE2));
    }
}

void read_mlp_defaults(default_file, default_fullname)
     FILE *default_file;
     char *default_fullname;
{
    int nargc;
    char *argv[256];

    if(default_file != NULL){
	nargc = fill_argv(argv, default_file, default_fullname);
	nargc = nnargs_part(nargc, argv, mlp_tab, MLP_TABSIZE);
	nargc = nnargs(nargc, argv, argtab2, TABSIZE2);
    } else {
	nepochs = 20;
    }
    use_etta_list = use_etta_list_arg;
    use_sigmoid_param_list = use_sigmoid_list_arg;
    /* algorithm specific flags are now set.  generic flags are taken
       from the argument list for the main screen */
}
