/***--------------------------------------------------------------------------
 * NAME:        grbf.c
 * PURPOSE:     Radial Basis Function Classifier driver (DEVGUIDE version)
 *		Executes rbf functions after setting command line parameters
 *
 * USAGE:	gseq	(from command line - produces control panel window)
 *
 * SEE ALSO:	gseq_main.c gseq.c gseq_subs.c rbf.c rbf_pack.c grbf_subs.c
 *
 * DESCRIPTION:
 *	
 *	grbf_display_params
 *	grbf_get_plot_label
 *	grbf_init
 *	grbf_set_clustering_active
 *	grbf_set_clustering_algorithm
 *	grbf_set_cluster_param_filename
 *	grbf_set_extra_hspread
 *	grbf_set_hspread
 *	grbf_set_max_iteration
 *	grbf_set_max_ratio
 *	grbf_set_bias
 *	grbf_set_split_percentage
 *	grbf_set_stop_percentage
 *	grbf_store_plot_command
 *		
 * BUGS AND LIMITATIONS:
 *
 *
 ------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/gseq/rbf/grbf.c,v 10.4 2001/08/23 20:14:30 craig Exp $";



#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <unistd.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h>
#include <gdd.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <xview/notify.h>
#include <xview/notice.h>
#include <nnlib.h>
#include "grbf_ui.h"
#include <rbf_seq_args.h>			/* nnargs definitions */
#include "../gseq/nn_params.h"
#include "../gseq/galgs.h"

#define TRUE 1
#define FALSE 0
#define MAXSTR 256

static char *clustering_algorithm_name = "kmeans";
static char param_ftmp[MAXSTR];

static int clustering_active_flag;
static int clustering_algorithm = KMEANS;

extern grbf_popup1_objects	*Grbf_popup1;

void set_float_string();
void grbf_set_clustering_algorithm();
float set_atof_value(), set_atof_range_value(), set_atof_low_value();

void grbf_display_params()
{
    xv_set (Grbf_popup1->clustering_active, PANEL_VALUE,
	    clustering_active_flag, NULL);
    xv_set(Grbf_popup1->clustering_algorithm, PANEL_VALUE,
	   clustering_algorithm, NULL);
    xv_set(Grbf_popup1->cluster_param_fname, PANEL_VALUE,
	   cluster_param_fname, NULL);
    grbf_set_clustering_algorithm(clustering_algorithm);
    grbf_set_cluster_param_fname();
    xv_set (Grbf_popup1->bias, PANEL_VALUE, bias, NULL);
    set_float_string (Grbf_popup1->min_var, "%g", minimum_variance);
    set_float_string (Grbf_popup1->hspread, "%5.2g", hspread);
    set_float_string (Grbf_popup1->extra_hspread, "%5.2g", extra_hspread);
    set_float_string (Grbf_popup1->max_ratio, "%5.2g", max_ratio);
}

char * grbf_get_plot_label (string)
     char *string;
{
    sprintf (string,"H:%g Extra H:%g", hspread, extra_hspread);
    if (bias) strcat (string, " Bias");
    
    return string;
}

void grbf_init ()
{
    grbf_display_params();
}

/*
 * Notify callback function for `clustering_active'.
 */
void
  grbf_create_clusters_handler(item, value, event)
Panel_item	item;
int		value;
Event		*event;/*ARGSUSED*/
{
    clustering_active_flag = value;
}


/*
 * Notify callback function for `clustering_algorithm'.
 */
void
  grbf_clustering_algorithm_handler(item, value, event)
Panel_item	item;
int		value;
Event		*event;/*ARGSUSED*/
{
    int temp;
    switch(value){
      case 0:
	grbf_set_clustering_algorithm(KMEANS);
	break;
    }
    temp = clustering_active_flag;
    clustering_active_flag = 1;
    grbf_set_cluster_param_fname();
    clustering_active_flag = temp;
}

/* needed by grbf_set_description_values */
void
grbf_set_clustering_algorithm (value)
     int value;
{
    clustering_algorithm = value;
    switch(value){
      case KMEANS:
	clustering_algorithm_name = "kmeans";
	break;
      default:
	break;
    }
    /* cluster_param_fname still needs to be set */
}
grbf_set_cluster_param_fname()
{
    if(clustering_active_flag){
	strcpy(param_ftmp,nn_params.exp_prefix);
	strcat(param_ftmp,clustering_algorithm_name);
	cluster_param_fname = strcat(param_ftmp,".param");
	xv_set(Grbf_popup1->cluster_param_fname,PANEL_VALUE,
	       cluster_param_fname, NULL);
    }
}

/*
 * Notify callback function for `clustering_parameters'.
 */
void
  grbf_clustering_parameters_handler(item, event)
Panel_item	item;
Event		*event;/*ARGSUSED*/
{
    void gkmeans_show_menu();

    switch(clustering_algorithm){
      case KMEANS:
	gkmeans_show_menu();
	break;
    }
    
}

/*
 * Notify callback function for `cluster_param_fname'.
 */
Panel_setting
  grbf_cluster_param_fname_handler(item, event)
Panel_item	item;
Event		*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    cluster_param_fname = value;
    return panel_text_notify(item, event);
}


/*
 * Notify callback function for `extra_hspread'.
 */
Panel_setting grbf_extra_hspread_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    extra_hspread =
      set_atof_low_value(value, "extra_hspread",
			 Grbf_popup1->extra_hspread, extra_hspread, 0.);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `hspread'.
 */
Panel_setting grbf_hspread_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    hspread =
      set_atof_low_value(value, "hspread", Grbf_popup1->hspread, hspread, 0.);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `max_ratio'.
 */
Panel_setting grbf_max_ratio_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    max_ratio =
      set_atof_low_value(value, "max_ratio", Grbf_popup1->max_ratio,
			 max_ratio, 1.);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `min_var'.
 */
Panel_setting grbf_min_var_handler(item, event)
     Panel_item	item;
     Event	*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    
    minimum_variance =
      set_atof_low_value(value, "minimum_variance",
			 Grbf_popup1->min_var, minimum_variance, 0.);
    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `bias'.
 */
void
grbf_bias_handler(item, value, event)
     Panel_item	item;
     int		value;
     Event		*event;/*ARGSUSED*/
{
    bias = value;
}

void
grbf_store_plot_command (shell_file)
     FILE *shell_file;
{
    fprintf (shell_file, " -exhspread %g", extra_hspread);
}

char * grbf_get_args ()
{
    return (nnargs_get_part_flags(rbf_tab, RBF_TABSIZE));
}

char *grbf_get_cluster_fname(){
    return cluster_param_fname;
}

int grbf_get_cluster_alg(){
    return clustering_algorithm;
}

int grbf_must_store_clustering(){
    return clustering_active_flag;
}

static int clustering_active_arg = 0;
static int bias_arg = 0;
static ARG argtab2[]=
{
    {"clus_active", INT, &clustering_active_arg,
       "write cluster calls on create"},
    {"clus_alg", INT, &clustering_algorithm, "clustering algorithm number"},
    {"bias_arg", INT, &bias_arg, "add bias node"},
};
#define TABSIZE2 (sizeof(argtab2)/sizeof(ARG))

void write_rbf_defaults(default_file)
     FILE *default_file;
{
    clustering_active_arg = clustering_active_flag;
    bias_arg = bias;
    if(default_file != NULL){
	fprintf (default_file,"rbf %s",
		 nnargs_get_part_flags(rbf_tab, RBF_TABSIZE));
	fprintf (default_file, "%s", nnargs_get_flags(argtab2, TABSIZE2));
    }
}

void read_rbf_defaults(default_file, default_fullname)
     FILE *default_file;
     char *default_fullname;
{
    int nargc;
    char *argv[256];

    if(default_file != NULL){
	nargc = fill_argv(argv, default_file, default_fullname);
	nargc = nnargs_part(nargc, argv, rbf_tab, RBF_TABSIZE);
	nargc = nnargs(nargc, argv, argtab2, TABSIZE2);
    } else {
	clustering_active_arg = 1;
    }
    clustering_active_flag = clustering_active_arg;
    strcpy(param_ftmp, cluster_param_fname);
    cluster_param_fname = param_ftmp;
    bias = bias_arg;
    /* algorithm specific flags are now set.  generic flags are taken
       from the argument list for the main screen */
}
