/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
#ifndef artmap_pack_h
#define artmap_pack_h

float artmap_norm (float *p, int plen);
void artmap_vector_norm (float *p, int plen);
void artmap_new_scale (int model, int ncol);
void artmap_learn_complement_mapping (int model, float **m1, int nrow,
				      int ncol);
void artmap_complement_code (int model, float **m1, int nrow, int ncol,
			       float **m2);
void artmap_complement_code_vector (int model, float *v1, int ncol, float *v2);
float artmap_renormalize_element (int model, float e1, int feature_index);
float artmap_unrenormalize_element (int model, float e1, int feature_index);
int artmap_build (int ninputs, int nclasses, int alloc_wj, int f2a_size,
		  int f2b_size);
void artmap_init (int model, int map_flag, int *nn_features,
		  int complement_code, int ARTnorm, float ro_a,
		  float beta_a, float alpha_a, float ro_b, float beta_b,
		  float alpha_b);
void artmap_set_model_params (int model, float beta_a, float alpha_a,
			      float ro_b, float beta_b, float alpha_b,
			      int ARTnorm);
void artmap_free (int model);
int artmap_read_model (FILE *fp, int *complement_coding, float *ro_a,
		       float *beta_a, float *alpha_a, float *ro_b,
		       float *beta_b, float *alpha_b, int *ARTnorm);
int artmap_write_model (int model, FILE *fp);
void *artmap_param (int model, char *param_name);
int artmap_inc_ctrain (int model, float *input, int desired_class,
		       float input_norm, int is_fast_commit);
int artmap_batch_update (int model, int npatterns);
void artmap_gc_init (int model);
void artmap_gc_mark (int model, float *inputs, int desired_class,
		     float im_norm, int threshold);
int artmap_gc_f2 (int model, int n_trials, int ninputs, int threshold,
		  int verbosity);
int artmap_classify (int model, float *inputs, float **outputs, float norm);

#ifdef artmap_pack_h_internal_types
#undef artmap_pack_h_internal_types

typedef struct {
    float       alpha;          /* Choice parameter */
    float       beta;           /* Learning rate parameter */
    float       ro;             /* Vigilance parameter */
    int         ninputs;        /* Size of the forthcoming vectors */
    int         num_f2;         /* Number of f2 nodes in use*/
    int         alloc_f2;       /* Number of f2 nodes allocated */
    float       **w;            /* Fuzzy weights */
    /* Things below here represent state specific to one input */
    float      *inv_norm_w_alpha;/* 1/(Norm of weights + alpha) */
    int         *yidx;          /* Index order of outputs*/
    float       *norm_x;        /* |I^wj| */
    int         which_y;        /* Index into yidx, or -1 if not converted */
    int         first_equal;   /* Which was the first idx found to be equal */
} FUZZY_ART;
  
/* This field is optimized for the fast learning case--it won't work under
   conditions of slow learning.  Carpenter et al describe this as 

            y_b ^ wj_ab f2_a, f2_b active
	    x_ab =   wj_ab       f2_a active
            yb          f2a  active
	    0           otherwise
   They then go on to say the map field vigilance parameter ro_ab is used as:

   | x_ab | < ro_ab | y_b |                                 R(1a)

   BUT(!) the way they describe y_b, wj_ab, and the updating of all of these 
   things, we can simplify all of these goodies (if we know we're using fast
   learning, and they suggest we should) with the following:

   If | x_ab | == 0, fail vigilance                         R(1b)
   Alternatively, if we either have a mismatch or no ART F2 node activated, 
   then we fail ab vigilance.
*/
typedef struct {
    int       alloc_wj;      /* Total size of wj, in entries */
    int       *wj_ab;        /*
			      * This is defined as a matrix in the text, but
			      * really it's a matrix with only one element set
			      * to 1, and all others set to 0.  We speed
			      * things up by only using a vector of the index
			      * of the entry with a 1 in it.  The index of the
			      * wj'th entry corresponds to the F2_a entry,
			      * while the value is the index of the
			      * correspondance to F2_b.  Got it?
			      *
			      */
}FAST_MAP_FIELD;

typedef struct {
	/* general model parameters */
    int         nfinputs;       /* Number of elements in file form of pattern
				 * Note that complement coding doubles this*/
    int		ninputs;	/* number of elements in patterns */
    int		nclasses;	/* number of classes */
    float       **class_maps;    /* Bitmap used for training */
    float	*outputs;	/* space for output values used by error
				 * report functions */
    /* Should be in Fuzzy ART's */
    int		*features;	/* features of raw data used */
    float       *slope;         /* Slope of complement code */
    float       *intercept;     /* Intercept of complement code */
	/* scalar entries */
    int		accum_trials;	/* total number of trials accumulated */ 
    int		epoch;	        /* total number of trials accumulated */ 
    int         ARTnorm;        /* Is the ARTnorm set? */

    /* fields for garbage collecting */
    int strict_err_cnt;
    int err_cnt;
    int* mark;

	/* training vectors and classes */
    FUZZY_ART   *ARTa;           /* Input side ART network */
    FUZZY_ART   *ARTb;           /* Output side ART network */
    FAST_MAP_FIELD *mf;          /* Map field */

} ARTMAP_PARAMS;

#ifdef artmap_pack_h_param_types
#undef artmap_pack_h_param_types
				/* Parameter Tables */
static PARAM artmap_scalars[] = {
    {"ARTnorm", P_INT},
    {"num_f2a",P_INT}, /* Number of F2 nodes */
    {"num_f2b",P_INT}, /* Number of F2 nodes */
    {"nfinputs", P_INT}, /* Encodes if complement coding occured */
    {"ninputs", P_INT},
    {"neq",P_INT}, /* Number of columns in slope and intercept */
    {"nclasses", P_INT},
    {"accum_trials", P_INT},
    {"epoch",P_INT},
    {"alloc_wj",P_INT}
};
#define N_SARTMAP (sizeof(artmap_scalars)/sizeof(PARAM))

static PARAM fuzzy_sparam[] = {
    {"alpha",P_FLOAT},
    {"beta",P_FLOAT},
    {"ro",P_FLOAT},
};
#define N_SFUZZY (sizeof(fuzzy_sparam)/sizeof(PARAM))
static PARAM fuzzy_aparam[] = {
    {"w",P_FLOAT_MATRIX,NULL,0,0}
};
#define N_AFUZZY (sizeof(fuzzy_aparam)/sizeof(PARAM))

static PARAM artmap_arrays[] = {
    {"features",P_INT_VECTOR,NULL,0,0},
    {"slope",P_FLOAT_VECTOR,NULL,0,0},
    {"intercept",P_FLOAT_VECTOR,NULL,0,0},
    {"wj_ab",P_INT_VECTOR,NULL,0,0}
};
#define N_AARTMAP (sizeof(artmap_arrays)/sizeof(PARAM))

#endif

#ifdef artmap_pack_statics
#undef artmap_pack_statics
static void p10 (char *s, float *ptr, int n);
static int artmap_learn_split_extrema (float **m1, int nrow, int ncol,
				       int *cross_index, int cv_nsplits,
				       float **min, float **max);
static void artmap_complement_mapping_from_extrema (int model, float **min,
						    float **max, int ncol,
						    char *train_mask,
						    int cv_nsplits);
static void artmap_renormalize_vector (int model, float *v1, int ncol,
				       float *v2);
static float artmap_complement_element (float e1);
static int artmap_arrival_order (int n, float *xarray, int *index);
static int artmap_fuzzy_thinking (FUZZY_ART *f, float *I, float ro_scaled);
static int artmap_match_track (FUZZY_ART *f, float ro);
static void artmap_update_wt (float beta, float *wj, float *I, int ninputs);
static int artmap_fuzzy_learning (FUZZY_ART *f, float *I, float norm_I,
				  int chosen, int is_learning,
				  int is_fast_commit);
static FUZZY_ART *artmap_fuzzy_build (int f2_size, int ninputs);
static int artmap_fuzzy_new_f2 (FUZZY_ART *f);
static void artmap_fuzzy_free (FUZZY_ART *f);
static int artmap_fast_map_resonance (FAST_MAP_FIELD *fmf, int chosen_f2a,
				      int chosen_f2b);
static FAST_MAP_FIELD *artmap_fast_map_build (int alloc_wj);
static void artmap_fast_map_free (FAST_MAP_FIELD *fmf);
static int artmap_read_fuzzy_art (FUZZY_ART *f, int num_f2, int ninputs,
				  FILE *fp);
static int artmap_write_fuzzy_art (FUZZY_ART *f, FILE *fp);

#endif

#endif

#endif
