/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* cknn_pack.h */

/* DEFINITIONS AND DECLARATIONS FOR cknn_pack.c */

/* DEFINITIONS OF INTERNAL STRUCTURES AND TYPES */
#ifdef cknn_pack_h_internal_types
#undef cknn_pack_h_internal_types
/* cknn_pack_h_internal_types is defined in `cknn_pack.c' */
typedef struct {
  /* general model features */
    int		ninputs;	/* number of elements in patterns */
    int		nclasses;	/* number of classes */
    int		*features;	/* features of raw data used */
    float	*outputs;	/* space for output values used by error
				 * report functions */
  /* scalar entries */
    int		k;		/* number of nearest neighbors used
				 * in training */
    int		accum_trials;	/* total number of trials accumulated */
    int		max_exemplars;	/* index of last vector in
					 * cknn->exemplars */
  /* training vectors and classes */
    float	**exemplars;	/* exemplars collected in training */
    int		*classes;	/* classes of exemplars */
  /* classification vectors */
    int		*class_vote;	/* to store histogram of classes */
    float	*near_dist;	/* to store the distances of the k nearest
				 * neighbors */
    int		*near_index;
    int		dist_model;

} CKNN_PARAMS;

				/* Parameter Tables */
static PARAM    paramtab[] = {
    {"k", P_INT},
    {"ninputs", P_INT},
    {"nclasses", P_INT},
    {"accum_trials", P_INT},
};
#define TABSIZE (sizeof(paramtab)/sizeof(PARAM))

static PARAM    paramtab_l[] = {
    {"classes", P_INT_VECTOR},
    {"exemplars", P_FLOAT_MATRIX},
    {"features", P_INT_VECTOR},
};
#define TABSIZE_L (sizeof(paramtab_l)/sizeof(PARAM))
#endif


#ifndef cknn_pack_h
#define cknn_pack_h

/* DECLARATION OF EXTERNAL FUNCTIONS */
int cknn_build (int ninputs, int nclasses, int npatterns, int k);
void cknn_init (int model, int accum_trials, int *nn_features);
void cknn_free (int model);
int cknn_read_model (FILE *fp, int *accum_trials);
int cknn_write_model (int model, FILE *fp);
void *cknn_param (int model, char *param_name);
void cknn_store_inputs (int model, float *inputs, int desired_class);
int cknn_inc_ctrain (int model, float *inputs, int desired_class);
int cknn_classify (int model, float *inputs);
int cknn_copy_exemplar (int model, int pattern, float *inputs);

#endif
