/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* em_clus_pack.h */

/* DEFINITIONS AND DECLARATIONS FOR em_clus_pack.c */

/* DEFINITIONS OF INTERNAL STRUCTURES AND TYPES */
#ifdef em_clus_pack_h_internal_types
#undef em_clus_pack_h_internal_types
/* em_clus_pack_h_internal_types is defined in `em_clus_pack.c' */
typedef struct {
    int    ninputs;	/* number of elements in patterns */
    int    ncenters;	/* desired number of cluster centers */
    int    max_centers;
    int    *features;	/* features of raw data used */
    float  *outputs;      	/* space for output values used by error
    				 * report functions */
    float  minimum_variance;
    float  variance_scale;
    int    splits;		/* number of splits so far */
    int    stop_flag;		/* indicates when done */
    int    total_vectors;	/* total number of vectors */
    int    total_iteration;	/* total iterations so far */
    int    max_iteration;	/* maximum iterations after any split */
    int    current_iteration;	/* number of iterations for this split */
    int    current_ncenters;	/* current number of cluster centers */
    float  total_logl;		/* total log likelihood */
    float  ave_logl;
    float  last_ave_logl;

    float *weighted_gauss_output;
    float *new_weight;
    float *weight;
    float *log_weight;
    float **means;
    float **new_means;
    float **diag_var;
    float **diag_new_var;
    float **diag_inv_var;
    float *log_det_covar;

    float    split_percentage;	/* +/- this percentage of sdev on split */
    float    stop_percentage;	/* stop iterating at one split when percentage
				 * change in rms err is less than this */
    int reduce_step;
    int add_random_offset;
}EM_CLUS_PARAMS;
#endif


#ifndef em_clus_pack_h
#define em_clus_pack_h

/* DECLARATION OF EXTERNAL FUNCTIONS */
int em_clus_build (int ninputs, int ncenters);
void em_clus_init (int model, int *nn_features, float variance_scale,
		   float minimum_variance, float stop_percentage,
		   float split_percentage, int max_iteration,
		   int reduce_step, int add_random_offset);
void em_clus_free (int model);
int em_clus_read_model (FILE *fp);
int em_clus_write_model (int model, int noutputs, FILE *fp);
int em_clus_write_model_list (int *model_list, int noutputs, FILE *fp);
void *em_clus_param (int model, char *param_name);
void em_clus_batch_train (int model, float input[]);
int em_clus_batch_update (int model, int *report_iterations,
			  int *report_actual_ncenters, float *report_ave_logl,
			  int *report_nvectors);
void em_clus_remove_ncenter (int model, int center, int nearest_center);
int em_clus_reduce_ncenter (int model, int goal);
int em_clus_cluster (int model, float *input);
void em_clus_copy_centers (int model, int ncenters,
			   float **means, float **variances);

#ifdef em_clus_pack_h_statics
#undef em_clus_pack_h_statics

static void em_clus_split(int model);
static void em_clus_local_init(int model);
#endif

#endif
