/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* gauss_pack.h */
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/include/gauss_pack.h,v 11.4 2001/08/23 20:14:33 craig Exp $*/
/* $Source: /data/speech/kukolich/cvsroot/lnknet/src/include/gauss_pack.h,v $*/

#ifndef gauss_pack_h
#define gauss_pack_h

#ifdef gauss_pack_h_internal_types
#undef gauss_pack_h_internal_types

/* Per Class Covariance Parameters */
typedef struct {
    float *mean;    /* mean for each class */
    float log_det;  /* determinant, if per_class variance */
    float *var;     /* per_class, diagonal */
    float **covar;  /* per_class, full */
} PCLCOV;

/* Gaussian Classifier Model Parameters */
typedef struct {
    int     ninputs;    /* number of elements in patterns */
    int     noutputs;   /* number of elements in patterns */
    int     *features;  /* features of raw data used */
    int     total_trials;
    int     *class_trials;  /* current count of input vectors */
    int     per_class;  /* variance type */
    int     full_covar; /* covariance type */
    float   minimum_variance;
    float   *outputs;
    float   *log_class_priors;
    float   log_det;    /* determinant, if grand variance */
    float   *var;       /* grand diagonal */
    float   **covar;    /* grand full */
    PCLCOV  *cv_class;  /* means and per class variances */
} GAUSS_PARAMS;


/*** Parameter Table ***/
static PARAM   paramtab[] = {
    {"ninputs", P_INT, NULL},
    {"noutputs", P_INT, NULL},
    {"total_trials", P_INT, NULL},
    {"full_covar", P_INT, NULL},
    {"per_class", P_INT, NULL},
};
#define  TABSIZE (sizeof( paramtab)/sizeof(PARAM))

static PARAM   paramtab_l[] = {
    {"features", P_INT_VECTOR, NULL, 0, 0},
    {"class_trials", P_INT_VECTOR, NULL, 0, 0},
};
#define  TABSIZE_L (sizeof(  paramtab_l)/sizeof(PARAM))

static PARAM means[] = {
    {"mean", P_FLOAT_VECTOR, NULL, 0, 0},
};
#define NMEANS (sizeof(means)/sizeof(PARAM))

static PARAM var_tab[] = {
    {"log_det", P_FLOAT, NULL},
    {"inv_var", P_FLOAT_VECTOR, NULL, 0, 0},
};
#define NVAR (sizeof(var_tab)/sizeof(PARAM))

static PARAM covar_tab[] = {
    {"log_det", P_FLOAT, NULL},
    {"inv_covar", P_FLOAT_MATRIX, NULL, 0, 0},
};
#define NCOVAR (sizeof(var_tab)/sizeof(PARAM))
#endif
extern int gauss_build(int ninputs, int noutputs, int per_class_variance,
               int full_covar);
extern void gauss_init(int model, int *nn_features, float minimum_variance);
extern void gauss_free(int model);
extern int gauss_read_model(FILE *fp);
extern int gauss_write_model(int model, FILE *fp);
extern void *gauss_param(int model, char *param_name, int class);
extern void gauss_batch_ctrain(int model, float *inputs, int desired_class);
extern void gauss_batch_update(int model, float max_ratio);
extern void gauss_test_build(int model);
extern void gauss_test_free(int model);
extern void gauss_test_init(int model, int adjust_priors, float *priors_list);
extern void gauss_eval(int model, float *inputs);
extern int gauss_classify(int model, float *inputs);
#endif
