/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* gmix_pack.h */

/* DEFINITIONS AND DECLARATIONS FOR gmix_pack.c */

/* DEFINITIONS OF INTERNAL STRUCTURES AND TYPES */
#ifdef gmix_pack_h_internal_types
#undef gmix_pack_h_internal_types
/* gmix_pack_h_internal_types is defined in `gmix_pack.c' */
struct gauss_desc {
    float weighted_gauss_output;	/* tau in equations */
    float weight;
    float log_weight;
    float new_weight;
    float log_l;
    float *means;
    float *new_means;
    float **inv_covar;
    float **new_covar;
    float *diag_inv_var;
    float *diag_new_var;
    float log_det_covar;
};

struct class_gauss {
    int npatterns;
    int ngauss;
    struct gauss_desc **gaussian;
    float **inv_covar;	/* for grand-per-class */
    float **new_covar;
    float *diag_inv_var;
    float *diag_new_var;
    float log_det_covar;
    float ave_logl;
};

typedef struct {
    int    tied_flag;
    int    grand_flag;
    int    full_flag;
    int    ninputs;	/* number of elements in patterns */
    int    noutputs;	/* number of elements in patterns */
    int   *features;	/* features of raw data used */
    float *outputs;	/* space for output values used by error
    			 * report functions */
    int    npatterns;	/* total number of patterns seen */
    int    ngauss;	/* number of tied gaussians */
    float minimum_variance;
    float eigen_ratio;
    float **inv_covar;	/* for grand full covariance */
    float **new_covar;	/* for grand full covariance */
    float *diag_inv_var;/* for grand diagonal covariance */
    float *diag_new_var;/* for grand diagonal covariance */
    float log_det_covar;	/* of covariance matrix */
    float ave_logl;
    struct gauss_desc  **tied_gaussian; /* for tied mixtures */
    struct class_gauss **class_mix;
    float total_npatterns;
    float *class_cnt;
    float *log_class_priors;
} GMIX_PARAMS;
				/* Parameter Tables */
static PARAM    paramtab[] = {
    {"ninputs", P_INT},
    {"noutputs", P_INT},
    {"ngauss", P_INT},
    {"tied_flag", P_INT},
    {"grand_flag", P_INT},
    {"full_flag", P_INT},
    {"total_npat", P_FLOAT},
};
#define TABSIZE (sizeof(paramtab)/sizeof(PARAM))

static PARAM    paramtab_l[] = {
    {"features", P_INT_VECTOR},
    {"ngauss", P_INT_VECTOR},
    {"class_cnt", P_FLOAT_VECTOR},
};
#define TABSIZE_L (sizeof(paramtab_l)/sizeof(PARAM))

static PARAM paramtab_w[] = {
    {"log_weights", P_FLOAT_VECTOR},
};
#define TABSIZE_W (sizeof(paramtab_w)/sizeof(PARAM))

static PARAM paramtab_m[] = {
    {"means", P_FLOAT_VECTOR},
};
#define TABSIZE_M (sizeof(paramtab_m)/sizeof(PARAM))

static PARAM paramtab_c[] = {
    {"inv_covar", P_FLOAT_MATRIX},
    {"diag_inv_var", P_FLOAT_VECTOR},
    {"log_det_covar", P_FLOAT},
};
#define TABSIZE_C (sizeof(paramtab_c)/sizeof(PARAM))
#endif


#ifndef gmix_pack_h
#define gmix_pack_h

/* DECLARATION OF EXTERNAL FUNCTIONS */
int gmix_build (int ninputs, int noutputs, int *ngauss_param, int tied_flag,
		int grand_flag, int full_flag);
void gmix_init (int model, int *nn_features, float minimum_variance,
		float eigen_ratio);
void gmix_test_init (int model, int adjust_priors, float *priors_list);
void gmix_free (int model);
int gmix_read_model (FILE *fp);
int gmix_write_model (int model, FILE *fp);
void *gmix_param (int model, char *param_name, int class, int g);
int gmix_batch_ctrain (int model, float *inputs, int desired_class);
void gmix_batch_update (int model);
void gmix_eval (int model, float *inputs);
int gmix_classify (int model, float *inputs);
void gmix_initialize_cluster_centers (int model, int clus_model,
				      float var_spread);
int gmix_read_centers (char *param_path, FILE *param_file, int *pninputs,
		       int *pnoutputs, int **ngauss_param, int *features,
		       int tied_flag);
#endif
