/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* histogram_pack.h */

/* DEFINITIONS AND DECLARATIONS FOR histogram_pack.c */

#ifndef histogram_pack_h
#define histogram_pack_h

#define U_WIDTH 0
#define U_NPAT 1
#define PER_CLASS 0
#define GRAND_INPUT 1
#define USER_BIN 2

/* DEFINITIONS OF INTERNAL STRUCTURES AND TYPES */
#ifdef histogram_pack_h_internal_types
#undef histogram_pack_h_internal_types
/* histogram_pack_h_internal_types is defined in `histogram_pack.c' */

/* HISTOGRAM_MODEL holds information for each layer of the network */
typedef struct {
    int *bin_npat;	/* TRAIN nB aR *//* npatterns per bin */
    float *bin_prob;	/* BOTH aB*//* probability of each bin */
    float mean, var, LNvar;/* iR iR iT */
} HISTOGRAM_PROB;

typedef struct {
    float *bin_ends;	/* BOTH aB */   /* start or end of each bin */
    float min, max;	/* TRAIN iR */
    float *sorted_inputs; /* TRAIN iB iR */ /* for U_NPAT bins */
    int sorted_length;	/* TRAIN nB iR */
} HISTOGRAM_BIN;

typedef struct {
    int		ninputs;	/* BOTH iB */
    int		noutputs;	/* BOTH iB */
    float	*outputs;	/* TEST nB aT */
    float	*gauss_outputs;	/* TEST nB aT */
    float	*class_cnt;	/* BOTH aB iR */
    float	total_npatterns;/* BOTH iR */
    float	*class_priors;	/* TEST nB aT iT */
    int		bin_type;	/* TRAIN a *//* U_WIDTH or U_NPAT */
    int		use_gauss;	/* TEST iT */
    int		*nbins;		/* BOTH aB *//* nbins per class */
    int		grand_bins;	/* BOTH aB */
    HISTOGRAM_BIN **bin_list;	/* BOTH aB */
    HISTOGRAM_BIN *grand_bin_list;/* BOTH aB */
    HISTOGRAM_BIN *user_bin;	/* BOTH aB */
    HISTOGRAM_PROB **prob_list; /* BOTH aB */
} HISTOGRAM_PARAMS;
				/* constant definitions */
#define MODEL_STORE_SIZE 10
#define PEQ(y) (0==strcmp(param_name,y))
#endif


/* DECLERATION OF EXTERNAL FUNCTIONS */
int histogram_build(int ninputs, int noutputs, int bin_type, int *nbins,
		    int grand_bins);
void histogram_train_init( int model);
void histogram_test_init( int model, int adjust_priors, float *priors_list,
			  int use_gauss);
void histogram_test_alloc( int model);
void histogram_free(int model);
int histogram_read_model( FILE *fp);
int histogram_write_model( int model, FILE *fp );
void * histogram_param( int model, char *param_name, int class, int input);
void histogram_inc_train1( int model, float *inputs, int desired_class);
void histogram_finish_train1( int model, float range_factor);
void histogram_set_bins( int model, float min, float max);
void histogram_inc_train2( int model, float *inputs, int desired_class);
void histogram_finish_train2( int model, float minimum_variance);
int histogram_classify( int model, float *inputs );
int histogram_eval( int model, float *inputs );
#ifdef histogram_pack_h_static_decl
#undef histogram_pack_h_static_decl
/* DECLARATION OF STATIC FUNCTIONS */
static void histogram_set_paramtab(HISTOGRAM_PARAMS *hpp);
static void histogram_set_paramtab_l( HISTOGRAM_PARAMS *hpp );
static void histogram_set_paramtab_bin( HISTOGRAM_BIN *bin, int b);
static void histogram_set_paramtab_prob( HISTOGRAM_PROB *prob, int b);
static void hist_insert_heap( float x, float *array, int n);
static void hist_sort_heap( float *array, int n);
static int hist_get_bin( HISTOGRAM_BIN *bin, float x, int nbins);
#endif
#endif
