/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* hyper_pack.h */

/* DEFINITIONS AND DECLARATIONS FOR hyper_pack.c */

/* DEFINITIONS OF INTERNAL STRUCTURES AND TYPES */
#ifdef hyper_pack_h_internal_types
#undef hyper_pack_h_internal_types
/* hyper_pack_h_internal_types is defined in `hyper_pack.c' */

typedef struct {
		/* generic parameters */ 
    int             ninputs;	/* number of elements in patterns */
    int             nclasses;	/* number of classes */
    int		   *features;	/* features of raw data used */
    float	   *outputs;	/* space for output values used by error
				 * report functions */
		/* scalars */
    int             accum_trials;/* total number of trials accumulated */
    int             max_spheres;/* index of last vector in hyper->centroids */
    float           r_init;
    int             force_class;
		/* training vectors and classes */
    float         **centroids;	/* centroids collected in training */
    float          *radii;	/* radii for cetroids */
    int		*classes;	/* classes of centroids */
		/* classification scalars */
    float           cor_dist;
    float           min_dist;
    int             cor_i;
    int             min_i;
		/* classification vectors */
    int            *correct_cnt;/* to store histogram of correct
				 * classifications */
    float          *distance;	/* to store the distances of the nearest
				 * neighbors */
}               HYPER_PARAMS;

/*** Parameter Tables ***/
static PARAM    paramtab[] = {
    {"ninputs", P_INT},
    {"nclasses", P_INT},
    {"accum_trials", P_INT},
    {"force_classify", P_BOOLEAN},
    {"r_init",P_FLOAT},
};
#define TABSIZE (sizeof(paramtab)/sizeof(PARAM))

static PARAM    paramtab_l[] = {
    {"classes", P_INT_VECTOR},
    {"centroids", P_FLOAT_MATRIX},
    {"radii", P_FLOAT_VECTOR},
    {"features",P_INT_VECTOR},
};
#define TABSIZE_L (sizeof(paramtab_l)/sizeof(PARAM))
#endif


#ifndef hyper_pack_h
#define hyper_pack_h

/* DECLARATION OF EXTERNAL FUNCTIONS */
int hyper_build (int ninputs, int nclasses, int npatterns);
void hyper_init (int model, int accum_trials, float r_init, int force_class,
		int *nn_features);
void hyper_free (int model);
int hyper_read_model (FILE *fp, int *accum_trials);
int hyper_write_model (int model, FILE *fp);
void *hyper_param (int model, char *param_name);
int hyper_inc_ctrain (int model, float *inputs, int desired_class);
int hyper_classify (int model, float *inputs);
void hyper_clear_cnts (int model);
int hyper_eval (int model, float *inputs, int desired_class);
void hyper_store_inputs (int model, float *inputs, int desired_class);
int hyper_prune (int model, int best_n, int correct_sort, float min);
void hyper_swap (int model, int fw, int bw);

#endif
