/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* irbf_pack.h */

/* DEFINITIONS AND DECLARATIONS FOR irbf_pack.c */

/* DEFINITIONS OF INTERNAL STRUCTURES AND TYPES */
#ifdef irbf_pack_h_internal_types
#undef irbf_pack_h_internal_types
/* irbf_pack_h_internal_types is defined in `irbf_pack.c' */
/* Define structure of the irbf_model */
typedef struct {
    int ninputs;	/* dimension of each input vector */
    int ngauss_hidden;	/* number of standard gaussian hidden
			 * nodes */
    int bias;	/* number of bias nodes, (0 or 1) */
    int nhidden;	/* total number of hidden irbf nodes */
    int nclasses;	/* number of output classes or nodes */
    int grandvar;
    float hspread;	/* factor to multiply all variances by */
    float extra_hspread; /* additional hspread for testing */
    float *hidden_out;	/* vector for hidden outputs */
    float *desired_outputs; /* for classification */
    float *outputs;	/* vector for outputs */
    float **means;	/* matrix of means */
    float mean_eta;	/* center stepsize */
    float *inv_var;	/* grand inverse variance */
    float **inv_var_m;	/* inverse variance for each center */
    float inv_var_eta;	/* stepsize for inverse variance */
    float **weights;	/* matrix of weights */
    float weight_eta;	/* stepsize for weights */
    int *features;	/* features of raw data used */
    float *class_cnt;	/* number of samples seen in each class */
    float total_npatterns;/*total number of samples seen */
    int adjust_priors;	/* whether to adjust testing priors */
    float *priors_list;
    int norm_outputs;
    float minimum_inv_var;
    float *cluster_g;	/* temporary sum for weight update */
    float *delta;
    float *new_inv_var;	/* temporary for grand variances */
    int cost_func;
    float cost_param;
}               IRBF_PARAMS;
/*** parameter table ***/
static PARAM    irbfscalars[] = {
    {"ninputs", P_INT},
    {"ngauss_hidden", P_INT},
    {"bias", P_INT},
    {"nhidden", P_INT},
    {"nclasses", P_INT},
    {"hspread", P_FLOAT},
    {"total_npat", P_FLOAT},
    {"norm_outs", P_INT},
    {"grandvar", P_INT},
    {"cost_func", P_INT},
    {"cost_param", P_FLOAT},
};
static PARAM    irbfarrays[] = {
    {"centroids", P_FLOAT_MATRIX, NULL, 0, 0},
    {"weights", P_FLOAT_MATRIX, NULL, 0, 0},
    {"features", P_INT_VECTOR, NULL, 0, 0},
    {"class_cnt", P_FLOAT_VECTOR, NULL, 0, 0},
};
static PARAM irbfvar_array[]= {
    {"inv_var", P_FLOAT_VECTOR, NULL, 0, 0},
};
static PARAM irbfvar_matrix[]={
    {"inv_var", P_FLOAT_MATRIX, NULL, 0, 0},
};
#define NIRBFSCALARS (sizeof(irbfscalars)/sizeof(PARAM))
#define NIRBFARRAYS (sizeof(irbfarrays)/sizeof(PARAM))
#define NIRBFVAR_ARRAY (1)
#define NIRBFVAR_MATRIX (1)
#endif


#ifndef irbf_pack_h
#define irbf_pack_h

/* DECLARATION OF EXTERNAL FUNCTIONS */
int irbf_build (int ninputs, int ngauss_hidden, int bias, int nclasses,
		int grandvar);
void irbf_init (int model, float hspread, int *features,
		float minimum_inv_var, int norm_outputs, float weight_eta,
		float mean_eta, float inv_var_eta, float max_magnitude,
		int cost_func, float cost_param);
void irbf_initialize_cluster_centers (int model, int clus_model);
void irbf_test_init (int model, int adjust_priors, float *priors_list,
		     float extra_hspread);
void irbf_free (int model);
int irbf_read_model (FILE *fp, float weight_eta, float mean_eta, float
		     inv_var_eta, float minimum_inv_var);
void irbf_write_model (int model, FILE *fp);
void *irbf_param (int model, char *param_name);
int irbf_inc_ctrain (int model, float *invec, int class);
void irbf_inc_train (int model, float *invec, float *desired);
int irbf_classify (int model, float *invec);
void irbf_eval (int model, float *invec);
int irbf_read_cluster (char *param_path, FILE *param_file, int *pninputs,
		       int *pncenters, int *features);

#ifdef irbf_pack_h_statics
#undef irbf_pack_h_statics
static void irbf_init_weights (IRBF_PARAMS *irpp, float max_magnitude);
static void irbf_set_scalar_params (IRBF_PARAMS *irpp);
static void irbf_set_array_params (IRBF_PARAMS *irpp);
static void irbf_set_var_array_params (IRBF_PARAMS *irpp);
static void irbf_set_var_matrix_params (IRBF_PARAMS *irpp);
static void irbf_backprop (int model, float *inputs, float *desired);
static void irbf_hidden_eval (int model, float *invec, float extra_hspread);
#endif

#endif
