/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* kmeans_pack.h */

/* DEFINITIONS AND DECLARATIONS FOR kmeans_pack.c */

#ifndef kmeans_pack_h
#define kmeans_pack_h

/* DEFINITIONS OF INTERNAL STRUCTURES AND TYPES */
#ifdef kmeans_pack_h_internal_types
#undef kmeans_pack_h_internal_types
/* kmeans_pack_h_internal_types is defined in `kmeans_pack.c' */

typedef struct {
    int      ninputs;		/* number of elements in patterns */
    int      ncenters;		/* desired number of cluster centers */
    int      max_centers;
    int     *features;		/* features of raw data used */
    float   *outputs;		/* space for output values used by error
    				 * report functions */
    int      splits;		/* number of splits so far */
    int      stop_flag;		/* indicates when done */
    int      total_vectors;	/* total number of vectors */
    int      total_iteration;	/* total iterations so far */
    int      max_iteration;	/* maximum iterations after any split */
    int      current_iteration;	/* number of iterations for this split */
    int      current_ncenters;	/* current number of cluster centers */
    int     *current_count;	/* number of nearest vectors */
    float  **current_center;	/* centers on this iteration */
    float  **current_variance;	/* variance on this iteration */
    double **current_sum;	/* accumulate nearest vectors */
    double **current_sum_squared;/* accumulate nearest vectors squared */
    float    minimum_variance;
    float    total_squared_error;/* total vq squared error */
    float    ave_squared_error;	/* ave over vectors on this iteration */
    float    last_ave_squared_error;	/* last iteration ave over vectors */
    float   *weight;		/* vector weighting (inverse variance) */
    float  **pcenter;		/* final centers to return */
    float  **pvariance;		/* variance of final centers */
    float    split_percentage;	/* +/- this percentage of sdev on split */
    float    stop_percentage;	/* stop iterating at one split when percentage
				 * change in rms err is less than this */
    int reduce_step;
    int add_random_offset;
} KMEANS_PARAMS;
#endif


/* DECLERATION OF EXTERNAL FUNCTIONS */
int kmeans_build (int ninputs, int ncenters);
void kmeans_init (int model, int *nn_features, float *pnorm_variance,
		  float stop_percentage, float split_percentage,
		  int max_iteration, int reduce_step, int add_random_offset);
void kmeans_free (int model);
int kmeans_read_model (FILE *fp);
int kmeans_write_model (int model, int noutputs, FILE *fp);
int kmeans_write_model_list (int *model_list, int noutputs, FILE *fp);
void *kmeans_param (int model, char *param_name);
void kmeans_batch_train (int model, float *input);
int kmeans_batch_update (int model, int *report_iterations,
			 int *report_actual_ncenters,
			 float *report_ave_squared_error,
			 int *report_nvectors);
void kmeans_remove_ncenter (int model, int center, int nearest_center);
int kmeans_reduce_ncenter (int model, int goal);
int kmeans_cluster (int model, float *input);
void kmeans_combine_models (int model1, int model2);
void merge_kmeans_files (char *param_path1, char *param_path2,
			 char *result_file);

#ifdef kmeans_pack_h_statics
#undef kmeans_pack_h_statics
static void kmeans_split(int model);
static void kmeans_local_init(int model);
#endif

#endif
