/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* lvq_pack.h */

/* DEFINITIONS AND DECLARATIONS FOR lvq_pack.c */

/* DEFINITIONS OF INTERNAL STRUCTURES AND TYPES */
#ifdef lvq_pack_h_internal_types
#undef lvq_pack_h_internal_types
/* lvq_pack_h_internal_types is defined in `lvq_pack.c' */

typedef struct {
    /* general model features */
    int		ninputs;	/* number of elements in patterns */
    int		nclasses;	/* number of classes */
    int		*features;	/* features of raw data used */
    float	*outputs;	/* space for output values used by error
				 * report functions */
    int		ncenters;	/* number of centers */
    int		lvq_type;
    float	init_alpha;	/* alpha(0) */
    float	epsilon;	/* epsilon for LVQ3 */
    float	window;		/* window for LVQ2 and LVQ3 */
    /* training vectors and classes */
    float	**centers;	/* centers */
    int		*classes;	/* classes of centers */
    int		**class_vote;	/* votes for labeling centers */
    float	*alphas;	/* alphas for OLVQ centers */
} LVQ_PARAMS;

/* Parameter Tables */
static PARAM    paramtab[] = {
    {"ninputs", P_INT},
    {"nclasses", P_INT},
    {"ncenters", P_INT},
    {"alpha0", P_FLOAT},
};
#define TABSIZE (sizeof(paramtab)/sizeof(PARAM))

static PARAM    paramtab_l[] = {
    {"classes", P_INT_VECTOR},
    {"centers", P_FLOAT_MATRIX},
    {"alphas", P_FLOAT_VECTOR},
    {"features", P_INT_VECTOR},
};
#define TABSIZE_L (sizeof(paramtab_l)/sizeof(PARAM))
#endif


#ifndef lvq_pack_h
#define lvq_pack_h

/* DECLARATION OF EXTERNAL FUNCTIONS */
int lvq_build (int ninputs, int nclasses, int ncenters);
void lvq_init (int model, int *nn_features, int lvq_type, float alpha0,
	       float epsilon, float window);
void lvq_free (int model);
int lvq_read_model (FILE *fp);
int lvq_write_model (int model, FILE *fp);
void *lvq_param (int model, char *param_name);
int lvq_inc_ctrain (int model, float *inputs, int desired_class, float talpha);
int lvq_classify (int model, float *inputs);
void lvq_copy_centers (int model, float **centers);
void lvq_inc_label (int model, float *inputs, int class);
void lvq_label_update (int model);
void lvq_read_cluster (char *param_path, FILE *param_file, int *pninputs,
		      int noutputs, int *pncenters, float ***centroids,
		      int *features);

#endif
