/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* mlp_pack.h */

/* DEFINITIONS AND DECLARATIONS FOR mlp_pack.c */

#ifndef mlp_pack_h
#define mlp_pack_h

/* DEFINITIONS OF INTERNAL STRUCTURES AND TYPES */
#ifdef mlp_pack_h_internal_types
#undef mlp_pack_h_internal_types
/* mlp_pack_h_internal_types is defined in `mlp_pack.c' */

#define CONSTANT_ETTA 0
#define DECREASE_ETTA 1
#define MAS 2

/* MLP_MODEL holds information for each layer of the network */
typedef struct {
    int		nodes;		/* number of nodes in this layer */
    int		sigm_type;	/* sigmoid function type */
    float	sigm_param;	/* sigmoid function parameter */
    float	*node_sum;	/* summed activity of nodes in current layer */
    float	*node_output;	/* output of nodes in current layer after
				 * being sent through (sigmoid) function */
    float	*error_delta;	/* storage of error delta for current node  */
    float	**weights;	/* 2-D array of weights n_nodes * n_nodes of
				 * previous layer */
    float	**weight_delta;	/* difference of weights between
				 * current and previous trials - used
				 * in momentum term */
    float	**batch_gradient;/* storage of accumulation of error
				  * gradients for batch update (does
				  * not include momentum or etta) */
    float	**old_gradient;	/* save of previous batch_gradient
				 * for etta update */
    float	**m_etta;	/* multiple weight changes parameters */
    float	etta;		/* etta for this layer */
    float	etta0;		/* initial etta for DECREASE_ETTA */
} MLP_MODEL;

typedef struct {
    int		nlayers;	/* number of layers (including input and
				 * outp) */
    int		*layers;	/* number of nodes in each layer */
    float	alpha;		/* momentum of descent */
    float	decay;		/*weight decay (multiply by weight at update)*/
    float	kappa;		/* constant for increment using
				 * mult.adap.etta */
    float	epsilon;	/* scale for decrement using
				 * mult.adap.etta */
    float	tolerance;	/* comparison tolerance */
    int		ninputs;	/* number of inputs to network */
    int		noutputs;	/* number of outputs from network */
    int		cost_func;	/* Error cost_func for output correction */
    int		accum_trials;	/* total number of trials accumulated */
    int		etta_type;	/* type of step size, CONSTANT_ETTA,
				 DECREASE_ETTA, MAS */
    int		etta_tau;	/* trial number for decrease (DECREASE_ETTA) */
    float	*outputs;	/* Network outputs */
    float	*desired_outputs;/* Points to desired output values */
    int		check_underflow;/* start checking for small errors */
    float	cost;	/*Root Mean Squared Error (Desired - Actual) */
    int		batch_on;	/*TRUE if a batch of updates is being accum. */
    int		batch_cnt;	/* count of training patterns during batch */
    int		*features;
    float	*class_cnt;	/* number of samples seen in each class */
    float	total_npatterns;/* total samples seen */
    int		adjust_priors;	/* whether to adjust testing priors */
    float	*priors_list;	/* when adjusting priors in testing */
    MLP_MODEL	*mlp_model;	/* Array of models each w/mult layers */
} MLP_PARAMS;
#endif


/* DECLERATION OF EXTERNAL FUNCTIONS */
int mlp_build(int nlayers_param, int *layers_param, int etta_type);
void mlp_init(int model, float max_magnitude, int hfunction, int ofunction,
	      float *etta_list, int *nn_features, int etta_tau);
void mlp_set_train_params(int model, int cost_func, float cost_param,
			  float *etta_list, float alpha, float decay,
			  float kappa, float epsilon, float tolerance,
			  float *sigmoid_param_list);
void mlp_test_init(int model, int adjust_priors, float *priors_list);
void mlp_free(int model);
int mlp_read_model(FILE *fp);
int mlp_write_model(int model, FILE *fp);
void * mlp_param(int model, char *param_name, int layer);
void mlp_inc_train(int model, float *inputs, float *desired_outputs);
int mlp_inc_ctrain(int model, float *inputs, int desired_class);
void mlp_batch_train(int model, float *inputs, float *desired_outputs);
int mlp_batch_ctrain(int model, float *inputs, int desired_class);
void mlp_batch_update_weights(int model);
int mlp_classify(int model, float *inputs);
void mlp_eval(int model, float *inputs);

#ifdef mlp_pack_h_statics
#undef mlp_pack_h_statics
/* these functions should be static but aren't */
static void initialize_storage(int model);
static void feed_fwd_one(int n_curr, int n_next, float *curr_node_sum,
			 float *next_node_sum, float **w_array, int type,
			 float param, float *curr_node_output);
static void back_prop(int model, float *desired_outputs, float *outputs,
		      float tolerance, int check_underflow);
static void calc_hidden_deltas(int model, int n_layers);
static void normal_back_prop(int model, int n_layers, float alpha,
			     int check_underflow);
static void set_etta_values(MLP_PARAMS *mpp);
#endif

#endif
