/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* nbayes_pack.h */

/* DEFINITIONS AND DECLARATIONS FOR nbayes_pack.c */

#ifndef nbayes_pack_h
#define nbayes_pack_h

/* DEFINITIONS OF INTERNAL STRUCTURES AND TYPES */
#ifdef nbayes_pack_h_internal_types
#undef nbayes_pack_h_internal_types
/* nbayes_pack_h_internal_types is defined in `nbayes_pack.c' */

typedef struct {
  int	ninputs;
  int	noutputs;
  float	*outputs;
  float	*class_cnt;
  float	total_npatterns;
  float	*log_class_priors;
  int	*nbins;		/* nbins per input */
  float ***prob_list;
  float **bin_ends;
  float *bin_min;
  float *bin_max;
  float *bin_width;
  int ignore_unseen; /* if input misses range on discrete data, ignore it
			(likelihood = 1, so no effect) */
  int unity_laplace;	/* constant for the laplace correction term */
  int discreteData;
} NBAYES_PARAMS;
				/* constant definitions */
#define MODEL_STORE_SIZE 10
#define PEQ(y) (0==strcmp(param_name,y))
#endif


/* DECLERATION OF EXTERNAL FUNCTIONS */
int nbayes_build(int ninputs, int noutputs, int *nbins);
void nbayes_train_init( int model, int ignore_unseen, int unity_laplace);
void nbayes_test_init( int model, int adjust_priors, float *priors_list);
void nbayes_test_alloc( int model);
void nbayes_free(int model);
int nbayes_read_model( FILE *fp);
int nbayes_write_model( int model, FILE *fp );
void * nbayes_param( int model, char *param_name, int class, int input);
void nbayes_inc_train1( int model, float *inputs, int desired_class);
void nbayes_finish_train1( int model, int discreteData, float rangeFactor);
void nbayes_set_bins( int model, float min, float max);
void nbayes_inc_train2( int model, float *inputs, int desired_class);
void nbayes_finish_train2( int model);
int nbayes_classify( int model, float *inputs );
int nbayes_eval( int model, float *inputs );
#ifdef nbayes_pack_h_static_decl
#undef nbayes_pack_h_static_decl
/* DECLARATION OF STATIC FUNCTIONS */
static void nbayes_set_paramtab(NBAYES_PARAMS *hpp);
static void nbayes_set_paramtab_l( NBAYES_PARAMS *hpp );
static int nbayes_get_bin( float x, float *bin_ends, int nbins);
#endif
#endif
