/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* nc_class_pack.h */

/* DEFINITIONS AND DECLARATIONS FOR nc_class_pack.c */

/* DEFINITIONS OF INTERNAL STRUCTURES AND TYPES */
#ifdef nc_class_pack_h_internal_types
#undef nc_class_pack_h_internal_types
/* nc_class_pack_h_internal_types is defined in `nc_class_pack.c' */
/* Define structure of the nc_class_model */
typedef struct {
    int		ninputs;	/* dimension of each input vector */
    int		ncenters;	/* number of centers */
    int		nclasses;	/* number of classes */
    int		*features;	/* features of raw data used */
    float	**pcenter;	/* vector of center vectors */
    float	**inv_variance;	/* vector of inverse variance vectors */
    float	*log_det_var;	/* vector of determinants variance vectors */
    int		**class_vote;	/* vector of class count vectors */
    int		*class;		/* classes assigned to centers */
    float	*class_cnt;	/* number of samples seen in each class */
    float	total_npatterns;/* total number of samples seen */
    int		dist_type;	/* flag for type of distance for assignment
				   of points to centers */
} NC_CLASS_PARAMS;

/*** Parameter Table ***/
static PARAM    nc_scalars[] = {
    {"ninputs", P_INT},
    {"ncenters", P_INT},
    {"nclasses", P_INT},
    {"total_npat", P_FLOAT},
    {"dist_type", P_INT},
};
static PARAM    nc_arrays[] = {
    {"class_cnt", P_FLOAT_VECTOR},
    {"features", P_INT_VECTOR},
    {"class", P_INT_VECTOR},
    {"centers", P_FLOAT_MATRIX},
    {"inv_vars", P_FLOAT_MATRIX},
    {"log_det_vars", P_FLOAT_VECTOR},
};
#define NNC_SCALARS (sizeof(nc_scalars)/sizeof(PARAM))
#define NNC_ARRAYS (sizeof(nc_arrays)/sizeof(PARAM))
#endif


#ifndef nc_class_pack_h
#define nc_class_pack_h

/* DECLARATION OF EXTERNAL FUNCTIONS */
int nc_class_build (int ninputs, int nclasses, int ncenters);
void nc_class_init (int model_num, float **pcentroids, float **pspreads,
		    int *features, int dist_type, float minimum_variance);
void nc_class_free (int model_num);
int nc_class_read_model (FILE *fp);
void nc_class_write_model (int model_num, FILE *fp);
void *nc_class_param (int, char *);
void nc_class_batch_ctrain (int model_num, float *invec, int class);
void nc_class_batch_update (int model_num);
int nc_class_classify (int model_num, float *invec);
void nc_class_read_cluster (char *param_path, FILE *param_file, int *pninputs,
			   int *pncenters, float ***centroids,
			   float ***spreads, int *features);

#ifdef nc_class_pack_h_statics
#undef nc_class_pack_h_statics
static int get_best_center (NC_CLASS_PARAMS *ncpp, float *invec);
#endif

#endif
