/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/***--------------------------------------------------------------------------
* NAME:		nnlib.h
* PURPOSE:	header file for nnlib.a 
* USAGE: 
* 	#include <nnlib.h>
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/include/nnlib.h,v 11.8 2004/01/27 20:53:25 kukolich Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/include/nnlib.h,v $
* CREATED: 25-Jan-91 rpl                             MIT LINCOLN LABORATORY
*---------------------------------------------------------------------------
* Copyright 1990 MIT Lincoln Laboratory.
* This software is not for release and should neither be copied
* nor distributed without explicit permission of MIT Lincoln Laboratory.
--------------------------------------------------------------------------***/

#ifndef nnlib_h
#define nnlib_h

/* macros for the number of an unknown class, used in some
   classifiers and if we ever put in outlyer detection */
#define UNKNOWN_CLASS (-1)

/* makesure math.h is included and define fast exp10() macro 
   math.h includes: M_LOG10E, M_LN10
-----------------------------------------------------------*/
#ifndef M_LN10
#include <math.h>
#endif

/* we still didn't get our math includes. We need to define these
   ourselves. */
#ifndef M_LN10
#endif

/* exp10 takes over twice as long as exp, so use the macro */
#define nn_exp10(x) (exp(M_LN10*(x)))

/* important machine-dependent values
   includes: MAXINT, MAXLONG, MAXSHORT, MAXDOUBLE,MAXFLOAT, MINDOUBLE, MINFLOAT
----------------------------------------------------------------------------*/
#ifdef SGI
#undef M_LN2
#undef M_PI
#undef M_SQRT2
#endif
#ifdef __CYGWIN__
#include <limits.h>
#include <ieeefp.h>
#define MAXINT INT_MAX
#define MAXLONG LONG_MAX
#define MAXSHORT SHRT_MAX
#define MAXDOUBLE __IEEE_DBL_MAXPOWTWO
/* #define MAXFLOAT __IEEE_FLT_MAXPOWTWO */
#define MINDOUBLE (1.0/__IEEE_DBL_MAXPOWTWO)
#define MINFLOAT (1.0/__IEEE_FLT_MAXPOWTWO)
#else
#include<values.h>
#endif

/* constants nice to have for Gaussian outputs */
#define LN_2PI 1.83787706640935
#define LOG_2PI 0.798179868358

/* macro definitions
-----------------------------*/
#ifndef SQR
#define SQR(x) ((x) * (x))
#endif
#ifndef MAX
#define MAX(x,y) ((x) > (y) ? (x) : (y))
#endif
#ifndef MIN
#define MIN(x,y) ((x) < (y) ? (x) : (y))
#endif
#ifndef AVRG
#define AVRG(x,y) ((double) 0.50 * ((x) + (y)))
#endif

/* function declarations
-----------------------------*/
/********** allocate.c *************/
/*---------------------- allocate.h ----------------------------*/
char *nn_malloc( unsigned int n, char *for_message );
char *nn_realloc( char *ptr, unsigned int n, char *for_message );
void nn_free( char *p, char *for_message );
char *nn_strdup(char *str );

float *allocate_vector( int n );
float *reallocate_vector( float *v, unsigned int n, unsigned int newn );
void deallocate_vector( float *v );
float **allocate_matrix( int nrow, int ncol );
float **reallocate_matrix( float **row_pointers, int nrow, int ncol,
			  int newrow );
float **reallocate_matrix_noncontig( float **row_pointers, int nrow,
				    int ncol, int newrow );
void deallocate_matrix( float **m );

int *allocate_ivector( int n );
int *reallocate_ivector( int *v, int n, int newn );
void deallocate_ivector( int *v );
int *allocate_iindex( int n );
int **allocate_imatrix( int nrow, int ncol );
int **reallocate_imatrix( int **row_pointers,
			 int nrow, int ncol, int newrow );
int **reallocate_imatrix_noncontig( int **row_pointers,
				   int nrow, int ncol, int newrow );
void deallocate_imatrix( int **m );

void **allocate_pvector( int n );
void **reallocate_pvector( void **v, int n, int newn );
void deallocate_pvector( void **v );
void ***allocate_pmatrix( int nrow, int ncol );
void ***reallocate_pmatrix( void ***row_pointers,
			   int nrow, int ncol, int newrow );
void ***reallocate_pmatrix_noncontig( void ***row_pointers,
				     int nrow, int ncol, int newrow );
void deallocate_pmatrix( void ***m );

void *allocate_structvector( int n, int size );
void *reallocate_structvector( void *v, int n, int newn, int size );
void deallocate_structvector( void *v );
void deallocate_structmatrix( void **m );
void **allocate_structmatrix( int nrow, int ncol, int size );
void **reallocate_structmatrix( void **row_pointers,
			       int nrow, int ncol, int newrow, int size );
void **reallocate_structmatrix_noncontig( void **row_pointers, int nrow,
					 int ncol, int newrow, int size );

void deallocate_dvector( double *v );
double *reallocate_dvector( double *v, int n, int newn );
double *allocate_dvector( int n );
double **allocate_dmatrix( int nrow, int ncol );
double **reallocate_dmatrix( double **row_pointers,
			    int nrow, int ncol, int newrow );
double **reallocate_dmatrix_noncontig( double **row_pointers,
				      int nrow, int ncol, int newrow );
void deallocate_dmatrix( double **m );

void nr_deallocate_vector( float *v, int nlow, int nhigh );
void nr_deallocate_matrix( float **m, int row_low, int row_high,
			  int col_low, int col_high );
float *nr_allocate_vector( int nlow, int nhigh );
float **nr_allocate_matrix( int row_low, int row_high,
			   int col_low, int col_high );
void deallocate_string_vector(char ** array, int length);

/********** class_data.c *************/

/*----------------------------------------------------------------
 * NAME: class_data.h
 * PURPOSE: read classification data file and return patterns to classifier
 *	on request.  Routine handles random selection of patterns,
 *	priors adjustment though pattern selection, and cross validation
 ------------------------------------------------------------------*/
int class_data_read( char *data_path, char *data_fname, int npatterns,
		    int nraw, int ninputs, int nclasses, int *features,
		    int normalization, int verbosity);
int class_data_single_init(char *data_path, char *data_fname, int npatterns,
			   int nraw, int ninputs, int noutputs, int *features,
			   int normalization, int verbosity);
int class_data_get_npatterns( int model);
int class_data_get_npatterns_class(int data_model, int class);
void class_data_priors_init( int model, float *priors, int priors_npat);
void class_data_random_init( int model);
void class_data_cv_init( int model, int cv_nsplits, int use_cvsplits,
			int *cv_splits, int cv_nfolds,
			char **train_mask, char **test_mask, int cv_random );
int class_data_start_cv_fold( int data_model, int fold, int is_train );
void class_data_end_cv_fold( int data_model);
void class_data_epoch_init( int data_model);
float *class_data_get_next( int data_model, int *class);
void class_data_exit( int data_model );
void class_data_set_n_read_zero(int dm);/* for testing only,
					   use class_data_epoch_init */
float **class_data_get_input_matrix( int );
int class_data_get_input_num(int data_model);

/********** clust_param.c *************/
int cluster_build( int ninputs, int ncenters, int noutputs,
		  int cluster_by_class, int *ncenters_list );
void cluster_init( int model, int *features );
void cluster_free( int model );
void cluster_write_model( int model, FILE *fp );
int cluster_write_paramfile( FILE *fp, int ninputs, int ncenters,
			    int noutputs, float ***pcenter,
			    float ***pvariance, int *features,
			    int cluster_by_class, int *ncenters_list );
int cluster_read_paramfile( FILE *fp, int *ninputs, int *noutputs,
			   int *ncenters, int *features );
int cluster_read_model( FILE *fp );

void *cluster_param( int model, char *param_name, int class_num );
void cluster_merge_centers( int model );

/********** cross.c *************/
int * build_cross_index(int nclasses, int n_trials, int n_splits,
			int *desired_classes, int use_splits, int *splits);
int *build_map_cross_index( int nclasses, int n_trials, int n_splits,
			   int use_splits, int *splits );
void build_cross_valid_masks( int nfolds, char **train_mask,
			     char **test_mask );
void cross_valid_check_args( int *pnfolds, int *pnsplits, int *cv_splits,
			   char **cv_train_mask, char **cv_test_mask );

/********** distance.c *************/
float l2dist(int n, float *vector1, float *vector2);
float l2dist_quit(int n, float *vector1, float *vector2, float max);
float l2dist_weight(int n, float *vector1, float *vector2, float *weight);
float l2dist_quit_weight( int n, float * vector1, float *vector2,
			 float * weight, float max);
float logadd( float x, float y );
float lnadd( float x, float y );

/********** errors.c *************/
void init_nnsubs( int , char **, char * );
void exit_nnsubs( void );

void coredump_error(char *fmt, ...);
void fatal_error(char *fmt, ...);
void warning(char *fmt, ...);
char *get_program_name( void );
char *get_command_line( void );

/********** fastdist.c *************/
int fastdist_build( int ninputs, int ncenters );
int fastdist_eval( int model, float *inputs, float *distances );
void fastdist_init( int model, float **pcenters, float *pvariances );
void fastdist_exit( int model );
void *fastdist_param( int model, char *param_name );

/********** feat_sel.c *************/

/***---------------------------------------------------------------------------
* NAME:		feat_sel.h
* PURPOSE:	function declerations for feat_sel.c
* USAGE: 	#include <nnlib.h>
*--------------------------------------------------------------------------***/
void feat_sel_build( int ninputs, int search_type, int search_n,
		     int verbosity);
void cat_sel_build( int ninputs, int search_type, int search_n,
		    int verbosity, int *categories);
int feat_sel_search_done(void);
void feat_sel_init_nfeatures(void);
int *feat_sel_next_features(void);
void feat_sel_check_best( float percent_error);
void feat_sel_pick_best(void);
void feat_sel_print_head(void);
void feat_sel_print_final_result(void);
void feat_sel_save_list( char *file_path, char *comment );
void feat_sel_free(void);
int feat_sel_read_model(FILE * fp,int *n_inputs, int * best, int **selected,
			float **errors);
int feat_sel_write_model(FILE *fp, int *selected, float *errors, int best,
			 int n_inputs);

/********** features.c *************/
int fix_features( int nraw, int ninputs, int* features);
int fix_windows(int window_width, int *window_frames);
int count_nwindow_frames(int window_width, int *window_frames);
int feature_read_paramfile (FILE *fp, int **pfeatures,
			    int *pnraw, int *pninputs);
int feature_write_paramfile (FILE *fp, int *features, int nraw, int ninputs);
int window_read_paramfile(FILE *fp, int *pnwindow_frames,
			  int **pwindow_frames);
int window_write_paramfile(FILE *fp, int *window_frames, int nwindow_frames);

/********** files.c *************/
FILE *nn_fopen( char *fname, char *type, char *for_message );
void nn_fclose( FILE *ptr, char *filename, char *for_message );

/********** gauss_out.c *************/
float get_log_gauss_output_full( int ninputs, float *inputs, float *means,
				float **inv_covar, float log_det_covar);
float get_log_gauss_output_diag( int ninputs, float *inputs, float *means,
				float *inv_var, float log_det_covar );
float get_log_gauss_output_sphere( int ninputs, float *inputs, float *means,
				  float inv_var);
float get_log_gauss_output_scalar(float input, float mean, float inv_var);
int invert_and_calc_det( int ninputs, float **inv_covar, float *eigenvalues,
			float max_ratio, float *p_log_det );
float vector_vector_multiply( float *v1, float *v2, int n);

/********** get_subs.c *************/
#include <get_subs.h>
int load_trial_data( FILE* data_file, int map_flag, int n_trials, int nraw,
		    int noutputs, int ninputs, int* trial_class,
		    float** desire, float** trial_data, int normalize,
		    char* data_fname, int* features);
int load_class_data( FILE* data_file, int n_trials, int nraw, int ninputs,
		    int* trial_class, float** trial_data, int normalize,
		    char* data_fname, int* features);
int load_map_data(FILE *data_file, int n_trials, int nraw, int noutputs,
		  int ninputs, float **desire, float **trial_data,
		  int normalize, char *data_fname, int *features);
int load_seq_files( char * path, char * filename, DATA_BLOCK ** block_list,
		   char ** labels, int normalize, int * features,
		   int ninputs, int nraw, int nsequences );
int load_error_file_data( FILE* fptr, char* fname, ERROR_LINE ** err_lines,
			 int error_file_verbosity, int ninputs, int noutputs);
ERROR_LINE *reallocate_error_lines( ERROR_LINE* error_lines, int old_size,
				   int new_size, int ninputs, int noutputs);
int get_class_line( FILE * fptr, char * filename, float * input, int nraw,
		   int norm_flag, int *features, int ninputs);
void write_class_line(FILE *fptr, char *filename, float *inputs, int class,
		      int ninputs);
int get_map_line( FILE* fptr, char* filename, float* input, float* output,
		 int nraw, int ninputs, int noutputs, int norm_flag,
		 int* features );
int get_err_line( FILE* fptr, char* filename, ERROR_LINE* err_line_p,
		 int error_file_verbosity);
int get_map_err_line( FILE* fptr, char* filename, ERROR_LINE* err_line_p,
		 int error_file_verbosity);
int get_seq_err_line( FILE* fptr, char* filename, ERROR_LINE* err_line_p,
		 int error_file_verbosity);
int get_seq_line( FILE * fptr, char *filename, float *input, int nraw,
		 int norm_flag, int *features, int ninputs);
int get_line_token( FILE * fptr, char *string, int length);
void skip_to_eol( FILE * fp);
int label2class( char * token, char ** labels);
int get_n_lines( FILE * fp);
int get_n_tokens( FILE *fp );
FILE *get_seq_file(FILE *list_file, char *data_path, char *filename,
		   DATA_BLOCK *bp, char **labels);
void find_class_priors( FILE *fp, int npatterns, int noutputs,
		       float * priors_list);
void find_seq_priors( FILE *fp, int npatterns, int noutputs,
		     float * priors_list);
int find_frame_priors( FILE *fp, int npatterns, int noutputs,
		      float *priors_list, char *data_path, char *filename,
		      char **labels );

/********** heap.c *************/

/*-------------- heap.h -----------------*/
void heap_init( int n, float *xarray, int *index );
void reheap( int n, float *xarray, int *index );
void heap_sort( int n, float *xarray, int *index );
void heap_sort2( int n, float *xarray, int *array2 );
void rev_heap_init( int n, float *xarray, int *index );
void rev_reheap( int n, float *xarray, int *index );
void rev_heap_sort( int n, float *xarray, int *index );
void rev_heap_sort2( int n, float *xarray, int *array2 );

void heap_iinit( int n, int *xarray, int *index );
void ireheap( int n, int *xarray, int *index );
void heap_isort( int n, int *xarray, int *index );
void rev_heap_iinit( int n, int *xarray, int *index );
void rev_ireheap( int n, int *xarray, int *index );
void rev_heap_isort( int n, int *xarray, int *index );

void heap_dinit( int n, double *xarray, int *index );
void dreheap( int n, double *xarray, int *index );
void heap_dsort( int n, double *xarray, int *index );
void rev_heap_dinit( int n, double *xarray, int *index );
void rev_dreheap( int n, double *xarray, int *index );
void rev_heap_dsort( int n, double *xarray, int *index );

/********** main_subs.c *************/
/*----------------------------------------------------------------
 * NAME: main_subs.h
 * PURPOSE: handle things that are done in all LNKnet main routines
 ------------------------------------------------------------------*/
/* data base type numbers for set_desc_args */
typedef enum {
    CLASS_DATA = 0, MAP_DATA = 1, SEQ_DATA = 2
} DATA_BASE_TYPE;
typedef enum {
    CONT = 0, BIN, CAT, CIRC
} INPUT_FEATURE_TYPE;
typedef struct {
    DATA_BASE_TYPE db_type;
    int ninputs;
    int noutputs;
    char **output_labels;
    char **input_labels;
    INPUT_FEATURE_TYPE *input_types;
    float *input_type_values;
  int *categories;
} DEFAULTS_FILE_STRUCT;

void set_desc_args(char *desc_fullname, char ***plabels, int *pnraw,
		   int *pnoutputs, int *pdb_type);
void read_defaults_file(char *desc_fullname,
			DEFAULTS_FILE_STRUCT *defaults_args);
void read_defaults_fp(FILE *desc_file, DEFAULTS_FILE_STRUCT *defaults_args);
void free_defaults_file_struct(DEFAULTS_FILE_STRUCT *df_args);
void check_desc_args(char *desc_fullname, char **labels, int *pnraw,
		     int *pnoutputs, int *pmap_flag, int *pnpatterns);
void write_defaults_stream(FILE *desc_file,
			   DEFAULTS_FILE_STRUCT *defaults_args);
void write_defaults_file(char *filename, DEFAULTS_FILE_STRUCT *defaults_args);
void write_desc_args(char *filename, char **labels, int ninputs,
		     int noutputs, int db_type);
char * get_input_label(int n, char **input_labels, int *features,
		       int normalization);
char * get_class_label(int n, char **class_labels);
char * make_comment_string (char *flag_string);
void print_log_footer( void );
void print_log_header( void );
void remove_pid_file(char *pid_fullname);
void store_pid (char *pid_fullname);
void display_model_params (int model, int ninputs, int normalization,
			   int *features, int verbosity);
void verify_model_params (int model, int ninputs, int model_ninputs,
			  int *features, int *model_features);
int fix_cross_valid_args(int *pcv, int *pnfolds, int *pnsplits, int *splits,
			 char **train_mask, char **test_mask, int from_file,
			 char *cv_fname);
void check_file_length(char *file_path, int *p_npat);
void fix_priors(int adjust_train, int train_uniform_priors,
		float **train_ppriors_list, int adjust_test,
		int test_uniform_priors, float **test_ppriors_list,
		int train_flag, int cross_valid, int noutputs);
void fix_input_types(DEFAULTS_FILE_STRUCT *df_args, int normalize,
		    int ninputs, int *features);

/********** map_data.c *************/
/*----------------------------------------------------------------
 * NAME: map_data.h
 * PURPOSE: read mapification data file and return patterns to mapifier
 *	on request.  Routine handles random selection of patterns
 *	and cross validation
 ------------------------------------------------------------------*/
int map_data_read( char *data_path, char *data_fname, int npatterns,
		  int nraw, int ninputs, int noutputs, int *features,
		  int normalization, int verbosity);
int map_data_get_npatterns( int  data_model);
void map_data_random_init( int  data_model);
void map_data_cv_init( int data_model, int cv_nsplits, int use_cvsplits,
		      int *cv_splits, int cv_nfolds, char **cv_train_mask,
		      char ** cv_test_mask);
int map_data_start_cv_fold( int data_model, int fold, int is_train);
void map_data_epoch_init( int data_model);
float *map_data_get_next( int data_model, float ** outputs);
void map_data_exit(int data_model);
int map_data_get_input_num(int data_model);
/* don't use the next routine! */
void map_data_set_n_read_zero(int dm);

/********** matrix.c *************/
float **invertM(int, float **, float  );
float **invertM_ev( int, float **, float, float **);
void matrix_vector_multiply( float **matrix, float *vector, float *result,
			     int rows, int columns );
void square_matrix_invert( int ninputs, float **input_matrix,
			   float **output_matrix, float *eigenvalues,
			   float max_ratio );
void square_matrix_multiply( int ninputs,
			     float **input_matrix_1, float **input_matrix_2,
			     float **output_matrix );
void square_matrix_transpose( int ninputs, float **input_matrix,
			      float **output_matrix );
void square_normalize_and_sort( int ninputs, float **eigenvectors,
				float *eigenvalues );
void square_singular_value_decomposition( int ninputs, float **input_matrix,
					  float **eigenvectors,
					  float *eigenvalues );
void transposed_matrix_vector_multiply( float **matrix, float *vector,
					float *result, int rows, int cols);

/********** model.c *************/
int model_get_number( void ***pmodel_list, int *pmlist_length,
			int model_size );

/********** nearest.c *************/
int find_nearest_neighbor( float *inputs, float **centers, int ninputs,
			     int ncenters );
void find_k_nearest_neighbors( float *inputs, float **centers, float *dists,
				 int *neighbors, int ninputs, int ncenters,
				 int k );
int int_max_pick( int *vec, int size );
int max_pick( float *vec, int size );
int max_pick_int( int *vec, int size );

/********** nnargs.c *************/

/***---------------------------------------------------------------------------
* NAME:		nnargs.h
* PURPOSE:	required for nnargs command line argument processor
* MODIFIED: 19-Oct-90 rpl                             MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
#define INT		0		
#define FLOAT		1
#define BOOLEAN		2
#define FLAG		BOOLEAN
#define CHAR		3
#define STRING		4
#define DOUBLE		5
#define INT_LIST	6		/* comma separated lists */
#define STRING_LIST	7
#define FLOAT_LIST	8
#define DOUBLE_LIST	9
#define INT_ALLOC_LIST	10
#define STRING_ALLOC_LIST	11
#define FLOAT_ALLOC_LIST	12
#define DOUBLE_ALLOC_LIST	13
#define LABELS		STRING_LIST
#define NET		INT_LIST
#define MAX_NNLIST	256
#define MAX_INT_LIST	MAX_NNLIST
#define MAX_FLOAT_LIST	MAX_NNLIST
#define MAX_DOUBLE_LIST	MAX_NNLIST
#define NNARGS_MAX_NET	MAX_NNLIST
#define MAX_STRING_LIST MAX_NNLIST
#define NNARGS_MAX_LABELS MAX_NNLIST
#define NNMAX_LINE 	2000
#define NNARGS_MAX_LINE NNMAX_LINE

typedef struct {
    char	*arg;		/* command line switch */
    unsigned	type;		/* variable type */
    int		*variable;	/* pointer to variable */
    char	*errmsg;	/* pointer to error message */
    double	lower_limit;	/* opt limit for int, float, double */
    double     	upper_limit;	/* opt limit for int, float, double */
} ARG;
/* subroutines in nnargs */
int nnargs(int argc, char **argv, ARG *tabp, int tabsize);
int nnargs_part(int argc, char **argv, ARG *tabp, int tabsize);
int fill_argv(char **argv, FILE *describe_file, char *dfname);
int nnargs_from_file(FILE *describe_file, char *describe_fname,
		     ARG *tabp, int tabsize);
void nnargs_print_table(ARG *tabp, int tabsize, FILE *fp);
char * nnargs_get_flags(ARG *tabp, int tabsize);
char * nnargs_get_part_flags(ARG *tabp, int tabsize);
char * nnargs_get_flags_one_line(ARG *tabp, int tabsize);
void nnargs_print_flags(ARG *tabp, int tabsize, FILE *fp);
void nnargs_get_double_list( char *string, double *array, int length);
void nnargs_get_float_list( char *string, float *array, int length);
void nnargs_get_int_list( char *string, int *array, int length);
void nnargs_get_string_list( char *string, char **array, int length);
void nnargs_set_variable(char *name, int *variable, ARG *tabp, int tabsize);
void nnargs_set_list_length(char *name, int length, ARG *tabp, int tabsize);
double *nnargs_get_double_alloc_list( char *string );
float *nnargs_get_float_alloc_list( char *string );
int *nnargs_get_int_alloc_list( char *string );
char **nnargs_get_string_alloc_list( char *string );
void nnargs_combine_argtabs( ARG *first_tab, int first_size,
			     ARG *second_tab, int second_size,
			     ARG **pnew_tab, int *pnew_size );
void nnargs_free_argtab( ARG *tab, int size );
char *nn_strtok( char *str, char *tokEnd );

/********** norm2c.c *************/
void get_norm_params( int model, int ninputs_, int * features_);
void print_norm_comments(void);
void print_norm_macros( char * nraw_name, char * ninputs_name);
void clear_norm_macros(void);
void print_normalize_code( char * normalize_name);
void print_copy_inputs(void);

/********** norm_subs.c *************/
/* macros for normalization types */
#define NONE 0
#define SIMPLE 1
#define PCA 2
#define LDA 3


/* norm_subs.c */
void norm_build( int ninputs, int nclasses, int normalization );
void norm_free(void);
void norm_setup_none( int ninputs, int noutputs);
void norm_init(int total_trials_param);
void norm_inc( float *inputs, int class );
void norm_inc2( float *inputs, int class );
void *norm_param( char *param_name );
void norm_update( void );
void norm_update2( void );
void norm_apply( float *inputs );
int norm_read_paramfile( FILE *fp, int *normalization );
int norm_write_paramfile( FILE *fp );
int norm_normalization( void );
float * norm_mean( void );
float * norm_sdev( void );
int *norm_calc_needed( void );
void make_transform_matrix( int ninputs, float **input_matrix,
			   float **output_matrix, float *eigenvalues,
			   float max_ratio, int *max_features );
char *norm_description( int normalization );
int read_norm_file( char *norm_filename );

/********** note.c *************/
/* note.h */
void diff_argv( FILE *fp, int argc1, char ** argv1, int argc2, char ** argv2 );
void store_train_note_command( FILE * shell_file, char * log_fname,
			      char * script_name, char * notebook_path);
void store_test_note_command( FILE * shell_file, char * log_fname,
			     char * script_name, char * notebook_path);
void store_cv_note_command( FILE * shell_file, char * log_fname,
			   char * script_name, char * notebook_path);
void store_roc_note_command( FILE * shell_file, char * log_fname,
			    char * script_name, char * notebook_path);
void store_chi_note_command( FILE * shell_file, char * log_fname,
			    char * script_name, char * notebook_path);

/********** param_util.c *************/

/***---------------------------------------------------------------------------
* NAME:		param_util.h
* PURPOSE:	typdefs needed for parameter file processor
* USAGE: 	#include "/u/es/hybrid/iwr/param/param_util.h"
* CREATED:	25-Sep-90 es                             MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
#define P_INT		0
#define P_FLOAT		1
#define P_BOOLEAN	2
#define P_FLAG		P_BOOLEAN
#define P_CHAR		3
#define P_STRING	4
#define P_DOUBLE	5
#define P_INT_VECTOR	6
#define P_STRING_VECTOR	7
#define P_FLOAT_VECTOR	8
#define P_DOUBLE_VECTOR	9
#define P_INT_MATRIX	10
#define P_FLOAT_MATRIX	11
#define P_DOUBLE_MATRIX	12

#define MAXFIELD	50
#define MAXLINE		100


typedef struct {
    char	*label;		/* parameter label */
    unsigned	type;		/* variable type */
    int		*variable;	/* pointer to variable */
    int		n_dim1;         /* dimension 1 */
    int		n_dim2;         /* dimension 2 */
} PARAM;

#define BEG_BLK_STR "<BEGIN_PARAM>"
#define END_BLK_STR "<END_PARAM>"
#define BEG_VEC_STR "{BEGIN_VECTOR"
#define END_VEC_STR "END_VECTOR}"
#define BEG_MAT_STR "{BEGIN_MATRIX"
#define END_MAT_STR "END_MATRIX}"

#define FLAG_TRUE	"YES"
#define FLAG_FALSE	"NO"

int param_putblk( FILE *fileptr, PARAM *tabp, int nparam );
int param_getblk( FILE *fileptr, PARAM *tabp, int nparam );
void param_setvarptr( char *name, int *varp, PARAM *tabp, int nparam );
void param_setdim( char *name, int dim1, int dim2,
		     PARAM *tabp, int nparam );
void param_inc_allptrs( int pincr, PARAM *tabp, int nparam );
void param_put( FILE *fileptr, PARAM *paramp );
int param_get( FILE *filepat, PARAM *tabp, int nparam );
void param_put_begblkstr( FILE *fp );
void param_put_endblkstr( FILE *fp );
int param_find_begblk( FILE *fileptr );
int param_find_endblk( FILE *fileptr );
PARAM *param_find( char *name, PARAM *tabp, int nparam );

/********** path_build.c *************/
char * path_build( char *str1, char *str2 );
char * path_create( char *str1, char *str2 );

/********** print_source.c *************/
/***-------------------------------------------------------------------------
* NAME:        print_source.h
* PURPOSE:     header file for print_source.c
* USAGE:       #include "/u/kukolich/lnknet/src/lib/print_source.h"
*
* $Header: 
* $Source: 
* CREATED: 20-AUG-93 deniz                             MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
#ifndef print_source_h
#define print_source_h

#define EXTERN 1
#define STATIC 0

#define NFLOATS_PER_LINE 4
#define NINTS_PER_LINE 8

extern void print_matrix_initialization( char *matrix_name, float **matrix,
					int isize, int jsize, char *iname,
					char *jname);
extern void print_vector_initialization( char *vector_name, float *vector,
					int size, char *iname);
extern void print_vector( float *vector, int isize);
extern void print_sampled_matrix_initialization( char *matrix_name,
						float **matrix, int isize,
						int jsize, char *iname,
						char *jname, int *features );
extern void print_sampled_vector_initialization( char *vector_name,
						float *vector, int isize,
						char *iname, int *features );
extern void print_imatrix_initialization( char *matrix_name, int **matrix,
					 int isize, int jsize, char *iname,
					 char *jname);
extern void print_ivector_initialization( char *vector_name, int *vector,
					 int isize, char *iname);

extern void print_ivector( int *vector, int isize);
#endif

/********** priors.c *************/
void init_priors_data( int *classes, int nclasses, int npatterns,
		      int random_flag, float * priors_list);
int get_priors_class( void );
int get_priors_index( int class );
void free_priors_data( void );

/********** random.c *************/
float random_uniform( void );
float random_gauss( float, float );
int random_event( float p );
float random_gauss_height( float, float, float );
int random_select( float *pprobs, int n );
int random_int( int n1, int n2 );
void randomize_index_list( int *index_list, int list_length );
void set_random_seed(int seed);
void set_nn_random(void);
/********** report.c *************/
/***-------------------------------------------------------------------
 * NAME: report.h
 * PURPOSE: results reporting functions for classification, mapping,
 *	and sequence recognition
 *
 -------------------------------------------------------------------***/
#define OVERALL_VERBOSITY -1
#define TABLE_VERBOSITY 0
#define CONFUSION_VERBOSITY 1
#define EPOCH_VERBOSITY 2
#define FLAGS_VERBOSITY 2

#define NONE_VERROR 0
#define RESULTS_VERROR 1
#define OUTPUTS_VERROR 2
#define INPUTS_VERROR 3

int report_build( int verbosity, int verror, int noutputs);
void report_free( int model);
void report_init( int model, int db_type, int ninputs, int cost_func,
		 int log_otuputs, int sum_to_one);
void report_roc_init(int model, int target, float lower, float upper);
void report_class_trial( int model, FILE * error_file, float * inputs,
			float *outputs, int desired_class, int class,
			int trial_num);
void report_cluster_trial( int model, FILE *error_file, float *inputs,
			  float *outputs, int class, int trial_num);
void report_map_trial( int model, FILE *error_file, float *inputs,
		      float *outputs, float *desired_outputs, int trial_num );
void report_overall(int model, FILE * log_file, char *error_fname,
		    char **labels);
void report_cluster_overall(int model, FILE * log_file, char *error_fname,
			    char **labels);
void report_epoch_header( int model, FILE *report_file, int n_trials);
void report_data_epoch_header(int model, FILE *report_file, int data_model);
void report_epoch( int model, FILE *report_file, int epoch);
void report_centers_epoch( int model, FILE *report_file, int epoch,
			  int centers);
void report_artmap_epoch(int model, FILE *report_file, int epoch, int err_cnt,
			 int num_a, int num_b);
void report_cluster_epoch( int model, FILE *report_file, int epoch,
			  int centers, float cost);
void report_test_result(int model, FILE *report_file, char * dataname);
void report_cv_result(int model, FILE *report_file, char *dataname,
		      int nfolds);
void report_last_epoch(int model, FILE *report_file, int epoch);
float report_get_avg_cost( int model);
void report_seq_trial( int model, FILE *error_file, int trial_num,
		      float *outputs, int desired_class, int class);
float report_get_percent_error( int model);

/********** roc_cost.c *************/
/* roc_cost.h */
void roc_cost_init( int noutputs, int nbins);
float roc_calc_cost( float *outputs, float * desired_outputs);
float roc_calc_cost_vector( float *outputs, float *desired_outputs,
			    float *cost_vector );
int roc_calc_deriv( float *outputs, float *desired_outputs,
		    float *derivative );
float roc_cost_train( float *outputs, float *desired_outputs );
void roc_cost_finish_epoch(void);
void roc_print_values( int target, FILE *fp );
void roc_print_slopes(int target, FILE *fp);
void roc_cost_free(void);
float roc_cost_get_area( int target );

/********** roc_pack.c *************/
/*  roc_pack.h */
/*  defines objects and operators for class roc_pack */

#define ROC_TINY 1.0e-30

typedef struct{
  int nallocpatterns;
  int npatterns;
  int npatterns_left; /* after rejections */
  int nclasses;
  float **output_val; /* last column is the desired value */

  int class;
  float reject;
  int *reject_flag;

  int table_length;
  int *ninclass;
  int *class_index;
  int *npat;
  int *ndr;
  int *nfa;
  float *threshold;
  float *false_alarm;
  float *detect_rate;
} roc;

int roc_build(int nclasses, int npatterns );
void roc_init(int model);
void roc_free( int model );
void roc_param( int model, roc **model_pointer );
void roc_batch_train( int model, int class, float *outputs );
void roc_log_train( int model, int class, float *outputs );
void roc_batch_compute( int model, int target_class, float reject);
void roc_print_table( FILE *dp,int model, int target, float reject,
		     int verbosity );
void roc_print_sampled_table( FILE *fp, int model, int target, float reject,
			     int verbosity, float table_begin,
			     float table_end, float table_step);
float roc_get_percent_fom( int model, int target, float reject,
			  float false_alarm_min, float false_alarm_max);
float roc_get_area( int model, int target, float reject);
float roc_get_eer( FILE *fp, int model, int target, float reject);

/********** seq_data.c *************/
/*----------------------------------------------------------------
 * NAME: seq_data.h
 * PURPOSE: read classification data file and return patterns to classifier
 *	on request.  Routine handles random selection of patterns
 *	and cross validation
 ------------------------------------------------------------------*/
int seq_data_read( char *data_path, char *data_fname, int nsequences,
		  int nraw, int ninputs, int noutputs, int *features,
		  int normalization, char **labels, int verbosity,
		  int nwindow_frames, int *window_frames);
int seq_data_get_nfiles( int model);
int seq_data_get_nfiles_class(int model, int class);
int seq_data_get_nframes( int model);
void seq_data_random_init( int model);
void seq_data_priors_init(int model, float *priors_list, int priors_npatterns);
void seq_data_cv_init(int model, int cv_nsplits, int use_cvsplits,
		      int *cv_splits, int cv_nfolds, char **train_mask,
		      char **test_mask, int cv_random );
int seq_data_start_cv_fold( int model, int fold, int is_train);
void seq_data_end_cv_fold( int model);
void seq_data_epoch_init( int model);
int seq_data_get_next_file( int model);
int seq_data_get_file_num(int model);
int seq_data_get_class(int model);
float *seq_data_get_next_frame( int model);
void seq_data_exit(int model);
void seq_data_set_n_read_zero(int dm);/* don't use this, it is a cheat*/

/********** sigm.c *************/
/**************************** sigm.h *************************/
/* cost function type definitions */
#define SQ_ERR (0)
#define CROSS_ENTROPY (1)
#define MAX_LIKE (2)
/* percepton convergence procedure, mlp only */
#define PC_PROC (3)
#define CFOM (4)
/* average log likelihood, for gmix */
#define LOG_LIKE (5)
/* number of centers added */
#define CENTERS (6)
/* no cost function */
#define NO_COST (7)
/* cost functions for clustering */
#define CLUS_SQ_ERR (8)
#define CLUS_LOG_LIKE (9)
/* cost for error reporting for artmap */
#define ARTMAP_COST (10)
/* ROC Cost */
#define ROC_COST (11)

/* sigmoid function type definitions */
#define SIGM (0)
#define SYM_SIGM (1)
#define STEP (3)
#define LINEAR (4)
float sigm( float val, int type, float param);
float dsigm( float val, int type, float param );
void init_cost(int cost_func, float param );
float calc_cost_vector(float *, float *, float *, int , int  );
float calc_cost ( float *, float *, int , int );
float calc_avg_cost( float, int, int );
void calc_final_sigm( float *inputs, float *outputs, int noutputs,
			int sigm_type, float sigm_param, int cost_func );
int calc_output_deriv( float *desired_outputs, float *outputs,
			 float tolerance, float *derivative, int noutputs,
			 int sigm_type, float sigm_param, int cost_func );
char *cost_string( int cost_func );

/********** sort_matrix.c *************/
/***---------------------------------------------------------------------------
* NAME:		sort_matrix.h
* PURPOSE:	function declerations for sort_matrix.c
* USAGE: 	#include "/u/kukolich/lnknet/lib/sort_matrix.h"
* CREATED:	14-Jul-93                               MIT LINCOLN LABORATORY
*--------------------------------------------------------------------------***/
#ifndef sort_matrix_h
#define sort_matrix_h

int sort_matrix_rows( int, int, int, float** );

#endif

/********** stat.c *************/
/* stat.h */
float chi_square( int n , int *dof, float *actual, float *predicted,
		  float *significance );
float chi_square_siqnificance( int dof, float x);

/********** svdcmp.c *************/
void svbksb(float **u, float *w, float **v, int m, int n, float *b, float *x);
void svdcmp( float **a, int m, int n, float *w, float **v );

/********** swap.c *************/
/***-------------------------------------------------------------------------
* NAME: swap.h
* PURPOSE:     swap functions to swap two variables of various types
* USAGE: #include "/u/kukolich/lnknet/src/lib/swap.h"
*
* $Header: 
* $Source: 
* CREATED: 09-AUG-93 deniz                             MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
#ifndef swap_h
#define swap_h

extern int swap_int( int*a, int*b );
extern int swap_float( float*a, float*b );
extern int swap_char( char*a, char*b );
extern int swap_string( char**a, char**b );

#endif

/********** system.c *************/
int nn_system( char *s );

/********** top2diff_cost.c *************/
void top2diff_init(float steepness); 
float top2diff_cost( float *, int, int );
void d_top2diff_cost( float *out_vector, int correct_class, int noutputs,
			float *d_vector );
/************ vonmises.c ******************/
float vonmises_mean( double cosSum, double sinSum );
float vonmises_inv_var( double cosSum, double sinSum, int n );
float vonmises_log_output_scalar( float rad_input, float rad_mean,
				 float inv_var );

/* sets of arguments so we don't always have to edit the whole world
   to add a feature */
typedef struct {
  int search_type;
  int search_n;
  int search_verbosity;
  char *search_param_fname;
  int *categories;
} SEARCH_ARGUMENTS;
typedef struct {
  int verror;
  char *error_fname;
  int cost_func;
  int roc_target;
  float roc_lower;
  float roc_upper;
} REPORT_ARGUMENTS;
typedef struct {
  int adjust_train;
  int priors_npatterns;
  float *train_priors_list;
  int adjust_test;
  float *test_priors_list;
} PRIORS_ARGUMENTS;
typedef struct {
  /* data as read from the file */
  char *data_path;
  char *data_fname;
  int map_flag;
  int nraw;
  int noutputs;
  char **labels;
  /* normalize it */
  char *norm_fname;
  int normalize;
  /* do feature selection */
  int ninputs;
  int *features;
  /* how many to read and in what order */
  int npatterns;
  int random_flag;
} DATA_ARGUMENTS;
typedef struct {
  int cv_nfolds;
  int cv_nsplits;
  int use_cv_splits;
  int *cv_splits;
  char **cv_train_mask;
  char **cv_test_mask;
  int cv_random;
} CV_ARGUMENTS;
#endif

