/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/***--------------------------------------------------------------------------
* NAME:		nnlib.h
* PURPOSE:	header file for nnlib.a 
* USAGE: 
* 	#include "/u/rpl/nn/public/lib/nnlib.h"
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/include/nnlibKR.h,v 11.2 1999/09/01 21:22:06 kukolich Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/include/nnlibKR.h,v $
* CREATED: 25-Jan-91 rpl                             MIT LINCOLN LABORATORY
*---------------------------------------------------------------------------
* Copyright 1990 MIT Lincoln Laboratory.
* This software is not for release and should neither be copied
* nor distributed without explicit permission of MIT Lincoln Laboratory.
--------------------------------------------------------------------------***/
/* makesure math.h is included and define fast exp10() macro 
   math.h includes: M_LOG10E, M_LN10
-----------------------------------------------------------*/
#ifndef M_LN10
#include <math.h>
#endif

/* exp10 takes over twice as long as exp, so use the macro */
#define nn_exp10(x) (exp(M_LN10*(x)))

/* important machine-dependent values
   includes: MAXINT, MAXLONG, MAXSHORT, MAXDOUBLE,MAXFLOAT, MINDOUBLE, MINFLOAT
----------------------------------------------------------------------------*/
#ifdef SGI
#undef M_LN2
#undef M_PI
#undef M_SQRT2
#endif
#include<values.h>

/* constants nice to have for Gaussian outputs */
#define LN_2PI 1.83787706640935
#define LOG_2PI 0.798179868358

/* macro definitions
-----------------------------*/
#ifndef SQR
#define SQR(x) ((x) * (x))
#endif
#ifndef MAX
#define MAX(x,y) ((x) > (y) ? (x) : (y))
#endif
#ifndef MIN
#define MIN(x,y) ((x) < (y) ? (x) : (y))
#endif
#ifndef AVRG
#define AVRG(x,y) ((double) 0.50 * ((x) + (y)))
#endif

/* function declarations
-----------------------------*/
int *build_cross_index(/* int nclasses, int n_trials, int n_splits,
			  int *desired_classes, int use_splits,
			  int *splits */);
int *build_map_cross_index(/* int nclasses, int n_trials, int n_splits,
			      int use_splits, int *splits */);
int calc_output_deriv(/* float *desired_outputs, float *outputs,
			 float tolerance, float *derivative, int noutputs,
			 int sigm_type, float sigm_param, int cost_func */);
int cluster_build(/* int ninputs, int ncenters, int noutputs,
		     int cluster_by_class, int *ncenters_list */);
int cluster_write_paramfile(/* FILE *fp, int ninputs, int ncenters,
			       int noutputs, float ***pcenter,
			       float ***pvariance, int *features,
			       int cluster_by_class, int *ncenters_list */);
int cluster_read_paramfile(/* FILE *fp, int *ninputs, int *noutputs,
			      int *ncenters, int *features */);
int cluster_read_model(/* FILE *fp */);
int cross_valid_check_args(/* int *pnfolds, int *pnsplits, int *cv_splits,
			      char **cv_train_mask, char **cv_test_mask */);
int fastdist_build(/* int ninputs, int ncenters */);
int fastdist_eval(/* int model, float *inputs, float *distances */);
int find_nearest_neighbor(/* float *inputs, float **centers, int ninputs,
			     int ncenters */);
int get_priors_class(/* void */);
int get_priors_index(/* void */);
int int_max_pick(/* int *vec, int size */);
int invert_and_calc_det(/* int ninputs, float **inv_covar, float *eigenvalues,
			   float *max_ratio, float *p_log_det */);
int model_get_number(/* void ***pmodel_list, int *pmlist_length,
			int model_size */);
int nn_system(/* char s */);
int max_pick(/* float *vec, int size */);
int random_int(/* int n1, int n2 */);
int random_select(/* float *pprobs, int n */);
int random_event(/* float p */);
int scan_trial_data(/* FILE *data_file, int map_flag, int n_data_inputs,
		       int n_outputs, int normalize, int n_patterns,
		       char *data_fname */);

void build_cross_valid_masks(/* int nfolds, char **train_mask,
				char **test_mask */);
void calc_final_sigm(/* float *inputs, float *outputs, int noutputs,
			int sigm_type, float sigm_param, int cost_func */);
void cluster_init(/* int model, int *features */);
void cluster_free(/* int model */);
void cluster_write_model(/* int model, FILE *fp */);
void *cluster_param(/* int model, char *param_name, int class_num */);
void cluster_merge_centers(/* int model */);
void coredump_error(/* va_alist */);
void d_top2diff_cost(/* float *out_vector, int correct_class, int noutputs,
			float *d_vector */);
void exit_nnsubs(/* void */);
void fatal_error(/* va_alist */);
void fastdist_init(/* int model, float **pcenters, float *pvariances */);
void fastdist_exit(/* int model */);
void *fastdist_param(/* int model, char *param_name */);
void find_k_nearest_neighbors(/* float *inputs, float **centers, float *dists,
				 int *neighbors, int ninputs, int ncenters,
				 int k */);
void free_priors_data(/* void */);
void init_cost(/*int cost_func, float param */);
void init_nnsubs(/* int argc, char *argv[], char *rcsid */);
void init_priors_data(/* int *classes, int nclasses, int npatterns,
			 int random_flag, float *priors_list */);
void make_transform_matrix(/* int ninputs, int *max_features,
			   float **input_matrix, float **output_matrix,
			   float max_ratio, float *eigenvalues */);
void matrix_vector_multiply(/* float **matrix, float *vector, float *result,
			       int rows, int columns */);
void nn_fclose(/* FILE *ptr, char *filename, char *for_message */);
void randomize_index_list(/* int *index_list, int list_length */);
void skip_to_eol(/* FILE fp */);
void square_matrix_invert(/* int ninputs, float **input_matrix,
			     float **output_matrix, float *eigenvalues,
			     float max_ratio */);
void square_matrix_multiply(/* int ninputs,
			       float **input_matrix_1, float **input_matrix_2,
			       float **output_matrix */);
void square_matrix_transpose(/* int ninputs, float **input_matrix,
				float **output_matrix */);
void square_normalize_and_sort(/* int ninputs, float **eigenvectors,
				  float *eigenvalues */);
void square_singular_value_decomposition(/* int ninputs, float **input_matrix,
					    float **eigenvectors,
					    float *eigenvalues */);
void svbksb(/* float **u, float w[], float **v, int m, int n,
	       float b[], float x[] */);
void svdcmp(/* float **a, int m, int n, float *w, float **v */);
void top2diff_init(); 
void transposed_matrix_vector_multiply(/* float **matrix, float *vector,
					  float *result,
					  int rows, int columns */);
void warning(/* va_alist */);

float calc_avg_cost(/* float cost_sum, int cost_sum, int cost_func */);
float calc_cost (/* float *, float *, int , int */);
float calc_cost_vector(/*float *, float *, float *, int , int  */);
float dsigm(/* float , int , float  */);
float get_log_gauss_output_full(/* int , float *, float *, float **, float */);
float get_log_gauss_output_diag(/* int , float *, float *, float *, float  */);
float get_log_gauss_output_sphere(/* int, float *, float *, float */);
float get_log_gauss_output_scalar(/* float, float, float */);
float **invertM(/*int, float **, float  */);
float **invertM_ev(/* int, float **, float, float ***/);
float l2dist(/* int , float *, float * */);
float l2dist_quit(/* int , float *, float *, float */);
float l2dist_weight(/* int , float *, float *, float * */);
float l2dist_quit_weight(/* int , float *, float *,
			    float *, float */);
float logadd(/* float, float  */);
float lnadd(/* float, float  */);
float random_uniform(/* void */);
float random_gauss(/* float, float */);
float random_gauss_height(/* float, float, float */);
float sigm(/* float, int, float */);
float top2diff_cost(/* float *, int, int */);
float vector_vector_multiply(/* float *, float *, int */);
float vonmises_rad_mean(/* double cosSum, double sinSum */);
float vonmises_inv_var(/* double cosSum, double sinSum, int n */);
float vonmises_log_output_scalar(/* float rad_input, float rad_mean,
				    float inv_var */);

char *cost_string(/* int cost_func */);
char * path_build(/* char *str1, char *str2 */);
char * path_create(/* char *str1, char *str2 */);
char *get_program_name(/* void */), *get_command_line(/* void */);

FILE *nn_fopen(/* char *fname, char *type, char *for_message */);
FILE *get_seq_file(/*FILE *list_file, char *filename, DATA_BLOCK *bp,
		     char **labels, int noutputs */);

/***---------------------------------------------------------------------------
* NAME:		nnargs.h
* PURPOSE:	required for nnargs command line argument processor
* MODIFIED: 19-Oct-90 rpl                             MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
#define INT		0		
#define FLOAT		1
#define BOOLEAN		2
#define FLAG		BOOLEAN
#define CHAR		3
#define STRING		4
#define DOUBLE		5
#define INT_LIST	6		/* comma separated lists */
#define STRING_LIST	7
#define FLOAT_LIST	8
#define DOUBLE_LIST	9
#define INT_ALLOC_LIST	10
#define STRING_ALLOC_LIST	11
#define FLOAT_ALLOC_LIST	12
#define DOUBLE_ALLOC_LIST	13
#define LABELS		STRING_LIST
#define NET		INT_LIST
#define MAX_NNLIST	256
#define MAX_INT_LIST	MAX_NNLIST
#define MAX_FLOAT_LIST	MAX_NNLIST
#define MAX_DOUBLE_LIST	MAX_NNLIST
#define NNARGS_MAX_NET	MAX_NNLIST
#define MAX_STRING_LIST MAX_NNLIST
#define NNARGS_MAX_LABELS MAX_NNLIST
#define NNMAX_LINE 	2000
#define NNARGS_MAX_LINE NNMAX_LINE

typedef struct {
    char	*arg;		/* command line switch */
    unsigned	type;		/* variable type */
    int		*variable;	/* pointer to variable */
    char	*errmsg;	/* pointer to error message */
    double	lower_limit;	/* opt limit for int, float, double */
    double     	upper_limit;	/* opt limit for int, float, double */
} ARG;
/* subroutines in nnargs */
int nnargs(/* int , char **, ARG *, int */);
int nnargs_part(/* int , char **, ARG *, int */);
int fill_argv(/* int, char **, ARG *, int */);
int nnargs_from_file(/* FILE *, char *, ARG *, int  */);
void nnargs_print_table(/* ARG *, int, FILE * */);
char *nnargs_get_flags(/* ARG *, int */);
char *nnargs_get_part_flags(/* ARG *, int */);
char *nnargs_get_flags_one_line(/* ARG *, int */);
void nnargs_print_flags(/* ARG *, int, FILE * */);
int nnargs_get_double_list(/* char *, double *, int */);
int nnargs_get_float_list(/* char *, float *, int */);
int nnargs_get_int_list(/* char *, int *, int */);
int nnargs_get_string_list(/* char *, char **, int */);
void nnargs_set_variable(/* char *name, int *variable, ARG *tabp, int tabsize */);
void nnargs_set_list_length(/* char *name, int length, ARG *tabp, int tabsize */);
double *nnargs_get_double_alloc_list(/* char * */);
float *nnargs_get_float_alloc_list(/* char * */);
int *nnargs_get_int_alloc_list(/* char * */);
char **nnargs_get_string_alloc_list(/* char * */);
void nnargs_combine_argtabs(/* ARG *, int, ARG *, int, ARG **, int * */);
void nnargs_free_argtab(/* ARG *, int */);
char *nn_strtok(/* char *, char * */);

/***---------------------------------------------------------------------------
* NAME:		param_util.h
* PURPOSE:	typdefs needed for parameter file processor
* USAGE: 	#include "/u/es/hybrid/iwr/param/param_util.h"
* CREATED:	25-Sep-90 es                             MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
#define P_INT		0
#define P_FLOAT		1
#define P_BOOLEAN	2
#define P_FLAG		P_BOOLEAN
#define P_CHAR		3
#define P_STRING	4
#define P_DOUBLE	5
#define P_INT_VECTOR	6
#define P_STRING_VECTOR	7
#define P_FLOAT_VECTOR	8
#define P_DOUBLE_VECTOR	9
#define P_INT_MATRIX	10
#define P_FLOAT_MATRIX	11
#define P_DOUBLE_MATRIX	12

#define MAXFIELD	50
#define MAXLINE		100


typedef struct {
    char	*label;		/* parameter label */
    unsigned	type;		/* variable type */
    int		*variable;	/* pointer to variable */
    int		n_dim1;         /* dimension 1 */
    int		n_dim2;         /* dimension 2 */
} PARAM;

#define BEG_BLK_STR "<BEGIN_PARAM>"
#define END_BLK_STR "<END_PARAM>"
#define BEG_VEC_STR "{BEGIN_VECTOR"
#define END_VEC_STR "END_VECTOR}"
#define BEG_MAT_STR "{BEGIN_MATRIX"
#define END_MAT_STR "END_MATRIX}"

#define FLAG_TRUE	"YES"
#define FLAG_FALSE	"NO"

int param_putblk(/* FILE *fileptr, PARAM *tabp, int nparam */);
int param_getblk(/* FILE *fileptr, PARAM *tabp, int nparam */);
void param_setvarptr(/* char *name, int *varp, PARAM *tabp, int nparam */);
void param_setdim(/* char *name, int dim1, int dim2,
		     PARAM *tabp, int nparam */);
void param_inc_allptrs(/* int pincr, PARAM *tabp, int nparam */);
int param_put(/* FILE *fileptr, PARAM *paramp */);
int param_get(/* FILE *filepat, PARAM *tabp, int nparam */);
void param_put_begblkstr(/* FILE *fp */);
void param_put_endblkstr(/* FILE *fp */);
int param_find_begblk(/* FILE *fileptr */);
int param_find_endblk(/* FILE *fileptr */);
PARAM *param_find(/* char *name, PARAM *tabp, int nparam */);

/**************************** sigm.h *************************/
/* cost function type definitions */
#define SQ_ERR (0)
#define CROSS_ENTROPY (1)
#define MAX_LIKE (2)
/* percepton convergence procedure, mlp only */
#define PC_PROC (3)
#define CFOM (4)
/* average log likelihood, for gmix */
#define LOG_LIKE (5)
/* number of centers added */
#define CENTERS (6)
/* no cost function */
#define NO_COST (7)
/* cost functions for clustering */
#define CLUS_SQ_ERR (8)
#define CLUS_LOG_LIKE (9)
/* cost for error reporting for artmap */
#define ARTMAP_COST (10)
/* ROC Cost */
#define ROC_COST (11)

/* sigmoid function type definitions */
#define SIGM (0)
#define SYM_SIGM (1)
#define STEP (3)
#define LINEAR (4)

/***---------------------------------------------------------------------------
* NAME:	      get_subs.h
* PURPOSE:    function declerations, type and macro definitions for get_subs.c
* USAGE:      #include "/u/kukolich/lnknet/lib/get_subs.h"
* CREATED:    20-Jul-93                               MIT LINCOLN LABORATORY
*--------------------------------------------------------------------------***/

#ifndef get_subs_h
#define get_subs_h

#define RTN '\n'
#define BLOCK_SIZE 256

typedef struct{
    int trial;
    int desired_class;
    int class;
    int err_f;
    float rmserr;
    float *inputs;
    int ninputs;
    float *outputs;
    float *desired_outputs;
    int noutputs;
} ERROR_LINE;

typedef struct{
    char fname[128];
    int class;
    int ninputs;
    int start;
    int npatterns;
    float ** data_matrix;
} DATA_BLOCK;

/* decleration of (some) functions defined in `get_subs.c' */

int load_trial_data(/* FILE*, int, int, int, int, int, int*, int,
		       float**, float**, char*, int* */);
int load_class_data(/* FILE*, int, int, int, int*, int,
		       float**, char*, int* */);
int load_map_data(/* FILE *, int, int, int, int, int, int **, int **,
		     char *, int * */);
int load_seq_files(/* char *, char *, DATA_BLOCK **, char **, int,
		      int *, int, int */);
int load_error_file_data(/* FILE*, char*, ERROR_LINE **, int, int, int */);
int get_class_line(/* FILE *, char *, float *, int, int, int *, int */);
int get_map_line(/* FILE*, char*, float*, float*, int, int, int,
		    int, int*  */);
int get_err_line(/* FILE* , char* , ERROR_LINE* , int */);
int get_seq_line(/* FILE *, char *, float *, int, int, int *, int */);
int fix_features(/* int, int, int* */);
int get_line_token(/* FILE *, char *, int */);
void skip_to_eol(/* FILE * */);
int label2class(/* char *, char ** */);
int get_n_lines(/* FILE * */);
int get_n_tokens(/* FILE * */);
FILE *get_seq_file(/* FILE *, char *, char *, DATA_BLOCK *, char ** */);
void find_class_priors(/* FILE *, int, float * */);
void find_seq_priors(/* FILE *, int, float * */);
int find_frame_priors(/* FILE *, int, float *, char * char * char ** */);
ERROR_LINE *reallocate_error_lines(/* ERROR_LINE*, int, int, int, int*/);

#endif

/***---------------------------------------------------------------------------
* NAME:		sort_matrix.h
* PURPOSE:	function declerations for sort_matrix.c
* USAGE: 	#include "/u/kukolich/lnknet/lib/sort_matrix.h"
* CREATED:	14-Jul-93                               MIT LINCOLN LABORATORY
*--------------------------------------------------------------------------***/
#ifndef sort_matrix_h
#define sort_matrix_h

int sort_matrix_rows(/* int, int, int, float** */);
int swap_successive_matrix_rows(/* int, int, float** */);

#endif



/***-------------------------------------------------------------------------
* NAME: swap.h
* PURPOSE:     swap functions to swap two variables of various types
* USAGE: #include "/u/kukolich/lnknet/src/lib/swap.h"
*
* $Header: 
* $Source: 
* CREATED: 09-AUG-93 deniz                             MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
#ifndef swap_h
#define swap_h

extern int swap_int(/* int*, int* */);
extern int swap_float(/* float*, float* */);
extern int swap_char(/* char*, char* */);
extern int swap_string(/* char**, char** */);

#endif


/***-------------------------------------------------------------------------
* NAME:        print_source.h
* PURPOSE:     header file for print_source.c
* USAGE:       #include "/u/kukolich/lnknet/src/lib/print_source.h"
*
* $Header: 
* $Source: 
* CREATED: 20-AUG-93 deniz                             MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
#ifndef print_source_h
#define print_source_h

#define EXTERN 1
#define STATIC 0

#define NFLOATS_PER_LINE 4
#define NINTS_PER_LINE 8

extern void print_matrix_initialization(/* char *, float**, int, int */);
extern void print_sampled_matrix_initialization(/* char *, float **, int, int,
						   int * */);
extern void print_sampled_vector_initialization(/* char *, float *, int,
						   int * */);
extern void print_vector_initialization(/* char *, float*, int */);
extern void print_vector(/* float *, int */);
extern void print_ivector(/* int *, int */);
extern void print_imatrix_initialization(/* char *, int**, int, int */);
extern void print_ivector_initialization(/* char *, int*, int */);

#endif


/***-------------------------------------------------------------------
 * NAME: report.h
 * PURPOSE: results reporting functions for classification, mapping,
 *	and sequence recognition
 *
 -------------------------------------------------------------------***/
#define OVERALL_VERBOSITY -1
#define TABLE_VERBOSITY 0
#define CONFUSION_VERBOSITY 1
#define EPOCH_VERBOSITY 2
#define FLAGS_VERBOSITY 2

#define NONE_VERROR 0
#define RESULTS_VERROR 1
#define OUTPUTS_VERROR 2
#define INPUTS_VERROR 3

int report_build(/* int, int, int */);
void report_free(/* int */);
void report_init(/* int, int, int, int, int */);
void report_class_trial(/* int, FILE *, float *, float *, int, int */);
void report_cluster_trial(/* int, FILE *, float *, float *, int, int */);
void report_map_trial(/* int, FILE *, float *, float *, float * */);
void report_seq_trial(/* int, FILE *, float *, int, int */);
void report_overall(/* int model, FILE * log_file, char *error_fname,
		       char **labels */);
void report_cluster_overall(/* int model, FILE * log_file, char *error_fname,
			       char **labels */);
void report_epoch_header(/* int, FILE *, int_trials */);
void report_epoch(/* int, FILE *, int */);
void report_centers_epoch(/* int , FILE *, int, int */);
void report_artmap_epoch();
void report_cluster_epoch(/* int , FILE *, int, int , float */);
float report_get_avg_cost(/* int */);
float report_get_percent_error(/* int */);
void report_test_result(/*int, FILE *, char * */);
void report_cv_result(/*int, FILE *, char *, int */);
void report_last_epoch(/*int, FILE *, int */);

/******************************************/
/* old style of error reporting functions */
/* should be deleted                      */
/******************************************/
int error_report_between_classes(/* FILE *log_file, int class1, int class2,
				    int verbosity */);
void error_report_build(/* int verbosity, int noutputs */);
void error_report_free(/* void */);
void error_report_init(/* int map_flag, int ninputs */);
void error_report_class_trial(/* FILE *error_file, int trial, int verror,
				 float rms_err, float *inputs, float *outputs,
				 int desired_class, int class */);
void error_report_map_trial(/* FILE *error_file, int trial, int verror,
			    float rms_err, float *inputs, float *outputs,
			    float *desired_outputs */);
void error_report_overall(/* FILE *log_file, char *error_fname,
			     char **labels */);
void error_report_part_build(/* int verbosity */);
void error_report_part_free(/* void */);
void error_report_part_init(/* void */);
void error_report_part(/* FILE *log_file, char *error_fname, char **labels */);
float error_report_calc_rmserr(/* int class, int desired_class,
				  float *outputs */);
float error_report_overall_error_rate(/* void */);
float error_report_overall_rms(/* void */);

/*----------------------------------------------------------------
 * NAME: class_data.h
 * PURPOSE: read classification data file and return patterns to classifier
 *	on request.  Routine handles random selection of patterns,
 *	priors adjustment though pattern selection, and cross validation
 ------------------------------------------------------------------*/
int class_data_read(/* char *, char *, int, int, int, int, int *, int, int */);
int class_data_get_npatterns(/* int */);
void class_data_priors_init(/* int, float *, int */);
void class_data_random_init(/* int */);
void class_data_cv_init(/* int, int, int, int *, int, char **, char ** */);
int class_data_start_cv_fold(/* int, int, int */);
void class_data_end_cv_fold(/* int */);
void class_data_epoch_init(/* int */);
float *class_data_get_next(/* int, int */);
float **class_data_get_input_matrix(/* int */);
void class_data_exit(/* int */);

/*----------------------------------------------------------------
 * NAME: map_data.h
 * PURPOSE: read mapification data file and return patterns to mapifier
 *	on request.  Routine handles random selection of patterns
 *	and cross validation
 ------------------------------------------------------------------*/
int map_data_read(/* char *, char *, int, int, int, int, int *, int, int */);
int map_data_get_npatterns(/* int  */);
void map_data_random_init(/* int  */);
void map_data_cv_init(/* int, int, int, int *, int, char **, char ** */);
int map_data_start_cv_fold(/* int, int, int */);
void map_data_epoch_init(/* int */);
float *map_data_get_next(/* int, float ** */);
void map_data_exit(/*int */);

/*----------------------------------------------------------------
 * NAME: seq_data.h
 * PURPOSE: read classification data file and return patterns to classifier
 *	on request.  Routine handles random selection of patterns
 *	and cross validation
 ------------------------------------------------------------------*/
int seq_data_read(/* char *, char *, int, int, int, int *, int, char *, int
		   */);
int seq_data_get_nfiles(/* int */);
int seq_data_get_nframes(/* int */);
void seq_data_random_init(/* int */);
void seq_data_priors_init();
void seq_data_cv_init(/* int, int, int, int *, int, char **, char ** */);
int seq_data_start_cv_fold(/* int, int, int */);
void seq_data_end_cv_fold();
void seq_data_epoch_init(/* int */);
int seq_data_get_next_file(/* int */);
float *seq_data_get_next_frame(/* int, int * */);
void seq_data_exit(/*int */);

/*----------------------------------------------------------------
 * NAME: main_subs.h
 * PURPOSE: handle things that are done in all LNKnet main routines
 ------------------------------------------------------------------*/
/* data base type numbers for set_desc_args */
typedef enum {
    CLASS_DATA = 0, MAP_DATA = 1, SEQ_DATA = 2
} DATA_BASE_TYPE;
typedef enum {
    CONT = 0, BIN, CAT, CIRC
} INPUT_FEATURE_TYPE;
typedef struct {
    DATA_BASE_TYPE db_type;
    int ninputs;
    int noutputs;
    char **output_labels;
    char **input_labels;
    INPUT_FEATURE_TYPE *input_types;
    float *input_type_values;
} DEFAULTS_FILE_STRUCT;

void set_desc_args(/* char *, char ***, int *, int *, int * */);
void read_defaults_file(/* char *, DEFAULTS_FILE_STRUCT * */);
void free_defaults_file_struct(/* DEFAULTS_FILE_STRUCT * */);
void read_defaults_fp(/* FILE *, DEFAULTS_FILE_STRUCT * */);
void check_desc_args(/* char *, char **, int *, int *, int *, int * */);
void write_defaults_stream(/* FILE *, DEFAULTS_FILE_STRUCT * */);
void write_defaults_file(/* char *, DEFAULTS_FILE_STRUCT * */);
void write_desc_args(/* char *, char **, int, int, int */);
char *get_input_label(/* n, char **, int *, int */);
char *get_class_label(/* n, char ** */);
void initialize_interrupt_handler();
void interrupt_handler(/* int, int, struct sigcontext *, char * */);
char *make_comment_string(/* char * */);
void print_log_footer(/* void */);
void print_log_header(/* void */);
void remove_pid_file(/* char * */);
void store_pid(/* char * */);
void display_model_params(/* int, int, int, int*, int */);
void verify_model_params(/* int, int, int, int*, int* */);
int fix_cross_valid_args(/* int *, int *, int *, int *, char **, char **,
			    int, char * */);
void check_file_length(/* char *, int * */);
void fix_priors(/* int, int, float**, int, int, float**, int, int, int */);
int fix_input_types(/* DEFAULTS_FILE_STRUCT *, int, int, int* */);

/*---------------------- allocate.h ----------------------------*/
char *nn_malloc(/* unsigned n, char *for_message */);
char *nn_realloc(/* char *ptr, unsigned n, char *for_message */);
void nn_free(/* char *p, char *for_message */);
char *nn_strdup(/*char *str */);

float *allocate_vector(/* int n */);
float *reallocate_vector(/* float *v, int n, int newn */);
void deallocate_vector(/* float *v */);
float **allocate_matrix(/* int nrow, int ncol */);
float **reallocate_matrix(/* float **row_pointers, int nrow, int ncol,
			     int newrow */);
float **reallocate_matrix_noncontig(/* float **row_pointers, int nrow,
				       int ncol, int newrow */);
void deallocate_matrix(/* float **m */);

int *allocate_ivector(/* int n */);
int *reallocate_ivector(/* int *v, int n, int newn */);
void deallocate_ivector(/* int *v */);
int *allocate_iindex(/* int n */);
int **allocate_imatrix(/* int nrow, int ncol */);
int **reallocate_imatrix(/* int **row_pointers,
			    int nrow, int ncol, int newrow */);
int **reallocate_imatrix_noncontig(/* int **row_pointers,
				      int nrow, int ncol, int newrow */);
void deallocate_imatrix(/* int **m */);

void **allocate_pvector(/* int n */);
void **reallocate_pvector(/* void **v, int n, int newn */);
void deallocate_pvector(/* void **v */);
void ***allocate_pmatrix(/* int nrow, int ncol */);
void ***reallocate_pmatrix(/* void ***row_pointers,
			      int nrow, int ncol, int newrow */);
void ***reallocate_pmatrix_noncontig(/* void ***row_pointers,
					int nrow, int ncol, int newrow */);
void deallocate_pmatrix(/* void ***m */);

void *allocate_structvector(/* int n, int size */);
void *reallocate_structvector(/* void *v, int n, int newn, int size */);
void deallocate_structvector(/* void *v */);
void deallocate_structmatrix(/* void **m */);
void **allocate_structmatrix(/* int nrow, int ncol, int size */);
void **reallocate_structmatrix(/* void **row_pointers,
				  int nrow, int ncol, int newrow, int size */);
void **reallocate_structmatrix_noncontig(/* void **row_pointers, int nrow,
					    int ncol, int newrow, int size */);

void deallocate_dvector(/* double *v */);
double *reallocate_dvector(/* double *v, int n, int newn */);
double *allocate_dvector(/* int n */);
double **allocate_dmatrix(/* int nrow, int ncol */);
double **reallocate_dmatrix(/* double **row_pointers,
			       int nrow, int ncol, int newrow */);
double **reallocate_dmatrix_noncontig(/* double **row_pointers,
					 int nrow, int ncol, int newrow */);
void deallocate_dmatrix(/* double **m */);

void nr_deallocate_vector(/* float *v, int nlow, int nhigh */);
void nr_deallocate_matrix(/* float **m, int row_low, int row_high,
			     int col_low, int col_high */);
float *nr_allocate_vector(/* int nlow, int nhigh */);
float **nr_allocate_matrix(/* int row_low, int row_high,
			      int col_low, int col_high */);

/*-------------- heap.h -----------------*/
void heap_sort(/* int n, int index[], float xarray[] */);
void heap_init(/* int n, int index[], float xarray[] */);
void reheap(/* int n, int index[], float xarray[] */);
void heap_sort2(/* int n, float xarray[], int *array2 */);
void rev_heap_sort(/* int n, int index[], float xarray[] */);
void rev_heap_init(/* int n, int index[], float xarray[] */);
void rev_reheap(/* int n, int index[], float xarray[] */);
void rev_heap_sort2(/* int n, float xarray[], int *array2 */);

void heap_dsort(/* int n, int index[], double xarray[] */);
void heap_dinit(/* int n, int index[], double xarray[] */);
void dreheap(/* int n, int index[], double xarray[] */);
void rev_heap_dsort(/* int n, int index[], double xarray[] */);
void rev_heap_dinit(/* int n, int index[], double xarray[] */);
void rev_dreheap(/* int n, int index[], double xarray[] */);

void heap_isort(/* int n, int index[], int xarray[] */);
void heap_iinit(/* int n, int index[], int xarray[] */);
void ireheap(/* int n, int index[], int xarray[] */);
void rev_heap_isort(/* int n, int index[], int xarray[] */);
void rev_heap_iinit(/* int n, int index[], int xarray[] */);
void rev_ireheap(/* int n, int index[], int xarray[] */);

/* macros for normalization types */
#define NONE 0
#define SIMPLE 1
#define PCA 2
#define LDA 3

/* norm2c.c */
void get_norm_params(/* int int, int * */);
void print_norm_comments();
void print_normalize_code(/* char * */);
void print_copy_iniputs();
void print_norm_macros(/* char *, char * */);
void clear_norm_macros();

/* norm_subs.c */
void norm_build(/* int ninputs, int nclasses, int normalization */);
void norm_free();
void norm_setup_none(/* int int */);
void norm_inc(/* float *inputs, int class */);
void norm_inc2(/* float *inputs, int class */);
void norm_init(/* int total_trials */);
void *norm_param(/* char *param_name */);
void norm_update(/* void */);
void norm_update2(/* void */);
void norm_apply(/* float *inputs */);
int norm_read_paramfile(/* FILE *fp, int *normalization */);
int norm_write_paramfile(/* FILE *fp */);
int norm_normalization(/* void */);
float * norm_mean(/* void */);
float * norm_sdev(/* void */);
int *norm_calc_needed(/* void */);
void make_transform_matrix(/* int, float **, float **, float *,
			      float, int * */);
char *norm_description(/* int normalization */);
int read_norm_file(/* char *norm_filename */);

/***---------------------------------------------------------------------------
* NAME:		feat_sel.h
* PURPOSE:	function declerations for feat_sel.c
* USAGE: 	#include <nnlib.h>
*--------------------------------------------------------------------------***/
void feat_sel_build(/* int, int, int, int */);
int feat_sel_search_done();
void feat_sel_init_nfeatures();
int *feat_sel_next_features();
void feat_sel_check_best(/* float */);
void feat_sel_pick_best();
void feat_sel_print_head();
void feat_sel_print_final_result();
void feat_sel_print_result(/* int *, float, int */);
void feat_sel_save_list(/* char *, char * */);
void feat_sel_free();

/*  roc_pack.h */
/*  defines objects and operators for class roc_pack */

#define ROC_TINY 1.0e-30

typedef struct{
  int nallocpatterns;
  int npatterns;
  int npatterns_left; /* after rejections */
  int nclasses;
  float **output_val; /* last column is the desired value */

  int class;
  float reject;
  int *reject_flag;

  int table_length;
  int *ninclass;
  int *class_index;
  int *npat;
  int *ndr;
  int *nfa;
  float *threshold;
  float *false_alarm;
  float *detect_rate;
} roc;

int roc_build(/*int, int  */);
void roc_free(/* int  */);
void roc_param(/* int , roc ** */);
void roc_batch_train(/* int , int , float * */);
void roc_batch_compute(/* int , int , float */);
void roc_print_table(/* FILE *,int , int , float , int  */);
void roc_print_sampled_table(/* FILE, int, int, float, int, float */);
float roc_get_percent_fom(/* int, int, float, float, float, float */);
float roc_get_area(/* int, int, float */);

/* macros for the number of an unknown class, used in some
   classifiers and if we ever put in outlyer detection */
#define UNKNOWN_CLASS (-1)

/* stat.h */
float chi_square(/* int, *int, *float, *float, *float */);
float chi_square_siqnificance(/* int, float */);

/* roc_cost.h */
void roc_cost_init(/* int, int */);
float roc_calc_cost(/* float *, float * */);
float roc_calc_cost_vector(/* float *, float *, float * */);
int roc_calc_deriv(/* float *, float *, float * */);
float roc_cost_train(/* float *, float * */);
void roc_cost_finish_epoch();
void roc_cost_free();
void roc_print_values(/* int, FILE * */);
float roc_cost_get_area(/* int */);

/* note.h */
void diff_argv(/* FILE *, int, int, char **, char ** */);
void store_train_note_command(/* FILE *, char *, char * */);
void store_test_note_command(/* FILE *, char *, char * */);
void store_cv_note_command(/* FILE *, char *, char * */);
void store_roc_note_command(/* FILE *, char *, char * */);
void store_chi_note_command(/* FILE *, char *, char * */);
