/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* parzen_pack.h */
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/include/parzen_pack.h,v 11.4 2001/08/23 20:14:37 craig Exp $*/
/* $Source: /data/speech/kukolich/cvsroot/lnknet/src/include/parzen_pack.h,v $*/

#ifndef parzen_pack_h
#define parzen_pack_h

#ifdef parzen_pack_h_internal_types
#undef parzen_pack_h_internal_types

/* Variance parameters */
typedef union {
    float **per_class_input;    /*[noutputs][ninputs]*/
    float *per_input;   /* [ninputs] */
    float *per_class;   /* [noutputs] */
    float grand;
} INVVAR;

typedef union {
    float *per_class;   /*[noutputs]*/
    float grand_class;
} LOGDET;

/* Parzen Window Classifier Model Parameters */
typedef struct {
    int     ninputs;    /* number of elements in patterns */
    int     *input_types;   /* types of inputs */
    float   *input_type_values; /* values for non-CONT/BIN types */
    int     noutputs;   /* number of elements in patterns */
    int     total_trials;
    int     *class_trials;  /* current count of input vectors */
    int     trials_allocd;
    int     *classes;   /* [total_trials] */
    float   **centers;  /* [total_trials][ninputs] */
    int     linear;     /* Kernal type, Step or Gauss*/
    int     per_class;
    int     per_input;
    float   minimum_variance;
    float   *outputs;   /* [noutputs] */
    float   *log_class_priors;  /* [noutputs] */
    float   **train_gauss_mean; /* [noutputs][ninputs] */
    float   **train_gauss_var;  /* [noutputs][ninputs] */
    float   **train_cos_sum;
    float   **train_sin_sum;
    INVVAR  inv_var;
    LOGDET  log_det;
} PARZEN_PARAMS;

#ifdef parzen_pack_h_param_types
/*** Parameter Table ***/
static PARAM   paramtab[] = {
    {"ninputs", P_INT, NULL},
    {"noutputs", P_INT, NULL},
    {"total_trials", P_INT, NULL},
    {"per_input", P_INT, NULL},
    {"per_class", P_INT, NULL},
};
#define  TABSIZE (sizeof( paramtab)/sizeof(PARAM))

static PARAM   paramtab_l[] = {
    {"class_trials", P_INT_VECTOR, NULL, 0, 0},
    {"means", P_FLOAT_MATRIX, NULL, 0, 0},
    {"classes", P_INT_VECTOR, NULL, 0, 0},
    {"input_types", P_INT_VECTOR, NULL, 0, 0},
    {"type_values", P_FLOAT_VECTOR, NULL, 0, 0},
};
#define TABSIZE_L (sizeof(paramtab_l)/sizeof(PARAM))

static PARAM var_matrix[] = {
    {"inv_var", P_FLOAT_MATRIX, NULL, 0, 0},
};

static PARAM var_vector[] = {
    {"inv_var", P_FLOAT_VECTOR, NULL, 0, 0},
};

static PARAM var_scalar[] = {
    {"inv_var", P_FLOAT, NULL, 0, 0},
};

static PARAM det_vector[] = {
    {"log_det", P_FLOAT_VECTOR, NULL, 0, 0},
};

static PARAM det_scalar[] = {
    {"log_det", P_FLOAT, NULL, 0, 0},
};
#define NVAR (1)
#endif
#endif
int parzen_build( int ninputs, int noutputs, int per_input, int per_class,
          INPUT_FEATURE_TYPE *input_types, float *input_type_values);
void parzen_init( int model, float minimum_variance);
void parzen_free( int model);
int parzen_read_model( FILE *fp );
int parzen_write_model( int model, FILE *fp );
void *parzen_param( int model, char *param_name );
void parzen_batch_ctrain( int model, float *inputs, int desired_class);
void parzen_batch_update( int model);
void parzen_test_init( int model, int adjust_priors, float *priors_list,
               float spread, int linear);
void parzen_eval( int model, float *inputs );
int parzen_classify( int model, float *inputs );

#ifdef parzen_pack_h_static_decl
#undef parzen_pack_h_static_decl
static void parzen_set_scalar_params( PARZEN_PARAMS *gpp );
static void parzen_set_vector_params( PARZEN_PARAMS *gpp );
static void parzen_set_var_params( PARZEN_PARAMS *gpp );
static float get_radians( float x, float range );
static float make_0_2PI( float x );
static int inside_circ(float x, float m, float var, float range );
#endif
#endif
