/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* plot_bound_args.h */

/*  Argument Variables for plot_bound program */
/* arguments needed by more than one plot */
#ifdef PLOT_BOUND

#define EXPERIMENT_PATH "experiment/"

static char *plot_exp_path = "./";
static char *plot_error_fname = "mlp.param";
static int plot_n_inputs = 0;
static int plot_n_outputs = 0;
static int no_graphics = 0;
/* I'm trying to see if these can be moved into plot_bound.c */
/*
static char **plot_labels = NULL; 
static char **plot_input_labels = NULL;
*/
#endif
/* arguments for just plot_bound.c */
static int mac_flag = 0;
static char *macdots_fname = "boundary_dots.mac";
static char *macscatter_fname = "scatter.mac";
static char *maclines_fname = "profile_lines.mac";
static char *machisto_fname = "histogram.mac";
static int mac_wireframe_flag = 0;
static int mac_wireframe_output = -1;
static char *macwireframe_fname = "wireframe.mac";
static int bound_autoscale = 0;
static char *bound_plot_fname = "region.plot"; 
static char *profile_plot_fname = "profile.plot"; 
static float bound_xmin = -3.0;
static float bound_xmax = 3.0;
static float bound_xstep = 1.0;
static float bound_ymin = -3.0;
static float bound_ymax = 3.0;
static float bound_ystep = 1.0;
static float profile_ymin = 0.;
static float profile_ymax = 1.5;
static float profile_ystep = .25;
static float distance_limit = .5;
static float internals_scale = 1.;
static char *bound_plot_label = "Decision Region Plot";
static char *profile_plot_label = "Profile Plot";
static int bound_dim_1 = 0;
static int bound_dim_2 = 1;
static int bound_npoints = 50;
static int internals_level = 1;
static int region_plot = 0;
static int scatter_plot = 0;
static int internals_plot = 0;
static int profile_plot = 0;
static int histogram_plot = 0;
static int show_all = 0;
static int show_color = 0;
static int show_misses = 0;
static int scatter_n_patterns = 0;
static char *scatter_data_fullname = "";
static char *scatter_desc_fullname = "";
static int plot_adjust_priors = 0;
static int plot_uniform_priors = 0;
static float plot_priors_list[NNARGS_MAX_LABELS] = {0.};
static int plot_map = 0;
static int no_norm = 0;
static float other_vars[NNARGS_MAX_LABELS] = {0.};

static ARG bound_argtab[]=
{
  {"autoscale", BOOLEAN, &bound_autoscale,
	"0 - supplied axes limits; 1 - autoscale both axes"},
  {"pathexp", STRING, (int *) &plot_exp_path, "path to experiment files"},
  {"fparam", STRING, (int *) &plot_error_fname, "name of parameter file"},
  {"fregion", STRING, (int *) &bound_plot_fname, "name of region plot file"},
  {"mac", BOOLEAN, &mac_flag, "create data files for Macintosh plotting"},
  {"mac_wireframe", BOOLEAN, &mac_wireframe_flag, "create data file for 3D discrim function wireframe plots"},
  {"mac_wireframe_output", INT, &mac_wireframe_output, "-1=max output, 0 to noutputs-1=only that output for 3D discrim function wireframe plots"},
  {"fmacdots", STRING, (int *)&macdots_fname,
     "Macintosh boundary dots data file name"},
  {"fmacscatter", STRING, (int *)&macscatter_fname,
     "Macintosh scatter plot data file name"},
  {"fmaclines", STRING, (int *)&maclines_fname,
     "Macintosh profile plot data file name"},
  {"fmachisto", STRING, (int *)&machisto_fname,
     "Macintosh histogram plot data file name"},
  {"fmacwireframe", STRING, (int *)&macwireframe_fname,
     "Macintosh 3D wireframe discrim function data file name"},
  {"fprofile", STRING, (int *) &profile_plot_fname,"name of profile plot file"},
  {"xmin", FLOAT, (int *) &bound_xmin, "lower x axis limit"},
  {"xmax", FLOAT, (int *) &bound_xmax, "upper x axis limit"},
  {"ymin", FLOAT, (int *) &bound_ymin, "lower y axis limit"},
  {"ymax", FLOAT, (int *) &bound_ymax, "upper y axis limit"},
  {"xstep", FLOAT, (int *) &bound_xstep, "step for x axis"},
  {"ystep", FLOAT, (int *) &bound_ystep, "step for y axis"},
  {"pymin", FLOAT, (int *) &profile_ymin, "lower y axis limit for profile"},
  {"pymax", FLOAT, (int *) &profile_ymax, "upper y axis limit for profile"},
  {"pystep", FLOAT, (int *) &profile_ystep, "y axis step for profile"},
  {"ninputs", INT, &plot_n_inputs,"number of dimensions to use in data"},
  {"noutputs", INT, &plot_n_outputs,"use nclasses instead"},
  {"tregion", STRING, (int *)&bound_plot_label, "region plot title"},
  {"tprofile", STRING, (int *)&profile_plot_label, "profile plot title"},
  {"first_dim", INT, &bound_dim_1, "first dimension from input vector"},
  {"second_dim", INT, &bound_dim_2, "second dimension from input vector"},
  {"other_vars", FLOAT_LIST, (int *)&other_vars[0],
     "settings for all dimensions (default: unplotted dimensions = 0)"},
  {"npatterns", INT, &scatter_n_patterns,
     "number of patterns for scatter plot"},
  {"npoints", INT, &bound_npoints, "points per dimension in grid"},
  {"region", BOOLEAN, &region_plot, "Plot boundary regions if on"},
  {"scatter", BOOLEAN, &scatter_plot, "Create scatter plot if on"},
  {"internals", BOOLEAN, &internals_plot,
     "Plot internal classifier parameters if on"},
  {"internals_scale", FLOAT, (int *)&internals_scale,
     "Scale factor for internal classifier parameters"},
  {"internals_level", INT, &internals_level,
     "Level of detail for internal classifier parameters"},
  {"profile", BOOLEAN, &profile_plot, "Plot boundary regions if on"},
  {"histogram", BOOLEAN, &histogram_plot, "Create scatter plot if on"},
  {"color", BOOLEAN, &show_color, "Use color to show decision regions"},
  {"misses", BOOLEAN, &show_misses, "Highlight missclassified points"},
  {"all", BOOLEAN, &show_all, "Show all points (no maximum distance)"},
  {"distance", FLOAT, (int *) &distance_limit,
     "Show points at this distance limit"},
  {"fdata", STRING, (int *) &scatter_data_fullname,
     "full path name of data file  for scatter plot"},
  {"fdesc", STRING, (int *)&scatter_desc_fullname,
     "full path name of description file for data base"},
  {"map", BOOLEAN, &plot_map, "scatter data is for mapping"},
  {"adjust_priors", BOOLEAN, &plot_adjust_priors,
     "adjust outputs to reflect class probabilities in priors_list"},
  {"uniform_priors", BOOLEAN, &plot_uniform_priors,
     "if adjusting priors, make class probabilities uniform"},
  {"priors_list", FLOAT_LIST, (int *)plot_priors_list,
     "adjustments to apriori class probabilities"},
  {"no_graphics", BOOLEAN, &no_graphics,
     "create plot file, but do not display it"},
  {"no_norm", BOOLEAN, &no_norm, "plot un-normalized data"},
};
#define BOUND_TABSIZE (sizeof(bound_argtab)/sizeof(ARG))

