/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* rbf_pack.h */

#ifndef rbf_pack_h
#define rbf_pack_h

#ifdef rbf_pack_h_internal_types
#undef rbf_pack_h_internal_types

/* Define structure of the rbf_model */
typedef struct {
    int ninputs;	/* dimension of each input vector */
    int ngauss_hidden;	/* number of standard gaussian hidden
			 * nodes */
    int bias;	/* number of bias nodes, (0 or 1) */
    int nhidden;	/* total number of hidden rbf nodes */
    int nclasses;	/* number of output classes or nodes */
    float hspread;	/* factor to multiply all spreads by */
    float extra_hspread;/* additional hspread for testing */
    float *hidden_out;	/* vector for hidden outputs */
    float *outputs;	/* vector for classifier outputs */
    float **centroids;	/* matrix of centroids */
    float **spreads;	/* matrix of spreads or variances */
    float **weights;	/* matrix of weights */
    float **correl_matrix;	/* temporary correlation matrix */
    float **partial_weight_matrix;	/* temporary partial weight
					 * matrix */
    int *features;	/* features of raw data used */
    float *class_cnt;	/* number of samples seen in each class */
    float total_npatterns;/*total number of samples seen */
    int adjust_priors;	/* whether to adjust testing priors */
    float *priors_list;
    int norm_outputs;
    int fast_train;
    int fast_nhidden;
    int *fast_index;
    float *fast_outputs;
} RBF_PARAMS;
#endif

extern int rbf_build(int ninputs, int ngauss_hidden, int bias, int nclasses);
extern void rbf_init(int model_num, float **pcentroids, float **pspreads,
		     float hspread, int xverbose, int *features,
		     float minimum_variance, int norm_outputs);
extern void rbf_test_init(int model_num, int adjust_priors,
			  float *priors_list, float extra_hspread);
extern void rbf_init_fast_train(int model_num, int fast_nhidden);
extern void rbf_free(int model_num);
extern int rbf_read_model(FILE *fp);
extern void rbf_write_model(int model_num, FILE *fp);
extern void *rbf_param(int model_num, char *param_name);
extern void rbf_batch_ctrain(int model_num, float *invec, int class);
extern void rbf_batch_train(int model_num, float *invec, float *desired);
extern void rbf_batch_update(int model_num, float max_ratio);
extern void rbf_eval(int model_num, float *invec);
extern int rbf_classify(int model_num, float *invec);
extern void rbf_hidden_eval(int model_num, float *invec);
extern void rbf_fast_hidden_eval(int model_num, float *invec);
extern void rbf_gen_probability(int model_num, float *invec,
				float extra_hspread);
extern void rbf_write_paramfile(int model_num, FILE *fp);
extern void rbf_read_paramfile(FILE *fp);
extern void print_centers_spreads(int model_num);
extern void rbf_read_cluster(char *param_path, FILE *param_file,
			    int *pninputs, int *pncenters,
			    float ***centroids, float ***spreads,
			    int *features);
#endif
