/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* svm_pack.h */

#ifndef svm_pack_h
#define svm_pack_h

/* DECLARATIONS c */
#define NOT_A_SUPPORT_VECTOR (-1)
/* describes alpha value for each pattern */
#define ZERO 0
#define UNBOUNDED 1
#define CBOUND 10
/* describes types of kernels */
#define SMO_LINEAR 0
#define SMO_GAUSSIAN 1
#define SMO_POLYNOMIAL 2
#define SMO_INHOMOGENEOUS_POLYNOMIAL 3
#define SMO_LINEAR_USE_SVS 4
#define MAX_SWEEPS 10000000


/* DEFINITIONS OF INTERNAL STRUCTURES AND TYPES */
#ifdef svm_pack_h_internal_types
#undef svm_pack_h_internal_types
/* svm_pack_h_internal_types is defined in `svm_pack.c' */
/* Define structure of the svm_model */
typedef struct {
    /* general model features */
    int         ninputs;        /* number of elements in patterns */
    int         noutputs;       /* number of elements in patterns */
    int 	verbosity;	/* for internal printouts during training */
    float       *outputs;       /* space for output values used by error
                                 * report functions */
    float 	**exemplars;	/* all training patterns */
	                        /* used to access sv patterns during train */
    int		*y;	        /* desired_outputs for all training patterns */
    float       *class_cnt;     /* number of samples seen in each class */
    int         npatterns;      /* number training patterns */

    /* algorith parameters */
    float cbound;	        /* upper bound on alpha for slack variables */
    int kernel_type;	        /* 0=lin,1=gauss,2=(x.x)^,3=(x.x+1)^p */
    float gaussian_sigma;	/* for Gaussian kernel */
    float gaussian_variance_times_two; /* calculated from gaussian_sigma */
    int polynomial_order;	/* for polynomial kernel, must be integer */
    float polynomial_divisor;	/* scale poly dot prod */
    float inv_poly_divisor;	/* computed from above */
    int sigmoid_fit;    /* flag to output approx post prob */
    float kkt_tolerance;       /* tolerance for kkt conditions out=+/-1?*/
    float final_check_tolerance;    /* twice kkt_tolerance */
   float epsilon_rounding;		/* for checking progress */
    float alpha_tolerance;		/* clamp sv's lower than this to zero*/
    float posterior_aexp;            /* post = 1/(1+exp(ax)) this is the a */
    int multi_class_type;	   /* 0=1 vs others, 1=all pairs */
    int multi_class_a;	   	   /* original index for class A */
    int multi_class_b;	   	   /* original index for class B */
    int multi_class_i;		   /* index for this binary classifier */


    /* counters */
    int nsweeps;			/* number of sweeps through all data */
    int total_pair_corrections;         /* total two-pattern pairs adapted */
    int total_kernel_evaluations;     /* times kernel functions evaluated */

    /* internal vectors kept for all training patterns */
    char *alpha_type;		/* alpha values for all training patterns */
    float *fcache_value;	/* cached values of F (output without bias) */
    int *fcache_flag;		/* flag to indicate Fcache value is good */ 
    int *pattern2sv;		/* translates from pattern index to sv */
    float *alpha;		/* alpha value for all training patterns */

    /* static variables */
    int sv_n;		/* number of support vectors */
    int n_positive_below_bound; /* number of sv's >0 and < C */
    int n_zero;		/* number training patterns  alpha = 0 */
    int n_at_cbound;	/* number of sv's at C */
    float threshold;  /* bias or threshold, valid after convergence */
    int verbose_flag; /* turn on an up  informative printout */
    int warning_flag;	/* turn on common warnings */
    float b_high;	/* upper bound for b or threshold */
    int i_high;
    int i_high_desired_output;
    float b_low;	/* lower bound for b or threshold */
    int i_low;
    int i_low_desired_output;

    /* vectors as long as the number of inputs )*/
    float *i_low_vector; /* store current low bias vector */
    float *i_high_vector; /* store curren high bias vector */
    float *v1_current;    /* store i1 vector */
    float *v2_current;    /* store i2 vector */
    float *weight;	/* weight vector for linear svm  */

    /* Kept for currently positive alpha support vectors.
    During training, the support_vectors matrix is empty
    and the exemplars matrix is used to access patterns
    for support vectors. After training is complete, the
    support vectors are filled in and used.
    This is indicated by the support_vectors_valid.
    Memory could be saved here by allocating more memory
    as the number of support vector grows, but it doesnt
    matter for long input vectors (>10) and for short input
    vectors it doesnt matter much either. 
    -----------------------------------------------------*/
    float *sv_alpha;	/* alpha value for internal support vectors */
    int *sv_y;		/* desired output for internal support vectors */
    char *sv_alpha_type; /* type for each support vector */
    int *sv_unique;       /* whether sv is not used in other component classifiers */
    int *sv2pattern;	/* translate internal sv index to pattern index */
    int support_vectors_valid;  /* indicates following matrix is ok */
    float  **support_vectors;	/* allocated and filled after training over */

} SVM_PARAMS;
#endif

/* Define structure of multi-class data */
typedef struct {
    /* general model features */
    int ninputs;   /* number input features */
    int noutputs;  /* number of classes */
    int type;	   /* 0=1 vs others, 1=all pairs */
    int n;	   /* number of classifiers */
    int sv_n;      /* total number of support vectors in component classifiers */
    int n_positive_below_bound; /* total number of sv's >0 and < C */
    int n_at_cbound;	/* total number of sv's at C */
    int sv_unique; /* total number of unique support vectors */
   int *component_model;   /* index for component model when created */
    float **component_outputs; /* two outputs for each component classifier */
    int *component_decisions; /* class 0=1, class 1=0 */
    int *votes;	   /* number of votes for each class */
    float *class_min; /* minimum output across combinatorial comparisons */
    int   adjust_priors;  /* flag for test priors adjustment */
    float *priors_list;   /* when adjusting priors in testing */
    int sigmoid_fit;    /* use sigmoid when testing */
    float *outputs;	/* used to pass results back to error routines */
    float   *class_cnt;     /* number of samples seen in each class */
  float npatterns;	/* total number of patterns */
} SVM_MULTI;

/*** parameter table ***/
/*
static PARAM    svmscalars[] = {
    {"ninputs", P_INT},
    {"nclasses", P_INT},
    {"cbound", P_FLOAT},        
    {"kernel_type", P_INT},
    {"gaussian_sigma", P_FLOAT},
    {"polynomial_order", P_INT},
    {"polynomial_divisor", P_FLOAT},
    {"kkt_tolerance", P_FLOAT},
    {"alpha_tolerance",P_FLOAT},
    {"posterior_aexp",P_FLOAT},
    {"nsweeps", P_INT},
    {"total_pair_corrections",P_INT},
    {"total_kernel_evaluations", P_INT},
    {"sv_n", P_INT},
    {"n_positive_below_bound", P_INT},
    {"n_zero", P_INT},
    {"n_at_cbound", P_INT},
    {"threshold", P_FLOAT},
    {"b_high", P_FLOAT},
    {"b_low", P_FLOAT},
};
static PARAM    svmsv_matrix[] = {
    {"sv_vector" , P_FLOAT_MATRIX, NULL, 0, 0},
};
static PARAM svmvectors[]={
    {"sv_alpha", P_FLOAT_VECTOR, NULL, 0, 0},
    {"sv_y", P_INT_VECTOR, NULL, 0, 0},
    {"sv_alpha_type", P_STRING_VECTOR, NULL, 0, 0},
    {"sv_unique", P_STRING_VECTOR, NULL, 0, 0},
    {"sv2pattern", P_INT_VECTOR, NULL, 0, 0},

};
#define NSVMSCALARS (sizeof(svmscalars)/sizeof(PARAM))
#define NSVMVECTORS (sizeof(svmvectors)/sizeof(PARAM))
#define NSVMMATRICES (1)
*/



/* DECLARATION OF EXTERNAL FUNCTIONS */
int 
svm_train_multi (int nraw, int ninputs, int noutputs,
                int normalize, int npatterns, int verbosity,
                char *data_path, char *data_fname, char *norm_fname,
                char *exp_path, char *param_fname, char *comment, char **labels,
                float cbound, int kernel_type, float kernel_sigma,
                int kernel_order, float kernel_poly_divisor,
                int sigmoid_fit,
                float kkt_tolerance, float alpha_tolerance,
                int multi_class_type, 
                int data_model, int param_file_flag, FILE *param_file, 
                char *tmp_path, int cv_flag, int cv_fold);
int svm_build (int ninputs, int nclasses, int npatterns);
int svm_build_multi (int ninputs, int noutputs, int type, 
                     int n_classifiers_to_build, 
                     int n_classifiers_that_will_be_trained);
void svm_init(int model, float cbound, int kernel_type, float kernel_sigma, 
              int kernel_order, float kernel_poly_divisor, 
              int sigmoid_fit,
              float kkt_tolerance, float alpha_tolerance, 
              int verbosity,
              int multi_class_type, 
              int multi_class_a, int multi_class_b, int multi_class_i);
void svm_run_training (int model, int verbosity, char *run_name, char **labels);
int svm_classify(int model, float *xinput);
int svm_classify_multi(int multi_model, float *xinput);
void svm_eval (int model, float *invec);
void svm_test_init_multi (int model, int adjust_priors, float *priors_list,
                    int sigmoid_fit);

void svm_free (int model);
void svm_free_multi (int model);
int svm_read_model (FILE *fp);
int svm_read_model_multi (FILE *fp, int verbosity);
int svm_write_model (int model, FILE *fp);
int svm_write_model_multi (int model, FILE *fp);
void svm_read_model_multi_end(int model, FILE *fp);
void *svm_param_multi (int multi_model, char *param_name);
void *svm_param (int model, char *param_name);
int svm_get_nsv(int model);
int svm_get_alpha_type(int model, int i);
int svm_get_nsv_pos_below_bound(int model);
int svm_get_nsv_at_bound(int model);
float svm_get_sv_pos_below_bound(int model, int i, int j);
float svm_get_sv_at_bound(int model, int i, int j);
void svm_store_inputs (int model, float *inputs, int desired_class, int i);
void svm_print_multi(int model, int vote_flag);
float *svm_get_outputs_multi(int model);
void svm_set_sv_multi(int model, int sv_n, int n_positive_below_bound, 
                       int n_at_cbound, int sv_unique);
void svm_set_sv_unique(int model, int i);
int svm_get_sv_unique(int model, int i);
void svm_set_component_model_multi(int multi_model, int model, int i);
void svm_set_class_cnt_multi(int multi_model, float count, int i);





#ifdef svm_pack_h_statics
#undef svm_pack_h_statics

/* internal functions 
-----------------------*/

/* fcache routines */
static void svm_init_fcache(SVM_PARAMS *svm, int ntrain);
static void svm_set_fcache(SVM_PARAMS *svm, int i, float x);
static float svm_get_fcache(SVM_PARAMS *svm, int i, int desired_output, float *p1);
static void svm_invalidate_fcache(SVM_PARAMS *svm, int i);
static void svm_update_fcache_positive_below_bound(SVM_PARAMS *svm, int i1, int i2, 
                   float a1old, float a1new, float a2old, float a2new, 
                   int y1, int y2, float *p1, float *p2);

/* translate indices */
static int svm_fsv2pattern( SVM_PARAMS *svm, int i);
static int svm_fpattern2sv( SVM_PARAMS *svm, int i);

/* compute kernel and other functions */
static float svm_compute_kernel(SVM_PARAMS *svm, float *p1, float *p2);
static float svm_compute_no_bias_output(SVM_PARAMS *svm, float *pinput);
static float svm_compute_output(SVM_PARAMS *svm, float *pinput);
static void svm_find_ab_patterns(SVM_PARAMS *svm, int ntrain);


static int svm_examine_example(SVM_PARAMS *svm, int i, int desired_out, float *input_vector);
static int svm_take_step (SVM_PARAMS *svm, int i1, int i2, int y1, int y2, 
                          float *p1, float *p2);
static void svm_compute_bounds(float y1, float y2, float alpha1, 
                               float alpha2, float *lbound, 
                               float *hbound, float *l1bound, 
                               float *h1bound, float cbound);
static void svm_print_positive_alphas(SVM_PARAMS *svm, int verbose_flag);
static void svm_update_alpha(SVM_PARAMS *svm, int i1, float new, float old, int desired_ouput, 
                             float *pinput);
static int svm_sv_add(SVM_PARAMS *svm, int i, float a, int desired_output, float *pinput, 
                      int atype);
static void svm_sv_update(SVM_PARAMS *svm, int i_sv, float alpha, int atype);
static void svm_sv_remove(SVM_PARAMS *svm, int i);
static void svm_low_update(SVM_PARAMS *svm, float f2, int i2, int y2, float *pinput);
static void svm_high_update(SVM_PARAMS *svm, float f2, int i2, int y2, float *pinput);
static void svm_update_pair_corrections(SVM_PARAMS *svm);
static void svm_print_model_parameters(int model);
static void svm_update_bias_bounds(SVM_PARAMS *svm, int i1, int i2, int y1, int y2, 
                                   float *p1, float *p2);
static void svm_print_summary_values(SVM_PARAMS *svm, int nsweeps, int n_changed, 
                                     int print_alpha_flag , int verbosity);
static void svm_fit_posterior_sigmoid(SVM_PARAMS *svm);
static void svm_print_summary_header(int verbosity);
static void svm_check_final_kkt_conditions(SVM_PARAMS *svm, int verbosity, char *run_name, char **labels, int bias_different_flag);
static void svm_save_final_support_vectors(SVM_PARAMS *svm);
static void svm_free_train(int model);
void svm_free_multi_train (int model);
#endif
#endif
