/***--------------------------------------------------------------------------
* NAME:		LNK2gobi.c
* PURPOSE:	reads LNKnet data and defaults file,
*		creates an xgobi col and color file
* USAGE:	LNK2gobi pbvowel.train pbvowel.defaults
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/lib/LNK2gobi.c,v 11.4 2001/08/23 20:14:41 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/lib/LNK2gobi.c,v $
* CREATED: 22-JUL-94 lck			MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/lib/LNK2gobi.c,v 11.4 2001/08/23 20:14:41 craig Exp $";


				/* Include necessary header files */
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include "nnlib.h"

static char *data_fname = "";
static char *desc_fname = "";
static char column_ext[] = ".col";
static char colors_ext[] = ".colors";
static char row_ext[] = ".row";
static char glyphs_ext[] = ".glyphs";

static ARG LNK2gobi_argtab[]=
{
    {"fdata", STRING, (int *)&data_fname, "name of file to be converted"},
    {"fdesc", STRING, (int *)&desc_fname,
       "name of data base description file for fdata"},
};
#define LNK2GOBI_TABSIZE (sizeof(LNK2gobi_argtab)/sizeof(ARG))

static char *gobi_color_table[] = {
    "Red", "Yellow", "Orchid", "SkyBlue", "Orange",
    "Green", "SlateBlue", "Peru", "HotPink", "YellowGreen"};
#define N_COLORS (sizeof(gobi_color_table)/sizeof(char *))

static int gobi_glyph_table[] = {
13, 3, 8, 23, 18, 28};
#define N_GLYPHS (sizeof (gobi_glyph_table)/sizeof(int))

int
main(register int argc, register char **argv)
{
    DEFAULTS_FILE_STRUCT df_args;
    int i, nlabels, desired_class;
    FILE *colors_fp, *columns_fp, *data_fp, *row_fp, *glyphs_fp;
    char *columns_fname, *colors_fname, *row_fname, *glyphs_fname;
    float *inputs;
				/* Process the command line arguments */
    init_nnsubs(argc, argv, rcsid);
    argc = nnargs(argc, argv, LNK2gobi_argtab, LNK2GOBI_TABSIZE);

    if(strcmp(data_fname, "") == 0)
      fatal_error("An input data file name must be specified");
    if(strcmp(desc_fname, "") == 0)
      fatal_error("A data base description file name must be specified");

    /* read defaults file to get labels and number of inputs and classes */
    read_defaults_file(desc_fname, &df_args);
    if(df_args.db_type != CLASS_DATA)
      fatal_error("%s only works on LNKnet classification data bases",
		  argv[0]);

    inputs = allocate_vector(df_args.ninputs);

    /* write the columns file */
    if(df_args.input_labels != NULL){
	nlabels = atoi(df_args.input_labels[0]);
	if(nlabels != 0){
	    columns_fname = nn_malloc(strlen(data_fname) +
				      strlen(column_ext) + 1,
				      "for columns_fname");
	    strcpy(columns_fname, data_fname);
	    strcat(columns_fname, column_ext);
	    columns_fp = nn_fopen(columns_fname, "w",
				  "for write of columns file");
	    fprintf(columns_fp, "class\n");
	    for(i = 1; i <= nlabels; i++){
		fprintf(columns_fp, "%s\n", df_args.input_labels[i]);
	    }
	    nn_fclose(columns_fp, columns_fname, "for write of columns file");
	    nn_free(columns_fname, "for columns_fname");
	}
    }
	
    /* write the colors file */
    colors_fname = nn_malloc(strlen(data_fname) +
			      strlen(colors_ext) + 1, "for colors_fname");
    strcpy(colors_fname, data_fname);
    strcat(colors_fname, colors_ext);
    colors_fp = nn_fopen(colors_fname, "w", "for write of colors file");
    data_fp = nn_fopen(data_fname, "r", "for read of data file");
    while(1){
	desired_class =
	  get_class_line (data_fp, data_fname, inputs,
			  df_args.ninputs, 0, NULL, df_args.ninputs);
	if(desired_class == EOF) break;
	fprintf(colors_fp, "%s\n", gobi_color_table[desired_class%N_COLORS]);
    }
    nn_fclose(colors_fp, colors_fname, "for write of colors file");
    nn_fclose(data_fp, data_fname, "for read of data file");
    nn_free(colors_fname, "for colors_fname");

    /* write the row file */
    if (df_args.output_labels != NULL){
	nlabels = atoi(df_args.output_labels[0]);
	if(nlabels != 0){
	    row_fname = nn_malloc(strlen(data_fname) +
				  strlen(row_ext) + 1, "for row_fname");
	    strcpy(row_fname, data_fname);
	    strcat(row_fname, row_ext);
	    row_fp = nn_fopen(row_fname, "w", "for write of row file");

	    data_fp = nn_fopen(data_fname, "r", "for read of data file");
	    while(1){
		desired_class =
		  get_class_line (data_fp, data_fname, inputs,
				  df_args.ninputs, 0, NULL, df_args.ninputs);
		if(desired_class == EOF) break;
		fprintf(row_fp, "%s\n", df_args.output_labels[desired_class+1]);
	    }
	    nn_fclose(row_fp, row_fname, "for write of row file");
	    nn_fclose(data_fp, data_fname, "for read of data file");
	    nn_free(row_fname, "for row_fname");
	}
    }

    /* write the glyphs file */
    glyphs_fname = nn_malloc(strlen(data_fname) +
			      strlen(glyphs_ext) + 1, "for glyphs_fname");
    strcpy(glyphs_fname, data_fname);
    strcat(glyphs_fname, glyphs_ext);
    glyphs_fp = nn_fopen(glyphs_fname, "w", "for write of glyphs file");
    data_fp = nn_fopen(data_fname, "r", "for read of data file");
    while(1){
	desired_class =
	  get_class_line (data_fp, data_fname, inputs,
			  df_args.ninputs, 0, NULL, df_args.ninputs);
	if(desired_class == EOF) break;
	fprintf(glyphs_fp, "%d\n", gobi_glyph_table[desired_class%N_GLYPHS]);
    }
    nn_fclose(glyphs_fp, glyphs_fname, "for write of glyphs file");
    nn_fclose(data_fp, data_fname, "for read of data file");
    nn_free(glyphs_fname, "for glyphs_fname");
    
    exit_nnsubs();
    exit(0);
}
