/***---------------------------------------------------------------------------
* NAME:		files.c 
* PURPOSE:	open and close files with error checking
* DESCRIPTION:
* nn_fopen 	- open a file with error checking 
* nn_fclose	- close a file with error checking 

* $Header: /data/speech/kukolich/cvsroot/lnknet/src/lib/files.c,v 11.3 2001/08/23 20:14:43 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/lib/files.c,v $
* CREATED: 26-July-90 rpl                             MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */


/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include "nnlib.h"

/***---------------------------------------------------------------------------
* NAME:		nn_fopen
* PURPOSE:	open a file with error checking 
* EXAMPLE:
*	FILE *p;
*	p = nn_fopen("eset.trainlist", "r", "for reading training list file" );
* NOTE:
*     The third argument is used in an error message if the fopen() fails.
*     It should describe what the file is for and begin with "for ...".
--------------------------------------------------------------------------***/
FILE *nn_fopen(char *fname,/* file name */
	       char *type,/* r, w, a, r+, w+, a+ */
	       char *for_message/* what file is for "for ...." */)
{
    FILE *p;

    p = fopen(fname, type);
    if ( p == NULL)
	fatal_error("nn_fopen: Can't open file %s %s.\n",fname, for_message);

    return(p);
}

/***---------------------------------------------------------------------------
* NAME:		nn_fclose
* PURPOSE:	close a file with error checking 
* EXAMPLE:
*
*  FILE *p;
*  p = nn_fopen("eset.trainlist", "r","for reading training data list file" );
*  nn_fclose(p, "eset.trainlist", "for reading training data list file" );
* NOTE:
*     The third argument is used in an error message if the fclose() fails.
*     It should describe what the file is for and begin with "for ...".
--------------------------------------------------------------------------***/
void nn_fclose(FILE *ptr,/* file pointer from nn_fopen or fopen */
	       char *filename,/* file name */
	       char *for_message/* what file is for "for ...." */)
{
    int i;

    i = fclose(ptr);
    if ( i != 0)
	warning( "nn_fclose: Can't close file %s which was opened %s.\n",
		      filename, for_message);
}

