/***---------------------------------------------------------------------------
* NAME:        main_subs.c
* PURPOSE:     Subroutines used by algorithm main programs
*
* USAGE:
*
* SEE ALSO:    mlp_main.c, rbf_main.c, kmeans_main.c, knn_main.c, etc.  
*
* DESCRIPTION: 
*		check_desc_args
*		make_comment_string	
*		print_log_footer
*		print_log_header
*		remove_pid_file
*		store_pid
*		display_model_params
*		verify_model_params
*		
* BUGS AND LIMITATIONS:
*
--------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */



/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STRINGS_H
#include <strings.h>
#endif
#include "nnlib.h"
/* for localtime */
#include <time.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#include "gnn_args.h"
#include "def_args.h"
#include "cv_args.h"

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#define MAXSTR (256)

/* uses new style defaults files */
void
set_desc_args(char *desc_fullname, char ***plabels, int *pnraw,
	      int *pnoutputs, int *pdb_type)
{
    DEFAULTS_FILE_STRUCT df_args;

    read_defaults_file(desc_fullname, &df_args);

    *plabels = df_args.output_labels;
    /* input labels pointer is lost */
    *pnraw = df_args.ninputs;
    *pnoutputs = df_args.noutputs;
    *pdb_type = df_args.db_type;
}

/* uses new style defaults files */
void
read_defaults_file(char *desc_fullname, DEFAULTS_FILE_STRUCT *defaults_args)
{
    FILE *desc_file;

    /* get file description arguments if present */
    desc_file = nn_fopen(desc_fullname, "r", "for data base description file");
    read_defaults_fp(desc_file, defaults_args);
    nn_fclose(desc_file, desc_fullname, "for reading data description");
}

/* uses new style defaults files */
void
read_defaults_fp(FILE *desc_file, DEFAULTS_FILE_STRUCT *defaults_args)
{
    int i, j, nlabels;

    /* clear old values for arguments,
       in case not all are present in the current file */
    def_map_flag = def_class_flag = def_seq_flag = 0;
    def_ninputs = def_noutputs = 0;
    def_npatterns = 0;
    if(def_labels != NULL){
	if(def_labels[0] != NULL)
	  deallocate_string_vector(def_labels, atoi(def_labels[0])+1);
	def_labels = NULL;
    }
    if(def_input_labels != NULL){
	if(def_input_labels[0] != NULL)
	  deallocate_string_vector(def_input_labels,
				   atoi(def_input_labels[0])+1);
	def_input_labels = NULL;
    }
    if(def_input_types != NULL){
	if(def_input_types[0] != NULL)
	  deallocate_string_vector(def_input_types,
				   atoi(def_input_types[0])+1);
	def_input_types = NULL;
    }
    if(def_categories != NULL){
      deallocate_ivector(def_categories);
      def_categories = NULL;
    }

    /* get file description arguments if present */
    i = nnargs_from_file(desc_file, "defaults file", def_argtab, DEF_TABSIZE);

    /* if there was no label list in the defaults file, set up one with
       length 0 */
    if(def_labels == NULL){
	def_labels = (char **)allocate_pvector(1);
	def_labels[0] = nn_malloc(2, "for label length string");
	strcpy(def_labels[0], "0");
    }
    if(def_input_labels == NULL){
	def_input_labels = (char **)allocate_pvector(1);
	def_input_labels[0] = nn_malloc(2, "for input label length string");
	strcpy(def_input_labels[0], "0");
    }
    if(def_input_types == NULL){
	def_input_types = (char **)allocate_pvector(1);
	def_input_types = (char **) nn_malloc(2, "for input types length string");
	strcpy(def_input_types[0], "0");
    }

    /* copy the output label list */
    nlabels = atoi(def_labels[0]);
    if(nlabels > 0){
	defaults_args->output_labels = (char **)allocate_pvector(nlabels+1);
	for (i = 0; i <= nlabels; i++) {
	    defaults_args->output_labels[i] =
	      nn_malloc(strlen(def_labels[i])+1,"for label string");
	    strcpy(defaults_args->output_labels[i], def_labels[i]);
	}
    } else {
	defaults_args->output_labels = NULL;
    }

    /* copy the input label list */
    nlabels = atoi(def_input_labels[0]);
    if(nlabels > 0){
	defaults_args->input_labels = (char **)allocate_pvector(nlabels+1);
	for (i = 0; i <= nlabels; i++) {
	    defaults_args->input_labels[i] =
	      nn_malloc(strlen(def_input_labels[i])+1,"for label string");
	    strcpy(defaults_args->input_labels[i], def_input_labels[i]);
	}
    } else {
	defaults_args->input_labels = NULL;
    }

    /* copy ninputs and noutputs */
    defaults_args->ninputs = def_ninputs;
    defaults_args->noutputs = def_noutputs;

    /* figure out the data base type, class is the default */
    if(def_class_flag) defaults_args->db_type = CLASS_DATA;
    else if (def_map_flag) defaults_args->db_type = MAP_DATA;
    else if(def_seq_flag) defaults_args->db_type = SEQ_DATA;
    else defaults_args->db_type = CLASS_DATA;

    /* parse the input types */
    defaults_args->input_types =
      (INPUT_FEATURE_TYPE *)allocate_ivector(def_ninputs+1);
    defaults_args->input_types[0] = def_ninputs;
    defaults_args->input_type_values = allocate_vector(def_ninputs+1);
    defaults_args->input_type_values[0] = def_ninputs;
    for(i = 1; i <= def_ninputs; i++){
	/* initialize everything to CONT and value 0 */
	defaults_args->input_types[i] = CONT;
	defaults_args->input_type_values[i] = 0;
    }
    /* copy the category groupings */
    if(def_categories){
      defaults_args->categories = allocate_ivector(def_categories[0]+1);
      for(i = 0; i <= def_categories[0]; i++){
	defaults_args->categories[i] = def_categories[i];
      }
    }

    nlabels = atoi(def_input_types[0]);
    if(nlabels > 0){
	for (i = j = 1; i <= def_ninputs && j <= nlabels; i++, j++){
	    if(!strcasecmp(def_input_types[j], "bin") ||
	       !strcasecmp(def_input_types[j], "binary")){
		defaults_args->input_types[i] = BIN;
		defaults_args->input_type_values[i] = 0.;

	    } else if (!strcasecmp(def_input_types[j], "cat") ||
		       !strcasecmp(def_input_types[j], "categorical")){
		defaults_args->input_types[i] = CAT;
		j++;
		if( j > nlabels ){
		    warning("Error parsing input types, missing n catagories");
		    return;
		}
		defaults_args->input_type_values[i] = atoi(def_input_types[j]);

	    } else if (!strcasecmp(def_input_types[j], "circ") ||
		       !strcasecmp(def_input_types[j], "circular")){
		defaults_args->input_types[i] = CIRC;
		j++;
		if ( j > nlabels ){
		    warning("Error parsing input types, missing circ base");
		    return;
		}
		defaults_args->input_type_values[i] = atof(def_input_types[j]);
	    }
	}
    }
    deallocate_string_vector(def_labels, atoi(def_labels[0])+1);
    def_labels = NULL;
    deallocate_string_vector(def_input_labels, atoi(def_input_labels[0])+1);
    def_input_labels = NULL;
    deallocate_string_vector(def_input_types, atoi(def_input_types[0])+1);
    def_input_types = NULL;
    deallocate_ivector(def_categories);
    def_categories = NULL;
}

void
free_defaults_file_struct(DEFAULTS_FILE_STRUCT *df_args)
{
    if(df_args == NULL) return;

    if(df_args->output_labels != NULL){
	deallocate_string_vector(df_args->output_labels,
				 atoi(df_args->output_labels[0])+1);
    }
    if(df_args->input_labels != NULL){
	deallocate_string_vector(df_args->input_labels,
				 atoi(df_args->input_labels[0])+1);
    }
    if(df_args->input_types != NULL){
	deallocate_ivector((int *)df_args->input_types);
    }
    if(df_args->input_type_values != NULL){
	deallocate_vector(df_args->input_type_values);
    }
    if(df_args->categories){
      deallocate_ivector(df_args->categories);
    }
    nn_free((char *)df_args, "for defaults argument table");
}

/* uses old style defaults files */
void
check_desc_args(char *desc_fullname, char **labels, int *pnraw,
		int *pnoutputs, int *pmap_flag, int *pnpatterns)
{
    int             i;
    FILE           *desc_file;

    /* get file description arguments if present */
    desc_file = fopen(desc_fullname, "r");
    if (desc_file == NULL)
	    warning("Data description parameter file %s is not found.%s\n",
		    desc_fullname,
		    "  This will be a fatal error in later versions");
    else {
	i = nnargs_from_file(desc_file, desc_fullname, gnn_argtab,
		GNN_TABSIZE);

	if (atoi(labels[0]) <= 0 && atoi(gnn_labels[0]) > 0) {
	    for (i = 0;
		    i <= atoi(gnn_labels[0]) && i <= NNARGS_MAX_LABELS; i++) {
		labels[i] = gnn_labels[i];
	    }
	}
	if (*pnraw != gnn_ninputs) {
	    warning("check_desc_args: %s (%d) %s (%d)\n",
		    "Number of inputs specified",*pnraw,
		    "not equal to number in data file", gnn_ninputs);
	    *pnraw = gnn_ninputs;
	}
	if (*pnoutputs != gnn_noutputs) {
	    warning("check_desc_args: %s (%d) %s (%d)\n",
		    "Number of outputs specified",*pnoutputs,
		    "not equal to number in data file", gnn_noutputs);
	    *pnoutputs = gnn_noutputs;
	}
	if (*pmap_flag != gnn_map_flag) {
	    warning("check_desc_args: %s (%d) %s (%d)\n",
		    "Map flag specified", *pmap_flag,
		    "not equal to map flag in data file", gnn_map_flag);
	    *pmap_flag = gnn_map_flag;
	}
	if (*pnpatterns > gnn_n_patterns) {
	    warning("check_desc_args: %s (%d) %s (%d)\n",
		    "Number of patterns specified", *pnpatterns,
		    "greater than number in data file", gnn_n_patterns);
	    *pnpatterns = gnn_n_patterns;
	}
	nn_fclose(desc_file, desc_fullname, "for reading data description");
    }
}

void
write_defaults_stream(FILE *desc_file, DEFAULTS_FILE_STRUCT *defaults_args)
{
    int i, j, nlabels;

    if(def_labels != NULL){
	if(def_labels[0] != NULL)
	  deallocate_string_vector(def_labels, atoi(def_labels[0])+1);
	def_labels = NULL;
    }
    if(defaults_args->output_labels != NULL){
	nlabels = atoi(defaults_args->output_labels[0]);
	if(nlabels > 0){
	    def_labels = (char **)allocate_pvector(nlabels+1);
	    for(i = 0; i <= nlabels; i++){
		def_labels[i] =
		  nn_malloc(strlen(defaults_args->output_labels[i])+1,
			    "for label string");
		strcpy(def_labels[i], defaults_args->output_labels[i]);
	    }
	}
    } else {
	def_labels = NULL;
    }
    if(def_input_labels != NULL){
	if(def_input_labels[0] != NULL)
	  deallocate_string_vector(def_input_labels,
				   atoi(def_input_labels[0])+1);
	def_input_labels = NULL;
    }
    if(defaults_args->input_labels != NULL){
	nlabels = atoi(defaults_args->input_labels[0]);
	if(nlabels > 0){
	    def_input_labels = (char **)allocate_pvector(nlabels+1);
	    for(i = 0; i <= nlabels; i++){
		def_input_labels[i] =
		  nn_malloc(strlen(defaults_args->input_labels[i])+1,
			    "for label string");
		strcpy(def_input_labels[i], defaults_args->input_labels[i]);
	    }
	}
    } else {
	def_input_labels = NULL;
    }
    if(def_input_types != NULL){
	if(def_input_types[0] != NULL)
	  deallocate_string_vector(def_input_types,
				   atoi(def_input_types[0])+1);
	def_input_types = NULL;
    }
    if(defaults_args->input_types != NULL){
	char numString[10];
	/* leave extra space for the value arguments */
	nlabels = 2*defaults_args->input_types[0];
	if(nlabels > 0){
	    def_input_types = (char **)allocate_pvector(nlabels+1);
	    for(i = j = 1; i <= nlabels && j <= defaults_args->input_types[0];
		j++, i++){
		switch(defaults_args->input_types[j]){
		  case BIN:
		    def_input_types[i] = nn_strdup("BIN");
		    break;
		  case CAT:
		    def_input_types[i] = nn_strdup("CAT");
		    i++;
		    sprintf(numString, "%.0f", defaults_args->input_type_values[j]);
		    def_input_types[i] = nn_strdup(numString);
		    break;
		  case CIRC:
		    def_input_types[i] = nn_strdup("CIRC");
		    i++;
		    sprintf(numString, "%f", defaults_args->input_type_values[j]);
		    def_input_types[i] = nn_strdup(numString);
		    break;
		  case CONT:
		  default:
		    def_input_types[i] = nn_strdup("CONT");
		    break;
		}
	    }
	}
    } else {
	def_input_types = NULL;
    }
    if(defaults_args->categories){
      def_categories = allocate_ivector(defaults_args->categories[0]+1);
      for(i = 0; i <= defaults_args->categories[0]; i++){
	def_categories[i] = defaults_args->categories[i];
      }
    } else {
      def_categories = NULL;
    }

    def_npatterns = 0;
    def_ninputs = defaults_args->ninputs;
    def_noutputs = defaults_args->noutputs;
    def_class_flag = def_map_flag = def_seq_flag = 0;
    switch(defaults_args->db_type){
      case CLASS_DATA:
	def_class_flag = 1; break;
      case MAP_DATA:
	def_map_flag = 1; break;
      case SEQ_DATA:
	def_seq_flag = 1; break;
    }
    fprintf(desc_file, "describe %s",
	    nnargs_get_flags(def_argtab, DEF_TABSIZE));
}

void
write_defaults_file(char *filename, DEFAULTS_FILE_STRUCT *defaults_args)
{
    FILE *desc_file;

    desc_file = nn_fopen(filename, "w", "for data base description file");
    write_defaults_stream(desc_file, defaults_args);
    nn_fclose(desc_file, filename, "for writing data description");
}

void
write_desc_args(char *filename, char **labels, int ninputs,
		int noutputs, int db_type)
{
    DEFAULTS_FILE_STRUCT df_args;

    df_args.ninputs = ninputs;
    df_args.noutputs = noutputs;
    df_args.db_type = db_type;
    df_args.output_labels = labels;
    df_args.input_labels = NULL;
    df_args.input_types = NULL;
    df_args.input_type_values = NULL;
    df_args.categories = NULL;
    write_defaults_file(filename, &df_args);
}

/* return the label for input n, after normalization and feature selection
   have taken place.
   BUGS AND LIMITATIONS: because the return value is sometimes a static,
   it can disappear on a second call to get_input_label
*/
char *
get_input_label(int n, char **input_labels, int *features, int normalization)
{
    int ninput_labels;
    static char static_string[32], *label;

    if(input_labels != NULL) ninput_labels = atoi(input_labels[0]);
    else ninput_labels = 0;
    if(normalization == PCA || normalization == LDA) ninput_labels = 0;
    if(features != NULL && features[0] > 0){
	if(n < features[0] && features[n+1] < ninput_labels)
	  label = input_labels[features[n+1]+1];
	else{
	    sprintf(static_string, "%d", features[n+1]);
	    label = static_string;
	}
    } else {
	if(n < ninput_labels)
	  label = input_labels[n+1];
	else {
	    sprintf(static_string, "%d", n);
	    label = static_string;
	}
    }
    return label;
}

/* return the label for class n
   BUGS AND LIMITATIONS: because the return value is sometimes a static,
   it can disappear on a second call to get_input_label
*/
char *
get_class_label(int n, char **class_labels)
{
    int nclass_labels;
    static char static_string[32], *label;

    if(class_labels != NULL) nclass_labels = atoi(class_labels[0]);
    else nclass_labels = 0;
    if(n < nclass_labels) label = class_labels[n+1];
    else {
	sprintf(static_string, "%d", n);
	label = static_string;
    }
    return label;
}

char *
make_comment_string (char *flag_string)
{
    static char comment[4096];
    time_t curTime;

    strcpy(comment, get_program_name());
    strcat (comment, "\n");
    strcat (comment, flag_string);
    curTime = time(NULL);
    strcat(comment, ctime(&curTime));

    return comment;
}

void
print_log_footer(void)
{
    printf("%s %s   END\n",
	"================================================================",
	get_program_name());
}

void
print_log_header(void)
{
    printf("\n%s %s BEGIN\n",
	"================================================================",
	get_program_name());
    fflush(stdout);	      /* make sure initial feedback on run is timely */
}

void
remove_pid_file(char *pid_fullname)
{
    char chartmp[MAXSTR];

    sprintf(chartmp, "rm \"%s\"",pid_fullname);
    system (chartmp);
}

void
store_pid (char *pid_fullname)
{
    FILE *pid_file;

    /* save pid in pid_fname file for use in signal from another process */
    pid_file = nn_fopen(pid_fullname,"w","for storage of pid");
    fprintf (pid_file, "%d", (int)getpid());
    nn_fclose(pid_file,pid_fullname,"for storage of pid");
}

void
display_model_params (int model, int ninputs, int normalization,
		      int *features, int verbosity)
{
    int i;
    
    if(verbosity > 4){
	printf ("\n%s %d %s %d  %s %s  %s %d", 
		"Model", model, "initialized:  ninputs =", ninputs, 
		"normalization =", norm_description (normalization), 
		"features =", features[0]);
	if (ninputs > 1)
	  for (i=1; i < features[0]; i++) {
	      printf(",%d", features[i+1]);
	  }
	printf("\n");
    }
}

void
verify_model_params (int model, int ninputs, int model_ninputs,
		     int *features, int *model_features)
{
    int i;
    
    if (ninputs != model_ninputs) 
      warning("%s %d %d %d (model value is used)", 
	      "verify_model_params: ninputs parameter", ninputs, 
	      "not equal to model value", model_ninputs);
            
    for (i=0; i<= features[0]; i++) {
        if (features[i] != model_features[i])
	  warning("%s%d%s %d %s %d (model value is used)", 
		  "verify_model_params: features[", i, "] parameter", 
		  features[i], "not equal to model value", model_features[i]);
    }
}

int
fix_cross_valid_args(int *pcv, int *pnfolds, int *pnsplits, int *splits,
		     char **train_mask, char **test_mask, int from_file,
		     char *cv_fname)
{
    FILE *cv_file;
    int use_splits;

    if(*pcv < 1) *pcv = 0;
    if(from_file){
	if((cv_file = fopen(cv_fname, "r")) == NULL){
	    warning("cross_valid_file %s not found", cv_fname);
	} else {
	    nnargs_set_variable("cv_splits", splits, cross_valid_argtab,
				CROSS_VALID_TABSIZE);
	    nnargs_set_variable("cv_train_mask", (int *)train_mask,
				cross_valid_argtab, CROSS_VALID_TABSIZE);
	    nnargs_set_variable("cv_test_mask", (int *)test_mask,
				cross_valid_argtab, CROSS_VALID_TABSIZE);
	    nnargs_from_file(cv_file, cv_fname, cross_valid_argtab,
			     CROSS_VALID_TABSIZE);
	    nn_fclose(cv_file, cv_fname, "for cross_valid_splits");
	}
	cross_valid_check_args(&cv_nfolds, &cv_nsplits, splits,
			       train_mask, test_mask);
	/* the real arguments are used in the classifier main driver.
	   they are also what is supposed to have just been filled, so
	   this should all be smooth, now */
	*pcv = cv_nfolds;
	*pnfolds = cv_nfolds;
	*pnsplits = cv_nsplits;
	use_splits = 1;
    } else {
	if(*pcv >= NNARGS_MAX_LABELS){
	    fatal_error("too many cross valid folds(%d). Maximum %d folds",
			*pcv, NNARGS_MAX_LABELS-1);
	}
	use_splits = 0;
	*pnfolds = *pnsplits = *pcv;
	build_cross_valid_masks(*pcv, train_mask, test_mask);
    }
    return use_splits;
}

void
check_file_length(char *file_path, int *p_npat)
{
    FILE *fp;
    int file_npat;

    /* count the number of lines in the list file */
    fp = nn_fopen(file_path, "r", "for checking file length");
    file_npat = get_n_lines(fp);
    nn_fclose(fp, file_path, "for checking file length");

    /* check file length against the requested number of patterns */
    if(file_npat < *p_npat || *p_npat == 0) *p_npat = file_npat;
}

/* check uniform priors, the length of priors list, and the priors adjustment
   flag.  return whether to adjust priors (1) or not (0)
*/
void
fix_priors(int adjust_train, int train_uniform_priors,
	   float **train_ppriors_list, int adjust_test,
	   int test_uniform_priors, float **test_ppriors_list,
	   int train_flag, int cross_valid, int noutputs)
{
    int i;
    float *priors_list, total;

    if(adjust_train && (train_flag || cross_valid > 0)){
	priors_list = *train_ppriors_list;
	if(train_uniform_priors){
	    /* set up priors list to give uniform priors */
	    /* check that there is space for the priors in priors_list */
	    if(priors_list != NULL && priors_list[0] < noutputs){
		deallocate_vector(priors_list);
		priors_list = NULL;
	    }
	    if(priors_list == NULL){
		priors_list = allocate_vector(noutputs+1);
		*train_ppriors_list = priors_list;
		priors_list[0] = noutputs;
	    }
	    for(i = 1; i <= noutputs; i++)
	      priors_list[i] = 1./(float)noutputs;
	}
	if(priors_list[0] != (float)noutputs){
	    fatal_error("fix_priors: %s %.0f, should be %d",
			"train_priors_list length is", priors_list[0],
			noutputs);
	} else {
	    /* if user supplied priors, normalize to sum to 1 */
	    if(!train_uniform_priors){
		total = 0;
		for(i = 1; i <= noutputs; i++) total += priors_list[i];
		for(i = 1; i <= noutputs; i++) priors_list[i] /= total;
	    }
	}
    }
    if(adjust_test && !train_flag){
	priors_list = *test_ppriors_list;
	if(test_uniform_priors){
	    /* set up priors list to give uniform priors */
	    /* check that there is space for the priors in priors_list */
	    if(priors_list != NULL && priors_list[0] < noutputs){
		deallocate_vector(priors_list);
		priors_list = NULL;
	    }
	    if(priors_list == NULL){
		priors_list = allocate_vector(noutputs+1);
		*test_ppriors_list = priors_list;
		priors_list[0] = noutputs;
	    }
	    for(i = 1; i <= noutputs; i++)
	      priors_list[i] = 1./(float)noutputs;
	}
	if(priors_list[0] != (float)noutputs){
	    fatal_error("fix_priors: %s %.0f, should be %d",
			"test_priors_list length is", priors_list[0],
			noutputs);
	} else {
	    /* if user supplied priors, normalize to sum to 1 */
	    if(!test_uniform_priors){
		total = 0;
		for(i = 1; i <= noutputs; i++) total += priors_list[i];
		for(i = 1; i <= noutputs; i++) priors_list[i] /= total;
	    }
	}
    }
}
    
/* fill in the input_types arrays, check for conflicts with normalization,
   and re-order the input_types arrays to match the feature selection */
/* it is necessary to call fix_features before fix_input_types is called !! */
void
fix_input_types(DEFAULTS_FILE_STRUCT *df_args, int normalize, int ninputs,
		int *features)
{
    int i;

    /* make sure that input_types and input_type_values are there and are
       as long as they need to be */
    if(df_args->input_types == NULL){
	df_args->input_types = (INPUT_FEATURE_TYPE *)
	  allocate_ivector(df_args->ninputs+1);
	df_args->input_types[0] = df_args->ninputs;
	for(i = 1; i <= df_args->ninputs; i++){
	    df_args->input_types[i] = CONT;
	}
    } else if (df_args->input_types[0] != df_args->ninputs) {
	fatal_error("the number of input types specified is not the same as the number of inputs in the data base");
    }
    if(df_args->input_type_values == NULL){
	df_args->input_type_values = allocate_vector(df_args->ninputs+1);
	df_args->input_type_values[0] = df_args->ninputs;
	for(i = 1; i <= df_args->ninputs; i++){
	    df_args->input_types[i] = 0.;
	}
    } else if (df_args->input_types[0] != df_args->ninputs) {
	fatal_error("the number of input type values specified is not the same as the number of inputs in the data base");
    }
    /* the number of input types and values is nraw */

    /* if we need to do feature selection, we need to play with the
       input_types stuff */
    if (ninputs != df_args->ninputs || (features != NULL && features[0] != 0)){
	if(features != NULL && features[0] != 0){
	    int *temp_ivector;
	    float *temp_vector;
	    int i;
	    /* reorder the list to be in the order of the selected features */
	    temp_ivector = allocate_ivector(features[0]+1);
	    temp_vector = allocate_vector(features[0]+1);
	    temp_ivector[0] = features[0];
	    temp_vector[0] = features[0];
	    for(i = 1; i <= ninputs; i++){
		temp_ivector[i] = df_args->input_types[features[i]+1];
		temp_vector[i] = df_args->input_type_values[features[i]+1];
	    }
	    deallocate_ivector((int *)df_args->input_types);
	    deallocate_vector(df_args->input_type_values);
	    df_args->input_types = (INPUT_FEATURE_TYPE *)temp_ivector;
	    df_args->input_type_values = temp_vector;
	} else {
	    /* truncate the list to be ninputs long */
	    df_args->input_types[0] = ninputs;
	    df_args->input_type_values[0] = ninputs;
	}
    }
    /* you can't do normalization when you have a circular variable */
    /* this is after feature selection, to give people a way out */
    if(normalize){
	/* check if we can do normalization */
	for(i = 1; i <= df_args->input_types[0]; i++){
	    if(df_args->input_types[i] == CIRC){
		fatal_error("Normalization cannot be used when there are circular variables");
	    }
	}
    }
}
