/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include "nnlib.h"

int find_nearest_neighbor(float *inputs, float **centers,
			  int ninputs, int ncenters)
{
    int i, min_i;
    float dist, min_dist;

    if(ncenters < 1 || ninputs < 1) return -1;

    min_dist = MAXFLOAT;
    min_i = 0;
    for(i = 0; i < ncenters; i++){
	dist = l2dist(ninputs, inputs, &centers[i][0]);
	if(dist < min_dist){
	    min_dist = dist;
	    min_i = i;
	}
    }
    return min_i;
}

void find_k_nearest_neighbors(float *inputs, float **centers,
			      float *dists, int *neighbors, int ninputs,
			      int ncenters, int k)
{
    int i, j;
    float dist;

    if(k < 1 || neighbors == NULL || dists == NULL) return;
    if(k > ncenters) k = ncenters;
    for(i = 0; i < k; i++){
	dists[i] = MAXFLOAT;
	neighbors[i] = -1;
    }
    if(ninputs < 1 || inputs == NULL || centers == NULL){
	return;
    }
    for(i = 0; i < ncenters; i++){
	dist = l2dist(ninputs, inputs, &centers[i][0]);
	if(dist < dists[k-1]){
	    for(j = k-1; j > 0 && dist < dists[j-1]; j--){
		dists[j] = dists[j-1];
		neighbors[j] = neighbors[j-1];
	    }
	    dists[j] = dist;
	    neighbors[j] = i;
	}
    }
}

/*****************************************************************************
* NAME: int max_pick()
* DESCRIPTION:
*       Returns integer corresponding to maximum value of vec
*****************************************************************************/

int int_max_pick (int *vec, int size)
{
    int max_value, tmp;
    register int max_class=0, i;

    max_value = *vec++;
    if (size>1) for (i=1; i<size; i++) {
        tmp = *vec++;
	if (tmp > max_value) {
	    max_class=i;
	    max_value=tmp;
	}
    }
    return max_class;
}
/*****************************************************************************
* NAME: max_pick()
* DESCRIPTION:
*       Returns integer corresponding to maximum float value of vec
*****************************************************************************/

int max_pick (float *vec, int size)
{
    float max_value, tmp;
    register int max_class=0, i;

    max_value = *vec++;
    if (size>1) for (i=1; i<size; i++) {
        tmp = *vec++;
	if (tmp > max_value) {
	    max_class=i;
	    max_value=tmp;
	}
    }
    return max_class;
}
/*****************************************************************************
* NAME: max_pick_int()
* DESCRIPTION:
*       Returns integer corresponding to maximum float value of vec
*****************************************************************************/

int max_pick_int (int *vec, int size)
{
    int max_value, tmp;
    register int max_class=0, i;

    max_value = *vec++;
    if (size>1) for (i=1; i<size; i++) {
        tmp = *vec++;
	if (tmp > max_value) {
	    max_class=i;
	    max_value=tmp;
	}
    }
    return max_class;
}
