/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

/* tee - read from standard input and write to standard output and files.
   Copyright (C) 1985, 1990, 1991 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* Mike Parker, Richard M. Stallman, and David MacKenzie */

/* Modification by Rob Cunningham rkc@xn.ll.mit.edu to
   handle other interrupts */

/* Modification by Linda Kukolich kukolich@sst.ll.mit.edu to
   work with LNKnet software */

/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <sys/types.h>
#include <signal.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#if HAVE_FCNTL_H
#include <fcntl.h>
#endif
#include <errno.h>
#include "nnlib.h"

int tee (int nfiles, char **files);

/* If nonzero, append to output files rather than truncating them. */
int append;

/* If nonzero, ignore interrupts. */
int ignore_interrupts;

/* If nonzero, handle hup interrupts. */
int handle_hup_interrupts;

/* If nonzero, stdout and stderr have been closed */
int std_closed;

/* The name that this program was run with. */
char *program_name;

/* argument table for use with nnargs */
static ARG argtab[]=
{
    {"a", BOOLEAN, &append, "append"},
    {"i", BOOLEAN, &ignore_interrupts, "ignore-interrupts"},
    {"h", BOOLEAN, &handle_hup_interrupts, "handle-missing-tty"},
};
#define TABSIZE (sizeof(argtab)/sizeof(ARG))


void
sigh(int s)
{
  FILE *t;
  t = fopen("tee_error","a");
  fprintf(t,"Caught sig %d\n",s);
  fclose(t);
}

void
close_std(int s)
{
  fprintf(stderr,"Caught hup signal\n");
  if (close (1) == -1)
    fatal_error ("error closing standard output %d", errno);
  if (close (2) == -1)
    fatal_error ("error closing standard error %d", errno);
  std_closed = 1;
}

int
main (int argc, char **argv)
{
  int errs;
  int pid;

  program_name = argv[0];
  std_closed = 0;

  /* parse arguments, stripping them from argv and decrementing argc */
  argc = nnargs(argc, argv, argtab, TABSIZE);

  if (ignore_interrupts)
    signal (SIGINT, SIG_IGN);

  if(handle_hup_interrupts)
  {
    pid = getpid();
    setpgid(pid, pid);/* Change our process group so an overzealous terminal
		       doesn't kill us. */
    signal (SIGHUP, close_std);
  }

  errs = tee (argc -1, &argv[1]);

  if (!std_closed && close (0) == -1)
    fatal_error ("error closing standard input %d", errno);
  if (!std_closed && close (1) == -1)
    fatal_error ("error closing standard output %d", errno);
  exit (errs);
}

/* Copy the standard input into each of the NFILES files in FILES
   and into the standard output.
   Return 0 if successful, 1 if any errors occur. */

int
tee (int nfiles, char **files)
{
  int *descriptors;
  char buffer[BUFSIZ];
  register int bytes_read, i, ret = 0, mode;

  descriptors = (int *) malloc (nfiles * sizeof (int));
  mode = O_WRONLY | O_CREAT;
  if (append)
    mode |= O_APPEND;
  else
    mode |= O_TRUNC;

  for (i = 0; i < nfiles; i++)
    {
      descriptors[i] = open (files[i], mode, 0666);
      if (descriptors[i] == -1)
	{
	    warning("error %d opening %s", errno, files[i]);
	    ret = 1;
	}
    }

  while ((bytes_read = read (0, buffer, sizeof buffer)) > 0)
    {
      if(!std_closed) write (1, buffer, bytes_read);
      for (i = 0; i < nfiles; i++)
	if (descriptors[i] != -1)
	  write (descriptors[i], buffer, bytes_read);
    }
  if (bytes_read == -1)
    {
      warning("read error %d", errno);
      ret = 1;
    }

  for (i = 0; i < nfiles; i++)
    if (descriptors[i] != -1 && close (descriptors[i]) == -1)
      {
	warning("error %d closing %s", errno, files[i]);
	ret = 1;
      }

  free(descriptors);
  return ret;
}


