/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
                                  /* norm2c.c */
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include "nnlib.h"

/* global variables */
static int norm_type, norm_ninputs;
static float *means, *sdevs, **transform_matrix;
static int *features;
static int ninputs;
static char *N2Cnraw_name = NULL, *N2Cninputs_name = NULL;
static char N2C1[16];/* space for numbers if names aren't specified */
static char N2C2[16];

/* subroutines */

void get_norm_params(int model, int ninputs_, int *features_)
{
    /* get norm parameters */
    ninputs = ninputs_;
    features = features_;
    norm_type = *(int *)norm_param("normalization");
    norm_ninputs = *(int *)norm_param("ninputs");
    if(norm_ninputs == 0) norm_ninputs = ninputs;
    means = *(float **)norm_param("means");
    sdevs = *(float **)norm_param("sdevs");
    transform_matrix = *(float ***)norm_param("transform");
}

void
print_norm_comments(void)
{
    
    /* print description of variables */
    switch (norm_type){
      case NONE:
	printf("/* norm: NONE */\n");
	break;
      case SIMPLE:
	printf("/* norm: SIMPLE */\n");
	break;
      case PCA:
	printf("/* norm: PCA with SIMPLE */\n");
	break;
      case LDA:
	printf("/* norm: LDA with SIMPLE */\n");
	break;
    }
    if(ninputs != norm_ninputs){
	printf("/* %d features ", ninputs);
	if(features[0] != 0){
	    int i;
	    printf(": ");
	    for(i = 1; i <= features[0]; i++) printf("%d ", features[i]);
	}
	printf("*/\n");
    } else{
	printf("/* %d (all) features */\n", ninputs);
    }
    
    printf("\n");
}

void
print_norm_macros(char *nraw_name, char *ninputs_name)
{
    N2Cnraw_name = nraw_name;
    N2Cninputs_name = ninputs_name;
    printf("#define %s %d\n", nraw_name, norm_ninputs);
    printf("#define %s %d\n", ninputs_name, ninputs);
}

void
clear_norm_macros(void)
{
    printf("#undef %s\n", N2Cnraw_name);
    printf("#undef %s\n", N2Cninputs_name);
}

void
print_normalize_code(char *normalize_name)
{
    int sample=0;

    if(ninputs != norm_ninputs && features[0] != 0){
	sample = 1;
    }

    /* fix up nraw stuff, in case no one told us about it */
    if(N2Cnraw_name == NULL){
	sprintf(N2C1, "%d", norm_ninputs);
	N2Cnraw_name = N2C1;
    }
    if(N2Cninputs_name == NULL){
	sprintf(N2C2, "%d", ninputs);
	N2Cninputs_name = N2C2;
    }
    printf("\n\n");
    printf("int %s(inputs)\n", normalize_name);
    printf("   float *inputs;\n");
    printf("{\n");
    if(norm_type == NONE && sample == 0){
	printf("  return((int) %s);\n", N2Cninputs_name);
	printf("}\n");
	return;
    }
    if(sample == 1){
	print_ivector_initialization("features", &features[1], ninputs,
				     N2Cninputs_name);
    }

    printf("  int n;\n");
    printf("  float new_inputs[%s];\n", N2Cnraw_name);

    if(sample == 1){
	/* print out parameter initializations for just the ones we need */
	printf("/* normalization parameters for selected features */\n");

	/* remember to take off the first element of the features array */
	if(norm_type == SIMPLE){
	    print_sampled_vector_initialization("means", means, ninputs,
						N2Cninputs_name,
						&features[1]);
	    print_sampled_vector_initialization("sdevs", sdevs, ninputs,
						N2Cninputs_name,
						&features[1]);
	}
	if(norm_type == PCA || norm_type == LDA){
	    printf("  int m;\n");
	    print_sampled_matrix_initialization("transform_matrix",
						transform_matrix,
						ninputs, norm_ninputs,
						N2Cninputs_name, N2Cnraw_name,
						&features[1]);
	    /* also do simple norm on transformed elements */
	    print_sampled_vector_initialization("means", means, ninputs,
						N2Cninputs_name, &features[1]);
	    print_sampled_vector_initialization("sdevs", sdevs, ninputs,
						N2Cninputs_name, &features[1]);
	}
    } else {
	switch(norm_type){
	  case PCA:
	  case LDA:
	    printf("  int m;\n");
	    print_matrix_initialization("transform_matrix", transform_matrix,
					ninputs, norm_ninputs,
					N2Cninputs_name, N2Cnraw_name);
	    /* fall through and do simple norm, too */
	  case SIMPLE:
	    print_vector_initialization("means", means, ninputs,
					N2Cninputs_name);
	    print_vector_initialization("sdevs", sdevs, ninputs,
					N2Cninputs_name);
	}
    }
    printf("\n");

    printf("  for(n = 0; n < %s; n++){\n", N2Cninputs_name);
    switch(norm_type){
      case NONE:
	if(sample)
	  printf("    new_inputs[n] = (float) inputs[features[n]];\n");
	else
	  printf("    new_inputs[n] = (float) inputs[n];\n");
	break;
      case PCA:
      case LDA:
	printf("\n");
	printf("    new_inputs[n] = (float) 0.0;\n");
	printf("    for(m = 0; m < %s; m++){\n", N2Cnraw_name);
	printf("      new_inputs[n] += (float) ");
	printf(" transform_matrix[n][m] * inputs[m];\n");
	printf("    }\n");
	printf("    new_inputs[n] = ");
	printf("(new_inputs[n]");
	printf("- means[n]) / sdevs[n];\n");
	break;
      case SIMPLE:
	printf("    new_inputs[n] = ");
	if(sample)
	  printf("(inputs[features[n]]");
	else
	  printf("(inputs[n]");
	printf("- means[n]) / sdevs[n];\n");
	break;
    }
    printf("  }\n");

    printf("\n");
    printf("  for(n = 0; n < %s; n++){\n", N2Cninputs_name);
    printf("    inputs[n] = new_inputs[n];\n");
    printf("  }\n");
    printf("  return((int) %s);\n", N2Cninputs_name);
    printf("}\n");
}
void
print_copy_inputs(void)
{
    /* fix up nraw stuff, in case no one told us about it */
    if(N2Cnraw_name == NULL){
	sprintf(N2C1, "%d", norm_ninputs);
	N2Cnraw_name = N2C1;
    }
    if(N2Cninputs_name == NULL){
	sprintf(N2C2, "%d", ninputs);
	N2Cninputs_name = N2C2;
    }

    printf("  float x[%s];\n", N2Cnraw_name);
    printf("  int i;\n");

    printf("\n  /* load inputs */\n");
    printf("  for(i = 0; i < %s; i++)\n", N2Cnraw_name);
    printf("    x[i] = inputs[i];\n");
}
