/***---------------------------------------------------------------------------
* NAME:		note.c 
* PURPOSE:	maintain a notebook file for a LNKnet session
* DESCRIPTION:
*	diff_argv(int argc1, char ** argv1, int argc2, char ** argv2)
*	diff_screen(char *screen_fname1, char *screen_fname2)
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/lib/note.c,v 11.4 2001/08/23 20:14:45 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/lib/note.c,v $
* CREATED: 26-July-90 rpl                             MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */


/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include "nnlib.h"

enum dir { insert, delete };
static struct change {
    enum dir direction;
    char *string;
} changes[256];
static int change_num = 0;

/* check whether there is a match for the current pair of arguments
   if there is, record the changes between the two entries */
static int
check_args(char **argv1, char **argv2, int match1, int match2,
	   int offset1, int offset2)
{
    int j;
    if(!strcmp(argv1[match1+offset1], argv2[match2+offset2])){
	/* if this is a simple deletion or insertion, offset1 or offset2
	   is zero, then the changes are just those here.
	   Otherwise, we need to back step one for both */
	/* I'm trying it a new way. Only print out the new version of
	   a modified string. move the insert one back a step
	   to display the constant string. move the delete one forward
	   to skip the changed argument */
	if(offset1 != 0 && offset2 != 0){
	    if(match1 > 0) match1--;
	    if(match2 > 0) match2++;
	    offset1++;
	    offset2--;
	}
	for(j = 0; j < offset1; j++){
	    changes[change_num].direction = insert;
	    changes[change_num].string = argv1[match1+j];
	    change_num++;
	}
	for(j = 0; j < offset2; j++){
	    changes[change_num].direction = delete;
	    changes[change_num].string = argv2[match2+j];
	    change_num++;
	}
	return 1;
    } else return 0;
}

/***---------------------------------------------------------------------------
* NAME:		diff_argv
* PURPOSE:	find differences between two argv lists, printing them
*		to the FILE p;
* EXAMPLE:
*	FILE *p;
*	diff_argv(p, argc1, argv1, argc2, argv2)
* NOTE:
--------------------------------------------------------------------------***/
void
diff_argv(FILE *fp, int argc1, char **argv1, int argc2, char **argv2)
{
    int match1, match2, i, search, done;
    enum dir curdir;
    
    change_num = 0;
    /* we assume here that argv1[0] and argv2[0] are command names and that
       they match */
    for(match1 = match2 = 1; match1 < argc1 && match2 < argc2;
	match1++, match2++){
	/* if these don't match */
	if ( strcmp(argv1[match1], argv2[match2])){
	    done = 0;
	    for (search = 1; !done ; search++){
		if(match1+search < argc1){
		    for(i = 0; i < search; i++){
			if(match2+i < argc2){
			    done = check_args(argv1, argv2, match1, match2,
					      search, i);
			    if(done){
				match1 += search;
				match2 += i;
				break;
			    }
			}
		    }
		}
		if(!done && match2+search < argc2){
		    for(i = 0; i < search; i++){
			if(match1+i < argc1){
			    done = check_args(argv1, argv2, match1, match2,
					      i, search);
			    if(done){
				match1 += i;
				match2 += search;
				break;
			    }
			}
		    }
		}
		if(!done && match2+search < argc2 && match1+search < argc1){
		    done = check_args(argv1, argv2, match1, match2,
				      search, search);
		    if(done){
			match1 += search;
			match2 += search;
		    }
		}
	    }
	}
    }
    if(change_num > 0){
	curdir = insert;
	/* print out changes */
	fprintf(fp, "{%s ", argv1[0]);
	for(i = 0; i < change_num; i++){
	    if(changes[i].direction == delete && curdir == insert)
	      fprintf(fp, "[");
	    else if (changes[i].direction == insert && curdir == delete)
	      fprintf(fp, "] ");
	    fprintf(fp, "%s", changes[i].string);
	    fprintf(fp, " ");
	    curdir = changes[i].direction;
	}
	if(changes[change_num-1].direction == delete) fprintf(fp, "] ");
	fprintf(fp, "} ");
    }
}

void
store_train_note_command(FILE *shell_file, char *log_fname, char *script_name,
			 char *notebook_path)
{
    /* insert a tail -1 to get the last one ... */
    fprintf(shell_file, "echo -n \"%s\" >> %s\n", script_name, notebook_path);
    fprintf(shell_file, "grep \"LAST TRAIN EPOCH\" %s | tail -1 >> %s\n",
	    log_fname, notebook_path);
}

void
store_test_note_command(FILE *shell_file, char *log_fname, char *script_name,
			char *notebook_path)
{
    fprintf(shell_file, "echo -n \"%s\" >> %s\n", script_name, notebook_path);
    fprintf(shell_file, "grep \"TEST\" %s | tail -1 >> %s\n",
	    log_fname, notebook_path);
}

void
store_cv_note_command(FILE *shell_file, char *log_fname, char *script_name,
		      char *notebook_path)
{
    fprintf(shell_file, "echo -n \"%s\" >> %s\n", script_name, notebook_path);
    fprintf(shell_file, "grep \"CV\" %s | tail -1 >> %s\n",
	    log_fname, notebook_path);
}

void
store_roc_note_command(FILE *shell_file, char *log_fname, char *script_name,
		       char *notebook_path)
{
    fprintf(shell_file, "echo -n \"%s\" >> %s\n", script_name, notebook_path);
    fprintf(shell_file, "grep \"ROC AREA\" %s | tail -1 >> %s\n",
	    log_fname, notebook_path);
}

void
store_chi_note_command(FILE *shell_file, char *log_fname, char *script_name,
		       char *notebook_path)
{
    fprintf(shell_file, "echo -n \"%s\" >> %s\n", script_name, notebook_path);
    fprintf(shell_file, "grep \"CHI\" %s | tail -1 >> %s\n",
	    log_fname, notebook_path);
}
