/***--------------------------------------------------------------------------
* NAME:  path_build.c
* PURPOSE:  Functions called by neural network software
* USAGE
*
* SEE ALSO: 
* GLOBALS:
* DESCRIPTION: 
*
*	path_build		Combines two strings and returns the result
*	
------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */


/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include "nnlib.h"

/*****************************************************************************
* NAME:path_build()
* PURPOSE:	Build a temporary path name from two character strings 
* USAGE: 
*	char *path_build (str1,str2)
*	char *str1;	pointer to first string
*	char *str2;	pointer to second string
*
* DESCRIPTION:
*       Combines two strings and returns the result in a static internal buffer
* RETURNS:
*	buffer:	pointer to static buffer containing result of concatenation
*
* CREATED: 06-Feb-91 dan                             MIT LINCOLN LABORATORY
******************************************************************************/
#define MAX_PATH_SIZE 256

char * path_build (char *str1, char *str2)
{
    static char buffer[MAX_PATH_SIZE];
    int len1, extra=0;

    len1 = strlen(str1);
    strcpy (buffer, str1);

    if (len1 > 0 && str1[len1-1] != '/') {
	strcat (buffer, "/");
	extra = 1;
    }
    if ((strlen(str1) + strlen(str2) + extra) > MAX_PATH_SIZE)
	warning("%s %s and %s are > %d, path_build aborted.",
	        "path_build: combined length of strings",str1,str2,
		MAX_PATH_SIZE);
    else
	strcat (buffer, str2);

#if DEBUG
    printf("path_build -- %s %s %s\n",str1,str2,buffer);
#endif

    return buffer;
}

/* just like path_build, except that the new string is allocated
   and must be freed (and doesn't disappear the next time someone
   uses the static! */
char *
path_create(char *str1, char *str2)
{
    char *buffer;
    int len1;

    /* allocate space for both strings
       plus one each for the '/' and the '\0' */
    buffer = nn_malloc(strlen(str1) + strlen(str2) + 2, "for creating a path");
    strcpy(buffer, str1);
    len1 = strlen(str1);
    if(len1 > 0 && str1[len1-1] != '/') strcat(buffer, "/");
    strcat(buffer, str2);
    return buffer;
}
