/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/***-------------------------------------------------------------------------
* NAME:        print_source.c
* PURPOSE:	functions to write array and matrix
*		declaration/initializations.
* USAGE:
*
* SEE ALSO:     
* DESCRIPTION: 
*
* GLOBAL FUNCTIONS:	
*	print_matrix_initialization();
*	print_vector_initialization();
*	print_imatrix_initialization();
*	print_ivector_initialization();
*
* ALGORITHM:
* REFERENCE:
*
* BUGS AND LIMITATIONS:
*
* $Header: 
* $Source: 
* CREATED: 20-AUG-93 deniz                             MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/

/* Include necessary header files */
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include "nnlib.h"
#include "print_source.h"

/************************************************************************
* NAME: print_matrix_initialization(matrix_name, matrix, isize, jsize)
* DESCRIPTION:
*	prints a source code to declare and initialize
*	a static (isize x jsize) float matrix.
*
*************************************************************************/
void
print_matrix_initialization(char *matrix_name, float **matrix,
			    int isize, int jsize, char *iname, char *jname)
{
    int i;
    
    printf("\n");
    if(iname == NULL)
      printf("static float %s[%d]", matrix_name, isize);
    else 
      printf("static float %s[%s]", matrix_name, iname);
    if(jname == NULL)
      printf("[%d] = {\n", jsize);
    else
      printf("[%s] = {\n", jname);

    for(i = 0; i < isize; i++){
	printf("\t{");
	print_vector(matrix[i], jsize);
	printf("},\n");
    }
    printf(" };\n");

}

/************************************************************************
* NAME: print_vector_initialization(vector_name, vector, isize)
* DESCRIPTION:
*	prints a source code to declare and initialize
*	a static (isize x jsize) float matrix.
*
*************************************************************************/
void
print_vector_initialization(char *vector_name, float *vector,
			    int isize, char *iname)
{
    printf("\n");
    if(iname == NULL)
      printf("static float %s[%d] = {\n\t", vector_name, isize);
    else
      printf("static float %s[%s] = {\n\t", vector_name, iname);
    print_vector( vector, isize);
    printf("};\n");

}

/************************************************************************
* NAME: print_vector(vector, isize)
* DESCRIPTION:
*	prints initialization of a vector of floats, WITHOUT BRACES
*
*************************************************************************/
void print_vector(float *vector, int isize)
{
    int i;
    
    for(i = 0; i < isize; i++){
	if(i && !(i % NFLOATS_PER_LINE))
	   printf("\n ");
	printf("%14.6e, ", vector[i]);
    }
}

/************************************************************************
* NAME: print_sampled_matrix_initialization(matrix_name, matrix, isize, jsize,
*		features)
* DESCRIPTION:
*	prints a source code to declare and initialize
*	a static (isize x jsize) float matrix.
*
*************************************************************************/
void
print_sampled_matrix_initialization(char *matrix_name, float **matrix,
				    int isize, int jsize, char *iname,
				    char *jname, int *features)
{
    int i;
    
    printf("\n");
    if(iname == NULL || jname == NULL)
      printf("static float %s[%d][%d] = {", matrix_name, isize, jsize);
    else
      printf("static float %s[%s][%s] = {", matrix_name, iname, jname);
    for(i = 0; i < isize; i++){
	printf("\n\t{");
	print_vector(matrix[features[i]], jsize);
	printf("}, ");
    }
    printf("\n };\n");

}


/************************************************************************
* NAME: print_sampled_vector_initialization(vector_name, vector, isize,
*			features)
* DESCRIPTION:
*	prints a source code to declare and initialize
*	a static (isize x jsize) float matrix.
*
*************************************************************************/
void
print_sampled_vector_initialization(char *vector_name, float *vector,
				    int isize, char *iname, int *features)
{
    int i;
    
    printf("\n");
    if(iname == NULL)
      printf("static float %s[%d] = {", vector_name, isize);
    else
      printf("static float %s[%s] = {", vector_name, iname);
    printf("\n\t");      
    for(i = 0; i < isize; i++){
	if(i && !(i % NFLOATS_PER_LINE))
	   printf("\n ");
	printf("%14.6e, ", vector[features[i]]);
    }
    printf("\n };\n");

}

/************************************************************************
* NAME: print_imatrix_initialization(matrix_name, matrix, isize, jsize)
* DESCRIPTION:
*	prints a source code to declare and initialize
*	a static (isize x jsize) float matrix.
*
*************************************************************************/
void
print_imatrix_initialization(char *matrix_name, int **matrix, int isize,
			     int jsize, char *iname, char *jname)
{
    int i;
    
    printf("\n");
    if(iname == NULL || jname == NULL)
      printf("static int %s[%d][%d] = {", matrix_name, isize, jsize);
    else
      printf("static int %s[%s][%s] = {", matrix_name, iname, jname);
    for(i = 0; i < isize; i++){
	printf("\n\t{");
	print_ivector(matrix[i], jsize);
	printf("}, ");
    }
    printf("\n };\n");
}

/************************************************************************
* NAME: print_ivector_initialization(vector_name, vector, isize)
* DESCRIPTION:
*	prints a source code to declare and initialize
*	a static (isize x jsize) float matrix.
*
*************************************************************************/
void
print_ivector_initialization(char *vector_name, int *vector, int isize,
			     char *iname)
{
    printf("\n");
    if(iname == NULL)
      printf("static int %s[%d] = {", vector_name, isize);
    else
      printf("static int %s[%s] = {", vector_name, iname);
    printf("\n\t");      
    print_ivector(vector, isize);
    printf(" };\n");

}

/************************************************************************
* NAME: print_ivector(vector, isize)
* DESCRIPTION:
*	prints initialization of a vector of ints, WITHOUT BRACES
*
*************************************************************************/
void print_ivector(int *vector, int isize)
{
    int i;
    
    for(i = 0; i < isize; i++){
	if(i && !(i % NINTS_PER_LINE))
	   printf("\n ");
	printf("%6d, ", vector[i]);
    }
}
