/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include "nnlib.h"


/*  Argument Variables for roc_test program */
static int target = 0;
static char *error_fname = "roc_test.err";
static char *std_out_fname = "std_roc.out";
static char *new_out_fname = "new_roc.out";
static char *new_deriv_fname = "new_deriv.out";
static char *rms_deriv_fname = "rms_deriv.out";
static char *slopes_fname = "slope.out";
static int noutputs = 0;
static int verbose = 3;
static float reject_rate= 0.0;
static int npatterns = 0;

static ARG roc_test_argtab[]=
{
  {"target", INT, (int *) &target, "target class [0 - nclasses-1]"},
  {"ferror",STRING,(int *) &error_fname,
     "error file in pathexp created with -verror 2"},
  {"fstd",STRING, (int *) &std_out_fname, "roc data file for std method"},
  {"fnew",STRING, (int *) &new_out_fname, "roc data file for new method"},
  {"fderiv",STRING, (int *) &new_deriv_fname,
     "roc derivative data file for new method"},
  {"frms_deriv",STRING, (int *) &rms_deriv_fname,
     "rms derivative data file for data"},
  {"fslopes", STRING, (int *) &slopes_fname, "file for slopes"},
  {"noutputs", INT, (int *) &noutputs, "number of outputs"},
  {"verbose", INT, &verbose,
     "0=No Table, 1=Area under ROC 2=Sampled Table 3=Full Table"},
  {"npatterns", INT, &npatterns, "number of patterns per epoch in error file"},
};

#define ROC_TEST_TABSIZE (sizeof(roc_test_argtab)/sizeof(ARG))


/* read patterns from an error file */
/* calculate the ROC for them, both ways */
/* print out both ROC's for use in plot files */

main(argc, argv)
     int argc;
     char **argv;
{
    FILE *err_file, *std_file, *new_file, *deriv_file, *rms_deriv_file, *slopes_file;
    int not_done, i, aroc, epochs, rv;
    int error_file_verbosity=2; /* long Error File */
    ERROR_LINE error_line;
    float *desired_outputs, *outputs, *derivative, x;
    roc *roc_struct;

    argc = nnargs(argc, argv, roc_test_argtab, ROC_TEST_TABSIZE);
    if(target < 0) target = 0;
    if(target > noutputs-1) target = noutputs - 1;

    err_file = nn_fopen(error_fname, "r", "error file");
    std_file = nn_fopen(std_out_fname, "w", "standard roc file");
    new_file = nn_fopen(new_out_fname, "w", "new roc file");
    deriv_file = nn_fopen(new_deriv_fname, "w", "deriv roc file");
    rms_deriv_file = nn_fopen(rms_deriv_fname, "w", "deriv rms file");
    slopes_file = nn_fopen(slopes_fname, "w", "slope file");
    
    not_done = 1;
    if(npatterns <= 0) npatterns = get_n_lines(err_file);
    aroc= roc_build(noutputs, npatterns);


    /* allocate space for vectors from error file */
    error_line.outputs = allocate_vector(noutputs);
    error_line.inputs = NULL;
    error_line.desired_outputs = allocate_vector(noutputs);
    error_line.ninputs = 0;
    error_line.noutputs = noutputs;

    desired_outputs = allocate_vector(noutputs);
    outputs = allocate_vector(noutputs);
    derivative = allocate_vector(noutputs);
    roc_cost_init(noutputs, 200);

    epochs = 0;
    while(not_done){
	epochs++;
	for(i = 0; i < npatterns; i++){
	    rv = get_err_line(err_file, error_fname, &error_line,
			      error_file_verbosity);
	    if(rv == EOF){
		not_done = 0;
		break;
	    }
	    /* do traditional way */
	    roc_batch_train(aroc,
			    error_line.desired_class, error_line.outputs);
	    /* do the new way */
	    desired_outputs[error_line.desired_class] = 1.;
	    roc_cost_train(error_line.outputs, desired_outputs);
	    desired_outputs[error_line.desired_class] = 0.;
	}
	if(not_done == 0) break;
	printf("std area for epoch %d: %f\n",
	       epochs, roc_get_area(aroc, target, reject_rate));
	fprintf(std_file, "ROC TABLE FOR EPOCH %d\n", epochs);
	roc_param(aroc, &roc_struct);
	roc_batch_compute(aroc, target, reject_rate);
	for(i = roc_struct->table_length-1; i >= 0; i--){
	    fprintf(std_file, "%f %f\n",
		    roc_struct->false_alarm[i], roc_struct->detect_rate[i]);
	}
	roc_free(aroc);
	aroc = roc_build(noutputs, npatterns);

	roc_cost_finish_epoch();
	printf("new area for epoch %d: %f\n",
	       epochs, roc_cost_get_area(target));
	fprintf(new_file, "NEW ROC TABLE FOR EPOCH %d\n", epochs);
	roc_print_values(target, new_file);
	fflush(stdout);
	/* print the roc derivatives for each output value */
	/* each of the noutputs roc curves has two derivative curves,
	   one for true hits, one for false alarms */
	desired_outputs[target] = 1.;
	for(x = 0.; x <= 1.; x += .01){
	    for(i = 0; i < noutputs; i++) outputs[i] = x;
	    fprintf(deriv_file, "%f ", x);
	    calc_output_deriv(desired_outputs, outputs, .1, derivative,
			      noutputs, SIGM, 1., ROC_COST);
	    for(i = 0; i < noutputs; i++){
		fprintf(deriv_file, "%f  ", derivative[i]);
	    }
	    fprintf(deriv_file, "\n");
	}
	/* check for RMS error cost, as well */
	for(x = 0.; x <= 1.; x += .01){
	    for(i = 0; i < noutputs; i++) outputs[i] = x;
	    fprintf(rms_deriv_file, "%f ", x);
	    calc_output_deriv(desired_outputs, outputs, .1, derivative,
			      noutputs, SIGM, 1., SQ_ERR);
	    for(i = 0; i < noutputs; i++){
		fprintf(rms_deriv_file, "%f  ", derivative[i]);
	    }
	    fprintf(rms_deriv_file, "\n");
	}
	desired_outputs[target] = 0.;
	roc_print_slopes(target, slopes_file);
    }
    nn_fclose(err_file, "error_file", "");
    nn_fclose(std_file, "std_roc_file", "");
    nn_fclose(new_file, "new_roc_file", "");
    nn_fclose(deriv_file, "new_deriv_file", "");
    nn_fclose(rms_deriv_file, "rms_deriv_file", "");
    nn_fclose(slopes_file, "slopes file", "");
    roc_cost_free();
    return 0;
}
