/***--------------------------------------------------------------------------
* NAME:         sort_matrix.c
* PURPOSE:      contains functions which sorts rows of a matrix with ascending
*             or descending order of values of a specified column
*
* SEE ALSO:	
*
* GLOBALS:
*
* DESCRIPTION:
*    Functions in this file
*       sort_matrix_rows             Sorts a matrix with descending order.
------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
/* to add functions which sorts matrices of strings */
#include <string.h>
#endif
#include "nnlib.h"

/*********************************************************************
* NAME: sort_matrix_rows(nrows, ncols, col, matrix)
* DESCRIPTION:
*       sorts the rows of a matrix according to the column given by: 'col' 
*       in a descending order: matrix[n][col] >= matrix[n+1][col]
*************************************************************************/
int sort_matrix_rows(int nrows, int ncols, int col, float **matrix)
{
    int i, j, changes = 0, max;
    float temp;

    if(matrix != NULL){
	/* go through the rows, finding the smallest entry and
	   swapping it in */
	for(i = nrows; i > 0; i--){
	    max = 0;
	    temp = matrix[0][col];
	    /* find the row with the smallest remaining entry */
	    for(j = 1; j < i; j++){
		if(matrix[j][col] <= temp){
		    max = j;
		    temp = matrix[j][col];
		}
	    }
	    if(max != i-1){
		++changes;
		/* swap the row with the smallest entry
		   into the current last place */
		for(j = 0; j < ncols; j++){
		    temp = matrix[max][j];
		    matrix[max][j] = matrix[i-1][j];
		    matrix[i-1][j] = temp;
		}
	    }
	}
	return(changes);
    } else return(-1);
}
