/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#if HAVE_MALLOC_H
#include <malloc.h>
#endif
#include "nnlib.h"

static struct record {
    int start;
    int duration;
    float score;
    char name[20];
};

int
main(void)
{
    int *piv, **pim;
    float *pfv, **pfm;
    double *pdv, **pdm;
    char *pc;
    void **ppv, ***ppm;
    struct record *psv, **psm;
    static char *msg = "cause";
    char dummy[80];
    int i, j, tmp;

    printf("\nTESTIING ALLOCATION\n");
    mallocmap();
    pc = nn_malloc(32, msg);
    printf("\n%x = nn_malloc(32)\n", (unsigned int)pc);
    mallocmap();
    pc = nn_realloc(pc, 64, msg);
    printf("\n%x = nn_realloc(pc, 64)\n", (unsigned int)pc);
    mallocmap();
    pc = nn_realloc(pc, 16, msg);
    printf("\n%x = nn_realloc(pc, 16)\n", (unsigned int)pc);
    mallocmap();
    nn_free(pc, msg);
    printf("\nnn_free(pc)\n");
    mallocmap();

    /* ------------------test integer routines-------------------------- */
    /*  --- test vector routines --- */
    piv = allocate_ivector(16);
    printf("\n%x = allocate_ivector(16)\n", (unsigned int)piv);
    mallocmap();
    /*- grow -*/
    piv = reallocate_ivector(piv, 16, 32);
    printf("\n%x = reallocate_ivector(piv, 16, 32)\n", (unsigned int)piv);
    mallocmap();
    /*- shrink -*/
    piv = reallocate_ivector(piv, 32, 8);
    printf("\n%x = reallocate_ivector(piv, 32, 8)\n", (unsigned int)piv);
    mallocmap();
    /*- delete -*/
    deallocate_ivector(piv);
    printf("\ndeallocate_ivector(piv)\n");
    mallocmap();

    /*  --- test matrix routines --- */
    /*- start from 0  -*/
    pim = allocate_imatrix(0, 32);
    printf("\n%x = allocate_imatrix(0, 32)\n", (unsigned int)pim);
    mallocmap();
    pim = reallocate_imatrix(pim, 0, 32, 16);
    printf("\nadd 16 rows: %x = reallocate_imatrix(pim, 0, 32, 16)\n", (unsigned int)pim);
    mallocmap();
    pim = reallocate_imatrix(pim, 16, 32, 0);
    printf("\nremove 16 rows: %x = reallocate_imatrix(pim, 16, 32, 0)\n", (unsigned int)pim);
    mallocmap();
    pim = reallocate_imatrix_noncontig(pim, 0, 32, 16);
    printf("\nadd 16 rows non-contig: %x = reallocate_imatrix_noncontig(pim, 0, 32, 16)\n", (unsigned int)pim);
    mallocmap();
    deallocate_imatrix(pim);
    printf("\ndeallocate_imatrix(pim)\n");
    mallocmap();
    /*- start from 16 -*/
    pim = allocate_imatrix(16, 32);
    printf("\n%x = allocate_imatrix(16, 32)\n", (unsigned int)pim);
    mallocmap();
    /*- grow -*/
    pim = reallocate_imatrix(pim, 16, 32, 32);
    printf("\nadd 16 rows: %x = reallocate_imatrix(pim, 16, 32, 32)\n", (unsigned int)pim);
    mallocmap();
    /*- shrink -*/
    pim = reallocate_imatrix(pim, 32, 32, 8);
    printf("\nremove 24 rows: %x = reallocate_imatrix(pim, 32, 32, 8)\n", (unsigned int)pim);
    mallocmap();
    /*- grow noncontig -*/
    pim = reallocate_imatrix_noncontig(pim, 8, 32, 40);
    printf("\nadd 32 rows: %x = reallocate_imatrix_noncontig(pim, 8, 32, 40)\n", (unsigned int)pim);
    mallocmap();
    /* shrink noncontig -*/
    pim = reallocate_imatrix_noncontig(pim, 40, 32, 24);
    printf("\nremove 16 rows: %x = reallocate_imatrix_noncontig(pim, 40, 32, 24)\n", (unsigned int)pim);
    mallocmap();
    /* grow noncontig again -*/
    pim = reallocate_imatrix_noncontig(pim, 24, 32, 32);
    printf("\nadd 8 rows: %x = reallocate_imatrix_noncontig(pim, 24, 32, 32)\n", (unsigned int)pim);
    mallocmap();
    /*- delete -*/
    deallocate_imatrix(pim);
    printf("\ndeallocate_imatrix(pim)\n");
    mallocmap();

    /* ------------------test float routines-------------------------- */
    /*  --- test vector routines --- */
    pfv = allocate_vector(16);
    printf("\n%x = allocate_vector(16)\n", (unsigned int)pfv);
    mallocmap();
    /*- grow -*/
    pfv = reallocate_vector(pfv, 16, 32);
    printf("\n%x = reallocate_vector(pfv, 16, 32)\n", (unsigned int)pfv);
    mallocmap();
    /*- shrink -*/
    pfv = reallocate_vector(pfv, 32, 8);
    printf("\n%x = reallocate_vector(pfv, 32, 8)\n", (unsigned int)pfv);
    mallocmap();
    /*- delete -*/
    deallocate_vector(pfv);
    printf("\ndeallocate_vector(pfv)\n");
    mallocmap();

    /*  --- test matrix routines --- */
    /*- start from 0  -*/
    pfm = allocate_matrix(0, 32);
    printf("\n%x = allocate_matrix(0, 32)\n", (unsigned int)pfm);
    mallocmap();
    pfm = reallocate_matrix(pfm, 0, 32, 16);
    printf("\nadd 16 rows: %x = reallocate_matrix(pfm, 0, 32, 16)\n", (unsigned int)pfm);
    mallocmap();
    pfm = reallocate_matrix(pfm, 16, 32, 0);
    printf("\nremove 16 rows: %x = reallocate_matrix(pfm, 16, 32, 0)\n", (unsigned int)pfm);
    mallocmap();
    pfm = reallocate_matrix_noncontig(pfm, 0, 32, 16);
    printf("\nadd 16 rows non-contig: %x = reallocate_matrix_noncontig(pfm, 0, 32, 16)\n", (unsigned int)pfm);
    mallocmap();
    deallocate_matrix(pfm);
    printf("\ndeallocate_matrix(pfm)\n");
    mallocmap();
    /*- start from 16 -*/
    pfm = allocate_matrix(16, 32);
    printf("\n%x = allocate_matrix(16, 32)\n", (unsigned int)pfm);
    mallocmap();
    /*- grow -*/
    pfm = reallocate_matrix(pfm, 16, 32, 32);
    printf("\nadd 16 rows: %x = reallocate_matrix(pfm, 16, 32, 32)\n", (unsigned int)pfm);
    mallocmap();
    /*- shrink -*/
    pfm = reallocate_matrix(pfm, 32, 32, 8);
    printf("\nremove 24 rows: %x = reallocate_matrix(pfm, 32, 32, 8)\n", (unsigned int)pfm);
    mallocmap();
    /*- grow noncontig -*/
    pfm = reallocate_matrix_noncontig(pfm, 8, 32, 40);
    printf("\nadd 32 rows: %x = reallocate_matrix_noncontig(pfm, 8, 32, 40)\n", (unsigned int)pfm);
    mallocmap();
    /*- shrink noncontig -*/
    pfm = reallocate_matrix_noncontig(pfm, 40, 32, 24);
    printf("\nremove 16 rows: %x = reallocate_matrix_noncontig(pfm, 40, 32, 24)\n", (unsigned int)pfm);
    mallocmap();
    /*- grow noncontig again -*/
    pfm = reallocate_matrix_noncontig(pfm, 24, 32, 32);
    printf("\nadd 8 rows: %x = reallocate_matrix_noncontig(pfm, 24, 32, 32)\n", (unsigned int)pfm);
    mallocmap();
    /*- delete -*/
    deallocate_matrix(pfm);
    printf("\ndeallocate_matrix(pfm)\n");
    mallocmap();

    /* ------------------test double routines-------------------------- */
    /*  --- test vector routines --- */
    pdv = allocate_dvector(16);
    printf("\n%x = allocate_dvector(16)\n", (unsigned int)pdv);
    mallocmap();
    /*- grow -*/
    pdv = reallocate_dvector(pdv, 16, 32);
    printf("\n%x = reallocate_dvector(pdv, 16, 32)\n", (unsigned int)pdv);
    mallocmap();
    /*- shrink -*/
    pdv = reallocate_dvector(pdv, 32, 8);
    printf("\n%x = reallocate_dvector(pdv, 32, 8)\n", (unsigned int)pdv);
    mallocmap();
    /*- delete -*/
    deallocate_dvector(pdv);
    printf("\ndeallocate_dvector(pdv)\n");
    mallocmap();

    /*  --- test matrix routines --- */
    /*- start from 0  -*/
    pdm = allocate_dmatrix(0, 32);
    printf("\n%x = allocate_dmatrix(0, 32)\n", (unsigned int)pdm);
    mallocmap();
    pdm = reallocate_dmatrix(pdm, 0, 32, 16);
    printf("\nadd 16 rows: %x = reallocate_dmatrix(pdm, 0, 32, 16)\n", (unsigned int)pdm);
    mallocmap();
    pdm = reallocate_dmatrix(pdm, 16, 32, 0);
    printf("\nremove 16 rows: %x = reallocate_dmatrix(pdm, 16, 32, 0)\n", (unsigned int)pdm);
    mallocmap();
    pdm = reallocate_dmatrix_noncontig(pdm, 0, 32, 16);
    printf("\nadd 16 rows non-contig: %x = reallocate_dmatrix_noncontig(pdm, 0, 32, 16)\n", (unsigned int)pdm);
    mallocmap();
    deallocate_dmatrix(pdm);
    printf("\ndeallocate_dmatrix(pdm)\n");
    mallocmap();
    /*- start from 16 -*/
    pdm = allocate_dmatrix(16, 32);
    printf("\n%x = allocate_dmatrix(16, 32)\n", (unsigned int)pdm);
    mallocmap();
    /*- grow -*/
    pdm = reallocate_dmatrix(pdm, 16, 32, 32);
    printf("\nadd 16 rows: %x = reallocate_dmatrix(pdm, 16, 32, 32)\n", (unsigned int)pdm);
    mallocmap();
    /*- shrink -*/
    pdm = reallocate_dmatrix(pdm, 32, 32, 8);
    printf("\nremove 24 rows: %x = reallocate_dmatrix(pdm, 32, 32, 8)\n", (unsigned int)pdm);
    mallocmap();
    /*- grow noncontig -*/
    pdm = reallocate_dmatrix_noncontig(pdm, 8, 32, 40);
    printf("\nadd 32 rows: %x = reallocate_dmatrix_noncontig(pdm, 8, 32, 40)\n", (unsigned int)pdm);
    mallocmap();
    /*- shrink noncontig -*/
    pdm = reallocate_dmatrix_noncontig(pdm, 40, 32, 24);
    printf("\nremove 16 rows: %x = reallocate_dmatrix_noncontig(pdm, 40, 32, 24)\n", (unsigned int)pdm);
    mallocmap();
    /*- grow noncontig again -*/
    pdm = reallocate_dmatrix_noncontig(pdm, 24, 32, 32);
    printf("\nadd 8 rows: %x = reallocate_dmatrix_noncontig(pdm, 24, 32, 32)\n", (unsigned int)pdm);
    mallocmap();
    /*- delete -*/
    deallocate_dmatrix(pdm);
    printf("\ndeallocate_dmatrix(pdm)\n");
    mallocmap();

    ppv = allocate_pvector(16);
    printf("\n%x = allocate_pvector(16)\n", (unsigned int)ppv);
    mallocmap();
    /*- grow -*/
    ppv = reallocate_pvector(ppv, 16, 32);
    printf("\n%x = reallocate_pvector(ppv, 16, 32)\n", (unsigned int)ppv);
    mallocmap();
    /*- shrink -*/
    ppv = reallocate_pvector(ppv, 32, 8);
    printf("\n%x = reallocate_pvector(ppv, 32, 8)\n", (unsigned int)ppv);
    mallocmap();
    /*- delete -*/
    deallocate_pvector(ppv);
    printf("\ndeallocate_pvector(ppv)\n");
    mallocmap();

    /*  --- test matrix routines --- */
    /*- start from 0  -*/
    ppm = allocate_pmatrix(0, 32);
    printf("\n%x = allocate_pmatrix(0, 32)\n", (unsigned int)ppm);
    mallocmap();
    ppm = reallocate_pmatrix(ppm, 0, 32, 16);
    printf("\nadd 16 rows: %x = reallocate_pmatrix(ppm, 0, 32, 16)\n", (unsigned int)ppm);
    mallocmap();
    ppm = reallocate_pmatrix(ppm, 16, 32, 0);
    printf("\nremove 16 rows: %x = reallocate_pmatrix(ppm, 16, 32, 0)\n", (unsigned int)ppm);
    mallocmap();
    ppm = reallocate_pmatrix_noncontig(ppm, 0, 32, 16);
    printf("\nadd 16 rows non-contig: %x = reallocate_pmatrix_noncontig(ppm, 0, 32, 16)\n", (unsigned int)ppm);
    mallocmap();
    deallocate_pmatrix(ppm);
    printf("\ndeallocate_pmatrix(ppm)\n");
    mallocmap();
    /*- start from 16 -*/
    ppm = allocate_pmatrix(16, 32);
    printf("\n%x = allocate_pmatrix(16, 32)\n", (unsigned int)ppm);
    mallocmap();
    /*- grow -*/
    ppm = reallocate_pmatrix(ppm, 16, 32, 32);
    printf("\nadd 16 rows: %x = reallocate_pmatrix(ppm, 16, 32, 32)\n", (unsigned int)ppm);
    mallocmap();
    /*- shrink -*/
    ppm = reallocate_pmatrix(ppm, 32, 32, 8);
    printf("\nremove 24 rows: %x = reallocate_pmatrix(ppm, 32, 32, 8)\n", (unsigned int)ppm);
    mallocmap();
    /*- grow noncontig -*/
    ppm = reallocate_pmatrix_noncontig(ppm, 8, 32, 40);
    printf("\nadd 32 rows: %x = reallocate_pmatrix_noncontig(ppm, 8, 32, 40)\n", (unsigned int)ppm);
    mallocmap();
    /*- shrink noncontig -*/
    ppm = reallocate_pmatrix_noncontig(ppm, 40, 32, 24);
    printf("\nremove 16 rows: %x = reallocate_pmatrix_noncontig(ppm, 40, 32, 24)\n", (unsigned int)ppm);
    mallocmap();
    /*- grow noncontig again -*/
    ppm = reallocate_pmatrix_noncontig(ppm, 24, 32, 32);
    printf("\nadd 8 rows: %x = reallocate_pmatrix_noncontig(ppm, 24, 32, 32)\n", (unsigned int)ppm);
    mallocmap();
    /*- delete -*/
    deallocate_pmatrix(ppm);
    printf("\ndeallocate_pmatrix(ppm)\n");
    mallocmap();

    /* ------------------test struct routines-------------------------- */
    /*  --- test vector routines --- */
    printf("sizeof record %d\n", sizeof(struct record));
    psv = allocate_structvector(16, sizeof(struct record));
    printf("\n%x = allocate_structvector(16, %d)\n",
	   (unsigned int)psv, sizeof(struct record));
    mallocmap();
    for(i = 0; i < 16; i++){
	psv[i].start = i;
	psv[i].duration = i+10;
	psv[i].score = 2.0*i*i;
	sprintf(dummy, "number: %6d", i);
	strcpy(psv[i].name, dummy);
    }
    /*- grow -*/
    psv = reallocate_structvector(psv, 16, 32, sizeof(struct record));
    printf("\n%x = reallocate_structvector(psv, 16, 32, %d)\n",
	   (unsigned int)psv, sizeof(struct record));
    mallocmap();
    for(i = 16; i < 32; i++){
	psv[i].start = i;
	psv[i].duration = i+10;
	psv[i].score = 2.0*i*i;
	sprintf(dummy, "number: %6d", i);
	strcpy(psv[i].name, dummy);
    }
    /* print out all structures */
    printf("\n\nCONTENTS OF VECTOR OF STRUCTURES\n");
    for(i = 0; i < 32; i++)
      printf("%2d %3d %3d %7.1f %s\n",
	     i, psv[i].start, psv[i].duration, psv[i].score, psv[i].name);

    /*- shrink -*/
    psv = reallocate_structvector(psv, 32, 8, sizeof(struct record));
    printf("\n%x = reallocate_structvector(psv, 32, 8, %d)\n",
	   (unsigned int)psv, sizeof(struct record));
    mallocmap();
    /*- delete -*/
    deallocate_structvector(psv);
    printf("\ndeallocate_structvector(psv)\n");
    mallocmap();

    /*  --- test matrix routines --- */
    /*- start from 0  -*/
    psm = (struct record **)allocate_structmatrix(0, 4, sizeof(struct record));
    printf("\n%x = allocate_structmatrix(0, 4, %d)\n", (unsigned int)psm, sizeof(struct record));
    mallocmap();
    psm = (struct record **)reallocate_structmatrix((void **)psm, 0, 4, 4, sizeof(struct record));
    printf("\nadd 4 rows: %x = reallocate_structmatrix(psm, 0, 4, 4, %d)\n", (unsigned int)psm, sizeof(struct record));
    mallocmap();
    for(i = 0; i < 4; i++)
      for(j = 0; j < 4; j++){
	  tmp = i*4+j;
	  psm[i][j].start = tmp;
	  psm[i][j].duration = tmp*10;
	  psm[i][j].score = 2.0*tmp*tmp;
	  sprintf(dummy, "number: %6d", tmp);
	  strcpy(psm[i][j].name, dummy);
      }
    /* print out all structures */
    printf("\n\nCONTENTS OF MATRIX OF STRUCTURES\n");
    for(i = 0; i < 4; i++)
      for(j = 0; j < 4; j++)
	printf("%2d %2d %3d %3d %7.1f %s\n",
	       i, j, psm[i][j].start, psm[i][j].duration, psm[i][j].score,
	       psm[i][j].name);
    psm = (struct record **)reallocate_structmatrix((void **)psm, 4, 4, 0, sizeof(struct record));
    printf("\nremove 4 rows: %x = reallocate_structmatrix(psm, 4, 4, 0, %d)\n", (unsigned int)psm, sizeof(struct record));
    mallocmap();
    psm = (struct record **)reallocate_structmatrix_noncontig((void **)psm, 0, 4, 4, sizeof(struct record));
    printf("\nadd 4 rows non-contig: %x = reallocate_structmatrix_noncontig(psm, 0, 4, 4)\n", (unsigned int)psm);
    mallocmap();
    for(i = 0; i < 4; i++)
      for(j = 0; j < 4; j++){
	  tmp = i*4+j;
	  psm[i][j].start = tmp;
	  psm[i][j].duration = tmp*10;
	  psm[i][j].score = 2.0*tmp*tmp;
	  sprintf(dummy, "number: %6d", tmp);
	  strcpy(psm[i][j].name, dummy);
      }
    /* print out all structures */
    printf("\n\nCONTENTS OF MATRIX OF STRUCTURES\n");
    for(i = 0; i < 4; i++)
      for(j = 0; j < 4; j++)
	printf("%2d %2d %3d %3d %7.1f %s\n",
	       i, j, psm[i][j].start, psm[i][j].duration, psm[i][j].score,
	       psm[i][j].name);
    deallocate_structmatrix((void **)psm);
    printf("\ndeallocate_structmatrix(psm)\n");
    mallocmap();
    /*- start from 16 -*/
    psm = (struct record **)allocate_structmatrix(4, 4, sizeof(struct record));
    printf("\n%x = allocate_structmatrix(4, 4, %d)\n",
	   (unsigned int)psm, sizeof(struct record));
    mallocmap();
    for(i = 0; i < 4; i++)
      for(j = 0; j < 4; j++){
	  tmp = i*4+j;
	  psm[i][j].start = tmp;
	  psm[i][j].duration = tmp*10;
	  psm[i][j].score = 2.0*tmp*tmp;
	  sprintf(dummy, "number: %6d", tmp);
	  strcpy(psm[i][j].name, dummy);
      }
    /*- grow -*/
    psm = (struct record **)reallocate_structmatrix((void **)psm, 4, 4, 8,
						    sizeof(struct record));
    printf("\nadd 4 rows: %x = reallocate_structmatrix(psm, 4, 4, 8, %d)\n",
	   (unsigned int)psm, sizeof(struct record));
    mallocmap();
    for(i = 4; i < 8; i++)
      for(j = 0; j < 4; j++){
	  tmp = i*4+j;
	  psm[i][j].start = tmp;
	  psm[i][j].duration = tmp*10;
	  psm[i][j].score = 2.0*tmp*tmp;
	  sprintf(dummy, "number: %6d", tmp);
	  strcpy(psm[i][j].name, dummy);
      }
    /* print out all structures */
    printf("\n\nCONTENTS OF MATRIX OF STRUCTURES\n");
    for(i = 0; i < 8; i++)
      for(j = 0; j < 4; j++)
	printf("%2d %2d %3d %3d %7.1f %s\n",
	       i, j, psm[i][j].start, psm[i][j].duration, psm[i][j].score,
	       psm[i][j].name);
    /*- shrink -*/
    psm = (struct record **)reallocate_structmatrix((void **)psm, 8, 4, 2,
						    sizeof(struct record));
    printf("\nremove 6 rows: %x = reallocate_structmatrix(psm, 8, 4, 2, %d)\n",
	   (unsigned int)psm, sizeof(struct record));
    mallocmap();
    psm = (struct record **)
      reallocate_structmatrix_noncontig((void **)psm, 2, 4, 12, sizeof(struct record));
    printf("\nadd 10 rows: %x = reallocate_structmatrix_noncontig(psm, 2, 4, 12, %d)\n",
	   (unsigned int)psm, sizeof(struct record));
    mallocmap();
    for(i = 2; i < 12; i++)
      for(j = 0; j < 4; j++){
	  tmp = i*4+j+1;
	  psm[i][j].start = tmp;
	  psm[i][j].duration = (tmp)*10;
	  psm[i][j].score = 2.0*(tmp)*(tmp);
	  sprintf(dummy, "number: %6d", i*4+j);
	  strcpy(psm[i][j].name, dummy);
      }
    /* print out all structures */
    printf("\n\nCONTENTS OF MATRIX OF STRUCTURES\n");
    for(i = 0; i < 12; i++)
      for(j = 0; j < 4; j++)
	printf("%2d %2d %3d %3d %7.1f %s\n",
	       i, j, psm[i][j].start, psm[i][j].duration, psm[i][j].score,
	       psm[i][j].name);
    psm = (struct record **)
      reallocate_structmatrix_noncontig((void **)psm, 12, 4, 8, sizeof(struct record));
    printf("\nremove 4 rows: %x = reallocate_structmatrix_noncontig(psm, 12, 4, %d)\n",
	   (unsigned int)psm, sizeof(struct record));
    mallocmap();
    /*- grow -*/
    psm = (struct record **)
      reallocate_structmatrix_noncontig((void **)psm, 8, 4, 10, sizeof(struct record));
    printf("\nadd 2 rows: %x = reallocate_structmatrix_noncontig(psm, 8, 4, 10, %d)\n",
	   (unsigned int)psm, sizeof(struct record));
    mallocmap();
    /*- delete -*/
    deallocate_structmatrix((void **)psm);
    printf("\ndeallocate_structmatrix(psm)\n");
    mallocmap();

    return(0);
}
