/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <nnlib.h>

/***---------------------------------------------------------------------------
* NAME:		diff_screen
* PURPOSE:	find differences between two screen files, printing them
*		to the FILE p;
* EXAMPLE:
*	FILE *p;
*	diff_screen(p, "X1mlp.screen", "LNKnet.note.log" );
* NOTE:
--------------------------------------------------------------------------***/
void
diff_screen(FILE *fp, char *fname1, char *fname2)
{
    FILE *p1, *p2;
    char *argv1[256], *argv2[256];
    int argc1, argc2, i;

    p1 = nn_fopen(fname1, "r", "in diff screen");
    p2 = nn_fopen(fname2, "r", "in diff screen");

    while (1){
	/* while there are commands left in the files */
	argc1 = fill_argv(argv1, p1, fname1);
	if (argc1 == EOF) break;
	/* copy things in argv1 to make it safe to call fill_argv again */
	for(i = 0; i < argc1; i++){
	    if((argv1[i] = nn_strdup(argv1[i])) == NULL)
	      fatal_error("%s: Cannot allocate space while copying argv1",
			  "diff_screen");
	}
	/* add a final argument to help the checking */
	if((argv1[argc1] = nn_strdup("FINAL_MATCH")) == NULL)
	  fatal_error("%s: Cannot allocate space while copying argv1",
		      "diff_screen");
	argc1++;

	argc2 = fill_argv(argv2, p2, fname2);
	if (argc2 == EOF) break;
	/* copy things in argv2 out of habit */
	for(i = 0; i < argc2; i++){
	    if((argv2[i] = nn_strdup(argv2[i])) == NULL)
	      fatal_error("%s: Cannot allocate space while copying argv1",
			  "diff_screen");
	}
	/* add a final argument to help the checking */
	if((argv2[argc2] = nn_strdup("FINAL_MATCH")) == NULL)
	  fatal_error("%s: Cannot allocate space while copying argv2",
		      "diff_screen");
	argc2++;

	/* find the differences between the arglists */
	if(strcmp(argv1[0], argv2[0])){
	    /* the argument names don't match. there is a problem with
	       the log file */
	    fprintf(fp, "log file commands out of synch %s %s",
		    argv1[0], argv2[0]);
	    for (i = 0; i < argc1; i++) free(argv1[i]);
	    for (i = 0; i < argc2; i++) free(argv2[i]);
	    break;
	} else {
	    diff_argv(fp, argc1, argv1, argc2, argv2);
	    fflush(fp);
	    /* I'd like to put a check in here to see if this is one
	       of the arg tables we actally care about */
	    /* free argv strings that we dup'ed */
	    for (i = 0; i < argc1; i++) free(argv1[i]);
	    for (i = 0; i < argc2; i++) free(argv2[i]);
	}
    }
}

void
main(int argc, char **argv)
{
    if(argc != 3){
	fprintf(stderr, "Usage: %s file1 file2\n", argv[0]);
	exit(-1);
    }
    diff_screen(stdout, argv[1], argv[2]);
    printf("\n");
    exit(0);
}
