/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

/*
 * Copyright 1989 by Metis Technology, Inc.
 *
 *                        All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software and its 
 * documentation for any purpose and without fee is hereby granted, 
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in 
 * supporting documentation.
 *
 * METIS TECHNOLOGY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL METIS TECHNOLOGY BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
 * USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 */

/*
 * Plot Widget - display standard UNIX plot(5) format file
 */
/*
 * 
 * Revision 1.3  95/02/15  16:25:15  kukolich
 * redid memory allocation of plot command buffer
 * buffers are allocated in pieces now, when a buffer is full a new buffer
 * is allocated and added to a list of buffers. If that list is ever
 * filled, (1024x1024 commands in the plot) the list is reallocated but the
 * piecewise buffers stay where they are.
 * I also changed the arguments to some of the commands that handle the
 * allocation and freeing of the command buffer
 * We also found a major memory leak. Strings allocated by GetString
 * in PlotCommand were never freed. The pointers were lost immediately.
 * Finally, the plot displays each file while it is being read, rather
 * than waiting until the whole file is in memory before starting the display
 * 
 */

/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <ctype.h>
#include <stdio.h>
#include <setjmp.h>

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <X11/Xatom.h>

#include "PlotP.h"
#include "PlotI.h"

#define offset(field) XtOffset(PlotWidget, field)

static XtResource resources[] = {
  { XtNbatchGraphics, XtCBatchGraphics, XtRBoolean, sizeof(Boolean),
      offset(plot.batch_graphics), XtRImmediate, (caddr_t) True },
  { XtNforeground, XtCForeground, XtRPixel, sizeof(Pixel),
      offset(plot.foreground), XtRString, "XtDefaultForeground" },
  { XtNfont, XtCFont, XtRFontStruct, sizeof (XFontStruct *),
      offset(plot.font), XtRString, "XtDefaultFont" },
  { XtNfile, XtCFile, XtRString, sizeof (String),
      offset(plot.file), XtRString, NULL},
  { XtNplotWidth, XtCPlotWidth, XtRInt, sizeof(int),
      offset(plot.plot_width), XtRImmediate, (caddr_t) DEF_PLOT_WIDTH },
  { XtNplotHeight, XtCPlotHeight, XtRInt, sizeof(int),
      offset(plot.plot_height), XtRImmediate, (caddr_t) DEF_PLOT_HEIGHT },
  { XtNhorizMargin, XtCMargin, XtRDimension, sizeof (Dimension),
      offset(plot.horiz_margin), XtRImmediate, (caddr_t) DEF_HORIZ_MARGIN },
  { XtNvertMargin, XtCMargin, XtRDimension, sizeof (Dimension),
      offset(plot.vert_margin), XtRImmediate, (caddr_t) DEF_VERT_MARGIN },

};

#undef offset

static void Initialize();
static void Realize();
static void Resize();
static void Redisplay();
static Boolean  SetValues();
static void Destroy();
static XtGeometryResult QueryGeometry();

PlotClassRec plotClassRec = {
  { /* core fields */
#define superclass          (&widgetClassRec)
    /* superclass       */  (WidgetClass) &widgetClassRec,
    /* class_name       */  "Plot",
    /* widget_size      */  sizeof(PlotRec),
    /* class_initialize     */  NULL,
    /* class_part_initialize    */  NULL,
    /* class_inited     */  FALSE,
    /* initialize       */  Initialize,
    /* initialize_hook      */  NULL,
    /* realize          */  Realize,
    /* actions          */  NULL,
    /* num_actions      */  0,
    /* resources        */  resources,
    /* num_resources        */  XtNumber(resources),
    /* xrm_class        */  NULLQUARK,
    /* compress_motion      */  TRUE,
    /* compress_exposure    */  TRUE,
    /* compress_enterleave  */  TRUE,
    /* visible_interest     */  TRUE,
    /* destroy          */  Destroy,
    /* resize           */  Resize,
    /* expose           */  Redisplay,
    /* set_values       */  SetValues,
    /* set_values_hook      */  NULL,
    /* set_values_almost    */  XtInheritSetValuesAlmost,
    /* get_values_hook      */  NULL,
    /* accept_focus     */  NULL,
    /* version          */  XtVersion,
    /* callback_private     */  NULL,
    /* tm_table         */  NULL,
    /* query_geometry       */  QueryGeometry,
    /* display_accelerator  */  XtInheritDisplayAccelerator,
    /* extension        */  NULL
  }
};

WidgetClass plotWidgetClass = (WidgetClass) &plotClassRec;

typedef struct _keyent {
  char *keyword;
  int   value;
} KeyTabEnt, *KeyTab;

static KeyTabEnt LineModeKeyTab[] =
{
  { "dotted",      PLOT_LINEMODE_DOTTED },
  { "solid",       PLOT_LINEMODE_SOLID  },
  { "longdashed",  PLOT_LINEMODE_LONGDASHED },
  { "shortdashed", PLOT_LINEMODE_SHORTDASHED },
  { "dotdashed",   PLOT_LINEMODE_DOTDASHED },
  { (char *) NULL, 0 }
};

static KeyTabEnt FillModeKeyTab[] =
{
  { "transparent", PLOT_FILLMODE_TRANSPARENT    },
  { "opaque",      PLOT_FILLMODE_OPAQUE     },
  { (char *) NULL, 0 }
};

static char ThrowReason[1024];  /* Throw Reason */
static jmp_buf  ThrowEnv;       /* Throw Environment */

static void
InitFontInfo(w)
     Widget w;
{
  register PlotWidget pw = (PlotWidget) w;
  register XFontStruct *font;
  int quad_width;

  /*
   * Use QUAD_WIDTH if available, otherwise use Space width.
   */
  font = pw->plot.font;
  if (!XGetFontProperty(font, XA_QUAD_WIDTH, (unsigned long *) &quad_width)) {
    if ((quad_width = XTextWidth(font, " ", 1)) == 0)
      XtWarning("GetFontInfo: zero space width");
  }
  pw->plot.quad_width = quad_width;
  pw->plot.fillmode = PLOT_FILLMODE_TRANSPARENT;
}
  

static void
InitGraphics(w)
     Widget w;
{
  register PlotWidget pw = (PlotWidget) w;
  Display *dpy;
  Screen *screen;
  Drawable drawable;
  unsigned valuemask;
  XGCValues values;

  /* Get a drawable to create private GC. */

  dpy = XtDisplay(w);
  screen = XtScreen(w);
  if (XtWindow(w) == (Window)NULL) {
    if (w->core.depth == DefaultDepthOfScreen(screen))
      drawable = RootWindowOfScreen(screen);
    else
      drawable = XCreatePixmap(dpy, screen->root, 1, 1, w->core.depth);
  } else
    drawable = (Drawable) XtWindow(w);

  /* Use Private GC for foreground drawing. */

  valuemask     = GCForeground | GCBackground | GCFont;
  values.foreground = pw->plot.foreground;
  values.background = pw->core.background_pixel;
  values.font       = pw->plot.font->fid;
  pw->plot.drawGC   = XCreateGC(dpy, drawable, valuemask, &values);

  /* Use Shared GC for erasing. */

  valuemask     = GCForeground | GCBackground;
  values.foreground = pw->core.background_pixel;
  values.background = pw->core.background_pixel;
  pw->plot.eraseGC  = XtGetGC(w, valuemask, &values);

  if (pw->plot.batch_graphics) {
    pw->plot.batch_buf     = XtMalloc((unsigned) DEF_BATCH_BUF_SIZE);
    pw->plot.batch_bufsize = DEF_BATCH_BUF_SIZE;
    pw->plot.batch_entries = 0;
    pw->plot.batch_state   = BATCH_NONE;
  }

  pw->plot.overlay_mode = False;
}

static void
InitSource(w)
     Widget w;
{
  register PlotWidget pw = (PlotWidget) w;

  if ( pw->plot.file ) {
    pw->plot.file = XtNewString ( pw->plot.file );
    if ( ( pw->plot.fp = fopen ( pw->plot.file, "r" ) ) == 0)
      XtError ( "Can't open plot source file" );
  }
  pw->plot.plotbuflist = (PlotCmdBuf *) NULL;

}

static void
UpdateCTM(w)
     Widget w;
{
  register PlotWidget pw = (PlotWidget) w;

  pw->plot.scale_x = ((double)pw->core.width / (double)pw->plot.space_width);

  pw->plot.scale_y = -1.0 *
    ((double)pw->core.height / (double)pw->plot.space_height);

  pw->plot.disp_x  = -1.0 * 
    ((double)pw->plot.space_disp_x * (double)pw->core.width) /
      ((double)pw->plot.space_width);
          
  pw->plot.disp_y  = (double)pw->core.height +
    (((double)pw->plot.space_disp_y * (double)pw->core.height) /
     ((double)pw->plot.space_height));
}   


static void
SetSpace(w, width, height, x, y)
     Widget w;
     short width, height, x, y;
{
  register PlotWidget pw = (PlotWidget) w;

  pw->plot.space_width  = width;
  pw->plot.space_height = height;
  pw->plot.space_disp_x = x;
  pw->plot.space_disp_y = y;
  
  UpdateCTM(w);
}

static void
InitSpace(w)
     Widget w;
{
  register PlotWidget pw = (PlotWidget) w;

  SetSpace(w, pw->plot.plot_width, pw->plot.plot_height, 0, 0);
  pw->core.width = pw->plot.plot_width;
  pw->core.height = pw->plot.plot_height;
}

/* ARGSUSED */
static void
Initialize(request, new)
     Widget request, new;
{
  InitFontInfo(new);
  InitGraphics(new);
  InitSource(new);
  InitSpace(new);
}

static void
Realize(w, valueMask, attributes)
     register Widget w;
     Mask *valueMask;
     XSetWindowAttributes *attributes;
{

  *valueMask |= CWBitGravity;
  attributes->bit_gravity = ForgetGravity;

  XtCreateWindow(w, (unsigned) InputOutput, (Visual *) CopyFromParent,
         *valueMask, attributes);
}

static char
GetCommand(fp)
     FILE *fp;
{
  register int c;

  if ((c = fgetc(fp)) == EOF) {
    if (ferror(fp))
      XtError("plot read command: I/O Error");
    return PLOT_END;
  }
  return (char) c;
}


static short
GetShort(fp)
     FILE *fp;
{
  register int c;
  register unsigned short s;
  
  if ((c = fgetc(fp)) != EOF) {
    s = (unsigned short) c;
    if ((c = fgetc(fp)) != EOF) {
      s |= ((unsigned short) c) << 8;
      return s;
    }
  }
  if (ferror(fp))
    XtError("plot command read: I/O error");
  else
    XtError("plot command read: premature EOF");
  return -1;
}

static String
GetString(fp, len)
     FILE *fp;
     int *len;
{
  char buf[1024];
  register char *cp = buf;
  register int c, cc = 0;

  while ((c = fgetc(fp)) != EOF) {
    if (cp >= &buf[1024])
      XtError("plot command read: string too long");
    if ((char) c == '\n' || (char) c == '\0') {
      *cp = '\0';
      *len = cc;
      return XtNewString(buf);
    }
    *cp++ = (char) c, cc++;
  }
  if (ferror(fp))
    XtError("plot command read: I/O error");
  else
    XtError("plot command read: premature EOF");
  *len = 0;
  return (String) NULL;
}

static void
TransformPoint(pw, x, y, point)
     register PlotWidget pw;
     register short x, y;
     register XPoint *point;
{
  point->x = (short) (((double) x * pw->plot.scale_x) + pw->plot.disp_x);
  point->y = (short) (((double) y * pw->plot.scale_y) + pw->plot.disp_y);
}

#define ABS(a) ((a) >= 0 ? (a) : -(a))
   
static void
TransformCircle(pw, x, y, r, arc)
     register PlotWidget pw;
     short x, y, r;
     register XArc *arc;
{
  double maj, min, center_x, center_y;

  maj = (double) r * pw->plot.scale_x;
  min = (double) r * ABS(pw->plot.scale_y);
  center_x = ((double) x * pw->plot.scale_x) + pw->plot.disp_x;
  center_y = ((double) y * pw->plot.scale_y) + pw->plot.disp_y;
  arc->x = (short) (center_x - maj);
  arc->y = (short) (center_y - min);
  arc->width = (short) (2.0 * maj);
  arc->height = (short) (2.0 * min);
  arc->angle1 = 0;
  arc->angle2 = 360 * 64;
#ifdef notdef
  fprintf(stderr, "TransformCircle: x %d, y %d, r %d\n", x, y, r);
  fprintf(stderr, "TransformCircle: arc x %d, y %d, w %d, h %d\n",
      arc->x, arc->y, arc->width, arc->height);
#endif
}

static int
FindKeyword(name, keytab)
     char *name;
     KeyTab  keytab;
{
  register unsigned char *cp, *bp;
  register KeyTabEnt *kp;

  cp = bp = XtNewString(name);
  while (*cp++) {
    if (isascii(*cp) && isupper(*cp))
      *cp = tolower(*cp);
  }
  cp = bp;
  for (kp = keytab; kp->keyword != (char *) NULL; kp++)
    if (strcmp(kp->keyword, cp) == 0)
      break;
  XtFree(bp);
  return kp->value;
}

static char *
FindKeywordName(keytab, value)
     KeyTab keytab;
     int value;
{
  register KeyTabEnt *kp;

  for (kp = keytab; kp->keyword; kp++) {
    if (kp->value == value)
      return XtNewString(kp->keyword);
  }
  return (char *) NULL;
}

static void
CreatePlotBuf(pw)
     PlotWidget pw;
{
  pw->plot.plotbuflist =
    (PlotCmdBuf *) XtCalloc((unsigned) DEF_PLOT_BUF_SIZE,
                (unsigned) sizeof (PlotCmdBuf));
  pw->plot.plotlistlength = DEF_PLOT_BUF_SIZE;
  pw->plot.plotlistsize = pw->plot.plotbuflength = pw->plot.plotbufsize = 0;
}

static int
FullPlotBuf(pw)
     PlotWidget  pw;
{
  return (pw->plot.plotbufsize >= pw->plot.plotbuflength);
}

static void
ExtendPlotBuf(pw)
     PlotWidget pw;
{
  if (pw->plot.plotlistsize >= pw->plot.plotlistlength){
      pw->plot.plotlistlength += DEF_PLOT_BUF_SIZE;
      pw->plot.plotbuflist = (PlotCmdBuf *)
    XtRealloc((char *) pw->plot.plotbuflist,
          (unsigned) pw->plot.plotlistlength *
          (unsigned) sizeof (PlotCmdBuf));
  }
  pw->plot.plotbuflength = DEF_PLOT_BUF_SIZE;
  pw->plot.plotbuflist[pw->plot.plotlistsize++] =
    (PlotCmdBuf)XtCalloc((unsigned) DEF_PLOT_BUF_SIZE,
             (unsigned) sizeof (PlotCmd));
  pw->plot.plotbufsize = 0;
}

static void
FreePlot(pw)
     register PlotWidget pw;
{
  register PlotCmdBuf *plotbuflist, plotbuf, pc;
  register int curbuflength, plotbuflength, i;

  if (! pw) return;

  plotbuflist = pw->plot.plotbuflist;
  curbuflength = pw->plot.plotbufsize;
  plotbuflength = pw->plot.plotbuflength;

  /* free the partially filled final buffer, first
     then free the fully filled other buffers. */
  for (i = pw->plot.plotlistsize-1; i >= 0; i--){
      plotbuf = plotbuflist[i];
      if ( ! plotbuf ) break;
      for (pc = plotbuf; pc - plotbuf < curbuflength &&
       pc->command != PLOT_END; pc++)
    switch (pc->command) {
      case PLOT_LABEL:
        XtFree(pc->args.label.label);
        break;
      case PLOT_LINEMODE:
        /* this is an integer, dont free it! */
        /*XtFree(pc->args.line_mode.mode);*/
        break;
#ifdef EXTENDED
      case PLOT_EXT_COLOR:
        XtFree(pc->args.color.color);
        break;
      case PLOT_EXT_FONT:
        XtFree(pc->args.font.font);
        break;
#endif /* EXTENDED */
      default:
        break;
    }
      curbuflength = plotbuflength;
      XtFree((char *) plotbuf);
  }

  XtFree((char *) plotbuflist);
}

static void
FlushBatch(pw)
     register PlotWidget pw;
{
  Display *dpy = XtDisplay((Widget) pw);
  Window win = XtWindow((Widget) pw);
  GC gc = pw->plot.drawGC;

  if (!pw->plot.batch_graphics) /* should never happen */
    XtError("FlushBatch: batching disabled!!");

  switch (pw->plot.batch_state) {

  case BATCH_NONE:      /* should never get here */
    XtError("FlushBatch: flushing before batched!!");
    break;

  case BATCH_LINES:
    XDrawLines(dpy, win, gc,
           (XPoint *) pw->plot.batch_buf, pw->plot.batch_entries,
           CoordModeOrigin);
    break;

  case BATCH_POINTS:
    XDrawPoints(dpy, win, gc,
        (XPoint *) pw->plot.batch_buf, pw->plot.batch_entries,
        CoordModeOrigin);
    break;

  case BATCH_ARCS:
    XDrawArcs(dpy, win, gc,
          (XArc *) pw->plot.batch_buf, pw->plot.batch_entries);
    break;

  case BATCH_POLY:
    XFillPolygon(dpy, win, gc,
        (XPoint *) pw->plot.batch_buf, pw->plot.batch_entries,
        Complex, CoordModeOrigin );
    break;

  }

  pw->plot.batch_state = BATCH_NONE;
  pw->plot.batch_entries = 0;

}

static void
BatchPlotCmd(pw, pc)
     register PlotWidget pw;
     register PlotCmdBuf pc;
{
  register int entries = pw->plot.batch_entries;
  register int bufsize = pw->plot.batch_bufsize;

  switch (pc->command) {
  case PLOT_CONT:
  case PLOT_LINE:
    {
      register XPoint *points = (XPoint *) pw->plot.batch_buf;
      
      if (((&points[entries + 1] - points) * sizeof (XPoint)) > bufsize)
    FlushBatch(pw), entries = pw->plot.batch_entries;
      if (pc->command == PLOT_CONT) {
    points[entries] = pw->plot.currentpoint, entries++;
    TransformPoint(pw,
               pc->args.cont.x0,
               pc->args.cont.y0,
               &points[entries]);
    pw->plot.currentpoint = points[entries], entries++;
      } else {
    TransformPoint(pw,
               pc->args.line.x0,
               pc->args.line.y0,
               &points[entries]), entries++;
    TransformPoint(pw,
               pc->args.line.x1,
               pc->args.line.y1,
               &points[entries]);
    pw->plot.currentpoint = points[entries], entries++;
      }
      pw->plot.batch_state = BATCH_LINES;
      break;
    }
  case PLOT_POINT:
    {
      register XPoint *points = (XPoint *) pw->plot.batch_buf;
      
      if (((&points[entries] - points) * sizeof (XPoint)) > bufsize)
    FlushBatch(pw), entries = pw->plot.batch_entries;
      TransformPoint(pw,
             pc->args.point.x0,
             pc->args.point.y0,
             &points[entries]);
      pw->plot.currentpoint = points[entries], entries++;
      if ( pw->plot.batch_state != BATCH_POLY )
    pw->plot.batch_state = BATCH_POINTS;
      break;
    }
  case PLOT_ARC:
    pw->plot.batch_state = BATCH_ARCS;
    break;
  }
  pw->plot.batch_entries = entries;
}


static void
PlotCommand(w, pc)
     Widget w;
     register PlotCmdBuf pc;
{
  register PlotWidget pw = (PlotWidget) w;
  register int batch;
  Display *dpy;
  Window win;
  GC gc;

  dpy = XtDisplay(w);
  win = XtWindow(w);
  gc  = pw->plot.drawGC;

  /*
   * Graphics Batching FSM (should be table driven)
   */
  if ((batch = pw->plot.batch_state) != BATCH_NONE) {
    switch (pc->command) {
#ifdef notdef
    case PLOT_MOVE:
      if (batch == BATCH_SEGMENTS) {
    BatchPlotCmd(pw, pc);
    return;
      }
      break;
#endif
    case PLOT_CONT:
      if (batch == BATCH_LINES) {
    BatchPlotCmd(pw, pc);
    return;
      }
      break;
    case PLOT_POINT:
      if ((batch == BATCH_POINTS) || (batch == BATCH_POLY)) {
    BatchPlotCmd(pw, pc);
    return;
      }
      break;
    case PLOT_ARC:
      if (batch == BATCH_ARCS) {
    BatchPlotCmd(pw, pc);
    return;
      }
      break;
    }
    FlushBatch(pw);
  }
  
  if (pw->plot.batch_graphics) {
    switch (pc->command) {
#ifdef notdef
    case PLOT_MOVE:
#endif
    case PLOT_CONT:
    case PLOT_LINE:
    case PLOT_POINT:
    case PLOT_ARC:
      BatchPlotCmd(pw, pc);
      return;
#ifdef EXTENDED
    case PLOT_EXT_POLY:
      pw->plot.batch_state = BATCH_POLY;
      return;
#endif
    }
  }
  
  switch (pc->command) {
    
    /* Standard Plot Commands */
    
  case PLOT_MOVE:
    {
      XPoint newpoint;
      
      TransformPoint(pw, pc->args.move.x0, pc->args.move.y0, &newpoint);
      pw->plot.currentpoint = newpoint;
      break;
    }
    
  case PLOT_CONT:
    {
      XPoint point[2];
      
      point[0]   = pw->plot.currentpoint;
      TransformPoint(pw, pc->args.cont.x0, pc->args.cont.y0, &point[1]);
      XDrawLine(dpy, win, gc,
        point[0].x, point[0].y,
        point[1].x, point[1].y);
      pw->plot.currentpoint = point[1];
      break;
    }
    
  case PLOT_POINT:
    {
      XPoint point[1];
      
      TransformPoint(pw, pc->args.point.x0, pc->args.point.y0, &point[0]);
      XDrawPoint(dpy, win, gc, point[0].x, point[0].y);
      pw->plot.currentpoint = point[0]; 
      break;
    }
    
  case PLOT_LINE:
    {
      XPoint point[2];
      
      TransformPoint(pw, pc->args.line.x0, pc->args.line.y0, &point[0]);
      TransformPoint(pw, pc->args.line.x1, pc->args.line.y1, &point[1]);
      XDrawLine(dpy, win, gc,
        point[0].x, point[0].y,
        point[1].x, point[1].y);
      pw->plot.currentpoint = point[1];
      break;
    }
    
  case PLOT_LABEL:
    {
      register char *text, *cp;
      int cc;
      XPoint start, end;
      
      start = pw->plot.currentpoint;
      text = pc->args.label.label;
      cc = pc->args.label.length;
      for (cp = text; cp && *cp == '\b'; cp++, cc--)
    start.x -= pw->plot.quad_width;
      switch ( pw->plot.fillmode ) {
      case PLOT_FILLMODE_OPAQUE:
    XDrawImageString(dpy, win, gc, start.x, start.y, cp, cc);
    break;
      case PLOT_FILLMODE_TRANSPARENT:
    XDrawString(dpy, win, gc, start.x, start.y, cp, cc);
    break;
      }
      end.x = start.x + XTextWidth(pw->plot.font, cp, cc);
      end.y = start.y;
      pw->plot.currentpoint = end;
      break;
    }
    
  case PLOT_ARC:
    {
      break;
    }
    
  case PLOT_CIRCLE:
    {
      XArc arc;
      XPoint end;
      
      TransformCircle(pw,
              pc->args.circle.center_x,
              pc->args.circle.center_y,
              pc->args.circle.radius,
              &arc);
      XDrawArcs(dpy, win, gc, &arc, 1);
      end.x = arc.x + arc.width;
      end.y = arc.y + (arc.height >> 1);
      pw->plot.currentpoint = end;
      break;
    }
    
  case PLOT_ERASE:
    {
      if ( ! pw->plot.overlay_mode ) {
    XClearWindow(dpy, win);
    XSetForeground(dpy, gc, pw->plot.foreground);
      }
      break;
    }
    
  case PLOT_LINEMODE:
    break;
    
  case PLOT_SPACE:
    SetSpace(w,
         pc->args.space.x1 - pc->args.space.x0,
         pc->args.space.y1 - pc->args.space.y0,
         pc->args.space.x0,
         pc->args.space.y0);
    break;
    
    /* Extended Plot Commands */
    
#ifdef EXTENDED

  case PLOT_EXT_BITMAP:
    break;

  case PLOT_EXT_COLOR:
    {
      register unsigned long pixel;
      XColor allowed, exact;
      
      if ((pixel = pc->args.color.pixel) == (unsigned long) -1) {
    if (strcmp(pc->args.color.color, "foreground")  == 0)
      pixel = pw->plot.foreground;
    else if (strcmp(pc->args.color.color, "background")  == 0)
      pixel = pw->core.background_pixel;
    else if ( XAllocNamedColor( dpy,
                    DefaultColormapOfScreen(XtScreen(w)),
                    pc->args.color.color,
                    &allowed,
                    &exact ) )
      pixel = allowed.pixel;
    else
      pixel = pw->plot.foreground;
      }
      XSetForeground(dpy, gc, pixel);
      pc->args.color.pixel = pixel;
      break;
    }
    
  case PLOT_EXT_DASH:
    break;

  case PLOT_EXT_FONT:
    break;

  case PLOT_EXT_FILL:
    pw->plot.fillmode = pc->args.fill_mode.mode;
    break;

  case PLOT_EXT_POLY:
    XtWarning("PlotCommand: polygons supported in batch mode only!");
    break;

  case PLOT_EXT_POLYDRAW:
    break;

#endif /* EXTENDED */
    
  default:
    break;
    
  } /* switch */
  
}

static void
ReadPlot(fp, pw)
     register FILE *fp;
     register PlotWidget pw;
{
  PlotCmdBuf pc;
  int dummy;

  if(!pw) return;

  if ( !pw->plot.plotbuflist )
    CreatePlotBuf(pw);
  else
    pc =
      &(pw->plot.plotbuflist[pw->plot.plotlistsize-1][pw->plot.plotbufsize]);
  
  while (1) {
    /*
     * Grow plot buffer as required.
     */
    if(FullPlotBuf(pw)) {
    /* this will be true for a new plot buffer list */
    ExtendPlotBuf(pw);
    pc = pw->plot.plotbuflist[pw->plot.plotlistsize-1];
    }
       
    pc->command = GetCommand(fp);

    switch (pc->command) {
    case PLOT_END:
      return;
    case PLOT_MOVE:
      pc->args.move.x0         = GetShort(fp);
      pc->args.move.y0         = GetShort(fp);
      break;
    case PLOT_CONT:
      pc->args.cont.x0         = GetShort(fp);
      pc->args.cont.y0         = GetShort(fp);
      break;
    case PLOT_POINT:
      pc->args.point.x0        = GetShort(fp);
      pc->args.point.y0        = GetShort(fp);
      break;
    case PLOT_LINE:
      pc->args.line.x0         = GetShort(fp);
      pc->args.line.y0         = GetShort(fp);
      pc->args.line.x1         = GetShort(fp);
      pc->args.line.y1         = GetShort(fp);
      break;
    case PLOT_LABEL:
      pc->args.label.label     = GetString(fp, &pc->args.label.length);
      break;
    case PLOT_ARC:
      pc->args.arc.center_x    = GetShort(fp);
      pc->args.arc.center_y    = GetShort(fp);
      pc->args.arc.start_x     = GetShort(fp);
      pc->args.arc.start_y     = GetShort(fp);
      pc->args.arc.end_x       = GetShort(fp);
      pc->args.arc.end_y       = GetShort(fp);
      break;
    case PLOT_CIRCLE:
      pc->args.circle.center_x = GetShort(fp);
      pc->args.circle.center_y = GetShort(fp);
      pc->args.circle.radius   = GetShort(fp);
      break;
    case PLOT_ERASE:
      break;
    case PLOT_LINEMODE:
      {
    char *mode;

    mode = GetString(fp, &dummy);
    pc->args.line_mode.mode = FindKeyword(mode, LineModeKeyTab);
    XtFree(mode);
    break;
      }
    case PLOT_SPACE:
      pc->args.space.x0        = GetShort(fp);
      pc->args.space.y0        = GetShort(fp);
      pc->args.space.x1        = GetShort(fp);
      pc->args.space.y1        = GetShort(fp);
      break;
#ifdef EXTENDED
    case PLOT_EXT_COLOR:
      pc->args.color.color     = GetString(fp, &pc->args.color.length);
      pc->args.color.pixel     = (unsigned long) -1;
      break;
    case PLOT_EXT_FONT:
      pc->args.font.font       = GetString(fp, &pc->args.font.length);
      break;
    case PLOT_EXT_FILL:
      {
    char *mode;

    mode = GetString(fp, &dummy);
    pc->args.fill_mode.mode = FindKeyword(mode, FillModeKeyTab);
    XtFree(mode);
    break;
      }
    case PLOT_EXT_POLY:
      pc->args.poly.count       = GetShort(fp);
      break;
    case PLOT_EXT_POLYDRAW:
      break;
#endif /* EXTENDED */
    }
    PlotCommand(pw, pc);
    pc++;
    pw->plot.plotbufsize++;
  }
  /*NOTREACHED*/
}

/*ARGSUSED*/
static void
ReplayPlot(w, region)
     Widget w;
     Region region;
{
  register PlotWidget pw = (PlotWidget) w;
  register PlotCmdBuf pc, plotbuf, *plotbuflist;
  register int i, plotlistsize, curbuflength;

  plotbuflist = pw->plot.plotbuflist;
  if ( ! plotbuflist && pw->plot.fp ) ReadPlot(pw->plot.fp, pw);
  else {
      if ( ! plotbuflist ) return;
      plotlistsize = pw->plot.plotlistsize;
      curbuflength = pw->plot.plotbuflength;
      for (i = 0; i < plotlistsize; i++){
      plotbuf = plotbuflist[i];
      if(i == plotlistsize-1) curbuflength = pw->plot.plotbufsize;
      for (pc = plotbuf; pc - plotbuf < curbuflength &&
           pc->command != PLOT_END; pc++)
        PlotCommand(w, pc);
      /* check if we stopped because we hit a PLOT_END */
      if (pc  - plotbuf < curbuflength) break;
      }
  }

  XFlush(XtDisplay(w));
}

/* ARGSUSED */
static void
Redisplay(w, event, region)
     Widget w;
     XEvent *event;
     Region region;
{
  if (w->core.visible && region != NULL) {
#ifdef notdef
    register XExposeEvent *e = (XExposeEvent *) event;
    fprintf(stderr, "Redisplay: %dx%d+%d+%d\n",
        e->width, e->height, e->x, e->y);
#endif
    ReplayPlot(w, region);
#ifdef notdef
  } else
    fprintf(stderr, "Redisplay: not visible or null region\n");
#else
  }
#endif

}

static void
Resize(w)
     Widget w;
{
  UpdateCTM(w);
  if (XtIsRealized(w))
    XClearWindow(XtDisplay(w), XtWindow(w));
}

/* ARGSUSED */
static Boolean
SetValues(current, request, new, args, num_args)
     Widget current, request, new;
     ArgList args;
     Cardinal *num_args;
{
  PlotWidget curpw = (PlotWidget) current;
  PlotWidget newpw = (PlotWidget) new;
  Boolean redisplay = False;

  if ( newpw->plot.file != curpw->plot.file ) {
    if ( newpw->plot.fp ) {
      (void) fclose ( newpw->plot.fp );
      newpw->plot.fp = (FILE *) NULL;
    }
    if ( newpw->plot.plotbuflist ) {
      FreePlot ( newpw );
      newpw->plot.plotbuflist = (PlotCmdBuf *) NULL;
    }
    if ( ( newpw->plot.fp = fopen ( newpw->plot.file, "r" )) == 0 )
      XtError ( "Can't open plot source file" );
    redisplay = True;
  }

  return redisplay || ( XtIsSensitive(current) != XtIsSensitive(new) );
}

static void
Destroy(w)
     Widget w;
{
  register PlotWidget pw = (PlotWidget) w;

  if (pw->plot.fp) {
    (void) fclose(pw->plot.fp);
    pw->plot.fp = (FILE *) NULL;
  }

  if (pw->plot.plotbuflist) {
    FreePlot(pw);
    pw->plot.plotbuflist = (PlotCmdBuf *) NULL;
  }

  if (pw->plot.batch_buf) {
    XtFree((caddr_t) pw->plot.batch_buf);
    pw->plot.batch_buf = (caddr_t) NULL;
  }
  XtFree(pw->plot.file);

  XFreeGC(XtDisplay(w), pw->plot.drawGC);   /* private GC */
  XtReleaseGC(w, pw->plot.eraseGC);         /* shared GC */
}

/* ARGSUSED */
static XtGeometryResult
QueryGeometry(w, intended, preferred)
     Widget w;
     XtWidgetGeometry *intended, *preferred;
{
  return XtGeometryYes;
}

/*
 * External Interface to Plot Widget
 *
 * PlotTranslate ( w, x, y, px_ret, py_ret )
 *
 *    Translate pixel coordinates within w to plot coordinates.
 *
 * Plot ( w, pc )
 *
 *    Given a plot widget w, append plot command pc to current plot
 *    buffer then execute plot command.
 *
 * ClearPlot ( w )
 *
 *    Given a plot widget w, clear the plot buffer history and erase
 *    associated window.
 *
 * ResetPlotColor ( w )
 *
 *    Reset the drawing color of plot to original foreground color.
 *    (Note: this only effects subsequent plot drawing).
 *
 * OverlayPlot ( w, fp )
 *
 *    Append the plot file given by fp to the current plot history of
 *    plot widget w;
 *
 * DumpPlot ( w, fp, extended )
 *
 *    Given a plot widget w, dump the current plot command buffer to
 *    stream fp.  If extended flag is non-nil, dump extended plot commands
 *    also.  Output to stream is in plot(5) format extended according
 *    to the definitions defined in PlotI.h.
 *
 */

void
PlotTranslate ( w, x, y, px_ret, py_ret )
     Widget w;
     int x, y;
     int *px_ret, *py_ret;
{
  register PlotWidget pw = (PlotWidget) w;

  if ( ! pw || ! px_ret || ! py_ret )
    return;
  if ( ( x < 0 ) || ( x >= pw->core.width ) ||
       ( y < 0 ) || ( y >= pw->core.height ) ) {
    *px_ret = -1;
    *py_ret = -1;
  } else {
    *px_ret = (int) (((double) x - pw->plot.disp_x) / pw->plot.scale_x );
    *py_ret = (int) (((double) y - pw->plot.disp_y) / pw->plot.scale_y );
  }
}

void
ClearPlot ( w )
     Widget w;
{
  register PlotWidget pw;
  Display *dpy;
  GC gc;

  if ( ! ( pw = (PlotWidget) w ) )
    return;
  dpy = XtDisplay ( w );
  gc  = pw->plot.drawGC;
  FreePlot ( pw);
  pw->plot.plotbuflist = (PlotCmdBuf *) NULL;
  pw->plot.plotlistlength = pw->plot.plotlistsize = 
    pw->plot.plotbuflength = pw->plot.plotbufsize = 0;
  pw->plot.fillmode = PLOT_FILLMODE_TRANSPARENT;
  pw->plot.overlay_mode = False;
  pw->plot.batch_entries = 0;
  pw->plot.batch_state   = BATCH_NONE;
  XSetForeground(dpy, gc, pw->plot.foreground);
  XClearWindow(dpy, XtWindow(w));
  XFlush(dpy);
}

void
Plot(w, pc)
     Widget w;
     PlotCmdBuf pc;
{
  register PlotWidget pw = (PlotWidget) w;

  if ( ! pw )
    return;
  if ( pc->command == PLOT_END ) {
    ClearPlot ( w );
    return;
  }
  if (!pw->plot.plotbuflist)
    CreatePlotBuf(pw);
  if (FullPlotBuf(pw))
    ExtendPlotBuf(pw);
  /* copying a structure with an equals. ok in all but very old compilers,
     I think */
  pw->plot.plotbuflist[pw->plot.plotlistsize-1][pw->plot.plotbufsize++] = *pc;
  PlotCommand(w, pc);
  XFlush(XtDisplay(w));
}

void
ResetPlotColor ( w )
     Widget w;
{
  register PlotWidget pw;
  Display *dpy;
  GC gc;

  if ( ! ( pw = (PlotWidget) w ) )
    return;
  dpy = XtDisplay ( w );
  gc  = pw->plot.drawGC;
  XSetForeground(dpy, gc, pw->plot.foreground);
}

void
OverlayPlot ( w, fp )
     Widget w;
     FILE *fp;
{
  register PlotWidget pw = (PlotWidget) w;

  pw->plot.overlay_mode = True;
  (void) ReadPlot ( fp, pw);
  ResetPlotColor ( w );
  /* this will replay the whole plot. I think we just replayed the
     new stuff when we did the overlay */
  ReplayPlot ( w, (Region) NULL );
}

static void
PutChar(fp, c)
     FILE *fp;
     char c;
{
  if (fputc(c, fp) == EOF) {
    (void) sprintf(ThrowReason, "PlotWidget: PutChar: Write Error");
    longjmp(ThrowEnv, 1);
    /*NOTREACHED*/
  }
}

static void
PutShort(fp, s)
     FILE *fp;
     register short s;
{
  if ((fputc(s & 0377, fp) == EOF) || (fputc((s >> 8) & 0377, fp)  == EOF)) {
    (void) sprintf(ThrowReason, "PlotWidget: PutShort: Write Error");
    longjmp(ThrowEnv, 1);
    /*NOTREACHED*/
  }
}

static void
PutString(fp, str)
     FILE *fp;
     char *str;
{
  if (fputs(str, fp) == EOF){
    (void) sprintf(ThrowReason, "PlotWidget: PutString: Write Error");
    longjmp(ThrowEnv, 1);
    /*NOTREACHED*/
  }
  PutChar(fp, '\0');        /* standard plot(5) string format */
}

static void
DumpPlotFile(fp, pw, extended)
     register FILE *fp;
     PlotWidget pw;
     int extended;
{
  register PlotCmdBuf pc, plotbuf;
  register int i, top_buffer_i, curbuflength;

  /* setup a jump point here (I think) to handle errors in Put routines */
  /* jmp allows "goto labels" that are not in the same routine as the
     goto itself */
  if (setjmp(ThrowEnv) != 0) {
    perror(ThrowReason);
    return;
  }
  curbuflength = pw->plot.plotbuflength;
  top_buffer_i = pw->plot.plotlistsize-1;
  for (i = 0; i <= top_buffer_i; i++){
    plotbuf = pw->plot.plotbuflist[i];
    if(i == top_buffer_i) curbuflength = pw->plot.plotbufsize;
    for (pc = plotbuf; pc < &plotbuf[curbuflength]; pc++) {

    switch (pc->command) {
    case PLOT_END:
      if (i != top_buffer_i && pc != &plotbuf[curbuflength - 1])
    fprintf(stderr, "PlotWidget: DumpPlotFile: Premature End of Plot\n");
      return;
    case PLOT_MOVE:
      PutChar(fp, pc->command);
      PutShort(fp, pc->args.move.x0);
      PutShort(fp, pc->args.move.y0);
      break;
    case PLOT_CONT:
      PutChar(fp, pc->command);
      PutShort(fp, pc->args.cont.x0);
      PutShort(fp, pc->args.cont.y0);
      break;
    case PLOT_POINT:
      PutChar(fp, pc->command);
      PutShort(fp, pc->args.point.x0);
      PutShort(fp, pc->args.point.y0);
      break;
    case PLOT_LINE:
      PutChar(fp, pc->command);
      PutShort(fp, pc->args.line.x0);
      PutShort(fp, pc->args.line.y0);
      PutShort(fp, pc->args.line.x1);
      PutShort(fp, pc->args.line.y1);
      break;
    case PLOT_LABEL:
      PutChar(fp, pc->command);
      PutString(fp, pc->args.label.label);
      break;
    case PLOT_ARC:
      PutChar(fp, pc->command);
      PutShort(fp, pc->args.arc.center_x);
      PutShort(fp, pc->args.arc.center_y);
      PutShort(fp, pc->args.arc.start_x);
      PutShort(fp, pc->args.arc.start_y);
      PutShort(fp, pc->args.arc.end_x);
      PutShort(fp, pc->args.arc.end_y);
      break;
    case PLOT_CIRCLE:
      PutChar(fp, pc->command);
      PutShort(fp, pc->args.circle.center_x);
      PutShort(fp, pc->args.circle.center_y);
      PutShort(fp, pc->args.circle.radius);
      break;
    case PLOT_ERASE:
      PutChar(fp, pc->command);
      break;
    case PLOT_LINEMODE:
      {
    register char *cp;

    cp = FindKeywordName(LineModeKeyTab, pc->args.line_mode.mode);
    if (!cp) {
      fprintf(stderr, "PlotWidget: DumpPlotFile: unknown line mode (%d)\n",
          pc->args.line_mode.mode);
      return;
    }
    PutChar(fp, pc->command);
    PutString(fp, cp);
    XtFree(cp);
    break;
      }
    case PLOT_SPACE:
      PutChar(fp, pc->command);
      PutShort(fp, pc->args.space.x0);
      PutShort(fp, pc->args.space.y0);
      PutShort(fp, pc->args.space.x1);
      PutShort(fp, pc->args.space.y1);
      break;
#ifdef EXTENDED
    case PLOT_EXT_COLOR:
      if (extended) {
    PutChar(fp, pc->command);
    PutString(fp, pc->args.color.color);
      }
      break;
    case PLOT_EXT_FILL:
      {
    register char *cp;

    if (extended) {
      cp = FindKeywordName(FillModeKeyTab, pc->args.line_mode.mode);
      if (!cp) {
        fprintf(stderr,
            "PlotWidget: DumpPlotFile: unknown fill mode (%d)\n",
            pc->args.line_mode.mode);
        return;
      }
      PutChar(fp, pc->command);
      PutString(fp, cp);
      XtFree(cp);
    }
    break;
      }
    case PLOT_EXT_POLY:
      if (extended) {
    PutChar(fp, pc->command);
    PutShort(fp, pc->args.poly.count);
      }
      break;
    case PLOT_EXT_POLYDRAW:
      if (extended)
    PutChar(fp, pc->command);
      break;
#endif /* EXTENDED */
    }/*switch*/
  }/* for pc */
  }/* for i */
}

void
DumpPlot(w, fp, extended)
     Widget w;
     FILE *fp;
     int extended;
{
  register PlotWidget pw = (PlotWidget) w;

  DumpPlotFile(fp, pw, extended);
}
