/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/*
 * Copyright 1989 by Metis Technology, Inc.
 *
 *                        All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software and its 
 * documentation for any purpose and without fee is hereby granted, 
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in 
 * supporting documentation.
 *
 * METIS TECHNOLOGY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL METIS TECHNOLOGY BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
 * USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 */



#ifndef _PlotI_h
#define _PlotI_h

/* defaults of various sorts */

#define DEF_BATCH_BUF_SIZE  1024    /* bytes */
#define DEF_HORIZ_MARGIN    4   /* pixels */
#define DEF_PLOT_BUF_SIZE   1024    /* plot commands */
#define DEF_PLOT_HEIGHT     780 /* plot coordinates */
#define DEF_PLOT_WIDTH      1023    /* plot coordinates */
#define DEF_VERT_MARGIN     4   /* pixels */

/* graphics batching state */
 
#define BATCH_NONE      0 /* no batching active */
#define BATCH_LINES     1 /* lines are being batched */
#define BATCH_POINTS        2 /* points are being batched */
#define BATCH_ARCS      3 /* arcs are being batched */
#define BATCH_SEGMENTS      4 /* segments are being batched */
#define BATCH_POLY      5 /* ploy vertices are being batched */

/* standard plot(5) commands */
 
#define PLOT_MOVE       'm' /* set new current point */
#define PLOT_CONT       'n' /* draw from current to new point */
#define PLOT_POINT      'p' /* draw point at location */
#define PLOT_LINE       'l' /* draw line from ponit1 to point2 */
#define PLOT_LABEL      't' /* draw text at current point */
#define PLOT_ARC        'a' /* draw arc at center, start to end */
#define PLOT_CIRCLE     'c' /* draw circle at center with radius */
#define PLOT_ERASE      'e' /* erase window */
#define PLOT_LINEMODE       'f' /* set line draw mode */
#define PLOT_SPACE      's' /* set coordinate space */
#define PLOT_END        '\000'  /* end of plot */

/* extended plot commands */

#define PLOT_EXT_BITMAP     'B' /* draw bitmap at point */
#define PLOT_EXT_COLOR      'C' /* set foreground drawing color */
#define PLOT_EXT_DASH       'D' /* set line dash pattern */
#define PLOT_EXT_FONT       'F' /* set text font */
#define PLOT_EXT_FILL       'G' /* set fill mode */
#define PLOT_EXT_POLY       'P' /* start polygon */
#define PLOT_EXT_POLYDRAW   'E' /* end polygon and draw */

/* xplot link commands */

#define PLOT_LINK_DISCONNECT    '\001'  /* disconnect from link */
#define PLOT_LINK_TERMINATE '\002'  /* disconnect and terminate */
#define PLOT_LINK_POINTERWAIT   '\003'  /* sync read pointer position */
#define PLOT_LINK_POINTERREAD   '\004'  /* async read pointer position */
#define PLOT_LINK_SETTITLE  '\005'  /* set plot title */

typedef struct _PlotCmdMoveRec {
  short x0, y0;
} PlotCmdMove;

typedef struct _PlotCmdContRec {
  short x0, y0;
} PlotCmdCont;

typedef struct _PlotCmdPointRec {
  short x0, y0;
} PlotCmdPoint;

typedef struct _PlotCmdLineRec {
  short x0, y0;
  short x1, y1;
} PlotCmdLine;

typedef struct _PlotCmdLabelRec {
  char *label;
  int  length;
} PlotCmdLabel;

typedef struct _PlotCmdArcRec {
  short center_x, center_y;
  short start_x, start_y;
  short end_x, end_y;
} PlotCmdArc;

typedef struct _PlotCmdCircleRec {
  short center_x, center_y;
  short radius;
} PlotCmdCircle;

typedef struct _PlotCmdEraseRec {
  char   dummy;
} PlotCmdErase;

typedef struct _PlotCmdLineModeRec {
  int mode;         /* converted to internal mode value */
} PlotCmdLineMode;
  
/* internal line modes */
#define PLOT_LINEMODE_DOTTED        1
#define PLOT_LINEMODE_SOLID     2
#define PLOT_LINEMODE_LONGDASHED    3
#define PLOT_LINEMODE_SHORTDASHED   4
#define PLOT_LINEMODE_DOTDASHED     5

typedef struct _PlotCmdSpaceRec {
  short x0, y0;
  short x1, y1;
} PlotCmdSpace;

typedef struct _PlotCmdColorRec {
  char *color;
  int  length;
  unsigned long pixel;
} PlotCmdColor;

typedef struct _PlotCmdFontRec {
  char *font;
  int  length;
} PlotCmdFont;

typedef struct _PlotCmdFillMode {
  int  mode;
} PlotCmdFillMode;

/* internal fill modes */
#define PLOT_FILLMODE_TRANSPARENT   1
#define PLOT_FILLMODE_OPAQUE        2

typedef struct _PlotCmdPoly {
  short count;          /* number of vertices to follow */
} PlotCmdPoly;

typedef struct _PlotCmdTimestampRec {
  int timestamp;
} PlotCmdTimestamp;

typedef struct _PlotCmdPointerRec {
  short x, y;
  short buttons, modifiers;
  int timestamp;
} PlotCmdPointer;

typedef struct _PlotCmdRec {
  char command;
  union {
    PlotCmdMove     move;
    PlotCmdCont     cont;
    PlotCmdPoint    point;
    PlotCmdLine     line;
    PlotCmdLabel    label;
    PlotCmdArc      arc;
    PlotCmdCircle   circle;
    PlotCmdErase    erase;
    PlotCmdLineMode line_mode;
    PlotCmdSpace    space;
#ifdef EXTENDED
    PlotCmdColor    color;
    PlotCmdFont     font;
    PlotCmdFillMode fill_mode;
    PlotCmdPoly     poly;
#endif /* EXTENDED */
    PlotCmdTimestamp    timestamp;
    PlotCmdPointer  pointer;
  } args;
} PlotCmd;

#endif /* _PlotI_h */
