/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

/*
 * Copyright 1989 by Metis Technology, Inc.
 *
 *                        All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software and its 
 * documentation for any purpose and without fee is hereby granted, 
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in 
 * supporting documentation.
 *
 * METIS TECHNOLOGY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL METIS TECHNOLOGY BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
 * USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 */

/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <ctype.h>
#if HAVE_STRINGS_H
#include <strings.h>
#endif
#include <setjmp.h>
#include <stdio.h>
#include <sysexits.h>

#include "PlotI.h"

typedef struct _PlotCmdRec *PlotCmdBuf;

#define XtError(str)        fprintf(stderr, "%s\n", str)
#define XtFree(ptr)     if (ptr) free(ptr)
#define XtNewString(str)    strcpy(XtMalloc((unsigned)strlen(str)+1),str)

typedef struct _keyent {
  char *keyword;
  int   value;
} KeyTabEnt, *KeyTab;

KeyTabEnt CommandKeyTab[] =
{
  { "move",        PLOT_MOVE },
  { "cont",        PLOT_CONT },
  { "line",        PLOT_LINE },
  { "point",       PLOT_POINT },
  { "label",       PLOT_LABEL },
  { "arc",         PLOT_ARC },
  { "circle",      PLOT_CIRCLE },
  { "erase",       PLOT_ERASE },
  { "linemode",    PLOT_LINEMODE },
  { "space",       PLOT_SPACE },
  { "bitmap",      PLOT_EXT_BITMAP },
  { "color",       PLOT_EXT_COLOR },
  { "dash",        PLOT_EXT_DASH },
  { "font",        PLOT_EXT_FONT },
  { "fillmode",    PLOT_EXT_FILL },
  { "poly",        PLOT_EXT_POLY },
  { "polydraw",    PLOT_EXT_POLYDRAW },
  { (char *) NULL, 0 }
};

KeyTabEnt LineModeKeyTab[] =
{
  { "dotted",      PLOT_LINEMODE_DOTTED },
  { "solid",       PLOT_LINEMODE_SOLID  },
  { "longdashed",  PLOT_LINEMODE_LONGDASHED },
  { "shortdashed", PLOT_LINEMODE_SHORTDASHED },
  { "dotdashed",   PLOT_LINEMODE_DOTDASHED },
  { (char *) NULL, 0 }
};

static KeyTabEnt FillModeKeyTab[] =
{
  { "transparent", PLOT_FILLMODE_TRANSPARENT    },
  { "opaque",      PLOT_FILLMODE_OPAQUE     },
  { (char *) NULL, 0 }
};

static char *ProgName;      /* Program Name (argv[0]) */
static char *TokenPtr;      /* Pointer into Token String */
static char *TokenStr;      /* Token String Buffer */
static char ThrowReason[1024];  /* Throw Reason */
static jmp_buf  ThrowEnv;       /* Throw Environment */

void MakePlot(FILE *fin, FILE *fout);

int
main(argc, argv)
     int argc;
     char **argv;
{
  FILE *fin, *fout;
  char buf[1024];

  ProgName = argv[0];

  if (argc == 3) {
    if ((fin = fopen(argv[1], "r")) == (FILE *) NULL) {
      (void) sprintf(buf, "%s: fopen: open of %s failed", ProgName, argv[1]);
      perror(buf);
      return EX_NOINPUT;
    }
    if ((fout = fopen(argv[2], "w+")) == (FILE *) NULL) {
      (void) sprintf(buf, "%s: fopen: open of %s failed", ProgName, argv[2]);
      perror(buf);
      return EX_CANTCREAT;
    }
  } else if (argc == 2) {
    if ((fin = fopen(argv[1], "r")) == (FILE *) NULL) {
      (void) sprintf(buf, "%s: fopen: open of %s failed", ProgName, argv[1]);
      perror(buf);
      return EX_NOINPUT;
    }
    fout = stdout;
  } else if (argc == 1) {
    fin = stdin;
    fout = stdout;
  } else {
    fprintf(stderr, "usage: %s [ascii-plot-file] [plot-file]\n", ProgName);
    return EX_USAGE;
  }
  MakePlot(fin, fout);
  return EX_OK;
}

char *
XtMalloc(len)
     unsigned len;
{
  char *ptr;
  
  if ((ptr = malloc(len)) == (char *) NULL) {
    XtError("XtMalloc: malloc failed");
    exit(1);
  }
  return ptr;
}

char *
XtCalloc(nelements, len)
     unsigned nelements;
     unsigned len;
{
  char *ptr;
  
  if ((ptr = calloc(nelements, len)) == (char *) NULL) {
    XtError("XtCalloc: calloc failed");
    exit(1);
  }
  return ptr;

}

char *
XtRealloc(ptr, len)
     char *ptr;
     unsigned len;
{
  if (ptr == (char *) NULL)
    return XtMalloc(len);
  else if ((ptr = realloc(ptr, len)) == (char *) NULL) {
    XtError("XtRealloc: realloc failed");
    exit(1);
  }
  return ptr;
}

static char *
my_strtok(string, sepset)
     char    *string, *sepset;
{
  register char   *p, *q, *r;
  
  if (string)
    p = TokenStr = string;
  else
    p = TokenPtr;

  if (p == 0)
    return (char *) NULL;
  
#if HAVE_STRSPN
  q = p + strspn(p, sepset);
#else
  q = p + strspn(p, sepset);
#endif
  
  if(*q == '\0')
    return (char *) NULL;
  
  if((r = strpbrk(q, sepset)) == (char *) NULL) {
    TokenPtr = 0;
#ifdef notdef
    *TokenStr = '\0';       /* truncate token buffer */
#endif
  } else {
    *r = '\0';
    TokenPtr = ++r;
  }
  return(q);
}

static int
FindKeyword(name, keytab)
     char *name;
     KeyTab  keytab;
{
  register unsigned char *cp, *bp;
  register KeyTabEnt *kp;

  cp = bp = XtNewString(name);
  while (*cp++) {
    if (isascii(*cp) && isupper(*cp))
      *cp = tolower(*cp);
  }
  cp = bp;
  for (kp = keytab; kp->keyword != (char *) NULL; kp++)
    if (strcmp(kp->keyword, cp) == 0)
      break;
  XtFree(bp);
  return kp->value;
}

static char *
FindKeywordName(keytab, value)
     KeyTab keytab;
     int value;
{
  register KeyTabEnt *kp;

  for (kp = keytab; kp->keyword; kp++) {
    if (kp->value == value)
      return XtNewString(kp->keyword);
  }
  return (char *) NULL;
}

static int
GetKeyword(buf, keytab)
     char *buf;
     KeyTab  keytab;
{
  register char *cp;
  register int key;

  cp = my_strtok(TokenPtr ? (char *) NULL : buf, " \t");
  if (!cp) {
    (void) sprintf(ThrowReason, "Missing Keyword");
    longjmp(ThrowEnv, 1);
    /*NOTREACHED*/
  }
  if ((key = FindKeyword(cp, keytab)) == 0) {
    (void) sprintf(ThrowReason, "Unknown Keyword (%s)", cp);
    longjmp(ThrowEnv, 1);
    /*NOTREACHED*/
  }
  return key;
}

static short
GetCoord(buf)
     char *buf;
{
  register char *cp;
  register short s;

  cp = my_strtok(TokenPtr ? (char *) NULL : buf, " \t");
  if (!cp) {
    (void) sprintf(ThrowReason, "Missing Coordinate");
    longjmp(ThrowEnv, 1);
    /*NOTREACHED*/
  }
  s = (short) atoi(cp);
#ifdef notdef
  if (s < 0) {
    (void) sprintf(ThrowReason, "Negative Coordinate (%d)", s);
    longjmp(ThrowEnv, 1);
  }
#endif
  return s;
}

static char *
GetQuotedString(buf, len)
     char *buf;
     int *len;
{
  register unsigned char *bp, *cp, *dp;
  register int cc;
  unsigned char buffer[1024];

  cp = TokenPtr ? TokenPtr : buf;
  while (cp && *cp && (*cp == ' ' || *cp == '\t'))
    cp++;
  dp = cp;          /* for error output */
  if (!cp || !*cp || *cp++ != '"') {
    if (len)
      *len = 0;
    TokenPtr = (char *) NULL;
    (void) sprintf(ThrowReason,
              (cp && *cp) ? "Unquoted String (%s)" : "Missing String", dp);
    longjmp(ThrowEnv, 1);
    /*NOTREACHED*/
  }
  for (bp = buffer, cc = 0; bp < &buffer[1024]; bp++, cp++, cc++) {
    if (!*cp) {
      (void) sprintf(ThrowReason, "Unbalanced Quotes (%s)", dp);
      longjmp(ThrowEnv, 1);
      /*NOTREACHED*/
    }
    if (*cp == '\\') {
      switch (*++cp) {
      case '\0':        /* escaped NULL ?!? */
    if (len)
      *len = 0;
    return (char *) NULL;
      case 'b':
    *bp = '\010';
    break;
      case 'f':
    *bp = '\014';
    break;
      case 'n':
    *bp = '\012';
    break;
      case 't':
    *bp = '\011';
    break;
      default:
    {
      int temp = 0;
      
      if (isdigit(*cp)) {
        while (isdigit(*cp))
          temp = temp * 8 + (int) (*cp - '0');
        *bp = (char) temp;
      } else
        *bp = *cp;
    }
      }
    } else if (*cp != '"')
      *bp = *cp;
    else {
      cp++;         /* skip over end quote */
      break;
    }
  }
  *bp = '\0';
  if (len)
    *len = cc;
  TokenPtr =  *cp ? cp :  NULL;
  return XtNewString(buffer);
}

static void
FreePlotCmdBuf(plotbuf, plotsize)
     PlotCmdBuf plotbuf;
     int plotsize;
{
  register PlotCmdBuf pc;

  for (pc = plotbuf; pc - plotbuf < plotsize && pc->command != PLOT_END; pc++)
    switch (pc->command) {
    case PLOT_LABEL:
      XtFree(pc->args.label.label);
      break;
    default:
      break;
    }
}

static PlotCmdBuf
ReadAsciiPlotFile(fp, plotsize)
     FILE *fp;
     int *plotsize;
{
  register char *bp = (char *) NULL;
  register PlotCmdBuf plotbuf, pc;
  register int plotbufsize = DEF_PLOT_BUF_SIZE;
  register int lineno = 0;
  char linebuf[1024], parsebuf[1024];

  pc = plotbuf = (PlotCmdBuf) XtCalloc((unsigned) plotbufsize,
                       (unsigned) sizeof (*pc));

  while (fgets(linebuf, 1024, fp) != (char *) NULL) {

    bp = strcpy(parsebuf, linebuf), lineno++;
    parsebuf[strlen(parsebuf) - 1] = '\0';

    /* skip blank lines and comment lines */
    if ((strlen(parsebuf) == 0) || parsebuf[0] == '#')
      continue;

    if (setjmp(ThrowEnv) != 0) {
      fprintf(stderr, "%s: ReadAsciiPlotFile: Line %d: %s\n",
          ProgName, lineno, ThrowReason);
      continue;
    }

    if (pc == &plotbuf[plotbufsize]) {
      plotbuf = (PlotCmdBuf) XtRealloc((char *) plotbuf,
                 (unsigned) (plotbufsize + DEF_PLOT_BUF_SIZE) *
                 (unsigned) sizeof (*pc));
      pc = &plotbuf[plotbufsize];
      plotbufsize += DEF_PLOT_BUF_SIZE;
    }

    pc->command = GetKeyword(bp, CommandKeyTab);

    switch (pc->command) {
    case PLOT_MOVE:
      pc->args.move.x0 = GetCoord(bp);
      pc->args.move.y0 = GetCoord(bp);
      break;
    case PLOT_CONT:
      pc->args.cont.x0 = GetCoord(bp);
      pc->args.cont.y0 = GetCoord(bp);
      break;
    case PLOT_POINT:
      pc->args.point.x0 = GetCoord(bp);
      pc->args.point.y0 = GetCoord(bp);
      break;
    case PLOT_LINE:
      pc->args.line.x0 = GetCoord(bp);
      pc->args.line.y0 = GetCoord(bp);
      pc->args.line.x1 = GetCoord(bp);
      pc->args.line.y1 = GetCoord(bp);
      break;
    case PLOT_LABEL:
      {
    int len;

    pc->args.label.label  = GetQuotedString(bp, &len);
    pc->args.label.length = len;
    break;
      }
    case PLOT_ARC:
      pc->args.arc.center_x = GetCoord(bp);
      pc->args.arc.center_y = GetCoord(bp);
      pc->args.arc.start_x  = GetCoord(bp);
      pc->args.arc.start_y  = GetCoord(bp);
      pc->args.arc.end_x    = GetCoord(bp);
      pc->args.arc.end_y    = GetCoord(bp);
      break;
    case PLOT_CIRCLE:
      pc->args.circle.center_x = GetCoord(bp);
      pc->args.circle.center_y = GetCoord(bp);
      pc->args.circle.radius   = GetCoord(bp);
      break;
    case PLOT_ERASE:
      break;
    case PLOT_LINEMODE:
      pc->args.line_mode.mode = GetKeyword(bp, LineModeKeyTab);
      break;
    case PLOT_SPACE:
      pc->args.space.x0 = GetCoord(bp);
      pc->args.space.y0 = GetCoord(bp);
      pc->args.space.x1 = GetCoord(bp);
      pc->args.space.y1 = GetCoord(bp);
      break;
#ifdef EXTENDED
    case PLOT_EXT_COLOR:
      {
    int len;

    pc->args.color.color  = GetQuotedString(bp, &len);
    pc->args.color.length = len;
    pc->args.color.pixel  = (unsigned long) -1;
    break;
      }
    case PLOT_EXT_FONT:
      {
    int len;

    pc->args.font.font  = GetQuotedString(bp, &len);
    pc->args.font.length = len;
    break;
      }
    case PLOT_EXT_FILL:
      pc->args.fill_mode.mode = GetKeyword(bp, FillModeKeyTab);
      break;
    case PLOT_EXT_POLY:
      pc->args.poly.count = GetCoord(bp);
      break;
    case PLOT_EXT_POLYDRAW:
      break;
#endif /* EXTENDED */
    default:
      break;
    }
    pc++;
  }

  if (ferror(fp)) {
    (void) sprintf(linebuf, "%s: ReadAsciiPlotFile: Read Error", ProgName);
    perror(linebuf);
    if (plotsize)
      *plotsize = 0;
    return  (PlotCmdBuf) NULL;
  }
  if (plotsize)
    *plotsize = pc - plotbuf;
  return plotbuf;
}

static void
PutChar(fp, c)
     FILE *fp;
     char c;
{
  if (fputc(c, fp) == EOF) {
    (void) sprintf(ThrowReason, "PutChar: Write Error");
    longjmp(ThrowEnv, 1);
    /*NOTREACHED*/
  }
}


static void
PutShort(fp, s)
     FILE *fp;
     register short s;
{
  if ((fputc(s & 0377, fp) == EOF) || (fputc((s >> 8) & 0377, fp)  == EOF)) {
    (void) sprintf(ThrowReason, "PutShort: Write Error");
    longjmp(ThrowEnv, 1);
    /*NOTREACHED*/
  }
}

static void
PutString(fp, str)
     FILE *fp;
     char *str;
{
  if (fputs(str, fp) == EOF){
    (void) sprintf(ThrowReason, "PutString: Write Error");
    longjmp(ThrowEnv, 1);
    /*NOTREACHED*/
  }
  PutChar(fp, '\0');        /* standard plot(5) string format */
}

static void
DumpPlotFile(fp, plotbuf, plotsize)
     register FILE *fp;
     PlotCmdBuf plotbuf;
     int plotsize;
{
  register PlotCmdBuf pc;

  if (setjmp(ThrowEnv) != 0) {
    perror(ThrowReason);
    return;
  }
  for (pc = plotbuf; pc < &plotbuf[plotsize]; pc++) {

    PutChar(fp, pc->command);

    switch (pc->command) {
    case PLOT_END:
      if (pc != &plotbuf[plotsize - 1])
    fprintf(stderr, "%s: DumpAsciiPlotFile: Premature End of Plot\n",
        ProgName);
      return;
    case PLOT_MOVE:
      PutShort(fp, pc->args.move.x0);
      PutShort(fp, pc->args.move.y0);
      break;
    case PLOT_CONT:
      PutShort(fp, pc->args.cont.x0);
      PutShort(fp, pc->args.cont.y0);
      break;
    case PLOT_POINT:
      PutShort(fp, pc->args.point.x0);
      PutShort(fp, pc->args.point.y0);
      break;
    case PLOT_LINE:
      PutShort(fp, pc->args.line.x0);
      PutShort(fp, pc->args.line.y0);
      PutShort(fp, pc->args.line.x1);
      PutShort(fp, pc->args.line.y1);
      break;
    case PLOT_LABEL:
      PutString(fp, pc->args.label.label);
      break;
    case PLOT_ARC:
      PutShort(fp, pc->args.arc.center_x);
      PutShort(fp, pc->args.arc.center_y);
      PutShort(fp, pc->args.arc.start_x);
      PutShort(fp, pc->args.arc.start_y);
      PutShort(fp, pc->args.arc.end_x);
      PutShort(fp, pc->args.arc.end_y);
      break;
    case PLOT_CIRCLE:
      PutShort(fp, pc->args.circle.center_x);
      PutShort(fp, pc->args.circle.center_y);
      PutShort(fp, pc->args.circle.radius);
      break;
    case PLOT_ERASE:
      break;
    case PLOT_LINEMODE:
      {
    register char *cp;

    cp = FindKeywordName(LineModeKeyTab, pc->args.line_mode.mode);
    if (!cp) {
      fprintf(stderr, "%s: DumpPlotFile: unknown line mode (%d)\n",
          ProgName, pc->args.line_mode.mode);
      exit(EX_DATAERR);
    }
    PutString(fp, cp);
    XtFree(cp);
    break;
      }
    case PLOT_SPACE:
      PutShort(fp, pc->args.space.x0);
      PutShort(fp, pc->args.space.y0);
      PutShort(fp, pc->args.space.x1);
      PutShort(fp, pc->args.space.y1);
      break;
#ifdef EXTENDED
    case PLOT_EXT_COLOR:
      PutString(fp, pc->args.color.color);
      break;
    case PLOT_EXT_FONT:
      PutString(fp, pc->args.font.font);
      break;
    case PLOT_EXT_FILL:
      {
    register char *cp;

    cp = FindKeywordName(FillModeKeyTab, pc->args.fill_mode.mode);
    if (!cp) {
      fprintf(stderr, "%s: DumpPlotFile: unknown fill mode (%d)\n",
          ProgName, pc->args.fill_mode.mode);
      exit(EX_DATAERR);
    }
    PutString(fp, cp);
    XtFree(cp);
    break;
      }
    case PLOT_EXT_POLY:
      PutShort(fp, pc->args.poly.count);
      break;
    case PLOT_EXT_POLYDRAW:
      break;
#endif /* EXTENDED */
    }
  }
}

void
MakePlot(fin, fout)
     FILE *fin, *fout;
{
  PlotCmdBuf plotbuf;
  int plotsize;

  plotbuf = ReadAsciiPlotFile(fin, &plotsize);
  if (plotbuf && (plotsize > 0)) {
    DumpPlotFile(fout, plotbuf, plotsize);
    FreePlotCmdBuf(plotbuf, plotsize);
  }
}
