/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

/*
 * Copyright 1989,1990,1991 by Metis Technology, Inc.
 *
 *                        All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software and its 
 * documentation for any purpose and without fee is hereby granted, 
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in 
 * supporting documentation.
 *
 * METIS TECHNOLOGY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL METIS TECHNOLOGY BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
 * USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 */

/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <ctype.h>
#include <stdio.h>
#include <sys/types.h>
/* for timeout for PlotPointerRead */
#if HAVE_SYS_TIME_H
#include <sys/time.h>
#endif
#if HAVE_UNISTD_H
#include <unistd.h>
#endif

#include "PlotLink.h"
#include "PlotI.h"

/*
 * Sample usage of PlotLink interface to xplot.  The regimen is as follows:
 *
 * PlotOpen ( w, h )
 *   Open plot link; create xplot window with width w, height h.
 *
 * Plot ( pc )
 *   Send plot command pc
 *
 * PointerWait ( x_ret, y_ret, but_ret, mod_ret, timeout )
 *   Wait on pointer event (ButtonPress), returning x & y in plot coordinates,
 *   button mask, and modifiers.  Timeout specifies max wait period.
 *
 *   Button values are as follows:
 *
 *  Button1     1
 *  Button2     2
 *  Button3     3
 *
 *   Modifier values are as follows:
 *
 *  ShiftMask   (1<<0)
 *  LockMask    (1<<1)
 *  ControlMask (1<<2)
 *  Mod1Mask    (1<<3)
 *  Mod2Mask    (1<<4)
 *  Mod3Mask    (1<<5)
 *  Mod4Mask    (1<<6)
 *  Mod5Mask    (1<<7)
 *  Button1Mask (1<<8)
 *  Button2Mask (1<<9)
 *  Button3Mask (1<<10)
 *  Button4Mask (1<<11)
 *  Button5Mask (1<<12)
 *
 *
 * PlotClose
 *   Disconnect from xplot and close plot link.
 */
int SendPlot(FILE *fp);

int
main ( argc, argv )
     int argc;
     char **argv;
{
  FILE *fp;

  argc--, argv++;
  if ( argc < 1 ) {
    fprintf ( stderr, "usage: link <plotfile>\n" );
    return 1;
  }

  /*
   * Open Plot Link
   */

  if ( PlotOpen ( (char **) NULL ) < 0 ) {
    fprintf ( stderr, "plotlink: can't open plot link\n" );
    return 1;
  }
  
  /*
   * Open Plot File
   */

  if ( ! ( fp = fopen ( argv[0], "r" ) ) ) {
    fprintf ( stderr, "plotlink: can't open \"%s\"\n", argv[0] );
    return 1;
  }

  /*
   * Read and Transmit Plot Commands
   */
  if ( SendPlot ( fp ) < 0 ) {
    fprintf ( stderr, "plotlink: plot send failed\n" );
    return 1;
  }

  while ( 1 ) {
    struct timeval timeout;
    int x, y, mod;
    char location [ 128 ];

    timeout.tv_sec = 30;
    timeout.tv_usec = 0;
    switch ( PlotPointerRead ( &x, &y, &mod, timeout ) ) {
    case -1:
      fprintf ( stderr, "plotlink: pointer read error\n" );
      return 1;
    case 0:
      fprintf ( stderr, "plotlink: pointer read timed out\n" );
      return 1;
    default:
      break;
    }
    sprintf ( location, "(%d+%d) mods (%x)\n", x, y, mod );
    if ( PlotSetTitle ( location ) < 0 ) {
      fprintf ( stderr, "plotlink: can't set title\n" );
      return 1;
    }
    if ( mod == 0x700 )
      break;
    sleep ( 5 );
  }

  /*
   * Disconnect Link
   */
 done:
  PlotClose ( 1 );

  return 0;
}

/*
 * Everthing below this point is concerned with reading plot(3) commands
 * from file and formatting them as PlotCmd structures which are sent
 * the Plot() interface of PlotLink.
 */

char
GetCommand(fp)
     FILE *fp;
{
  register int c;

  if ((c = fgetc(fp)) == EOF) {
    if (ferror(fp))
      perror ( "GetCommand: plot read command: I/O Error" );
    return PLOT_END;
  }
  return (char) c;
}


static short
GetShort(fp)
     FILE *fp;
{
  register int c;
  register unsigned short s;
  
  if ((c = fgetc(fp)) != EOF) {
    s = (unsigned short) c;
    if ((c = fgetc(fp)) != EOF) {
      s |= ((unsigned short) c) << 8;
      return s;
    }
  }
  if (ferror(fp))
    perror ( "GetShort: plot command read: I/O error" );
  else
    fprintf ( stderr, "GetShort: plot command read: premature EOF\n" );
  return -1;
}

static char *
  newstring ( str )
char *str;
{
  char *s;
  if ( ! str ) return (char *) NULL;
  if ( ! ( s = malloc ( strlen ( str ) + 1 ) ) ) return (char *) NULL;
  return strcpy ( s, str );
}

static char *
GetString(fp, len)
     FILE *fp;
     int *len;
{
  char buf[1024];
  register char *cp = buf;
  register int c, cc = 0;

  while ((c = fgetc(fp)) != EOF) {
    if (cp >= &buf[1024])
      fprintf ( stderr, "GetString: plot command read: string too long\n" );
    if ((char) c == '\n' || (char) c == '\0') {
      *cp = '\0';
      *len = cc;
      return newstring(buf);
    }
    *cp++ = (char) c, cc++;
  }
  if (ferror(fp))
    perror ( "GetString: plot command read: I/O error" );
  else
    fprintf ( stderr, "GetString: plot command read: premature EOF\n" );
  *len = 0;
  return (char *) NULL;
}

typedef struct _keyent {
  char *keyword;
  int   value;
} KeyTabEnt, *KeyTab;

static KeyTabEnt LineModeKeyTab[] =
{
  { "dotted",      PLOT_LINEMODE_DOTTED },
  { "solid",       PLOT_LINEMODE_SOLID  },
  { "longdashed",  PLOT_LINEMODE_LONGDASHED },
  { "shortdashed", PLOT_LINEMODE_SHORTDASHED },
  { "dotdashed",   PLOT_LINEMODE_DOTDASHED },
  { (char *) NULL, 0 }
};

static int
FindKeyword(name, keytab)
     char *name;
     KeyTab  keytab;
{
  register unsigned char *cp, *bp;
  register KeyTabEnt *kp;

  cp = bp = newstring(name);
  while (*cp++) {
    if (isascii(*cp) && isupper(*cp))
      *cp = tolower(*cp);
  }
  cp = bp;
  for (kp = keytab; kp->keyword != (char *) NULL; kp++)
    if (strcmp(kp->keyword, cp) == 0)
      break;
  free(bp);
  return kp->value;
}

int
SendPlot ( fp )
     register FILE *fp;
{
  PlotCmd plotcmd;
  register PlotCmd *pc = &plotcmd;
  int dummy;

  while (1) {

    pc->command = GetCommand(fp);

    switch (pc->command) {
    case PLOT_END:
      return 1;
    case PLOT_MOVE:
      pc->args.move.x0         = GetShort(fp);
      pc->args.move.y0         = GetShort(fp);
      break;
    case PLOT_CONT:
      pc->args.cont.x0         = GetShort(fp);
      pc->args.cont.y0         = GetShort(fp);
      break;
    case PLOT_POINT:
      pc->args.point.x0        = GetShort(fp);
      pc->args.point.y0        = GetShort(fp);
      break;
    case PLOT_LINE:
      pc->args.line.x0         = GetShort(fp);
      pc->args.line.y0         = GetShort(fp);
      pc->args.line.x1         = GetShort(fp);
      pc->args.line.y1         = GetShort(fp);
      break;
    case PLOT_LABEL:
      pc->args.label.label     = GetString(fp, &pc->args.label.length);
      break;
    case PLOT_ARC:
      pc->args.arc.center_x    = GetShort(fp);
      pc->args.arc.center_y    = GetShort(fp);
      pc->args.arc.start_x     = GetShort(fp);
      pc->args.arc.start_y     = GetShort(fp);
      pc->args.arc.end_x       = GetShort(fp);
      pc->args.arc.end_y       = GetShort(fp);
      break;
    case PLOT_CIRCLE:
      pc->args.circle.center_x = GetShort(fp);
      pc->args.circle.center_y = GetShort(fp);
      pc->args.circle.radius   = GetShort(fp);
      break;
    case PLOT_ERASE:
      break;
    case PLOT_LINEMODE:
      {
    char *mode;

    mode = GetString(fp, &dummy);
    pc->args.line_mode.mode = FindKeyword(mode, LineModeKeyTab);
    break;
      }
    case PLOT_SPACE:
      pc->args.space.x0        = GetShort(fp);
      pc->args.space.y0        = GetShort(fp);
      pc->args.space.x1        = GetShort(fp);
      pc->args.space.y1        = GetShort(fp);
      break;
#ifdef EXTENDED
    case PLOT_EXT_COLOR:
      pc->args.color.color     = GetString(fp, &pc->args.color.length);
      pc->args.color.pixel     = (unsigned long) -1;
      break;
    case PLOT_EXT_FONT:
      pc->args.font.font       = GetString(fp, &pc->args.font.length);
      break;
#endif /* EXTENDED */
    }

    if ( Plot ( pc ) < 0 ) {
      fprintf ( stderr, "SendPlot: plot command send failed\n" );
      return 0;
    }
  }

  /*NOTREACHED*/
}
