/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

/*
 * Copyright 1989 by Metis Technology, Inc.
 *
 *                        All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software and its 
 * documentation for any purpose and without fee is hereby granted, 
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in 
 * supporting documentation.
 *
 * METIS TECHNOLOGY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL METIS TECHNOLOGY BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
 * USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 */

/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <ctype.h>
#include <stdio.h>
#include <sysexits.h>

#include "PlotI.h"

typedef struct _PlotCmdRec *PlotCmdBuf;

#define XtError(str)        fprintf(stderr, "%s\n", str)
#define XtFree(ptr)     if (ptr) free(ptr)
#define XtNewString(str)    strcpy(XtMalloc((unsigned)strlen(str)+1),str)

typedef struct _keyent {
  char *keyword;
  int   value;
} KeyTabEnt, *KeyTab;

KeyTabEnt LineModeKeyTab[] =
{
  { "dotted",      PLOT_LINEMODE_DOTTED },
  { "solid",       PLOT_LINEMODE_SOLID  },
  { "longdashed",  PLOT_LINEMODE_LONGDASHED },
  { "shortdashed", PLOT_LINEMODE_SHORTDASHED },
  { "dotdashed",   PLOT_LINEMODE_DOTDASHED },
  { (char *) NULL, 0 }
};

static KeyTabEnt FillModeKeyTab[] =
{
  { "transparent", PLOT_FILLMODE_TRANSPARENT    },
  { "opaque",      PLOT_FILLMODE_OPAQUE     },
  { (char *) NULL, 0 }
};

char *ProgName;

void ShowPlot(FILE *fin, FILE *fout);

int
main(argc, argv)
     int argc;
     char **argv;
{
  FILE *fin, *fout;
  char buf[1024];

  ProgName = argv[0];

  if (argc == 3) {
    if ((fin = fopen(argv[1], "r")) == (FILE *) NULL) {
      (void) sprintf(buf, "%s: fopen: open of %s failed", ProgName, argv[1]);
      perror(buf);
      return EX_NOINPUT;
    }
    if ((fout = fopen(argv[2], "w+")) == (FILE *) NULL) {
      (void) sprintf(buf, "%s: fopen: open of %s failed", ProgName, argv[2]);
      perror(buf);
      return EX_CANTCREAT;
    }
  } else if (argc == 2) {
    if ((fin = fopen(argv[1], "r")) == (FILE *) NULL) {
      (void) sprintf(buf, "%s: fopen: open of %s failed", ProgName, argv[1]);
      perror(buf);
      return EX_NOINPUT;
    }
    fout = stdout;
  } else if (argc == 1) {
    fin = stdin;
    fout = stdout;
  } else {
    fprintf(stderr,"usage: %s [plot-file] [ascii-plot-file]\n", ProgName);
    return EX_USAGE;
  }
  ShowPlot(fin, fout);
  return EX_OK;
}

char *
XtMalloc(len)
     unsigned len;
{
  char *ptr;
  
  if ((ptr = malloc(len)) == (char *) NULL) {
    XtError("XtMalloc: malloc failed");
    exit(1);
  }
  return ptr;
}

char *
XtCalloc(nelements, len)
     unsigned nelements;
     unsigned len;
{
  char *ptr;
  
  if ((ptr = calloc(nelements, len)) == (char *) NULL) {
    XtError("XtCalloc: calloc failed");
    exit(1);
  }
  return ptr;

}

char *
XtRealloc(ptr, len)
     char *ptr;
     unsigned len;
{
  if (ptr == (char *) NULL)
    return XtMalloc(len);
  else if ((ptr = realloc(ptr, len)) == (char *) NULL) {
    XtError("XtRealloc: realloc failed");
    exit(1);
  }
  return ptr;
}

static int
FindKeyword(name, keytab)
     char *name;
     KeyTab  keytab;
{
  register unsigned char *cp, *bp;
  register KeyTabEnt *kp;

  cp = bp = XtNewString(name);
  while (*cp++) {
    if (isascii(*cp) && isupper(*cp))
      *cp = tolower(*cp);
  }
  cp = bp;
  for (kp = keytab; kp->keyword != (char *) NULL; kp++)
    if (strcmp(kp->keyword, cp) == 0)
      break;
  XtFree(bp);
  return kp->value;
}

static char *
FindKeywordName(keytab, value)
     KeyTab keytab;
     int value;
{
  register KeyTabEnt *kp;

  for (kp = keytab; kp->keyword; kp++) {
    if (kp->value == value)
      return XtNewString(kp->keyword);
  }
  return (char *) NULL;
}

static char
GetCommand(fp)
     FILE *fp;
{
  register int c;

  if ((c = fgetc(fp)) == EOF) {
    if (ferror(fp))
      XtError("plot read command: I/O Error");
    return PLOT_END;
  }
  return (char) c;
}


static short
GetShort(fp)
     FILE *fp;
{
  register int c;
  register unsigned short s;
  
  if ((c = fgetc(fp)) != EOF) {
    s = (unsigned short) c;
    if ((c = fgetc(fp)) != EOF) {
      s |= ((unsigned short) c) << 8;
      return s;
    }
  }
  if (ferror(fp))
    XtError("plot command read: I/O error");
  else
    XtError("plot command read: premature EOF");
  return -1;
}

static char *
GetString(fp, len)
     FILE *fp;
     int *len;
{
  char buf[1024];
  register char *cp = buf;
  register int c, cc = 0;

  while ((c = fgetc(fp)) != EOF) {
    if (cp >= &buf[1024])
      XtError("plot command read: string too long");
    if ((char) c == '\n' || (char) c == '\0') {
      *cp = '\0';
      *len = cc;
      return XtNewString(buf);
    }
    *cp++ = (char) c, cc++;
  }
  if (ferror(fp))
    XtError("plot command read: I/O error");
  else
    XtError("plot command read: premature EOF");
  *len = 0;
  return (char *) NULL;
}

static PlotCmdBuf
ReadPlotFile(fp, plotsize)
     register FILE *fp;
     int *plotsize;
{
  PlotCmdBuf plotbuf, pc;
  register int plotbufsize = DEF_PLOT_BUF_SIZE;
  int dummy;

  pc = plotbuf = (PlotCmdBuf) XtCalloc((unsigned) plotbufsize,
                       (unsigned) sizeof (*pc));

  while (1) {
    /*
     * Grow plot buffer as required.
     */
    if (pc == &plotbuf[plotbufsize]) {
      plotbuf = (PlotCmdBuf) XtRealloc((char *) plotbuf,
                 (unsigned) (plotbufsize + DEF_PLOT_BUF_SIZE) *
                 (unsigned) sizeof (*pc));
      pc = &plotbuf[plotbufsize];
      plotbufsize += DEF_PLOT_BUF_SIZE;
    }

    pc->command = GetCommand(fp);

    switch (pc->command) {
    case PLOT_END:
      if (plotsize) 
    *plotsize = pc - plotbuf;
      return plotbuf;
    case PLOT_MOVE:
      pc->args.move.x0         = GetShort(fp);
      pc->args.move.y0         = GetShort(fp);
      break;
    case PLOT_CONT:
      pc->args.cont.x0         = GetShort(fp);
      pc->args.cont.y0         = GetShort(fp);
      break;
    case PLOT_POINT:
      pc->args.point.x0        = GetShort(fp);
      pc->args.point.y0        = GetShort(fp);
      break;
    case PLOT_LINE:
      pc->args.line.x0         = GetShort(fp);
      pc->args.line.y0         = GetShort(fp);
      pc->args.line.x1         = GetShort(fp);
      pc->args.line.y1         = GetShort(fp);
      break;
    case PLOT_LABEL:
      pc->args.label.label     = GetString(fp, &pc->args.label.length);
      break;
    case PLOT_ARC:
      pc->args.arc.center_x    = GetShort(fp);
      pc->args.arc.center_y    = GetShort(fp);
      pc->args.arc.start_x     = GetShort(fp);
      pc->args.arc.start_y     = GetShort(fp);
      pc->args.arc.end_x       = GetShort(fp);
      pc->args.arc.end_y       = GetShort(fp);
      break;
    case PLOT_CIRCLE:
      pc->args.circle.center_x = GetShort(fp);
      pc->args.circle.center_y = GetShort(fp);
      pc->args.circle.radius   = GetShort(fp);
      break;
    case PLOT_ERASE:
      break;
    case PLOT_LINEMODE:
      {
    char *mode;

    mode = GetString(fp, &dummy);
    pc->args.line_mode.mode = FindKeyword(mode, LineModeKeyTab);
    break;
      }
    case PLOT_SPACE:
      pc->args.space.x0        = GetShort(fp);
      pc->args.space.y0        = GetShort(fp);
      pc->args.space.x1        = GetShort(fp);
      pc->args.space.y1        = GetShort(fp);
      break;
#ifdef EXTENDED
    case PLOT_EXT_COLOR:
      pc->args.color.color     = GetString(fp, &pc->args.color.length);
      pc->args.color.pixel     = (unsigned long) -1;
      break;
    case PLOT_EXT_FONT:
      pc->args.font.font       = GetString(fp, &pc->args.font.length);
      break;
    case PLOT_EXT_FILL:
      {
    char *mode;

    mode = GetString(fp, &dummy);
    pc->args.fill_mode.mode = FindKeyword(mode, FillModeKeyTab);
    break;
      }
    case PLOT_EXT_POLY:
      pc->args.poly.count       = GetShort(fp);
      break;
    case PLOT_EXT_POLYDRAW:
      break;
#endif
    }
    pc++;
  }
  /*NOTREACHED*/
}

static void
FreePlotCmdBuf(plotbuf, plotsize)
     PlotCmdBuf plotbuf;
     int plotsize;
{
  register PlotCmdBuf pc;

  for (pc = plotbuf; pc - plotbuf < plotsize && pc->command != PLOT_END; pc++)
    switch (pc->command) {
    case PLOT_LABEL:
      XtFree(pc->args.label.label);
      break;
    default:
      break;
    }

  XtFree((char *) plotbuf);
}

static void
DumpAsciiPlotFile(fout, plotbuf, plotsize)
     FILE *fout;
     PlotCmdBuf plotbuf;
     int plotsize;
{
  register PlotCmdBuf pc;

  for (pc = plotbuf; pc < &plotbuf[plotsize]; pc++) {
    switch (pc->command) {
    case PLOT_END:
      if (pc != &plotbuf[plotsize - 1])
    fprintf(stderr, "%s: plot ended before buffer\n", ProgName);
      return;
    case PLOT_MOVE:
      fprintf(fout, "move %d %d\n", pc->args.move.x0, pc->args.move.y0);
      break;
    case PLOT_CONT:
      fprintf(fout, "cont %d %d\n", pc->args.cont.x0, pc->args.cont.y0);
      break;
    case PLOT_POINT:
      fprintf(fout, "point %d %d\n", pc->args.point.x0, pc->args.point.y0);
      break;
    case PLOT_LINE:
      fprintf(fout, "line %d %d %d %d\n",
          pc->args.line.x0, pc->args.line.y0,
          pc->args.line.x1, pc->args.line.y1);
      break;
    case PLOT_LABEL:
      {
    register char *ptr = pc->args.label.label;
    
    fprintf(fout, "label \"");
    while (*ptr && *ptr == '\b') {
      fprintf(fout, "\\b");
      ptr++;
    }
    if (ptr[strlen(ptr) - 1] == '\n')
      fprintf(fout, "%s\"", ptr);
    else
      fprintf(fout, "%s\"\n", ptr);
    break;
      }
    case PLOT_ARC:
      fprintf(fout, "arc %d %d %d %d %d %d\n",
          pc->args.arc.center_x, pc->args.arc.center_y,
          pc->args.arc.start_x, pc->args.arc.start_y,
          pc->args.arc.end_x, pc->args.arc.end_y);
      break;
    case PLOT_CIRCLE:
      fprintf(fout, "circle %d %d %d\n",
          pc->args.circle.center_x, pc->args.circle.center_y,
          pc->args.circle.radius);
      break;
    case PLOT_ERASE:
      fprintf(fout, "erase\n");
      break;
    case PLOT_LINEMODE:
      {
    register char *mode;

    mode = FindKeywordName(LineModeKeyTab, pc->args.line_mode.mode);
    fprintf(fout, "linemode %s\n", mode);
    break;
      }
    case PLOT_SPACE:
      fprintf(fout, "space %d %d %d %d\n",
          pc->args.space.x0, pc->args.space.y0,
          pc->args.space.x1, pc->args.space.y1);
      break;
#ifdef EXTENDED
    case PLOT_EXT_COLOR:
      {
    register char *ptr = pc->args.color.color;
    
    fprintf(fout, "color \"");
    while (*ptr && *ptr == '\b') {
      fprintf(fout, "\\b");
      ptr++;
    }
    if (ptr[strlen(ptr) - 1] == '\n')
      fprintf(fout, "%s\"", ptr);
    else
      fprintf(fout, "%s\"\n", ptr);
    break;
      }
    case PLOT_EXT_FONT:
      {
    register char *ptr = pc->args.font.font;
    
    fprintf(fout, "font \"");
    while (*ptr && *ptr == '\b') {
      fprintf(fout, "\\b");
      ptr++;
    }
    if (ptr[strlen(ptr) - 1] == '\n')
      fprintf(fout, "%s\"", ptr);
    else
      fprintf(fout, "%s\"\n", ptr);
    break;
      }
    case PLOT_EXT_FILL:
      {
    register char *mode;

    mode = FindKeywordName(FillModeKeyTab, pc->args.fill_mode.mode);
    fprintf(fout, "fillmode %s\n", mode);
    break;
      }
    case PLOT_EXT_POLY:
      fprintf(fout, "poly %d\n", pc->args.poly.count);
      break;
    case PLOT_EXT_POLYDRAW:
      fprintf(fout, "polydraw\n" );
      break;
#endif
    }
  }
}

void
ShowPlot(fin, fout)
     FILE *fin, *fout;
{
  PlotCmdBuf plotbuf;
  int plotsize;

  plotbuf = ReadPlotFile(fin, &plotsize);
  if (plotbuf && (plotsize > 0)) {
    DumpAsciiPlotFile(fout, plotbuf, plotsize);
    FreePlotCmdBuf(plotbuf, plotsize);
  }
}
