/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

/*
 * Copyright 1989 by Metis Technology, Inc.
 *
 *                        All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software and its 
 * documentation for any purpose and without fee is hereby granted, 
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in 
 * supporting documentation.
 *
 * METIS TECHNOLOGY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL METIS TECHNOLOGY BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
 * USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 */



/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#if HAVE_TERMIO_H
#include <termio.h>
#endif

#include <sys/types.h>
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#if HAVE_FCNTL_H
#include <fcntl.h>
#endif
#include <sys/socket.h>
/* this has all sorts of socket things that man says IBM wants */
/* #include <sys/socketvar.h>*/
#include <sys/un.h>
/* this is used for internet sockets, which this isn't */
#include <netinet/in.h>

#ifdef __CYGWIN__
#define TIOCNOTTY 17
#endif

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

/* do some error checking on the compile flags.
   Allowable combinations:
   BUTTONS & ATHENA
   BUTTONS & OPENLOOK
   no BUTTONS & no ATHENA & no OPENLOOK

   ICCCM is independent */
#ifndef BUTTONS
#undef OPENLOOK
#undef ATHENA
#undef MOTIF
#else
/* BUTTONS on */
#ifdef ATHENA
#undef OPENLOOK
#undef MOTIF
#else
/* BUTTONS on and no ATHENA, check MOTIF */
#ifdef MOTIF
#undef OPENLOOK
#else
/* BUTTONS on and no ATHENA or MOTIF, check OPENLOOK */
#ifndef OPENLOOK
#undef BUTTONS
#endif
#endif
#endif
#endif

#ifdef OPENLOOK
/* this can only handle X11R4 (and isn't compilable without it) */
#include <Xol/OpenLook.h>
#include <Xol/Caption.h>
#include <Xol/ControlAre.h>
#include <Xol/Exclusives.h>
#include <Xol/Form.h>
#include <Xol/MenuButton.h>
#include <Xol/OblongButt.h>
#include <Xol/PopupWindo.h>
#include <Xol/RectButton.h>
#include <Xol/StaticText.h>
#include <Xol/TextField.h>
#endif

#ifdef ATHENA
/* this assumes at least X11R4 but works with R5 */
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Dialog.h>
#include <X11/Xaw/Label.h>
#endif

#ifdef MOTIF
/* this assumes Motif 1.2 which assumes X11R5 */
#include <X11/StringDefs.h>
#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/Separator.h>
#include <Xm/TextF.h>
#include <Xm/Label.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeB.h>
#endif

#include "Plot.h"
#include "PlotI.h"

XtAppContext app;
Widget	top = NULL;	/* top level and plot widgets */
Widget plotWidget = NULL;
char * printer;		/* printer name for printing plots */
char * Pprinter = "";	/* -P flag added to printer, if printer is set */
#ifdef BUTTONS
Widget titleWidget = NULL;		/* title widget */
#ifdef ATHENA
Widget savedialog = NULL;	/* save dialog widget */
Widget printdialog = NULL;	/* print dialog widget */
#else
Widget savetext = NULL;	/* save text widget */
Widget printertext = NULL;	/* printer name text widget */
#endif
Widget savepop = NULL;		/* save popup widget */
Widget sfoottext = NULL;	/* save footer text widget */
Widget printerpop = NULL;	/* printer popup widget */
Widget curprinter = NULL;	/* button widget with the current printer name on it */
Widget Helppop = NULL;
#ifdef MOTIF
Widget saveform = NULL;
Widget printerform = NULL;
Widget helpform = NULL;
#endif
#endif

static XrmOptionDescRec Options[] = {
  { "-plotwidth",  "*Plot.plotWidth",	  XrmoptionSepArg, (caddr_t) NULL },
  { "-plotheight", "*Plot.plotHeight",	  XrmoptionSepArg, (caddr_t) NULL },
  { "-nobatch",	   "*Plot.batchGraphics", XrmoptionNoArg,  (caddr_t) "off" },
};

static char *		   NewString();
static void 		   DoEvent();
#ifdef ICCCM
static void DoClientMessage();
#endif
#ifdef BUTTONS
static void		   DoCallback(), OverlayCallback();
#endif
static void		   LinkInput();

char *		ProgName		= (char *) NULL;
char **		PlotFiles     		= (char **) NULL;
char *		TempFileName		= (char *) NULL;
int    		CurPlotFile		= 0;
int    		MaxPlotFile		= 0;
Boolean		OverlayPlotMode		= False;
char *		LinkName		= (char *) NULL;
Boolean		LinkMode		= False;
int		LinkSocket		= -1;
XtInputId	LinkId			= (XtInputId) 0;
int		LinkWaitTimestamp	= 0;
#ifdef ICCCM
Atom		ProtProp		=  None;
Atom		Protocols[3]		= { None, None, None };
#endif


#define XPLOT_TEMPLATE		"/tmp/xplotXXXXXX"
#define XPLOT_EXTENSION		".plot"
#define PRINTER_PREFIX		"-P"
#define PLOT2PS_CMD		"plot2ps"
#define LPR_CMD			"lpr"
#define IMAGEN_CMD		"lipiplot"

#ifdef BUTTONS
static char *HelpText =
"lnkplot - X tool for displaying plot(8) files\n\
\n\
Keyboard commands:\n\
h - print a HARDCOPY of the current plot on a postscript printer\n\
P - bring up PRINTER popup for chosing a postscript printer\n\
s - bring up SAVE popup for saving files\n\
c - CLEAR plot window\n\
n - display the NEXT plot\n\
p - display the PREVIOUS plot\n\
o - toggle the OVERLAY mode\n\
   When on, overlay mode permits display of multile plots.\n\
   Use n and p to add plots to the overlay display.\n\
q - QUIT lnkplot\n\
? - display this help window\n\
";
#endif

int
main(argc, argv)
     int argc;
     char **argv;
{
  Arg args[20];
  int nargs = 0;
  register char **pf = NULL;
/* set application context and fallback resources for X11R4 things */
#ifndef OPENLOOK
  XtAppContext app;
#ifdef MOTIF
    static String fallback_resources[] = {
	"*background: grey",
	"*plot.background: white",
	"*title.background: white",
	"*foreground: black",
	"*XmText.background: lightBlue",
	"*XmTextField.background: lightBlue",
	"*filePull.XmCascadeButton.background: lightGrey",
	"*XmPushButton.background: lightGrey",
	"*fontList: -*-helvetica-bold-r-*-*-*-120-*-*-*-*-iso8859-1",
	"*XmTextField.fontList:-*-courier-medium-r-*-*-*-100-*-*-*-*-iso8859-1",
	"*XmText.fontList:-*-courier-medium-r-*-*-*-100-*-*-*-*-iso8859-1",
	NULL
    };
#else
  static String fallback_resources[] = {"", NULL};
#endif
#endif
#ifdef MOTIF
  XmString xmstr;
#endif
  struct sockaddr_un saddr;
  int s, rc;

  char buf[BUFSIZ];
  register FILE *tf;
  register int cc;

  /* initialize top level shell (window) */
#ifdef OPENLOOK
  top = OlInitialize(argv[0], "Olxplot", Options, XtNumber(Options), &argc, argv);
#else
  top = XtAppInitialize(&app, "Lnkplot", Options, XtNumber(Options),
#ifndef XlibSpecificationRelease
			(Cardinal *) &argc,
#else
#if (XlibSpecificationRelease>=5)
			&argc,
#else
			(Cardinal *) &argc,
#endif
#endif
			argv, fallback_resources, NULL, 0);
#endif

  /* allow input from the keyboard to the top level shell */
  nargs = 0;
#ifdef MOTIF
  XtSetArg ( args [ nargs ], XmNinput, True ), nargs++;
#else
  XtSetArg ( args [ nargs ], XtNinput, True ), nargs++;
#endif
  XtSetValues ( top, args, nargs );

  /* set the program name, if possible */
  if ( ! ( ProgName = NewString ( argv[0] ) ) )
    ProgName = "xplot";
  
  /* read arguments until we hit something without a - first.
     the only flag permitted is -linkname and it must come before
     or instead of any file names that are read below */
  argc--, argv++;
  while ( argc && ( argv[0][0] == '-' ) ) {
    switch ( argv[0][1] ) {
    case 'l':
      if ( strcmp ( & argv[0][1], "linkname" ) == 0 ) {
	argc--, argv++;
	LinkName = NewString ( argv[0] );
	LinkMode = True;
      } else
	fprintf ( stderr, "%s: ignoring unknown option: %s\n",
		  ProgName, argv[0] );
      break;
    default:
      fprintf ( stderr, "%s: ignoring unknown option: %s\n",
	        ProgName, argv[0] );
    }
    argc--, argv++;
  }

  /* if we didn't have the -linkname flag set ... */
  if ( ! LinkMode ) {
      /* if we were given a plot file name just for displaying ...
	 the data will be coming from the file(s) */
    if ( ( MaxPlotFile = argc ) > 0) {
      pf = PlotFiles = (char **) XtMalloc ( ( argc + 1 ) * sizeof (char *) );
      while ( *argv )
	*pf++ = NewString ( *argv++ );
    } else if ( ! isatty ( fileno ( stdin ) ) ) {
	/* this isn't a tty so it must be a pipe. the data will be
	   coming in from stdin */
      
      /* create a temporary file to store the data */
      pf = PlotFiles = (char **) XtMalloc ( 2 * sizeof (char *) );
      (void) strcpy ( buf, XPLOT_TEMPLATE );
      if ( ! ( tf = fdopen ( mkstemp ( buf ), "w" ) ) ) {
	(void) fprintf ( stderr, "%s: can't open temp file %s.\n",
			 ProgName, buf );
	return 1;
      }
      /* write the input data to the temp file one buffer's worth at a time */
      TempFileName = NewString ( buf );
      while ( ( cc = fread ( buf, 1, sizeof (buf), stdin ) ) > 0 ) {
	if ( fwrite ( buf, 1, cc, tf ) == 0 ) {
	  perror ( "Temporary file write error" );
	  (void) unlink ( TempFileName );
	  XtFree(TempFileName);
	  return 1;
	}
      }
      /* check if we had an error while reading the input data */
      if ( ferror ( stdin ) ) {
	perror ( "pipe read error" );
	(void) unlink ( TempFileName );
	XtFree(TempFileName);
	return 1;
      }
      (void) fclose ( stdin );
      (void) fclose ( tf );
      *pf++ = TempFileName, MaxPlotFile = 1;
    }
    /* no link, no name, and no pipe. There is no plot */
    if ( ! pf ) {
      fprintf ( stderr, "%s: no input plot specified\n", ProgName );
      return 1;
    }
    /* make PlotFiles be a NULL terminated array of plot file names */
    *pf = (char *) NULL;
  }
  
  if ( LinkMode ) {
    /* open a two-way unix socket, then connect it to the socket
     specified by LinkName. That socket name is set in PlotOpen which is
     in PlotLink.c (in the xlippic directory) */
#if HAVE_SOCKET
    if ( ( s = socket ( PF_UNIX, SOCK_STREAM, 0 ) ) < 0 ) {
      perror ( "lnkplot: can't open socket");
      return 1;
    }
#else
    if ( ( s = socket ( PF_UNIX, SOCK_STREAM, 0 ) ) < 0 ) {
      perror ( "lnkplot: can't open socket");
      return 1;
    }
#endif
    saddr.sun_family = AF_UNIX;
    strcpy ( saddr.sun_path, LinkName );
    /* try this for the sizeof: */
    /*rc = connect ( s, (struct sockaddr *)&saddr,
      sizeof(saddr) - sizeof(saddr.sun_path) + strlen(LinkName)); */
    rc = connect ( s, (struct sockaddr *)&saddr, sizeof(saddr));
    if(rc < 0){
	perror ( "lnkplot: connect failed");
	fprintf (stderr, "Error code: %d\n", rc);
	return 1;
    }
    /* tell X that when there is data to read on the socket, call LinkInput */
    LinkSocket = s;
    LinkId = XtAppAddInput ( XtWidgetToApplicationContext ( top ),
			     LinkSocket, 
			     (XtPointer) XtInputReadMask,
			     LinkInput,
			     (XtPointer) NULL );
  }

#ifdef BUTTONS
  /* create the menu buttons that go up top */
  {
    Widget f, ca, mp1, b, m;
#ifdef MOTIF
    Widget separator, neighbor, mp2;
#endif

    /* create a form widget to hold the control area that holds the buttons,
       the plot label, the help button, and the plot widget */
#ifdef MOTIF
    nargs    = 0;
    XtSetArg (args[nargs], XmNresizePolicy, XmRESIZE_GROW); nargs++;
    f        = XtCreateManagedWidget ( "form", xmFormWidgetClass, top, args, nargs );
#else
    nargs    = 0;
    f        = XtCreateManagedWidget ( "form", formWidgetClass, top,
				      args, nargs );
#endif
    
#ifdef OPENLOOK
    /* create a control area widget. The control area widget keeps the
       children in neat rows and columns, as requested by the user 
       The request is for all in one row, referenced to a position in
       the form widget above, f, attached to the right of the form widget,
       and allowing the form widget to resize the control area, which it
       won't do because the control area is precisely the size it wants to
       be based on the size of its children */
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNborderWidth, 0 ),		nargs++;
    XtSetArg ( args [ nargs ], XtNlayoutType, OL_FIXEDROWS ),	nargs++;
    XtSetArg ( args [ nargs ], XtNmeasure, 1 ),			nargs++;
    XtSetArg ( args [ nargs ], XtNxRefWidget, f ),		nargs++;
    XtSetArg ( args [ nargs ], XtNyRefWidget, f ),		nargs++;
    XtSetArg ( args [ nargs ], XtNxResizable, True ),		nargs++;
    XtSetArg ( args [ nargs ], XtNxAttachRight, False ),	nargs++;
    ca       = XtCreateManagedWidget ( "controls", controlAreaWidgetClass, f,
				      args, nargs );
#endif
#ifdef ATHENA
    /* create a box widget. The box widget keeps the children widgets
       in a box in the order in which they are added.
       Referenced to a position in
       the form widget above, f, attached to the left of the form widget.
       */
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNborderWidth, 0 ),		nargs++;
    XtSetArg ( args [ nargs ], XtNorientation, XtorientHorizontal ), nargs++;

    XtSetArg ( args [ nargs ], XtNfromHoriz, NULL ),		nargs++;
    XtSetArg ( args [ nargs ], XtNfromVert, NULL ),		nargs++;
    XtSetArg ( args [ nargs ], XtNresizable, True ),		nargs++;
    ca       = XtCreateManagedWidget ( "controls", boxWidgetClass, f,
				      args, nargs );

#endif
#ifdef MOTIF
    nargs = 0;
    XtSetArg ( args [ nargs ], XtNborderWidth, 0); nargs++;
    XtSetArg ( args [ nargs ], XmNtopOffset, 0); nargs++;
    XtSetArg ( args [ nargs ], XmNleftAttachment, XmATTACH_FORM), nargs++;
    XtSetArg ( args [ nargs ], XmNrightAttachment, XmATTACH_FORM), nargs++;
    XtSetArg ( args [ nargs ], XmNleftOffset, 0), nargs++;
    XtSetArg ( args [ nargs ], XmNrightOffset, 0), nargs++;
    /* this is really a specialized RowColumn */
    ca = XmCreateMenuBar ( f, "controls", args, nargs );
    XtManageChild(ca);
#endif

#ifdef OPENLOOK
    /* create the buttons */
    /* first the file menu button and its children */
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "File" ), nargs++;
    m        = XtCreateManagedWidget ( "file", menuButtonWidgetClass, ca,
				      args, nargs );
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNmenuPane, (XtArgVal) &mp1 ), nargs++;
    XtGetValues ( m, args, nargs ); /* get the associated menuPane, mp1 */

    /* attach buttons to the file menu */
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "Print" ), nargs++;
    b        = XtCreateManagedWidget ( "print", menuButtonWidgetClass, mp1,
				      args, nargs );

    /* attach a menu to the print button and add two buttons to that
       menu */
    /* these use mp2, presumably menu pane 2, that is, the menu pane
       created for the Print menu, so we don't loose the pointer to
       the menu pane attached to the file menu */
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNmenuPane, (XtArgVal) &mp2 ), nargs++;
    XtGetValues ( b, args, nargs );

    printer  = NewString("");
    Pprinter = NewString("");
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "(default)" ), nargs++;
    curprinter = XtCreateManagedWidget ( "cur-printer",
					oblongButtonWidgetClass, mp2,
					args, nargs );
    XtAddCallback ( curprinter, XtNselect, DoCallback, (XtPointer) 'h' );
    
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "Printer..." ), nargs++;
    b        = XtCreateManagedWidget ( "printer", oblongButtonWidgetClass, mp2,
				      args, nargs );
    XtAddCallback ( b, XtNselect, DoCallback, (XtPointer) 'P' );
    
    /* attach the save button */
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "Save..." ), nargs++;
    b        = XtCreateManagedWidget ( "save", oblongButtonWidgetClass, mp1,
				      args, nargs );
    XtAddCallback ( b, XtNselect, DoCallback, (XtPointer) 's' );
    
    /* attach the quit button, too */
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "Quit" ), nargs++;
    b        = XtCreateManagedWidget ( "quit", oblongButtonWidgetClass, mp1,
				      args, nargs );
    XtAddCallback ( b, XtNselect, DoCallback, (XtPointer) 'q' );
#endif
#ifdef ATHENA
    
    /* create the buttons */
    /* first the file menu button and its children */
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "File" ), nargs++;
    XtSetArg ( args [ nargs ], XtNmenuName, "File" ), nargs++;
    m        = XtCreateManagedWidget ( "file", menuButtonWidgetClass, ca,
				      args, nargs );

    nargs = 0;
    mp1 = XtCreatePopupShell("File", simpleMenuWidgetClass, m,
			      args, nargs);

    /* attach buttons to the file menu */
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "Save..." ), nargs++;
    b        = XtCreateManagedWidget ( "save", smeBSBObjectClass, mp1,
				      args, nargs );
    XtAddCallback ( b, XtNcallback, DoCallback, (XtPointer) 's' );
    

    /* I have a cascade menu here other places. That is hard
     with athena widgets because the menu buttons are supposed to
     be gadgets, not widgets */
    printer  = NewString("");
    Pprinter = NewString("");

    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "Print to (default)" ), nargs++;
    curprinter = XtCreateManagedWidget ( "cur-printer",
					smeBSBObjectClass, mp1,
					args, nargs );
    XtAddCallback ( curprinter, XtNcallback, DoCallback, (XtPointer) 'h' );
    
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "Set Printer..." ), nargs++;
    b        = XtCreateManagedWidget ( "printer", smeBSBObjectClass, mp1,
				      args, nargs );
    XtAddCallback ( b, XtNcallback, DoCallback, (XtPointer) 'P' );

    /* attach the quit button, too */
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "Quit" ), nargs++;
    b        = XtCreateManagedWidget ( "quit", smeBSBObjectClass, mp1,
				      args, nargs );
    XtAddCallback ( b, XtNcallback, DoCallback, (XtPointer) 'q' );
#endif
#ifdef MOTIF
    /* create the buttons */
    /* first the file menu button and its children */
    nargs    = 0;
    xmstr = XmStringCreateLocalized("File");
    XtSetArg ( args [ nargs ], XmNlabelString, xmstr ), nargs++;
    m = XtCreateManagedWidget ( "fileCascade", xmCascadeButtonWidgetClass, ca,
				 args, nargs );
    XmStringFree(xmstr);

    /* pulldown menus are specialized RowColumns. use the convenience routine */
    nargs = 0;
    mp1 = XmCreatePulldownMenu ( XtParent(m), "filePull", args, nargs);

    nargs = 0;
    XtSetArg ( args [ nargs ], XmNsubMenuId, mp1); nargs++;
    XtSetValues ( m, args, nargs );

    /* attach buttons to the file menu */
    nargs    = 0;
    xmstr = XmStringCreateLocalized("Print");
    XtSetArg ( args [ nargs ], XmNlabelString, xmstr ), nargs++;
    m = XtCreateManagedWidget ("printCascade", xmCascadeButtonWidgetClass, mp1,
			       args, nargs);
    XmStringFree(xmstr);
    nargs = 0;
    mp2 = XmCreatePulldownMenu ( XtParent(m), "printPull", args, nargs);

    nargs = 0;
    XtSetArg ( args [ nargs ], XmNsubMenuId, mp2); nargs++;
    XtSetValues ( m, args, nargs );

    printer  = NewString("");
    Pprinter = NewString("");

    nargs    = 0;
    xmstr = XmStringCreateLocalized("(default)");
    XtSetArg ( args [ nargs ], XmNlabelString, xmstr ), nargs++;
    curprinter = XtCreateManagedWidget ( "cur-printer",
					xmPushButtonWidgetClass, mp2, args, nargs );
    XtAddCallback ( curprinter, XmNactivateCallback, DoCallback, (XtPointer) 'h' );
    XmStringFree(xmstr);
    
    nargs    = 0;
    xmstr = XmStringCreateLocalized("Printer...");
    XtSetArg ( args [ nargs ], XmNlabelString, xmstr ), nargs++;
    b        = XtCreateManagedWidget ( "printer", xmPushButtonWidgetClass, mp2,
				      args, nargs );
    XtAddCallback ( b, XmNactivateCallback, DoCallback, (XtPointer) 'P' );
    XmStringFree(xmstr);
    
    /* attach the save button */
    nargs    = 0;
    xmstr = XmStringCreateLocalized("Save...");
    XtSetArg ( args [ nargs ], XmNlabelString, xmstr ), nargs++;
    b        = XtCreateManagedWidget ( "save", xmPushButtonWidgetClass, mp1,
				      args, nargs );
    XtAddCallback ( b, XmNactivateCallback, DoCallback, (XtPointer) 's' );
    XmStringFree(xmstr);
    
    /* attach the quit button, too */
    nargs    = 0;
    xmstr = XmStringCreateLocalized("Quit");
    XtSetArg ( args [ nargs ], XmNlabelString, xmstr ), nargs++;
    b        = XtCreateManagedWidget ( "quit", xmPushButtonWidgetClass, mp1,
				      args, nargs );
    XtAddCallback ( b, XmNactivateCallback, DoCallback, (XtPointer) 'q' );
    XmStringFree(xmstr);
    
#endif
    
    /* for the MOTIF version, the clear, next and prev buttons go
       on the bottom of the screen and need some layout stuff */
    /* add a clear button to the top control area (ca) */
#ifdef OPENLOOK
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "Clear" ), nargs++;
    b        = XtCreateManagedWidget ( "clear", oblongButtonWidgetClass, ca,
				      args, nargs );
    XtAddCallback ( b, XtNselect, DoCallback, (XtPointer) 'c' );
#endif
#ifdef ATHENA
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "Clear" ), nargs++;
    b        = XtCreateManagedWidget ( "clear", commandWidgetClass, ca,
				      args, nargs );
    XtAddCallback ( b, XtNcallback, DoCallback, (XtPointer) 'c' );
#endif
#ifdef MOTIF
   nargs    = 0;
    xmstr = XmStringCreateLocalized("Clear");
    XtSetArg ( args [ nargs ], XmNlabelString, xmstr ), nargs++;
    XtSetArg ( args [ nargs ], XmNtopAttachment, XmATTACH_NONE); nargs++;
    XtSetArg ( args [ nargs ], XmNbottomAttachment, XmATTACH_FORM); nargs++;
    XtSetArg ( args [ nargs ], XmNleftAttachment, XmATTACH_FORM); nargs++;
    XtSetArg ( args [ nargs ], XmNbottomOffset, 10); nargs++;
    XtSetArg ( args [ nargs ], XmNleftOffset, 20); nargs++;
    /* I need to turn keyboard traversal off for this button because when
       it is on, no keyboard events go to the Plot window and all the neat
       shortcuts on the help popup don't work */
    XtSetArg ( args [ nargs ], XmNtraversalOn, False), nargs++;
    b        = XtCreateManagedWidget ( "clear", xmPushButtonWidgetClass, f,
				      args, nargs );
    XtAddCallback ( b, XmNactivateCallback, DoCallback, (XtPointer) 'c' );
    XmStringFree(xmstr);
    neighbor = b;
#endif

    /* add buttons to the next menu */
#ifdef OPENLOOK
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "Next" ), nargs++;
    b        = XtCreateManagedWidget ( "next", oblongButtonWidgetClass, ca,
				      args, nargs );
    XtAddCallback ( b, XtNselect, DoCallback, (XtPointer) 'n' );
#endif
#ifdef ATHENA
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "Next" ), nargs++;
    b        = XtCreateManagedWidget ( "next", commandWidgetClass, ca,
				      args, nargs );
    XtAddCallback ( b, XtNcallback, DoCallback, (XtPointer) 'n' );
#endif
#ifdef MOTIF
    nargs    = 0;
    xmstr = XmStringCreateLocalized("Next");
    XtSetArg ( args [ nargs ], XmNlabelString, xmstr ), nargs++;
    XtSetArg ( args [ nargs ], XmNtopAttachment, XmATTACH_NONE); nargs++;
    XtSetArg ( args [ nargs ], XmNbottomAttachment, XmATTACH_FORM); nargs++;
    XtSetArg ( args [ nargs ], XmNleftAttachment, XmATTACH_WIDGET); nargs++;
    XtSetArg ( args [ nargs ], XmNrightAttachment, XmATTACH_NONE); nargs++;
    XtSetArg ( args [ nargs ], XmNleftWidget, neighbor); nargs++;
    XtSetArg ( args [ nargs ], XmNbottomOffset, 10); nargs++;
    XtSetArg ( args [ nargs ], XmNleftOffset, 20); nargs++;
    /* I need to turn keyboard traversal off for this button because when
       it is on, no keyboard events go to the Plot window and all the neat
       shortcuts on the help popup don't work */
    XtSetArg ( args [ nargs ], XmNtraversalOn, False), nargs++;
    b        = XtCreateManagedWidget ( "next", xmPushButtonWidgetClass, f,
				      args, nargs );
    XtAddCallback ( b, XmNactivateCallback, DoCallback, (XtPointer) 'n' );
    XmStringFree(xmstr);
    neighbor = b;
#endif
    
#ifdef OPENLOOK
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "Prev" ), nargs++;
    b        = XtCreateManagedWidget ( "prev", oblongButtonWidgetClass, ca,
				      args, nargs );
    XtAddCallback ( b, XtNselect, DoCallback, (XtPointer) 'p' );
#endif
#ifdef ATHENA
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "Prev" ), nargs++;
    b        = XtCreateManagedWidget ( "prev", commandWidgetClass, ca,
				      args, nargs );
    XtAddCallback ( b, XtNcallback, DoCallback, (XtPointer) 'p' );
#endif
#ifdef MOTIF
    nargs    = 0;
    xmstr = XmStringCreateLocalized("Prev");
    XtSetArg ( args [ nargs ], XmNlabelString, xmstr ), nargs++;
    XtSetArg ( args [ nargs ], XmNtopAttachment, XmATTACH_NONE); nargs++;
    XtSetArg ( args [ nargs ], XmNbottomAttachment, XmATTACH_FORM); nargs++;
    XtSetArg ( args [ nargs ], XmNleftAttachment, XmATTACH_WIDGET); nargs++;
    XtSetArg ( args [ nargs ], XmNrightAttachment, XmATTACH_NONE); nargs++;
    XtSetArg ( args [ nargs ], XmNleftWidget, neighbor); nargs++;
    XtSetArg ( args [ nargs ], XmNbottomOffset, 10); nargs++;
    XtSetArg ( args [ nargs ], XmNleftOffset, 20); nargs++;
    /* I need to turn keyboard traversal off for this button because when
       it is on, no keyboard events go to the Plot window and all the neat
       shortcuts on the help popup don't work */
    XtSetArg ( args [ nargs ], XmNtraversalOn, False), nargs++;
    b        = XtCreateManagedWidget ( "prev", xmPushButtonWidgetClass, f,
				      args, nargs );
    XmStringFree(xmstr);
    XtAddCallback ( b, XmNactivateCallback, DoCallback, (XtPointer) 'p' );
#endif
    
#ifdef MOTIF
    /* Rich wants a Close button and a separator,
       so this looks like the new windows */
    nargs = 0;
    xmstr = XmStringCreateLocalized("Close");
    XtSetArg ( args [ nargs ], XmNlabelString, xmstr ), nargs++;
    XtSetArg ( args [ nargs ], XmNtopAttachment, XmATTACH_NONE), nargs++;
    XtSetArg ( args [ nargs ], XmNbottomAttachment, XmATTACH_FORM), nargs++;
    XtSetArg ( args [ nargs ], XmNbottomOffset, 10), nargs++;
    XtSetArg ( args [ nargs ], XmNleftAttachment, XmATTACH_NONE), nargs++;
    XtSetArg ( args [ nargs ], XmNrightAttachment, XmATTACH_FORM), nargs++;
    XtSetArg ( args [ nargs ], XmNrightOffset, 10), nargs++;
    /* I need to turn keyboard traversal off for this button because when
       it is on, no keyboard events go to the Plot window and all the neat
       shortcuts on the help popup don't work */
    XtSetArg ( args [ nargs ], XmNtraversalOn, False), nargs++;

    b = XtCreateManagedWidget ( "close", xmPushButtonWidgetClass, f, args, nargs);
    XtAddCallback(b, XmNactivateCallback, DoCallback, (XtPointer) 'q');
    XmStringFree(xmstr);
    neighbor = b;

    nargs = 0;
    XtSetArg ( args [ nargs ], XmNtopAttachment, XmATTACH_NONE), nargs++;
    XtSetArg ( args [ nargs ], XmNbottomAttachment, XmATTACH_WIDGET), nargs++;
    XtSetArg ( args [ nargs ], XmNleftAttachment, XmATTACH_FORM), nargs++;
    XtSetArg ( args [ nargs ], XmNrightAttachment, XmATTACH_FORM), nargs++;
    XtSetArg ( args [ nargs ], XmNrightOffset, 0), nargs++;
    XtSetArg ( args [ nargs ], XmNleftOffset, 0), nargs++;
    XtSetArg ( args [ nargs ], XmNbottomOffset, 10), nargs++;
    XtSetArg ( args [ nargs ], XmNbottomWidget, neighbor), nargs++;
    separator = XtCreateManagedWidget ( "separator", xmSeparatorWidgetClass, f, args, nargs);
#endif
    /* add the overlay menu */
#ifdef ATHENA
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "Overlay" ), nargs++;
    XtSetArg ( args [ nargs ], XtNmenuName, "Overlay" ), nargs++;
    m        = XtCreateManagedWidget ( "overlay", menuButtonWidgetClass, ca,
				      args, nargs );
#endif
#ifdef OPENLOOK
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "Overlay" ), nargs++;
    m        = XtCreateManagedWidget ( "overlay", menuButtonWidgetClass, ca,
				      args, nargs );
#endif
#ifdef MOTIF
    nargs = 0;
    xmstr = XmStringCreateLocalized("Overlay");
    XtSetArg ( args [ nargs ], XmNlabelString, xmstr ), nargs++;
    m = XtCreateManagedWidget ("overlay", xmCascadeButtonWidgetClass, ca, args, nargs);
    XmStringFree(xmstr);
#endif
#ifdef OPENLOOK
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNmenuPane, (XtArgVal) &mp1 ), nargs++;
    XtGetValues ( m, args, nargs );
#endif
#ifdef ATHENA
    nargs = 0;
    mp1 = XtCreatePopupShell("Overlay", simpleMenuWidgetClass, m,
			      args, nargs);
#endif
#ifdef MOTIF
    nargs = 0;
    mp1 = XmCreatePulldownMenu(XtParent(m), "overlayPull", args, nargs);

    nargs = 0;
    XtSetArg ( args [ nargs ], XmNsubMenuId, mp1); nargs++;
    XtSetValues ( m, args, nargs );
#endif

#ifdef OPENLOOK
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "Enable" ), nargs++;
    b        = XtCreateManagedWidget ( "enable", oblongButtonWidgetClass, mp1,
				      args, nargs );
    XtAddCallback ( b, XtNselect, OverlayCallback, (XtPointer) True );
#endif
#ifdef ATHENA
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "Enable" ), nargs++;
    b        = XtCreateManagedWidget ( "enable", smeBSBObjectClass, mp1,
				      args, nargs );
    XtAddCallback ( b, XtNcallback, OverlayCallback, (XtPointer) True );
#endif
#ifdef MOTIF
    nargs = 0;
    xmstr = XmStringCreateLocalized("Enable");
    XtSetArg ( args [ nargs ], XmNlabelString, xmstr ), nargs++;
    b = XtCreateManagedWidget ( "enable", xmPushButtonWidgetClass, mp1, args, nargs );
    XmStringFree(xmstr);
    XtAddCallback ( b, XmNactivateCallback, OverlayCallback, (XtPointer) True );
#endif

#ifdef OPENLOOK
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "Disable" ), nargs++;
    b        = XtCreateManagedWidget ( "disable", oblongButtonWidgetClass, mp1,
				      args, nargs );
    XtAddCallback ( b, XtNselect, OverlayCallback, (XtPointer) False );
#endif
#ifdef ATHENA
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "Disable" ), nargs++;
    b        = XtCreateManagedWidget ( "disable", smeBSBObjectClass, mp1,
				      args, nargs );
    XtAddCallback ( b, XtNcallback, OverlayCallback, (XtPointer) False );
#endif
#ifdef MOTIF
    nargs = 0;
    xmstr = XmStringCreateLocalized("Disable");
    XtSetArg ( args [ nargs ], XmNlabelString, xmstr ), nargs++;
    b = XtCreateManagedWidget ( "disable", xmPushButtonWidgetClass, mp1, args, nargs );
    XmStringFree(xmstr);
    XtAddCallback ( b, XmNactivateCallback, OverlayCallback, (XtPointer) False );
#endif

    /* create a text widget with the title of the current plot
     I guess that all the fancy stuff is to keep it centered even when
     things are resized */
    /* the title goes on the form widget, centered in the x direction on
       the form, below the control area widget */
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNborderWidth, 0 ),		nargs++;
#ifdef OPENLOOK
    if ( PlotFiles && PlotFiles [ CurPlotFile ] )
      XtSetArg ( args [ nargs ], XtNstring, PlotFiles [ CurPlotFile ] ), nargs++;
    else
      XtSetArg ( args [ nargs ], XtNstring, "Unnamed Plot" ),	nargs++;
    XtSetArg ( args [ nargs ], XtNalignment, OL_CENTER ),	nargs++;
    XtSetArg ( args [ nargs ], XtNrecomputeSize, False ),	nargs++;
    XtSetArg ( args [ nargs ], XtNxRefWidget, f ),		nargs++;
    XtSetArg ( args [ nargs ], XtNyRefWidget, ca ), 		nargs++;
    XtSetArg ( args [ nargs ], XtNyAddHeight, True ),		nargs++;
    XtSetArg ( args [ nargs ], XtNxResizable, True ), 		nargs++;
    XtSetArg ( args [ nargs ], XtNxAttachRight, True ),		nargs++;
    titleWidget = XtCreateManagedWidget ( "title", staticTextWidgetClass, f,
					 args, nargs );
#endif
#ifdef ATHENA
    if ( PlotFiles && PlotFiles [ CurPlotFile ] )
      XtSetArg ( args [ nargs ], XtNlabel, PlotFiles [ CurPlotFile ] ), nargs++;
    else
      XtSetArg ( args [ nargs ], XtNlabel, "Unnamed Plot" ),	nargs++;
    XtSetArg ( args [ nargs ], XtNjustify, XtJustifyCenter ),	nargs++;
    XtSetArg ( args [ nargs ], XtNfromVert, ca ), 		nargs++;
    XtSetArg ( args [ nargs ], XtNvertDistance, 0 ),		nargs++;
    XtSetArg ( args [ nargs ], XtNresizable, True ), 		nargs++;
    titleWidget = XtCreateManagedWidget ( "title", labelWidgetClass, f,
					 args, nargs );
#endif
#ifdef MOTIF
    if ( PlotFiles && PlotFiles [ CurPlotFile ] )
      xmstr = XmStringCreateLocalized( PlotFiles [ CurPlotFile ] );
    else
      xmstr = XmStringCreateLocalized( "Unnamed Plot" );

    XtSetArg ( args [ nargs ], XmNlabelString, xmstr ),	nargs++;
    XtSetArg ( args [ nargs ], XmNtopAttachment, XmATTACH_WIDGET ), nargs++;
    XtSetArg ( args [ nargs ], XmNbottomAttachment, XmATTACH_NONE ), nargs++;
    XtSetArg ( args [ nargs ], XmNleftAttachment, XmATTACH_FORM ), nargs++;
    XtSetArg ( args [ nargs ], XmNrightAttachment, XmATTACH_FORM ), nargs++;
    XtSetArg ( args [ nargs ], XmNtopWidget, ca ), nargs++;
    XtSetArg ( args [ nargs ], XmNtopOffset, 0 ), nargs++;
    XtSetArg ( args [ nargs ], XmNbottomOffset, 0 ), nargs++;
    XtSetArg ( args [ nargs ], XmNleftOffset, 0 ), nargs++;
    XtSetArg ( args [ nargs ], XmNrightOffset, 0 ), nargs++;
    titleWidget = XtCreateManagedWidget ( "title", xmLabelWidgetClass, f,
					 args, nargs );
    XmStringFree(xmstr);
#endif
    
    /* create a help button in the right top corner of the form widget */
#ifdef OPENLOOK
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "Help" ), nargs++;
    XtSetArg ( args [ nargs ], XtNxRefWidget, f ),		nargs++;
    XtSetArg ( args [ nargs ], XtNyRefWidget, f ),		nargs++;
    XtSetArg ( args [ nargs ], XtNxResizable, False ),		nargs++;
    XtSetArg ( args [ nargs ], XtNxAttachRight, True ),	nargs++;
    XtSetArg ( args [ nargs ], XtNxVaryOffset, True ),	nargs++;
    XtSetArg ( args [ nargs ], XtNxAttachOffset, 10 ),	nargs++;
    XtSetArg ( args [ nargs ], XtNyOffset, 5 ),	nargs++;
    b        = XtCreateManagedWidget ( "help", oblongButtonWidgetClass, f,
				      args, nargs );
    XtAddCallback ( b, XtNselect, DoCallback, (XtPointer) '?' );
#endif
#ifdef ATHENA
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "Help" ), nargs++;
/*
    XtSetArg ( args [ nargs ], XtNresizable, False ),		nargs++;
    XtSetArg ( args [ nargs ], XtNtop, XtChainTop ),		nargs++;
    XtSetArg ( args [ nargs ], XtNright, XtChainRight ),	nargs++;
*/
    b        = XtCreateManagedWidget ( "help", commandWidgetClass, ca,
				      args, nargs );
    XtAddCallback ( b, XtNcallback, DoCallback, (XtPointer) '?' );
#endif
#ifdef MOTIF
    nargs    = 0;
    xmstr = XmStringCreateLocalized("Help");
    XtSetArg ( args [ nargs ], XmNlabelString, xmstr ), nargs++;
    m        = XtCreateManagedWidget ( "helpCascade",
				      xmCascadeButtonWidgetClass, ca,
				      args, nargs );
    XmStringFree(xmstr);
    nargs = 0;
    mp1 = XmCreatePulldownMenu(XtParent(m), "helpPull", args, nargs);

    nargs = 0;
    XtSetArg ( args [ nargs ], XmNsubMenuId, mp1); nargs++;
    XtSetValues ( m, args, nargs );
    
    nargs = 0;
    xmstr = XmStringCreateLocalized("Help");
    XtSetArg ( args [ nargs ], XmNlabelString, xmstr ), nargs++;
    b        = XtCreateManagedWidget ( "help",
				      xmPushButtonWidgetClass, mp1,
				      args, nargs );
    XmStringFree(xmstr);
    XtAddCallback ( b, XmNactivateCallback, DoCallback, (XtPointer) '?' );

    /* make the help button the help menu item */
    nargs = 0;
    XtSetArg ( args [ nargs ], XmNmenuHelpWidget, b), nargs++;
    XtSetValues(ca, args, nargs);
#endif

    /* create a plot widget for putting the plot into */
    /* the plot widget is defined in Plot.c */
    /* the plot widget is placed on the form, below the text widget for the
       plot label, attached to the bottom of the form */
    nargs    = 0;
    XtSetArg ( args [ nargs ], XtNborderWidth, 0 ), nargs++;
    if ( PlotFiles && PlotFiles [ CurPlotFile ] )
      XtSetArg ( args [ nargs ], XtNfile, PlotFiles [ CurPlotFile ] ), nargs++;
#ifdef OPENLOOK
    XtSetArg ( args [ nargs ], XtNxRefWidget, f ),		nargs++;
    XtSetArg ( args [ nargs ], XtNyRefWidget, titleWidget ),	nargs++;
    XtSetArg ( args [ nargs ], XtNyAddHeight, True ),		nargs++;
    XtSetArg ( args [ nargs ], XtNxResizable, True ), 		nargs++;
    XtSetArg ( args [ nargs ], XtNyResizable, True ), 		nargs++;
    XtSetArg ( args [ nargs ], XtNyAttachBottom, True ),	nargs++;
#endif
#ifdef ATHENA
    XtSetArg ( args [ nargs ], XtNfromVert, titleWidget ),	nargs++;
    XtSetArg ( args [ nargs ], XtNvertDistance, 0 ),		nargs++;
    XtSetArg ( args [ nargs ], XtNresizable, True ), 		nargs++;
    XtSetArg ( args [ nargs ], XtNbottom, XtChainBottom ),	nargs++;
#endif
#ifdef MOTIF
    XtSetArg ( args [ nargs ], XmNtopAttachment, XmATTACH_WIDGET ), nargs++;
    XtSetArg ( args [ nargs ], XmNbottomAttachment, XmATTACH_WIDGET ), nargs++;
    XtSetArg ( args [ nargs ], XmNleftAttachment, XmATTACH_FORM ), nargs++;
    XtSetArg ( args [ nargs ], XmNrightAttachment, XmATTACH_FORM ), nargs++;
    XtSetArg ( args [ nargs ], XmNtopWidget, titleWidget ), nargs++;
    XtSetArg ( args [ nargs ], XmNbottomWidget, separator ), nargs++;
    XtSetArg ( args [ nargs ], XmNtopOffset, 0 ), nargs++;
    XtSetArg ( args [ nargs ], XmNbottomOffset, 0 ), nargs++;
    XtSetArg ( args [ nargs ], XmNleftOffset, 0 ), nargs++;
    XtSetArg ( args [ nargs ], XmNrightOffset, 0 ), nargs++;
#endif
    plotWidget       = XtCreateManagedWidget ( "plot", plotWidgetClass, f,
					      args, nargs );
  }

#else

  /* this is the plain version without the buttons */
  /* create a plot widget and set it to read from the given file */
  /* the plot widget is defined in Plot.c */
  nargs    = 0;
  if ( PlotFiles && PlotFiles [ CurPlotFile ] )
    XtSetArg ( args [ nargs ], XtNfile, PlotFiles [ CurPlotFile ] ), nargs++;
  plotWidget = XtCreateManagedWidget( "plot", plotWidgetClass, top, args, nargs );

#endif

#ifdef ICCCM
  XtAddEventHandler ( top,
		      NoEventMask,
		      True,
		      DoClientMessage,
		      (XtPointer) NULL );
#endif

  XtAddEventHandler ( plotWidget,
		      KeyReleaseMask | ButtonPressMask,
		      False,
		      DoEvent,
		      (XtPointer) NULL );

  XtRealizeWidget(top);

#ifdef ICCCM
  /* get the atoms for the double quoted strings */
  /* REPLACE the WM_PROTOCOLS property which is an ATOM to a 32 bit
     number which is somewhow made of the three WM_ atoms */
  /* these are all three of the standard window manager protocols
     associated with ICCCM */
  /* so what does that do? */
  {
    Display *dpy = XtDisplay ( top );
    Atom type;
    ProtProp        = XInternAtom ( dpy, "WM_PROTOCOLS", False );
    type            = XInternAtom ( dpy, "ATOM", False );
    Protocols [ 0 ] = XInternAtom ( dpy, "WM_TAKE_FOCUS", False );
    Protocols [ 1 ] = XInternAtom ( dpy, "WM_SAVE_YOURSELF", False );
    Protocols [ 2 ] = XInternAtom ( dpy, "WM_DELETE_WINDOW", False );
    XChangeProperty ( dpy, XtWindow ( top ),
		      ProtProp, type, 32, PropModeReplace,
		      (char *) Protocols, 3 );
  }
#endif
  
#ifdef OPENLOOK
  XtMainLoop();
#else
  XtAppMainLoop(app);
#endif

  return 0;
}

#ifdef ICCCM
/* if we have gotten a message from the client (i don't know how) */
/* the only thing we are really handling is to delete ourselves and
   then the only thing we really do about it is to unlink file
   in the tmp directory (created to hold the data from a pipe) */
static void
DoClientMessage ( w, closure, event, cont )
     Widget w;
     XtPointer closure;
     XEvent *event;
     Boolean *cont;
{
  XClientMessageEvent *ce;
  Atom protocol;

  switch ( event->type ) {
  case ClientMessage:
    if ( ProtProp == None )
      break;
    ce = (XClientMessageEvent *) event;
    if ( ( ce->message_type != ProtProp ) ||
	 ( ce->format != 32 ) )
      break;
    protocol = ce->data.l[0];
    if ( protocol == Protocols [ 0 ] ) {	/* WM_TAKE_FOCUS */

    } else if ( protocol == Protocols [ 1 ] ) {	/* WM_SAVE_YOURSELF */

    } else if ( protocol == Protocols [ 2 ] ) { /* WM_DELETE_WINDOW */
      if ( TempFileName )
	(void) unlink ( TempFileName );
      exit ( 0 );
    }
    break;
  default:
    break;
  }
  if ( cont )
    *cont = TRUE;
}
#endif

/* there is input in the socket from an xlippic generated plot */
/*ARGSUSED*/
static void
LinkInput ( data, source, curId )
     XtPointer data;
     int *source;
     XtInputId *curId;
{
  PlotCmd plotbuf;
  register PlotCmd *pc = &plotbuf;
  int cc;
  char strbuf [ 1024 ];
#ifdef BUTTONS
  Arg args[20];
  int nargs = 0;
#endif
  int terminate = 0;
#ifdef MOTIF
  XmString xmstr;
#endif

  /* if there is no current socket, quit */
  if ( LinkSocket < 0 ) {
    XtRemoveInput ( *curId );
    return;
  }

  /* read a command, if there is an error, close the socket and
     do a remove input */
  if ( ( cc = read ( *source, (char *) pc, sizeof (*pc) ) ) < 0 ) {
    perror ( "LinkInput: read error" );
    goto down;
  }
  switch ( pc->command ) {
  case PLOT_LINK_TERMINATE:
      /* disconnect AND terminate */
    terminate = 1;
  case PLOT_LINK_DISCONNECT:
    /* just disconnect */
    (void) shutdown ( *source, 0 ); /* shut our side of the socket */
    pc->command = PLOT_LINK_DISCONNECT;
    /* send a disconnect command down the other side of the pipe to
       tell it to shut down too */
    (void) write ( *source, (char *) pc, sizeof (*pc) );

    /* do the things done at "down" */
    XtRemoveInput ( *curId );
    (void) close (LinkSocket);
    LinkSocket = -1;

    if ( terminate )
      exit ( 0 );
    return;

  case PLOT_LINK_POINTERWAIT:
    LinkWaitTimestamp = pc->args.timestamp.timestamp;
    break;
  case PLOT_LINK_POINTERREAD:
    {
	/* ask where the pointer is, translate it from plot widget
	   coordinates to xlippic uv coordinates (presumably) */
      Window root, child = None;
      int root_x, root_y, win_x, win_y, mask, px, py;
      PlotCmd retplotbuf;
      register PlotCmd *rpc = &retplotbuf;
      (void) XQueryPointer ( XtDisplay ( plotWidget ),
			     XtWindow  ( plotWidget ),
			     &root, &child,
			     &root_x, &root_y, &win_x, &win_y, &mask );
      PlotTranslate ( plotWidget, win_x, win_y, &px, &py );
      rpc->command = PLOT_LINK_POINTERREAD;
      rpc->args.pointer.x = px;
      rpc->args.pointer.y = py;
      rpc->args.pointer.buttons = 0;
      rpc->args.pointer.modifiers = mask;
      rpc->args.pointer.timestamp = pc->args.timestamp.timestamp;
      if ( ( cc = write ( LinkSocket, (char *) rpc, sizeof (*rpc) ) ) < 0 ) {
	perror ( "LinkInput: write" );
	goto down;
      }
      if ( cc != sizeof (*rpc) ) {
	fprintf ( stderr, "LinkInput: link write out of sync\n" );
	goto down;
      }
      break;
    }
  default:
    /* if this is a simple command (not one of the other cases,
       send the the plot command onto the Plot function to be handled and
       then return */
    switch ( pc->command ) {
    case PLOT_LABEL:
    case PLOT_EXT_COLOR:
    case PLOT_EXT_FONT:
    case PLOT_LINK_SETTITLE:
      break;
    default:
      Plot ( plotWidget, pc );
      return;
    }
    /* otherwise, this was one of the harder commands and we need to read
       some more arguments from the input */
    if ( read ( *source, strbuf, 1 ) != 1 ) {
      perror ( "LinkInput: read error" );
      goto down;
    }
    if ( read ( *source, &strbuf[1], strbuf[0] ) != strbuf[0] ) {
      perror ( "LinkInput: read error" );
      goto down;
    }
    /* set the plot label, color, font, or title to the new value */
    strbuf [ strbuf [ 0 ] + 1 ] = '\0';
    switch ( pc->command ) {
    case PLOT_LABEL:
      pc->args.label.label = NewString ( & strbuf [ 1 ] );
      break;
    case PLOT_EXT_COLOR:
      pc->args.color.color = NewString ( & strbuf [ 1 ] );
      break;
    case PLOT_EXT_FONT:
      pc->args.font.font = NewString ( & strbuf [ 1 ] );
      break;
    case PLOT_LINK_SETTITLE:
#ifdef BUTTONS
      /* if we are to set the title, if this is openlook so we have
	 a title to set, set the string attached to tw to the new string */
      nargs = 0;
#ifdef OPENLOOK
      if ( strbuf [ 0 ] )
	XtSetArg ( args [ nargs ], XtNstring, & strbuf [ 1 ] ), nargs ++;
      else
	XtSetArg ( args [ nargs ], XtNstring, "" ), nargs ++;
#endif
#ifdef ATHENA
      if ( strbuf [ 0 ] )
	XtSetArg ( args [ nargs ], XtNlabel, & strbuf [ 1 ] ), nargs ++;
      else
	XtSetArg ( args [ nargs ], XtNlabel, "" ), nargs ++;
#endif
#ifdef MOTIF
      if ( strbuf [ 0 ] )
	xmstr = XmStringCreateLocalized ( & strbuf[1] );
      else
	xmstr = XmStringCreateLocalized ( "" );
      XtSetArg (args [nargs], XmNlabelString, xmstr), nargs++;
#endif
      XtSetValues ( titleWidget, args, nargs );
#ifdef MOTIF
      XmStringFree(xmstr);
#endif
#endif
      return;
    }
    Plot ( plotWidget, pc );
  }
  return;

 down:
  XtRemoveInput ( *curId );
  (void) close ( LinkSocket );
  LinkSocket = -1;
}

#ifdef BUTTONS
/* Save popup */
/*ARGSUSED*/
static void
SaveCallback ( w, client, call )
     Widget w;
     XtPointer client, call;
{
  FILE *df;
  char *buf = (char *) NULL;
#ifndef ATHENA
  int nargs;
  Arg args[20];
#endif
#ifdef OPENLOOK
  char errbuf[128];
#endif

#ifdef OPENLOOK
  nargs    = 0;
  XtSetArg ( args [ nargs ], XtNstring, &buf ), nargs++;
  XtGetValues ( savetext, args, nargs );
#endif
#ifdef ATHENA
  buf = XawDialogGetValueString(savedialog);
#endif
#ifdef MOTIF
  nargs = 0;
  XtSetArg ( args [ nargs ], XmNvalue, &buf), nargs++;
  XtGetValues ( savetext, args, nargs );
#endif

  if ( buf ) {
    if ( (df = fopen ( buf, "w+" )) ) {
      DumpPlot ( plotWidget, df, True );
      (void) fclose ( df );
    } else {
#ifdef OPENLOOK
      extern int errno;
      extern char *sys_errlist[];

      /* post error to footer text */

      sprintf ( errbuf, "Can't create \"%s: %s", buf, sys_errlist [ errno ] );
      nargs    = 0;
      XtSetArg ( args [ nargs ], XtNstring, errbuf ), nargs++;
      XtSetValues ( sfoottext, args, nargs );
#endif
      return;
    }
#ifdef MOTIF
    XtFree(buf);
#endif
  }

#ifdef OPENLOOK
  /* clear footer text */
  errbuf [ 0 ] = '\0';
  nargs    = 0;
  XtSetArg ( args [ nargs ], XtNstring, errbuf ), nargs++;
  XtSetValues ( sfoottext, args, nargs );
#endif
  XtPopdown ( savepop );
}

/*ARGSUSED*/
static void
CancelSaveCallback ( w, client, call )
     Widget w;
     XtPointer client, call;
{
#ifdef OPENLOOK
  int nargs;
  Arg args[20];
  char c = '\0';

  /* clear message text field */
  nargs    = 0;
  XtSetArg ( args [ nargs ], XtNstring, &c ), nargs++;
  XtSetValues ( sfoottext, args, nargs );
#endif

  XtPopdown ( savepop );
}

static Widget
CreateSavePopup ( )
{
  int nargs;
  Arg args[20];
  Widget p, b;
#ifdef OPENLOOK
  Widget uc, lc, fc;
#endif
#ifdef MOTIF
  Widget cw, f, s;
  Widget attachWidget;
  XmString xmstr;
#endif

  /* create the popup shell */
  nargs    = 0;
  XtSetArg ( args [ nargs ], XtNtitle, "Save Plot..." ), nargs++;
#ifdef OPENLOOK
  p        = XtCreatePopupShell ( "savePopup", popupWindowShellWidgetClass,
				 top, args, nargs );

  /* get the control areas */
  nargs    = 0;
  XtSetArg ( args [ nargs ], XtNupperControlArea, &uc ), nargs++;
  XtSetArg ( args [ nargs ], XtNlowerControlArea, &lc ), nargs++;
  XtSetArg ( args [ nargs ], XtNfooterPanel, &fc ),      nargs++;
  XtGetValues ( p, args, nargs );

#ifdef notdef
  XtRemoveAllCallbacks ( lc, XtNpostSelect );
#endif

  /* put the file text widget in the upper box */
  nargs    = 0;
  XtSetArg ( args [ nargs ], XtNlabel, "Filename:" ),   nargs++;
  cw       = XtCreateManagedWidget ( "savePopupCaption", captionWidgetClass,
				    uc, args, nargs );

  nargs    = 0;
  savetext = XtCreateManagedWidget ( "savePopupText", textFieldWidgetClass, cw,
				    args, nargs );
  XtAddCallback ( savetext, XtNverification, SaveCallback, (XtPointer) NULL );

  /* put the save and cancel button widgets in the lower box */
  nargs    = 0;
  XtSetArg ( args [ nargs ], XtNlabel, "Save" ),        nargs++;
  XtSetArg ( args [ nargs ], XtNdefault, True ),          nargs++;
  b        = XtCreateManagedWidget ( "savePopupSaveButton",
				    oblongButtonWidgetClass, lc,
				    args, nargs );
  XtAddCallback ( b, XtNselect, SaveCallback, (XtPointer) NULL );
  XtRemoveAllCallbacks ( b, XtNpostSelect );

  nargs    = 0;
  XtSetArg ( args [ nargs ], XtNlabel, "Cancel" ),        nargs++;
  b        = XtCreateManagedWidget ( "savePopupCancelButton",
				    oblongButtonWidgetClass, lc,
				    args, nargs );
  XtAddCallback ( b, XtNselect, CancelSaveCallback, (XtPointer) NULL );

  /* put a static text widget in the footer */
  nargs    = 0;
  XtSetArg ( args [ nargs ], XtNwidth, 400 ),          nargs++;
  sfoottext = XtCreateManagedWidget ( "savePopupFooterText",
				     staticTextWidgetClass, fc,
				    args, nargs );
#endif
#ifdef ATHENA
  p        = XtCreatePopupShell ( "savePopup", transientShellWidgetClass,
				 top, args, nargs );

  /* create a dialog compound widget in the popup shell */
  /* label is the text at the top of the widget
     value is the default value for the text to be typed in
     setting value activates the text field*/
     
  nargs    = 0;
  XtSetArg ( args [ nargs ], XtNlabel, "Filename:" ),   nargs++;
  XtSetArg ( args [ nargs ], XtNvalue, ""), nargs++;
  savedialog      = XtCreateManagedWidget ( "savePopupDialog",
					   dialogWidgetClass,
					   p , args, nargs );

  /* put the save and cancel button widgets in the lower box */
  nargs    = 0;
  XtSetArg ( args [ nargs ], XtNlabel, "Save" ),        nargs++;
  b        = XtCreateManagedWidget ( "savePopupSaveButton",
				    commandWidgetClass, savedialog,
				    args, nargs );
  XtAddCallback ( b, XtNcallback, SaveCallback, (XtPointer) NULL );
/*  XtRemoveAllCallbacks ( b, XtNpostSelect );*/

  nargs    = 0;
  XtSetArg ( args [ nargs ], XtNlabel, "Cancel" ),        nargs++;
  b        = XtCreateManagedWidget ( "savePopupCancelButton",
				    commandWidgetClass, savedialog,
				    args, nargs );
  XtAddCallback ( b, XtNcallback, CancelSaveCallback, (XtPointer) NULL );
#ifdef notdef
  XtRemoveAllCallbacks ( lc, XtNpostSelect );
#endif

#endif
#ifdef MOTIF
  p        = XtCreatePopupShell ( "savePopup", xmDialogShellWidgetClass,
				 top, args, nargs );

  /* create a form to sit in the dialog */
  nargs = 0;
  XtSetArg ( args [ nargs ], XmNresizePolicy, XmRESIZE_GROW ); nargs++;
  f = XtCreateWidget( "saveForm", xmFormWidgetClass, p, args, nargs );
  saveform = f;
     
  /* put the label and text widget in the top */
  nargs    = 0;
  xmstr = XmStringCreateLocalized("Filename:");
  XtSetArg ( args [ nargs ], XmNlabelString, xmstr ),   nargs++;
  XtSetArg ( args [ nargs ], XmNtopAttachment, XmATTACH_FORM ); nargs++;
  XtSetArg ( args [ nargs ], XmNleftAttachment, XmATTACH_FORM ); nargs++;
  XtSetArg ( args [ nargs ], XmNtopOffset, 5 ); nargs++;
  XtSetArg ( args [ nargs ], XmNleftOffset, 10 ); nargs++;
  cw       = XtCreateManagedWidget ( "saveLabel", xmLabelWidgetClass,
				    f, args, nargs );
  XmStringFree(xmstr);

  nargs    = 0;
  savetext = XtCreateManagedWidget ( "savePopupText", xmTextFieldWidgetClass, f,
				    args, nargs );
  XtAddCallback ( savetext, XmNactivateCallback, SaveCallback, (XtPointer) NULL );

  /* put the save and cancel button widgets in the bottom of the window */
  nargs = 0;
  s = XtCreateManagedWidget ( "saveSeparator", xmSeparatorWidgetClass, f,
			     args, nargs );

  nargs    = 0;
  xmstr = XmStringCreateLocalized("Save");
  XtSetArg ( args [ nargs ], XmNlabelString, xmstr ),        nargs++;
  XtSetArg ( args [ nargs ], XmNtopAttachment, XmATTACH_NONE); nargs++;
  XtSetArg ( args [ nargs ], XmNbottomAttachment, XmATTACH_FORM ), nargs++;
  XtSetArg ( args [ nargs ], XmNleftAttachment, XmATTACH_FORM ), nargs++;
  XtSetArg ( args [ nargs ], XmNrightAttachment, XmATTACH_NONE ), nargs++;
  XtSetArg ( args [ nargs ], XmNbottomOffset, 10 ), nargs++;
  XtSetArg ( args [ nargs ], XmNleftOffset, 10 ), nargs++;
  b        = XtCreateManagedWidget ( "savePopupSaveButton",
				    xmPushButtonWidgetClass, f,
				    args, nargs );
  XtAddCallback ( b, XmNactivateCallback, SaveCallback, (XtPointer) NULL );
  XmStringFree(xmstr);
  attachWidget = b;

  nargs    = 0;
  xmstr = XmStringCreateLocalized("Cancel");
  XtSetArg ( args [ nargs ], XmNlabelString, xmstr ),        nargs++;
  XtSetArg ( args [ nargs ], XmNtopAttachment, XmATTACH_NONE); nargs++;
  XtSetArg ( args [ nargs ], XmNbottomAttachment, XmATTACH_FORM ), nargs++;
  XtSetArg ( args [ nargs ], XmNleftAttachment, XmATTACH_WIDGET ), nargs++;
  XtSetArg ( args [ nargs ], XmNrightAttachment, XmATTACH_NONE ), nargs++;
  XtSetArg ( args [ nargs ], XmNleftWidget, attachWidget ), nargs++;
  XtSetArg ( args [ nargs ], XmNbottomOffset, 10 ), nargs++;
  XtSetArg ( args [ nargs ], XmNleftOffset, 20 ), nargs++;
  XtSetArg ( args [ nargs ], XmNrightOffset, 0 ), nargs++;
  b        = XtCreateManagedWidget ( "savePopupCancelButton",
				    xmPushButtonWidgetClass, f,
				    args, nargs );
  XtAddCallback ( b, XmNactivateCallback, CancelSaveCallback, (XtPointer) NULL );
  XmStringFree(xmstr);
  attachWidget = b;

  nargs = 0;
  xmstr = XmStringCreateLocalized("Close"), nargs++;
  XtSetArg ( args [ nargs ], XmNlabelString, xmstr ), nargs ++;
  XtSetArg ( args [ nargs ], XmNtopAttachment, XmATTACH_NONE); nargs++;
  XtSetArg ( args [ nargs ], XmNbottomAttachment, XmATTACH_FORM ), nargs++;
  XtSetArg ( args [ nargs ], XmNleftAttachment, XmATTACH_WIDGET ), nargs++;
  XtSetArg ( args [ nargs ], XmNrightAttachment, XmATTACH_FORM ), nargs++;
  XtSetArg ( args [ nargs ], XmNleftWidget, attachWidget ), nargs++;
  XtSetArg ( args [ nargs ], XmNbottomOffset, 10 ), nargs++;
  XtSetArg ( args [ nargs ], XmNleftOffset, 20 ), nargs++;
  XtSetArg ( args [ nargs ], XmNrightOffset, 10 ), nargs++;
  b = XtCreateManagedWidget ("saveClosePBtn", xmPushButtonWidgetClass, f,
			     args, nargs );
  XtAddCallback ( b, XmNactivateCallback, CancelSaveCallback, (XtPointer) NULL );
  XmStringFree(xmstr);
  attachWidget = b;

  /* fix up the attachments for the separator and the textfield */
  nargs = 0;
  XtSetArg ( args [ nargs ], XmNtopAttachment, XmATTACH_NONE); nargs++;
  XtSetArg ( args [ nargs ], XmNbottomAttachment, XmATTACH_WIDGET ), nargs++;
  XtSetArg ( args [ nargs ], XmNleftAttachment, XmATTACH_FORM ), nargs++;
  XtSetArg ( args [ nargs ], XmNrightAttachment, XmATTACH_FORM ), nargs++;
  XtSetArg ( args [ nargs ], XmNbottomWidget, attachWidget ), nargs++;
  XtSetArg ( args [ nargs ], XmNbottomOffset, 10 ), nargs++;
  XtSetArg ( args [ nargs ], XmNleftOffset, 0 ), nargs++;
  XtSetArg ( args [ nargs ], XmNrightOffset, 0 ), nargs++;
  XtSetValues (s, args, nargs);

  nargs = 0;
  XtSetArg ( args [ nargs ], XmNtopAttachment, XmATTACH_WIDGET ); nargs++;
  XtSetArg ( args [ nargs ], XmNbottomAttachment, XmATTACH_WIDGET ); nargs++;
  XtSetArg ( args [ nargs ], XmNleftAttachment, XmATTACH_FORM ); nargs++;
  XtSetArg ( args [ nargs ], XmNrightAttachment, XmATTACH_FORM ); nargs++;
  XtSetArg ( args [ nargs ], XmNtopWidget, cw ); nargs++;
  XtSetArg ( args [ nargs ], XmNbottomWidget, s ); nargs++;
  XtSetArg ( args [ nargs ], XmNtopOffset, 0 ); nargs++;
  XtSetArg ( args [ nargs ], XmNbottomOffset, 10 ); nargs++;
  XtSetArg ( args [ nargs ], XmNleftOffset, 10 ); nargs++;
  XtSetArg ( args [ nargs ], XmNrightOffset, 10 ); nargs++;
  XtSetValues(savetext, args, nargs);
#endif
  return p;
}

#endif

#ifdef BUTTONS
/* commands for the printer popup */

/*ARGSUSED*/
static void
PrinterCallback ( w, client, call )
     Widget w;
     XtPointer client, call;
{
  char *buf = (char *) NULL;
  int nargs;
  Arg args[20];
#ifdef MOTIF
  XmString xmstr;
#endif

#ifdef OPENLOOK
  nargs    = 0;
  XtSetArg ( args [ nargs ], XtNstring, &buf ), nargs++;
  XtGetValues ( printertext, args, nargs );
#endif
#ifdef ATHENA
  buf = XawDialogGetValueString(printdialog);
#endif
#ifdef MOTIF
  nargs = 0;
  XtSetArg ( args [ nargs ], XmNvalue, &buf), nargs++;
  XtGetValues ( printertext, args, nargs );
#endif
  /* set the printer string and set it as the label on the other printer
     button */
  if ( buf ) {
      /* copy the printer name and set the label on the menu */
      if (!printer) XtFree (printer);
      printer = NewString(buf);
#ifdef MOTIF
      nargs = 0;
      xmstr = XmStringCreateLocalized(printer);
      XtSetArg ( args [ nargs ] , XmNlabelString, xmstr ), nargs++;
      XtSetValues ( curprinter, args, nargs );
      XmStringFree(xmstr);
#else
      nargs = 0;
      XtSetArg ( args [ nargs ] , XtNlabel, printer ), nargs++;
      XtSetValues ( curprinter, args, nargs );
#endif

      /* set the -P flag for printing with lpr */
      if (strlen(printer) > 0){
	  if (!Pprinter) XtFree(Pprinter);
	  Pprinter =
	    XtMalloc ((unsigned) strlen(printer) + sizeof (PRINTER_PREFIX));
	  sprintf(Pprinter, "%s%s", PRINTER_PREFIX, printer);
      }
#ifdef MOTIF
      XtFree(buf);
#endif
  }

  XtPopdown ( printerpop );
}

#ifdef MOTIF
static void
CancelPrinterCallback ( w, client, call )
     Widget w;
     XtPointer client, call;
{
  XtPopdown ( printerpop );
}
#endif

static Widget
CreatePrinterPopup ( )
{
  int nargs;
  Arg args[20];
  Widget p, b;
#ifdef OPENLOOK
  Widget lc, uc;
#endif
#ifdef MOTIF
  Widget attachWidget, s, f, cw;
  XmString xmstr;
#endif

  /* create the popup shell */
  nargs    = 0;
  XtSetArg ( args [ nargs ], XtNtitle, "Set Printer..." ), nargs++;
#ifdef OPENLOOK
  p        = XtCreatePopupShell ( "printerPopup", popupWindowShellWidgetClass,
				 top, args, nargs );

  nargs    = 0;
  XtSetArg ( args [ nargs ], XtNupperControlArea, &uc ), nargs++;
  XtSetArg ( args [ nargs ], XtNlowerControlArea, &lc ), nargs++;
  XtGetValues ( p, args, nargs );

#ifdef notdef
  XtRemoveAllCallbacks ( lc, XtNpostSelect );
#endif

  /* put the file text widget in the upper box */
  nargs    = 0;
  XtSetArg ( args [ nargs ], XtNlabel, "Printer Name:" ),   nargs++;
  cw       = XtCreateManagedWidget ( "printerPopupCaption", captionWidgetClass,
				    uc, args, nargs );

  nargs    = 0;
  printertext = XtCreateManagedWidget ( "printerPopupText", textFieldWidgetClass, cw,
				    args, nargs );
  XtAddCallback ( printertext, XtNverification, PrinterCallback, (XtPointer) NULL );

  /* put the ok button widget in the lower box */
  nargs    = 0;
  XtSetArg ( args [ nargs ], XtNlabel, "OK" ),        nargs++;
  XtSetArg ( args [ nargs ], XtNdefault, True ),          nargs++;
  b        = XtCreateManagedWidget ( "printerPopupPrinterButton",
				    oblongButtonWidgetClass, lc,
				    args, nargs );
  XtAddCallback ( b, XtNselect, PrinterCallback, (XtPointer) NULL );
  XtRemoveAllCallbacks ( b, XtNpostSelect );

#endif
#ifdef ATHENA
  p        = XtCreatePopupShell ( "printerPopup", transientShellWidgetClass,
				 top, args, nargs );

  /* create a dialog box and populate it with appropriate buttons */
  nargs    = 0;
  XtSetArg ( args [ nargs ], XtNlabel, "Printer Name:" ),   nargs++;
  XtSetArg ( args [ nargs ], XtNvalue, "" ),   nargs++;
  printdialog       = XtCreateManagedWidget ( "printerPopupDialog",
					     dialogWidgetClass,
					     p, args, nargs );

  /* put the ok button widget in the lower box */
  nargs    = 0;
  XtSetArg ( args [ nargs ], XtNlabel, "OK" ),        nargs++;
  b        = XtCreateManagedWidget ( "printerPopupOKButton",
				    commandWidgetClass, printdialog,
				    args, nargs );
  XtAddCallback ( b, XtNcallback, PrinterCallback, (XtPointer) NULL );

#endif
#ifdef MOTIF
  p        = XtCreatePopupShell ( "printPopup", xmDialogShellWidgetClass,
				 top, args, nargs );

  /* create a form to sit in the dialog */
  nargs = 0;
  XtSetArg ( args [ nargs ], XmNresizePolicy, XmRESIZE_GROW ); nargs++;
  f = XtCreateWidget( "printForm", xmFormWidgetClass, p, args, nargs );
  printerform = f;
     
  /* put the label and text widget in the top */
  nargs    = 0;
  xmstr = XmStringCreateLocalized("Printer Name:");
  XtSetArg ( args [ nargs ], XmNlabelString, xmstr ),   nargs++;
  XtSetArg ( args [ nargs ], XmNtopAttachment, XmATTACH_FORM ); nargs++;
  XtSetArg ( args [ nargs ], XmNleftAttachment, XmATTACH_FORM ); nargs++;
  XtSetArg ( args [ nargs ], XmNtopOffset, 5 ); nargs++;
  XtSetArg ( args [ nargs ], XmNleftOffset, 10 ); nargs++;
  cw       = XtCreateManagedWidget ( "printerLabel", xmLabelWidgetClass,
				    f, args, nargs );
  XmStringFree(xmstr);

  nargs    = 0;
  printertext = XtCreateManagedWidget ( "printerPopupText",
				       xmTextFieldWidgetClass, f,
				       args, nargs );
  XtAddCallback ( printertext, XmNactivateCallback, PrinterCallback, (XtPointer) NULL );

  /* put the printer and cancel button widgets in the bottom of the window */
  nargs = 0;
  s = XtCreateManagedWidget ( "printerSeparator", xmSeparatorWidgetClass, f,
			     args, nargs );

  nargs    = 0;
  xmstr = XmStringCreateLocalized("OK");
  XtSetArg ( args [ nargs ], XmNlabelString, xmstr ),        nargs++;
  XtSetArg ( args [ nargs ], XmNtopAttachment, XmATTACH_NONE); nargs++;
  XtSetArg ( args [ nargs ], XmNbottomAttachment, XmATTACH_FORM ), nargs++;
  XtSetArg ( args [ nargs ], XmNleftAttachment, XmATTACH_FORM ), nargs++;
  XtSetArg ( args [ nargs ], XmNrightAttachment, XmATTACH_NONE ), nargs++;
  XtSetArg ( args [ nargs ], XmNleftWidget, XmATTACH_FORM ), nargs++;
  XtSetArg ( args [ nargs ], XmNbottomOffset, 10 ), nargs++;
  XtSetArg ( args [ nargs ], XmNleftOffset, 10 ), nargs++;
  b        = XtCreateManagedWidget ( "printerOKButton",
				    xmPushButtonWidgetClass, f,
				    args, nargs );
  XtAddCallback ( b, XmNactivateCallback, PrinterCallback, (XtPointer) NULL );
  XmStringFree(xmstr);
  attachWidget = b;

  nargs = 0;
  xmstr = XmStringCreateLocalized("Close"), nargs++;
  XtSetArg ( args [ nargs ], XmNlabelString, xmstr ), nargs ++;
  XtSetArg ( args [ nargs ], XmNtopAttachment, XmATTACH_NONE); nargs++;
  XtSetArg ( args [ nargs ], XmNbottomAttachment, XmATTACH_FORM ), nargs++;
  XtSetArg ( args [ nargs ], XmNleftAttachment, XmATTACH_NONE ), nargs++;
  XtSetArg ( args [ nargs ], XmNrightAttachment, XmATTACH_FORM ), nargs++;
  XtSetArg ( args [ nargs ], XmNbottomOffset, 10 ), nargs++;
  XtSetArg ( args [ nargs ], XmNrightOffset, 10 ), nargs++;
  b = XtCreateManagedWidget ("printerClosePBtn", xmPushButtonWidgetClass, f,
			     args, nargs );
  XtAddCallback ( b, XmNactivateCallback, CancelPrinterCallback, (XtPointer) NULL );
  XmStringFree(xmstr);
  attachWidget = b;

  /* fix up the attachments for the separator and the textfield */
  nargs = 0;
  XtSetArg ( args [ nargs ], XmNtopAttachment, XmATTACH_NONE); nargs++;
  XtSetArg ( args [ nargs ], XmNbottomAttachment, XmATTACH_WIDGET ), nargs++;
  XtSetArg ( args [ nargs ], XmNleftAttachment, XmATTACH_FORM ), nargs++;
  XtSetArg ( args [ nargs ], XmNrightAttachment, XmATTACH_FORM ), nargs++;
  XtSetArg ( args [ nargs ], XmNbottomWidget, attachWidget ), nargs++;
  XtSetArg ( args [ nargs ], XmNbottomOffset, 10 ), nargs++;
  XtSetArg ( args [ nargs ], XmNleftOffset, 0 ), nargs++;
  XtSetArg ( args [ nargs ], XmNrightOffset, 0 ), nargs++;
  XtSetValues (s, args, nargs);

  nargs = 0;
  XtSetArg ( args [ nargs ], XmNtopAttachment, XmATTACH_WIDGET ); nargs++;
  XtSetArg ( args [ nargs ], XmNbottomAttachment, XmATTACH_WIDGET ); nargs++;
  XtSetArg ( args [ nargs ], XmNleftAttachment, XmATTACH_FORM ); nargs++;
  XtSetArg ( args [ nargs ], XmNrightAttachment, XmATTACH_FORM ); nargs++;
  XtSetArg ( args [ nargs ], XmNtopWidget, cw ); nargs++;
  XtSetArg ( args [ nargs ], XmNbottomWidget, s ); nargs++;
  XtSetArg ( args [ nargs ], XmNtopOffset, 0 ); nargs++;
  XtSetArg ( args [ nargs ], XmNbottomOffset, 10 ); nargs++;
  XtSetArg ( args [ nargs ], XmNleftOffset, 10 ); nargs++;
  XtSetArg ( args [ nargs ], XmNrightOffset, 10 ); nargs++;
  XtSetValues(printertext, args, nargs);
#endif

  return p;
}

#endif

#ifdef BUTTONS
/* commands for the help popup */
/*ARGSUSED*/
static void
HelpCallback ( w, client, call )
     Widget w;
     XtPointer client, call;
{
  XtPopdown ( Helppop );
}

static Widget
CreateHelpPopup ( )
{
  int nargs;
  Arg args[20];
  Widget p, b;
#ifdef ATHENA
  Widget dialog;
#endif
#ifdef MOTIF
  Widget cw, f, s, attachWidget;
  XmString xmstr;
#endif
#ifdef OPENLOOK
  Widget uc, lc;
#endif

  /* create the popup shell */
  nargs    = 0;
  XtSetArg ( args [ nargs ], XtNtitle, "Lnkplot Help..." ), nargs++;
#ifdef OPENLOOK
  p        = XtCreatePopupShell ( "helpPopup", popupWindowShellWidgetClass,
				 top, args, nargs );

  nargs    = 0;
  XtSetArg ( args [ nargs ], XtNupperControlArea, &uc ), nargs++;
  XtSetArg ( args [ nargs ], XtNlowerControlArea, &lc ), nargs++;
  XtGetValues ( p, args, nargs );

  /* put the static text widget in the upper box */
  /* put a static text widget in the footer */
  nargs    = 0;
  XtSetArg ( args [ nargs ], XtNwidth, 400 ),          nargs++;
  XtSetArg ( args [ nargs ], XtNstring, HelpText ),nargs++;
  b = XtCreateManagedWidget ( "helpText", staticTextWidgetClass, uc,
			     args, nargs );

  /* put the ok button widget in the lower box */
  nargs    = 0;
  XtSetArg ( args [ nargs ], XtNlabel, "OK" ),        nargs++;
  XtSetArg ( args [ nargs ], XtNdefault, True ),          nargs++;
  b        = XtCreateManagedWidget ( "helpPopupHelpButton",
				    oblongButtonWidgetClass, lc,
				    args, nargs );
  XtAddCallback ( b, XtNselect, HelpCallback, (XtPointer) NULL );
  XtRemoveAllCallbacks ( b, XtNpostSelect );
#endif
#ifdef ATHENA
  p        = XtCreatePopupShell ( "helpPopup", transientShellWidgetClass,
				 top, args, nargs );

  /* create a dialog widget to manage the children */
  nargs = 0;
  XtSetArg ( args [ nargs ], XtNlabel, HelpText ), nargs++;
  dialog = XtCreateManagedWidget("dialog", dialogWidgetClass, p, args, nargs);

  /* put the static text widget in the upper box */
  /* put a static text widget in the footer */
  nargs    = 0;
  XtSetArg ( args [ nargs ], XtNwidth, 400 ),          nargs++;
  XtSetArg ( args [ nargs ], XtNlabel, HelpText ),nargs++;

  /* put the ok button widget in the lower box */
  nargs    = 0;
  XtSetArg ( args [ nargs ], XtNlabel, "OK" ),        nargs++;
  b        = XtCreateManagedWidget ( "helpPopupOKButton",
				    commandWidgetClass, dialog,
				    args, nargs );
  XtAddCallback ( b, XtNcallback, HelpCallback, (XtPointer) NULL );
#endif
#ifdef MOTIF
  p        = XtCreatePopupShell ( "helpPopup", xmDialogShellWidgetClass,
				 top, args, nargs );

  /* create a form to sit in the dialog */
  nargs = 0;
  XtSetArg ( args [ nargs ], XmNresizePolicy, XmRESIZE_GROW ); nargs++;
  f = XtCreateWidget( "helpForm", xmFormWidgetClass, p, args, nargs );
  helpform = f;
     
  /* put the label and text widget in the top */
  nargs    = 0;
  xmstr = XmStringCreateLtoR(HelpText, XmFONTLIST_DEFAULT_TAG);
  XtSetArg ( args [ nargs ], XmNlabelString, xmstr ),   nargs++;
  XtSetArg ( args [ nargs ], XmNalignment, XmALIGNMENT_BEGINNING ),   nargs++;
  cw       = XtCreateManagedWidget ( "helpLabel", xmLabelWidgetClass,
				    f, args, nargs );
  XmStringFree(xmstr);

  /* put the help and cancel button widgets in the bottom of the window */
  nargs = 0;
  s = XtCreateManagedWidget ( "helpSeparator", xmSeparatorWidgetClass, f,
			     args, nargs );

  nargs = 0;
  xmstr = XmStringCreateLocalized("Close"), nargs++;
  XtSetArg ( args [ nargs ], XmNlabelString, xmstr ), nargs ++;
  XtSetArg ( args [ nargs ], XmNtopAttachment, XmATTACH_NONE); nargs++;
  XtSetArg ( args [ nargs ], XmNbottomAttachment, XmATTACH_FORM ), nargs++;
  XtSetArg ( args [ nargs ], XmNleftAttachment, XmATTACH_NONE ), nargs++;
  XtSetArg ( args [ nargs ], XmNrightAttachment, XmATTACH_FORM ), nargs++;
  XtSetArg ( args [ nargs ], XmNbottomOffset, 10 ), nargs++;
  XtSetArg ( args [ nargs ], XmNleftOffset, 20 ), nargs++;
  XtSetArg ( args [ nargs ], XmNrightOffset, 10 ), nargs++;
  b = XtCreateManagedWidget ("helpClosePBtn", xmPushButtonWidgetClass, f,
			     args, nargs );
  XtAddCallback ( b, XmNactivateCallback, HelpCallback, (XtPointer) NULL );
  XmStringFree(xmstr);
  attachWidget = b;

  /* fix up the attachments for the separator and the label */
  nargs = 0;
  XtSetArg ( args [ nargs ], XmNtopAttachment, XmATTACH_NONE); nargs++;
  XtSetArg ( args [ nargs ], XmNbottomAttachment, XmATTACH_WIDGET ), nargs++;
  XtSetArg ( args [ nargs ], XmNleftAttachment, XmATTACH_FORM ), nargs++;
  XtSetArg ( args [ nargs ], XmNrightAttachment, XmATTACH_FORM ), nargs++;
  XtSetArg ( args [ nargs ], XmNbottomWidget, attachWidget ), nargs++;
  XtSetArg ( args [ nargs ], XmNbottomOffset, 10 ), nargs++;
  XtSetArg ( args [ nargs ], XmNleftOffset, 0 ), nargs++;
  XtSetArg ( args [ nargs ], XmNrightOffset, 0 ), nargs++;
  XtSetValues (s, args, nargs);

  nargs = 0;
  XtSetArg ( args [ nargs ], XmNtopAttachment, XmATTACH_FORM ); nargs++;
  XtSetArg ( args [ nargs ], XmNbottomAttachment, XmATTACH_WIDGET ); nargs++;
  XtSetArg ( args [ nargs ], XmNleftAttachment, XmATTACH_FORM ); nargs++;
  XtSetArg ( args [ nargs ], XmNrightAttachment, XmATTACH_FORM ); nargs++;
  XtSetArg ( args [ nargs ], XmNbottomWidget, s ); nargs++;
  XtSetArg ( args [ nargs ], XmNtopOffset, 5 ); nargs++;
  XtSetArg ( args [ nargs ], XmNbottomOffset, 10 ); nargs++;
  XtSetArg ( args [ nargs ], XmNleftOffset, 10 ); nargs++;
  XtSetArg ( args [ nargs ], XmNrightOffset, 10 ); nargs++;
  XtSetValues(cw, args, nargs);
#endif
  return p;
}

#endif

/* when the user has requested one of the functions available from the
   menus or by typing to the window */
static void
Dispatch ( pw, c )
     Widget pw;
     int c;
{
  Arg args[20];
  int nargs;
  FILE *fp;
#ifdef MOTIF
  XmString xmstr;
#endif
  int tty;
  char tempfile [ 32 ], cmdbuf [ 1024 ];
  FILE *tf;
#ifdef BUTTONS
  char *title = (char *) NULL, *filename, *cp;
#endif

  if ( ! pw || ! c )
    return;

  switch ( c ) {
  case 'h':	/* hardcopy device 2 (laserwriter) */
    (void) strcpy ( tempfile, XPLOT_TEMPLATE );
    if ( ! ( tf = fdopen ( mkstemp ( tempfile ), "w" ) ) ) {
      (void) fprintf ( stderr, "%s: can't open temp file %s.\n",
		       ProgName, tempfile );
      return;
    }
    DumpPlot ( pw, tf, False );
    (void) fclose ( tf );
    switch ( fork () ) {
    case -1:
      fprintf ( stderr, "%s: fork failed\n", ProgName );
      break;
    case 0:
      if ( ( tty = open ( "/dev/tty", O_RDWR ) ) > 0 ) {
	(void) ioctl ( tty, TIOCNOTTY, NULL );
	(void) close ( tty );
      }
#if SETPGRP_VOID      
      setpgrp ();
#else
      /* what are my other choices here? */
      setpgrp ();
#endif
      sprintf ( cmdbuf, "%s < %s | %s %s",
		PLOT2PS_CMD, tempfile, LPR_CMD, Pprinter);
      /*fprintf (stdout, "%s\n", cmdbuf);*/
      (void) system ( cmdbuf );
      (void) unlink ( tempfile );
      exit ( 0 );
    default:
      break;
    }
    break;
  case 'P':
#ifdef BUTTONS
    {
      if ( ! printerpop )
	printerpop = CreatePrinterPopup();
      
      nargs    = 0;
#ifdef OPENLOOK
      XtSetArg ( args [ nargs ], XtNstring, printer ), nargs++;
      XtSetValues ( printertext, args, nargs );
#endif
#ifdef ATHENA
      XtSetArg ( args [ nargs ], XtNvalue, printer ), nargs++;
      XtSetValues ( printdialog, args, nargs );
#endif
#ifdef MOTIF
      XtSetArg ( args [ nargs ], XtNvalue, printer ), nargs++;
      XtSetValues ( printertext, args, nargs );
#endif
      
#ifdef MOTIF
      XtManageChild( printerform );
#endif
#ifdef ATHENA
      XtManageChild (printdialog);
#endif
      XtPopup ( printerpop, XtGrabNone );
    }
#endif
    break;
  case 's':
#ifdef BUTTONS
    {
      if ( ! savepop )
	savepop = CreateSavePopup();
      
#ifdef OPENLOOK
      /* the string returned here is the one stored in the widget,
	 not a newly allocated copy... */
      nargs = 0;
      XtSetArg ( args [ nargs ], XtNstring, &title ), nargs ++;
      XtGetValues ( titleWidget, args, nargs );
#endif
#ifdef ATHENA
      nargs = 0;
      XtSetArg ( args [ nargs ], XtNlabel, &title ), nargs ++;
      XtGetValues ( titleWidget, args, nargs );
#endif
#ifdef MOTIF
      nargs = 0;
      XtSetArg ( args [ nargs ], XmNlabelString, &xmstr ), nargs ++;
      XtGetValues ( titleWidget, args, nargs );
      if(xmstr == NULL) title = NULL;
      else XmStringGetLtoR(xmstr, XmFONTLIST_DEFAULT_TAG, &title);
#endif

      if (!title) title = "unnamed";
      if (( cp = strpbrk ( title, ". \t\n" )) )
	*cp = '\0';
      filename =
	XtMalloc ( (unsigned) strlen ( title ) + sizeof (XPLOT_EXTENSION) );
      sprintf ( filename, "%s%s", title, XPLOT_EXTENSION );

#ifdef OPENLOOK
      nargs    = 0;
      XtSetArg ( args [ nargs ], XtNstring, filename ), nargs++;
      XtSetValues ( savetext, args, nargs );
#endif
#ifdef ATHENA
      /*XtFree(title);*/ /* I'm not sure whether to free this for athena */
      nargs    = 0;
      XtSetArg ( args [ nargs ], XtNvalue, filename ), nargs++;
      XtSetValues ( savedialog, args, nargs );
#endif
#ifdef MOTIF
      XtFree(title);
      XmStringFree(xmstr);
      nargs    = 0;
      XtSetArg ( args [ nargs ], XmNvalue, filename ), nargs++;
      XtSetValues ( savetext, args, nargs );
#endif
      XtFree ( filename );
      
#ifdef MOTIF
      XtManageChild(saveform);
#endif
      XtPopup ( savepop, XtGrabNone );
    }
#endif
    break;
  case 'c':
    ClearPlot ( pw );
#ifdef OPENLOOK
    nargs = 0;
    XtSetArg ( args [ nargs ], XtNstring, "Unnamed Plot" ), nargs ++;
    XtSetValues ( titleWidget, args, nargs );
#endif
#ifdef ATHENA
    nargs = 0;
    XtSetArg ( args [ nargs ], XtNlabel, "Unnamed Plot" ), nargs ++;
    XtSetValues ( titleWidget, args, nargs );
#endif
#ifdef MOTIF
    nargs = 0;
    xmstr = XmStringCreateLocalized("Unnamed Plot");
    XtSetArg ( args [ nargs ], XtNlabel, xmstr ), nargs ++;
    XtSetValues ( titleWidget, args, nargs );
    XmStringFree(xmstr);
#endif
    break;
  case 'n':
    if ( LinkMode || ( ( CurPlotFile + 1 ) == MaxPlotFile ) ) {
      XBell ( XtDisplay ( pw ), 100 );
      break;
    }
    CurPlotFile ++;
    if ( ! OverlayPlotMode ) {
      nargs = 0;
      XtSetArg ( args [ nargs ], XtNfile, PlotFiles[CurPlotFile] ), nargs ++;
      XtSetValues ( pw, args, nargs );
#ifdef OPENLOOK
      nargs = 0;
      XtSetArg ( args [ nargs ], XtNstring, PlotFiles[CurPlotFile] ), nargs ++;
      XtSetValues ( titleWidget, args, nargs );
#endif
#ifdef ATHENA
      nargs = 0;
      XtSetArg ( args [ nargs ], XtNlabel, PlotFiles[CurPlotFile] ), nargs ++;
      XtSetValues ( titleWidget, args, nargs );
#endif
#ifdef MOTIF
      nargs = 0;
      xmstr = XmStringCreateLocalized(PlotFiles[CurPlotFile]);
      XtSetArg ( args [ nargs ], XmNlabelString, xmstr ), nargs++;
      XtSetValues ( titleWidget, args, nargs );
      XmStringFree(xmstr);
#endif
    } else {
      if ( ( fp = fopen ( PlotFiles [ CurPlotFile ], "r+" ) ) == 0 ) {
	(void) fprintf ( stderr, "%s: unable to open plot %s\n",
			ProgName, PlotFiles [ CurPlotFile ] );
	break;
      }
      OverlayPlot ( pw, fp );
      (void) fclose ( fp );
#ifdef OPENLOOK
      nargs = 0;
      XtSetArg ( args [ nargs ], XtNstring, "Overlaid Plot" ), nargs ++;
      XtSetValues ( titleWidget, args, nargs );
#endif
#ifdef ATHENA
      nargs = 0;
      XtSetArg ( args [ nargs ], XtNlabel, "Overlaid Plot" ), nargs ++;
      XtSetValues ( titleWidget, args, nargs );
#endif
#ifdef MOTIF
      nargs = 0;
      xmstr = XmStringCreateLocalized("Overlaid Plot");
      XtSetArg ( args [ nargs ], XmNlabelString, xmstr ), nargs ++;
      XtSetValues ( titleWidget, args, nargs );
      XmStringFree(xmstr);
#endif
    }
    break;
  case 'o':
    OverlayPlotMode = ~ OverlayPlotMode;
    break;
  case 'p':
    if ( LinkMode || ( CurPlotFile == 0 ) ) {
      XBell ( XtDisplay ( pw ), 100 );
      break;
    }
    CurPlotFile --;
    if ( ! OverlayPlotMode ) {
      nargs = 0;
      XtSetArg ( args [ nargs ], XtNfile, PlotFiles[CurPlotFile] ), nargs ++;
      XtSetValues ( pw, args, nargs );
#ifdef OPENLOOK
      nargs = 0;
      XtSetArg ( args [ nargs ], XtNstring, PlotFiles[CurPlotFile] ), nargs ++;
      XtSetValues ( titleWidget, args, nargs );
#endif
#ifdef ATHENA
      nargs = 0;
      XtSetArg ( args [ nargs ], XtNlabel, PlotFiles[CurPlotFile] ), nargs ++;
      XtSetValues ( titleWidget, args, nargs );
#endif
#ifdef MOTIF
      nargs = 0;
      xmstr = XmStringCreateLocalized(PlotFiles[CurPlotFile]);
      XtSetArg ( args [ nargs ], XmNlabelString, xmstr ), nargs ++;
      XtSetValues ( titleWidget, args, nargs );
      XmStringFree(xmstr);
#endif
    } else {
      if ( ( fp = fopen ( PlotFiles [ CurPlotFile ], "r+" ) ) == 0 ) {
	(void) fprintf ( stderr, "%s: unable to open plot %s\n",
			ProgName, PlotFiles [ CurPlotFile ] );
	break;
      }
      OverlayPlot ( pw, fp );
      (void) fclose ( fp );
#ifdef OPENLOOK
      nargs = 0;
      XtSetArg ( args [ nargs ], XtNstring, "Overlaid Plot" ), nargs ++;
      XtSetValues ( titleWidget, args, nargs );
#endif
#ifdef ATHENA
      nargs = 0;
      XtSetArg ( args [ nargs ], XtNlabel, "Overlaid Plot" ), nargs ++;
      XtSetValues ( titleWidget, args, nargs );
#endif
#ifdef MOTIF
      nargs = 0;
      xmstr = XmStringCreateLocalized("Overlaid Plot");
      XtSetArg ( args [ nargs ], XtNlabel, xmstr ), nargs ++;
      XtSetValues ( titleWidget, args, nargs );
      XmStringFree(xmstr);
#endif
    }
    break;
  case '?':
#ifdef BUTTONS
    {
      if ( ! Helppop )
	Helppop = CreateHelpPopup();

#ifdef MOTIF
      XtManageChild( helpform );
#endif
      XtPopup ( Helppop, XtGrabNone );
    }
#endif
    break;
  case 'q':
    if ( TempFileName )
      (void) unlink ( TempFileName );
    ClearPlot(pw);
    exit ( 0 );
    /*NOTREACHED*/
  default:
    break;
  }
}

/*ARGSUSED*/
static void
DoKey ( pw, closure, event )
     Widget pw;
     XtPointer closure;
     XKeyEvent *event;
{
  int cc;
  char buf[128];

  cc = XLookupString ( event, buf, sizeof (buf),
		      (KeySym *) NULL, (XComposeStatus *) NULL );

  if ( cc > 0 )
    Dispatch ( pw, (int) ( buf [ 0 ] ) );

}

/*ARGSUSED*/
static void
DoButton ( pw, closure, event )
     Widget pw;
     XtPointer closure;
     XButtonEvent *event;
{
  PlotCmd plotbuf;
  register PlotCmd *pc = &plotbuf;
  int px, py, cc;

  /* if we were waiting for a second mouse button to be pressed ... */
  if ( LinkWaitTimestamp && LinkMode && ( LinkSocket >= 0 ) ) {
    PlotTranslate ( pw, event->x, event->y, &px, &py );
    pc->command = PLOT_LINK_POINTERWAIT;
    pc->args.pointer.x = px;
    pc->args.pointer.y = py;
    pc->args.pointer.buttons = event->button;
    pc->args.pointer.modifiers = event->state;
    pc->args.pointer.timestamp = LinkWaitTimestamp;
    LinkWaitTimestamp = 0;
    if ( ( cc = write ( LinkSocket, (char *) pc, sizeof (*pc) ) ) < 0 ) {
      perror ( "DoButton: write" );
      return;
    }
    if ( cc != sizeof (*pc) ) {
      fprintf ( stderr, "DoButton: link write out of sync\n" );
      return;
    }
  }
} 

/*ARGSUSED*/
static void
DoEvent ( w, closure, event, cont )
     Widget w;
     XtPointer closure;
     XEvent *event;
     Boolean *cont;
{
  switch ( event->type ) {
  case KeyPress:
  case KeyRelease:
    DoKey ( plotWidget, closure, (XKeyEvent *) event );
    break;
  case ButtonPress:
  case ButtonRelease:
    DoButton ( plotWidget, closure, (XButtonEvent *) event );
    break;
  default:
    break;
  }
  if ( cont )
    *cont = TRUE;
}

#ifdef BUTTONS
/*ARGSUSED*/
static void
DoCallback ( w, client, call )
     Widget w;
     XtPointer client, call;
{
    /* the client data was the letter associated with the command */
  Dispatch ( plotWidget, (int) client );
}

/*ARGSUSED*/
static void
OverlayCallback ( w, client, call )
     Widget w;
     XtPointer client, call;
{
  OverlayPlotMode = client ? True : False;
}
#endif

static char *
NewString ( s )
     char *s;
{
  char *ns;

  if ( ! s )
    return (char *) NULL;
  if ( ! ( ns = XtMalloc ( (unsigned) strlen ( s ) + 1 ) ) )
    return (char *) NULL;
  return strcpy ( ns, s );
}
