/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/***---------------------------------------------------------------------------
* NAME:		dots2mif
* PURPOSE:	translate x y pairs into lots of mif circles 
* USAGE: 
* 	dots2mif < mac_lines.dots > temp.mif 
*
* DESCRIPTION:
*  
* R. Lippmann May 1994
--------------------------------------------------------------------------***/
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include "plot2mif.h"

FILE *fout;
int debug_flag = 0;
int blocks_to_dots_flag=0; 	
float radius=.01;
int first_plot = 1;
float height_fract = -1.0;
float width_fract = 1.0;
float x_fract = 0.0;
float y_fract = 1.0;
int	linmod	= SOLID;
static int color_index = 0;			/* black */
static int last_color_index = 0;	/* black */
int lastx, lasty, mlastx, mlasty;

float xtoinch(/*float x*/), ytoinch(/*float y*/);
void print_color(void);
void fplt(FILE *fin);
void mif_circle(/*float x, float y, float r, int black_fill_flag*/);
/* map.c */
void mapinit(/*float x, float y, float xf, float yf*/);

int
main(argc,argv)  
char **argv; 
	{
	int files_provided=0;
	FILE *fin;

	while(argc-- > 1) 
		{
		/* pick off any flags */
		if(*argv[1] == '-')
			switch(argv[1][1]) 
				{
				case 'b':
					blocks_to_dots_flag = 1;
					break;
				case 'd':
					debug_flag = 1;
					break;
				case 'h':
					height_fract = atof(argv[1]+2);
					break;
				case 'w':
					width_fract = atof(argv[1]+2);
					break;
				case 'r':
					x_fract = atof(argv[1]+2);
					break;
				case 'u':
					y_fract = atof(argv[1]+2);
					break;
				default:
					fprintf(stderr, "ptof: bad flag character\n");
			fprintf(stderr, "Allowable flags are: -b -d -h -w -r -u\n");
			fprintf(stderr, "-b turn small blocks into dots to plot decision regions\n");
			fprintf(stderr, "-d debug\n");
			fprintf(stderr, "-h n  height fraction\n");
			fprintf(stderr, "-w n  width fraction\n");
			fprintf(stderr, "-r n  right fraction\n");
			fprintf(stderr, "-u n  up fraction\n");
					exit(1);
					
				}
		else 
			{
			files_provided++;
			if ((fin = fopen(argv[1], "r")) == NULL) 
				{
				fprintf(stderr, "ptof: can't open %s\n", argv[1]);
				exit(1);
				}
			fplt(fin);
			if ( fclose(fin) == EOF ) 
				{
				fprintf(stderr, "ptof: can't close %s\n", argv[1]);
				exit(1);
				}
			}
		argv++;
		}
	if (files_provided==0)
		fplt( stdin );

	exit(0);
	}

void
fplt(fin)  
FILE *fin; 
	{
	int clast;
	float x,y;

	/* initialize for first file  */
	if (first_plot == 1)
		{
		/* scale so full size is reasonable on paper */
		mapinit(x_fract*.76, y_fract*.76, width_fract*.76, height_fract*.76);
		printf("<MIFFile 3.00> # Generated by plot2mif\n");
		first_plot = 0;
		}
		

	clast=0;
	while(fscanf(fin, "%f %f", &x, &y) == 2)
		{
		mif_circle(xtoinch(x), ytoinch(y), radius, 1);
		}
	}

/* convert x coordinates ranging from 0 to 1023 to frame maker inches
----------------------------------------------------------------------*/
float xtoinch(x)
float x;
    {
    float xinch;

    xinch = (x+2.0)*1.5;
    return(xinch);
    }

/* convert y coordinates ranging from 0 to 780 to frame maker inches
----------------------------------------------------------------------*/
float ytoinch(y)
float y;
    {
    float yinch;

    yinch = (y+2.0)*1.5;
    return(yinch);
    }



/* output circle in mif format
--------------------------------*/
void
mif_circle(x, y, r, black_fill_flag)
float x,y,r;
int black_fill_flag;
	{
	printf(" <Ellipse\n");
		print_color();
		/* solid black */
		if ( black_fill_flag)
			 printf("\t<Fill 0>\n");
		/* transparent */
		else
			 printf("\t<Fill 15>\n");
        printf("  <BRect  %6.3f\" %6.3f\" %6.3f\" %6.3f\">\n", 
	        x-r, y-r, 2.0*r, 2.0*r);
       printf(" > # end of Ellipse\n");
	}
/* output line to add color to line or ellipse
---------------------------------------------*/
void print_color()
	{
	if ( color_index != last_color_index)
		{
		printf(" <Separation %d>\n", color_index);
		last_color_index = color_index;
		}
	}

/* output line to add color to line or ellipse
---------------------------------------------*/
void print_fcolor()
	{
	if ( color_index != last_color_index)
		{
		printf(" <FSeparation %d>\n", color_index);
		last_color_index = color_index;
		}
	}
