/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>

#define X_MAX 1023
#define Y_MAX 780

/* scaling values from flags */
static int flag_xoff = 0;   /* x offset */
static int flag_yoff = 0;   /* y offset */
static float flag_xfract = 1.0;     /* x multiplication factor */
static float flag_yfract = 1.0; /* y multiplication factor */

/* total values to used taking into account both parameter and size values */
static int xoff = 0;    /* x offset */
static int yoff = 0;    /* y offset */
static float xfract = 1.0;  /* x multiplication factor */
static float yfract = 1.0;  /* y multiplication factor */

/* flags when mapping is required */
static int xmapped = 0;
static int ymapped = 0;

void
mapinit(x, y, xf, yf)
float x,y;
float xf, yf;
{
if ( xf == 0 || yf == 0)
    {
    fprintf(stderr, "plot2fig: Zero scaling used for -h,  or -w\n");
    fprintf(stderr, "width fraction = %f, height fraction = %f\n", xf, yf); 
    exit(1);
    }
flag_xoff = x*X_MAX;
flag_yoff = y*Y_MAX;
flag_xfract = xf;
flag_yfract = yf;

/* actual values used */
xoff = flag_xoff;   
yoff = flag_yoff;
xfract = flag_xfract;
yfract = flag_yfract;

/* set mapping flags */
if ( xoff == 0 && xfract == 1.0)
    xmapped = 0;
else
    xmapped = 1;

if ( yoff == 0 && yfract == 1.0)
    ymapped = 0;
else
    ymapped = 1;
                    
}

void
space(x0, y0, x1, y1)
int x0, y0, x1, y1;
{
float space_xoff, space_yoff, space_xfract, space_yfract;
if ( (x0-x1) == 0)
    {
    fprintf(stderr, "plot2fig: x range is zero in Space command \n");
    exit(1);
    }
if ( (y0-y1) == 0)
    {
    fprintf(stderr, "plot2fig: y range is zero in Space command \n");
    exit(1);
    }
/* calculate offsets and multiplicative fractions for this space command */
space_xfract = (X_MAX*1.0)/(x1-x0);
space_yfract = (Y_MAX*1.0)/(y1-y0);
space_xoff = -x0*space_xfract;
space_yoff = -y0*space_yfract;

/* calculate total offsets and multiplicative fractions */
xoff = flag_xoff + flag_xfract*space_xoff;
yoff = flag_yoff + flag_yfract*space_yoff;
xfract = flag_xfract*space_xfract;
yfract = flag_yfract*space_yfract;

/* set mapping flags */
if ( xoff == 0 && xfract == 1.0)
    xmapped = 0;
else
    xmapped = 1;

if ( yoff == 0 && yfract == 1.0)
    ymapped = 0;
else
    ymapped = 1;

}

int
xmap(x)
int x;
{   
    int u;
    if (xmapped == 0)
        u = x;
    else
        u = xoff + x*xfract;
    return(u);
}

int
ymap(y)
int y;
{   
    int v;
    if (ymapped == 0)
        v = y;
    else
        v = yoff + y*yfract;
    return(v);
}
