/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/***---------------------------------------------------------------------------
* NAME:		plot2mif
* PURPOSE:	translate unix plot format to frame maker mif format 
* USAGE: 
* 	plot2mif < temp.plot > temp.mif 
*
* DESCRIPTION:
*   Translates plot(5) commands read in from an optional file or 
*   from the standard input into mif commands for incorporation in frame maker.
*   Plot files generated with lipgraph work fine.
*   Allows the following colors: black, white, red, green blue, 
*   cyan, magenta, yellow
* HACKS:
*	The -b flag turns small blocks that are used to generate decision
*   region boundaries into dots. This creates a prettier frame maker
*   decision region boundary plot without jagged edges. Blocks that are
*   less than 8 (lippic) by 8 are turned into dots. This leaves symbols
*   in scatter plots alone. A decision region plot needs to have a
*   resolution of at least 400x400 to look really smooth.
*  
* R. Lippmann Fri Jan  11 1992
--------------------------------------------------------------------------***/
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <nnlib.h>
#include <plot_colors.h>
#include "plot2mif.h"
#define TEXT 0
#define LINE 1
#define ELLIPSE 2
#define POLYGON 3
static char *color_catalog = " <ColorCatalog \n <Color \n <ColorTag `Black'>\n <ColorCyan  0.0>\n <ColorMagenta  0.0>\n <ColorYellow  0.0>\n <ColorBlack  100.0>\n <ColorAttribute ColorIsBlack >\n <ColorAttribute ColorIsReserved >\n > # end of Color\n <Color \n <ColorTag `Red'>\n <ColorCyan  0.0>\n <ColorMagenta  100.0>\n <ColorYellow  100.0>\n <ColorBlack  0.0>\n <ColorAttribute ColorIsRed >\n <ColorAttribute ColorIsReserved >\n > # end of Color\n <Color \n <ColorTag `Blue'>\n <ColorCyan  100.0>\n <ColorMagenta  100.0>\n <ColorYellow  0.0>\n <ColorBlack  0.0>\n <ColorAttribute ColorIsBlue >\n <ColorAttribute ColorIsReserved >\n > # end of Color\n <Color \n <ColorTag `White'>\n <ColorCyan  0.0>\n <ColorMagenta  0.0>\n <ColorYellow  0.0>\n <ColorBlack  0.0>\n <ColorAttribute ColorIsWhite >\n <ColorAttribute ColorIsReserved >\n > # end of Color\n <Color \n <ColorTag `Red'>\n <ColorCyan  0.0>\n <ColorMagenta  100.0>\n <ColorYellow  100.0>\n <ColorBlack  0.0>\n <ColorAttribute ColorIsRed >\n <ColorAttribute ColorIsReserved >\n > # end of Color\n <Color \n <ColorTag `Green'>\n <ColorCyan  100.0>\n <ColorMagenta  0.0>\n <ColorYellow  100.0>\n <ColorBlack  0.0>\n <ColorAttribute ColorIsGreen >\n <ColorAttribute ColorIsReserved >\n > # end of Color\n <Color \n <ColorTag `Cyan'>\n <ColorCyan  100.0>\n <ColorMagenta  0.0>\n <ColorYellow  0.0>\n <ColorBlack  0.0>\n <ColorAttribute ColorIsCyan >\n <ColorAttribute ColorIsReserved >\n > # end of Color\n <Color \n <ColorTag `Magenta'>\n <ColorCyan  0.0>\n <ColorMagenta  100.0>\n <ColorYellow  0.0>\n <ColorBlack  0.0>\n <ColorAttribute ColorIsMagenta >\n <ColorAttribute ColorIsReserved >\n > # end of Color\n <Color \n <ColorTag `Yellow'>\n <ColorCyan  0.0>\n <ColorMagenta  0.0>\n <ColorYellow  100.0>\n <ColorBlack  0.0>\n <ColorAttribute ColorIsYellow >\n <ColorAttribute ColorIsReserved >\n > # end of Color\n <Color \n <ColorTag `red'>\n <ColorCyan  0.0>\n <ColorMagenta  100.0>\n <ColorYellow  100.0>\n <ColorBlack  0.0>\n > # end of Color\n <Color \n <ColorTag `LightGoldenrod'>\n <ColorCyan  0.0>\n <ColorMagenta  6.599991>\n <ColorYellow  42.319992>\n <ColorBlack  6.700012>\n > # end of Color\n <Color \n <ColorTag `violet'>\n <ColorCyan  0.0>\n <ColorMagenta  42.319992>\n <ColorYellow  0.0>\n <ColorBlack  6.700012>\n > # end of Color\n <Color \n <ColorTag `blue'>\n <ColorCyan  100.0>\n <ColorMagenta  100.0>\n <ColorYellow  0.0>\n <ColorBlack  0.0>\n > # end of Color\n <Color \n <ColorTag `salmon'>\n <ColorCyan  0.0>\n <ColorMagenta  47.850006>\n <ColorYellow  53.329987>\n <ColorBlack  1.960007>\n > # end of Color\n <Color \n <ColorTag `LimeGreen'>\n <ColorCyan  60.800003>\n <ColorMagenta  0.0>\n <ColorYellow  60.800003>\n <ColorBlack  19.600006>\n > # end of Color\n <Color \n <ColorTag `SteelBlue'>\n <ColorCyan  43.179993>\n <ColorMagenta  19.599991>\n <ColorYellow  0.0>\n <ColorBlack  29.419998>\n > # end of Color\n <Color \n <ColorTag `brown'>\n <ColorCyan  0.0>\n <ColorMagenta  48.230011>\n <ColorYellow  48.230011>\n <ColorBlack  35.300003>\n > # end of Color\n <Color \n <ColorTag `PaleVioletRed'>\n <ColorCyan  0.0>\n <ColorMagenta  41.959976>\n <ColorYellow  28.239990>\n <ColorBlack  14.120010>\n > # end of Color\n <Color \n <ColorTag `OliveDrab'>\n <ColorCyan  13.720001>\n <ColorMagenta  0.0>\n <ColorYellow  41.959991>\n <ColorBlack  44.320007>\n > # end of Color\n <Color \n <ColorTag `DeepSkyBlue'>\n <ColorCyan  100.0>\n <ColorMagenta  25.099991>\n <ColorYellow  0.0>\n <ColorBlack  0.0>\n > # end of Color\n <Color \n <ColorTag `HotPink'>\n <ColorCyan  0.0>\n <ColorMagenta  58.829987>\n <ColorYellow  29.419998>\n <ColorBlack  0.0>\n > # end of Color\n <Color \n <ColorTag `turquoise'>\n <ColorCyan  62.699997>\n <ColorMagenta  0.0>\n <ColorYellow  6.199997>\n <ColorBlack  12.200012>\n > # end of Color\n <Color \n <ColorTag `yellow'>\n <ColorCyan  0.0>\n <ColorMagenta  0.0>\n <ColorYellow  100.0>\n <ColorBlack  0.0>\n > # end of Color\n <Color \n <ColorTag `MediumBlue'>\n <ColorCyan  80.389999>\n <ColorMagenta  80.389999>\n <ColorYellow  0.0>\n <ColorBlack  19.610001>\n > # end of Color\n <Color \n <ColorTag `maroon'>\n <ColorCyan  0.0>\n <ColorMagenta  50.199997>\n <ColorYellow  31.369980>\n <ColorBlack  30.980011>\n > # end of Color\n <Color \n <ColorTag `chocholate'>\n <ColorCyan  0.0>\n <ColorMagenta  41.179993>\n <ColorYellow  70.589996>\n <ColorBlack  17.650009>\n > # end of Color\n <Color \n <ColorTag `LightSlateGray'>\n <ColorCyan  13.340012>\n <ColorMagenta  6.669998>\n <ColorYellow  0.0>\n <ColorBlack  40.0>\n > # end of Color\n <Color \n <ColorTag `DarkSeaGreen'>\n <ColorCyan  17.639999>\n <ColorMagenta  0.0>\n <ColorYellow  17.649994>\n <ColorBlack  26.279999>\n > # end of Color\n <Color \n <ColorTag `orange'>\n <ColorCyan  0.0>\n <ColorMagenta  35.300003>\n <ColorYellow  100.0>\n <ColorBlack  0.0>\n > # end of Color\n <Color \n <ColorTag `LightBlue'>\n <ColorCyan  22.389999>\n <ColorMagenta  5.489990>\n <ColorYellow  0.0>\n <ColorBlack  9.810013>\n > # end of Color\n <Color \n <ColorTag `purple'>\n <ColorCyan  31.369995>\n <ColorMagenta  81.569992>\n <ColorYellow  0.0>\n <ColorBlack  5.890015>\n > # end of Color\n <Color \n <ColorTag `gray'>\n <ColorCyan  0.0>\n <ColorMagenta  0.0>\n <ColorYellow  0.0>\n <ColorBlack  25.0>\n > # end of Color\n <Color \n <ColorTag `MediumSeaGreen'>\n <ColorCyan  46.669998>\n <ColorMagenta  0.0>\n <ColorYellow  25.879990>\n <ColorBlack  29.809998>\n > # end of Color\n <Color \n <ColorTag `cyan'>\n <ColorCyan  100.0>\n <ColorMagenta  0.0>\n <ColorYellow  0.0>\n <ColorBlack  0.0>\n > # end of Color\n <Color \n <ColorTag `DeepPink'>\n <ColorCyan  0.0>\n <ColorMagenta  92.160004>\n <ColorYellow  42.360001>\n <ColorBlack  0.0>\n > # end of Color\n <Color \n <ColorTag `tan'>\n <ColorCyan  0.0>\n <ColorMagenta  11.769989>\n <ColorYellow  27.449982>\n <ColorBlack  17.650009>\n > # end of Color\n <Color \n <ColorTag `pink'>\n <ColorCyan  0.0>\n <ColorMagenta  24.710007>\n <ColorYellow  20.400009>\n <ColorBlack  0.0>\n > # end of Color\n <Color \n <ColorTag `SkyBlue'>\n <ColorCyan  39.210007>\n <ColorMagenta  11.369980>\n <ColorYellow  0.0>\n <ColorBlack  7.850006>\n > # end of Color\n <Color \n <ColorTag `VioletRed'>\n <ColorCyan  0.0>\n <ColorMagenta  69.020004>\n <ColorYellow  25.090012>\n <ColorBlack  18.440002>\n > # end of Color\n <Color \n <ColorTag `DarkOrange'>\n <ColorCyan  0.0>\n <ColorMagenta  45.100006>\n <ColorYellow  100.0>\n <ColorBlack  0.0>\n > # end of Color\n <Color \n <ColorTag `LightSlateBlue'>\n <ColorCyan  48.240005>\n <ColorMagenta  56.080002>\n <ColorYellow  0.0>\n <ColorBlack  0.0>\n > # end of Color\n <Color \n <ColorTag `goldenrod'>\n <ColorCyan  0.0>\n <ColorMagenta  20.789993>\n <ColorYellow  72.949997>\n <ColorBlack  14.510010>\n > # end of Color\n <Color \n <ColorTag `SpringGreen'>\n <ColorCyan  100.0>\n <ColorMagenta  0.0>\n <ColorYellow  50.199997>\n <ColorBlack  0.0>\n > # end of Color\n <Color \n <ColorTag `LightGrey'>\n <ColorCyan  0.0>\n <ColorMagenta  0.0>\n <ColorYellow  0.0>\n <ColorBlack  17.0>\n > # end of Color\n <Color \n <ColorTag `white'>\n <ColorCyan  0.0>\n <ColorMagenta  0.0>\n <ColorYellow  0.0>\n <ColorBlack  0.0>\n > # end of Color\n <Color \n <ColorTag `black'>\n <ColorCyan  0.0>\n <ColorMagenta  0.0>\n <ColorYellow  0.0>\n <ColorBlack  100.0>\n > # end of Color\n > # end of ColorCatalog\n ";

static char *standard_colors[8] = {"black", "white", "red", "green",
				   "blue", "cyan", "magenta", "yellow"};

FILE *fout;
int debug_flag = 0;
int blocks_to_dots_flag=0; 	
int blocks_to_dots_radius=1;
int first_plot = 1;
float height_fract = -1.0;
float width_fract = 1.0;
float x_fract = 0.0;
float y_fract = 1.0;
int	linmod	= SOLID;
static int color_index = 0;			/* 0 to 7 are standard, above are lnknet */
static int last_color_index = -1;	
static int nlnknet_colors = N_PLOT_COLORS;
int lastx, lasty, mlastx, mlasty;

float line_seg(int i);
float xtoinch(int i), ytoinch(int i);
void mif_line(int x1, int y1, int x2, int y2);
void print_color(int type);
void fplt(FILE *fin);
void lip_gets(char *s, FILE *fin);
int getsi(FILE *fin);
void mif_polygon(int npoints, int *xarray, int *yarray);
void mif_circle(int x, int y, int r, int black_fill_flag);
/* map.c */
void mapinit(/*float x, float y, float xf, float yf*/);
void space (int x0, int y0, int x1, int y1);
int xmap(int x);
int ymap(int y);

int
main(argc,argv)  
char **argv; 
	{
	int files_provided=0;
	FILE *fin;

	while(argc-- > 1) 
		{
		/* pick off any flags */
		if(*argv[1] == '-')
			switch(argv[1][1]) 
				{
				case 'b':
					blocks_to_dots_flag = 1;
					break;
				case 'd':
					debug_flag = 1;
					break;
				case 'h':
					height_fract = atof(argv[1]+2);
					break;
				case 'w':
					width_fract = atof(argv[1]+2);
					break;
				case 'r':
					x_fract = atof(argv[1]+2);
					break;
				case 'u':
					y_fract = atof(argv[1]+2);
					break;
				default:
					fprintf(stderr, "ptof: bad flag character\n");
			fprintf(stderr, "Allowable flags are: -b -d -h -w -r -u\n");
			fprintf(stderr, "-b turn small blocks into dots to plot decision regions\n");
			fprintf(stderr, "-d debug\n");
			fprintf(stderr, "-h n  height fraction\n");
			fprintf(stderr, "-w n  width fraction\n");
			fprintf(stderr, "-r n  right fraction\n");
			fprintf(stderr, "-u n  up fraction\n");
					exit(1);
					
				}
		else 
			{
			files_provided++;
			if ((fin = fopen(argv[1], "r")) == NULL) 
				{
				fprintf(stderr, "ptof: can't open %s\n", argv[1]);
				exit(1);
				}
			fplt(fin);
			if ( fclose(fin) == EOF ) 
				{
				fprintf(stderr, "ptof: can't close %s\n", argv[1]);
				exit(1);
				}
			}
		argv++;
		}
	if (files_provided==0)
		fplt( stdin );

	exit(0);
	}


void
fplt(fin)  
FILE *fin; 
	{
	int i, found_color_flag, length;
	int c,clast;
	int first_text=0;
	int ci;
	char s[256];
	char *pstring;
	int xi,yi,x0,y0,x1,y1,r,dx,n;
	int poly_flag=0;
	int xpoly[MAX_POLY],ypoly[MAX_POLY];

	/* initialize for first file  */
	if (first_plot == 1)
		{
		/* scale so full size is reasonable on paper */
		mapinit(x_fract*.76, y_fract*.76, width_fract*.76, height_fract*.76);
		printf("<MIFFile 3.00> # Generated by plot2mif\n");
		/* write out color catalog */
                length = strlen(color_catalog);
                clast = 'a';
                for(i=0; i<length; i++)
			{
                        c = color_catalog[i]; 
                        if ( c == '\\')
				{
				printf("\n");
				}
			else if ( clast == '\\')
				{
				}
			else
				{
				putchar(c);
				}
			clast = c;
			}
                           
		first_plot = 0;
		}
		

	clast=0;
	while((c=getc(fin)) != EOF){
		switch(c){

		/* colors */
		case 'C':
		    lip_gets(s,fin);
			found_color_flag=0;
			/* foreground and background */
			if ( strcasecmp(s, "foreground") == 0)
				{
				found_color_flag=1;
				color_index = 0;
				}
			if ( strcasecmp(s, "background") == 0)
				{
				found_color_flag=1;
				color_index = 1;
				}
			/* find eight framemaker standard colors */
			for(i=0; i<8 && found_color_flag==0; i++)
				{
				if ( strcasecmp(s, standard_colors[i]) == 0)
					{
					found_color_flag =1;
					color_index = i;
					}
				}
			/* find lnknet colors */
			for(i=0; i<nlnknet_colors && found_color_flag==0; i++)
				{
				if ( strcasecmp(s, color_table[i]) == 0)
					{
					found_color_flag =1;
					color_index = i+8;
					}
				}
			if ( found_color_flag == 0)
				{
				fprintf(stderr, 
				"WARNING plot2mif - Ignored invalid color: %s\n",s);
				}
			else
				{
				if ( debug_flag)
					printf("color %s\n",s);
				}
			break;

		/* ignore extra plot commands that work under x windows */
		/* ignore font command */
		case 'F':
		    lip_gets(s,fin);
		    break;

		/* ignore fill mode command */
		case 'G':
		    lip_gets(s,fin);
		    break;

		/* poly start command */
		case 'P':
			x1 = xmap(getsi(fin));
			poly_flag=1;
			if ( debug_flag)
				printf("Poly Start\n");
			break;
		/* poly draw command */
		case 'E':
			if ( debug_flag)
				printf("Poly Draw\n");
			else
				{
				mif_polygon( poly_flag -1, xpoly, ypoly);
				/*
				lastx = (xpoly[0] +xpoly[1])/2;
				lasty = (ypoly[0]+ypoly[1])/2;
				mif_line(lastx, lasty, lastx+1, lasty+1);
				mif_line(lastx, lasty+1, lastx+1, lasty);
				mif_line(lastx, lasty, lastx, lasty);
				*/
				}
			poly_flag=0;
			break;
		case 'm':
			lastx = xmap(getsi(fin));
			lasty = ymap(getsi(fin));
			mlastx = lastx;
			mlasty = lasty;
			if ( debug_flag)
				printf("Move: %d, %d\n", lastx, lasty);
			break;
		case 'l':
			x1 = xmap(getsi(fin));
			y1 = ymap(getsi(fin));
			lastx = xmap(getsi(fin));
			lasty = ymap(getsi(fin));
			if ( debug_flag)
				printf("Line: %d, %d, %d, %d\n", x1, y1, lastx, lasty);
			else
				mif_line(lastx, lasty, x1, y1);
			break;
		case 't':
			lip_gets(s,fin);
			if (debug_flag)
				{
				printf("String: %s\n", s);
				i=0;
				while((ci=s[i++]) != '\0')
					if (ci=='\b')
						printf("\\b");
					else
						putchar(ci);
				printf("\n");
				}
			else
				{
				/* count and strip off leading backspaces */
				pstring=s;
				i=0;
				while( *pstring == '\b')
					{
					pstring++;
					i++;
					}
				printf("<TextLine\n"); 
				printf(" <TLAlignment Left >\n");
				printf(" <Angle 0>\n");
				printf(" <TLOrigin  %6.3f\" %6.3f\">\n",
					  xtoinch(lastx-4-8*i), 
					  ytoinch(lasty));
				print_color(TEXT);
				if ( first_text == 0)
					{
					first_text++;
					printf(" <Font \n");
					printf("  <FFamily `Courier\'>\n");
					printf("  <FVar `Regular\'>\n");
					printf("  <FWeight `Regular\'>\n");
					printf("  <FAngle `Regular\'>\n");
					printf("  <FSize  10.0 pt>\n");
					printf("  <FUnderline No >\n");
					printf("  <FOverline No >\n");
					printf("  <FStrike No >\n");
					printf("  <FSupScript No >\n");
					printf("  <FSubScript No >\n");
					printf("  <FChangeBar No >\n");
					printf("  <FOutline No >\n");
					printf("  <FShadow No >\n");
					printf("  <FPairKern Yes >\n");
					printf("  <FDoubleUnderline No >\n");
					printf("  <FNumericUnderline No >\n");
					printf("  <FDX  0.0 pt>\n");
					printf("  <FDY  0.0 pt>\n");
					printf("  <FDW  0.0 pt>\n");
					printf("  <FSeparation 0>\n");
				        printf(" > # end of Font\n");
					}

			       printf(" <String `%s\'>\n",pstring);
			       printf("> # end of TextLine\n");
				}
			break;

		case 'e':
			if ( debug_flag)
				printf("Erase screen\n");
			break;
		case 'p':
			lastx = xmap(getsi(fin));
			lasty = ymap(getsi(fin));
			if ( debug_flag)
				printf("Point: %d, %d\n", lastx, lasty);
			else
				{
				/* store the first and 3rd points of a blob
				   and only plot the center point of a box */
				if ( poly_flag >= 1)
					{
					xpoly[poly_flag-1]=lastx;
					ypoly[poly_flag-1]=lasty;
					poly_flag++;
					if ( poly_flag >= MAX_POLY)
						{
						warning("Too many points in a poly object %d", 
								poly_flag);
						poly_flag = MAX_POLY -1;
						}
					}
				else
					mif_line(lastx, lasty, lastx, lasty);
				}
			break;
		case 'n':
			x1 = xmap(getsi(fin));
			y1 = ymap(getsi(fin));
			if ( debug_flag)
				printf("Next: %d, %d\n", x1, y1);
			else
				{
				mif_line(lastx, lasty, x1, y1);
				}
			lastx = x1;
			lasty = y1;
			break;
		case 's':
			x0 = getsi(fin);
			y0 = getsi(fin);
			x1 = getsi(fin);
			y1 = getsi(fin);
			if ( debug_flag)
				printf("Space: %d, %d, %d, %d\n", x0, y0, x1, y1);
			space(x0, y0, x1, y1);
			break;
		case 'a':
			xi = xmap(getsi(fin));
			yi = ymap(getsi(fin));
			x0 = xmap(getsi(fin));
			y0 = ymap(getsi(fin));
			x1 = xmap(getsi(fin));
			y1 = ymap(getsi(fin));
			if ( debug_flag)
				printf("Arc: center=%d, %d   ends=%d, %d, %d, %d\n", 
						  xi, yi, x0, y0, x1, y1);
			/* arcs are not supported */
			break;
		case 'c':
			xi = xmap(getsi(fin));
			yi = ymap(getsi(fin));
			r = xmap(getsi(fin));
			if ( debug_flag)
				printf("Circle: center=%d, %d   radius=%d\n", xi, yi, r);
			else
				{
				mif_circle(xi, yi, r, 0);
				lastx = xi + r;
				lasty = xi;
				}
			break;
		case 'f':
			lip_gets(s,fin);
			switch (s[3])
				{
				case 'i':
					if ( debug_flag)
						printf("linetype solid\n");
					linmod = SOLID;
					break;
				case 't':
					if ( debug_flag)
						printf("linetype dotted\n");
					linmod = DOTTED;
					break;
				case 'd':
					if ( debug_flag)
						printf("linetype dashed\n");
					linmod = DASHED;
					break;
				case 'r':
					if ( debug_flag)
						printf("linetype solid\n");
					linmod = DOTTED;
					break;
				case 'g':
					if ( debug_flag)
						printf("linetype dotted\n");
					linmod = DASHED;
					break;
				default:
					if ( debug_flag)
						printf("linetype default=solid\n");
					linmod = SOLID;
				}
			break;
		case 'd':
			xi = getsi(fin);
			yi = getsi(fin);
			dx = getsi(fin);
			n = getsi(fin);
			/* dummied out
			for(i=0; i<n; i++)pat[i] = getsi(fin);
			dot(xi,yi,dx,n,pat);
			*/
			break;
		default:
			printf("plot2mif: bad input char = %c\n",c);
			}
		clast = c;
		}
	}

int
getsi(fin)  FILE *fin; {	/* get an integer stored in 2 ascii bytes. */
	short a, b;
	if((b = getc(fin)) == EOF)
		return(EOF);
	if((a = getc(fin)) == EOF)
		return(EOF);
	b = b;
	a = a;
	a = a<<8;
	return(a|b);
}
void
lip_gets(s,fin)  char *s;  FILE *fin; {
	for( ; (*s = getc(fin)) != '\0'; s++)
		if(*s == '\n')
			break;
	*s = '\0';
	return;
}
/* return distance for line segment */
float line_seg(i)
int	i;			/* linmod */
	{
	float dist;		/* length of line segment */
	if ( i == SOLID )
			dist = 0.0;
	else if ( i == DASHED )
			dist = 4.0;
	else if ( i == DOTTED )
			dist = 3.2;
	else
			dist = 0.0;
	return(dist);
	}

/* convert x coordinates ranging from 0 to 1023 to frame maker inches
----------------------------------------------------------------------*/
float xtoinch(x)
int x;
    {
    float xinch;

    xinch = (8.5*x)/1023.0;
    return(xinch);
    }

/* convert y coordinates ranging from 0 to 780 to frame maker inches
----------------------------------------------------------------------*/
float ytoinch(y)
int y;
    {
    float yinch;

    yinch = 0.6*(y*11.0/780.0);
    return(yinch);
    }



/* plot a line
----------------*/
static int first_line=0;
void mif_line( x1, y1, x2, y2)
int x1, y1, x2, y2;
	{
	printf("<PolyLine\n");
	print_color(LINE);
	if ( first_line == 0)
	    {
	    first_line++;
	    printf(" <Pen 0>\n");
	    printf(" <PenWidth  0.5 pt>\n");
	    printf(" <Separation 0>\n");
	    printf(" <HeadCap Square >\n");
	    printf(" <TailCap Square >\n"); 
	    }
		
	printf(" <NumPoints 2>\n");
	printf(" <Point  %6.3f\" %6.3f\">\n", 
		 xtoinch(x1), ytoinch(y1));
	printf(" <Point  %6.3f\" %6.3f\">\n", 
		 xtoinch(x2), ytoinch(y2));
	printf("> # end of PolyLine\n");

	}

/* plot a polygon
------------------*/
void
mif_polygon( npoints, xarray, yarray)
int npoints;
int *xarray, *yarray;
	{
	int i;
	int radius, x, y;

	/* turn solid blocks into dots for decision region plot 
		when the blocks are small */
	if ( npoints == 5 && blocks_to_dots_flag &&
				xarray[1] - xarray[0] < 8)
		{
		x = (xarray[0] + xarray[1])/2;
		y = (yarray[1] + yarray[2])/2;
		radius = blocks_to_dots_radius;
		mif_circle(x, y, radius, 1);
		}
	/* put out polygon */
	else
		{
		printf("<Polygon\n"); 
		print_color(POLYGON);
		if ( first_line == 0)
			{
			first_line++;
			printf(" <Pen 0>\n");
			printf(" <PenWidth  0.5 pt>\n");
		/*	printf(" <Separation 0>\n"); */
			printf(" <HeadCap Square >\n");
			printf(" <TailCap Square >\n"); 
			}
		 /* solid fill */
		 printf("\t<Fill 0>\n");
		 printf("\t<NumPoints %d>\n", npoints);
		 for(i=0; i<npoints; i++)
				printf(" <Point  %6.3f\" %6.3f\">\n", 
						 xtoinch(xarray[i]), ytoinch(yarray[i]));
		printf("> # end of Polygon\n");
		}
	}

/* output circle in mif format
--------------------------------*/
void
mif_circle(x, y, r, black_fill_flag)
int x, y, r; 
int black_fill_flag;
	{
	printf(" <Ellipse\n");
		print_color(ELLIPSE);
		/* solid black */
		if ( black_fill_flag)
			 printf("\t<Fill 0>\n");
		/* transparent */
		else
			 printf("\t<Fill 15>\n");
        printf("  <BRect  %6.3f\" %6.3f\" %6.3f\" %6.3f\">\n", 
	        xtoinch(x-r),
		ytoinch(y-r),
		ytoinch(2*r),
		ytoinch(2*r));
       printf(" > # end of Ellipse\n");
       }


/* output line to add color to line or ellipse
---------------------------------------------*/
void print_color(type)
int type;
	{
	static int last_type = -1;

	if ( color_index != last_color_index || type != last_type )
		{
		/* eight supported colors */
		if ( color_index < 8 )
		  {
		  if ( type == TEXT )
			printf(" <FSeparation %d>\n", color_index);
		  else
			  printf(" <Separation %d>\n", color_index);
		  printf(" <ObColor `%s'>\n", standard_colors[color_index]);
		  }
		/* lnknet colors */
		else
		  {
		  printf(" <ObColor `%s'>\n", color_table[color_index-8]);
		  }
		last_color_index = color_index;
		last_type = type;
		}
	}
