/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */


/* The following program may be used, copied, and modified by anyone at will,
 * so long as this message is not removed or altered.
 * 'tlw' was written by Steve Rapaport at grouptheory systems inc., October 85.
 * {decvax, etc...}!utzoo!dciem!nrcaer!grpthry!rap
 * Copyright 1985 grouptheory systems inc.  Permission to reproduce & update
 * granted.
 */

/*
 *
 * plot2ps -- a filter to turn output from plot(3) library routines into postscript
 *        instructions for the Apple LaserWriter.   (or any PostScript device).
 *        Compatible with all Berkeley 4.2 PLOT(5) files.  (I think!)
 *        To install, just compile it with the math library (-lm), and
 *        put it somewhere that everyone can get at.  (e.g. /usr/bin/plot2ps).
 *
 *        Bugs: until end of Dec '85, mail me at utzoo!dciem!nrcaer!grpthry!rap
 *        After that, you're on your own....
 *
 *        Example of use:  Let 'plotter' be a C program containing 
 *        calls to plot(3) routines.  The following command will
 *        put the output of 'plotter' on the laserwriter:
 *
 *        plotter | plot2ps | <whatever you use to send PostScript to the lw>
 *
 *        In addition, plot2ps may be added as a 'terminal' filter to the 
 *        plot(1G) shell script (tplot(1G) in system 3) 
 *        by adding the following line to plot (or tplot):
 *
 *           -Tlw)  exec /usr/bin/plot2ps $* ;;
 *
 *        (or wherever you put this program)  and using -Tlw as plot option.
 *
 *        Portability:  This program has only been tested on a Plexus P/60
 *        running System 3.  It uses no fancies, though, and outputs plain
 *        ascii.  The only problem I can foresee involves the input of
 *        plot(5) files.  
 *
 *        I am using (int) variables to hold the
 *        integer parameters; if your C thinks (int) is less than 16 bits,
 *        change that declaration to (long) in main() and readswpd().
 *
 *        Customization:  The program takes no options, but certain macros 
 *        may be changed in the source to customize for your installation.
 *
 *            DEFXSCALE, DEFYSCALE:  These are the size of the laserwriter
 *            raster area, in points.  Legal size paper will require
 *            DEFYSCALE to change to 14.0 * 72.0.  
 *            to use another PostScript device (e.g., some Linotron typesetters)
 *            change these to match the raster area for your machine.  
 *            *Note* that they are (float). (If these constants are defined 
 *            without decimals, the program will behave *strangely*)
 *
 *            MARGX, MARGY:  compensate for the clipping boundaries of
 *            the laserwriter.  They are measured in points, and their
 *            effect is to reduce the page size to fit within the lw's
 *            raster area.  The numbers given here work okay on my machine,
 *            you may wish to change them to fit yours.
 *            MARGX refers to the amount that the short dimension differs
 *            from 8.5 inches (612 points), and MARGY refers to the 
 *            analagous difference from 11 inches (792 points).
 *            Note that these constants are (float).
 *
 *            FLIP:  For the sake of most graphs, defining FLIP 
 *            flips the x and y axes to make the page wider than it is tall.
 *            To reverse this effect, undefine FLIP (comment out the #define).
 *            You may want to keep two versions of the filter in your library,
 *            named, say, plot2psh and plot2psv, one with FLIP defined, one not.
 *      (not any more - MAZ)
 *
 *            FONT, PITCH:  These are the parameters that postscript takes.
 *            10 point letters seem to fit well with what the standard
 *            programs expect.  You may wish to change them, though.  If
 *            you prefer, say, Helvetica to Times Roman, you might change FONT
 *            to "/Helvetica".  PITCH is assumed integer.
 *
 *            LINEWIDTH :  This linewidth (.2 of a point) suits my taste.
 *            You may have your own opinions.  Note that the width is
 *            independent of the scale of the space.  Assumed (float).
 *
 */
/*
MODIFICATION HISTORY

Greg Duckworth  Jun 24, 1986
    added the following commands to be consistent with driver.c
    modified for Musicus' updated graph and plot(5) routines ....
    T  long label terminated by NULL
    F  plmode  (probably doesn't do what you want it to)
    U  flush

gld and gildea 25 Jun 86
    changed int's to short ints where 16-bits was important.

    swapped order of "translate" and "scale" commands in space()
        block of switch so they work correctly together.

gld 25 June 86
    if label begins with string of 0x08 (ctl-H or \b) this is stripped
        off, and the label is moved to the left by a "rmoveto" command by
    the appropriate amount. (number of 0x08's times newpitch)
    The movement is undone at the end of the labeling so that lines
    can continue normally

    added -line and -pitch options to change line thickness and label
    pitch.

Greg Duckworth Jun 29, 1986
    to make transfers to the laserwriter more efficient,
    some of the commonly used commands have been given 
    shorter names:
    old ps command       new ps command
    moveto          m
    lineto          l
    rmoveto         r
    show            s
    currentpoint stroke moveto  csm

Greg Duckworth July 9, 1986
    added -n option for number of copies desired (limited to 10)

Marc Zissman 31 October 1989
    changed name from tlw to plot2ps
    added "-flip" and "-font" to command line parameters

Rich Lippmann 4 Mar  1991
    Added code to plot filled polygons as in olxplot.
    Added code to ignore other extra plot(3) commands used 
    by olxplot.
    Increased PITCH to 15, changed divisor to determine backspace
    amount for text from 2 to 2.6, changed relative y offset divisor
    for text from 2 to 8. This should probably be figured out correctly
    for any PITCH.

*/
/* 
 installation-dependent constants: 
 (change these at will)
 */


#define DEFXSCALE   612.0
#define DEFYSCALE   792.0
#define MARGX       39.0
#define MARGY       20.0
#define FONT        "/Times-Roman"
#define PITCH       15
#define LINEWIDTH   .2

/* 
 * change the following at your own peril: 
 */

/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <math.h>

#define FOREVER     1
#define DEGREES     * 180.0 / 3.141592654
#define DOTS        / scale
#define min(a,b)   ( (a < b) ? a : b )
#define MARGSCALE   min(( DEFXSCALE - MARGX )/DEFXSCALE,  \
                        ( DEFYSCALE - MARGY )/DEFYSCALE)
#define MAXNCOPIES 10
int ncopies;

int argck(int argc, char **argv, char *arg);
int set_color(char *string, float *red, float *green, float *blue);
void initcoords(int flip);
void readswpd(unsigned short int *location, FILE *stream);
void fin(int error);

int
main(argc,argv)  
     int argc;
     char **argv;

/*  hacker's notes:  (if you're reading this far, you are probably one such.)
 *
 *  Program assumes you may use the 'space' scaling function once, many times,
 *  or not at all.  This creates some extra (dumb-looking) PostScript output.
 *  If you try to optimize, keep this assumption in mind.
 *
 *  Upon finding errors in the input, my version skips up to the next newline
 *  and continues reading.  This is good if you want to put comments into your
 *  plot (5) files.  If not, just making the default case in the switch a
 *  null switch (comment out the one executable statement in it) will
 *  ignore each invalid character.
 *  
 *  You may notice a strange moveto in the label section.  This is my 
 *  (trial-and-error determined) kludge to make the first letter of the
 *  label come out CENTERED on the current point.  Plot seems to expect that.
 *
 *  Erase just erases the current page without printing.  I think that's how
 *  it's supposed to work.  If you want it to flush instead, change the 
 *  printf("erasepage\n") to printf("showpage\n");
 *
 */
{
    /* rpl mod for filled blobs */
    int poly_start_flag=0;
    /* rpl mod for changing color */
    float xred, xgreen, xblue;
    int i,j;
    short int x, y, x2, y2, xc, yc;  /* or whatever 16 bits is in your C */
                 /* if a short int is less than 16 bits */
                 /* this program won't work unless you  */
                 /* change this declaration.            */
                 /* see also the change in readswpd     */
    float r, ang1, ang2;
    float xscale, yscale;
    float scale = 1.0;
    float newpitch = (float) PITCH;
    float newlinew = LINEWIDTH;
    char command;
    char *dashstyle;
    char string [257];
    char font [257];
    float pitch = (float) PITCH;
    float linewidth = (float) LINEWIDTH;
    int flip=0; /* no flip by default */

    if((i=argck(argc,argv,"-pitch"))!=0)   /* set pitch size */
      {
      sscanf(argv[i+1],"%f",&pitch);
      newpitch = pitch;
    }

    if((i=argck(argc,argv,"-line"))!=0)   /* set line width */
      {
      sscanf(argv[i+1],"%f",&linewidth);
      newlinew = linewidth;
    }

    if((i=argck(argc,argv,"-font"))!=0)   /* set font */
      {
      sscanf(argv[i+1],"%s",font);
      }
    else {
      strcpy(font, FONT);
    }

    if((i=argck(argc,argv,"-flip"))!=0)   /* flip axes */
      {
      flip = 1;
    }

    ncopies = 1;
    if((i=argck(argc,argv,"-ncopies"))!=0)   /* number of copies */
      {
      sscanf(argv[i+1],"%d",&ncopies);
    }
    if(ncopies > MAXNCOPIES) ncopies = MAXNCOPIES;

    printf("%%!PS-Adobe-1.0\n");
    printf ("%% -------------------------------------------------- %%\n");
    printf ("%% This is a PostScript (TM) program produced by plot2ps,\n");
    printf ("%% from standard plot(5) output. \n");
    printf ("%% (S.J. Rapaport, grouptheory systems, 1985)\n");
    printf ("%% modified by Greg Duckworth, MIT, June 1986\n");
    printf ("%% rev 2   June 29, 1986\n");

    printf ("\n\n");
    printf ("/plot2pscontext save def\n");
    printf ("%f setlinewidth\n", linewidth);
    printf ("%s findfont %.0f scalefont setfont\n",font, pitch);

        /* new stuff by gld for economy of transfer */
    printf ("/m {moveto} def\n");
    printf ("/l {lineto} def\n");
    printf ("/r {rmoveto} def\n");
    printf ("/s {show} def\n");
    printf ("/csm {currentpoint stroke moveto} def\n");
    /* end new stuff */


    initcoords (flip);

    do {
        command = getchar();
        switch (command)
        {
            default:   
               /* if invalid command char, skip rest of line.
               This is good if you want to put comments into your
               plot (5) files.  If not, just commenting out the 
               next line will ignore each invalid character.
                     */

                if(command != EOF)
                  fprintf(stderr,
                      "unrecognized command %c  %x\n",
                      command,(int)command);
                if (! gets(string)) fin (0);
            break;

            case 'm':   /*   move (x, y)   */
                readswpd (&x, stdin);
                readswpd (&y, stdin);
                printf ("%d %d m\n",x, y); 
            break;

            case 'n':   /*   cont (x, y)   */
                readswpd (&x, stdin);
                readswpd (&y, stdin);
                printf ("%d %d l\n", x, y);
                printf ("csm\n");
            break;

            case 'p':   /*   point (x, y)  */
                readswpd (&x, stdin);
                readswpd (&y, stdin);
                if (poly_start_flag)
                    {
                    /* first point in poly fill */
                    if ( poly_start_flag <= 1)
                        {
                        /*
                        printf("%d %d moveto\n", x, y);
                        */
                        printf("%d %d m\n", x, y);
                        poly_start_flag++;
                        }
                    /* another point in poly fill */
                    else    
                        printf("%d %d l\n", x, y);
                        /*
                        printf("%d %d lineto\n", x, y);
                        */
                    }
                    else
                    {
                    printf ("gsave \n");
                    printf ("1 setlinecap\n");
                printf ("currentlinewidth 2 mul setlinewidth\n");
                    printf ("%d %d m\n", x, y);
                    printf ("0 0  rlineto\n");
                    printf ("stroke\n");
                    printf ("grestore\n");
                    }
                break;

            case 'a':   /*   arc (xc, yc, x, y, x2, y2)   */
                readswpd (&xc, stdin);
                readswpd (&yc, stdin);
                readswpd (&x, stdin);
                readswpd (&y, stdin);
                readswpd (&x2, stdin);
                readswpd (&y2, stdin);
                r = sqrt ((float) ((x-xc) * (x-xc) + (y-yc) * (y-yc)));
                ang1 = atan ( (float) (y-yc) / (float) (x-xc) ) DEGREES;
                ang2 = atan ( (float) (y2-yc) / (float) (x2-xc) ) DEGREES;
                if ((x - xc) < 0) ang1 = 180.0 + ang1;
                if ((x2 - xc) < 0) ang2 = 180.0 + ang2;
                printf ("newpath \n");
                printf ("%d %d %6.3f %6.3f %6.3f arc\n", xc, yc, r, ang1, ang2);
                printf ("csm\n");
            break;

            case 'c':   /*   circle (xc, yc, r)   */
                readswpd (&xc, stdin);
                readswpd (&yc, stdin);
                readswpd (&x, stdin);
                printf ("newpath \n");
                printf ("%d %d %d 0 360 arc\n", xc, yc, x);
                printf ("csm\n");
            break;

            case 'l':   /*   line (x, y, x2, y2)   */
                readswpd (&x, stdin);
                readswpd (&y, stdin);
                readswpd (&x2, stdin);
                readswpd (&y2, stdin);
                printf ("%d %d m\n", x, y); 
                printf ("%d %d l\n", x2, y2);
                printf ("csm\n");
            break;

            case 't':   /*   label ( string )   */
                if (! gets(string) ) fin (0);
                printf ("%6.3f %6.3f r\n", -newpitch/4.0, -newpitch/8.0);
                /* backspacing stuff added by gld */
                j = 0;
                while(string[j] == (char)0x08)  j++;
                
                if(j != 0)
                  printf("%6.3f %6.3f r\n", -newpitch*j/2.6,0.);
                /* gld added +j in next line */
                printf ("\t(%s) s\n", string+j);
                /* gld added to restore loc */
                if(j != 0)
                  printf("%6.3f %6.3f r\n",
                     newpitch*j/2.,0.);

                printf ("%6.3f %6.3f r\n", newpitch/4.0, newpitch/8.0);
            break;

                /* ignore extra plot commands that work under x windows */
                /* set color */
                case 'C':
                    if (! gets(string) ) fin (0);
                    x = set_color(string, 
                            &xred, &xgreen, &xblue );
                    if ( x > 0)
                        printf("%f %f %f  setrgbcolor\n", 
                           xred, xgreen, xblue);
                    break;

                /* ignore font command */
                case 'F':
                    if (! gets(string) ) fin (0);
                    break;
                /* ignore fill mode command */
                case 'G':
                    if (! gets(string) ) fin (0);
                    break;
                /* poly start command */
                case 'P':
                    readswpd (&x, stdin);
                    poly_start_flag=1;
                    printf("newpath\n");
                    break;
                /* poly draw command */
                case 'E':
                    printf("closepath\n");
                    printf("fill\n");
                    poly_start_flag=0;
                    break;

            case 'T':
                printf ("%6.3f %6.3f r\n", -newpitch/4, -newpitch/4);
                i = 0;
                while((string[i++]=getc(stdin)) != '\0') {
                  if(i==255) {
                    string[i] = '\0';
                    /* backspacing stuff added by gld */
                    j = 0;
                    while(string[j] == (char)0x08)  j++;
                    
                    if(j != 0)
                      printf("%6.3f %6.3f r\n",
                         -newpitch*j/2.,0.);
                    /* gld added +j in next line */
                    printf ("\t(%s) s\n", string+j);
                    /* gld added to restore loc */
                    if(j != 0)
                      printf("%6.3f %6.3f r\n",
                         newpitch*j/2.,0.);



/* old line             printf ("\t(%s) s\n", string); */
                    i = 0;
                  }
                }
                /* backspacing stuff added by gld */
                j = 0;
                while(string[j] == (char)0x08)  j++;
                
                if(j != 0)
                  printf("%6.3f %6.3f r\n",
                     -newpitch*j/2.,0.);
                /* gld added +j in next line */
                printf ("\t(%s) s\n", string+j);
                /* gld added to restore loc */
                if(j != 0)
                  printf("%6.3f %6.3f r\n",
                     newpitch*j/2.,0.);
/* old line         printf ("\t(%s) s\n", string); */
                printf ("%6.3f %6.3f r\n", newpitch/4, newpitch/4);
            break;

            case 'e':    /*  erase ()   */
                    printf ("erasepage\n");  /* just erase current page */
            break;

            case 'U':    /*  plflush ()   */
                    printf ("showpage\n");  /* just eject current page */
            break;

            /* rpl commented out to allow new olxplot command to be
               ignored 
            case 'F':
                i = 0;
                while(string[i++] = getc(stdin));
                plmode(string);
                        break;
            */
            case 'f':    /*  linemod(dashstyle)   */
                if (! gets(string) ) fin (0);
                dashstyle = string;

                if (strcmp (dashstyle, "dotted") == 0) {
                printf("[%6.3f %6.3f] 0 setdash\n", 1 DOTS, 4 DOTS);
                }
                else if (strcmp (dashstyle, "solid") == 0) {
                printf("[] 0 setdash\n");
                }
                else if (strcmp (dashstyle, "longdashed") == 0) {
                printf("[%6.3f %6.3f] 0 setdash\n", 4 DOTS, 1 DOTS);
                }
                else if (strcmp (dashstyle, "shortdashed") == 0) {
                printf("[%6.3f %6.3f] 0 setdash\n", 2 DOTS, 3 DOTS);
                }
                else if (strcmp (dashstyle, "dotdashed") == 0) {
                printf("[%6.3f %6.3f %6.3f %6.3f] 0 setdash\n",5 DOTS, 2 DOTS, 1 DOTS, 2 DOTS);
                }
                else break;   /* ignore any other styles */
            break;

            case 's':   /*   space (x, y, x2, y2)   */
                readswpd (&x, stdin);
                readswpd (&y, stdin);
                readswpd (&x2, stdin);
                readswpd (&y2, stdin);

                xscale =  DEFXSCALE / (float) (x2 - x);
                yscale =  DEFYSCALE / (float) (y2 - y);
                scale = (xscale < yscale) ? xscale : yscale;  /* min */
                newlinew = linewidth / scale;
                newpitch = pitch / scale;

                initcoords(flip);
                /* gildea 25Jun86: 
                   exchanged order of next two lines */
                printf ("%6.3f %6.3f scale\n", scale, scale);
                printf ("%d %d translate\n", -x, -y);
                printf ("%6.3f setlinewidth\n", newlinew);
                printf ("%s findfont ", font);
                printf ("%6.3f scalefont ", newpitch);
                printf ("setfont\n");
            break;

        } /* end of switch */

    } while (FOREVER);      
}

void
initcoords (flip)  /* set up coordinate system default  (before scaling) */
int flip; /* added - MAZ */
{
  if (flip) {
    printf("initmatrix\n");
    printf("90 rotate\n");
    printf("0 %f translate \n", -DEFXSCALE);
    printf("%4.1f %4.1f translate \n", MARGY / 2, MARGX / 2);
    printf("%f %f scale \n", MARGSCALE, MARGSCALE);
  }
  else {
    printf("initmatrix\n");
    printf("%4.1f %4.1f translate \n", MARGX / 2, MARGY / 2);
    printf("%f %f scale \n", MARGSCALE, MARGSCALE);
  }
}

void
readswpd (location, stream) 
unsigned short int *location;
FILE *stream;
{
    unsigned short int nextch;

    nextch = getc(stream);
    *location = getc(stream);
    *location = ((0377 & (*location)) << 8) + (0377 & nextch);
}

void
fin (error)  /* end of input file */
int error;
{
    if (! error)  /* flush output at end of file */ {
        printf ("\n%d {copypage} repeat showpage\n",ncopies-1);
        printf ("\nplot2pscontext restore\n");
    }
    else    /* fatal error (none provided in original source) */
      {
        printf ("%% fatal error in input file: no page produced.\n");
        fprintf (stderr,"%% fatal error in input file: no page produced.\n");
          }
    exit (error);
}
/* plmode added by Greg Duckworth */

void
plmode(s)
char *s ;
{
        int c ;
        putc('F',stdout) ;  
        do {
                putc(c = *s++,stdout) ;
        } while(c) ;
        return ;
}


char *gets(s)
char *s;
{
   char *saves;  int c;

   saves = s;
   while ( (c=getchar()) != '\0' /** for lipgraph **/
        && c != '\n'         /** for unix graph **/
        && c != EOF )
       (*(s++)) = c;

   (*(s++)) = '\0';

   if (c == EOF) return((char *) NULL);

   return(saves);
}
