/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/***---------------------------------------------------------------------------
* NAME:     set_color()
* PURPOSE:  get rgb values for x open windows color name
* USAGE: 
*   int set_color(string, red, green, blue)
*       char    *string;        - x color name 
*       float *red, *green, *blue;  - rgb values from 0.0 to 1.0
* 
* RETURN VALUE:
*   1 if found color, otherwise 0
*
* EXAMPLE:
*   x = set_color("yellow", &red, &green, &blue);
*
* DESCRIPTION:
*   Compares color string name to names recognized by the Lincoln
*       openwindows server. Sets rgb values to those used by the server.
*       Does not interrogate server but uses names originally stored
*       in $OPENWINHOME/lib/rgb.txt. This allows the plot2ps program
*       to be ported anywhere where x isn't running. It also requires
*       modification of this program when the rgb.txt file changes.
*       Use the shell script gen_colors to simplify conversion to
*       a format that can be used in this program.
*       Use gen_colors to generate file with the color table.
*       Then add by hand the colors foregound and background.
* 
* BUGS AND LIMITATIONS:
*   May have to generate a new color table when Openwindows
*       changes.
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/lippic/plot2ps/set_color.c,v 11.4 2001/11/29 18:57:58 kukolich Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/lippic/plot2ps/set_color.c,v $
* CREATED: 1-Feb-90 rpl                             MIT LINCOLN LABORATORY
*---------------------------------------------------------------------------
* Copyright 1990 MIT Lincoln Laboratory.
* This software is not for release and should neither be copied
* nor distributed without explicit permission of MIT Lincoln Laboratory.
--------------------------------------------------------------------------***/
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#if HAVE_STRINGS_H
#include <strings.h>
#endif
#include <ctype.h>
struct color_grayscale
    {
    float red_value;
    float green_value;
    float blue_value;
    char *name;
    };

static struct color_grayscale grayscale_table[] = {
{0, 0, 0,   "foreground"},
{0, .392157,    0,  "darkgreen"},
{0, .392157,    0,  "dark green"},
{0, .407843,    .545098,    "deepskyblue4"},
{0, .52549, .545098,    "turquoise4"},
{0, .545098,    .270588,    "springgreen4"},
{0, .545098,    .545098,    "cyan4"},
{0, .545098,    0,  "green4"},
{0, .603922,    .803922,    "deepskyblue3"},
{0, .698039,    .933333,    "deepskyblue2"},
{0, .74902, 1,  "deepskyblue"},
{0, .74902, 1,  "deepskyblue1"},
{0, .74902, 1,  "deep sky blue"},
{0, .772549,    .803922,    "turquoise3"},
{0, .803922,    .4, "springgreen3"},
{0, .803922,    .803922,    "cyan3"},
{0, .803922,    0,  "green3"},
{0, .807843,    .819608,    "darkturquoise"},
{0, .807843,    .819608,    "dark turquoise"},
{0, .898039,    .933333,    "turquoise2"},
{0, .933333,    .462745,    "springgreen2"},
{0, .933333,    .933333,    "cyan2"},
{0, .933333,    0,  "green2"},
{0, .960784,    1,  "turquoise1"},
{0, .980392,    .603922,    "mediumspringgreen"},
{0, .980392,    .603922,    "medium spring green"},
{0, 0,  .501961,    "navyblue"},
{0, 0,  .501961,    "navy"},
{0, 0,  .501961,    "navy blue"},
{0, 0,  .545098,    "blue4"},
{0, 0,  .803922,    "mediumblue"},
{0, 0,  .803922,    "blue3"},
{0, 0,  .803922,    "medium blue"},
{0, 0,  .933333,    "blue2"},
{0, 0,  0,  "black"},
{0, 0,  0,  "gray0"},
{0, 0,  0,  "grey0"},
{0, 0,  1,  "blue"},
{0, 0,  1,  "blue1"},
{0, 1,  .498039,    "springgreen"},
{0, 1,  .498039,    "springgreen1"},
{0, 1,  .498039,    "spring green"},
{0, 1,  0,  "green"},
{0, 1,  0,  "green1"},
{0, 1,  1,  "cyan"},
{0, 1,  1,  "cyan1"},
{0.0117647, 0.0117647,  0.0117647,  "gray1"},
{0.0117647, 0.0117647,  0.0117647,  "grey1"},
{0.0196078, 0.0196078,  0.0196078,  "gray2"},
{0.0196078, 0.0196078,  0.0196078,  "grey2"},
{0.0313725, 0.0313725,  0.0313725,  "gray3"},
{0.0313725, 0.0313725,  0.0313725,  "grey3"},
{0.0392157, 0.0392157,  0.0392157,  "gray4"},
{0.0392157, 0.0392157,  0.0392157,  "grey4"},
{0.0509804, 0.0509804,  0.0509804,  "gray5"},
{0.0509804, 0.0509804,  0.0509804,  "grey5"},
{0.0588235, 0.0588235,  0.0588235,  "gray6"},
{0.0588235, 0.0588235,  0.0588235,  "grey6"},
{0.0627451, .305882,    .545098,    "dodgerblue4"},
{0.0705882, 0.0705882,  0.0705882,  "gray7"},
{0.0705882, 0.0705882,  0.0705882,  "grey7"},
{0.0784314, 0.0784314,  0.0784314,  "gray8"},
{0.0784314, 0.0784314,  0.0784314,  "grey8"},
{0.0901961, 0.0901961,  0.0901961,  "gray9"},
{0.0901961, 0.0901961,  0.0901961,  "grey9"},
{0.0941176, .454902,    .803922,    "dodgerblue3"},
{0.0980392, 0.0980392,  .439216,    "midnightblue"},
{0.0980392, 0.0980392,  .439216,    "midnight blue"},
{.101961,   .101961,    .101961,    "gray10"},
{.101961,   .101961,    .101961,    "grey10"},
{.109804,   .109804,    .109804,    "gray11"},
{.109804,   .109804,    .109804,    "grey11"},
{.109804,   .52549, .933333,    "dodgerblue2"},
{.117647,   .564706,    1,  "dodgerblue"},
{.117647,   .564706,    1,  "dodgerblue1"},
{.117647,   .564706,    1,  "dodger blue"},
{.121569,   .121569,    .121569,    "gray12"},
{.121569,   .121569,    .121569,    "grey12"},
{.12549,    .698039,    .666667,    "lightseagreen"},
{.12549,    .698039,    .666667,    "light sea green"},
{.129412,   .129412,    .129412,    "gray13"},
{.129412,   .129412,    .129412,    "grey13"},
{.133333,   .545098,    .133333,    "forestgreen"},
{.133333,   .545098,    .133333,    "forest green"},
{.141176,   .141176,    .141176,    "gray14"},
{.141176,   .141176,    .141176,    "grey14"},
{.14902,    .14902, .14902, "gray15"},
{.14902,    .14902, .14902, "grey15"},
{.152941,   .25098, .545098,    "royalblue4"},
{.160784,   .160784,    .160784,    "gray16"},
{.160784,   .160784,    .160784,    "grey16"},
{.168627,   .168627,    .168627,    "gray17"},
{.168627,   .168627,    .168627,    "grey17"},
{.180392,   .180392,    .180392,    "gray18"},
{.180392,   .180392,    .180392,    "grey18"},
{.180392,   .545098,    .341176,    "seagreen"},
{.180392,   .545098,    .341176,    "seagreen4"},
{.180392,   .545098,    .341176,    "sea green"},
{.184314,   .309804,    .309804,    "darkslategray"},
{.184314,   .309804,    .309804,    "darkslategrey"},
{.184314,   .309804,    .309804,    "dark slate gray"},
{.184314,   .309804,    .309804,    "dark slate grey"},
{.188235,   .188235,    .188235,    "gray19"},
{.188235,   .188235,    .188235,    "grey19"},
{.196078,   .803922,    .196078,    "limegreen"},
{.196078,   .803922,    .196078,    "lime green"},
{.2,    .2, .2, "gray20"},
{.2,    .2, .2, "grey20"},
{.211765,   .211765,    .211765,    "gray21"},
{.211765,   .211765,    .211765,    "grey21"},
{.211765,   .392157,    .545098,    "steelblue4"},
{.219608,   .219608,    .219608,    "gray22"},
{.219608,   .219608,    .219608,    "grey22"},
{.227451,   .372549,    .803922,    "royalblue3"},
{.231373,   .231373,    .231373,    "gray23"},
{.231373,   .231373,    .231373,    "grey23"},
{.235294,   .701961,    .443137,    "mediumseagreen"},
{.235294,   .701961,    .443137,    "medium sea green"},
{.239216,   .239216,    .239216,    "gray24"},
{.239216,   .239216,    .239216,    "grey24"},
{.25098,    .25098, .25098, "gray25"},
{.25098,    .25098, .25098, "grey25"},
{.25098,    .878431,    .815686,    "turquoise"},
{.254902,   .411765,    .882353,    "royalblue"},
{.254902,   .411765,    .882353,    "royal blue"},
{.258824,   .258824,    .258824,    "gray26"},
{.258824,   .258824,    .258824,    "grey26"},
{.262745,   .431373,    .933333,    "royalblue2"},
{.262745,   .803922,    .501961,    "seagreen3"},
{.270588,   .270588,    .270588,    "gray27"},
{.270588,   .270588,    .270588,    "grey27"},
{.270588,   .545098,    .454902,    "aquamarine4"},
{.270588,   .545098,    0,  "chartreuse4"},
{.27451,    .509804,    .705882,    "steelblue"},
{.27451,    .509804,    .705882,    "steel blue"},
{.278431,   .235294,    .545098,    "slateblue4"},
{.278431,   .278431,    .278431,    "gray28"},
{.278431,   .278431,    .278431,    "grey28"},
{.282353,   .239216,    .545098,    "darkslateblue"},
{.282353,   .239216,    .545098,    "dark slate blue"},
{.282353,   .462745,    1,  "royalblue1"},
{.282353,   .819608,    .8, "mediumturquoise"},
{.282353,   .819608,    .8, "medium turquoise"},
{.290196,   .290196,    .290196,    "gray29"},
{.290196,   .290196,    .290196,    "grey29"},
{.290196,   .439216,    .545098,    "skyblue4"},
{.301961,   .301961,    .301961,    "gray30"},
{.301961,   .301961,    .301961,    "grey30"},
{.305882,   .933333,    .580392,    "seagreen2"},
{.309804,   .309804,    .309804,    "gray31"},
{.309804,   .309804,    .309804,    "grey31"},
{.309804,   .580392,    .803922,    "steelblue3"},
{.321569,   .321569,    .321569,    "gray32"},
{.321569,   .321569,    .321569,    "grey32"},
{.321569,   .545098,    .545098,    "darkslategray4"},
{.32549,    .52549, .545098,    "cadetblue4"},
{.329412,   .329412,    .329412,    "gray33"},
{.329412,   .329412,    .329412,    "grey33"},
{.329412,   .545098,    .329412,    "palegreen4"},
{.329412,   1,  .623529,    "seagreen1"},
{.333333,   .101961,    .545098,    "purple4"},
{.333333,   .419608,    .184314,    "darkolivegreen"},
{.333333,   .419608,    .184314,    "dark olive green"},
{.341176,   .341176,    .341176,    "gray34"},
{.341176,   .341176,    .341176,    "grey34"},
{.34902,    .34902, .34902, "gray35"},
{.34902,    .34902, .34902, "grey35"},
{.360784,   .360784,    .360784,    "gray36"},
{.360784,   .360784,    .360784,    "grey36"},
{.360784,   .67451, .933333,    "steelblue2"},
{.364706,   .278431,    .545098,    "mediumpurple4"},
{.368627,   .368627,    .368627,    "gray37"},
{.368627,   .368627,    .368627,    "grey37"},
{.372549,   .619608,    .627451,    "cadetblue"},
{.372549,   .619608,    .627451,    "cadet blue"},
{.376471,   .482353,    .545098,    "lightskyblue4"},
{.380392,   .380392,    .380392,    "gray38"},
{.380392,   .380392,    .380392,    "grey38"},
{.388235,   .388235,    .388235,    "gray39"},
{.388235,   .388235,    .388235,    "grey39"},
{.388235,   .721569,    1,  "steelblue1"},
{.392157,   .584314,    .929412,    "cornflowerblue"},
{.392157,   .584314,    .929412,    "cornflower blue"},
{.4,    .4, .4, "gray40"},
{.4,    .4, .4, "grey40"},
{.4,    .545098,    .545098,    "paleturquoise4"},
{.4,    .803922,    .666667,    "mediumaquamarine"},
{.4,    .803922,    .666667,    "aquamarine3"},
{.4,    .803922,    .666667,    "medium aquamarine"},
{.4,    .803922,    0,  "chartreuse3"},
{.407843,   .133333,    .545098,    "darkorchid4"},
{.407843,   .513725,    .545098,    "lightblue4"},
{.411765,   .34902, .803922,    "slateblue3"},
{.411765,   .411765,    .411765,    "dimgray"},
{.411765,   .411765,    .411765,    "dimgrey"},
{.411765,   .411765,    .411765,    "dim gray"},
{.411765,   .411765,    .411765,    "dim grey"},
{.411765,   .411765,    .411765,    "gray41"},
{.411765,   .411765,    .411765,    "grey41"},
{.411765,   .545098,    .133333,    "olivedrab4"},
{.411765,   .545098,    .411765,    "darkseagreen4"},
{.415686,   .352941,    .803922,    "slateblue"},
{.415686,   .352941,    .803922,    "slate blue"},
{.419608,   .419608,    .419608,    "gray42"},
{.419608,   .419608,    .419608,    "grey42"},
{.419608,   .556863,    .137255,    "olivedrab"},
{.419608,   .556863,    .137255,    "olive drab"},
{.423529,   .482353,    .545098,    "slategray4"},
{.423529,   .65098, .803922,    "skyblue3"},
{.431373,   .431373,    .431373,    "gray43"},
{.431373,   .431373,    .431373,    "grey43"},
{.431373,   .482353,    .545098,    "lightsteelblue4"},
{.431373,   .545098,    .239216,    "darkolivegreen4"},
{.439216,   .439216,    .439216,    "gray44"},
{.439216,   .439216,    .439216,    "grey44"},
{.439216,   .501961,    .564706,    "slategray"},
{.439216,   .501961,    .564706,    "slategrey"},
{.439216,   .501961,    .564706,    "slate gray"},
{.439216,   .501961,    .564706,    "slate grey"},
{.45098,    .45098, .45098, "gray45"},
{.45098,    .45098, .45098, "grey45"},
{.458824,   .458824,    .458824,    "gray46"},
{.458824,   .458824,    .458824,    "grey46"},
{.462745,   .933333,    .776471,    "aquamarine2"},
{.462745,   .933333,    0,  "chartreuse2"},
{.466667,   .533333,    .6, "lightslategray"},
{.466667,   .533333,    .6, "lightslategrey"},
{.466667,   .533333,    .6, "light slate gray"},
{.466667,   .533333,    .6, "light slate grey"},
{.470588,   .470588,    .470588,    "gray47"},
{.470588,   .470588,    .470588,    "grey47"},
{.47451,    .803922,    .803922,    "darkslategray3"},
{.478431,   .215686,    .545098,    "mediumorchid4"},
{.478431,   .403922,    .933333,    "slateblue2"},
{.478431,   .478431,    .478431,    "gray48"},
{.478431,   .478431,    .478431,    "grey48"},
{.478431,   .545098,    .545098,    "lightcyan4"},
{.478431,   .772549,    .803922,    "cadetblue3"},
{.482353,   .407843,    .933333,    "mediumslateblue"},
{.482353,   .407843,    .933333,    "medium slate blue"},
{.486275,   .803922,    .486275,    "palegreen3"},
{.486275,   .988235,    0,  "lawngreen"},
{.486275,   .988235,    0,  "lawn green"},
{.490196,   .14902, .803922,    "purple3"},
{.490196,   .490196,    .490196,    "gray49"},
{.490196,   .490196,    .490196,    "grey49"},
{.494118,   .752941,    .933333,    "skyblue2"},
{.498039,   .498039,    .498039,    "gray50"},
{.498039,   .498039,    .498039,    "grey50"},
{.498039,   1,  .831373,    "aquamarine"},
{.498039,   1,  .831373,    "aquamarine1"},
{.498039,   1,  0,  "chartreuse"},
{.498039,   1,  0,  "chartreuse1"},
{.509804,   .509804,    .509804,    "gray51"},
{.509804,   .509804,    .509804,    "grey51"},
{.513725,   .435294,    1,  "slateblue1"},
{.513725,   .545098,    .513725,    "honeydew4"},
{.513725,   .545098,    .545098,    "azure4"},
{.517647,   .439216,    1,  "lightslateblue"},
{.517647,   .439216,    1,  "light slate blue"},
{.521569,   .521569,    .521569,    "gray52"},
{.521569,   .521569,    .521569,    "grey52"},
{.529412,   .529412,    .529412,    "gray53"},
{.529412,   .529412,    .529412,    "grey53"},
{.529412,   .807843,    .921569,    "skyblue"},
{.529412,   .807843,    .921569,    "sky blue"},
{.529412,   .807843,    .980392,    "lightskyblue"},
{.529412,   .807843,    .980392,    "light sky blue"},
{.529412,   .807843,    1,  "skyblue1"},
{.537255,   .407843,    .803922,    "mediumpurple3"},
{.541176,   .168627,    .886275,    "blueviolet"},
{.541176,   .168627,    .886275,    "blue violet"},
{.541176,   .541176,    .541176,    "gray54"},
{.541176,   .541176,    .541176,    "grey54"},
{.545098,   .101961,    .101961,    "firebrick4"},
{.545098,   .109804,    .384314,    "maroon4"},
{.545098,   .133333,    .321569,    "violetred4"},
{.545098,   .137255,    .137255,    "brown4"},
{.545098,   .145098,    0,  "orangered4"},
{.545098,   .211765,    .14902, "tomato4"},
{.545098,   .227451,    .227451,    "indianred4"},
{.545098,   .227451,    .384314,    "hotpink4"},
{.545098,   .243137,    .184314,    "coral4"},
{.545098,   .270588,    0,  "darkorange4"},
{.545098,   .270588,    0.0745098,  "saddlebrown"},
{.545098,   .270588,    0.0745098,  "chocolate4"},
{.545098,   .270588,    0.0745098,  "saddle brown"},
{.545098,   .278431,    .14902, "sienna4"},
{.545098,   .278431,    .364706,    "palevioletred4"},
{.545098,   .278431,    .537255,    "orchid4"},
{.545098,   .298039,    .223529,    "salmon4"},
{.545098,   .341176,    .258824,    "lightsalmon4"},
{.545098,   .352941,    .168627,    "tan4"},
{.545098,   .352941,    0,  "orange4"},
{.545098,   .372549,    .396078,    "lightpink4"},
{.545098,   .388235,    .423529,    "pink4"},
{.545098,   .396078,    0.0313725,  "darkgoldenrod4"},
{.545098,   .4, .545098,    "plum4"},
{.545098,   .411765,    .411765,    "rosybrown4"},
{.545098,   .411765,    0.0784314,  "goldenrod4"},
{.545098,   .45098, .333333,    "burlywood4"},
{.545098,   .458824,    0,  "gold4"},
{.545098,   .466667,    .396078,    "peachpuff4"},
{.545098,   .47451, .368627,    "navajowhite4"},
{.545098,   .482353,    .545098,    "thistle4"},
{.545098,   .490196,    .419608,    "bisque4"},
{.545098,   .490196,    .482353,    "mistyrose4"},
{.545098,   .494118,    .4, "wheat4"},
{.545098,   .505882,    .298039,    "lightgoldenrod4"},
{.545098,   .513725,    .470588,    "antiquewhite4"},
{.545098,   .513725,    .52549, "lavenderblush4"},
{.545098,   .52549, .305882,    "khaki4"},
{.545098,   .52549, .509804,    "seashell4"},
{.545098,   .533333,    .470588,    "cornsilk4"},
{.545098,   .537255,    .439216,    "lemonchiffon4"},
{.545098,   .537255,    .537255,    "snow4"},
{.545098,   .545098,    .478431,    "lightyellow4"},
{.545098,   .545098,    .513725,    "ivory4"},
{.545098,   .545098,    0,  "yellow4"},
{.545098,   0,  .545098,    "magenta4"},
{.545098,   0,  0,  "red4"},
{.545098,   0.0392157,  .313725,    "deeppink4"},
{.54902,    .54902, .54902, "gray55"},
{.54902,    .54902, .54902, "grey55"},
{.552941,   .713725,    .803922,    "lightskyblue3"},
{.552941,   .933333,    .933333,    "darkslategray2"},
{.556863,   .898039,    .933333,    "cadetblue2"},
{.560784,   .560784,    .560784,    "gray56"},
{.560784,   .560784,    .560784,    "grey56"},
{.560784,   .737255,    .560784,    "darkseagreen"},
{.560784,   .737255,    .560784,    "dark sea green"},
{.564706,   .933333,    .564706,    "palegreen2"},
{.568627,   .172549,    .933333,    "purple2"},
{.568627,   .568627,    .568627,    "gray57"},
{.568627,   .568627,    .568627,    "grey57"},
{.576471,   .439216,    .858824,    "mediumpurple"},
{.576471,   .439216,    .858824,    "medium purple"},
{.580392,   .580392,    .580392,    "gray58"},
{.580392,   .580392,    .580392,    "grey58"},
{.580392,   0,  .827451,    "darkviolet"},
{.580392,   0,  .827451,    "dark violet"},
{.588235,   .588235,    .588235,    "gray59"},
{.588235,   .588235,    .588235,    "grey59"},
{.588235,   .803922,    .803922,    "paleturquoise3"},
{.592157,   1,  1,  "darkslategray1"},
{.596078,   .960784,    1,  "cadetblue1"},
{.596078,   .984314,    .596078,    "palegreen"},
{.596078,   .984314,    .596078,    "pale green"},
{.6,    .196078,    .8, "darkorchid"},
{.6,    .196078,    .8, "dark orchid"},
{.6,    .6, .6, "gray60"},
{.6,    .6, .6, "grey60"},
{.603922,   .196078,    .803922,    "darkorchid3"},
{.603922,   .752941,    .803922,    "lightblue3"},
{.603922,   .803922,    .196078,    "olivedrab3"},
{.603922,   .803922,    .196078,    "yellowgreen"},
{.603922,   .803922,    .196078,    "yellow green"},
{.603922,   1,  .603922,    "palegreen1"},
{.607843,   .188235,    1,  "purple1"},
{.607843,   .803922,    .607843,    "darkseagreen3"},
{.611765,   .611765,    .611765,    "gray61"},
{.611765,   .611765,    .611765,    "grey61"},
{.619608,   .619608,    .619608,    "gray62"},
{.619608,   .619608,    .619608,    "grey62"},
{.623529,   .47451, .933333,    "mediumpurple2"},
{.623529,   .713725,    .803922,    "slategray3"},
{.627451,   .12549, .941176,    "purple"},
{.627451,   .321569,    .176471,    "sienna"},
{.631373,   .631373,    .631373,    "gray63"},
{.631373,   .631373,    .631373,    "grey63"},
{.635294,   .709804,    .803922,    "lightsteelblue3"},
{.635294,   .803922,    .352941,    "darkolivegreen3"},
{.639216,   .639216,    .639216,    "gray64"},
{.639216,   .639216,    .639216,    "grey64"},
{.643137,   .827451,    .933333,    "lightskyblue2"},
{.647059,   .164706,    .164706,    "brown"},
{.65098,    .65098, .65098, "gray65"},
{.65098,    .65098, .65098, "grey65"},
{.658824,   .658824,    .658824,    "gray66"},
{.658824,   .658824,    .658824,    "grey66"},
{.670588,   .509804,    1,  "mediumpurple1"},
{.670588,   .670588,    .670588,    "gray67"},
{.670588,   .670588,    .670588,    "grey67"},
{.678431,   .678431,    .678431,    "gray68"},
{.678431,   .678431,    .678431,    "grey68"},
{.678431,   .847059,    .901961,    "lightblue"},
{.678431,   .847059,    .901961,    "light blue"},
{.678431,   1,  .184314,    "greenyellow"},
{.678431,   1,  .184314,    "green yellow"},
{.682353,   .933333,    .933333,    "paleturquoise2"},
{.686275,   .933333,    .933333,    "paleturquoise"},
{.686275,   .933333,    .933333,    "pale turquoise"},
{.690196,   .188235,    .376471,    "maroon"},
{.690196,   .690196,    .690196,    "gray69"},
{.690196,   .690196,    .690196,    "grey69"},
{.690196,   .768627,    .870588,    "lightsteelblue"},
{.690196,   .768627,    .870588,    "light steel blue"},
{.690196,   .878431,    .901961,    "powderblue"},
{.690196,   .878431,    .901961,    "powder blue"},
{.690196,   .886275,    1,  "lightskyblue1"},
{.698039,   .133333,    .133333,    "firebrick"},
{.698039,   .227451,    .933333,    "darkorchid2"},
{.698039,   .87451, .933333,    "lightblue2"},
{.701961,   .701961,    .701961,    "gray70"},
{.701961,   .701961,    .701961,    "grey70"},
{.701961,   .933333,    .227451,    "olivedrab2"},
{.705882,   .321569,    .803922,    "mediumorchid3"},
{.705882,   .803922,    .803922,    "lightcyan3"},
{.705882,   .933333,    .705882,    "darkseagreen2"},
{.709804,   .709804,    .709804,    "gray71"},
{.709804,   .709804,    .709804,    "grey71"},
{.721569,   .52549, 0.0431373,  "darkgoldenrod"},
{.721569,   .52549, 0.0431373,  "dark goldenrod"},
{.721569,   .721569,    .721569,    "gray72"},
{.721569,   .721569,    .721569,    "grey72"},
{.72549,    .827451,    .933333,    "slategray2"},
{.729412,   .333333,    .827451,    "mediumorchid"},
{.729412,   .333333,    .827451,    "medium orchid"},
{.729412,   .729412,    .729412,    "gray73"},
{.729412,   .729412,    .729412,    "grey73"},
{.733333,   1,  1,  "paleturquoise1"},
{.737255,   .560784,    .560784,    "rosybrown"},
{.737255,   .560784,    .560784,    "rosy brown"},
{.737255,   .823529,    .933333,    "lightsteelblue2"},
{.737255,   .933333,    .407843,    "darkolivegreen2"},
{.741176,   .717647,    .419608,    "darkkhaki"},
{.741176,   .717647,    .419608,    "dark khaki"},
{.741176,   .741176,    .741176,    "gray74"},
{.741176,   .741176,    .741176,    "grey74"},
{.74902,    .243137,    1,  "darkorchid1"},
{.74902,    .74902, .74902, "gray75"},
{.74902,    .74902, .74902, "grey75"},
{.74902,    .937255,    1,  "lightblue1"},
{.752941,   .752941,    .752941,    "gray"},
{.752941,   .752941,    .752941,    "grey"},
{.752941,   1,  .243137,    "olivedrab1"},
{.756863,   .803922,    .756863,    "honeydew3"},
{.756863,   .803922,    .803922,    "azure3"},
{.756863,   1,  .756863,    "darkseagreen1"},
{.760784,   .760784,    .760784,    "gray76"},
{.760784,   .760784,    .760784,    "grey76"},
{.768627,   .768627,    .768627,    "gray77"},
{.768627,   .768627,    .768627,    "grey77"},
{.776471,   .886275,    1,  "slategray1"},
{.780392,   .780392,    .780392,    "gray78"},
{.780392,   .780392,    .780392,    "grey78"},
{.780392,   0.0823529,  .521569,    "mediumvioletred"},
{.780392,   0.0823529,  .521569,    "medium violet red"},
{.788235,   .788235,    .788235,    "gray79"},
{.788235,   .788235,    .788235,    "grey79"},
{.792157,   .882353,    1,  "lightsteelblue1"},
{.792157,   1,  .439216,    "darkolivegreen1"},
{.8,    .8, .8, "gray80"},
{.8,    .8, .8, "grey80"},
{.803922,   .14902, .14902, "firebrick3"},
{.803922,   .160784,    .564706,    "maroon3"},
{.803922,   .196078,    .470588,    "violetred3"},
{.803922,   .2, .2, "brown3"},
{.803922,   .215686,    0,  "orangered3"},
{.803922,   .309804,    .223529,    "tomato3"},
{.803922,   .333333,    .333333,    "indianred3"},
{.803922,   .356863,    .270588,    "coral3"},
{.803922,   .360784,    .360784,    "indianred"},
{.803922,   .360784,    .360784,    "indian red"},
{.803922,   .376471,    .564706,    "hotpink3"},
{.803922,   .4, .113725,    "chocolate3"},
{.803922,   .4, 0,  "darkorange3"},
{.803922,   .407843,    .223529,    "sienna3"},
{.803922,   .407843,    .537255,    "palevioletred3"},
{.803922,   .411765,    .788235,    "orchid3"},
{.803922,   .439216,    .329412,    "salmon3"},
{.803922,   .505882,    .384314,    "lightsalmon3"},
{.803922,   .521569,    .247059,    "peru"},
{.803922,   .521569,    .247059,    "tan3"},
{.803922,   .521569,    0,  "orange3"},
{.803922,   .54902, .584314,    "lightpink3"},
{.803922,   .568627,    .619608,    "pink3"},
{.803922,   .584314,    0.0470588,  "darkgoldenrod3"},
{.803922,   .588235,    .803922,    "plum3"},
{.803922,   .607843,    .113725,    "goldenrod3"},
{.803922,   .607843,    .607843,    "rosybrown3"},
{.803922,   .666667,    .490196,    "burlywood3"},
{.803922,   .678431,    0,  "gold3"},
{.803922,   .686275,    .584314,    "peachpuff3"},
{.803922,   .701961,    .545098,    "navajowhite3"},
{.803922,   .709804,    .803922,    "thistle3"},
{.803922,   .717647,    .619608,    "bisque3"},
{.803922,   .717647,    .709804,    "mistyrose3"},
{.803922,   .729412,    .588235,    "wheat3"},
{.803922,   .745098,    .439216,    "lightgoldenrod3"},
{.803922,   .752941,    .690196,    "antiquewhite3"},
{.803922,   .756863,    .772549,    "lavenderblush3"},
{.803922,   .772549,    .74902, "seashell3"},
{.803922,   .776471,    .45098, "khaki3"},
{.803922,   .784314,    .694118,    "cornsilk3"},
{.803922,   .788235,    .647059,    "lemonchiffon3"},
{.803922,   .788235,    .788235,    "snow3"},
{.803922,   .803922,    .705882,    "lightyellow3"},
{.803922,   .803922,    .756863,    "ivory3"},
{.803922,   .803922,    0,  "yellow3"},
{.803922,   0,  .803922,    "magenta3"},
{.803922,   0,  0,  "red3"},
{.803922,   0.0627451,  .462745,    "deeppink3"},
{.811765,   .811765,    .811765,    "gray81"},
{.811765,   .811765,    .811765,    "grey81"},
{.815686,   .12549, .564706,    "violetred"},
{.815686,   .12549, .564706,    "violet red"},
{.819608,   .372549,    .933333,    "mediumorchid2"},
{.819608,   .819608,    .819608,    "gray82"},
{.819608,   .819608,    .819608,    "grey82"},
{.819608,   .933333,    .933333,    "lightcyan2"},
{.823529,   .411765,    .117647,    "chocolate"},
{.823529,   .705882,    .54902, "tan"},
{.827451,   .827451,    .827451,    "lightgray"},
{.827451,   .827451,    .827451,    "lightgrey"},
{.827451,   .827451,    .827451,    "light gray"},
{.827451,   .827451,    .827451,    "light grey"},
{.831373,   .831373,    .831373,    "gray83"},
{.831373,   .831373,    .831373,    "grey83"},
{.839216,   .839216,    .839216,    "gray84"},
{.839216,   .839216,    .839216,    "grey84"},
{.847059,   .74902, .847059,    "thistle"},
{.85098,    .85098, .85098, "gray85"},
{.85098,    .85098, .85098, "grey85"},
{.854902,   .439216,    .839216,    "orchid"},
{.854902,   .647059,    .12549, "goldenrod"},
{.858824,   .439216,    .576471,    "palevioletred"},
{.858824,   .439216,    .576471,    "pale violet red"},
{.858824,   .858824,    .858824,    "gray86"},
{.858824,   .858824,    .858824,    "grey86"},
{.862745,   .862745,    .862745,    "gainsboro"},
{.866667,   .627451,    .866667,    "plum"},
{.870588,   .721569,    .529412,    "burlywood"},
{.870588,   .870588,    .870588,    "gray87"},
{.870588,   .870588,    .870588,    "grey87"},
{.878431,   .4, 1,  "mediumorchid1"},
{.878431,   .878431,    .878431,    "gray88"},
{.878431,   .878431,    .878431,    "grey88"},
{.878431,   .933333,    .878431,    "honeydew2"},
{.878431,   .933333,    .933333,    "azure2"},
{.878431,   1,  1,  "lightcyan"},
{.878431,   1,  1,  "lightcyan1"},
{.878431,   1,  1,  "light cyan"},
{.890196,   .890196,    .890196,    "gray89"},
{.890196,   .890196,    .890196,    "grey89"},
{.898039,   .898039,    .898039,    "gray90"},
{.898039,   .898039,    .898039,    "grey90"},
{.901961,   .901961,    .980392,    "lavender"},
{.909804,   .909804,    .909804,    "gray91"},
{.909804,   .909804,    .909804,    "grey91"},
{.913725,   .588235,    .478431,    "darksalmon"},
{.913725,   .588235,    .478431,    "dark salmon"},
{.921569,   .921569,    .921569,    "gray92"},
{.921569,   .921569,    .921569,    "grey92"},
{.929412,   .929412,    .929412,    "gray93"},
{.929412,   .929412,    .929412,    "grey93"},
{.933333,   .172549,    .172549,    "firebrick2"},
{.933333,   .188235,    .654902,    "maroon2"},
{.933333,   .227451,    .54902, "violetred2"},
{.933333,   .231373,    .231373,    "brown2"},
{.933333,   .25098, 0,  "orangered2"},
{.933333,   .360784,    .258824,    "tomato2"},
{.933333,   .388235,    .388235,    "indianred2"},
{.933333,   .415686,    .313725,    "coral2"},
{.933333,   .415686,    .654902,    "hotpink2"},
{.933333,   .462745,    .129412,    "chocolate2"},
{.933333,   .462745,    0,  "darkorange2"},
{.933333,   .47451, .258824,    "sienna2"},
{.933333,   .47451, .623529,    "palevioletred2"},
{.933333,   .478431,    .913725,    "orchid2"},
{.933333,   .509804,    .384314,    "salmon2"},
{.933333,   .509804,    .933333,    "violet"},
{.933333,   .584314,    .447059,    "lightsalmon2"},
{.933333,   .603922,    .286275,    "tan2"},
{.933333,   .603922,    0,  "orange2"},
{.933333,   .635294,    .678431,    "lightpink2"},
{.933333,   .662745,    .721569,    "pink2"},
{.933333,   .678431,    0.054902,   "darkgoldenrod2"},
{.933333,   .682353,    .933333,    "plum2"},
{.933333,   .705882,    .133333,    "goldenrod2"},
{.933333,   .705882,    .705882,    "rosybrown2"},
{.933333,   .772549,    .568627,    "burlywood2"},
{.933333,   .788235,    0,  "gold2"},
{.933333,   .796078,    .678431,    "peachpuff2"},
{.933333,   .811765,    .631373,    "navajowhite2"},
{.933333,   .823529,    .933333,    "thistle2"},
{.933333,   .835294,    .717647,    "bisque2"},
{.933333,   .835294,    .823529,    "mistyrose2"},
{.933333,   .847059,    .682353,    "wheat2"},
{.933333,   .862745,    .509804,    "lightgoldenrod2"},
{.933333,   .866667,    .509804,    "lightgoldenrod"},
{.933333,   .866667,    .509804,    "light goldenrod"},
{.933333,   .87451, .8, "antiquewhite2"},
{.933333,   .878431,    .898039,    "lavenderblush2"},
{.933333,   .898039,    .870588,    "seashell2"},
{.933333,   .901961,    .521569,    "khaki2"},
{.933333,   .909804,    .666667,    "palegoldenrod"},
{.933333,   .909804,    .666667,    "pale goldenrod"},
{.933333,   .909804,    .803922,    "cornsilk2"},
{.933333,   .913725,    .74902, "lemonchiffon2"},
{.933333,   .913725,    .913725,    "snow2"},
{.933333,   .933333,    .819608,    "lightyellow2"},
{.933333,   .933333,    .878431,    "ivory2"},
{.933333,   .933333,    0,  "yellow2"},
{.933333,   0,  .933333,    "magenta2"},
{.933333,   0,  0,  "red2"},
{.933333,   0.0705882,  .537255,    "deeppink2"},
{.941176,   .501961,    .501961,    "lightcoral"},
{.941176,   .501961,    .501961,    "light coral"},
{.941176,   .901961,    .54902, "khaki"},
{.941176,   .941176,    .941176,    "gray94"},
{.941176,   .941176,    .941176,    "grey94"},
{.941176,   .972549,    1,  "aliceblue"},
{.941176,   .972549,    1,  "alice blue"},
{.941176,   1,  .941176,    "honeydew"},
{.941176,   1,  .941176,    "honeydew1"},
{.941176,   1,  1,  "azure"},
{.941176,   1,  1,  "azure1"},
{.94902,    .94902, .94902, "gray95"},
{.94902,    .94902, .94902, "grey95"},
{.956863,   .643137,    .376471,    "sandybrown"},
{.956863,   .643137,    .376471,    "sandy brown"},
{.960784,   .870588,    .701961,    "wheat"},
{.960784,   .960784,    .862745,    "beige"},
{.960784,   .960784,    .960784,    "whitesmoke"},
{.960784,   .960784,    .960784,    "gray96"},
{.960784,   .960784,    .960784,    "grey96"},
{.960784,   .960784,    .960784,    "white smoke"},
{.960784,   1,  .980392,    "mintcream"},
{.960784,   1,  .980392,    "mint cream"},
{.968627,   .968627,    .968627,    "gray97"},
{.968627,   .968627,    .968627,    "grey97"},
{.972549,   .972549,    1,  "ghostwhite"},
{.972549,   .972549,    1,  "ghost white"},
{.980392,   .501961,    .447059,    "salmon"},
{.980392,   .921569,    .843137,    "antiquewhite"},
{.980392,   .921569,    .843137,    "antique white"},
{.980392,   .941176,    .901961,    "linen"},
{.980392,   .980392,    .823529,    "lightgoldenrodyellow"},
{.980392,   .980392,    .823529,    "light goldenrod yellow"},
{.980392,   .980392,    .980392,    "gray98"},
{.980392,   .980392,    .980392,    "grey98"},
{.988235,   .988235,    .988235,    "gray99"},
{.988235,   .988235,    .988235,    "grey99"},
{.992157,   .960784,    .901961,    "oldlace"},
{.992157,   .960784,    .901961,    "old lace"},
{1, .188235,    .188235,    "firebrick1"},
{1, .203922,    .701961,    "maroon1"},
{1, .243137,    .588235,    "violetred1"},
{1, .25098, .25098, "brown1"},
{1, .270588,    0,  "orangered"},
{1, .270588,    0,  "orangered1"},
{1, .270588,    0,  "orange red"},
{1, .388235,    .278431,    "tomato"},
{1, .388235,    .278431,    "tomato1"},
{1, .411765,    .705882,    "hotpink"},
{1, .411765,    .705882,    "hot pink"},
{1, .415686,    .415686,    "indianred1"},
{1, .431373,    .705882,    "hotpink1"},
{1, .447059,    .337255,    "coral1"},
{1, .498039,    .141176,    "chocolate1"},
{1, .498039,    .313725,    "coral"},
{1, .498039,    0,  "darkorange1"},
{1, .509804,    .278431,    "sienna1"},
{1, .509804,    .670588,    "palevioletred1"},
{1, .513725,    .980392,    "orchid1"},
{1, .54902, .411765,    "salmon1"},
{1, .54902, 0,  "darkorange"},
{1, .54902, 0,  "dark orange"},
{1, .627451,    .478431,    "lightsalmon"},
{1, .627451,    .478431,    "lightsalmon1"},
{1, .627451,    .478431,    "light salmon"},
{1, .647059,    .309804,    "tan1"},
{1, .647059,    0,  "orange"},
{1, .647059,    0,  "orange1"},
{1, .682353,    .72549, "lightpink1"},
{1, .709804,    .772549,    "pink1"},
{1, .713725,    .756863,    "lightpink"},
{1, .713725,    .756863,    "light pink"},
{1, .72549, 0.0588235,  "darkgoldenrod1"},
{1, .733333,    1,  "plum1"},
{1, .752941,    .796078,    "pink"},
{1, .756863,    .145098,    "goldenrod1"},
{1, .756863,    .756863,    "rosybrown1"},
{1, .827451,    .607843,    "burlywood1"},
{1, .843137,    0,  "gold"},
{1, .843137,    0,  "gold1"},
{1, .854902,    .72549, "peachpuff"},
{1, .854902,    .72549, "peachpuff1"},
{1, .854902,    .72549, "peach puff"},
{1, .870588,    .678431,    "navajowhite"},
{1, .870588,    .678431,    "navajowhite1"},
{1, .870588,    .678431,    "navajo white"},
{1, .882353,    1,  "thistle1"},
{1, .894118,    .709804,    "moccasin"},
{1, .894118,    .768627,    "bisque"},
{1, .894118,    .768627,    "bisque1"},
{1, .894118,    .882353,    "mistyrose"},
{1, .894118,    .882353,    "mistyrose1"},
{1, .894118,    .882353,    "misty rose"},
{1, .905882,    .729412,    "wheat1"},
{1, .921569,    .803922,    "blanchedalmond"},
{1, .921569,    .803922,    "blanched almond"},
{1, .92549, .545098,    "lightgoldenrod1"},
{1, .937255,    .835294,    "papayawhip"},
{1, .937255,    .835294,    "papaya whip"},
{1, .937255,    .858824,    "antiquewhite1"},
{1, .941176,    .960784,    "lavenderblush"},
{1, .941176,    .960784,    "lavenderblush1"},
{1, .941176,    .960784,    "lavender blush"},
{1, .960784,    .933333,    "seashell"},
{1, .960784,    .933333,    "seashell1"},
{1, .964706,    .560784,    "khaki1"},
{1, .972549,    .862745,    "cornsilk"},
{1, .972549,    .862745,    "cornsilk1"},
{1, .980392,    .803922,    "lemonchiffon"},
{1, .980392,    .803922,    "lemonchiffon1"},
{1, .980392,    .803922,    "lemon chiffon"},
{1, .980392,    .941176,    "floralwhite"},
{1, .980392,    .941176,    "floral white"},
{1, .980392,    .980392,    "snow"},
{1, .980392,    .980392,    "snow1"},
{1, 0,  0,  "red"},
{1, 0,  0,  "red1"},
{1, 0,  1,  "magenta"},
{1, 0,  1,  "magenta1"},
{1, 0.0784314,  .576471,    "deeppink"},
{1, 0.0784314,  .576471,    "deeppink1"},
{1, 0.0784314,  .576471,    "deep pink"},
{1, 1,  .878431,    "lightyellow"},
{1, 1,  .878431,    "lightyellow1"},
{1, 1,  .878431,    "light yellow"},
{1, 1,  .941176,    "ivory"},
{1, 1,  .941176,    "ivory1"},
{1, 1,  0,  "yellow"},
{1, 1,  0,  "yellow1"},
{1, 1,  1,  "gray100"},
{1, 1,  1,  "grey100"},
{1, 1,  1,  "white"},
{1, 1, 1,   "background"},
};

int set_color(string, red, green, blue)
     unsigned char *string;
     float *red, *green, *blue;
{
int i;
int done_flag=0;
int nitems = sizeof(grayscale_table)/sizeof(struct color_grayscale);

/* map upper case characters in string to lower case */
for(i=0; i<strlen(string); i++)
    if (isascii(string[i]) && isupper(string[i]))
        string[i] = tolower(string[i]);

/* see if string matches any x color strings */
for(i=0; (i< nitems) && (done_flag <= 0); i++)
    {
    if (strcmp( string, grayscale_table[i].name) == 0)
        {
        done_flag++;
        *red = grayscale_table[i].red_value;
        *green = grayscale_table[i].green_value;
        *blue = grayscale_table[i].blue_value;
        }
    }
if( done_flag <= 0)
    fprintf(stderr, "WARNING plot2ps: unknown color = %s\n", string);

return(done_flag);
}
