/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

/*
 * Copyright 1989,1990,1991 by Metis Technology, Inc.
 *
 *                        All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software and its 
 * documentation for any purpose and without fee is hereby granted, 
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in 
 * supporting documentation.
 *
 * METIS TECHNOLOGY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL METIS TECHNOLOGY BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
 * USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 */



/* for malloc */
#if HAVE_CONFIG_H
#include <config.h>
#endif
/* for NULL */
#include <stdio.h>
#if HAVE_MALLOC_H
#include <malloc.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
/* for memset */
#include <string.h>
#endif
#include <sys/types.h>
/* for timeout in select */
#if TIME_WITH_SYS_TIME
#include <time.h>
#endif
#if HAVE_SYS_TIME_H
#include <sys/time.h>
#endif
#include <sys/socket.h>
#include <sys/un.h>
#if HAVE_UNISTD_H
/* for fork */
#include <unistd.h>
#endif

#if !HAVE_WORKING_VFORK
# define vfork fork
#endif

#include "PlotI.h"

#define XPLOTLINKNAME   "/tmp/xplotXXXXXX"

#define XPLOTCMD    "lnkplot"

#ifndef XPLOTTIMEOUT
#define XPLOTTIMEOUT    30
#endif

typedef struct _PlotLinkRec PlotLinkRec, *PlotLink;
struct _PlotLinkRec {
  int   flags;      /* link flags */
  char *linkname;   /* link socket pathname */
  int   socket;     /* socket to xplot instance */
  int   child;      /* xplot child process id */
};

#define PLOpen  00001
#define PLError 00002


static PlotLinkRec  _PlotLink;
int Plot ( register PlotCmd *pc );

static char *
  newstr ( str )
char *str;
{
  char *s;
  if ( ! str ) return (char *) NULL;
  if ( ! ( s = malloc ( strlen ( str ) + 1 ) ) ) return (char *) NULL;
  return strcpy ( s, str );
}

/*
 * Create xplot instance and establish IPC.
 */

int
PlotOpen ( register char **argv )
{
  register PlotLink pl = & _PlotLink;
  char pathbuf [ sizeof (XPLOTLINKNAME) + 1 ];
  struct sockaddr_un saddr;
  int s, ns, pid, saddrlen = sizeof (struct sockaddr_un);
  struct timeval timeout;
  fd_set readfds;

  if ( pl->flags & PLOpen ) {
    fprintf ( stderr, "PlotOpen: link already open\n" );
    return -1;
  }

  saddr.sun_family = AF_UNIX;
  if ( ! mktemp ( strcpy ( pathbuf, XPLOTLINKNAME ) ) ) {
    fprintf ( stderr, "PlotOpen: can't create unique plot link name\n" );
    return -1;
  }
  strcpy ( saddr.sun_path, pathbuf );

  if ( ( s = socket ( PF_UNIX, SOCK_STREAM, 0 ) ) < 0 ) {
    perror ( "PlotOpen: can't create socket" );
    return -1;
  }

  if ( bind ( s, (struct sockaddr *) &saddr,
         sizeof ( saddr.sun_family ) + strlen ( saddr.sun_path ) ) < 0 ) {
    fprintf( stderr, "PlotOpen: can't bind socket to path \"%s\": ", pathbuf );
    perror ( "" );
    goto err;
  }

  if ( listen ( s, 1 ) < 0 ) {
    perror ( "PlotOpen: can't listen on socket" );
    goto err;
  }

  switch ( pid = vfork () ) {
  case -1:
    perror ( "PlotOpen: fork failed" );
    goto err;
  case 0:
    {
      char **args;
      register char **av;
      register int argc;

      for ( av = argv, argc = 0; av && *av; av++ )
    argc++;
      av = args = (char **) malloc ( ( argc + 4 ) * sizeof (char *) );
      if ( ! av ) {
    fprintf ( stderr, "PlotOpen: can't allocate arg vector\n" );
    return -1;
      }
      *av++ = XPLOTCMD;
      *av++ = "-linkname";
      *av++ = pathbuf;
      while ( argv && *argv )
    *av++ = *argv++;
      *av++ = (char *) NULL;

      execvp ( XPLOTCMD, args );
      perror ( "PlotOpen: exec failed" );
      exit ( 1 );
    }
  default:
    break;
  }

  timeout.tv_sec  = XPLOTTIMEOUT;
  timeout.tv_usec = 0;
  FD_ZERO ( &readfds );
  FD_SET  ( s, &readfds );
  switch ( select ( 32, &readfds, (fd_set *) 0, (fd_set *) 0, &timeout ) ) {
  case -1:
    perror ( "PlotOpen: select error" );
    goto err;
  case 0:
    fprintf ( stderr,
      "PlotOpen: xplot not responding: timed out during accept\n" );
    goto err;
  }

  if ( ( ns = accept ( s, (struct sockaddr *)&saddr, &saddrlen ) ) < 0 ) {
    perror ( "PlotOpen: accept failed" );
    goto err;
  }
  (void) close ( s );

  pl->linkname = newstr ( pathbuf );
  pl->socket   = ns;
  pl->child    = pid;
  pl->flags    = PLOpen;

  return 0;

 err:
  (void) close ( s );
  (void) unlink ( pathbuf );
  return -1;
}

/*
 * Shutdown xplot and IPC link.
 */

void
PlotClose ( int terminate )
{
  register PlotLink pl = & _PlotLink;
  PlotCmd plotbuf;
  register PlotCmd *pc = &plotbuf;
  struct timeval timeout;
  fd_set readfds;

  /* initalize plotbuf, to make purify happy */
  memset(&plotbuf, 0, sizeof(plotbuf));

  if ( ! ( pl->flags & PLOpen ) )
    return;

  if ( terminate )
    pc->command = PLOT_LINK_TERMINATE;
  else
    pc->command = PLOT_LINK_DISCONNECT;
  (void) Plot ( pc );

  (void) shutdown ( pl->socket, 1 );
  while ( 1 ) {
    timeout.tv_sec  = XPLOTTIMEOUT;
    timeout.tv_usec = 0;
    FD_ZERO ( &readfds );
    FD_SET  ( pl->socket, &readfds );
    switch ( select ( 32, &readfds, (fd_set *) 0, (fd_set *) 0, &timeout ) ) {
    case 0:
    case -1:
      break;
    default:
      if ( read ( pl->socket, (char *) pc, sizeof (*pc) ) > 0 )
    if ( pc->command != PLOT_LINK_DISCONNECT )
      continue;
      break;
    }
    break;
  }

  (void) close ( pl->socket );
  pl->flags = 0;
}

/*
 * Transmit plot command to xplot.
 */

int
Plot ( register PlotCmd *pc )
{
  register PlotLink pl = & _PlotLink;
  int cc;
  char *str;
  char strbuf [ 256 ];

  if ( ! pc || ! ( pl->flags & PLOpen ) || ( pl->flags & PLError ) )
    return -1;
  if ( ( cc = write ( pl->socket, (char *) pc, sizeof (*pc) ) ) < 0 ) {
    perror ( "Plot: link write error" );
    pl->flags |= PLError;
    return -1;
  }
  if ( cc != sizeof (*pc) ) {
    fprintf ( stderr, "Plot: link write out of sync\n" );
    pl->flags |= PLError;
    return -1;
  }
  switch ( pc->command ) {
  case PLOT_LABEL:
    str = pc->args.label.label;
    break;
  case PLOT_EXT_COLOR:
    str = pc->args.color.color;
    break;
  case PLOT_EXT_FONT:
    str = pc->args.font.font;
    break;
  default:
    return 0;
  }
  if ( ! str )
    strbuf [ 0 ] = 0;
  else {
    register int slen;
    if ( ( slen = strlen ( str ) ) > ( sizeof ( strbuf ) - 2 ) ) {
      switch(pc->command){
      case PLOT_LABEL:
    fprintf ( stderr, "Plot: label too long\n" );
    break;
      case PLOT_EXT_COLOR:
    fprintf( stderr, "Plot: color too long\n");
    break;
      case PLOT_EXT_FONT:
    fprintf(stderr, "Plot: font too long\n");
    break;
      }
      return -1;
    }
    strbuf [ 0 ] = strlen ( str );
  }
  strcpy ( & strbuf [ 1 ], str );
  if ( ( cc = write ( pl->socket, strbuf, strbuf [ 0 ] + 1 ) ) < 0 ) {
    perror ( "Plot: link write error" );
    pl->flags |= PLError;
    return -1;
  }
  if ( cc != ( strbuf [ 0 ] + 1 ) ) {
    fprintf ( stderr, "Plot: link write out of sync\n" );
    pl->flags |= PLError;
    return -1;
  }
  return 0;
}

/*
 * Synchronously read xplot pointer, i.e., wait for button press.
 */

int
PlotPointerWait ( int *x_ret, int *y_ret, int *but_ret, int *mod_ret,
          struct timeval timeout )
{
  register PlotLink pl = & _PlotLink;
  PlotCmd plotbuf;
  register PlotCmd *pc = &plotbuf;
  int cc;
  time_t now;
  fd_set readfds;

  /* initalize plotbuf, to make purify happy */
  memset(&plotbuf, 0, sizeof(plotbuf));

  if ( ! ( pl->flags & PLOpen ) || ( pl->flags & PLError ) )
    return -1;
  pc->command = PLOT_LINK_POINTERWAIT;
  time ( &now );
  pc->args.timestamp.timestamp = now;
  if ( ( cc = write ( pl->socket, (char *) pc, sizeof (*pc) ) ) < 0 ) {
    perror ( "PointerWait: link write error" );
    pl->flags |= PLError;
    return -1;
  }
  if ( cc != sizeof (*pc) ) {
    fprintf ( stderr, "PointerWait: link write out of sync\n" );
    pl->flags |= PLError;
    return -1;
  }

  while ( 1 ) {
    FD_ZERO ( &readfds );
    FD_SET  ( pl->socket, &readfds );
    switch ( select ( 32, &readfds, (fd_set *) 0, (fd_set *) 0, &timeout ) ) {
    case 0:
      return 0;
    case -1:
      perror ( "PointerWait: select error" );
      goto err;
    default:
      if ( ( cc = read ( pl->socket, (char *) pc, sizeof (*pc) ) ) < 0 ) {
    perror ( "PointerWait: read error" );
    goto err;
      }
      if ( cc != sizeof (*pc) ) {
    fprintf ( stderr, "PointerWait: link read out of sync\n" );
    goto err;
      }
      if ( ( pc->command != PLOT_LINK_POINTERWAIT ) ||
       ( pc->args.pointer.timestamp < now ) )
    continue;
      break;
    }
    break;
  }
  
  if ( x_ret )
    *x_ret = pc->args.pointer.x;
  if ( y_ret )
    *y_ret = pc->args.pointer.y;
  if ( but_ret )
    *but_ret = pc->args.pointer.buttons;
  if ( mod_ret )
    *mod_ret = pc->args.pointer.modifiers;
  return 1;
 err:
  pl->flags |= PLError;
  return -1;
}

/*
 * Asynchronously read xplot pointer, i.e., don't wait for button press.
 */

int
PlotPointerRead ( int *x_ret, int *y_ret, int *mod_ret, 
              struct timeval timeout )
{
  register PlotLink pl = & _PlotLink;
  PlotCmd plotbuf;
  register PlotCmd *pc = &plotbuf;
  int cc;
  time_t now;
  fd_set readfds;

  /* initalize plotbuf, to make purify happy */
  memset(&plotbuf, 0, sizeof(plotbuf));

  if ( ! ( pl->flags & PLOpen ) || ( pl->flags & PLError ) )
    return -1;
  pc->command = PLOT_LINK_POINTERREAD;
  time ( &now );
  pc->args.timestamp.timestamp = now;
  if ( ( cc = write ( pl->socket, (char *) pc, sizeof (*pc) ) ) < 0 ) {
    perror ( "PointerRead: link write error" );
    pl->flags |= PLError;
    return -1;
  }
  if ( cc != sizeof (*pc) ) {
    fprintf ( stderr, "PointerRead: link write out of sync\n" );
    pl->flags |= PLError;
    return -1;
  }

  while ( 1 ) {
    FD_ZERO ( &readfds );
    FD_SET  ( pl->socket, &readfds );
    switch ( select ( 32, &readfds, (fd_set *) 0, (fd_set *) 0, &timeout ) ) {
    case 0:
      return 0;
    case -1:
      perror ( "PointerRead: select error" );
      goto err;
    default:
      if ( ( cc = read ( pl->socket, (char *) pc, sizeof (*pc) ) ) < 0 ) {
    perror ( "PointerRead: read error" );
    goto err;
      }
      if ( cc != sizeof (*pc) ) {
    fprintf ( stderr, "PointerRead: link read out of sync\n" );
    goto err;
      }
      if ( ( pc->command != PLOT_LINK_POINTERREAD ) ||
       ( pc->args.pointer.timestamp < now ) )
    continue;
      break;
    }
    break;
  }
  
  if ( x_ret )
    *x_ret = pc->args.pointer.x;
  if ( y_ret )
    *y_ret = pc->args.pointer.y;
  if ( mod_ret )
    *mod_ret = pc->args.pointer.modifiers;
  return 1;
 err:
  pl->flags |= PLError;
  return -1;
}

/*
 * Set Plot Title
 */

int
PlotSetTitle ( char *title )
{
  register PlotLink pl = & _PlotLink;
  PlotCmd plotbuf;
  register PlotCmd *pc = &plotbuf;
  int cc;
  char strbuf [ 256 ];

  /* initalize plotbuf, to make purify happy */
  memset(&plotbuf, 0, sizeof(plotbuf));

  if ( ! ( pl->flags & PLOpen ) || ( pl->flags & PLError ) )
    return -1;
  pc->command = PLOT_LINK_SETTITLE;
#if HAVE_SOCKET
  if ( ( cc = write ( pl->socket, (char *) pc, sizeof (*pc) ) ) < 0 ) {
    perror ( "PlotTitle: link write error" );
    pl->flags |= PLError;
    return -1;
  }
#else
  if ( ( cc = write ( pl->socket, (char *) pc, sizeof (*pc) ) ) < 0 ) {
    perror ( "PlotTitle: link write error" );
    pl->flags |= PLError;
    return -1;
  }
#endif
  if ( cc != sizeof (*pc) ) {
    fprintf ( stderr, "PlotTitle: link write out of sync\n" );
    pl->flags |= PLError;
    return -1;
  }

  if ( ! title )
    strbuf [ 0 ] = 0;
  else {
    register int slen;
    if ( ( slen = strlen ( title ) ) > ( sizeof ( strbuf ) - 2 ) ) {
      fprintf ( stderr, "PlotTitle: title too long\n" );
      return -1;
    }
    strbuf [ 0 ] = slen;
  }
  strcpy ( & strbuf [ 1 ], title );
#if HAVE_SOCKET
  if ( ( cc = write ( pl->socket, strbuf, strbuf [ 0 ] + 1 ) ) < 0 ) {
    perror ( "PlotTitle: link write error" );
    pl->flags |= PLError;
    return -1;
  }
#else
  if ( ( cc = write ( pl->socket, strbuf, strbuf [ 0 ] + 1 ) ) < 0 ) {
    perror ( "PlotTitle: link write error" );
    pl->flags |= PLError;
    return -1;
  }
#endif
  if ( cc != ( strbuf [ 0 ] + 1 ) ) {
    fprintf ( stderr, "PlotTitle: link write out of sync\n" );
    pl->flags |= PLError;
    return -1;
  }

  return 0;
}
