/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
# include <stdio.h>
# include <ctype.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif

# define MAXLINE    80  /* max num of chars on a line*/
# define MAX_TRIES    5     /* maximum number of tries before error */
# define DEFAULT    '\0'    /* Carriage Return          */
# define DIGITS      0      /* String with digits only      */
# define MIXED        1     /* Mixed string             */
int g_int_token(char *);

/***---------------------------------------------------------------------------
* NAME:     aski
* PURPOSE:  friendly user interface to get an integer from a terminal
* USAGE: 
*   aski(&x,prompt,def,min,max);
*   int x;              - value returned
*   char    *prompt;    - prompting string
*   int def;            - default value
*   int min;            - minimum acceptable value
*   int max;            - maximum acceptable value
*
* RETURN VALUE:
*   Zero if ok
*   -1 if no acceptable number is typed in after MAXTRIES tries
*   -2 if parameters are bad
*
* EXAMPLE:
*   int x;
*   int i;
*   i = aski(&x, "How old are you ? ", 25, 1, 100);
*
* SEE ALSO:
*   asks(), g_int_token()
*
* DESCRIPTION:
*   Prompts for a number using the prompting string, then accepts
*   valid number, substitutes default value if CR hit, and prints
*   acceptable range and default value if ? or invalid number typed
*   in. User gets MAXTRIES tries at entering a correct number before
*   the function returns with an error code.
*   Allows a beginning + or - but no expressions.
*
* BUGS AND LIMITATIONS:
*   Doesn't know about intelligent terminals thus adds a CR LF
*   after a lone CR, then prints default. 
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/aski.c,v 11.6 2001/11/29 18:57:58 kukolich Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/aski.c,v $
* CREATED: 12-Jun-80 rpl    LAST UPDATE: 12-Jun-84 rpl  MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
int
aski(x,prompt,def,min,max)
int *x;         /* variable returned            */
char *prompt;           /* prompt message           */
int def;            /* default value            */
int min;            /* minimum allowable value      */
int max;            /* maximum allowable value      */
{
    int count;      /* number of tries          */
    char dummy[MAXLINE];    /* entered string           */
    int y;          /* entered number           */
    int sbad;       /* string not an integer number     */
    int i;

    /* check for errors in parameters */
    if ( max < min )
        {
        fprintf(stderr, "aski: max greater than min; max=%d, min=%d\n",max,min);
        return(-2);
        }
    
    /* allow  MAX_TRIES tries at an input number        */
    for (count=1;count<=MAX_TRIES;count++)
        {
        /* print prompt*/
        printf("%s ",prompt);

        /* get typed in string and set flag if any non digits */
        sbad = g_int_token(dummy);

        /* default value    */
        if (dummy[0] == DEFAULT)
            {
            *x=def;
            for(i=1;i<=strlen(prompt)-10;i++)
                printf(" ");
            printf("default =  %d\n",def);
            break;
            }
        /* valid number*/
        else if ( sbad==0 && (y=atoi(dummy))>=min && y<=max ) 
            {
            *x = y;
            break;
            }
        /* bad number, or ? , or mixed char string  */
        else
           printf("---------------------------------\n");
           printf("Minimum = %d     Maximum = %d\nDefault (CR) = %d\n",
                    min,max,def);
           printf("---------------------------------\n");
        }

    if ( count > MAX_TRIES )
        return(-1);
    else
        return(0);
}

/***---------------------------------------------------------------------------
* NAME:     g_int_token 
* PURPOSE:  get a token from the standard input and set a flag if
*       the token is an acceptable integer string (+ or - followed
*       by digits)
* USAGE: 
*   g_int_token(string)
*       char    *string;        - prompt to put token into
* 
* RETURN VALUE:
*   Zero if token is acceptable integer, -1 otherwise
*
* SEE ALSO: ctype(3),aski,atoi(3)
* 
* BUGS AND LIMITATIONS:
*   CR character must be '\n' and adds '\0' at end of returned string
* 
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/aski.c,v 11.6 2001/11/29 18:57:58 kukolich Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/aski.c,v $
* CREATED: 12-Jun-80 rpl    LAST UPDATE: 26-Apr-83 MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
int
g_int_token(string)
char string[];
{
    int stype;      /* token type       */
    unsigned char    c;      /* single char      */
    int i;

    /* get typed in string and set flag if not acceptable integer */
    stype = DIGITS;
    for(i=0;(c=string[i]=getchar())!='\n' && i < MAXLINE;i++)
        if ( stype==DIGITS && isdigit(c)==0 )
            {
            if (i==0 && ( c=='+' || c=='-'))
                ;
            else
                stype = MIXED;
            }
    if (i >= MAXLINE) i = MAXLINE-1;
    /* turn into a string       */
    string[i] = '\0'; 
    /* return           */
    if ( stype == DIGITS )
        return(0);
    else
        return(-1);
}
