/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#define MAXLINE	80		/* max num of chars on a line*/
#define NO 		0
#define YES		1
#define MAX_TRIES    5		/* maximum number of tries before error */
#define DEFAULT    012		/* Carriage Return			*/

/***---------------------------------------------------------------------------
* NAME:		asks
* PURPOSE:	friendly user interface to get character string from a terminal
* USAGE: 
* 	asks(x, prompt, default, ntoken, pptoken)
*	char	x[80];		- char array to contain return string 
* 	char	*prompt;	- prompting string
*	char	*default;    - default string
*	int		ntoken;		- number of acceptable strings in pptoken
*	char	*pptoken[];	- array of acceptable strings
*
* RETURN VALUE:
*   0 to ntoken = index into *ptoken for typed in string 
*                 (0=*ptoken[0], 1=*ptoken[1], ... 
*	-1 if no acceptable string typed in
*	-2 if input parameters are wrong
*
* EXAMPLE:
*	int i;
*	char	x[80];		
*	static char *pyn[] = 	{ "yes", "y", "no", "n", "quit", "q" };
*	    i = asks(x, "Do you want to go to lunch ? ", pyn[0], 6, pyn);
* SEE ALSO:
*	aski(),askf()
*
* DESCRIPTON:
*	Prompts for a string using prompting string, then accepts valid
*	string, substitutes default value if CR hit, and prints acceptable
*	values and default if incorrect string or ? typed.
*   Default must be contained in *ptoken[] list.
*	User gets MAXTRIES tries at entering correct string before
*	function returns with error code.
*	Entered strings can be up to MAXLINE chars long.
*	The string "*" in *ptoken[] is a wild card that matches any string.
*	If "*" is in *ptoken[] then ? still provides information.
*
* BUGS AND LIMITATIONS:
* 	Doesn't know about intelligent terminals thus adds a CR LF
*	after a lone CR and then prints default.
*	If *ptoken[] contains "*", then an isolated question
*	mark can't be entered.
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/asks.c,v 11.5 2001/08/23 20:15:05 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/asks.c,v $
* CREATED: 25-Apr-83 rpl    LAST UPDATE: 12-Jun-84 rpl  MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
int
asks(px, prompt, pdefault, ntoken, ptoken)
char	*px;		/* string returned */
char	*prompt;	/* prompting string */
char 	*pdefault;	/* default string */
int		ntoken;		/* number of acceptable strings	*/
char	*ptoken[];	/* array of acceptable strings */
{
	int count;		/* number of tries			*/
	char dummy[MAXLINE];	/* entered string*/
	int i;
	int found;	/* flag when searching tokens */
	int tindex=0;	/* index into ptoken that string matches */
	int ndefault=0; /* index into ptoken for default string */
	int any_flag=0;	/* "*" is present in ptoken */
	int any_index=0;	/* location of "*" in ptoken */

	/* check for wildcard */
	for(i=0; i< ntoken; i++)
	if (strcmp("*", ptoken[i])==0)
		{
		any_flag++;
		any_index=i;
		break;
		}

	/* check for errors in parameters */
	if ( ntoken < 0 )
		{
	fprintf(stderr,"asks: number of strings in ntoken=%d invalid\n",ntoken);
		return(-2);
		}
	
	/* is default in ptoken list */
	found = 0;
	for(i=0; i< ntoken; i++)
		if (strcmp( pdefault, ptoken[i]) == 0 )
			{
			found++;
			ndefault=i;
			}

	if ( found == 0 && any_flag == 0)
		{
	fprintf(stderr,
		"asks: default string=%s not in list of acceptable strings\n",pdefault);
		return(-2);
		}
		
	
	/* allow  MAX_TRIES tries at an input number	 	*/
	for (count=1;count<=MAX_TRIES;count++)
		{
		/* print prompt*/
		printf("%s ",prompt);

	       /* get typed in string */
	       for(i=0;(dummy[i]=getchar())!='\n' && i < MAXLINE ;i++);
	       if (i >= MAXLINE) i = MAXLINE-1;

	       /* turn into a string		*/
	       dummy[i] = 0; 

		/*
	       printf("string = %s\n",dummy);
	       printf("first char = %c  %o  %d\n",*dummy,*dummy,*dummy);
	       */

		/* default value	*/
		if (dummy[0] == 0)
			{
			strcpy(px,pdefault);
			for(i=1;i<=strlen(prompt)-10;i++)
				printf(" ");
			printf("default =  %s\n",pdefault);
			tindex = ndefault;
			break;
			}

		/* valid string equal to a token */
		found = NO;
		if ( strcmp("?",dummy) != 0 )
			{
			/* search through ptoken list */
		    for( i=0; i<ntoken; i++)
				if ( strcmp(dummy,ptoken[i]) == 0 )
					{
					found = YES;
					tindex = i;
					break;
					}
			/* wildcard matches anything */
			if (found == NO && any_flag != 0)
				{
				found = YES;
				tindex = any_index;
				}
			if ( found == YES)
				{
				strcpy(px,dummy);
				break;
				}
			}
		

		/* bad string, or ? */
		printf("-----------------\nAcceptable tokens\n");
		for( i=0; i<=ntoken-1; i++)
		    printf("  %s\n",ptoken[i]);

		printf("Default (CR)\n  %s\n-----------------\n", 
							pdefault);

		}

	if ( count > MAX_TRIES )
		return(-1);
	else
		return(tindex);
}
