/***---------------------------------------------------------------------------
* NAME:     errors.c 
* PURPOSE:  error reporting and debugging utilities 
* DESCRIPTION:
* lippic_fatal_error    - print error message using printf format and terminate 
* lippic_warning    - print warning message using printf format 
* 
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/errors.c,v 11.6 2004/01/15 19:28:35 kukolich Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/errors.c,v $
* CREATED: 26-July-90 rpl                             MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */



/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <math.h>
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#endif
#include <xlippic.h>

/***---------------------------------------------------------------------------
* NAME:     lippic_fatal_error 
* PURPOSE:  print error message using printf format and terminate 
* EXAMPLE:
*   lippic_fatal_error("nnargs: Can't open file %s after %d tries", filename, n);
*
* NOTE:     Should initialize with init_nnsubs, adds final carriage return
--------------------------------------------------------------------------***/
void lippic_fatal_error (char *fmt, ...)
    {
    va_list args;

    gexit(1);

    fprintf(stderr,"\nERROR: \n----------------\n");

    /* print error message using variable number of arguments */
    va_start(args, fmt);
    vfprintf(stderr, fmt, args);
    va_end(args);
    fprintf(stderr, "\n");

    exit(-1);
    }

/***---------------------------------------------------------------------------
* NAME:     lippic_warning 
* PURPOSE:  print warning message using printf format 
* EXAMPLE:
*           lippic_warning("Could not open file %s after %d tries", filename, n);
*
* NOTE:     Should be initialized with init_nnsubs, adds final carriage return
--------------------------------------------------------------------------***/
void lippic_warning (char *fmt, ...)
    {
    va_list args;


    fprintf(stderr,"\nWARNING: \n----------------\n");

    /* print warning message using variable number of arguments */
    va_start(args, fmt);
    vfprintf(stderr, fmt, args);
    va_end(args);
    fprintf(stderr, "\n");

    }

