/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/*-----------------------------------------------------------------------------
* NAME:     ghardcopy 
* PURPOSE:  turn hardcopy feature on and off
* USAGE: 
*   ghardcopy(flag) 
*       int flag   -  0 = turn hardcopy feature off  
*                     1 = turn hardcopy feature on 
* DESCRIPTION:
*           When turned on, drawing a vector, moving to a new location, or
*       printing a string on the graphics screen causes a character string
*       containing the UNIX standard graphics commands (see plot(5)) to be
*       sent to an internal buffer.  When gsave() is called, this internal 
*       buffer is written out to a file called by default
*       frame1.plot, frame2.plot, ..
*       or any name a user desires. When gversatec() is
*       called a gsave() call is issued, the resulting file is
*       piped through vplot to the versatec plotter/printer, and 
*       the resulting file is erased. 
*       Things are really a little
*       more complicated because parts of a plot file are written out to 
*       a temporary file called /usr/tmp/lipplotXXXXXX
*   if the internal hardcopy buffer
*   overflows. The hardcopy buffer is cleared but not written out
*   on every gclear() call, on a ghardcopy(1) call, and on a hc_reset() 
*   call.
* EXAMPLE:
*   char *gsave();
*
*       ginit("");
*       ghardcopy(1);
*        ........
*        printf("Created file %s\n",gsave(0,0));
*        .......
*        gversatec(1);
*    gexit(0);
*
* SEE ALSO: gsave(), gversatec(), plot(5)
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/g_hardcopy.c,v 11.10 2001/08/27 19:15:32 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/g_hardcopy.c,v $
* CREATED: 15-Mar-84 rpl LAST UPDATE: 27-Jun-84 rpl MIT LINCOLN LABORATORY
*---------------------------------------------------------------------------*/
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
/* for lseek */
#include <sys/types.h>
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
/* system V include for creat */
#include <sys/stat.h>
#include "g_gvt.h"
/* for open */
#if HAVE_FCNTL_H
#include <fcntl.h>
#endif
#include <errno.h>
#include "xlippic.h"

/* in line macros for efficiency */
#define hc_putc(cc)         /* char cc */\
{   if(GG_.hardcopy==1)\
    {   if ( GGhc_count >= HC_BUFFER)\
        {   write(fildes, GGhc_buff, HC_BUFFER);\
            GGhc_count = 0;\
        }\
        GGhc_buff[GGhc_count] = (cc);\
        ++GGhc_count;\
    }\
}

/* put an integer into the plot buffer */
#define hc_putint(i)        /* int i; integer to put into buffer */\
{   hc_putc((char)(i));\
    hc_putc((char)((i)>>8));\
}

extern char GGhc_buff[];
extern int GGhc_count;
static int hc_file_count = 0;       /* number of files created so far */
static char temp_plotname[] = "/usr/tmp/lipplotXXXXXX";
static int fildes;          /* file descriptor */
static int temp_open = 0;       /* indicates temporary file is open */
static int temp_exists= 0;      /* indicates temporary file exists */
static char fname[512];         /* file name */
static char command[512];       /* command line for shell */
static char *linetexture[] = {
              "solid",
              "dotted",
              "dotdashed",
              "shortdashed",
              "longdashed"
              };

void
ghardcopy(flag)
int flag;
{
  int s1, s2, s3, s4;
  char buf[64];
  int ReadFlags;
  mode_t ModeFlags;

  if (flag !=0) {
    if (temp_open !=0)
      close(fildes);

    /* create new temp file or set current to zero length 
       ---------------------------------------------------------*/
    /* this business with the flags shouldn't be necessary but it
     doesn't work without them and does work with them */
    ReadFlags =  O_RDWR | O_CREAT | O_EXCL;
    ModeFlags = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH;
    strcpy(temp_plotname, "/usr/tmp/lipplotXXXXXX"); 
    if(!mktemp(temp_plotname))
      {
        perror("mktemp failed in /usr/tmp");
        lippic_fatal_error("ghardcopy: can't create file %s", temp_plotname);
      }
    if ((fildes = open ( temp_plotname ,ReadFlags, ModeFlags)) == -1){
      perror("can't create file in /usr/tmp");
      lippic_fatal_error("ghardcopy: can't create file %s", temp_plotname);
    }
#ifdef notdef
    /* this broke when we switched to solaris */
    if ( (fildes = mkstemp ( temp_plotname )) == -1)
      lippic_fatal_error("ghardcopy: can't create file %s", temp_plotname);
#endif

    /* initialize all flags */
    temp_open = 1;
    temp_exists = 1;
    GG_.hardcopy = 1;
    GGhc_count = 0;

    /* put header info in buffer */
    s1 = -GG_.uoffset/GG_.ufract;
    s2 = -GG_.voffset/GG_.vfract;
    s3 = (G_XMAX-GG_.uoffset)/GG_.ufract;
    s4 = (G_YMAX-GG_.voffset)/GG_.vfract;
    hc_space( s1, s2, s3, s4);

    /* removed for compatibility with implot and verplot 18 Aug 1984 rpl
       because it produces a blank page.
       hc_erase();
       */
  } /* turn off */ else {
    /* get rid of temporary file */
    if ( temp_open != 0 )
      close(fildes);
    if ( temp_exists != 0){
      strcpy(buf, "rm -f \'");
      strcat(buf, temp_plotname);
      strcat(buf, "\'");
      system(buf);
    }

    /* zero flags */
    temp_open = 0;
    temp_exists = 0;
    GG_.hardcopy = 0;
  }
}

/***---------------------------------------------------------------------------
* NAME:     gimagen
* PURPOSE:  create hardcopy of current screen on imagen laser printer/plotter
* USAGE: 
*   gimagen(verbose)
*       int verbose    - 0=silent, 1=report creation of hardcopy
* DESCRIPTION:
*   Creates a plot file using gsave(), spawns a shell command that
*   processes the plot file using implot, waits for child to complete
*   then returns. File created remains after call is completed.
* SEE ALSO: gsave(), ghardcopy(), gversatec(),plot(5)
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/g_hardcopy.c,v 11.10 2001/08/27 19:15:32 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/g_hardcopy.c,v $
* CREATED: 27-Jun-84 rpl    LAST UPDATE: 27-Apr-83 rpl  MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
void
gimagen(verbose)
     int verbose;       /* 0=silent, 1=report creation of hardcopy */
{
  char  *p;

  /* create plot file and get file name */
  p = gsave(NULL, 0);

  /* report that file is being plotted */
  if (verbose != 0) {
    printf("Plotting file %s on the Imagen . . . .",p);
  }

  /* create command line and issue using system call */
  sprintf(command, "lipiplot %s", p);
  if (system(command) == 127) {
    gexit(1);
    lippic_fatal_error("gimagen: couldn't issue shell command:\n%s",command);
  }

  /* done creating file */
  if (verbose != 0) {
    printf(" . . . done!\n");
  }
}

/***---------------------------------------------------------------------------
* NAME:     gsave 
* PURPOSE:  save current screen in a plot file 
* USAGE: 
*   char *gsave(pfname, verbose)
*       char *pfname  - 0=automatically create a unique file name 
*                          of the form frame1.plot, frame2.plot, .....
*                          to store plot file.
*                        Valid string = use the passed file name
*       int verbose    - 0=silent, 1=report file name and creation 
* DESCRIPTION:
*   Creates file with Slightly modified standard Unix plot(5) commands
*   that can be  used to create a plot using verplot, implot, or gvtplot.
*   Returns a pointer to the filename.
* SEE ALSO: ghardcopy(), gversatec(), gimagen(), plot(5)
* 
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/g_hardcopy.c,v 11.10 2001/08/27 19:15:32 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/g_hardcopy.c,v $
* CREATED: 27-Jun-84 rpl    LAST UPDATE: 27-Apr-83 rpl  MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
char *
gsave(name, verbose)
     char name[];       /* file name or null */
     int verbose;       /* 0=silent, 1=report file name and creation */
{
  int fdtemp;       /* temporary file descriptor */
  int ReadFlags, ModeFlags;

  if ( GG_.hardcopy == 0 || temp_open == 0 ) {
    gexit(1);
    lippic_fatal_error("gsave: can't write hardcopy file if hardcopy flag isn't on");
  }

  /* put a bunch of last commands on file to guarantee last vector
     is plotted */
  hc_move( 0,0);
  hc_move( 0,0);
  hc_texture(0);


  /* create a unique file name */
  if ( name == 0 ) {
    /* find first unused file name of type framen.plot */
    for(;;) {
      sprintf(fname, "frame%d.plot", ++hc_file_count);
      if ((fdtemp = open(fname, O_RDONLY)) == -1 )
    break;
      close(fdtemp);
    }
  }
  /* complete name is passed */
  else
    strcpy(fname, name);

  /* empty hardcopy buffer into temp file and close temp file */
  if ( write(fildes, GGhc_buff, GGhc_count) < 0 ) {
    gexit(1);
    lippic_fatal_error("gsave: can't write to file %s", temp_plotname);
  }
  if ( close(fildes) < 0 ) {
    gexit(1);
    lippic_fatal_error("gsave: can't close file %s", temp_plotname);
  }
  temp_open = 0;
  GGhc_count = 0;

  /* copy contents of temp to file with desired name */
  sprintf(command, "cp %s %s\n", temp_plotname, fname);
  if (system(command) == 127) {
    gexit(1);
    lippic_fatal_error("gsave: couldn't issue shell command:\n%s",command);
  }

  /* report file name */
  if (verbose != 0) {
    printf("Created file %s\n",fname);
  }

  /* reopen temp file and point to end */
  ReadFlags =  O_RDWR;
  ModeFlags = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH;

  if ( (fildes = open(temp_plotname, ReadFlags, ModeFlags)) < 0 ) {
    gexit(1);
    lippic_fatal_error("gsave: can't re-open file %s for writing",
               temp_plotname);
  }

  /* move write pointer to end of temporary file */
  if (lseek(fildes, 0L, 2) < 0L) {
    gexit(1);
    lippic_fatal_error("gsave: can't move pointer to end of file %s",
               temp_plotname);
  }

  /* initialize flags */
  temp_open = 1;
  temp_exists = 1;
  GGhc_count = 0;

  /* return pointer to name*/
  return(fname);
}

/* erase current frame and start another one */
void
hc_reset()
{
  ghardcopy(1);
}

#if 0
hc_putc(cc)
register char cc;
{
if ( GG_.hardcopy != 1)
    return;

if ( GGhc_count >= HC_BUFFER)
    {
    write(fildes, GGhc_buff, HC_BUFFER);
    GGhc_count = 0;
    }

hc_buff[GGhc_count] = cc;
++GGhc_count;
}
#endif

#if 0
/* put an integer into the plot buffer */
void
hc_putint(i)
int i;      /* integer to put into buffer */
{
    hc_putc((char)i);
    hc_putc((char)(i>>8));
}
#endif

void
hc_space(x0,y0,x1,y1)
int x0, y0, x1, y1; /* space for plot */
{
    hc_putc('s');
    hc_putint(x0);
    hc_putint(y0);
    hc_putint(x1);
    hc_putint(y1);
}
void
hc_erase()
{   
    hc_putc('e');
}
void
hc_move(xi,yi)
register int xi, yi;
{
    hc_putc('m');
    hc_putint(xi);
    hc_putint(yi);
}
void
hc_point(xi,yi)
register int xi, yi;
{
    hc_putc('p');
    hc_putint(xi);
    hc_putint(yi);
}
void
hc_cont(xi,yi)
register int xi, yi;
{
    hc_putc('n');
    hc_putint(xi);
    hc_putint(yi);
}
void
hc_circle( xi, yi, ri )
int xi, yi, ri;
{
  hc_putc('c');
  hc_putint(xi);
  hc_putint(yi);
  hc_putint(ri);
}

/* fill in area in a closed curve */
void
hc_blob(npoints, xarray, yarray)
     int npoints, *xarray, *yarray;
{
    int i;
    hc_poly(npoints);
    /* plot all points */
    for(i=0; i<npoints; i++)
    hc_point(xarray[i], yarray[i]);
    hc_polydraw();
}
void
hc_texture(mode)
register int mode;
{
    register int i,cc;

    if (mode < 0)
        mode = 0;
    else if (mode > 4)
        mode = 4;

    hc_putc('f');
    for(i=0;(cc = *(linetexture[mode]+i)) != 0; i++)
        hc_putc(cc);
    hc_putc('\0');
}

void
hc_label(s)
register char *s;
{
    hc_putc('t');
    hc_putstring(s);
}
void
hc_color(s)
register char *s;
{
    hc_putc('C');
    hc_putstring(s);
}
void
hc_font(s)
register char *s;
{
    hc_putc('F');
    hc_putstring(s);
}
void
hc_fill(mode)
register int mode;
{
    hc_putc('G');
    if ( mode <= 0)
        hc_putstring("transparent");
        else
        hc_putstring("opaque");
}
void
hc_poly(count)
register int count;
{
    hc_putc('P');
    hc_putint(count);
}
void
hc_polydraw()
    {
    hc_putc('E');
    }
void
hc_putstring(s)
char *s;
{
    int i;
    for(i=0;s[i]!='\0'; i++)
        hc_putc(s[i]);
    hc_putc('\000');
}
