/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/*-----------------------------------------------------------------------------
* NAME:		gprintf 
* PURPOSE:	graphics printf at current location with optional centering 
* USAGE: 
* 	gprintf( center, format [,arg] ...) 
*	int center - 0 = first character is above right of current position
*	             1 = first character is centered on current position
*	             2 = complete string is centered on current position
*	char *format - same as format for printf
*	
* EXAMPLE:
*	int i;
*	float y;
*	gprintf(1, "The value of i = %d   y=%f",i, y);
* DESCRIPTION:
*	Prints formatted output on the graphics display screen starting
*   at the point where the last point was plotted. 
*   Uses the same conversion routines as printf. Centers 
*   the first character or the whole string or puts the first character 
*   above right of the last location plotted.
*   Blanking can be turned on with gblank() for gvt terminals.
* LIMITATIONS:
*	Do not use imbedded CR's ('\n') because these will be handled 
*  incorrectly. Print only one line per gprintf call.
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/gprintf.c,v 11.7 2004/01/15 19:29:28 kukolich Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/gprintf.c,v $
* CREATED: 26-Mar-84 rpl    LAST UPDATE: 26-Mar-84 rpl	MIT LINCOLN LABORATORY
*---------------------------------------------------------------------------*/
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include	<stdio.h>
#include "g_gvt.h"
#include "xlippic.h"
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#endif
void
gprintf(int center, char *fmt, ...)
/* center first character flag */
{
	va_list args;
	int i;
	int length;
	int u, v;

	char tempbuf[DOPRNT_BUFFER];
	char withback[DOPRNT_BUFFER*2];

	/* format into temporary buffer */
	va_start(args, fmt);
	vsprintf(tempbuf, fmt, args);
	va_end(args);
	length = strlen(tempbuf);


	/* error if graphics not initialized */
	if ( !GG_.inityet)
		g_initerr();


	/* get current location */
	u = GG_.ucur;
	v = GG_.vcur;

	/* make sure cursor is at correct location */
	if (center == 0 && GG_.blanking == 0 )
		uvmove(u, v);

	/* center character */
	if ( center !=0 )
	    {
	    u += GGc[GG_.csize];
	    v += GGd[GG_.csize];
		uvmove(u, v);
	    }

	/* center complete string using backspaces */
	withback[0] = '\0';
	if ( center == 2 && length > 1)
	    {
	    for ( i=0; i<length/2; i++)
		withback[i] = '\b';
	    withback[length/2] = '\0';
	    }

	tempbuf[length] = '\0';
	strcat(withback, tempbuf);
	x_plot_label(withback);

	/* hardcopy */
	if ( GG_.hardcopy == 1)
	    hc_label(withback);

}
