/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/*-----------------------------------------------------------------------------
* NAME:		grcmove 
* PURPOSE:	relative character movement specified in lines and spaces 
* USAGE: 
* 	grcmove( line_offset, column_offset)
*	int line_offset   - offset in lines from current line ( -34 to +34)
*	int column_offset - offset in columns from current column (-72 to +72)
*	
* DESCRIPTION:
*	Moves graphics cursor to a point specified in lines and columns relative
*   to the current point.  Use with gprint and gputc without centering.
*   Like gcmove but character location is relative.
*   Grcmov( 1, 0) moves down one line, grcmove(-1, 0) moves up a 
*   line , grcmove(0, 1) moves right a character, and grcmove(0, -1)
*   moves left a character.
* LIMITATIONS:
*      Multiple relative moves starting at a point defined by gcmove()
*   will eventually lead to round off errors in the absolute position
*   as compared to the absolute position used by gcmove(). This is normally
*   not important, but can become visible after 20 or so relative row
*   movements.
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/grcmove.c,v 11.3 2001/08/23 20:15:08 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/grcmove.c,v $
* CREATED: 26-Mar-84 rpl    LAST UPDATE: 26-Mar-84 rpl	MIT LINCOLN LABORATORY
*---------------------------------------------------------------------------*/
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include	<stdio.h>
#include "g_gvt.h"
#include "xlippic.h"
void
grcmove(l_offset, c_offset)
int l_offset;	
int c_offset;
{
	int csize;
	int udelta, vdelta;

	/* limit to +/-34 lines, +/-72 columns */
	if ( l_offset < -34)
		l_offset = -34;
	else if ( l_offset > 34)
		l_offset = 34;
	if ( c_offset < -72)
		c_offset = -72;
	else if ( c_offset > 72)
		c_offset = 72;



	/* get character size */
	csize=GG_.csize;

	/* calculate offsets */
	/* note only first entry of a and b are used */
	udelta = (int)(0.5 + c_offset*GGb[csize]);
	vdelta = -1*(int)(0.5 + l_offset*GGa[csize]);

	/* enter graphics mode and go to desired position */
	uvrmove(udelta, vdelta);

}
