/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/*-----------------------------------------------------------------------------
* NAME:		gscale 
* PURPOSE:	scale size and offset of all graphics output on gvt screen 
* USAGE: 
* 	gscale( uoffset, voffset, width, height)
*	float uoffset, voffset - u,v offsets expressed in terms of a fraction
*                              off the full screen excursion in both directions.
*                              Setting both to 0.5 moves the point (0,0)
*                              to the middle of the screen
*	float width, height - u,v multiplicative scale factors 
*                         Setting both to 0.5 halves the size of the graph.
* EXAMPLE:
* 	gscale( 0.0, 0.5, 1.0, 0.5);		- One plot in top half of screen 
* 	gscale( 0.0, 0.0, 1.0, 1.0);		- Default setting 
* DESCRIPTION:
*   Scales all points plotted on the screen using the formulas
*       unew = uoffset*G_XMAX + u*width 
*		vnew = voffset*G_YMAX + v*height
*   This is compatible with the offset parameters available
*   in the lipiplot and lipgplot commands.
*   Scaling during hardcopy is obtained using the size g(3) graphics
*   command.
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/gscale.c,v 11.3 2001/08/23 20:15:08 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/gscale.c,v $
* CREATED: 10-Dec-84 rpl   LAST UPDATE: 10-Dec-84 rpl	MIT LINCOLN LABORATORY
*---------------------------------------------------------------------------*/
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include "g_gvt.h"
#include <stdio.h>

#include "xlippic.h"
void
gscale(x, y, xf, yf)
float x,y;
float xf, yf;
{
int s1, s2, s3, s4;		/* parameters for hardcopy space routine */

if ( xf == 0 || yf == 0)
	{
	lippic_fatal_error("%s\nwidth fraction = %f, height fraction = %f",
	     "gscale: height or width fraction is zero", xf, yf);
	 }

/* set offset and scale factor */
GG_.uoffset = x*G_XMAX;
GG_.voffset = y*G_YMAX;
GG_.ufract = xf;
GG_.vfract = yf;

/* transfer scaling to parameters for space routine and issue space command */
s1 = -GG_.uoffset/GG_.ufract;
s2 = -GG_.voffset/GG_.vfract;
s3 = (G_XMAX-GG_.uoffset)/GG_.ufract;
s4 = (G_YMAX-GG_.voffset)/GG_.vfract;
x_plot_space( s1, s2, s3, s4);

if (GG_.hardcopy == 1)
	hc_space( s1, s2, s3, s4);
}
