/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/*-----------------------------------------------------------------------------
* NAME:		gsymbol 
* PURPOSE:	draw one of eight symbols at the current point   
* USAGE: 
* 	gsymbol(symbol, radius )
*       int symbol - symbol to plot
*                     0 = open square, 1 = open triangle, 2 = plus ,3 = x
*                     4 = open diamond, 5 = open upside down triangle
*		      6 = filled square, 7 = circle, 8 = filled circle 
*		      9 = right arrow head, 10 = left arrow head
*		      11 = up arrow head, 12 = down arrow head
*             13 = upper part of error bar, 14 = lower part of error bar
*		      Other values of symbol produce an error
*       int radius - radius of symbol in uv units (0-1023)
* DESCRIPTION: 
*	Plots symbol. Blanking can be turned on with gblank()
*   for gvt terminals.
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/gsymbol.c,v 11.3 2001/08/23 20:15:08 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/gsymbol.c,v $
* CREATED: 15-Mar-84 rpl    LAST UPDATE: 15-Mar-84 rpl	MIT LINCOLN LABORATORY
*---------------------------------------------------------------------------*/
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include "g_gvt.h"
#include "xlippic.h"
void
gsymbol(symbol, r)
char symbol;	
int r;
{
	int i,j;
	int u, v;
	/* check symbol type */
	if ( symbol <0 || symbol >14 )
	    {
	    lippic_fatal_error("gsymbol: symbol index should be 0-8, not %d:", symbol);
	    }

	/* blank out area */
	if ( GG_.blanking != 0 && symbol != 6)
		{
		u = GG_.ucur;
		v = GG_.vcur;
		uvblock("background", u-r, v-r, u+r, v+r);
		uvmove(u, v);
		}
	
	/* determine symbol type and draw */
	switch (symbol)
	    {
	    /* open square */ 
	    case 0:
		    uvrmove(r, r);
		    uvrcont(-2*r, 0);
		    uvrcont(0, -2*r);
		    uvrcont(2*r, 0);
		    uvrcont(0, 2*r);
		    uvrmove(-r, -r);
		    break;
	    /* open triangle */
	    case 1:
		    i = (int)(.866027*r);
		    j = (int)(1.732*r);
		    uvrmove(0 , r);
		    uvrcont(-i, -(r+r/2));
		    uvrcont(j, 0);
		    uvrcont(-i, (r+r/2));
		    uvrmove(0, -r);
		    break;
	    /* plus */
	    case 2:
		    uvrcont(-r, 0);
		    uvrcont(2*r, 0);
		    uvrcont(-r, 0);
		    uvrcont(0, -r);
		    uvrcont(0, 2*r);
		    uvrcont( 0, -r);
		    break;
	    /* x */
	    case 3:
		    i = (int)(.707*r);
		    j = (int)(2.0*.707*r);
		    uvrcont(-i, i);
		    uvrcont(j, -j);
		    uvrcont(-i, i);
		    uvrcont(-i, -i);
		    uvrcont(j, j);
		    uvrcont(-i, -i);
		    break;
	    /* diamond */
	    case 4:
		    uvrmove(0 , r);
		    uvrcont(-r, -r);
		    uvrcont(r, -r);
		    uvrcont(r, r);
		    uvrcont(-r, r);
		    uvrmove(0 , -r);
		    break;
	    /* upside down triangle */
	    case 5:
		    i = (int)(.866027*r);
		    j = (int)(1.732*r);
		    uvrmove(0 , -r);
		    uvrcont(-i, (r+r/2));
		    uvrcont(j, 0);
		    uvrcont(-i, -(r+r/2));
		    uvrmove(0, r);
		    break;
	    /* filled square */
	    case 6:
		    u = GG_.ucur;
		    v = GG_.vcur;
		    uvblock((char *)NULL, u-r, v-r, u+r, v+r);
		    uvmove(u, v);
		    break;
	    /* circle */
	    case 7:
		    u = GG_.ucur;
		    v = GG_.vcur;
		    uvcircle( u, v, r);
		    uvmove(u, v);
		    break;
	    /* filled circle */
	    case 8:
		    u = GG_.ucur;
		    v = GG_.vcur;
		    uvdot((char *)NULL, u, v, r);
		    uvmove(u, v);
		    break;
	    /* right arrow head */
	    case 9:
		    uvrcont(r , r);
		    uvrmove(-r, -r);
		    uvrcont(r, -r);
		    uvrmove(-r, r);
		    break;
	    /* left arrow head */
	    case 10:
		    uvrcont(-r , r);
		    uvrmove(r, -r);
		    uvrcont(-r, -r);
		    uvrmove(r, r);
		    break;
	    /* up arrow head */
	    case 11:
		    uvrcont(-r , -r);
		    uvrmove(r, r);
		    uvrcont(r, -r);
		    uvrmove(-r, r);
		    break;
	    /* down arrow head */
	    case 12:
		    uvrcont(-r , r);
		    uvrmove(r, -r);
		    uvrcont(r, r);
		    uvrmove(-r, -r);
		    break;
		/* upper edge of error bar */
		case 13:
			uvrmove(-r, 0);
			uvrcont(2*r, 0);
			uvrmove(-r, 0);
			uvrcont(0, -r);
			uvrmove(0, r);
			break;
		/* lower edge of error bar */
		case 14:
			uvrmove(-r, 0);
			uvrcont(2*r, 0);
			uvrmove(-r, 0);
			uvrcont(0, r);
			uvrmove(0, -r);
			break;


		}
}
