/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/*-----------------------------------------------------------------------------
* NAME:     uvarray 
* PURPOSE:  plot lines connecting array of y integer values 
* USAGE: 
*   uvarray(x0, y0, y, n, xdelta, power2, texture, color) 
*   int x0, y0  - screen coordinates where x=0, y=0 (0 to 1023) 
*   int y[]     - array containing y values
*   int n       - number of points to plot 
*   int xdelta  - increment in x for each point plotted (0 to 1023)
*   int power2  - scale factor in powers of two to multiply
*                     the y values by (-31 to +31)
*   int texture - line type as in gtexture (0=solid, 1=dotted, 2=dot-dashed
*                       3=short-dashed, 4=long-dashed)
*   char *color - color for lines
* DESCRIPTION:
*       Modified version of putarray written by Doug Paul 4/1/84
*       without local array to allow use on 11/44.
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/uvarray.c,v 11.3 2001/08/23 20:15:11 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/uvarray.c,v $
* CREATED: 15-Mar-84 rpl    LAST UPDATE: 15-Mar-84 rpl  MIT LINCOLN LABORATORY
*---------------------------------------------------------------------------*/
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include "g_gvt.h"
#include "xlippic.h"
/* declarations of statics */
static int g_scale(/* int y, int power2 */);
void
uvarray(x0,y0,y,n,xdelta,power2,texture, color)
int x0; /* starting point */
register int y0;
int y[];    /* y array */
int n;      /* number of points */
register int xdelta;    /* delta x for each point */
int power2; /* power of 2 scale factor */
int texture;    /* line texture 0=solid, 1=dotted, 2=dot-dashed, 3=short-dashed
                4=long-dashed */
char *color;
{   
    register int temp1, temp2, temp3; /* three successive values to plot */
    register int i;
    register int x;         /* x value */
    int old_texture;        /* save texture for restoration */
    char save_color[MAX_COLOR_STRING];
    strcpy(save_color, GG_.color);
    gcolor(color);

    /* plot only one point */
    if(n==1)
        {   
        uvpoint( x0, g_scale(y[0], power2) + y0);
        gcolor(save_color);
        return;
        }

    /* save current line texture and set new texture */
    old_texture = GG_.texture;
    gtexture(texture);

    x = x0;

    /* plot first point */
    temp1 = g_scale(y[0],power2);
    uvpoint( x, temp1 + y0);

    /* plot middle points */
    temp2 = g_scale(y[1],power2);
    for(i=1;i<n-1;i++)
        {   
        /* increment x */
        x += xdelta;
        temp3 = g_scale(y[i+1],power2);

        /* plot if not linear interpolation */
        if(temp1 + temp3 != temp2*2)
            uvcont( x, temp2 + y0);

        temp1 = temp2;
        temp2 = temp3;
        }

    /* plot last  point */
    x += xdelta;
    uvcont( x, g_scale(y[n-1], power2) + y0);

    /* restore old line texture  and color */
    gtexture(old_texture);
    gcolor(save_color);
}
static int g_scale(y, power2)
register int y;
register int power2;
{
    register int k;

    if(power2>=0) 
        return( y<<power2 );
    else
        {
        /* round off */
        k=1<<(power2-1);        
            return( (y+k) >> -power2);
        }
}
