/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/*-----------------------------------------------------------------------------
* NAME:     uvbeam 
* PURPOSE:  draw variable width line between two points 
* USAGE: 
*   uvbeam (u1, v1, u2, v2, width)  
*   int u1, v1  - point at one end of line (currently 0-1023, 0-780)
*   int u2, v2  - point at other end of line (currently 0-1023, 0-780)
*   int width   - beam width (currently 0-1023)
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/uvbeam.c,v 11.4 2001/11/01 20:02:58 kukolich Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/uvbeam.c,v $
* CREATED: 15-Mar-84 rpl    LAST UPDATE: 15-Mar-84 rpl  MIT LINCOLN LABORATORY
*---------------------------------------------------------------------------*/
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include "g_gvt.h"
#include "xlippic.h"
void
uvbeam (u1,v1,u2,v2,width)  /* draw line from (u1,v1) to (u2,v2) */
     int u1,v1,u2,v2;
     int width;
{
#define DISTINCT_SPACING 2  /* minimum offset between lines that produces
			       a distinct line */
#define Y_FUNCT_OF_X 2      /* Test line, y is function of x */
#define X_FUNCT_OF_Y 3      /* Test line, x is function of y */

 int delta;      /* offset for this line from center line */
 int x1, x2, y1, y2; /* endpoints of added lines */
 int n;
 int old_above, old_below; /* last above and below offsets */

 /* add trunks */
 if (width > 0)
   {
     /* always draw center line */
     uvline(u1, v1, u2, v2);

     delta = 0;
     old_above = 0;
     old_below = 0;
     for ( n = 1; n <= width; n++)
       {
	 /* add trunks evenly so link expands from center (odd below)*/
	 if ( n%2 == 1 )
	   {
	     delta = -1*(n/2);
	     /* skip if line not distinct from last */
	     if ( (old_below - delta) < DISTINCT_SPACING)
	       continue;
	     old_below = delta;
	   }
	 /* above */
	 else
	   {
	     delta = (n/2);
	     /* skip if line not distinct from last */
	     if ( (delta - old_above) < DISTINCT_SPACING)
	       continue;
	     old_above = delta;
	   }

	 /* draw line by fattening on longest side */
	 if ( abs(u1-u2) >= abs(v1-v2) )
	   {
	     x1 = u1;
	     x2 = u2;
	     y1 = v1 +delta;
	     y2 = v2 +delta;
	   }
	 else
	   {
	     x1 = u1+delta;
	     x2 = u2+delta;
	     y1 = v1;
	     y2 = v2;
	   }
	 uvline(x1, y1, x2, y2);
       }
   }
}
