/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/*-----------------------------------------------------------------------------
* NAME:     uvpairs 
* PURPOSE:  plot lines connecting pairs of x, y integer points 
* USAGE: 
*   uvpairs(x0, y0, x, y, n, texture, color) 
*   int x0, y0  - screen coordinates where x=0, y=0 (0 to 1023) 
*   int x[]     - array containing x values
*   int y[]     - array containing y values
*   int n       - number of points to plot 
*   int texture - line type as in gtexture (0=solid, 1=dotted, 2=dot-dashed
*                       3=short-dashed, 4=long-dashed)
*       char *color - color string
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/uvpairs.c,v 11.3 2001/08/23 20:15:12 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/uvpairs.c,v $
* CREATED: 15-Mar-84 rpl    LAST UPDATE: 15-Mar-84 rpl  MIT LINCOLN LABORATORY
*---------------------------------------------------------------------------*/
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include "g_gvt.h"
#include "xlippic.h"
void
uvpairs(x0, y0, x, y, n, texture, color) 
int x0,y0;  /* starting point */
int x[];        /* x array */
int y[];    /* y array */
int n;      /* number of points */
int texture;    /* line texture 0=solid, 1=dotted, 2=dot-dashed, 3=short-dashed
                4=long-dashed */
char *color;    /* color string */
{   
    int i;
    int old_texture;        /* save texture for restoration */
    char save_color[MAX_COLOR_STRING];
    strcpy(save_color, GG_.color);
    gcolor(color);

    /* plot only one point */
    if(n==1)
        {   
        uvpoint( x0 + x[0], y0 + y[0] );
        gcolor(save_color);
        return;
        }

    /* save current line texture and set new texture */
    old_texture = GG_.texture;
    gtexture(texture);

    /* plot first point */
    uvpoint( x0+x[0], y0+y[0]);

    /* plot middle points */
    for(i=0;i<n-1;i++)
        {
        /* plot only points that are not linear interpolations */
    if( ((y[i]-y[i-1])*(x[i+1]-x[i-1])) != ((x[i]-x[i-1])*(y[i+1]-y[i-1])))
            uvcont( x0 + x[i], y0 + y[i] );
        }

    /* plot last point */
    uvcont( x0 + x[n-1], y0 + y[n-1]);

    /* restore old line and color texture */
    gcolor(save_color);
    gtexture(old_texture);

}
