/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/*-----------------------------------------------------------------------------
* NAME:     uvspairs 
* PURPOSE:  plot symbols at pairs of x, y integer points with optional line 
* USAGE: 
*   uvpairs(x0, y0, x, y, n, mode, symbol, sradius, texture, color) 
*   int x0, y0  - screen coordinates where x=0, y=0 (0 to 1023) 
*   int x[]     - array containing x values
*   int y[]     - array containing y values
*   int n       - number of points to plot 
*       int mode    - 0 = plot only symbols
*                     1 = plot symbols and lines
*                     2 = plot only lines (same as uvpairs)
*       char symbol - symbol to plot
*                     If the value of symbol is between 32 and 126 then
*                     the symbol is assumed to be an ascii character and that 
*                     character is printed at every point.
*                     If the symbol is 0 to 12 then the symbol plotted is:
*                     0 = open square, 1 = open triangle, 2 = plus ,3 = x
*                     4 = open diamond, 5 = open upside down triangle
*             6 = filled square, 7 = circle, 8=filled circle
*                     9=right arrow head, 10=left arrow head
*             11=up arrow head, 12=down arrow head
*             Other values of symbol produce an error
*                     if mode is 0 or 1.
*       int sradius - radius of symbol in uv units (0-1023)
*             Ignored if symbol is 32 to 126 or if mode is 2.
*   int texture - line type as in gtexture (0=solid, 1=dotted, 2=dot-dashed
*                       3=short-dashed, 4=long-dashed)
*             Ignored if mode is 0.
*       char *color - color for lines and symbols
* DESCRIPTION:
*     Plots only symbols, lines plus symbols, or lines only on coordinates
*  defined using x y pairs.
*  Blanking can be turned on with gblank() for gvt terminals.
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/uvspairs.c,v 11.3 2001/08/23 20:15:14 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/uvspairs.c,v $
* CREATED: 15-Mar-84 rpl    LAST UPDATE: 15-Mar-84 rpl  MIT LINCOLN LABORATORY
*---------------------------------------------------------------------------*/
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include "g_gvt.h"
#include "xlippic.h"
/* declarations of statics */
static void g_symbol(int symbol, int sradius);
void
uvspairs(x0, y0, x, y, n, mode, symbol, sradius, texture, color) 
int x0,y0;  /* starting point */
int x[];        /* x array */
int y[];    /* y array */
int n;      /* number of points */
int mode;   /* 0=symbol only, 1=symbol+lines, 2=lines only */
char symbol;    /* 0-12 = special symbols, 32-126 = ascii characters */
int sradius;    /* radius of special symbols */
int texture;    /* line texture 0=solid, 1=dotted, 2=dot-dashed, 3=short-dashed
                4=long-dashed */
char *color;
{   
    int i;
    int old_texture;        /* save texture for restoration */
    char save_color[MAX_COLOR_STRING];

    /* check range of mode flag */
    if ( mode < 0 || mode > 2)
        {
        gexit(1);
        fatal_error("uvspairs: mode should be 0 to 2, not %d",mode);
        }

    /* plot lines 
    ---------------------------*/
    if ( mode == 1 || mode == 2)
        uvpairs( x0, y0, x, y, n, texture, color);

    /* plot symbols 
    ---------------------------*/
    if (mode == 0 || mode == 1)
        {
        /* check symbol type */
        if ( symbol <0 || (symbol >12 && symbol < 32) || symbol > 126)
        {
        gexit(1);
        lippic_fatal_error("%s not %d",
           "uvspairs: symbol should be 0-12 or ascii char", symbol);
            }


        /* save current color */
        strcpy(save_color, GG_.color);
        gcolor(color);

        /* offset all to eliminate per character centering offset */
        if ( symbol >= 32)
        {
        x0 += GGc[GG_.csize];
        y0 += GGd[GG_.csize];
        }

        /* plot only one symbol */
        if(n==1)
        {   
        uvmove( x0 + x[0], y0 + y[0] );
        g_symbol( symbol, sradius);
        gcolor(save_color);
        return;
        }

        /* save current line texture and set to solid lines for symbols */
        old_texture = GG_.texture;
        gtexture(0);


        /* plot first symbol */
        uvmove( x0+x[0] ,y0+y[0]);
        g_symbol( symbol, sradius);

        /* plot rest of symbols */
        for(i=0;i<n;i++)
        {
        uvmove( x0 + x[i], y0 + y[i] );
        g_symbol( symbol, sradius);
        }

        /* restore old line texture and color*/
        gtexture(old_texture);
        gcolor(save_color);
        }
}
/* plot characters or symbols */
static void g_symbol(int symbol, int sradius)
{
if ( symbol > 12)
    gputc(0, symbol);
else
    gsymbol(symbol, sradius);
}
