/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/***---------------------------------------------------------------------------
* NAME:		xplot_init
* PURPOSE:	intialize openwindows lippic graphics routines 
* USAGE: 
* 	xplot_init("string");
*   	char	*string1;		- valid xview command line string 
*                                         to set color, position, size
*                                         of graphics window
* 
* RETURN VALUE:
*	Zero if ok, -1 if error
*
* EXAMPLE:
*	xplot_init("");	 -  default window size and placement 
*	xplot_init("-geometry 1023x780+100+100 -fg blue -bg yellow");	
*                        - specify HEIGHTxWIDTH+xoffset+yoffset,
*			   also foreground and background color
*
* DESCRIPTION:
*	Opens up a separate xview window for plotting graphics.
*       Pass an optional string of command line flags to specify
*       size, location, and color of window. The default fixed width
*       font and size (1023x780) were selected to emulate a tektronix
*       display. The string allows all aspects of the plotting window
*       that can be changed by xview(1) command line flags to be changed.
*       These flags include:
*	-fg color		- foreground color
*	-bg color		- background color
*	-geometry HxW+x+y	- height,width,xoffset,yoffset in pixels
*	-title			- window title or header
* 
* SEE ALSO:
* 	Description of allowable xview flags in SUN Openwindows
*       Deskset Environment Reference Guide (Pages 264-260).
*       Colors can be examined using the openwindows demo dislay program.
*       Click on News, then Colors, then Color Names.
*       A list of all standard available colors is provided with the command
*		showrgb
*
*       Common useful combinations of foreground and background colors are:
*
*		background	foreground
*		----------	----------
*		white		mediumseagreen	
*		white		steelblue	
*		black		white	
*		white		black
*		black		palegreen	
*		black		lightgrey	
*		yellow		dimgray	
*		black		wheat	
*		palegreen	steelblue	
*		green		dimgrey	
*		slateblue	wheat	
*		slateblue	yellow	
*
*       A file containing all common colors is available in
*		$OPENWINHOME/lib/rgb.txt
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/xplot_init.c,v 1.9 2001/08/23 20:15:15 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/xplot_init.c,v $
* CREATED: 1-Feb-90 rpl                             MIT LINCOLN LABORATORY
*---------------------------------------------------------------------------
* Copyright 1990 MIT Lincoln Laboratory.
* This software is not for release and should neither be copied
* nor distributed without explicit permission of MIT Lincoln Laboratory.
--------------------------------------------------------------------------***/
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include "PlotI.h"

int xplot_init(command_string)
char *command_string;
{
int ntokens, length;
char **argv; 
char *string;
PlotCmd plotcmd;
PlotCmd *pc = &plotcmd;
char *dummy;

length = strlen(command_string);

/* make argv a null pointer if command line string is empty
-------------------------------------------------------*/
if ( command_string == NULL || length <= 0 )
    {
    ntokens = 0;
    argv = (char **) lippic_malloc( sizeof(char *), 
				"for openwindow argument list");
    argv[0] = (char *) NULL;
    }

/* separate string into tokens and put into argv structure
   terminated with NULL pointer 
   --------------------------------------------------------*/
else 
    {
    /* allocate the maximum number of tokens */
    length = strlen(command_string);
    argv = (char **) lippic_malloc ( (sizeof(char *))*( 1 + length/2), 
				"in xplot_init for openwindow argument list");

    /* put pointers to tokens into argv */
    ntokens = 0;
	dummy = newstring(command_string);
    string = strtok(dummy, " \t");
    while ( string != NULL)
	    {
	    argv[ntokens] = string;
	    ntokens++;
	    string = strtok(NULL, " \t");
	    }
    /* terminate argv with null */
    argv[ntokens] = (char *) NULL;
    free(dummy);
    }

/* call routines to fork process and open window
--------------------------------------------------*/
  if ( PlotOpen ( argv ) < 0)
	{
	ghardcopy(0);
	lippic_fatal_error("xplot_init: couldn't open plot window");
	}
  else
	return(0);

x_plot_space(0,0,1023,780);
}
