/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/***---------------------------------------------------------------------------
* NAME:		x_plot
* PURPOSE:	subroutines to create plots with olxplot 
* 
* gmouse(x, y, button)
* gmouse_wait(x, y, button)
* x_plot_arc(center_x,center_y, start_x, start_y, end_x, end_y)
* x_plot_blob(npoints, xarray, yarray)
* x_plot_block(x0,y0, x1, y1)
* x_plot_circle(center_x,center_y, radius)
* x_plot_color(string)
* x_plot_cont(x,y)
* x_plot_end()
* x_plot_erase()
* x_plot_exit(destroy_flag)	
* x_plot_fill(flag)
* x_plot_font(string)
* x_plot_init(command_string)
* x_plot_label(string)
* x_plot_line(x0,y0, x1, y1)
* x_plot_linemode(string)
* x_plot_move(x,y)
* x_plot_point(x,y)
* x_plot_poly(count)
* x_plot_polydraw()
* x_plot_send_command()
* x_plot_space(x0,y0,x1,y1)
* x_plot_window_title(string)
* 
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/xplot_pack.c,v 11.5 2001/11/29 18:57:59 kukolich Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/xplot_pack.c,v $
* CREATED: 1-Feb-90 rpl                             MIT LINCOLN LABORATORY
*---------------------------------------------------------------------------
* Copyright 1990 MIT Lincoln Laboratory.
* This software is not for release and should neither be copied
* nor distributed without explicit permission of MIT Lincoln Laboratory.
--------------------------------------------------------------------------***/
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <ctype.h>
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <sys/types.h>
/* sys/time is for PlotPointerRead */
#if HAVE_SYS_TIME_H
#include <sys/time.h>
#endif
#if HAVE_MALLOC_H
#include <malloc.h>
#endif
#include "g_gvt.h"
#include "PlotI.h"
#include "xlippic.h"

/* these functions are found in PlotLink.c */
int PlotSetTitle(char *title);
int Plot(PlotCmd *pc);
int PlotOpen(char **argv);
void PlotClose(int terminate);
int PlotPointerRead(int *x_ret, int *y_ret, int *mod_ret, struct timeval timeout);
int PlotPointerWait(int *x_ret, int *y_ret, int *but_ret, int *mod_ret, struct timeval timeout);

typedef struct _keyent {
  char *keyword;
  int   value;
} KeyTabEnt, *KeyTab;


static KeyTabEnt LineModeKeyTab[] =
{
  { "dotted",	   PLOT_LINEMODE_DOTTED	},
  { "solid",	   PLOT_LINEMODE_SOLID	},
  { "longdashed",  PLOT_LINEMODE_LONGDASHED },
  { "shortdashed", PLOT_LINEMODE_SHORTDASHED },
  { "dotdashed",   PLOT_LINEMODE_DOTDASHED },
  { (char *) NULL, 0 }
};

static char *
newstring ( str )
     char *str;
{
    char *s;
    if ( ! str ) return (char *) NULL;
    if ( ! ( s = malloc ( strlen ( str ) + 1 ) ) ) return (char *) NULL;
    return strcpy ( s, str );
}

static  PlotCmd plotcmd;
static PlotCmd *pc = &plotcmd;

/* declarations of static functions */
static int FindKeyword(/* char *name, KeyTab keytab */);

void x_plot_end()
    {
    pc->command = PLOT_END;
    x_plot_send_command();
    }

void x_plot_move(x,y)
int x,y;
    {
    pc->command = PLOT_MOVE;
    pc->args.move.x0 = x;
    pc->args.move.y0 = y;
    x_plot_send_command();
    }

void x_plot_cont(x,y)
int x,y;
    {
    pc->command = PLOT_CONT;
    pc->args.cont.x0 = x;
    pc->args.cont.y0 = y;
    x_plot_send_command();
    }

void x_plot_point(x,y)
int x,y;
    {
    pc->command = PLOT_POINT;
    pc->args.point.x0 = x;
    pc->args.point.y0 = y;
    x_plot_send_command();
    }

void x_plot_line(x0,y0, x1, y1)
int x0,y0,x1,y1;
    {
    pc->command = PLOT_LINE;
    pc->args.line.x0 = x0;
    pc->args.line.y0 = y0;
    pc->args.line.x1 = x1;
    pc->args.line.y1 = y1;
    x_plot_send_command();
    }
 
void x_plot_poly(count)
int count;
    {
    pc->command = PLOT_EXT_POLY;
    pc->args.poly.count = count;
    x_plot_send_command();
    }

void x_plot_polydraw()
    {
    pc->command = PLOT_EXT_POLYDRAW;
    x_plot_send_command();
    }

void x_plot_fill(flag)
int flag;
    {
    pc->command = PLOT_EXT_FILL;
    if ( flag <= 0)
	pc->args.fill_mode.mode = PLOT_FILLMODE_TRANSPARENT;
    else
	pc->args.fill_mode.mode = PLOT_FILLMODE_OPAQUE;
    x_plot_send_command();
    }

/* fill in area in a closed curve */
void x_plot_blob(npoints, xarray, yarray)
int npoints, *xarray, *yarray;
    {
    int i;
    x_plot_poly(npoints);
    /* plot all points */
    for(i=0; i<npoints; i++)
	x_plot_point(xarray[i], yarray[i]);
    x_plot_polydraw();
    }

void x_plot_block(x0,y0, x1, y1)
int x0,y0,x1,y1;
    {
    x_plot_poly(5);
    x_plot_point(x0,y0);
    x_plot_point(x1,y0);
    x_plot_point(x1,y1);
    x_plot_point(x0,y1);
    x_plot_point(x0,y0);
    x_plot_polydraw();
    }

void x_plot_label(string)
     char *string;
{
  int i;

  pc->command = PLOT_LABEL;
  pc->args.label.label = string;
  pc->args.label.length = strlen(string);
  /* in Plot(), the command that will actually send this,
     there is a firm upper limit of 7 bits to transmit the length of
     this string. It had better be no more than 127 characters long... */
  if (pc->args.label.length > 127) {
    for(i = 0; i < 127; i++){
      string[i] = string[pc->args.label.length-127+i];
    }
    /* put an ellipses at the front so they know we did something then
       null terminate the string */
    if (string[0] != '\b') string[0] = '.';
    if (string[1] != '\b') string[1] = '.';
    if (string[2] != '\b') string[2] = '.';
    string[127] = '\0';
    pc->args.label.length = 127;
  }
  x_plot_send_command();
}

void x_plot_window_title(string)
char *string;
    {
    if ( GG_.noscreen )
	    return;
    if ( PlotSetTitle(string) < 0 )
	{
	lippic_fatal_error("x_plot_window_title: Can't set title to %s", string);
	}
    }

void x_plot_arc(center_x,center_y, start_x, start_y, end_x, end_y)
int center_x,center_y, start_x, start_y, end_x, end_y;
    {
    pc->command = PLOT_ARC;
    pc->args.arc.center_x = center_x;
    pc->args.arc.center_y = center_y;
    pc->args.arc.start_x = start_x;
    pc->args.arc.start_y = start_y;
    pc->args.arc.end_x = end_x;
    pc->args.arc.end_y = end_y;
    x_plot_send_command();
    }

void x_plot_circle(center_x,center_y, radius)
int center_x, center_y, radius;
    {
    pc->command = PLOT_CIRCLE;
    pc->args.circle.center_x = center_x;
    pc->args.circle.center_y = center_y;
    pc->args.circle.radius   = radius;
    x_plot_send_command();
    }

void x_plot_erase()
    {
    pc->command = PLOT_ERASE;
    x_plot_send_command();
    }

void x_plot_linemode(string)
char *string;
    {
    pc->command = PLOT_LINEMODE;
    pc->args.line_mode.mode = FindKeyword(string, LineModeKeyTab);
    x_plot_send_command();
    }

void x_plot_space(x0,y0,x1,y1)
int x0,y0,x1,y1;
      {
      pc->command = PLOT_SPACE;
      pc->args.space.x0 = x0;
      pc->args.space.y0 = y0;
      pc->args.space.x1 = x1;
      pc->args.space.y1 = y1;
      x_plot_send_command();
      }

void x_plot_color(string)
char *string;
      {
      pc->command = PLOT_EXT_COLOR;
      pc->args.color.color     = string;
      pc->args.color.pixel     = (unsigned long) -1;
      pc->args.color.length     = strlen(string);
      x_plot_send_command();
      }

void x_plot_font(string)
char *string;
      {
      pc->command = PLOT_EXT_FONT;
      pc->args.font.font = string;
      pc->args.font.length = strlen(string);
      x_plot_send_command();
      }

void x_plot_send_command()
	{
	if ( GG_.noscreen)
		return;
	if ( Plot ( pc ) < 0 ) 
	  {
	  lippic_fatal_error("%s %c %s",
	    "x_plot_send_command: couldn't send plot command", pc->command,
	    "to plot window");
          }
	}


/* decode string with line type into constant symbol
----------------------------------------------------*/
static int
FindKeyword(name, keytab)
     char *name;
     KeyTab  keytab;
{
  register unsigned char *cp;
  register char *bp;
  register KeyTabEnt *kp;

  cp = bp = newstring(name);
  while (*cp++) {
    if (isascii(*cp) && isupper(*cp))
      *cp = tolower(*cp);
  }
  cp = bp;
  for (kp = keytab; kp->keyword != (char *) NULL; kp++)
    if (strcmp(kp->keyword, cp) == 0)
      break;
  free(bp);
  /* couldn't find match for this name */
  if ( kp->keyword == (char *)NULL)
	{
	lippic_fatal_error("FindKeyword: Invalid string in xplot command = %s", name);
	}
  return kp->value;
}

/***---------------------------------------------------------------------------
* NAME:		x_plot_init
* PURPOSE:	intialize openwindows lippic graphics routines 
* USAGE: 
* 	x_plot_init("string");
*   	char	*string1;		- valid xview command line string 
*                                         to set color, position, size
*                                         of graphics window
* 
* RETURN VALUE:
*	Zero if ok, -1 if error
*
* EXAMPLE:
*	x_plot_init("");	 -  default window size and placement 
*	x_plot_init("-geometry 1023x780+100+100 -fg blue -bg yellow");	
*                        - specify HEIGHTxWIDTH+xoffset+yoffset,
*			   also foreground and background color
*
* DESCRIPTION:
*	Opens up a separate xview window for plotting graphics.
*       Pass an optional string of command line flags to specify
*       size, location, and color of window. The default fixed width
*       font and size (1023x780) were selected to emulate a tektronix
*       display. The string allows all aspects of the plotting window
*       that can be changed by xview(1) command line flags to be changed.
*       These flags include:
*	-fg color		- foreground color
*	-bg color		- background color
*	-geometry HxW+x+y	- height,width,xoffset,yoffset in pixels
*	-title			- window title or header
* 
* SEE ALSO:
* 	Description of allowable xview flags in SUN Openwindows
*       Deskset Environment Reference Guide (Pages 264-260).
*       Colors can be examined using the openwindows demo dislay program.
*       Click on News, then Colors, then Color Names.
*       A list of all standard available colors is provided with the command
*		showrgb
*
*       Common useful combinations of foreground and background colors are:
*
*		background	foreground
*		----------	----------
*		white		mediumseagreen	
*		white		steelblue	
*		black		white	
*		white		black
*		black		palegreen	
*		black		lightgrey	
*		yellow		dimgray	
*		black		wheat	
*		palegreen	steelblue	
*		green		dimgrey	
*		slateblue	wheat	
*		slateblue	yellow	
*
*       A file containing all common colors is available in
*		$OPENWINHOME/lib/rgb.txt
*
* CREATED: 1-Feb-90 rpl                             MIT LINCOLN LABORATORY
*---------------------------------------------------------------------------
* Copyright 1990 MIT Lincoln Laboratory.
* This software is not for release and should neither be copied
* nor distributed without explicit permission of MIT Lincoln Laboratory.
--------------------------------------------------------------------------***/

int
x_plot_init(command_string)
     char *command_string;
{
    int ntokens, length;
    char **argv; 
    char *string;
    char *dummy = NULL;

    if (GG_.noscreen)
      return -1;

    length = strlen(command_string);

    if ( command_string == NULL || length <= 0 ) {
	/* make argv a null pointer if command line string is empty
	   -------------------------------------------------------*/
	ntokens = 0;
	argv = (char **) lippic_malloc( sizeof(char *), 
				       "for openwindow argument list");
	argv[0] = (char *) NULL;
    } else {
	/* separate string into tokens and put into argv structure
	   terminated with NULL pointer 
	   --------------------------------------------------------*/
	/* allocate the maximum number of tokens */
	length = strlen(command_string);
	argv = (char **) lippic_malloc ( (sizeof(char *))*( 1 + length/2), 
					"in x_plot_init for argument list");

	/* put pointers to tokens into argv */
	ntokens = 0;
	dummy = newstring(command_string);
	string = strtok(dummy, " \t");
	while ( string != NULL) {
	    argv[ntokens] = string;
	    ntokens++;
	    string = strtok(NULL, " \t");
	}
	/* terminate argv with null */
	argv[ntokens] = (char *) NULL;
    }

    /* call routines to fork process and open window
       --------------------------------------------------*/
    if ( PlotOpen ( argv ) < 0) {
	lippic_fatal_error("x_plot_init: couldn't open plot window");
    } else {
	x_plot_space(0,0,1023,780);
    }
    if(dummy != NULL) free(dummy);
    free(argv);
    return(0);
}
/*-----------------------------------------------------------------------------
* NAME:		x_plot_exit 
* PURPOSE:	exit x graphics with optional destruction of graphics window 
* USAGE: 
* 	x_plot_exit(destroy_flag)
*	int destroy_flag - 1 = destroy graphics window before exit
*		         - 0 = don't destroy graphics window before exit
*---------------------------------------------------------------------------*/
void x_plot_exit(destroy_flag)	
int destroy_flag;
{
	if ( GG_.noscreen )
		return;
        PlotClose ( destroy_flag ); 
}

/***---------------------------------------------------------------------------
* NAME:		gmouse
* PURPOSE:	get current location of mouse on graphics window
* USAGE: 
* 	gmouse( x, y, button);
*	int *x,*y;	mouse position (-1 if mouse off graphics window)
*       int *button;	0=no button, 1=left button currently down, 
*			2= middle button, 3=right button, 4=shift key,
*                       5 = ctrl key
*
* RETURN VALUE:
*       0 If normal return with coordinates.
*
* DESCRIPTION:
*	Returns current coordinates of mouse without waiting for button 
*       push.  Also indicates if any buttons are down.
* 
--------------------------------------------------------------------------***/
int gmouse(x, y, button)
int *x, *y, *button;
{
struct timeval timeout;
int mod;
int result;

if ( GG_.noscreen)
	return -1;

timeout.tv_sec = 30;		/* timeout for response from plot process */
timeout.tv_usec = 0;

result = PlotPointerRead ( x, y, &mod, timeout );

/* error */
if ( result == -1)
      {
      lippic_fatal_error("gmouse: pointer read error");
      }
/* timed out */
else if ( result == 0)
      {
      lippic_fatal_error("gmouse: timed out waiting for mouse coordinates");
      }
/* ok */
else
      {
      if ( mod == 256 )
		*button = 1;
      else if ( mod == 512 )
		*button = 2;
      else if ( mod == 1024 )
		*button = 3;
      else if ( mod == 1 )
		*button = 4;
      else if ( mod == 4 )
		*button = 5;
      else
	*button = 0;
      }
      return 0;
}

/***---------------------------------------------------------------------------
* NAME:		gmouse_wait
* PURPOSE:	wait for button push, then get location of mouse 
* USAGE: 
* 	gmouse_wait( &x, &y, &button);
*	int x,y; 	mouse position 
*       int button;	1=button1 pushed, 2=button2, 3=button3 
*
* RETURN VALUE:
*       0 If normal return with coordinates.
*
* DESCRIPTION:
*	Returns coordinates of mouse after next button push.
*       Indicates which button was pushed.
* 
--------------------------------------------------------------------------***/
int gmouse_wait(x, y, button)
int *x, *y, *button;
{
struct timeval timeout;
int mod, but_ret;
int result;

if ( GG_.noscreen)
	return -1;

timeout.tv_sec = 24*3600;   /* wait forever for response from plot process */
timeout.tv_usec = 0;

result = PlotPointerWait ( x, y, &but_ret, &mod, timeout );

/* error */
if ( result == -1)
      {
      lippic_fatal_error("gmouse_wait: pointer read error");
      }
/* timed out */
else if ( result == 0)
      {
      lippic_fatal_error("gmouse_wait: timed out waiting for mouse coordinates");
      }
/* ok */
else
      {
      *button = but_ret;
      }
      return 0;
}
