/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/*-----------------------------------------------------------------------------
* NAME:     xyarray 
* PURPOSE:  plot lines connecting array of floating point y values 
* USAGE: 
*   xyarray(x0, y0, y, n, xdelta, power2, texture, color) 
*   int x0, y0  - screen coordinates where x=0, y=0 (0 to 1023) 
*   float y[]   - array containing y values
*   int n       - number of points to plot 
*   int xdelta  - increment in x for each point plotted (0 to 1023)
*   int power2  - scale factor in powers of two to multiply
*                     the y values by (-31 to +31)
*   int texture - line type as in gtexture (0=solid, 1=dotted, 2=dot-dashed
*                       3=short-dashed, 4=long-dashed)
*       char *color - color of lines
* DESCRIPTION:
*       Modified version of old group24 putarray written by Doug Paul 4/1/84
*       without local array to allow use on 11/44.
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/xyarray.c,v 11.3 2001/08/23 20:15:15 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/xyarray.c,v $
* CREATED: 15-Mar-84 rpl    LAST UPDATE: 15-Mar-84 rpl  MIT LINCOLN LABORATORY
*---------------------------------------------------------------------------*/
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include "g_gvt.h"
#if HAVE_STRINGS_H
#include <strings.h>
#endif
#include "xlippic.h"
static double scale_factor;     /* power of two scale factor */

/* declarations for static functions */
static int ARscale(/* float y */);
void
xyarray(x0,y0,y,n,xdelta,power2,texture, color)
int x0,y0;  /* starting point */
float y[];  /* y array */
int n;      /* number of points */
int xdelta; /* delta x for each point */
int power2; /* power of 2 scale factor */
int texture;    /* line texture 0=solid, 1=dotted, 2=dot-dashed, 3=short-dashed
                4=long-dashed */
char *color;
{   
    int i;
    int x;                      /* x value */
    int old_texture;        /* save texture for restoration */
    char save_color[MAX_COLOR_STRING];
    strcpy(save_color, GG_.color);
    gcolor(color);


    /* calculate scale factor */
    if(power2>=0) 
        scale_factor =(double)(1<<power2);
    else scale_factor = 1./(1<<-power2);


    /* plot only one point */
    if(n==1)
        {   
        uvpoint( x0, ARscale(y[0]) + y0);
        gcolor(save_color);
        return;
        }

    /* save current line texture and set new texture */
    old_texture = GG_.texture;
    gtexture(texture);

    x = x0;

    /* plot first point */
    uvpoint( x, ARscale(y[0]) + y0);

    /* plot middle points */
    for(i=1;i<n-1;i++)
        {   
        /* increment x */
        x += xdelta;

        /* plot if not linear interpolation */
        if( (ARscale(y[i-1])+ARscale(y[i+1])) != 2*ARscale(y[i])) 
        uvcont( x, ARscale(y[i]) + y0);
        }

    /* plot last  point */
    x += xdelta;
    uvcont( x, ARscale(y[n-1]) + y0);

    /* restore old line texture and color */
    gtexture(old_texture);
    gtexture(texture);
    gcolor(save_color);

}
static int ARscale(y)
float y;
{
    return( (int)(y*scale_factor + .5) );
}
