/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/***---------------------------------------------------------------------------
* NAME:		xypie
* PURPOSE:	plot pie chart from floating numbers 
* USAGE: 
* xypie(y, n, graph_title, shading_type, label_location, labels,
*         size_fract, xoffset_fract, yoffset_fract)
*
*	float y[] - array containing floating numbers.
*                   The sum of all n numbers is calculated and then percentages
*                   of this sum for each number are alloted to percentages of
*                   the pie.
*	int n      - number slices in the pie and numbers in y.
*   char *graph_title - string containing title of graph
*   int shading_type - 0=no shading (slices are not colored in)
*                      1= slices are alternately uncolored and filled
*   int label_location - 0 = labels are outside the pie 
*                        1 = labels are inside the pie 
*   char *labels[]   - Pointer array for labels.  Set to 0 (NULL) if there 
*                      are no labels.
*	float size_fract - fraction of screen to use for pie 
*	float xoffset_fract, yoffset_fract - fractional x and y offsets
*
* EXAMPLE:
*	# include "gvt.h"
*	float  y[4];
*	static char *labels[] = { "apples", "peaches", "pears", "cheeries"};
*
*	ginit(4,1);
*       xypie(y, 4, "Fruit Types", 1, 0, labels, 1.0, 0.0, 0.0);
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/xypie.c,v 11.3 2001/08/23 20:15:17 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/lippic/xlippic/xypie.c,v $
* CREATED: 22-Jun-84 rpl    LAST UPDATE: 22-Jun-84 rpl	MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include "gvt.h"
#include "xlippic.h"

#define	INF	1.e+37
#define	F	.25

/* define size of pie and offset for labels */
#define TITLE_ROOM 30
#define TOP 780
#define LABEL_ROOM 114
#define MIN_RADIUS 20
#define CHAR_HEIGHT_PLUS 25

/* function declarations */
static void PIscale (/* int *pu0, int *pv0, int *pr0, int *pxtitle,
		       int *pytitle, int lflag, int *ploffset,
		       float size, float xoffset, float yoffset */);
static void PIplot (/* int u0, int v0, int r0, float *y, int n, int stype,
		      int lloc, char **labels, int loffset */);
static void PItitle(/* int u, int v, char *title */);


void
xypie(y, n, graph_title, shading_type, label_location, labels,
        size_fract, xoffset_fract, yoffset_fract)

float y[];
int n;
char *graph_title;
int shading_type, label_location;
char *labels[];
float size_fract, xoffset_fract, yoffset_fract;
{


	int u0, v0, r0;
	int xtitle, ytitle;
	int loffset;		/* offset from pie to label center */

	greset();
	PIscale( &u0, &v0, &r0, &xtitle, &ytitle, label_location, &loffset,
			   size_fract, xoffset_fract, yoffset_fract);
	PIplot( u0, v0, r0, y, n, shading_type, 
			   label_location, labels, loffset);
	PItitle(xtitle, ytitle, graph_title);


}
static void
PIscale( pu0, pv0, pr0, pxtitle, pytitle, lflag, ploffset, size, xoffset, yoffset)
int *pu0;
int *pv0;
int *pr0;
int *pxtitle, *pytitle;
int lflag;
int *ploffset;
float size, xoffset, yoffset;
{
    int botx;
    int boty;
    int width;
    int radius;
    int ytitle;
    int twidth;
    int lwidth;

    width = TOP*size;
    botx = xoffset*TOP;
    boty = yoffset*TOP;
    twidth = TITLE_ROOM;
    lwidth = LABEL_ROOM;

    if (lflag == 0)
	    {
	    radius = (width - 2*lwidth)/2;
	    ytitle = boty + width - CHAR_HEIGHT_PLUS/2;
	    }
    else
	    {
	    radius = (width - twidth)/2;
	    ytitle = boty + width - twidth/2;
	    }

    if (radius < MIN_RADIUS)
	radius = MIN_RADIUS;

    *pr0 = radius;
    *pu0 = botx +width/2;
    *pv0 = boty + (width -twidth)/2;
    *pxtitle = *pu0;
    *pytitle = ytitle;
    *ploffset = lwidth/2;

}
#define NARCS 20
#define RAD_245 4.27
#define RAD_315 5.49
static void
PIplot( u0, v0, r0, y, n, stype, lloc, labels, loffset)
int u0, v0, r0, n;
float y[];
int stype, lloc;
char *labels[];
int loffset;
{
	int i,r;
	int u1, v1;
	int abegin, aend;
	int aradius, astep;
	float total;
	double rad_begin, rad_end;
	double rad_middle;
	double dtorad;

	dtorad= 3.14159*2.0/360.0;

	gtexture(0);

	/* form sum */
	total = 0;
	for(i=0; i< n; i++)
		total +=y[i];

	astep = r0/NARCS;
	abegin = 0;
	rad_begin = 0.0;
	uvcircle(u0, v0, r0);

	/* plot pie slices 
	-----------------------*/
	for(i=0; i< n; i++)
		{
		/* ending angle and radian */
		if (i==n-1)
			aend = 360;
		else
			aend = (int)((y[i]/total)*360.0) + abegin;
		rad_end = aend*dtorad;

		/* bottom solid line */
		u1 = u0 + (int)(r0*cos(rad_begin));
		v1 = v0 + (int)(r0*sin(rad_begin));
		uvline( u0, v0, u1, v1);

		/* solid pie area */
		if ( i%2 == 0 && stype == 1)
		for(aradius=astep; aradius<r0; aradius+=astep)
			uvarc(u0, v0, aradius, abegin, aend);

		/* put in labels */
		  if ( lloc == 0)
		    {
		    rad_middle = (rad_begin + rad_end)*0.5;
		    /* label on bottom of graph is too low */
		    if ( rad_middle > RAD_245 && rad_middle < RAD_315)
			r = r0 + CHAR_HEIGHT_PLUS;
		    else
			r = r0 + loffset;

		    u1 = u0 + (int)(r*cos(rad_middle));
		    v1 = v0 + (int)(r*sin(rad_middle));
		    }
		  else
		    {
		    u1 = u0 + (int)((r0/2)*cos((rad_begin+rad_end)/2.0));
		    v1 = v0 + (int)((r0/2)*sin((rad_begin+rad_end)/2.0));
		    }
	    

		  /* labels */
		  if ( labels != 0)
		      uvprintf(u1, v1, 2, "%s", labels[i]);

		  /* percentages */
		  /* complex because printf doesn't allow final '%' */
		  uvmove(u1, v1);
		  /* linefeed */
		  if (labels !=0)
			grcmove(1,0);
		  gprintf(2, "%4.1f", y[i]*100.0/total);
		  uvmove(u1, v1);
		  if (labels !=0)
			grcmove(1,0);
		  grcmove(0,3);
		  gputc(1,'%');
		  /*
		  sprintf(dummy, "%4.1f", y[i]*100.0/total);
		  dummy[5] = '%';
		  dummy[6] = '\0';
		  uvmove(u1, v1);
		  if (labels !=0)
			grcmove(1,0);
		  grcmove(0,-3);
		  for(j=0;j<6;j++)
			  {
			  gputc(0, dummy[j]);
			  grcmove(0,1);
			  }
		*/

		abegin = aend;
	        rad_begin = rad_end;
		}
}
static void
PItitle(u, v, title)
int u,v;
char *title;
{
/*
aprintf("Xtitle = %d, Ytitle=%d\n", u,v);
*/
uvprintf(u, v, 2, "%s", title);
}
