void window_callback(Widget w, XtPointer client, XtPointer call);
void window_request(String request);
void read_window_defaults(int argc, char **argv);
void write_window_defaults(FILE *default_file);
String window_get_plot_label(String);
String window_get_args();
String window_get_important_args();

typedef struct {
    int int_flag;
    int int_radio;
    int int_text;
    char * int_text_string;
    float float_text;
    char *float_text_string;
    char *stringText;
    int *int_list;
    char *int_list_string;
    float *float_list;
    char *float_list_string;
    char **string_list;
    char *string_list_string;
    /* axes scales */
    int autoscale;
    float xmin;
    char *xmin_string;
    float xmax;
    char *xmax_string;
    float xstep;
    char *xstep_string;
    float ymin;
    char *ymin_string;
    float ymax;
    char *ymax_string;
    float ystep;
    char *ystep_string;
} WindowParams;

 
void
window_callback(Widget w, XtPointer client, XtPointer call)
{
    static int int_text_buttons_mapped = False;
    static int float_text_buttons_mapped = False;
    static int stringText_buttons_mapped = False;
    static int int_list_buttons_mapped = False;
    static int float_list_buttons_mapped = False;
    static int string_list_buttons_mapped = False;
    /* a set for the axes scales */
    static int xmin_buttons_mapped = False;
    static int xmax_buttons_mapped = False;
    static int xstep_buttons_mapped = False;
    static int ymin_buttons_mapped = False;
    static int ymax_buttons_mapped = False;
    static int ystep_buttons_mapped = False;

    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    WindowWidgets *ip;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
    if (starting){
	/*
	 * toggle callbacks for flag
	 */
    } else if (!strcmp(client, "int_flagToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "window_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	window_params.int_flag = state->set;
	/*
	 * end of toggle callbacks for flag
	 */

	/*
	 * toggle callbacks for radio
	 */
    } else if (!strcmp(client, "int_radio1Toggle") ||
	       !strcmp(client, "int_radio2Toggle") ||
	       !strcmp(client, "int_radio3Toggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "window_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set) window_params.int_radio =
	  get_radio_value(ip->int_radioRadioBox);
	/*
	 * end of toggle callbacks for radio
	 */

	/*
	 * text callbacks for integers
	 */
    } else if (!strcmp(client, "int_textText")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	int_text_buttons_mapped =
	  okTextValueChanged(ip->int_textText, ip->int_textOKPBtn,
			     ip->int_textCancelPBtn,
			     ip->windowDialogShell,
			     int_text_buttons_mapped,
			     window_params.int_text_string);
	
    } else if ((!strcmp(client, "int_textTextAct")) ||
	       (!strcmp(client, "int_textOKPBtn"))) {
	String int_textTextval;

	/*fprintf (stderr, "window_callback: %s\n", client );*/
	int_text_buttons_mapped =
	  okTextActivate(ip->int_textText, ip->int_textOKPBtn,
			 ip->int_textCancelPBtn, &int_textTextval);
	/* I won't actually be using this copy */
	XtFree (int_textTextval);
	
	/* verify the string as an integer */
	window_params.int_text =
	  set_atoi_value(ip->int_textText, "int_text_set",
			     window_params.int_text);

	/* save the current text for future comparisons */
	XtFree(window_params.int_text_string);
	window_params.int_text_string = get_string_text(ip->int_textText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "int_textCancelPBtn")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	int_text_buttons_mapped = 
	  okTextCancel(ip->int_textText, ip->int_textOKPBtn,
		       ip->int_textCancelPBtn,
		       window_params.int_text_string);
	/* 
	 * end of text callbacks for integers
	 */
	
	/*
	 * text callbacks for floats
	 */
    } else if (!strcmp(client, "float_textText")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	float_text_buttons_mapped =
	  okTextValueChanged(ip->float_textText, ip->float_textOKPBtn,
			     ip->float_textCancelPBtn,
			     ip->windowDialogShell,
			     float_text_buttons_mapped,
			     window_params.float_text_string);
	
    } else if ((!strcmp(client, "float_textTextAct")) ||
	       (!strcmp(client, "float_textOKPBtn"))) {
	String float_textTextval;

	/*fprintf (stderr, "window_callback: %s\n", client );*/
	float_text_buttons_mapped =
	  okTextActivate(ip->float_textText, ip->float_textOKPBtn,
			 ip->float_textCancelPBtn, &float_textTextval);
	/* I won't actually be using this copy */
	XtFree (float_textTextval);
	
	/* verify the string as a float */
	window_params.float_text =
	  set_atof_value(ip->float_textText, "float_text_set",
			 window_params.float_text);

	/* save the current text for future comparisons */
	XtFree(window_params.float_text_string);
	window_params.float_text_string = get_string_text(ip->float_textText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "float_textCancelPBtn")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	float_text_buttons_mapped = 
	  okTextCancel(ip->float_textText, ip->float_textOKPBtn,
		       ip->float_textCancelPBtn,
		       window_params.float_text_string);
	
	/*
	 * end of text callbacks for floats
	 */
	
	/* 
	 * text callbacks for strings
	 */
    } else if (!strcmp(client, "stringTextText")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	stringText_buttons_mapped =
	  okTextValueChanged(ip->stringTextText, ip->stringTextOKPBtn,
			     ip->stringTextCancelPBtn,
			     ip->windowDialogShell,
			     stringText_buttons_mapped,
			     window_params.stringText);
	
    } else if ((!strcmp(client, "stringTextTextAct")) ||
	       (!strcmp(client, "stringTextOKPBtn"))) {
	String stringTextTextval;

	/*fprintf (stderr, "window_callback: %s\n", client );*/
	stringText_buttons_mapped =
	  okTextActivate(ip->stringTextText, ip->stringTextOKPBtn,
			 ip->stringTextCancelPBtn, &stringTextTextval);
	
	XtFree(window_params.stringText);
	window_params.stringText = XtNewString(stringTextTextval);
	XtFree (stringTextTextval);
	
	/* do any other things needed */

    } else if (!strcmp(client, "stringTextCancelPBtn")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	stringText_buttons_mapped = 
	  okTextCancel(ip->stringTextText, ip->stringTextOKPBtn,
		       ip->stringTextCancelPBtn,
		       window_params.stringText);
	
	/*
	 * end of text callbacks for strings
	 */

	/* 
	 * text callbacks for int lists
	 */
    } else if (!strcmp(client, "int_listText")) {
	/*fprintf (stderr, "window_callback: %s\n", client);*/
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	int_list_buttons_mapped =
	  okTextValueChanged(ip->int_listText, ip->int_listOKPBtn,
			     ip->int_listCancelPBtn,
			     ip->windowDialogShell,
			     int_list_buttons_mapped,
			     window_params.int_list_string);
    } else if ((!strcmp(client, "int_listTextAct")) ||
	       (!strcmp(client, "int_listOKPBtn"))) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	String int_listTextval;
	int_list_buttons_mapped =
	  okTextActivate(ip->int_listText,
			 ip->int_listOKPBtn,
			 ip->int_listCancelPBtn,
			 &int_listTextval);
	
	if(window_params.int_list != NULL)
	  deallocate_ivector(window_params.int_list);
	/* take spaces and tabs out of the list before handing it to nnargs */
	if(int_listTextval != NULL){
	    char *int_listTemp = int_listTextval;
	    int_listTextval = strtok(int_listTextval, " \t");
	    window_params.int_list =
	      nnargs_get_int_alloc_list(int_listTextval);
	    XtFree(int_listTemp);
	}
	
	/* make a copy of the string as it has been interpretted */
	XtFree(window_params.int_list_string);
	window_params.int_list_string =
	  make_int_string(window_params.int_list);
	show_string_text(ip->int_listText,
			 window_params.int_list_string);
	
	/* do anything else necessary */
	
    } else if (!strcmp(client, "int_listCancelPBtn")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	int_list_buttons_mapped = 
	  okTextCancel(ip->int_listText, ip->int_listOKPBtn,
		       ip->int_listCancelPBtn,
		       window_params.int_list_string);
    	/*
	 * end of text callbacks for int lists
	 */

	/*
	 * text callbacks for float lists
	 */
    } else if (!strcmp(client, "float_listText")) {
	/*fprintf (stderr, "window_callback: %s\n", client);*/
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	float_list_buttons_mapped =
	  okTextValueChanged(ip->float_listText, ip->float_listOKPBtn,
			     ip->float_listCancelPBtn,
			     ip->windowDialogShell,
			     float_list_buttons_mapped,
			     window_params.float_list_string);
    } else if ((!strcmp(client, "float_listTextAct")) ||
	       (!strcmp(client, "float_listOKPBtn"))) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	String float_listTextval;
	float_list_buttons_mapped =
	  okTextActivate(ip->float_listText,
			 ip->float_listOKPBtn,
			 ip->float_listCancelPBtn,
			 &float_listTextval);
	
	if(window_params.float_list != NULL)
	  deallocate_vector(window_params.float_list);
	/* take spaces and tabs out of the list before handing it to nnargs */
	if(float_listTextval != NULL){
	    char *float_listTemp = float_listTextval;
	    float_listTextval = strtok(float_listTextval, " \t");
	    window_params.float_list =
	      nnargs_get_float_alloc_list(float_listTextval);
	    XtFree(float_listTemp);
	}
	
	/* make a copy of the string as it has been interpretted */
	XtFree(window_params.float_list_string);
	window_params.float_list_string =
	  make_float_string(window_params.float_list);
	show_string_text(ip->float_listText,
			 window_params.float_list_string);
	
	/* do anything else necessary */
	
    } else if (!strcmp(client, "float_listCancelPBtn")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	float_list_buttons_mapped = 
	  okTextCancel(ip->float_listText, ip->float_listOKPBtn,
		       ip->float_listCancelPBtn,
		       window_params.float_list_string);
	/*
	 * end of text callbacks for float lists
	 */
    
	/*
	 * text callbacks for string lists
	 */
    } else if (!strcmp(client, "string_listText")) {
	/*fprintf (stderr, "window_callback: %s\n", client);*/
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	string_list_buttons_mapped =
	  okTextValueChanged(ip->string_listText, ip->string_listOKPBtn,
			     ip->string_listCancelPBtn,
			     ip->windowDialogShell,
			     string_list_buttons_mapped,
			     window_params.string_list_string);
    } else if ((!strcmp(client, "string_listTextAct")) ||
	       (!strcmp(client, "string_listOKPBtn"))) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	String string_listTextval;
	string_list_buttons_mapped =
	  okTextActivate(ip->string_listText,
			 ip->string_listOKPBtn,
			 ip->string_listCancelPBtn,
			 &string_listTextval);
	
	if(window_params.string_list != NULL)
	  deallocate_string_vector(window_params.string_list,
				   atoi(window_params.string_list[0])+1);
	window_params.string_list = NULL;
	/* take spaces and tabs out of the list before handing it to nnargs */
	if(string_listTextval != NULL){
	    char *string_listTemp = string_listTextval;
	    string_listTextval = strtok(string_listTextval, " \t");
	    window_params.string_list =
	      nnargs_get_string_alloc_list(string_listTextval);
	    XtFree(string_listTemp);
	}

	/* make a copy of the string as it has been interpretted */
	XtFree(window_params.string_list_string);
	window_params.string_list_string =
	  make_labels_string(window_params.string_list);
	show_string_text(ip->string_listText,
			 window_params.string_list_string);
	
	/* do anything else necessary */
 
    } else if (!strcmp(client, "string_listCancelPBtn")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	string_list_buttons_mapped = 
	  okTextCancel(ip->string_listText, ip->string_listOKPBtn,
		       ip->string_listCancelPBtn,
		       window_params.string_list_string);
	/*
	 * end of text callbacks for string lists
	 */

	/*
	 * start of axes scales callbacks
	 */
    } else if (!strcmp(client, "autoscaleToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "window_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	window_params.autoscale = state->set;
	gray_out_item(WindowClone->axesForm, state->set);
 
    } else if (!strcmp(client, "xminText")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	xmin_buttons_mapped =
	  okTextValueChanged(ip->xminText, ip->xminOKPBtn,
			     ip->xminCancelPBtn,
			     ip->windowDialogShell,
			     xmin_buttons_mapped,
			     window_params.xmin_string);
      
    } else if (!strcmp(client, "xminTextAct") ||
	       !strcmp(client, "xminOKPBtn")) {
	String xminTextval;

	/*fprintf (stderr, "window_callback: %s\n", client );*/
	xmin_buttons_mapped = okTextActivate(ip->xminText, ip->xminOKPBtn,
					     ip->xminCancelPBtn, &xminTextval);

	XtFree (xminTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	window_params.xmin = set_atof_value(ip->xminText, "window_xmin",
					  window_params.xmin);
	XtFree(window_params.xmin_string);
	window_params.xmin_string = get_string_text(ip->xminText);
 
    } else if (!strcmp(client, "xminCancelPBtn")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	xmin_buttons_mapped = 
	  okTextCancel(ip->xminText, ip->xminOKPBtn,
		       ip->xminCancelPBtn,
		       window_params.xmin_string);
 
    } else if (!strcmp(client, "xmaxText")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	xmax_buttons_mapped =
	  okTextValueChanged(ip->xmaxText, ip->xmaxOKPBtn,
			     ip->xmaxCancelPBtn,
			     ip->windowDialogShell,
			     xmax_buttons_mapped,
			     window_params.xmax_string);
      
    } else if (!strcmp(client, "xmaxTextAct") ||
	       !strcmp(client, "xmaxOKPBtn")) {
	String xmaxTextval;

	/*fprintf (stderr, "window_callback: %s\n", client );*/
	xmax_buttons_mapped = okTextActivate(ip->xmaxText, ip->xmaxOKPBtn,
					     ip->xmaxCancelPBtn, &xmaxTextval);
	
	XtFree (xmaxTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	window_params.xmax = set_atof_value(ip->xmaxText, "window_xmax",
					  window_params.xmax);
	XtFree(window_params.xmax_string);
	window_params.xmax_string = get_string_text(ip->xmaxText);
 
    } else if (!strcmp(client, "xmaxCancelPBtn")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	xmax_buttons_mapped = 
	  okTextCancel(ip->xmaxText, ip->xmaxOKPBtn,
		       ip->xmaxCancelPBtn,
		       window_params.xmax_string);
 
    } else if (!strcmp(client, "xstepText")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	xstep_buttons_mapped =
	  okTextValueChanged(ip->xstepText, ip->xstepOKPBtn,
			     ip->xstepCancelPBtn,
			     ip->windowDialogShell,
			     xstep_buttons_mapped,
			     window_params.xstep_string);
      
    } else if (!strcmp(client, "xstepTextAct") ||
	       !strcmp(client, "xstepOKPBtn")) {
	String xstepTextval;
      
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	xstep_buttons_mapped =
	  okTextActivate(ip->xstepText, ip->xstepOKPBtn,
			 ip->xstepCancelPBtn, &xstepTextval);

	XtFree (xstepTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	window_params.xstep = set_atof_value(ip->xstepText, "window_xstep",
					   window_params.xstep);
	XtFree(window_params.xstep_string);
	window_params.xstep_string = get_string_text(ip->xstepText);
 
    } else if (!strcmp(client, "xstepCancelPBtn")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	xstep_buttons_mapped = 
	  okTextCancel(ip->xstepText, ip->xstepOKPBtn,
		       ip->xstepCancelPBtn,
		       window_params.xstep_string);
	
    } else if (!strcmp(client, "yminText")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	ymin_buttons_mapped =
	  okTextValueChanged(ip->yminText, ip->yminOKPBtn,
			     ip->yminCancelPBtn,
			     ip->windowDialogShell,
			     ymin_buttons_mapped,
			     window_params.ymin_string);
      
    } else if (!strcmp(client, "yminTextAct") ||
	       !strcmp(client, "yminOKPBtn")) {
	String yminTextval;
      
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	ymin_buttons_mapped = okTextActivate(ip->yminText, ip->yminOKPBtn,
					     ip->yminCancelPBtn, &yminTextval);

	XtFree (yminTextval);/* I won't actually be using this copy */
	
	/* check that this value is ok */
	window_params.ymin = set_atof_value(ip->yminText, "window_ymin",
					  window_params.ymin);
	XtFree(window_params.ymin_string);
	window_params.ymin_string = get_string_text(ip->yminText);
	
    } else if (!strcmp(client, "yminCancelPBtn")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	ymin_buttons_mapped = 
	  okTextCancel(ip->yminText, ip->yminOKPBtn,
		       ip->yminCancelPBtn,
		       window_params.ymin_string);
	
    } else if (!strcmp(client, "ymaxText")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	ymax_buttons_mapped =
	  okTextValueChanged(ip->ymaxText, ip->ymaxOKPBtn,
			     ip->ymaxCancelPBtn,
			     ip->windowDialogShell,
			     ymax_buttons_mapped,
			     window_params.ymax_string);
      
    } else if (!strcmp(client, "ymaxTextAct") ||
	       !strcmp(client, "ymaxOKPBtn")) {
	String ymaxTextval;
      
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	ymax_buttons_mapped = okTextActivate(ip->ymaxText, ip->ymaxOKPBtn,
					     ip->ymaxCancelPBtn, &ymaxTextval);

	XtFree (ymaxTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	window_params.ymax = set_atof_value(ip->ymaxText, "window_ymax",
					  window_params.ymax);
	XtFree(window_params.ymax_string);
	window_params.ymax_string = get_string_text(ip->ymaxText);
 
    } else if (!strcmp(client, "ymaxCancelPBtn")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	ymax_buttons_mapped = 
	  okTextCancel(ip->ymaxText, ip->ymaxOKPBtn,
		       ip->ymaxCancelPBtn,
		       window_params.ymax_string);
 
    } else if (!strcmp(client, "ystepText")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	ystep_buttons_mapped =
	  okTextValueChanged(ip->ystepText, ip->ystepOKPBtn,
			     ip->ystepCancelPBtn,
			     ip->windowDialogShell,
			     ystep_buttons_mapped,
			     window_params.ystep_string);
      
    } else if (!strcmp(client, "ystepTextAct") ||
	       !strcmp(client, "ystepOKPBtn")) {
	String ystepTextval;
      
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	ystep_buttons_mapped =
	  okTextActivate(ip->ystepText, ip->ystepOKPBtn,
			 ip->ystepCancelPBtn, &ystepTextval);

	XtFree (ystepTextval);/* I won't actually be using this copy */
	
	/* check that this value is ok */
	window_params.ystep = set_atof_value(ip->ystepText, "window_ystep",
					   window_params.ystep);
	XtFree(window_params.ystep_string);
	window_params.ystep_string = get_string_text(ip->ystepText);
 
    } else if (!strcmp(client, "ystepCancelPBtn")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	ystep_buttons_mapped = 
	  okTextCancel(ip->ystepText, ip->ystepOKPBtn,
		       ip->ystepCancelPBtn,
		       window_params.ystep_string);
	/*
	 * end of axes scales callbacks
	 */
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	close_popup(ip->windowForm);

    } else {
	fprintf(stderr, "window_callback: unknown request: %s\n", request);
    }
}

void
window_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(WindowClone->windowForm);
    } else if (!strcmp(request, "init")){
	/* flag */
	window_params.int_flag = 0;
	/* radio */
	window_params.int_radio = 0;
	/* int text */
	window_params.int_text = 0;
	window_params.int_text_string = NULL;
	/* float text */
	window_params.float_text = 0.;
	window_params.float_text_string = NULL;
	/* string text */
	window_params.stringText = NULL;
	/* int_list text */
	window_params.int_list = NULL;
	window_params.int_list_string = NULL;
	/* float_list text */
	window_params.float_list = NULL;
	window_params.float_list_string = NULL;
	/* string_list text */
	window_params.string_list = NULL;
	window_params.string_list_string = NULL;

	/* axes scales */
	window_params.xmin = 0;
	window_params.xmin_string = NULL;
	window_params.xmax = 0;
	window_params.xmax_string = NULL;
	window_params.xstep = 0;
	window_params.xstep_string = NULL;
	window_params.ymin = 0;
	window_params.ymin_string = NULL;
	window_params.ymax = 0;
	window_params.ymax_string = NULL;
	window_params.ystep = 0;
	window_params.ystep_string = NULL;

    } else if (!strcmp(request, "display")){
	char string[10];
	/* flag */
	show_toggle_value(WindowClone->int_flagToggle, window_params.int_flag);
	/* radioBox */
	show_radio_value(WindowClone->int_radioRadioBox, window_params.int_radio);
	/* int text */
	XtFree(window_params.int_text_string);
	sprintf(string, "%d", window_params.int_text);
	window_params.int_text_string = XtNewString(string);
	show_string_text(WindowClone->int_textText,
			 window_params.int_text_string);
	/* float text */
	XtFree(window_params.float_text_string);
	sprintf(string, "%5.2g", window_params.float_text);
	window_params.float_text_string = XtNewString(string);
	show_string_text(WindowClone->float_textText,
			 window_params.float_text_string);
	/* string text */
	show_string_text(WindowClone->stringTextText,
			 window_params.stringText);
	/* int_list text */
	XtFree(window_params.int_list_string);
	window_params.int_list_string =
	  make_int_string(window_params.int_list);
	show_string_text(WindowClone->int_listText,
			 window_params.int_list_string);
	/* float_list text */
	XtFree(window_params.float_list_string);
	window_params.float_list_string =
	  make_float_string(window_params.float_list);
	show_string_text(WindowClone->float_listText,
			 window_params.float_list_string);
	/* string_list text */
	XtFree(window_params.string_list_string);
	window_params.string_list_string =
	  make_labels_string(window_params.string_list);
	show_string_text(WindowClone->string_listText,
			 window_params.string_list_string);
	
	/* axes scales */
	show_toggle_value (WindowClone->autoscaleToggle,
			   window_params.autoscale);
	gray_out_item(WindowClone->axesForm, window_params.autoscale);
	XtFree(window_params.xmin_string);
	sprintf(string, "%5.2g", window_params.xmin);
	window_params.xmin_string = XtNewString(string);
	show_string_text (WindowClone->xminText,
			  window_params.xmin_string);

	XtFree(window_params.xmax_string);
	sprintf(string, "%5.2g", window_params.xmax);
	window_params.xmax_string = XtNewString(string);
	show_string_text (WindowClone->xmaxText,
			  window_params.xmax_string);

	XtFree(window_params.xstep_string);
	sprintf(string, "%5.2g", window_params.xstep);
	window_params.xstep_string = XtNewString(string);
	show_string_text (WindowClone->xstepText,
			  window_params.xstep_string);

	XtFree(window_params.ymin_string);
	sprintf(string, "%5.2g", window_params.ymin);
	window_params.ymin_string = XtNewString(string);
	show_string_text (WindowClone->yminText,
			  window_params.ymin_string);

	XtFree(window_params.ymax_string);
	sprintf(string, "%5.2g", window_params.ymax);
	window_params.ymax_string = XtNewString(string);
	show_string_text (WindowClone->ymaxText,
			  window_params.ymax_string);

	XtFree(window_params.ystep_string);
	sprintf(string, "%5.2g", window_params.ystep);
	window_params.ystep_string = XtNewString(string);
	show_string_text (WindowClone->ystepText,
			 window_params.ystep_string);

    } else {
	fprintf(stderr, "window_request: unknown request: %s\n", request);
    }
}

String
window_get_plot_label(String string)
{
    string = StringConcat(string, "more stuff");
    return string;
}

static void
window_set_read_variables()
{
    window_params.int_flag = int_flag;
    window_params.int_radio = int_radio;
    window_params.int_text = int_text;
    window_params.float_text = float_text;
    window_params.stringText = stringText;
    if(window_params.int_list)
      deallocate_ivector(window_params.int_list);
    window_params.int_list = int_list;
    if(window_params.float_list)
      deallocate_vector(window_params.float_list);
    window_params.float_list = float_list;
    if(window_params.float_list)
      deallocate_string_vector(window_params.string_list,
			       atoi(window_params.string_list[0])+1);
    window_params.string_list = string_list;
    window_params.autoscale = autoscale;
    window_params.xmin = xmin;
    window_params.xmax = xmax;
    window_params.xstep = xstep;
    window_params.ymin = ymin;
    window_params.ymax = ymax;
    window_params.ystep = ystep;
}

static void
window_set_write_variables()
{
    int_flag = window_params.int_flag;
    int_radio = window_params.int_radio;
    int_text = window_params.int_text;
    float_text = window_params.float_text;
    stringText = window_params.stringText;
    int_list = window_params.int_list;
    float_list = window_params.float_list;
    string_list = window_params.string_list;
    autoscale = window_params.autoscale;
    xmin = window_params.xmin;
    xmax = window_params.xmax;
    xstep = window_params.xstep;
    ymin = window_params.ymin;
    ymax = window_params.ymax;
    ystep = window_params.ystep;
}

static ARG windowtab[] =
{
    {"int_flag", BOOLEAN, (int *)NULL,},
    {"int_radio", INT, (int *)NULL, },
    {"int_text", INT, (int *)NULL,},
    {"float_text", FLOAT, (int *)NULL,},
    {"stringText", STRING, (int *)NULL,},
    {"int_list", INT_ALLOC_LIST, (int *)NULL,},
    {"float_list", FLOAT_ALLOC_LIST, (int *)NULL,},
    {"string_list", STRING_ALLOC_LIST, (int *)NULL,},
    {"autoscale", FLAG, (int *)NULL, "autoscale plot"},
    {"xmin", FLOAT, (int *)NULL,},
    {"xmax", FLOAT, (int *)NULL,},
    {"xstep", FLOAT, (int *)NULL,},
    {"ymin", FLOAT, (int *)NULL,},
    {"ymax", FLOAT, (int *)NULL,},
    {"ystep", FLOAT, (int *)NULL,},
};
#define TABSIZE (sizeof(windowtab)/sizeof(ARG))

static void
window_set_defaults_variables(dp)
     WindowParams *dp;
{
    nnargs_set_variable("int_flag", (int *)&dp->int_flag, windowtab, TABSIZE);
    nnargs_set_variable("int_radio", (int *)&dp->int_radio, windowtab, TABSIZE);
    nnargs_set_variable("int_text", (int *)&dp->int_text, windowtab, TABSIZE);
    nnargs_set_variable("float_text", (int *)&dp->float_text, windowtab, TABSIZE);
    nnargs_set_variable("stringText", (int *)&dp->stringText, windowtab, TABSIZE);
    nnargs_set_variable("int_list", (int *)&dp->int_list, windowtab, TABSIZE);
    nnargs_set_variable("float_list", (int *)&dp->float_list, windowtab, TABSIZE);
    nnargs_set_variable("string_list", (int *)&dp->string_list, windowtab, TABSIZE);
    nnargs_set_variable("autoscale", &dp->autoscale, windowtab, TABSIZE);
    nnargs_set_variable("xmin", (int *)&dp->xmin, windowtab, TABSIZE);
    nnargs_set_variable("xmax", (int *)&dp->xmax, windowtab, TABSIZE);
    nnargs_set_variable("xstep", (int *)&dp->xstep, windowtab, TABSIZE);
    nnargs_set_variable("ymin", (int *)&dp->ymin, windowtab, TABSIZE);
    nnargs_set_variable("ymax", (int *)&dp->ymax, windowtab, TABSIZE);
    nnargs_set_variable("ystep", (int *)&dp->ystep, windowtab, TABSIZE);

}

void
read_window_defaults(argc, argv)
     int argc;
     char **argv;
{
    /* clear flags and strings which are not printed if they are NULL */
    window_params.int_flag = 0;
    if (window_params.stringText != NULL){
	XtFree(window_params.stringText);
	window_params.stringText = NULL;
    }
    if (window_params.int_list != NULL){
	deallocate_ivector(window_params.int_list);
	window_params.int_list = NULL;
    }
    if (window_params.float_list != NULL){
	deallocate_vector(window_params.float_list);
	window_params.float_list = NULL;
    }
    if (window_params.string_list != NULL){
	deallocate_string_vector(window_params.string_list,
				 atoi(window_params.string_list[0])+1);
	window_params.string_list = NULL;
    }
    window_params.autoscale = 0;
    /*OR*/
    int_flag = 0;
    stringText = NULL;
    int_list = NULL;
    /*OR*/ int_list[0] = 0;
    float_list= NULL;
    /*OR*/ float_list[0] = 0.;
    string_list= NULL;
    /*OR*/ string_list[0] = "0";

    window_set_defaults_variables(&window_params);

    if(argc > 0){
	argc = nnargs(argc, argv, windowtab, TABSIZE);
	/*OR*/
	argc = nnargs(argc, argv, window_tab, WINDOW_TABSIZE);
	window_set_read_variables();
	
	/* copy the strings into new memory */
	window_params.stringText = XtNewString(window_params.stringText);
	/* the lists are properly copied into new space by nnargs */

    } else {
	/* set any defaults */
    }

    window_request("display");
}

void
write_window_defaults(FILE *default_file)
{
    window_set_defaults_variables(&window_params);
    /* OR */
    window_set_write_variables();

    if(default_file != NULL){
	fprintf(default_file, "window %s",
		nnargs_get_flags(windowtab, TABSIZE));
	/* OR */
	fprintf(default_file, "window %s",
		nnargs_get_flags(window_tab, WINDOW_TABSIZE));
    }
}

String
window_get_args ()
{
    window_set_write_variables();
    return (nnargs_get_part_flags(window_tab, WINDOW_TABSIZE));
}

String
window_get_important_args ()
{
    window_set_write_variables();
    return (nnargs_get_flags_one_line(window_tab, WINDOW_TABSIZE));
}
