/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/artmap/artmap_callback.c,v 11.4 2001/08/23 20:15:29 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                       Lnknet Window Interface                        ****
 ****                           Artmap Window                              ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  14-Sep-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include "../main/lnknet_subs.h"
#include "artmap.h"
#include <nnlib.h>
#include <artmap_args.h>
/*
 * External Functions
 */
#include "../plots/plots_func.h"
#include "artmap_func.h"
 
/*
 * Externals
 */
extern ArtmapWidgets *ArtmapClone;
#include "artmap_params.h"
ArtmapParams artmap_params; /* this is the real one */

/*
 * Statics
 */
static void artmap_set_read_variables();
static void artmap_set_write_variables();
static void artmap_set_defaults_variables(ArtmapParams *);
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: artmap_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   artmap_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  20-Nov-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
artmap_callback(Widget w, XtPointer client, XtPointer call)
{
    /* SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    ArtmapWidgets *ip;
    static int alpha_buttons_mapped = False;
    static int beta_buttons_mapped = False;
    static int artATrain_buttons_mapped = False;
    static int artATest_buttons_mapped = False;
    static int artBTrainTest_buttons_mapped = False;
    static int epochs_buttons_mapped = False;
    static int pruneThresh_buttons_mapped = False;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "complementCodeToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "artmap_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	artmap_params.complementCode = state->set;

    } else if (!strcmp(client, "sumToOneToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "artmap_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	artmap_params.sumToOne = state->set;

    } else if (!strcmp(client, "alphaText")) {
	/*fprintf (stderr, "artmap_callback: %s\n", client );*/
	alpha_buttons_mapped =
	  okTextValueChanged(ip->alphaText, ip->alphaOKPBtn,
			     ip->alphaCancelPBtn,
			     ip->artmapDialogShell,
			     alpha_buttons_mapped,
			     artmap_params.alpha_string);
 
  } else if ((!strcmp(client, "alphaTextAct")) ||
	     (!strcmp(client, "alphaOKPBtn"))) {
      String alphaTextval;

      /*fprintf (stderr, "artmap_callback: %s\n", client );*/
	alpha_buttons_mapped =
	  okTextActivate(ip->alphaText, ip->alphaOKPBtn,
			 ip->alphaCancelPBtn, &alphaTextval);
	/* I won't actually be using this copy */
	XtFree (alphaTextval);
	
	/* verify the string as an integer */
	artmap_params.alpha =
	  set_atof_range_value(ip->alphaText, "alpha_set",
			       artmap_params.alpha, 0., 1.);

	/* save the current text for future comparisons */
	XtFree(artmap_params.alpha_string);
	artmap_params.alpha_string = get_string_text(ip->alphaText);
 
  } else if (!strcmp(client, "alphaCancelPBtn")) {
    /*fprintf (stderr, "artmap_callback: %s\n", client );*/
	alpha_buttons_mapped = 
	  okTextCancel(ip->alphaText, ip->alphaOKPBtn,
		       ip->alphaCancelPBtn,
		       artmap_params.alpha_string);

    } else if (!strcmp(client, "betaText")) {
	/*fprintf (stderr, "artmap_callback: %s\n", client );*/
	beta_buttons_mapped =
	  okTextValueChanged(ip->betaText, ip->betaOKPBtn,
			     ip->betaCancelPBtn,
			     ip->artmapDialogShell,
			     beta_buttons_mapped,
			     artmap_params.beta_string);
 
  } else if ((!strcmp(client, "betaTextAct")) ||
	     (!strcmp(client, "betaOKPBtn"))) {
      String betaTextval;

      /*fprintf (stderr, "artmap_callback: %s\n", client );*/
	beta_buttons_mapped =
	  okTextActivate(ip->betaText, ip->betaOKPBtn,
			 ip->betaCancelPBtn, &betaTextval);
	/* I won't actually be using this copy */
	XtFree (betaTextval);
	
	/* verify the string as an integer */
	artmap_params.beta =
	  set_atof_range_value(ip->betaText, "beta_set",
			       artmap_params.beta, 0., 1.);

	/* save the current text for future comparisons */
	XtFree(artmap_params.beta_string);
	artmap_params.beta_string = get_string_text(ip->betaText);
 
  } else if (!strcmp(client, "betaCancelPBtn")) {
    /*fprintf (stderr, "artmap_callback: %s\n", client );*/
	beta_buttons_mapped = 
	  okTextCancel(ip->betaText, ip->betaOKPBtn,
		       ip->betaCancelPBtn,
		       artmap_params.beta_string);

    } else if (!strcmp(client, "artATrainText")) {
	/*fprintf (stderr, "artmap_callback: %s\n", client );*/
	artATrain_buttons_mapped =
	  okTextValueChanged(ip->artATrainText, ip->artATrainOKPBtn,
			     ip->artATrainCancelPBtn,
			     ip->artmapDialogShell,
			     artATrain_buttons_mapped,
			     artmap_params.artATrain_string);
 
  } else if ((!strcmp(client, "artATrainTextAct")) ||
	     (!strcmp(client, "artATrainOKPBtn"))) {
      String artATrainTextval;

      /*fprintf (stderr, "artmap_callback: %s\n", client );*/
	artATrain_buttons_mapped =
	  okTextActivate(ip->artATrainText, ip->artATrainOKPBtn,
			 ip->artATrainCancelPBtn, &artATrainTextval);
	/* I won't actually be using this copy */
	XtFree (artATrainTextval);
	
	/* verify the string as an integer */
	artmap_params.artATrain =
	  set_atof_range_value(ip->artATrainText, "artATrain_set",
			       artmap_params.artATrain, 0., 1.);

	/* save the current text for future comparisons */
	XtFree(artmap_params.artATrain_string);
	artmap_params.artATrain_string = get_string_text(ip->artATrainText);
 
  } else if (!strcmp(client, "artATrainCancelPBtn")) {
    /*fprintf (stderr, "artmap_callback: %s\n", client );*/
	artATrain_buttons_mapped = 
	  okTextCancel(ip->artATrainText, ip->artATrainOKPBtn,
		       ip->artATrainCancelPBtn,
		       artmap_params.artATrain_string);

    } else if (!strcmp(client, "artATestText")) {
	/*fprintf (stderr, "artmap_callback: %s\n", client );*/
	artATest_buttons_mapped =
	  okTextValueChanged(ip->artATestText, ip->artATestOKPBtn,
			     ip->artATestCancelPBtn,
			     ip->artmapDialogShell,
			     artATest_buttons_mapped,
			     artmap_params.artATest_string);
 
  } else if ((!strcmp(client, "artATestTextAct")) ||
	     (!strcmp(client, "artATestOKPBtn"))) {
      String artATestTextval;

      /*fprintf (stderr, "artmap_callback: %s\n", client );*/
	artATest_buttons_mapped =
	  okTextActivate(ip->artATestText, ip->artATestOKPBtn,
			 ip->artATestCancelPBtn, &artATestTextval);
	/* I won't actually be using this copy */
	XtFree (artATestTextval);
	
	/* verify the string as an integer */
	artmap_params.artATest =
	  set_atof_range_value(ip->artATestText, "artATest_set",
			       artmap_params.artATest, 0., 1.);

	/* save the current text for future comparisons */
	XtFree(artmap_params.artATest_string);
	artmap_params.artATest_string = get_string_text(ip->artATestText);
 
  } else if (!strcmp(client, "artATestCancelPBtn")) {
    /*fprintf (stderr, "artmap_callback: %s\n", client );*/
	artATest_buttons_mapped = 
	  okTextCancel(ip->artATestText, ip->artATestOKPBtn,
		       ip->artATestCancelPBtn,
		       artmap_params.artATest_string);

    } else if (!strcmp(client, "artBTrainTestText")) {
	/*fprintf (stderr, "artmap_callback: %s\n", client );*/
	artBTrainTest_buttons_mapped =
	  okTextValueChanged(ip->artBTrainTestText, ip->artBTrainTestOKPBtn,
			     ip->artBTrainTestCancelPBtn,
			     ip->artmapDialogShell,
			     artBTrainTest_buttons_mapped,
			     artmap_params.artBTrainTest_string);
 
  } else if ((!strcmp(client, "artBTrainTestTextAct")) ||
	     (!strcmp(client, "artBTrainTestOKPBtn"))) {
      String artBTrainTestTextval;

      /*fprintf (stderr, "artmap_callback: %s\n", client );*/
	artBTrainTest_buttons_mapped =
	  okTextActivate(ip->artBTrainTestText, ip->artBTrainTestOKPBtn,
			 ip->artBTrainTestCancelPBtn, &artBTrainTestTextval);
	/* I won't actually be using this copy */
	XtFree (artBTrainTestTextval);
	
	/* verify the string as an integer */
	artmap_params.artBTrainTest =
	  set_atof_range_value(ip->artBTrainTestText, "artBTrainTest_set",
			       artmap_params.artBTrainTest, 0., 1.);

	/* save the current text for future comparisons */
	XtFree(artmap_params.artBTrainTest_string);
	artmap_params.artBTrainTest_string = get_string_text(ip->artBTrainTestText);
 
  } else if (!strcmp(client, "artBTrainTestCancelPBtn")) {
    /*fprintf (stderr, "artmap_callback: %s\n", client );*/
	artBTrainTest_buttons_mapped = 
	  okTextCancel(ip->artBTrainTestText, ip->artBTrainTestOKPBtn,
		       ip->artBTrainTestCancelPBtn,
		       artmap_params.artBTrainTest_string);

    } else if (!strcmp(client, "epochsText")) {
	/*fprintf (stderr, "artmap_callback: %s\n", client );*/
	epochs_buttons_mapped =
	  okTextValueChanged(ip->epochsText, ip->epochsOKPBtn,
			     ip->epochsCancelPBtn,
			     ip->artmapDialogShell,
			     epochs_buttons_mapped,
			     artmap_params.epochs_string);
 
  } else if ((!strcmp(client, "epochsTextAct")) ||
	     (!strcmp(client, "epochsOKPBtn"))) {
      String epochsTextval;

      /*fprintf (stderr, "artmap_callback: %s\n", client );*/
	epochs_buttons_mapped =
	  okTextActivate(ip->epochsText, ip->epochsOKPBtn,
			 ip->epochsCancelPBtn, &epochsTextval);
	/* I won't actually be using this copy */
	XtFree (epochsTextval);
	
	/* verify the string as an integer */
	artmap_params.epochs =
	  set_atoi_low_value(ip->epochsText, "epochs_set",
			     artmap_params.epochs, 0);

	/* save the current text for future comparisons */
	XtFree(artmap_params.epochs_string);
	artmap_params.epochs_string = get_string_text(ip->epochsText);
 
  } else if (!strcmp(client, "epochsCancelPBtn")) {
    /*fprintf (stderr, "artmap_callback: %s\n", client );*/
	epochs_buttons_mapped = 
	  okTextCancel(ip->epochsText, ip->epochsOKPBtn,
		       ip->epochsCancelPBtn,
		       artmap_params.epochs_string);

    } else if (!strcmp(client, "pruneThreshText")) {
	/*fprintf (stderr, "artmap_callback: %s\n", client );*/
	pruneThresh_buttons_mapped =
	  okTextValueChanged(ip->pruneThreshText, ip->pruneThreshOKPBtn,
			     ip->pruneThreshCancelPBtn,
			     ip->artmapDialogShell,
			     pruneThresh_buttons_mapped,
			     artmap_params.pruneThresh_string);
 
  } else if ((!strcmp(client, "pruneThreshTextAct")) ||
	     (!strcmp(client, "pruneThreshOKPBtn"))) {
      String pruneThreshTextval;

      /*fprintf (stderr, "artmap_callback: %s\n", client );*/
	pruneThresh_buttons_mapped =
	  okTextActivate(ip->pruneThreshText, ip->pruneThreshOKPBtn,
			 ip->pruneThreshCancelPBtn, &pruneThreshTextval);
	/* I won't actually be using this copy */
	XtFree (pruneThreshTextval);
	
	/* verify the string as an integer */
	artmap_params.pruneThresh =
	  set_atoi_low_value(ip->pruneThreshText, "pruneThresh_set",
			     artmap_params.pruneThresh, -1);

	/* save the current text for future comparisons */
	XtFree(artmap_params.pruneThresh_string);
	artmap_params.pruneThresh_string = get_string_text(ip->pruneThreshText);
 
  } else if (!strcmp(client, "pruneThreshCancelPBtn")) {
    /*fprintf (stderr, "artmap_callback: %s\n", client );*/
	pruneThresh_buttons_mapped = 
	  okTextCancel(ip->pruneThreshText, ip->pruneThreshOKPBtn,
		       ip->pruneThreshCancelPBtn,
		       artmap_params.pruneThresh_string);

    } else if (!strcmp(client, "trainCycleToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "artmap_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	artmap_params.trainCycle = state->set;

    } else if (!strcmp(client, "pruneToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "artmap_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	artmap_params.prune = state->set;
	gray_out_item(ip->pruneThreshForm, !artmap_params.prune);

  } else if (!strcmp(client, "closePBtn")) {
      /*fprintf (stderr, "artmap_callback: %s\n", client);*/
      close_popup(ArtmapClone->artmapForm);

  } else {
      fprintf (stderr, "artmap_callback: unknown client: %s\n", (char *)client);
  }
}

void
artmap_request(String request)
{
    if (!strcmp(request, "open")) {
	open_popup(ArtmapClone->artmapForm);

    } else if (!strcmp(request, "init")){
	/* this should only get called once, before ANYTHING 
	   gets allocated or there will be memory leaks */
	artmap_params.complementCode = 0;
	artmap_params.sumToOne = 0;
	artmap_params.alpha = 0.;
	artmap_params.alpha_string = NULL;
	artmap_params.beta = 0.;
	artmap_params.beta_string = NULL;
	artmap_params.artATrain = 0.;
	artmap_params.artATrain_string = NULL;
	artmap_params.artATest = 0.;
	artmap_params.artATest_string = NULL;
	artmap_params.artBTrainTest = 0.;
	artmap_params.artBTrainTest_string = NULL;
	artmap_params.epochs = 0;
	artmap_params.epochs_string = NULL;
	artmap_params.trainCycle = 0;
	artmap_params.prune = 0;
	artmap_params.pruneThresh = 0;
	artmap_params.pruneThresh_string = NULL;

    } else if (!strcmp(request, "display")){
	char string[10]; /* for creating int and float text */

	show_toggle_value(ArtmapClone->complementCodeToggle, artmap_params.complementCode);
	show_toggle_value(ArtmapClone->sumToOneToggle, artmap_params.sumToOne);

	sprintf(string, "%.2g", artmap_params.alpha);
	XtFree(artmap_params.alpha_string);
	artmap_params.alpha_string = XtNewString(string);
	show_string_text(ArtmapClone->alphaText, artmap_params.alpha_string);

	sprintf(string, "%.2g", artmap_params.beta);
	XtFree(artmap_params.beta_string);
	artmap_params.beta_string = XtNewString(string);
	show_string_text(ArtmapClone->betaText, artmap_params.beta_string);

	sprintf(string, "%.2g", artmap_params.artATrain);
	XtFree(artmap_params.artATrain_string);
	artmap_params.artATrain_string = XtNewString(string);
	show_string_text(ArtmapClone->artATrainText, artmap_params.artATrain_string);

	sprintf(string, "%.2g", artmap_params.artATest);
	XtFree(artmap_params.artATest_string);
	artmap_params.artATest_string = XtNewString(string);
	show_string_text(ArtmapClone->artATestText, artmap_params.artATest_string);

	sprintf(string, "%.3g", artmap_params.artBTrainTest);
	XtFree(artmap_params.artBTrainTest_string);
	artmap_params.artBTrainTest_string = XtNewString(string);
	show_string_text(ArtmapClone->artBTrainTestText, artmap_params.artBTrainTest_string);

	sprintf(string, "%d", artmap_params.epochs);
	XtFree(artmap_params.epochs_string);
	artmap_params.epochs_string = XtNewString(string);
	show_string_text(ArtmapClone->epochsText, artmap_params.epochs_string);

	show_toggle_value(ArtmapClone->trainCycleToggle, artmap_params.trainCycle);
	show_toggle_value(ArtmapClone->pruneToggle, artmap_params.prune);
	gray_out_item(ArtmapClone->pruneThreshForm, !artmap_params.prune);

	sprintf(string, "%d", artmap_params.pruneThresh);
	XtFree(artmap_params.pruneThresh_string);
	artmap_params.pruneThresh_string = XtNewString(string);
	show_string_text(ArtmapClone->pruneThreshText, artmap_params.pruneThresh_string);

    } else {
	fprintf (stderr, "artmap_request: unknown request: %s\n", request);
    }
}

String
artmap_get_plot_label(String string)
{
    return string;
}

static ARG artmaptab[] =
{
    {"prune", BOOLEAN, (int *)NULL, },
};
#define TABSIZE (sizeof(artmaptab)/sizeof(ARG))

static void
artmap_set_read_variables()
{
    artmap_params.complementCode = !no_complement_code;
    artmap_params.sumToOne = input_sum_to_one;
    artmap_params.alpha = alpha;
    artmap_params.beta = beta;
    artmap_params.artATrain = ro_a_train;
    artmap_params.artATest = ro_a_test;
    artmap_params.artBTrainTest = ro_b;
    artmap_params.epochs = nepochs;
    artmap_params.pruneThresh = prune_threshold;
    artmap_params.trainCycle = train_cycle;
}

static void
artmap_set_write_variables()
{
    no_complement_code = !artmap_params.complementCode;
    input_sum_to_one = artmap_params.sumToOne;
    alpha = artmap_params.alpha;
    beta = artmap_params.beta;
    ro_a_train = artmap_params.artATrain;
    ro_a_test = artmap_params.artATest;
    ro_b = artmap_params.artBTrainTest;
    nepochs = artmap_params.epochs;
    prune_threshold = artmap_params.pruneThresh;
    train_cycle = artmap_params.trainCycle;
}

static void
artmap_set_defaults_variables(ArtmapParams *dp)
{
    nnargs_set_variable("prune", (int *)&dp->prune, artmaptab, TABSIZE);
}

void
read_artmap_defaults(int argc, char **argv)
{
    no_complement_code = 0;
    input_sum_to_one = 0;
    artmap_params.prune = 0;
    train_cycle = 0;
    artmap_set_defaults_variables(&artmap_params);

    if(argc > 0){
	argc = nnargs_part(argc, argv, artmaptab, TABSIZE);
	argc = nnargs(argc, argv, artmap_tab, ARTMAP_TABSIZE);
	artmap_set_read_variables();
    } else {
	artmap_params.complementCode = 1;
	artmap_params.sumToOne = 0;
	artmap_params.beta = 1.;
	artmap_params.alpha = 0.001;
	artmap_params.artATrain = 0.95;
	artmap_params.artATest = 0.9;
	artmap_params.artBTrainTest = 0.999;
	artmap_params.epochs = 5;
	artmap_params.trainCycle = 0;
	artmap_params.prune = 0;
	artmap_params.pruneThresh = -1;
    }
    artmap_request("display");
}

void
write_artmap_defaults(default_file)
     FILE *default_file;
{
    artmap_set_defaults_variables(&artmap_params);
    artmap_set_write_variables();
    if(default_file != NULL){
	fprintf(default_file, "artmap %s",
		nnargs_get_part_flags(artmaptab, TABSIZE));
	fprintf(default_file, "%s",
		nnargs_get_flags(artmap_tab, ARTMAP_TABSIZE));
    }
}

String
artmap_get_args ()
{
    artmap_set_write_variables();
    if(!artmap_params.prune){
	prune_threshold = -1;
    }
    return (nnargs_get_part_flags(artmap_tab, ARTMAP_TABSIZE));
}

String
artmap_get_important_args ()
{
    artmap_set_write_variables();
    if(!artmap_params.prune){
	prune_threshold = -1;
    }
    return (nnargs_get_flags_one_line(artmap_tab, ARTMAP_TABSIZE));
}
