/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/bintree/bintree_callback.c,v 11.4 2001/08/23 20:15:32 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                       Lnknet Window Interface                        ****
 ****                           Bintree Window                             ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  14-Sep-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include "../main/lnknet_subs.h"
#include "bintree.h"
#include <nnlib.h>
#include <bintree_args.h>
/*
 * External Functions
 */
#include "../plots/plots_func.h"
#include "bintree_func.h"
 
/*
 * Externals
 */
extern BintreeWidgets *BintreeClone;
#include "bintree_params.h"
BintreeParams bintree_params; /* this is the real one */

/*
 * Statics
 */
static void bintree_set_read_variables();
static void bintree_set_write_variables();
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: bintree_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   bintree_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  20-Nov-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
bintree_callback(Widget w, XtPointer client, XtPointer call)
{
    /* SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    BintreeWidgets *ip;
    static int leafMin_buttons_mapped = False;
    static int maxNodes_buttons_mapped = False;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "singleFeatureToggle") ||
	!strcmp(client, "linearToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "bintree_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set){
	    bintree_params.linear =
	      get_radio_value(ip->linearRadioBox);
	    set_plot_labels();
	}
	
    } else if (!strcmp(client, "trainFullToggle") ||
	       !strcmp(client, "stopEarlyToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "bintree_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set){
	    bintree_params.stopEarly =
	      get_radio_value(ip->stopEarlyRadioBox);
	    gray_out_item(ip->leafMinForm, !bintree_params.stopEarly);
	}
	
    } else if (!strcmp(client, "leafMinText")) {
	/*fprintf (stderr, "bintree_callback: %s\n", client );*/
	leafMin_buttons_mapped =
	  okTextValueChanged(ip->leafMinText, ip->leafMinOKPBtn,
			     ip->leafMinCancelPBtn,
			     ip->bintreeDialogShell,
			     leafMin_buttons_mapped,
			     bintree_params.leafMin_string);
	
    } else if ((!strcmp(client, "leafMinTextAct")) ||
	       (!strcmp(client, "leafMinOKPBtn"))) {
	String leafMinTextval;

	/*fprintf (stderr, "bintree_callback: %s\n", client );*/
	leafMin_buttons_mapped =
	  okTextActivate(ip->leafMinText, ip->leafMinOKPBtn,
			 ip->leafMinCancelPBtn, &leafMinTextval);
	/* I won't actually be using this copy */
	XtFree (leafMinTextval);
	
	/* verify the string as an integer */
	bintree_params.leafMin =
	  set_atoi_value(ip->leafMinText, "leafMin_set",
			     bintree_params.leafMin);

	/* save the current text for future comparisons */
	XtFree(bintree_params.leafMin_string);
	bintree_params.leafMin_string = get_string_text(ip->leafMinText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "leafMinCancelPBtn")) {
	/*fprintf (stderr, "bintree_callback: %s\n", client );*/
	leafMin_buttons_mapped = 
	  okTextCancel(ip->leafMinText, ip->leafMinOKPBtn,
		       ip->leafMinCancelPBtn,
		       bintree_params.leafMin_string);
	
    } else if (!strcmp(client, "testFullToggle") ||
	       !strcmp(client, "pruneToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "bintree_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set){
	    bintree_params.prune =
	      get_radio_value(ip->pruneRadioBox);
	    gray_out_item(ip->maxNodesForm, !bintree_params.prune);
	    set_plot_labels();
	}
	
    } else if (!strcmp(client, "maxNodesText")) {
	/*fprintf (stderr, "bintree_callback: %s\n", client );*/
	maxNodes_buttons_mapped =
	  okTextValueChanged(ip->maxNodesText, ip->maxNodesOKPBtn,
			     ip->maxNodesCancelPBtn,
			     ip->bintreeDialogShell,
			     maxNodes_buttons_mapped,
			     bintree_params.maxNodes_string);
	
    } else if ((!strcmp(client, "maxNodesTextAct")) ||
	       (!strcmp(client, "maxNodesOKPBtn"))) {
	String maxNodesTextval;

	/*fprintf (stderr, "bintree_callback: %s\n", client );*/
	maxNodes_buttons_mapped =
	  okTextActivate(ip->maxNodesText, ip->maxNodesOKPBtn,
			 ip->maxNodesCancelPBtn, &maxNodesTextval);
	/* I won't actually be using this copy */
	XtFree (maxNodesTextval);
	
	/* verify the string as an integer */
	bintree_params.maxNodes =
	  set_atoi_value(ip->maxNodesText, "maxNodes_set",
			     bintree_params.maxNodes);

	/* save the current text for future comparisons */
	XtFree(bintree_params.maxNodes_string);
	bintree_params.maxNodes_string = get_string_text(ip->maxNodesText);
 
	/* do any other things needed */
	set_plot_labels();

    } else if (!strcmp(client, "maxNodesCancelPBtn")) {
	/*fprintf (stderr, "bintree_callback: %s\n", client );*/
	maxNodes_buttons_mapped = 
	  okTextCancel(ip->maxNodesText, ip->maxNodesOKPBtn,
		       ip->maxNodesCancelPBtn,
		       bintree_params.maxNodes_string);

    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "bintree_callback: %s\n", client);*/
	close_popup(BintreeClone->bintreeForm);

    } else {
	fprintf (stderr, "bintree_callback: unknown client: %s\n", (char *)client);
    }
}

void
bintree_request(String request)
{
    if (!strcmp(request, "open")) {
	open_popup(BintreeClone->bintreeForm);

    } else if (!strcmp(request, "init")){
	bintree_params.linear = 0;
	bintree_params.stopEarly = 0;
	bintree_params.leafMin = 0;
	bintree_params.leafMin_string = NULL;
	bintree_params.prune = 0;
	bintree_params.maxNodes = 0;
	bintree_params.maxNodes_string = NULL;

    } else if (!strcmp(request, "display")){
	char string[10]; /* for creating int and float text */
	show_radio_value(BintreeClone->linearRadioBox, bintree_params.linear);
	show_radio_value(BintreeClone->stopEarlyRadioBox, bintree_params.stopEarly);
	gray_out_item(BintreeClone->leafMinForm, !bintree_params.stopEarly);

	XtFree(bintree_params.leafMin_string);
	sprintf(string, "%d", bintree_params.leafMin);
	bintree_params.leafMin_string = XtNewString(string);
	show_string_text(BintreeClone->leafMinText,
			 bintree_params.leafMin_string);

	show_radio_value(BintreeClone->pruneRadioBox, bintree_params.prune);
	gray_out_item(BintreeClone->maxNodesForm, !bintree_params.prune);

	XtFree(bintree_params.maxNodes_string);
	sprintf(string, "%d", bintree_params.maxNodes);
	bintree_params.maxNodes_string = XtNewString(string);
	show_string_text(BintreeClone->maxNodesText,
			 bintree_params.maxNodes_string);

    } else {
	fprintf (stderr, "bintree_request: unknown request: %s\n", request);
    }
}

String
bintree_get_plot_label(String string)
{
    char numString[10];
    if(bintree_params.linear)
      string = StringConcat(string, " Linear");
    if(bintree_params.prune){
	string = StringConcat(string, " Test nodes:");
	sprintf(numString, "%d", bintree_params.maxNodes);
	string = StringConcat(string, numString);
    } else {
	string = StringConcat(string, " Test nodes:all");
    }
    return string;
}

static void
bintree_set_read_variables()
{
    bintree_params.linear = linear;
    bintree_params.stopEarly = stop_train_early;
    bintree_params.leafMin = leaf_min_npatterns;
    bintree_params.prune = prune_tree;
    bintree_params.maxNodes = max_nodes;
}

static void
bintree_set_write_variables()
{
    linear = bintree_params.linear;
    stop_train_early = bintree_params.stopEarly;
    leaf_min_npatterns = bintree_params.leafMin;
    prune_tree = bintree_params.prune;
    max_nodes = bintree_params.maxNodes;
}

void
read_bintree_defaults(int argc, char **argv)
{
    linear = 0;
    stop_train_early = 0;
    prune_tree = 0;

    if(argc > 0){
	argc = nnargs(argc, argv, bintree_tab, BINTREE_TABSIZE);
	bintree_set_read_variables();
    } else {
	bintree_params.maxNodes = 100;
	bintree_params.linear = 0;
	bintree_params.stopEarly = 0;
	bintree_params.prune = 0;
	bintree_params.leafMin = 5;
    }
    bintree_request("display");
}

void
write_bintree_defaults(default_file)
     FILE *default_file;
{
    bintree_set_write_variables();
    if(default_file != NULL){
	fprintf(default_file, "bintree %s",
		nnargs_get_flags(bintree_tab, BINTREE_TABSIZE));
    }
}

String
bintree_get_args ()
{
    bintree_set_write_variables();
    return (nnargs_get_part_flags(bintree_tab, BINTREE_TABSIZE));
}

String
bintree_get_important_args ()
{
    bintree_set_write_variables();
    return (nnargs_get_flags_one_line(bintree_tab, BINTREE_TABSIZE));
}

void
bintree_store_plot_command(FILE *shell_file)
{
    if(bintree_params.prune){
	fprintf(shell_file, " -prune_tree");
	fprintf(shell_file, " -max_nodes %d", bintree_params.maxNodes);
    }
}
