/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/ccode/ccode_callback.c,v 11.5 2001/08/23 20:15:37 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                                ccode                                 ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  13-Nov-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <nnlib.h>
#include <Xm/Xm.h>
#include "ccode.h"
/*
 * External Functions
 */
#include "../main/lnknet_subs.h"
#include "../lnk/lnk_func.h"
#include "../reports/reports_func.h"
#include "ccode_func.h"
 
/*
 * Externals
 */
extern CcodeWidgets *CcodeClone;
#include "ccode_params.h"
ccodeParams ccode_params;
#include "../data/data_params.h"
extern DataParams data_params;
#include "../lnk/lnk_params.h"
extern LnkParams lnk_params;
#include "../reports/reports_params.h"
extern ReportsParams reports_params;
#include "../priors/priors_params.h"
extern PriorsParams priors_params;


/*
 * Statics
 */ 
static void ccode_set_defaults_variables(ccodeParams *dp);
static void ccode_run();
static int store_ccode_commands(FILE *shell_file);
#include <gen_2c_args.h>

/******************************************************************************
 ******************************************************************************
 *
 * Module Name: ccode_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   ccode_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  13-Nov-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
ccode_callback(Widget w, XtPointer client, XtPointer call)
{
    CcodeWidgets *ip;
    static int script_buttons_mapped = False;
    static int suffix_buttons_mapped = False;
    static int cFile_buttons_mapped = False;
  
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
  
	
    if (!strcmp(client, "scriptText")) {
	/*fprintf (stderr, "ccode_callback: %s\n", client );*/
	script_buttons_mapped =
	  okTextValueChanged(ip->scriptText, ip->scriptOKPBtn,
			     ip->scriptCancelPBtn,
			     ip->ccodeDialogShell,
			     script_buttons_mapped,
			     ccode_params.script);
	
    } else if ((!strcmp(client, "scriptTextAct")) ||
	       (!strcmp(client, "scriptOKPBtn"))) {
	String scriptTextval;

	/*fprintf (stderr, "ccode_callback: %s\n", client );*/
	script_buttons_mapped =
	  okTextActivate(ip->scriptText, ip->scriptOKPBtn,
			 ip->scriptCancelPBtn, &scriptTextval);
	XtFree(scriptTextval);
	
	scriptTextval = set_string_not_empty(ip->scriptText,
					       "ccode data base",
					       ccode_params.script);
	XtFree(ccode_params.script);
	ccode_params.script = scriptTextval;
	
	/* do any other things needed */

    } else if (!strcmp(client, "scriptCancelPBtn")) {
	/*fprintf (stderr, "ccode_callback: %s\n", client );*/
	script_buttons_mapped = 
	  okTextCancel(ip->scriptText, ip->scriptOKPBtn,
		       ip->scriptCancelPBtn,
		       ccode_params.script);

    } else if (!strcmp(client, "suffixText")) {
	/*fprintf (stderr, "ccode_callback: %s\n", client );*/
	suffix_buttons_mapped =
	  okTextValueChanged(ip->suffixText, ip->suffixOKPBtn,
			     ip->suffixCancelPBtn,
			     ip->ccodeDialogShell,
			     suffix_buttons_mapped,
			     ccode_params.suffix);
	
    } else if ((!strcmp(client, "suffixTextAct")) ||
	       (!strcmp(client, "suffixOKPBtn"))) {
	String suffixTextval;

	/*fprintf (stderr, "ccode_callback: %s\n", client );*/
	suffix_buttons_mapped =
	  okTextActivate(ip->suffixText, ip->suffixOKPBtn,
			 ip->suffixCancelPBtn, &suffixTextval);

	XtFree(ccode_params.suffix);
	ccode_params.suffix = suffixTextval;
	
	/* do any other things needed */

    } else if (!strcmp(client, "suffixCancelPBtn")) {
	/*fprintf (stderr, "ccode_callback: %s\n", client );*/
	suffix_buttons_mapped = 
	  okTextCancel(ip->suffixText, ip->suffixOKPBtn,
		       ip->suffixCancelPBtn,
		       ccode_params.suffix);

    } else if (!strcmp(client, "cFileText")) {
	/*fprintf (stderr, "ccode_callback: %s\n", client );*/
	cFile_buttons_mapped =
	  okTextValueChanged(ip->cFileText, ip->cFileOKPBtn,
			     ip->cFileCancelPBtn,
			     ip->ccodeDialogShell,
			     cFile_buttons_mapped,
			     ccode_params.cFile);
	
    } else if ((!strcmp(client, "cFileTextAct")) ||
	       (!strcmp(client, "cFileOKPBtn"))) {
	String cFileTextval;

	/*fprintf (stderr, "ccode_callback: %s\n", client );*/
	cFile_buttons_mapped =
	  okTextActivate(ip->cFileText, ip->cFileOKPBtn,
			 ip->cFileCancelPBtn, &cFileTextval);
	XtFree(cFileTextval);
	
	cFileTextval = set_string_not_empty(ip->cFileText,
					       "ccode data base",
					       ccode_params.cFile);
	XtFree(ccode_params.cFile);
	ccode_params.cFile = cFileTextval;
	
	/* do any other things needed */

    } else if (!strcmp(client, "cFileCancelPBtn")) {
	/*fprintf (stderr, "ccode_callback: %s\n", client );*/
	cFile_buttons_mapped = 
	  okTextCancel(ip->cFileText, ip->cFileOKPBtn,
		       ip->cFileCancelPBtn,
		       ccode_params.cFile);
	
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "ccode_callback: %s\n", client );*/
	close_popup(ip->ccodeForm);
	
    } else if (!strcmp(client, "generatePushButton")) {
	/*fprintf (stderr, "ccode_callback: %s\n", client );*/
	ccode_run();
	
    } else if (!strcmp(client, "cancelPushButton")) {
	/*fprintf (stderr, "ccode_callback: %s\n", client );*/
	stop_command_file(ccode_params.pid);
	ccode_params.pid = 0;
	close_popup(ip->ccodeForm);
	
    } else {
	fprintf(stderr, "ccode_callback: unknown client: %s\n", (char *)client);
    }
}

void
ccode_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(CcodeClone->ccodeForm);
    } else if (!strcmp(request, "init")){
	ccode_params.script = NULL;
	ccode_params.suffix = NULL;
	ccode_params.cFile = NULL;

    } else if (!strcmp(request, "display")){
	show_string_label(CcodeClone->paramFnameLabel,
			  reports_params.param_fname);
	ccode_check_file();
	if(ccode_params.script != NULL)
	  show_string_text(CcodeClone->scriptText,
			   ccode_params.script);
	if(ccode_params.suffix != NULL)
	  show_string_text(CcodeClone->suffixText,
			   ccode_params.suffix);
	if(ccode_params.cFile != NULL)
	  show_string_text(CcodeClone->cFileText,
			   ccode_params.cFile);

    } else if (!strcmp(request, "paramFname")){
	show_string_label(CcodeClone->paramFnameLabel,
			  reports_params.param_fname);

        XtFree(ccode_params.cFile);
	ccode_params.cFile = XtNewString(lnk_params.exp_name);
	ccode_params.cFile = StringConcat(ccode_params.cFile, ".c");
	show_string_text(CcodeClone->cFileText, ccode_params.cFile);

        XtFree(ccode_params.suffix);
	ccode_params.suffix = XtNewString(lnk_params.exp_name);
	show_string_text(CcodeClone->suffixText, ccode_params.suffix);

        XtFree(ccode_params.script);
	ccode_params.script = XtNewString(lnk_params.exp_name);
	ccode_params.script = StringConcat(ccode_params.script, ".c.run");
	show_string_text(CcodeClone->scriptText, ccode_params.script);

	ccode_check_file();

    } else {
	fprintf(stderr, "ccode_request: unknown request: %s\n",
		request);
    }
}

static ARG ccodetab[] =
{
    {"script", STRING, (int *)NULL,},
    {"suffix", STRING, (int *)NULL,},
    {"cFile", STRING, (int *)NULL,},
};
#define TABSIZE (sizeof(ccodetab)/sizeof(ARG))

static void
ccode_set_defaults_variables(dp)
     ccodeParams *dp;
{
    nnargs_set_variable("script", (int *)&dp->script,
			ccodetab, TABSIZE);
    nnargs_set_variable("suffix", (int *)&dp->suffix,
			ccodetab, TABSIZE);
    nnargs_set_variable("cFile", (int *)&dp->cFile,
			ccodetab, TABSIZE);
}

void
read_ccode_defaults(argc, argv)
     int argc;
     char **argv;
{
    /* clear flags and strings which are not printed if they are NULL */
    if (ccode_params.script != NULL){
	XtFree(ccode_params.script);
	ccode_params.script = NULL;
    }
    if (ccode_params.suffix != NULL){
	XtFree(ccode_params.suffix);
	ccode_params.suffix = NULL;
    }
    if (ccode_params.cFile != NULL){
	XtFree(ccode_params.cFile);
	ccode_params.cFile = NULL;
    }
    ccode_set_defaults_variables(&ccode_params);
    if(argc > 0){
	argc = nnargs(argc, argv, ccodetab, TABSIZE);
	/* copy the strings into new memory */
	ccode_params.script =
	  XtNewString(ccode_params.script);
	ccode_params.suffix =
	  XtNewString(ccode_params.suffix);
	ccode_params.cFile =
	  XtNewString(ccode_params.cFile);

    }

    ccode_request("display");
}

void
write_ccode_defaults(FILE *default_file)
{
    ccode_set_defaults_variables(&ccode_params);
    if(default_file != NULL){
	fprintf(default_file, "ccode %s",
		nnargs_get_flags(ccodetab, TABSIZE));
    }
}

static void
ccode_run()
{
    char *shell_path;
    FILE *shell_file;

    shell_path = path_build(reports_params.exp_path,
			    ccode_params.script);
    if(overwrite_ok(CcodeClone->ccodeForm, shell_path)){
	shell_file = command_file_open(CcodeClone->ccodeForm,
				       shell_path);
	if(shell_file == NULL) return;
	
	if(store_ccode_commands(shell_file)) return;

	shell_path =
	  path_build(reports_params.exp_path, ccode_params.script);
	command_file_close(shell_file, shell_path, "for ccode shell file");

	if(!lnk_params.store_only)
	  ccode_params.pid =
	    chdir_and_execute_command(ccode_params.script);
    }
}

static int
store_ccode_commands(FILE *shell_file)
{
    char *plot_command_line;

    param_fname = reports_params.param_fname;
    class_suff = ccode_params.suffix;
    adjust_test = priors_params.adjustTest;
    test_uniform_priors = priors_params.testUniform;
    test_priors_list = priors_params.testPriorsList;
    plot_command_line = XtNewString(lnk_params.classifier_name);
    plot_command_line = StringConcat(plot_command_line, "2c ");
    plot_command_line = StringConcat(plot_command_line,
				     nnargs_get_flags(argtab2c, TABSIZE2C));
    *strrchr(plot_command_line, '\n') = ' ';	/* get rid of newline at end */
    fprintf(shell_file, "%s", plot_command_line);
    /* put in classifier dependent call if needed */
    g_get_test_args(shell_file);
    fprintf(shell_file, " >! %s\n", ccode_params.cFile);
    return 0;
}

void
activate_ccode(int flag)
{
    gray_out_item(CcodeClone->ccodeForm, !flag);
}

void
ccode_check_file()
{
    if(file_exists(path_build(reports_params.exp_path,
			      reports_params.param_fname))){
	show_string_label(CcodeClone->errorLabel, " ");
    } else {
	show_string_label(CcodeClone->errorLabel, 
			  "Error: Classifier Parameter File does not exist");
    }
}
