/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/cknn/cknn_callback.c,v 11.4 2001/08/23 20:15:43 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            CKNN                              ****
 ****                          Cknn Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  30-Oct-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include "../main/lnknet_subs.h"
#include "cknn.h"
#include <nnlib.h>
#include <cknn_args.h>
/*
 * External Functions
 */
#include "../plots/plots_func.h"

/*
 * Externals
 */
extern CknnWidgets *CknnClone;
#include "cknn_params.h"
CknnParams cknn_params; /* this is the real one */
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: cknn_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   cknn_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  30-Oct-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
cknn_callback(Widget w, XtPointer client, XtPointer call)
{
    CknnWidgets *ip;
    static int epochs_buttons_mapped = False;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
    
    if (!strcmp(client, "epochsText")) {
	/*fprintf (stderr, "cknn_callback: %s\n", client );*/
	epochs_buttons_mapped =
	  okTextValueChanged(ip->epochsText, ip->epochsOKPBtn,
			     ip->epochsCancelPBtn,
			     ip->cknnDialogShell,
			     epochs_buttons_mapped,
			     cknn_params.epochs_string);
	
    } else if ((!strcmp(client, "epochsTextAct")) ||
	       (!strcmp(client, "epochsOKPBtn"))) {
	String epochsTextval;

	/*fprintf (stderr, "cknn_callback: %s\n", client );*/
	epochs_buttons_mapped =
	  okTextActivate(ip->epochsText, ip->epochsOKPBtn,
			 ip->epochsCancelPBtn, &epochsTextval);
	/* I won't actually be using this copy */
	XtFree (epochsTextval);
	
	/* verify the string as an integer */
	cknn_params.epochs =
	  set_atoi_value(ip->epochsText, "epochs_set", cknn_params.epochs);

	/* save the current text for future comparisons */
	XtFree(cknn_params.epochs_string);
	cknn_params.epochs_string = get_string_text(ip->epochsText);
 
	/* do any other things needed */
	set_plot_labels();

    } else if (!strcmp(client, "epochsCancelPBtn")) {
	/*fprintf (stderr, "cknn_callback: %s\n", client );*/
	epochs_buttons_mapped = 
	  okTextCancel(ip->epochsText, ip->epochsOKPBtn,
		       ip->epochsCancelPBtn,
		       cknn_params.epochs_string);
	
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "cknn_callback: %s\n", client );*/
	close_popup(CknnClone->cknnForm);
	
    } else {
	fprintf(stderr, "cknn_callback: unknown client: %s\n", (char *)client);
    }
}

void
cknn_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(CknnClone->cknnForm);
    } else if (!strcmp(request, "init")){
	/* flag */
	cknn_params.epochs = 0;
	cknn_params.epochs_string = NULL;
    } else if (!strcmp(request, "display")){
	char string[10];
	/* flag */
	XtFree(cknn_params.epochs_string);
	sprintf(string, "%d", cknn_params.epochs);
	cknn_params.epochs_string = XtNewString(string);
	show_string_text(CknnClone->epochsText,
			 cknn_params.epochs_string);
    } else {
	fprintf(stderr, "cknn_request: unknown request: %s\n", request);
    }
}

String
cknn_get_plot_label(String string)
{
    char num[10];
    string = StringConcat(string, " Epochs:");
    sprintf(num, "%d", cknn_params.epochs);
    string = StringConcat(string, num);
    return string;
}

static void
cknn_set_read_variables()
{
    cknn_params.epochs = nepochs;
}

static void
cknn_set_write_variables()
{
    k = 1;
    nepochs = cknn_params.epochs;
}

void
read_cknn_defaults(argc, argv)
     int argc;
     char **argv;
{
    /* clear flags and strings which are not printed if they are NULL */

    if(argc > 0){
	argc = nnargs(argc, argv, cknn_tab, CKNN_TABSIZE);
	cknn_set_read_variables();
    } else {
	/* set any defaults */
	cknn_params.epochs = 5;
    }

    cknn_request("display");
}

void
write_cknn_defaults(FILE *default_file)
{
    cknn_set_write_variables();
    if(default_file != NULL){
	fprintf(default_file, "cknn %s",
		nnargs_get_flags(cknn_tab, CKNN_TABSIZE));
    }
}

String
cknn_get_args ()
{
    cknn_set_write_variables();
    return (nnargs_get_part_flags(cknn_tab, CKNN_TABSIZE));
}

String
cknn_get_important_args ()
{
    cknn_set_write_variables();
    return (nnargs_get_flags_one_line(cknn_tab, CKNN_TABSIZE));
}

void
cknn_set_epochs (int e)
{
    cknn_params.epochs = e;
}

int
cknn_get_epochs()
{
    return cknn_params.epochs;
}
