/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/committee/committee_callback.c,v 11.4 2001/08/23 20:15:47 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                       Lnknet Window Interface                        ****
 ****                           Committee Window                              ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  14-Sep-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include "../main/lnknet_subs.h"
#include "committee.h"
#include <nnlib.h>
#include <committee_args.h>
/*
 * External Functions
 */
#include "../plots/plots_func.h"
#include "committee_func.h"
 
/*
 * Externals
 */
extern CommitteeWidgets *CommitteeClone;
#include "committee_params.h"
CommitteeParams committee_params; /* this is the real one */

/*
 * Statics
 */
static void committee_set_read_variables();
static void committee_set_write_variables();
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: committee_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   committee_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  20-Nov-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
committee_callback(Widget w, XtPointer client, XtPointer call)
{
    /* SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    CommitteeWidgets *ip;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "averageToggle") ||
	!strcmp(client, "medianToggle") ||
	!strcmp(client, "voteToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "committee_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set){
	    committee_params.classType =
	      get_radio_value(ip->classTypeRadioBox);
	    set_plot_labels();
	}

    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "committee_callback: %s\n", client);*/
	close_popup(CommitteeClone->committeeForm);

    } else {
	fprintf (stderr, "committee_callback: unknown client: %s\n", (char *)client);
    }
}

void
committee_request(String request)
{
    if (!strcmp(request, "open")) {
	open_popup(CommitteeClone->committeeForm);

    } else if (!strcmp(request, "init")){
	/* this should only get called once, before ANYTHING 
	   gets allocated or there will be memory leaks */
	committee_params.classType = 0;

    } else if (!strcmp(request, "display")){
	show_radio_value(CommitteeClone->classTypeRadioBox, committee_params.classType);

    } else {
	fprintf (stderr, "committee_request: unknown request: %s\n", request);
    }
}

String
committee_get_plot_label(String string)
{
    if(committee_params.classType == 0)
      string = StringConcat(string, "Average");
    else if (committee_params.classType == 1)
      string = StringConcat(string, "Median");
    else
      string = StringConcat(string, "Vote");

    return string;
}

static void
committee_set_read_variables()
{
    committee_params.classType = classifier_type;
}

static void
committee_set_write_variables()
{
    classifier_type = committee_params.classType;
}

void
read_committee_defaults(int argc, char **argv)
{
    if(argc > 0){
	argc = nnargs(argc, argv, committee_tab, COMMITTEE_TABSIZE);
	committee_set_read_variables();
    } else {
	committee_params.classType = 0;
    }
    committee_request("display");
}

void
write_committee_defaults(default_file)
     FILE *default_file;
{
    committee_set_write_variables();
    if(default_file != NULL){
	fprintf(default_file, "committee %s",
		nnargs_get_flags(committee_tab, COMMITTEE_TABSIZE));
    }
}

String
committee_get_args ()
{
    committee_set_write_variables();
    return (nnargs_get_part_flags(committee_tab, COMMITTEE_TABSIZE));
}

String
committee_get_important_args ()
{
    committee_set_write_variables();
    return (nnargs_get_flags_one_line(committee_tab, COMMITTEE_TABSIZE));
}
