/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/committeeData/committeeData_callback.c,v 11.7 2001/08/23 20:15:55 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            COMMITTEEDATA                             ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  13-Nov-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <nnlib.h>
#include <Xm/Xm.h>
#include "committeeData.h"
/*
 * External Functions
 */
#include "../main/lnknet_subs.h"
#include "../lnk/lnk_func.h"
#include "../reports/reports_func.h"
#include "committeeData_func.h"
 
/*
 * Externals
 */
extern CommitteeDataWidgets *CommitteeDataClone;
#include "committeeData_params.h"
CommitteeDataParams committeeData_params;
#include "../data/data_params.h"
extern DataParams data_params;
#include "../lnk/lnk_params.h"
extern LnkParams lnk_params;
#include "../reports/reports_params.h"
extern ReportsParams reports_params;


/*
 * Statics
 */ 
static void committeeData_set_defaults_variables(CommitteeDataParams *dp);
static void committeeData_set_status_strings();
static char * committeeData_file_ok(char *);
static void committeeData_set_ninputs_msg();
static void committeeData_run();
static char ** committeeData_get_input_labels(int nexperiments, int noutputs);
static int committeeData_store_specific_command(FILE *shell_file, char *status_string, char **trainList, char *ext);
static void store_committeeData_commands(FILE *shell_file, char **comm_file_list, char *ext);
#include <err2data_args.h>

/******************************************************************************
 ******************************************************************************
 *
 * Module Name: committeeData_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   committeeData_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  13-Nov-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
committeeData_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    CommitteeDataWidgets *ip;
    static int expList_buttons_mapped = False;
    static int dataBase_buttons_mapped = False;
  
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
  
    if (!strcmp(client, "expListText")) {
	/*fprintf (stderr, "committeeData_callback: %s\n", client);*/
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	expList_buttons_mapped =
	  okTextValueChanged(ip->expListText, ip->expListOKPBtn,
			     ip->expListCancelPBtn,
			     ip->committeeDataDialogShell,
			     expList_buttons_mapped,
			     committeeData_params.expList_string);

    } else if ((!strcmp(client, "expListTextAct")) ||
	       (!strcmp(client, "expListOKPBtn"))) {
	/*fprintf (stderr, "committeeData_callback: %s\n", client );*/
	String expListTextval;
	expList_buttons_mapped =
	  okTextActivate(ip->expListText,
			 ip->expListOKPBtn,
			 ip->expListCancelPBtn,
			 &expListTextval);
	
	if(committeeData_params.expList != NULL)
	  deallocate_string_vector(committeeData_params.expList,
				   atoi(committeeData_params.expList[0])+1);
	committeeData_params.expList = NULL;
	/* take spaces and tabs out of the list before handing it to nnargs */
	if(expListTextval != NULL){
	    char *expListTemp = expListTextval;
	    expListTextval = strtok(expListTextval, " \t");
	    if(expListTextval != NULL)
	      committeeData_params.expList =
		nnargs_get_string_alloc_list(expListTextval);
	    XtFree(expListTemp);
	}
	if(committeeData_params.expList == NULL){
	    committeeData_params.expList =
	      nnargs_get_string_alloc_list("");
	}

	/* make a copy of the string as it has been interpretted */
	XtFree(committeeData_params.expList_string);
	committeeData_params.expList_string =
	  make_labels_string(committeeData_params.expList);
	show_string_text(ip->expListText,
			 committeeData_params.expList_string);
	
	/* do anything else necessary */
	if(committeeData_params.expList != NULL)
	  committeeData_params.nexperiments =
	    atoi(committeeData_params.expList[0]);
	else
	  committeeData_params.nexperiments = 0;
	committeeData_set_lists();
	committeeData_set_ninputs_msg();
	
 
    } else if (!strcmp(client, "expListCancelPBtn")) {
	/*fprintf (stderr, "committeeData_callback: %s\n", client );*/
	expList_buttons_mapped = 
	  okTextCancel(ip->expListText, ip->expListOKPBtn,
		       ip->expListCancelPBtn,
		       committeeData_params.expList_string);
	fprintf (stderr, "committeeData_callback: %s\n", (char *)client );
	
    } else if (!strcmp(client, "selectTrainToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "committeeData_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	committeeData_params.selectTrain = state->set;
	
    } else if (!strcmp(client, "selectEvalToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "committeeData_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	committeeData_params.selectEval = state->set;
	
    } else if (!strcmp(client, "selectTestToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "committeeData_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	committeeData_params.selectTest = state->set;
	
    } else if (!strcmp(client, "dataBaseText")) {
	/*fprintf (stderr, "committeeData_callback: %s\n", client );*/
	dataBase_buttons_mapped =
	  okTextValueChanged(ip->dataBaseText, ip->dataBaseOKPBtn,
			     ip->dataBaseCancelPBtn,
			     ip->committeeDataDialogShell,
			     dataBase_buttons_mapped,
			     committeeData_params.dataBase);
	
    } else if ((!strcmp(client, "dataBaseTextAct")) ||
	       (!strcmp(client, "dataBaseOKPBtn"))) {
	String dataBaseTextval;

	/*fprintf (stderr, "committeeData_callback: %s\n", client );*/
	dataBase_buttons_mapped =
	  okTextActivate(ip->dataBaseText, ip->dataBaseOKPBtn,
			 ip->dataBaseCancelPBtn, &dataBaseTextval);
	XtFree(dataBaseTextval);
	
	dataBaseTextval = set_string_not_empty(ip->dataBaseText,
					       "committeeData data base",
					       committeeData_params.dataBase);
	XtFree(committeeData_params.dataBase);
	committeeData_params.dataBase = dataBaseTextval;
	
	/* do any other things needed */
	XtFree(committeeData_params.script);
	committeeData_params.script =
	  XtNewString(committeeData_params.dataBase);
	committeeData_params.script = StringConcat(committeeData_params.script,
						   ".run");
	show_string_label(ip->scriptLabel, committeeData_params.script);
	committeeData_set_ninputs_msg();

    } else if (!strcmp(client, "dataBaseCancelPBtn")) {
	/*fprintf (stderr, "committeeData_callback: %s\n", client );*/
	dataBase_buttons_mapped = 
	  okTextCancel(ip->dataBaseText, ip->dataBaseOKPBtn,
		       ip->dataBaseCancelPBtn,
		       committeeData_params.dataBase);
	
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "committeeData_callback: %s\n", client );*/
	close_popup(ip->committeeDataForm);
	
    } else if (!strcmp(client, "generatePushButton")) {
	/*fprintf (stderr, "committeeData_callback: %s\n", client );*/
	committeeData_run();
	
    } else if (!strcmp(client, "cancelPushButton")) {
	/*fprintf (stderr, "committeeData_callback: %s\n", client );*/
	stop_command_file(committeeData_params.pid);
	committeeData_params.pid = 0;
	close_popup(ip->committeeDataForm);
	
    } else if (!strcmp(client, "updateStatusPBtn")) {
	/*fprintf (stderr, "committeeData_callback: %s\n", client );*/
	committeeData_set_status_strings();

    } else {
	fprintf(stderr, "committeeData_callback: unknown client: %s\n", (char *)client);
    }
}

void
committeeData_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(CommitteeDataClone->committeeDataForm);
    } else if (!strcmp(request, "init")){
	committeeData_params.expList = NULL;
	committeeData_params.expList_string = NULL;
	committeeData_params.selectTrain = 0;
	committeeData_params.selectEval = 0;
	committeeData_params.selectTest = 0;
	committeeData_params.dataBase = NULL;
	committeeData_params.trainList = NULL;

    } else if (!strcmp(request, "display")){
	show_string_label(CommitteeDataClone->expPathLabel,
			  reports_params.exp_path);
	XtFree(committeeData_params.expList_string);
	committeeData_params.expList_string =
	  make_labels_string(committeeData_params.expList);
	if(committeeData_params.expList)
	  committeeData_params.nexperiments =
	    atoi(committeeData_params.expList[0]);
	show_string_text(CommitteeDataClone->expListText,
			 committeeData_params.expList_string);
	show_toggle_value(CommitteeDataClone->selectTrainToggle,
			  committeeData_params.selectTrain);
	show_toggle_value(CommitteeDataClone->selectEvalToggle,
			  committeeData_params.selectEval);
	show_toggle_value(CommitteeDataClone->selectTestToggle,
			  committeeData_params.selectTest);
	committeeData_set_status_strings();
	if(committeeData_params.dataBase != NULL)
	  show_string_text(CommitteeDataClone->dataBaseText,
			   committeeData_params.dataBase);
	show_int_label(CommitteeDataClone->nOutputsLabel,
		       data_params.noutputs);
	show_string_label(CommitteeDataClone->scriptLabel,
			  committeeData_params.script);
	show_string_label(CommitteeDataClone->errorLabel, "");
	committeeData_set_ninputs_msg();
    } else if (!strcmp(request, "expPath")){
	show_string_label(CommitteeDataClone->expPathLabel,
			  reports_params.exp_path);
	committeeData_set_status_strings();

    } else {
	fprintf(stderr, "committeeData_request: unknown request: %s\n",
		request);
    }
}

static ARG committeeDatatab[] =
{
    {"expList", STRING_ALLOC_LIST, (int *)NULL,},
    {"selectTrain", BOOLEAN, (int *)NULL,},
    {"selectEval", BOOLEAN, (int *)NULL,},
    {"selectTest", BOOLEAN, (int *)NULL,},
    {"dataBase", STRING, (int *)NULL,},
};
#define TABSIZE (sizeof(committeeDatatab)/sizeof(ARG))

static void
committeeData_set_defaults_variables(dp)
     CommitteeDataParams *dp;
{
    nnargs_set_variable("expList", (int *)&dp->expList,
			committeeDatatab, TABSIZE);
    nnargs_set_variable("selectTrain", (int *)&dp->selectTrain,
			committeeDatatab, TABSIZE);
    nnargs_set_variable("selectEval", (int *)&dp->selectEval,
			committeeDatatab, TABSIZE);
    nnargs_set_variable("selectTest", (int *)&dp->selectTest,
			committeeDatatab, TABSIZE);
    nnargs_set_variable("dataBase", (int *)&dp->dataBase,
			committeeDatatab, TABSIZE);

}

void
read_committeeData_defaults(argc, argv)
     int argc;
     char **argv;
{
    /* clear flags and strings which are not printed if they are NULL */
    if (committeeData_params.expList != NULL){
	deallocate_string_vector(committeeData_params.expList,
				 atoi(committeeData_params.expList[0])+1);
	committeeData_params.expList = NULL;
    }
    committeeData_params.selectTrain = 0;
    committeeData_params.selectEval = 0;
    committeeData_params.selectTest = 0;
    if (committeeData_params.dataBase != NULL){
	XtFree(committeeData_params.dataBase);
	committeeData_params.dataBase = NULL;
    }
    committeeData_set_defaults_variables(&committeeData_params);
    if(argc > 0){
	argc = nnargs(argc, argv, committeeDatatab, TABSIZE);
	/* copy the strings into new memory */
	committeeData_params.dataBase =
	  XtNewString(committeeData_params.dataBase);

    } else {
	/* set any defaults */
	committeeData_params.selectTrain = 1;
	committeeData_params.selectEval = 1;
	committeeData_params.selectTest = 1;
    }
    if(committeeData_params.expList == NULL){
	committeeData_params.expList =
	  nnargs_get_string_alloc_list("");
    }

    committeeData_set_lists();
    committeeData_request("display");
}

void
write_committeeData_defaults(FILE *default_file)
{
    committeeData_set_defaults_variables(&committeeData_params);
    if(default_file != NULL){
	fprintf(default_file, "committeeData %s",
		nnargs_get_flags(committeeDatatab, TABSIZE));
    }
}

static void
committeeData_set_status_strings()
{
    int nfiles, i;

    if(committeeData_params.expList == NULL) nfiles = 0;
    else nfiles = atoi(committeeData_params.expList[0]);

    XtFree(committeeData_params.trainStatus);
    committeeData_params.trainStatus = XtNewString("");
    XtFree(committeeData_params.evalStatus);
    committeeData_params.evalStatus = XtNewString("");
    XtFree(committeeData_params.testStatus);
    committeeData_params.testStatus = XtNewString("");

    for(i = 1; i <= nfiles; i++){
	/* check the training file */
	committeeData_params.trainStatus =
	  StringConcat(committeeData_params.trainStatus,
		       committeeData_file_ok(committeeData_params.trainList[i]));
	if(i < nfiles) committeeData_params.trainStatus = StringConcat(committeeData_params.trainStatus, ",");

	/* check the evaluation file */
	committeeData_params.evalStatus = StringConcat(committeeData_params.evalStatus, committeeData_file_ok(committeeData_params.evalList[i]));
	if(i < nfiles) committeeData_params.evalStatus = StringConcat(committeeData_params.evalStatus, ",");

	/* check the testing file */
	committeeData_params.testStatus = StringConcat(committeeData_params.testStatus, committeeData_file_ok(committeeData_params.testList[i]));
	if(i < nfiles) committeeData_params.testStatus = StringConcat(committeeData_params.testStatus, ",");
    }
    show_string_label(CommitteeDataClone->trainStatusLabel,
		      committeeData_params.trainStatus);
    show_string_label(CommitteeDataClone->evalStatusLabel,
		      committeeData_params.evalStatus);
    show_string_label(CommitteeDataClone->testStatusLabel,
		      committeeData_params.testStatus);
}

void
committeeData_set_lists()
{
    int nfiles, i;
    char *tmp;

    if(committeeData_params.expList == NULL) return;
    else nfiles = atoi(committeeData_params.expList[0]);

    /* build train list */
    if(committeeData_params.trainList != NULL)
      deallocate_string_vector(committeeData_params.trainList,
			       atoi(committeeData_params.trainList[0])+1);
    committeeData_params.trainList = (char **)allocate_pvector(nfiles+1);
    committeeData_params.trainList[0] =
      XtNewString(committeeData_params.expList[0]);
    for(i = 1; i <= nfiles; i++){
	tmp = XtNewString(committeeData_params.expList[i]);
	tmp = StringConcat(tmp, ".err");
	tmp = StringConcat(tmp, data_params.ext_test_train);
	committeeData_params.trainList[i] = tmp;
    }

    /* build test list */
    if(committeeData_params.testList != NULL)
      deallocate_string_vector(committeeData_params.testList,
			       atoi(committeeData_params.testList[0])+1);
    committeeData_params.testList = (char **)allocate_pvector(nfiles+1);
    committeeData_params.testList[0] =
      XtNewString(committeeData_params.expList[0]);
    for(i = 1; i <= nfiles; i++){
	tmp = XtNewString(committeeData_params.expList[i]);
	tmp = StringConcat(tmp, ".err");
	tmp = StringConcat(tmp, data_params.ext_test);
	committeeData_params.testList[i] = tmp;
    }
    
    /* build eval list */
    if(committeeData_params.evalList != NULL)
      deallocate_string_vector(committeeData_params.evalList,
			       atoi(committeeData_params.evalList[0])+1);
    committeeData_params.evalList = (char **)allocate_pvector(nfiles+1);
    committeeData_params.evalList[0] =
      XtNewString(committeeData_params.expList[0]);
    for(i = 1; i <= nfiles; i++){
	tmp = XtNewString(committeeData_params.expList[i]);
	tmp = StringConcat(tmp, ".err");
	tmp = StringConcat(tmp, data_params.ext_eval);
	committeeData_params.evalList[i] = tmp;
    }
    committeeData_set_status_strings();
}

static char *
committeeData_file_ok(char *file_name)
{
    FILE *fp;
    int ntoken;
    
    if(file_exists(path_build(reports_params.exp_path, file_name))){
	/* check the number of tokens on each line, should be
	   5 + noutputs */
	if((fp = fopen(path_build(reports_params.exp_path, file_name),"r"))
	   != NULL){
	    ntoken = get_n_tokens(fp);
	    fclose(fp);
	    if(ntoken >= 5+data_params.noutputs)
	      return "1";
	    else return "*";
	}
    }
    /* if the file wasn't ok, we will drop through the ifs to here */
    return "0";
}

static void
committeeData_set_ninputs_msg()
{
    char *ninputs_label, num[10];
    ninputs_label = XtNewString(committeeData_params.dataBase);
    ninputs_label = StringConcat(ninputs_label, " will have ");
    sprintf(num, "%d", committeeData_params.nexperiments*data_params.noutputs);
    ninputs_label = StringConcat(ninputs_label, num);
    ninputs_label = StringConcat(ninputs_label, " input features.");
    show_string_label(CommitteeDataClone->ninputsLabel,
		      ninputs_label);
    XtFree(ninputs_label);
}

static void
committeeData_run()
{
    char *shell_path, *dbase;
    FILE *shell_file;
    DEFAULTS_FILE_STRUCT df_args;

    /* make sure status strings are up to date */
    committeeData_set_status_strings();
	
    /* no experiments to include! */
    if(committeeData_params.expList == NULL) return;

    shell_path = path_build(reports_params.exp_path,
			    committeeData_params.script);
    if(overwrite_ok(CommitteeDataClone->committeeDataForm, shell_path)){
	shell_file = command_file_open(CommitteeDataClone->committeeDataForm,
				       shell_path);
	if(shell_file == NULL) return;
	
	set_temp_exp_path(shell_file);
	exp_path = reports_params.exp_path;

	/* store for each of the file types selected */
	if(committeeData_params.selectTrain){
	    fprintf(shell_file, "\n#create training file\n");
	    if(committeeData_store_specific_command(shell_file, committeeData_params.trainStatus, committeeData_params.trainList, data_params.ext_train) != 0) {
		restore_exp_path();
		return;
	    }
	}
	if(committeeData_params.selectEval){
	    fprintf(shell_file, "\n#create evaluation file\n");
	    if(committeeData_store_specific_command(shell_file, committeeData_params.evalStatus, committeeData_params.evalList, data_params.ext_eval) != 0){
		restore_exp_path();
		return;
	    }
	}
	if(committeeData_params.selectTest){
	    fprintf(shell_file, "\n#create testing file\n");
	    if(committeeData_store_specific_command(shell_file, committeeData_params.testStatus, committeeData_params.testList, data_params.ext_test) != 0){
		restore_exp_path();
		return;
	    }
	}

	/* save the defaults file */
	dbase = path_create(reports_params.exp_path, committeeData_params.dataBase);
	dbase = StringConcat(dbase, DEFAULTS_SUFFIX);
	df_args.ninputs = data_params.noutputs * committeeData_params.nexperiments;
	df_args.noutputs = data_params.noutputs;
	df_args.db_type = CLASS_DATA;
	df_args.input_labels =
	  committeeData_get_input_labels(committeeData_params.nexperiments,
					 data_params.noutputs);
	df_args.output_labels = data_params.class_labels;
	df_args.input_types = NULL;
	df_args.input_type_values = NULL;
	df_args.categories = NULL;
	fprintf(shell_file, "\n#create description file\n");
	fprintf(shell_file, "cat << EOF > %s\n", dbase);
	write_defaults_stream(shell_file, &df_args);
	fprintf(shell_file, "EOF\n");
	XtFree(dbase);

	deallocate_string_vector(df_args.input_labels,
				 atoi(df_args.input_labels[0])+1);

	restore_exp_path();

	shell_path =
	  path_build(reports_params.exp_path, committeeData_params.script);
	command_file_close(shell_file, shell_path, "for committeeData shell file");

	show_string_label(CommitteeDataClone->errorLabel, "");
	if(!lnk_params.store_only)
	  committeeData_params.pid =
	    chdir_and_execute_command(committeeData_params.script);
    }
}

/* build an input label list for the committeeData data base */
/* X1mlp0,X1mlp1,X1knn0,X1knn1 */
static char **
committeeData_get_input_labels(nexperiments, noutputs)
     int nexperiments, noutputs;
{
    int i, j;
    char *input_labels_string, **input_labels, temp[16];

    /* fill the string */
    input_labels_string = XtNewString("");
    for(i = 0; i < nexperiments; i++){
	for(j = 0; j < noutputs; j++){
	    input_labels_string =
	      StringConcat(input_labels_string,
			   committeeData_params.expList[i+1]);
	    sprintf(temp, "%d", j);
	    input_labels_string = StringConcat(input_labels_string, temp);
	    input_labels_string = StringConcat(input_labels_string, ",");
	}
    }
    /* take off the final comma */
    input_labels_string[strlen(input_labels_string)] = '\0';

    /* translate the string into a label list */
    input_labels = nnargs_get_string_alloc_list(input_labels_string);
    /* free input_labels_string */
    nn_free(input_labels_string, "for input_labels_string");
    return input_labels;
}

static int
committeeData_store_specific_command(FILE *shell_file, char *status_string,
				     char **expList,
				     char *ext)
{
    /* check if all the status string says the files are ok */
#if HAVE_STRSTR
    if(strstr(status_string, "0") || strstr(status_string, "*")){
#else
    if(strstr(status_string, "0") || strstr(status_string, "*")){
#endif
	warningMessageDialog(CommitteeDataClone->committeeDataDialogShell,
			     "Error: Missing or Short Error Files\nCommitteeData Base Creation Aborted");
	show_string_label(CommitteeDataClone->errorLabel,
			  "Error: Missing or Short Error Files");
	command_file_close(shell_file, committeeData_params.script,
			   "for committeeData shell file");
	return -1;
    }
    store_committeeData_commands(shell_file,
				 expList, ext);
    return 0;
}

static void
store_committeeData_commands(FILE *shell_file, char **comm_file_list, char *ext)
{
    /* setup arguments for nnargs */
    file_list = comm_file_list;
    noutputs = data_params.noutputs;
    output_file = XtNewString(committeeData_params.dataBase);
    output_file = StringConcat(output_file, ext);
    /* exp_path gets set above */
    
    /* I cannot redirect the output to a file for store_only mode, because
       there is no log file for the committeeData shell script */
    fprintf(shell_file, "err2data %s",
	    nnargs_get_flags(err2data_tab, ERR2DATA_TABSIZE));
    XtFree(output_file);
}

void
set_committee_data_base(char *value)
{
    XtFree(committeeData_params.dataBase);
    committeeData_params.dataBase = XtNewString(value);
    committeeData_params.dataBase = StringConcat(committeeData_params.dataBase,
						 "_comm");
    XtFree(committeeData_params.script);
    committeeData_params.script = XtNewString(committeeData_params.dataBase);
    committeeData_params.script = StringConcat(committeeData_params.script,
					       ".run");
    show_string_text(CommitteeDataClone->dataBaseText,
		     committeeData_params.dataBase);
    show_string_label(CommitteeDataClone->scriptLabel,
		      committeeData_params.script);
    committeeData_set_ninputs_msg();
}

void
committeeData_update_noutputs()
{
    noutputs = data_params.noutputs;
    show_int_label(CommitteeDataClone->nOutputsLabel,
		   data_params.noutputs);
    committeeData_set_ninputs_msg();
}

