/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/data/dataSelect_callback.c,v 11.7 2001/09/17 21:22:49 kukolich Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            LNKNET                              ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  30-Aug-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
/* for strrchr, etc, for lots of string manipulations */
#include <string.h>
#endif
/* for opendir, etc, for filling file list */
#if HAVE_DIRENT_H
#include <dirent.h>
#endif
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <Xm/Xm.h>
#include <Xm/Text.h>
#include <Xm/List.h>
#include <nnlib.h>
#include "../main/lnknet_subs.h"
#include "dataSelect.h"
#include "data_params.h"
#include "data_func.h"
/*
 * External Functions
 */
int dataSelectFillFilesList(char *homedir);
 
/*
 * Externals
 */
extern DataSelectWidgets *DataSelectClone;
extern DataParams data_params;

/******************************************************************************
 ******************************************************************************
 *
 * Module Name: dataSelect_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   dataSelect_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  30-Aug-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
dataSelect_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    DataSelectWidgets *ip;
    static int dir_buttons_mapped = False; /* for dirText OK/Cancel buttons */
    static int selectOther_buttons_mapped = False;
    /* for selectOtherText OK/Cancel buttons */
 
  /*
   * Get the parent structure
   */
  XtVaGetValues (w, XmNuserData, &ip, NULL);
 
  if (!strcmp(client, "dirText")) {
      /*this is the value changed callback,
    make the OK/CANCEL buttons appear */
      /* fprintf (stderr, "dataSelect_callback: %s\n", (char *)client );*/
      dir_buttons_mapped =
    okTextValueChanged(ip->dirText, ip->dirOKPBtn,
               ip->dirCancelPBtn,
               ip->dataSelectDialogShell,
               dir_buttons_mapped,
               data_params.select_data_path);
  } else if ((!strcmp(client, "dirTextAct")) ||
         (!strcmp(client, "dirOKPBtn"))) {
      /* fprintf (stderr, "dataSelect_callback: %s\n", (char *)client );*/
      String dirTextval;
      dir_buttons_mapped =
    okTextActivate(ip->dirText, ip->dirOKPBtn,
               ip->dirCancelPBtn, &dirTextval);
      
      if (dirTextval[0] != '/'){
      /* this is not a full path, cd to the requested directory
         and get the full path from it */
      String currentDir;
      /* store the current LNKnet working directory */
      currentDir = XtMalloc(256);
      getcwd(currentDir,256);

      /* relative paths are relative to the path on the data
         window */
      chdir(data_params.select_data_path);
      /* let UNIX interpret the directory string for us */
      if (chdir(dirTextval) == -1){
          char entry[128];
          sprintf (entry, "%s\nis NOT a valid directory name",
               dirTextval);
          warningMessageDialog(w, entry);
          show_string_text(ip->dirText, data_params.select_data_path);
          return;
      }
      /* this is a real directory, get the full path */
      dirTextval = XtRealloc(dirTextval, 256);
      getcwd(dirTextval, 256);
      /* go back to the current LNKnet directory */
      chdir(currentDir);
      XtFree(currentDir);
      }   
      /*
       * get contents of directory and
       * display in filesList.
       */
      if (!dataSelectFillFilesList(dirTextval)) {
      /* invalid directory, complain and redisplay the
         last valid path */
      char entry[128];
      sprintf (entry, "%s\nis NOT a valid directory name",
           dirTextval);
      warningMessageDialog(w, entry);
      show_string_text(ip->dirText, data_params.select_data_path);
      } else {
      /* actually successful! copy the directory */
      
      if(data_params.select_data_path != NULL)
        XtFree(data_params.select_data_path);
      data_params.select_data_path = XtNewString(dirTextval);
      show_string_text(ip->dirText, data_params.select_data_path);
      }
      XtFree (dirTextval);
      /* we could tell the user right now that things have changed,
     or we could wait until the Apply or Dismiss buttons have
     been selected and tell them then. I chose to wait */
 
  } else if (!strcmp(client, "dirCancelPBtn")) {
      /*fprintf (stderr, "dataSelect_callback: %s\n", (char *)client );*/
      dir_buttons_mapped = 
    okTextCancel(ip->dirText, ip->dirOKPBtn, ip->dirCancelPBtn,
             data_params.select_data_path);
 
  } else if (!strcmp(client, "filesList")) {
      Boolean doit;
      String dirTextval, selection;
      char newname[BUFSIZ];
      int selectedItem;
      XmListCallbackStruct  *cbs = (XmListCallbackStruct *) call;

      /* fprintf (stderr, "dataSelect_callback: %s\n", client );*/
     /* get the selected item number and string in a manner that
        works with lesstif, rpl 10/99 */
      if( XmStringGetLtoR ( cbs->item, XmFONTLIST_DEFAULT_TAG, &selection) != True)
      {
      fprintf (stderr, "couldnt convert list item string\n" );
      return;
          }
      selectedItem = cbs->item_position;
      /* fprintf (stderr, "selected item: %s %d\n", selection, selectedItem);*/

      /* user double-clicked; get the current selection */
      doit = True;

      dirTextval = XmTextGetString(ip->dirText);
      /* fprintf (stderr, "dirText %s\n", dirTextval );*/
      
     /* this doesnt work with lesstif for an XmList widget, 
        it returns null. I  replaced it by extracting the selected item
        number and string from the callback structure 
      XtVaGetValues(w,
            XmNselectedItems, &selectedItems, 
            XmNselectedItemCount, &selectedItemCount,
            NULL);
     */
      if(selectedItem <= 0){
      fprintf (stderr, "Selected Item <=0\n" );
      return;
      }
    
      if(strlen(selection) == 0){
      fprintf (stderr, "zero length selection\n" );
      XtFree(selection);
      return;
      }
      if (!strcmp (PREVDIR, selection)) {
      /* they clicked "../" */
      /* get the current directory and extract the next level up */
      if (!strcmp(dirTextval, "/")) {
          /* 
           * then we're already at the root directory and 
           * there's no place to go 
           */
          doit = False;
      } else {
          int i;
          char * slashloc;

          /* get the name of the next directory up one level */
          slashloc = strrchr(dirTextval, '/');
          i = strlen(dirTextval)-strlen(slashloc);
          strncpy (newname, dirTextval, i);
          newname[i] = '\0';
      }
      } else {
      if ('/' == selection[strlen(selection)-1]) {
          /* user double-clicked on a directory name */
          /* go down a level to the requested directory */
          sprintf (newname, "%s/%s", dirTextval, selection);
          newname[strlen(newname)-1] = '\0';
      } else {
          /* user double-clicked on a filename (not a directory name) */
          XtCallActionProc (ip->applyPBtn, "Arm", 
                acs->event, NULL, 0);

          dataSelect_callback(ip->applyPBtn, "applyPBtn", call);

          XtCallActionProc (ip->applyPBtn, "Activate", 
                acs->event, NULL, 0);
          doit = False;
      }
      }
 
      /* free up allocated memory */
      XtFree (selection);
      XtFree (dirTextval);

      /* read the contents of the directory and display it */
      if (doit) {
    if (!dataSelectFillFilesList (newname)) {
        /* directory was not good. tell the user and reset to the
           old one */
        char entry[128];
        sprintf(entry,  "%s\nis NOT is valid directory name", newname);
        warningMessageDialog (w, entry);
        XmTextSetString(ip->dirText, data_params.select_data_path);
    } else {
        /* a sucessful directory change! copy the name over */
        if(data_params.select_data_path != NULL)
          XtFree(data_params.select_data_path);
        data_params.select_data_path = XtNewString(newname);
        
        XmTextSetString(ip->dirText, newname);
    }
      }
  } else if (!strcmp(client, "filesList_single")) {
      String selection, strend;
      int selectedItem;

      XmListCallbackStruct  *cbs = (XmListCallbackStruct *) call;

     /* get the selected item number and string in a manner that
        works with lesstif, rpl 10/99 */
      if( XmStringGetLtoR ( cbs->item, XmFONTLIST_DEFAULT_TAG, &selection) != True)
      {
      fprintf (stderr, "couldnt convert list item string\n" );
      return;
          }
      selectedItem = cbs->item_position;
      if(selectedItem <= 0){
      return;
      }
      if(strlen(selection) == 0){
      XtFree(selection);
      return;
      }

      if ((!strcmp (PREVDIR, selection)) ||
      ('/' == selection[strlen(selection)-1])) {
      /* they selected a directory, ignore it */
      } else {
      /* get the root of the defaults file (without the .defaults...)
         and display it in the select_data_base text field */
#if HAVE_STRSTR
      strend = strstr(selection, DEFAULTS_SUFFIX);
#else
      strend = strstr(selection, DEFAULTS_SUFFIX);
#endif
      if(strend != NULL) strend[0] = '\0';
      if(data_params.select_data_base != NULL)
        XtFree(data_params.select_data_base);
      data_params.select_data_base = XtNewString(selection);
      show_string_text(ip->selectOtherText, data_params.select_data_base);
      }
      XtFree(selection);

  } else if (!strcmp(client, "selectOtherText")) {
      /*this is the value changed callback,
    make the OK/CANCEL buttons appear */
      /*fprintf (stderr, "dataSelect_callback: %s\n", client );*/
      selectOther_buttons_mapped =
    okTextValueChanged(ip->selectOtherText, ip->selectOtherOKPBtn,
               ip->selectOtherCancelPBtn,
               ip->dataSelectDialogShell,
               selectOther_buttons_mapped,
               data_params.select_data_base);
  } else if ((!strcmp(client, "selectOtherTextAct")) ||
         (!strcmp(client, "selectOtherOKPBtn"))) {
      /*fprintf (stderr, "dataSelect_callback: %s\n", client );*/
      String selectOtherTextval, with_defaults;
      int listPos;
      XmString xmstr;
      selectOther_buttons_mapped =
    okTextActivate(ip->selectOtherText, ip->selectOtherOKPBtn,
               ip->selectOtherCancelPBtn,
               &selectOtherTextval);

      /* save the new value */
      if(data_params.select_data_base != NULL)
    XtFree(data_params.select_data_base);
      data_params.select_data_base = XtNewString(selectOtherTextval);

      /* deselect any existing selection */
      XmListDeselectAllItems(ip->filesList);
      /* check if the string is in the current file list.
     highlight it and display it if it is */
      with_defaults = XtMalloc(strlen(selectOtherTextval) +
                   strlen(DEFAULTS_SUFFIX) + 1);
      strcpy(with_defaults, selectOtherTextval);
      strcat(with_defaults, DEFAULTS_SUFFIX);

      xmstr = XmStringCreateSimple(with_defaults);
      listPos = XmListItemPos(ip->filesList, xmstr);
      XmStringFree(xmstr);
      if(listPos > 0){
      int top, visible;
      /* the string is on the list, select it */
      XmListSelectPos(ip->filesList, listPos, False);
      XtVaGetValues(ip->filesList,
            XmNtopItemPosition, &top,
            XmNvisibleItemCount, &visible,
            NULL);
      if(listPos < top)
        XmListSetPos(ip->filesList, listPos);
      else if(listPos >= top+visible)
        XmListSetBottomPos(ip->filesList, listPos);
      }

      XtFree (selectOtherTextval);
    
      /* we could tell the user right now that things have changed,
     or we could wait until the Apply or Dismiss buttons have
     been selected and tell them then. I chose to wait */
 
  } else if (!strcmp(client, "selectOtherCancelPBtn")) {
      /*fprintf (stderr, "dataSelect_callback: %s\n", client );*/
      selectOther_buttons_mapped =
    okTextCancel(ip->selectOtherText, ip->selectOtherOKPBtn,
             ip->selectOtherCancelPBtn,
              data_params.select_data_base);
 
  } else if (!strcmp(client, "applyPBtn")) {
      String dirTextval, selection, strend;
      XmString *selectedItems;
      char newname[BUFSIZ];
      int selectedItemCount, i;

      /*fprintf (stderr, "dataSelect_callback: %s\n", client );*/
      dirTextval = XmTextGetString(ip->dirText);
      XtVaGetValues(ip->filesList, 
            XmNselectedItems, &selectedItems, 
            XmNselectedItemCount, &selectedItemCount,
            NULL);
      /* selectedItemCount should be 1 because we are only doing singles */
      for (i=0; i<selectedItemCount; i++) {
      /* get selection */
      if(XmStringGetLtoR(selectedItems[i], XmFONTLIST_DEFAULT_TAG,
                 &selection) != True) continue;
      /* build filename with path */
      sprintf (newname, "%s/%s", dirTextval, selection);

      /* strip defaults_suffix off to get just the database name
         and display it*/
#if HAVE_STRSTR
      strend = strstr(selection, DEFAULTS_SUFFIX);
#else
      strend = strstr(selection, DEFAULTS_SUFFIX);
#endif
      if(strend != NULL) strend[0] = '\0';
      if(data_params.select_data_base != NULL)
        XtFree(data_params.select_data_base);
      data_params.select_data_base = XtNewString(selection);
      show_string_text(ip->selectOtherText, data_params.select_data_base);

      XtFree(selection);
      }
      XtFree(dirTextval);
      
      /* because we have hit apply, tell the world about our selection
     and pop the window down, freeing the rest of the application
     from our APPLICATION_MODAL grab */
      if(data_params.data_path != NULL) XtFree(data_params.data_path);
      data_params.data_path = data_params.select_data_path;
      data_params.select_data_path = NULL;

      if(data_params.data_base != NULL) XtFree(data_params.data_base);
      data_params.data_base = data_params.select_data_base;
      data_params.select_data_base = NULL;

      data_request("select_data");
      close_popup(ip->dataSelectForm);
 
  } else if (!strcmp(client, "applyPBtn_arm")) {
      /*fprintf (stderr, "dataSelect_callback: %s\n", client );*/
      /* 
       * NOTE: this callback is here just so the visual effect of the
       * the Add button being pushed in will work.
       * BTW: I stumbled over this requirement by chance.  It is not
       * documented anywhere that I have found.  GRRRRR!
       * lss - 03-JUN-1994
       */
 
  } else if (!strcmp(client, "dismissPBtn")) {
      /*fprintf (stderr, "dataSelect_callback: %s\n", client );*/
      close_popup(ip->dataSelectForm);
 
  } else if (!strcmp(client, "helpPBtn")) {
      fprintf (stderr, "dataSelect_callback: %s\n", (char *)client );
 
  } else if (!strcmp(client, "selectAllPBtn")) {
      fprintf (stderr, "dataSelect_callback: %s\n", (char *)client );
 
  } else {
    fprintf (stderr, "dataSelect_callback: unknown client: %s\n", (char *)client);
  }
}

/******************************************************************************
******************************************************************************
*
* Module Name: fileEntryQsort
*
* Purpose: sort the filename array.
* 
* Calling Sequence:
*   qsort (filelist.file, filelist.n_files, sizeof(FileEntry), fileEntryQsort);
*   i = filenameQsort(fileEntry1, fileEntry2);
*
* Inputs:
*   FileEntry *fileEntry1     one element in filename array
*   FileEntry *fileEntry2     another element in filename array
*
* Outputs:
*   none
*
* Return Value:
*   +1  if (score1.start  < score2.start)
*    0  if (score1.start == score2.start)
*   -1  if (score1.start  > score2.start)
*
* Side Effects:
*   The qsort function re-arranges the elements of a given array 
*   according to this function's return value.
*
* Restrictions:
*   none
*
* Modification History:
*  25-Mar-1994     Linda Sue Sohn  initial creation
*
******************************************************************************
******************************************************************************
*/
 
int
fileEntryQsort(fileEntry1, fileEntry2)
     FileEntry *fileEntry1;
     FileEntry *fileEntry2;
{
  return (strcmp(fileEntry1->name, fileEntry2->name));
}

/******************************************************************************
******************************************************************************
*
* Module Name: dataSelectFillFilesList
*
* Purpose: 
*   Read the contents of the directory and put them in the file list.
*   Only put directories and files with the correct extension (.rec & .pprec)
*   in the list.
* 
* Calling Sequence:
*   dataSelectFillFilesList(homedir)
*
* Inputs:
*   char *homedir    directory name
*
* Outputs:
*   none
*
* Return Value:
*   False if "homedir" is not a valid directory name.
*   True otherwise.
*
* Side Effects:
*   The DataSelectClone->filesList and DataSelectClone->selectionLabel are updated.
*
* Restrictions:
*   none
*
* Modification History:
*  01-Jun-1994     Linda Sue Sohn  initial creation
*
******************************************************************************
******************************************************************************
*/
int dataSelectFillFilesList(homedir)
     char *homedir;
{
  int root = (!strcmp(homedir, "/"));
  struct dirent *dp;
  DIR *dirp, *dirp2;
  char filename[BUFSIZ], *dotextension;
  char message[BUFSIZ];
  int i;
  int ok;
  int directory;
  XmString xmstr;
  int dirnum;
  int filenum;
  
  /*
   * If this is not a valid directory name, then return.
   */
  if ((dirp=opendir(homedir)) == NULL) return False;
  
  /*
   * Re-initialize the list, so we don't mix up the
   * the contents of this directory list with any other.
   */
  for (i=0; i<data_params.select_filelist.n_files; i++) {
    XtFree(data_params.select_filelist.file[i].name);
  }
  XtFree(data_params.select_filelist.dirname);
  
  /*
   * Get number of entries in directory, so we can allocate the
   * maximum amount of space needed.  We'll reallocate when we're done.
   */
  i = 0;
  for (dp=readdir(dirp); 
       dp!=NULL; 
       dp=readdir(dirp)) {
    i++;
  }
  rewinddir(dirp);
  data_params.select_filelist.dirname = XtNewString(homedir);
  data_params.select_filelist.file =
    (FileEntry *)XtRealloc((char *)data_params.select_filelist.file, 
               (i+1) * sizeof(FileEntry));

  /* First entry is always "../ (Go up 1 directory level)" */
  data_params.select_filelist.file[0].name = XtNewString(PREVDIR);
  data_params.select_filelist.file[0].directory = True;
  
  /* fill in the file entries */
  for (dp=readdir(dirp), i=1; 
       dp!=NULL; 
       dp=readdir(dirp)) {
    if ((strcmp(dp->d_name, ".") != 0) && (strcmp(dp->d_name,"..") != 0)) {
      ok = FALSE; /* we don't know if we're going to list the file yet */
      if (root) {
    sprintf(filename, "/%s", dp->d_name);
      } else {
    sprintf(filename, "%s/%s", homedir, dp->d_name);
      }
      if ((dirp2=opendir(filename)) == NULL) {
    /* filename found; test for the right extension */
    if ((dotextension=strrchr(dp->d_name,'.')) != NULL) {
      /* '.' found; test for the right extension */
      if (!strcmp(dotextension, DEFAULTS_SUFFIX)){
        /* the file has the right extension; keep it */
        ok = TRUE;
        directory = False;
      }
    }
      } else {
    /* directory found; keep it */
    if (closedir(dirp2) != 0) perror(filename);
    ok = TRUE;
    directory = True;
      }
      /* insert the entry */
      if (ok) {
    if (directory) {
      int len;
      /* not just a XtNewString, because we need room for the slash */
      data_params.select_filelist.file[i].name =
        (char *)XtMalloc((strlen(dp->d_name)+2) * sizeof(char));
      strcpy (data_params.select_filelist.file[i].name, dp->d_name);
      /* put a "/" at the end of the filename to signify directory */
      len = strlen(dp->d_name);
      data_params.select_filelist.file[i].name[len+1] = '\0';
      data_params.select_filelist.file[i].name[len] = '/';
    } else {
      data_params.select_filelist.file[i].name = XtNewString(dp->d_name);
    }
    data_params.select_filelist.file[i].directory = directory;
    i++;
      }
    }
  }
  
  /* close the directory */
  if (closedir(dirp) != 0) perror(homedir);
  
  /* adjust allocated space of data_params.select_filelist.file */
  data_params.select_filelist.n_files = i;
  data_params.select_filelist.file =
    (FileEntry *)XtRealloc((char *)data_params.select_filelist.file, 
               i * sizeof(FileEntry));
  
  /* sort the entries alphabetically */
  qsort (data_params.select_filelist.file, data_params.select_filelist.n_files,
     sizeof(FileEntry), fileEntryQsort);

  /* display all the entries */
  XmListDeleteAllItems(DataSelectClone->filesList);
  for (i=0, dirnum=0, filenum=0; 
       i<data_params.select_filelist.n_files; 
       i++) {
    xmstr = XmStringCreateLtoR(data_params.select_filelist.file[i].name,
                   XmFONTLIST_DEFAULT_TAG);
    XmListAddItem (DataSelectClone->filesList, xmstr, 0);
    XmStringFree (xmstr);
    /* directory name or filename */
    if (data_params.select_filelist.file[i].directory) {
      dirnum++;
    } else {
      filenum++;
    }
  }

  /* Update the selection label */
  if ((1 == dirnum) && (1 == filenum)) {
    sprintf (message, "1 directory, 1 file");
  } else if ((1 == dirnum) && (1 != filenum)) {
    sprintf (message, "1 directory, %d files", filenum);
  } else if ((1 != dirnum) && (1 == filenum)) {
    sprintf (message, "%d directories, 1 file", dirnum);
  } else {
    sprintf (message, "%d directories, %d files", dirnum, filenum);
  }

  xmstr = XmStringCreateSimple(message);
  XtVaSetValues (DataSelectClone->selectionReportLabel,
         XmNlabelString, xmstr, NULL);
  XmStringFree(xmstr);

  return True;
}


void
dataSelect_request(String request)
{
    if(!strcmp(request, "open")){
    open_popup (DataSelectClone->dataSelectForm);
    } else if (!strcmp(request, "display")){
    /* display these values on the dataSelect window */
    show_string_text(DataSelectClone->dirText,
             data_params.select_data_path);
    dataSelectFillFilesList(data_params.select_data_path);
    show_string_text(DataSelectClone->selectOtherText,
             data_params.select_data_base);

    } else {
    fprintf(stderr, "data_request: unknown request: %s\n", request);
    }
}
