/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/data/data_callback.c,v 11.5 2001/08/23 20:16:09 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                      LNKNET Window Interface                         ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source: /data/speech/kukolich/cvsroot/lnknet/src/mclass/data/data_callback.c,v $
 *
 * Language: C
 *
 * File Modification History:
 *  23-Aug-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <Xm/Xm.h>
#include <nnlib.h>
#include "data_params.h"
/*
 * External Functions
 */
#include "data_func.h"
#include "../main/lnknet_subs.h"
#include "../lnk/lnk_func.h"
#include "../committeeData/committeeData_func.h"
 
/*
 * Externals
 */
#include "data.h"
extern DataParams data_params;
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: data_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   data_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  23-Aug-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
data_callback(Widget w, XtPointer client, XtPointer call)
{
    DataWidgets *ip;
    static int trainExt_buttons_mapped = False;
    static int evalExt_buttons_mapped = False;
    static int testExt_buttons_mapped = False;
    static int trainNpatterns_buttons_mapped = False;
    static int evalNpatterns_buttons_mapped = False;
    static int testNpatterns_buttons_mapped = False;
    FILE *fp;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "trainExtText")) {
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	/*fprintf (stderr, "data_callback: %s\n", client );*/
	trainExt_buttons_mapped =
	  okTextValueChanged(ip->trainExtText,
			     ip->trainExtOKPBtn,
			     ip->trainExtCancelPBtn,
			     ip->dataDialogShell,
			     trainExt_buttons_mapped,
			     data_params.ext_train);

    } else if ((!strcmp(client, "trainExtTextAct")) ||
	       (!strcmp(client, "trainExtOKPBtn"))) {
	String trainExtTextval;
	/*fprintf (stderr, "data_callback: %s\n", client );*/
	trainExt_buttons_mapped =
	  okTextActivate(ip->trainExtText,
			 ip->trainExtOKPBtn,
			 ip->trainExtCancelPBtn,
			 &trainExtTextval);
	/* we won't use this copy */
	XtFree(trainExtTextval);
 
	trainExtTextval =
	  set_string_not_empty(ip->trainExtText, "notebook file",
			       data_params.ext_train);
	XtFree(data_params.ext_train);
	data_params.ext_train = trainExtTextval;

	/* do whatever else is necessary */
	set_train_file();
	g_set_cv_filename();
	committeeData_set_lists();

    } else if (!strcmp(client, "trainExtCancelPBtn")) {
	/*fprintf (stderr, "data_callback: %s\n", client );*/
	trainExt_buttons_mapped = 
	  okTextCancel(ip->trainExtText, ip->trainExtOKPBtn,
		       ip->trainExtCancelPBtn,
		       data_params.ext_train);
 
    } else if (!strcmp(client, "trainNpatternsText")) {
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	/*fprintf (stderr, "data_callback: %s\n", client );*/
	trainNpatterns_buttons_mapped =
	  okTextValueChanged(ip->trainNpatternsText,
			     ip->trainNpatternsOKPBtn,
			     ip->trainNpatternsCancelPBtn,
			     ip->dataDialogShell,
			     trainNpatterns_buttons_mapped,
			     data_params.npatterns_train_string);
    } else if ((!strcmp(client, "trainNpatternsTextAct")) ||
	       (!strcmp(client, "trainNpatternsOKPBtn"))) {
	String trainNpatternsTextval;
	/*fprintf (stderr, "data_callback: %s\n", client );*/
	trainNpatterns_buttons_mapped =
	  okTextActivate(ip->trainNpatternsText, ip->trainNpatternsOKPBtn,
			 ip->trainNpatternsCancelPBtn, &trainNpatternsTextval);
	/* I won't actually be using this copy */
	XtFree (trainNpatternsTextval);
	
	/* verify the string as an integer */
	data_params.npatterns_train =
	  set_atoi_value(ip->trainNpatternsText, "data_trainNpatterns_set",
			 data_params.npatterns_train);

	/* save the current text for future comparisons and
	   set npatterns_train on any other windows as necessary */
	set_npatterns_train(data_params.npatterns_train);

	/* do anything else you need to */
	fp = open_data_base_file(data_params.ext_train);
	set_train_priors_string(fp);
	close_data_base_file(fp);
	
    } else if (!strcmp(client, "trainNpatternsCancelPBtn")) {
	/*fprintf (stderr, "data_callback: %s\n", client );*/
	trainNpatterns_buttons_mapped = 
	  okTextCancel(ip->trainNpatternsText, ip->trainNpatternsOKPBtn,
		       ip->trainNpatternsCancelPBtn,
		       data_params.npatterns_train_string);
 
    } else if (!strcmp(client, "evalExtText")) {
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	/*fprintf (stderr, "data_callback: %s\n", client );*/
	evalExt_buttons_mapped =
	  okTextValueChanged(ip->evalExtText,
			     ip->evalExtOKPBtn,
			     ip->evalExtCancelPBtn,
			     ip->dataDialogShell,
			     evalExt_buttons_mapped,
			     data_params.ext_eval);

    } else if ((!strcmp(client, "evalExtTextAct")) ||
	       (!strcmp(client, "evalExtOKPBtn"))) {
	String evalExtTextval;
	/*fprintf (stderr, "data_callback: %s\n", client );*/
	evalExt_buttons_mapped =
	  okTextActivate(ip->evalExtText,
			 ip->evalExtOKPBtn,
			 ip->evalExtCancelPBtn,
			 &evalExtTextval);
	/* we won't use this copy */
	XtFree(evalExtTextval);
 
	evalExtTextval =
	  set_string_not_empty(ip->evalExtText, "notebook file",
			       data_params.ext_eval);
	XtFree(data_params.ext_eval);
	data_params.ext_eval = evalExtTextval;

	/* do whatever else is necessary */
	set_eval_file();
	committeeData_set_lists();

    } else if (!strcmp(client, "evalExtCancelPBtn")) {
	/*fprintf (stderr, "data_callback: %s\n", client );*/
	evalExt_buttons_mapped = 
	  okTextCancel(ip->evalExtText, ip->evalExtOKPBtn,
		       ip->evalExtCancelPBtn,
		       data_params.ext_eval);
 
    } else if (!strcmp(client, "evalNpatternsText")) {
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	/*fprintf (stderr, "data_callback: %s\n", client );*/
	evalNpatterns_buttons_mapped =
	  okTextValueChanged(ip->evalNpatternsText,
			     ip->evalNpatternsOKPBtn,
			     ip->evalNpatternsCancelPBtn,
			     ip->dataDialogShell,
			     evalNpatterns_buttons_mapped,
			     data_params.npatterns_eval_string);

    } else if ((!strcmp(client, "evalNpatternsTextAct")) ||
	       (!strcmp(client, "evalNpatternsOKPBtn"))) {
	String evalNpatternsTextval;
	/*fprintf (stderr, "data_callback: %s\n", client );*/
	evalNpatterns_buttons_mapped =
	  okTextActivate(ip->evalNpatternsText, ip->evalNpatternsOKPBtn,
			 ip->evalNpatternsCancelPBtn, &evalNpatternsTextval);
	/* I won't actually be using this copy */
	XtFree (evalNpatternsTextval);
	
	/* verify the string as an integer */
	data_params.npatterns_eval =
	  set_atoi_value(ip->evalNpatternsText, "data_evalNpatterns_set",
			 data_params.npatterns_eval);

	/* save the current text for future comparisons */
	XtFree(data_params.npatterns_eval_string);
	data_params.npatterns_eval_string =
	  get_string_text(ip->evalNpatternsText);
	
	/* do anything else you need to */
	fp = open_data_base_file(data_params.ext_eval);
	set_eval_priors_string(fp);
	close_data_base_file(fp);
	
    } else if (!strcmp(client, "evalNpatternsCancelPBtn")) {
	/*fprintf (stderr, "data_callback: %s\n", client );*/
	evalNpatterns_buttons_mapped = 
	  okTextCancel(ip->evalNpatternsText, ip->evalNpatternsOKPBtn,
		       ip->evalNpatternsCancelPBtn,
		       data_params.npatterns_eval_string);
 
    } else if (!strcmp(client, "testExtText")) {
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	/*fprintf (stderr, "data_callback: %s\n", client );*/
	testExt_buttons_mapped =
	  okTextValueChanged(ip->testExtText,
			     ip->testExtOKPBtn,
			     ip->testExtCancelPBtn,
			     ip->dataDialogShell,
			     testExt_buttons_mapped,
			     data_params.ext_test);

    } else if ((!strcmp(client, "testExtTextAct")) ||
	       (!strcmp(client, "testExtOKPBtn"))) {
	String testExtTextval;
	/*fprintf (stderr, "data_callback: %s\n", client );*/
	testExt_buttons_mapped =
	  okTextActivate(ip->testExtText,
			 ip->testExtOKPBtn,
			 ip->testExtCancelPBtn,
			 &testExtTextval);
	/* we won't use this copy */
	XtFree(testExtTextval);
 
	testExtTextval =
	  set_string_not_empty(ip->testExtText, "notebook file",
			       data_params.ext_test);
	XtFree(data_params.ext_test);
	data_params.ext_test = testExtTextval;

	/* do whatever else is necessary */
	set_test_file();
	committeeData_set_lists();

    } else if (!strcmp(client, "testExtCancelPBtn")) {
	/*fprintf (stderr, "data_callback: %s\n", client );*/
	testExt_buttons_mapped = 
	  okTextCancel(ip->testExtText, ip->testExtOKPBtn,
		       ip->testExtCancelPBtn,
		       data_params.ext_test);

    } else if (!strcmp(client, "testNpatternsText")) {
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	/*fprintf (stderr, "data_callback: %s\n", client );*/
	testNpatterns_buttons_mapped =
	  okTextValueChanged(ip->testNpatternsText,
			     ip->testNpatternsOKPBtn,
			     ip->testNpatternsCancelPBtn,
			     ip->dataDialogShell,
			     testNpatterns_buttons_mapped,
			     data_params.npatterns_test_string);
    } else if ((!strcmp(client, "testNpatternsTextAct")) ||
	       (!strcmp(client, "testNpatternsOKPBtn"))) {
	String testNpatternsTextval;
	/*fprintf (stderr, "data_callback: %s\n", client );*/
	testNpatterns_buttons_mapped =
	  okTextActivate(ip->testNpatternsText, ip->testNpatternsOKPBtn,
			 ip->testNpatternsCancelPBtn, &testNpatternsTextval);
	/* I won't actually be using this copy */
	XtFree (testNpatternsTextval);
	
	/* verify the string as an integer */
	data_params.npatterns_test =
	  set_atoi_value(ip->testNpatternsText, "data_testNpatterns_set",
			 data_params.npatterns_test);

	/* save the current text for future comparisons */
	XtFree(data_params.npatterns_test_string);
	data_params.npatterns_test_string =
	  get_string_text(ip->testNpatternsText);

	/* do anything else you need to */
	fp = open_data_base_file(data_params.ext_test);
	set_test_priors_string(fp);
	close_data_base_file(fp);
	
    } else if (!strcmp(client, "testNpatternsCancelPBtn")) {
	/*fprintf (stderr, "data_callback: %s\n", client );*/
	testNpatterns_buttons_mapped = 
	  okTextCancel(ip->testNpatternsText, ip->testNpatternsOKPBtn,
		       ip->testNpatternsCancelPBtn,
		       data_params.npatterns_test_string);
 
    } else if (!strcmp(client, "defaultsPBtn")) {
	/*fprintf (stderr, "data_callback: %s\n", client );*/
	data_params.defaults_nraw = data_params.nraw;
	data_params.defaults_noutputs = data_params.noutputs;
	data_params.defaults_db_type = data_params.db_type;
	/* copy the input and class labels lists */
	if(data_params.input_labels != NULL)
	  data_params.defaults_input_labels =
	    copy_string_vector(data_params.input_labels,
			       atoi(data_params.input_labels[0])+1);
	else data_params.defaults_input_labels = NULL;
	if(data_params.class_labels != NULL)
	  data_params.defaults_class_labels =
	    copy_string_vector(data_params.class_labels,
			       atoi(data_params.class_labels[0])+1);
	else data_params.defaults_class_labels = NULL;
	if(data_params.categories != NULL)
	  data_params.defaults_categories =
	    copy_static2alloc_ivector(data_params.categories);

	defaults_request("display");
	defaults_request("open");

  } else if (!strcmp(client, "dataSelectionPBtn")) {
      /*fprintf (stderr, "data_callback: %s\n", client );*/
      /* initialize the path and data base fields */
      if(data_params.select_data_path != NULL)
	XtFree(data_params.select_data_path);
      data_params.select_data_path = XtNewString(data_params.data_path);

      if(data_params.select_data_base != NULL)
	XtFree(data_params.select_data_base);
      data_params.select_data_base = XtNewString(data_params.data_base);

      dataSelect_request("display");
      dataSelect_request("open");

    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "data_callback: %s\n", client );*/
	close_popup(ip->dataForm);

    } else {
	fprintf (stderr, "data_callback: unknown client: %s\n", (char *)client);
    }
}

