/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/data/data_request.c,v 11.7 2001/09/17 21:22:49 kukolich Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            LNKNET                                    ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source: /data/speech/kukolich/cvsroot/lnknet/src/mclass/data/data_request.c,v $
 *
 * Language: C
 *
 * File Modification History:
 *  23-Aug-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <Xm/Xm.h>
#include <sys/param.h>
#include <nnlib.h>
#include "data_params.h"
/*
 * External Functions
 */
#include "../main/lnknet_subs.h"
#include "data_func.h"
#include "../lnk/lnk_func.h"
#include "../reports/reports_func.h"
#include "../norm/norm_func.h"
#include "../select/select_func.h"
#include "../priors/priors_func.h"
#include "../region/region_func.h"
#include "../profile/profile_func.h"
#include "../prob/prob_func.h"
#include "../detect/detect_func.h"
#include "../committeeData/committeeData_func.h"
 
/*
 * Externals
 */
#include "data.h"
extern DataWidgets *DataClone;
DataParams data_params; /* this is the real one */
#include "../main/lnknet.h"
extern LnkGlyphs *LnknetClone;
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: data_request
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   data_request(request)
 *
 * Inputs:
 *   String request   Text of request
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   depend on the request
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  23-Aug-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
void
data_request(String request)
{
    if (!strcmp(request, "open")) {
    open_popup(DataClone->dataForm);
    } else if (!strcmp(request, "init")){
    /* this should only get called once, before ANYTHING 
       gets allocated or there will be memory leaks */
    data_params.data_path = NULL;
    data_params.data_base = NULL;
    data_params.desc_fname = NULL;
    data_params.ext_train = NULL;
    data_params.ext_test_train = ".test_on_train";
    data_params.ext_eval = NULL;
    data_params.ext_test = NULL;
    data_params.train_fname = NULL;
    data_params.eval_fname = NULL;
    data_params.test_fname = NULL;
    data_params.npatterns_train = 0;
    data_params.npatterns_train_string = NULL;
    data_params.npatterns_eval = 0;
    data_params.npatterns_eval_string = NULL;
    data_params.npatterns_test = 0;
    data_params.npatterns_test_string = NULL;
    data_params.nraw = 0;
    data_params.noutputs = 0;
    data_params.db_type = 0;
    data_params.class_labels = NULL;
    data_params.input_labels = NULL;
    data_params.categories = NULL;
    data_params.priors_string = NULL;
    data_params.priors_list = NULL;
    data_params.select_data_path = NULL;
    data_params.select_data_base = NULL;
    data_params.defaults_nraw = 0;
    data_params.defaults_nraw_string = NULL;
    data_params.defaults_noutputs = 0;
    data_params.defaults_noutputs_string = NULL;
    data_params.defaults_input_labels = NULL;
    data_params.defaults_input_labels_string = NULL;
    data_params.defaults_class_labels = NULL;
    data_params.defaults_class_labels_string = NULL;
    data_params.defaults_categories = NULL;
    data_params.defaults_categories_string = NULL;
    data_params.defaults_db_type = 0;
    data_params.select_filelist.n_files = 0;
    data_params.select_filelist.dirname = NULL;
    data_params.select_filelist.file = NULL;
    } else if (!strcmp(request, "display")){
    data_set_data_base();

    show_string_text(DataClone->evalExtText, data_params.ext_eval);
    show_string_text(DataClone->testExtText, data_params.ext_test);
    show_string_text(DataClone->trainExtText, data_params.ext_train);

    } else if (!strcmp(request, "select_data")){
    data_set_data_base();
    norm_check_file();
    select_check_file();
    } else {
    fprintf (stderr, "data_request: unknown request: %s\n", request);
    }
}

 
/******************************************************************************
 ******************************************************************************
 *
 * Module Names: read_data_defaults, write_data_defaults
 *
 * Purpose: 
 *   Handle reading and writing the state of the data and data base defaults
 *   windows to a screen/defaults file.
 * 
 * Calling Sequence:
 *   # open defaults file for reading
 *   # read arg tables up until the data one
 *   # fill arg list for data one
 *   read_data_defaults(argc, argv)
 *
 *  OR
 *   # open a defaults file for writings
 *   # write arg tables up until the data one
 *   write_data_defaults(default_file)
 *
 * Inputs:
 *   int argc; # the number of strings in argv
 *   char **argv; # pointer to an array String pointers
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   read sets values in data_params structure according to flags found in argv
 *   write writes flags to default_file according to the settings of
 *   data_params structure entries
 *
 * Restrictions:
 *   On read, if the argument list in argv doesn't match the arguments
 *   in the data argument table, a fatal error will occur.
 *
 * Modification History:
 *  23-Aug-1995     Linda C Kukolich  modified original xview versions
 *
 ******************************************************************************
 ******************************************************************************
 */
static ARG datatab[] =
{
    {"data_path", STRING, (int *)NULL, "full or relative path to data"},
    {"data_base", STRING, (int *)NULL, "data base name"},
    {"ext_train", STRING, (int *)NULL, "extension for training file"},
    {"ext_eval", STRING, (int *)NULL, "extension for eval file"},
    {"ext_test", STRING, (int *)NULL, "extension for test file"},
    {"npatterns_train", INT, (int *)NULL, "npatterns to read from train"},
    {"npatterns_eval", INT, (int *)NULL, "npatterns to read from eval"},
    {"npatterns_test", INT, (int *)NULL, "npatterns to read from test"},
    {"db_type", INT, (int *)NULL, "data base type, class, seq, or map"},
    {"ninputs", INT, (int *)NULL, "number of inputs in this data base"},
    {"noutputs", INT, (int *)NULL, "number of outputs in this data base"},
    {"input_labels", STRING_ALLOC_LIST, (int *)NULL, "input feature labels"},
    {"class_labels", STRING_ALLOC_LIST, (int *)NULL, "class labels"},
};
#define TABSIZE (sizeof(datatab)/sizeof(ARG))

static void
data_set_defaults_variables(dp)
     DataParams *dp;
{
    nnargs_set_variable("data_path", (int *)&dp->data_path, datatab, TABSIZE);
    nnargs_set_variable("data_base", (int *)&dp->data_base, datatab, TABSIZE);
    nnargs_set_variable("ext_train", (int *)&dp->ext_train, datatab, TABSIZE);
    nnargs_set_variable("ext_eval", (int *)&dp->ext_eval, datatab, TABSIZE);
    nnargs_set_variable("ext_test", (int *)&dp->ext_test, datatab, TABSIZE);
    nnargs_set_variable("npatterns_train", &dp->npatterns_train,
            datatab, TABSIZE);
    nnargs_set_variable("npatterns_eval", &dp->npatterns_eval,
            datatab, TABSIZE);
    nnargs_set_variable("npatterns_test", &dp->npatterns_test,
            datatab, TABSIZE);
    nnargs_set_variable("db_type", &dp->db_type, datatab, TABSIZE);
    nnargs_set_variable("ninputs", &dp->nraw, datatab, TABSIZE);
    nnargs_set_variable("noutputs", &dp->noutputs, datatab, TABSIZE);
    nnargs_set_variable("input_labels", (int *)&dp->input_labels,
            datatab, TABSIZE);
    nnargs_set_variable("class_labels", (int *)&dp->class_labels,
            datatab, TABSIZE);
}

void
read_data_defaults(argc, argv)
     int argc;
     char **argv;
{
    /* clear input_labels and class_labels, which are not there if they
       were NULL before */
    if (data_params.data_path != NULL){
    XtFree(data_params.data_path);
    data_params.data_path = NULL;
    }
    if (data_params.data_base != NULL){
    XtFree(data_params.data_base);
    data_params.data_base = NULL;
    }
    if (data_params.ext_train != NULL){
    XtFree(data_params.ext_train);
    data_params.ext_train = NULL;
    }
    if (data_params.ext_eval != NULL){
    XtFree(data_params.ext_eval);
    data_params.ext_eval = NULL;
    }
    if (data_params.ext_test != NULL){
    XtFree(data_params.ext_test);
    data_params.ext_test = NULL;
    }
    if (data_params.input_labels != NULL){
    deallocate_string_vector(data_params.input_labels,
                 atoi(data_params.input_labels[0]));
    data_params.input_labels = NULL;
    }
    if (data_params.class_labels != NULL){
    deallocate_string_vector(data_params.class_labels,
                 atoi(data_params.class_labels[0]));
    data_params.class_labels = NULL;
    }
    /*    if (data_params.categories != NULL){
      deallocate_ivector(data_params.categories);
      data_params.categories = NULL;
      }*/
    data_params.categories = NULL;

    data_set_defaults_variables(&data_params);

    if(argc > 0){
    argc = nnargs(argc, argv, datatab, TABSIZE);
    /* copy the strings into new memory */
    /* Linda Sue checks that this is a valid directory ... */
    data_params.data_path = XtNewString(data_params.data_path);
    data_params.data_base = XtNewString(data_params.data_base);
    data_params.ext_eval = XtNewString(data_params.ext_eval);
    data_params.ext_test = XtNewString(data_params.ext_test);
    data_params.ext_train = XtNewString(data_params.ext_train);
    /* the input_labels and class_labels are properly copied into
       new space by nnargs */
    } else {
    char current_dir[MAXPATHLEN];
    /* use the current directory as the default */
    if(getcwd(current_dir,MAXPATHLEN) == 0){
        fatal_error("read_data_defaults: getcwd returned %s",
            current_dir);
    }
    data_params.data_path = XtNewString(current_dir);
    data_params.data_base = XtNewString("vowel");
    data_params.ext_train = XtNewString(".train");
    data_params.ext_eval = XtNewString(".eval");
    data_params.ext_test = XtNewString(".test");
    }

    data_request("display");
}

void
write_data_defaults(default_file)
     FILE *default_file;
{
    data_set_defaults_variables(&data_params);

    if(default_file != NULL){
    fprintf(default_file, "data %s",
        nnargs_get_flags(datatab, TABSIZE));
    }
}
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Names: set_train_file, set_eval_file, set_test_file
 *
 * Purpose: 
 *   Set the training, evaluation, or testing, file, after the data base
 *   or file name extension or path has changed
 * 
 * Calling Sequence:
 *   # set the data base name and file extension
 *   set_train_file()
 *
 * Inputs:
 *   none
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   counts the number of patterns in the appropriate file, and sets the
 *   selected number of patterns from the file to that value.
 *   counts the priors numbers for the appropriate file and displays them
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  28-Aug-1995     Linda C Kukolich  copied from xview versions
 *
 ******************************************************************************
 ******************************************************************************
 */

FILE *
open_data_base_file(String ext)
{
    FILE *fp;
#define MAXSTR 256
    char data_base_tmp[MAXSTR];

    strcpy(data_base_tmp, data_params.data_base);
    strcat(data_base_tmp, ext);
    fp = fopen(path_build(data_params.data_path, data_base_tmp), "r");
    return fp;
}

void
close_data_base_file(fp)
     FILE *fp;
{
    if(fp != NULL) fclose(fp);
}

void
set_priors_string(fp, npatterns)
    FILE *fp;
    int npatterns;
{
    if(fp != NULL){
    if(data_params.priors_string != NULL)
      XtFree(data_params.priors_string);
    if(data_params.db_type == CLASS_DATA){
        if(data_params.priors_list != NULL){
        find_class_priors(fp, npatterns, data_params.noutputs,
                  data_params.priors_list);
        data_params.priors_string = 
          make_float_string(data_params.priors_list);
        } else {
        data_params.priors_string = XtNewString("noutputs unknown");
        }
    } else {
        data_params.priors_string = XtNewString("Not classification data");
    }
    } else {
    if(data_params.db_type == CLASS_DATA){
        data_params.priors_string = XtNewString("No File");
    } else {
        data_params.priors_string = XtNewString("Not classification data");
    }
    }
}

void
set_npatterns_train(int value)
{
    char string [10];
    data_params.npatterns_train = value;

    /* save the current text for future comparisons */
    sprintf(string, "%d", value);
    XtFree(data_params.npatterns_train_string);
    data_params.npatterns_train_string = XtNewString(string);
    show_string_text(DataClone->trainNpatternsText,
             data_params.npatterns_train_string);
    priors_request("set_npatterns");
}

void
set_train_priors_string(FILE *fp)
{
    set_priors_string(fp, data_params.npatterns_train);
    show_string_label(DataClone->trainPatternsPerClassLabel,
              data_params.priors_string);
    priors_request("show_train_priors");/* quick, while the string is valid! */
}

void
set_train_file()
{
    FILE *fp;

    fp = open_data_base_file(data_params.ext_train);
    if(fp != NULL) set_npatterns_train(get_n_lines(fp));
    else set_npatterns_train(0);
    set_train_priors_string(fp);
    close_data_base_file(fp);
    XtFree(data_params.train_fname);
    data_params.train_fname = XtNewString(data_params.data_base);
    data_params.train_fname = StringConcat(data_params.train_fname,
                       data_params.ext_train);
    show_string_label(DataClone->trainFileLabel, data_params.train_fname);
}

void
set_npatterns_eval(int value)
{
    char string [10];
    data_params.npatterns_eval = value;

    /* save the current text for future comparisons */
    sprintf(string, "%d", value);
    XtFree(data_params.npatterns_eval_string);
    data_params.npatterns_eval_string = XtNewString(string);
    show_string_text(DataClone->evalNpatternsText,
             data_params.npatterns_eval_string);
}

void
set_eval_priors_string(FILE *fp)
{
    set_priors_string(fp, data_params.npatterns_eval);
    show_string_label(DataClone->evalPatternsPerClassLabel,
              data_params.priors_string);
    priors_request("show_eval_priors");/* quick, while the string is valid! */
}

void
set_eval_file()
{
    FILE *fp;

    fp = open_data_base_file(data_params.ext_eval);
    if(fp != NULL) set_npatterns_eval(get_n_lines(fp));
    else set_npatterns_eval(0);
    set_eval_priors_string(fp);
    close_data_base_file(fp);
    XtFree(data_params.eval_fname);
    data_params.eval_fname = XtNewString(data_params.data_base);
    data_params.eval_fname = StringConcat(data_params.eval_fname,
                      data_params.ext_eval);
    show_string_label(DataClone->evalFileLabel, data_params.eval_fname);
}

void
set_npatterns_test(int value)
{
    char string [10];
    data_params.npatterns_test = value;

    /* save the current text for future comparisons */
    sprintf(string, "%d", value);
    XtFree(data_params.npatterns_test_string);
    data_params.npatterns_test_string = XtNewString(string);
    show_string_text(DataClone->testNpatternsText,
             data_params.npatterns_test_string);
}

void
set_test_priors_string(FILE *fp)
{
    set_priors_string(fp, data_params.npatterns_test);
    show_string_label(DataClone->testPatternsPerClassLabel,
              data_params.priors_string);
    priors_request("show_test_priors");/* quick, while the string is valid! */
}

void
set_test_file()
{
    FILE *fp;

    fp = open_data_base_file(data_params.ext_test);
    if(fp != NULL) set_npatterns_test(get_n_lines(fp));
    else set_npatterns_test(0);
    set_test_priors_string(fp);
    close_data_base_file(fp);
    XtFree(data_params.test_fname);
    data_params.test_fname = XtNewString(data_params.data_base);
    data_params.test_fname = StringConcat(data_params.test_fname,
                      data_params.ext_test);
    show_string_label(DataClone->testFileLabel, data_params.test_fname);
}
int
set_description_values_data_base ()
{
    char *desc_fullname;
    FILE *desc_file;
    int rv;
    DEFAULTS_FILE_STRUCT df_args;
    char *labels_string;
    static int direction = 1;
    Dimension height, width;
    
    if(data_params.desc_fname != NULL) XtFree(data_params.desc_fname);
    data_params.desc_fname = XtMalloc(strlen(data_params.data_base) +
                      strlen(DEFAULTS_SUFFIX) + 1);
    strcpy(data_params.desc_fname, data_params.data_base);
    strcat(data_params.desc_fname, DEFAULTS_SUFFIX);

    /* reset flags */
    df_args.input_labels = NULL;
    df_args.output_labels = NULL;
    data_params.db_type = df_args.db_type = CLASS_DATA;
    df_args.ninputs = df_args.noutputs = 0;
    df_args.input_types = NULL;
    df_args.input_type_values = NULL;
    df_args.categories = NULL;

    desc_fullname = path_create(data_params.data_path, data_params.desc_fname);
    if ((desc_file = fopen(desc_fullname, "r")) == NULL) {
    rv = -1;
    } else {
    read_defaults_file(desc_fullname, &df_args);
    nn_fclose(desc_file, desc_fullname, "for reading data description");
    rv = 0;
    }
    nn_free(desc_fullname, "for reading data description");

    data_params.nraw = df_args.ninputs;
    data_params.noutputs = df_args.noutputs;
    if(data_params.class_labels != NULL){
    deallocate_string_vector(data_params.class_labels,
                 atoi(data_params.class_labels[0])+1);
    }
    data_params.class_labels = df_args.output_labels;
    if(data_params.input_labels != NULL){
    deallocate_string_vector(data_params.input_labels,
                 atoi(data_params.input_labels[0])+1);
    }
    data_params.input_labels = df_args.input_labels;
    if(data_params.categories != NULL){
      deallocate_ivector(data_params.categories);
    }
    data_params.categories = df_args.categories;

    /* keep the window the same size instead of automatically growing
       as soon as the labels string gets longer */
    XtVaGetValues(DataClone->dataForm, XmNheight, &height,
          XmNwidth, &width, NULL);
    /*printf("dataForm width:%d height:%d\n", width, height);*/

    /* display the information from the defaults file */
    show_int_label(DataClone->nInputsLabel, data_params.nraw);
    show_int_label(DataClone->nOutputsLabel, data_params.noutputs);

    labels_string = make_labels_string(data_params.input_labels);
    show_string_label(DataClone->inputLabelsLabel, labels_string);
    nn_free(labels_string, "for make_labels_string");

    labels_string = make_labels_string(data_params.class_labels);
    show_string_label(DataClone->classLabelsLabel, labels_string);
    nn_free(labels_string, "for make_labels_string");

    /* set the window width to the same as it was */
    XtVaSetValues(DataClone->dataForm,
          XmNheight, height,
          XmNwidth, width+direction, NULL);
    direction *= -1;
    
    /* update ninputs setting */
    update_nraw_and_ninputs();
    set_feature_labels_string();
    committeeData_update_noutputs();
    region_set_input_label();
    profile_set_input_label();

    /* update class labels on plots */
    prob_set_target_label();
    detect_set_target_label();

    return rv;
}

/******************************************************************************
 ******************************************************************************
 *
 * Module Name: data_set_data_base()
 *
 * Purpose: 
 * This is a combination of what used to be update_data_base and
 * data_set_data_base. Since update_data_base was only called along with
 * data_set_data_base in the Motif version and since it already called
 * update_data_base, I decided to combine them.
 *
 * data_set_data_base is used when a new data base is selected 
 * and when reading the defaults from a screen file, for setting data
 * base dependent file names
 * setting the number of training, testing, and evaluation npatterns,
 * and the priors strings,
 * it also checks if the warning should be posted for having a bad
 * data base name.
 *
 * Calling Sequence:
 *   data_set_data_base()
 *
 * Inputs:
 *   none
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   data_set_data_base sets the normalization, feature search, cross
 *     validation and committee data base names. finds the data base
 *     description file, sets the number
 *     of patterns and the number of patterns per class for each file type
 *     if the data base description file is bad, sets the warning flag
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  23-Aug-1995     Linda C Kukolich  brought over from Xview version
 *
 ******************************************************************************
 ******************************************************************************
 */

void
data_set_data_base()
{
    /* display data base as selection */
    show_string_label(DataClone->dataPathLabel, data_params.data_path);
    show_string_label(DataClone->dataBaseLabel, data_params.data_base);

    g_set_normalize_filename();
    g_create_search_filename();

    set_committee_data_base(data_params.data_base);
    g_set_cv_filename();

    /* gray out the data base list and data file text widget so no one
       does a selection until this is done. */
    show_string_label(DataClone->errorLabel,
              "Counting patterns per class in data files");
    show_cursor(LnknetClone->busyCursor);

    /* update the data base info on defaults file */
    /* read the data base description when filename changes */

    if(data_params.priors_list != NULL){
    deallocate_vector(data_params.priors_list);
    data_params.priors_list = NULL;
    }
    if(set_description_values_data_base() < 0){
    /* set error message to say couldn't find data base */
    show_string_label(DataClone->errorLabel,
              "Error: Cannot read data base defaults file");
    show_image_label(DataClone->warningGlyphLabel,
             LnknetClone->largeWarning);
    lnk_request("dataWarningOn");
    data_params.priors_list = NULL;
    } else {
    show_string_label(DataClone->errorLabel, "");
    show_image_label(DataClone->warningGlyphLabel,
             LnknetClone->largeBlank);
    lnk_request("dataWarningOff");
    data_params.priors_list = allocate_vector(data_params.noutputs+1);
    data_params.priors_list[0] = (float)data_params.noutputs;
    }
    set_test_file();
    set_eval_file();
    set_train_file();
    
    /* fix the cursor */
    show_cursor(LnknetClone->cursor);

    /* tell notebook that something important changed */
    reports_request("major_note");

}

