/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/data/defaults_callback.c,v 11.5 2001/08/23 20:16:13 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            Lnknet                                    ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source: /data/speech/kukolich/cvsroot/lnknet/src/mclass/data/defaults_callback.c,v $
 *
 * Language: C
 *
 * File Modification History:
 *  23-Aug-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <Xm/Xm.h>
#include <nnlib.h>
#include "../main/lnknet_subs.h"
#include "data_func.h"

/*
 * External Functions
 */
static int data_check_df_args(int saving);
#define SAVING 1
#define NOT_SAVING 0
 
/*
 * Externals
 */
#include "defaults.h"
extern DefaultsWidgets *DefaultsClone;
#include "data_params.h"
extern DataParams data_params;
#include "../main/lnknet.h"
extern LnkGlyphs *LnknetClone;
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: defaults_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   defaults_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  23-Aug-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
defaults_callback(Widget w, XtPointer client, XtPointer call)
{
    DefaultsWidgets *ip;
    static int outputLabels_buttons_mapped = False;
    static int categories_buttons_mapped = False;
    static int inputLabels_buttons_mapped = False;
    static int nOutputs_buttons_mapped = False;
    static int nInputs_buttons_mapped = False;
    
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "outputLabelsText")) {
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	/*fprintf (stderr, "defaults_callback: %s\n", client );*/
	outputLabels_buttons_mapped =
	  okTextValueChanged(ip->outputLabelsText,
			     ip->outputLabelsOKPBtn,
			     ip->outputLabelsCancelPBtn,
			     ip->defaultsDialogShell,
			     outputLabels_buttons_mapped,
			     data_params.defaults_class_labels_string);
    } else if ((!strcmp(client, "outputLabelsTextAct")) ||
	       (!strcmp(client, "outputLabelsOKPBtn"))) {
	/*fprintf (stderr, "defaults_callback: %s\n", client );*/
	String outputLabelsTextval;
	outputLabels_buttons_mapped =
	  okTextActivate(ip->outputLabelsText,
			 ip->outputLabelsOKPBtn,
			 ip->outputLabelsCancelPBtn,
			 &outputLabelsTextval);

	/* convert the string to a label list and free it */
	if(data_params.defaults_class_labels != NULL)
	  deallocate_string_vector(data_params.defaults_class_labels,
				   atoi(data_params.defaults_class_labels[0])+1);
	data_params.defaults_class_labels = NULL;
	/* take spaces and tabs out of the list before handing it to nnargs */
	if(outputLabelsTextval != NULL){
	    char *outputLabelsTemp = outputLabelsTextval;
	    outputLabelsTextval = strtok(outputLabelsTextval, " \t");
	    if(outputLabelsTextval != NULL){
		data_params.defaults_class_labels =
		  nnargs_get_string_alloc_list(outputLabelsTextval);
		XtFree(outputLabelsTemp);
	    }
	}

	/* make a copy of the string as it has been interpretted */
	XtFree(data_params.defaults_class_labels_string);
	data_params.defaults_class_labels_string =
	  make_labels_string(data_params.defaults_class_labels);
	show_string_text(ip->outputLabelsText,
			 data_params.defaults_class_labels_string);

	data_check_df_args(NOT_SAVING);
 
    } else if (!strcmp(client, "outputLabelsCancelPBtn")) {
	/*fprintf (stderr, "defaults_callback: %s\n", client );*/
	outputLabels_buttons_mapped = 
	  okTextCancel(ip->outputLabelsText, ip->outputLabelsOKPBtn,
		       ip->outputLabelsCancelPBtn,
		       data_params.defaults_class_labels_string);

    } else if (!strcmp(client, "categoriesText")) {
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	/*fprintf (stderr, "defaults_callback: %s\n", client );*/
	categories_buttons_mapped =
	  okTextValueChanged(ip->categoriesText,
			     ip->categoriesOKPBtn,
			     ip->categoriesCancelPBtn,
			     ip->defaultsDialogShell,
			     categories_buttons_mapped,
			     data_params.defaults_categories_string);
    } else if ((!strcmp(client, "categoriesTextAct")) ||
	       (!strcmp(client, "categoriesOKPBtn"))) {
	/*fprintf (stderr, "defaults_callback: %s\n", client );*/
	String categoriesTextval;
	categories_buttons_mapped =
	  okTextActivate(ip->categoriesText,
			 ip->categoriesOKPBtn,
			 ip->categoriesCancelPBtn,
			 &categoriesTextval);

	/* convert the string to a label list and free it */
	if(data_params.defaults_categories != NULL)
	  deallocate_ivector(data_params.defaults_categories);
	data_params.defaults_categories = NULL;
	/* take spaces and tabs out of the list before handing it to nnargs */
	if(categoriesTextval != NULL){
	    char *categoriesTemp = categoriesTextval;
	    categoriesTextval = strtok(categoriesTextval, " \t");
	    if(categoriesTextval != NULL){
		data_params.defaults_categories =
		  nnargs_get_int_alloc_list(categoriesTextval);
		XtFree(categoriesTemp);
	    }
	}

	/* make a copy of the string as it has been interpretted */
	XtFree(data_params.defaults_categories_string);
	data_params.defaults_categories_string =
	  make_int_string(data_params.defaults_categories);
	show_string_text(ip->categoriesText,
			 data_params.defaults_categories_string);

	data_check_df_args(NOT_SAVING);
 
    } else if (!strcmp(client, "categoriesCancelPBtn")) {
	/*fprintf (stderr, "defaults_callback: %s\n", client );*/
	categories_buttons_mapped = 
	  okTextCancel(ip->categoriesText, ip->categoriesOKPBtn,
		       ip->categoriesCancelPBtn,
		       data_params.defaults_categories_string);

    } else if (!strcmp(client, "inputLabelsText")) {
	/*fprintf (stderr, "defaults_callback: %s\n", client);*/
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	inputLabels_buttons_mapped =
	  okTextValueChanged(ip->inputLabelsText, ip->inputLabelsOKPBtn,
			     ip->inputLabelsCancelPBtn,
			     ip->defaultsDialogShell,
			     inputLabels_buttons_mapped,
			     data_params.defaults_input_labels_string);
    } else if ((!strcmp(client, "inputLabelsTextAct")) ||
	       (!strcmp(client, "inputLabelsOKPBtn"))) {
	/*fprintf (stderr, "defaults_callback: %s\n", client );*/
	String inputLabelsTextval;
	inputLabels_buttons_mapped =
	  okTextActivate(ip->inputLabelsText,
			 ip->inputLabelsOKPBtn,
			 ip->inputLabelsCancelPBtn,
			 &inputLabelsTextval);
	
	if(data_params.defaults_input_labels != NULL)
	  deallocate_string_vector(data_params.defaults_input_labels,
				   atoi(data_params.defaults_input_labels[0])+1);
	data_params.defaults_input_labels = NULL;
	/* take spaces and tabs out of the list before handing it to nnargs */
	if(inputLabelsTextval != NULL){
	    char *inputLabelsTemp = inputLabelsTextval;
	    inputLabelsTextval = strtok(inputLabelsTextval, " \t");
	    if(inputLabelsTextval != NULL){
		data_params.defaults_input_labels =
		  nnargs_get_string_alloc_list(inputLabelsTextval);
		XtFree(inputLabelsTemp);
	    }
	}

	/* make a copy of the string as it has been interpretted */
	XtFree(data_params.defaults_input_labels_string);
	data_params.defaults_input_labels_string =
	  make_labels_string(data_params.defaults_input_labels);
	show_string_text(ip->inputLabelsText,
			 data_params.defaults_input_labels_string);
	
	data_check_df_args(NOT_SAVING);
 
    } else if (!strcmp(client, "inputLabelsCancelPBtn")) {
	/*fprintf (stderr, "defaults_callback: %s\n", client );*/
	inputLabels_buttons_mapped = 
	  okTextCancel(ip->inputLabelsText, ip->inputLabelsOKPBtn,
		       ip->inputLabelsCancelPBtn,
		       data_params.defaults_input_labels_string);

    } else if (!strcmp(client, "nOutputsText")) {
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	/*fprintf (stderr, "defaults_callback: %s\n", client );*/
	nOutputs_buttons_mapped =
	  okTextValueChanged(ip->nOutputsText, ip->nOutputsOKPBtn,
			     ip->nOutputsCancelPBtn,
			     ip->defaultsDialogShell,
			     nOutputs_buttons_mapped,
			     data_params.defaults_noutputs_string);
    } else if ((!strcmp(client, "nOutputsTextAct")) ||
	       (!strcmp(client, "nOutputsOKPBtn"))) {
	/*fprintf (stderr, "defaults_callback: %s\n", client );*/
	String nOutputsTextval;
	nOutputs_buttons_mapped =
	  okTextActivate(ip->nOutputsText, ip->nOutputsOKPBtn,
			 ip->nOutputsCancelPBtn, &nOutputsTextval);
	/* I won't actually be using this copy */
	XtFree (nOutputsTextval);
	
	/* verify the string as an integer */
	data_params.defaults_noutputs =
	  set_atoi_value(ip->nOutputsText, "data_noutputs_set",
			 data_params.defaults_noutputs);
	data_check_df_args(NOT_SAVING);

	/* save the current text for future comparisons */
	XtFree(data_params.defaults_noutputs_string);
	data_params.defaults_noutputs_string =
	  get_string_text(ip->nOutputsText);
	
    } else if (!strcmp(client, "nOutputsCancelPBtn")) {
	/*fprintf (stderr, "defaults_callback: %s\n", client );*/
	nOutputs_buttons_mapped = 
	  okTextCancel(ip->nOutputsText, ip->nOutputsOKPBtn,
		       ip->nOutputsCancelPBtn,
		       data_params.defaults_noutputs_string);

    } else if (!strcmp(client, "nInputsText")) {
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	/*fprintf (stderr, "defaults_callback: %s\n", client );*/
	nInputs_buttons_mapped =
	  okTextValueChanged(ip->nInputsText, ip->nInputsOKPBtn,
			     ip->nInputsCancelPBtn,
			     ip->defaultsDialogShell,
			     nInputs_buttons_mapped,
			     data_params.defaults_nraw_string);
    } else if ((!strcmp(client, "nInputsTextAct")) ||
	       (!strcmp(client, "nInputsOKPBtn"))) {
	/*fprintf (stderr, "defaults_callback: %s\n", client );*/
	String nInputsTextval;
	nInputs_buttons_mapped =
	  okTextActivate(ip->nInputsText, ip->nInputsOKPBtn,
			 ip->nInputsCancelPBtn, &nInputsTextval);
	/* I won't actually be using this copy */
	XtFree (nInputsTextval);
	
	/* verify the string as an integer */
	data_params.defaults_nraw =
	  set_atoi_value(ip->nInputsText, "data_ninputs_set",
			 data_params.defaults_nraw);
	data_check_df_args(NOT_SAVING);

	/* save the current text for future comparisons */
	XtFree(data_params.defaults_nraw_string);
	data_params.defaults_nraw_string = get_string_text(ip->nInputsText);
	
    } else if (!strcmp(client, "nInputsCancelPBtn")) {
	/*fprintf (stderr, "defaults_callback: %s\n", client );*/
	nInputs_buttons_mapped = 
	  okTextCancel(ip->nInputsText, ip->nInputsOKPBtn,
		       ip->nInputsCancelPBtn,
		       data_params.defaults_nraw_string);
 
    } else if (!strcmp(client, "cancelPBtn")) {
	/*fprintf (stderr, "defaults_callback: %s\n", client );*/
	close_popup(ip->defaultsForm);
 
    } else if (!strcmp(client, "generatePBtn")) {
	/*fprintf (stderr, "defaults_callback: %s\n", client );*/
	DEFAULTS_FILE_STRUCT df_args;

	/* LNKnet can only deal with classification data */
	data_params.defaults_db_type = CLASS_DATA;

	/* check arguments, complain if there is a problem and don't
	   save the file */
	if (data_check_df_args(SAVING)) {
	    df_args.input_labels = data_params.defaults_input_labels;
	    df_args.output_labels = data_params.defaults_class_labels;
	    df_args.db_type = data_params.defaults_db_type;
	    df_args.ninputs = data_params.defaults_nraw;
	    df_args.noutputs = data_params.defaults_noutputs;
	    df_args.input_types = NULL;
	    df_args.input_type_values = NULL;
	    df_args.categories = data_params.defaults_categories;

	    write_defaults_file(path_build(data_params.data_path,
					   data_params.desc_fname), &df_args);
	    /* reselect the data base to read the new defaults file and
	       display the values in it */
	    data_set_data_base();

	    /* close the popup */
	    close_popup(ip->defaultsForm);
	}

    } else {
	fprintf (stderr, "defaults_callback: unknown client: %s\n", (char *)client);
    }
}


static int
defaults_check_categories(char *error_string, int ok){
  int *categoriesCheck;
  int i, s, f, j;
  if(!data_params.defaults_categories ||
     data_params.defaults_categories[0] == 0){
    return ok;
  }
  categoriesCheck = allocate_ivector(data_params.defaults_nraw);
  for(i=1; i < data_params.defaults_categories[0]; i += 2){
    s = data_params.defaults_categories[i];
    f = data_params.defaults_categories[i+1];
    if(s > f){
      strcat(error_string, "start of category pair > finish");
      deallocate_ivector(categoriesCheck);
      return 0;
    }
    if(s < 0 || s >= data_params.defaults_nraw ||
       f < 0 || f >= data_params.defaults_nraw) {
      strcat(error_string, "category entry out of range 0..# Inputs-1");
      deallocate_ivector(categoriesCheck);
      return 0;
    }
    for(j = s; j <= f; j++){
      categoriesCheck[j]++;
    }
  }
  for(i = 0; i < data_params.defaults_nraw; i++){
    if(categoriesCheck[i] == 0){
      strcat(error_string, "missed an input in category assignments");
      deallocate_ivector(categoriesCheck);
      return 0;
    }
    if(categoriesCheck[i] > 1){
      strcat(error_string, "feature found in categories twice");
      deallocate_ivector(categoriesCheck);
      return 0;
    }
  }
  deallocate_ivector(categoriesCheck);
  return ok;
}

/* check that the number of inputs matches the input label list and
   the number of outputs matches the output label list */
static int
data_check_df_args(saving)
     int saving;
{
    int length;
    int ok = 1;
    char error_string[128];/* max is still 100, but watch out */

    strcpy(error_string, "");
    if(data_params.defaults_class_labels != NULL){
	length = atoi(data_params.defaults_class_labels[0]);
	if(length != 0 && length != data_params.defaults_noutputs){
	    strcat(error_string, "# Class Labels != # Classes    ");
	    ok = 0;
	}
    }
    if(data_params.defaults_input_labels != NULL){
	length = atoi(data_params.defaults_input_labels[0]);
	if(length != 0 && length != data_params.defaults_nraw){
	    strcat(error_string, "# Input Labels != # Inputs   ");
	    ok = 0;
	}
    }
    if(data_params.defaults_categories != NULL){
      if(data_params.defaults_categories[0] % 2){
	strcat(error_string, "categories length must be even");
	ok = 0;
      } else {
	ok = defaults_check_categories(error_string, ok);
      }
    }
    if(saving){
	if(ok == 0){
	    strcat(error_string, "\nLNKnet will not create description file");
	    warningMessageDialog(DefaultsClone->defaultsDialogShell,
				 error_string);
	}
    } else {
	show_string_label(DefaultsClone->errorLabel, error_string);
	if(ok == 0)
	  show_image_label(DefaultsClone->errorGlyphLabel,
			   LnknetClone->largeWarning);
	else
	  show_image_label(DefaultsClone->errorGlyphLabel,
			   LnknetClone->largeBlank);
	  
    }
    return ok;
}

void
defaults_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(DefaultsClone->defaultsForm);
    } else if (!strcmp(request, "display")){
	char string[10]; /* for creating the int text */

	/* display these values on the defaults window */
	show_string_label(DefaultsClone->dataPathLabel, data_params.data_path);
	show_string_label(DefaultsClone->dataBaseLabel, data_params.data_base);

	sprintf(string, "%d", data_params.defaults_nraw);
	XtFree(data_params.defaults_nraw_string);
	data_params.defaults_nraw_string = XtNewString(string);
	show_string_text(DefaultsClone->nInputsText,
			 data_params.defaults_nraw_string);

	sprintf(string, "%d", data_params.defaults_noutputs);
	XtFree(data_params.defaults_noutputs_string);
	data_params.defaults_noutputs_string = XtNewString(string);
	show_string_text(DefaultsClone->nOutputsText,
			 data_params.defaults_noutputs_string);

	XtFree(data_params.defaults_input_labels_string);
	data_params.defaults_input_labels_string =
	  make_labels_string(data_params.defaults_input_labels);
	show_string_text(DefaultsClone->inputLabelsText,
			 data_params.defaults_input_labels_string);
	
	XtFree(data_params.defaults_class_labels_string);
	data_params.defaults_class_labels_string =
	  make_labels_string(data_params.defaults_class_labels);
	show_string_text(DefaultsClone->outputLabelsText,
			 data_params.defaults_class_labels_string);
	
	XtFree(data_params.defaults_categories_string);
	data_params.defaults_categories_string =
	  make_int_string(data_params.defaults_categories);
	show_string_text(DefaultsClone->categoriesText,
			 data_params.defaults_categories_string);
	/* also check and display the warning message */
	data_check_df_args(NOT_SAVING);
    } else {
	fprintf(stderr, "data_request: unknown request: %s\n", request);
    }
}
