/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            EM_CLUS                                   ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  18-Oct-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <Xm/Xm.h>
#include <nnlib.h>
#include <em_clus_args.h>
#include "em_clus.h"
/*
 * External Functions
 */
#include "../main/lnknet_subs.h"
#include "../plots/plots_func.h"
#include "../cost/cost_func.h"
#include "em_clus_func.h"

/*
 * Externals
 */
extern Em_clusWidgets *Em_clusClone;
#include "em_clus_params.h"
Em_clusParams em_clus_params; /* this is the real one */
#include "../data/data_params.h"
extern DataParams data_params;

/*
 * Statics
 */
static void em_clus_set_read_variables();
static void em_clus_set_write_variables();
static void em_clus_set_defaults_variables();
static void em_clus_show_ncenters_choices();
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: em_clus_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   em_clus_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  18-Oct-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
em_clus_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    Em_clusWidgets *ip;
    static int ncenters_buttons_mapped = False;
    static int ncentersList_buttons_mapped = False;
    static int reduceStep_buttons_mapped = False;
    static int stopPercentage_buttons_mapped = False;
    static int maxIteration_buttons_mapped = False;
    static int splitPercentage_buttons_mapped = False;
    static int varSpread_buttons_mapped = False;
    static int minVar_buttons_mapped = False;
    static int kmeansStop_buttons_mapped = False;
 
  /*
   * Get the parent structure
   */
  XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "ignoreClassToggle") || 
	!strcmp(client, "clusterByClassToggle")) {
	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	if(state->set){
	    em_clus_params.cluster_by_class =
	      get_radio_value(ip->clusterByClassRadioBox);
	    em_clus_show_ncenters_choices();
	    em_clus_set_total_ncenters();
	    set_plot_labels();
	}

    } else if (!strcmp(client, "equalNcentersToggle") ||
	       !strcmp(client, "ncentersByClassToggle")) {
	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	if(state->set){
	    em_clus_params.specify_by_class =
	      get_radio_value(ip->ncentersByClassRadioBox);
	    em_clus_show_ncenters_choices();
	    em_clus_set_total_ncenters();
	    set_plot_labels();
	}
	
    } else if (!strcmp(client, "ncentersText")) {
	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	ncenters_buttons_mapped =
	  okTextValueChanged(ip->ncentersText, ip->ncentersOKPBtn,
			     ip->ncentersCancelPBtn,
			     ip->em_clusDialogShell,
			     ncenters_buttons_mapped,
			     em_clus_params.ncenters_string);
	
    } else if ((!strcmp(client, "ncentersTextAct")) ||
	       (!strcmp(client, "ncentersOKPBtn"))) {
	String ncentersTextval;

      /*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	ncenters_buttons_mapped =
	  okTextActivate(ip->ncentersText, ip->ncentersOKPBtn,
			 ip->ncentersCancelPBtn, &ncentersTextval);
	/* I won't actually be using this copy */
	XtFree (ncentersTextval);
	
	/* verify the string as an integer */
	em_clus_params.ncenters =
	  set_atoi_value(ip->ncentersText, "ncenters_set",
			     em_clus_params.ncenters);

	/* save the current text for future comparisons */
	XtFree(em_clus_params.ncenters_string);
	em_clus_params.ncenters_string = get_string_text(ip->ncentersText);
 
	/* do any other things needed */
	em_clus_set_total_ncenters();
	set_plot_labels();
  } else if (!strcmp(client, "ncentersCancelPBtn")) {
    /*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	ncenters_buttons_mapped = 
	  okTextCancel(ip->ncentersText, ip->ncentersOKPBtn,
		       ip->ncentersCancelPBtn,
		       em_clus_params.ncenters_string);
	
    } else if (!strcmp(client, "ncentersListText")) {
	/*fprintf (stderr, "em_clus_callback: %s\n", client);*/
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	ncentersList_buttons_mapped =
	  okTextValueChanged(ip->ncentersListText, ip->ncentersListOKPBtn,
			     ip->ncentersListCancelPBtn,
			     ip->em_clusDialogShell,
			     ncentersList_buttons_mapped,
			     em_clus_params.ncentersList_string);
    } else if ((!strcmp(client, "ncentersListTextAct")) ||
	       (!strcmp(client, "ncentersListOKPBtn"))) {
	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	String ncentersListTextval;
	ncentersList_buttons_mapped =
	  okTextActivate(ip->ncentersListText,
			 ip->ncentersListOKPBtn,
			 ip->ncentersListCancelPBtn,
			 &ncentersListTextval);
	
	if(em_clus_params.ncentersList != NULL)
	  deallocate_ivector(em_clus_params.ncentersList);
	em_clus_params.ncentersList = NULL;
	/* take spaces and tabs out of the list before handing it to nnargs */
	if(ncentersListTextval){
	    char *ncentersListTemp = ncentersListTextval;
	    ncentersListTextval = strtok(ncentersListTextval, " \t");
	    if(ncentersListTextval){
		em_clus_params.ncentersList =
		  nnargs_get_int_alloc_list(ncentersListTextval);
		XtFree(ncentersListTemp);
	    }
	}
	
	/* make a copy of the string as it has been interpretted */
	XtFree(em_clus_params.ncentersList_string);
	em_clus_params.ncentersList_string =
	  make_int_string(em_clus_params.ncentersList);
	show_string_text(ip->ncentersListText,
			 em_clus_params.ncentersList_string);
	
	/* do anything else necessary */
	em_clus_set_total_ncenters();
	set_plot_labels();
	
    } else if (!strcmp(client, "ncentersListCancelPBtn")) {
	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	ncentersList_buttons_mapped = 
	  okTextCancel(ip->ncentersListText, ip->ncentersListOKPBtn,
		       ip->ncentersListCancelPBtn,
		       em_clus_params.ncentersList_string);
	
    } else if (!strcmp(client, "reduceStepText")) {
	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	reduceStep_buttons_mapped =
	  okTextValueChanged(ip->reduceStepText, ip->reduceStepOKPBtn,
			     ip->reduceStepCancelPBtn,
			     ip->em_clusDialogShell,
			     reduceStep_buttons_mapped,
			     em_clus_params.reduceStep_string);
	
    } else if ((!strcmp(client, "reduceStepTextAct")) ||
	       (!strcmp(client, "reduceStepOKPBtn"))) {
	String reduceStepTextval;

	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	reduceStep_buttons_mapped =
	  okTextActivate(ip->reduceStepText, ip->reduceStepOKPBtn,
			 ip->reduceStepCancelPBtn, &reduceStepTextval);
	/* I won't actually be using this copy */
	XtFree (reduceStepTextval);
	
	/* verify the string as an integer */
	em_clus_params.reduceStep =
	  set_atoi_value(ip->reduceStepText, "reduceStep_set",
			     em_clus_params.reduceStep);

	/* save the current text for future comparisons */
	XtFree(em_clus_params.reduceStep_string);
	em_clus_params.reduceStep_string = get_string_text(ip->reduceStepText);
 
	/* do any other things needed */
	em_clus_set_total_ncenters();
	set_plot_labels();

    } else if (!strcmp(client, "reduceStepCancelPBtn")) {
	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	reduceStep_buttons_mapped = 
	  okTextCancel(ip->reduceStepText, ip->reduceStepOKPBtn,
		       ip->reduceStepCancelPBtn,
		       em_clus_params.reduceStep_string);

    } else if (!strcmp(client, "stopPercentageText")) {
	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	stopPercentage_buttons_mapped =
	  okTextValueChanged(ip->stopPercentageText, ip->stopPercentageOKPBtn,
			     ip->stopPercentageCancelPBtn,
			     ip->em_clusDialogShell,
			     stopPercentage_buttons_mapped,
			     em_clus_params.stopPercentage_string);
	
    } else if ((!strcmp(client, "stopPercentageTextAct")) ||
	       (!strcmp(client, "stopPercentageOKPBtn"))) {
	String stopPercentageTextval;

	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	stopPercentage_buttons_mapped =
	  okTextActivate(ip->stopPercentageText, ip->stopPercentageOKPBtn,
			 ip->stopPercentageCancelPBtn, &stopPercentageTextval);
	/* I won't actually be using this copy */
	XtFree (stopPercentageTextval);
	
	/* verify the string as a float */
	em_clus_params.stopPercentage =
	  set_atof_low_value(ip->stopPercentageText, "stopPercentage_set",
			     em_clus_params.stopPercentage, 0.);

	/* save the current text for future comparisons */
	XtFree(em_clus_params.stopPercentage_string);
	em_clus_params.stopPercentage_string = get_string_text(ip->stopPercentageText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "stopPercentageCancelPBtn")) {
	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	stopPercentage_buttons_mapped = 
	  okTextCancel(ip->stopPercentageText, ip->stopPercentageOKPBtn,
		       ip->stopPercentageCancelPBtn,
		       em_clus_params.stopPercentage_string);
	
    } else if (!strcmp(client, "maxIterationText")) {
	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	maxIteration_buttons_mapped =
	  okTextValueChanged(ip->maxIterationText, ip->maxIterationOKPBtn,
			     ip->maxIterationCancelPBtn,
			     ip->em_clusDialogShell,
			     maxIteration_buttons_mapped,
			     em_clus_params.maxIteration_string);
	
    } else if ((!strcmp(client, "maxIterationTextAct")) ||
	       (!strcmp(client, "maxIterationOKPBtn"))) {
	String maxIterationTextval;

	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	maxIteration_buttons_mapped =
	  okTextActivate(ip->maxIterationText, ip->maxIterationOKPBtn,
			 ip->maxIterationCancelPBtn, &maxIterationTextval);
	/* I won't actually be using this copy */
	XtFree (maxIterationTextval);
	
	/* verify the string as an integer */
	em_clus_params.maxIteration =
	  set_atoi_value(ip->maxIterationText, "maxIteration_set",
			     em_clus_params.maxIteration);

	/* save the current text for future comparisons */
	XtFree(em_clus_params.maxIteration_string);
	em_clus_params.maxIteration_string = get_string_text(ip->maxIterationText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "maxIterationCancelPBtn")) {
	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	maxIteration_buttons_mapped = 
	  okTextCancel(ip->maxIterationText, ip->maxIterationOKPBtn,
		       ip->maxIterationCancelPBtn,
		       em_clus_params.maxIteration_string);

    } else if (!strcmp(client, "splitPercentageText")) {
	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	splitPercentage_buttons_mapped =
	  okTextValueChanged(ip->splitPercentageText, ip->splitPercentageOKPBtn,
			     ip->splitPercentageCancelPBtn,
			     ip->em_clusDialogShell,
			     splitPercentage_buttons_mapped,
			     em_clus_params.splitPercentage_string);
	
    } else if ((!strcmp(client, "splitPercentageTextAct")) ||
	       (!strcmp(client, "splitPercentageOKPBtn"))) {
	String splitPercentageTextval;

	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	splitPercentage_buttons_mapped =
	  okTextActivate(ip->splitPercentageText, ip->splitPercentageOKPBtn,
			 ip->splitPercentageCancelPBtn, &splitPercentageTextval);
	/* I won't actually be using this copy */
	XtFree (splitPercentageTextval);
	
	/* verify the string as a float */
	em_clus_params.splitPercentage =
	  set_atof_low_value(ip->splitPercentageText, "splitPercentage_set",
			     em_clus_params.splitPercentage, 0.);

	/* save the current text for future comparisons */
	XtFree(em_clus_params.splitPercentage_string);
	em_clus_params.splitPercentage_string = get_string_text(ip->splitPercentageText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "splitPercentageCancelPBtn")) {
	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	splitPercentage_buttons_mapped = 
	  okTextCancel(ip->splitPercentageText, ip->splitPercentageOKPBtn,
		       ip->splitPercentageCancelPBtn,
		       em_clus_params.splitPercentage_string);
	
    } else if (!strcmp(client, "addRandomOffsetToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "em_clus_callback: %s %s\n", client,
	  (state->set) ? "on", "off" );*/
	em_clus_params.addRandomOffset = state->set;
	
    } else if (!strcmp(client, "varSpreadText")) {
	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	varSpread_buttons_mapped =
	  okTextValueChanged(ip->varSpreadText, ip->varSpreadOKPBtn,
			     ip->varSpreadCancelPBtn,
			     ip->em_clusDialogShell,
			     varSpread_buttons_mapped,
			     em_clus_params.varSpread_string);
	
    } else if ((!strcmp(client, "varSpreadTextAct")) ||
	       (!strcmp(client, "varSpreadOKPBtn"))) {
	String varSpreadTextval;

	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	varSpread_buttons_mapped =
	  okTextActivate(ip->varSpreadText, ip->varSpreadOKPBtn,
			 ip->varSpreadCancelPBtn, &varSpreadTextval);
	/* I won't actually be using this copy */
	XtFree (varSpreadTextval);
	
	/* verify the string as a float */
	em_clus_params.varSpread =
	  set_atof_low_value(ip->varSpreadText, "varSpread_set",
			     em_clus_params.varSpread, 0.);

	/* save the current text for future comparisons */
	XtFree(em_clus_params.varSpread_string);
	em_clus_params.varSpread_string = get_string_text(ip->varSpreadText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "varSpreadCancelPBtn")) {
	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	varSpread_buttons_mapped = 
	  okTextCancel(ip->varSpreadText, ip->varSpreadOKPBtn,
		       ip->varSpreadCancelPBtn,
		       em_clus_params.varSpread_string);
 
    } else if (!strcmp(client, "minVarText")) {
	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	minVar_buttons_mapped =
	  okTextValueChanged(ip->minVarText, ip->minVarOKPBtn,
			     ip->minVarCancelPBtn,
			     ip->em_clusDialogShell,
			     minVar_buttons_mapped,
			     em_clus_params.minVar_string);
	
    } else if ((!strcmp(client, "minVarTextAct")) ||
	       (!strcmp(client, "minVarOKPBtn"))) {
	String minVarTextval;

	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	minVar_buttons_mapped =
	  okTextActivate(ip->minVarText, ip->minVarOKPBtn,
			 ip->minVarCancelPBtn, &minVarTextval);
	/* I won't actually be using this copy */
	XtFree (minVarTextval);
	
	/* verify the string as a float */
	em_clus_params.minVar =
	  set_atof_low_value(ip->minVarText, "minVar_set",
			     em_clus_params.minVar, 0.);

	/* save the current text for future comparisons */
	XtFree(em_clus_params.minVar_string);
	em_clus_params.minVar_string = get_string_text(ip->minVarText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "minVarCancelPBtn")) {
	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	minVar_buttons_mapped = 
	  okTextCancel(ip->minVarText, ip->minVarOKPBtn,
		       ip->minVarCancelPBtn,
		       em_clus_params.minVar_string);
 
  } else if (!strcmp(client, "useKmeansToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "em_clus_callback: %s %s\n", client,
	  (state->set) ? "on", "off" );*/
	em_clus_params.useKmeans = state->set;
	gray_out_item(ip->kmeansStopForm, !em_clus_params.useKmeans);
	if(em_clus_params.useKmeans)
	  cost_set_cost_func(CLUS_SQ_ERR);
	else
	  cost_set_cost_func(CLUS_LOG_LIKE);
 
    } else if (!strcmp(client, "kmeansStopText")) {
	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	kmeansStop_buttons_mapped =
	  okTextValueChanged(ip->kmeansStopText, ip->kmeansStopOKPBtn,
			     ip->kmeansStopCancelPBtn,
			     ip->em_clusDialogShell,
			     kmeansStop_buttons_mapped,
			     em_clus_params.kmeansStop_string);
	
    } else if ((!strcmp(client, "kmeansStopTextAct")) ||
	       (!strcmp(client, "kmeansStopOKPBtn"))) {
	String kmeansStopTextval;

	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	kmeansStop_buttons_mapped =
	  okTextActivate(ip->kmeansStopText, ip->kmeansStopOKPBtn,
			 ip->kmeansStopCancelPBtn, &kmeansStopTextval);
	/* I won't actually be using this copy */
	XtFree (kmeansStopTextval);
	
	/* verify the string as a float */
	em_clus_params.kmeansStop =
	  set_atof_low_value(ip->kmeansStopText, "kmeansStop_set",
			     em_clus_params.kmeansStop, 0.);

	/* save the current text for future comparisons */
	XtFree(em_clus_params.kmeansStop_string);
	em_clus_params.kmeansStop_string = get_string_text(ip->kmeansStopText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "kmeansStopCancelPBtn")) {
	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	kmeansStop_buttons_mapped = 
	  okTextCancel(ip->kmeansStopText, ip->kmeansStopOKPBtn,
		       ip->kmeansStopCancelPBtn,
		       em_clus_params.kmeansStop_string);
 
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "em_clus_callback: %s\n", client );*/
	close_popup(Em_clusClone->em_clusForm);

    } else {
	fprintf(stderr, "em_clus_callback: unknown client: %s\n", (char *)client);
    }
}

void
em_clus_request(String request)
{
    if (!strcmp(request, "open")) {
	open_popup(Em_clusClone->em_clusForm);
    } else if (!strcmp(request, "init")){
	/* this should only get called once, before ANYTHING 
	   gets allocated or there will be memory leaks */
	em_clus_params.cluster_by_class = False;
	em_clus_params.specify_by_class = False;
	em_clus_params.ncenters = 0;
	em_clus_params.ncenters_string = NULL;
	em_clus_params.ncentersList = NULL;
	em_clus_params.ncentersList_string = NULL;
	em_clus_params.total_ncenters = 0;
	em_clus_params.reduceStep = 0;
	em_clus_params.reduceStep_string = 0;
	em_clus_params.stopPercentage = 0.;
	em_clus_params.stopPercentage_string = NULL;
	em_clus_params.maxIteration = 0;
	em_clus_params.maxIteration_string = NULL;
	em_clus_params.splitPercentage = 0.;
	em_clus_params.splitPercentage_string = NULL;
	em_clus_params.addRandomOffset = False;
	em_clus_params.varSpread = 0.;
	em_clus_params.minVar = 0.;
	em_clus_params.useKmeans = False;
	em_clus_params.kmeansStop = 0.;
    } else if (!strcmp(request, "display")){
	char string[10]; /* for creating int and float text */
	show_radio_value(Em_clusClone->clusterByClassRadioBox,
			 em_clus_params.cluster_by_class);
	show_radio_value(Em_clusClone->ncentersByClassRadioBox,
			 em_clus_params.specify_by_class);

	sprintf(string, "%d", em_clus_params.ncenters);
	XtFree(em_clus_params.ncenters_string);
	em_clus_params.ncenters_string = XtNewString(string);
	show_string_text(Em_clusClone->ncentersText,
			 em_clus_params.ncenters_string);

	XtFree(em_clus_params.ncentersList_string);
	em_clus_params.ncentersList_string =
	  make_int_string(em_clus_params.ncentersList);
	show_string_text(Em_clusClone->ncentersListText,
			 em_clus_params.ncentersList_string);

	em_clus_show_ncenters_choices();
	em_clus_set_total_ncenters();

	sprintf(string, "%d", em_clus_params.reduceStep);
	XtFree(em_clus_params.reduceStep_string);
	em_clus_params.reduceStep_string = XtNewString(string);
	show_string_text(Em_clusClone->reduceStepText,
			 em_clus_params.reduceStep_string);

	sprintf(string, "%5.2g", em_clus_params.stopPercentage);
	XtFree(em_clus_params.stopPercentage_string);
	em_clus_params.stopPercentage_string = XtNewString(string);
	show_string_text(Em_clusClone->stopPercentageText,
			 em_clus_params.stopPercentage_string);

	sprintf(string, "%d", em_clus_params.maxIteration);
	XtFree(em_clus_params.maxIteration_string);
	em_clus_params.maxIteration_string = XtNewString(string);
	show_string_text(Em_clusClone->maxIterationText,
			 em_clus_params.maxIteration_string);

	sprintf(string, "%5.2g", em_clus_params.splitPercentage);
	XtFree(em_clus_params.splitPercentage_string);
	em_clus_params.splitPercentage_string = XtNewString(string);
	show_string_text(Em_clusClone->splitPercentageText,
			 em_clus_params.splitPercentage_string);

	show_toggle_value(Em_clusClone->addRandomOffsetToggle,
			  em_clus_params.addRandomOffset);

	sprintf(string, "%5.2g", em_clus_params.varSpread);
	XtFree(em_clus_params.varSpread_string);
	em_clus_params.varSpread_string = XtNewString(string);
	show_string_text(Em_clusClone->varSpreadText,
			 em_clus_params.varSpread_string);

	sprintf(string, "%5.2g", em_clus_params.minVar);
	XtFree(em_clus_params.minVar_string);
	em_clus_params.minVar_string = XtNewString(string);
	show_string_text(Em_clusClone->minVarText,
			 em_clus_params.minVar_string);

	show_toggle_value(Em_clusClone->useKmeansToggle,
			  em_clus_params.useKmeans);
	gray_out_item(Em_clusClone->kmeansStopForm, !em_clus_params.useKmeans);

	sprintf(string, "%5.2g", em_clus_params.kmeansStop);
	XtFree(em_clus_params.kmeansStop_string);
	em_clus_params.kmeansStop_string = XtNewString(string);
	show_string_text(Em_clusClone->kmeansStopText,
			 em_clus_params.kmeansStop_string);

    } else {
	fprintf (stderr, "em_clus_request: unknown request: %s\n", request);
    }
}

String
em_clus_get_plot_label(String string)
{
    char intchars[10];
    sprintf(intchars, "%d", em_clus_params.total_ncenters);
    string = StringConcat(string, " Centers:");
    string = StringConcat(string, intchars);
    return string;
}

static ARG em_clus_tab2[] =
{
    {"specify_by_class", BOOLEAN, (int *)NULL,
       "use ncenters list for nclusters by class"},
};
#define EM_CLUS_TABSIZE2 (sizeof(em_clus_tab2)/sizeof(ARG))

static void
em_clus_set_read_variables()
{
    em_clus_params.cluster_by_class = cluster_by_class;
    em_clus_params.ncenters = ncenters;
    if(em_clus_params.ncentersList != NULL){
	deallocate_ivector(em_clus_params.ncentersList);
    }
    em_clus_params.ncentersList = ncenters_list;
    ncenters_list = NULL;
    em_clus_params.splitPercentage = split_percentage;
    em_clus_params.addRandomOffset = add_random_offset;
    em_clus_params.maxIteration = max_iteration;
    em_clus_params.stopPercentage = stop_percentage;
    em_clus_params.reduceStep = reduce_step;
    em_clus_params.useKmeans = use_kmeans;
    em_clus_params.kmeansStop = kmeans_stop_percentage;
    em_clus_params.varSpread = variance_scale;
    em_clus_params.minVar = minimum_variance;
}

static void
em_clus_set_write_variables()
{
    cluster_by_class = em_clus_params.cluster_by_class;
    ncenters = em_clus_params.ncenters;
    ncenters_list = em_clus_params.ncentersList;
    split_percentage = em_clus_params.splitPercentage;
    add_random_offset = em_clus_params.addRandomOffset;
    max_iteration = em_clus_params.maxIteration;
    stop_percentage = em_clus_params.stopPercentage;
    reduce_step = em_clus_params.reduceStep;
    use_kmeans = em_clus_params.useKmeans;
    kmeans_stop_percentage = em_clus_params.kmeansStop;
    variance_scale = em_clus_params.varSpread;
    minimum_variance = em_clus_params.minVar;
}

static void
em_clus_set_defaults_variables()
{
    nnargs_set_variable("specify_by_class", &em_clus_params.specify_by_class,
			em_clus_tab2, EM_CLUS_TABSIZE2);
}
void
read_em_clus_defaults(int argc, char **argv)
{
    cluster_by_class = 0;
    add_random_offset = 0;
    ncenters_list = NULL;/* the space for this list is always
			    owned by the em_clus_params structure */
    use_kmeans = 0;
    em_clus_params.specify_by_class = 0;

    em_clus_set_defaults_variables();
    if(argc > 0){
	argc = nnargs_part(argc, argv, em_clus_tab2, EM_CLUS_TABSIZE2);
	argc = nnargs(argc, argv, em_clus_tab, EM_CLUS_TABSIZE);
	em_clus_set_read_variables();
    } else {
	em_clus_params.addRandomOffset = True;
	em_clus_params.reduceStep = 1;
	em_clus_params.maxIteration = 10;
	em_clus_params.ncenters = 16;
	em_clus_params.splitPercentage = 100.;
	em_clus_params.stopPercentage = 1.;
	em_clus_params.specify_by_class = 0;
	em_clus_params.kmeansStop = 1.;
	em_clus_params.varSpread = 4.;
	em_clus_params.minVar = 1e-5;
    }
    em_clus_request("display");
}

void
write_em_clus_defaults(default_file)
     FILE *default_file;
{
    em_clus_set_write_variables();
    em_clus_set_defaults_variables();
    if(default_file != NULL){
	fprintf(default_file, "em_clus %s",
		nnargs_get_part_flags(em_clus_tab, EM_CLUS_TABSIZE));
	fprintf(default_file, "%s",
		nnargs_get_flags(em_clus_tab2, EM_CLUS_TABSIZE2));
    }
}

String
em_clus_get_args ()
{
    em_clus_set_write_variables();
    /* fix up ncenters list, in case it isn't wanted */
    if(em_clus_params.cluster_by_class && em_clus_params.specify_by_class){
	ncenters = 0;
    } else {
	ncenters_list = NULL;
    }
    return (nnargs_get_part_flags(em_clus_tab, EM_CLUS_TABSIZE));
}

String
em_clus_get_important_args ()
{
    em_clus_set_write_variables();
    /* fix up ncenters list, in case it isn't wanted */
    if(em_clus_params.specify_by_class){
	ncenters = 0;
    } else {
	ncenters_list = NULL;
    }
    return (nnargs_get_flags_one_line(em_clus_tab, EM_CLUS_TABSIZE));
}

void
em_clus_set_total_ncenters()
{
    static char total_label[64];

    if(em_clus_params.cluster_by_class){
	if(em_clus_params.specify_by_class &&
	   em_clus_params.ncentersList != NULL){
	    int i;
	    for(i = 1, em_clus_params.total_ncenters = 0;
		i <= em_clus_params.ncentersList[0]; i++){
		em_clus_params.total_ncenters += em_clus_params.ncentersList[i];
	    }
	} else {
	    em_clus_params.total_ncenters =
	      data_params.noutputs * em_clus_params.ncenters;
	}
    } else {
	em_clus_params.total_ncenters = em_clus_params.ncenters;
    }
    sprintf(total_label, "This will create a total of %d centers.",
	    em_clus_params.total_ncenters);
    show_string_label(Em_clusClone->totalNcentersLabel, total_label);
}

static void
em_clus_show_ncenters_choices()
{
    if(em_clus_params.cluster_by_class){
	gray_out_item(Em_clusClone->ncentersByClassRadioBox, FALSE);
	if(em_clus_params.specify_by_class){
	    gray_out_item(Em_clusClone->ncentersListForm, FALSE);
	    gray_out_item(Em_clusClone->ncentersForm, TRUE);
	} else {
	    gray_out_item(Em_clusClone->ncentersListForm, TRUE);
	    gray_out_item(Em_clusClone->ncentersForm, FALSE);
	}
    } else {
	gray_out_item(Em_clusClone->ncentersByClassRadioBox, TRUE);
	gray_out_item(Em_clusClone->ncentersListForm, TRUE);
	gray_out_item(Em_clusClone->ncentersForm, FALSE);
    }
}

int
em_clus_get_cost_func()
{
    if(em_clus_params.useKmeans) return CLUS_SQ_ERR;
    else return CLUS_LOG_LIKE;
}
