/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/gauss/gauss_callback.c,v 11.4 2001/08/23 20:16:27 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                       Lnknet Window Interface                        ****
 ****                          Gauss Window                                ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  14-Sep-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include "../main/lnknet_subs.h"
#include "gauss.h"
#include <nnlib.h>
#include <gauss_args.h>
/*
 * External Functions
 */
#include "../plots/plots_func.h"
 
/*
 * Externals
 */
extern GaussWidgets *GaussClone;
#include "gauss_params.h"
GaussParams gauss_params; /* this is the real one */
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: gauss_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   gauss_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  14-Sep-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
gauss_callback(Widget w, XtPointer client, XtPointer call)
{
    /* SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    GaussWidgets *ip;
    static int minVar_buttons_mapped = False;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "minVarText")) {
	/*fprintf (stderr, "gauss_callback: %s\n", client );*/
	minVar_buttons_mapped =
	  okTextValueChanged(ip->minVarText, ip->minVarOKPBtn,
			     ip->minVarCancelPBtn,
			     ip->gaussDialogShell,
			     minVar_buttons_mapped,
			     gauss_params.minVarString);
 
  } else if ((!strcmp(client, "minVarTextAct")) ||
	     (!strcmp(client, "minVarOKPBtn"))) {
      String minVarTextval;

      /*fprintf (stderr, "gauss_callback: %s\n", client );*/
	minVar_buttons_mapped =
	  okTextActivate(ip->minVarText, ip->minVarOKPBtn,
			 ip->minVarCancelPBtn, &minVarTextval);
	/* I won't actually be using this copy */
	XtFree (minVarTextval);
	
	/* verify the string as an integer */
	gauss_params.minVar =
	  set_atof_low_value(ip->minVarText, "minVar_set",
			     gauss_params.minVar, 0.);

	/* save the current text for future comparisons */
	XtFree(gauss_params.minVarString);
	gauss_params.minVarString = get_string_text(ip->minVarText);
 
  } else if (!strcmp(client, "minVarCancelPBtn")) {
    /*fprintf (stderr, "gauss_callback: %s\n", client );*/
	minVar_buttons_mapped = 
	  okTextCancel(ip->minVarText, ip->minVarOKPBtn,
		       ip->minVarCancelPBtn,
		       gauss_params.minVarString);
 
  } else if (!strcmp(client, "fullDiagonalToggle") ||
	     !strcmp(client, "fullFullToggle")){
      XmToggleButtonCallbackStruct *state =
	(XmToggleButtonCallbackStruct *)acs;
      /*fprintf (stderr, "gauss_callback: %s %s\n", client,
	(state->set)? "on": "off" );*/
      if(state->set){
	  gauss_params.full = get_radio_value(ip->fullRadioBox);
	  set_plot_labels();
      }
 
  } else if (!strcmp(client, "grandPerClassToggle") ||
	     !strcmp(client, "grandGrandToggle")) {
      XmToggleButtonCallbackStruct *state =
	(XmToggleButtonCallbackStruct *)acs;
      /*fprintf (stderr, "gauss_callback: %s %s\n", client,
	(state->set)? "on": "off" );*/
      if(state->set){
	  int temp;
	  temp = get_radio_value(ip->grandRadioBox);
	  /* reverse the sense of this */
	  gauss_params.per_class = (temp) ? 0 : 1;
	  set_plot_labels();
      }

  } else if (!strcmp(client, "closePBtn")) {
      /*fprintf (stderr, "gauss_callback: %s\n", client);*/
      close_popup(GaussClone->gaussForm);

  } else {
      fprintf (stderr, "gauss_callback: unknown client: %s\n", (char *)client);
  }
}

void
gauss_request(String request)
{
    if (!strcmp(request, "open")) {
	open_popup(GaussClone->gaussForm);
    } else if (!strcmp(request, "init")){
	/* this should only get called once, before ANYTHING 
	   gets allocated or there will be memory leaks */
	gauss_params.full = 0;
	gauss_params.per_class = 0;
	gauss_params.minVar = 0.;
	gauss_params.minVarString = NULL;
    } else if (!strcmp(request, "display")){
	char string[10]; /* for creating int and float text */

	show_radio_value(GaussClone->fullRadioBox, gauss_params.full);
	show_radio_value(GaussClone->grandRadioBox, !gauss_params.per_class);
	sprintf(string, "%5.2g", gauss_params.minVar);
	XtFree(gauss_params.minVarString);
	gauss_params.minVarString = XtNewString(string);
	show_string_text(GaussClone->minVarText, gauss_params.minVarString);
    } else {
	fprintf (stderr, "gauss_request: unknown request: %s\n", request);
    }
}

String
gauss_get_plot_label(String string)
{
    if(gauss_params.full) string = StringConcat(string, " Full");
    else string = StringConcat(string, " Diagonal");
    if(gauss_params.per_class) string = StringConcat(string, " Per Class");
    else string = StringConcat(string, " Grand");
    return string;
}

static void
gauss_set_read_variables()
{
    gauss_params.full = covariance_type;
    gauss_params.per_class = variance_type;
    gauss_params.minVar = minimum_variance;
    gauss_params.maxRatio = max_ratio;
}

static void
gauss_set_write_variables()
{
    covariance_type = gauss_params.full;
    variance_type = gauss_params.per_class;
    minimum_variance = gauss_params.minVar;
    max_ratio = gauss_params.maxRatio;
}

void
read_gauss_defaults(int argc, char **argv)
{
    variance_type = 0;
    covariance_type = 0;
    if(argc > 0){
	argc = nnargs(argc, argv, gauss_tab, GAUSS_TABSIZE);
	gauss_set_read_variables();
    } else {
	gauss_params.full = 0;
	gauss_params.per_class = 0;
	gauss_params.minVar = 0.00001;
	gauss_params.maxRatio = 1e6;
    }
    gauss_request("display");
}

void
write_gauss_defaults(default_file)
     FILE *default_file;
{
    gauss_set_write_variables();
    if(default_file != NULL){
	fprintf(default_file, "gauss %s",
		nnargs_get_flags(gauss_tab, GAUSS_TABSIZE));
    }
}

String
gauss_get_args ()
{
    gauss_set_write_variables();
    return (nnargs_get_part_flags(gauss_tab, GAUSS_TABSIZE));
}

String
gauss_get_important_args ()
{
    gauss_set_write_variables();
    return (nnargs_get_flags_one_line(gauss_tab, GAUSS_TABSIZE));
}
