/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/gmix/gmix_callback.c,v 11.4 2001/08/23 20:16:31 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            GMIX                              ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  8-Nov-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include "../main/lnknet_subs.h"
#include "gmix.h"
#include <nnlib.h>
#include <gmix_args.h>
/*
 * External Functions
 */
#include "gmix_func.h"
#include "../plots/plots_func.h"
#include "../kmeans/kmeans_func.h"
#include "../em_clus/em_clus_func.h"
#include "../lead_clus/lead_clus_func.h"
#include "../ran_clus/ran_clus_func.h"

/*
 * Externals
 */
extern GmixWidgets *GmixClone;
#include "gmix_params.h"
GmixParams gmix_params; /* this is the real one */
#include "../lnk/lnk_params.h"
LnkParams lnk_params;

/*
 * Statics
 */
static void gmix_set_clustering_alg(int);

/******************************************************************************
 ******************************************************************************
 *
 * Module Name: gmix_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   gmix_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  8-Nov-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
gmix_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    GmixWidgets *ip;
    static int varSpread_buttons_mapped = False;
    static int epochs_buttons_mapped = False;
    static int stopPercent_buttons_mapped = False;
    static int maxRatio_buttons_mapped = False;
    static int minVar_buttons_mapped = False;
    
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
    
    if (!strcmp(client, "createClustersToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "gmix_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	gmix_params.createClusters = state->set;
 
    } else if (!strcmp(client, "kmeansPBtn")) {
	/*fprintf (stderr, "gmix_callback: %s\n", client );*/
	gmix_set_clustering_alg(KMEANS);

    } else if (!strcmp(client, "em_clusPBtn")) {
	/*fprintf (stderr, "gmix_callback: %s\n", client );*/
	gmix_set_clustering_alg(EM_CLUS);
 
    } else if (!strcmp(client, "lead_clusPBtn")) {
	/*fprintf (stderr, "gmix_callback: %s\n", client );*/
	gmix_set_clustering_alg(LEAD_CLUS);
 
    } else if (!strcmp(client, "ran_clusPBtn")) {
	/*fprintf (stderr, "gmix_callback: %s\n", client );*/
	gmix_set_clustering_alg(RAN_CLUS);
 
    } else if (!strcmp(client, "clustParamsPBtn")) {
	/*fprintf (stderr, "gmix_callback: %s\n", client );*/
	switch(gmix_params.clusterNum){
	  case KMEANS:
	    kmeans_request("open");
	    break;
	  case EM_CLUS:
	    em_clus_request("open");
	    break;
	  case LEAD_CLUS:
	    lead_clus_request("open");
	    break;
	  case RAN_CLUS:
	    ran_clus_request("open");
	    break;
	}
 
    } else if (!strcmp(client, "perClassToggle") ||
	       !strcmp(client, "grandClassToggle") ||
	       !strcmp(client, "grandOverallToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "gmix_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set) gmix_params.grandType =
	  get_radio_value(ip->grandTypeRadioBox);
	set_plot_labels();
	
    } else if (!strcmp(client, "diagonalToggle") ||
	       !strcmp(client, "fullToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "gmix_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set) gmix_params.covType =
	  get_radio_value(ip->covTypeRadioBox);
	set_plot_labels();
	
    } else if (!strcmp(client, "unTiedToggle") ||
	       !strcmp(client, "tiedToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "gmix_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set) gmix_params.mixType =
	  get_radio_value(ip->mixTypeRadioBox);
	set_plot_labels();
	
    } else if (!strcmp(client, "varSpreadText")) {
	/*fprintf (stderr, "gmix_callback: %s\n", client );*/
	varSpread_buttons_mapped =
	  okTextValueChanged(ip->varSpreadText, ip->varSpreadOKPBtn,
			     ip->varSpreadCancelPBtn,
			     ip->gmixDialogShell,
			     varSpread_buttons_mapped,
			     gmix_params.varSpread_string);
	
    } else if ((!strcmp(client, "varSpreadTextAct")) ||
	       (!strcmp(client, "varSpreadOKPBtn"))) {
	String varSpreadTextval;

	/*fprintf (stderr, "gmix_callback: %s\n", client );*/
	varSpread_buttons_mapped =
	  okTextActivate(ip->varSpreadText, ip->varSpreadOKPBtn,
			 ip->varSpreadCancelPBtn, &varSpreadTextval);
	/* I won't actually be using this copy */
	XtFree (varSpreadTextval);
	
	/* verify the string as a float */
	gmix_params.varSpread =
	  set_atof_low_value(ip->varSpreadText, "varSpread_set",
			     gmix_params.varSpread, 0.);

	/* save the current text for future comparisons */
	XtFree(gmix_params.varSpread_string);
	gmix_params.varSpread_string = get_string_text(ip->varSpreadText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "varSpreadCancelPBtn")) {
	/*fprintf (stderr, "gmix_callback: %s\n", client );*/
	varSpread_buttons_mapped = 
	  okTextCancel(ip->varSpreadText, ip->varSpreadOKPBtn,
		       ip->varSpreadCancelPBtn,
		       gmix_params.varSpread_string);
	
    } else if (!strcmp(client, "epochsText")) {
	/*fprintf (stderr, "gmix_callback: %s\n", client );*/
	epochs_buttons_mapped =
	  okTextValueChanged(ip->epochsText, ip->epochsOKPBtn,
			     ip->epochsCancelPBtn,
			     ip->gmixDialogShell,
			     epochs_buttons_mapped,
			     gmix_params.epochs_string);
	
    } else if ((!strcmp(client, "epochsTextAct")) ||
	       (!strcmp(client, "epochsOKPBtn"))) {
	String epochsTextval;

	/*fprintf (stderr, "gmix_callback: %s\n", client );*/
	epochs_buttons_mapped =
	  okTextActivate(ip->epochsText, ip->epochsOKPBtn,
			 ip->epochsCancelPBtn, &epochsTextval);
	/* I won't actually be using this copy */
	XtFree (epochsTextval);
	
	/* verify the string as an integer */
	gmix_params.epochs =
	  set_atoi_low_value(ip->epochsText, "epochs_set",
			     gmix_params.epochs, 0);

	/* save the current text for future comparisons */
	XtFree(gmix_params.epochs_string);
	gmix_params.epochs_string = get_string_text(ip->epochsText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "epochsCancelPBtn")) {
	/*fprintf (stderr, "gmix_callback: %s\n", client );*/
	epochs_buttons_mapped = 
	  okTextCancel(ip->epochsText, ip->epochsOKPBtn,
		       ip->epochsCancelPBtn,
		       gmix_params.epochs_string);
	
    } else if (!strcmp(client, "stopPercentText")) {
	/*fprintf (stderr, "gmix_callback: %s\n", client );*/
	stopPercent_buttons_mapped =
	  okTextValueChanged(ip->stopPercentText, ip->stopPercentOKPBtn,
			     ip->stopPercentCancelPBtn,
			     ip->gmixDialogShell,
			     stopPercent_buttons_mapped,
			     gmix_params.stopPercent_string);
	
    } else if ((!strcmp(client, "stopPercentTextAct")) ||
	       (!strcmp(client, "stopPercentOKPBtn"))) {
	String stopPercentTextval;

	/*fprintf (stderr, "gmix_callback: %s\n", client );*/
	stopPercent_buttons_mapped =
	  okTextActivate(ip->stopPercentText, ip->stopPercentOKPBtn,
			 ip->stopPercentCancelPBtn, &stopPercentTextval);
	/* I won't actually be using this copy */
	XtFree (stopPercentTextval);
	
	/* verify the string as a float */
	gmix_params.stopPercent =
	  set_atof_low_value(ip->stopPercentText, "stopPercent_set",
			     gmix_params.stopPercent, 0.);

	/* save the current text for future comparisons */
	XtFree(gmix_params.stopPercent_string);
	gmix_params.stopPercent_string = get_string_text(ip->stopPercentText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "stopPercentCancelPBtn")) {
	/*fprintf (stderr, "gmix_callback: %s\n", client );*/
	stopPercent_buttons_mapped = 
	  okTextCancel(ip->stopPercentText, ip->stopPercentOKPBtn,
		       ip->stopPercentCancelPBtn,
		       gmix_params.stopPercent_string);
	
    } else if (!strcmp(client, "maxRatioText")) {
	/*fprintf (stderr, "gmix_callback: %s\n", client );*/
	maxRatio_buttons_mapped =
	  okTextValueChanged(ip->maxRatioText, ip->maxRatioOKPBtn,
			     ip->maxRatioCancelPBtn,
			     ip->gmixDialogShell,
			     maxRatio_buttons_mapped,
			     gmix_params.maxRatio_string);
	
    } else if ((!strcmp(client, "maxRatioTextAct")) ||
	       (!strcmp(client, "maxRatioOKPBtn"))) {
	String maxRatioTextval;

	/*fprintf (stderr, "gmix_callback: %s\n", client );*/
	maxRatio_buttons_mapped =
	  okTextActivate(ip->maxRatioText, ip->maxRatioOKPBtn,
			 ip->maxRatioCancelPBtn, &maxRatioTextval);
	/* I won't actually be using this copy */
	XtFree (maxRatioTextval);
	
	/* verify the string as a float */
	gmix_params.maxRatio =
	  set_atof_low_value(ip->maxRatioText, "maxRatio_set",
			     gmix_params.maxRatio, 1.);

	/* save the current text for future comparisons */
	XtFree(gmix_params.maxRatio_string);
	gmix_params.maxRatio_string = get_string_text(ip->maxRatioText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "maxRatioCancelPBtn")) {
	/*fprintf (stderr, "gmix_callback: %s\n", client );*/
	maxRatio_buttons_mapped = 
	  okTextCancel(ip->maxRatioText, ip->maxRatioOKPBtn,
		       ip->maxRatioCancelPBtn,
		       gmix_params.maxRatio_string);
	
    } else if (!strcmp(client, "minVarText")) {
	/*fprintf (stderr, "gmix_callback: %s\n", client );*/
	minVar_buttons_mapped =
	  okTextValueChanged(ip->minVarText, ip->minVarOKPBtn,
			     ip->minVarCancelPBtn,
			     ip->gmixDialogShell,
			     minVar_buttons_mapped,
			     gmix_params.minVar_string);
	
    } else if ((!strcmp(client, "minVarTextAct")) ||
	       (!strcmp(client, "minVarOKPBtn"))) {
	String minVarTextval;

	/*fprintf (stderr, "gmix_callback: %s\n", client );*/
	minVar_buttons_mapped =
	  okTextActivate(ip->minVarText, ip->minVarOKPBtn,
			 ip->minVarCancelPBtn, &minVarTextval);
	/* I won't actually be using this copy */
	XtFree (minVarTextval);
	
	/* verify the string as a float */
	gmix_params.minVar =
	  set_atof_low_value(ip->minVarText, "minVar_set",
			     gmix_params.minVar, 0.);

	/* save the current text for future comparisons */
	XtFree(gmix_params.minVar_string);
	gmix_params.minVar_string = get_string_text(ip->minVarText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "minVarCancelPBtn")) {
	/*fprintf (stderr, "gmix_callback: %s\n", client );*/
	minVar_buttons_mapped = 
	  okTextCancel(ip->minVarText, ip->minVarOKPBtn,
		       ip->minVarCancelPBtn,
		       gmix_params.minVar_string);
	
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "gmix_callback: %s\n", client );*/
	close_popup(GmixClone->gmixForm);
	
    } else {
	fprintf(stderr, "gmix_callback: unknown client: %s\n", (char *)client);
    }
}

void
gmix_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(GmixClone->gmixForm);
    } else if (!strcmp(request, "init")){
	gmix_params.createClusters = 0;
	gmix_params.clusterNum = KMEANS;
	gmix_params.clustName = "kmeans";
	gmix_params.clustFname = NULL;
	gmix_params.grandType = 0;
	gmix_params.covType = 0;
	gmix_params.mixType = 0;
	gmix_params.varSpread = 0.;
	gmix_params.varSpread_string = NULL;
	gmix_params.epochs = 0;
	gmix_params.epochs_string = NULL;
	gmix_params.stopPercent = 0.;
	gmix_params.stopPercent_string = NULL;
	gmix_params.maxRatio = 0.;
	gmix_params.maxRatio_string = NULL;
	gmix_params.minVar = 0.;
	gmix_params.minVar_string = NULL;
    } else if (!strcmp(request, "display")){
	char string[10];
	show_toggle_value(GmixClone->createClustersToggle,
			  gmix_params.createClusters);
	switch(gmix_params.clusterNum){
	  case KMEANS:
	    show_option_value(GmixClone->clustAlgOptionMenu, 0);
	    break;
	  case EM_CLUS:
	    show_option_value(GmixClone->clustAlgOptionMenu, 1);
	    break;
	  case LEAD_CLUS:
	    show_option_value(GmixClone->clustAlgOptionMenu, 2);
	    break;
	  case RAN_CLUS:
	    show_option_value(GmixClone->clustAlgOptionMenu, 3);
	    break;
	}
	show_string_label(GmixClone->clustFnameLabel, gmix_params.clustFname);
	show_radio_value(GmixClone->grandTypeRadioBox, gmix_params.grandType);
	show_radio_value(GmixClone->covTypeRadioBox, gmix_params.covType);
	show_radio_value(GmixClone->mixTypeRadioBox, gmix_params.mixType);
	XtFree(gmix_params.varSpread_string);
	sprintf(string, "%5.2g", gmix_params.varSpread);
	gmix_params.varSpread_string = XtNewString(string);
	show_string_text(GmixClone->varSpreadText,
			 gmix_params.varSpread_string);
	XtFree(gmix_params.epochs_string);
	sprintf(string, "%d", gmix_params.epochs);
	gmix_params.epochs_string = XtNewString(string);
	show_string_text(GmixClone->epochsText,
			 gmix_params.epochs_string);
	XtFree(gmix_params.stopPercent_string);
	sprintf(string, "%5.2g", gmix_params.stopPercent);
	gmix_params.stopPercent_string = XtNewString(string);
	show_string_text(GmixClone->stopPercentText,
			 gmix_params.stopPercent_string);
	XtFree(gmix_params.maxRatio_string);
	sprintf(string, "%5.2g", gmix_params.maxRatio);
	gmix_params.maxRatio_string = XtNewString(string);
	show_string_text(GmixClone->maxRatioText,
			 gmix_params.maxRatio_string);
	XtFree(gmix_params.minVar_string);
	sprintf(string, "%5.2g", gmix_params.minVar);
	gmix_params.minVar_string = XtNewString(string);
	show_string_text(GmixClone->minVarText,
			 gmix_params.minVar_string);

    } else {
	fprintf(stderr, "gmix_request: unknown request: %s\n", request);
    }
}

String
gmix_get_plot_label(String string)
{
    if(gmix_params.mixType){
	string = StringConcat(string, " Tied ");
	if(gmix_params.grandType)
	  string = StringConcat(string, "Grand ");
	else
	  string = StringConcat(string, "per Component ");
    } else {
	string = StringConcat(string, " Untied ");
	if(gmix_params.grandType > 1)
	  string = StringConcat(string, "Grand ");
	else if (gmix_params.grandType == 1)
	  string = StringConcat(string, "Grand per Class ");
	else
	  string = StringConcat(string, "per Component ");
    }
    if(gmix_params.covType)
      string = StringConcat(string, "Full");
    else
      string = StringConcat(string, "Diagonal");
    
    switch(gmix_params.clusterNum){
      case KMEANS:
	string = kmeans_get_plot_label(string);
	break;
      case EM_CLUS:
	string = em_clus_get_plot_label(string);
	break;
      case LEAD_CLUS:
	string = lead_clus_get_plot_label(string);
	break;
      case RAN_CLUS:
	string = ran_clus_get_plot_label(string);
	break;
    }
    return string;
}

static void
gmix_set_read_variables()
{
    XtFree(gmix_params.clustFname);
    gmix_params.clustFname = XtNewString(cluster_param_fname);
    gmix_params.grandType = grand_flag;
    gmix_params.covType = full_flag;
    gmix_params.mixType = tied_flag;
    gmix_params.varSpread = var_spread;
    gmix_params.epochs = nepochs;
    gmix_params.stopPercent = gmix_stop_percentage;
    gmix_params.maxRatio = max_ratio;
    gmix_params.minVar = minimum_variance;
}

static void
gmix_set_write_variables()
{
    cluster_param_fname = gmix_params.clustFname;
    grand_flag = gmix_params.grandType;
    full_flag = gmix_params.covType;
    tied_flag = gmix_params.mixType;
    var_spread = gmix_params.varSpread;
    nepochs = gmix_params.epochs;
    gmix_stop_percentage = gmix_params.stopPercent;
    max_ratio = gmix_params.maxRatio;
    minimum_variance = gmix_params.minVar;
}

static ARG gmixtab[] =
{
    {"createClusters", BOOLEAN, (int *)NULL,},
    {"clusterNum", INT, (int *)NULL,},
};
#define TABSIZE (sizeof(gmixtab)/sizeof(ARG))

static void
gmix_set_defaults_variables(dp)
     GmixParams *dp;
{
    nnargs_set_variable("createClusters", (int *)&dp->createClusters, gmixtab, TABSIZE);
    nnargs_set_variable("clusterNum", (int *)&dp->clusterNum, gmixtab, TABSIZE);
}

void
read_gmix_defaults(argc, argv)
     int argc;
     char **argv;
{
    gmix_params.createClusters = 0;
    grand_flag = 0;
    full_flag = 0;
    tied_flag = 0;
    gmix_set_defaults_variables(&gmix_params);
    if(argc > 0){
	argc = nnargs_part(argc, argv, gmixtab, TABSIZE);
	argc = nnargs(argc, argv, gmix_tab, GMIX_TABSIZE);
	gmix_set_read_variables();
	switch(gmix_params.clusterNum){
	  case KMEANS:
	    gmix_params.clustName = "kmeans";
	    break;
	  case EM_CLUS:
	    gmix_params.clustName = "em_clus";
	    break;
	  case LEAD_CLUS:
	    gmix_params.clustName = "lead_clus";
	    break;
	  case RAN_CLUS:
	    gmix_params.clustName = "ran_clus";
	    break;
	}
    } else {
	/* set any defaults */
	gmix_params.createClusters = 1;
	gmix_params.clusterNum = KMEANS;
	gmix_params.clustName = "kmeans";
	gmix_params.grandType = 0;
	gmix_params.covType = 0;
	gmix_params.mixType = 0;
	gmix_params.minVar = 1e-6;
	gmix_params.maxRatio = 1e6;
	gmix_params.stopPercent = 1.;
	gmix_params.epochs = 10;
	gmix_params.varSpread = 4.;
    }

    gmix_request("display");
}

void
write_gmix_defaults(FILE *default_file)
{
    gmix_set_write_variables();
    gmix_set_defaults_variables(&gmix_params);

    if(default_file != NULL){
	fprintf(default_file, "gmix %s",
		nnargs_get_part_flags(gmixtab, TABSIZE));
	fprintf(default_file, "%s",
		nnargs_get_flags(gmix_tab, GMIX_TABSIZE));
    }
}

String
gmix_get_args ()
{
    gmix_set_write_variables();
    return (nnargs_get_part_flags(gmix_tab, GMIX_TABSIZE));
}

String
gmix_get_important_args ()
{
    gmix_set_write_variables();
    return (nnargs_get_flags_one_line(gmix_tab, GMIX_TABSIZE));
}

static void
gmix_set_clustering_alg(value)
     int value;
{
    gmix_params.clusterNum = value;
    switch(gmix_params.clusterNum){
      case KMEANS:
	gmix_params.clustName = "kmeans";
	break;
      case EM_CLUS:
	gmix_params.clustName = "em_clus";
	break;
      case LEAD_CLUS:
	gmix_params.clustName = "lead_clus";
	break;
      case RAN_CLUS:
	gmix_params.clustName = "ran_clus";
	break;
    }
    XtFree(gmix_params.clustFname);
    gmix_params.clustFname = XtNewString(lnk_params.exp_prefix);
    gmix_params.clustFname = StringConcat(gmix_params.clustFname,
					 gmix_params.clustName);
    gmix_params.clustFname = StringConcat(gmix_params.clustFname,
					 ".param");
    show_string_label(GmixClone->clustFnameLabel, gmix_params.clustFname);
    set_plot_labels();
}

void
gmix_set_cluster_param_fname()
{
    if(gmix_params.createClusters){
	XtFree(gmix_params.clustFname);
	gmix_params.clustFname = XtNewString(lnk_params.exp_prefix);
	gmix_params.clustFname = StringConcat(gmix_params.clustFname,
					     gmix_params.clustName);
	gmix_params.clustFname = StringConcat(gmix_params.clustFname,
					     ".param");
	show_string_label(GmixClone->clustFnameLabel, gmix_params.clustFname);
    }
}

char *
gmix_get_cluster_fname(){
    return gmix_params.clustFname;
}

int
gmix_get_cluster_alg(){
    return gmix_params.clusterNum;
}

int
gmix_must_store_clustering(){
    return gmix_params.createClusters;
}

int
gmix_get_epochs(){
    return gmix_params.epochs;
}

void gmix_set_epochs(int e)
{
    gmix_params.epochs = e;
}
