/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/histogram/histogram_callback.c,v 11.4 2001/08/23 20:16:34 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                             HISTOGRAM                                ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  8-Nov-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include "../main/lnknet_subs.h"
#include "histogram.h"
#include <nnlib.h>
#include <histogram_args.h>
/*
 * External Functions
 */
#include "histogram_func.h"
#include "../plots/plots_func.h"
 
/*
 * Externals
 */
extern HistogramWidgets *HistogramClone;
#include "histogram_params.h"
HistogramParams histogram_params; /* this is the real one */
#include "../data/data_params.h"
extern DataParams data_params;

/*
 * Statics
 */ 
static void histogram_show_grand_choices(HistogramWidgets *ip);

/******************************************************************************
 ******************************************************************************
 *
 * Module Name: histogram_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   histogram_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  8-Nov-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
histogram_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    HistogramWidgets *ip;
    static int nBins_buttons_mapped = False;
    static int nBinsPerClass_buttons_mapped = False;
    static int userMin_buttons_mapped = False;
    static int userMax_buttons_mapped = False;
    static int rangeFactor_buttons_mapped = False;
    static int minVar_buttons_mapped = False;

    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "equalNbinsToggle") ||
	!strcmp(client, "specifyBinsToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "histogram_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set){
	    histogram_params.specifyBins =
	      get_radio_value(ip->specifyBinsRadioBox);
	    gray_out_item(ip->nBinsForm, histogram_params.specifyBins);
	    gray_out_item(ip->nBinsPerClassForm,
			  !histogram_params.specifyBins);
	}
	
    } else if (!strcmp(client, "nBinsText")) {
	/*fprintf (stderr, "histogram_callback: %s\n", client );*/
	nBins_buttons_mapped =
	  okTextValueChanged(ip->nBinsText, ip->nBinsOKPBtn,
			     ip->nBinsCancelPBtn,
			     ip->histogramDialogShell,
			     nBins_buttons_mapped,
			     histogram_params.nBins_string);
	
    } else if ((!strcmp(client, "nBinsTextAct")) ||
	       (!strcmp(client, "nBinsOKPBtn"))) {
	String nBinsTextval;

	/*fprintf (stderr, "histogram_callback: %s\n", client );*/
	nBins_buttons_mapped =
	  okTextActivate(ip->nBinsText, ip->nBinsOKPBtn,
			 ip->nBinsCancelPBtn, &nBinsTextval);
	/* I won't actually be using this copy */
	XtFree (nBinsTextval);
	
	/* verify the string as an integer */
	histogram_params.nBins =
	  set_atoi_value(ip->nBinsText, "nBins_set",
			     histogram_params.nBins);

	/* save the current text for future comparisons */
	XtFree(histogram_params.nBins_string);
	histogram_params.nBins_string = get_string_text(ip->nBinsText);
 
	/* do any other things needed */
	histogram_set_total_nbins();
	set_plot_labels();

    } else if (!strcmp(client, "nBinsCancelPBtn")) {
	/*fprintf (stderr, "histogram_callback: %s\n", client );*/
	nBins_buttons_mapped = 
	  okTextCancel(ip->nBinsText, ip->nBinsOKPBtn,
		       ip->nBinsCancelPBtn,
		       histogram_params.nBins_string);
	
    } else if (!strcmp(client, "nBinsPerClassText")) {
	/*fprintf (stderr, "histogram_callback: %s\n", client);*/
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	nBinsPerClass_buttons_mapped =
	  okTextValueChanged(ip->nBinsPerClassText, ip->nBinsPerClassOKPBtn,
			     ip->nBinsPerClassCancelPBtn,
			     ip->histogramDialogShell,
			     nBinsPerClass_buttons_mapped,
			     histogram_params.nBinsPerClass_string);
    } else if ((!strcmp(client, "nBinsPerClassTextAct")) ||
	       (!strcmp(client, "nBinsPerClassOKPBtn"))) {
	/*fprintf (stderr, "histogram_callback: %s\n", client );*/
	String nBinsPerClassTextval;
	nBinsPerClass_buttons_mapped =
	  okTextActivate(ip->nBinsPerClassText,
			 ip->nBinsPerClassOKPBtn,
			 ip->nBinsPerClassCancelPBtn,
			 &nBinsPerClassTextval);
	
	if(histogram_params.nBinsPerClass != NULL)
	  deallocate_ivector(histogram_params.nBinsPerClass);
	/* take spaces and tabs out of the list before handing it to nnargs */
	if(nBinsPerClassTextval != NULL){
	    char *nBinsPerClassTemp = nBinsPerClassTextval;
	    nBinsPerClassTextval = strtok(nBinsPerClassTextval, " \t");
	    histogram_params.nBinsPerClass =
	      nnargs_get_int_alloc_list(nBinsPerClassTextval);
	    XtFree(nBinsPerClassTemp);
	}
	
	/* make a copy of the string as it has been interpretted */
	XtFree(histogram_params.nBinsPerClass_string);
	histogram_params.nBinsPerClass_string =
	  make_int_string(histogram_params.nBinsPerClass);
	show_string_text(ip->nBinsPerClassText,
			 histogram_params.nBinsPerClass_string);
	
	/* do anything else necessary */
	histogram_set_total_nbins();
	set_plot_labels();
	
    } else if (!strcmp(client, "nBinsPerClassCancelPBtn")) {
	/*fprintf (stderr, "histogram_callback: %s\n", client );*/
	nBinsPerClass_buttons_mapped = 
	  okTextCancel(ip->nBinsPerClassText, ip->nBinsPerClassOKPBtn,
		       ip->nBinsPerClassCancelPBtn,
		       histogram_params.nBinsPerClass_string);

    } else if (!strcmp(client, "perComponentToggle") ||
	       !strcmp(client, "perFeatureToggle") ||
	       !strcmp(client, "grandToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "histogram_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set){
	    histogram_params.grandBins =
	      get_radio_value(ip->grandBinsRadioBox);
	    histogram_show_grand_choices(ip);
	    histogram_set_total_nbins();
	    set_plot_labels();
	}
	
    } else if (!strcmp(client, "userMinText")) {
	/*fprintf (stderr, "histogram_callback: %s\n", client );*/
	userMin_buttons_mapped =
	  okTextValueChanged(ip->userMinText, ip->userMinOKPBtn,
			     ip->userMinCancelPBtn,
			     ip->histogramDialogShell,
			     userMin_buttons_mapped,
			     histogram_params.userMin_string);
	
    } else if ((!strcmp(client, "userMinTextAct")) ||
	       (!strcmp(client, "userMinOKPBtn"))) {
	String userMinTextval;

	/*fprintf (stderr, "histogram_callback: %s\n", client );*/
	userMin_buttons_mapped =
	  okTextActivate(ip->userMinText, ip->userMinOKPBtn,
			 ip->userMinCancelPBtn, &userMinTextval);
	/* I won't actually be using this copy */
	XtFree (userMinTextval);
	
	/* verify the string as a float */
	histogram_params.userMin =
	  set_atof_value(ip->userMinText, "userMin_set",
			 histogram_params.userMin);

	/* save the current text for future comparisons */
	XtFree(histogram_params.userMin_string);
	histogram_params.userMin_string = get_string_text(ip->userMinText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "userMinCancelPBtn")) {
	/*fprintf (stderr, "histogram_callback: %s\n", client );*/
	userMin_buttons_mapped = 
	  okTextCancel(ip->userMinText, ip->userMinOKPBtn,
		       ip->userMinCancelPBtn,
		       histogram_params.userMin_string);
	
    } else if (!strcmp(client, "userMaxText")) {
	/*fprintf (stderr, "histogram_callback: %s\n", client );*/
	userMax_buttons_mapped =
	  okTextValueChanged(ip->userMaxText, ip->userMaxOKPBtn,
			     ip->userMaxCancelPBtn,
			     ip->histogramDialogShell,
			     userMax_buttons_mapped,
			     histogram_params.userMax_string);
	
    } else if ((!strcmp(client, "userMaxTextAct")) ||
	       (!strcmp(client, "userMaxOKPBtn"))) {
	String userMaxTextval;

	/*fprintf (stderr, "histogram_callback: %s\n", client );*/
	userMax_buttons_mapped =
	  okTextActivate(ip->userMaxText, ip->userMaxOKPBtn,
			 ip->userMaxCancelPBtn, &userMaxTextval);
	/* I won't actually be using this copy */
	XtFree (userMaxTextval);
	
	/* verify the string as a float */
	histogram_params.userMax =
	  set_atof_value(ip->userMaxText, "userMax_set",
			 histogram_params.userMax);

	/* save the current text for future comparisons */
	XtFree(histogram_params.userMax_string);
	histogram_params.userMax_string = get_string_text(ip->userMaxText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "userMaxCancelPBtn")) {
	/*fprintf (stderr, "histogram_callback: %s\n", client );*/
	userMax_buttons_mapped = 
	  okTextCancel(ip->userMaxText, ip->userMaxOKPBtn,
		       ip->userMaxCancelPBtn,
		       histogram_params.userMax_string);
	
    } else if (!strcmp(client, "equalWidthToggle") ||
	       !strcmp(client, "equalNumToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "histogram_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set) histogram_params.binType =
	  get_radio_value(ip->binTypeRadioBox);
	
    } else if (!strcmp(client, "rangeFactorText")) {
	/*fprintf (stderr, "histogram_callback: %s\n", client );*/
	rangeFactor_buttons_mapped =
	  okTextValueChanged(ip->rangeFactorText, ip->rangeFactorOKPBtn,
			     ip->rangeFactorCancelPBtn,
			     ip->histogramDialogShell,
			     rangeFactor_buttons_mapped,
			     histogram_params.rangeFactor_string);
	
    } else if ((!strcmp(client, "rangeFactorTextAct")) ||
	       (!strcmp(client, "rangeFactorOKPBtn"))) {
	String rangeFactorTextval;

	/*fprintf (stderr, "histogram_callback: %s\n", client );*/
	rangeFactor_buttons_mapped =
	  okTextActivate(ip->rangeFactorText, ip->rangeFactorOKPBtn,
			 ip->rangeFactorCancelPBtn, &rangeFactorTextval);
	/* I won't actually be using this copy */
	XtFree (rangeFactorTextval);
	
	/* verify the string as a float */
	histogram_params.rangeFactor =
	  set_atof_low_value(ip->rangeFactorText, "rangeFactor_set",
			     histogram_params.rangeFactor, 0.);

	/* save the current text for future comparisons */
	XtFree(histogram_params.rangeFactor_string);
	histogram_params.rangeFactor_string = get_string_text(ip->rangeFactorText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "rangeFactorCancelPBtn")) {
	/*fprintf (stderr, "histogram_callback: %s\n", client );*/
	rangeFactor_buttons_mapped = 
	  okTextCancel(ip->rangeFactorText, ip->rangeFactorOKPBtn,
		       ip->rangeFactorCancelPBtn,
		       histogram_params.rangeFactor_string);
	
    } else if (!strcmp(client, "unknownToggle") ||
	       !strcmp(client, "useGaussToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "histogram_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set){
	    histogram_params.useGauss =
	      get_radio_value(ip->useGaussRadioBox);
	    gray_out_item(ip->minVarForm, !histogram_params.useGauss);
	    set_plot_labels();
	}
	
    } else if (!strcmp(client, "minVarText")) {
	/*fprintf (stderr, "histogram_callback: %s\n", client );*/
	minVar_buttons_mapped =
	  okTextValueChanged(ip->minVarText, ip->minVarOKPBtn,
			     ip->minVarCancelPBtn,
			     ip->histogramDialogShell,
			     minVar_buttons_mapped,
			     histogram_params.minVar_string);
	
    } else if ((!strcmp(client, "minVarTextAct")) ||
	       (!strcmp(client, "minVarOKPBtn"))) {
	String minVarTextval;

	/*fprintf (stderr, "histogram_callback: %s\n", client );*/
	minVar_buttons_mapped =
	  okTextActivate(ip->minVarText, ip->minVarOKPBtn,
			 ip->minVarCancelPBtn, &minVarTextval);
	/* I won't actually be using this copy */
	XtFree (minVarTextval);
	
	/* verify the string as a float */
	histogram_params.minVar =
	  set_atof_low_value(ip->minVarText, "minVar_set",
			     histogram_params.minVar, 0.);

	/* save the current text for future comparisons */
	XtFree(histogram_params.minVar_string);
	histogram_params.minVar_string = get_string_text(ip->minVarText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "minVarCancelPBtn")) {
	/*fprintf (stderr, "histogram_callback: %s\n", client );*/
	minVar_buttons_mapped = 
	  okTextCancel(ip->minVarText, ip->minVarOKPBtn,
		       ip->minVarCancelPBtn,
		       histogram_params.minVar_string);
	
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "histogram_callback: %s\n", client );*/
	close_popup(HistogramClone->histogramForm);
	
    } else {
	fprintf(stderr, "histogram_callback: unknown client: %s\n", (char *)client);
    }
}

void
histogram_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(HistogramClone->histogramForm);
    } else if (!strcmp(request, "init")){
	histogram_params.specifyBins = 0;
	histogram_params.nBins = 0;
	histogram_params.nBins_string = NULL;
	histogram_params.nBinsPerClass = NULL;
	histogram_params.nBinsPerClass_string = NULL;
	histogram_params.grandBins = 0;
	histogram_params.userMin = 0.;
	histogram_params.userMin_string = NULL;
	histogram_params.userMax = 0.;
	histogram_params.userMax_string = NULL;
	histogram_params.binType = 0;
	histogram_params.rangeFactor = 0.;
	histogram_params.rangeFactor_string = NULL;
	histogram_params.useGauss = 0;
	histogram_params.minVar = 0.;
	histogram_params.minVar_string = NULL;
    } else if (!strcmp(request, "display")){
	char string[10];
	show_radio_value(HistogramClone->specifyBinsRadioBox,
			 histogram_params.specifyBins);
	gray_out_item(HistogramClone->nBinsForm, histogram_params.specifyBins);
	gray_out_item(HistogramClone->nBinsPerClassForm,
		      !histogram_params.specifyBins);
	XtFree(histogram_params.nBins_string);
	sprintf(string, "%d", histogram_params.nBins);
	histogram_params.nBins_string = XtNewString(string);
	show_string_text(HistogramClone->nBinsText,
			 histogram_params.nBins_string);
	XtFree(histogram_params.nBinsPerClass_string);
	histogram_params.nBinsPerClass_string =
	  make_int_string(histogram_params.nBinsPerClass);
	show_string_text(HistogramClone->nBinsPerClassText,
			 histogram_params.nBinsPerClass_string);
	show_radio_value(HistogramClone->grandBinsRadioBox,
			 histogram_params.grandBins);
	histogram_show_grand_choices(HistogramClone);
	XtFree(histogram_params.userMin_string);
	sprintf(string, "%3.2g", histogram_params.userMin);
	histogram_params.userMin_string = XtNewString(string);
	show_string_text(HistogramClone->userMinText,
			 histogram_params.userMin_string);
	XtFree(histogram_params.userMax_string);
	sprintf(string, "%3.2g", histogram_params.userMax);
	histogram_params.userMax_string = XtNewString(string);
	show_string_text(HistogramClone->userMaxText,
			 histogram_params.userMax_string);
	show_radio_value(HistogramClone->binTypeRadioBox,
			 histogram_params.binType);
	XtFree(histogram_params.rangeFactor_string);
	sprintf(string, "%5.2g", histogram_params.rangeFactor);
	histogram_params.rangeFactor_string = XtNewString(string);
	show_string_text(HistogramClone->rangeFactorText,
			 histogram_params.rangeFactor_string);
	show_radio_value(HistogramClone->useGaussRadioBox,
			 histogram_params.useGauss);
	gray_out_item(HistogramClone->minVarForm, !histogram_params.useGauss);
	XtFree(histogram_params.minVar_string);
	sprintf(string, "%5.2g", histogram_params.minVar);
	histogram_params.minVar_string = XtNewString(string);
	show_string_text(HistogramClone->minVarText,
			 histogram_params.minVar_string);
    } else {
	fprintf(stderr, "histogram_request: unknown request: %s\n", request);
    }
}

String
histogram_get_plot_label(String string)
{
    char numString[10];
    string = StringConcat(string, "Nbins: ");
    sprintf(numString, "%d", histogram_params.total_nbins);
    string = StringConcat(string, numString);
    if(histogram_params.useGauss)
      string = StringConcat(string, "Use Gauss");
    else
      string = StringConcat(string, "Allow Unknown");
    return string;
}

static void
histogram_set_read_variables()
{
    histogram_params.nBins = nbins;
    histogram_params.nBinsPerClass = nBinsPerClass;
    histogram_params.grandBins = grand_bins;
    histogram_params.userMin = user_min;
    histogram_params.userMax = user_max;
    histogram_params.binType = bin_type;
    histogram_params.rangeFactor = range_factor;
    histogram_params.useGauss = use_gauss;
    histogram_params.minVar = minimum_var;
}

static void
histogram_set_write_variables()
{
    nbins = histogram_params.nBins;
    nBinsPerClass = histogram_params.nBinsPerClass;
    grand_bins = histogram_params.grandBins;
    user_min = histogram_params.userMin;
    user_max = histogram_params.userMax;
    bin_type = histogram_params.binType;
    range_factor = histogram_params.rangeFactor;
    use_gauss = histogram_params.useGauss;
    minimum_var = histogram_params.minVar;
}

static ARG histogramtab[] =
{
    {"specifyBins", BOOLEAN, (int *)NULL,},
};
#define TABSIZE (sizeof(histogramtab)/sizeof(ARG))

static void
histogram_set_defaults_variables(dp)
     HistogramParams *dp;
{
    nnargs_set_variable("specifyBins", (int *)&dp->specifyBins, histogramtab, TABSIZE);
}

void
read_histogram_defaults(argc, argv)
     int argc;
     char **argv;
{
    histogram_params.specifyBins = 0;
    bin_type = 0;
    use_gauss = 0;
    nBinsPerClass = NULL;
    histogram_set_defaults_variables(&histogram_params);
    if(argc > 0){
	argc = nnargs_part(argc, argv, histogramtab, TABSIZE);
	argc = nnargs(argc, argv, histogram_tab, HISTOGRAM_TABSIZE);
	histogram_set_read_variables();

    } else {
	/* set any defaults */
	histogram_params.useGauss = 1;
	histogram_params.nBins = 2;
	histogram_params.minVar = 1e-6;
	histogram_params.rangeFactor = 1.5;
	histogram_params.userMin = -3.;
	histogram_params.userMax = 3.;
	histogram_params.grandBins = 0;
	histogram_params.binType = 0;
    }

    histogram_request("display");
}

void
write_histogram_defaults(FILE *default_file)
{
    histogram_set_write_variables();
    histogram_set_defaults_variables(&histogram_params);

    if(default_file != NULL){
	fprintf(default_file, "histogram %s",
		nnargs_get_part_flags(histogramtab, TABSIZE));
	fprintf(default_file, "%s",
		nnargs_get_flags(histogram_tab, HISTOGRAM_TABSIZE));
    }
}

String
histogram_get_args ()
{
    histogram_set_write_variables();
    if(!histogram_params.specifyBins) nBinsPerClass = NULL;
    return (nnargs_get_part_flags(histogram_tab, HISTOGRAM_TABSIZE));
}

String
histogram_get_important_args ()
{
    histogram_set_write_variables();
    if(!histogram_params.specifyBins) nBinsPerClass = NULL;
    return (nnargs_get_flags_one_line(histogram_tab, HISTOGRAM_TABSIZE));
}

static void
histogram_show_grand_choices(HistogramWidgets *ip)
{
    gray_out_item(ip->specifyBinsRadioBox,
		  histogram_params.grandBins != 0);
    gray_out_item(ip->nBinsPerClassForm,
		  histogram_params.grandBins != 0 ||
		  !histogram_params.specifyBins);
    gray_out_item(ip->nBinsForm,
		  histogram_params.grandBins == 0 &&
		  histogram_params.specifyBins);
    gray_out_item(ip->binTypeRadioBox,
		  histogram_params.grandBins == 2);
    gray_out_item(ip->hyperCubeLabel,
		  histogram_params.grandBins != 2);
    gray_out_item(ip->userMinForm,
		  histogram_params.grandBins != 2);
    gray_out_item(ip->userMaxForm,
		  histogram_params.grandBins != 2);
}

void
histogram_set_total_nbins()
{
    static char total_label[64];
    if(histogram_params.grandBins == 0){
	if(histogram_params.specifyBins &&
	   histogram_params.nBinsPerClass != NULL){
	    int i;
	    for(i = 1, histogram_params.total_nbins = 0;
		i <= histogram_params.nBinsPerClass[0]; i++){
		histogram_params.total_nbins +=
		  histogram_params.nBinsPerClass[i];
	    }
	} else {
	    histogram_params.total_nbins =
	      data_params.noutputs * histogram_params.nBins;
	}
    } else if (histogram_params.grandBins){
	histogram_params.total_nbins = histogram_params.nBins;
    }
    sprintf(total_label, "This will create %d bins per dimension.",
	    histogram_params.total_nbins);
    show_string_label(HistogramClone->totalNbinsLabel, total_label);
}

void
histogram_store_plot_command(FILE *shell_file)
{
    if(histogram_params.useGauss)
      fprintf(shell_file, "\\\n -use_gauss");
}
