/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/hyper/hyper_callback.c,v 11.4 2001/08/23 20:16:37 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                       Lnknet Window Interface                        ****
 ****                           Hyper Window                              ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  14-Sep-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include "../main/lnknet_subs.h"
#include "hyper.h"
#include <nnlib.h>
#include <hyper_args.h>
/*
 * External Functions
 */
#include "../plots/plots_func.h"
#include "hyper_func.h"
 
/*
 * Externals
 */
extern HyperWidgets *HyperClone;
#include "hyper_params.h"
HyperParams hyper_params; /* this is the real one */

/*
 * Statics
 */
static void hyper_set_read_variables();
static void hyper_set_write_variables();
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: hyper_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   hyper_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  20-Nov-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
hyper_callback(Widget w, XtPointer client, XtPointer call)
{
    /* SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    HyperWidgets *ip;
    static int nEpochs_buttons_mapped = False;
    static int rInit_buttons_mapped = False;
    static int nCenters_buttons_mapped = False;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "nEpochsText")) {
	/*fprintf (stderr, "hyper_callback: %s\n", client );*/
	nEpochs_buttons_mapped =
	  okTextValueChanged(ip->nEpochsText, ip->nEpochsOKPBtn,
			     ip->nEpochsCancelPBtn,
			     ip->hyperDialogShell,
			     nEpochs_buttons_mapped,
			     hyper_params.nEpochs_string);
	
    } else if ((!strcmp(client, "nEpochsTextAct")) ||
	       (!strcmp(client, "nEpochsOKPBtn"))) {
	String nEpochsTextval;

	/*fprintf (stderr, "hyper_callback: %s\n", client );*/
	nEpochs_buttons_mapped =
	  okTextActivate(ip->nEpochsText, ip->nEpochsOKPBtn,
			 ip->nEpochsCancelPBtn, &nEpochsTextval);
	/* I won't actually be using this copy */
	XtFree (nEpochsTextval);
	
	/* verify the string as an integer */
	hyper_params.nEpochs =
	  set_atoi_value(ip->nEpochsText, "nEpochs_set",
			     hyper_params.nEpochs);

	/* save the current text for future comparisons */
	XtFree(hyper_params.nEpochs_string);
	hyper_params.nEpochs_string = get_string_text(ip->nEpochsText);
 
	/* do any other things needed */
	set_plot_labels();

    } else if (!strcmp(client, "nEpochsCancelPBtn")) {
	/*fprintf (stderr, "hyper_callback: %s\n", client );*/
	nEpochs_buttons_mapped = 
	  okTextCancel(ip->nEpochsText, ip->nEpochsOKPBtn,
		       ip->nEpochsCancelPBtn,
		       hyper_params.nEpochs_string);

    } else if (!strcmp(client, "rInitText")) {
	/*fprintf (stderr, "hyper_callback: %s\n", client );*/
	rInit_buttons_mapped =
	  okTextValueChanged(ip->rInitText, ip->rInitOKPBtn,
			     ip->rInitCancelPBtn,
			     ip->hyperDialogShell,
			     rInit_buttons_mapped,
			     hyper_params.rInit_string);
	
    } else if ((!strcmp(client, "rInitTextAct")) ||
	       (!strcmp(client, "rInitOKPBtn"))) {
	String rInitTextval;

	/*fprintf (stderr, "hyper_callback: %s\n", client );*/
	rInit_buttons_mapped =
	  okTextActivate(ip->rInitText, ip->rInitOKPBtn,
			 ip->rInitCancelPBtn, &rInitTextval);
	/* I won't actually be using this copy */
	XtFree (rInitTextval);
	
	/* verify the string as a float */
	hyper_params.rInit =
	  set_atof_low_value(ip->rInitText, "rInit_set",
			     hyper_params.rInit, 0.);

	/* save the current text for future comparisons */
	XtFree(hyper_params.rInit_string);
	hyper_params.rInit_string = get_string_text(ip->rInitText);
 
	/* do any other things needed */
	set_plot_labels();

    } else if (!strcmp(client, "rInitCancelPBtn")) {
	/*fprintf (stderr, "hyper_callback: %s\n", client );*/
	rInit_buttons_mapped = 
	  okTextCancel(ip->rInitText, ip->rInitOKPBtn,
		       ip->rInitCancelPBtn,
		       hyper_params.rInit_string);

    } else if (!strcmp(client, "pruneToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "hyper_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	hyper_params.prune = state->set;
	gray_out_item(ip->useRadioBox, !hyper_params.prune);
	gray_out_item(ip->nCentersForm, !hyper_params.prune);
 
    } else if (!strcmp(client, "sizeToggle") ||
	       !strcmp(client, "useToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "hyper_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set){
	    hyper_params.use =
	      get_radio_value(ip->useRadioBox);
	    set_plot_labels();
	}
 
    } else if (!strcmp(client, "nCentersText")) {
	/*fprintf (stderr, "hyper_callback: %s\n", client );*/
	nCenters_buttons_mapped =
	  okTextValueChanged(ip->nCentersText, ip->nCentersOKPBtn,
			     ip->nCentersCancelPBtn,
			     ip->hyperDialogShell,
			     nCenters_buttons_mapped,
			     hyper_params.nCenters_string);
	
    } else if ((!strcmp(client, "nCentersTextAct")) ||
	       (!strcmp(client, "nCentersOKPBtn"))) {
	String nCentersTextval;

	/*fprintf (stderr, "hyper_callback: %s\n", client );*/
	nCenters_buttons_mapped =
	  okTextActivate(ip->nCentersText, ip->nCentersOKPBtn,
			 ip->nCentersCancelPBtn, &nCentersTextval);
	/* I won't actually be using this copy */
	XtFree (nCentersTextval);
	
	/* verify the string as an integer */
	hyper_params.nCenters =
	  set_atoi_value(ip->nCentersText, "nCenters_set",
			     hyper_params.nCenters);

	/* save the current text for future comparisons */
	XtFree(hyper_params.nCenters_string);
	hyper_params.nCenters_string = get_string_text(ip->nCentersText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "nCentersCancelPBtn")) {
	/*fprintf (stderr, "hyper_callback: %s\n", client );*/
	nCenters_buttons_mapped = 
	  okTextCancel(ip->nCentersText, ip->nCentersOKPBtn,
		       ip->nCentersCancelPBtn,
		       hyper_params.nCenters_string);
  } else if (!strcmp(client, "nCentersTextAct")) {
    fprintf (stderr, "hyper_callback: %s\n", (char *)client );
 
  } else if (!strcmp(client, "nCentersText")) {
    fprintf (stderr, "hyper_callback: %s\n", (char *)client );
 
  } else if (!strcmp(client, "nCentersOKPBtn")) {
    fprintf (stderr, "hyper_callback: %s\n", (char *)client );
 
  } else if (!strcmp(client, "nCentersCancelPBtn")) {
    fprintf (stderr, "hyper_callback: %s\n", (char *)client );
 
  } else if (!strcmp(client, "unknownToggle") ||
	     !strcmp(client, "forceToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "hyper_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set){
	    hyper_params.force =
	      get_radio_value(ip->forceRadioBox);
	    set_plot_labels();
	}

    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "hyper_callback: %s\n", client);*/
	close_popup(HyperClone->hyperForm);
	
    } else {
	fprintf (stderr, "hyper_callback: unknown client: %s\n", (char *)client);
    }
}

void
hyper_request(String request)
{
    if (!strcmp(request, "open")) {
	open_popup(HyperClone->hyperForm);

    } else if (!strcmp(request, "init")){
	/* this should only get called once, before ANYTHING 
	   gets allocated or there will be memory leaks */
	hyper_params.nEpochs = 0;
	hyper_params.nEpochs_string = NULL;
	hyper_params.rInit = 0.;
	hyper_params.rInit_string = NULL;
	hyper_params.prune = 0;
	hyper_params.use = 0;
	hyper_params.nCenters = 0;
	hyper_params.nCenters_string = NULL;
	hyper_params.force = 0;

    } else if (!strcmp(request, "display")){
	char string[10]; /* for creating int and float text */

	XtFree(hyper_params.nEpochs_string);
	sprintf(string, "%d", hyper_params.nEpochs);
	hyper_params.nEpochs_string = XtNewString(string);
	show_string_text(HyperClone->nEpochsText,
			 hyper_params.nEpochs_string);

	XtFree(hyper_params.rInit_string);
	sprintf(string, "%5.2g", hyper_params.rInit);
	hyper_params.rInit_string = XtNewString(string);
	show_string_text(HyperClone->rInitText,
			 hyper_params.rInit_string);

	show_toggle_value(HyperClone->pruneToggle, hyper_params.prune);
	gray_out_item(HyperClone->useRadioBox, !hyper_params.prune);
	gray_out_item(HyperClone->nCentersForm, !hyper_params.prune);

	show_radio_value(HyperClone->useRadioBox, hyper_params.use);

	XtFree(hyper_params.nCenters_string);
	sprintf(string, "%d", hyper_params.nCenters);
	hyper_params.nCenters_string = XtNewString(string);
	show_string_text(HyperClone->nCentersText,
			 hyper_params.nCenters_string);

	show_radio_value(HyperClone->forceRadioBox, hyper_params.force);


    } else {
	fprintf (stderr, "hyper_request: unknown request: %s\n", request);
    }
}

String
hyper_get_plot_label(String string)
{
    char numString[10];
    string = StringConcat(string, " epochs:");
    sprintf(numString, "%d", hyper_params.nEpochs);
    string = StringConcat(string, numString);
    string = StringConcat(string, " radius:");
    sprintf(numString, "%.2g", hyper_params.rInit);
    string = StringConcat(string, numString);
    if(hyper_params.prune){
	string = StringConcat(string, " Prune by ");
	if(hyper_params.use){
	    string = StringConcat(string, "times used");
	} else {
	    string = StringConcat(string, "size");
	}
    }
    if(hyper_params.force){
	string = StringConcat(string, "Use NN");
    } else {
	string = StringConcat(string, "Allow Unknown");
    }
    return string;
}

static void
hyper_set_read_variables()
{
    hyper_params.nEpochs = nepochs;
    hyper_params.rInit = r_init;
    hyper_params.prune = prune_flag;
    hyper_params.use = correct_sort;
    hyper_params.nCenters = prune_n;
    hyper_params.force = force_classification;
}

static void
hyper_set_write_variables()
{
    nepochs = hyper_params.nEpochs;
    r_init = hyper_params.rInit;
    prune_flag = hyper_params.prune;
    correct_sort = hyper_params.use;
    prune_n = hyper_params.nCenters;
    force_classification = hyper_params.force;
}

void
read_hyper_defaults(int argc, char **argv)
{
    prune_flag = 0;
    correct_sort = 0;
    force_classification = 0;
    add_flag = 0;
    best_n = 0;

    if(argc > 0){
	argc = nnargs(argc, argv, hyper_tab, HYPER_TABSIZE);
	hyper_set_read_variables();
    } else {
	add_flag = 1;
	best_n = 1;
	hyper_params.use = 0;
	hyper_params.nEpochs = 5;
	hyper_params.nCenters = 100;
	hyper_params.force = 1;
	hyper_params.prune = 1;
	hyper_params.rInit = 0.5;
    }
    hyper_request("display");
}

void
write_hyper_defaults(default_file)
     FILE *default_file;
{
    hyper_set_write_variables();
    if(default_file != NULL){
	fprintf(default_file, "hyper %s",
		nnargs_get_flags(hyper_tab, HYPER_TABSIZE));
    }
}

String
hyper_get_args ()
{
    hyper_set_write_variables();
    add_flag = 1;
    best_n = 1;
    return (nnargs_get_part_flags(hyper_tab, HYPER_TABSIZE));
}

String
hyper_get_important_args ()
{
    hyper_set_write_variables();
    add_flag = 1;
    best_n = 1;
    return (nnargs_get_flags_one_line(hyper_tab, HYPER_TABSIZE));
}

int
hyper_get_epochs()
{
    return hyper_params.nEpochs;
}

void
hyper_set_epochs(int e)
{
    hyper_params.nEpochs = e;
}
