/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/irbf/irbf_callback.c,v 11.4 2001/08/23 20:16:40 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                                 IRBF                                 ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  7-Nov-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include "../main/lnknet_subs.h"
#include "irbf.h"
#include <nnlib.h>
#include <irbf_args.h>
/*
 * External Functions
 */
#include "../plots/plots_func.h"
#include "../kmeans/kmeans_func.h"
#include "../em_clus/em_clus_func.h"
#include "../lead_clus/lead_clus_func.h"
#include "../ran_clus/ran_clus_func.h"
#include "../cost/cost_func.h"

/*
 * Externals
 */
extern IrbfWidgets *IrbfClone;
#include "irbf_params.h"
IrbfParams irbf_params; /* this is the real one */
#include "../lnk/lnk_params.h"
LnkParams lnk_params;

/*
 * Statics
 */
static void irbf_set_clustering_alg(int);
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: irbf_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   irbf_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  7-Nov-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
irbf_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    IrbfWidgets *ip;
    static int nEpochs_buttons_mapped = False;
    int hSpread_buttons_mapped = False;
    int extraHspread_buttons_mapped = False;
    static int weightEtta_buttons_mapped = False;
    static int meanEtta_buttons_mapped = False;
    static int varEtta_buttons_mapped = False;
    static int costParam_buttons_mapped = False;
    int maxMag_buttons_mapped = False;
    int minVar_buttons_mapped = False;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "createClustersToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "irbf_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	irbf_params.createClusters = state->set;
 
    } else if (!strcmp(client, "kmeansPBtn")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	irbf_set_clustering_alg(KMEANS);

    } else if (!strcmp(client, "em_clusPBtn")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	irbf_set_clustering_alg(EM_CLUS);
 
    } else if (!strcmp(client, "lead_clusPBtn")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	irbf_set_clustering_alg(LEAD_CLUS);
 
    } else if (!strcmp(client, "ran_clusPBtn")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	irbf_set_clustering_alg(RAN_CLUS);
 
    } else if (!strcmp(client, "clustParamsPBtn")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	switch(irbf_params.clusterNum){
	  case KMEANS:
	    kmeans_request("open");
	    break;
	  case EM_CLUS:
	    em_clus_request("open");
	    break;
	  case LEAD_CLUS:
	    lead_clus_request("open");
	    break;
	  case RAN_CLUS:
	    ran_clus_request("open");
	    break;
	}
 
    } else if (!strcmp(client, "nEpochsText")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	nEpochs_buttons_mapped =
	  okTextValueChanged(ip->nEpochsText, ip->nEpochsOKPBtn,
			     ip->nEpochsCancelPBtn,
			     ip->irbfDialogShell,
			     nEpochs_buttons_mapped,
			     irbf_params.nEpochs_string);
	
    } else if ((!strcmp(client, "nEpochsTextAct")) ||
	       (!strcmp(client, "nEpochsOKPBtn"))) {
	String nEpochsTextval;

	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	nEpochs_buttons_mapped =
	  okTextActivate(ip->nEpochsText, ip->nEpochsOKPBtn,
			 ip->nEpochsCancelPBtn, &nEpochsTextval);
	/* I won't actually be using this copy */
	XtFree (nEpochsTextval);
	
	/* verify the string as an integer */
	irbf_params.nEpochs =
	  set_atoi_value(ip->nEpochsText, "nEpochs_set",
			     irbf_params.nEpochs);

	/* save the current text for future comparisons */
	XtFree(irbf_params.nEpochs_string);
	irbf_params.nEpochs_string = get_string_text(ip->nEpochsText);
 
	/* do any other things needed */
	set_plot_labels();

    } else if (!strcmp(client, "nEpochsCancelPBtn")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	nEpochs_buttons_mapped = 
	  okTextCancel(ip->nEpochsText, ip->nEpochsOKPBtn,
		       ip->nEpochsCancelPBtn,
		       irbf_params.nEpochs_string);

    } else if (!strcmp(client, "hSpreadText")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	hSpread_buttons_mapped =
	  okTextValueChanged(ip->hSpreadText, ip->hSpreadOKPBtn,
			     ip->hSpreadCancelPBtn,
			     ip->irbfDialogShell,
			     hSpread_buttons_mapped,
			     irbf_params.hSpread_string);
	
    } else if ((!strcmp(client, "hSpreadTextAct")) ||
	       (!strcmp(client, "hSpreadOKPBtn"))) {
	String hSpreadTextval;

	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	hSpread_buttons_mapped =
	  okTextActivate(ip->hSpreadText, ip->hSpreadOKPBtn,
			 ip->hSpreadCancelPBtn, &hSpreadTextval);
	/* I won't actually be using this copy */
	XtFree (hSpreadTextval);
	
	/* verify the string as a float */
	irbf_params.hSpread =
	  set_atof_low_value(ip->hSpreadText, "hSpread_set",
			     irbf_params.hSpread, 0.);

	/* save the current text for future comparisons */
	XtFree(irbf_params.hSpread_string);
	irbf_params.hSpread_string = get_string_text(ip->hSpreadText);
 
	/* do any other things needed */
	set_plot_labels();

    } else if (!strcmp(client, "hSpreadCancelPBtn")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	hSpread_buttons_mapped = 
	  okTextCancel(ip->hSpreadText, ip->hSpreadOKPBtn,
		       ip->hSpreadCancelPBtn,
		       irbf_params.hSpread_string);
 
    } else if (!strcmp(client, "extraHspreadText")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	extraHspread_buttons_mapped =
	  okTextValueChanged(ip->extraHspreadText, ip->extraHspreadOKPBtn,
			     ip->extraHspreadCancelPBtn,
			     ip->irbfDialogShell,
			     extraHspread_buttons_mapped,
			     irbf_params.extraHspread_string);
	
    } else if ((!strcmp(client, "extraHspreadTextAct")) ||
	       (!strcmp(client, "extraHspreadOKPBtn"))) {
	String extraHspreadTextval;

	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	extraHspread_buttons_mapped =
	  okTextActivate(ip->extraHspreadText, ip->extraHspreadOKPBtn,
			 ip->extraHspreadCancelPBtn, &extraHspreadTextval);
	/* I won't actually be using this copy */
	XtFree (extraHspreadTextval);
	
	/* verify the string as a float */
	irbf_params.extraHspread =
	  set_atof_low_value(ip->extraHspreadText, "extraHspread_set",
			     irbf_params.extraHspread, 0.);

	/* save the current text for future comparisons */
	XtFree(irbf_params.extraHspread_string);
	irbf_params.extraHspread_string = get_string_text(ip->extraHspreadText);
 
	/* do any other things needed */
	set_plot_labels();

    } else if (!strcmp(client, "extraHspreadCancelPBtn")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	extraHspread_buttons_mapped = 
	  okTextCancel(ip->extraHspreadText, ip->extraHspreadOKPBtn,
		       ip->extraHspreadCancelPBtn,
		       irbf_params.extraHspread_string);
 
 
    } else if (!strcmp(client, "weightEttaText")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	weightEtta_buttons_mapped =
	  okTextValueChanged(ip->weightEttaText, ip->weightEttaOKPBtn,
			     ip->weightEttaCancelPBtn,
			     ip->irbfDialogShell,
			     weightEtta_buttons_mapped,
			     irbf_params.weightEtta_string);
	
    } else if ((!strcmp(client, "weightEttaTextAct")) ||
	       (!strcmp(client, "weightEttaOKPBtn"))) {
	String weightEttaTextval;

	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	weightEtta_buttons_mapped =
	  okTextActivate(ip->weightEttaText, ip->weightEttaOKPBtn,
			 ip->weightEttaCancelPBtn, &weightEttaTextval);
	/* I won't actually be using this copy */
	XtFree (weightEttaTextval);
	
	/* verify the string as a float */
	irbf_params.weightEtta =
	  set_atof_low_value(ip->weightEttaText, "weightEtta_set",
			     irbf_params.weightEtta, 0.);

	/* save the current text for future comparisons */
	XtFree(irbf_params.weightEtta_string);
	irbf_params.weightEtta_string = get_string_text(ip->weightEttaText);
 
	/* do any other things needed */
	set_plot_labels();

    } else if (!strcmp(client, "weightEttaCancelPBtn")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	weightEtta_buttons_mapped = 
	  okTextCancel(ip->weightEttaText, ip->weightEttaOKPBtn,
		       ip->weightEttaCancelPBtn,
		       irbf_params.weightEtta_string);
	
    } else if (!strcmp(client, "meanEttaText")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	meanEtta_buttons_mapped =
	  okTextValueChanged(ip->meanEttaText, ip->meanEttaOKPBtn,
			     ip->meanEttaCancelPBtn,
			     ip->irbfDialogShell,
			     meanEtta_buttons_mapped,
			     irbf_params.meanEtta_string);
	
    } else if ((!strcmp(client, "meanEttaTextAct")) ||
	       (!strcmp(client, "meanEttaOKPBtn"))) {
	String meanEttaTextval;

	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	meanEtta_buttons_mapped =
	  okTextActivate(ip->meanEttaText, ip->meanEttaOKPBtn,
			 ip->meanEttaCancelPBtn, &meanEttaTextval);
	/* I won't actually be using this copy */
	XtFree (meanEttaTextval);
	
	/* verify the string as a float */
	irbf_params.meanEtta =
	  set_atof_low_value(ip->meanEttaText, "meanEtta_set",
			     irbf_params.meanEtta, 0.);

	/* save the current text for future comparisons */
	XtFree(irbf_params.meanEtta_string);
	irbf_params.meanEtta_string = get_string_text(ip->meanEttaText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "meanEttaCancelPBtn")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	meanEtta_buttons_mapped = 
	  okTextCancel(ip->meanEttaText, ip->meanEttaOKPBtn,
		       ip->meanEttaCancelPBtn,
		       irbf_params.meanEtta_string);
	
    } else if (!strcmp(client, "varEttaText")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	varEtta_buttons_mapped =
	  okTextValueChanged(ip->varEttaText, ip->varEttaOKPBtn,
			     ip->varEttaCancelPBtn,
			     ip->irbfDialogShell,
			     varEtta_buttons_mapped,
			     irbf_params.varEtta_string);
	
    } else if ((!strcmp(client, "varEttaTextAct")) ||
	       (!strcmp(client, "varEttaOKPBtn"))) {
	String varEttaTextval;

	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	varEtta_buttons_mapped =
	  okTextActivate(ip->varEttaText, ip->varEttaOKPBtn,
			 ip->varEttaCancelPBtn, &varEttaTextval);
	/* I won't actually be using this copy */
	XtFree (varEttaTextval);
	
	/* verify the string as a float */
	irbf_params.varEtta =
	  set_atof_low_value(ip->varEttaText, "varEtta_set",
			     irbf_params.varEtta, 0.);

	/* save the current text for future comparisons */
	XtFree(irbf_params.varEtta_string);
	irbf_params.varEtta_string = get_string_text(ip->varEttaText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "varEttaCancelPBtn")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	varEtta_buttons_mapped = 
	  okTextCancel(ip->varEttaText, ip->varEttaOKPBtn,
		       ip->varEttaCancelPBtn,
		       irbf_params.varEtta_string);
	
    } else if (!strcmp(client, "costParamText")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	costParam_buttons_mapped =
	  okTextValueChanged(ip->costParamText, ip->costParamOKPBtn,
			     ip->costParamCancelPBtn,
			     ip->irbfDialogShell,
			     costParam_buttons_mapped,
			     irbf_params.costParam_string);
	
    } else if ((!strcmp(client, "costParamTextAct")) ||
	       (!strcmp(client, "costParamOKPBtn"))) {
	String costParamTextval;

	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	costParam_buttons_mapped =
	  okTextActivate(ip->costParamText, ip->costParamOKPBtn,
			 ip->costParamCancelPBtn, &costParamTextval);
	/* I won't actually be using this copy */
	XtFree (costParamTextval);
	
	/* verify the string as a float */
	irbf_params.costParam =
	  set_atof_value(ip->costParamText, "costParam_set",
			 irbf_params.costParam);

	/* save the current text for future comparisons */
	XtFree(irbf_params.costParam_string);
	irbf_params.costParam_string = get_string_text(ip->costParamText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "costParamCancelPBtn")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	costParam_buttons_mapped = 
	  okTextCancel(ip->costParamText, ip->costParamOKPBtn,
		       ip->costParamCancelPBtn,
		       irbf_params.costParam_string);
	
    } else if (!strcmp(client, "separateVarToggle") ||
	       !strcmp(client, "grandVarToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "irbf_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set) irbf_params.grandVar =
	  get_radio_value(ip->grandVarRadioBox);

    } else if (!strcmp(client, "sqErrToggle") ||
	       !strcmp(client, "top2diffToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "irbf_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set){
	    irbf_params.costFunc =
	      get_radio_value(ip->costFuncRadioBox);
	    gray_out_item(ip->costParamForm, !irbf_params.costFunc);
	    if(irbf_params.costFunc){
		cost_set_cost_func(CFOM);
	    } else {
		cost_set_cost_func(SQ_ERR);
	    }
	}

    } else if (!strcmp(client, "costParamText")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	costParam_buttons_mapped =
	  okTextValueChanged(ip->costParamText, ip->costParamOKPBtn,
			     ip->costParamCancelPBtn,
			     ip->irbfDialogShell,
			     costParam_buttons_mapped,
			     irbf_params.costParam_string);
	
    } else if ((!strcmp(client, "costParamTextAct")) ||
	       (!strcmp(client, "costParamOKPBtn"))) {
	String costParamTextval;

	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	costParam_buttons_mapped =
	  okTextActivate(ip->costParamText, ip->costParamOKPBtn,
			 ip->costParamCancelPBtn, &costParamTextval);
	/* I won't actually be using this copy */
	XtFree (costParamTextval);
	
	/* verify the string as a float */
	irbf_params.costParam =
	  set_atof_value(ip->costParamText, "costParam_set",
			 irbf_params.costParam);

	/* save the current text for future comparisons */
	XtFree(irbf_params.costParam_string);
	irbf_params.costParam_string = get_string_text(ip->costParamText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "costParamCancelPBtn")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	costParam_buttons_mapped = 
	  okTextCancel(ip->costParamText, ip->costParamOKPBtn,
		       ip->costParamCancelPBtn,
		       irbf_params.costParam_string);
	
    } else if (!strcmp(client, "biasToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "irbf_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	irbf_params.bias = state->set;
	set_plot_labels();
 
    } else if (!strcmp(client, "maxMagText")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	maxMag_buttons_mapped =
	  okTextValueChanged(ip->maxMagText, ip->maxMagOKPBtn,
			     ip->maxMagCancelPBtn,
			     ip->irbfDialogShell,
			     maxMag_buttons_mapped,
			     irbf_params.maxMag_string);
	
    } else if ((!strcmp(client, "maxMagTextAct")) ||
	       (!strcmp(client, "maxMagOKPBtn"))) {
	String maxMagTextval;

	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	maxMag_buttons_mapped =
	  okTextActivate(ip->maxMagText, ip->maxMagOKPBtn,
			 ip->maxMagCancelPBtn, &maxMagTextval);
	/* I won't actually be using this copy */
	XtFree (maxMagTextval);
	
	/* verify the string as a float */
	irbf_params.maxMag =
	  set_atof_low_value(ip->maxMagText, "maxMag_set",
			     irbf_params.maxMag, 0.);

	/* save the current text for future comparisons */
	XtFree(irbf_params.maxMag_string);
	irbf_params.maxMag_string = get_string_text(ip->maxMagText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "maxMagCancelPBtn")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	maxMag_buttons_mapped = 
	  okTextCancel(ip->maxMagText, ip->maxMagOKPBtn,
		       ip->maxMagCancelPBtn,
		       irbf_params.maxMag_string);
 
    } else if (!strcmp(client, "minVarText")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	minVar_buttons_mapped =
	  okTextValueChanged(ip->minVarText, ip->minVarOKPBtn,
			     ip->minVarCancelPBtn,
			     ip->irbfDialogShell,
			     minVar_buttons_mapped,
			     irbf_params.minVar_string);
	
    } else if ((!strcmp(client, "minVarTextAct")) ||
	       (!strcmp(client, "minVarOKPBtn"))) {
	String minVarTextval;

	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	minVar_buttons_mapped =
	  okTextActivate(ip->minVarText, ip->minVarOKPBtn,
			 ip->minVarCancelPBtn, &minVarTextval);
	/* I won't actually be using this copy */
	XtFree (minVarTextval);
	
	/* verify the string as a float */
	irbf_params.minVar =
	  set_atof_low_value(ip->minVarText, "minVar_set",
			     irbf_params.minVar, 0.);

	/* save the current text for future comparisons */
	XtFree(irbf_params.minVar_string);
	irbf_params.minVar_string = get_string_text(ip->minVarText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "minVarCancelPBtn")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	minVar_buttons_mapped = 
	  okTextCancel(ip->minVarText, ip->minVarOKPBtn,
		       ip->minVarCancelPBtn,
		       irbf_params.minVar_string);
 
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "irbf_callback: %s\n", client );*/
	close_popup(ip->irbfForm);
 
    } else {
	fprintf(stderr, "irbf_callback: unknown client: %s\n", (char *)client);
    }
}

void
irbf_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(IrbfClone->irbfForm);
    } else if (!strcmp(request, "init")){
	irbf_params.createClusters = 0;
	irbf_params.clusterNum = KMEANS;
	irbf_params.clustName = "kmeans";
	irbf_params.clustFname = NULL;
	irbf_params.nEpochs = 0;
	irbf_params.nEpochs_string = NULL;
	irbf_params.hSpread = 0.;
	irbf_params.hSpread_string = NULL;
	irbf_params.extraHspread = 0.;
	irbf_params.extraHspread_string = NULL;
	irbf_params.weightEtta = 0.;
	irbf_params.weightEtta_string = NULL;
	irbf_params.meanEtta = 0.;
	irbf_params.meanEtta_string = NULL;
	irbf_params.varEtta = 0.;
	irbf_params.varEtta_string = NULL;
	irbf_params.grandVar = 0;
	irbf_params.costFunc = 0;
	irbf_params.bias = 0;
	irbf_params.costParam = 0.;
	irbf_params.costParam_string = NULL;
	irbf_params.maxMag = 0.;
	irbf_params.maxMag_string = NULL;
	irbf_params.minVar = 0.;
	irbf_params.minVar_string = NULL;
    } else if (!strcmp(request, "display")){
	char string[10];
	show_toggle_value(IrbfClone->createClustersToggle,
			  irbf_params.createClusters);
	switch(irbf_params.clusterNum){
	  case KMEANS:
	    show_option_value(IrbfClone->clustAlgOptionMenu, 0);
	    break;
	  case EM_CLUS:
	    show_option_value(IrbfClone->clustAlgOptionMenu, 1);
	    break;
	  case LEAD_CLUS:
	    show_option_value(IrbfClone->clustAlgOptionMenu, 2);
	    break;
	  case RAN_CLUS:
	    show_option_value(IrbfClone->clustAlgOptionMenu, 3);
	    break;
	}
	show_string_label(IrbfClone->clustFnameLabel, irbf_params.clustFname);

	XtFree(irbf_params.nEpochs_string);
	sprintf(string, "%d", irbf_params.nEpochs);
	irbf_params.nEpochs_string = XtNewString(string);
	show_string_text(IrbfClone->nEpochsText,
			 irbf_params.nEpochs_string);

	XtFree(irbf_params.hSpread_string);
	sprintf(string, "%5.2g", irbf_params.hSpread);
	irbf_params.hSpread_string = XtNewString(string);
	show_string_text(IrbfClone->hSpreadText,
			 irbf_params.hSpread_string);
	XtFree(irbf_params.extraHspread_string);
	sprintf(string, "%5.2g", irbf_params.extraHspread);
	irbf_params.extraHspread_string = XtNewString(string);
	show_string_text(IrbfClone->extraHspreadText,
			 irbf_params.extraHspread_string);


	XtFree(irbf_params.weightEtta_string);
	sprintf(string, "%5.2g", irbf_params.weightEtta);
	irbf_params.weightEtta_string = XtNewString(string);
	show_string_text(IrbfClone->weightEttaText,
			 irbf_params.weightEtta_string);
	XtFree(irbf_params.meanEtta_string);
	sprintf(string, "%5.2g", irbf_params.meanEtta);
	irbf_params.meanEtta_string = XtNewString(string);
	show_string_text(IrbfClone->meanEttaText,
			 irbf_params.meanEtta_string);
	XtFree(irbf_params.varEtta_string);
	sprintf(string, "%5.2g", irbf_params.varEtta);
	irbf_params.varEtta_string = XtNewString(string);
	show_string_text(IrbfClone->varEttaText,
			 irbf_params.varEtta_string);

	show_radio_value(IrbfClone->grandVarRadioBox,
			 irbf_params.grandVar);

	show_radio_value(IrbfClone->costFuncRadioBox,
			 irbf_params.costFunc);
	gray_out_item(IrbfClone->costParamForm, !irbf_params.costFunc);
	XtFree(irbf_params.costParam_string);
	sprintf(string, "%5.2g", irbf_params.costParam);
	irbf_params.costParam_string = XtNewString(string);
	show_string_text(IrbfClone->costParamText,
			 irbf_params.costParam_string);

	show_toggle_value(IrbfClone->biasToggle, irbf_params.bias);

	XtFree(irbf_params.maxMag_string);
	sprintf(string, "%5.2g", irbf_params.maxMag);
	irbf_params.maxMag_string = XtNewString(string);
	show_string_text(IrbfClone->maxMagText,
			 irbf_params.maxMag_string);
	XtFree(irbf_params.minVar_string);
	sprintf(string, "%5.2g", irbf_params.minVar);
	irbf_params.minVar_string = XtNewString(string);
	show_string_text(IrbfClone->minVarText,
			 irbf_params.minVar_string);
    } else {
	fprintf(stderr, "irbf_request: unknown request: %s\n", request);
    }
}

String
irbf_get_plot_label(String string)
{
    char numString[10];
    string = StringConcat(string, "H:");
    sprintf(numString, "%g", irbf_params.hSpread);
    string = StringConcat(string, numString);
    string = StringConcat(string, " Extra H:");
    sprintf(numString, "%g", irbf_params.extraHspread);
    string = StringConcat(string, numString);
    if(irbf_params.bias) string = StringConcat(string, " Bias");
    string = StringConcat(string, " Weight step:");
    sprintf(numString, "%.2g", irbf_params.weightEtta);
    string = StringConcat(string, numString);
    string = StringConcat(string, " Epochs:");
    sprintf(numString, "%d", irbf_params.nEpochs);
    string = StringConcat(string, numString);
    
    switch(irbf_params.clusterNum){
      case KMEANS:
	string = kmeans_get_plot_label(string);
	break;
      case EM_CLUS:
	string = em_clus_get_plot_label(string);
	break;
      case LEAD_CLUS:
	string = lead_clus_get_plot_label(string);
	break;
      case RAN_CLUS:
	string = ran_clus_get_plot_label(string);
	break;
    }
    return string;
}

static void
irbf_set_read_variables()
{
    XtFree(irbf_params.clustFname);
    irbf_params.clustFname = XtNewString(cluster_param_fname);
    irbf_params.nEpochs = nepochs;
    irbf_params.hSpread = hspread;
    irbf_params.extraHspread = extra_hspread;
    irbf_params.weightEtta = weight_eta;
    irbf_params.meanEtta = mean_eta;
    irbf_params.varEtta = var_eta;
    irbf_params.grandVar = grand_flag;
    if(cost_func == CFOM)
      irbf_params.costFunc = 1;
    else
      irbf_params.costFunc = 0;
    irbf_params.costParam = cost_param;
    irbf_params.bias = bias;
    irbf_params.maxMag = max_magnitude;
    irbf_params.minVar = minimum_variance;
}

static void
irbf_set_write_variables()
{
    cluster_param_fname = irbf_params.clustFname;
    nepochs = irbf_params.nEpochs;
    hspread = irbf_params.hSpread;
    extra_hspread = irbf_params.extraHspread;
    weight_eta = irbf_params.weightEtta;
    mean_eta = irbf_params.meanEtta;
    var_eta = irbf_params.varEtta;
    grand_flag = irbf_params.grandVar;
    cost_func = (irbf_params.costFunc == 0) ? SQ_ERR : CFOM;
    cost_param = irbf_params.costParam;
    bias = irbf_params.bias;
    max_magnitude = irbf_params.maxMag;
    minimum_variance = irbf_params.minVar;
}
static ARG irbftab[] =
{
    {"createClusters", BOOLEAN, (int *)NULL,},
    {"clusterNum", INT, (int *)NULL,},
};
#define TABSIZE (sizeof(irbftab)/sizeof(ARG))

static void
irbf_set_defaults_variables(dp)
     IrbfParams *dp;
{
    nnargs_set_variable("createClusters", (int *)&dp->createClusters, irbftab, TABSIZE);
    nnargs_set_variable("clusterNum", (int *)&dp->clusterNum, irbftab, TABSIZE);
}

void
read_irbf_defaults(argc, argv)
     int argc;
     char **argv;
{
    irbf_params.createClusters = 0;
    bias = 0;
    grand_flag = 0;
    irbf_set_defaults_variables(&irbf_params);
    if(argc > 0){
	argc = nnargs_part(argc, argv, irbftab, TABSIZE);
	argc = nnargs(argc, argv, irbf_tab, IRBF_TABSIZE);
	irbf_set_read_variables();
	switch(irbf_params.clusterNum){
	  case KMEANS:
	    irbf_params.clustName = "kmeans";
	    break;
	  case EM_CLUS:
	    irbf_params.clustName = "em_clus";
	    break;
	  case LEAD_CLUS:
	    irbf_params.clustName = "lead_clus";
	    break;
	  case RAN_CLUS:
	    irbf_params.clustName = "ran_clus";
	    break;
	}
    } else {
	/* set any defaults */
	irbf_params.createClusters = 1;
	irbf_params.clusterNum = KMEANS;
	irbf_params.nEpochs = 10;
	irbf_params.hSpread = 1;
	irbf_params.extraHspread = 1;
	irbf_params.weightEtta = 0.1;
	irbf_params.meanEtta = 0.01;
	irbf_params.varEtta = 0.01;
	irbf_params.grandVar = 0;
	irbf_params.costFunc = 0;
	irbf_params.costParam = 1.;
	irbf_params.maxMag = 0.01;
	irbf_params.minVar = 1e-6;
    }

    irbf_request("display");
}

void
write_irbf_defaults(FILE *default_file)
{
    irbf_set_write_variables();
    irbf_set_defaults_variables(&irbf_params);

    if(default_file != NULL){
	fprintf(default_file, "irbf %s",
		nnargs_get_part_flags(irbftab, TABSIZE));
	fprintf(default_file, "irbf %s",
		nnargs_get_flags(irbf_tab, IRBF_TABSIZE));
    }
}

String
irbf_get_args ()
{
    irbf_set_write_variables();
    return (nnargs_get_part_flags(irbf_tab, IRBF_TABSIZE));
}

String
irbf_get_important_args ()
{
    irbf_set_write_variables();
    return (nnargs_get_flags_one_line(irbf_tab, IRBF_TABSIZE));
}

static void
irbf_set_clustering_alg(value)
     int value;
{
    irbf_params.clusterNum = value;
    switch(irbf_params.clusterNum){
      case KMEANS:
	irbf_params.clustName = "kmeans";
	break;
      case EM_CLUS:
	irbf_params.clustName = "em_clus";
	break;
      case LEAD_CLUS:
	irbf_params.clustName = "lead_clus";
	break;
      case RAN_CLUS:
	irbf_params.clustName = "ran_clus";
	break;
    }
    XtFree(irbf_params.clustFname);
    irbf_params.clustFname = XtNewString(lnk_params.exp_prefix);
    irbf_params.clustFname = StringConcat(irbf_params.clustFname,
					 irbf_params.clustName);
    irbf_params.clustFname = StringConcat(irbf_params.clustFname,
					 ".param");
    show_string_label(IrbfClone->clustFnameLabel, irbf_params.clustFname);
    set_plot_labels();
}

void
irbf_set_cluster_param_fname()
{
    if(irbf_params.createClusters){
	XtFree(irbf_params.clustFname);
	irbf_params.clustFname = XtNewString(lnk_params.exp_prefix);
	irbf_params.clustFname = StringConcat(irbf_params.clustFname,
					     irbf_params.clustName);
	irbf_params.clustFname = StringConcat(irbf_params.clustFname,
					     ".param");
	show_string_label(IrbfClone->clustFnameLabel, irbf_params.clustFname);
    }
}

void
irbf_store_plot_command(FILE * shell_file)
{
    fprintf(shell_file, " -exhspread %g", irbf_params.extraHspread);
}

char *
irbf_get_cluster_fname(){
    return irbf_params.clustFname;
}

int
irbf_get_cluster_alg(){
    return irbf_params.clusterNum;
}

int
irbf_must_store_clustering(){
    return irbf_params.createClusters;
}

int
irbf_get_epochs(){
    return irbf_params.nEpochs;
}

void irbf_set_epochs(int e)
{
    irbf_params.nEpochs = e;
}

int
irbf_get_cost_func(){
    if(irbf_params.costFunc == 0)
      return SQ_ERR;
    else
      return CFOM;
}
