/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/kmeans/kmeans_callback.c,v 11.4 2001/08/23 20:16:43 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            KMEANS                              ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  5-Oct-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <Xm/Xm.h>
#include <nnlib.h>
#include <kmeans_args.h>
#include "kmeans.h"
/*
 * External Functions
 */
#include "../main/lnknet_subs.h"
#include "../plots/plots_func.h"
#include "kmeans_func.h"
 
/*
 * Externals
 */
extern KmeansWidgets *KmeansClone;
#include "kmeans_params.h"
KmeansParams kmeans_params; /* this is the real one */
#include "../data/data_params.h"
extern DataParams data_params;

/*
 * Statics
 */
static void kmeans_show_ncenters_choices();

/******************************************************************************
 ******************************************************************************
 *
 * Module Name: kmeans_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   kmeans_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  5-Oct-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
kmeans_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    KmeansWidgets *ip;
    static int ncenters_buttons_mapped = False;
    static int ncentersList_buttons_mapped = False;
    static int reduceStep_buttons_mapped = False;
    static int stopPercentage_buttons_mapped = False;
    static int maxIteration_buttons_mapped = False;
    static int splitPercentage_buttons_mapped = False;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "ignoreClassToggle") || 
	!strcmp(client, "clusterByClassToggle")) {
	/*fprintf (stderr, "kmeans_callback: %s\n", client );*/
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	if(state->set){
	    kmeans_params.cluster_by_class =
	      get_radio_value(ip->clusterByClassRadioBox);
	    kmeans_show_ncenters_choices();
	    kmeans_set_total_ncenters();
	    set_plot_labels();
	}

    } else if (!strcmp(client, "equalNcentersToggle") ||
	       !strcmp(client, "ncentersByClassToggle")) {
	/*fprintf (stderr, "kmeans_callback: %s\n", client );*/
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	if(state->set){
	    kmeans_params.specify_by_class =
	      get_radio_value(ip->ncentersByClassRadioBox);
	    kmeans_show_ncenters_choices();
	    kmeans_set_total_ncenters();
	    set_plot_labels();
	}
	
    } else if (!strcmp(client, "ncentersText")) {
	/*fprintf (stderr, "kmeans_callback: %s\n", client );*/
	ncenters_buttons_mapped =
	  okTextValueChanged(ip->ncentersText, ip->ncentersOKPBtn,
			     ip->ncentersCancelPBtn,
			     ip->kmeansDialogShell,
			     ncenters_buttons_mapped,
			     kmeans_params.ncenters_string);
	
    } else if ((!strcmp(client, "ncentersTextAct")) ||
	       (!strcmp(client, "ncentersOKPBtn"))) {
	String ncentersTextval;

      /*fprintf (stderr, "kmeans_callback: %s\n", client );*/
	ncenters_buttons_mapped =
	  okTextActivate(ip->ncentersText, ip->ncentersOKPBtn,
			 ip->ncentersCancelPBtn, &ncentersTextval);
	/* I won't actually be using this copy */
	XtFree (ncentersTextval);
	
	/* verify the string as an integer */
	kmeans_params.ncenters =
	  set_atoi_value(ip->ncentersText, "ncenters_set",
			     kmeans_params.ncenters);

	/* save the current text for future comparisons */
	XtFree(kmeans_params.ncenters_string);
	kmeans_params.ncenters_string = get_string_text(ip->ncentersText);
 
	/* do any other things needed */
	kmeans_set_total_ncenters();
	set_plot_labels();
  } else if (!strcmp(client, "ncentersCancelPBtn")) {
    /*fprintf (stderr, "kmeans_callback: %s\n", client );*/
	ncenters_buttons_mapped = 
	  okTextCancel(ip->ncentersText, ip->ncentersOKPBtn,
		       ip->ncentersCancelPBtn,
		       kmeans_params.ncenters_string);
	
    } else if (!strcmp(client, "ncentersListText")) {
	/*fprintf (stderr, "kmeans_callback: %s\n", client);*/
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	ncentersList_buttons_mapped =
	  okTextValueChanged(ip->ncentersListText, ip->ncentersListOKPBtn,
			     ip->ncentersListCancelPBtn,
			     ip->kmeansDialogShell,
			     ncentersList_buttons_mapped,
			     kmeans_params.ncentersList_string);
    } else if ((!strcmp(client, "ncentersListTextAct")) ||
	       (!strcmp(client, "ncentersListOKPBtn"))) {
	/*fprintf (stderr, "kmeans_callback: %s\n", client );*/
	String ncentersListTextval;
	ncentersList_buttons_mapped =
	  okTextActivate(ip->ncentersListText,
			 ip->ncentersListOKPBtn,
			 ip->ncentersListCancelPBtn,
			 &ncentersListTextval);
	
	if(kmeans_params.ncentersList != NULL)
	  deallocate_ivector(kmeans_params.ncentersList);
	kmeans_params.ncentersList = NULL;
	/* take spaces and tabs out of the list before handing it to nnargs */
	if(ncentersListTextval != NULL){
	    char *ncentersListTemp = ncentersListTextval;
	    ncentersListTextval = strtok(ncentersListTextval, " \t");
	    if(ncentersListTextval != NULL){
		kmeans_params.ncentersList =
		  nnargs_get_int_alloc_list(ncentersListTextval);
		XtFree(ncentersListTemp);
	    }
	}
	
	/* make a copy of the string as it has been interpretted */
	XtFree(kmeans_params.ncentersList_string);
	kmeans_params.ncentersList_string =
	  make_int_string(kmeans_params.ncentersList);
	show_string_text(ip->ncentersListText,
			 kmeans_params.ncentersList_string);
	
	/* do anything else necessary */
	kmeans_set_total_ncenters();
	set_plot_labels();
	
    } else if (!strcmp(client, "ncentersListCancelPBtn")) {
	/*fprintf (stderr, "kmeans_callback: %s\n", client );*/
	ncentersList_buttons_mapped = 
	  okTextCancel(ip->ncentersListText, ip->ncentersListOKPBtn,
		       ip->ncentersListCancelPBtn,
		       kmeans_params.ncentersList_string);
	
    } else if (!strcmp(client, "reduceStepText")) {
	/*fprintf (stderr, "kmeans_callback: %s\n", client );*/
	reduceStep_buttons_mapped =
	  okTextValueChanged(ip->reduceStepText, ip->reduceStepOKPBtn,
			     ip->reduceStepCancelPBtn,
			     ip->kmeansDialogShell,
			     reduceStep_buttons_mapped,
			     kmeans_params.reduceStep_string);
	
    } else if ((!strcmp(client, "reduceStepTextAct")) ||
	       (!strcmp(client, "reduceStepOKPBtn"))) {
	String reduceStepTextval;

	/*fprintf (stderr, "kmeans_callback: %s\n", client );*/
	reduceStep_buttons_mapped =
	  okTextActivate(ip->reduceStepText, ip->reduceStepOKPBtn,
			 ip->reduceStepCancelPBtn, &reduceStepTextval);
	/* I won't actually be using this copy */
	XtFree (reduceStepTextval);
	
	/* verify the string as an integer */
	kmeans_params.reduceStep =
	  set_atoi_value(ip->reduceStepText, "reduceStep_set",
			     kmeans_params.reduceStep);

	/* save the current text for future comparisons */
	XtFree(kmeans_params.reduceStep_string);
	kmeans_params.reduceStep_string = get_string_text(ip->reduceStepText);
 
	/* do any other things needed */
	kmeans_set_total_ncenters();
	set_plot_labels();

    } else if (!strcmp(client, "reduceStepCancelPBtn")) {
	/*fprintf (stderr, "kmeans_callback: %s\n", client );*/
	reduceStep_buttons_mapped = 
	  okTextCancel(ip->reduceStepText, ip->reduceStepOKPBtn,
		       ip->reduceStepCancelPBtn,
		       kmeans_params.reduceStep_string);

    } else if (!strcmp(client, "stopPercentageText")) {
	/*fprintf (stderr, "kmeans_callback: %s\n", client );*/
	stopPercentage_buttons_mapped =
	  okTextValueChanged(ip->stopPercentageText, ip->stopPercentageOKPBtn,
			     ip->stopPercentageCancelPBtn,
			     ip->kmeansDialogShell,
			     stopPercentage_buttons_mapped,
			     kmeans_params.stopPercentage_string);
	
    } else if ((!strcmp(client, "stopPercentageTextAct")) ||
	       (!strcmp(client, "stopPercentageOKPBtn"))) {
	String stopPercentageTextval;

	/*fprintf (stderr, "kmeans_callback: %s\n", client );*/
	stopPercentage_buttons_mapped =
	  okTextActivate(ip->stopPercentageText, ip->stopPercentageOKPBtn,
			 ip->stopPercentageCancelPBtn, &stopPercentageTextval);
	/* I won't actually be using this copy */
	XtFree (stopPercentageTextval);
	
	/* verify the string as a float */
	kmeans_params.stopPercentage =
	  set_atof_low_value(ip->stopPercentageText, "stopPercentage_set",
			     kmeans_params.stopPercentage, 0.);

	/* save the current text for future comparisons */
	XtFree(kmeans_params.stopPercentage_string);
	kmeans_params.stopPercentage_string = get_string_text(ip->stopPercentageText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "stopPercentageCancelPBtn")) {
	/*fprintf (stderr, "kmeans_callback: %s\n", client );*/
	stopPercentage_buttons_mapped = 
	  okTextCancel(ip->stopPercentageText, ip->stopPercentageOKPBtn,
		       ip->stopPercentageCancelPBtn,
		       kmeans_params.stopPercentage_string);
	
    } else if (!strcmp(client, "maxIterationText")) {
	/*fprintf (stderr, "kmeans_callback: %s\n", client );*/
	maxIteration_buttons_mapped =
	  okTextValueChanged(ip->maxIterationText, ip->maxIterationOKPBtn,
			     ip->maxIterationCancelPBtn,
			     ip->kmeansDialogShell,
			     maxIteration_buttons_mapped,
			     kmeans_params.maxIteration_string);
	
    } else if ((!strcmp(client, "maxIterationTextAct")) ||
	       (!strcmp(client, "maxIterationOKPBtn"))) {
	String maxIterationTextval;

	/*fprintf (stderr, "kmeans_callback: %s\n", client );*/
	maxIteration_buttons_mapped =
	  okTextActivate(ip->maxIterationText, ip->maxIterationOKPBtn,
			 ip->maxIterationCancelPBtn, &maxIterationTextval);
	/* I won't actually be using this copy */
	XtFree (maxIterationTextval);
	
	/* verify the string as an integer */
	kmeans_params.maxIteration =
	  set_atoi_value(ip->maxIterationText, "maxIteration_set",
			     kmeans_params.maxIteration);

	/* save the current text for future comparisons */
	XtFree(kmeans_params.maxIteration_string);
	kmeans_params.maxIteration_string = get_string_text(ip->maxIterationText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "maxIterationCancelPBtn")) {
	/*fprintf (stderr, "kmeans_callback: %s\n", client );*/
	maxIteration_buttons_mapped = 
	  okTextCancel(ip->maxIterationText, ip->maxIterationOKPBtn,
		       ip->maxIterationCancelPBtn,
		       kmeans_params.maxIteration_string);

    } else if (!strcmp(client, "splitPercentageText")) {
	/*fprintf (stderr, "kmeans_callback: %s\n", client );*/
	splitPercentage_buttons_mapped =
	  okTextValueChanged(ip->splitPercentageText, ip->splitPercentageOKPBtn,
			     ip->splitPercentageCancelPBtn,
			     ip->kmeansDialogShell,
			     splitPercentage_buttons_mapped,
			     kmeans_params.splitPercentage_string);
	
    } else if ((!strcmp(client, "splitPercentageTextAct")) ||
	       (!strcmp(client, "splitPercentageOKPBtn"))) {
	String splitPercentageTextval;

	/*fprintf (stderr, "kmeans_callback: %s\n", client );*/
	splitPercentage_buttons_mapped =
	  okTextActivate(ip->splitPercentageText, ip->splitPercentageOKPBtn,
			 ip->splitPercentageCancelPBtn, &splitPercentageTextval);
	/* I won't actually be using this copy */
	XtFree (splitPercentageTextval);
	
	/* verify the string as a float */
	kmeans_params.splitPercentage =
	  set_atof_low_value(ip->splitPercentageText, "splitPercentage_set",
			     kmeans_params.splitPercentage, 0.);

	/* save the current text for future comparisons */
	XtFree(kmeans_params.splitPercentage_string);
	kmeans_params.splitPercentage_string = get_string_text(ip->splitPercentageText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "splitPercentageCancelPBtn")) {
	/*fprintf (stderr, "kmeans_callback: %s\n", client );*/
	splitPercentage_buttons_mapped = 
	  okTextCancel(ip->splitPercentageText, ip->splitPercentageOKPBtn,
		       ip->splitPercentageCancelPBtn,
		       kmeans_params.splitPercentage_string);
	
    } else if (!strcmp(client, "addRandomOffsetToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "kmeans_callback: %s %s\n", client,
	  (state->set) ? "on", "off" );*/
	kmeans_params.addRandomOffset = state->set;
	
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "kmeans_callback: %s\n", client );*/
	close_popup(KmeansClone->kmeansForm);
      
    } else {
	fprintf(stderr, "kmeans_callback: unknown client: %s\n", (char *)client);
    }
}

void
kmeans_request(String request)
{
    if (!strcmp(request, "open")) {
	open_popup(KmeansClone->kmeansForm);
    } else if (!strcmp(request, "init")){
	/* this should only get called once, before ANYTHING 
	   gets allocated or there will be memory leaks */
	kmeans_params.cluster_by_class = False;
	kmeans_params.specify_by_class = False;
	kmeans_params.ncenters = 0;
	kmeans_params.ncenters_string = NULL;
	kmeans_params.ncentersList = NULL;
	kmeans_params.ncentersList_string = NULL;
	kmeans_params.total_ncenters = 0;
	kmeans_params.reduceStep = 0;
	kmeans_params.reduceStep_string = 0;
	kmeans_params.stopPercentage = 0.;
	kmeans_params.stopPercentage_string = NULL;
	kmeans_params.maxIteration = 0;
	kmeans_params.maxIteration_string = NULL;
	kmeans_params.splitPercentage = 0.;
	kmeans_params.splitPercentage_string = NULL;
	kmeans_params.addRandomOffset = False;
    } else if (!strcmp(request, "display")){
	char string[10]; /* for creating int and float text */
	show_radio_value(KmeansClone->clusterByClassRadioBox,
			 kmeans_params.cluster_by_class);
	show_radio_value(KmeansClone->ncentersByClassRadioBox,
			 kmeans_params.specify_by_class);

	sprintf(string, "%d", kmeans_params.ncenters);
	XtFree(kmeans_params.ncenters_string);
	kmeans_params.ncenters_string = XtNewString(string);
	show_string_text(KmeansClone->ncentersText,
			 kmeans_params.ncenters_string);

	XtFree(kmeans_params.ncentersList_string);
	kmeans_params.ncentersList_string =
	  make_int_string(kmeans_params.ncentersList);
	show_string_text(KmeansClone->ncentersListText,
			 kmeans_params.ncentersList_string);

	kmeans_show_ncenters_choices();
	kmeans_set_total_ncenters();

	sprintf(string, "%d", kmeans_params.reduceStep);
	XtFree(kmeans_params.reduceStep_string);
	kmeans_params.reduceStep_string = XtNewString(string);
	show_string_text(KmeansClone->reduceStepText,
			 kmeans_params.reduceStep_string);

	sprintf(string, "%5.2g", kmeans_params.stopPercentage);
	XtFree(kmeans_params.stopPercentage_string);
	kmeans_params.stopPercentage_string = XtNewString(string);
	show_string_text(KmeansClone->stopPercentageText,
			 kmeans_params.stopPercentage_string);

	sprintf(string, "%d", kmeans_params.maxIteration);
	XtFree(kmeans_params.maxIteration_string);
	kmeans_params.maxIteration_string = XtNewString(string);
	show_string_text(KmeansClone->maxIterationText,
			 kmeans_params.maxIteration_string);

	sprintf(string, "%5.2g", kmeans_params.splitPercentage);
	XtFree(kmeans_params.splitPercentage_string);
	kmeans_params.splitPercentage_string = XtNewString(string);
	show_string_text(KmeansClone->splitPercentageText,
			 kmeans_params.splitPercentage_string);

	show_toggle_value(KmeansClone->addRandomOffsetToggle,
			  kmeans_params.addRandomOffset);
    } else {
	fprintf (stderr, "kmeans_request: unknown request: %s\n", request);
    }
}

String
kmeans_get_plot_label(String string)
{
    char intchars[10];
    sprintf(intchars, "%d", kmeans_params.total_ncenters);
    string = StringConcat(string, " Centers:");
    string = StringConcat(string, intchars);
    return string;
}

static ARG kmeans_tab2[] =
{
    {"specify_by_class", BOOLEAN, (int *)NULL,
       "use ncenters list for nclusters by class"},
};
#define KMEANS_TABSIZE2 (sizeof(kmeans_tab2)/sizeof(ARG))

static void
kmeans_set_read_variables()
{
    kmeans_params.cluster_by_class = cluster_by_class;
    kmeans_params.ncenters = ncenters;
    if(kmeans_params.ncentersList != NULL){
	deallocate_ivector(kmeans_params.ncentersList);
    }
    kmeans_params.ncentersList = ncenters_list;
    ncenters_list = NULL;
    kmeans_params.splitPercentage = split_percentage;
    kmeans_params.addRandomOffset = add_random_offset;
    kmeans_params.maxIteration = max_iteration;
    kmeans_params.stopPercentage = stop_percentage;
    kmeans_params.reduceStep = reduce_step;
}

static void
kmeans_set_write_variables()
{
    cluster_by_class = kmeans_params.cluster_by_class;
    ncenters = kmeans_params.ncenters;
    ncenters_list = kmeans_params.ncentersList;
    split_percentage = kmeans_params.splitPercentage;
    add_random_offset = kmeans_params.addRandomOffset;
    max_iteration = kmeans_params.maxIteration;
    stop_percentage = kmeans_params.stopPercentage;
    reduce_step = kmeans_params.reduceStep;
}

static void
kmeans_set_defaults_variables()
{
    nnargs_set_variable("specify_by_class", &kmeans_params.specify_by_class,
			kmeans_tab2, KMEANS_TABSIZE2);
}
void
read_kmeans_defaults(int argc, char **argv)
{
    cluster_by_class = 0;
    add_random_offset = 0;
    ncenters_list = NULL;/* the space for this list is always
			    owned by the kmeans_params structure */
    kmeans_params.specify_by_class = 0;

    kmeans_set_defaults_variables();
    if(argc > 0){
	argc = nnargs_part(argc, argv, kmeans_tab2, KMEANS_TABSIZE2);
	argc = nnargs(argc, argv, kmeans_tab, KMEANS_TABSIZE);
	kmeans_set_read_variables();
    } else {
	kmeans_params.addRandomOffset = True;
	kmeans_params.reduceStep = 1;
	kmeans_params.maxIteration = 10;
	kmeans_params.ncenters = 16;
	kmeans_params.splitPercentage = 1.;
	kmeans_params.stopPercentage = 1.;
	kmeans_params.specify_by_class = 0;
    }
    kmeans_request("display");
}

void
write_kmeans_defaults(default_file)
     FILE *default_file;
{
    kmeans_set_write_variables();
    kmeans_set_defaults_variables();
    if(default_file != NULL){
	fprintf(default_file, "kmeans %s",
		nnargs_get_part_flags(kmeans_tab, KMEANS_TABSIZE));
	fprintf(default_file, "%s",
		nnargs_get_flags(kmeans_tab2, KMEANS_TABSIZE2));
    }
}

String
kmeans_get_args ()
{
    kmeans_set_write_variables();
    /* fix up ncenters list, in case it isn't wanted */
    if(kmeans_params.cluster_by_class && kmeans_params.specify_by_class){
	ncenters = 0;
    } else {
	ncenters_list = NULL;
    }
    return (nnargs_get_part_flags(kmeans_tab, KMEANS_TABSIZE));
}

String
kmeans_get_important_args ()
{
    kmeans_set_write_variables();
    /* fix up ncenters list, in case it isn't wanted */
    if(kmeans_params.specify_by_class){
	ncenters = 0;
    } else {
	ncenters_list = NULL;
    }
    return (nnargs_get_flags_one_line(kmeans_tab, KMEANS_TABSIZE));
}

void
kmeans_set_total_ncenters()
{
    static char total_label[64];

    if(kmeans_params.cluster_by_class){
	if(kmeans_params.specify_by_class &&
	   kmeans_params.ncentersList != NULL){
	    int i;
	    for(i = 1, kmeans_params.total_ncenters = 0;
		i <= kmeans_params.ncentersList[0]; i++){
		kmeans_params.total_ncenters += kmeans_params.ncentersList[i];
	    }
	} else {
	    kmeans_params.total_ncenters =
	      data_params.noutputs * kmeans_params.ncenters;
	}
    } else {
	kmeans_params.total_ncenters = kmeans_params.ncenters;
    }
    sprintf(total_label, "This will create a total of %d centers.",
	    kmeans_params.total_ncenters);
    show_string_label(KmeansClone->totalNcentersLabel, total_label);
}

static void
kmeans_show_ncenters_choices()
{
    if(kmeans_params.cluster_by_class){
	gray_out_item(KmeansClone->ncentersByClassRadioBox, FALSE);
	if(kmeans_params.specify_by_class){
	    gray_out_item(KmeansClone->ncentersListForm, FALSE);
	    gray_out_item(KmeansClone->ncentersForm, TRUE);
	} else {
	    gray_out_item(KmeansClone->ncentersListForm, TRUE);
	    gray_out_item(KmeansClone->ncentersForm, FALSE);
	}
    } else {
	gray_out_item(KmeansClone->ncentersByClassRadioBox, TRUE);
	gray_out_item(KmeansClone->ncentersListForm, TRUE);
	gray_out_item(KmeansClone->ncentersForm, FALSE);
    }
}
