/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/knn/knn_callback.c,v 11.5 2001/08/23 20:16:46 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            KNN                              ****
 ****                          Knn Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  30-Oct-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include "../main/lnknet_subs.h"
#include "knn.h"
#include <nnlib.h>
#include <knn_args.h>
/*
 * External Functions
 */
#include "../plots/plots_func.h"

/*
 * Externals
 */
extern KnnWidgets *KnnClone;
#include "knn_params.h"
KnnParams knn_params; /* this is the real one */
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: knn_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   knn_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  30-Oct-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
knn_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    KnnWidgets *ip;
    static int k_buttons_mapped = False;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
    
    if (!strcmp(client, "leave1OutToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "knn_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	knn_params.leave1Out = state->set;
      
    } else if (!strcmp(client, "kText")) {
	/*fprintf (stderr, "knn_callback: %s\n", client );*/
	k_buttons_mapped =
	  okTextValueChanged(ip->kText, ip->kOKPBtn,
			     ip->kCancelPBtn,
			     ip->knnDialogShell,
			     k_buttons_mapped,
			     knn_params.k_string);
	
    } else if ((!strcmp(client, "kTextAct")) ||
	       (!strcmp(client, "kOKPBtn"))) {
	String kTextval;

	/*fprintf (stderr, "knn_callback: %s\n", client );*/
	k_buttons_mapped =
	  okTextActivate(ip->kText, ip->kOKPBtn,
			 ip->kCancelPBtn, &kTextval);
	/* I won't actually be using this copy */
	XtFree (kTextval);
	
	/* verify the string as an integer */
	knn_params.k =
	  set_atoi_value(ip->kText, "k_set",
			     knn_params.k);

	/* save the current text for future comparisons */
	XtFree(knn_params.k_string);
	knn_params.k_string = get_string_text(ip->kText);
 
	/* do any other things needed */
	set_plot_labels();

    } else if (!strcmp(client, "kCancelPBtn")) {
	/*fprintf (stderr, "knn_callback: %s\n", client );*/
	k_buttons_mapped = 
	  okTextCancel(ip->kText, ip->kOKPBtn,
		       ip->kCancelPBtn,
		       knn_params.k_string);
	
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "knn_callback: %s\n", client );*/
	close_popup(KnnClone->knnForm);
	
    } else {
	fprintf(stderr, "knn_callback: unknown client: %s\n", (char *)client);
    }
}

void
knn_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(KnnClone->knnForm);
    } else if (!strcmp(request, "init")){
	/* flag */
	knn_params.leave1Out = 0;
	knn_params.k = 0;
	knn_params.k_string = NULL;
    } else if (!strcmp(request, "display")){
	char string[10];
	/* flag */
	show_toggle_value(KnnClone->leave1OutToggle, knn_params.leave1Out);
	XtFree(knn_params.k_string);
	sprintf(string, "%d", knn_params.k);
	knn_params.k_string = XtNewString(string);
	show_string_text(KnnClone->kText,
			 knn_params.k_string);
    } else {
	fprintf(stderr, "knn_request: unknown request: %s\n", request);
    }
}

String
knn_get_plot_label(String string)
{
    char num[10];
    string = StringConcat(string, " K:");
    sprintf(num, "%d", knn_params.k);
    string = StringConcat(string, num);
    return string;
}

static void
knn_set_read_variables()
{
    knn_params.leave1Out = cvtest_flag;
    knn_params.k = k;
}

static void
knn_set_write_variables()
{
    cvtest_flag = knn_params.leave1Out;
    k = knn_params.k;
}

void
read_knn_defaults(argc, argv)
     int argc;
     char **argv;
{
    /* clear flags and strings which are not printed if they are NULL */
    cvtest_flag = 0;

    if(argc > 0){
	argc = nnargs(argc, argv, knn_tab, KNN_TABSIZE);
	knn_set_read_variables();
    } else {
	/* set any defaults */
        knn_params.k = 1;
    }

    knn_request("display");
}

void
write_knn_defaults(FILE *default_file)
{
    knn_set_write_variables();

    if(default_file != NULL){
	fprintf(default_file, "knn %s",
		nnargs_get_flags(knn_tab, KNN_TABSIZE));
    }
}

String
knn_get_args ()
{
    knn_set_write_variables();
    return (nnargs_get_part_flags(knn_tab, KNN_TABSIZE));
}

String
knn_get_important_args ()
{
    knn_set_write_variables();
    return (nnargs_get_flags_one_line(knn_tab, KNN_TABSIZE));
}

void
knn_store_plot_command (FILE *shell_file)
{
    fprintf(shell_file, "\\\n -k %d", k);
}
