/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/lead_clus/lead_clus_callback.c,v 11.4 2001/08/23 20:16:50 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            LEAD_CLUS                              ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  23-Oct-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <Xm/Xm.h>
#include <nnlib.h>
#include <lead_clus_args.h>
#include "lead_clus.h"
/*
 * External Functions
 */
#include "../main/lnknet_subs.h"
#include "../plots/plots_func.h"
#include "lead_clus_func.h"

/*
 * Externals
 */
extern Lead_clusWidgets *Lead_clusClone;
#include "lead_clus_params.h"
Lead_ClusParams lead_clus_params; /* this is the real one */

/*
 * Statics
 */
static void lead_clus_set_read_variables();
static void lead_clus_set_write_variables();
static void lead_clus_set_defaults_variables();
static void lead_clus_show_delta_choices();

/******************************************************************************
 ******************************************************************************
 *
 * Module Name: lead_clus_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   lead_clus_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  23-Oct-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */

void
lead_clus_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    Lead_clusWidgets *ip;
    static int delta_buttons_mapped = False;
    static int deltaList_buttons_mapped = False;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "ignoreClassToggle") || 
	!strcmp(client, "clusterByClassToggle")) {
	/*fprintf (stderr, "lead_clus_callback: %s\n", client );*/
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	if(state->set){
	    lead_clus_params.cluster_by_class =
	      get_radio_value(ip->clusterByClassRadioBox);
	    lead_clus_show_delta_choices();
	    set_plot_labels();
	}

    } else if (!strcmp(client, "equalDeltaToggle") ||
	       !strcmp(client, "deltaByClassToggle")) {
	/*fprintf (stderr, "lead_clus_callback: %s\n", client );*/
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	if(state->set){
	    lead_clus_params.specify_by_class =
	      get_radio_value(ip->deltaByClassRadioBox);
	    lead_clus_show_delta_choices();
	    set_plot_labels();
	}
	
    } else if (!strcmp(client, "deltaText")) {
	/*fprintf (stderr, "lead_clus_callback: %s\n", client );*/
	delta_buttons_mapped =
	  okTextValueChanged(ip->deltaText, ip->deltaOKPBtn,
			     ip->deltaCancelPBtn,
			     ip->lead_clusDialogShell,
			     delta_buttons_mapped,
			     lead_clus_params.delta_string);
	
    } else if ((!strcmp(client, "deltaTextAct")) ||
	       (!strcmp(client, "deltaOKPBtn"))) {
	String deltaTextval;

      /*fprintf (stderr, "lead_clus_callback: %s\n", client );*/
	delta_buttons_mapped =
	  okTextActivate(ip->deltaText, ip->deltaOKPBtn,
			 ip->deltaCancelPBtn, &deltaTextval);
	/* I won't actually be using this copy */
	XtFree (deltaTextval);
	
	/* verify the string as an integer */
	lead_clus_params.delta =
	  set_atof_value(ip->deltaText, "delta_set",
			     lead_clus_params.delta);

	/* save the current text for future comparisons */
	XtFree(lead_clus_params.delta_string);
	lead_clus_params.delta_string = get_string_text(ip->deltaText);
 
	/* do any other things needed */
	set_plot_labels();
  } else if (!strcmp(client, "deltaCancelPBtn")) {
    /*fprintf (stderr, "lead_clus_callback: %s\n", client );*/
	delta_buttons_mapped = 
	  okTextCancel(ip->deltaText, ip->deltaOKPBtn,
		       ip->deltaCancelPBtn,
		       lead_clus_params.delta_string);
	
    } else if (!strcmp(client, "deltaListText")) {
	/*fprintf (stderr, "lead_clus_callback: %s\n", client);*/
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	deltaList_buttons_mapped =
	  okTextValueChanged(ip->deltaListText, ip->deltaListOKPBtn,
			     ip->deltaListCancelPBtn,
			     ip->lead_clusDialogShell,
			     deltaList_buttons_mapped,
			     lead_clus_params.deltaList_string);
    } else if ((!strcmp(client, "deltaListTextAct")) ||
	       (!strcmp(client, "deltaListOKPBtn"))) {
	/*fprintf (stderr, "lead_clus_callback: %s\n", client );*/
	String deltaListTextval;
	deltaList_buttons_mapped =
	  okTextActivate(ip->deltaListText,
			 ip->deltaListOKPBtn,
			 ip->deltaListCancelPBtn,
			 &deltaListTextval);
	
	if(lead_clus_params.deltaList != NULL)
	  deallocate_vector(lead_clus_params.deltaList);
	lead_clus_params.deltaList = NULL;
	/* take spaces and tabs out of the list before handing it to nnargs */
	if(deltaListTextval){
	    char *deltaListTemp = deltaListTextval;
	    deltaListTextval = strtok(deltaListTextval, " \t");
	    if(deltaListTextval){
		lead_clus_params.deltaList =
		  nnargs_get_float_alloc_list(deltaListTextval);
		XtFree(deltaListTemp);
	    }
	}
	
	/* make a copy of the string as it has been interpretted */
	XtFree(lead_clus_params.deltaList_string);
	lead_clus_params.deltaList_string =
	  make_float_string(lead_clus_params.deltaList);
	show_string_text(ip->deltaListText,
			 lead_clus_params.deltaList_string);
	
	/* do anything else necessary */
	set_plot_labels();
	
    } else if (!strcmp(client, "deltaListCancelPBtn")) {
	/*fprintf (stderr, "lead_clus_callback: %s\n", client );*/
	deltaList_buttons_mapped = 
	  okTextCancel(ip->deltaListText, ip->deltaListOKPBtn,
		       ip->deltaListCancelPBtn,
		       lead_clus_params.deltaList_string);
 
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "lead_clus_callback: %s\n", client );*/
	close_popup(Lead_clusClone->lead_clusForm);
 
    } else {
	fprintf(stderr, "lead_clus_callback: unknown client: %s\n", (char *)client);
    }
}

void
lead_clus_request(String request)
{
    if (!strcmp(request, "open")) {
	open_popup(Lead_clusClone->lead_clusForm);
    } else if (!strcmp(request, "init")){
	/* this should only get called once, before ANYTHING 
	   gets allocated or there will be memory leaks */
	lead_clus_params.cluster_by_class = False;
	lead_clus_params.specify_by_class = False;
	lead_clus_params.delta = 0;
	lead_clus_params.delta_string = NULL;
	lead_clus_params.deltaList = NULL;
	lead_clus_params.deltaList_string = NULL;
    } else if (!strcmp(request, "display")){
	char string[10]; /* for creating int and float text */
	show_radio_value(Lead_clusClone->clusterByClassRadioBox,
			 lead_clus_params.cluster_by_class);
	show_radio_value(Lead_clusClone->deltaByClassRadioBox,
			 lead_clus_params.specify_by_class);

	sprintf(string, "%5.2g", lead_clus_params.delta);
	XtFree(lead_clus_params.delta_string);
	lead_clus_params.delta_string = XtNewString(string);
	show_string_text(Lead_clusClone->deltaText,
			 lead_clus_params.delta_string);

	XtFree(lead_clus_params.deltaList_string);
	lead_clus_params.deltaList_string =
	  make_float_string(lead_clus_params.deltaList);
	show_string_text(Lead_clusClone->deltaListText,
			 lead_clus_params.deltaList_string);

	lead_clus_show_delta_choices();

    } else {
	fprintf (stderr, "lead_clus_request: unknown request: %s\n", request);
    }
}

String
lead_clus_get_plot_label(String string)
{
    char intchars[10];
    if(lead_clus_params.cluster_by_class && lead_clus_params.specify_by_class){
	string = StringConcat(string, " Delta List:");
	string = StringConcat(string, lead_clus_params.deltaList_string);
    } else {
	sprintf(intchars, "%.2f", lead_clus_params.delta);
	string = StringConcat(string, " Delta:");
	string = StringConcat(string, intchars);
    }
    return string;
}

static ARG lead_clus_tab2[] =
{
    {"specify_by_class", BOOLEAN, (int *)NULL,
       "use delta list for nclusters by class"},
};
#define LEAD_CLUS_TABSIZE2 (sizeof(lead_clus_tab2)/sizeof(ARG))

static void
lead_clus_set_read_variables()
{
    lead_clus_params.cluster_by_class = cluster_by_class;
    lead_clus_params.delta = delta;
    if(lead_clus_params.deltaList != NULL){
	deallocate_vector(lead_clus_params.deltaList);
    }
    lead_clus_params.deltaList = delta_list;
    delta_list = NULL;
}

static void
lead_clus_set_write_variables()
{
    cluster_by_class = lead_clus_params.cluster_by_class;
    delta = lead_clus_params.delta;
    delta_list = lead_clus_params.deltaList;
}

static void
lead_clus_set_defaults_variables()
{
    nnargs_set_variable("specify_by_class", &lead_clus_params.specify_by_class,
			lead_clus_tab2, LEAD_CLUS_TABSIZE2);
}
void
read_lead_clus_defaults(int argc, char **argv)
{
    cluster_by_class = 0;
    delta_list = NULL;/* the space for this list is always
			    owned by the lead_clus_params structure */
    lead_clus_params.specify_by_class = 0;

    lead_clus_set_defaults_variables();
    if(argc > 0){
	argc = nnargs_part(argc, argv, lead_clus_tab2, LEAD_CLUS_TABSIZE2);
	argc = nnargs(argc, argv, lead_clus_tab, LEAD_CLUS_TABSIZE);
	lead_clus_set_read_variables();
    } else {
	lead_clus_params.delta = 1.0;
	lead_clus_params.specify_by_class = 0;
    }
    lead_clus_request("display");
}

void
write_lead_clus_defaults(default_file)
     FILE *default_file;
{
    lead_clus_set_write_variables();
    lead_clus_set_defaults_variables();
    if(default_file != NULL){
	fprintf(default_file, "lead_clus %s",
		nnargs_get_part_flags(lead_clus_tab, LEAD_CLUS_TABSIZE));
	fprintf(default_file, "%s",
		nnargs_get_flags(lead_clus_tab2, LEAD_CLUS_TABSIZE2));
    }
}

String
lead_clus_get_args ()
{
    lead_clus_set_write_variables();
    /* fix up delta list, in case it isn't wanted */
    if(lead_clus_params.cluster_by_class && lead_clus_params.specify_by_class){
	delta = 0.;
    } else {
	delta_list = NULL;
    }
    return (nnargs_get_part_flags(lead_clus_tab, LEAD_CLUS_TABSIZE));
}

String
lead_clus_get_important_args ()
{
    lead_clus_set_write_variables();
    /* fix up delta list, in case it isn't wanted */
    if(lead_clus_params.specify_by_class){
	delta = 0.;
    } else {
	delta_list = NULL;
    }
    return (nnargs_get_flags_one_line(lead_clus_tab, LEAD_CLUS_TABSIZE));
}

static void
lead_clus_show_delta_choices()
{
    if(lead_clus_params.cluster_by_class){
	gray_out_item(Lead_clusClone->deltaByClassRadioBox, FALSE);
	if(lead_clus_params.specify_by_class){
	    gray_out_item(Lead_clusClone->deltaListForm, FALSE);
	    gray_out_item(Lead_clusClone->deltaForm, TRUE);
	} else {
	    gray_out_item(Lead_clusClone->deltaListForm, TRUE);
	    gray_out_item(Lead_clusClone->deltaForm, FALSE);
	}
    } else {
	gray_out_item(Lead_clusClone->deltaByClassRadioBox, TRUE);
	gray_out_item(Lead_clusClone->deltaListForm, TRUE);
	gray_out_item(Lead_clusClone->deltaForm, FALSE);
    }
}
