/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/lnk/lnk_algs.c,v 11.11 2001/09/13 20:39:50 kukolich Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            LNKNET                              ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $LNKNETHOME/src/lnk_algs.c
 *
 * Language: C
 *
 * File Modification History:
 *  23-Aug-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
/*
#include <sys/param.h>
#include <sys/types.h>
*/
#include <Xm/Xm.h>
#include <nnlib.h>
#if STDC_HEADERS
#include <stdlib.h>
/* stdlib.h is included for getenv */
#endif
/*
 * External Functions
 */
#include "../main/lnknet_subs.h"
#include "lnk_func.h"
#include "../reports/reports_func.h"
#include "../data/data_func.h"
#include "../norm/norm_func.h"
#include "../select/select_func.h"
#include "../priors/priors_func.h"
#include "../plots/plots_func.h"
#include "../region/region_func.h"
#include "../profile/profile_func.h"
#include "../struct/struct_func.h"
#include "../cost/cost_func.h"
#include "../perr/perr_func.h"
#include "../prob/prob_func.h"
#include "../detect/detect_func.h"
#include "../reject/reject_func.h"
#include "../print/print_func.h"
#include "../ccode/ccode_func.h"
#include "../committeeData/committeeData_func.h"
#include "../mlp/mlp_func.h"
#include "../rbf/rbf_func.h"
#include "../irbf/irbf_func.h"
#include "../gauss/gauss_func.h"
#include "../gmix/gmix_func.h"
#include "../histogram/histogram_func.h"
#include "../nbayes/nbayes_func.h"
#include "../svm/svm_func.h"
#include "../parzen/parzen_func.h"
#include "../knn/knn_func.h"
#include "../cknn/cknn_func.h"
#include "../lvq/lvq_func.h"
#include "../nc_class/nc_class_func.h"
#include "../bintree/bintree_func.h"
#include "../hyper/hyper_func.h"
#include "../kmeans/kmeans_func.h"
#include "../em_clus/em_clus_func.h"
#include "../ran_clus/ran_clus_func.h"
#include "../lead_clus/lead_clus_func.h"
#include "../artmap/artmap_func.h"
#include "../committee/committee_func.h"
 
/*
 * Externals
 */
#include "lnk_algs.h"
#include "lnk.h"
extern	LnkWidgets	*LnkClone;
#include "lnk_params.h"
extern  LnkParams	lnk_params;

#include "../norm/norm_params.h"
extern NormParams	norm_params;
/* Globals */

/******************************************************************************
 ******************************************************************************
 *
 * Module Name: lnk_algs
 *
 * Purpose: 
 *   setting algorithms and setting general screen things that depend on
 *   which algorithm is set
 * 
 * Calling Sequence:
 *   none
 *
 * Inputs:
 *   none
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  14-Sep-1995     Linda C Kukolich  started port from Xview, gnn/galgs.c
 *
 ******************************************************************************
 ******************************************************************************
 */

void
set_algorithm(ALGORITHM_TYPE value)
{
    set_short_algorithm(value);
    set_experiment_name(); /* adds in classifier name and
			      tells dependents */
    set_plot_labels();
}

void
set_short_algorithm(ALGORITHM_TYPE value)
{
    lnk_params.classifier_num = value;

    if(alg_is_clus()){
	/* KMEANS EM_CLUS RAN_CLUS LEAD_CLUS */
	region_request("colorOff");
    } else {
	region_request("colorOn");
    }
    if(lnk_params.classifier_num == ARTMAP ||
       lnk_params.classifier_num == COMMITTEE){
	activate_ccode(FALSE);
    } else {
	activate_ccode(TRUE);
    }
    if(alg_is_clus() || lnk_params.classifier_num == ARTMAP ||
       lnk_params.classifier_num == COMMITTEE){
	selectGen_request("alg_select_off");
    } else {
	selectGen_request("alg_select_on");
    }
    if(alg_is_epoch_class()){
	/* CKNN HYPER MLP IRBF GMIX LVQ ARTMAP */
	plots_arg_request("epochPlots",TRUE);
	plots_arg_request("perr",TRUE);
	if(alg_is_continuous_outputs())
	  plots_arg_request("cost",TRUE);
	else 
	  plots_arg_request("cost",FALSE);
    } else {
	plots_arg_request("epochPlots",FALSE);
	plots_arg_request("cost",FALSE);
	plots_arg_request("perr",FALSE);
    }
    if(alg_is_continuous_outputs()){
	/* MLP GAUSS RBF IRBF GMIX COMMITTEE */
	activate_test_priors(TRUE);
	plots_arg_request("profile",TRUE);
	plots_arg_request("testError",TRUE);
    } else {
	activate_test_priors(FALSE);
	plots_arg_request("profile",FALSE);
	plots_arg_request("testError",FALSE);
    }
    if(alg_is_likelihood()){
	prob_request("sumToOne");
	reject_request("sumToOne");
    }
    if(lnk_params.classifier_num == HISTOGRAM ||
       lnk_params.classifier_num == COMMITTEE)
      activate_train_priors(FALSE);
    else
      activate_train_priors(TRUE);

    if(lnk_params.classifier_num == COMMITTEE)
      plots_arg_request("region",FALSE);
    else
      plots_arg_request("region",TRUE);

    if(alg_has_struct_plot()){
	/* BINTREE MLP GAUSS RBF IRBF GMIX SVM */
	plots_arg_request("struct",TRUE);

	if(lnk_params.classifier_num == BINTREE || lnk_params.classifier_num == GAUSS || lnk_params.classifier_num == SVM ){
	    /* no weights, so no threshold either */
	    struct_request("noWeights");
	} else {
	    struct_request("weights");
	}
	if(lnk_params.classifier_num == MLP){
	    /* turn on dispay bias */
	    struct_request("bias");
	} else {
	    struct_request("noBias");
	}
    } else {
	plots_arg_request("struct",FALSE);
    }
    /* set up cost plot error functions */
    switch(lnk_params.classifier_num){
      case MLP:
	cost_set_cost_func(mlp_get_cost_func());
	break;
      case ARTMAP:
	cost_set_cost_func(ARTMAP_COST);
	break;
      case IRBF:
	cost_set_cost_func(irbf_get_cost_func());
	break;
      case GMIX:
	cost_set_cost_func(LOG_LIKE);
	break;
      case CKNN:
      case HYPER:
	cost_set_cost_func(CENTERS);
	break;
      case BINTREE:
      case GAUSS:
      case KNN:
      case RBF:
      case COMMITTEE:
	cost_set_cost_func(SQ_ERR);
	break;
      case LVQ:
      case NC_CLASS:
	cost_set_cost_func(NO_COST);
	break;
      case KMEANS:
	cost_set_cost_func(CLUS_SQ_ERR);
	break;
      case EM_CLUS:
	cost_set_cost_func(em_clus_get_cost_func());
	break;
    default:
      break;
    }

    lnk_params.classifier_name = classifier_names[lnk_params.classifier_num];
    lnk_params.classifier_string = classifier_strings[lnk_params.classifier_num];
    /* tell notebook that something important happened */
    reports_request("major_note");
}

Boolean
alg_is_clus()
{
    switch(lnk_params.classifier_num){
      case KMEANS:
      case EM_CLUS:
      case RAN_CLUS:
      case LEAD_CLUS:
	return 1;
      default:
	return 0;
    }
}

Boolean
alg_is_epoch_class()
{
    switch(lnk_params.classifier_num){
      case MLP:
      case IRBF:
      case GMIX:
      case CKNN:
      case LVQ:
      case HYPER:
      case ARTMAP:
	return 1;
      default:
	return 0;
    }
}

Boolean
alg_is_continuous_outputs()
{
    switch(lnk_params.classifier_num){
      case MLP:
      case RBF:
      case IRBF:
      case GAUSS:
      case GMIX:
      case HISTOGRAM:
      case NBAYES:
      case SVM:
      case PARZEN:
      case BINTREE:
	return 1;
      default:
	return 0;
    }
}

Boolean
alg_is_likelihood()
{
    switch(lnk_params.classifier_num){
      case GAUSS:
      case GMIX:
      case HISTOGRAM:
      case PARZEN:
	return 1;
      default:
	return 0;
    }
}

Boolean
alg_is_bayesian()
{
    switch(lnk_params.classifier_num){
      case MLP:
      case RBF:
      case IRBF:
      case BINTREE:
	return 1;
      default:
	return 0;
    }
}

Boolean
alg_has_struct_plot()
{
    switch(lnk_params.classifier_num){
      case MLP:
      case RBF:
      case IRBF:
      case GAUSS:
      case GMIX:
      case BINTREE:
    case SVM:
	return 1;
      default:
	return 0;
    }
}

void
show_algorithm_popup()
{
    switch (lnk_params.classifier_num) {
      case MLP:
	mlp_request("open");
	break;
      case RBF:
	rbf_request("open");
	break;
      case IRBF:
	irbf_request("open");
	break;
      case GAUSS:
	gauss_request("open");
	break;
      case GMIX:
	gmix_request("open");
	break;
      case HISTOGRAM:
	histogram_request("open");
	break;
      case NBAYES:
	nbayes_request("open");
	break;
      case SVM:
	svm_request("open");
	break;
      case PARZEN:
	parzen_request("open");
	break;
      case KNN:
	knn_request("open");
	break;
      case CKNN:
	cknn_request("open");
	break;
      case NC_CLASS:
	nc_class_request("open");
	break;
      case LVQ:
	lvq_request("open");
	break;
      case BINTREE:
	bintree_request("open");
	break;
      case HYPER:
	hyper_request("open");
	break;
      case COMMITTEE:
	committee_request("open");
	break;
      case KMEANS:
	kmeans_request("open");
	break;
      case EM_CLUS:
	em_clus_request("open");
	break;
      case LEAD_CLUS:
	lead_clus_request("open");
	break;
      case RAN_CLUS:
	ran_clus_request("open");
	break;
      case ARTMAP:
	artmap_request("open");
	break;
      default:
	break;
	}
}

char *
g_get_plot_label()
{
    String string;

    string = XtNewString("Norm:");
    string = StringConcat(string, norm_description(norm_params.normalize));

    switch (lnk_params.classifier_num) {
      case MLP:
	return mlp_get_plot_label(string);
      case RBF:
	return rbf_get_plot_label(string);
      case IRBF:
	return irbf_get_plot_label(string);
      case GAUSS:
	return gauss_get_plot_label(string);
      case GMIX:
	return gmix_get_plot_label(string);
      case HISTOGRAM:
	return histogram_get_plot_label(string);
      case NBAYES:
	return nbayes_get_plot_label(string);
      case SVM:
	return svm_get_plot_label(string);
      case PARZEN:
	return parzen_get_plot_label(string);
      case KNN:
	return knn_get_plot_label(string);
      case CKNN:
	return cknn_get_plot_label(string);
      case NC_CLASS:
	return nc_class_get_plot_label(string);
      case LVQ:
	return lvq_get_plot_label(string);
      case BINTREE:
	return bintree_get_plot_label(string);
      case HYPER:
	return hyper_get_plot_label(string);
      case COMMITTEE:
	return committee_get_plot_label(string);
      case KMEANS:
	return kmeans_get_plot_label(string);
      case EM_CLUS:
	return em_clus_get_plot_label(string);
      case LEAD_CLUS:
	return lead_clus_get_plot_label(string);
      case RAN_CLUS:
	return ran_clus_get_plot_label(string);
      case ARTMAP:
	return artmap_get_plot_label(string);
      default:
	return string;
    }
}

void
g_update_ninputs()
{
    kmeans_set_total_ncenters();
    em_clus_set_total_ncenters();
    ran_clus_set_total_ncenters();
    mlp_request("update_ninputs");
    histogram_set_total_nbins();
    svm_set_total_nclass();
}

void
g_get_test_args(shell_file)
     FILE *shell_file;
{
    switch (lnk_params.classifier_num) {
      case RBF:
	rbf_store_plot_command (shell_file);
	break;
      case IRBF:
	irbf_store_plot_command (shell_file);
	break;
      case HISTOGRAM:
	histogram_store_plot_command(shell_file);
	break;
      case NBAYES:
	nbayes_store_plot_command(shell_file);
	break;
      case PARZEN:
	parzen_store_plot_command(shell_file);
	break;
      case KNN:
	knn_store_plot_command (shell_file);
	break;
      case BINTREE:
	bintree_store_plot_command(shell_file);
	break;
    case SVM:
      svm_store_plot_command(shell_file);
      break;
      default:
	break;
    }
}

char *
g_get_alg_args()
{
    switch (lnk_params.classifier_num) {
      case MLP:
	return mlp_get_args ();
      case RBF:
	return rbf_get_args ();
      case IRBF:
	return irbf_get_args ();
      case GAUSS:
	return gauss_get_args ();
      case GMIX:
	return gmix_get_args ();
      case HISTOGRAM:
	return histogram_get_args();
      case NBAYES:
	return nbayes_get_args();
      case SVM:
	return svm_get_args();
      case PARZEN:
	return parzen_get_args();
      case KNN:
	return knn_get_args ();
      case CKNN:
	return cknn_get_args ();
      case NC_CLASS:
	return nc_class_get_args ();
      case LVQ:
	return lvq_get_args ();
      case BINTREE:
	return bintree_get_args ();
      case HYPER:
	return hyper_get_args ();
      case COMMITTEE:
	return committee_get_args ();
      case KMEANS:
	return kmeans_get_args ();
      case EM_CLUS:
	return em_clus_get_args ();
      case LEAD_CLUS:
	return lead_clus_get_args ();
      case RAN_CLUS:
	return ran_clus_get_args ();
      case ARTMAP:
	return artmap_get_args();
      default:
	return "";
    }
}

char *
g_get_important_alg_args()
{
    switch (lnk_params.classifier_num) {
      case MLP:
	return mlp_get_important_args ();
      case RBF:
	return rbf_get_important_args ();
      case IRBF:
	return irbf_get_important_args ();
      case GAUSS:
	return gauss_get_important_args ();
      case GMIX:
	return gmix_get_important_args ();
      case HISTOGRAM:
	return histogram_get_important_args();
      case NBAYES:
	return nbayes_get_important_args();
      case SVM:
	return svm_get_important_args();
      case PARZEN:
	return parzen_get_important_args();
      case KNN:
	return knn_get_important_args ();
      case CKNN:
	return cknn_get_important_args ();
      case LVQ:
	return lvq_get_important_args ();
      case NC_CLASS:
	return nc_class_get_important_args ();
      case BINTREE:
	return bintree_get_important_args ();
      case HYPER:
	return hyper_get_important_args ();
      case COMMITTEE:
	return committee_get_important_args ();
      case KMEANS:
	return kmeans_get_important_args ();
      case EM_CLUS:
	return em_clus_get_important_args ();
      case LEAD_CLUS:
	return lead_clus_get_important_args ();
      case RAN_CLUS:
	return ran_clus_get_important_args ();
      case ARTMAP:
	return artmap_get_important_args();
      default:
	return NULL;
    }
}

Boolean
must_store_clustering(){
    switch(lnk_params.classifier_num){
      case RBF:
	return rbf_must_store_clustering();
      case IRBF:
	return irbf_must_store_clustering();
      case GMIX:
	return gmix_must_store_clustering();
      case LVQ:
	return lvq_must_store_clustering();
      case NC_CLASS:
	return nc_class_must_store_clustering();
      default:
	return 0;
    }
}

char *
g_get_cluster_fname()
{
    switch(lnk_params.classifier_num){
      case RBF:
	return rbf_get_cluster_fname();
      case IRBF:
	return irbf_get_cluster_fname();
      case GMIX:
	return gmix_get_cluster_fname();
      case LVQ:
	return lvq_get_cluster_fname();
      case NC_CLASS:
	return nc_class_get_cluster_fname();
      default:
	return 0;
    }
}

ALGORITHM_TYPE
g_get_cluster_alg(){
    int cluster_alg;
    switch(lnk_params.classifier_num){
      case RBF:
	cluster_alg = rbf_get_cluster_alg();
	break;
      case IRBF:
	cluster_alg = irbf_get_cluster_alg();
	break;
      case GMIX:
	cluster_alg = gmix_get_cluster_alg();
	break;
      case LVQ:
	cluster_alg = lvq_get_cluster_alg();
	break;
      case NC_CLASS:
	cluster_alg = nc_class_get_cluster_alg();
	break;
      default:
	cluster_alg = 0;
	break;
    }
    return cluster_alg;
}

Boolean
alg_uses_clustering(){
    switch(lnk_params.classifier_num){
      case RBF:
      case IRBF:
      case GMIX:
      case LVQ:
      case NC_CLASS:
	return 1;
      default:
	return 0;
    }
}

/* these are intended for use when gnn is storing cluster commands,
   gnn must temporarily change the algorithm to the appropriate clustering
   algorithm */
ALGORITHM_TYPE
g_get_alg(){
    return lnk_params.classifier_num;
}

void
g_set_alg(ALGORITHM_TYPE alg){
    lnk_params.classifier_num = alg;
    lnk_params.classifier_name = classifier_names[alg];
}

/*
 * get the number of epochs from the current algorithm
 */
int
g_get_epochs()
{
    switch (lnk_params.classifier_num) {
      case MLP:
	return mlp_get_epochs();
      case IRBF:
	return irbf_get_epochs();
      case GMIX:
	return gmix_get_epochs();
      case CKNN:
	return cknn_get_epochs();
      case LVQ:
	return lvq_get_epochs();
      case HYPER:
	return hyper_get_epochs();
      case RBF:
      case GAUSS:
      case KNN:
      case NC_CLASS:
      case KMEANS:
      case EM_CLUS:
      case LEAD_CLUS:
      case RAN_CLUS:
      case ARTMAP:
      default:
	return 1;
    }
}

/*
 * set the number of epochs from the current algorithm
 */
void
g_set_epochs(e)
     int e;
{
    switch (lnk_params.classifier_num) {
      case MLP:
	mlp_set_epochs(e);
	break;
      case IRBF:
	irbf_set_epochs(e);
	break;
      case GMIX:
	gmix_set_epochs(e);
	break;
      case CKNN:
	cknn_set_epochs(e);
	break;
      case LVQ:
	lvq_set_epochs(e);
	break;
      case HYPER:
	hyper_set_epochs(e);
	break;
      case RBF:
      case GAUSS:
      case KNN:
      case NC_CLASS:
      case KMEANS:
      case EM_CLUS:
      case LEAD_CLUS:
      case RAN_CLUS:
      case ARTMAP:
      default:
	break;
    }
}

void
g_set_cluster_names()
{
    rbf_set_cluster_param_fname();
    irbf_set_cluster_param_fname();
    gmix_set_cluster_param_fname();
    lvq_set_cluster_param_fname();
    nc_class_set_cluster_param_fname();
}

void
set_common_flags(which_data)
     int which_data;
{
    gnn_set_common_flags(which_data);
}


/******************************************************************************
 ******************************************************************************
 *
 * Module Name: read_defaults, write_defaults
 *
 * Purpose: Read/write state of all LNKnet windows from/to a screen
 *	or defaults file
 * 
 * Calling Sequence:
 *   read_defaults(screen_fname);
 *   OR
 *   write_defaults();
 *
 * Inputs:
 *   String screen_fname;  full path to the defaults file
 *
 * Outputs:
 *   generates a screen file with the state of all windows
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  25-Aug-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */

void
read_defaults(String screen_fname)
{
    FILE *defaults_file;
    char *argv[256], *list_name;
    int argc;

    if(screen_fname == NULL || strcmp(screen_fname, "") == 0){
	screen_fname = path_build(getenv("HOME"), DEFAULTS_FILENAME);
    }
    defaults_file = fopen(screen_fname, "r");
    if(defaults_file != NULL){
	while(1){
	    argc = fill_argv(argv, defaults_file, screen_fname);
	    if(argc == EOF) break;
	    if(argc == 0) continue;
	    list_name = argv[0];
	    if(!strcmp(list_name, "LNKnet")){
		read_main_defaults(argc, argv);
	    } else if(!strcmp(list_name, "reports")){
		read_reports_defaults(argc, argv);
	    } else if(!strcmp(list_name, "data")){
		read_data_defaults(argc, argv);
	    } else if(!strcmp(list_name, "norm")){
		read_norm_defaults(argc, argv);
	    } else if(!strcmp(list_name, "select")){
		read_select_defaults(argc, argv);
	    } else if(!strcmp(list_name, "priors")){
		read_priors_defaults(argc, argv);
	    } else if(!strcmp(list_name, "plots")){
		read_plots_defaults(argc, argv);
	    } else if(!strcmp(list_name, "region")){
		read_region_defaults(argc, argv);
	    } else if(!strcmp(list_name, "profile")){
		read_profile_defaults(argc, argv);
	    } else if(!strcmp(list_name, "struct")){
		read_struct_defaults(argc, argv);
	    } else if(!strcmp(list_name, "cost")){
		read_cost_defaults(argc, argv);
	    } else if(!strcmp(list_name, "perr")){
		read_perr_defaults(argc, argv);
	    } else if(!strcmp(list_name, "prob")){
		read_prob_defaults(argc, argv);
	    } else if(!strcmp(list_name, "detect")){
		read_detect_defaults(argc, argv);
	    } else if(!strcmp(list_name, "reject")){
		read_reject_defaults(argc, argv);
	    } else if(!strcmp(list_name, "print")){
		read_print_defaults(argc, argv);
	    } else if(!strcmp(list_name, "ccode")){
		read_ccode_defaults(argc, argv);
	    } else if(!strcmp(list_name, "committeeData")){
		read_committeeData_defaults(argc, argv);
	    } else if(!strcmp(list_name, "mlp")){
		read_mlp_defaults(argc, argv);
	    } else if(!strcmp(list_name, "rbf")){
		read_rbf_defaults(argc,argv);
	    } else if(!strcmp(list_name, "irbf")){
		read_irbf_defaults(argc,argv);
	    } else if(!strcmp(list_name, "gauss")){
		read_gauss_defaults(argc,argv);
	    } else if(!strcmp(list_name, "gmix")){
		read_gmix_defaults(argc,argv);
	    } else if(!strcmp(list_name, "histogram")){
		read_histogram_defaults(argc, argv);
	    } else if(!strcmp(list_name, "nbayes")){
		read_nbayes_defaults(argc, argv);
	    } else if(!strcmp(list_name, "svm")){
		read_svm_defaults(argc, argv);
	    } else if(!strcmp(list_name, "parzen")){
		read_parzen_defaults(argc, argv);
	    } else if(!strcmp(list_name, "knn")){
		read_knn_defaults(argc,argv);
	    } else if(!strcmp(list_name, "cknn")){
		read_cknn_defaults(argc,argv);
	    } else if(!strcmp(list_name, "lvq")){
		read_lvq_defaults(argc,argv);
	    } else if(!strcmp(list_name, "nc_class")){
		read_nc_class_defaults(argc,argv);
	    } else if(!strcmp(list_name, "bintree")){
		read_bintree_defaults(argc, argv);
	    } else if(!strcmp(list_name, "hyper")){
		read_hyper_defaults(argc, argv);
	    } else if(!strcmp(list_name, "committee")){
		read_committee_defaults(argc, argv);
	    } else if(!strcmp(list_name, "kmeans")){
		read_kmeans_defaults(argc,argv);
	    } else if(!strcmp(list_name, "em_clus")){
		read_em_clus_defaults(argc,argv);
	    } else if(!strcmp(list_name, "lead_clus")){
		read_lead_clus_defaults(argc,argv);
	    } else if(!strcmp(list_name, "ran_clus")){
		read_ran_clus_defaults(argc,argv);
	    } else if(!strcmp(list_name, "artmap")){
		read_artmap_defaults(argc, argv);
	    } else {
		warning("Unknown argument list: %s", list_name);
	    }
	}
    } else {
	read_init_defaults();
    }
    if(defaults_file == NULL){
	/* update algorithm derived file names */
	set_algorithm(lnk_params.classifier_num);
	/* update data base derived file names */
	data_set_data_base();
    } else {
	/* some things are derived and not stored whole */
	set_algorithm(lnk_params.classifier_num);
	g_set_cv_filename();
	fclose(defaults_file);
    }
    display_defaults();
    /* write messages, like class priors for files, that are generated
       not stored */
}
void
read_init_defaults()
{
    int argc = 0;
    char **argv = NULL;

    read_main_defaults(argc, argv);
    read_reports_defaults(argc, argv);
    read_data_defaults(argc, argv);
    read_norm_defaults(argc, argv);
    read_select_defaults(argc, argv);
    read_priors_defaults(argc, argv);
    read_plots_defaults(argc, argv);
    read_region_defaults(argc, argv);
    read_profile_defaults(argc, argv);
    read_struct_defaults(argc, argv);
    read_cost_defaults(argc, argv);
    read_perr_defaults(argc, argv);
    read_prob_defaults(argc, argv);
    read_detect_defaults(argc, argv);
    read_reject_defaults(argc, argv);
    read_print_defaults(argc, argv);
    read_ccode_defaults(argc, argv);
    read_committeeData_defaults(argc, argv);
    read_mlp_defaults(argc, argv);
    read_rbf_defaults(argc,argv);
    read_irbf_defaults(argc,argv);
    read_gauss_defaults(argc,argv);
    read_gmix_defaults(argc,argv);
    read_histogram_defaults(argc, argv);
    read_nbayes_defaults(argc, argv);
    read_svm_defaults(argc, argv);
    read_parzen_defaults(argc, argv);
    read_knn_defaults(argc,argv);
    read_cknn_defaults(argc,argv);
    read_lvq_defaults(argc,argv);
    read_nc_class_defaults(argc,argv);
    read_bintree_defaults(argc, argv);
    read_hyper_defaults(argc, argv);
    read_committee_defaults(argc, argv);
    read_kmeans_defaults(argc,argv);
    read_em_clus_defaults(argc,argv);
    read_lead_clus_defaults(argc,argv);
    read_ran_clus_defaults(argc,argv);
    read_artmap_defaults(argc, argv);
}

void
write_defaults(String screen_fname)
{
    FILE *dfp;

    if (screen_fname == NULL || strcmp(screen_fname, "") == 0){
	screen_fname = path_build(getenv("HOME"), DEFAULTS_FILENAME);
    }
    dfp = fopen(screen_fname, "w");
    if(dfp == NULL){
	warning("cannot write to defaults/screen file %s",
		screen_fname);
	return;
    }
    write_main_defaults(dfp);
    write_reports_defaults(dfp);
    write_data_defaults(dfp);
    write_norm_defaults(dfp);
    write_select_defaults(dfp);
    write_priors_defaults(dfp);
    write_plots_defaults(dfp);
    write_region_defaults(dfp);
    write_profile_defaults(dfp);
    write_struct_defaults(dfp);
    write_cost_defaults(dfp);
    write_perr_defaults(dfp);
    write_prob_defaults(dfp);
    write_detect_defaults(dfp);
    write_reject_defaults(dfp);
    write_print_defaults(dfp);
    write_ccode_defaults(dfp);
    write_committeeData_defaults(dfp);
    write_mlp_defaults(dfp);
    write_rbf_defaults(dfp);
    write_irbf_defaults(dfp);
    write_gauss_defaults(dfp);
    write_gmix_defaults(dfp);
    write_histogram_defaults(dfp);
    write_nbayes_defaults(dfp);
    write_svm_defaults(dfp);
    write_parzen_defaults(dfp);
    write_knn_defaults(dfp);
    write_cknn_defaults(dfp);
    write_lvq_defaults(dfp);
    write_nc_class_defaults(dfp);
    write_bintree_defaults(dfp);
    write_hyper_defaults(dfp);
    write_committee_defaults(dfp);
    write_kmeans_defaults(dfp);
    write_em_clus_defaults(dfp);
    write_lead_clus_defaults(dfp);
    write_ran_clus_defaults(dfp);
    write_artmap_defaults(dfp);
    printf ("\nLNKnet default values saved in %s\n", screen_fname);
    fclose(dfp);
}

void
display_defaults()
{
    lnk_request("display");
    reports_request("display");
    data_request("display");
    norm_request("display");
    select_request("display");
    priors_request("display");
    plots_request("display");
    region_request("display");
    profile_request("display");
    struct_request("display");
    cost_request("display");
    perr_request("display");
    prob_request("display");
    detect_request("display");
    reject_request("display");
    print_request("display");
    ccode_request("display");
    committeeData_request("display");
    mlp_request("display");
    rbf_request("display");
    irbf_request("display");
    gauss_request("display");
    gmix_request("display");
    histogram_request("display");
    nbayes_request("display");
    svm_request("display");
    parzen_request("display");
    knn_request("display");
    cknn_request("display");
    lvq_request("display");
    nc_class_request("display");
    bintree_request("display");
    hyper_request("display");
    committee_request("display");
    kmeans_request("display");
    em_clus_request("display");
    lead_clus_request("display");
    ran_clus_request("display");
    artmap_request("display");
}
