/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/lnk/lnk_callback.c,v 11.7 2001/08/23 20:16:55 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            LNKNET                              ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $LNKNETHOME/src/lnk_callback.c
 *
 * Language: C
 *
 * File Modification History:
 *  23-Aug-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include <nnlib.h>
#include "lnk.h"
#include "lnk_params.h"
#include "../main/lnknet_subs.h"
#if STDC_HEADERS
/* stdlib.h is included for getenv */
#include <stdlib.h>
#endif
/*
 * External Functions
 */
#include "lnk_func.h"
#include "../data/data_func.h"
#include "../reports/reports_func.h"
#include "../norm/norm_func.h"
#include "../select/select_func.h"
#include "../priors/priors_func.h"
#include "../plots/plots_func.h"
#include "../print/print_func.h"
#include "../ccode/ccode_func.h"
#include "../committeeData/committeeData_func.h"
#include "../region/region_func.h"
#include "../profile/profile_func.h"

/*
 * Externals
 */
extern LnkWidgets *LnkClone;
extern LnkParams lnk_params;

/******************************************************************************
 ******************************************************************************
 *
 * Module Name: lnk_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   lnk_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  23-Aug-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
lnk_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    LnkWidgets *ip;
    static int cvFolds_buttons_mapped = False;
    static int extCv_buttons_mapped = False;
    static int seed_buttons_mapped = False;
    static int experimentName_buttons_mapped = False;

    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "experimentNameText")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	experimentName_buttons_mapped =
	  okTextValueChanged(ip->experimentNameText, ip->experimentNameOKPBtn,
			     ip->experimentNameCancelPBtn,
			     ip->lnkTopLevelShell,
			     experimentName_buttons_mapped,
			     lnk_params.exp_prefix);
	
    } else if ((!strcmp(client, "experimentNameTextAct")) ||
	       (!strcmp(client, "experimentNameOKPBtn"))) {
	String experimentNameTextval;

	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	experimentName_buttons_mapped =
	  okTextActivate(ip->experimentNameText, ip->experimentNameOKPBtn,
			 ip->experimentNameCancelPBtn, &experimentNameTextval);
	XtFree (experimentNameTextval);
	experimentNameTextval =
	  set_string_not_empty(ip->experimentNameText,
			       "exp_prefix", lnk_params.exp_prefix);
	XtFree(lnk_params.exp_prefix);
	lnk_params.exp_prefix = experimentNameTextval;
	
	/* do any other things needed */
	set_experiment_name();

    } else if (!strcmp(client, "experimentNameCancelPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	experimentName_buttons_mapped = 
	  okTextCancel(ip->experimentNameText, ip->experimentNameOKPBtn,
		       ip->experimentNameCancelPBtn,
		       lnk_params.exp_prefix);
 
    } else if (!strcmp(client, "algorithmParamsPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	show_algorithm_popup();
 
    } else if (!strcmp(client, "fileCascadeButton")) {
      fprintf (stderr, "lnk_callback: %s\n", (char *)client );
 
    } else if (!strcmp(client, "fileQuitPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	exit(0);
 
    } else if (!strcmp(client, "algorithmsCascadeButton")) {
      fprintf (stderr, "lnk_callback: %s\n", (char *)client );
 
    } else if (!strcmp(client, "algMlpPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	set_algorithm(MLP);
 
    } else if (!strcmp(client, "algRbfPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	set_algorithm(RBF);
 
    } else if (!strcmp(client, "algIrbfPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	set_algorithm(IRBF);
 
    } else if (!strcmp(client, "algGaussPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	set_algorithm(GAUSS);
 
    } else if (!strcmp(client, "algGmixPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	set_algorithm(GMIX);
 
    } else if (!strcmp(client, "algHistogramPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	set_algorithm(HISTOGRAM);
 
    } else if (!strcmp(client, "algNbayesPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	set_algorithm(NBAYES);

    } else if (!strcmp(client, "algSvmPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	set_algorithm(SVM);
 
    } else if (!strcmp(client, "algParzenPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	set_algorithm(PARZEN);
 
    } else if (!strcmp(client, "algKnnPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	set_algorithm(KNN);
 
    } else if (!strcmp(client, "algCknnPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	set_algorithm(CKNN);
 
    } else if (!strcmp(client, "algNc_classPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	set_algorithm(NC_CLASS);

    } else if (!strcmp(client, "algLvqPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	set_algorithm(LVQ);
 
    } else if (!strcmp(client, "algBintreePBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	set_algorithm(BINTREE);
 
    } else if (!strcmp(client, "algHyperPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	set_algorithm(HYPER);

    } else if (!strcmp(client, "algCommitteePBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	set_algorithm(COMMITTEE);
 
    } else if (!strcmp(client, "algKmeansPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	set_algorithm(KMEANS);
 
    } else if (!strcmp(client, "algEm_clusPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	set_algorithm(EM_CLUS);
 
    } else if (!strcmp(client, "algLead_clusPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	set_algorithm(LEAD_CLUS);
 
    } else if (!strcmp(client, "algRan_clusPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	set_algorithm(RAN_CLUS);
 
    } else if (!strcmp(client, "algArtmapPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	set_algorithm(ARTMAP);
 
    } else if (!strcmp(client, "printPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	print_request("open");
 
    } else if (!strcmp(client, "ccodePBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	ccode_request("open");
 
    } else if (!strcmp(client, "committeeDataPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	committeeData_request("open");
 
    } else if (!strcmp(client, "reportPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	reports_request("open");
 
    } else if (!strcmp(client, "dataPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	data_request("open");
 
    } else if (!strcmp(client, "normPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	norm_request("open");
 
    } else if (!strcmp(client, "featurePBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	select_request("open");
 
    } else if (!strcmp(client, "priorsPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	priors_request("open");
 
    } else if (!strcmp(client, "plotPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	plots_request("open");
 
    } else if (!strcmp(client, "saveDefaultsPBtn")) {
	String temp_filename;
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	temp_filename = path_build(getenv("HOME"), DEFAULTS_FILENAME);
	if (overwrite_ok (w, temp_filename)) {
	    write_defaults (temp_filename);
	}
 
    } else if (!strcmp(client, "restoreDefaultsPBtn")) {
	void read_defaults();
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	read_defaults("");/* use defaults file for getting data */
 
    } else if (!strcmp(client, "actionSelectTrainToggle") ||
	       !strcmp(client, "actionSelectTestToggle") ||
	       !strcmp(client, "actionSelectTrainTestToggle") ||
	       !strcmp(client, "actionSelectCrossValidToggle")){
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	if(state->set){
	    lnk_params.action = get_radio_value(ip->actionSelectRadioBox);
	    if(lnk_params.action == TEST)
	      activate_features(FALSE);
	    else
	      activate_features(TRUE);
	    g_set_cvtest_flag(lnk_params.action == CROSS_VALID);

	    /* update the plot names on the print page*/
	    region_request("plotFname");
	    profile_request("plotFname");
	}
 
    } else if (!strcmp(client, "testSelectTrainToggle") ||
	       !strcmp(client, "testSelectEvalToggle") ||
	       !strcmp(client, "testSelectTestToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "lnk_callback: %s %s\n", client,
		 state->set ? "On" : "Off" );*/
	if(state->set){
	    lnk_params.test_select = get_radio_value(ip->testSelectRadioBox);
	    /* update the plot namss on the print page*/
	    region_request("plotFname");
	    profile_request("plotFname");
	}
 
    } else if (!strcmp(client, "extCvText")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	extCv_buttons_mapped =
	  okTextValueChanged(ip->extCvText, ip->extCvOKPBtn,
			     ip->extCvCancelPBtn,
			     ip->lnkTopLevelShell,
			     extCv_buttons_mapped,
			     lnk_params.ext_cv);
	
    } else if ((!strcmp(client, "extCvTextAct")) ||
	       (!strcmp(client, "extCvOKPBtn"))) {
	String extCvTextval;

	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	extCv_buttons_mapped =
	  okTextActivate(ip->extCvText, ip->extCvOKPBtn,
			 ip->extCvCancelPBtn, &extCvTextval);
	XtFree(extCvTextval);

	extCvTextval = set_string_not_empty(ip->extCvText,
					    "ext_cv", lnk_params.ext_cv);
	XtFree(lnk_params.ext_cv);
	lnk_params.ext_cv = XtNewString(extCvTextval);
	XtFree (extCvTextval);
	
	/* do any other things needed */
	lnk_params.ext_cv = extCvTextval;
	g_set_cv_filename();

    } else if (!strcmp(client, "extCvCancelPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	extCv_buttons_mapped = 
	  okTextCancel(ip->extCvText, ip->extCvOKPBtn,
		       ip->extCvCancelPBtn,
		       lnk_params.ext_cv);

    } else if (!strcmp(client, "cvAutoSplitToggleButton") ||
	       !strcmp(client, "cvFromFileToggleButton")){
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	if(state->set){
	    Boolean cvFileOff;
	    lnk_params.cv_from_file = get_radio_value(ip->cvFromFileRadioBox);
	    if(lnk_params.cv_from_file){
		/* cross_valid_from_file = 1; */
		cvFileOff = False;
	    } else {
		/* cross_valid_from_file = 0; */
		cvFileOff = True;
	    }
	    gray_out_item(ip->extCvForm, cvFileOff);
	    gray_out_item(ip->cvFilenameLabel, cvFileOff);
	    gray_out_item(ip->cvFilenameLabel1, cvFileOff);
	    gray_out_item(ip->cvFoldsForm, !cvFileOff);
	    gray_out_item(ip->cvRandomToggle, !cvFileOff);
	}
 
    } else if (!strcmp(client, "cvFoldsText")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	cvFolds_buttons_mapped =
	  okTextValueChanged(ip->cvFoldsText, ip->cvFoldsOKPBtn,
			     ip->cvFoldsCancelPBtn,
			     ip->lnkTopLevelShell,
			     cvFolds_buttons_mapped,
			     lnk_params.cvFolds_string);
	
    } else if ((!strcmp(client, "cvFoldsTextAct")) ||
	       (!strcmp(client, "cvFoldsOKPBtn"))) {
	String cvFoldsTextval;

	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	cvFolds_buttons_mapped =
	  okTextActivate(ip->cvFoldsText, ip->cvFoldsOKPBtn,
			 ip->cvFoldsCancelPBtn, &cvFoldsTextval);
	/* I won't actually be using this copy */
	XtFree (cvFoldsTextval);
	
	/* verify the string as an integer */
	lnk_params.cv_nfolds =
	  set_atoi_value(ip->cvFoldsText, "cv_nfolds", lnk_params.cv_nfolds);

	/* save the current text for future comparisons */
	XtFree(lnk_params.cvFolds_string);
	lnk_params.cvFolds_string = get_string_text(ip->cvFoldsText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "cvFoldsCancelPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	cvFolds_buttons_mapped = 
	  okTextCancel(ip->cvFoldsText, ip->cvFoldsOKPBtn,
		       ip->cvFoldsCancelPBtn,
		       lnk_params.cvFolds_string);

    } else if (!strcmp(client, "cvRandomToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	lnk_params.cv_random = state->set;
 
    } else if (!strcmp(client, "seedText")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	seed_buttons_mapped =
	  okTextValueChanged(ip->seedText, ip->seedOKPBtn,
			     ip->seedCancelPBtn,
			     ip->lnkTopLevelShell,
			     seed_buttons_mapped,
			     lnk_params.seed_string);
	
    } else if ((!strcmp(client, "seedTextAct")) ||
	       (!strcmp(client, "seedOKPBtn"))) {
	String seedTextval;

	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	seed_buttons_mapped =
	  okTextActivate(ip->seedText, ip->seedOKPBtn,
			 ip->seedCancelPBtn, &seedTextval);
	/* I won't actually be using this copy */
	XtFree (seedTextval);
	
	/* verify the string as an integer */
	lnk_params.seed =
	  set_atoi_value(ip->seedText, "seed_set",
			     lnk_params.seed);

	/* save the current text for future comparisons */
	XtFree(lnk_params.seed_string);
	lnk_params.seed_string = get_string_text(ip->seedText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "seedCancelPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	seed_buttons_mapped = 
	  okTextCancel(ip->seedText, ip->seedOKPBtn,
		       ip->seedCancelPBtn,
		       lnk_params.seed_string);

    } else if (!strcmp(client, "randomToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	lnk_params.random_flag = state->set;

    } else if (!strcmp(client, "startPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	lnk_params.create_flag = TRUE;
	lnk_params.append_flag = FALSE;
	lnk_params.continue_flag = FALSE;
	g_run (EXP_AND_PLOT);
 
    } else if (!strcmp(client, "stopPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	stop_command_file(lnk_params.exper_pid);
	lnk_params.exper_pid = 0;
 
    } else if (!strcmp(client, "continuePBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	lnk_params.create_flag = FALSE;
	lnk_params.append_flag = TRUE;
	lnk_params.continue_flag = TRUE;
	g_run (EXP_AND_PLOT);
 
    } else if (!strcmp(client, "plotOnlyPBtn")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	lnk_params.continue_flag = FALSE;
	g_run (PLOT_ONLY);
	
    } else if (!strcmp(client, "storeOnlyToggle")) {
	/*fprintf (stderr, "lnk_callback: %s\n", client );*/
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	lnk_params.store_only = state->set;
 
    } else {
	fprintf (stderr, "lnk_callback: unknown client: %s\n", (char *)client);
    }
    return;
}
