/*SUPPRESS 592*/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/lnk/lnk_script.c,v 11.7 2001/08/23 20:16:55 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                writing common flags to shell script                  ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source: /data/speech/kukolich/cvsroot/lnknet/src/mclass/lnk/lnk_script.c,v $
 *
 * Language: C
 *
 * File Modification History:
 *  26-Sep-95     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <sys/types.h>
#if HAVE_SYS_WAIT_H
#include <sys/wait.h>
#endif
#include <Xm/Xm.h>
#include <nnlib.h>
#include <gen_class_args.h>
/*
 * External Functions
 */
#include "../main/lnknet_subs.h"
#include "lnk_func.h"
#include "../reports/reports_func.h"
#include "../plots/plots_func.h"

/*
 * Externals
 */
#include "lnk_params.h"
extern LnkParams lnk_params;
#include "../reports/reports_params.h"
extern ReportsParams reports_params;
#include "../data/data_params.h"
extern DataParams data_params;
#include "../norm/norm_params.h"
extern NormParams norm_params;
#include "../select/select_params.h"
extern SelectParams select_params;
#include "../priors/priors_params.h"
extern PriorsParams priors_params;
#include "../plots/plots_params.h"
extern PlotsParams plots_params;
#include "lnk.h"
extern LnkWidgets *LnkClone;
/* for classifier names */
#include "lnk_algs.h"

/*
 * Statics
 */
static int
store_epochs_train(FILE *shell_file, int run_flag);
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: gnn_set_common_flags
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   gnn_set_common_flags()
 *
 * Inputs:
 *   none
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  26-Sep-95     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */

void
gnn_set_common_flags(which_data)
     int which_data;
{
    String err_exten = NULL;
    static String tmp_error_fname_space = NULL;

    train_flag = lnk_params.train_flag;
    create_flag = lnk_params.create_flag;
    exp_path = reports_params.exp_path;
    /* build error_fname */
    switch(which_data){
      case TRAIN_TRAIN:
	err_exten = data_params.ext_train;
	break;
      case CLASS_TRAIN:
	err_exten = data_params.ext_test_train;
	break;
      case CLASS_EVAL:
	err_exten = data_params.ext_eval;
	break;
      case CLASS_TEST:
	err_exten = data_params.ext_test;
	break;
      case CV_TRAIN:
	err_exten = lnk_params.ext_cv;
	break;
    }
    if(tmp_error_fname_space) XtFree(tmp_error_fname_space);
    tmp_error_fname_space = XtNewString(reports_params.error_fname);
    tmp_error_fname_space = StringConcat(tmp_error_fname_space, err_exten);
    error_fname = tmp_error_fname_space;
    
    param_fname = reports_params.param_fname;
    data_path = data_params.data_path;

    /* build data_fname */
    switch(which_data){
      case CLASS_TRAIN:
      case TRAIN_TRAIN:
      case CV_TRAIN:
	npatterns = data_params.npatterns_train;
	data_fname = data_params.train_fname;
	break;
      case CLASS_EVAL:
	npatterns = data_params.npatterns_eval;
	data_fname = data_params.eval_fname;
	break;
      case CLASS_TEST:
	npatterns = data_params.npatterns_test;
	data_fname = data_params.test_fname;
	break;
    }

    desc_fname = data_params.desc_fname;
    /* npatterns is set with data_fname */
    ninputs = select_params.ninputs;
    features = select_params.features;

    normalize = norm_params.normalize;
    norm_fname = norm_params.norm_fname;
    if(lnk_params.action == CROSS_VALID){
	if(lnk_params.cv_from_file){
	    cross_valid = 0;
	    cross_valid_from_file = 1;
	    cross_valid_fname = lnk_params.cv_filename;
	} else {
	    cross_valid_from_file = 0;
	    cross_valid = lnk_params.cv_nfolds;
	    cv_random = lnk_params.cv_random;
	}
    } else {
	cross_valid = 0;
	cross_valid_from_file = 0;
	cv_random = 0;
	cross_valid_fname = NULL;
    }
    random_flag = lnk_params.random_flag;
    seed = lnk_params.seed;
    
    adjust_train = priors_params.adjustTrain;
    if(adjust_train){
	priors_npatterns = priors_params.priorsNpatterns;
	train_uniform_priors = priors_params.trainUniform;
	if(train_uniform_priors){
	    train_priors_list = NULL;
	} else {
	    train_priors_list = priors_params.trainPriorsList;
	}
    }
    adjust_test = priors_params.adjustTest;
    if(adjust_test){
	test_uniform_priors = priors_params.testUniform;
	if(test_uniform_priors){
	    test_priors_list = NULL;
	} else {
	    test_priors_list = priors_params.testPriorsList;
	}
    }

    verbosity = reports_params.verbosity;
    if(which_data == TRAIN_TRAIN)
	verror = reports_params.no_train_verror ? 0 : reports_params.verror;
    else
      verror = reports_params.verror;

    /* we don't set this anywhere yet */
    log_outputs = 0;
}

void
store_command_head(shell_file)
    FILE *shell_file;
{
    if(reports_params.verbosity > 1)
      fprintf(shell_file, "(time ");
    fprintf(shell_file, "%s_lnk\\\n %s",
	    lnk_params.classifier_name, g_get_gen_class_args());
}

void
store_command_end(shell_file)
     FILE *shell_file;
{
    if(reports_params.verbosity > 1)
      fprintf(shell_file, ")");
    if(lnk_params.store_only){
	fprintf(shell_file, ">");
	if(lnk_params.append_flag) fprintf(shell_file, ">");
	fprintf(shell_file, "& %s\n", reports_params.log_fname);
    } else {
	fprintf(shell_file, "|& nn_tee -h ");
	if(lnk_params.append_flag) fprintf(shell_file, "-a ");
	fprintf(shell_file, "%s\n", reports_params.log_fname);
    }
}

char *
g_get_gen_class_args()
{
    return nnargs_get_part_flags(gen_tab, GEN_TABSIZE);
}

static ARG import_tab[] =
{
    {"train_uniform_priors", BOOLEAN, &train_uniform_priors,
      "if adjusting train priors, make class probabilities uniform"},
    {"train_priors_list", FLOAT_ALLOC_LIST, (int *)&train_priors_list,
       "class probabilities list for sampling in training"},
    {"test_uniform_priors", BOOLEAN, &test_uniform_priors,
       "if adjusting priors in testing, make class probabilities uniform"},
    {"test_priors_list", FLOAT_ALLOC_LIST, (int *)&test_priors_list,
       "class probabilities list for scaling in testing"},
    {"features", INT_ALLOC_LIST, (int *)&features,
       "features of normalized data to use"},
};
#define IMPORT_TABSIZE (sizeof(import_tab)/sizeof(ARG))

char *
g_get_important_args()
{
    return nnargs_get_flags_one_line(import_tab, IMPORT_TABSIZE);
}

/***---------------------------------------------------------------------------
* NAME:		diff_screen
* PURPOSE:	find differences between two screen files, printing them
*		to the FILE p;
* EXAMPLE:
*	FILE *p;
*	diff_screen(p, "X1mlp.screen", "LNKnet.note.screen" );
* NOTE:
--------------------------------------------------------------------------***/
void
diff_screen(fp, fname1, fname2)
     FILE *fp;
     char *fname1;
     char *fname2;
{
    FILE *p1, *p2;
    char *argv1[256], *argv2[256];
    int argc1, argc2, i;

    p1 = nn_fopen(fname1, "r", "in diff screen");
    p2 = nn_fopen(fname2, "r", "in diff screen");

    while (1){
	/* while there are commands left in the files */
	argc1 = fill_argv(argv1, p1, fname1);
	if (argc1 == EOF) break;
	/* copy things in argv1 to make it safe to call fill_argv again */
	for(i = 0; i < argc1; i++){
	    if((argv1[i] = nn_strdup(argv1[i])) == NULL)
	      fatal_error("%s: Cannot allocate space while copying argv1",
			  "diff_screen");
	}
	/* add a final argument to help the checking */
	if((argv1[argc1] = nn_strdup("FINAL_MATCH")) == NULL)
	  fatal_error("%s: Cannot allocate space while copying argv1",
		      "diff_screen");
	argc1++;

	argc2 = fill_argv(argv2, p2, fname2);
	if (argc2 == EOF) break;
	/* copy things in argv2 out of habit */
	for(i = 0; i < argc2; i++){
	    if((argv2[i] = nn_strdup(argv2[i])) == NULL)
	      fatal_error("%s: Cannot allocate space while copying argv1",
			  "diff_screen");
	}
	/* add a final argument to help the checking */
	if((argv2[argc2] = nn_strdup("FINAL_MATCH")) == NULL)
	  fatal_error("%s: Cannot allocate space while copying argv2",
		      "diff_screen");
	argc2++;

	/* find the differences between the arglists */
	if(strcmp(argv1[0], argv2[0])){
	    /* the argument names don't match. there is a problem with
	       the log file */
	    fprintf(fp, "log file commands out of synch %s %s",
		    argv1[0], argv2[0]);
	    for (i = 0; i < argc1; i++) free(argv1[i]);
	    for (i = 0; i < argc2; i++) free(argv2[i]);
	    break;
	} else {
	    if (notebook_important_change(argv1[0]))
	      diff_argv(fp, argc1, argv1, argc2, argv2);
	    /* I'd like to put a check in here to see if this is one
	       of the arg tables we actally care about */
	    /* free argv strings that we dup'ed */
	    for (i = 0; i < argc1; i++) free(argv1[i]);
	    for (i = 0; i < argc2; i++) free(argv2[i]);
	}
    }
    nn_fclose(p1, fname1, "in diff_screen");
    nn_fclose(p2, fname2, "in diff_screen");
}
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Names: read_main_defaults, write_main_defaults
 *
 * Purpose: 
 *   Handle reading and writing the state of the main window
 *   to a screen/defaults file.
 * 
 * Calling Sequence:
 *   # open defaults file for reading
 *   # read arg tables up until the LNKnet one
 *   # fill arg list for main one
 *   read_main_defaults(argc, argv)
 *
 *  OR
 *   # open a defaults file for writings
 *   # write arg tables up until the LNKnet one
 *   write_main_defaults(default_file)
 *
 * Inputs:
 *   int argc; # the number of strings in argv
 *   char **argv; # pointer to an array String pointers
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   read sets values in lnk_params structure according to flags found in argv
 *   write writes flags to default_file according to the settings of
 *   lnk_params structure entries
 *
 * Restrictions:
 *   On read, if the argument list in argv doesn't match the arguments
 *   in the LNKnet argument table, a fatal error will occur.
 *
 * Modification History:
 *  23-Aug-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */

static ARG maintab[] =
{
    {"store_only", FLAG, (int *)NULL, ""},
    {"action", INT, (int *)NULL, "0-TRAIN,1-TEST,2-TRAIN&TEST,3-CV"},
    {"test_select", INT, (int *)NULL, "0-train,1-eval,2-test"},
    {"cv_nfolds", INT, (int *)NULL, ""},
    {"cv_from_file", FLAG, (int *)NULL, ""},
    {"cv_random", FLAG, (int *)NULL, ""},
    {"ext_cv", STRING, (int *)NULL, ""},
    {"random", FLAG, (int *)NULL, ""},
    {"algorithm", INT, (int *)NULL, "MLP,RBF,IRBF,GAUSS,GMIX,HISTOGRAM,PARZEN,KNN,CKNN,NC_CLASS,LVQ,BINTREE,HYPER,NBAYES,SVM,COMMITTEE,ARTMAP,KMEANS,EM_CLUS,LEAD_CLUS,RAN_CLUS"},
    {"exp_name", STRING, (int *)NULL, ""},
    {"seed", INT, (int *)NULL, ""},
};
#define TABSIZE (sizeof(maintab)/sizeof(ARG))

static void
main_set_defaults_variables(LnkParams *lp)
{
    nnargs_set_variable("store_only", &lp->store_only, maintab, TABSIZE);
    nnargs_set_variable("action", (int *)&lp->action, maintab, TABSIZE);
    nnargs_set_variable("test_select", (int *)&lp->test_select,
			maintab, TABSIZE);
    nnargs_set_variable("cv_nfolds", &lp->cv_nfolds, maintab, TABSIZE);
    nnargs_set_variable("cv_from_file", &lp->cv_from_file, maintab, TABSIZE);
    nnargs_set_variable("cv_random", &lp->cv_random, maintab, TABSIZE);
    nnargs_set_variable("ext_cv", (int *)&lp->ext_cv, maintab, TABSIZE);
    nnargs_set_variable("random", &lp->random_flag, maintab, TABSIZE);
    nnargs_set_variable("algorithm", (int *)&lp->classifier_num,
			maintab, TABSIZE);
    nnargs_set_variable("exp_name", (int *)&lp->exp_prefix, maintab, TABSIZE);
    nnargs_set_variable("seed", &lp->seed, maintab, TABSIZE);
}

void
read_main_defaults(int argc, char **argv)
{
    /* set variables that are not included if they are NULL or 0 */
    if(lnk_params.exp_prefix != NULL){
	XtFree(lnk_params.exp_prefix);
	lnk_params.exp_prefix = NULL;
    }
    if(lnk_params.ext_cv != NULL){
	XtFree(lnk_params.ext_cv);
	lnk_params.ext_cv = NULL;
    }
    lnk_params.store_only = 0;
    lnk_params.cv_from_file = 0;
    lnk_params.cv_random = 0;
    lnk_params.random_flag = 0;

    main_set_defaults_variables(&lnk_params);

    if(argc > 0){
	argc = nnargs(argc, argv, maintab, TABSIZE);
	lnk_params.exp_prefix = XtNewString(lnk_params.exp_prefix);
	lnk_params.ext_cv = XtNewString(lnk_params.ext_cv);
    } else {
	/* set some defaults for initial startup */
	lnk_params.random_flag = 1;	/* turn on randomization */
	lnk_params.classifier_num = 0;	/* MLP, usually */
	lnk_params.ext_cv = XtNewString(".cv");
	lnk_params.cv_nfolds = 4;
	lnk_params.exp_prefix = XtNewString("X1");
	lnk_params.cv_random = 1;
    }
}

void
write_main_defaults (FILE *default_file)
{
    main_set_defaults_variables(&lnk_params);

    if(default_file != NULL){
	fprintf (default_file, "LNKnet %s",
		 nnargs_get_flags(maintab, TABSIZE));
    }
}

int
store_commands (String filename, RUN_TYPE run_flag)
{
    FILE *shell_file;

    shell_file = command_file_open (LnkClone->lnkForm, filename);
    if(shell_file == NULL) return -1;
    
    set_temp_exp_path(shell_file);

    /* store train commands */
    if (lnk_params.action == TRAIN || lnk_params.action == TRAIN_TEST) {
	lnk_params.train_flag = TRUE;
	set_common_flags(TRAIN_TRAIN);
	if(run_flag != PLOT_ONLY){
	    /* setup for notebook */
	    if(plots_params.epochPlots){
		if(store_epochs_train(shell_file, run_flag)
		   == 0){
		    /* not enough epochs to do more than one training run */
		    store_commands_specific (shell_file, run_flag);
		    if(alg_is_epoch_class() && reports_params.notebook_on)
		      store_train_note_command(shell_file,
					       reports_params.log_fname,
					       reports_params.shell_fname,
					       reports_params.notebook_path);
		}
	    } else
	      {
		  fprintf(shell_file, "\n#train\n");
		  store_commands_specific (shell_file, run_flag);
		  if(alg_is_epoch_class() && reports_params.notebook_on)
		    store_train_note_command(shell_file,
					     reports_params.log_fname,
					     reports_params.shell_fname,
					     reports_params.notebook_path);
	      }
	}
	lnk_params.append_flag = TRUE;
	if(plots_params.doTrainPlots){
	    if(lnk_params.store_only)
	      store_alg_plot_commands(shell_file, NO_GRAPHICS, TRAIN_TRAIN);
	    else 
	      store_alg_plot_commands(shell_file, SHOW_GRAPHICS, TRAIN_TRAIN);
	}
	store_train_plot_commands(shell_file);
    }
    lnk_params.train_flag = FALSE;

    /* store test commands */
    if(lnk_params.action == TEST || lnk_params.action == TRAIN_TEST) {
	lnk_params.create_flag = TRUE;
	set_common_flags(lnk_params.test_select);
	if(run_flag != PLOT_ONLY) fprintf(shell_file, "\n#test\n");
	store_commands_specific (shell_file, run_flag);
	lnk_params.append_flag = TRUE;
	/* store command to get result */
	if(run_flag != PLOT_ONLY){
	    if(reports_params.notebook_on)
	      store_test_note_command(shell_file, reports_params.log_fname,
				      reports_params.shell_fname,
				      reports_params.notebook_path);
	}
	/* 1 is for show_graphics, not shown with epoch plots */
	if(lnk_params.store_only)
	  store_alg_plot_commands(shell_file, NO_GRAPHICS,
				  lnk_params.test_select);
	else
	  store_alg_plot_commands(shell_file, SHOW_GRAPHICS,
				  lnk_params.test_select);
	store_test_plot_commands(shell_file);
    }
    /* store cross valid commands */
    if(lnk_params.action == CROSS_VALID){
	lnk_params.create_flag = TRUE;
	set_common_flags(CV_TRAIN);
	if(run_flag != PLOT_ONLY) fprintf(shell_file, "\n#cross validation\n");
	if((lnk_params.cv_nfolds > 1) || lnk_params.cv_from_file){
	    store_commands_specific (shell_file, run_flag);
	    /* store command to get result */
	    if(run_flag != PLOT_ONLY){
		if(reports_params.notebook_on)
		  store_cv_note_command(shell_file, reports_params.log_fname,
					reports_params.shell_fname,
					reports_params.notebook_path);
	    }
	    if(lnk_params.store_only)
	      store_alg_plot_commands(shell_file, NO_GRAPHICS,
				      lnk_params.test_select);
	    else
	      store_alg_plot_commands(shell_file, SHOW_GRAPHICS,
				      lnk_params.test_select);
	    store_test_plot_commands(shell_file);
	}
    }

    restore_exp_path();
    
    /* tell the log file where we are */
    /* telling sooner risks the log file being overwritten */
    fprintf(shell_file, "echo \"current directory:\" >> %s\n",
	    reports_params.log_fname);
    fprintf(shell_file, "echo $loc >> %s\n", reports_params.log_fname);
    
    command_file_close (shell_file, filename, "for run shell file");

    /* store screen file */
    write_defaults(reports_params.screen_path);
    write_notebook_entry(run_flag);

    return 0;
}

void
store_commands_specific (FILE *shell_file, RUN_TYPE run_flag)
{
    char *temp_fname;
    int clus_alg, temp_alg;
    
    if(run_flag != PLOT_ONLY){
	/* store clustering commands */
	if(must_store_clustering() &&
	   ((lnk_params.train_flag && lnk_params.create_flag) ||
	    cross_valid > 1)){

	    /* set param_fname and algorithm */
	    temp_fname = param_fname;
	    param_fname = g_get_cluster_fname();
	    temp_alg = g_get_alg();
	    clus_alg = g_get_cluster_alg();
	    g_set_alg(clus_alg);
	    
	    store_command_head(shell_file);
	    fprintf(shell_file, "%s", g_get_alg_args());
	    store_command_end(shell_file);
	    
	    /* reset param_fname and algorithm */
	    param_fname = temp_fname;
	    g_set_alg(temp_alg);
	    lnk_params.append_flag = TRUE;
	}
	/* store train, test, or cv command */
	store_command_head(shell_file);
	fprintf(shell_file, "%s", g_get_alg_args());
	store_command_end(shell_file);
    }
}

void
write_notebook_entry(RUN_TYPE run_flag)
{
    FILE *fp;
    String command;
    String note_screen_fname;
    int status;

    if(reports_params.notebook_on == 0) return;
    if(run_flag == PLOT_ONLY) return;
    /* add changes to the notebook file */
    fp = fopen(reports_params.notebook_path, "a");
    if(fp == NULL){
	String entry;
	entry = XtNewString("LNKnet: cannot open notebook file ");
	entry = StringConcat(entry, reports_params.notebook_path);
	warningMessageDialog(LnkClone->lnkTopLevelShell, entry);
	XtFree(entry);
	return;
    }
    /* setup for screen_fname */
    note_screen_fname = XtNewString(reports_params.notebook_fname);
    note_screen_fname = StringConcat(note_screen_fname, ".screen");

    /* write the shell script name as the start of any line */
    fprintf(fp, "\n%s ", reports_params.shell_fname);
    if(reports_params.major_note){
	char *g_get_alg_args();

	/* print out the important things:
	   experiment name (done)
	   data base selection
	   important args from current classifier */
	fprintf(fp, "%s ", data_params.data_base );
	fprintf(fp, "%s ",
		(norm_params.normalize == NONE)? "none" :
		(norm_params.normalize == SIMPLE) ? "simple" :
		(norm_params.normalize == PCA) ? "pca" : "lda");
	fprintf(fp, "%s ", g_get_important_args());

	if(select_params.ninputs != data_params.nraw)
	  fprintf(fp, "-ninputs %d ", select_params.ninputs);

	/* differences in these three flags will not be caught later */
	if(lnk_params.action == CROSS_VALID){
	    if(lnk_params.cv_nfolds > 1)
	      fprintf(fp, "-cross_valid %d %s", lnk_params.cv_nfolds,
		      lnk_params.cv_random ? "-cv_random " : "");
	    else if(lnk_params.cv_from_file)
	      fprintf(fp, "-cross_valid_from_file -fcross_valid %s ",
		      lnk_params.cv_filename);
	}
	if(plots_params.epochPlots)
	  fprintf(fp, "-movie %d ", plots_params.ePerPlot);
	if(lnk_params.continue_flag) fprintf(fp, "-continue ");

	if(must_store_clustering()){
	    char *temp_fname,  *g_get_cluster_fname();
	    int clus_alg, temp_alg;
	    
	    /* temporarily set the algorithm */
	    temp_fname = param_fname;
	    param_fname = g_get_cluster_fname();
	    temp_alg = g_get_alg();
	    clus_alg = g_get_cluster_alg();
	    g_set_alg(clus_alg);
	    
	    fprintf(fp, " %s_lnk ", lnk_params.classifier_name);
	    fprintf(fp, "%s", g_get_important_alg_args());

	    /* and set it back */
	    param_fname = temp_fname;
	    g_set_alg(temp_alg);
	}
	fprintf(fp, " %s_lnk ", lnk_params.classifier_name);
	fprintf(fp, "%s", g_get_important_alg_args());
    } else { /* this is a change, not a new experiment */
	/* make sure there is a notebook log file */
	if (file_exists (note_screen_fname)){
	    if(lnk_params.continue_flag) fprintf(fp, "-continue ");
	    diff_screen(fp, reports_params.screen_path, note_screen_fname);
	}
    }
    fprintf(fp, "\n");
    nn_fclose(fp, reports_params.notebook_path,
	      "to append to the notebook file");
    /* copy the current screen file into the notebook screen file */
    command = XtNewString("cp ");
    command = StringConcat(command, reports_params.screen_path);
    command = StringConcat(command, " ");
    command = StringConcat(command, note_screen_fname);
    system(command);
    XtFree(command);
    XtFree(note_screen_fname);
    wait(&status);
    reports_request("reset_major_note");
}

/* pick out which screen file lines are important to the notebook */
/* only the algorithm and select lines are here. changes to
   any of the other lines (data, main, norm) would have triggered
   a major_note and we wouldn't be doing this */
Boolean
notebook_important_change(argv0)
     char *argv0;
{
    if(!strcmp(argv0, "select") ||
       !strcmp(argv0, lnk_params.classifier_name) ||
       (alg_uses_clustering() &&
	!strcmp(argv0, classifier_names[g_get_cluster_alg()]))) return 1;
    else return 0;
}

static int
store_epochs_train(FILE *shell_file, int run_flag)
{
    int epochs;
    
    /* get the number of epochs we are supposed to do */
    epochs = g_get_epochs();
    /* if we can only do one pass, do it normally */
    if(epochs <= plots_params.ePerPlot) return 0;
    /* if e_per_plot set wrong, it causes a core dump */
    if(plots_params.ePerPlot <= 0) return 0;
    
    /* set up variables in the shell file */
    fprintf(shell_file, "\n#initialize plot file names and numbers\n");
    setup_epoch_plot_calls(shell_file);
    fprintf(shell_file, "\n#initialize total number of epochs\n");
    fprintf(shell_file, "set epochs_left = %d\n", epochs);
    
    /* set variables to do the first training run */
    g_set_epochs(plots_params.ePerPlot);
    
    fprintf(shell_file, "\n# training and plots for first movie frame\n");
    store_commands_specific(shell_file, run_flag);
    /* 0 is for show_graphics, not shown with epoch plots */
    if(plots_params.doTrainPlots)
      store_alg_plot_commands(shell_file, NO_GRAPHICS, TRAIN_TRAIN);
    
    /* the rest of the training will be done in continue mode */
    lnk_params.create_flag = FALSE;
    create_flag = FALSE;
    lnk_params.append_flag = TRUE;
    
    /* create a call to display the set of plots */
    fprintf(shell_file, "\n# start list of plot numbers for movie\n");
    fprintf(shell_file, "set plotnum_list = $plotnum\n");
    fprintf(shell_file, "@ plotnum++\n");
    
    fprintf(shell_file, "\n# decrease epochs left \n");
    fprintf(shell_file, "@ epochs_left -= %d\n", plots_params.ePerPlot);
    
    /* loop through most of the training epochs */
    fprintf(shell_file, "\n######## start movie mode loop #############\n");
    fprintf(shell_file, "while($epochs_left > %d)\n", plots_params.ePerPlot);
    fprintf(shell_file, "set bpfname = \"$bplotfile$plotnum\"\n");
    fprintf(shell_file, "set ppfname = \"$pplotfile$plotnum\"\n");
    
    fprintf(shell_file, "\n# train\n");
    store_commands_specific(shell_file, run_flag);
    /* 0 is for show_graphics, not shown with epoch plots */
    if(plots_params.doTrainPlots)
      store_alg_plot_commands(shell_file, NO_GRAPHICS, TRAIN_TRAIN);
    

    fprintf(shell_file, "\n# update list of plot numbers for movie\n");
    fprintf(shell_file, "set plotnum_list = $plotnum_list,$plotnum\n");
    fprintf(shell_file, "@ plotnum++\n");

    fprintf(shell_file, "\n# decrease epochs left\n");
    fprintf(shell_file, "@ epochs_left -= %d\n", plots_params.ePerPlot);
    fprintf(shell_file, "end\n");
    fprintf(shell_file, "\n######## end movie mode loop ################\n");
    
    /* do the last training call, which may want fewer epochs */
    fprintf(shell_file, "\n# do last %d epochs of training \n",
	    plots_params.ePerPlot);
    fprintf(shell_file, "set bpfname = \"$bplotfile$plotnum\"\n");
    fprintf(shell_file, "set ppfname = \"$pplotfile$plotnum\"\n");
    
    /* calculate the number of epochs remaining (e_per_plot or less) */
    /* e_per_plot must be > 0 or this dumps core */
    if(epochs%plots_params.ePerPlot != 0)
      g_set_epochs(epochs%plots_params.ePerPlot);
    fprintf(shell_file, "\n#train\n");
    store_commands_specific(shell_file, run_flag);

    /* store the command in the notebook file */
    if(alg_is_epoch_class() && reports_params.notebook_on){
	char *note_command;
	char string[10];
	note_command = XtNewString(reports_params.shell_fname);
	note_command = StringConcat(note_command, " MOVIE EPOCH: ");
	sprintf(string,"%d",epochs);
	note_command = StringConcat(note_command, string);
	store_train_note_command(shell_file, reports_params.log_fname,
				 note_command,
				 reports_params.notebook_path);
	XtFree(note_command);
    }

    /* 0 is for show_graphics, not shown with epoch plots */
    if(plots_params.doTrainPlots)
      store_alg_plot_commands(shell_file, NO_GRAPHICS, TRAIN_TRAIN);
    
    fprintf(shell_file, "\n#add last plot number to list\n");
    fprintf(shell_file, "set plotnum_list = $plotnum_list,$plotnum\n");
    
    /* display the plots */
    fprintf(shell_file, "# display movie frames in olxplot windows\n");
    finish_epoch_plot_calls(shell_file);
    g_set_epochs(epochs);
    
    return(1);
}
