/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/lvq/lvq_callback.c,v 11.4 2001/08/23 20:16:57 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                                 LVQ                                  ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  7-Nov-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include "../main/lnknet_subs.h"
#include "lvq.h"
#include <nnlib.h>
#include <lvq_args.h>
/*
 * External Functions
 */
#include "../plots/plots_func.h"
#include "../kmeans/kmeans_func.h"
#include "../em_clus/em_clus_func.h"
#include "../lead_clus/lead_clus_func.h"
#include "../ran_clus/ran_clus_func.h"

/*
 * Externals
 */
extern LvqWidgets *LvqClone;
#include "lvq_params.h"
LvqParams lvq_params; /* this is the real one */
#include "../lnk/lnk_params.h"
LnkParams lnk_params;

/*
 * Statics
 */
static void lvq_set_lvq_type_graying();
static void lvq_set_clustering_alg(int);
static void lvq_set_help_text();
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: lvq_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   lvq_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  7-Nov-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
lvq_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    LvqWidgets *ip;
    static int nEpochs_buttons_mapped = False;
    static int alpha_buttons_mapped = False;
    static int window_buttons_mapped = False;
    static int epsilon_buttons_mapped = False;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "createClustersToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "lvq_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	lvq_params.createClusters = state->set;
 
    } else if (!strcmp(client, "kmeansPBtn")) {
	/*fprintf (stderr, "lvq_callback: %s\n", client );*/
	lvq_set_clustering_alg(KMEANS);

    } else if (!strcmp(client, "em_clusPBtn")) {
	/*fprintf (stderr, "lvq_callback: %s\n", client );*/
	lvq_set_clustering_alg(EM_CLUS);
 
    } else if (!strcmp(client, "lead_clusPBtn")) {
	/*fprintf (stderr, "lvq_callback: %s\n", client );*/
	lvq_set_clustering_alg(LEAD_CLUS);
 
    } else if (!strcmp(client, "ran_clusPBtn")) {
	/*fprintf (stderr, "lvq_callback: %s\n", client );*/
	lvq_set_clustering_alg(RAN_CLUS);
 
    } else if (!strcmp(client, "clustParamsPBtn")) {
	/*fprintf (stderr, "lvq_callback: %s\n", client );*/
	switch(lvq_params.clusterNum){
	  case KMEANS:
	    kmeans_request("open");
	    break;
	  case EM_CLUS:
	    em_clus_request("open");
	    break;
	  case LEAD_CLUS:
	    lead_clus_request("open");
	    break;
	  case RAN_CLUS:
	    ran_clus_request("open");
	    break;
	}
 
    } else if (!strcmp(client, "olvqToggle") ||
	       !strcmp(client, "lvq1Toggle") ||
	       !strcmp(client, "lvq2Toggle") ||
	       !strcmp(client, "lvq3Toggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "lvq_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set){
	    lvq_params.lvqType =
	      get_radio_value(ip->lvqTypeRadioBox);
	    lvq_set_lvq_type_graying();
	    set_plot_labels();
	    lvq_set_help_text();
	}

    } else if (!strcmp(client, "nEpochsText")) {
	/*fprintf (stderr, "lvq_callback: %s\n", client );*/
	nEpochs_buttons_mapped =
	  okTextValueChanged(ip->nEpochsText, ip->nEpochsOKPBtn,
			     ip->nEpochsCancelPBtn,
			     ip->lvqDialogShell,
			     nEpochs_buttons_mapped,
			     lvq_params.nEpochs_string);
	
    } else if ((!strcmp(client, "nEpochsTextAct")) ||
	       (!strcmp(client, "nEpochsOKPBtn"))) {
	String nEpochsTextval;

	/*fprintf (stderr, "lvq_callback: %s\n", client );*/
	nEpochs_buttons_mapped =
	  okTextActivate(ip->nEpochsText, ip->nEpochsOKPBtn,
			 ip->nEpochsCancelPBtn, &nEpochsTextval);
	/* I won't actually be using this copy */
	XtFree (nEpochsTextval);
	
	/* verify the string as an integer */
	lvq_params.nEpochs =
	  set_atoi_value(ip->nEpochsText, "nEpochs_set",
			     lvq_params.nEpochs);

	/* save the current text for future comparisons */
	XtFree(lvq_params.nEpochs_string);
	lvq_params.nEpochs_string = get_string_text(ip->nEpochsText);
 
	/* do any other things needed */
	set_plot_labels();

    } else if (!strcmp(client, "nEpochsCancelPBtn")) {
	/*fprintf (stderr, "lvq_callback: %s\n", client );*/
	nEpochs_buttons_mapped = 
	  okTextCancel(ip->nEpochsText, ip->nEpochsOKPBtn,
		       ip->nEpochsCancelPBtn,
		       lvq_params.nEpochs_string);

 
    } else if (!strcmp(client, "alphaText")) {
	/*fprintf (stderr, "lvq_callback: %s\n", client );*/
	alpha_buttons_mapped =
	  okTextValueChanged(ip->alphaText, ip->alphaOKPBtn,
			     ip->alphaCancelPBtn,
			     ip->lvqDialogShell,
			     alpha_buttons_mapped,
			     lvq_params.alpha_string);
	
    } else if ((!strcmp(client, "alphaTextAct")) ||
	       (!strcmp(client, "alphaOKPBtn"))) {
	String alphaTextval;

	/*fprintf (stderr, "lvq_callback: %s\n", client );*/
	alpha_buttons_mapped =
	  okTextActivate(ip->alphaText, ip->alphaOKPBtn,
			 ip->alphaCancelPBtn, &alphaTextval);
	/* I won't actually be using this copy */
	XtFree (alphaTextval);
	
	/* verify the string as a float */
	lvq_params.alpha =
	  set_atof_range_value(ip->alphaText, "alpha_set",
			       lvq_params.alpha, 0., 1.);

	/* save the current text for future comparisons */
	XtFree(lvq_params.alpha_string);
	lvq_params.alpha_string = get_string_text(ip->alphaText);
 
	/* do any other things needed */
	set_plot_labels();

    } else if (!strcmp(client, "alphaCancelPBtn")) {
	/*fprintf (stderr, "lvq_callback: %s\n", client );*/
	alpha_buttons_mapped = 
	  okTextCancel(ip->alphaText, ip->alphaOKPBtn,
		       ip->alphaCancelPBtn,
		       lvq_params.alpha_string);
	
    } else if (!strcmp(client, "windowText")) {
	/*fprintf (stderr, "lvq_callback: %s\n", client );*/
	window_buttons_mapped =
	  okTextValueChanged(ip->windowText, ip->windowOKPBtn,
			     ip->windowCancelPBtn,
			     ip->lvqDialogShell,
			     window_buttons_mapped,
			     lvq_params.window_string);
	
    } else if ((!strcmp(client, "windowTextAct")) ||
	       (!strcmp(client, "windowOKPBtn"))) {
	String windowTextval;

	/*fprintf (stderr, "lvq_callback: %s\n", client );*/
	window_buttons_mapped =
	  okTextActivate(ip->windowText, ip->windowOKPBtn,
			 ip->windowCancelPBtn, &windowTextval);
	/* I won't actually be using this copy */
	XtFree (windowTextval);
	
	/* verify the string as a float */
	lvq_params.window =
	  set_atof_range_value(ip->windowText, "window_set",
			       lvq_params.window, 0., 1.);

	/* save the current text for future comparisons */
	XtFree(lvq_params.window_string);
	lvq_params.window_string = get_string_text(ip->windowText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "windowCancelPBtn")) {
	/*fprintf (stderr, "lvq_callback: %s\n", client );*/
	window_buttons_mapped = 
	  okTextCancel(ip->windowText, ip->windowOKPBtn,
		       ip->windowCancelPBtn,
		       lvq_params.window_string);
	
    } else if (!strcmp(client, "epsilonText")) {
	/*fprintf (stderr, "lvq_callback: %s\n", client );*/
	epsilon_buttons_mapped =
	  okTextValueChanged(ip->epsilonText, ip->epsilonOKPBtn,
			     ip->epsilonCancelPBtn,
			     ip->lvqDialogShell,
			     epsilon_buttons_mapped,
			     lvq_params.epsilon_string);
	
    } else if ((!strcmp(client, "epsilonTextAct")) ||
	       (!strcmp(client, "epsilonOKPBtn"))) {
	String epsilonTextval;

	/*fprintf (stderr, "lvq_callback: %s\n", client );*/
	epsilon_buttons_mapped =
	  okTextActivate(ip->epsilonText, ip->epsilonOKPBtn,
			 ip->epsilonCancelPBtn, &epsilonTextval);
	/* I won't actually be using this copy */
	XtFree (epsilonTextval);
	
	/* verify the string as a float */
	lvq_params.epsilon =
	  set_atof_range_value(ip->epsilonText, "epsilon_set",
			     lvq_params.epsilon, 0., 1.);

	/* save the current text for future comparisons */
	XtFree(lvq_params.epsilon_string);
	lvq_params.epsilon_string = get_string_text(ip->epsilonText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "epsilonCancelPBtn")) {
	/*fprintf (stderr, "lvq_callback: %s\n", client );*/
	epsilon_buttons_mapped = 
	  okTextCancel(ip->epsilonText, ip->epsilonOKPBtn,
		       ip->epsilonCancelPBtn,
		       lvq_params.epsilon_string);
	
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "lvq_callback: %s\n", client );*/
	close_popup(ip->lvqForm);
 
    } else {
	fprintf(stderr, "lvq_callback: unknown client: %s\n", (char *)client);
    }
}

void
lvq_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(LvqClone->lvqForm);
    } else if (!strcmp(request, "init")){
	lvq_params.createClusters = 0;
	lvq_params.clusterNum = KMEANS;
	lvq_params.clustName = "kmeans";
	lvq_params.clustFname = NULL;
	lvq_params.lvqType = 0;
	lvq_params.nEpochs = 0;
	lvq_params.nEpochs_string = NULL;
	lvq_params.alpha = 0.;
	lvq_params.alpha_string = NULL;
	lvq_params.window = 0.;
	lvq_params.window_string = NULL;
	lvq_params.epsilon = 0.;
	lvq_params.epsilon_string = NULL;
    } else if (!strcmp(request, "display")){
	char string[10];
	show_toggle_value(LvqClone->createClustersToggle,
			  lvq_params.createClusters);
	switch(lvq_params.clusterNum){
	  case KMEANS:
	    show_option_value(LvqClone->clustAlgOptionMenu, 0);
	    break;
	  case EM_CLUS:
	    show_option_value(LvqClone->clustAlgOptionMenu, 1);
	    break;
	  case LEAD_CLUS:
	    show_option_value(LvqClone->clustAlgOptionMenu, 2);
	    break;
	  case RAN_CLUS:
	    show_option_value(LvqClone->clustAlgOptionMenu, 3);
	    break;
	}
	show_string_label(LvqClone->clustFnameLabel, lvq_params.clustFname);

	show_radio_value(LvqClone->lvqTypeRadioBox, lvq_params.lvqType);
	lvq_set_lvq_type_graying();
	lvq_set_help_text();

	XtFree(lvq_params.nEpochs_string);
	sprintf(string, "%d", lvq_params.nEpochs);
	lvq_params.nEpochs_string = XtNewString(string);
	show_string_text(LvqClone->nEpochsText,
			 lvq_params.nEpochs_string);

	XtFree(lvq_params.alpha_string);
	sprintf(string, "%5.2g", lvq_params.alpha);
	lvq_params.alpha_string = XtNewString(string);
	show_string_text(LvqClone->alphaText,
			 lvq_params.alpha_string);
	XtFree(lvq_params.window_string);
	sprintf(string, "%5.2g", lvq_params.window);
	lvq_params.window_string = XtNewString(string);
	show_string_text(LvqClone->windowText,
			 lvq_params.window_string);
	XtFree(lvq_params.epsilon_string);
	sprintf(string, "%5.2g", lvq_params.epsilon);
	lvq_params.epsilon_string = XtNewString(string);
	show_string_text(LvqClone->epsilonText,
			 lvq_params.epsilon_string);
    } else {
	fprintf(stderr, "lvq_request: unknown request: %s\n", request);
    }
}

String
lvq_get_plot_label(String string)
{
    char numString[10];
    switch(lvq_params.lvqType){
      case OLVQ:
	string = StringConcat(string, " OLVQ");
	break;
      case LVQ1:
	string = StringConcat(string, " LVQ1");
	break;
      case LVQ2:
	string = StringConcat(string, " LVQ2");
	break;
      case LVQ3:
	string = StringConcat(string, " LVQ3");
	break;
    }
    string = StringConcat(string, " Stepsize:");
    sprintf(numString, "%.2g", lvq_params.alpha);
    string = StringConcat(string, numString);
    string = StringConcat(string, " Epochs:");
    sprintf(numString, "%d", lvq_params.nEpochs);
    string = StringConcat(string, numString);
    
    switch(lvq_params.clusterNum){
      case KMEANS:
	string = kmeans_get_plot_label(string);
	break;
      case EM_CLUS:
	string = em_clus_get_plot_label(string);
	break;
      case LEAD_CLUS:
	string = lead_clus_get_plot_label(string);
	break;
      case RAN_CLUS:
	string = ran_clus_get_plot_label(string);
	break;
    }
    return string;
}

static void
lvq_set_read_variables()
{
    XtFree(lvq_params.clustFname);
    lvq_params.clustFname = XtNewString(cluster_param_fname);
    lvq_params.nEpochs = nepochs;
    lvq_params.alpha = alpha;
    lvq_params.window = window;
    lvq_params.epsilon = epsilon;
    lvq_params.lvqType = lvq_type;
}

static void
lvq_set_write_variables()
{
    cluster_param_fname = lvq_params.clustFname;
    nepochs = lvq_params.nEpochs;
    alpha = lvq_params.alpha;
    window = lvq_params.window;
    epsilon = lvq_params.epsilon;
    lvq_type = lvq_params.lvqType;
}
static ARG lvqtab[] =
{
    {"createClusters", BOOLEAN, (int *)NULL,},
    {"clusterNum", INT, (int *)NULL,},
};
#define TABSIZE (sizeof(lvqtab)/sizeof(ARG))

static void
lvq_set_defaults_variables(dp)
     LvqParams *dp;
{
    nnargs_set_variable("createClusters", (int *)&dp->createClusters, lvqtab, TABSIZE);
    nnargs_set_variable("clusterNum", (int *)&dp->clusterNum, lvqtab, TABSIZE);
}

void
read_lvq_defaults(argc, argv)
     int argc;
     char **argv;
{
    lvq_params.createClusters = 0;
    lvq_type = 0;
    lvq_set_defaults_variables(&lvq_params);
    if(argc > 0){
	argc = nnargs_part(argc, argv, lvqtab, TABSIZE);
	argc = nnargs(argc, argv, lvq_tab, LVQ_TABSIZE);
	lvq_set_read_variables();
	switch(lvq_params.clusterNum){
	  case KMEANS:
	    lvq_params.clustName = "kmeans";
	    break;
	  case EM_CLUS:
	    lvq_params.clustName = "em_clus";
	    break;
	  case LEAD_CLUS:
	    lvq_params.clustName = "lead_clus";
	    break;
	  case RAN_CLUS:
	    lvq_params.clustName = "ran_clus";
	    break;
	}
    } else {
	/* set any defaults */
	lvq_params.createClusters = 1;
	lvq_params.clusterNum = KMEANS;
	lvq_params.nEpochs = 10;
	lvq_params.alpha = 0.1;
	lvq_params.window = 0.01;
	lvq_params.epsilon = 0.01;
	lvq_params.lvqType = 0;
    }

    lvq_request("display");
}

void
write_lvq_defaults(FILE *default_file)
{
    lvq_set_write_variables();
    lvq_set_defaults_variables(&lvq_params);

    if(default_file != NULL){
	fprintf(default_file, "lvq %s",
		nnargs_get_part_flags(lvqtab, TABSIZE));
	fprintf(default_file, "lvq %s",
		nnargs_get_flags(lvq_tab, LVQ_TABSIZE));
    }
}

String
lvq_get_args ()
{
    lvq_set_write_variables();
    return (nnargs_get_part_flags(lvq_tab, LVQ_TABSIZE));
}

String
lvq_get_important_args ()
{
    lvq_set_write_variables();
    return (nnargs_get_flags_one_line(lvq_tab, LVQ_TABSIZE));
}

static void
lvq_set_clustering_alg(value)
     int value;
{
    lvq_params.clusterNum = value;
    switch(lvq_params.clusterNum){
      case KMEANS:
	lvq_params.clustName = "kmeans";
	break;
      case EM_CLUS:
	lvq_params.clustName = "em_clus";
	break;
      case LEAD_CLUS:
	lvq_params.clustName = "lead_clus";
	break;
      case RAN_CLUS:
	lvq_params.clustName = "ran_clus";
	break;
    }
    XtFree(lvq_params.clustFname);
    lvq_params.clustFname = XtNewString(lnk_params.exp_prefix);
    lvq_params.clustFname = StringConcat(lvq_params.clustFname,
					 lvq_params.clustName);
    lvq_params.clustFname = StringConcat(lvq_params.clustFname,
					 ".param");
    show_string_label(LvqClone->clustFnameLabel, lvq_params.clustFname);
    set_plot_labels();
}

void
lvq_set_cluster_param_fname()
{
    if(lvq_params.createClusters){
	XtFree(lvq_params.clustFname);
	lvq_params.clustFname = XtNewString(lnk_params.exp_prefix);
	lvq_params.clustFname = StringConcat(lvq_params.clustFname,
					     lvq_params.clustName);
	lvq_params.clustFname = StringConcat(lvq_params.clustFname,
					     ".param");
	show_string_label(LvqClone->clustFnameLabel, lvq_params.clustFname);
    }
}

char *
lvq_get_cluster_fname(){
    return lvq_params.clustFname;
}

int
lvq_get_cluster_alg(){
    return lvq_params.clusterNum;
}

int
lvq_must_store_clustering(){
    return lvq_params.createClusters;
}

int
lvq_get_epochs(){
    return lvq_params.nEpochs;
}

void lvq_set_epochs(int e)
{
    lvq_params.nEpochs = e;
}

static void
lvq_set_lvq_type_graying()
{
    gray_out_item(LvqClone->windowForm, lvq_params.lvqType < LVQ2);
    gray_out_item(LvqClone->epsilonForm, lvq_params.lvqType < LVQ3);
}

static void
lvq_set_help_text()
{
    XmString xmstring;
    switch(lvq_params.lvqType){
      case OLVQ:
	xmstring = XmStringCreateLtoR("Closest center moves toward pattern if classes match,\naway from pattern otherwise.\nEach center has its own stepsize\n ",
				      XmFONTLIST_DEFAULT_TAG);
	break;
      case LVQ1:
	xmstring = XmStringCreateLtoR("Closest center moves toward pattern if classes match,\naway from pattern otherwise.\nStepsize decreases to 0 over N epochs\n ",
				      XmFONTLIST_DEFAULT_TAG);
	break;
      case LVQ2:
	xmstring = XmStringCreateLtoR("Move the two closest centers if one matches\nthe pattern's class and the other doesn't\nand the distance ratio is inside the window.\nLarger windows let the distance ratio be larger.",
				      XmFONTLIST_DEFAULT_TAG);
	break;
      case LVQ3:
	xmstring = XmStringCreateLtoR("Move the two closest centers if one matches\nthe pattern's class and the other doesn't\nand the distance ratio is inside a window.\nIf both match the class, move with scaled stepsize.",
				      XmFONTLIST_DEFAULT_TAG);
	break;
    }
    XtVaSetValues(LvqClone->algHelpLabel,  XmNlabelString, xmstring, NULL);
    XmStringFree(xmstring);

}
