/*SUPPRESS 592*/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/main/okText.c,v 11.3 2001/08/23 20:17:00 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                      Ok Text Widget Collection                       ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source: /data/speech/kukolich/cvsroot/lnknet/src/mclass/main/okText.c,v $
 *
 * Purpose:
 *   routines for handling mechanics of the okText Form
 *
 * Language: C
 *
 * File Modification History:
 *  13-Sep-95     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include <Xm/Text.h>
/*
 * External Functions
 */
 
/*
 * Externals
 */
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Name:  okTextValueChanged
 *
 * Purpose: 
 *  Called from the value changed callback of the text widget associated
 *  with an okText form to check to see if the ok/cancel buttons should
 *  be mapped (visible).
 *  The buttons should be mapped when:
 *   - not currently mapped
 *   - the dialog or top level shell they are on is realized
 *   - the text has actually changed
 * 
 * Calling Sequence:
 *  buttonsMapped = okTextValueChanged(textWidget, dialogShell,
 *		                       okButton, cancelButton,
 *                                     buttonsMapped, oldText)
 *
 * Inputs:
 *   Widget textWidget;		Text widget in this okText collection
 *   Widget dialogShell;	Parent dialog or Top level shell of okText
 *   Widget okButton;		Ok PushButton widget in collection
 *   Widget cancelButton;	Cancel PushButton widget in collection
 *   int buttonsMapped;		flag for whether ok/cancel are currently mapped
 *   String oldText;		stored value for the string set by text widget
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   whether the buttons are currently mapped
 *   (True for Mapped, still 0 for not)
 *
 * Side Effects:
 *   Changes the okButton and cancelButton to be mapped
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  13-Sep-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
int
okTextValueChanged(Widget textWidget, Widget okButton, Widget cancelButton,
		   Widget dialogShell, int buttonsMapped, String oldText)
{
    String newText = XmTextGetString(textWidget);

    if (!buttonsMapped && XtIsRealized(dialogShell)) {
	if((oldText == NULL) || strcmp(oldText, newText)) {
	    /* 
	     * Only map these widgets if the window is realized (visible),
	     * AND if the textfield value has changed.
	     * Otherwise, X-windows mapping error occurs.
	     */
	    XtMapWidget(okButton);
	    XtMapWidget(cancelButton);
	    buttonsMapped = True;
	}
    }
    XtFree (newText);
    return buttonsMapped;
}
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Name:  okTextactivate
 *
 * Purpose: 
 *  get the text from 
 *
 *  Calling Sequence:
 *  buttonsMapped = okTextactivate(textWidget, 
 *		                   okButton, cancelButton,
 *                                 &newText)
 *
 * Inputs:
 *   Widget textWidget;		Text widget in this okText collection
 *   Widget okButton;		Ok PushButton widget in collection
 *   Widget cancelButton;	Cancel PushButton widget in collection
 *   int buttonsMapped;		flag for whether ok/cancel are currently mapped
 *   String newText;		
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   whether the buttons are currently mapped
 *   (True for Mapped, still 0 for not)
 *
 * Side Effects:
 *   Changes the okButton and cancelButton to be unmapped
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  13-Sep-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
int
okTextActivate(Widget textWidget, Widget okButton, Widget cancelButton,
	       String *newText)
{
    /* save the new value */
    *newText = XmTextGetString(textWidget);

    /* unmap the OK/CANCEL buttons */
    XtUnmapWidget(okButton);
    XtUnmapWidget(cancelButton);
    return False;
}

/* makes ok/cancel buttons unmapped and puts the old text back in */
int
okTextCancel(Widget textWidget, Widget okButton, Widget cancelButton,
	     String oldText)
{
    /* unmap the buttons */
    XtUnmapWidget(okButton);
    XtUnmapWidget(cancelButton);

    /* put the old value back */
    XmTextSetString(textWidget, oldText);
    return False;
}
