/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/mlp/mlpCost_callback.c,v 11.4 2001/08/23 20:17:04 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            MLPCOST                              ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  3-Nov-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include <nnlib.h>
#include "mlpCost.h"
/*
 * External Functions
 */
#include "../main/lnknet_subs.h"
#include "../plots/plots_func.h"
#include "../cost/cost_func.h"

/*
 * Externals
 */
extern MlpCostWidgets *MlpCostClone;
#include "mlpNode.h"
extern MlpNodeWidgets *MlpNodeClone;
#include "mlp.h"
extern MlpWidgets *MlpClone;
#include "mlpWeight.h"
extern MlpWeightWidgets *MlpWeightClone;

#include "mlp_params.h"
extern MlpParams mlp_params;
#include "../select/select_params.h"
extern SelectParams select_params;
#include "../data/data_params.h"
extern DataParams data_params;

/******************************************************************************
 ******************************************************************************
 *
 * Module Name: mlpCost_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   mlpCost_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  3-Nov-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
mlpCost_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    MlpCostWidgets *ip;
    int costParam_buttons_mapped = False;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "rmsErrorToggle") ||
	!strcmp(client, "crossEntToggle") ||
	!strcmp(client, "maxLikeToggle") ||
	!strcmp(client, "pcpToggle") ||
	!strcmp(client, "top2diffToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "mlpCost_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set){
	    mlp_params.costFunc =
	      get_radio_value(ip->costFuncRadioBox);
	    if(mlp_params.costFunc == PC_PROC){
		gray_out_item(MlpNodeClone->ofunctionRadioBox, True);
		gray_out_item(MlpClone->layersForm, True);
		if(mlp_params.layers != NULL && mlp_params.layers[0] >= 2){
		    mlp_params.layers[0] = 2;
		    mlp_params.layers[1] = select_params.ninputs;
		    mlp_params.layers[2] = data_params.noutputs;

		    mlp_params.layers_string =
		      make_int_string(mlp_params.layers);
		    show_string_text(MlpClone->layersText,
				     mlp_params.layers_string);
		}
		gray_out_item(MlpWeightClone->momentumForm, True);
	    } else {
		gray_out_item(MlpNodeClone->ofunctionRadioBox, False);
		gray_out_item(MlpClone->layersForm, False);
		gray_out_item(MlpWeightClone->momentumForm, False);
		if(mlp_params.costFunc == SQ_ERR ||
		   mlp_params.costFunc == CFOM){
		    gray_out_item(MlpNodeClone->ofunctionRadioBox, False);
		} else {
		    gray_out_item(MlpNodeClone->ofunctionRadioBox, True);
		}
	    }
	    gray_out_item(MlpCostClone->costParamForm,
			  (mlp_params.costFunc != CFOM));
	    set_plot_labels();
	    cost_set_cost_func(mlp_params.costFunc);
	}
 
    } else if (!strcmp(client, "costParamText")) {
	/*fprintf (stderr, "mlpCost_callback: %s\n", client );*/
	costParam_buttons_mapped =
	  okTextValueChanged(ip->costParamText, ip->costParamOKPBtn,
			     ip->costParamCancelPBtn,
			     ip->mlpCostDialogShell,
			     costParam_buttons_mapped,
			     mlp_params.costParam_string);
	
    } else if ((!strcmp(client, "costParamTextAct")) ||
	       (!strcmp(client, "costParamOKPBtn"))) {
	String costParamTextval;

	/*fprintf (stderr, "mlpCost_callback: %s\n", client );*/
	costParam_buttons_mapped =
	  okTextActivate(ip->costParamText, ip->costParamOKPBtn,
			 ip->costParamCancelPBtn, &costParamTextval);
	/* I won't actually be using this copy */
	XtFree (costParamTextval);
	
	/* verify the string as a float */
	mlp_params.costParam =
	  set_atof_value(ip->costParamText, "costParam_set",
			 mlp_params.costParam);

	/* save the current text for future comparisons */
	XtFree(mlp_params.costParam_string);
	mlp_params.costParam_string = get_string_text(ip->costParamText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "costParamCancelPBtn")) {
	/*fprintf (stderr, "mlpCost_callback: %s\n", client );*/
	costParam_buttons_mapped = 
	  okTextCancel(ip->costParamText, ip->costParamOKPBtn,
		       ip->costParamCancelPBtn,
		       mlp_params.costParam_string);
 
  } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "mlpCost_callback: %s\n", client );*/
	close_popup(MlpCostClone->mlpCostForm);
 
  } else {
    fprintf(stderr, "mlpCost_callback: unknown client: %s\n", (char *)client);
  }
}

void
mlpCost_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(MlpCostClone->mlpCostForm);
    } else if (!strcmp(request, "init")){
	mlp_params.costFunc = 0;
	mlp_params.costParam = 0.;
	mlp_params.costParam_string = NULL;
    } else if (!strcmp(request, "display")){
	char string[10];
	show_radio_value(MlpCostClone->costFuncRadioBox, mlp_params.costFunc);
	XtFree(mlp_params.costParam_string);
	sprintf(string, "%5.2g", mlp_params.costParam);
	mlp_params.costParam_string = XtNewString(string);
	show_string_text(MlpCostClone->costParamText,
			 mlp_params.costParam_string);
    } else {
	fprintf(stderr, "mlpCost_request: unknown request: %s\n", request);
    }
}
