/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/mlp/mlpNode_callback.c,v 11.4 2001/08/23 20:17:04 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            MLPNODE                              ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  3-Nov-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include <nnlib.h>
#include "mlpNode.h"
/*
 * External Functions
 */
#include "../main/lnknet_subs.h"

/*
 * Externals
 */
extern MlpNodeWidgets *MlpNodeClone;
#include "mlp_params.h"
extern MlpParams mlp_params;

/******************************************************************************
 ******************************************************************************
 *
 * Module Name: mlpNode_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   mlpNode_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  3-Nov-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
mlpNode_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    MlpNodeWidgets *ip;
    int sigmoidParam_buttons_mapped = False;
    int sigmoidList_buttons_mapped = False;
    
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
    
    if (!strcmp(client, "standardOfuncToggle") ||
	!strcmp(client, "symetricOfuncToggle") ||
	!strcmp(client, "linearOfuncToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "mlpNode_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set){
	    mlp_params.ofunction =
	      get_radio_value(ip->ofunctionRadioBox);
	}
      
    } else if (!strcmp(client, "standardHfuncToggle") ||
	       !strcmp(client, "symetricHfuncToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "mlpNode_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set) mlp_params.hfunction =
	  get_radio_value(ip->hfunctionRadioBox);
	
    } else if (!strcmp(client, "sigmoidParamText")) {
	/*fprintf (stderr, "mlpNode_callback: %s\n", client );*/
	sigmoidParam_buttons_mapped =
	  okTextValueChanged(ip->sigmoidParamText, ip->sigmoidParamOKPBtn,
			     ip->sigmoidParamCancelPBtn,
			     ip->mlpNodeDialogShell,
			     sigmoidParam_buttons_mapped,
			     mlp_params.sigmoidParam_string);
	
    } else if ((!strcmp(client, "sigmoidParamTextAct")) ||
	       (!strcmp(client, "sigmoidParamOKPBtn"))) {
	String sigmoidParamTextval;

	/*fprintf (stderr, "mlpNode_callback: %s\n", client );*/
	sigmoidParam_buttons_mapped =
	  okTextActivate(ip->sigmoidParamText, ip->sigmoidParamOKPBtn,
			 ip->sigmoidParamCancelPBtn, &sigmoidParamTextval);
	/* I won't actually be using this copy */
	XtFree (sigmoidParamTextval);
	
	/* verify the string as a float */
	mlp_params.sigmoidParam =
	  set_atof_value(ip->sigmoidParamText, "sigmoidParam_set",
			 mlp_params.sigmoidParam);

	/* save the current text for future comparisons */
	XtFree(mlp_params.sigmoidParam_string);
	mlp_params.sigmoidParam_string = get_string_text(ip->sigmoidParamText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "sigmoidParamCancelPBtn")) {
	/*fprintf (stderr, "mlpNode_callback: %s\n", client );*/
	sigmoidParam_buttons_mapped = 
	  okTextCancel(ip->sigmoidParamText, ip->sigmoidParamOKPBtn,
		       ip->sigmoidParamCancelPBtn,
		       mlp_params.sigmoidParam_string);
 
    } else if (!strcmp(client, "useSigmoidListToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "mlpNode_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	mlp_params.useSigmoidList = state->set;
	gray_out_item(ip->sigmoidListForm, !mlp_params.useSigmoidList);

    } else if (!strcmp(client, "sigmoidListText")) {
	/*fprintf (stderr, "mlpNode_callback: %s\n", client);*/
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	sigmoidList_buttons_mapped =
	  okTextValueChanged(ip->sigmoidListText, ip->sigmoidListOKPBtn,
			     ip->sigmoidListCancelPBtn,
			     ip->mlpNodeDialogShell,
			     sigmoidList_buttons_mapped,
			     mlp_params.sigmoidList_string);
    } else if ((!strcmp(client, "sigmoidListTextAct")) ||
	       (!strcmp(client, "sigmoidListOKPBtn"))) {
	/*fprintf (stderr, "mlpNode_callback: %s\n", client );*/
	String sigmoidListTextval;
	sigmoidList_buttons_mapped =
	  okTextActivate(ip->sigmoidListText,
			 ip->sigmoidListOKPBtn,
			 ip->sigmoidListCancelPBtn,
			 &sigmoidListTextval);
	
	if(mlp_params.sigmoidList != NULL)
	  deallocate_vector(mlp_params.sigmoidList);
	/* take spaces and tabs out of the list before handing it to nnargs */
	if(sigmoidListTextval != NULL){
	    char *sigmoidListTemp = sigmoidListTextval;
	    sigmoidListTextval = strtok(sigmoidListTextval, " \t");
	    mlp_params.sigmoidList =
	      nnargs_get_float_alloc_list(sigmoidListTextval);
	    XtFree(sigmoidListTemp);
	}
	
	/* make a copy of the string as it has been interpretted */
	XtFree(mlp_params.sigmoidList_string);
	mlp_params.sigmoidList_string =
	  make_float_string(mlp_params.sigmoidList);
	show_string_text(ip->sigmoidListText,
			 mlp_params.sigmoidList_string);
	
	/* do anything else necessary */
	
    } else if (!strcmp(client, "sigmoidListCancelPBtn")) {
	/*fprintf (stderr, "mlpNode_callback: %s\n", client );*/
	sigmoidList_buttons_mapped = 
	  okTextCancel(ip->sigmoidListText, ip->sigmoidListOKPBtn,
		       ip->sigmoidListCancelPBtn,
		       mlp_params.sigmoidList_string);
 
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "mlpNode_callback: %s\n", client );*/
	close_popup(MlpNodeClone->mlpNodeForm);
	
    } else {
	fprintf(stderr, "mlpNode_callback: unknown client: %s\n", (char *)client);
    }
}

void
mlpNode_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(MlpNodeClone->mlpNodeForm);
    } else if (!strcmp(request, "init")){
	mlp_params.ofunction = 0;
	mlp_params.hfunction = 0;
	mlp_params.sigmoidParam = 0.;
	mlp_params.sigmoidParam_string = NULL;
	mlp_params.useSigmoidList = 0;
	mlp_params.sigmoidList = NULL;
	mlp_params.sigmoidList_string = NULL;
    } else if (!strcmp(request, "display")){
	char string[10];
	show_radio_value(MlpNodeClone->ofunctionRadioBox, mlp_params.ofunction);
	show_radio_value(MlpNodeClone->hfunctionRadioBox, mlp_params.hfunction);
	XtFree(mlp_params.sigmoidParam_string);
	sprintf(string, "%5.2g", mlp_params.sigmoidParam);
	mlp_params.sigmoidParam_string = XtNewString(string);
	show_string_text(MlpNodeClone->sigmoidParamText,
			 mlp_params.sigmoidParam_string);

	show_toggle_value(MlpNodeClone->useSigmoidListToggle, mlp_params.useSigmoidList);
	gray_out_item(MlpNodeClone->sigmoidListForm, !mlp_params.useSigmoidList);

	XtFree(mlp_params.sigmoidList_string);
	mlp_params.sigmoidList_string =
	  make_float_string(mlp_params.sigmoidList);
	show_string_text(MlpNodeClone->sigmoidListText,
			 mlp_params.sigmoidList_string);
    } else {
	fprintf(stderr, "mlpNode_request: unknown request: %s\n", request);
    }
}
