/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/mlp/mlpWeight_callback.c,v 11.5 2001/08/23 20:17:05 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            MLPWEIGHT                              ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  3-Nov-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include <nnlib.h>
#include "mlpWeight.h"
/*
 * External Functions
 */
#include "../main/lnknet_subs.h"
 
/*
 * Externals
 */
extern MlpWeightWidgets *MlpWeightClone;
#include "mlp_params.h"
extern MlpParams mlp_params;

/******************************************************************************
 ******************************************************************************
 *
 * Module Name: mlpWeight_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   mlpWeight_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  3-Nov-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
mlpWeight_callback(Widget w, XtPointer client, XtPointer call)
{
  /*SUPPRESS 765*/
  XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
  MlpWeightWidgets *ip;
  int ettaNepochs_buttons_mapped = False;
  int kappa_buttons_mapped = False;
  int epsilon_buttons_mapped = False;
  int ettaList_buttons_mapped = False;
  int decay_buttons_mapped = False;
  int momentum_buttons_mapped = False;
  int tolerance_buttons_mapped = False;
  int maxWeight_buttons_mapped = False;

  /*
   * Get the parent structure
   */
  XtVaGetValues (w, XmNuserData, &ip, NULL);
 
  if (!strcmp(client, "constEttaToggle") ||
      !strcmp(client, "cOverTToggle") ||
      !strcmp(client, "masToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "mlpWeight_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set){
	    mlp_params.ettaChange =
	      get_radio_value(ip->ettaChangeRadioBox);
	    gray_out_item(ip->ettaNepochsForm,
			  (mlp_params.ettaChange != 1));
	    gray_out_item(ip->kappaForm,
			  (mlp_params.ettaChange != 2));
	    gray_out_item(ip->epsilonForm,
			  (mlp_params.ettaChange != 2));
	    gray_out_item(ip->weightUpdateRadioBox,
			  (mlp_params.ettaChange != 0));
	    if(mlp_params.ettaChange == 1){
		mlp_params.weightUpdate = 0;
	    } else if(mlp_params.ettaChange == 2){
		mlp_params.weightUpdate = 1;
	    }
	    show_radio_value(ip->weightUpdateRadioBox,
			     mlp_params.weightUpdate);
	}
	
 
    } else if (!strcmp(client, "ettaNepochsText")) {
	/*fprintf (stderr, "mlpWeight_callback: %s\n", client );*/
	ettaNepochs_buttons_mapped =
	  okTextValueChanged(ip->ettaNepochsText, ip->ettaNepochsOKPBtn,
			     ip->ettaNepochsCancelPBtn,
			     ip->mlpWeightDialogShell,
			     ettaNepochs_buttons_mapped,
			     mlp_params.ettaNepochs_string);
	
    } else if ((!strcmp(client, "ettaNepochsTextAct")) ||
	       (!strcmp(client, "ettaNepochsOKPBtn"))) {
	String ettaNepochsTextval;

	/*fprintf (stderr, "mlpWeight_callback: %s\n", client );*/
	ettaNepochs_buttons_mapped =
	  okTextActivate(ip->ettaNepochsText, ip->ettaNepochsOKPBtn,
			 ip->ettaNepochsCancelPBtn, &ettaNepochsTextval);
	/* I won't actually be using this copy */
	XtFree (ettaNepochsTextval);
	
	/* verify the string as a float */
	mlp_params.ettaNepochs =
	  set_atof_value(ip->ettaNepochsText, "ettaNepochs_set",
			 mlp_params.ettaNepochs);

	/* save the current text for future comparisons */
	XtFree(mlp_params.ettaNepochs_string);
	mlp_params.ettaNepochs_string = get_string_text(ip->ettaNepochsText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "ettaNepochsCancelPBtn")) {
	/*fprintf (stderr, "mlpWeight_callback: %s\n", client );*/
	ettaNepochs_buttons_mapped = 
	  okTextCancel(ip->ettaNepochsText, ip->ettaNepochsOKPBtn,
		       ip->ettaNepochsCancelPBtn,
		       mlp_params.ettaNepochs_string);
 
    } else if (!strcmp(client, "kappaText")) {
	/*fprintf (stderr, "mlpWeight_callback: %s\n", client );*/
	kappa_buttons_mapped =
	  okTextValueChanged(ip->kappaText, ip->kappaOKPBtn,
			     ip->kappaCancelPBtn,
			     ip->mlpWeightDialogShell,
			     kappa_buttons_mapped,
			     mlp_params.kappa_string);
	
    } else if ((!strcmp(client, "kappaTextAct")) ||
	       (!strcmp(client, "kappaOKPBtn"))) {
	String kappaTextval;

	/*fprintf (stderr, "mlpWeight_callback: %s\n", client );*/
	kappa_buttons_mapped =
	  okTextActivate(ip->kappaText, ip->kappaOKPBtn,
			 ip->kappaCancelPBtn, &kappaTextval);
	/* I won't actually be using this copy */
	XtFree (kappaTextval);
	
	/* verify the string as a float */
	mlp_params.kappa =
	  set_atof_low_value(ip->kappaText, "kappa_set",
			     mlp_params.kappa, 0.);

	/* save the current text for future comparisons */
	XtFree(mlp_params.kappa_string);
	mlp_params.kappa_string = get_string_text(ip->kappaText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "kappaCancelPBtn")) {
	/*fprintf (stderr, "mlpWeight_callback: %s\n", client );*/
	kappa_buttons_mapped = 
	  okTextCancel(ip->kappaText, ip->kappaOKPBtn,
		       ip->kappaCancelPBtn,
		       mlp_params.kappa_string);

    } else if (!strcmp(client, "epsilonText")) {
	/*fprintf (stderr, "mlpWeight_callback: %s\n", client );*/
	epsilon_buttons_mapped =
	  okTextValueChanged(ip->epsilonText, ip->epsilonOKPBtn,
			     ip->epsilonCancelPBtn,
			     ip->mlpWeightDialogShell,
			     epsilon_buttons_mapped,
			     mlp_params.epsilon_string);
	
    } else if ((!strcmp(client, "epsilonTextAct")) ||
	       (!strcmp(client, "epsilonOKPBtn"))) {
	String epsilonTextval;

	/*fprintf (stderr, "mlpWeight_callback: %s\n", client );*/
	epsilon_buttons_mapped =
	  okTextActivate(ip->epsilonText, ip->epsilonOKPBtn,
			 ip->epsilonCancelPBtn, &epsilonTextval);
	/* I won't actually be using this copy */
	XtFree (epsilonTextval);
	
	/* verify the string as a float */
	mlp_params.epsilon =
	  set_atof_range_value(ip->epsilonText, "epsilon_set",
			       mlp_params.epsilon, 0., 1.);

	/* save the current text for future comparisons */
	XtFree(mlp_params.epsilon_string);
	mlp_params.epsilon_string = get_string_text(ip->epsilonText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "epsilonCancelPBtn")) {
	/*fprintf (stderr, "mlpWeight_callback: %s\n", client );*/
	epsilon_buttons_mapped = 
	  okTextCancel(ip->epsilonText, ip->epsilonOKPBtn,
		       ip->epsilonCancelPBtn,
		       mlp_params.epsilon_string);
 
  } else if (!strcmp(client, "noEttaListToggle") ||
	     !strcmp(client, "useEttaListToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "mlpWeight_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set){
	    mlp_params.useEttaList =
	      get_radio_value(ip->useEttaListRadioBox);
	    gray_out_item(ip->ettaListForm, !mlp_params.useEttaList);
	}
 
    } else if (!strcmp(client, "ettaListText")) {
	/*fprintf (stderr, "mlpWeight_callback: %s\n", client);*/
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	ettaList_buttons_mapped =
	  okTextValueChanged(ip->ettaListText, ip->ettaListOKPBtn,
			     ip->ettaListCancelPBtn,
			     ip->mlpWeightDialogShell,
			     ettaList_buttons_mapped,
			     mlp_params.ettaList_string);
    } else if ((!strcmp(client, "ettaListTextAct")) ||
	       (!strcmp(client, "ettaListOKPBtn"))) {
	/*fprintf (stderr, "mlpWeight_callback: %s\n", client );*/
	String ettaListTextval;
	ettaList_buttons_mapped =
	  okTextActivate(ip->ettaListText,
			 ip->ettaListOKPBtn,
			 ip->ettaListCancelPBtn,
			 &ettaListTextval);
	
	if(mlp_params.ettaList != NULL)
	  deallocate_vector(mlp_params.ettaList);
	/* take spaces and tabs out of the list before handing it to nnargs */
	if(ettaListTextval != NULL){
	    char *ettaListTemp = ettaListTextval;
	    ettaListTextval = strtok(ettaListTextval, " \t");
	    mlp_params.ettaList =
	      nnargs_get_float_alloc_list(ettaListTextval);
	    XtFree(ettaListTemp);
	}
	
	/* make a copy of the string as it has been interpretted */
	XtFree(mlp_params.ettaList_string);
	mlp_params.ettaList_string =
	  make_float_string(mlp_params.ettaList);
	show_string_text(ip->ettaListText,
			 mlp_params.ettaList_string);
	
	/* do anything else necessary */
	
    } else if (!strcmp(client, "ettaListCancelPBtn")) {
	/*fprintf (stderr, "mlpWeight_callback: %s\n", client );*/
	ettaList_buttons_mapped = 
	  okTextCancel(ip->ettaListText, ip->ettaListOKPBtn,
		       ip->ettaListCancelPBtn,
		       mlp_params.ettaList_string);

    } else if (!strcmp(client, "decayText")) {
	/*fprintf (stderr, "mlpWeight_callback: %s\n", client );*/
	decay_buttons_mapped =
	  okTextValueChanged(ip->decayText, ip->decayOKPBtn,
			     ip->decayCancelPBtn,
			     ip->mlpWeightDialogShell,
			     decay_buttons_mapped,
			     mlp_params.decay_string);
	
    } else if ((!strcmp(client, "decayTextAct")) ||
	       (!strcmp(client, "decayOKPBtn"))) {
	String decayTextval;

	/*fprintf (stderr, "mlpWeight_callback: %s\n", client );*/
	decay_buttons_mapped =
	  okTextActivate(ip->decayText, ip->decayOKPBtn,
			 ip->decayCancelPBtn, &decayTextval);
	/* I won't actually be using this copy */
	XtFree (decayTextval);
	
	/* verify the string as a float */
	mlp_params.decay =
	  set_atof_range_value(ip->decayText, "decay_set",
			       mlp_params.decay, 0., 1.);

	/* save the current text for future comparisons */
	XtFree(mlp_params.decay_string);
	mlp_params.decay_string = get_string_text(ip->decayText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "decayCancelPBtn")) {
	/*fprintf (stderr, "mlpWeight_callback: %s\n", client );*/
	decay_buttons_mapped = 
	  okTextCancel(ip->decayText, ip->decayOKPBtn,
		       ip->decayCancelPBtn,
		       mlp_params.decay_string);

    } else if (!strcmp(client, "momentumText")) {
	/*fprintf (stderr, "mlpWeight_callback: %s\n", client );*/
	momentum_buttons_mapped =
	  okTextValueChanged(ip->momentumText, ip->momentumOKPBtn,
			     ip->momentumCancelPBtn,
			     ip->mlpWeightDialogShell,
			     momentum_buttons_mapped,
			     mlp_params.momentum_string);
	
    } else if ((!strcmp(client, "momentumTextAct")) ||
	       (!strcmp(client, "momentumOKPBtn"))) {
	String momentumTextval;

	/*fprintf (stderr, "mlpWeight_callback: %s\n", client );*/
	momentum_buttons_mapped =
	  okTextActivate(ip->momentumText, ip->momentumOKPBtn,
			 ip->momentumCancelPBtn, &momentumTextval);
	/* I won't actually be using this copy */
	XtFree (momentumTextval);
	
	/* verify the string as a float */
	mlp_params.momentum =
	  set_atof_range_value(ip->momentumText, "momentum_set",
			       mlp_params.momentum, 0., 1.);

	/* save the current text for future comparisons */
	XtFree(mlp_params.momentum_string);
	mlp_params.momentum_string = get_string_text(ip->momentumText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "momentumCancelPBtn")) {
	/*fprintf (stderr, "mlpWeight_callback: %s\n", client );*/
	momentum_buttons_mapped = 
	  okTextCancel(ip->momentumText, ip->momentumOKPBtn,
		       ip->momentumCancelPBtn,
		       mlp_params.momentum_string);
 
    } else if (!strcmp(client, "toleranceText")) {
	/*fprintf (stderr, "mlpWeight_callback: %s\n", client );*/
	tolerance_buttons_mapped =
	  okTextValueChanged(ip->toleranceText, ip->toleranceOKPBtn,
			     ip->toleranceCancelPBtn,
			     ip->mlpWeightDialogShell,
			     tolerance_buttons_mapped,
			     mlp_params.tolerance_string);
	
    } else if ((!strcmp(client, "toleranceTextAct")) ||
	       (!strcmp(client, "toleranceOKPBtn"))) {
	String toleranceTextval;

	/*fprintf (stderr, "mlpWeight_callback: %s\n", client );*/
	tolerance_buttons_mapped =
	  okTextActivate(ip->toleranceText, ip->toleranceOKPBtn,
			 ip->toleranceCancelPBtn, &toleranceTextval);
	/* I won't actually be using this copy */
	XtFree (toleranceTextval);
	
	/* verify the string as a float */
	mlp_params.tolerance =
	  set_atof_low_value(ip->toleranceText, "tolerance_set",
			     mlp_params.tolerance, 0.);

	/* save the current text for future comparisons */
	XtFree(mlp_params.tolerance_string);
	mlp_params.tolerance_string = get_string_text(ip->toleranceText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "toleranceCancelPBtn")) {
	/*fprintf (stderr, "mlpWeight_callback: %s\n", client );*/
	tolerance_buttons_mapped = 
	  okTextCancel(ip->toleranceText, ip->toleranceOKPBtn,
		       ip->toleranceCancelPBtn,
		       mlp_params.tolerance_string);
 
    } else if (!strcmp(client, "maxWeightText")) {
	/*fprintf (stderr, "mlpWeight_callback: %s\n", client );*/
	maxWeight_buttons_mapped =
	  okTextValueChanged(ip->maxWeightText, ip->maxWeightOKPBtn,
			     ip->maxWeightCancelPBtn,
			     ip->mlpWeightDialogShell,
			     maxWeight_buttons_mapped,
			     mlp_params.maxWeight_string);
	
    } else if ((!strcmp(client, "maxWeightTextAct")) ||
	       (!strcmp(client, "maxWeightOKPBtn"))) {
	String maxWeightTextval;

	/*fprintf (stderr, "mlpWeight_callback: %s\n", client );*/
	maxWeight_buttons_mapped =
	  okTextActivate(ip->maxWeightText, ip->maxWeightOKPBtn,
			 ip->maxWeightCancelPBtn, &maxWeightTextval);
	/* I won't actually be using this copy */
	XtFree (maxWeightTextval);
	
	/* verify the string as a float */
	mlp_params.maxWeight =
	  set_atof_value(ip->maxWeightText, "maxWeight_set",
			 mlp_params.maxWeight);

	/* save the current text for future comparisons */
	XtFree(mlp_params.maxWeight_string);
	mlp_params.maxWeight_string = get_string_text(ip->maxWeightText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "maxWeightCancelPBtn")) {
	/*fprintf (stderr, "mlpWeight_callback: %s\n", client );*/
	maxWeight_buttons_mapped = 
	  okTextCancel(ip->maxWeightText, ip->maxWeightOKPBtn,
		       ip->maxWeightCancelPBtn,
		       mlp_params.maxWeight_string);
 
    } else if (!strcmp(client, "trialUpdateToggle") ||
	       !strcmp(client, "batchUpdateToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "mlpWeight_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set) mlp_params.weightUpdate =
	  get_radio_value(ip->weightUpdateRadioBox); 

    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "mlpWeight_callback: %s\n", client );*/
	close_popup(MlpWeightClone->mlpWeightForm);
 
    } else {
	fprintf(stderr, "mlpWeight_callback: unknown client: %s\n", (char *)client);
    }
}
void
mlpWeight_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(MlpWeightClone->mlpWeightForm);
    } else if (!strcmp(request, "init")){
	mlp_params.ettaChange = 0;
	mlp_params.ettaNepochs = 0.;
	mlp_params.ettaNepochs_string = NULL;
	mlp_params.kappa = 0.;
	mlp_params.kappa_string = NULL;
	mlp_params.epsilon = 0.;
	mlp_params.epsilon_string = NULL;
	mlp_params.useEttaList = 0;
	mlp_params.ettaList = NULL;
	mlp_params.ettaList_string = NULL;
	mlp_params.decay = 0.;
	mlp_params.decay_string = NULL;
	mlp_params.momentum = 0.;
	mlp_params.momentum_string = NULL;
	mlp_params.tolerance = 0.;
	mlp_params.tolerance_string = NULL;
	mlp_params.maxWeight = 0.;
	mlp_params.maxWeight_string = NULL;
	mlp_params.weightUpdate = 0;
    } else if (!strcmp(request, "display")){
	char string[10];
	show_radio_value(MlpWeightClone->ettaChangeRadioBox, mlp_params.ettaChange);
	gray_out_item(MlpWeightClone->ettaNepochsForm,
		      (mlp_params.ettaChange != 1));
	gray_out_item(MlpWeightClone->kappaForm,
		      (mlp_params.ettaChange != 2));
	gray_out_item(MlpWeightClone->epsilonForm,
		      (mlp_params.ettaChange != 2));
	gray_out_item(MlpWeightClone->weightUpdateRadioBox,
		      (mlp_params.ettaChange != 0));
	
	XtFree(mlp_params.ettaNepochs_string);
	sprintf(string, "%5.2g", mlp_params.ettaNepochs);
	mlp_params.ettaNepochs_string = XtNewString(string);
	show_string_text(MlpWeightClone->ettaNepochsText,
			 mlp_params.ettaNepochs_string);

	XtFree(mlp_params.kappa_string);
	sprintf(string, "%5.2g", mlp_params.kappa);
	mlp_params.kappa_string = XtNewString(string);
	show_string_text(MlpWeightClone->kappaText,
			 mlp_params.kappa_string);

	XtFree(mlp_params.epsilon_string);
	sprintf(string, "%5.2g", mlp_params.epsilon);
	mlp_params.epsilon_string = XtNewString(string);
	show_string_text(MlpWeightClone->epsilonText,
			 mlp_params.epsilon_string);
	show_radio_value(MlpWeightClone->useEttaListRadioBox, mlp_params.useEttaList);
	gray_out_item(MlpWeightClone->ettaListForm, !mlp_params.useEttaList);

	XtFree(mlp_params.ettaList_string);
	mlp_params.ettaList_string =
	  make_float_string(mlp_params.ettaList);
	show_string_text(MlpWeightClone->ettaListText,
			 mlp_params.ettaList_string);

	XtFree(mlp_params.decay_string);
	sprintf(string, "%5.2g", mlp_params.decay);
	mlp_params.decay_string = XtNewString(string);
	show_string_text(MlpWeightClone->decayText,
			 mlp_params.decay_string);

	XtFree(mlp_params.momentum_string);
	sprintf(string, "%5.2g", mlp_params.momentum);
	mlp_params.momentum_string = XtNewString(string);
	show_string_text(MlpWeightClone->momentumText,
			 mlp_params.momentum_string);

	XtFree(mlp_params.tolerance_string);
	sprintf(string, "%5.2g", mlp_params.tolerance);
	mlp_params.tolerance_string = XtNewString(string);
	show_string_text(MlpWeightClone->toleranceText,
			 mlp_params.tolerance_string);

	XtFree(mlp_params.maxWeight_string);
	sprintf(string, "%5.2g", mlp_params.maxWeight);
	mlp_params.maxWeight_string = XtNewString(string);
	show_string_text(MlpWeightClone->maxWeightText,
			 mlp_params.maxWeight_string);
	show_radio_value(MlpWeightClone->weightUpdateRadioBox, mlp_params.weightUpdate);
    } else {
	fprintf(stderr, "mlpWeight_request: unknown request: %s\n", request);
    }
}
