/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/mlp/mlp_callback.c,v 11.5 2001/08/23 20:17:05 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            MLP                              ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  3-Nov-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include <nnlib.h>
#include "mlp.h"

/*
 * External Functions
 */
#include "../main/lnknet_subs.h"
#include "../plots/plots_func.h"
#include "mlp_func.h"

 
/*
 * Externals
 */
extern MlpWidgets *MlpClone;
#include "mlp_params.h"
MlpParams mlp_params;
#include "../data/data_params.h"
extern DataParams data_params;
#include "../select/select_params.h"
extern SelectParams select_params;

/*
 * Statics
 */
#include <mlp_args.h>

/******************************************************************************
 ******************************************************************************
 *
 * Module Name: mlp_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   mlp_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  3-Nov-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
mlp_callback(Widget w, XtPointer client, XtPointer call)
{
    MlpWidgets *ip;
    static int epochs_buttons_mapped = False;
    static int layers_buttons_mapped = False;
    static int etta_buttons_mapped = False;
    
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
    
    if (!strcmp(client, "epochsText")) {
	/*fprintf (stderr, "mlp_callback: %s\n", client );*/
	epochs_buttons_mapped =
	  okTextValueChanged(ip->epochsText, ip->epochsOKPBtn,
			     ip->epochsCancelPBtn,
			     ip->mlpDialogShell,
			     epochs_buttons_mapped,
			     mlp_params.epochs_string);
	
    } else if ((!strcmp(client, "epochsTextAct")) ||
	       (!strcmp(client, "epochsOKPBtn"))) {
	String epochsTextval;

	/*fprintf (stderr, "mlp_callback: %s\n", client );*/
	epochs_buttons_mapped =
	  okTextActivate(ip->epochsText, ip->epochsOKPBtn,
			 ip->epochsCancelPBtn, &epochsTextval);
	/* I won't actually be using this copy */
	XtFree (epochsTextval);
	
	/* verify the string as an integer */
	mlp_params.epochs =
	  set_atoi_value(ip->epochsText, "epochs_set",
			     mlp_params.epochs);

	/* save the current text for future comparisons */
	XtFree(mlp_params.epochs_string);
	mlp_params.epochs_string = get_string_text(ip->epochsText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "epochsCancelPBtn")) {
	/*fprintf (stderr, "mlp_callback: %s\n", client );*/
	epochs_buttons_mapped = 
	  okTextCancel(ip->epochsText, ip->epochsOKPBtn,
		       ip->epochsCancelPBtn,
		       mlp_params.epochs_string);
	
    } else if (!strcmp(client, "layersText")) {
	/*fprintf (stderr, "mlp_callback: %s\n", client);*/
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	layers_buttons_mapped =
	  okTextValueChanged(ip->layersText, ip->layersOKPBtn,
			     ip->layersCancelPBtn,
			     ip->mlpDialogShell,
			     layers_buttons_mapped,
			     mlp_params.layers_string);
    } else if ((!strcmp(client, "layersTextAct")) ||
	       (!strcmp(client, "layersOKPBtn"))) {
	/*fprintf (stderr, "mlp_callback: %s\n", client );*/
	String layersTextval;
	layers_buttons_mapped =
	  okTextActivate(ip->layersText,
			 ip->layersOKPBtn,
			 ip->layersCancelPBtn,
			 &layersTextval);
	
	if(mlp_params.layers != NULL)
	  deallocate_ivector(mlp_params.layers);
	/* take spaces and tabs out of the list before handing it to nnargs */
	if(layersTextval != NULL){
	    char *layersTemp = layersTextval;
	    layersTextval = strtok(layersTextval, " \t");
	    mlp_params.layers =
	      nnargs_get_int_alloc_list(layersTextval);
	    XtFree(layersTemp);
	}
	/* fix the ninputs and noutputs entries */
	if(mlp_params.layers[0] < 2) mlp_params.layers[0] = 2;
	mlp_params.layers[1] = select_params.ninputs;
	mlp_params.layers[mlp_params.layers[0]] = data_params.noutputs;
	
	/* make a copy of the string as it has been interpretted */
	XtFree(mlp_params.layers_string);
	mlp_params.layers_string =
	  make_int_string(mlp_params.layers);
	show_string_text(ip->layersText,
			 mlp_params.layers_string);
	
	/* do anything else necessary */
	set_plot_labels();
	
    } else if (!strcmp(client, "layersCancelPBtn")) {
	/*fprintf (stderr, "mlp_callback: %s\n", client );*/
	layers_buttons_mapped = 
	  okTextCancel(ip->layersText, ip->layersOKPBtn,
		       ip->layersCancelPBtn,
		       mlp_params.layers_string);
	
    } else if (!strcmp(client, "ettaText")) {
	/*fprintf (stderr, "mlp_callback: %s\n", client );*/
	etta_buttons_mapped =
	  okTextValueChanged(ip->ettaText, ip->ettaOKPBtn,
			     ip->ettaCancelPBtn,
			     ip->mlpDialogShell,
			     etta_buttons_mapped,
			     mlp_params.etta_string);
	
    } else if ((!strcmp(client, "ettaTextAct")) ||
	       (!strcmp(client, "ettaOKPBtn"))) {
	String ettaTextval;

	/*fprintf (stderr, "mlp_callback: %s\n", client );*/
	etta_buttons_mapped =
	  okTextActivate(ip->ettaText, ip->ettaOKPBtn,
			 ip->ettaCancelPBtn, &ettaTextval);
	/* I won't actually be using this copy */
	XtFree (ettaTextval);
	
	/* verify the string as a float */
	mlp_params.etta =
	  set_atof_value(ip->ettaText, "etta_set",
			 mlp_params.etta);

	/* save the current text for future comparisons */
	XtFree(mlp_params.etta_string);
	mlp_params.etta_string = get_string_text(ip->ettaText);
 
	/* do any other things needed */
	set_plot_labels();

    } else if (!strcmp(client, "ettaCancelPBtn")) {
	/*fprintf (stderr, "mlp_callback: %s\n", client );*/
	etta_buttons_mapped = 
	  okTextCancel(ip->ettaText, ip->ettaOKPBtn,
		       ip->ettaCancelPBtn,
		       mlp_params.etta_string);
	
    } else if (!strcmp(client, "weightPBtn")) {
	/*fprintf (stderr, "mlp_callback: %s\n", client );*/
	mlpWeight_request("open");
	
    } else if (!strcmp(client, "costPBtn")) {
	/*fprintf (stderr, "mlp_callback: %s\n", client );*/
	mlpCost_request("open");
	
    } else if (!strcmp(client, "nodePBtn")) {
	/*fprintf (stderr, "mlp_callback: %s\n", client );*/
	mlpNode_request("open");
	
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "mlp_callback: %s\n", client );*/
	close_popup(MlpClone->mlpForm);
	
    } else {
	fprintf(stderr, "mlp_callback: unknown client: %s\n", (char *)client);
    }
}

void
mlp_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(MlpClone->mlpForm);
    } else if (!strcmp(request, "init")){
	mlp_params.epochs = 0;
	mlp_params.epochs_string = NULL;
	mlp_params.layers = NULL;
	mlp_params.layers_string = NULL;
	mlp_params.etta = 0.;
	mlp_params.etta_string = NULL;
	mlpWeight_request("init");
	mlpCost_request("init");
	mlpNode_request("init");
    } else if (!strcmp(request, "display")){
	char string[10];

	XtFree(mlp_params.epochs_string);
	sprintf(string, "%d", mlp_params.epochs);
	mlp_params.epochs_string = XtNewString(string);
	show_string_text(MlpClone->epochsText,
			 mlp_params.epochs_string);

	XtFree(mlp_params.layers_string);
	mlp_params.layers_string =
	  make_int_string(mlp_params.layers);
	show_string_text(MlpClone->layersText,
			 mlp_params.layers_string);

	XtFree(mlp_params.etta_string);
	sprintf(string, "%5.2g", mlp_params.etta);
	mlp_params.etta_string = XtNewString(string);
	show_string_text(MlpClone->ettaText,
			 mlp_params.etta_string);

	mlpWeight_request("display");
	mlpCost_request("display");
	mlpNode_request("display");
    } else if (!strcmp(request, "update_ninputs")){
	if(mlp_params.layers != NULL && mlp_params.layers[0] >= 2){
	    mlp_params.layers[1] = select_params.ninputs;
	    mlp_params.layers[mlp_params.layers[0]] = data_params.noutputs;
	    mlp_params.layers_string =
	      make_int_string(mlp_params.layers);
	    show_string_text(MlpClone->layersText,
			     mlp_params.layers_string);
	    set_plot_labels();
	}
    } else {
	fprintf(stderr, "mlp_request: unknown request: %s\n", request);
    }
}

String
mlp_get_plot_label(String string)
{
    char numstring[32];
    string = StringConcat(string, " Net:");
    string = StringConcat(string, make_int_string(mlp_params.layers));
    string = StringConcat(string, " Step:");
    sprintf(numstring, "%g", mlp_params.etta);
    string = StringConcat(string, numstring);
    return string;
}

static void
mlp_set_read_variables()
{
    mlp_params.epochs = nepochs;
    mlp_params.layers = copy_static2alloc_ivector(nnodes);
    mlp_params.etta = etta;
    mlp_params.ettaChange = step_change;
    mlp_params.ettaNepochs = etta_nepochs;
    mlp_params.kappa = kappa;
    mlp_params.epsilon = epsilon;
    mlp_params.useEttaList = use_etta_list;
    if(mlp_params.ettaList != NULL){
	deallocate_vector(mlp_params.ettaList);
    }
    mlp_params.ettaList = etta_list;
    mlp_params.decay = decay;
    mlp_params.momentum = alpha;
    mlp_params.tolerance = tolerance;
    mlp_params.maxWeight = max_magnitude;
    mlp_params.costFunc = cost_func;
    mlp_params.costParam = cost_param;
    switch(ofunction){
      case 0:
      case 1:
	mlp_params.ofunction = ofunction;
	break;
      case LINEAR:
	mlp_params.ofunction = 2;
	break;
      case STEP:
	mlp_params.ofunction = 0;
	break;
    }
    switch(hfunction){
      case 0:
      case 1:
	mlp_params.hfunction = hfunction;
	break;
      case LINEAR:
      case STEP:
	mlp_params.hfunction = 0;
    }
    mlp_params.sigmoidParam = sigmoid_param;
    mlp_params.useSigmoidList = use_sigmoid_param_list;
    if(mlp_params.sigmoidList != NULL){
	deallocate_vector(mlp_params.sigmoidList);
    }
    mlp_params.sigmoidList = sigmoid_param_list;
}

static void
mlp_set_write_variables()
{
    nepochs = mlp_params.epochs;
    copy_alloc2static_ivector(mlp_params.layers, nnodes);
    etta = mlp_params.etta;
    step_change = mlp_params.ettaChange;
    etta_nepochs = mlp_params.ettaNepochs;
    kappa = mlp_params.kappa;
    epsilon = mlp_params.epsilon;
    use_etta_list = mlp_params.useEttaList;
    etta_list = mlp_params.ettaList;
    decay = mlp_params.decay;
    alpha = mlp_params.momentum;
    tolerance = mlp_params.tolerance;
    max_magnitude = mlp_params.maxWeight;
    cost_func = mlp_params.costFunc;
    cost_param = mlp_params.costParam;
    switch(mlp_params.ofunction){
      case 0:
      case 1:
	ofunction = mlp_params.ofunction;
	break;
      case 2:
	ofunction = LINEAR;
    }
    if(mlp_params.costFunc == PC_PROC){
	ofunction = STEP;
	alpha = 0.;
    } else if(mlp_params.costFunc != SQ_ERR && mlp_params.costFunc != CFOM){
	ofunction = SIGM;
    }
    hfunction = mlp_params.hfunction;
    sigmoid_param = mlp_params.sigmoidParam;
    use_sigmoid_param_list = mlp_params.useSigmoidList;
    sigmoid_param_list = mlp_params.sigmoidList;
    batch[0] = 3;
    switch(mlp_params.weightUpdate){
      case 0:	/* trial by trial */
	batch[1]=1;
	batch[2]=1;
	batch[3]=0;
	break;
      case 1:	/* batch update, once per epoch */
	batch[1]=data_params.npatterns_train;
	batch[2]=data_params.npatterns_train;
	batch[3]=0;
	break;
      case 2:	/* batch update, variable rate (every 1, then 2, etc)*/
	/* this one is defunct but I may as well keep it around a bit */
	batch[1]=1;
	batch[2]=data_params.npatterns_train;
	batch[3]=1;
	break;
      default:	/* trial by trial */
	batch[1]=1;
	batch[2]=1;
	batch[3]=0;
	break;
    }
}
static ARG mlptab[] =
{
    {"weightUpdate", INT, (int *)NULL, },
};
#define TABSIZE (sizeof(mlptab)/sizeof(ARG))

static void
mlp_set_defaults_variables(dp)
     MlpParams *dp;
{
    nnargs_set_variable("weightUpdate", (int *)&dp->weightUpdate, mlptab, TABSIZE);
}

void
read_mlp_defaults(argc, argv)
     int argc;
     char **argv;
{
    mlp_params.weightUpdate = 0;
    nnodes[0] = 0;
    use_etta_list = 0;
    etta_list = NULL;
    use_sigmoid_param_list = 0;
    sigmoid_param_list = NULL;
    batch[0] = 0;
    mlp_set_defaults_variables(&mlp_params);

    if(argc > 0){
	argc = nnargs_part(argc, argv, mlptab, TABSIZE);
	argc = nnargs(argc, argv, mlp_tab, MLP_TABSIZE);
	mlp_set_read_variables();
    } else {
	/* set any defaults */
	if(mlp_params.layers != NULL)
	  deallocate_ivector(mlp_params.layers);
	mlp_params.layers = allocate_ivector(4);
	mlp_params.layers[0] = 3;
	mlp_params.layers[1] = select_params.ninputs;
	mlp_params.layers[2] = 25;
	mlp_params.layers[3] = data_params.noutputs;

	mlp_params.momentum = 0.6;
	mlp_params.etta = 0.1;
	mlp_params.epsilon = 0.1;
	mlp_params.kappa = 0.01;
	mlp_params.decay = 0.;
	mlp_params.tolerance = 0.01;
	mlp_params.hfunction = 0;
	mlp_params.ofunction = 0;
	mlp_params.sigmoidParam = 1.0;
	mlp_params.costFunc = 0;
	mlp_params.costParam = 1.;
	mlp_params.epochs = 30;
	mlp_params.maxWeight = 0.1;
    }

    mlp_request("display");
}

void
write_mlp_defaults(FILE *default_file)
{
    mlp_set_defaults_variables(&mlp_params);
    mlp_set_write_variables();

    if(default_file != NULL){
	fprintf(default_file, "mlp %s",
		nnargs_get_part_flags(mlptab, TABSIZE));
	fprintf(default_file, "mlp %s",
		nnargs_get_flags(mlp_tab, MLP_TABSIZE));
    }
}

String
mlp_get_args ()
{
    mlp_set_write_variables();
    return (nnargs_get_part_flags(mlp_tab, MLP_TABSIZE));
}

String
mlp_get_important_args ()
{
    mlp_set_write_variables();
    return (nnargs_get_flags_one_line(mlp_tab, MLP_TABSIZE));
}

int
mlp_get_epochs()
{
    return mlp_params.epochs;
}

void
mlp_set_epochs(int e)
{
    mlp_params.epochs = e;
}

int
mlp_get_cost_func()
{
    return mlp_params.costFunc;
}
