/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/nbayes/nbayes_callback.c,v 1.2 2001/08/23 20:17:08 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                             NBAYES                                ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  8-Nov-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include "../main/lnknet_subs.h"
#include "nbayes.h"
#include <nnlib.h>
#include <nbayes_args.h>
/*
 * External Functions
 */
#include "nbayes_func.h"
#include "../plots/plots_func.h"
 
/*
 * Externals
 */
extern NbayesWidgets *NbayesClone;
#include "nbayes_params.h"
NbayesParams nbayes_params; /* this is the real one */
#include "../data/data_params.h"
extern DataParams data_params;

/*
 * Statics
 */ 

/******************************************************************************
 ******************************************************************************
 *
 * Module Name: nbayes_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   nbayes_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  8-Nov-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
nbayes_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    NbayesWidgets *ip;
    static int nBins_buttons_mapped = False;
    static int nBinsPerInput_buttons_mapped = False;

    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "equalNbinsToggle") ||
	!strcmp(client, "specifyBinsToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "nbayes_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set){
	    nbayes_params.specifyBins =
	      get_radio_value(ip->specifyBinsRadioBox);
	    gray_out_item(ip->nBinsForm, nbayes_params.specifyBins);
	    gray_out_item(ip->nBinsPerInputForm,
			  !nbayes_params.specifyBins);
	}
	
    } else if (!strcmp(client, "nBinsText")) {
	/*fprintf (stderr, "nbayes_callback: %s\n", client );*/
	nBins_buttons_mapped =
	  okTextValueChanged(ip->nBinsText, ip->nBinsOKPBtn,
			     ip->nBinsCancelPBtn,
			     ip->nbayesDialogShell,
			     nBins_buttons_mapped,
			     nbayes_params.nBins_string);
	
    } else if ((!strcmp(client, "nBinsTextAct")) ||
	       (!strcmp(client, "nBinsOKPBtn"))) {
	String nBinsTextval;

	/*fprintf (stderr, "nbayes_callback: %s\n", client );*/
	nBins_buttons_mapped =
	  okTextActivate(ip->nBinsText, ip->nBinsOKPBtn,
			 ip->nBinsCancelPBtn, &nBinsTextval);
	/* I won't actually be using this copy */
	XtFree (nBinsTextval);
	
	/* verify the string as an integer */
	nbayes_params.nBins =
	  set_atoi_value(ip->nBinsText, "nBins_set",
			     nbayes_params.nBins);

	/* save the current text for future comparisons */
	XtFree(nbayes_params.nBins_string);
	nbayes_params.nBins_string = get_string_text(ip->nBinsText);
 
	/* do any other things needed */
	set_plot_labels();

    } else if (!strcmp(client, "nBinsCancelPBtn")) {
	/*fprintf (stderr, "nbayes_callback: %s\n", client );*/
	nBins_buttons_mapped = 
	  okTextCancel(ip->nBinsText, ip->nBinsOKPBtn,
		       ip->nBinsCancelPBtn,
		       nbayes_params.nBins_string);
	
    } else if (!strcmp(client, "nBinsPerInputText")) {
	/*fprintf (stderr, "nbayes_callback: %s\n", client);*/
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	nBinsPerInput_buttons_mapped =
	  okTextValueChanged(ip->nBinsPerInputText, ip->nBinsPerInputOKPBtn,
			     ip->nBinsPerInputCancelPBtn,
			     ip->nbayesDialogShell,
			     nBinsPerInput_buttons_mapped,
			     nbayes_params.nBinsPerInput_string);
    } else if ((!strcmp(client, "nBinsPerInputTextAct")) ||
	       (!strcmp(client, "nBinsPerInputOKPBtn"))) {
	/*fprintf (stderr, "nbayes_callback: %s\n", client );*/
	String nBinsPerInputTextval;
	nBinsPerInput_buttons_mapped =
	  okTextActivate(ip->nBinsPerInputText,
			 ip->nBinsPerInputOKPBtn,
			 ip->nBinsPerInputCancelPBtn,
			 &nBinsPerInputTextval);
	
	if(nbayes_params.nBinsPerInput != NULL)
	  deallocate_ivector(nbayes_params.nBinsPerInput);
	/* take spaces and tabs out of the list before handing it to nnargs */
	if(nBinsPerInputTextval != NULL){
	    char *nBinsPerInputTemp = nBinsPerInputTextval;
	    nBinsPerInputTextval = strtok(nBinsPerInputTextval, " \t");
	    nbayes_params.nBinsPerInput =
	      nnargs_get_int_alloc_list(nBinsPerInputTextval);
	    XtFree(nBinsPerInputTemp);
	}
	
	/* make a copy of the string as it has been interpretted */
	XtFree(nbayes_params.nBinsPerInput_string);
	nbayes_params.nBinsPerInput_string =
	  make_int_string(nbayes_params.nBinsPerInput);
	show_string_text(ip->nBinsPerInputText,
			 nbayes_params.nBinsPerInput_string);
	
	/* do anything else necessary */
	set_plot_labels();
	
    } else if (!strcmp(client, "nBinsPerInputCancelPBtn")) {
	/*fprintf (stderr, "nbayes_callback: %s\n", client );*/
	nBinsPerInput_buttons_mapped = 
	  okTextCancel(ip->nBinsPerInputText, ip->nBinsPerInputOKPBtn,
		       ip->nBinsPerInputCancelPBtn,
		       nbayes_params.nBinsPerInput_string);

    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "nbayes_callback: %s\n", client );*/
	close_popup(NbayesClone->nbayesForm);
	
    } else {
	fprintf(stderr, "nbayes_callback: unknown client: %s\n", (char *)client);
    }
}

void
nbayes_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(NbayesClone->nbayesForm);
    } else if (!strcmp(request, "init")){
	nbayes_params.specifyBins = 0;
	nbayes_params.nBins = 0;
	nbayes_params.nBins_string = NULL;
	nbayes_params.nBinsPerInput = NULL;
	nbayes_params.nBinsPerInput_string = NULL;
    } else if (!strcmp(request, "display")){
	char string[10];
	show_radio_value(NbayesClone->specifyBinsRadioBox,
			 nbayes_params.specifyBins);
	gray_out_item(NbayesClone->nBinsForm, nbayes_params.specifyBins);
	gray_out_item(NbayesClone->nBinsPerInputForm,
		      !nbayes_params.specifyBins);
	XtFree(nbayes_params.nBins_string);
	sprintf(string, "%d", nbayes_params.nBins);
	nbayes_params.nBins_string = XtNewString(string);
	show_string_text(NbayesClone->nBinsText,
			 nbayes_params.nBins_string);
	XtFree(nbayes_params.nBinsPerInput_string);
	nbayes_params.nBinsPerInput_string =
	  make_int_string(nbayes_params.nBinsPerInput);
	show_string_text(NbayesClone->nBinsPerInputText,
			 nbayes_params.nBinsPerInput_string);
    } else {
	fprintf(stderr, "nbayes_request: unknown request: %s\n", request);
    }
}

String
nbayes_get_plot_label(String string)
{
    return string;
}

static void
nbayes_set_read_variables()
{
    nbayes_params.nBins = nbins;
    nbayes_params.nBinsPerInput = nBinsPerInput;
}

static void
nbayes_set_write_variables()
{
    nbins = nbayes_params.nBins;
    nBinsPerInput = nbayes_params.nBinsPerInput;
}

static ARG nbayestab[] =
{
    {"specifyBins", BOOLEAN, (int *)NULL,},
};
#define TABSIZE (sizeof(nbayestab)/sizeof(ARG))

static void
nbayes_set_defaults_variables(dp)
     NbayesParams *dp;
{
    nnargs_set_variable("specifyBins", (int *)&dp->specifyBins, nbayestab, TABSIZE);
}

void
read_nbayes_defaults(argc, argv)
     int argc;
     char **argv;
{
    nbayes_params.specifyBins = 0;
    nBinsPerInput = NULL;
    nbayes_set_defaults_variables(&nbayes_params);
    if(argc > 0){
	argc = nnargs_part(argc, argv, nbayestab, TABSIZE);
	argc = nnargs(argc, argv, nbayes_tab, NBAYES_TABSIZE);
	nbayes_set_read_variables();

    } else {
	/* set any defaults */
	nbayes_params.nBins = 10;
    }

    nbayes_request("display");
}

void
write_nbayes_defaults(FILE *default_file)
{
    nbayes_set_write_variables();
    nbayes_set_defaults_variables(&nbayes_params);

    if(default_file != NULL){
	fprintf(default_file, "nbayes %s",
		nnargs_get_part_flags(nbayestab, TABSIZE));
	fprintf(default_file, "%s",
		nnargs_get_flags(nbayes_tab, NBAYES_TABSIZE));
    }
}

String
nbayes_get_args ()
{
    nbayes_set_write_variables();
    if(!nbayes_params.specifyBins) nBinsPerInput = NULL;
    return (nnargs_get_part_flags(nbayes_tab, NBAYES_TABSIZE));
}

String
nbayes_get_important_args ()
{
    nbayes_set_write_variables();
    if(!nbayes_params.specifyBins) nBinsPerInput = NULL;
    return (nnargs_get_flags_one_line(nbayes_tab, NBAYES_TABSIZE));
}

void
nbayes_store_plot_command(FILE *shell_file)
{
}
