/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/nc_class/nc_class_callback.c,v 11.4 2001/08/23 20:17:11 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                              NC_CLASS                                ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  7-Nov-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include "../main/lnknet_subs.h"
#include "nc_class.h"
#include <nnlib.h>
#include <nc_class_args.h>
/*
 * External Functions
 */
#include "../plots/plots_func.h"
#include "../kmeans/kmeans_func.h"
#include "../em_clus/em_clus_func.h"
#include "../lead_clus/lead_clus_func.h"
#include "../ran_clus/ran_clus_func.h"

/*
 * Externals
 */
extern Nc_classWidgets *Nc_classClone;
#include "nc_class_params.h"
Nc_ClassParams nc_class_params; /* this is the real one */
#include "../lnk/lnk_params.h"
LnkParams lnk_params;

/*
 * Statics
 */
static void nc_class_set_clustering_alg(int);
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: nc_class_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   nc_class_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  7-Nov-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
nc_class_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    Nc_classWidgets *ip;
    int minVar_buttons_mapped = False;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "createClustersToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "nc_class_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	nc_class_params.createClusters = state->set;
 
    } else if (!strcmp(client, "kmeansPBtn")) {
	/*fprintf (stderr, "nc_class_callback: %s\n", client );*/
	nc_class_set_clustering_alg(KMEANS);

    } else if (!strcmp(client, "em_clusPBtn")) {
	/*fprintf (stderr, "nc_class_callback: %s\n", client );*/
	nc_class_set_clustering_alg(EM_CLUS);
 
    } else if (!strcmp(client, "lead_clusPBtn")) {
	/*fprintf (stderr, "nc_class_callback: %s\n", client );*/
	nc_class_set_clustering_alg(LEAD_CLUS);
 
    } else if (!strcmp(client, "ran_clusPBtn")) {
	/*fprintf (stderr, "nc_class_callback: %s\n", client );*/
	nc_class_set_clustering_alg(RAN_CLUS);
 
    } else if (!strcmp(client, "clustParamsPBtn")) {
	/*fprintf (stderr, "nc_class_callback: %s\n", client );*/
	switch(nc_class_params.clusterNum){
	  case KMEANS:
	    kmeans_request("open");
	    break;
	  case EM_CLUS:
	    em_clus_request("open");
	    break;
	  case LEAD_CLUS:
	    lead_clus_request("open");
	    break;
	  case RAN_CLUS:
	    ran_clus_request("open");
	    break;
	}
 
    } else if (!strcmp(client, "euclidToggle") ||
	       !strcmp(client, "gaussToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "window_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set){
	    nc_class_params.gauss =
	      get_radio_value(ip->gaussRadioBox);
	    set_plot_labels();
	}
 
    } else if (!strcmp(client, "minVarText")) {
	/*fprintf (stderr, "nc_class_callback: %s\n", client );*/
	minVar_buttons_mapped =
	  okTextValueChanged(ip->minVarText, ip->minVarOKPBtn,
			     ip->minVarCancelPBtn,
			     ip->nc_classDialogShell,
			     minVar_buttons_mapped,
			     nc_class_params.minVar_string);
	
    } else if ((!strcmp(client, "minVarTextAct")) ||
	       (!strcmp(client, "minVarOKPBtn"))) {
	String minVarTextval;

	/*fprintf (stderr, "nc_class_callback: %s\n", client );*/
	minVar_buttons_mapped =
	  okTextActivate(ip->minVarText, ip->minVarOKPBtn,
			 ip->minVarCancelPBtn, &minVarTextval);
	/* I won't actually be using this copy */
	XtFree (minVarTextval);
	
	/* verify the string as a float */
	nc_class_params.minVar =
	  set_atof_low_value(ip->minVarText, "minVar_set",
			     nc_class_params.minVar, 0.);

	/* save the current text for future comparisons */
	XtFree(nc_class_params.minVar_string);
	nc_class_params.minVar_string = get_string_text(ip->minVarText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "minVarCancelPBtn")) {
	/*fprintf (stderr, "nc_class_callback: %s\n", client );*/
	minVar_buttons_mapped = 
	  okTextCancel(ip->minVarText, ip->minVarOKPBtn,
		       ip->minVarCancelPBtn,
		       nc_class_params.minVar_string);
 
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "nc_class_callback: %s\n", client );*/
	close_popup(Nc_classClone->nc_classForm);
 
    } else {
	fprintf(stderr, "nc_class_callback: unknown client: %s\n", (char *)client);
    }
}

void
nc_class_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(Nc_classClone->nc_classForm);
    } else if (!strcmp(request, "init")){
	nc_class_params.createClusters = 0;
	nc_class_params.clusterNum = KMEANS;
	nc_class_params.clustName = "kmeans";
	nc_class_params.clustFname = NULL;
	nc_class_params.gauss = 0;
	nc_class_params.minVar = 0.;
	nc_class_params.minVar_string = NULL;
    } else if (!strcmp(request, "display")){
	char string[10];
	show_toggle_value(Nc_classClone->createClustersToggle,
			  nc_class_params.createClusters);
	switch(nc_class_params.clusterNum){
	  case KMEANS:
	    show_option_value(Nc_classClone->clustAlgOptionMenu, 0);
	    break;
	  case EM_CLUS:
	    show_option_value(Nc_classClone->clustAlgOptionMenu, 1);
	    break;
	  case LEAD_CLUS:
	    show_option_value(Nc_classClone->clustAlgOptionMenu, 2);
	    break;
	  case RAN_CLUS:
	    show_option_value(Nc_classClone->clustAlgOptionMenu, 3);
	    break;
	}
	show_string_label(Nc_classClone->clustFnameLabel, nc_class_params.clustFname);

	show_radio_value(Nc_classClone->gaussRadioBox, nc_class_params.gauss);

	XtFree(nc_class_params.minVar_string);
	sprintf(string, "%5.2g", nc_class_params.minVar);
	nc_class_params.minVar_string = XtNewString(string);
	show_string_text(Nc_classClone->minVarText,
			 nc_class_params.minVar_string);
    } else {
	fprintf(stderr, "nc_class_request: unknown request: %s\n", request);
    }
}

String
nc_class_get_plot_label(String string)
{
    if(nc_class_params.gauss) string = StringConcat(string, " Mahalanobis");
    else string = StringConcat(string, " Euclidean");
    switch(nc_class_params.clusterNum){
      case KMEANS:
	string = kmeans_get_plot_label(string);
	break;
      case EM_CLUS:
	string = em_clus_get_plot_label(string);
	break;
      case LEAD_CLUS:
	string = lead_clus_get_plot_label(string);
	break;
      case RAN_CLUS:
	string = ran_clus_get_plot_label(string);
	break;
    }
    return string;
}

static void
nc_class_set_read_variables()
{
    XtFree(nc_class_params.clustFname);
    nc_class_params.clustFname = XtNewString(cluster_param_fname);
    nc_class_params.gauss = dist_type;
    nc_class_params.minVar = minimum_variance;
}

static void
nc_class_set_write_variables()
{
    cluster_param_fname = nc_class_params.clustFname;
    dist_type = nc_class_params.gauss;
    minimum_variance = nc_class_params.minVar;
}
static ARG nc_classtab[] =
{
    {"createClusters", BOOLEAN, (int *)NULL,},
    {"clusterNum", INT, (int *)NULL,},
};
#define TABSIZE (sizeof(nc_classtab)/sizeof(ARG))

static void
nc_class_set_defaults_variables(dp)
     Nc_ClassParams *dp;
{
    nnargs_set_variable("createClusters", (int *)&dp->createClusters, nc_classtab, TABSIZE);
    nnargs_set_variable("clusterNum", (int *)&dp->clusterNum, nc_classtab, TABSIZE);
}

void
read_nc_class_defaults(argc, argv)
     int argc;
     char **argv;
{
    nc_class_params.createClusters = 0;
    dist_type = 0;
    nc_class_set_defaults_variables(&nc_class_params);
    if(argc > 0){
	argc = nnargs_part(argc, argv, nc_classtab, TABSIZE);
	argc = nnargs(argc, argv, nc_class_tab, NC_CLASS_TABSIZE);
	nc_class_set_read_variables();
	switch(nc_class_params.clusterNum){
	  case KMEANS:
	    nc_class_params.clustName = "kmeans";
	    break;
	  case EM_CLUS:
	    nc_class_params.clustName = "em_clus";
	    break;
	  case LEAD_CLUS:
	    nc_class_params.clustName = "lead_clus";
	    break;
	  case RAN_CLUS:
	    nc_class_params.clustName = "ran_clus";
	    break;
	}
    } else {
	/* set any defaults */
	nc_class_params.createClusters = 1;
	nc_class_params.clusterNum = KMEANS;
	nc_class_params.gauss = 0;
	nc_class_params.minVar = 1e-6;
    }

    nc_class_request("display");
}

void
write_nc_class_defaults(FILE *default_file)
{
    nc_class_set_write_variables();
    nc_class_set_defaults_variables(&nc_class_params);

    if(default_file != NULL){
	fprintf(default_file, "nc_class %s",
		nnargs_get_part_flags(nc_classtab, TABSIZE));
	fprintf(default_file, "nc_class %s",
		nnargs_get_flags(nc_class_tab, NC_CLASS_TABSIZE));
    }
}

String
nc_class_get_args ()
{
    nc_class_set_write_variables();
    return (nnargs_get_part_flags(nc_class_tab, NC_CLASS_TABSIZE));
}

String
nc_class_get_important_args ()
{
    nc_class_set_write_variables();
    return (nnargs_get_flags_one_line(nc_class_tab, NC_CLASS_TABSIZE));
}

static void
nc_class_set_clustering_alg(value)
     int value;
{
    nc_class_params.clusterNum = value;
    switch(nc_class_params.clusterNum){
      case KMEANS:
	nc_class_params.clustName = "kmeans";
	break;
      case EM_CLUS:
	nc_class_params.clustName = "em_clus";
	break;
      case LEAD_CLUS:
	nc_class_params.clustName = "lead_clus";
	break;
      case RAN_CLUS:
	nc_class_params.clustName = "ran_clus";
	break;
    }
    XtFree(nc_class_params.clustFname);
    nc_class_params.clustFname = XtNewString(lnk_params.exp_prefix);
    nc_class_params.clustFname = StringConcat(nc_class_params.clustFname,
					 nc_class_params.clustName);
    nc_class_params.clustFname = StringConcat(nc_class_params.clustFname,
					 ".param");
    show_string_label(Nc_classClone->clustFnameLabel, nc_class_params.clustFname);
    set_plot_labels();
}

void
nc_class_set_cluster_param_fname()
{
    if(nc_class_params.createClusters){
	XtFree(nc_class_params.clustFname);
	nc_class_params.clustFname = XtNewString(lnk_params.exp_prefix);
	nc_class_params.clustFname = StringConcat(nc_class_params.clustFname,
					     nc_class_params.clustName);
	nc_class_params.clustFname = StringConcat(nc_class_params.clustFname,
					     ".param");
	show_string_label(Nc_classClone->clustFnameLabel, nc_class_params.clustFname);
    }
}

char *
nc_class_get_cluster_fname(){
    return nc_class_params.clustFname;
}

int
nc_class_get_cluster_alg(){
    return nc_class_params.clusterNum;
}

int
nc_class_must_store_clustering(){
    return nc_class_params.createClusters;
}

