/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/norm/normGen_callback.c,v 11.5 2001/08/23 20:17:15 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            NORMGEN                              ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  2-Oct-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <Xm/Xm.h>
#include <nnlib.h>
#include "normGen.h"
/*
 * External Functions
 */
#include "norm_func.h"
#include "../main/lnknet_subs.h"
#include "../reports/reports_func.h"

/*
 * Externals
 */
#define NORM_PROGRAM_NAME "norm_lnk"
#define PLOT_NORM "plot_norm"

extern NormGenWidgets *NormGenClone;
#include "norm_params.h"
extern NormParams norm_params;
#include "../lnk/lnk_params.h"
extern LnkParams lnk_params;
#include "../reports/reports_params.h"
extern ReportsParams reports_params;
#include "../data/data_params.h"
extern DataParams data_params;

/* Static functions */
static void norm_run(RUN_TYPE);
static void norm_set_common_flags();
static int store_norm_commands(char *, RUN_TYPE);
static void activate_norm_plots(int activate_flag);
 
/* variables for shell scripts */
#include <norm_args.h>
#include <plot_norm_args.h>

/******************************************************************************
 ******************************************************************************
 *
 * Module Name: normGen_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   normGen_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  2-Oct-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
normGen_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    NormGenWidgets *ip;
    static int xmin_buttons_mapped = False;
    static int xmax_buttons_mapped = False;
    static int xstep_buttons_mapped = False;
    static int ymin_buttons_mapped = False;
    static int ymax_buttons_mapped = False;
    static int ystep_buttons_mapped = False;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "genPlotToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "normGen_callback: %s\n", client );*/
	norm_params.plot = state->set;
	gray_out_item(NormGenClone->plotForm, !state->set);
 
    } else if (!strcmp(client, "autoscaleToggle")) {
	/*fprintf (stderr, "normGen_callback: %s\n", client );*/
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	norm_params.autoscale = state->set;
	gray_out_item(ip->axesForm, norm_params.autoscale);
 
    } else if (!strcmp(client, "xminText")) {
	/*fprintf (stderr, "normGen_callback: %s\n", client );*/
	xmin_buttons_mapped =
	  okTextValueChanged(ip->xminText, ip->xminOKPBtn,
			     ip->xminCancelPBtn,
			     ip->normGenDialogShell,
			     xmin_buttons_mapped,
			     norm_params.xmin_string);
      
    } else if (!strcmp(client, "xminTextAct") ||
	       !strcmp(client, "xminOKPBtn")) {
	String xminTextval;

	/*fprintf (stderr, "normGen_callback: %s\n", client );*/
	xmin_buttons_mapped = okTextActivate(ip->xminText, ip->xminOKPBtn,
					     ip->xminCancelPBtn, &xminTextval);

	XtFree (xminTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	norm_params.xmin = set_atof_value(ip->xminText, "norm_xmin",
					  norm_params.xmin);
	XtFree(norm_params.xmin_string);
	norm_params.xmin_string = get_string_text(ip->xminText);
 
    } else if (!strcmp(client, "xminCancelPBtn")) {
	/*fprintf (stderr, "normGen_callback: %s\n", client );*/
	xmin_buttons_mapped = 
	  okTextCancel(ip->xminText, ip->xminOKPBtn,
		       ip->xminCancelPBtn,
		       norm_params.xmin_string);
 
    } else if (!strcmp(client, "xmaxText")) {
	/*fprintf (stderr, "normGen_callback: %s\n", client );*/
	xmax_buttons_mapped =
	  okTextValueChanged(ip->xmaxText, ip->xmaxOKPBtn,
			     ip->xmaxCancelPBtn,
			     ip->normGenDialogShell,
			     xmax_buttons_mapped,
			     norm_params.xmax_string);
      
    } else if (!strcmp(client, "xmaxTextAct") ||
	       !strcmp(client, "xmaxOKPBtn")) {
	String xmaxTextval;

	/*fprintf (stderr, "normGen_callback: %s\n", client );*/
	xmax_buttons_mapped = okTextActivate(ip->xmaxText, ip->xmaxOKPBtn,
					     ip->xmaxCancelPBtn, &xmaxTextval);
	
	XtFree (xmaxTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	norm_params.xmax = set_atof_value(ip->xmaxText, "norm_xmax",
					  norm_params.xmax);
	XtFree(norm_params.xmax_string);
	norm_params.xmax_string = get_string_text(ip->xmaxText);
 
    } else if (!strcmp(client, "xmaxCancelPBtn")) {
	/*fprintf (stderr, "normGen_callback: %s\n", client );*/
	xmax_buttons_mapped = 
	  okTextCancel(ip->xmaxText, ip->xmaxOKPBtn,
		       ip->xmaxCancelPBtn,
		       norm_params.xmax_string);
 
    } else if (!strcmp(client, "xstepText")) {
	/*fprintf (stderr, "normGen_callback: %s\n", client );*/
	xstep_buttons_mapped =
	  okTextValueChanged(ip->xstepText, ip->xstepOKPBtn,
			     ip->xstepCancelPBtn,
			     ip->normGenDialogShell,
			     xstep_buttons_mapped,
			     norm_params.xstep_string);
      
    } else if (!strcmp(client, "xstepTextAct") ||
	       !strcmp(client, "xstepOKPBtn")) {
	String xstepTextval;
      
	/*fprintf (stderr, "normGen_callback: %s\n", client );*/
	xstep_buttons_mapped =
	  okTextActivate(ip->xstepText, ip->xstepOKPBtn,
			 ip->xstepCancelPBtn, &xstepTextval);

	XtFree (xstepTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	norm_params.xstep = set_atof_value(ip->xstepText, "norm_xstep",
					   norm_params.xstep);
	XtFree(norm_params.xstep_string);
	norm_params.xstep_string = get_string_text(ip->xstepText);
 
    } else if (!strcmp(client, "xstepCancelPBtn")) {
	/*fprintf (stderr, "normGen_callback: %s\n", client );*/
	xstep_buttons_mapped = 
	  okTextCancel(ip->xstepText, ip->xstepOKPBtn,
		       ip->xstepCancelPBtn,
		       norm_params.xstep_string);
	
    } else if (!strcmp(client, "yminText")) {
	/*fprintf (stderr, "normGen_callback: %s\n", client );*/
	ymin_buttons_mapped =
	  okTextValueChanged(ip->yminText, ip->yminOKPBtn,
			     ip->yminCancelPBtn,
			     ip->normGenDialogShell,
			     ymin_buttons_mapped,
			     norm_params.ymin_string);
      
    } else if (!strcmp(client, "yminTextAct") ||
	       !strcmp(client, "yminOKPBtn")) {
	String yminTextval;
      
	/*fprintf (stderr, "normGen_callback: %s\n", client );*/
	ymin_buttons_mapped = okTextActivate(ip->yminText, ip->yminOKPBtn,
					     ip->yminCancelPBtn, &yminTextval);

	XtFree (yminTextval);/* I won't actually be using this copy */
	
	/* check that this value is ok */
	norm_params.ymin = set_atof_value(ip->yminText, "norm_ymin",
					  norm_params.ymin);
	XtFree(norm_params.ymin_string);
	norm_params.ymin_string = get_string_text(ip->yminText);
	
    } else if (!strcmp(client, "yminCancelPBtn")) {
	/*fprintf (stderr, "normGen_callback: %s\n", client );*/
	ymin_buttons_mapped = 
	  okTextCancel(ip->yminText, ip->yminOKPBtn,
		       ip->yminCancelPBtn,
		       norm_params.ymin_string);
	
    } else if (!strcmp(client, "ymaxText")) {
	/*fprintf (stderr, "normGen_callback: %s\n", client );*/
	ymax_buttons_mapped =
	  okTextValueChanged(ip->ymaxText, ip->ymaxOKPBtn,
			     ip->ymaxCancelPBtn,
			     ip->normGenDialogShell,
			     ymax_buttons_mapped,
			     norm_params.ymax_string);
      
    } else if (!strcmp(client, "ymaxTextAct") ||
	       !strcmp(client, "ymaxOKPBtn")) {
	String ymaxTextval;
      
	/*fprintf (stderr, "normGen_callback: %s\n", client );*/
	ymax_buttons_mapped = okTextActivate(ip->ymaxText, ip->ymaxOKPBtn,
					     ip->ymaxCancelPBtn, &ymaxTextval);

	XtFree (ymaxTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	norm_params.ymax = set_atof_value(ip->ymaxText, "norm_ymax",
					  norm_params.ymax);
	XtFree(norm_params.ymax_string);
	norm_params.ymax_string = get_string_text(ip->ymaxText);
 
    } else if (!strcmp(client, "ymaxCancelPBtn")) {
	/*fprintf (stderr, "normGen_callback: %s\n", client );*/
	ymax_buttons_mapped = 
	  okTextCancel(ip->ymaxText, ip->ymaxOKPBtn,
		       ip->ymaxCancelPBtn,
		       norm_params.ymax_string);
 
    } else if (!strcmp(client, "ystepText")) {
	/*fprintf (stderr, "normGen_callback: %s\n", client );*/
	ystep_buttons_mapped =
	  okTextValueChanged(ip->ystepText, ip->ystepOKPBtn,
			     ip->ystepCancelPBtn,
			     ip->normGenDialogShell,
			     ystep_buttons_mapped,
			     norm_params.ystep_string);
      
    } else if (!strcmp(client, "ystepTextAct") ||
	       !strcmp(client, "ystepOKPBtn")) {
	String ystepTextval;
      
	/*fprintf (stderr, "normGen_callback: %s\n", client );*/
	ystep_buttons_mapped =
	  okTextActivate(ip->ystepText, ip->ystepOKPBtn,
			 ip->ystepCancelPBtn, &ystepTextval);

	XtFree (ystepTextval);/* I won't actually be using this copy */
	
	/* check that this value is ok */
	norm_params.ystep = set_atof_value(ip->ystepText, "norm_ystep",
					   norm_params.ystep);
	XtFree(norm_params.ystep_string);
	norm_params.ystep_string = get_string_text(ip->ystepText);
 
    } else if (!strcmp(client, "ystepCancelPBtn")) {
	/*fprintf (stderr, "normGen_callback: %s\n", client );*/
	ystep_buttons_mapped = 
	  okTextCancel(ip->ystepText, ip->ystepOKPBtn,
		       ip->ystepCancelPBtn,
		       norm_params.ystep_string);
 
    } else if (!strcmp(client, "runPBtn")) {
	/*fprintf (stderr, "normGen_callback: %s\n", client );*/
	if(norm_params.normalize != NONE) norm_run(EXP_AND_PLOT);
 
    } else if (!strcmp(client, "plotOnlyPBtn")) {
	/*fprintf (stderr, "normGen_callback: %s\n", client );*/
	norm_run(PLOT_ONLY);
 
    } else if (!strcmp(client, "stopPBtn")) {
	/*fprintf (stderr, "normGen_callback: %s\n", client );*/
	stop_command_file(norm_params.gen_pid);
	norm_params.gen_pid = 0;
	
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "normGen_callback: %s\n", client );*/
	norm_check_file();
	close_popup(ip->normGenForm);
 
    } else {
	fprintf(stderr, "normGen_callback: unknown client: %s\n", (char *)client);
    }
}

void
normGen_request(String request)
{
    if(!strcmp(request, "open")) {
	open_popup(NormGenClone->normGenForm);
    } else if (!strcmp(request, "init")){
	char string[32];/* for creating the float Text */

	/* these parameters were initialized to NULL before */
	XtFree (norm_params.script_fname);
	norm_params.script_fname = XtNewString(norm_params.norm_fname);
	norm_params.script_fname =
	  StringConcat(norm_params.script_fname, ".run");
	
	XtFree (norm_params.log_fname);
	norm_params.log_fname = XtNewString(norm_params.norm_fname);
	norm_params.log_fname =
	  StringConcat(norm_params.log_fname, ".log");
	
	XtFree (norm_params.plot_fname);
	norm_params.plot_fname = XtNewString(norm_params.norm_fname);
	norm_params.plot_fname =
	  StringConcat(norm_params.plot_fname, ".plot");
	
	sprintf(string, "%5.2g", norm_params.xmin);
	XtFree(norm_params.xmin_string);
	norm_params.xmin_string = XtNewString(string);

	sprintf(string, "%5.2g", norm_params.xmax);
	XtFree(norm_params.xmax_string);
	norm_params.xmax_string = XtNewString(string);

	sprintf(string, "%5.2g", norm_params.xstep);
	XtFree(norm_params.xstep_string);
	norm_params.xstep_string = XtNewString(string);

	sprintf(string, "%5.2g", norm_params.ymin);
	XtFree(norm_params.ymin_string);
	norm_params.ymin_string = XtNewString(string);

	sprintf(string, "%5.2g", norm_params.ymax);
	XtFree(norm_params.ymax_string);
	norm_params.ymax_string = XtNewString(string);

	sprintf(string, "%5.2g", norm_params.ystep);
	XtFree(norm_params.ystep_string);
	norm_params.ystep_string = XtNewString(string);

	XtFree(norm_params.plot_label);
	norm_params.plot_label = XtNewString("Norm: ");
	norm_params.plot_label =
	  StringConcat(norm_params.plot_label,
		       norm_description(norm_params.normalize));
	
    } else if (!strcmp(request, "display")){
	show_string_label(NormGenClone->normalizationLabel,
			  norm_description(norm_params.normalize));
	show_string_label(NormGenClone->normFnameLabel,
			  norm_params.norm_fname);
	show_string_label(NormGenClone->normScriptLabel,
			  norm_params.script_fname);
	show_string_label(NormGenClone->normLogLabel,
			  norm_params.log_fname);
	show_toggle_value(NormGenClone->genPlotToggle, norm_params.plot);
	gray_out_item(NormGenClone->plotForm, !norm_params.plot);
	show_string_label(NormGenClone->plotFnameLabel,
			  norm_params.plot_fname);
	show_toggle_value (NormGenClone->autoscaleToggle,
			norm_params.autoscale);
	gray_out_item(NormGenClone->axesForm, norm_params.autoscale);
	show_string_text (NormGenClone->xminText,
			  norm_params.xmin_string);
	show_string_text (NormGenClone->xmaxText,
			  norm_params.xmax_string);
	show_string_text (NormGenClone->yminText,
			  norm_params.ymin_string);
	show_string_text (NormGenClone->ymaxText,
			  norm_params.ymax_string);
	show_string_text (NormGenClone->xstepText,
			  norm_params.xstep_string);
	show_string_text (NormGenClone->ystepText,
			 norm_params.ystep_string);
	show_string_label(NormGenClone->plotLabelLabel,
			  norm_params.plot_label);
	activate_norm_plots(norm_params.normalize == PCA ||
			    norm_params.normalize == LDA);
    } else {
	fprintf(stderr, "norm_request: unknown request: %s\n", request);
    }
}

static void
activate_norm_plots(int activate_flag)
{
    gray_out_item(NormGenClone->plotForm, !activate_flag);
    gray_out_item(NormGenClone->genPlotToggle, !activate_flag);
}

static void
norm_run(RUN_TYPE run_flag)
{
    char *shell_path;
    shell_path = path_create(reports_params.exp_path,
			     norm_params.script_fname);
    if(overwrite_ok(NormGenClone->normGenForm, shell_path)){
	if(!store_norm_commands(shell_path, run_flag)){
	    if(!lnk_params.store_only){
		norm_params.gen_pid =
		  chdir_and_execute_command(norm_params.script_fname);
	    }
	}
    }
    nn_free(shell_path, "for norm script");
}

static void
norm_set_common_flags()
{
    /* norm flags */
    data_path = data_params.data_path;
    data_fname = data_params.train_fname;
    desc_fname = data_params.desc_fname;
    normalization = norm_params.normalize;
    norm_fname = norm_params.norm_fname;
    debug = 0;
    verbosity = reports_params.verbosity;
    
    /* plot_norm flags */
    plot_exp_path = reports_params.exp_path;
    plot_error_fname = norm_params.norm_fname;
    plot_n_inputs = data_params.nraw;
    plot_n_outputs = data_params.noutputs;
    /* can't do plot_labels, its pre-allocated */
    norm_data_path = data_params.data_path;
    norm_plot_fname = norm_params.plot_fname;
    norm_auto_scale_flag = norm_params.autoscale;
    norm_xmin = norm_params.xmin;
    norm_xmax = norm_params.xmax;
    norm_xstep = norm_params.xstep;
    norm_ymin = norm_params.ymin;
    norm_ymax = norm_params.ymax;
    norm_ystep = norm_params.ystep;
    /* add double quotes here to eliminate problems with spaces */
    norm_plot_label = XtMalloc(strlen(norm_params.plot_label) + 3);
    sprintf(norm_plot_label, "\"%s\"", norm_params.plot_label);
    mac_norm_flag = 0;
    mac_normlines_fname = "norm.mac";
    /* set no_graphics below */
}

static int
store_norm_commands(char *shell_path, RUN_TYPE run_flag)
{
    FILE *shell_file;

    norm_set_common_flags();
    shell_file = command_file_open(NormGenClone->normGenForm, shell_path);
    if (shell_file == NULL) return -1;

    /* store the norm command */
    if(run_flag != PLOT_ONLY){
	fprintf(shell_file, "\n#calculate normalization parameters\n");
	fprintf (shell_file, "%s %s", NORM_PROGRAM_NAME,
		 nnargs_get_part_flags(argtab, TABSIZE));
	if(lnk_params.store_only)
	  fprintf(shell_file, ">& %s\n", norm_params.log_fname);
	else
	  fprintf(shell_file, "|& nn_tee -h %s\n", norm_params.log_fname);
    }
    /* store the plot command */
    if(norm_params.plot || run_flag == PLOT_ONLY){
	if(norm_params.normalize == PCA ||
	   norm_params.normalize == LDA){
	    String plot_command_line;

	    fprintf(shell_file, "\n#plot normalization parameters\n");
	    if(lnk_params.store_only) no_graphics = 1;
	    else no_graphics = 0;

	    if(lnk_params.store_only)
	      fprintf(shell_file, "(time ");
	    plot_command_line = XtNewString(PLOT_NORM);
	    plot_command_line = StringConcat(plot_command_line, " ");
	    plot_command_line =
	      StringConcat(plot_command_line,
			   nnargs_get_flags(norm_argtab, NORM_TABSIZE));
	    /* replace the last \n with a space */
	    *strrchr(plot_command_line, '\n') = ' ';

	    fprintf(shell_file, "%s", plot_command_line);

	    if(lnk_params.store_only)
	      fprintf(shell_file, "\\\n) >>& %s\n", norm_params.log_fname);
	    else fprintf(shell_file, "\n");
	}
    }
    /* we allocated space for norm_plot_label in set_common_flags.
       free it now */
    XtFree(norm_plot_label);
    norm_plot_label = NULL;

    command_file_close(shell_file, shell_path, "for norm script");
    return 0;
}
