/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/norm/norm_callback.c,v 11.6 2004/01/15 19:30:53 kukolich Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            NORM WINDOW                               ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  29-Sep-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include <nnlib.h>
/* for checking time of training and data files */
#include <sys/types.h>
#include <sys/stat.h>
/*
 * External Functions
 */
#include "norm_func.h"
#include "../main/lnknet_subs.h"
#include "../lnk/lnk_func.h"
#include "../select/select_func.h"
#include "../reports/reports_func.h"
#include "../plots/plots_func.h"
#include "../region/region_func.h"
#include "../profile/profile_func.h"

/*
 * Externals
 */
#include "norm_params.h"
NormParams norm_params; /* this is the real one */
#include "../data/data_params.h"
extern DataParams data_params;/* for data base path and name and extensions */
#include "norm.h"
extern NormWidgets *NormClone;
#include "../main/lnknet.h"
extern LnkGlyphs *LnknetClone;/* for warning glyphs */

/******************************************************************************
 ******************************************************************************
 *
 * Module Name: norm_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   norm_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  29-Sep-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
norm_callback(Widget w, XtPointer client, XtPointer call)
{
  /*SUPPRESS 765*/
  XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
  NormWidgets *ip;
 
  /*
   * Get the parent structure
   */
  XtVaGetValues (w, XmNuserData, &ip, NULL);
 
  if (!strcmp(client, "normalizeNoneToggle") ||
      !strcmp(client, "normalizeSimpleToggle") ||
      !strcmp(client, "normalizePcaToggle") ||
      !strcmp(client, "normalizeLdaToggle")) {
      XmToggleButtonCallbackStruct *state =
	(XmToggleButtonCallbackStruct *)acs;
      /*fprintf (stderr, "norm_callback: %s\n", client );*/
      if(state->set){
	  norm_params.normalize = get_radio_value(ip->normalizeRadioBox);
	  select_request("set_ninputs");
	  g_set_normalize_filename();
	  g_create_search_filename();

	  set_plot_labels();
	  region_set_input_label();
	  profile_set_input_label();

	  /* tell notebook that something important changed */
	  reports_request("major_note");
      }
 
  } else if (!strcmp(client, "generatePBtn")) {
      /*fprintf (stderr, "norm_callback: %s\n", client );*/
      normGen_request("init");
      normGen_request("display");
      normGen_request("open");

  } else if (!strcmp(client, "updatePBtn")) {
      /*fprintf (stderr, "norm_callback: %s\n", client );*/
      norm_check_file();

  } else if (!strcmp(client, "closePBtn")) {
      /*fprintf (stderr, "norm_callback: %s\n", client );*/
      close_popup(ip->normForm);
 
  } else {
    fprintf(stderr, "norm_callback: unknown client: %s\n", (char *)client);
  }
}

void
norm_request(String request)
{
    if(!strcmp(request, "open")) {
	open_popup(NormClone->normForm);
    } else if (!strcmp(request, "init")){
	norm_params.normalize = NONE;
	norm_params.norm_fname = NULL;
	/* gen window */
	norm_params.gen_pid = 0;
	norm_params.script_fname = NULL;
	norm_params.log_fname = NULL;
	norm_params.plot = False;
	norm_params.plot_fname = NULL;
	norm_params.autoscale = True;
	norm_params.xmin = -3;
	norm_params.xmin_string = NULL;
	norm_params.xmax = 3;
	norm_params.xmax_string = NULL;
	norm_params.xstep = 1;
	norm_params.xstep_string = NULL;
	norm_params.ymin = -3;
	norm_params.ymin_string = NULL;
	norm_params.ymax = 3;
	norm_params.ymax_string = NULL;
	norm_params.ystep = 1;
	norm_params.ystep_string = NULL;
	norm_params.plot_label = NULL;
    } else if (!strcmp(request, "display")){
	show_radio_value( NormClone->normalizeRadioBox, norm_params.normalize);
	show_string_label( NormClone->normFileLabel, norm_params.norm_fname);
	norm_check_file();
    } else {
	fprintf(stderr, "norm_request: unknown request: %s\n", (char *)request);
    }
}

void
g_set_normalize_filename()
{
    static char *norm_string[4] = {"none", "simple", "pca", "lda"};

    XtFree(norm_params.norm_fname);
    norm_params.norm_fname = XtNewString(data_params.data_base);
    norm_params.norm_fname = StringConcat(norm_params.norm_fname, ".norm.");
    norm_params.norm_fname = StringConcat(norm_params.norm_fname,
					  norm_string[norm_params.normalize]);
    show_string_label(NormClone->normFileLabel, norm_params.norm_fname);
    norm_check_file();
}
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Names: read_norm_defaults, write_norm_defaults
 *
 * Purpose: 
 *   Handle reading and writing the state of the norm and norm gen defaults
 *   windows to a screen/defaults file.
 * 
 * Calling Sequence:
 *   # open defaults file for reading
 *   # read arg tables up until the norm one
 *   # fill arg list for norm one
 *   read_norm_defaults(argc, argv)
 *
 *  OR
 *   # open a defaults file for writings
 *   # write arg tables up until the norm one
 *   write_norm_defaults(default_file)
 *
 * Inputs:
 *   int argc; # the number of strings in argv
 *   char **argv; # pointer to an array String pointers
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   read sets values in norm_params structure according to flags found in argv
 *   write writes flags to default_file according to the settings of
 *   norm_params structure entries
 *
 * Restrictions:
 *   On read, if the argument list in argv doesn't match the arguments
 *   in the norm argument table, a fatal error will occur.
 *
 * Modification History:
 *  2-Oct-1995     Linda C Kukolich  modified original xview versions
 *
 ******************************************************************************
 ******************************************************************************
 */
static ARG normtab[] =
{
    {"normalize", INT, (int *)NULL, "normalization: NONE, SIMPLE, PCA, LDA"},
    /* parameters on the generation/plot window */
    {"plot", FLAG, (int *)NULL, "generate plots"},
    {"autoscale", FLAG, (int *)NULL, "autoscale norm_plot"},
    {"xmin", FLOAT, (int *)NULL,},
    {"xmax", FLOAT, (int *)NULL,},
    {"xstep", FLOAT, (int *)NULL,},
    {"ymin", FLOAT, (int *)NULL,},
    {"ymax", FLOAT, (int *)NULL,},
    {"ystep", FLOAT, (int *)NULL,},
};
#define TABSIZE (sizeof(normtab)/sizeof(ARG))

static void
norm_set_defaults_variables(dp)
     NormParams *dp;
{
    nnargs_set_variable("normalize", &dp->normalize, normtab, TABSIZE);
    nnargs_set_variable("plot", &dp->plot, normtab, TABSIZE);
    nnargs_set_variable("autoscale", &dp->autoscale, normtab, TABSIZE);
    nnargs_set_variable("xmin", (int *)&dp->xmin, normtab, TABSIZE);
    nnargs_set_variable("xmax", (int *)&dp->xmax, normtab, TABSIZE);
    nnargs_set_variable("xstep", (int *)&dp->xstep, normtab, TABSIZE);
    nnargs_set_variable("ymin", (int *)&dp->ymin, normtab, TABSIZE);
    nnargs_set_variable("ymax", (int *)&dp->ymax, normtab, TABSIZE);
    nnargs_set_variable("ystep", (int *)&dp->ystep, normtab, TABSIZE);
}

void
read_norm_defaults(int argc, char **argv)
{
    /* clear things that produce no flag when not set */
    norm_params.plot = 0;
    norm_params.autoscale = 0;

    norm_set_defaults_variables(&norm_params);

    if(argc > 0){
	argc = nnargs(argc, argv, normtab, TABSIZE);
	/* copy the strings into new memory */
    } else {
	norm_params.normalize = SIMPLE;
	norm_params.plot = 1;
	norm_params.autoscale = 1;
    }

    norm_request("display");
}

void
write_norm_defaults(FILE *default_file)
{
    norm_set_defaults_variables(&norm_params);

    if(default_file != NULL){
	fprintf(default_file, "norm %s",
		nnargs_get_flags(normtab, TABSIZE));
    }
}

static void
norm_error(string)
     char *string;
{
    show_string_label(NormClone->warningLabel, string);
    show_image_label(NormClone->warningGlyph,
		     LnknetClone->largeWarning);
    lnk_request("normWarningOn");
}

static void
norm_clear_error()
{
    void set_main_warningGlyph();
    /* clear error message if we have made it this far */
    show_string_label(NormClone->warningLabel, "");
    show_image_label(NormClone->warningGlyph,
		     LnknetClone->largeBlank);
    lnk_request("normWarningOff");
}

void
norm_check_file()
{
    time_t train_time, norm_time;
    struct stat buf;
    char *train_path, *norm_path;

    if(norm_params.normalize == NONE)
      norm_clear_error();
    else {
	/* check if the file exists */
	norm_path = path_create(data_params.data_path, norm_params.norm_fname);
	if(file_exists(norm_path)){
	    /* check that the training data file is older
	       than the normalization file */
	    train_path = path_create(data_params.data_path,
				    data_params.data_base);
	    train_path = StringConcat(train_path, data_params.ext_train);
	    if(stat(train_path, &buf) == 0) train_time = buf.st_mtime;
	    else train_time = 0;
	    XtFree(train_path);
	    if(stat(norm_path, &buf) == 0) norm_time = buf.st_mtime;
	    else norm_time = 1;
	    if(norm_time < train_time){
		norm_error("Warning: Normalization file is older than training file");
	    } else{
		norm_clear_error();
	    }
	} else {
	    norm_error("Error: Normalization file does not exist");
	}
	nn_free(norm_path,"");
    }
}
