/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/parzen/parzen_callback.c,v 11.4 2001/08/23 20:17:18 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                       Lnknet Window Interface                        ****
 ****                          Parzen Window                               ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  09-Nov-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include "../main/lnknet_subs.h"
#include "parzen.h"
#include <nnlib.h>
#include <parzen_args.h>
/*
 * External Functions
 */
#include "../plots/plots_func.h"
 
/*
 * Externals
 */
extern ParzenWidgets *ParzenClone;
#include "parzen_params.h"
ParzenParams parzen_params; /* this is the real one */
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: parzen_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   parzen_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  14-Sep-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
parzen_callback(Widget w, XtPointer client, XtPointer call)
{
    /* SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    ParzenWidgets *ip;
    static int minVar_buttons_mapped = False;
    static int spread_buttons_mapped = False;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "perClassToggle") ||
	!strcmp(client, "grandToggle")){
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "parzen_callback: %s %s\n", client,
	  (state->set)? "on": "off" );*/
	if(state->set){
	    parzen_params.grand = get_radio_value(ip->grandRadioBox);
	    set_plot_labels();
	}
	
    } else if (!strcmp(client, "uniformToggle") ||
	       !strcmp(client, "perInputToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "parzen_callback: %s %s\n", client,
	  (state->set)? "on": "off" );*/
	if(state->set){
	    parzen_params.perInput = get_radio_value(ip->perInputRadioBox);
	    set_plot_labels();
	}

    } else if (!strcmp(client, "minVarText")) {
	/*fprintf (stderr, "parzen_callback: %s\n", client );*/
	minVar_buttons_mapped =
	  okTextValueChanged(ip->minVarText, ip->minVarOKPBtn,
			     ip->minVarCancelPBtn,
			     ip->parzenDialogShell,
			     minVar_buttons_mapped,
			     parzen_params.minVar_string);
 
    } else if ((!strcmp(client, "minVarTextAct")) ||
	       (!strcmp(client, "minVarOKPBtn"))) {
	String minVarTextval;

	/*fprintf (stderr, "parzen_callback: %s\n", client );*/
	minVar_buttons_mapped =
	  okTextActivate(ip->minVarText, ip->minVarOKPBtn,
			 ip->minVarCancelPBtn, &minVarTextval);
	/* I won't actually be using this copy */
	XtFree (minVarTextval);
	
	/* verify the string as an integer */
	parzen_params.minVar =
	  set_atof_low_value(ip->minVarText, "minVar_set",
			     parzen_params.minVar, 0.);

	/* save the current text for future comparisons */
	XtFree(parzen_params.minVar_string);
	parzen_params.minVar_string = get_string_text(ip->minVarText);
 
    } else if (!strcmp(client, "minVarCancelPBtn")) {
	/*fprintf (stderr, "parzen_callback: %s\n", client );*/
	minVar_buttons_mapped = 
	  okTextCancel(ip->minVarText, ip->minVarOKPBtn,
		       ip->minVarCancelPBtn,
		       parzen_params.minVar_string);
 
    } else if (!strcmp(client, "spreadText")) {
	/*fprintf (stderr, "parzen_callback: %s\n", client );*/
	spread_buttons_mapped =
	  okTextValueChanged(ip->spreadText, ip->spreadOKPBtn,
			     ip->spreadCancelPBtn,
			     ip->parzenDialogShell,
			     spread_buttons_mapped,
			     parzen_params.spread_string);
 
    } else if ((!strcmp(client, "spreadTextAct")) ||
	       (!strcmp(client, "spreadOKPBtn"))) {
	String spreadTextval;

	/*fprintf (stderr, "parzen_callback: %s\n", client );*/
	spread_buttons_mapped =
	  okTextActivate(ip->spreadText, ip->spreadOKPBtn,
			 ip->spreadCancelPBtn, &spreadTextval);
	/* I won't actually be using this copy */
	XtFree (spreadTextval);
	
	/* verify the string as an integer */
	parzen_params.spread =
	  set_atof_low_value(ip->spreadText, "spread_set",
			     parzen_params.spread, 0.);

	/* save the current text for future comparisons */
	XtFree(parzen_params.spread_string);
	parzen_params.spread_string = get_string_text(ip->spreadText);
 
    } else if (!strcmp(client, "spreadCancelPBtn")) {
	/*fprintf (stderr, "parzen_callback: %s\n", client );*/
	spread_buttons_mapped = 
	  okTextCancel(ip->spreadText, ip->spreadOKPBtn,
		       ip->spreadCancelPBtn,
		       parzen_params.spread_string);

    } else if (!strcmp(client, "gaussToggle") ||
	       !strcmp(client, "linearToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "parzen_callback: %s %s\n", client,
	  (state->set)? "on": "off" );*/
	if(state->set){
	    parzen_params.linear = get_radio_value(ip->linearRadioBox);
	    set_plot_labels();
	}
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "parzen_callback: %s\n", client);*/
	close_popup(ParzenClone->parzenForm);

    } else {
	fprintf (stderr, "parzen_callback: unknown client: %s\n", (char *)client);
    }
}

void
parzen_request(String request)
{
    if (!strcmp(request, "open")) {
	open_popup(ParzenClone->parzenForm);
    } else if (!strcmp(request, "init")){
	/* this should only get called once, before ANYTHING 
	   gets allocated or there will be memory leaks */
	parzen_params.grand = 0;
	parzen_params.perInput = 0;
	parzen_params.minVar = 0.;
	parzen_params.minVar_string = NULL;
	parzen_params.spread = 0.;
	parzen_params.spread_string = NULL;
    } else if (!strcmp(request, "display")){
	char string[10]; /* for creating int and float text */

	show_radio_value(ParzenClone->grandRadioBox, parzen_params.grand);
	show_radio_value(ParzenClone->perInputRadioBox,
			 parzen_params.perInput);
	sprintf(string, "%5.2g", parzen_params.minVar);
	XtFree(parzen_params.minVar_string);
	parzen_params.minVar_string = XtNewString(string);
	show_string_text(ParzenClone->minVarText, parzen_params.minVar_string);

	sprintf(string, "%5.2g", parzen_params.spread);
	XtFree(parzen_params.spread_string);
	parzen_params.spread_string = XtNewString(string);
	show_string_text(ParzenClone->spreadText, parzen_params.spread_string);

	show_radio_value(ParzenClone->linearRadioBox,
			 parzen_params.linear);
    } else {
	fprintf (stderr, "parzen_request: unknown request: %s\n", request);
    }
}

String
parzen_get_plot_label(String string)
{
    if(parzen_params.linear) string = StringConcat(string, " Step Functions");
    else string = StringConcat(string, " Gaussian Functions");
    if(parzen_params.perInput) string = StringConcat(string, " Ellipse");
    else string = StringConcat(string, " Sphere");
    if(parzen_params.grand) string = StringConcat(string, " Grand");
    else string = StringConcat(string, " Per Class");
    return string;
}

static void
parzen_set_read_variables()
{
    parzen_params.grand = !per_class;
    parzen_params.perInput = per_input;
    parzen_params.linear = linear;
    parzen_params.minVar = minimum_variance;
    parzen_params.spread = spread;
}

static void
parzen_set_write_variables()
{
    per_class = !parzen_params.grand;
    per_input = parzen_params.perInput;
    minimum_variance = parzen_params.minVar;
    spread = parzen_params.spread;
    linear = parzen_params.linear;
}

void
read_parzen_defaults(int argc, char **argv)
{
    per_input = 0;
    per_class = 0;
    linear = 0;
    if(argc > 0){
	argc = nnargs(argc, argv, parzen_tab, PARZEN_TABSIZE);
	parzen_set_read_variables();
    } else {
	parzen_params.grand = 1;
	parzen_params.perInput = 0;
	parzen_params.minVar = 0.00001;
	parzen_params.spread = 1.;
	parzen_params.linear = 0;
    }
    parzen_request("display");
}

void
write_parzen_defaults(default_file)
     FILE *default_file;
{
    parzen_set_write_variables();
    if(default_file != NULL){
	fprintf(default_file, "parzen %s",
		nnargs_get_flags(parzen_tab, PARZEN_TABSIZE));
    }
}

String
parzen_get_args ()
{
    parzen_set_write_variables();
    return (nnargs_get_part_flags(parzen_tab, PARZEN_TABSIZE));
}

String
parzen_get_important_args ()
{
    parzen_set_write_variables();
    return (nnargs_get_flags_one_line(parzen_tab, PARZEN_TABSIZE));
}

void
parzen_store_plot_command(FILE *shell_file)
{
    if(parzen_params.linear)
      fprintf(shell_file, "\\\n -linear -spread %f", parzen_params.spread);
    else
      fprintf(shell_file, "\\\n -spread %f", parzen_params.spread);
}
