/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/perr/perr_callback.c,v 11.4 2001/08/23 20:17:20 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            PERR                              ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  6-Nov-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include <nnlib.h>
#include "perr.h"
/*
 * External Functions
 */
#include "perr_func.h"
#include "../main/lnknet_subs.h"
#include "../lnk/lnk_func.h"
#include "../plots/plots_func.h"
#include "../print/print_func.h"
 
/*
 * Externals
 */
extern PerrWidgets *PerrClone;
#include "perr_params.h"
PerrParams perr_params; /* this is the real one */
#include "../data/data_params.h"
extern DataParams data_params;/* for data base path and name and extensions */
#include "../plots/plots_params.h"
extern PlotsParams plots_params;
#include "../lnk/lnk_params.h"
extern LnkParams lnk_params;
#include "../reports/reports_params.h"
extern ReportsParams reports_params;
#include "../priors/priors_params.h"
extern PriorsParams priors_params;

/*
 * Statics
 */
static void perr_set_read_variables();
static void perr_set_write_variables();
static void perr_set_common_flags();
#define PLOT_PERR "plot_perr"
#include <plot_perr_args.h>
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: perr_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   perr_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  6-Nov-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
perr_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    PerrWidgets *ip;
    static int nPoints_buttons_mapped = False;
    static int xmin_buttons_mapped = False;
    static int xmax_buttons_mapped = False;
    static int xstep_buttons_mapped = False;
    static int ymin_buttons_mapped = False;
    static int ymax_buttons_mapped = False;
    static int ystep_buttons_mapped = False;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "nPointsText")) {
	/*fprintf (stderr, "perr_callback: %s\n", client );*/
	nPoints_buttons_mapped =
	  okTextValueChanged(ip->nPointsText, ip->nPointsOKPBtn,
			     ip->nPointsCancelPBtn,
			     ip->perrDialogShell,
			     nPoints_buttons_mapped,
			     perr_params.nPoints_string);
	
    } else if ((!strcmp(client, "nPointsTextAct")) ||
	       (!strcmp(client, "nPointsOKPBtn"))) {
	String nPointsTextval;

	/*fprintf (stderr, "perr_callback: %s\n", client );*/
	nPoints_buttons_mapped =
	  okTextActivate(ip->nPointsText, ip->nPointsOKPBtn,
			 ip->nPointsCancelPBtn, &nPointsTextval);
	/* I won't actually be using this copy */
	XtFree (nPointsTextval);
	
	/* verify the string as an integer */
	perr_params.nPoints =
	  set_atoi_value(ip->nPointsText, "nPoints_set",
			     perr_params.nPoints);

	/* save the current text for future comparisons */
	XtFree(perr_params.nPoints_string);
	perr_params.nPoints_string = get_string_text(ip->nPointsText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "nPointsCancelPBtn")) {
	/*fprintf (stderr, "perr_callback: %s\n", client );*/
	nPoints_buttons_mapped = 
	  okTextCancel(ip->nPointsText, ip->nPointsOKPBtn,
		       ip->nPointsCancelPBtn,
		       perr_params.nPoints_string);
 
    } else if (!strcmp(client, "autoscaleToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "perr_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	perr_params.autoscale = state->set;
	gray_out_item(PerrClone->axesForm, state->set);
 
    } else if (!strcmp(client, "xminText")) {
	/*fprintf (stderr, "perr_callback: %s\n", client );*/
	xmin_buttons_mapped =
	  okTextValueChanged(ip->xminText, ip->xminOKPBtn,
			     ip->xminCancelPBtn,
			     ip->perrDialogShell,
			     xmin_buttons_mapped,
			     perr_params.xmin_string);
      
    } else if (!strcmp(client, "xminTextAct") ||
	       !strcmp(client, "xminOKPBtn")) {
	String xminTextval;

	/*fprintf (stderr, "perr_callback: %s\n", client );*/
	xmin_buttons_mapped = okTextActivate(ip->xminText, ip->xminOKPBtn,
					     ip->xminCancelPBtn, &xminTextval);

	XtFree (xminTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	perr_params.xmin = set_atof_value(ip->xminText, "perr_xmin",
					  perr_params.xmin);
	XtFree(perr_params.xmin_string);
	perr_params.xmin_string = get_string_text(ip->xminText);
 
    } else if (!strcmp(client, "xminCancelPBtn")) {
	/*fprintf (stderr, "perr_callback: %s\n", client );*/
	xmin_buttons_mapped = 
	  okTextCancel(ip->xminText, ip->xminOKPBtn,
		       ip->xminCancelPBtn,
		       perr_params.xmin_string);
 
    } else if (!strcmp(client, "xmaxText")) {
	/*fprintf (stderr, "perr_callback: %s\n", client );*/
	xmax_buttons_mapped =
	  okTextValueChanged(ip->xmaxText, ip->xmaxOKPBtn,
			     ip->xmaxCancelPBtn,
			     ip->perrDialogShell,
			     xmax_buttons_mapped,
			     perr_params.xmax_string);
      
    } else if (!strcmp(client, "xmaxTextAct") ||
	       !strcmp(client, "xmaxOKPBtn")) {
	String xmaxTextval;

	/*fprintf (stderr, "perr_callback: %s\n", client );*/
	xmax_buttons_mapped = okTextActivate(ip->xmaxText, ip->xmaxOKPBtn,
					     ip->xmaxCancelPBtn, &xmaxTextval);
	
	XtFree (xmaxTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	perr_params.xmax = set_atof_value(ip->xmaxText, "perr_xmax",
					  perr_params.xmax);
	XtFree(perr_params.xmax_string);
	perr_params.xmax_string = get_string_text(ip->xmaxText);
 
    } else if (!strcmp(client, "xmaxCancelPBtn")) {
	/*fprintf (stderr, "perr_callback: %s\n", client );*/
	xmax_buttons_mapped = 
	  okTextCancel(ip->xmaxText, ip->xmaxOKPBtn,
		       ip->xmaxCancelPBtn,
		       perr_params.xmax_string);
 
    } else if (!strcmp(client, "xstepText")) {
	/*fprintf (stderr, "perr_callback: %s\n", client );*/
	xstep_buttons_mapped =
	  okTextValueChanged(ip->xstepText, ip->xstepOKPBtn,
			     ip->xstepCancelPBtn,
			     ip->perrDialogShell,
			     xstep_buttons_mapped,
			     perr_params.xstep_string);
      
    } else if (!strcmp(client, "xstepTextAct") ||
	       !strcmp(client, "xstepOKPBtn")) {
	String xstepTextval;
      
	/*fprintf (stderr, "perr_callback: %s\n", client );*/
	xstep_buttons_mapped =
	  okTextActivate(ip->xstepText, ip->xstepOKPBtn,
			 ip->xstepCancelPBtn, &xstepTextval);

	XtFree (xstepTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	perr_params.xstep = set_atof_value(ip->xstepText, "perr_xstep",
					   perr_params.xstep);
	XtFree(perr_params.xstep_string);
	perr_params.xstep_string = get_string_text(ip->xstepText);
 
    } else if (!strcmp(client, "xstepCancelPBtn")) {
	/*fprintf (stderr, "perr_callback: %s\n", client );*/
	xstep_buttons_mapped = 
	  okTextCancel(ip->xstepText, ip->xstepOKPBtn,
		       ip->xstepCancelPBtn,
		       perr_params.xstep_string);
	
    } else if (!strcmp(client, "yminText")) {
	/*fprintf (stderr, "perr_callback: %s\n", client );*/
	ymin_buttons_mapped =
	  okTextValueChanged(ip->yminText, ip->yminOKPBtn,
			     ip->yminCancelPBtn,
			     ip->perrDialogShell,
			     ymin_buttons_mapped,
			     perr_params.ymin_string);
      
    } else if (!strcmp(client, "yminTextAct") ||
	       !strcmp(client, "yminOKPBtn")) {
	String yminTextval;
      
	/*fprintf (stderr, "perr_callback: %s\n", client );*/
	ymin_buttons_mapped = okTextActivate(ip->yminText, ip->yminOKPBtn,
					     ip->yminCancelPBtn, &yminTextval);

	XtFree (yminTextval);/* I won't actually be using this copy */
	
	/* check that this value is ok */
	perr_params.ymin = set_atof_value(ip->yminText, "perr_ymin",
					  perr_params.ymin);
	XtFree(perr_params.ymin_string);
	perr_params.ymin_string = get_string_text(ip->yminText);
	
    } else if (!strcmp(client, "yminCancelPBtn")) {
	/*fprintf (stderr, "perr_callback: %s\n", client );*/
	ymin_buttons_mapped = 
	  okTextCancel(ip->yminText, ip->yminOKPBtn,
		       ip->yminCancelPBtn,
		       perr_params.ymin_string);
	
    } else if (!strcmp(client, "ymaxText")) {
	/*fprintf (stderr, "perr_callback: %s\n", client );*/
	ymax_buttons_mapped =
	  okTextValueChanged(ip->ymaxText, ip->ymaxOKPBtn,
			     ip->ymaxCancelPBtn,
			     ip->perrDialogShell,
			     ymax_buttons_mapped,
			     perr_params.ymax_string);
      
    } else if (!strcmp(client, "ymaxTextAct") ||
	       !strcmp(client, "ymaxOKPBtn")) {
	String ymaxTextval;
      
	/*fprintf (stderr, "perr_callback: %s\n", client );*/
	ymax_buttons_mapped = okTextActivate(ip->ymaxText, ip->ymaxOKPBtn,
					     ip->ymaxCancelPBtn, &ymaxTextval);

	XtFree (ymaxTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	perr_params.ymax = set_atof_value(ip->ymaxText, "perr_ymax",
					  perr_params.ymax);
	XtFree(perr_params.ymax_string);
	perr_params.ymax_string = get_string_text(ip->ymaxText);
 
    } else if (!strcmp(client, "ymaxCancelPBtn")) {
	/*fprintf (stderr, "perr_callback: %s\n", client );*/
	ymax_buttons_mapped = 
	  okTextCancel(ip->ymaxText, ip->ymaxOKPBtn,
		       ip->ymaxCancelPBtn,
		       perr_params.ymax_string);
 
    } else if (!strcmp(client, "ystepText")) {
	/*fprintf (stderr, "perr_callback: %s\n", client );*/
	ystep_buttons_mapped =
	  okTextValueChanged(ip->ystepText, ip->ystepOKPBtn,
			     ip->ystepCancelPBtn,
			     ip->perrDialogShell,
			     ystep_buttons_mapped,
			     perr_params.ystep_string);
      
    } else if (!strcmp(client, "ystepTextAct") ||
	       !strcmp(client, "ystepOKPBtn")) {
	String ystepTextval;
      
	/*fprintf (stderr, "perr_callback: %s\n", client );*/
	ystep_buttons_mapped =
	  okTextActivate(ip->ystepText, ip->ystepOKPBtn,
			 ip->ystepCancelPBtn, &ystepTextval);

	XtFree (ystepTextval);/* I won't actually be using this copy */
	
	/* check that this value is ok */
	perr_params.ystep = set_atof_value(ip->ystepText, "perr_ystep",
					   perr_params.ystep);
	XtFree(perr_params.ystep_string);
	perr_params.ystep_string = get_string_text(ip->ystepText);
 
    } else if (!strcmp(client, "ystepCancelPBtn")) {
	/*fprintf (stderr, "perr_callback: %s\n", client );*/
	ystep_buttons_mapped = 
	  okTextCancel(ip->ystepText, ip->ystepOKPBtn,
		       ip->ystepCancelPBtn,
		       perr_params.ystep_string);
	/*
	 * end of axes scales callbacks
	 */
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "perr_callback: %s\n", client );*/
	close_popup(PerrClone->perrForm);

 
    } else {
	fprintf(stderr, "perr_callback: unknown client: %s\n", (char *)client);
    }
}

void
perr_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(PerrClone->perrForm);
    } else if (!strcmp(request, "init")){
	perr_params.nPoints = 0;
	perr_params.nPoints_string = NULL;
	perr_params.xmin = 0;
	perr_params.xmin_string = NULL;
	perr_params.xmax = 0;
	perr_params.xmax_string = NULL;
	perr_params.xstep = 0;
	perr_params.xstep_string = NULL;
	perr_params.ymin = 0;
	perr_params.ymin_string = NULL;
	perr_params.ymax = 0;
	perr_params.ymax_string = NULL;
	perr_params.ystep = 0;
	perr_params.ystep_string = NULL;

    } else if (!strcmp(request, "display")){
	char string[10];
	show_string_label(PerrClone->plotFnameLabel,
			  perr_params.plotFname);
	XtFree(perr_params.nPoints_string);
	sprintf(string, "%d", perr_params.nPoints);
	perr_params.nPoints_string = XtNewString(string);
	show_string_text(PerrClone->nPointsText,
			 perr_params.nPoints_string);
	show_toggle_value (PerrClone->autoscaleToggle,
			   perr_params.autoscale);
	gray_out_item(PerrClone->axesForm, perr_params.autoscale);
	XtFree(perr_params.xmin_string);
	sprintf(string, "%5.2g", perr_params.xmin);
	perr_params.xmin_string = XtNewString(string);
	show_string_text (PerrClone->xminText,
			  perr_params.xmin_string);

	XtFree(perr_params.xmax_string);
	sprintf(string, "%5.2g", perr_params.xmax);
	perr_params.xmax_string = XtNewString(string);
	show_string_text (PerrClone->xmaxText,
			  perr_params.xmax_string);

	XtFree(perr_params.xstep_string);
	sprintf(string, "%5.2g", perr_params.xstep);
	perr_params.xstep_string = XtNewString(string);
	show_string_text (PerrClone->xstepText,
			  perr_params.xstep_string);

	XtFree(perr_params.ymin_string);
	sprintf(string, "%5.2g", perr_params.ymin);
	perr_params.ymin_string = XtNewString(string);
	show_string_text (PerrClone->yminText,
			  perr_params.ymin_string);

	XtFree(perr_params.ymax_string);
	sprintf(string, "%5.2g", perr_params.ymax);
	perr_params.ymax_string = XtNewString(string);
	show_string_text (PerrClone->ymaxText,
			  perr_params.ymax_string);

	XtFree(perr_params.ystep_string);
	sprintf(string, "%5.2g", perr_params.ystep);
	perr_params.ystep_string = XtNewString(string);
	show_string_text (PerrClone->ystepText,
			 perr_params.ystep_string);

	show_string_label(PerrClone->plotLabelLabel,
			  plots_params.plot_label);
    } else if (!strcmp(request, "plotFname")){
	XtFree(perr_params.plotFname);
	perr_params.plotFname = XtNewString(lnk_params.exp_name);
	perr_params.plotFname = StringConcat(perr_params.plotFname,
					     ".perr.plot");
	show_string_label(PerrClone->plotFnameLabel,
			  perr_params.plotFname);
	print_set_perr_status();

    } else if (!strcmp(request, "plotLabel")){
	show_string_label(PerrClone->plotLabelLabel,
			  plots_params.plot_label);

    } else if (!strcmp(request, "nPoints")){
	char string[10];
	perr_params.nPoints = priors_params.priorsNpatterns;
	XtFree(perr_params.nPoints_string);
	sprintf(string, "%d", perr_params.nPoints);
	perr_params.nPoints_string = XtNewString(string);
	show_string_text(PerrClone->nPointsText,
			 perr_params.nPoints_string);

    } else {
	fprintf(stderr, "perr_request: unknown request: %s\n", request);
    }
}

static void
perr_set_read_variables()
{
    perr_params.nPoints = plot_trial_average;
    perr_params.autoscale = perr_auto_scale_flag;
    perr_params.xmin = perr_xmin;
    perr_params.xmax = perr_xmax;
    perr_params.xstep = perr_xstep;
    perr_params.ymin = perr_ymin;
    perr_params.ymax = perr_ymax;
    perr_params.ystep = perr_ystep;
}

static void
perr_set_write_variables()
{
    plot_exp_path = NULL;
    plot_error_fname = NULL;
    plot_trial_average = perr_params.nPoints;
    no_graphics = 0;

    perr_auto_scale_flag = perr_params.autoscale;
    perr_xmin = perr_params.xmin;
    perr_xmax = perr_params.xmax;
    perr_xstep = perr_params.xstep;
    perr_ymin = perr_params.ymin;
    perr_ymax = perr_params.ymax;
    perr_ystep = perr_params.ystep;
    perr_plot_label = NULL;
    mac_perr_flag = 0;
    mac_perrlines_fname = NULL;
    class_data = 0;
    seq_data = 0;
    map_data = 0;
}

void
read_perr_defaults(argc, argv)
     int argc;
     char **argv;
{
    plot_exp_path = NULL;
    plot_error_fname = NULL;
    no_graphics = 0;
    perr_auto_scale_flag = 0;
    perr_plot_fname = NULL;
    perr_plot_label = NULL;
    mac_perrlines_fname = NULL;
    class_data = 0;
    seq_data = 0;
    map_data = 0;
    if(argc > 0){
	argc = nnargs(argc, argv, perr_argtab, PERR_TABSIZE);
	perr_set_read_variables();
    } else {
	/* set any defaults */
	perr_params.autoscale = 1;
	perr_params.xmin = 0.;
	perr_params.xmax = 10000.;
	perr_params.xstep = 1000.;
	perr_params.ymin = 0;
	perr_params.ymax = 100.;
	perr_params.ystep = 10.;
    }
    perr_request("display");
}

void
write_perr_defaults(FILE *default_file)
{
    perr_set_write_variables();
    if(default_file != NULL){
	fprintf(default_file, "perr %s",
		nnargs_get_flags(perr_argtab, PERR_TABSIZE));
    }
}

void
store_perr_plot_command(FILE *shell_file)
{
    char *plot_command_line;

    if(lnk_params.store_only) no_graphics = 1;
    else no_graphics = 0;
    class_data = 1;
    map_data = seq_data = 0;

    store_plot_start(shell_file);
    perr_set_common_flags();
    plot_command_line =
      XtNewString(nnargs_get_flags(perr_argtab, PERR_TABSIZE));
    *strrchr(plot_command_line, '\n') = ' ';	/* get rid of newline at end */
    fprintf(shell_file, "%s\\\n %s",
	    PLOT_PERR, plot_command_line);
    store_plot_end(shell_file);

    /* free things allocated in common_flags */
    XtFree(perr_plot_label);
    XtFree(plot_error_fname);
    XtFree(plot_command_line);
}

static void
perr_set_common_flags()
{
    plot_exp_path = reports_params.exp_path;
    plot_error_fname = XtNewString(reports_params.error_fname);
    plot_error_fname = StringConcat(plot_error_fname, data_params.ext_train);
    plot_trial_average = perr_params.nPoints;
    /* no_graphics is already set */
    perr_plot_fname = perr_params.plotFname;
    perr_auto_scale_flag = perr_params.autoscale;
    perr_xmin = perr_params.xmin;
    perr_xmax = perr_params.xmax;
    perr_xstep = perr_params.xstep;
    perr_ymin = perr_params.ymin;
    perr_ymax = perr_params.ymax;
    perr_ystep = perr_params.ystep;

    /* mac flags */
    mac_perr_flag = 0;
    mac_perrlines_fname = "perr.mac";

    /* add double quotes here to eliminate problems with spaces */
    perr_plot_label = XtMalloc(strlen(plots_params.plot_label) + 3);
    sprintf(perr_plot_label, "\"%s\"",plots_params.plot_label);
}
