/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/plots/plots_callback.c,v 11.4 2001/08/23 20:17:22 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            PLOTS                                     ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  24-Oct-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include <nnlib.h>
#include "plots.h"
#include "plots_params.h"
#include "../main/lnknet_subs.h"
/*
 * External Functions
 */
#include "../lnk/lnk_func.h"
#include "plots_func.h"
#include "../reports/reports_func.h"
#include "../region/region_func.h"
#include "../profile/profile_func.h"
#include "../struct/struct_func.h"
#include "../cost/cost_func.h"
#include "../perr/perr_func.h"
#include "../prob/prob_func.h"
#include "../detect/detect_func.h"
#include "../reject/reject_func.h"
 
/*
 * Externals
 */
extern PlotsWidgets *PlotsClone;
PlotsParams plots_params;
#include "../reports/reports_params.h"
extern ReportsParams reports_params;
#include "../lnk/lnk_params.h"
extern LnkParams lnk_params;
#include "../region/region_params.h"
extern RegionParams region_params;
#include "../profile/profile_params.h"
extern ProfileParams profile_params;

/*
 * Statics
 */
static void plots_set_defaults_variables(PlotsParams *dp);
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: plots_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   plots_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  24-Oct-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
plots_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    PlotsWidgets *ip;
    static int ePerPlot_buttons_mapped = False;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "regionToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "plots_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	plots_params.region = state->set;
 
    } else if (!strcmp(client, "scatterToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "plots_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	plots_params.scatter = state->set;
 
    } else if (!strcmp(client, "internalsToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "plots_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	plots_params.internals = state->set;
 
    } else if (!strcmp(client, "regionParamsPBtn")) {
	/*fprintf (stderr, "plots_callback: %s\n", client );*/
	region_request("open");
 
    } else if (!strcmp(client, "profileToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "plots_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	plots_params.profile = state->set;
 
    } else if (!strcmp(client, "histogramToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "plots_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	plots_params.histogram = state->set;
 
    } else if (!strcmp(client, "profileParamsPBtn")) {
	/*fprintf (stderr, "plots_callback: %s\n", client );*/
	profile_request("open");
 
    } else if (!strcmp(client, "noTrainPlotsToggle") ||
	       !strcmp(client, "doTrainPlotsToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "plots_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set) plots_params.doTrainPlots =
	  get_radio_value(ip->doTrainPlotsRadioBox);
 
    } else if (!strcmp(client, "epochPlotsToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "plots_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	plots_params.epochPlots = state->set;
	gray_out_item(ip->ePerPlotForm, !plots_params.epochPlots);
 
    } else if (!strcmp(client, "ePerPlotText")) {
	/*fprintf (stderr, "plots_callback: %s\n", client );*/
	ePerPlot_buttons_mapped =
	  okTextValueChanged(ip->ePerPlotText, ip->ePerPlotOKPBtn,
			     ip->ePerPlotCancelPBtn,
			     ip->plotsDialogShell,
			     ePerPlot_buttons_mapped,
			     plots_params.ePerPlot_string);
	
    } else if ((!strcmp(client, "ePerPlotTextAct")) ||
	       (!strcmp(client, "ePerPlotOKPBtn"))) {
	String ePerPlotTextval;

	/*fprintf (stderr, "plots_callback: %s\n", client );*/
	ePerPlot_buttons_mapped =
	  okTextActivate(ip->ePerPlotText, ip->ePerPlotOKPBtn,
			 ip->ePerPlotCancelPBtn, &ePerPlotTextval);
	/* I won't actually be using this copy */
	XtFree (ePerPlotTextval);
	
	/* verify the string as an integer */
	plots_params.ePerPlot =
	  set_atoi_value(ip->ePerPlotText, "ePerPlot_set",
			     plots_params.ePerPlot);

	/* save the current text for future comparisons */
	XtFree(plots_params.ePerPlot_string);
	plots_params.ePerPlot_string = get_string_text(ip->ePerPlotText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "ePerPlotCancelPBtn")) {
	/*fprintf (stderr, "plots_callback: %s\n", client );*/
	ePerPlot_buttons_mapped = 
	  okTextCancel(ip->ePerPlotText, ip->ePerPlotOKPBtn,
		       ip->ePerPlotCancelPBtn,
		       plots_params.ePerPlot_string);
 
    } else if (!strcmp(client, "structToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "plots_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	plots_params.struct_plot = state->set;
 
    } else if (!strcmp(client, "structParamsPBtn")) {
	/*fprintf (stderr, "plots_callback: %s\n", client );*/
	struct_request("open");
 
    } else if (!strcmp(client, "costToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "plots_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	plots_params.cost = state->set;
	plots_set_verror();
 
    } else if (!strcmp(client, "costParamsPBtn")) {
	/*fprintf (stderr, "plots_callback: %s\n", client );*/
	cost_request("open");
 
    } else if (!strcmp(client, "perrToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "plots_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	plots_params.perr = state->set;
	plots_set_verror();
 
    } else if (!strcmp(client, "perrParamsPBtn")) {
	/*fprintf (stderr, "plots_callback: %s\n", client );*/
	perr_request("open");
 
    } else if (!strcmp(client, "probToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "plots_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	plots_params.prob = state->set;
	plots_set_verror();
 
    } else if (!strcmp(client, "probParamsPBtn")) {
	/*fprintf (stderr, "plots_callback: %s\n", client );*/
 	prob_request("open");

    } else if (!strcmp(client, "detectToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "plots_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	plots_params.detect = state->set;
	plots_set_verror();
 
    } else if (!strcmp(client, "detectParamsPBtn")) {
	/*fprintf (stderr, "plots_callback: %s\n", client );*/
	detect_request("open");
 
    } else if (!strcmp(client, "rejectToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "plots_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	plots_params.reject = state->set;
	plots_set_verror();
 
    } else if (!strcmp(client, "rejectParamsPBtn")) {
	/*fprintf (stderr, "plots_callback: %s\n", client );*/
	reject_request("open");
 
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "plots_callback: %s\n", client );*/
	close_popup(PlotsClone->plotsForm);
 
    } else {
	fprintf(stderr, "plots_callback: unknown client: %s\n", (char *)client);
    }
}

void
plots_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(PlotsClone->plotsForm);
    } else if (!strcmp(request, "init")){
	plots_params.region = 0;
	plots_params.scatter = 0;
	plots_params.internals = 0;
	plots_params.profile = 0;
	plots_params.histogram = 0;
	plots_params.doTrainPlots = 0;
	plots_params.epochPlots = 0;
	plots_params.ePerPlot = 0;
	plots_params.ePerPlot_string = NULL;
	plots_params.storeEpoch = 0;
	plots_params.regionEpochFname = "$bpfname";
	plots_params.profileEpochFname = "$ppfname";
	plots_params.struct_plot = 0;
	plots_params.cost = 0;
	plots_params.perr = 0;
	plots_params.prob = 0;
	plots_params.detect = 0;
	plots_params.reject = 0;
	plots_params.plot_label = NULL;
    } else if (!strcmp(request, "display")){
	char string[10];
	show_toggle_value(PlotsClone->regionToggle, plots_params.region);
	show_toggle_value(PlotsClone->scatterToggle, plots_params.scatter);
	show_toggle_value(PlotsClone->internalsToggle, plots_params.internals);
	show_toggle_value(PlotsClone->profileToggle, plots_params.profile);
	show_toggle_value(PlotsClone->histogramToggle, plots_params.histogram);
	show_radio_value(PlotsClone->doTrainPlotsRadioBox, plots_params.doTrainPlots);
	show_toggle_value(PlotsClone->epochPlotsToggle, plots_params.epochPlots);
	gray_out_item(PlotsClone->ePerPlotForm, !plots_params.epochPlots);
	XtFree(plots_params.ePerPlot_string);
	sprintf(string, "%d", plots_params.ePerPlot);
	plots_params.ePerPlot_string = XtNewString(string);
	show_string_text(PlotsClone->ePerPlotText,
			 plots_params.ePerPlot_string);
	show_toggle_value(PlotsClone->structToggle, plots_params.struct_plot);
	show_toggle_value(PlotsClone->costToggle, plots_params.cost);
	show_toggle_value(PlotsClone->perrToggle, plots_params.perr);
	show_toggle_value(PlotsClone->probToggle, plots_params.prob);
	show_toggle_value(PlotsClone->detectToggle, plots_params.detect);
	show_toggle_value(PlotsClone->rejectToggle, plots_params.reject);
    } else {
	fprintf(stderr, "plots_request: unknown request: %s\n", request);
    }
}

void
plots_arg_request(String request, int arg)
{
    if (!strcmp(request, "cost")){
	gray_out_item(PlotsClone->costForm, !arg);
	if(arg){
	    plots_params.cost = get_toggle_value(PlotsClone->costToggle);
	    plots_set_verror();
	} else {
	    plots_params.cost = 0;
	}
    } else if (!strcmp(request, "perr")){
	gray_out_item(PlotsClone->perrForm, !arg);
	if(arg){
	    plots_params.perr = get_toggle_value(PlotsClone->perrToggle);
	    plots_set_verror();
	} else {
	    plots_params.perr = 0;
	}
    } else if (!strcmp(request, "testError")){
	gray_out_item(PlotsClone->testErrorForm, !arg);
	if(arg){
	    plots_params.prob = get_toggle_value(PlotsClone->probToggle);
	    plots_params.detect = get_toggle_value(PlotsClone->detectToggle);
	    plots_params.reject = get_toggle_value(PlotsClone->rejectToggle);
	    plots_set_verror();
	} else {
	    plots_params.prob = 0;
	    plots_params.detect = 0;
	    plots_params.reject = 0;
	}
    } else if (!strcmp(request, "profile")){
	gray_out_item(PlotsClone->profileForm, !arg);
	if(arg){
	    plots_params.profile = get_toggle_value(PlotsClone->profileToggle);
	    plots_params.histogram = get_toggle_value(PlotsClone->histogramToggle);
	} else {
	    plots_params.profile = 0;
	    plots_params.histogram = 0;
	}
    } else if (!strcmp(request, "region")){
	gray_out_item(PlotsClone->regionForm, !arg);
	if(arg){
	    plots_params.region = get_toggle_value(PlotsClone->regionToggle);
	    plots_params.scatter = get_toggle_value(PlotsClone->scatterToggle);
	    plots_params.internals = get_toggle_value(PlotsClone->internalsToggle);
	} else {
	    plots_params.region = 0;
	    plots_params.scatter = 0;
	    plots_params.internals = 0;
	}
    } else if (!strcmp(request, "struct")){
	gray_out_item(PlotsClone->structForm, !arg);
	if(arg){
	    plots_params.struct_plot =
	      get_toggle_value(PlotsClone->structToggle);
	} else {
	    plots_params.struct_plot = 0;
	}
    } else if (!strcmp(request, "epochPlots")){
	gray_out_item(PlotsClone->epochPlotsToggle, !arg);
	gray_out_item(PlotsClone->ePerPlotForm, !arg);
	if(arg){
	    plots_params.epochPlots = get_toggle_value(PlotsClone->epochPlotsToggle);
	    gray_out_item(PlotsClone->ePerPlotForm, !plots_params.epochPlots);
	} else {
	    plots_params.epochPlots = 0;
	}
    } else {
	fprintf(stderr, "plots_arg_request: unknown request: %s\n", request);
    }
}

int
plots_verror_value_ok(value)
     int value;
{
    if ((plots_params.detect || plots_params.reject || plots_params.prob) && value < 2)
      return 0;
    else if ((plots_params.cost || plots_params.perr) && value < 1) return 0;
    else return 1;
}

int
plots_train_verror_value_ok(value)
     int value;
{
    if ((plots_params.cost || plots_params.perr) && value < 1) return 0;
    else return 1;
}

/* this string is assumed to be long enough at the moment */
/* let people do a realloc and strcat to make it the correct length!!!! */
void
set_plot_labels()
{
    XtFree(plots_params.plot_label);
    plots_params.plot_label = g_get_plot_label();
    region_request("plotLabel");
    profile_request("plotLabel");
    cost_request("plotLabel");
    perr_request("plotLabel");
    prob_request("plotLabel");
    detect_request("plotLabel");
    reject_request("plotLabel");
    
}

void
store_alg_plot_commands(FILE *shell_file, int show_plot,
			int which_data)
{
    if(plots_params.region || plots_params.scatter || plots_params.internals){
	fprintf(shell_file, "\n#decision region plot\n");
	store_region_plot_command(shell_file, show_plot, which_data);
    }
    if(plots_params.profile || plots_params.histogram){
	fprintf(shell_file, "\n#profile plot\n");
	store_profile_plot_command(shell_file, show_plot, which_data);
    }
}

/* store plots for training results (cost, perr, structure) */
void
store_train_plot_commands(shell_file)
     FILE *shell_file;
{
    if(plots_params.cost) {
	fprintf(shell_file, "\n#cost plot\n");
	store_cost_plot_command(shell_file);
    }
    if(plots_params.perr) {
	fprintf(shell_file, "\n#percent error plot\n");
	store_perr_plot_command(shell_file);
    }
    if(plots_params.struct_plot){
	fprintf(shell_file, "\n#structure plot\n");
	store_struct_plot_command(shell_file);
    }
}
/* store plots based on testing results (prob, detect, reject) */
void
store_test_plot_commands(shell_file)
     FILE *shell_file;
{
    char *note_command;
    note_command = XtNewString(reports_params.shell_fname);
    if(plots_params.prob) {
	fprintf(shell_file, "\n#prob plot\n");
	store_prob_plot_command(shell_file);
	if(reports_params.notebook_on)
	  store_chi_note_command(shell_file, reports_params.log_fname,
				 note_command,
				 reports_params.notebook_path);
    }
    if(plots_params.detect) {
	fprintf(shell_file, "\n#detect plot\n");
	store_detect_plot_command(shell_file);
	if(reports_params.notebook_on)
	  store_roc_note_command(shell_file, reports_params.log_fname,
				 note_command,
				 reports_params.notebook_path);
    }
    if(plots_params.reject) {
	fprintf(shell_file, "\n#reject plot\n");
	store_reject_plot_command(shell_file);
    }
    XtFree (note_command);
}

void
setup_epoch_plot_calls(shell_file)
     FILE *shell_file;
{
    plots_params.storeEpoch = True;
    fprintf(shell_file, "set bplotfile = %s\n", region_params.regionFname);
    fprintf(shell_file, "set pplotfile = %s\n", profile_params.profileFname);
    fprintf(shell_file, "set plotnum = 0\n");
    fprintf(shell_file, "set bpfname = $bplotfile$plotnum\n");
    fprintf(shell_file, "set ppfname = $pplotfile$plotnum\n");
}

void
finish_epoch_plot_calls(shell_file)
     FILE *shell_file;
{
    plots_params.storeEpoch = False;
    if(plots_params.doTrainPlots){
	if(plots_params.scatter || plots_params.region ||
	   plots_params.internals){
	    if(lnk_params.store_only) fprintf(shell_file, "#");
	    fprintf(shell_file, "lnkplot $bplotfile{$plotnum_list} &\n");
	}
	if(plots_params.profile || plots_params.histogram){
	    if(lnk_params.store_only) fprintf(shell_file, "#");
	    fprintf(shell_file, "lnkplot $pplotfile{$plotnum_list} &\n");
	}
    }
}

void
store_plot_start(FILE *shell_file)
{
    if(reports_params.verbosity > 1 && lnk_params.store_only){
	fprintf(shell_file, "(time ");
    }
}

void
store_plot_end(FILE *shell_file){
    if(reports_params.verbosity > 1 && lnk_params.store_only){
	fprintf(shell_file, "\\\n ) >>& %s\n", reports_params.log_fname);
    } else {
	fprintf(shell_file, "\n");
    }
}


/* finish up a plot command where we are making sure that the outputs
   end up in the log file */
/* the problem here is that the lnkplot that is started when a plot is
   run interactively prevents the pipe from closing for the tee so
   we need to do the tee on the plot without the interactive graphics
   and then display the plot in the background at the end */
void
store_plot_end_outputs(shell_file, plot_title, plot_fname)
     FILE *shell_file;
     char *plot_title, *plot_fname;
{
    if((reports_params.verbosity > 1) && lnk_params.store_only){
	fprintf(shell_file, "\\\n) >>& %s\n", reports_params.log_fname);
    } else {
	fprintf(shell_file, "\\\n |& nn_tee -h -a %s\n",
		reports_params.log_fname);
	fprintf(shell_file, "lnkplot -geometry 500x520 -title %s %s&\n",
		plot_title, plot_fname);
    }
}

void
create_plot_file_names()
{
    region_request("plotFname");
    profile_request("plotFname");
    struct_request("plotFname");
    cost_request("plotFname");
    perr_request("plotFname");
    prob_request("plotFname");
    detect_request("plotFname");
    reject_request("plotFname");
}

static ARG plotstab[] =
{
    {"region", BOOLEAN, (int *)NULL,},
    {"scatter", BOOLEAN, (int *)NULL,},
    {"internals", BOOLEAN, (int *)NULL,},
    {"profile", BOOLEAN, (int *)NULL,},
    {"histogram", BOOLEAN, (int *)NULL,},
    {"doTrainPlots", BOOLEAN, (int *)NULL,},
    {"epochPlots", BOOLEAN, (int *)NULL, },
    {"ePerPlot", INT, (int *)NULL,},
    {"struct", BOOLEAN, (int *)NULL,},
    {"cost", BOOLEAN, (int *)NULL,},
    {"perr", BOOLEAN, (int *)NULL,},
    {"prob", BOOLEAN, (int *)NULL,},
    {"detect", BOOLEAN, (int *)NULL,},
    {"reject", BOOLEAN, (int *)NULL,},
};
#define TABSIZE (sizeof(plotstab)/sizeof(ARG))

static void
plots_set_defaults_variables(dp)
     PlotsParams *dp;
{
    nnargs_set_variable("region", (int *)&dp->region, plotstab, TABSIZE);
    nnargs_set_variable("scatter", (int *)&dp->scatter, plotstab, TABSIZE);
    nnargs_set_variable("internals", (int *)&dp->internals, plotstab, TABSIZE);
    nnargs_set_variable("profile", (int *)&dp->profile, plotstab, TABSIZE);
    nnargs_set_variable("histogram", (int *)&dp->histogram, plotstab, TABSIZE);
    nnargs_set_variable("doTrainPlots", (int *)&dp->doTrainPlots, plotstab, TABSIZE);
    nnargs_set_variable("epochPlots", (int *)&dp->epochPlots, plotstab, TABSIZE);
    nnargs_set_variable("ePerPlot", (int *)&dp->ePerPlot, plotstab, TABSIZE);
    nnargs_set_variable("struct", (int *)&dp->struct_plot, plotstab, TABSIZE);
    nnargs_set_variable("cost", (int *)&dp->cost, plotstab, TABSIZE);
    nnargs_set_variable("perr", (int *)&dp->perr, plotstab, TABSIZE);
    nnargs_set_variable("prob", (int *)&dp->prob, plotstab, TABSIZE);
    nnargs_set_variable("detect", (int *)&dp->detect, plotstab, TABSIZE);
    nnargs_set_variable("reject", (int *)&dp->reject, plotstab, TABSIZE);
}

void
read_plots_defaults(argc, argv)
     int argc;
     char **argv;
{
    /* clear flags and strings which are not printed if they are NULL */
    plots_params.region = 0;
    plots_params.scatter = 0;
    plots_params.internals = 0;
    plots_params.profile = 0;
    plots_params.histogram = 0;
    plots_params.doTrainPlots = 0;
    plots_params.epochPlots = 0;
    plots_params.struct_plot = 0;
    plots_params.cost = 0;
    plots_params.perr = 0;
    plots_params.prob = 0;
    plots_params.detect = 0;
    plots_params.reject = 0;
    plots_set_defaults_variables(&plots_params);

    if(argc > 0){
	argc = nnargs(argc, argv, plotstab, TABSIZE);
	/* the lists are properly copied into new space by nnargs */
    } else {
	/* set any defaults */
    }

    plots_request("display");
}

void
write_plots_defaults(FILE *default_file)
{
    plots_set_defaults_variables(&plots_params);

    if(default_file != NULL){
	fprintf(default_file, "plots %s",
		nnargs_get_flags(plotstab, TABSIZE));
    }
}
