/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/print/print_callback.c,v 11.4 2001/08/23 20:17:26 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            PRINT                              ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  16-Nov-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <Xm/Xm.h>
#include <nnlib.h>
#include "print.h"
/*
 * External Functions
 */
#include "print_func.h"
#include "../main/lnknet_subs.h"
 
/*
 * Externals
 */
extern PrintWidgets *PrintClone;
#include "print_params.h"
PrintParams print_params; /* this is the real one */
#include "../reports/reports_params.h"
extern ReportsParams reports_params;
#include "../region/region_params.h"
extern RegionParams region_params;
#include "../profile/profile_params.h"
extern ProfileParams profile_params;
#include "../cost/cost_params.h"
extern CostParams cost_params;
#include "../perr/perr_params.h"
extern PerrParams perr_params;
#include "../struct/struct_params.h"
extern StructParams struct_params;
#include "../prob/prob_params.h"
extern ProbParams prob_params;
#include "../detect/detect_params.h"
extern DetectParams detect_params;
#include "../reject/reject_params.h"
extern RejectParams reject_params;

typedef enum{
    DO_PRINT, DO_PREVIEW, DO_PS, DO_MIF,
} do_process;

/*
 * Statics
 */
static void print_set_defaults_variables(PrintParams *dp);
static void print_set_file_status(Widget nameLabel, Widget existsLabel,
				  Widget psLabel, Widget mifLabel,
				  char *rootName);
static void print_process_files(do_process which);
static void print_process_plot_file(do_process which, char *file_name, char *geometry);


/* required interactions */
/* Change any of the file names:
   change the printed copy here, change the status message,
   if it was the log file, maybe clear the log file window and unpin it
   for the status messages, check for the existance of the three files
   and clear the print message. If we change back to something old, the
   print message goes away. Too bad.
 */

/******************************************************************************
 ******************************************************************************
 *
 * Module Name: print_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   print_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  16-Nov-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
print_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    PrintWidgets *ip;
    static int printer_buttons_mapped = False;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "logToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "print_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	print_params.log = state->set;
 
    } else if (!strcmp(client, "regionToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "print_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	print_params.region = state->set;
	
    } else if (!strcmp(client, "profileToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "print_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	print_params.profile = state->set;
	
    } else if (!strcmp(client, "costToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "print_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	print_params.cost = state->set;
	
    } else if (!strcmp(client, "perrToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "print_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	print_params.perr = state->set;
	
    } else if (!strcmp(client, "structToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "print_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	print_params.struct_flag = state->set;
	
    } else if (!strcmp(client, "probToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "print_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	print_params.prob = state->set;
	
    } else if (!strcmp(client, "detectToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "print_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	print_params.detect = state->set;
	
    } else if (!strcmp(client, "rejectToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "print_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	print_params.reject = state->set;
	
    } else if (!strcmp(client, "printerText")) {
	/*fprintf (stderr, "print_callback: %s\n", client );*/
	printer_buttons_mapped =
	  okTextValueChanged(ip->printerText, ip->printerOKPBtn,
			     ip->printerCancelPBtn,
			     ip->printDialogShell,
			     printer_buttons_mapped,
			     print_params.printer);
	
    } else if ((!strcmp(client, "printerTextAct")) ||
	       (!strcmp(client, "printerOKPBtn"))) {
	String printerTextval;

	/*fprintf (stderr, "print_callback: %s\n", client );*/
	printer_buttons_mapped =
	  okTextActivate(ip->printerText, ip->printerOKPBtn,
			 ip->printerCancelPBtn, &printerTextval);
	
	XtFree(print_params.printer);
	print_params.printer = XtNewString(printerTextval);
	XtFree (printerTextval);
	
	/* do any other things needed */

    } else if (!strcmp(client, "printerCancelPBtn")) {
	/*fprintf (stderr, "print_callback: %s\n", client );*/
	printer_buttons_mapped = 
	  okTextCancel(ip->printerText, ip->printerOKPBtn,
		       ip->printerCancelPBtn,
		       print_params.printer);
	
    } else if (!strcmp(client, "previewPBtn")) {
	/*fprintf (stderr, "print_callback: %s\n", client );*/
	print_process_files(DO_PREVIEW);
	
    } else if (!strcmp(client, "printPBtn")) {
	/*fprintf (stderr, "print_callback: %s\n", client );*/
	print_process_files(DO_PRINT);
	
    } else if (!strcmp(client, "psPBtn")) {
	/*fprintf (stderr, "print_callback: %s\n", client );*/
	print_process_files(DO_PS);
	
    } else if (!strcmp(client, "mifPBtn")) {
	/*fprintf (stderr, "print_callback: %s\n", client );*/
	print_process_files(DO_MIF);
	
    } else if (!strcmp(client, "updatePBtn")) {
	/*fprintf (stderr, "print_callback: %s\n", client );*/
	print_set_all_status();
	
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "print_callback: %s\n", client );*/
	close_popup(ip->printForm);
	
    } else {
	fprintf(stderr, "print_callback: unknown client: %s\n", (char *)client);
    }
}

void
print_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(PrintClone->printForm);
    } else if (!strcmp(request, "init")){
	print_params.log = 0;
	print_params.region = 0;
	print_params.profile = 0;
	print_params.cost = 0;
	print_params.perr = 0;
	print_params.struct_flag = 0;
	print_params.prob = 0;
	print_params.detect = 0;
	print_params.reject = 0;
	print_params.printer = NULL;
    } else if (!strcmp(request, "display")){
	print_set_all_status();
	show_toggle_value(PrintClone->logToggle, print_params.log);
	show_toggle_value(PrintClone->regionToggle, print_params.region);
	show_toggle_value(PrintClone->profileToggle, print_params.profile);
	show_toggle_value(PrintClone->costToggle, print_params.cost);
	show_toggle_value(PrintClone->perrToggle, print_params.perr);
	show_toggle_value(PrintClone->structToggle, print_params.struct_flag);
	show_toggle_value(PrintClone->probToggle, print_params.prob);
	show_toggle_value(PrintClone->detectToggle, print_params.detect);
	show_toggle_value(PrintClone->rejectToggle, print_params.reject);
	if(print_params.printer != NULL)
	  show_string_text(PrintClone->printerText, print_params.printer);
#ifdef notdef
	else
	  show_string_text(PrintClone->printerText, "");
#endif

    } else {
	fprintf(stderr, "print_request: unknown request: %s\n", request);
    }
}
static ARG printtab[] =
{
    {"log", BOOLEAN, (int *)NULL,},
    {"region", BOOLEAN, (int *)NULL,},
    {"profile", BOOLEAN, (int *)NULL,},
    {"cost", BOOLEAN, (int *)NULL,},
    {"perr", BOOLEAN, (int *)NULL,},
    {"struct", BOOLEAN, (int *)NULL,},
    {"prob", BOOLEAN, (int *)NULL,},
    {"detect", BOOLEAN, (int *)NULL,},
    {"reject", BOOLEAN, (int *)NULL,},
    {"printer", STRING, (int *)NULL,},
};
#define TABSIZE (sizeof(printtab)/sizeof(ARG))

static void
print_set_defaults_variables(dp)
     PrintParams *dp;
{
    nnargs_set_variable("log", (int *)&dp->log, printtab, TABSIZE);
    nnargs_set_variable("region", (int *)&dp->region, printtab, TABSIZE);
    nnargs_set_variable("profile", (int *)&dp->profile, printtab, TABSIZE);
    nnargs_set_variable("cost", (int *)&dp->cost, printtab, TABSIZE);
    nnargs_set_variable("perr", (int *)&dp->perr, printtab, TABSIZE);
    nnargs_set_variable("struct", (int *)&dp->struct_flag, printtab, TABSIZE);
    nnargs_set_variable("prob", (int *)&dp->prob, printtab, TABSIZE);
    nnargs_set_variable("detect", (int *)&dp->detect, printtab, TABSIZE);
    nnargs_set_variable("reject", (int *)&dp->reject, printtab, TABSIZE);
    nnargs_set_variable("printer", (int *)&dp->printer, printtab, TABSIZE);

}

void
read_print_defaults(argc, argv)
     int argc;
     char **argv;
{
    /* clear flags and strings which are not printed if they are NULL */
    print_params.log = 0;
    print_params.region = 0;
    print_params.profile = 0;
    print_params.cost = 0;
    print_params.perr = 0;
    print_params.struct_flag = 0;
    print_params.prob = 0;
    print_params.detect = 0;
    print_params.reject = 0;
    if (print_params.printer != NULL){
	XtFree(print_params.printer);
	print_params.printer = NULL;
    }
    print_set_defaults_variables(&print_params);

    if(argc > 0){
	argc = nnargs(argc, argv, printtab, TABSIZE);
	/* copy the strings into new memory */
	print_params.printer = XtNewString(print_params.printer);
    }

    print_request("display");
}

void
write_print_defaults(FILE *default_file)
{
    print_set_defaults_variables(&print_params);
    if(default_file != NULL){
	fprintf(default_file, "print %s",
		nnargs_get_flags(printtab, TABSIZE));
    }
}

void
print_set_all_status()
{
    print_set_log_status();
    print_set_region_status();
    print_set_profile_status();
    print_set_cost_status();
    print_set_perr_status();
    print_set_struct_status();
    print_set_prob_status();
    print_set_detect_status();
    print_set_reject_status();
}

void
print_set_log_status()
{
    print_set_file_status(PrintClone->logNameLabel, PrintClone->logExistsLabel,
			  NULL, NULL,
			  reports_params.log_fname);
}

void
print_set_region_status()
{
    print_set_file_status(PrintClone->regionNameLabel,
			  PrintClone->regionExistsLabel,
			  PrintClone->regionPsLabel,
			  PrintClone->regionMifLabel,
			  region_params.regionFname);
}

void
print_set_profile_status()
{
    print_set_file_status(PrintClone->profileNameLabel,
			  PrintClone->profileExistsLabel,
			  PrintClone->profilePsLabel,
			  PrintClone->profileMifLabel,
			  profile_params.profileFname);
}

void
print_set_cost_status()
{
    print_set_file_status(PrintClone->costNameLabel,
			  PrintClone->costExistsLabel,
			  PrintClone->costPsLabel,
			  PrintClone->costMifLabel,
			  cost_params.plotFname);
}

void
print_set_perr_status()
{
    print_set_file_status(PrintClone->perrNameLabel,
			  PrintClone->perrExistsLabel,
			  PrintClone->perrPsLabel,
			  PrintClone->perrMifLabel,
			  perr_params.plotFname);
}

void
print_set_struct_status()
{
    print_set_file_status(PrintClone->structNameLabel,
			  PrintClone->structExistsLabel,
			  PrintClone->structPsLabel,
			  PrintClone->structMifLabel,
			  struct_params.plotFname);
}

void
print_set_prob_status()
{
    print_set_file_status(PrintClone->probNameLabel,
			  PrintClone->probExistsLabel,
			  PrintClone->probPsLabel,
			  PrintClone->probMifLabel,
			  prob_params.plotFname);
}

void
print_set_detect_status()
{ 
    print_set_file_status(PrintClone->detectNameLabel,
			  PrintClone->detectExistsLabel,
			  PrintClone->detectPsLabel,
			  PrintClone->detectMifLabel,
			  detect_params.plotFname);
}

void
print_set_reject_status()
{
    print_set_file_status(PrintClone->rejectNameLabel,
			  PrintClone->rejectExistsLabel,
			  PrintClone->rejectPsLabel,
			  PrintClone->rejectMifLabel,
			  reject_params.plotFname);
}

static void
print_set_file_status(Widget nameLabel, Widget existsLabel,
		      Widget psLabel, Widget mifLabel,
		      char *rootName)
{
    char *tmpString;

    if(rootName == NULL) return;
    if(nameLabel != NULL){
	show_string_label(nameLabel, rootName);
    }
    if(existsLabel != NULL){
	if(file_exists(path_build(reports_params.exp_path, rootName)))
	  show_string_label(existsLabel, "Yes");
	else
	  show_string_label(existsLabel, "No");
    }
    
    if(psLabel != NULL){
	tmpString = XtNewString(rootName);
	tmpString = StringConcat(tmpString, ".ps");
	if(file_exists(path_build(reports_params.exp_path, tmpString)))
	  show_string_label(psLabel, "Yes");
	else
	  show_string_label(psLabel, "No");
	XtFree(tmpString);
    }

    if(mifLabel != NULL){
	tmpString = XtNewString(rootName);
	tmpString = StringConcat(tmpString, ".mif");
	if(file_exists(path_build(reports_params.exp_path, tmpString)))
	  show_string_label(mifLabel, "Yes");
	else
	  show_string_label(mifLabel, "No");
	XtFree(tmpString);
    }
}

static void
print_process_files(which)
     do_process which;
{
    char *file_name, *cmd, *geometry;
    if(print_params.log){
	if(which == DO_PRINT){
	    file_name = path_build(reports_params.exp_path,
				   reports_params.log_fname);
	    if(file_exists(file_name)){
		cmd = XtNewString("lpr ");
		if(print_params.printer != NULL &&
		   strcmp(print_params.printer, "") != 0){
		    cmd = StringConcat(cmd, "-P");
		    cmd = StringConcat(cmd, print_params.printer);
		    cmd = StringConcat(cmd, " ");
		}
		cmd = StringConcat(cmd, file_name);
		printf("%s\n", cmd);
		nn_system(cmd);
	    }
	} else if (which == DO_PREVIEW){
	    file_name = path_build(reports_params.exp_path,
				   reports_params.log_fname);
	    if(file_exists(file_name)){
		review_request("display");
		review_request("open");
	    }
	}
    }
    if (print_params.region) {
	file_name =
	  path_build(reports_params.exp_path, region_params.regionFname);
	geometry = "-geometry 510x490 -title decision_plot";
	print_process_plot_file(which, file_name, geometry);
    }
    if (print_params.profile) {
	file_name =
	  path_build(reports_params.exp_path, profile_params.profileFname);
	geometry = "-geometry 510x490 -title profile_plot";
	print_process_plot_file(which, file_name, geometry);
    }
    if (print_params.cost) {
	file_name =
	  path_build(reports_params.exp_path, cost_params.plotFname);
	geometry = "-geometry 600x440 -title cost_plot";
	print_process_plot_file(which, file_name, geometry);
    }
    if(print_params.perr) {
	file_name =
	  path_build(reports_params.exp_path, perr_params.plotFname);
	geometry = "-geometry 600x440 -title %_err_plot";
	print_process_plot_file(which, file_name, geometry);
    }
    if(print_params.struct_flag) {
	file_name =
	  path_build(reports_params.exp_path, struct_params.plotFname);
	geometry = "-geometry 510x490 -title struct_plot";
	print_process_plot_file(which, file_name, geometry);
    }
    if(print_params.prob) {
	file_name =
	  path_build(reports_params.exp_path, prob_params.plotFname);
	geometry = "-geometry 510x520 -title prob_plot";
	print_process_plot_file(which, file_name, geometry);
    }
    if(print_params.detect) {
	file_name =
	  path_build(reports_params.exp_path, detect_params.plotFname);
	geometry = "-geometry 510x520 -title detect_plot";
	print_process_plot_file(which, file_name, geometry);
    }
    if(print_params.reject) {
	file_name =
	  path_build(reports_params.exp_path, reject_params.plotFname);
	geometry = "-geometry 510x520 -title reject_plot";
	print_process_plot_file(which, file_name, geometry);
    }
    print_set_all_status();
}

static void
print_process_plot_file(do_process which, char *file_name,
			char *geometry)
{
    char unique_name[32];
    char *cmd;
    if(file_name != NULL && file_exists(file_name)){
	if(which == DO_PRINT){
	    sprintf(unique_name, "/tmp/LNKplot%.5ld.ps", random() % 100000);
	    /* make a .ps file and print it */
	    cmd = XtNewString("plot2ps < ");
	    cmd = StringConcat(cmd, file_name);
	    cmd = StringConcat(cmd, " > ");
	    cmd = StringConcat(cmd, unique_name);
	    cmd = StringConcat(cmd, "; lpr ");
	    if(print_params.printer != NULL &&
	       strcmp(print_params.printer, "")) {
		cmd = StringConcat(cmd, "-P");
		cmd = StringConcat(cmd, print_params.printer);
		cmd = StringConcat(cmd, " ");
	    }
	    cmd = StringConcat(cmd, unique_name);
	    cmd = StringConcat(cmd, "; rm ");
	    cmd = StringConcat(cmd, unique_name);
	    printf("%s\n", cmd);
	    nn_system(cmd);
	    XtFree(cmd);
	} else if (which == DO_PREVIEW){
	    cmd = XtNewString("lnkplot ");
	    cmd = StringConcat(cmd, geometry);
	    cmd = StringConcat(cmd, " ");
	    cmd = StringConcat(cmd, file_name);
	    cmd = StringConcat(cmd, "&");
	    /* start lnkplot for this file */
	    nn_system(cmd);
	    XtFree(cmd);
	} else if (which == DO_PS){
	    /* make a .ps file and print it */
	    cmd = XtNewString("plot2ps < ");
	    cmd = StringConcat(cmd, file_name);
	    cmd = StringConcat(cmd, " > ");
	    cmd = StringConcat(cmd, file_name);
	    cmd = StringConcat(cmd, ".ps");
	    printf("%s\n", cmd);
	    nn_system(cmd);
	    XtFree(cmd);
	} else if (which == DO_MIF){
	    /* make a .mif file and print it */
	    cmd = XtNewString("plot2mif < ");
	    cmd = StringConcat(cmd, file_name);
	    cmd = StringConcat(cmd, " > ");
	    cmd = StringConcat(cmd, file_name);
	    cmd = StringConcat(cmd, ".mif");
	    printf("%s\n", cmd);
	    nn_system(cmd);
	    XtFree(cmd);
	}
    }
}
