/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/print/review_callback.c,v 11.4 2001/08/23 20:17:27 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                               REVIEW                                 ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  16-Nov-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include <Xm/Text.h>
#include "review.h"
#include <nnlib.h>
/* for checking out the log file before displaying it */
#include <sys/types.h>
#include <sys/stat.h>
/*
 * External Functions
 */
#include "../main/lnknet_subs.h"
#include "print_func.h"
 
/*
 * Externals
 */
extern ReviewWidgets *ReviewClone;
#include "print_params.h"
extern PrintParams print_params;
#include "../reports/reports_params.h"
extern ReportsParams reports_params;
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: review_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   review_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  16-Nov-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
review_callback(Widget w, XtPointer client, XtPointer call)
{
  ReviewWidgets *ip;
 
  /*
   * Get the parent structure
   */
  XtVaGetValues (w, XmNuserData, &ip, NULL);
 
  if (!strcmp(client, "closePBtn")) {
      /*fprintf (stderr, "review_callback: %s\n", client );*/
      close_popup(ip->reviewForm);
 
  } else {
    fprintf(stderr, "review_callback: unknown client: %s\n", (char *)client);
  }
}

void
review_request(char *request)
{
    if(!strcmp(request, "open")){
	open_popup(ReviewClone->reviewForm);
    } else if (!strcmp(request, "init")){
    } else if (!strcmp(request, "display")){
	char *textValue, *filename;
	struct stat statb;
	FILE *fp;

	show_string_label(ReviewClone->logNameLabel, reports_params.log_fname);
	/* this is copped out of the OReily Motif book. it might not
	   be the best but I didn't have to figure it out myself...*/
	if (reports_params.log_fname != NULL &&
	    strcmp(reports_params.log_fname, "")){

	    /* check if we can open the file and view it */
	    filename = path_build(reports_params.exp_path,
				  reports_params.log_fname);
	    if(stat(filename, &statb) == -1 ||
	       (statb.st_mode & S_IFMT) != S_IFREG ||
	       !(fp = fopen(filename, "r"))){
		if((statb.st_mode & S_IFMT) == S_IFREG){
		    /* this is a regular file, there was some other problem */
		    textValue = XtNewString("File cannot be opened");
		} else {
		    textValue = XtNewString("File is not regular");
		}
	    } else {
		/* allocate space for the file */
		textValue = XtMalloc((unsigned)(statb.st_size + 1));
		if(!fread(textValue, sizeof(char), statb.st_size+1, fp))
		  warning("Log File Review Window: could not read whole log file");
		textValue[statb.st_size] = '\0'; /* NULL terminate the string*/
	    }
	    show_string_text(ReviewClone->reviewText, textValue);
	    XtFree(textValue);
	} else {
	    XmTextSetString(ReviewClone->reviewText, "");
	}
    } else {
	fprintf(stderr, "review_request: unknown request: %s\n", request);
    }
}
