/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/priors/priors_callback.c,v 11.4 2001/08/23 20:17:30 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            PRIORS                                    ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  18-Oct-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include <nnlib.h>
#include "priors.h"
/*
 * External Functions
 */
#include "priors_func.h"
#include "../main/lnknet_subs.h"
#include "../reports/reports_func.h"
#include "../cost/cost_func.h"
#include "../perr/perr_func.h"

/*
 * Externals
 */
extern PriorsWidgets *PriorsClone;
#include "priors_params.h"
PriorsParams priors_params;	/* this is the real one */
#include "../data/data_params.h"
DataParams data_params;
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: priors_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   priors_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  18-Oct-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
priors_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    PriorsWidgets *ip;
    static int priorsNpatterns_buttons_mapped = False;
    static int trainPriorsList_buttons_mapped = False;
    static int testPriorsList_buttons_mapped = False;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "trainAsIsToggle") ||
	!strcmp(client, "adjustTrainToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "select_callback: %s %s\n", client,
	  (state->set) ? "on", "off" );*/
	if(state->set){
	    priors_params.adjustTrain =
	      get_radio_value(ip->adjustTrainRadioBox);
	    gray_out_item(ip->samplePriorsForm, !priors_params.adjustTrain);
	    reports_request("major_note");
	}

    } else if (!strcmp(client, "trainSpecifyPriorsToggle") ||
	       !strcmp(client, "trainUniformToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "select_callback: %s %s\n", client,
	  (state->set) ? "on", "off" );*/
	if(state->set){
	    priors_params.trainUniform =
	      get_radio_value(ip->trainUniformRadioBox);
	    gray_out_item(ip->trainPriorsListForm, priors_params.trainUniform);
	    reports_request("major_note");
	}
 
    } else if (!strcmp(client, "trainPriorsListText")) {
	/*fprintf (stderr, "priors_callback: %s\n", client);*/
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	trainPriorsList_buttons_mapped =
	  okTextValueChanged(ip->trainPriorsListText, ip->trainPriorsListOKPBtn,
			     ip->trainPriorsListCancelPBtn,
			     ip->priorsDialogShell,
			     trainPriorsList_buttons_mapped,
			     priors_params.trainPriorsList_string);
    } else if ((!strcmp(client, "trainPriorsListTextAct")) ||
	       (!strcmp(client, "trainPriorsListOKPBtn"))) {
	/*fprintf (stderr, "priors_callback: %s\n", client );*/
	String trainPriorsListTextval;
	trainPriorsList_buttons_mapped =
	  okTextActivate(ip->trainPriorsListText,
			 ip->trainPriorsListOKPBtn,
			 ip->trainPriorsListCancelPBtn,
			 &trainPriorsListTextval);
	
	if(priors_params.trainPriorsList != NULL)
	  deallocate_vector(priors_params.trainPriorsList);
	priors_params.trainPriorsList = NULL;
	/* take spaces and tabs out of the list before handing it to nnargs */
	if(trainPriorsListTextval != NULL){
	    char *trainPriorsListTemp = trainPriorsListTextval;
	    trainPriorsListTextval = strtok(trainPriorsListTextval, " \t");
	    if(trainPriorsListTextval != NULL){
		priors_params.trainPriorsList =
		  nnargs_get_float_alloc_list(trainPriorsListTextval);
		XtFree(trainPriorsListTemp);
	    }
	}
	
	/* make a copy of the string as it has been interpretted */
	XtFree(priors_params.trainPriorsList_string);
	priors_params.trainPriorsList_string =
	  make_float_string(priors_params.trainPriorsList);
	show_string_text(ip->trainPriorsListText,
			 priors_params.trainPriorsList_string);
	
	/* do anything else necessary */
	reports_request("major_note");
	
    } else if (!strcmp(client, "trainPriorsListCancelPBtn")) {
	/*fprintf (stderr, "priors_callback: %s\n", client );*/
	trainPriorsList_buttons_mapped = 
	  okTextCancel(ip->trainPriorsListText, ip->trainPriorsListOKPBtn,
		       ip->trainPriorsListCancelPBtn,
		       priors_params.trainPriorsList_string);

    } else if (!strcmp(client, "priorsNpatternsText")) {
	/*fprintf (stderr, "priors_callback: %s\n", client );*/
	priorsNpatterns_buttons_mapped =
	  okTextValueChanged(ip->priorsNpatternsText,
			     ip->priorsNpatternsOKPBtn,
			     ip->priorsNpatternsCancelPBtn,
			     ip->priorsDialogShell,
			     priorsNpatterns_buttons_mapped,
			     priors_params.priorsNpatterns_string);
	
    } else if ((!strcmp(client, "priorsNpatternsTextAct")) ||
	       (!strcmp(client, "priorsNpatternsOKPBtn"))) {
	String priorsNpatternsTextval;

	/*fprintf (stderr, "priors_callback: %s\n", client );*/
	priorsNpatterns_buttons_mapped =
	  okTextActivate(ip->priorsNpatternsText, ip->priorsNpatternsOKPBtn,
			 ip->priorsNpatternsCancelPBtn, &priorsNpatternsTextval);
	/* I won't actually be using this copy */
	XtFree (priorsNpatternsTextval);
	
	/* verify the string as an integer */
	priors_params.priorsNpatterns =
	  set_atoi_value(ip->priorsNpatternsText, "priorsNpatterns_set",
			     priors_params.priorsNpatterns);

	/* save the current text for future comparisons */
	XtFree(priors_params.priorsNpatterns_string);
	priors_params.priorsNpatterns_string =
	  get_string_text(ip->priorsNpatternsText);
 
	/* do any other things needed */
	cost_request("nPoints");
	perr_request("nPoints");

    } else if (!strcmp(client, "priorsNpatternsCancelPBtn")) {
	/*fprintf (stderr, "priors_callback: %s\n", client );*/
	priorsNpatterns_buttons_mapped = 
	  okTextCancel(ip->priorsNpatternsText, ip->priorsNpatternsOKPBtn,
		       ip->priorsNpatternsCancelPBtn,
		       priors_params.priorsNpatterns_string);
 
    } else if (!strcmp(client, "testAsIsToggle") ||
	       !strcmp(client, "adjustTestToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "select_callback: %s %s\n", client,
	  (state->set) ? "on", "off" );*/
	if(state->set){
	    priors_params.adjustTest =
	      get_radio_value(ip->adjustTestRadioBox);
	    gray_out_item(ip->scalePriorsForm, !priors_params.adjustTest);
	    reports_request("major_note");
	}
 
    } else if (!strcmp(client, "testSpecifyPriorsToggle") ||
	       !strcmp(client, "testUniformToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "select_callback: %s %s\n", client,
		 state->set ? "on" : "off" );*/
	if(state->set){
	    priors_params.testUniform =
	      get_radio_value(ip->testUniformRadioBox);
	    gray_out_item(ip->testPriorsListForm,
			  priors_params.testUniform);
	    reports_request("major_note");
	}
    } else if (!strcmp(client, "testPriorsListText")) {
	/*fprintf (stderr, "priors_callback: %s\n", client);*/
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	testPriorsList_buttons_mapped =
	  okTextValueChanged(ip->testPriorsListText, ip->testPriorsListOKPBtn,
			     ip->testPriorsListCancelPBtn,
			     ip->priorsDialogShell,
			     testPriorsList_buttons_mapped,
			     priors_params.testPriorsList_string);
    } else if ((!strcmp(client, "testPriorsListTextAct")) ||
	       (!strcmp(client, "testPriorsListOKPBtn"))) {
	/*fprintf (stderr, "priors_callback: %s\n", client );*/
	String testPriorsListTextval;
	testPriorsList_buttons_mapped =
	  okTextActivate(ip->testPriorsListText,
			 ip->testPriorsListOKPBtn,
			 ip->testPriorsListCancelPBtn,
			 &testPriorsListTextval);
	
	if(priors_params.testPriorsList != NULL)
	  deallocate_vector(priors_params.testPriorsList);
	priors_params.testPriorsList = NULL;
	/* take spaces and tabs out of the list before handing it to nnargs */
	if(testPriorsListTextval != NULL){
	    char *testPriorsListTemp = testPriorsListTextval;
	    testPriorsListTextval = strtok(testPriorsListTextval, " \t");
	    if(testPriorsListTextval != NULL){
		priors_params.testPriorsList =
		  nnargs_get_float_alloc_list(testPriorsListTextval);
		XtFree(testPriorsListTemp);
	    }
	}
	
	/* make a copy of the string as it has been interpretted */
	XtFree(priors_params.testPriorsList_string);
	priors_params.testPriorsList_string =
	  make_float_string(priors_params.testPriorsList);
	show_string_text(ip->testPriorsListText,
			 priors_params.testPriorsList_string);
	
	/* do anything else necessary */
	reports_request("major_note");
	
    } else if (!strcmp(client, "testPriorsListCancelPBtn")) {
	/*fprintf (stderr, "priors_callback: %s\n", client );*/
	testPriorsList_buttons_mapped = 
	  okTextCancel(ip->testPriorsListText, ip->testPriorsListOKPBtn,
		       ip->testPriorsListCancelPBtn,
		       priors_params.testPriorsList_string);
 
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "priors_callback: %s\n", client );*/
	close_popup(PriorsClone->priorsForm);
 
    } else {
	fprintf(stderr, "priors_callback: unknown client: %s\n", (char *)client);
    }
}

void
priors_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(PriorsClone->priorsForm);
    } else if (!strcmp(request, "init")){
	priors_params.adjustTrain = 0;
	priors_params.adjustTest = 0;
	priors_params.trainUniform = 0;
	priors_params.testUniform = 0;
	priors_params.priorsNpatterns = 0;
	priors_params.priorsNpatterns_string = NULL;
	priors_params.trainPriorsList = NULL;
	priors_params.trainPriorsList_string = NULL;
	priors_params.testPriorsList = NULL;
	priors_params.testPriorsList_string = NULL;

    } else if (!strcmp(request, "display")){
	char string[10];
	show_radio_value(PriorsClone->adjustTrainRadioBox,
			 priors_params.adjustTrain);
	gray_out_item(PriorsClone->samplePriorsForm,
		      !priors_params.adjustTrain);
	show_radio_value(PriorsClone->trainUniformRadioBox,
			 priors_params.trainUniform);
	gray_out_item(PriorsClone->trainPriorsListForm,
		      priors_params.trainUniform);
	show_radio_value(PriorsClone->adjustTestRadioBox,
			 priors_params.adjustTest);
	gray_out_item(PriorsClone->scalePriorsForm,
		      !priors_params.adjustTest);
	show_radio_value(PriorsClone->testUniformRadioBox,
			 priors_params.testUniform);
	gray_out_item(PriorsClone->testPriorsListForm,
		      priors_params.testUniform);

	XtFree(priors_params.priorsNpatterns_string);
	sprintf(string, "%d", priors_params.priorsNpatterns);
	priors_params.priorsNpatterns_string = XtNewString(string);
	show_string_text(PriorsClone->priorsNpatternsText,
			 priors_params.priorsNpatterns_string);
	XtFree(priors_params.trainPriorsList_string);
	priors_params.trainPriorsList_string =
	  make_float_string(priors_params.trainPriorsList);
	show_string_text(PriorsClone->trainPriorsListText,
			 priors_params.trainPriorsList_string);
	XtFree(priors_params.testPriorsList_string);
	priors_params.testPriorsList_string =
	  make_float_string(priors_params.testPriorsList);
	show_string_text(PriorsClone->testPriorsListText,
			 priors_params.testPriorsList_string);

    } else if (!strcmp(request, "set_npatterns")) {
	char string[10];
	/* set the value to what data has, then display it and update
	   anyone who cares */
	priors_params.priorsNpatterns = data_params.npatterns_train;

	XtFree(priors_params.priorsNpatterns_string);
	sprintf(string, "%d", priors_params.priorsNpatterns);
	priors_params.priorsNpatterns_string = XtNewString(string);
	show_string_text(PriorsClone->priorsNpatternsText,
			 priors_params.priorsNpatterns_string);
	cost_request("nPoints");
	perr_request("nPoints");
	
    } else if (!strcmp(request, "show_train_priors")) {
	/* timing is everything on these... */
	show_string_label(PriorsClone->trainFilePriorsLabel,
			  data_params.priors_string);
    } else if (!strcmp(request, "show_eval_priors")) {
	show_string_label(PriorsClone->evalFilePriorsLabel,
			  data_params.priors_string);
    } else if (!strcmp(request, "show_test_priors")) {
	show_string_label(PriorsClone->testFilePriorsLabel,
			  data_params.priors_string);
    } else {
	fprintf(stderr, "priors_request: unknown request: %s\n", request);
    }
}

static ARG priorstab[] =
{
    {"adjustTrain", FLAG, (int *)NULL, },
    {"trainUniform", FLAG, (int *)NULL, },
    {"trainPriorsList", FLOAT_ALLOC_LIST, (int *)NULL,},
    {"priorsNpatterns", INT, (int *)NULL,},
    {"adjustTest", FLAG, (int *)NULL, },
    {"testUniform", FLAG, (int *)NULL, },
    {"testPriorsList", FLOAT_ALLOC_LIST, (int *)NULL,},
};
#define TABSIZE (sizeof(priorstab)/sizeof(ARG))

static void
priors_set_defaults_variables(dp)
     PriorsParams *dp;
{
    nnargs_set_variable("adjustTrain", (int *)&dp->adjustTrain, priorstab, TABSIZE);
    nnargs_set_variable("adjustTest", (int *)&dp->adjustTest, priorstab, TABSIZE);
    nnargs_set_variable("trainUniform", (int *)&dp->trainUniform, priorstab, TABSIZE);
    nnargs_set_variable("testUniform", (int *)&dp->testUniform, priorstab, TABSIZE);
    nnargs_set_variable("priorsNpatterns", (int *)&dp->priorsNpatterns, priorstab, TABSIZE);
    nnargs_set_variable("trainPriorsList", (int *)&dp->trainPriorsList, priorstab, TABSIZE);
    nnargs_set_variable("testPriorsList", (int *)&dp->testPriorsList, priorstab, TABSIZE);
}

void
read_priors_defaults(argc, argv)
     int argc;
     char **argv;
{
    /* clear flags and strings which are not printed if they are NULL */
    priors_params.adjustTrain = 0;
    priors_params.adjustTest = 0;
    priors_params.trainUniform = 0;
    priors_params.testUniform = 0;
    if (priors_params.trainPriorsList != NULL){
	deallocate_vector(priors_params.trainPriorsList);
	priors_params.trainPriorsList = NULL;
    }
    if (priors_params.testPriorsList != NULL){
	deallocate_vector(priors_params.testPriorsList);
	priors_params.testPriorsList = NULL;
    }

    priors_set_defaults_variables(&priors_params);

    if(argc > 0){
	argc = nnargs(argc, argv, priorstab, TABSIZE);
	/* the lists are properly copied into new space by nnargs */
    } else {
	priors_params.priorsNpatterns = 0;
    }

    priors_request("display");
}

void
write_priors_defaults(FILE *default_file)
{
    priors_set_defaults_variables(&priors_params);

    if(default_file != NULL){
	fprintf(default_file, "priors %s",
		nnargs_get_flags(priorstab, TABSIZE));
    }
}

void
activate_train_priors(activate_flag)
     int activate_flag;
{
    gray_out_item(PriorsClone->trainForm, !activate_flag);
    if(activate_flag){
	priors_params.adjustTrain =
	  get_radio_value(PriorsClone->adjustTrainRadioBox);
    } else {
	priors_params.adjustTrain = 0;
    }
}

void
activate_test_priors(activate_flag)
     int activate_flag;
{
    gray_out_item(PriorsClone->testForm, !activate_flag);
    if(activate_flag){
	priors_params.adjustTest =
	  get_radio_value(PriorsClone->adjustTestRadioBox);
    } else {
	priors_params.adjustTest = 0;
    }
}
