/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/prob/prob_callback.c,v 11.5 2001/08/23 20:17:33 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            PROB                              ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  7-Nov-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include <nnlib.h>
#include "prob.h"
/*
 * External Functions
 */
#include "prob_func.h"
#include "../main/lnknet_subs.h"
#include "../lnk/lnk_func.h"
#include "../plots/plots_func.h"
#include "../print/print_func.h"
 
/*
 * Externals
 */
extern ProbWidgets *ProbClone;
#include "prob_params.h"
ProbParams prob_params; /* this is the real one */
#include "../data/data_params.h"
extern DataParams data_params;/* for data base path and name and extensions */
#include "../plots/plots_params.h"
extern PlotsParams plots_params;
#include "../lnk/lnk_params.h"
extern LnkParams lnk_params;
#include "../reports/reports_params.h"
extern ReportsParams reports_params;

extern ProbWidgets *ProbClone;
/*
 * Statics
 */
static void prob_set_read_variables();
static void prob_set_write_variables();
static void prob_set_common_flags();
#define PLOT_PROB "plot_prob"
#include <plot_prob_args.h>
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: prob_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   prob_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  7-Nov-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
prob_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    ProbWidgets *ip;
    static int target_buttons_mapped = False;
    static int nBins_buttons_mapped = False;
    static int binEnds_buttons_mapped = False;
    static int minBinCount_buttons_mapped = False;
    static int xmin_buttons_mapped = False;
    static int xmax_buttons_mapped = False;
    static int xstep_buttons_mapped = False;
    static int ymin_buttons_mapped = False;
    static int ymax_buttons_mapped = False;
    static int ystep_buttons_mapped = False;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "targetText")) {
	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	target_buttons_mapped =
	  okTextValueChanged(ip->targetText, ip->targetOKPBtn,
			     ip->targetCancelPBtn,
			     ip->probDialogShell,
			     target_buttons_mapped,
			     prob_params.target_string);
	
    } else if ((!strcmp(client, "targetTextAct")) ||
	       (!strcmp(client, "targetOKPBtn"))) {
	String targetTextval;

	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	target_buttons_mapped =
	  okTextActivate(ip->targetText, ip->targetOKPBtn,
			 ip->targetCancelPBtn, &targetTextval);
	/* I won't actually be using this copy */
	XtFree (targetTextval);
	
	/* verify the string as an integer */
	prob_params.target =
	  set_atoi_value(ip->targetText, "target_set",
			     prob_params.target);

	/* save the current text for future comparisons */
	XtFree(prob_params.target_string);
	prob_params.target_string = get_string_text(ip->targetText);
 
	/* do any other things needed */
	prob_set_target_label();

    } else if (!strcmp(client, "targetCancelPBtn")) {
	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	target_buttons_mapped = 
	  okTextCancel(ip->targetText, ip->targetOKPBtn,
		       ip->targetCancelPBtn,
		       prob_params.target_string);
	
    } else if (!strcmp(client, "scatPlotToggle") ||
	       !strcmp(client, "binPlotToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "prob_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set) prob_params.binPlot =
	  get_radio_value(ip->binPlotRadioBox);
	
    } else if (!strcmp(client, "constBinToggle") ||
	       !strcmp(client, "varBinToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "prob_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set){
	    prob_params.varBin =
	      get_radio_value(ip->varBinRadioBox);
	    gray_out_item(ip->nBinsForm, prob_params.varBin);
	    gray_out_item(ip->binEndsForm, !prob_params.varBin);
	}
	
    } else if (!strcmp(client, "nBinsText")) {
	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	nBins_buttons_mapped =
	  okTextValueChanged(ip->nBinsText, ip->nBinsOKPBtn,
			     ip->nBinsCancelPBtn,
			     ip->probDialogShell,
			     nBins_buttons_mapped,
			     prob_params.nBins_string);
	
    } else if ((!strcmp(client, "nBinsTextAct")) ||
	       (!strcmp(client, "nBinsOKPBtn"))) {
	String nBinsTextval;

	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	nBins_buttons_mapped =
	  okTextActivate(ip->nBinsText, ip->nBinsOKPBtn,
			 ip->nBinsCancelPBtn, &nBinsTextval);
	/* I won't actually be using this copy */
	XtFree (nBinsTextval);
	
	/* verify the string as an integer */
	prob_params.nBins =
	  set_atoi_low_value(ip->nBinsText, "nBins_set",
			     prob_params.nBins, 0);

	/* save the current text for future comparisons */
	XtFree(prob_params.nBins_string);
	prob_params.nBins_string = get_string_text(ip->nBinsText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "nBinsCancelPBtn")) {
	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	nBins_buttons_mapped = 
	  okTextCancel(ip->nBinsText, ip->nBinsOKPBtn,
		       ip->nBinsCancelPBtn,
		       prob_params.nBins_string);
	
    } else if (!strcmp(client, "binEndsText")) {
	/*fprintf (stderr, "prob_callback: %s\n", client);*/
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	binEnds_buttons_mapped =
	  okTextValueChanged(ip->binEndsText, ip->binEndsOKPBtn,
			     ip->binEndsCancelPBtn,
			     ip->probDialogShell,
			     binEnds_buttons_mapped,
			     prob_params.binEnds_string);

    } else if ((!strcmp(client, "binEndsTextAct")) ||
	       (!strcmp(client, "binEndsOKPBtn"))) {
	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	String binEndsTextval;
	binEnds_buttons_mapped =
	  okTextActivate(ip->binEndsText,
			 ip->binEndsOKPBtn,
			 ip->binEndsCancelPBtn,
			 &binEndsTextval);
	
	if(prob_params.binEnds != NULL)
	  deallocate_vector(prob_params.binEnds);
	/* take spaces and tabs out of the list before handing it to nnargs */
	if(binEndsTextval != NULL){
	    char *binEndsTemp = binEndsTextval;
	    binEndsTextval = strtok(binEndsTextval, " \t");
	    prob_params.binEnds =
	      nnargs_get_float_alloc_list(binEndsTextval);
	    XtFree(binEndsTemp);
	}
	
	/* make a copy of the string as it has been interpretted */
	XtFree(prob_params.binEnds_string);
	prob_params.binEnds_string =
	  make_float_string(prob_params.binEnds);
	show_string_text(ip->binEndsText,
			 prob_params.binEnds_string);
	
	/* do anything else necessary */
	
    } else if (!strcmp(client, "binEndsCancelPBtn")) {
	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	binEnds_buttons_mapped = 
	  okTextCancel(ip->binEndsText, ip->binEndsOKPBtn,
		       ip->binEndsCancelPBtn,
		       prob_params.binEnds_string);
	
    } else if (!strcmp(client, "minBinCountText")) {
	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	minBinCount_buttons_mapped =
	  okTextValueChanged(ip->minBinCountText, ip->minBinCountOKPBtn,
			     ip->minBinCountCancelPBtn,
			     ip->probDialogShell,
			     minBinCount_buttons_mapped,
			     prob_params.minBinCount_string);
	
    } else if ((!strcmp(client, "minBinCountTextAct")) ||
	       (!strcmp(client, "minBinCountOKPBtn"))) {
	String minBinCountTextval;

	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	minBinCount_buttons_mapped =
	  okTextActivate(ip->minBinCountText, ip->minBinCountOKPBtn,
			 ip->minBinCountCancelPBtn, &minBinCountTextval);
	/* I won't actually be using this copy */
	XtFree (minBinCountTextval);
	
	/* verify the string as an integer */
	prob_params.minBinCount =
	  set_atoi_low_value(ip->minBinCountText, "minBinCount_set",
			     prob_params.minBinCount, 0);

	/* save the current text for future comparisons */
	XtFree(prob_params.minBinCount_string);
	prob_params.minBinCount_string = get_string_text(ip->minBinCountText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "minBinCountCancelPBtn")) {
	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	minBinCount_buttons_mapped = 
	  okTextCancel(ip->minBinCountText, ip->minBinCountOKPBtn,
		       ip->minBinCountCancelPBtn,
		       prob_params.minBinCount_string);
	
    } else if (!strcmp(client, "chiSqToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "prob_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	prob_params.chiSq = state->set;
	
    } else if (!strcmp(client, "noTextToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "prob_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	prob_params.noText = state->set;
	
    } else if (!strcmp(client, "verboseToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "prob_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	prob_params.verbose = state->set;
	
    } else if (!strcmp(client, "sumToOneToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "prob_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	prob_params.sumToOne = state->set;
	
    } else if (!strcmp(client, "xminText")) {
	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	xmin_buttons_mapped =
	  okTextValueChanged(ip->xminText, ip->xminOKPBtn,
			     ip->xminCancelPBtn,
			     ip->probDialogShell,
			     xmin_buttons_mapped,
			     prob_params.xmin_string);
      
    } else if (!strcmp(client, "xminTextAct") ||
	       !strcmp(client, "xminOKPBtn")) {
	String xminTextval;

	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	xmin_buttons_mapped = okTextActivate(ip->xminText, ip->xminOKPBtn,
					     ip->xminCancelPBtn, &xminTextval);

	XtFree (xminTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	prob_params.xmin = set_atof_value(ip->xminText, "prob_xmin",
					  prob_params.xmin);
	XtFree(prob_params.xmin_string);
	prob_params.xmin_string = get_string_text(ip->xminText);
 
    } else if (!strcmp(client, "xminCancelPBtn")) {
	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	xmin_buttons_mapped = 
	  okTextCancel(ip->xminText, ip->xminOKPBtn,
		       ip->xminCancelPBtn,
		       prob_params.xmin_string);
 
    } else if (!strcmp(client, "xmaxText")) {
	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	xmax_buttons_mapped =
	  okTextValueChanged(ip->xmaxText, ip->xmaxOKPBtn,
			     ip->xmaxCancelPBtn,
			     ip->probDialogShell,
			     xmax_buttons_mapped,
			     prob_params.xmax_string);
      
    } else if (!strcmp(client, "xmaxTextAct") ||
	       !strcmp(client, "xmaxOKPBtn")) {
	String xmaxTextval;

	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	xmax_buttons_mapped = okTextActivate(ip->xmaxText, ip->xmaxOKPBtn,
					     ip->xmaxCancelPBtn, &xmaxTextval);
	
	XtFree (xmaxTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	prob_params.xmax = set_atof_value(ip->xmaxText, "prob_xmax",
					  prob_params.xmax);
	XtFree(prob_params.xmax_string);
	prob_params.xmax_string = get_string_text(ip->xmaxText);
 
    } else if (!strcmp(client, "xmaxCancelPBtn")) {
	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	xmax_buttons_mapped = 
	  okTextCancel(ip->xmaxText, ip->xmaxOKPBtn,
		       ip->xmaxCancelPBtn,
		       prob_params.xmax_string);
 
    } else if (!strcmp(client, "xstepText")) {
	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	xstep_buttons_mapped =
	  okTextValueChanged(ip->xstepText, ip->xstepOKPBtn,
			     ip->xstepCancelPBtn,
			     ip->probDialogShell,
			     xstep_buttons_mapped,
			     prob_params.xstep_string);
      
    } else if (!strcmp(client, "xstepTextAct") ||
	       !strcmp(client, "xstepOKPBtn")) {
	String xstepTextval;
      
	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	xstep_buttons_mapped =
	  okTextActivate(ip->xstepText, ip->xstepOKPBtn,
			 ip->xstepCancelPBtn, &xstepTextval);

	XtFree (xstepTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	prob_params.xstep = set_atof_value(ip->xstepText, "prob_xstep",
					   prob_params.xstep);
	XtFree(prob_params.xstep_string);
	prob_params.xstep_string = get_string_text(ip->xstepText);
 
    } else if (!strcmp(client, "xstepCancelPBtn")) {
	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	xstep_buttons_mapped = 
	  okTextCancel(ip->xstepText, ip->xstepOKPBtn,
		       ip->xstepCancelPBtn,
		       prob_params.xstep_string);
	
    } else if (!strcmp(client, "yminText")) {
	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	ymin_buttons_mapped =
	  okTextValueChanged(ip->yminText, ip->yminOKPBtn,
			     ip->yminCancelPBtn,
			     ip->probDialogShell,
			     ymin_buttons_mapped,
			     prob_params.ymin_string);
      
    } else if (!strcmp(client, "yminTextAct") ||
	       !strcmp(client, "yminOKPBtn")) {
	String yminTextval;
      
	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	ymin_buttons_mapped = okTextActivate(ip->yminText, ip->yminOKPBtn,
					     ip->yminCancelPBtn, &yminTextval);

	XtFree (yminTextval);/* I won't actually be using this copy */
	
	/* check that this value is ok */
	prob_params.ymin = set_atof_value(ip->yminText, "prob_ymin",
					  prob_params.ymin);
	XtFree(prob_params.ymin_string);
	prob_params.ymin_string = get_string_text(ip->yminText);
	
    } else if (!strcmp(client, "yminCancelPBtn")) {
	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	ymin_buttons_mapped = 
	  okTextCancel(ip->yminText, ip->yminOKPBtn,
		       ip->yminCancelPBtn,
		       prob_params.ymin_string);
	
    } else if (!strcmp(client, "ymaxText")) {
	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	ymax_buttons_mapped =
	  okTextValueChanged(ip->ymaxText, ip->ymaxOKPBtn,
			     ip->ymaxCancelPBtn,
			     ip->probDialogShell,
			     ymax_buttons_mapped,
			     prob_params.ymax_string);
      
    } else if (!strcmp(client, "ymaxTextAct") ||
	       !strcmp(client, "ymaxOKPBtn")) {
	String ymaxTextval;
      
	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	ymax_buttons_mapped = okTextActivate(ip->ymaxText, ip->ymaxOKPBtn,
					     ip->ymaxCancelPBtn, &ymaxTextval);

	XtFree (ymaxTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	prob_params.ymax = set_atof_value(ip->ymaxText, "prob_ymax",
					  prob_params.ymax);
	XtFree(prob_params.ymax_string);
	prob_params.ymax_string = get_string_text(ip->ymaxText);
 
    } else if (!strcmp(client, "ymaxCancelPBtn")) {
	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	ymax_buttons_mapped = 
	  okTextCancel(ip->ymaxText, ip->ymaxOKPBtn,
		       ip->ymaxCancelPBtn,
		       prob_params.ymax_string);
 
    } else if (!strcmp(client, "ystepText")) {
	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	ystep_buttons_mapped =
	  okTextValueChanged(ip->ystepText, ip->ystepOKPBtn,
			     ip->ystepCancelPBtn,
			     ip->probDialogShell,
			     ystep_buttons_mapped,
			     prob_params.ystep_string);
      
    } else if (!strcmp(client, "ystepTextAct") ||
	       !strcmp(client, "ystepOKPBtn")) {
	String ystepTextval;
      
	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	ystep_buttons_mapped =
	  okTextActivate(ip->ystepText, ip->ystepOKPBtn,
			 ip->ystepCancelPBtn, &ystepTextval);

	XtFree (ystepTextval);/* I won't actually be using this copy */
	
	/* check that this value is ok */
	prob_params.ystep = set_atof_value(ip->ystepText, "prob_ystep",
					   prob_params.ystep);
	XtFree(prob_params.ystep_string);
	prob_params.ystep_string = get_string_text(ip->ystepText);
 
    } else if (!strcmp(client, "ystepCancelPBtn")) {
	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	ystep_buttons_mapped = 
	  okTextCancel(ip->ystepText, ip->ystepOKPBtn,
		       ip->ystepCancelPBtn,
		       prob_params.ystep_string);
	
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "prob_callback: %s\n", client );*/
	close_popup(ProbClone->probForm);
	
    } else {
	fprintf(stderr, "prob_callback: unknown client: %s\n", (char *)client);
    }
}

void
prob_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(ProbClone->probForm);
    } else if (!strcmp(request, "init")){
	prob_params.plotFname = NULL;
	prob_params.target = 0;
	prob_params.target_string = NULL;
	prob_params.binPlot = 0;
	prob_params.varBin = 0;
	prob_params.nBins = 0;
	prob_params.nBins_string = NULL;
	prob_params.binEnds = NULL;
	prob_params.binEnds_string = NULL;
	prob_params.minBinCount = 0;
	prob_params.minBinCount_string = NULL;
	prob_params.chiSq = 0;
	prob_params.noText = 0;
	prob_params.verbose = 0;
	prob_params.sumToOne = 0;
	prob_params.xmin = 0;
	prob_params.xmin_string = NULL;
	prob_params.xmax = 0;
	prob_params.xmax_string = NULL;
	prob_params.xstep = 0;
	prob_params.xstep_string = NULL;
	prob_params.ymin = 0;
	prob_params.ymin_string = NULL;
	prob_params.ymax = 0;
	prob_params.ymax_string = NULL;
	prob_params.ystep = 0;
	prob_params.ystep_string = NULL;

    } else if (!strcmp(request, "display")){
	char string[10];

	show_string_label(ProbClone->plotFnameLabel, prob_params.plotFname);

	XtFree(prob_params.target_string);
	sprintf(string, "%d", prob_params.target);
	prob_params.target_string = XtNewString(string);
	show_string_text(ProbClone->targetText,
			 prob_params.target_string);
	prob_set_target_label();
	show_radio_value(ProbClone->binPlotRadioBox, prob_params.binPlot);
	show_radio_value(ProbClone->varBinRadioBox, prob_params.varBin);
	gray_out_item(ProbClone->nBinsForm, prob_params.varBin);
	gray_out_item(ProbClone->binEndsForm, !prob_params.varBin);
	XtFree(prob_params.nBins_string);
	sprintf(string, "%d", prob_params.nBins);
	prob_params.nBins_string = XtNewString(string);
	show_string_text(ProbClone->nBinsText,
			 prob_params.nBins_string);
	XtFree(prob_params.binEnds_string);
	prob_params.binEnds_string =
	  make_float_string(prob_params.binEnds);
	show_string_text(ProbClone->binEndsText,
			 prob_params.binEnds_string);
	XtFree(prob_params.minBinCount_string);
	sprintf(string, "%d", prob_params.minBinCount);
	prob_params.minBinCount_string = XtNewString(string);
	show_string_text(ProbClone->minBinCountText,
			 prob_params.minBinCount_string);
	show_toggle_value(ProbClone->chiSqToggle, prob_params.chiSq);
	show_toggle_value(ProbClone->noTextToggle, prob_params.noText);
	show_toggle_value(ProbClone->verboseToggle, prob_params.verbose);
	show_toggle_value(ProbClone->sumToOneToggle, prob_params.sumToOne);
	XtFree(prob_params.xmin_string);
	sprintf(string, "%5.2g", prob_params.xmin);
	prob_params.xmin_string = XtNewString(string);
	show_string_text (ProbClone->xminText,
			  prob_params.xmin_string);

	XtFree(prob_params.xmax_string);
	sprintf(string, "%5.2g", prob_params.xmax);
	prob_params.xmax_string = XtNewString(string);
	show_string_text (ProbClone->xmaxText,
			  prob_params.xmax_string);

	XtFree(prob_params.xstep_string);
	sprintf(string, "%5.2g", prob_params.xstep);
	prob_params.xstep_string = XtNewString(string);
	show_string_text (ProbClone->xstepText,
			  prob_params.xstep_string);

	XtFree(prob_params.ymin_string);
	sprintf(string, "%5.2g", prob_params.ymin);
	prob_params.ymin_string = XtNewString(string);
	show_string_text (ProbClone->yminText,
			  prob_params.ymin_string);

	XtFree(prob_params.ymax_string);
	sprintf(string, "%5.2g", prob_params.ymax);
	prob_params.ymax_string = XtNewString(string);
	show_string_text (ProbClone->ymaxText,
			  prob_params.ymax_string);

	XtFree(prob_params.ystep_string);
	sprintf(string, "%5.2g", prob_params.ystep);
	prob_params.ystep_string = XtNewString(string);
	show_string_text (ProbClone->ystepText,
			 prob_params.ystep_string);

    } else if (!strcmp(request, "plotFname")){
	XtFree(prob_params.plotFname);
	prob_params.plotFname = XtNewString(lnk_params.exp_name);
	prob_params.plotFname = StringConcat(prob_params.plotFname,
					     ".prob.plot");
	show_string_label(ProbClone->plotFnameLabel,
			  prob_params.plotFname);
	print_set_prob_status();

    } else if (!strcmp(request, "plotLabel")){
	show_string_label(ProbClone->plotLabelLabel,
			  plots_params.plot_label);

    } else if (!strcmp("sumToOne", request)){
	if(!prob_params.sumToOne){
	    prob_params.sumToOne = 1;
	    show_toggle_value(ProbClone->sumToOneToggle, prob_params.sumToOne);
	}
    } else {
	fprintf(stderr, "prob_request: unknown request: %s\n", request);
    }
}

static void
prob_set_read_variables()
{
    prob_params.target = target;
    prob_params.binPlot = bin_plot;
    prob_params.varBin = var_bin_flag;
    prob_params.nBins = nbins;
    if(prob_params.binEnds != NULL)
      deallocate_vector(prob_params.binEnds);
    prob_params.binEnds = var_bin_ends;
    prob_params.minBinCount = min_bin_count;
    prob_params.chiSq = chi_square_flag;
    prob_params.noText = no_text;
    prob_params.verbose = verbose;
    prob_params.sumToOne = sum_to_one;
    prob_params.xmin = prob_xmin;
    prob_params.xmax = prob_xmax;
    prob_params.xstep = prob_xstep;
    prob_params.ymin = prob_ymin;
    prob_params.ymax = prob_ymax;
    prob_params.ystep = prob_ystep;
}

static void
prob_set_write_variables()
{
    target = prob_params.target;
    bin_plot = prob_params.binPlot;
    var_bin_flag = prob_params.varBin;
    nbins = prob_params.nBins;
    var_bin_ends = prob_params.binEnds;
    min_bin_count = prob_params.minBinCount;
    chi_square_flag = prob_params.chiSq;
    plot_exp_path = NULL;
    error_fname = NULL;
    npatterns = 0;
    plot_prob_fname = NULL;
    mac_problines_fname = NULL;
    mac_prob_flag = 0;
    no_text = prob_params.noText;
    verbose = prob_params.verbose;
    sum_to_one = prob_params.sumToOne;
    prob_xmin = prob_params.xmin;
    prob_xmax = prob_params.xmax;
    prob_xstep = prob_params.xstep;
    prob_ymin = prob_params.ymin;
    prob_ymax = prob_params.ymax;
    prob_ystep = prob_params.ystep;
    prob_plot_label = NULL;
}

void
read_prob_defaults(argc, argv)
     int argc;
     char **argv;
{
    bin_plot = 0;
    var_bin_flag = 0;
    chi_square_flag = 0;
    no_text = 0;
    verbose = 0;
    sum_to_one = 0;
    var_bin_ends = NULL;
    if(argc > 0){
	argc = nnargs(argc, argv, prob_argtab, PROB_TABSIZE);
	prob_set_read_variables();
    } else {
	/* set any defaults */
	prob_params.binPlot = 0;
	prob_params.target = 0;
	prob_params.nBins = 5;
	prob_params.varBin = 0;
	prob_params.minBinCount = 5;
	prob_params.chiSq = 1;
	prob_params.xmin = 0.;
	prob_params.xmax = 100.;
	prob_params.xstep = 10.;
	prob_params.ymin = 0.;
	prob_params.ymax = 100.;
	prob_params.ystep = 10.;
	prob_params.verbose = 1;
	prob_params.noText = 0;
	prob_params.sumToOne = 0;
    }

    prob_request("display");
}

void
write_prob_defaults(FILE *default_file)
{
    prob_set_write_variables();

    if(default_file != NULL){
	fprintf(default_file, "prob %s",
		nnargs_get_flags(prob_argtab, PROB_TABSIZE));
    }
}

void
store_prob_plot_command(FILE *shell_file)
{
    char *plot_command_line;

    /* this plot has outputs that must go to the log file and stdout using
       a call to nn_tee. This means a pipe that would hang until olxplot
       exited unless no_prob_graphics is set */
    no_prob_graphics = 1;

    store_plot_start(shell_file);
    prob_set_common_flags();
    plot_command_line =
      XtNewString(nnargs_get_flags(prob_argtab, PROB_TABSIZE));
    *strrchr(plot_command_line, '\n') = ' ';	/* get rid of newline at end */
    fprintf(shell_file, "%s\\\n %s",
	    PLOT_PROB, plot_command_line);
    store_plot_end_outputs(shell_file, "prob_plot", prob_params.plotFname);

    /* free things allocated in common_flags */
    XtFree(prob_plot_label);
    XtFree(error_fname);
    XtFree(plot_command_line);
}

static void
prob_set_common_flags()
{
    TEST_TYPE which_data = gnn_get_which_data();

    target = prob_params.target;
    bin_plot = prob_params.binPlot;
    var_bin_flag = prob_params.varBin;
    nbins = prob_params.nBins;
    var_bin_ends = prob_params.binEnds;
    min_bin_count = prob_params.minBinCount;
    chi_square_flag = prob_params.chiSq;

    plot_exp_path = reports_params.exp_path;
    error_fname = XtNewString(reports_params.error_fname);
    switch(which_data){
      case TRAIN_TRAIN:
	error_fname =
	  StringConcat(error_fname, data_params.ext_train);
	npatterns = data_params.npatterns_train;
	break;
      case CV_TRAIN:
	error_fname =
	  StringConcat(error_fname, lnk_params.ext_cv);
	npatterns = data_params.npatterns_train;
	break;
      case CLASS_TRAIN:
	error_fname =
	  StringConcat(error_fname, data_params.ext_test_train);
	npatterns = data_params.npatterns_train;
	break;
      case CLASS_EVAL:
	error_fname =
	  StringConcat(error_fname, data_params.ext_eval);
	npatterns = data_params.npatterns_eval;
	break;
      case CLASS_TEST:
	error_fname =
	  StringConcat(error_fname, data_params.ext_test);
	npatterns = data_params.npatterns_test;
	break;
    }
    plot_prob_fname = prob_params.plotFname;
    
    noutputs = data_params.noutputs;

    prob_xmin = prob_params.xmin;
    prob_xmax = prob_params.xmax;
    prob_xstep = prob_params.xstep;
    prob_ymin = prob_params.ymin;
    prob_ymax = prob_params.ymax;
    prob_ystep = prob_params.ystep;

    verbose = prob_params.verbose;
    no_text = prob_params.noText;
    sum_to_one = prob_params.sumToOne;

    /* mac flags */
    mac_prob_flag = 0;
    mac_problines_fname = "prob.mac";

    /* add double quotes here to eliminate problems with spaces */
    prob_plot_label = XtMalloc(strlen(plots_params.plot_label) + 3);
    sprintf(prob_plot_label, "\"%s\"",plots_params.plot_label);
}

void
prob_set_target_label()
{
    char *label;
    if(0 <= prob_params.target && prob_params.target < data_params.noutputs){
	label = get_class_label(prob_params.target, data_params.class_labels);
    } else {
	label = "Error: Bad Target Class";
    }
    show_string_label(ProbClone->targetLabelLabel, label);
}
