/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/profile/profile_callback.c,v 11.4 2001/08/23 20:17:36 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            PROFILE                                    ****
 ****                          Profile Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  27-Oct-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include <nnlib.h>
#include "profile.h"
/*
 * External Functions
 */
#include "profile_func.h"
#include "../main/lnknet_subs.h"
#include "../lnk/lnk_func.h"
#include "../plots/plots_func.h"
#include "../print/print_func.h"
 
/*
 * Externals
 */
extern ProfileWidgets *ProfileClone;
#include "profile_params.h"
ProfileParams profile_params; /* this is the real one */
#include "../data/data_params.h"
extern DataParams data_params;/* for data base path and name and extensions */
#include "../plots/plots_params.h"
extern PlotsParams plots_params;
#include "../lnk/lnk_params.h"
extern LnkParams lnk_params;
#include "../reports/reports_params.h"
extern ReportsParams reports_params;
#include "../priors/priors_params.h"
extern PriorsParams priors_params;
#include "../select/select_params.h"
extern SelectParams select_params;
#include "../norm/norm_params.h"
extern NormParams norm_params;

/* variables for shell scripts */
#define PLOT_BOUND "_plot_bound"
#include <plot_bound_args.h>

/*
 * Statics 
 */
static void profile_set_read_variables();
static void profile_set_write_variables();
static void profile_set_common_flags(TEST_TYPE);
static void create_profile_plot_fname(TEST_TYPE);
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: profile_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   profile_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  27-Oct-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
profile_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    ProfileWidgets *ip;
    static int xDimen_buttons_mapped = False;
    static int otherVar_buttons_mapped = False;
    static int nPoints_buttons_mapped = False;
    static int xmin_buttons_mapped = False;
    static int xmax_buttons_mapped = False;
    static int xstep_buttons_mapped = False;
    static int ymin_buttons_mapped = False;
    static int ymax_buttons_mapped = False;
    static int ystep_buttons_mapped = False;
    static int distLim_buttons_mapped = False;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "xDimenText")) {
	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	xDimen_buttons_mapped =
	  okTextValueChanged(ip->xDimenText, ip->xDimenOKPBtn,
			     ip->xDimenCancelPBtn,
			     ip->profileDialogShell,
			     xDimen_buttons_mapped,
			     profile_params.xDimen_string);
	
    } else if ((!strcmp(client, "xDimenTextAct")) ||
	       (!strcmp(client, "xDimenOKPBtn"))) {
	String xDimenTextval;

	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	xDimen_buttons_mapped =
	  okTextActivate(ip->xDimenText, ip->xDimenOKPBtn,
			 ip->xDimenCancelPBtn, &xDimenTextval);
	/* I won't actually be using this copy */
	XtFree (xDimenTextval);
	
	/* verify the string as an integer */
	profile_params.xDimen =
	  set_atoi_value(ip->xDimenText, "xDimen_set",
			     profile_params.xDimen);

	/* save the current text for future comparisons */
	XtFree(profile_params.xDimen_string);
	profile_params.xDimen_string = get_string_text(ip->xDimenText);
 
	/* do any other things needed */
	profile_set_input_label();

    } else if (!strcmp(client, "xDimenCancelPBtn")) {
	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	xDimen_buttons_mapped = 
	  okTextCancel(ip->xDimenText, ip->xDimenOKPBtn,
		       ip->xDimenCancelPBtn,
		       profile_params.xDimen_string);
	

    } else if (!strcmp(client, "otherVarText")) {
	/*fprintf (stderr, "profile_callback: %s\n", client);*/
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	otherVar_buttons_mapped =
	  okTextValueChanged(ip->otherVarText, ip->otherVarOKPBtn,
			     ip->otherVarCancelPBtn,
			     ip->profileDialogShell,
			     otherVar_buttons_mapped,
			     profile_params.otherVar_string);
    } else if ((!strcmp(client, "otherVarTextAct")) ||
	       (!strcmp(client, "otherVarOKPBtn"))) {
	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	String otherVarTextval;
	otherVar_buttons_mapped =
	  okTextActivate(ip->otherVarText,
			 ip->otherVarOKPBtn,
			 ip->otherVarCancelPBtn,
			 &otherVarTextval);
	
	if(profile_params.otherVar != NULL)
	  deallocate_vector(profile_params.otherVar);
	/* take spaces and tabs out of the list before handing it to nnargs */
	if(otherVarTextval != NULL){
	    char *otherVarTemp = otherVarTextval;
	    otherVarTextval = strtok(otherVarTextval, " \t");
	    profile_params.otherVar =
	      nnargs_get_float_alloc_list(otherVarTextval);
	    XtFree(otherVarTemp);
	}
	
	/* make a copy of the string as it has been interpretted */
	XtFree(profile_params.otherVar_string);
	profile_params.otherVar_string =
	  make_float_string(profile_params.otherVar);
	show_string_text(ip->otherVarText,
			 profile_params.otherVar_string);
	
	/* do anything else necessary */
	
    } else if (!strcmp(client, "otherVarCancelPBtn")) {
	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	otherVar_buttons_mapped = 
	  okTextCancel(ip->otherVarText, ip->otherVarOKPBtn,
		       ip->otherVarCancelPBtn,
		       profile_params.otherVar_string);

    } else if (!strcmp(client, "nPointsText")) {
	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	nPoints_buttons_mapped =
	  okTextValueChanged(ip->nPointsText, ip->nPointsOKPBtn,
			     ip->nPointsCancelPBtn,
			     ip->profileDialogShell,
			     nPoints_buttons_mapped,
			     profile_params.nPoints_string);
	
    } else if ((!strcmp(client, "nPointsTextAct")) ||
	       (!strcmp(client, "nPointsOKPBtn"))) {
	String nPointsTextval;

	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	nPoints_buttons_mapped =
	  okTextActivate(ip->nPointsText, ip->nPointsOKPBtn,
			 ip->nPointsCancelPBtn, &nPointsTextval);
	/* I won't actually be using this copy */
	XtFree (nPointsTextval);
	
	/* verify the string as an integer */
	profile_params.nPoints =
	  set_atoi_value(ip->nPointsText, "nPoints_set",
			     profile_params.nPoints);

	/* save the current text for future comparisons */
	XtFree(profile_params.nPoints_string);
	profile_params.nPoints_string = get_string_text(ip->nPointsText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "nPointsCancelPBtn")) {
	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	nPoints_buttons_mapped = 
	  okTextCancel(ip->nPointsText, ip->nPointsOKPBtn,
		       ip->nPointsCancelPBtn,
		       profile_params.nPoints_string);
	
    } else if (!strcmp(client, "autoscaleToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "profile_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	profile_params.autoscale = state->set;
	gray_out_item(ProfileClone->axesForm, state->set);
 
    } else if (!strcmp(client, "xminText")) {
	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	xmin_buttons_mapped =
	  okTextValueChanged(ip->xminText, ip->xminOKPBtn,
			     ip->xminCancelPBtn,
			     ip->profileDialogShell,
			     xmin_buttons_mapped,
			     profile_params.xmin_string);
      
    } else if (!strcmp(client, "xminTextAct") ||
	       !strcmp(client, "xminOKPBtn")) {
	String xminTextval;

	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	xmin_buttons_mapped = okTextActivate(ip->xminText, ip->xminOKPBtn,
					     ip->xminCancelPBtn, &xminTextval);

	XtFree (xminTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	profile_params.xmin = set_atof_value(ip->xminText, "profile_xmin",
					  profile_params.xmin);
	XtFree(profile_params.xmin_string);
	profile_params.xmin_string = get_string_text(ip->xminText);
 
    } else if (!strcmp(client, "xminCancelPBtn")) {
	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	xmin_buttons_mapped = 
	  okTextCancel(ip->xminText, ip->xminOKPBtn,
		       ip->xminCancelPBtn,
		       profile_params.xmin_string);
 
    } else if (!strcmp(client, "xmaxText")) {
	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	xmax_buttons_mapped =
	  okTextValueChanged(ip->xmaxText, ip->xmaxOKPBtn,
			     ip->xmaxCancelPBtn,
			     ip->profileDialogShell,
			     xmax_buttons_mapped,
			     profile_params.xmax_string);
      
    } else if (!strcmp(client, "xmaxTextAct") ||
	       !strcmp(client, "xmaxOKPBtn")) {
	String xmaxTextval;

	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	xmax_buttons_mapped = okTextActivate(ip->xmaxText, ip->xmaxOKPBtn,
					     ip->xmaxCancelPBtn, &xmaxTextval);
	
	XtFree (xmaxTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	profile_params.xmax = set_atof_value(ip->xmaxText, "profile_xmax",
					  profile_params.xmax);
	XtFree(profile_params.xmax_string);
	profile_params.xmax_string = get_string_text(ip->xmaxText);
 
    } else if (!strcmp(client, "xmaxCancelPBtn")) {
	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	xmax_buttons_mapped = 
	  okTextCancel(ip->xmaxText, ip->xmaxOKPBtn,
		       ip->xmaxCancelPBtn,
		       profile_params.xmax_string);
 
    } else if (!strcmp(client, "xstepText")) {
	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	xstep_buttons_mapped =
	  okTextValueChanged(ip->xstepText, ip->xstepOKPBtn,
			     ip->xstepCancelPBtn,
			     ip->profileDialogShell,
			     xstep_buttons_mapped,
			     profile_params.xstep_string);
      
    } else if (!strcmp(client, "xstepTextAct") ||
	       !strcmp(client, "xstepOKPBtn")) {
	String xstepTextval;
      
	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	xstep_buttons_mapped =
	  okTextActivate(ip->xstepText, ip->xstepOKPBtn,
			 ip->xstepCancelPBtn, &xstepTextval);

	XtFree (xstepTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	profile_params.xstep = set_atof_value(ip->xstepText, "profile_xstep",
					   profile_params.xstep);
	XtFree(profile_params.xstep_string);
	profile_params.xstep_string = get_string_text(ip->xstepText);
 
    } else if (!strcmp(client, "xstepCancelPBtn")) {
	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	xstep_buttons_mapped = 
	  okTextCancel(ip->xstepText, ip->xstepOKPBtn,
		       ip->xstepCancelPBtn,
		       profile_params.xstep_string);
	
    } else if (!strcmp(client, "yminText")) {
	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	ymin_buttons_mapped =
	  okTextValueChanged(ip->yminText, ip->yminOKPBtn,
			     ip->yminCancelPBtn,
			     ip->profileDialogShell,
			     ymin_buttons_mapped,
			     profile_params.ymin_string);
      
    } else if (!strcmp(client, "yminTextAct") ||
	       !strcmp(client, "yminOKPBtn")) {
	String yminTextval;
      
	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	ymin_buttons_mapped = okTextActivate(ip->yminText, ip->yminOKPBtn,
					     ip->yminCancelPBtn, &yminTextval);

	XtFree (yminTextval);/* I won't actually be using this copy */
	
	/* check that this value is ok */
	profile_params.ymin = set_atof_value(ip->yminText, "profile_ymin",
					  profile_params.ymin);
	XtFree(profile_params.ymin_string);
	profile_params.ymin_string = get_string_text(ip->yminText);
	
    } else if (!strcmp(client, "yminCancelPBtn")) {
	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	ymin_buttons_mapped = 
	  okTextCancel(ip->yminText, ip->yminOKPBtn,
		       ip->yminCancelPBtn,
		       profile_params.ymin_string);
	
    } else if (!strcmp(client, "ymaxText")) {
	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	ymax_buttons_mapped =
	  okTextValueChanged(ip->ymaxText, ip->ymaxOKPBtn,
			     ip->ymaxCancelPBtn,
			     ip->profileDialogShell,
			     ymax_buttons_mapped,
			     profile_params.ymax_string);
      
    } else if (!strcmp(client, "ymaxTextAct") ||
	       !strcmp(client, "ymaxOKPBtn")) {
	String ymaxTextval;
      
	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	ymax_buttons_mapped = okTextActivate(ip->ymaxText, ip->ymaxOKPBtn,
					     ip->ymaxCancelPBtn, &ymaxTextval);

	XtFree (ymaxTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	profile_params.ymax = set_atof_value(ip->ymaxText, "profile_ymax",
					  profile_params.ymax);
	XtFree(profile_params.ymax_string);
	profile_params.ymax_string = get_string_text(ip->ymaxText);
 
    } else if (!strcmp(client, "ymaxCancelPBtn")) {
	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	ymax_buttons_mapped = 
	  okTextCancel(ip->ymaxText, ip->ymaxOKPBtn,
		       ip->ymaxCancelPBtn,
		       profile_params.ymax_string);
 
    } else if (!strcmp(client, "ystepText")) {
	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	ystep_buttons_mapped =
	  okTextValueChanged(ip->ystepText, ip->ystepOKPBtn,
			     ip->ystepCancelPBtn,
			     ip->profileDialogShell,
			     ystep_buttons_mapped,
			     profile_params.ystep_string);
      
    } else if (!strcmp(client, "ystepTextAct") ||
	       !strcmp(client, "ystepOKPBtn")) {
	String ystepTextval;
      
	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	ystep_buttons_mapped =
	  okTextActivate(ip->ystepText, ip->ystepOKPBtn,
			 ip->ystepCancelPBtn, &ystepTextval);

	XtFree (ystepTextval);/* I won't actually be using this copy */
	
	/* check that this value is ok */
	profile_params.ystep = set_atof_value(ip->ystepText, "profile_ystep",
					   profile_params.ystep);
	XtFree(profile_params.ystep_string);
	profile_params.ystep_string = get_string_text(ip->ystepText);
 
    } else if (!strcmp(client, "ystepCancelPBtn")) {
	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	ystep_buttons_mapped = 
	  okTextCancel(ip->ystepText, ip->ystepOKPBtn,
		       ip->ystepCancelPBtn,
		       profile_params.ystep_string);
	
    } else if (!strcmp(client, "noNormToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "profile_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	profile_params.noNorm = state->set;
	profile_set_input_label();
	
    } else if (!strcmp(client, "showAllToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "profile_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	profile_params.showAll = state->set;
	gray_out_item(ip->distLimForm, profile_params.showAll);
	
    } else if (!strcmp(client, "distLimText")) {
	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	distLim_buttons_mapped =
	  okTextValueChanged(ip->distLimText, ip->distLimOKPBtn,
			     ip->distLimCancelPBtn,
			     ip->profileDialogShell,
			     distLim_buttons_mapped,
			     profile_params.distLim_string);
	
    } else if ((!strcmp(client, "distLimTextAct")) ||
	       (!strcmp(client, "distLimOKPBtn"))) {
	String distLimTextval;

	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	distLim_buttons_mapped =
	  okTextActivate(ip->distLimText, ip->distLimOKPBtn,
			 ip->distLimCancelPBtn, &distLimTextval);
	/* I won't actually be using this copy */
	XtFree (distLimTextval);
	
	/* verify the string as a float */
	profile_params.distLim =
	  set_atof_value(ip->distLimText, "distLim_set",
			 profile_params.distLim);

	/* save the current text for future comparisons */
	XtFree(profile_params.distLim_string);
	profile_params.distLim_string = get_string_text(ip->distLimText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "distLimCancelPBtn")) {
	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	distLim_buttons_mapped = 
	  okTextCancel(ip->distLimText, ip->distLimOKPBtn,
		       ip->distLimCancelPBtn,
		       profile_params.distLim_string);
	
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "profile_callback: %s\n", client );*/
	close_popup(ProfileClone->profileForm);
	
    } else {
	fprintf(stderr, "profile_callback: unknown client: %s\n", (char *)client);
    }
}

void
profile_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(ProfileClone->profileForm);
    } else if (!strcmp(request, "init")){
	profile_params.xDimen = 0;
	profile_params.xDimen_string = NULL;
	profile_params.otherVar = NULL;
	profile_params.otherVar_string = NULL;
	profile_params.nPoints = 0;
	profile_params.nPoints_string = NULL;
	profile_params.xmin = 0;
	profile_params.xmin_string = NULL;
	profile_params.xmax = 0;
	profile_params.xmax_string = NULL;
	profile_params.xstep = 0;
	profile_params.xstep_string = NULL;
	profile_params.ymin = 0;
	profile_params.ymin_string = NULL;
	profile_params.ymax = 0;
	profile_params.ymax_string = NULL;
	profile_params.ystep = 0;
	profile_params.ystep_string = NULL;
	profile_params.noNorm = 0;
	profile_params.showAll = 0;
	profile_params.distLim = 0.;
	profile_params.distLim_string = NULL;

    } else if (!strcmp(request, "display")){
	char string[10];
	show_string_label(ProfileClone->profilePrefixLabel,
			  profile_params.profilePrefix);
	XtFree(profile_params.xDimen_string);
	sprintf(string, "%d", profile_params.xDimen);
	profile_params.xDimen_string = XtNewString(string);
	show_string_text(ProfileClone->xDimenText,
			 profile_params.xDimen_string);

	profile_set_input_label();

	XtFree(profile_params.otherVar_string);
	profile_params.otherVar_string =
	  make_float_string(profile_params.otherVar);
	show_string_text(ProfileClone->otherVarText,
			 profile_params.otherVar_string);

	XtFree(profile_params.nPoints_string);
	sprintf(string, "%d", profile_params.nPoints);
	profile_params.nPoints_string = XtNewString(string);
	show_string_text(ProfileClone->nPointsText,
			 profile_params.nPoints_string);

	show_toggle_value (ProfileClone->autoscaleToggle,
			   profile_params.autoscale);
	gray_out_item(ProfileClone->axesForm, profile_params.autoscale);

	XtFree(profile_params.xmin_string);
	sprintf(string, "%5.2g", profile_params.xmin);
	profile_params.xmin_string = XtNewString(string);
	show_string_text (ProfileClone->xminText,
			  profile_params.xmin_string);

	XtFree(profile_params.xmax_string);
	sprintf(string, "%5.2g", profile_params.xmax);
	profile_params.xmax_string = XtNewString(string);
	show_string_text (ProfileClone->xmaxText,
			  profile_params.xmax_string);

	XtFree(profile_params.xstep_string);
	sprintf(string, "%5.2g", profile_params.xstep);
	profile_params.xstep_string = XtNewString(string);
	show_string_text (ProfileClone->xstepText,
			  profile_params.xstep_string);

	XtFree(profile_params.ymin_string);
	sprintf(string, "%5.2g", profile_params.ymin);
	profile_params.ymin_string = XtNewString(string);
	show_string_text (ProfileClone->yminText,
			  profile_params.ymin_string);

	XtFree(profile_params.ymax_string);
	sprintf(string, "%5.2g", profile_params.ymax);
	profile_params.ymax_string = XtNewString(string);
	show_string_text (ProfileClone->ymaxText,
			  profile_params.ymax_string);

	XtFree(profile_params.ystep_string);
	sprintf(string, "%5.2g", profile_params.ystep);
	profile_params.ystep_string = XtNewString(string);
	show_string_text (ProfileClone->ystepText,
			  profile_params.ystep_string);

	show_toggle_value(ProfileClone->noNormToggle, profile_params.noNorm);
	show_toggle_value(ProfileClone->showAllToggle, profile_params.showAll);

	gray_out_item(ProfileClone->distLimForm, profile_params.showAll);
	XtFree(profile_params.distLim_string);
	sprintf(string, "%5.2g", profile_params.distLim);
	profile_params.distLim_string = XtNewString(string);
	show_string_text(ProfileClone->distLimText,
			 profile_params.distLim_string);

	show_string_label(ProfileClone->plotLabelLabel,
			  plots_params.plot_label);

    } else if (!strcmp(request, "plotFname")){
	XtFree(profile_params.profilePrefix);
	profile_params.profilePrefix = XtNewString(lnk_params.exp_name);
	profile_params.profilePrefix = StringConcat(profile_params.profilePrefix,
						  ".profile.plot");
	show_string_label(ProfileClone->profilePrefixLabel,
			  profile_params.profilePrefix);
	create_profile_plot_fname(gnn_get_which_data());
	print_set_profile_status();

    } else if (!strcmp(request, "plotLabel")){
	show_string_label(ProfileClone->plotLabelLabel,
			  plots_params.plot_label);

    } else {
	fprintf(stderr, "profile_request: unknown request: %s\n", request);
    }
}

void
read_profile_defaults(argc, argv)
     int argc;
     char **argv;
{
    /* clear flags and strings which are not printed if they are NULL */
    bound_autoscale = 0;
    plot_exp_path = NULL;
    plot_error_fname = NULL;
    bound_plot_fname = NULL;
    mac_flag = 0;
    mac_wireframe_flag = 0;
    mac_wireframe_output = -1;
    macdots_fname = NULL;
    macscatter_fname = NULL;
    maclines_fname = NULL;
    machisto_fname = NULL;
    macwireframe_fname = NULL;
    profile_plot_fname = NULL;
    bound_plot_label = NULL;
    profile_plot_label = NULL;
    other_vars[0] = 0.;
    region_plot = 0;
    scatter_plot = 0;
    internals_plot = 0;
    profile_plot = 0;
    histogram_plot = 0;
    show_color = 1; /* this is always on here */
    show_misses = 0;
    show_all = 0;
    scatter_data_fullname = NULL;
    scatter_desc_fullname = NULL;
    plot_map = 0;
    plot_adjust_priors = 0;
    plot_uniform_priors = 0;
    plot_priors_list[0] = 0.;
    no_graphics = 0;
    no_norm = 0;

    if(argc > 0){
	argc = nnargs(argc, argv, bound_argtab, BOUND_TABSIZE);
	profile_set_read_variables();
    } else {
	/* set any defaults */
	profile_params.autoscale = True;
	profile_params.showAll = True;
	profile_params.xDimen = 0;
	profile_params.nPoints = 50;
	profile_params.xmin = -3;
	profile_params.xmax = 3;
	profile_params.xstep = 1;
	profile_params.ymin = -3;
	profile_params.ymax = 3;
	profile_params.ystep = 1;
	profile_params.distLim = 0.5;
	profile_params.noNorm = False;
    }

    profile_request("display");
}

void
write_profile_defaults(FILE *default_file)
{
    profile_set_write_variables();
    if(default_file != NULL){
	fprintf(default_file, "profile %s",
		nnargs_get_flags(bound_argtab, BOUND_TABSIZE));
    }
}

static void
profile_set_read_variables()
{
    profile_params.autoscale = bound_autoscale;
    profile_params.xmin = bound_xmin;
    profile_params.xmax = bound_xmax;
    profile_params.xstep = bound_xstep;
    profile_params.ymin = profile_ymin;
    profile_params.ymax = profile_ymax;
    profile_params.ystep = profile_ystep;
    profile_params.distLim = distance_limit;
    profile_params.xDimen = bound_dim_1;
    profile_params.nPoints = bound_npoints;
    profile_params.showAll = show_all;
    profile_params.noNorm = no_norm;
    if (profile_params.otherVar != NULL){
	deallocate_vector(profile_params.otherVar);
	profile_params.otherVar = NULL;
    }
    profile_params.otherVar = copy_static2alloc_vector(other_vars);
}

/* copy the variables for writing to screen file.
   zero out the uninteresting ones (derived or copied data) */
static void
profile_set_write_variables()
{
    plot_exp_path = NULL;
    plot_error_fname = NULL;
    plot_n_inputs = 0;
    plot_n_outputs = 0;
    no_graphics = 0;

    /* mac flags */
    mac_flag = 0;
    macdots_fname = NULL;
    macscatter_fname = NULL;
    maclines_fname = NULL;
    machisto_fname = NULL;
    mac_wireframe_flag = 0;
    mac_wireframe_output = -1;
    macwireframe_fname = NULL;

    bound_autoscale = profile_params.autoscale;

    scatter_desc_fullname = NULL;

    scatter_data_fullname = NULL;
    scatter_n_patterns = 0;
    bound_plot_fname = NULL;
    bound_plot_label = NULL;
    profile_plot_fname = NULL;
    profile_plot_label = NULL;

    bound_xmin = profile_params.xmin;
    bound_xmax = profile_params.xmax;
    bound_xstep = profile_params.xstep;
    profile_ymin = profile_params.ymin;
    profile_ymax = profile_params.ymax;
    profile_ystep = profile_params.ystep;

    distance_limit = profile_params.distLim;
    internals_scale = 0.;
    bound_dim_1 = profile_params.xDimen;
    bound_npoints = profile_params.nPoints;
    internals_level = 0.;
    region_plot = 0;
    scatter_plot = 0;
    internals_plot = 0;
    profile_plot = 0;
    histogram_plot = 0;
    show_all = profile_params.showAll;
    show_color = 1;	/* this is always on here */
    show_misses = 0;

    plot_adjust_priors = 0;
    plot_uniform_priors = 0;
    plot_priors_list[0] = 0.;
    plot_map = 0;
    no_norm = profile_params.noNorm;
    copy_alloc2static_vector(profile_params.otherVar, other_vars);
}

void
store_profile_plot_command(FILE *shell_file, int show_plot, int which_data)
{
    char *plot_command_line;

    if(show_plot) no_graphics = 0;
    else no_graphics = 1;

    store_plot_start(shell_file);
    profile_set_common_flags(which_data);
    plot_command_line =
      XtNewString(nnargs_get_flags(bound_argtab, BOUND_TABSIZE));
    *strrchr(plot_command_line, '\n') = ' ';	/* get rid of newline at end */
    fprintf(shell_file, "%s%s\\\n %s",
	    lnk_params.classifier_name, PLOT_BOUND, plot_command_line);
    /* put in classifier dependent call if needed */
    g_get_test_args (shell_file);
    store_plot_end(shell_file);
    /* free things allocated in common_flags */
    XtFree(scatter_desc_fullname);
    XtFree(scatter_data_fullname);
    XtFree(profile_plot_label);
    XtFree(plot_command_line);
}

static void
profile_set_common_flags(TEST_TYPE which_data)
{
    plot_exp_path = reports_params.exp_path;
    plot_error_fname = reports_params.param_fname;
    plot_n_inputs = data_params.nraw;
    plot_n_outputs = data_params.noutputs;
    /* no_graphics is already set */

    /* mac flags */
    mac_flag = 0;
    macdots_fname = "boundary_dots.mac";
    macscatter_fname = "scatter.mac";
    maclines_fname = "profile_lines.mac";
    machisto_fname = "profile_histo.mac";
    mac_wireframe_flag = 0;
    mac_wireframe_output = -1;
    macwireframe_fname = "boundary_wire.mac";

    bound_autoscale = profile_params.autoscale;

    scatter_desc_fullname =
	   path_create(data_params.data_path, data_params.desc_fname);

    switch(which_data){
      case CLASS_TRAIN:
      case TRAIN_TRAIN:
      case CV_TRAIN:
	scatter_data_fullname =
	  path_create(data_params.data_path, data_params.train_fname);
	scatter_n_patterns = data_params.npatterns_train;
	break;
      case CLASS_EVAL:
	scatter_data_fullname =
	  path_create(data_params.data_path, data_params.eval_fname);
	scatter_n_patterns = data_params.npatterns_eval;
	break;
      case CLASS_TEST:
	scatter_data_fullname =
	  path_create(data_params.data_path, data_params.test_fname);
	scatter_n_patterns = data_params.npatterns_test;
	break;
    }

    if(plots_params.storeEpoch){
	profile_plot_fname = plots_params.profileEpochFname;
    } else {
	create_profile_plot_fname(which_data);
	profile_plot_fname = profile_params.profileFname;
    }
    /* clear bound plot strings */
    bound_plot_fname = NULL;
    bound_plot_label = NULL;

    bound_xmin = profile_params.xmin;
    bound_xmax = profile_params.xmax;
    bound_xstep = profile_params.xstep;
    profile_ymin = profile_params.ymin;
    profile_ymax = profile_params.ymax;
    profile_ystep = profile_params.ystep;

    distance_limit = profile_params.distLim;
    internals_scale = 0.;
    /* add double quotes here to eliminate problems with spaces */
    profile_plot_label = XtMalloc(strlen(plots_params.plot_label) + 3);
    sprintf(profile_plot_label, "\"%s\"",plots_params.plot_label);

    bound_plot_label = NULL;
    bound_dim_1 = profile_params.xDimen;
    bound_npoints = profile_params.nPoints;
    internals_level = 0.;
    region_plot = 0;
    scatter_plot = 0;
    internals_plot = 0;
    profile_plot = plots_params.profile;
    histogram_plot = plots_params.histogram;
    show_all = profile_params.showAll;
    show_color = 1; /* this is always on here */
    show_misses = 0;

    plot_adjust_priors = priors_params.adjustTest;
    plot_uniform_priors = priors_params.testUniform;
    copy_alloc2static_vector(priors_params.testPriorsList, plot_priors_list);
    plot_map = data_params.db_type;
    no_norm = profile_params.noNorm;
    copy_alloc2static_vector(profile_params.otherVar, other_vars);
}

static void
create_profile_plot_fname(TEST_TYPE which_data)
{
    XtFree(profile_params.profileFname);
    profile_params.profileFname = XtNewString(profile_params.profilePrefix);
    switch (which_data){
      case TRAIN_TRAIN:
      case CV_TRAIN:
	profile_params.profileFname = StringConcat(profile_params.profileFname,
						 data_params.ext_train);
	break;
      case CLASS_TRAIN:
	profile_params.profileFname = StringConcat(profile_params.profileFname,
						 data_params.ext_test_train);
	break;
      case CLASS_EVAL:
	profile_params.profileFname = StringConcat(profile_params.profileFname,
						 data_params.ext_eval);
	break;
      case CLASS_TEST:
	profile_params.profileFname = StringConcat(profile_params.profileFname,
						 data_params.ext_test);
	break;
    }
}

void
profile_set_input_label()
{
    char * label;
    int upper_limit;
    
    if(profile_params.noNorm) upper_limit = data_params.nraw;
    else upper_limit = select_params.ninputs;

    if(0 <= profile_params.xDimen && profile_params.xDimen < upper_limit){
	if(profile_params.noNorm){
	    label = get_input_label(profile_params.xDimen,
				    data_params.input_labels,
				    NULL, 0);
	} else {
	    label = get_input_label(profile_params.xDimen,
				    data_params.input_labels,
				    select_params.features,
				    norm_params.normalize);
	}
    } else {
	label = "Error: Bad Plot Feature";
    }
    show_string_label(ProfileClone->xDimenLabelLabel, label);
}
