/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/ran_clus/ran_clus_callback.c,v 11.4 2001/08/23 20:17:39 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            RAN_CLUS                              ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  27-Oct-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <Xm/Xm.h>
#include <nnlib.h>
#include <ran_clus_args.h>
#include "ran_clus.h"
/*
 * External Functions
 */
#include "../main/lnknet_subs.h"
#include "../plots/plots_func.h"
#include "ran_clus_func.h"

/*
 * Externals
 */
extern Ran_clusWidgets *Ran_clusClone;
#include "ran_clus_params.h"
Ran_ClusParams ran_clus_params;	/* this is the real one */
#include "../data/data_params.h"
extern DataParams data_params;
 
/*
 * Statics
 */
static void ran_clus_show_ncenters_choices();

/******************************************************************************
 ******************************************************************************
 *
 * Module Name: ran_clus_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   ran_clus_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  27-Oct-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
ran_clus_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    Ran_clusWidgets *ip;
    static int ncenters_buttons_mapped = False;
    static int ncentersList_buttons_mapped = False;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
    
    if (!strcmp(client, "ignoreClassToggle") || 
	!strcmp(client, "clusterByClassToggle")) {
	/*fprintf (stderr, "ran_clus_callback: %s\n", client );*/
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	if(state->set){
	    ran_clus_params.cluster_by_class =
	      get_radio_value(ip->clusterByClassRadioBox);
	    ran_clus_show_ncenters_choices();
	    ran_clus_set_total_ncenters();
	    set_plot_labels();
	}

    } else if (!strcmp(client, "equalNcentersToggle") ||
	       !strcmp(client, "ncentersByClassToggle")) {
	/*fprintf (stderr, "ran_clus_callback: %s\n", client );*/
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	if(state->set){
	    ran_clus_params.specify_by_class =
	      get_radio_value(ip->ncentersByClassRadioBox);
	    ran_clus_show_ncenters_choices();
	    ran_clus_set_total_ncenters();
	    set_plot_labels();
	}
	
    } else if (!strcmp(client, "ncentersText")) {
	/*fprintf (stderr, "ran_clus_callback: %s\n", client );*/
	ncenters_buttons_mapped =
	  okTextValueChanged(ip->ncentersText, ip->ncentersOKPBtn,
			     ip->ncentersCancelPBtn,
			     ip->ran_clusDialogShell,
			     ncenters_buttons_mapped,
			     ran_clus_params.ncenters_string);
	
    } else if ((!strcmp(client, "ncentersTextAct")) ||
	       (!strcmp(client, "ncentersOKPBtn"))) {
	String ncentersTextval;

      /*fprintf (stderr, "ran_clus_callback: %s\n", client );*/
	ncenters_buttons_mapped =
	  okTextActivate(ip->ncentersText, ip->ncentersOKPBtn,
			 ip->ncentersCancelPBtn, &ncentersTextval);
	/* I won't actually be using this copy */
	XtFree (ncentersTextval);
	
	/* verify the string as an integer */
	ran_clus_params.ncenters =
	  set_atoi_value(ip->ncentersText, "ncenters_set",
			     ran_clus_params.ncenters);

	/* save the current text for future comparisons */
	XtFree(ran_clus_params.ncenters_string);
	ran_clus_params.ncenters_string = get_string_text(ip->ncentersText);
 
	/* do any other things needed */
	ran_clus_set_total_ncenters();
	set_plot_labels();
  } else if (!strcmp(client, "ncentersCancelPBtn")) {
    /*fprintf (stderr, "ran_clus_callback: %s\n", client );*/
	ncenters_buttons_mapped = 
	  okTextCancel(ip->ncentersText, ip->ncentersOKPBtn,
		       ip->ncentersCancelPBtn,
		       ran_clus_params.ncenters_string);
	
    } else if (!strcmp(client, "ncentersListText")) {
	/*fprintf (stderr, "ran_clus_callback: %s\n", client);*/
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	ncentersList_buttons_mapped =
	  okTextValueChanged(ip->ncentersListText, ip->ncentersListOKPBtn,
			     ip->ncentersListCancelPBtn,
			     ip->ran_clusDialogShell,
			     ncentersList_buttons_mapped,
			     ran_clus_params.ncentersList_string);
    } else if ((!strcmp(client, "ncentersListTextAct")) ||
	       (!strcmp(client, "ncentersListOKPBtn"))) {
	/*fprintf (stderr, "ran_clus_callback: %s\n", client );*/
	String ncentersListTextval;
	ncentersList_buttons_mapped =
	  okTextActivate(ip->ncentersListText,
			 ip->ncentersListOKPBtn,
			 ip->ncentersListCancelPBtn,
			 &ncentersListTextval);
	
	if(ran_clus_params.ncentersList != NULL)
	  deallocate_ivector(ran_clus_params.ncentersList);
	ran_clus_params.ncentersList = NULL;
	/* take spaces and tabs out of the list before handing it to nnargs */
	if(ncentersListTextval != NULL){
	    char *ncentersListTemp = ncentersListTextval;
	    ncentersListTextval = strtok(ncentersListTextval, " \t");
	    if(ncentersListTextval != NULL){
		ran_clus_params.ncentersList =
		  nnargs_get_int_alloc_list(ncentersListTextval);
		XtFree(ncentersListTemp);
	    }
	}
	
	/* make a copy of the string as it has been interpretted */
	XtFree(ran_clus_params.ncentersList_string);
	ran_clus_params.ncentersList_string =
	  make_int_string(ran_clus_params.ncentersList);
	show_string_text(ip->ncentersListText,
			 ran_clus_params.ncentersList_string);
	
	/* do anything else necessary */
	ran_clus_set_total_ncenters();
	set_plot_labels();
	
    } else if (!strcmp(client, "ncentersListCancelPBtn")) {
	/*fprintf (stderr, "ran_clus_callback: %s\n", client );*/
	ncentersList_buttons_mapped = 
	  okTextCancel(ip->ncentersListText, ip->ncentersListOKPBtn,
		       ip->ncentersListCancelPBtn,
		       ran_clus_params.ncentersList_string);
      
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "ran_clus_callback: %s\n", client );*/
	close_popup(Ran_clusClone->ran_clusForm);
      
  } else {
      fprintf(stderr, "ran_clus_callback: unknown client: %s\n", (char *)client);
  }
}

void
ran_clus_request(String request)
{
    if (!strcmp(request, "open")) {
	open_popup(Ran_clusClone->ran_clusForm);
    } else if (!strcmp(request, "init")){
	/* this should only get called once, before ANYTHING 
	   gets allocated or there will be memory leaks */
	ran_clus_params.cluster_by_class = False;
	ran_clus_params.specify_by_class = False;
	ran_clus_params.ncenters = 0;
	ran_clus_params.ncenters_string = NULL;
	ran_clus_params.ncentersList = NULL;
	ran_clus_params.ncentersList_string = NULL;
	ran_clus_params.total_ncenters = 0;
    } else if (!strcmp(request, "display")){
	char string[10]; /* for creating int and float text */
	show_radio_value(Ran_clusClone->clusterByClassRadioBox,
			 ran_clus_params.cluster_by_class);
	show_radio_value(Ran_clusClone->ncentersByClassRadioBox,
			 ran_clus_params.specify_by_class);

	sprintf(string, "%d", ran_clus_params.ncenters);
	XtFree(ran_clus_params.ncenters_string);
	ran_clus_params.ncenters_string = XtNewString(string);
	show_string_text(Ran_clusClone->ncentersText,
			 ran_clus_params.ncenters_string);

	XtFree(ran_clus_params.ncentersList_string);
	ran_clus_params.ncentersList_string =
	  make_int_string(ran_clus_params.ncentersList);
	show_string_text(Ran_clusClone->ncentersListText,
			 ran_clus_params.ncentersList_string);

	ran_clus_show_ncenters_choices();
	ran_clus_set_total_ncenters();

    } else {
	fprintf (stderr, "ran_clus_request: unknown request: %s\n", request);
    }
}

String
ran_clus_get_plot_label(String string)
{
    char intchars[10];
    sprintf(intchars, "%d", ran_clus_params.total_ncenters);
    string = StringConcat(string, " Centers:");
    string = StringConcat(string, intchars);
    return string;
}

static ARG ran_clus_tab2[] =
{
    {"specify_by_class", BOOLEAN, (int *)NULL,
       "use ncenters list for nclusters by class"},
};
#define RAN_CLUS_TABSIZE2 (sizeof(ran_clus_tab2)/sizeof(ARG))

static void
ran_clus_set_read_variables()
{
    ran_clus_params.cluster_by_class = cluster_by_class;
    ran_clus_params.ncenters = ncenters;
    if(ran_clus_params.ncentersList != NULL){
	deallocate_ivector(ran_clus_params.ncentersList);
    }
    ran_clus_params.ncentersList = ncenters_list;
    ncenters_list = NULL;
}

static void
ran_clus_set_write_variables()
{
    cluster_by_class = ran_clus_params.cluster_by_class;
    ncenters = ran_clus_params.ncenters;
    ncenters_list = ran_clus_params.ncentersList;
}

static void
ran_clus_set_defaults_variables()
{
    nnargs_set_variable("specify_by_class", &ran_clus_params.specify_by_class,
			ran_clus_tab2, RAN_CLUS_TABSIZE2);
}
void
read_ran_clus_defaults(int argc, char **argv)
{
    cluster_by_class = 0;
    ncenters_list = NULL;/* the space for this list is always
			    owned by the ran_clus_params structure */
    ran_clus_params.specify_by_class = 0;

    ran_clus_set_defaults_variables();
    if(argc > 0){
	argc = nnargs_part(argc, argv, ran_clus_tab2, RAN_CLUS_TABSIZE2);
	argc = nnargs(argc, argv, ran_clus_tab, RAN_CLUS_TABSIZE);
	ran_clus_set_read_variables();
    } else {
	ran_clus_params.ncenters = 16;
	ran_clus_params.specify_by_class = 0;
    }
    ran_clus_request("display");
}

void
write_ran_clus_defaults(default_file)
     FILE *default_file;
{
    ran_clus_set_write_variables();
    ran_clus_set_defaults_variables();
    if(default_file != NULL){
	fprintf(default_file, "ran_clus %s",
		nnargs_get_part_flags(ran_clus_tab, RAN_CLUS_TABSIZE));
	fprintf(default_file, "%s",
		nnargs_get_flags(ran_clus_tab2, RAN_CLUS_TABSIZE2));
    }
}

String
ran_clus_get_args ()
{
    ran_clus_set_write_variables();
    /* fix up ncenters list, in case it isn't wanted */
    if(ran_clus_params.cluster_by_class && ran_clus_params.specify_by_class){
	ncenters = 0;
    } else {
	ncenters_list = NULL;
    }
    return (nnargs_get_part_flags(ran_clus_tab, RAN_CLUS_TABSIZE));
}

String
ran_clus_get_important_args ()
{
    ran_clus_set_write_variables();
    /* fix up ncenters list, in case it isn't wanted */
    if(ran_clus_params.specify_by_class){
	ncenters = 0;
    } else {
	ncenters_list = NULL;
    }
    return (nnargs_get_flags_one_line(ran_clus_tab, RAN_CLUS_TABSIZE));
}

void
ran_clus_set_total_ncenters()
{
    static char total_label[64];

    if(ran_clus_params.cluster_by_class){
	if(ran_clus_params.specify_by_class &&
	   ran_clus_params.ncentersList != NULL){
	    int i;
	    for(i = 1, ran_clus_params.total_ncenters = 0;
		i <= ran_clus_params.ncentersList[0]; i++){
		ran_clus_params.total_ncenters += ran_clus_params.ncentersList[i];
	    }
	} else {
	    ran_clus_params.total_ncenters =
	      data_params.noutputs * ran_clus_params.ncenters;
	}
    } else {
	ran_clus_params.total_ncenters = ran_clus_params.ncenters;
    }
    sprintf(total_label, "This will create a total of %d centers.",
	    ran_clus_params.total_ncenters);
    show_string_label(Ran_clusClone->totalNcentersLabel, total_label);
}

static void
ran_clus_show_ncenters_choices()
{
    if(ran_clus_params.cluster_by_class){
	gray_out_item(Ran_clusClone->ncentersByClassRadioBox, FALSE);
	if(ran_clus_params.specify_by_class){
	    gray_out_item(Ran_clusClone->ncentersListForm, FALSE);
	    gray_out_item(Ran_clusClone->ncentersForm, TRUE);
	} else {
	    gray_out_item(Ran_clusClone->ncentersListForm, TRUE);
	    gray_out_item(Ran_clusClone->ncentersForm, FALSE);
	}
    } else {
	gray_out_item(Ran_clusClone->ncentersByClassRadioBox, TRUE);
	gray_out_item(Ran_clusClone->ncentersListForm, TRUE);
	gray_out_item(Ran_clusClone->ncentersForm, FALSE);
    }
}
