/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/rbf/rbf_callback.c,v 11.4 2001/08/23 20:17:42 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                                 RBF                                  ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  7-Nov-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include "../main/lnknet_subs.h"
#include "rbf.h"
#include <nnlib.h>
#include <rbf_args.h>
/*
 * External Functions
 */
#include "../plots/plots_func.h"
#include "../kmeans/kmeans_func.h"
#include "../em_clus/em_clus_func.h"
#include "../lead_clus/lead_clus_func.h"
#include "../ran_clus/ran_clus_func.h"

/*
 * Externals
 */
extern RbfWidgets *RbfClone;
#include "rbf_params.h"
RbfParams rbf_params; /* this is the real one */
#include "../lnk/lnk_params.h"
LnkParams lnk_params;

/*
 * Statics
 */
static void rbf_set_clustering_alg(int);
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: rbf_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   rbf_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  7-Nov-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
rbf_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    RbfWidgets *ip;
    int hSpread_buttons_mapped = False;
    int extraHspread_buttons_mapped = False;
    int fastNhidden_buttons_mapped = False;
    int maxRatio_buttons_mapped = False;
    int minVar_buttons_mapped = False;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "createClustersToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "rbf_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	rbf_params.createClusters = state->set;
 
    } else if (!strcmp(client, "kmeansPBtn")) {
	/*fprintf (stderr, "rbf_callback: %s\n", client );*/
	rbf_set_clustering_alg(KMEANS);

    } else if (!strcmp(client, "em_clusPBtn")) {
	/*fprintf (stderr, "rbf_callback: %s\n", client );*/
	rbf_set_clustering_alg(EM_CLUS);
 
    } else if (!strcmp(client, "lead_clusPBtn")) {
	/*fprintf (stderr, "rbf_callback: %s\n", client );*/
	rbf_set_clustering_alg(LEAD_CLUS);
 
    } else if (!strcmp(client, "ran_clusPBtn")) {
	/*fprintf (stderr, "rbf_callback: %s\n", client );*/
	rbf_set_clustering_alg(RAN_CLUS);
 
    } else if (!strcmp(client, "clustParamsPBtn")) {
	/*fprintf (stderr, "rbf_callback: %s\n", client );*/
	switch(rbf_params.clusterNum){
	  case KMEANS:
	    kmeans_request("open");
	    break;
	  case EM_CLUS:
	    em_clus_request("open");
	    break;
	  case LEAD_CLUS:
	    lead_clus_request("open");
	    break;
	  case RAN_CLUS:
	    ran_clus_request("open");
	    break;
	}
 
    } else if (!strcmp(client, "hSpreadText")) {
	/*fprintf (stderr, "rbf_callback: %s\n", client );*/
	hSpread_buttons_mapped =
	  okTextValueChanged(ip->hSpreadText, ip->hSpreadOKPBtn,
			     ip->hSpreadCancelPBtn,
			     ip->rbfDialogShell,
			     hSpread_buttons_mapped,
			     rbf_params.hSpread_string);
	
    } else if ((!strcmp(client, "hSpreadTextAct")) ||
	       (!strcmp(client, "hSpreadOKPBtn"))) {
	String hSpreadTextval;

	/*fprintf (stderr, "rbf_callback: %s\n", client );*/
	hSpread_buttons_mapped =
	  okTextActivate(ip->hSpreadText, ip->hSpreadOKPBtn,
			 ip->hSpreadCancelPBtn, &hSpreadTextval);
	/* I won't actually be using this copy */
	XtFree (hSpreadTextval);
	
	/* verify the string as a float */
	rbf_params.hSpread =
	  set_atof_low_value(ip->hSpreadText, "hSpread_set",
			     rbf_params.hSpread, 0.);

	/* save the current text for future comparisons */
	XtFree(rbf_params.hSpread_string);
	rbf_params.hSpread_string = get_string_text(ip->hSpreadText);
 
	/* do any other things needed */
	set_plot_labels();

    } else if (!strcmp(client, "hSpreadCancelPBtn")) {
	/*fprintf (stderr, "rbf_callback: %s\n", client );*/
	hSpread_buttons_mapped = 
	  okTextCancel(ip->hSpreadText, ip->hSpreadOKPBtn,
		       ip->hSpreadCancelPBtn,
		       rbf_params.hSpread_string);
 
    } else if (!strcmp(client, "extraHspreadText")) {
	/*fprintf (stderr, "rbf_callback: %s\n", client );*/
	extraHspread_buttons_mapped =
	  okTextValueChanged(ip->extraHspreadText, ip->extraHspreadOKPBtn,
			     ip->extraHspreadCancelPBtn,
			     ip->rbfDialogShell,
			     extraHspread_buttons_mapped,
			     rbf_params.extraHspread_string);
	
    } else if ((!strcmp(client, "extraHspreadTextAct")) ||
	       (!strcmp(client, "extraHspreadOKPBtn"))) {
	String extraHspreadTextval;

	/*fprintf (stderr, "rbf_callback: %s\n", client );*/
	extraHspread_buttons_mapped =
	  okTextActivate(ip->extraHspreadText, ip->extraHspreadOKPBtn,
			 ip->extraHspreadCancelPBtn, &extraHspreadTextval);
	/* I won't actually be using this copy */
	XtFree (extraHspreadTextval);
	
	/* verify the string as a float */
	rbf_params.extraHspread =
	  set_atof_low_value(ip->extraHspreadText, "extraHspread_set",
			     rbf_params.extraHspread, 0.);

	/* save the current text for future comparisons */
	XtFree(rbf_params.extraHspread_string);
	rbf_params.extraHspread_string = get_string_text(ip->extraHspreadText);
 
	/* do any other things needed */
	set_plot_labels();

    } else if (!strcmp(client, "extraHspreadCancelPBtn")) {
	/*fprintf (stderr, "rbf_callback: %s\n", client );*/
	extraHspread_buttons_mapped = 
	  okTextCancel(ip->extraHspreadText, ip->extraHspreadOKPBtn,
		       ip->extraHspreadCancelPBtn,
		       rbf_params.extraHspread_string);
 
    } else if (!strcmp(client, "fastTrainToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "rbf_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	rbf_params.fastTrain = state->set;
	gray_out_item(ip->fastNhiddenForm, !rbf_params.fastTrain);
 
    } else if (!strcmp(client, "fastNhiddenText")) {
	/*fprintf (stderr, "rbf_callback: %s\n", client );*/
	fastNhidden_buttons_mapped =
	  okTextValueChanged(ip->fastNhiddenText, ip->fastNhiddenOKPBtn,
			     ip->fastNhiddenCancelPBtn,
			     ip->rbfDialogShell,
			     fastNhidden_buttons_mapped,
			     rbf_params.fastNhidden_string);
	
    } else if ((!strcmp(client, "fastNhiddenTextAct")) ||
	       (!strcmp(client, "fastNhiddenOKPBtn"))) {
	String fastNhiddenTextval;

	/*fprintf (stderr, "rbf_callback: %s\n", client );*/
	fastNhidden_buttons_mapped =
	  okTextActivate(ip->fastNhiddenText, ip->fastNhiddenOKPBtn,
			 ip->fastNhiddenCancelPBtn, &fastNhiddenTextval);
	/* I won't actually be using this copy */
	XtFree (fastNhiddenTextval);
	
	/* verify the string as an integer */
	rbf_params.fastNhidden =
	  set_atoi_value(ip->fastNhiddenText, "fastNhidden_set",
			     rbf_params.fastNhidden);

	/* save the current text for future comparisons */
	XtFree(rbf_params.fastNhidden_string);
	rbf_params.fastNhidden_string = get_string_text(ip->fastNhiddenText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "fastNhiddenCancelPBtn")) {
	/*fprintf (stderr, "rbf_callback: %s\n", client );*/
	fastNhidden_buttons_mapped = 
	  okTextCancel(ip->fastNhiddenText, ip->fastNhiddenOKPBtn,
		       ip->fastNhiddenCancelPBtn,
		       rbf_params.fastNhidden_string);
 
    } else if (!strcmp(client, "biasToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "rbf_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	rbf_params.bias = state->set;
	set_plot_labels();
 
    } else if (!strcmp(client, "maxRatioText")) {
	/*fprintf (stderr, "rbf_callback: %s\n", client );*/
	maxRatio_buttons_mapped =
	  okTextValueChanged(ip->maxRatioText, ip->maxRatioOKPBtn,
			     ip->maxRatioCancelPBtn,
			     ip->rbfDialogShell,
			     maxRatio_buttons_mapped,
			     rbf_params.maxRatio_string);
	
    } else if ((!strcmp(client, "maxRatioTextAct")) ||
	       (!strcmp(client, "maxRatioOKPBtn"))) {
	String maxRatioTextval;

	/*fprintf (stderr, "rbf_callback: %s\n", client );*/
	maxRatio_buttons_mapped =
	  okTextActivate(ip->maxRatioText, ip->maxRatioOKPBtn,
			 ip->maxRatioCancelPBtn, &maxRatioTextval);
	/* I won't actually be using this copy */
	XtFree (maxRatioTextval);
	
	/* verify the string as a float */
	rbf_params.maxRatio =
	  set_atof_low_value(ip->maxRatioText, "maxRatio_set",
			     rbf_params.maxRatio, 1.);

	/* save the current text for future comparisons */
	XtFree(rbf_params.maxRatio_string);
	rbf_params.maxRatio_string = get_string_text(ip->maxRatioText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "maxRatioCancelPBtn")) {
	/*fprintf (stderr, "rbf_callback: %s\n", client );*/
	maxRatio_buttons_mapped = 
	  okTextCancel(ip->maxRatioText, ip->maxRatioOKPBtn,
		       ip->maxRatioCancelPBtn,
		       rbf_params.maxRatio_string);
 
    } else if (!strcmp(client, "minVarText")) {
	/*fprintf (stderr, "rbf_callback: %s\n", client );*/
	minVar_buttons_mapped =
	  okTextValueChanged(ip->minVarText, ip->minVarOKPBtn,
			     ip->minVarCancelPBtn,
			     ip->rbfDialogShell,
			     minVar_buttons_mapped,
			     rbf_params.minVar_string);
	
    } else if ((!strcmp(client, "minVarTextAct")) ||
	       (!strcmp(client, "minVarOKPBtn"))) {
	String minVarTextval;

	/*fprintf (stderr, "rbf_callback: %s\n", client );*/
	minVar_buttons_mapped =
	  okTextActivate(ip->minVarText, ip->minVarOKPBtn,
			 ip->minVarCancelPBtn, &minVarTextval);
	/* I won't actually be using this copy */
	XtFree (minVarTextval);
	
	/* verify the string as a float */
	rbf_params.minVar =
	  set_atof_low_value(ip->minVarText, "minVar_set",
			     rbf_params.minVar, 0.);

	/* save the current text for future comparisons */
	XtFree(rbf_params.minVar_string);
	rbf_params.minVar_string = get_string_text(ip->minVarText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "minVarCancelPBtn")) {
	/*fprintf (stderr, "rbf_callback: %s\n", client );*/
	minVar_buttons_mapped = 
	  okTextCancel(ip->minVarText, ip->minVarOKPBtn,
		       ip->minVarCancelPBtn,
		       rbf_params.minVar_string);
 
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "rbf_callback: %s\n", client );*/
	close_popup(ip->rbfForm);
 
    } else {
	fprintf(stderr, "rbf_callback: unknown client: %s\n", (char *)client);
    }
}

void
rbf_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(RbfClone->rbfForm);
    } else if (!strcmp(request, "init")){
	rbf_params.createClusters = 0;
	rbf_params.clusterNum = KMEANS;
	rbf_params.clustName = "kmeans";
	rbf_params.clustFname = NULL;
	rbf_params.hSpread = 0.;
	rbf_params.hSpread_string = NULL;
	rbf_params.extraHspread = 0.;
	rbf_params.extraHspread_string = NULL;
	rbf_params.fastTrain = 0;
	rbf_params.fastNhidden = 0;
	rbf_params.fastNhidden_string = NULL;
	rbf_params.bias = 0;
	rbf_params.maxRatio = 0.;
	rbf_params.maxRatio_string = NULL;
	rbf_params.minVar = 0.;
	rbf_params.minVar_string = NULL;
    } else if (!strcmp(request, "display")){
	char string[10];
	show_toggle_value(RbfClone->createClustersToggle,
			  rbf_params.createClusters);
	switch(rbf_params.clusterNum){
	  case KMEANS:
	    show_option_value(RbfClone->clustAlgOptionMenu, 0);
	    break;
	  case EM_CLUS:
	    show_option_value(RbfClone->clustAlgOptionMenu, 1);
	    break;
	  case LEAD_CLUS:
	    show_option_value(RbfClone->clustAlgOptionMenu, 2);
	    break;
	  case RAN_CLUS:
	    show_option_value(RbfClone->clustAlgOptionMenu, 3);
	    break;
	}
	show_string_label(RbfClone->clustFnameLabel, rbf_params.clustFname);

	XtFree(rbf_params.hSpread_string);
	sprintf(string, "%5.2g", rbf_params.hSpread);
	rbf_params.hSpread_string = XtNewString(string);
	show_string_text(RbfClone->hSpreadText,
			 rbf_params.hSpread_string);
	XtFree(rbf_params.extraHspread_string);
	sprintf(string, "%5.2g", rbf_params.extraHspread);
	rbf_params.extraHspread_string = XtNewString(string);
	show_string_text(RbfClone->extraHspreadText,
			 rbf_params.extraHspread_string);

	show_toggle_value(RbfClone->fastTrainToggle, rbf_params.fastTrain);
	gray_out_item(RbfClone->fastNhiddenForm, !rbf_params.fastTrain);
	XtFree(rbf_params.fastNhidden_string);
	sprintf(string, "%d", rbf_params.fastNhidden);
	rbf_params.fastNhidden_string = XtNewString(string);
	show_string_text(RbfClone->fastNhiddenText,
			 rbf_params.fastNhidden_string);

	show_toggle_value(RbfClone->biasToggle, rbf_params.bias);

	XtFree(rbf_params.maxRatio_string);
	sprintf(string, "%5.2g", rbf_params.maxRatio);
	rbf_params.maxRatio_string = XtNewString(string);
	show_string_text(RbfClone->maxRatioText,
			 rbf_params.maxRatio_string);
	XtFree(rbf_params.minVar_string);
	sprintf(string, "%5.2g", rbf_params.minVar);
	rbf_params.minVar_string = XtNewString(string);
	show_string_text(RbfClone->minVarText,
			 rbf_params.minVar_string);
    } else {
	fprintf(stderr, "rbf_request: unknown request: %s\n", request);
    }
}

String
rbf_get_plot_label(String string)
{
    char numString[10];
    string = StringConcat(string, "H:");
    sprintf(numString, "%g", rbf_params.hSpread);
    string = StringConcat(string, numString);
    string = StringConcat(string, " Extra H:");
    sprintf(numString, "%g", rbf_params.extraHspread);
    string = StringConcat(string, numString);
    if(rbf_params.bias) string = StringConcat(string, " Bias");
    switch(rbf_params.clusterNum){
      case KMEANS:
	string = kmeans_get_plot_label(string);
	break;
      case EM_CLUS:
	string = em_clus_get_plot_label(string);
	break;
      case LEAD_CLUS:
	string = lead_clus_get_plot_label(string);
	break;
      case RAN_CLUS:
	string = ran_clus_get_plot_label(string);
	break;
    }
    return string;
}

static void
rbf_set_read_variables()
{
    XtFree(rbf_params.clustFname);
    rbf_params.clustFname = XtNewString(cluster_param_fname);
    rbf_params.hSpread = hspread;
    rbf_params.extraHspread = extra_hspread;
    rbf_params.fastTrain = fast_train;
    rbf_params.fastNhidden = fast_nhidden;
    rbf_params.bias = bias;
    rbf_params.maxRatio = max_ratio;
    rbf_params.minVar = minimum_variance;
}

static void
rbf_set_write_variables()
{
    cluster_param_fname = rbf_params.clustFname;
    hspread = rbf_params.hSpread;
    extra_hspread = rbf_params.extraHspread;
    fast_train = rbf_params.fastTrain;
    fast_nhidden = rbf_params.fastNhidden;
    bias = rbf_params.bias;
    max_ratio = rbf_params.maxRatio;
    minimum_variance = rbf_params.minVar;
}
static ARG rbftab[] =
{
    {"createClusters", BOOLEAN, (int *)NULL,},
    {"clusterNum", INT, (int *)NULL,},
};
#define TABSIZE (sizeof(rbftab)/sizeof(ARG))

static void
rbf_set_defaults_variables(dp)
     RbfParams *dp;
{
    nnargs_set_variable("createClusters", (int *)&dp->createClusters, rbftab, TABSIZE);
    nnargs_set_variable("clusterNum", (int *)&dp->clusterNum, rbftab, TABSIZE);
}

void
read_rbf_defaults(argc, argv)
     int argc;
     char **argv;
{
    rbf_params.createClusters = 0;
    fast_train = 0;
    bias = 0;
    rbf_set_defaults_variables(&rbf_params);
    if(argc > 0){
	argc = nnargs_part(argc, argv, rbftab, TABSIZE);
	argc = nnargs(argc, argv, rbf_tab, RBF_TABSIZE);
	rbf_set_read_variables();
	switch(rbf_params.clusterNum){
	  case KMEANS:
	    rbf_params.clustName = "kmeans";
	    break;
	  case EM_CLUS:
	    rbf_params.clustName = "em_clus";
	    break;
	  case LEAD_CLUS:
	    rbf_params.clustName = "lead_clus";
	    break;
	  case RAN_CLUS:
	    rbf_params.clustName = "ran_clus";
	    break;
	}
    } else {
	/* set any defaults */
	rbf_params.createClusters = 1;
	rbf_params.clusterNum = KMEANS;
	rbf_params.hSpread = 1;
	rbf_params.extraHspread = 1;
	rbf_params.fastTrain = 0;
	rbf_params.fastNhidden = 0;
	rbf_params.maxRatio = 1e6;
	rbf_params.minVar = 1e-6;
    }

    rbf_request("display");
}

void
write_rbf_defaults(FILE *default_file)
{
    rbf_set_write_variables();
    rbf_set_defaults_variables(&rbf_params);

    if(default_file != NULL){
	fprintf(default_file, "rbf %s",
		nnargs_get_part_flags(rbftab, TABSIZE));
	fprintf(default_file, "rbf %s",
		nnargs_get_flags(rbf_tab, RBF_TABSIZE));
    }
}

String
rbf_get_args ()
{
    rbf_set_write_variables();
    return (nnargs_get_part_flags(rbf_tab, RBF_TABSIZE));
}

String
rbf_get_important_args ()
{
    rbf_set_write_variables();
    return (nnargs_get_flags_one_line(rbf_tab, RBF_TABSIZE));
}

static void
rbf_set_clustering_alg(value)
     int value;
{
    rbf_params.clusterNum = value;
    switch(rbf_params.clusterNum){
      case KMEANS:
	rbf_params.clustName = "kmeans";
	break;
      case EM_CLUS:
	rbf_params.clustName = "em_clus";
	break;
      case LEAD_CLUS:
	rbf_params.clustName = "lead_clus";
	break;
      case RAN_CLUS:
	rbf_params.clustName = "ran_clus";
	break;
    }
    XtFree(rbf_params.clustFname);
    rbf_params.clustFname = XtNewString(lnk_params.exp_prefix);
    rbf_params.clustFname = StringConcat(rbf_params.clustFname,
					 rbf_params.clustName);
    rbf_params.clustFname = StringConcat(rbf_params.clustFname,
					 ".param");
    show_string_label(RbfClone->clustFnameLabel, rbf_params.clustFname);
    set_plot_labels();
}

void
rbf_set_cluster_param_fname()
{
    if(rbf_params.createClusters){
	XtFree(rbf_params.clustFname);
	rbf_params.clustFname = XtNewString(lnk_params.exp_prefix);
	rbf_params.clustFname = StringConcat(rbf_params.clustFname,
					     rbf_params.clustName);
	rbf_params.clustFname = StringConcat(rbf_params.clustFname,
					     ".param");
	show_string_label(RbfClone->clustFnameLabel, rbf_params.clustFname);
    }
}

void
rbf_store_plot_command(FILE * shell_file)
{
    fprintf(shell_file, " -exhspread %g", rbf_params.extraHspread);
}

char *
rbf_get_cluster_fname(){
    return rbf_params.clustFname;
}

int
rbf_get_cluster_alg(){
    return rbf_params.clusterNum;
}

int
rbf_must_store_clustering(){
    return rbf_params.createClusters;
}

