/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/region/region_callback.c,v 11.4 2001/08/23 20:17:46 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            REGION                                    ****
 ****                          Region Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  27-Oct-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include <nnlib.h>
#include "region.h"
/*
 * External Functions
 */
#include "region_func.h"
#include "../main/lnknet_subs.h"
#include "../lnk/lnk_func.h"
#include "../plots/plots_func.h"
#include "../print/print_func.h"
 
/*
 * Externals
 */
extern RegionWidgets *RegionClone;
#include "region_params.h"
RegionParams region_params; /* this is the real one */
#include "../data/data_params.h"
extern DataParams data_params;/* for data base path and name and extensions */
#include "../plots/plots_params.h"
extern PlotsParams plots_params;
#include "../lnk/lnk_params.h"
extern LnkParams lnk_params;
#include "../reports/reports_params.h"
extern ReportsParams reports_params;
#include "../priors/priors_params.h"
extern PriorsParams priors_params;
#include "../select/select_params.h"
extern SelectParams select_params;
#include "../norm/norm_params.h"
extern NormParams norm_params;

/* variables for shell scripts */
#define PLOT_BOUND "_plot_bound"
#include <plot_bound_args.h>

/*
 * Statics 
 */
static void region_set_read_variables(void);
static void region_set_write_variables(void);
static void region_set_common_flags(TEST_TYPE);
static void create_region_plot_fname(TEST_TYPE);
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: region_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   region_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  27-Oct-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
region_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    RegionWidgets *ip;
    static int xDimen_buttons_mapped = False;
    static int yDimen_buttons_mapped = False;
    static int otherVar_buttons_mapped = False;
    static int nPoints_buttons_mapped = False;
    static int xmin_buttons_mapped = False;
    static int xmax_buttons_mapped = False;
    static int xstep_buttons_mapped = False;
    static int ymin_buttons_mapped = False;
    static int ymax_buttons_mapped = False;
    static int ystep_buttons_mapped = False;
    static int distLim_buttons_mapped = False;
    static int internalsLevel_buttons_mapped = False;
    static int internalsScale_buttons_mapped = False;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "xDimenText")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	xDimen_buttons_mapped =
	  okTextValueChanged(ip->xDimenText, ip->xDimenOKPBtn,
			     ip->xDimenCancelPBtn,
			     ip->regionDialogShell,
			     xDimen_buttons_mapped,
			     region_params.xDimen_string);
	
    } else if ((!strcmp(client, "xDimenTextAct")) ||
	       (!strcmp(client, "xDimenOKPBtn"))) {
	String xDimenTextval;

	/*fprintf (stderr, "region_callback: %s\n", client );*/
	xDimen_buttons_mapped =
	  okTextActivate(ip->xDimenText, ip->xDimenOKPBtn,
			 ip->xDimenCancelPBtn, &xDimenTextval);
	/* I won't actually be using this copy */
	XtFree (xDimenTextval);
	
	/* verify the string as an integer */
	region_params.xDimen =
	  set_atoi_value(ip->xDimenText, "xDimen_set",
			     region_params.xDimen);

	/* save the current text for future comparisons */
	XtFree(region_params.xDimen_string);
	region_params.xDimen_string = get_string_text(ip->xDimenText);
 
	/* do any other things needed */
	region_set_input_label();

    } else if (!strcmp(client, "xDimenCancelPBtn")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	xDimen_buttons_mapped = 
	  okTextCancel(ip->xDimenText, ip->xDimenOKPBtn,
		       ip->xDimenCancelPBtn,
		       region_params.xDimen_string);
	
    } else if (!strcmp(client, "yDimenText")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	yDimen_buttons_mapped =
	  okTextValueChanged(ip->yDimenText, ip->yDimenOKPBtn,
			     ip->yDimenCancelPBtn,
			     ip->regionDialogShell,
			     yDimen_buttons_mapped,
			     region_params.yDimen_string);
	
    } else if ((!strcmp(client, "yDimenTextAct")) ||
	       (!strcmp(client, "yDimenOKPBtn"))) {
	String yDimenTextval;

	/*fprintf (stderr, "region_callback: %s\n", client );*/
	yDimen_buttons_mapped =
	  okTextActivate(ip->yDimenText, ip->yDimenOKPBtn,
			 ip->yDimenCancelPBtn, &yDimenTextval);
	/* I won't actually be using this copy */
	XtFree (yDimenTextval);
	
	/* verify the string as an integer */
	region_params.yDimen =
	  set_atoi_value(ip->yDimenText, "yDimen_set",
			     region_params.yDimen);

	/* save the current text for future comparisons */
	XtFree(region_params.yDimen_string);
	region_params.yDimen_string = get_string_text(ip->yDimenText);
 
	/* do any other things needed */
	region_set_input_label();

    } else if (!strcmp(client, "yDimenCancelPBtn")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	yDimen_buttons_mapped = 
	  okTextCancel(ip->yDimenText, ip->yDimenOKPBtn,
		       ip->yDimenCancelPBtn,
		       region_params.yDimen_string);

    } else if (!strcmp(client, "otherVarText")) {
	/*fprintf (stderr, "region_callback: %s\n", client);*/
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	otherVar_buttons_mapped =
	  okTextValueChanged(ip->otherVarText, ip->otherVarOKPBtn,
			     ip->otherVarCancelPBtn,
			     ip->regionDialogShell,
			     otherVar_buttons_mapped,
			     region_params.otherVar_string);
    } else if ((!strcmp(client, "otherVarTextAct")) ||
	       (!strcmp(client, "otherVarOKPBtn"))) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	String otherVarTextval;
	otherVar_buttons_mapped =
	  okTextActivate(ip->otherVarText,
			 ip->otherVarOKPBtn,
			 ip->otherVarCancelPBtn,
			 &otherVarTextval);
	
	if(region_params.otherVar != NULL)
	  deallocate_vector(region_params.otherVar);
	/* take spaces and tabs out of the list before handing it to nnargs */
	if(otherVarTextval != NULL){
	    char *otherVarTemp = otherVarTextval;
	    otherVarTextval = strtok(otherVarTextval, " \t");
	    region_params.otherVar =
	      nnargs_get_float_alloc_list(otherVarTextval);
	    XtFree(otherVarTemp);
	}
	
	/* make a copy of the string as it has been interpretted */
	XtFree(region_params.otherVar_string);
	region_params.otherVar_string =
	  make_float_string(region_params.otherVar);
	show_string_text(ip->otherVarText,
			 region_params.otherVar_string);
	
	/* do anything else necessary */
	
    } else if (!strcmp(client, "otherVarCancelPBtn")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	otherVar_buttons_mapped = 
	  okTextCancel(ip->otherVarText, ip->otherVarOKPBtn,
		       ip->otherVarCancelPBtn,
		       region_params.otherVar_string);

    } else if (!strcmp(client, "nPointsText")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	nPoints_buttons_mapped =
	  okTextValueChanged(ip->nPointsText, ip->nPointsOKPBtn,
			     ip->nPointsCancelPBtn,
			     ip->regionDialogShell,
			     nPoints_buttons_mapped,
			     region_params.nPoints_string);
	
    } else if ((!strcmp(client, "nPointsTextAct")) ||
	       (!strcmp(client, "nPointsOKPBtn"))) {
	String nPointsTextval;

	/*fprintf (stderr, "region_callback: %s\n", client );*/
	nPoints_buttons_mapped =
	  okTextActivate(ip->nPointsText, ip->nPointsOKPBtn,
			 ip->nPointsCancelPBtn, &nPointsTextval);
	/* I won't actually be using this copy */
	XtFree (nPointsTextval);
	
	/* verify the string as an integer */
	region_params.nPoints =
	  set_atoi_value(ip->nPointsText, "nPoints_set",
			     region_params.nPoints);

	/* save the current text for future comparisons */
	XtFree(region_params.nPoints_string);
	region_params.nPoints_string = get_string_text(ip->nPointsText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "nPointsCancelPBtn")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	nPoints_buttons_mapped = 
	  okTextCancel(ip->nPointsText, ip->nPointsOKPBtn,
		       ip->nPointsCancelPBtn,
		       region_params.nPoints_string);
	
    } else if (!strcmp(client, "autoscaleToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "region_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	region_params.autoscale = state->set;
	gray_out_item(RegionClone->axesForm, state->set);
 
    } else if (!strcmp(client, "xminText")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	xmin_buttons_mapped =
	  okTextValueChanged(ip->xminText, ip->xminOKPBtn,
			     ip->xminCancelPBtn,
			     ip->regionDialogShell,
			     xmin_buttons_mapped,
			     region_params.xmin_string);
      
    } else if (!strcmp(client, "xminTextAct") ||
	       !strcmp(client, "xminOKPBtn")) {
	String xminTextval;

	/*fprintf (stderr, "region_callback: %s\n", client );*/
	xmin_buttons_mapped = okTextActivate(ip->xminText, ip->xminOKPBtn,
					     ip->xminCancelPBtn, &xminTextval);

	XtFree (xminTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	region_params.xmin = set_atof_value(ip->xminText, "region_xmin",
					  region_params.xmin);
	XtFree(region_params.xmin_string);
	region_params.xmin_string = get_string_text(ip->xminText);
 
    } else if (!strcmp(client, "xminCancelPBtn")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	xmin_buttons_mapped = 
	  okTextCancel(ip->xminText, ip->xminOKPBtn,
		       ip->xminCancelPBtn,
		       region_params.xmin_string);
 
    } else if (!strcmp(client, "xmaxText")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	xmax_buttons_mapped =
	  okTextValueChanged(ip->xmaxText, ip->xmaxOKPBtn,
			     ip->xmaxCancelPBtn,
			     ip->regionDialogShell,
			     xmax_buttons_mapped,
			     region_params.xmax_string);
      
    } else if (!strcmp(client, "xmaxTextAct") ||
	       !strcmp(client, "xmaxOKPBtn")) {
	String xmaxTextval;

	/*fprintf (stderr, "region_callback: %s\n", client );*/
	xmax_buttons_mapped = okTextActivate(ip->xmaxText, ip->xmaxOKPBtn,
					     ip->xmaxCancelPBtn, &xmaxTextval);
	
	XtFree (xmaxTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	region_params.xmax = set_atof_value(ip->xmaxText, "region_xmax",
					  region_params.xmax);
	XtFree(region_params.xmax_string);
	region_params.xmax_string = get_string_text(ip->xmaxText);
 
    } else if (!strcmp(client, "xmaxCancelPBtn")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	xmax_buttons_mapped = 
	  okTextCancel(ip->xmaxText, ip->xmaxOKPBtn,
		       ip->xmaxCancelPBtn,
		       region_params.xmax_string);
 
    } else if (!strcmp(client, "xstepText")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	xstep_buttons_mapped =
	  okTextValueChanged(ip->xstepText, ip->xstepOKPBtn,
			     ip->xstepCancelPBtn,
			     ip->regionDialogShell,
			     xstep_buttons_mapped,
			     region_params.xstep_string);
      
    } else if (!strcmp(client, "xstepTextAct") ||
	       !strcmp(client, "xstepOKPBtn")) {
	String xstepTextval;
      
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	xstep_buttons_mapped =
	  okTextActivate(ip->xstepText, ip->xstepOKPBtn,
			 ip->xstepCancelPBtn, &xstepTextval);

	XtFree (xstepTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	region_params.xstep = set_atof_value(ip->xstepText, "region_xstep",
					   region_params.xstep);
	XtFree(region_params.xstep_string);
	region_params.xstep_string = get_string_text(ip->xstepText);
 
    } else if (!strcmp(client, "xstepCancelPBtn")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	xstep_buttons_mapped = 
	  okTextCancel(ip->xstepText, ip->xstepOKPBtn,
		       ip->xstepCancelPBtn,
		       region_params.xstep_string);
	
    } else if (!strcmp(client, "yminText")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	ymin_buttons_mapped =
	  okTextValueChanged(ip->yminText, ip->yminOKPBtn,
			     ip->yminCancelPBtn,
			     ip->regionDialogShell,
			     ymin_buttons_mapped,
			     region_params.ymin_string);
      
    } else if (!strcmp(client, "yminTextAct") ||
	       !strcmp(client, "yminOKPBtn")) {
	String yminTextval;
      
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	ymin_buttons_mapped = okTextActivate(ip->yminText, ip->yminOKPBtn,
					     ip->yminCancelPBtn, &yminTextval);

	XtFree (yminTextval);/* I won't actually be using this copy */
	
	/* check that this value is ok */
	region_params.ymin = set_atof_value(ip->yminText, "region_ymin",
					  region_params.ymin);
	XtFree(region_params.ymin_string);
	region_params.ymin_string = get_string_text(ip->yminText);
	
    } else if (!strcmp(client, "yminCancelPBtn")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	ymin_buttons_mapped = 
	  okTextCancel(ip->yminText, ip->yminOKPBtn,
		       ip->yminCancelPBtn,
		       region_params.ymin_string);
	
    } else if (!strcmp(client, "ymaxText")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	ymax_buttons_mapped =
	  okTextValueChanged(ip->ymaxText, ip->ymaxOKPBtn,
			     ip->ymaxCancelPBtn,
			     ip->regionDialogShell,
			     ymax_buttons_mapped,
			     region_params.ymax_string);
      
    } else if (!strcmp(client, "ymaxTextAct") ||
	       !strcmp(client, "ymaxOKPBtn")) {
	String ymaxTextval;
      
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	ymax_buttons_mapped = okTextActivate(ip->ymaxText, ip->ymaxOKPBtn,
					     ip->ymaxCancelPBtn, &ymaxTextval);

	XtFree (ymaxTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	region_params.ymax = set_atof_value(ip->ymaxText, "region_ymax",
					  region_params.ymax);
	XtFree(region_params.ymax_string);
	region_params.ymax_string = get_string_text(ip->ymaxText);
 
    } else if (!strcmp(client, "ymaxCancelPBtn")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	ymax_buttons_mapped = 
	  okTextCancel(ip->ymaxText, ip->ymaxOKPBtn,
		       ip->ymaxCancelPBtn,
		       region_params.ymax_string);
 
    } else if (!strcmp(client, "ystepText")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	ystep_buttons_mapped =
	  okTextValueChanged(ip->ystepText, ip->ystepOKPBtn,
			     ip->ystepCancelPBtn,
			     ip->regionDialogShell,
			     ystep_buttons_mapped,
			     region_params.ystep_string);
      
    } else if (!strcmp(client, "ystepTextAct") ||
	       !strcmp(client, "ystepOKPBtn")) {
	String ystepTextval;
      
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	ystep_buttons_mapped =
	  okTextActivate(ip->ystepText, ip->ystepOKPBtn,
			 ip->ystepCancelPBtn, &ystepTextval);

	XtFree (ystepTextval);/* I won't actually be using this copy */
	
	/* check that this value is ok */
	region_params.ystep = set_atof_value(ip->ystepText, "region_ystep",
					   region_params.ystep);
	XtFree(region_params.ystep_string);
	region_params.ystep_string = get_string_text(ip->ystepText);
 
    } else if (!strcmp(client, "ystepCancelPBtn")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	ystep_buttons_mapped = 
	  okTextCancel(ip->ystepText, ip->ystepOKPBtn,
		       ip->ystepCancelPBtn,
		       region_params.ystep_string);
	
    } else if (!strcmp(client, "noNormToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "region_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	region_params.noNorm = state->set;
	region_set_input_label();
	
    } else if (!strcmp(client, "colorToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "region_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	region_params.color = state->set;
	
    } else if (!strcmp(client, "showMissesToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "region_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	region_params.showMisses = state->set;
	
    } else if (!strcmp(client, "showAllToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "region_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	region_params.showAll = state->set;
	gray_out_item(ip->distLimForm, region_params.showAll);
	
    } else if (!strcmp(client, "distLimText")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	distLim_buttons_mapped =
	  okTextValueChanged(ip->distLimText, ip->distLimOKPBtn,
			     ip->distLimCancelPBtn,
			     ip->regionDialogShell,
			     distLim_buttons_mapped,
			     region_params.distLim_string);
	
    } else if ((!strcmp(client, "distLimTextAct")) ||
	       (!strcmp(client, "distLimOKPBtn"))) {
	String distLimTextval;

	/*fprintf (stderr, "region_callback: %s\n", client );*/
	distLim_buttons_mapped =
	  okTextActivate(ip->distLimText, ip->distLimOKPBtn,
			 ip->distLimCancelPBtn, &distLimTextval);
	/* I won't actually be using this copy */
	XtFree (distLimTextval);
	
	/* verify the string as a float */
	region_params.distLim =
	  set_atof_value(ip->distLimText, "distLim_set",
			 region_params.distLim);

	/* save the current text for future comparisons */
	XtFree(region_params.distLim_string);
	region_params.distLim_string = get_string_text(ip->distLimText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "distLimCancelPBtn")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	distLim_buttons_mapped = 
	  okTextCancel(ip->distLimText, ip->distLimOKPBtn,
		       ip->distLimCancelPBtn,
		       region_params.distLim_string);
	
    } else if (!strcmp(client, "internalsLevelText")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	internalsLevel_buttons_mapped =
	  okTextValueChanged(ip->internalsLevelText, ip->internalsLevelOKPBtn,
			     ip->internalsLevelCancelPBtn,
			     ip->regionDialogShell,
			     internalsLevel_buttons_mapped,
			     region_params.internalsLevel_string);
	
    } else if ((!strcmp(client, "internalsLevelTextAct")) ||
	       (!strcmp(client, "internalsLevelOKPBtn"))) {
	String internalsLevelTextval;

	/*fprintf (stderr, "region_callback: %s\n", client );*/
	internalsLevel_buttons_mapped =
	  okTextActivate(ip->internalsLevelText, ip->internalsLevelOKPBtn,
			 ip->internalsLevelCancelPBtn, &internalsLevelTextval);
	/* I won't actually be using this copy */
	XtFree (internalsLevelTextval);
	
	/* verify the string as an integer */
	region_params.internalsLevel =
	  set_atoi_value(ip->internalsLevelText, "internalsLevel_set",
			     region_params.internalsLevel);

	/* save the current text for future comparisons */
	XtFree(region_params.internalsLevel_string);
	region_params.internalsLevel_string = get_string_text(ip->internalsLevelText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "internalsLevelCancelPBtn")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	internalsLevel_buttons_mapped = 
	  okTextCancel(ip->internalsLevelText, ip->internalsLevelOKPBtn,
		       ip->internalsLevelCancelPBtn,
		       region_params.internalsLevel_string);
	
    } else if (!strcmp(client, "internalsScaleText")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	internalsScale_buttons_mapped =
	  okTextValueChanged(ip->internalsScaleText, ip->internalsScaleOKPBtn,
			     ip->internalsScaleCancelPBtn,
			     ip->regionDialogShell,
			     internalsScale_buttons_mapped,
			     region_params.internalsScale_string);
	
    } else if ((!strcmp(client, "internalsScaleTextAct")) ||
	       (!strcmp(client, "internalsScaleOKPBtn"))) {
	String internalsScaleTextval;

	/*fprintf (stderr, "region_callback: %s\n", client );*/
	internalsScale_buttons_mapped =
	  okTextActivate(ip->internalsScaleText, ip->internalsScaleOKPBtn,
			 ip->internalsScaleCancelPBtn, &internalsScaleTextval);
	/* I won't actually be using this copy */
	XtFree (internalsScaleTextval);
	
	/* verify the string as a float */
	region_params.internalsScale =
	  set_atof_value(ip->internalsScaleText, "internalsScale_set",
			 region_params.internalsScale);

	/* save the current text for future comparisons */
	XtFree(region_params.internalsScale_string);
	region_params.internalsScale_string = get_string_text(ip->internalsScaleText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "internalsScaleCancelPBtn")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	internalsScale_buttons_mapped = 
	  okTextCancel(ip->internalsScaleText, ip->internalsScaleOKPBtn,
		       ip->internalsScaleCancelPBtn,
		       region_params.internalsScale_string);
	
	
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "region_callback: %s\n", client );*/
	close_popup(RegionClone->regionForm);
	
    } else {
	fprintf(stderr, "region_callback: unknown client: %s\n", (char *)client);
    }
}

void
region_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(RegionClone->regionForm);
    } else if (!strcmp(request, "init")){
	region_params.xDimen = 0;
	region_params.xDimen_string = NULL;
	region_params.yDimen = 0;
	region_params.yDimen_string = NULL;
	region_params.otherVar = NULL;
	region_params.otherVar_string = NULL;
	region_params.nPoints = 0;
	region_params.nPoints_string = NULL;
	region_params.xmin = 0;
	region_params.xmin_string = NULL;
	region_params.xmax = 0;
	region_params.xmax_string = NULL;
	region_params.xstep = 0;
	region_params.xstep_string = NULL;
	region_params.ymin = 0;
	region_params.ymin_string = NULL;
	region_params.ymax = 0;
	region_params.ymax_string = NULL;
	region_params.ystep = 0;
	region_params.ystep_string = NULL;
	region_params.noNorm = 0;
	region_params.color = 0;
	region_params.showMisses = 0;
	region_params.showAll = 0;
	region_params.distLim = 0.;
	region_params.distLim_string = NULL;
	region_params.internalsLevel = 0;
	region_params.internalsLevel_string = NULL;
	region_params.internalsScale = 0.;
	region_params.internalsScale_string = NULL;

    } else if (!strcmp(request, "display")){
	char string[10];
	show_string_label(RegionClone->regionPrefixLabel,
			  region_params.regionPrefix);
	XtFree(region_params.xDimen_string);
	sprintf(string, "%d", region_params.xDimen);
	region_params.xDimen_string = XtNewString(string);
	show_string_text(RegionClone->xDimenText,
			 region_params.xDimen_string);

	XtFree(region_params.yDimen_string);
	sprintf(string, "%d", region_params.yDimen);
	region_params.yDimen_string = XtNewString(string);
	show_string_text(RegionClone->yDimenText,
			 region_params.yDimen_string);

	region_set_input_label();

	XtFree(region_params.otherVar_string);
	region_params.otherVar_string =
	  make_float_string(region_params.otherVar);
	show_string_text(RegionClone->otherVarText,
			 region_params.otherVar_string);

	XtFree(region_params.nPoints_string);
	sprintf(string, "%d", region_params.nPoints);
	region_params.nPoints_string = XtNewString(string);
	show_string_text(RegionClone->nPointsText,
			 region_params.nPoints_string);

	show_toggle_value (RegionClone->autoscaleToggle,
			   region_params.autoscale);
	gray_out_item(RegionClone->axesForm, region_params.autoscale);

	XtFree(region_params.xmin_string);
	sprintf(string, "%5.2g", region_params.xmin);
	region_params.xmin_string = XtNewString(string);
	show_string_text (RegionClone->xminText,
			  region_params.xmin_string);

	XtFree(region_params.xmax_string);
	sprintf(string, "%5.2g", region_params.xmax);
	region_params.xmax_string = XtNewString(string);
	show_string_text (RegionClone->xmaxText,
			  region_params.xmax_string);

	XtFree(region_params.xstep_string);
	sprintf(string, "%5.2g", region_params.xstep);
	region_params.xstep_string = XtNewString(string);
	show_string_text (RegionClone->xstepText,
			  region_params.xstep_string);

	XtFree(region_params.ymin_string);
	sprintf(string, "%5.2g", region_params.ymin);
	region_params.ymin_string = XtNewString(string);
	show_string_text (RegionClone->yminText,
			  region_params.ymin_string);

	XtFree(region_params.ymax_string);
	sprintf(string, "%5.2g", region_params.ymax);
	region_params.ymax_string = XtNewString(string);
	show_string_text (RegionClone->ymaxText,
			  region_params.ymax_string);

	XtFree(region_params.ystep_string);
	sprintf(string, "%5.2g", region_params.ystep);
	region_params.ystep_string = XtNewString(string);
	show_string_text (RegionClone->ystepText,
			  region_params.ystep_string);

	show_toggle_value(RegionClone->noNormToggle, region_params.noNorm);
	show_toggle_value(RegionClone->colorToggle, region_params.color);
	show_toggle_value(RegionClone->showMissesToggle,
			  region_params.showMisses);
	show_toggle_value(RegionClone->showAllToggle, region_params.showAll);

	gray_out_item(RegionClone->distLimForm, region_params.showAll);
	XtFree(region_params.distLim_string);
	sprintf(string, "%5.2g", region_params.distLim);
	region_params.distLim_string = XtNewString(string);
	show_string_text(RegionClone->distLimText,
			 region_params.distLim_string);

	XtFree(region_params.internalsLevel_string);
	sprintf(string, "%d", region_params.internalsLevel);
	region_params.internalsLevel_string = XtNewString(string);
	show_string_text(RegionClone->internalsLevelText,
			 region_params.internalsLevel_string);

	XtFree(region_params.internalsScale_string);
	sprintf(string, "%5.2g", region_params.internalsScale);
	region_params.internalsScale_string = XtNewString(string);
	show_string_text(RegionClone->internalsScaleText,
			 region_params.internalsScale_string);

	show_string_label(RegionClone->plotLabelLabel,
			  plots_params.plot_label);

    } else if (!strcmp(request, "plotFname")){
	XtFree(region_params.regionPrefix);
	region_params.regionPrefix = XtNewString(lnk_params.exp_name);
	region_params.regionPrefix = StringConcat(region_params.regionPrefix,
						  ".region.plot");
	show_string_label(RegionClone->regionPrefixLabel,
			  region_params.regionPrefix);
	create_region_plot_fname(gnn_get_which_data());
	print_set_region_status();

    } else if (!strcmp(request, "plotLabel")){
	show_string_label(RegionClone->plotLabelLabel,
			  plots_params.plot_label);

    } else if (!strcmp(request, "colorOn")){
	gray_out_item(RegionClone->colorToggle, False);
	
    } else if (!strcmp(request, "colorOff")){
	gray_out_item(RegionClone->colorToggle, True);

    } else {
	fprintf(stderr, "region_request: unknown request: %s\n", request);
    }
}

void
read_region_defaults(argc, argv)
     int argc;
     char **argv;
{
    /* clear flags and strings which are not printed if they are NULL */
    bound_autoscale = 0;
    plot_exp_path = NULL;
    plot_error_fname = NULL;
    bound_plot_fname = NULL;
    mac_flag = 0;
    mac_wireframe_flag = 0;
    mac_wireframe_output = -1;
    macdots_fname = NULL;
    macscatter_fname = NULL;
    maclines_fname = NULL;
    machisto_fname = NULL;
    macwireframe_fname = NULL;
    profile_plot_fname = NULL;
    bound_plot_label = NULL;
    profile_plot_label = NULL;
    other_vars[0] = 0.;
    region_plot = 0;
    scatter_plot = 0;
    internals_plot = 0;
    profile_plot = 0;
    histogram_plot = 0;
    show_color = 0;
    show_misses = 0;
    show_all = 0;
    scatter_data_fullname = NULL;
    scatter_desc_fullname = NULL;
    plot_map = 0;
    plot_adjust_priors = 0;
    plot_uniform_priors = 0;
    plot_priors_list[0] = 0.;
    no_graphics = 0;
    no_norm = 0;

    if(argc > 0){
	argc = nnargs(argc, argv, bound_argtab, BOUND_TABSIZE);
	region_set_read_variables();
    } else {
	/* set any defaults */
	region_params.color = True;
	region_params.autoscale = True;
	region_params.showAll = True;
	region_params.xDimen = 0;
	region_params.yDimen = 1;
	region_params.nPoints = 50;
	region_params.internalsScale = 1.;
	region_params.xmin = -3;
	region_params.xmax = 3;
	region_params.xstep = 1;
	region_params.ymin = -3;
	region_params.ymax = 3;
	region_params.ystep = 1;
	region_params.distLim = 0.5;
	region_params.noNorm = False;
    }

    region_request("display");
}

void
write_region_defaults(FILE *default_file)
{
    region_set_write_variables();
    if(default_file != NULL){
	fprintf(default_file, "region %s",
		nnargs_get_flags(bound_argtab, BOUND_TABSIZE));
    }
}

static void
region_set_read_variables()
{
    region_params.autoscale = bound_autoscale;
    region_params.xmin = bound_xmin;
    region_params.xmax = bound_xmax;
    region_params.xstep = bound_xstep;
    region_params.ymin = bound_ymin;
    region_params.ymax = bound_ymax;
    region_params.ystep = bound_ystep;
    region_params.distLim = distance_limit;
    region_params.internalsScale = internals_scale;
    region_params.internalsLevel = internals_level;
    region_params.xDimen = bound_dim_1;
    region_params.yDimen = bound_dim_2;
    region_params.nPoints = bound_npoints;
    region_params.showAll = show_all;
    region_params.color = show_color;
    region_params.showMisses = show_misses;
    region_params.noNorm = no_norm;
    if (region_params.otherVar != NULL){
	deallocate_vector(region_params.otherVar);
	region_params.otherVar = NULL;
    }
    region_params.otherVar = copy_static2alloc_vector(other_vars);
}

/* copy the variables for writing to screen file.
   zero out the uninteresting ones (derived or copied data) */
static void
region_set_write_variables()
{
    plot_exp_path = NULL;
    plot_error_fname = NULL;
    plot_n_inputs = 0;
    plot_n_outputs = 0;
    no_graphics = 0;

    /* mac flags */
    mac_flag = 0;
    macdots_fname = NULL;
    macscatter_fname = NULL;
    maclines_fname = NULL;
    machisto_fname = NULL;
    mac_wireframe_flag = 0;
    mac_wireframe_output = -1;
    macwireframe_fname = NULL;

    bound_autoscale = region_params.autoscale;

    scatter_desc_fullname = NULL;

    scatter_data_fullname = NULL;
    scatter_n_patterns = 0;
    bound_plot_fname = NULL;
    bound_plot_label = NULL;
    profile_plot_fname = NULL;
    profile_plot_label = NULL;

    bound_xmin = region_params.xmin;
    bound_xmax = region_params.xmax;
    bound_xstep = region_params.xstep;
    bound_ymin = region_params.ymin;
    bound_ymax = region_params.ymax;
    bound_ystep = region_params.ystep;

    distance_limit = region_params.distLim;
    internals_scale = region_params.internalsScale;
    bound_dim_1 = region_params.xDimen;
    bound_dim_2 = region_params.yDimen;
    bound_npoints = region_params.nPoints;
    internals_level = region_params.internalsLevel;
    region_plot = 0;
    scatter_plot = 0;
    internals_plot = 0;
    profile_plot = 0;
    histogram_plot = 0;
    show_all = region_params.showAll;
    show_color = region_params.color;
    show_misses = region_params.showMisses;

    plot_adjust_priors = 0;
    plot_uniform_priors = 0;
    plot_priors_list[0] = 0.;
    plot_map = 0;
    no_norm = region_params.noNorm;
    copy_alloc2static_vector(region_params.otherVar, other_vars);
}

void
store_region_plot_command(FILE *shell_file, int show_plot, int which_data)
{
    char *plot_command_line;

    if(show_plot) no_graphics = 0;
    else no_graphics = 1;

    store_plot_start(shell_file);
    region_set_common_flags(which_data);
    plot_command_line =
      XtNewString(nnargs_get_flags(bound_argtab, BOUND_TABSIZE));
    *strrchr(plot_command_line, '\n') = ' ';	/* get rid of newline at end */
    fprintf(shell_file, "%s%s\\\n %s",
	    lnk_params.classifier_name, PLOT_BOUND, plot_command_line);
    /* put in classifier dependent call if needed */
    g_get_test_args (shell_file);
    store_plot_end(shell_file);
    /* free things allocated in common_flags */
    XtFree(scatter_desc_fullname);
    XtFree(scatter_data_fullname);
    XtFree(bound_plot_label);
    XtFree(plot_command_line);
}

static void
region_set_common_flags(TEST_TYPE which_data)
{
    plot_exp_path = reports_params.exp_path;
    plot_error_fname = reports_params.param_fname;
    plot_n_inputs = data_params.nraw;
    plot_n_outputs = data_params.noutputs;
    /* no_graphics is already set */

    /* mac flags */
    mac_flag = 0;
    macdots_fname = "boundary_dots.mac";
    macscatter_fname = "scatter.mac";
    maclines_fname = "profile_lines.mac";
    machisto_fname = "profile_histo.mac";
    mac_wireframe_flag = 0;
    mac_wireframe_output = -1;
    macwireframe_fname = "boundary_wire.mac";

    bound_autoscale = region_params.autoscale;

    scatter_desc_fullname =
	   path_create(data_params.data_path, data_params.desc_fname);

    switch(which_data){
      case CLASS_TRAIN:
      case TRAIN_TRAIN:
      case CV_TRAIN:
	scatter_data_fullname =
	  path_create(data_params.data_path, data_params.train_fname);
	scatter_n_patterns = data_params.npatterns_train;
	break;
      case CLASS_EVAL:
	scatter_data_fullname =
	  path_create(data_params.data_path, data_params.eval_fname);
	scatter_n_patterns = data_params.npatterns_eval;
	break;
      case CLASS_TEST:
	scatter_data_fullname =
	  path_create(data_params.data_path, data_params.test_fname);
	scatter_n_patterns = data_params.npatterns_test;
	break;
    }

    if(plots_params.storeEpoch){
	bound_plot_fname = plots_params.regionEpochFname;
    } else {
	create_region_plot_fname(which_data);
	bound_plot_fname = region_params.regionFname;
    }
    /* clear profile plot strings */
    profile_plot_fname = NULL;
    profile_plot_label = NULL;

    bound_xmin = region_params.xmin;
    bound_xmax = region_params.xmax;
    bound_xstep = region_params.xstep;
    bound_ymin = region_params.ymin;
    bound_ymax = region_params.ymax;
    bound_ystep = region_params.ystep;

    distance_limit = region_params.distLim;
    internals_scale = region_params.internalsScale;
    /* add double quotes here to eliminate problems with spaces */
    bound_plot_label = XtMalloc(strlen(plots_params.plot_label) + 3);
    sprintf(bound_plot_label, "\"%s\"",plots_params.plot_label);

    bound_dim_1 = region_params.xDimen;
    bound_dim_2 = region_params.yDimen;
    bound_npoints = region_params.nPoints;
    internals_level = region_params.internalsLevel;
    region_plot = plots_params.region;
    scatter_plot = plots_params.scatter;
    internals_plot = plots_params.internals;
    profile_plot = False;
    histogram_plot = False;
    show_all = region_params.showAll;
    show_color = region_params.color;
    show_misses = region_params.showMisses;

    plot_adjust_priors = priors_params.adjustTest;
    plot_uniform_priors = priors_params.testUniform;
    copy_alloc2static_vector(priors_params.testPriorsList, plot_priors_list);
    plot_map = data_params.db_type;
    no_norm = region_params.noNorm;
    copy_alloc2static_vector(region_params.otherVar, other_vars);
}

void
set_region_plot_fname(String name)
{
    XtFree(region_params.regionFname);
    region_params.regionFname = XtNewString(name);
}

static void
create_region_plot_fname(TEST_TYPE which_data)
{
    XtFree(region_params.regionFname);
    region_params.regionFname = XtNewString(region_params.regionPrefix);
    switch (which_data){
      case TRAIN_TRAIN:
      case CV_TRAIN:
	region_params.regionFname = StringConcat(region_params.regionFname,
						 data_params.ext_train);
	break;
      case CLASS_TRAIN:
	region_params.regionFname = StringConcat(region_params.regionFname,
						 data_params.ext_test_train);
	break;
      case CLASS_EVAL:
	region_params.regionFname = StringConcat(region_params.regionFname,
						 data_params.ext_eval);
	break;
      case CLASS_TEST:
	region_params.regionFname = StringConcat(region_params.regionFname,
						 data_params.ext_test);
	break;
    }
}

void
region_set_input_label()
{
    char * label;
    int upper_limit;
    
    if(region_params.noNorm) upper_limit = data_params.nraw;
    else upper_limit = select_params.ninputs;

    if(0 <= region_params.xDimen && region_params.xDimen < upper_limit){
	if(region_params.noNorm){
	    label = get_input_label(region_params.xDimen,
				    data_params.input_labels,
				    NULL, 0);
	} else {
	    label = get_input_label(region_params.xDimen,
				    data_params.input_labels,
				    select_params.features,
				    norm_params.normalize);
	}
    } else {
	label = "Error: Bad Plot Feature";
    }
    show_string_label(RegionClone->xDimenLabelLabel, label);

    if(0 <= region_params.yDimen && region_params.yDimen < upper_limit){
	if(region_params.noNorm){
	    label = get_input_label(region_params.yDimen,
				    data_params.input_labels,
				    NULL, 0);
	} else {
	    label = get_input_label(region_params.yDimen,
				    data_params.input_labels,
				    select_params.features,
				    norm_params.normalize);
	}
    } else {
	label = "Error: Bad Plot Feature";
    }
    show_string_label(RegionClone->yDimenLabelLabel, label);
}
