/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/reject/reject_callback.c,v 11.5 2001/08/23 20:17:51 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                               REJECT                                 ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  6-Nov-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include <nnlib.h>
#include "reject.h"
/*
 * External Functions
 */
#include "reject_func.h"
#include "../main/lnknet_subs.h"
#include "../lnk/lnk_func.h"
#include "../plots/plots_func.h"
#include "../print/print_func.h"
 
/*
 * Externals
 */
extern RejectWidgets *RejectClone;
#include "reject_params.h"
RejectParams reject_params; /* this is the real one */
#include "../data/data_params.h"
extern DataParams data_params;/* for data base path and name and extensions */
#include "../plots/plots_params.h"
extern PlotsParams plots_params;
#include "../lnk/lnk_params.h"
extern LnkParams lnk_params;
#include "../reports/reports_params.h"
extern ReportsParams reports_params;

/*
 * Statics
 */
static void reject_set_read_variables();
static void reject_set_write_variables();
static void reject_set_common_flags();
#define PLOT_REJECT "plot_reject"
#include <plot_reject_args.h>
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: reject_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   reject_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  6-Nov-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
reject_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    RejectWidgets *ip;
    static int tableBegin_buttons_mapped = False;
    static int tableEnd_buttons_mapped = False;
    static int tableStep_buttons_mapped = False;
    static int xmin_buttons_mapped = False;
    static int xmax_buttons_mapped = False;
    static int xstep_buttons_mapped = False;
    static int ymin_buttons_mapped = False;
    static int ymax_buttons_mapped = False;
    static int ystep_buttons_mapped = False;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 
    if (!strcmp(client, "noTablePBtn")) {
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	reject_params.verbose = 0;
	gray_out_item(ip->tableBeginForm, (reject_params.verbose != 1));
	gray_out_item(ip->tableEndForm, (reject_params.verbose != 1));
	gray_out_item(ip->tableStepForm, (reject_params.verbose != 1));
	
    } else if (!strcmp(client, "ratePBtn")) {
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	reject_params.verbose = 1;
	gray_out_item(ip->tableBeginForm, (reject_params.verbose != 1));
	gray_out_item(ip->tableEndForm, (reject_params.verbose != 1));
	gray_out_item(ip->tableStepForm, (reject_params.verbose != 1));
	
    } else if (!strcmp(client, "fullPBtn")) {
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	reject_params.verbose = 2;
	gray_out_item(ip->tableBeginForm, (reject_params.verbose != 1));
	gray_out_item(ip->tableEndForm, (reject_params.verbose != 1));
	gray_out_item(ip->tableStepForm, (reject_params.verbose != 1));
 
    } else if (!strcmp(client, "tableBeginText")) {
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	tableBegin_buttons_mapped =
	  okTextValueChanged(ip->tableBeginText, ip->tableBeginOKPBtn,
			     ip->tableBeginCancelPBtn,
			     ip->rejectDialogShell,
			     tableBegin_buttons_mapped,
			     reject_params.tableBegin_string);
	
    } else if ((!strcmp(client, "tableBeginTextAct")) ||
	       (!strcmp(client, "tableBeginOKPBtn"))) {
	String tableBeginTextval;

	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	tableBegin_buttons_mapped =
	  okTextActivate(ip->tableBeginText, ip->tableBeginOKPBtn,
			 ip->tableBeginCancelPBtn, &tableBeginTextval);
	/* I won't actually be using this copy */
	XtFree (tableBeginTextval);
	
	/* verify the string as a float */
	reject_params.tableBegin =
	  set_atof_range_value(ip->tableBeginText, "tableBegin_set",
			       reject_params.tableBegin, 0.,100.);

	/* save the current text for future comparisons */
	XtFree(reject_params.tableBegin_string);
	reject_params.tableBegin_string = get_string_text(ip->tableBeginText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "tableBeginCancelPBtn")) {
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	tableBegin_buttons_mapped = 
	  okTextCancel(ip->tableBeginText, ip->tableBeginOKPBtn,
		       ip->tableBeginCancelPBtn,
		       reject_params.tableBegin_string);

    } else if (!strcmp(client, "tableEndText")) {
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	tableEnd_buttons_mapped =
	  okTextValueChanged(ip->tableEndText, ip->tableEndOKPBtn,
			     ip->tableEndCancelPBtn,
			     ip->rejectDialogShell,
			     tableEnd_buttons_mapped,
			     reject_params.tableEnd_string);
	
    } else if ((!strcmp(client, "tableEndTextAct")) ||
	       (!strcmp(client, "tableEndOKPBtn"))) {
	String tableEndTextval;

	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	tableEnd_buttons_mapped =
	  okTextActivate(ip->tableEndText, ip->tableEndOKPBtn,
			 ip->tableEndCancelPBtn, &tableEndTextval);
	/* I won't actually be using this copy */
	XtFree (tableEndTextval);
	
	/* verify the string as a float */
	reject_params.tableEnd =
	  set_atof_range_value(ip->tableEndText, "tableEnd_set",
			       reject_params.tableEnd, 0.,100.);

	/* save the current text for future comparisons */
	XtFree(reject_params.tableEnd_string);
	reject_params.tableEnd_string = get_string_text(ip->tableEndText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "tableEndCancelPBtn")) {
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	tableEnd_buttons_mapped = 
	  okTextCancel(ip->tableEndText, ip->tableEndOKPBtn,
		       ip->tableEndCancelPBtn,
		       reject_params.tableEnd_string);

    } else if (!strcmp(client, "tableStepText")) {
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	tableStep_buttons_mapped =
	  okTextValueChanged(ip->tableStepText, ip->tableStepOKPBtn,
			     ip->tableStepCancelPBtn,
			     ip->rejectDialogShell,
			     tableStep_buttons_mapped,
			     reject_params.tableStep_string);
	
    } else if ((!strcmp(client, "tableStepTextAct")) ||
	       (!strcmp(client, "tableStepOKPBtn"))) {
	String tableStepTextval;

	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	tableStep_buttons_mapped =
	  okTextActivate(ip->tableStepText, ip->tableStepOKPBtn,
			 ip->tableStepCancelPBtn, &tableStepTextval);
	/* I won't actually be using this copy */
	XtFree (tableStepTextval);
	
	/* verify the string as a float */
	reject_params.tableStep =
	  set_atof_range_value(ip->tableStepText, "tableStep_set",
			       reject_params.tableStep, 0.,100.);

	/* save the current text for future comparisons */
	XtFree(reject_params.tableStep_string);
	reject_params.tableStep_string = get_string_text(ip->tableStepText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "tableStepCancelPBtn")) {
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	tableStep_buttons_mapped = 
	  okTextCancel(ip->tableStepText, ip->tableStepOKPBtn,
		       ip->tableStepCancelPBtn,
		       reject_params.tableStep_string);
 
    } else if (!strcmp(client, "sumToOneToggle")) {
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "reject_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	reject_params.sumToOne = state->set;

    } else if (!strcmp(client, "xminText")) {
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	xmin_buttons_mapped =
	  okTextValueChanged(ip->xminText, ip->xminOKPBtn,
			     ip->xminCancelPBtn,
			     ip->rejectDialogShell,
			     xmin_buttons_mapped,
			     reject_params.xmin_string);
      
    } else if (!strcmp(client, "xminTextAct") ||
	       !strcmp(client, "xminOKPBtn")) {
	String xminTextval;

	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	xmin_buttons_mapped = okTextActivate(ip->xminText, ip->xminOKPBtn,
					     ip->xminCancelPBtn, &xminTextval);

	XtFree (xminTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	reject_params.xmin = set_atof_value(ip->xminText, "reject_xmin",
					  reject_params.xmin);
	XtFree(reject_params.xmin_string);
	reject_params.xmin_string = get_string_text(ip->xminText);
 
    } else if (!strcmp(client, "xminCancelPBtn")) {
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	xmin_buttons_mapped = 
	  okTextCancel(ip->xminText, ip->xminOKPBtn,
		       ip->xminCancelPBtn,
		       reject_params.xmin_string);
 
    } else if (!strcmp(client, "xmaxText")) {
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	xmax_buttons_mapped =
	  okTextValueChanged(ip->xmaxText, ip->xmaxOKPBtn,
			     ip->xmaxCancelPBtn,
			     ip->rejectDialogShell,
			     xmax_buttons_mapped,
			     reject_params.xmax_string);
      
    } else if (!strcmp(client, "xmaxTextAct") ||
	       !strcmp(client, "xmaxOKPBtn")) {
	String xmaxTextval;

	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	xmax_buttons_mapped = okTextActivate(ip->xmaxText, ip->xmaxOKPBtn,
					     ip->xmaxCancelPBtn, &xmaxTextval);
	
	XtFree (xmaxTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	reject_params.xmax = set_atof_value(ip->xmaxText, "reject_xmax",
					  reject_params.xmax);
	XtFree(reject_params.xmax_string);
	reject_params.xmax_string = get_string_text(ip->xmaxText);
 
    } else if (!strcmp(client, "xmaxCancelPBtn")) {
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	xmax_buttons_mapped = 
	  okTextCancel(ip->xmaxText, ip->xmaxOKPBtn,
		       ip->xmaxCancelPBtn,
		       reject_params.xmax_string);
 
    } else if (!strcmp(client, "xstepText")) {
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	xstep_buttons_mapped =
	  okTextValueChanged(ip->xstepText, ip->xstepOKPBtn,
			     ip->xstepCancelPBtn,
			     ip->rejectDialogShell,
			     xstep_buttons_mapped,
			     reject_params.xstep_string);
      
    } else if (!strcmp(client, "xstepTextAct") ||
	       !strcmp(client, "xstepOKPBtn")) {
	String xstepTextval;
      
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	xstep_buttons_mapped =
	  okTextActivate(ip->xstepText, ip->xstepOKPBtn,
			 ip->xstepCancelPBtn, &xstepTextval);

	XtFree (xstepTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	reject_params.xstep = set_atof_value(ip->xstepText, "reject_xstep",
					   reject_params.xstep);
	XtFree(reject_params.xstep_string);
	reject_params.xstep_string = get_string_text(ip->xstepText);
 
    } else if (!strcmp(client, "xstepCancelPBtn")) {
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	xstep_buttons_mapped = 
	  okTextCancel(ip->xstepText, ip->xstepOKPBtn,
		       ip->xstepCancelPBtn,
		       reject_params.xstep_string);
	
    } else if (!strcmp(client, "yminText")) {
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	ymin_buttons_mapped =
	  okTextValueChanged(ip->yminText, ip->yminOKPBtn,
			     ip->yminCancelPBtn,
			     ip->rejectDialogShell,
			     ymin_buttons_mapped,
			     reject_params.ymin_string);
      
    } else if (!strcmp(client, "yminTextAct") ||
	       !strcmp(client, "yminOKPBtn")) {
	String yminTextval;
      
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	ymin_buttons_mapped = okTextActivate(ip->yminText, ip->yminOKPBtn,
					     ip->yminCancelPBtn, &yminTextval);

	XtFree (yminTextval);/* I won't actually be using this copy */
	
	/* check that this value is ok */
	reject_params.ymin = set_atof_value(ip->yminText, "reject_ymin",
					  reject_params.ymin);
	XtFree(reject_params.ymin_string);
	reject_params.ymin_string = get_string_text(ip->yminText);
	
    } else if (!strcmp(client, "yminCancelPBtn")) {
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	ymin_buttons_mapped = 
	  okTextCancel(ip->yminText, ip->yminOKPBtn,
		       ip->yminCancelPBtn,
		       reject_params.ymin_string);
	
    } else if (!strcmp(client, "ymaxText")) {
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	ymax_buttons_mapped =
	  okTextValueChanged(ip->ymaxText, ip->ymaxOKPBtn,
			     ip->ymaxCancelPBtn,
			     ip->rejectDialogShell,
			     ymax_buttons_mapped,
			     reject_params.ymax_string);
      
    } else if (!strcmp(client, "ymaxTextAct") ||
	       !strcmp(client, "ymaxOKPBtn")) {
	String ymaxTextval;
      
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	ymax_buttons_mapped = okTextActivate(ip->ymaxText, ip->ymaxOKPBtn,
					     ip->ymaxCancelPBtn, &ymaxTextval);

	XtFree (ymaxTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	reject_params.ymax = set_atof_value(ip->ymaxText, "reject_ymax",
					  reject_params.ymax);
	XtFree(reject_params.ymax_string);
	reject_params.ymax_string = get_string_text(ip->ymaxText);
 
    } else if (!strcmp(client, "ymaxCancelPBtn")) {
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	ymax_buttons_mapped = 
	  okTextCancel(ip->ymaxText, ip->ymaxOKPBtn,
		       ip->ymaxCancelPBtn,
		       reject_params.ymax_string);
 
    } else if (!strcmp(client, "ystepText")) {
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	ystep_buttons_mapped =
	  okTextValueChanged(ip->ystepText, ip->ystepOKPBtn,
			     ip->ystepCancelPBtn,
			     ip->rejectDialogShell,
			     ystep_buttons_mapped,
			     reject_params.ystep_string);
      
    } else if (!strcmp(client, "ystepTextAct") ||
	       !strcmp(client, "ystepOKPBtn")) {
	String ystepTextval;
      
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	ystep_buttons_mapped =
	  okTextActivate(ip->ystepText, ip->ystepOKPBtn,
			 ip->ystepCancelPBtn, &ystepTextval);

	XtFree (ystepTextval);/* I won't actually be using this copy */
	
	/* check that this value is ok */
	reject_params.ystep = set_atof_value(ip->ystepText, "reject_ystep",
					   reject_params.ystep);
	XtFree(reject_params.ystep_string);
	reject_params.ystep_string = get_string_text(ip->ystepText);
 
    } else if (!strcmp(client, "ystepCancelPBtn")) {
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	ystep_buttons_mapped = 
	  okTextCancel(ip->ystepText, ip->ystepOKPBtn,
		       ip->ystepCancelPBtn,
		       reject_params.ystep_string);
	/*
	 * end of axes scales callbacks
	 */
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "reject_callback: %s\n", client );*/
	close_popup(RejectClone->rejectForm);

 
    } else {
	fprintf(stderr, "reject_callback: unknown client: %s\n", (char *)client);
    }
}

void
reject_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(RejectClone->rejectForm);
    } else if (!strcmp(request, "init")){
	reject_params.plotFname = NULL;
	reject_params.tableBegin = 0.;
	reject_params.tableBegin_string = NULL;
	reject_params.tableEnd = 0.;
	reject_params.tableEnd_string = NULL;
	reject_params.tableStep = 0.;
	reject_params.tableStep_string = NULL;
	reject_params.xmin = 0;
	reject_params.xmin_string = NULL;
	reject_params.xmax = 0;
	reject_params.xmax_string = NULL;
	reject_params.xstep = 0;
	reject_params.xstep_string = NULL;
	reject_params.ymin = 0;
	reject_params.ymin_string = NULL;
	reject_params.ymax = 0;
	reject_params.ymax_string = NULL;
	reject_params.ystep = 0;
	reject_params.ystep_string = NULL;

    } else if (!strcmp(request, "display")){
	char string[10];
	show_string_label(RejectClone->plotFnameLabel,
			  reject_params.plotFname);
	show_option_value(RejectClone->verboseOptionMenu,
			  reject_params.verbose);
	gray_out_item(RejectClone->tableBeginForm, (reject_params.verbose != 1));
	gray_out_item(RejectClone->tableEndForm, (reject_params.verbose != 1));
	gray_out_item(RejectClone->tableStepForm, (reject_params.verbose != 1));

	XtFree(reject_params.tableBegin_string);
	sprintf(string, "%.1f", reject_params.tableBegin);
	reject_params.tableBegin_string = XtNewString(string);
	show_string_text(RejectClone->tableBeginText,
			 reject_params.tableBegin_string);
	XtFree(reject_params.tableEnd_string);
	sprintf(string, "%.1f", reject_params.tableEnd);
	reject_params.tableEnd_string = XtNewString(string);
	show_string_text(RejectClone->tableEndText,
			 reject_params.tableEnd_string);
	XtFree(reject_params.tableStep_string);
	sprintf(string, "%.1f", reject_params.tableStep);
	reject_params.tableStep_string = XtNewString(string);
	show_string_text(RejectClone->tableStepText,
			 reject_params.tableStep_string);

	show_toggle_value(RejectClone->sumToOneToggle,
			  reject_params.sumToOne);

	XtFree(reject_params.xmin_string);
	sprintf(string, "%.1f", reject_params.xmin);
	reject_params.xmin_string = XtNewString(string);
	show_string_text (RejectClone->xminText,
			  reject_params.xmin_string);

	XtFree(reject_params.xmax_string);
	sprintf(string, "%.1f", reject_params.xmax);
	reject_params.xmax_string = XtNewString(string);
	show_string_text (RejectClone->xmaxText,
			  reject_params.xmax_string);

	XtFree(reject_params.xstep_string);
	sprintf(string, "%.1f", reject_params.xstep);
	reject_params.xstep_string = XtNewString(string);
	show_string_text (RejectClone->xstepText,
			  reject_params.xstep_string);

	XtFree(reject_params.ymin_string);
	sprintf(string, "%.1f", reject_params.ymin);
	reject_params.ymin_string = XtNewString(string);
	show_string_text (RejectClone->yminText,
			  reject_params.ymin_string);

	XtFree(reject_params.ymax_string);
	sprintf(string, "%.1f", reject_params.ymax);
	reject_params.ymax_string = XtNewString(string);
	show_string_text (RejectClone->ymaxText,
			  reject_params.ymax_string);

	XtFree(reject_params.ystep_string);
	sprintf(string, "%.1f", reject_params.ystep);
	reject_params.ystep_string = XtNewString(string);
	show_string_text (RejectClone->ystepText,
			 reject_params.ystep_string);

	show_string_label(RejectClone->plotLabelLabel,
			  plots_params.plot_label);

    } else if (!strcmp(request, "plotFname")){
	XtFree(reject_params.plotFname);
	reject_params.plotFname = XtNewString(lnk_params.exp_name);
	reject_params.plotFname = StringConcat(reject_params.plotFname,
					     ".reject.plot");
	show_string_label(RejectClone->plotFnameLabel,
			  reject_params.plotFname);
	print_set_reject_status();

    } else if (!strcmp(request, "plotLabel")){
	show_string_label(RejectClone->plotLabelLabel,
			  plots_params.plot_label);

    } else if (!strcmp("sumToOne", request)){
	if(!reject_params.sumToOne){
	    reject_params.sumToOne = 1;
	    show_toggle_value(RejectClone->sumToOneToggle,
			      reject_params.sumToOne);
	}

    } else {
	fprintf(stderr, "reject_request: unknown request: %s\n", request);
    }
}

static void
reject_set_read_variables()
{
    reject_params.verbose = verbose;
    reject_params.tableBegin = table_begin;
    reject_params.tableEnd = table_end;
    reject_params.tableStep = table_step;
    reject_params.sumToOne = sum_to_one;
    reject_params.xmin = reject_xmin;
    reject_params.xmax = reject_xmax;
    reject_params.xstep = reject_xstep;
    reject_params.ymin = reject_ymin;
    reject_params.ymax = reject_ymax;
    reject_params.ystep = reject_ystep;
}

static void
reject_set_write_variables()
{
    plot_exp_path = NULL;
    plot_reject_fname = NULL;
    error_fname = NULL;
    table_begin = reject_params.tableBegin;
    table_end = reject_params.tableEnd;
    table_step = reject_params.tableStep;
    no_graphics = 0;

    verbose = reject_params.verbose;
    sum_to_one = reject_params.sumToOne;
    reject_xmin = reject_params.xmin;
    reject_xmax = reject_params.xmax;
    reject_xstep = reject_params.xstep;
    reject_ymin = reject_params.ymin;
    reject_ymax = reject_params.ymax;
    reject_ystep = reject_params.ystep;
    reject_plot_label = NULL;
    mac_reject_flag = 0;
    mac_rejectlines_fname = NULL;
}

void
read_reject_defaults(argc, argv)
     int argc;
     char **argv;
{
    plot_exp_path = NULL;
    error_fname = NULL;
    no_graphics = 0;
    plot_reject_fname = NULL;
    reject_plot_label = NULL;
    mac_rejectlines_fname = NULL;
    sum_to_one = 0;
    mac_reject_flag = 0;
    if(argc > 0){
	argc = nnargs(argc, argv, reject_argtab, REJECT_TABSIZE);
	reject_set_read_variables();
    } else {
	/* set any defaults */
	reject_params.xmin = 0.;
	reject_params.xmax = 100.;
	reject_params.xstep = 10.;
	reject_params.ymin = 0;
	reject_params.ymax = 100.;
	reject_params.ystep = 10.;
	reject_params.verbose = 1;
	reject_params.tableBegin = 0.;
	reject_params.tableEnd = 100.;
	reject_params.tableStep = 10.;
    }
    reject_request("display");
}

void
write_reject_defaults(FILE *default_file)
{
    reject_set_write_variables();
    if(default_file != NULL){
	fprintf(default_file, "reject %s",
		nnargs_get_flags(reject_argtab, REJECT_TABSIZE));
    }
}

void
store_reject_plot_command(FILE *shell_file)
{
    char *plot_command_line;

    no_graphics = 1;

    store_plot_start(shell_file);
    reject_set_common_flags();
    plot_command_line =
      XtNewString(nnargs_get_flags(reject_argtab, REJECT_TABSIZE));
    *strrchr(plot_command_line, '\n') = ' ';	/* get rid of newline at end */
    fprintf(shell_file, "%s\\\n %s",
	    PLOT_REJECT, plot_command_line);
    store_plot_end_outputs(shell_file, "reject_plot",
			   reject_params.plotFname);

    /* free things allocated in common_flags */
    XtFree(reject_plot_label);
    XtFree(error_fname);
    XtFree(plot_command_line);
}

static void
reject_set_common_flags()
{
    TEST_TYPE which_data = gnn_get_which_data();

    plot_exp_path = reports_params.exp_path;
    error_fname = XtNewString(reports_params.error_fname);
   switch(which_data){
      case TRAIN_TRAIN:
	error_fname =
	  StringConcat(error_fname, data_params.ext_train);
	npatterns = data_params.npatterns_train;
	break;
      case CV_TRAIN:
	error_fname =
	  StringConcat(error_fname, lnk_params.ext_cv);
	npatterns = data_params.npatterns_train;
	break;
      case CLASS_TRAIN:
	error_fname =
	  StringConcat(error_fname, data_params.ext_test_train);
	npatterns = data_params.npatterns_train;
	break;
      case CLASS_EVAL:
	error_fname =
	  StringConcat(error_fname, data_params.ext_eval);
	npatterns = data_params.npatterns_eval;
	break;
      case CLASS_TEST:
	error_fname =
	  StringConcat(error_fname, data_params.ext_test);
	npatterns = data_params.npatterns_test;
	break;
    }
    noutputs = data_params.noutputs;

    verbose = reject_params.verbose;
    sum_to_one = reject_params.sumToOne;

    table_begin = reject_params.tableBegin;
    table_end = reject_params.tableEnd;
    table_step = reject_params.tableStep;

    /* no_graphics is already set */
    plot_reject_fname = reject_params.plotFname;
    reject_xmin = reject_params.xmin;
    reject_xmax = reject_params.xmax;
    reject_xstep = reject_params.xstep;
    reject_ymin = reject_params.ymin;
    reject_ymax = reject_params.ymax;
    reject_ystep = reject_params.ystep;

    /* mac flags */
    mac_reject_flag = 0;
    mac_rejectlines_fname = "reject.mac";

    /* add double quotes here to eliminate problems with spaces */
    reject_plot_label = XtMalloc(strlen(plots_params.plot_label) + 3);
    sprintf(reject_plot_label, "\"%s\"",plots_params.plot_label);
}
