/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/reports/reports_callback.c,v 11.6 2001/09/11 15:37:21 kukolich Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            REPORTS                                   ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  21-Sep-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <Xm/Xm.h>
#include <nnlib.h>
#include "../main/lnknet_subs.h"
#include "reports.h"
/*
 * External Functions
 */
#include "reports_func.h"
#include "../lnk/lnk_func.h"
#include "../committeeData/committeeData_func.h"
#include "../print/print_func.h"
 
/*
 * Externals
 */
extern ReportsWidgets *ReportsClone;
#include "reports_params.h"
ReportsParams reports_params; /* this is the real one */
#include "../lnk/lnk_params.h"
extern LnkParams lnk_params;
#include "../plots/plots_params.h"
#include "../plots/plots_func.h"
extern PlotsParams plots_params;/* for setting verror according
				   to plot flags */
/*
 * Statics
 */
static void reports_set_defaults_variables(ReportsParams *dp);
static void display_verror();
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: reports_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   reports_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  21-Sep-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
reports_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    ReportsWidgets *ip;
    static int notebookFile_buttons_mapped = False;
    static int experimentPath_buttons_mapped = False;
    
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
    
    if (!strcmp(client, "verrorNonePBtn")) {
	/*fprintf (stderr, "reports_callback: %s\n", client );*/
	if(plots_verror_value_ok(NONE_VERROR)){
	    reports_params.verror = NONE_VERROR;
	} else display_verror();
	
    } else if (!strcmp(client, "verrorResultsPBtn")) {
	/*fprintf (stderr, "reports_callback: %s\n", client );*/
	if(plots_verror_value_ok(RESULTS_VERROR)){
	    reports_params.verror = RESULTS_VERROR;
	} else display_verror();

    } else if (!strcmp(client, "verrorOutputsPBtn")) {
	/*fprintf (stderr, "reports_callback: %s\n", client );*/
	if(plots_verror_value_ok(OUTPUTS_VERROR)){
	    reports_params.verror = OUTPUTS_VERROR;
	} else display_verror();
      
    } else if (!strcmp(client, "verrorInputsPBtn")) {
	/*fprintf (stderr, "reports_callback: %s\n", client );*/
	if(plots_verror_value_ok(INPUTS_VERROR)){
	    reports_params.verror = INPUTS_VERROR;
	} else display_verror();
      
    } else if (!strcmp(client, "verbosityOverallPBtn")) {
	/*fprintf (stderr, "reports_callback: %s\n", client );*/
	/* the verbosity defined values are set to one less than
	   the desired value */
	reports_params.verbosity = OVERALL_VERBOSITY+1;
      
    } else if (!strcmp(client, "verbositySummaryPBtn")) {
	/*fprintf (stderr, "reports_callback: %s\n", client );*/
	reports_params.verbosity = TABLE_VERBOSITY+1;
      
    } else if (!strcmp(client, "verbosityConfusionPBtn")) {
	/*fprintf (stderr, "reports_callback: %s\n", client );*/
	reports_params.verbosity = CONFUSION_VERBOSITY+1;
      
    } else if (!strcmp(client, "verbosityFlagsPBtn")) {
	/*fprintf (stderr, "reports_callback: %s\n", client );*/
	reports_params.verbosity = FLAGS_VERBOSITY+1;
      
    } else if (!strcmp(client, "restoreScreenPBtn")) {
	FILE *default_file;
	/*fprintf (stderr, "reports_callback: %s\n", client );*/
	if ((default_file =
	     fopen(reports_params.screen_path, "r")) == NULL) {
	    warning("Default parameter file %s not found.\n",
		    reports_params.screen_path);
	} else {
	    nn_fclose(default_file, reports_params.screen_path,
		      "for reading lnknet defaults");
	    read_defaults(reports_params.screen_path);
	}
	
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "reports_callback: %s\n", client );*/
	close_popup(ip->reportsForm);

    } else if (!strcmp(client, "noTrainVerrorToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "reports_callback: %s\n", client );*/
	if(plots_train_verror_value_ok(state->set ? 0 : reports_params.verror))
	  reports_params.no_train_verror = state->set;
	else display_verror();
	
    } else if (!strcmp(client, "notebookOnToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "reports_callback: %s\n", client );*/
	reports_params.notebook_on = state->set;
	
    } else if (!strcmp(client, "notebookFileText")) {
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	/*fprintf (stderr, "reports_callback: %s\n", client );*/
	notebookFile_buttons_mapped =
	  okTextValueChanged(ip->notebookFileText,
			     ip->notebookFileOKPBtn,
			     ip->notebookFileCancelPBtn,
			     ip->reportsDialogShell,
			     notebookFile_buttons_mapped,
			     reports_params.notebook_fname);

    } else if ((!strcmp(client, "notebookFileTextAct")) ||
	       (!strcmp(client, "notebookFileOKPBtn"))) {
	/*fprintf (stderr, "reports_callback: %s\n", client );*/
	String notebookFileTextval;
	notebookFile_buttons_mapped =
	  okTextActivate(ip->notebookFileText,
			 ip->notebookFileOKPBtn,
			 ip->notebookFileCancelPBtn,
			 &notebookFileTextval);
	/* we won't use this copy */
	XtFree(notebookFileTextval);
 
	notebookFileTextval =
	  set_string_not_empty(ip->notebookFileText, "notebook file",
			       reports_params.notebook_fname);
	XtFree(reports_params.notebook_fname);
	reports_params.notebook_fname = notebookFileTextval;
	XtFree(reports_params.notebook_path);
	reports_params.notebook_path =
	  path_create(reports_params.working_directory,
		      reports_params.notebook_fname);
	reports_request("major_note");

    } else if (!strcmp(client, "notebookFileCancelPBtn")) {
	/*fprintf (stderr, "reports_callback: %s\n", client );*/
	notebookFile_buttons_mapped = 
	  okTextCancel(ip->notebookFileText, ip->notebookFileOKPBtn,
		       ip->notebookFileCancelPBtn,
		       reports_params.notebook_fname);
 
    } else if (!strcmp(client, "experimentPathText")) {
	/*this is the value changed callback,
	  make the OK/CANCEL buttons appear */
	/*fprintf (stderr, "reports_callback: %s\n", client );*/
	experimentPath_buttons_mapped =
	  okTextValueChanged(ip->experimentPathText,
			     ip->experimentPathOKPBtn,
			     ip->experimentPathCancelPBtn,
			     ip->reportsDialogShell,
			     experimentPath_buttons_mapped,
			     reports_params.exp_path);

    } else if ((!strcmp(client, "experimentPathTextAct")) ||
	       (!strcmp(client, "experimentPathOKPBtn"))) {
	/*fprintf (stderr, "reports_callback: %s\n", client );*/
	String experimentPathTextval;
	void ccode_check_file();

	experimentPath_buttons_mapped =
	  okTextActivate(ip->experimentPathText,
			 ip->experimentPathOKPBtn,
			 ip->experimentPathCancelPBtn,
			 &experimentPathTextval);
	/* we won't use this copy */
	XtFree(experimentPathTextval);

	experimentPathTextval =
	  set_string_not_empty(ip->experimentPathText, "experimentPath",
			       reports_params.exp_path);
	XtFree(reports_params.exp_path);
	reports_params.exp_path = experimentPathTextval;
	XtFree(reports_params.shell_path);
	reports_params.shell_path =
	  path_create(reports_params.exp_path, reports_params.shell_fname);
	reports_params.screen_path =
	  path_create(reports_params.exp_path, reports_params.screen_fname);
	ccode_check_file();
	committeeData_request("expPath");
	print_set_all_status();

    } else if (!strcmp(client, "experimentPathCancelPBtn")) {
	/*fprintf (stderr, "reports_callback: %s\n", client );*/
	experimentPath_buttons_mapped = 
	  okTextCancel(ip->experimentPathText, ip->experimentPathOKPBtn,
		       ip->experimentPathCancelPBtn,
		       reports_params.exp_path);
 
    } else {
	fprintf(stderr, "reports_callback: unknown client: %s\n", (char *)client);
    }
}

void
reports_request(String request)
{
    if (!strcmp(request, "open")) {
	open_popup(ReportsClone->reportsForm);
    } else if (!strcmp(request, "init")){
	char working_directory[256];
	if(getcwd(working_directory,256) == 0){
	    warning("couldn't get directory name. %s",
		    working_directory);
	    strcpy(working_directory, "./");
	}
	reports_params.working_directory = XtNewString(working_directory);
	reports_params.exp_path = NULL;
	reports_params.param_fname = NULL;
	reports_params.shell_fname = NULL;
	reports_params.shell_path = NULL;
	reports_params.log_fname = NULL;
	reports_params.screen_fname = NULL;
	reports_params.screen_path = NULL;
	reports_params.error_fname = NULL;
	reports_params.notebook_fname = NULL;
	reports_params.notebook_path = NULL;
	reports_params.verbosity = 0;
	reports_params.verror = 0;
	reports_params.no_train_verror = 1;
    } else if (!strcmp(request, "display")){
	show_string_label (ReportsClone->wdirNameLabel,
			  reports_params.working_directory);
	show_string_text(ReportsClone->experimentPathText,
			 reports_params.exp_path);
	show_string_label(ReportsClone->shellFileLabel,
			  reports_params.shell_fname);
	show_string_label(ReportsClone->logFileLabel,
			  reports_params.log_fname);
	show_string_label(ReportsClone->paramFileLabel,
			  reports_params.param_fname);
	show_string_label(ReportsClone->errorFileLabel,
			  reports_params.error_fname);
	show_string_label(ReportsClone->screenFileLabel,
			  reports_params.screen_fname);
	show_string_text(ReportsClone->notebookFileText,
			 reports_params.notebook_fname);
	show_option_value(ReportsClone->verbosityOptionMenu,
			  reports_params.verbosity);
	show_option_value(ReportsClone->verrorOptionMenu,
			  reports_params.verror);
	show_toggle_value(ReportsClone->noTrainVerrorToggle,
			  reports_params.no_train_verror);
	show_toggle_value(ReportsClone->notebookOnToggle,
			  reports_params.notebook_on);
    } else if (!strcmp(request, "major_note")){
	reports_params.major_note = 1;
    } else if (!strcmp(request, "reset_major_note")){
	reports_params.major_note = 0;
    } else {
	fprintf (stderr, "reports_request: unknown request: %s\n", request);
    }
}

/******************************************************************************
 ******************************************************************************
 *
 * Module Names: read_reports_defaults, write_reports_defaults
 *
 * Purpose: 
 *   Handle reading and writing the state of the reports
 *   window to a screen/defaults file.
 * 
 * Calling Sequence:
 *   # open defaults file for reading
 *   # read arg tables up until the reports one
 *   # fill arg list for reports one
 *   read_reports_defaults(argc, argv)
 *
 *  OR
 *   # open a defaults file for writings
 *   # write arg tables up until the reports one
 *   write_reports_defaults(default_file)
 *
 * Inputs:
 *   int argc; # the number of strings in argv
 *   char **argv; # pointer to an array String pointers
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   read sets values in reports_params structure according to flags found in argv
 *   write writes flags to default_file according to the settings of
 *   reports_params structure entries
 *
 * Restrictions:
 *   On read, if the argument list in argv doesn't match the arguments
 *   in the reports argument table, a fatal error will occur.
 *
 * Modification History:
 *  21-Sep-1995     Linda C Kukolich  modified original xview versions
 *
 ******************************************************************************
 ******************************************************************************
 */
static ARG reportstab[] =
{
    {"exp_path", STRING, (int *)NULL, "full or relative path to experiment"},
    {"notebook_fname", STRING, (int *)NULL, "notebook file"},
    {"notebook_on", FLAG, (int *)NULL, "store important data in notebook"},
    {"verbosity", INT, (int *)NULL, "log file verbosity"},
    {"verror", INT, (int *)NULL, "error file verbosity"},
    {"no_train_verror", FLAG, (int *)NULL, "no training error file"},
};
#define TABSIZE (sizeof(reportstab)/sizeof(ARG))

static void
reports_set_defaults_variables(dp)
     ReportsParams *dp;
{
    nnargs_set_variable("exp_path", (int *)&dp->exp_path, reportstab, TABSIZE);
    nnargs_set_variable("notebook_fname", (int *)&dp->notebook_fname,
			reportstab, TABSIZE);
    nnargs_set_variable("notebook_on", &dp->notebook_on, reportstab, TABSIZE);
    nnargs_set_variable("verbosity", &dp->verbosity, reportstab, TABSIZE);
    nnargs_set_variable("verror", &dp->verror, reportstab, TABSIZE);
    nnargs_set_variable("no_train_verror", &dp->no_train_verror,
			reportstab, TABSIZE);
}

void
read_reports_defaults(argc, argv)
     int argc;
     char **argv;
{
    /* clear input_labels and class_labels, which are not there if they
       were NULL before */
    if (reports_params.exp_path != NULL){
	XtFree(reports_params.exp_path);
	reports_params.exp_path = NULL;
    }
    if (reports_params.notebook_fname != NULL){
	XtFree(reports_params.notebook_fname);
	reports_params.notebook_fname = NULL;
    }
    reports_params.notebook_on = 0;
    reports_params.no_train_verror = 0;

    reports_set_defaults_variables(&reports_params);

    if(argc > 0){
	argc = nnargs(argc, argv, reportstab, TABSIZE);
	/* copy the strings into new memory */
	/* Linda Sue checks that this is a valid directory ... */
	reports_params.exp_path = XtNewString(reports_params.exp_path);
	reports_params.notebook_fname = XtNewString(reports_params.notebook_fname);
    } else {
	reports_params.exp_path = XtNewString("./");
	reports_params.notebook_fname = XtNewString("LNKnet.note");
	reports_params.notebook_on = 1;
	reports_params.no_train_verror = 1;
	reports_params.verbosity = 3;
	reports_params.verror = 0;
    }
    /* put together the notebook path */
    nn_free(reports_params.notebook_path,"");
    reports_params.notebook_path =
      path_create(reports_params.working_directory,
		  reports_params.notebook_fname);
    create_experiment_file_names();
    reports_request("display");
}

void
write_reports_defaults(default_file)
     FILE *default_file;
{
    reports_set_defaults_variables(&reports_params);

    if(default_file != NULL){
	fprintf(default_file, "reports %s",
		nnargs_get_flags(reportstab, TABSIZE));
    }
}

void
create_experiment_file_names()
{
    /* shell script name */
    XtFree(reports_params.shell_fname);
    reports_params.shell_fname = XtNewString(lnk_params.exp_name);
    reports_params.shell_fname =
      StringConcat(reports_params.shell_fname, ".run");
    show_string_label(ReportsClone->shellFileLabel,
		      reports_params.shell_fname);
    nn_free(reports_params.shell_path,"");
    reports_params.shell_path = 
      path_create(reports_params.exp_path, reports_params.shell_fname);

    /* log file */
    XtFree(reports_params.log_fname);
    reports_params.log_fname = XtNewString(lnk_params.exp_name);
    reports_params.log_fname = StringConcat(reports_params.log_fname, ".log");
    show_string_label(ReportsClone->logFileLabel, reports_params.log_fname);
    print_set_log_status();

    /* parameter file name */
    XtFree(reports_params.param_fname);
    reports_params.param_fname = XtNewString( lnk_params.exp_name);
    reports_params.param_fname =
      StringConcat(reports_params.param_fname, ".param");
    show_string_label(ReportsClone->paramFileLabel,
		      reports_params.param_fname);
    
    /* error name prefix */
    XtFree(reports_params.error_fname);
    reports_params.error_fname = XtNewString(lnk_params.exp_name);
    reports_params.error_fname =
      StringConcat(reports_params.error_fname, ".err");
    show_string_label(ReportsClone->errorFileLabel,
		      reports_params.error_fname);
    
    /* screen file */
    XtFree(reports_params.screen_fname);
    reports_params.screen_fname = XtNewString (lnk_params.exp_name);
    reports_params.screen_fname =
      StringConcat(reports_params.screen_fname, ".screen");
    show_string_label(ReportsClone->screenFileLabel,
		      reports_params.screen_fname);
    nn_free(reports_params.screen_path,"");
    reports_params.screen_path =
      path_create(reports_params.exp_path, reports_params.screen_fname);
}

static void
display_verror()
{
    show_option_value( ReportsClone->verrorOptionMenu, reports_params.verror);

    show_toggle_value (ReportsClone->noTrainVerrorToggle,
		       reports_params.no_train_verror);

}

void
plots_set_verror()
{
    int change = 0;

    if((plots_params.detect || plots_params.reject || plots_params.prob)
       && reports_params.verror < 2){
	reports_params.verror = 2;
	change = 1;
    }
    if((plots_params.cost || plots_params.perr) && reports_params.verror < 1){
	reports_params.verror = 1;
	change = 1;
    }
    if((plots_params.cost || plots_params.perr)
       && reports_params.no_train_verror){
	reports_params.no_train_verror = 0;
	change = 1;
    }
    if (change) display_verror();
}

int
chdir_and_execute_command(String shell_fname)
{
    int pgrp;

    if(chdir(reports_params.exp_path))
      warning("couldn't change directory to %s", reports_params.exp_path);
    pgrp = execute_command_file(shell_fname);
    if(chdir(reports_params.working_directory))
      warning("couldn't change directory to %s", reports_params.working_directory);
    return pgrp;
}

/*********************************************************************
 module: set_temp_exp_path, restore_exp_path
 set_temp_exp_path sets lnk_params.exp_path to $loc for use in shell scripts
      and puts the set command in the shell script
 restore_exp_path returns lnk_params.exp_path to its old value
*********************************************************************/
static char *exp_path_tmp;
void
set_temp_exp_path(FILE *shell_file)
{
    exp_path_tmp = reports_params.exp_path;
    reports_params.exp_path = "$loc";
    
    /* print out variable necessary for running remotely */
    fprintf(shell_file, "set loc=`pwd`\n");
}

void
restore_exp_path()
{
    reports_params.exp_path = exp_path_tmp;
}
