/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/select/selectGen_callback.c,v 11.7 2001/08/23 20:17:57 craig Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            SELECTGEN                                 ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  13-Oct-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <nnlib.h>
#include <Xm/Xm.h>
#include "selectGen.h"
/*
 * External Functions
 */
#include "../main/lnknet_subs.h"
#include "../lnk/lnk_func.h"
#include "select_func.h"
#include "../reports/reports_func.h"

/*
 * Externals
 */
extern SelectGenWidgets *SelectGenClone;
#include "select_params.h"
extern SelectParams select_params;
#include "../norm/norm_params.h"
extern NormParams norm_params;
#include "../data/data_params.h"
extern DataParams data_params;
#include "../lnk/lnk_params.h"
extern LnkParams lnk_params;
#include "../reports/reports_params.h"
extern ReportsParams reports_params;

/*
 * Statics
 */
static void select_set_plot_label();
static void select_set_common_flags();
static void select_run();
static int store_select_commands();
static void select_store_run_command();
static void store_select_plot_command();
static void selectGen_update_filenames();
static void select_store_notebook(FILE *shell_file);

/* variables for shell scripts */
#define SELECT_PROGRAM_NAME "feat_sel"
#define PLOT_SELECT "plot_select"
#include <feat_sel_args.h>
#include <gen_search_args.h>
#include <plot_select_args.h>

/******************************************************************************
 ******************************************************************************
 *
 * Module Name: selectGen_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   selectGen_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  13-Oct-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
selectGen_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    SelectGenWidgets *ip;
    static int nFolds_buttons_mapped = False;
    static int genSearchN_buttons_mapped = False;
    static int rocTarget_buttons_mapped = False;
    static int xmin_buttons_mapped = False;
    static int xmax_buttons_mapped = False;
    static int xstep_buttons_mapped = False;
    static int ymin_buttons_mapped = False;
    static int ymax_buttons_mapped = False;
    static int ystep_buttons_mapped = False;
    
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
    
    if (!strcmp(client, "searchAlgNNToggle") ||	
	!strcmp(client, "searchAlgAlgToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "selectGen_callback: %s %s\n", client,
	  (state->set) ? "on", "off" );*/
	if(state->set){
	    select_params.searchAlg =
	      get_radio_value(ip->searchAlgRadioBox);
	    gray_out_item(ip->algorithmForm, !select_params.searchAlg);
	    gray_out_item(ip->searchCriterionFrame,
			  (!select_params.searchAlg ||
			   !alg_is_continuous_outputs()));
	    select_set_plot_label();
	}
	
    } else if (!strcmp(client, "algorithmParamsPBtn")) {
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	show_algorithm_popup();
	
    } else if (!strcmp(client, "nFoldsText")) {
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	nFolds_buttons_mapped =
	  okTextValueChanged(ip->nFoldsText, ip->nFoldsOKPBtn,
			     ip->nFoldsCancelPBtn,
			     ip->selectGenDialogShell,
			     nFolds_buttons_mapped,
			     select_params.nFolds_string);
	
    } else if ((!strcmp(client, "nFoldsTextAct")) ||
	       (!strcmp(client, "nFoldsOKPBtn"))) {
	String nFoldsTextval;

	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	nFolds_buttons_mapped =
	  okTextActivate(ip->nFoldsText, ip->nFoldsOKPBtn,
			 ip->nFoldsCancelPBtn, &nFoldsTextval);
	/* I won't actually be using this copy */
	XtFree (nFoldsTextval);
	
	/* verify the string as an integer */
	select_params.nFolds =
	  set_atoi_value(ip->nFoldsText, "nFolds_set",
			     select_params.nFolds);

	/* save the current text for future comparisons */
	XtFree(select_params.nFolds_string);
	select_params.nFolds_string = get_string_text(ip->nFoldsText);
 
	/* do any other things needed */
	select_set_plot_label();

    } else if (!strcmp(client, "nFoldsCancelPBtn")) {
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	nFolds_buttons_mapped = 
	  okTextCancel(ip->nFoldsText, ip->nFoldsOKPBtn,
		       ip->nFoldsCancelPBtn,
		       select_params.nFolds_string);
	
    } else if (!strcmp(client, "directionForwardToggle") ||
	       !strcmp(client, "directionBackwardToggle") ||
	       !strcmp(client, "directionForBackToggle") ||
	       !strcmp(client, "directionFloatingToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "selectGen_callback: %s %s\n", client,
	  (state->set) ? "on", "off" );*/
	if(state->set) select_params.searchDirection =
	  get_radio_value(ip->directionRadioBox);
	g_create_search_filename();
	selectGen_update_filenames();
	show_string_label(SelectGenClone->listFileLabel,
			  select_params.listFile);
	show_string_label(SelectGenClone->scriptLabel,
			  select_params.scriptFname);
	show_string_label(SelectGenClone->logLabel,
			  select_params.logFname);
	show_string_label(SelectGenClone->plotFnameLabel,
			  select_params.plotFname);
	
	/* update the filenames displayed here !!! */
	select_set_plot_label();
	
    } else if (!strcmp(client, "stopEarlyAllToggle") ||
	       !strcmp(client, "stopEarlyStopToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "selectGen_callback: %s %s\n", client,
	  (state->set) ? "on", "off" );*/
	if(state->set){
	    select_params.stopEarly =
	      get_radio_value(ip->stopEarlyRadioBox);
	    gray_out_item(ip->genSearchNForm, !select_params.stopEarly);
	}
	
    } else if (!strcmp(client, "genSearchNText")) {
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	genSearchN_buttons_mapped =
	  okTextValueChanged(ip->genSearchNText, ip->genSearchNOKPBtn,
			     ip->genSearchNCancelPBtn,
			     ip->selectGenDialogShell,
			     genSearchN_buttons_mapped,
			     select_params.genSearchN_string);
	
    } else if ((!strcmp(client, "genSearchNTextAct")) ||
	       (!strcmp(client, "genSearchNOKPBtn"))) {
	String genSearchNTextval;

	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	genSearchN_buttons_mapped =
	  okTextActivate(ip->genSearchNText, ip->genSearchNOKPBtn,
			 ip->genSearchNCancelPBtn, &genSearchNTextval);
	/* I won't actually be using this copy */
	XtFree (genSearchNTextval);
	
	/* verify the string as an integer */
	select_params.genSearchN =
	  set_atoi_value(ip->genSearchNText, "genSearchN_set",
			     select_params.genSearchN);

	/* save the current text for future comparisons */
	XtFree(select_params.genSearchN_string);
	select_params.genSearchN_string = get_string_text(ip->genSearchNText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "genSearchNCancelPBtn")) {
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	genSearchN_buttons_mapped = 
	  okTextCancel(ip->genSearchNText, ip->genSearchNOKPBtn,
		       ip->genSearchNCancelPBtn,
		       select_params.genSearchN_string);
	
    } else if (!strcmp(client, "perrToggle") ||
	       !strcmp(client, "rocToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "selectGen_callback: %s %s\n", client,
	  (state->set) ? "on", "off" );*/
	if(state->set){
	    select_params.rocCost =
	      get_radio_value(ip->searchCriterionRadioBox);
	    gray_out_item(ip->rocTargetForm, !select_params.rocCost);
	}
    } else if (!strcmp(client, "rocTargetText")) {
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	rocTarget_buttons_mapped =
	  okTextValueChanged(ip->rocTargetText, ip->rocTargetOKPBtn,
			     ip->rocTargetCancelPBtn,
			     ip->selectGenDialogShell,
			     rocTarget_buttons_mapped,
			     select_params.rocTarget_string);
	
    } else if ((!strcmp(client, "rocTargetTextAct")) ||
	       (!strcmp(client, "rocTargetOKPBtn"))) {
	String rocTargetTextval;

	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	rocTarget_buttons_mapped =
	  okTextActivate(ip->rocTargetText, ip->rocTargetOKPBtn,
			 ip->rocTargetCancelPBtn, &rocTargetTextval);
	/* I won't actually be using this copy */
	XtFree (rocTargetTextval);
	
	/* verify the string as an integer */
	select_params.rocTarget =
	  set_atoi_value(ip->rocTargetText, "rocTarget_set",
			     select_params.rocTarget);

	/* save the current text for future comparisons */
	XtFree(select_params.rocTarget_string);
	select_params.rocTarget_string = get_string_text(ip->rocTargetText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "rocTargetCancelPBtn")) {
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	rocTarget_buttons_mapped = 
	  okTextCancel(ip->rocTargetText, ip->rocTargetOKPBtn,
		       ip->rocTargetCancelPBtn,
		       select_params.rocTarget_string);

    } else if (!strcmp(client, "genPlotToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "selectGen_callback: %s %s\n", client,
	  (state->set) ? "on", "off" );*/
	select_params.genPlot = state->set;
	gray_out_item(ip->plotForm, !select_params.genPlot);

    } else if (!strcmp(client, "autoscaleToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "window_callback: %s %s\n", client,
	  (state->set) ? "on", "off" );*/
	select_params.autoscale = state->set;
	gray_out_item(SelectGenClone->axesForm, state->set);
 
    } else if (!strcmp(client, "xminText")) {
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	xmin_buttons_mapped =
	  okTextValueChanged(ip->xminText, ip->xminOKPBtn,
			     ip->xminCancelPBtn,
			     ip->selectGenDialogShell,
			     xmin_buttons_mapped,
			     select_params.xmin_string);
      
    } else if (!strcmp(client, "xminTextAct") ||
	       !strcmp(client, "xminOKPBtn")) {
	String xminTextval;

	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	xmin_buttons_mapped = okTextActivate(ip->xminText, ip->xminOKPBtn,
					     ip->xminCancelPBtn, &xminTextval);

	XtFree (xminTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	select_params.xmin = set_atof_value(ip->xminText, "selectGen_xmin",
					    select_params.xmin);
	XtFree(select_params.xmin_string);
	select_params.xmin_string = get_string_text(ip->xminText);
 
    } else if (!strcmp(client, "xminCancelPBtn")) {
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	xmin_buttons_mapped = 
	  okTextCancel(ip->xminText, ip->xminOKPBtn,
		       ip->xminCancelPBtn,
		       select_params.xmin_string);
 
    } else if (!strcmp(client, "xmaxText")) {
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	xmax_buttons_mapped =
	  okTextValueChanged(ip->xmaxText, ip->xmaxOKPBtn,
			     ip->xmaxCancelPBtn,
			     ip->selectGenDialogShell,
			     xmax_buttons_mapped,
			     select_params.xmax_string);
      
    } else if (!strcmp(client, "xmaxTextAct") ||
	       !strcmp(client, "xmaxOKPBtn")) {
	String xmaxTextval;

	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	xmax_buttons_mapped = okTextActivate(ip->xmaxText, ip->xmaxOKPBtn,
					     ip->xmaxCancelPBtn, &xmaxTextval);
	
	XtFree (xmaxTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	select_params.xmax = set_atof_value(ip->xmaxText, "selectGen_xmax",
					  select_params.xmax);
	XtFree(select_params.xmax_string);
	select_params.xmax_string = get_string_text(ip->xmaxText);
 
    } else if (!strcmp(client, "xmaxCancelPBtn")) {
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	xmax_buttons_mapped = 
	  okTextCancel(ip->xmaxText, ip->xmaxOKPBtn,
		       ip->xmaxCancelPBtn,
		       select_params.xmax_string);
 
    } else if (!strcmp(client, "xstepText")) {
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	xstep_buttons_mapped =
	  okTextValueChanged(ip->xstepText, ip->xstepOKPBtn,
			     ip->xstepCancelPBtn,
			     ip->selectGenDialogShell,
			     xstep_buttons_mapped,
			     select_params.xstep_string);
      
    } else if (!strcmp(client, "xstepTextAct") ||
	       !strcmp(client, "xstepOKPBtn")) {
	String xstepTextval;
      
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	xstep_buttons_mapped =
	  okTextActivate(ip->xstepText, ip->xstepOKPBtn,
			 ip->xstepCancelPBtn, &xstepTextval);

	XtFree (xstepTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	select_params.xstep = set_atof_value(ip->xstepText, "selectGen_xstep",
					   select_params.xstep);
	XtFree(select_params.xstep_string);
	select_params.xstep_string = get_string_text(ip->xstepText);
 
    } else if (!strcmp(client, "xstepCancelPBtn")) {
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	xstep_buttons_mapped = 
	  okTextCancel(ip->xstepText, ip->xstepOKPBtn,
		       ip->xstepCancelPBtn,
		       select_params.xstep_string);
	
    } else if (!strcmp(client, "yminText")) {
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	ymin_buttons_mapped =
	  okTextValueChanged(ip->yminText, ip->yminOKPBtn,
			     ip->yminCancelPBtn,
			     ip->selectGenDialogShell,
			     ymin_buttons_mapped,
			     select_params.ymin_string);
      
    } else if (!strcmp(client, "yminTextAct") ||
	       !strcmp(client, "yminOKPBtn")) {
	String yminTextval;
      
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	ymin_buttons_mapped = okTextActivate(ip->yminText, ip->yminOKPBtn,
					     ip->yminCancelPBtn, &yminTextval);

	XtFree (yminTextval);/* I won't actually be using this copy */
	
	/* check that this value is ok */
	select_params.ymin = set_atof_value(ip->yminText, "selectGen_ymin",
					  select_params.ymin);
	XtFree(select_params.ymin_string);
	select_params.ymin_string = get_string_text(ip->yminText);
	
    } else if (!strcmp(client, "yminCancelPBtn")) {
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	ymin_buttons_mapped = 
	  okTextCancel(ip->yminText, ip->yminOKPBtn,
		       ip->yminCancelPBtn,
		       select_params.ymin_string);
	
    } else if (!strcmp(client, "ymaxText")) {
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	ymax_buttons_mapped =
	  okTextValueChanged(ip->ymaxText, ip->ymaxOKPBtn,
			     ip->ymaxCancelPBtn,
			     ip->selectGenDialogShell,
			     ymax_buttons_mapped,
			     select_params.ymax_string);
      
    } else if (!strcmp(client, "ymaxTextAct") ||
	       !strcmp(client, "ymaxOKPBtn")) {
	String ymaxTextval;
      
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	ymax_buttons_mapped = okTextActivate(ip->ymaxText, ip->ymaxOKPBtn,
					     ip->ymaxCancelPBtn, &ymaxTextval);

	XtFree (ymaxTextval);/* I won't actually be using this copy */

	/* check that this value is ok */
	select_params.ymax = set_atof_value(ip->ymaxText, "selectGen_ymax",
					  select_params.ymax);
	XtFree(select_params.ymax_string);
	select_params.ymax_string = get_string_text(ip->ymaxText);
 
    } else if (!strcmp(client, "ymaxCancelPBtn")) {
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	ymax_buttons_mapped = 
	  okTextCancel(ip->ymaxText, ip->ymaxOKPBtn,
		       ip->ymaxCancelPBtn,
		       select_params.ymax_string);
 
    } else if (!strcmp(client, "ystepText")) {
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	ystep_buttons_mapped =
	  okTextValueChanged(ip->ystepText, ip->ystepOKPBtn,
			     ip->ystepCancelPBtn,
			     ip->selectGenDialogShell,
			     ystep_buttons_mapped,
			     select_params.ystep_string);
      
    } else if (!strcmp(client, "ystepTextAct") ||
	       !strcmp(client, "ystepOKPBtn")) {
	String ystepTextval;
      
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	ystep_buttons_mapped =
	  okTextActivate(ip->ystepText, ip->ystepOKPBtn,
			 ip->ystepCancelPBtn, &ystepTextval);

	XtFree (ystepTextval);/* I won't actually be using this copy */
	
	/* check that this value is ok */
	select_params.ystep = set_atof_value(ip->ystepText, "selectGen_ystep",
					   select_params.ystep);
	XtFree(select_params.ystep_string);
	select_params.ystep_string = get_string_text(ip->ystepText);
 
    } else if (!strcmp(client, "ystepCancelPBtn")) {
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	ystep_buttons_mapped = 
	  okTextCancel(ip->ystepText, ip->ystepOKPBtn,
		       ip->ystepCancelPBtn,
		       select_params.ystep_string);

    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	select_check_file();
	close_popup(ip->selectGenForm);
	
    } else if (!strcmp(client, "runPBtn")) {
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	select_run(EXP_AND_PLOT);
	
    } else if (!strcmp(client, "plotOnlyPBtn")) {
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	select_run(PLOT_ONLY);

    } else if (!strcmp(client, "stopPBtn")) {
	/*fprintf (stderr, "selectGen_callback: %s\n", client );*/
	stop_command_file(select_params.pid);
	select_params.pid = 0;

    } else {
	fprintf(stderr, "selectGen_callback: unknown client: %s\n", (char *)client);
    }
}

void
selectGen_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(SelectGenClone->selectGenForm);
    } else if (!strcmp(request, "init")){
	char string[10];

	if(select_params.searchAlgActive){
	    gray_out_item(SelectGenClone->searchAlgRadioBox, False);
	    gray_out_item(SelectGenClone->algorithmForm,
			  !select_params.searchAlg);
	} else {
	    gray_out_item(SelectGenClone->searchAlgRadioBox, True);
	    gray_out_item(SelectGenClone->algorithmForm, True);
	}

	/* build generate window file names */
	selectGen_update_filenames();
	
	XtFree(select_params.nFolds_string);
	sprintf(string, "%d", select_params.nFolds);
	select_params.nFolds_string = XtNewString(string);

	/* axes scales */
	sprintf(string, "%5.2g", select_params.xmin);
	XtFree(select_params.xmin_string);
	select_params.xmin_string = XtNewString(string);

	sprintf(string, "%5.2g", select_params.xmax);
	XtFree(select_params.xmax_string);
	select_params.xmax_string = XtNewString(string);

	sprintf(string, "%5.2g", select_params.xstep);
	XtFree(select_params.xstep_string);
	select_params.xstep_string = XtNewString(string);

	sprintf(string, "%5.2g", select_params.ymin);
	XtFree(select_params.ymin_string);
	select_params.ymin_string = XtNewString(string);

	sprintf(string, "%5.2g", select_params.ymax);
	XtFree(select_params.ymax_string);
	select_params.ymax_string = XtNewString(string);

	sprintf(string, "%5.2g", select_params.ystep);
	XtFree(select_params.ystep_string);
	select_params.ystep_string = XtNewString(string);

	select_set_plot_label();

    } else if (!strcmp(request, "display")){
	char string[10];
	show_string_label(SelectGenClone->listFileLabel,
			  select_params.listFile);
	show_string_label(SelectGenClone->scriptLabel,
			  select_params.scriptFname);
	show_string_label(SelectGenClone->logLabel,
			  select_params.logFname);
	show_radio_value(SelectGenClone->searchAlgRadioBox,
			 select_params.searchAlg);
	gray_out_item(SelectGenClone->algorithmForm, !select_params.searchAlg);
	show_string_label(SelectGenClone->algorithmLabel,
			  lnk_params.classifier_string);

	show_string_text(SelectGenClone->nFoldsText,
			 select_params.nFolds_string);
	show_radio_value(SelectGenClone->directionRadioBox,
			 select_params.searchDirection);
	show_radio_value(SelectGenClone->stopEarlyRadioBox,
			 select_params.stopEarly);
	if(select_params.stopEarly){
	    gray_out_item(SelectGenClone->genSearchNForm, False);
	} else {
	    gray_out_item(SelectGenClone->genSearchNForm, True);
	}
	XtFree(select_params.genSearchN_string);
	sprintf(string, "%d", select_params.genSearchN);
	select_params.genSearchN_string = XtNewString(string);
	show_string_text(SelectGenClone->genSearchNText,
			 select_params.genSearchN_string);

	if(alg_is_continuous_outputs() && (select_params.searchAlg != NN)){
	  gray_out_item(SelectGenClone->searchCriterionFrame, FALSE);
	} else {
	  gray_out_item(SelectGenClone->searchCriterionFrame, TRUE);
	}
	show_radio_value(SelectGenClone->searchCriterionRadioBox,
			 select_params.rocCost);
	if(select_params.rocCost){
	  gray_out_item(SelectGenClone->rocTargetForm, False);
	} else {
	  gray_out_item(SelectGenClone->rocTargetForm, True);
	}
	XtFree(select_params.rocTarget_string);
	sprintf(string, "%d", select_params.rocTarget);
	select_params.rocTarget_string = XtNewString(string);
	show_string_text(SelectGenClone->rocTargetText,
			 select_params.rocTarget_string);
	
	show_toggle_value(SelectGenClone->genPlotToggle,
			  select_params.genPlot);
	gray_out_item(SelectGenClone->plotForm, !select_params.genPlot);

	show_string_label(SelectGenClone->plotFnameLabel,
			  select_params.plotFname);
	/* axes scales */
	show_toggle_value (SelectGenClone->autoscaleToggle,
			   select_params.autoscale);
	gray_out_item(SelectGenClone->axesForm, select_params.autoscale);
	show_string_text (SelectGenClone->xminText,
			  select_params.xmin_string);
	show_string_text (SelectGenClone->xmaxText,
			  select_params.xmax_string);
	show_string_text (SelectGenClone->yminText,
			  select_params.ymin_string);
	show_string_text (SelectGenClone->ymaxText,
			  select_params.ymax_string);
	show_string_text (SelectGenClone->xstepText,
			  select_params.xstep_string);
	show_string_text (SelectGenClone->ystepText,
			  select_params.ystep_string);
	show_string_label(SelectGenClone->plotLabelLabel,
			  select_params.plotLabel);

    } else if (!strcmp(request, "alg_select_on")){
	select_params.searchAlgActive = True;
	
    } else if (!strcmp(request, "alg_select_off")){
	select_params.searchAlgActive = False;

    } else {
	fprintf(stderr, "selectGen_request: unknown request: %s\n", request);
    }
}

static void
select_set_plot_label()
{
    XtFree(select_params.plotLabel);
    switch(select_params.searchDirection){
      case FORWARD:
	select_params.plotLabel = XtNewString("Forward Search: Norm:");
	select_params.plotLabel =
	  StringConcat(select_params.plotLabel,
		       norm_description(norm_params.normalize));
	break;
      case BACKWARD:
	select_params.plotLabel = XtNewString("Backward Search: Norm:");
	select_params.plotLabel =
	  StringConcat(select_params.plotLabel,
		       norm_description(norm_params.normalize));
	break;
      case FOR_AND_BACK:
	select_params.plotLabel =
	  XtNewString(" Forward and Back Search: Norm:");
	select_params.plotLabel =
	  StringConcat(select_params.plotLabel,
		       norm_description(norm_params.normalize));
	break;
    }
    if(!select_params.searchAlgActive || select_params.searchAlg == NN)
      select_params.plotLabel =
	StringConcat(select_params.plotLabel, "NN leave-1-out");
    else{
	char string[10];
	select_params.plotLabel =
	  StringConcat(select_params.plotLabel, lnk_params.classifier_name);
	sprintf(string, "%d", select_params.nFolds);
	select_params.plotLabel =
	  StringConcat(select_params.plotLabel, string);
	select_params.plotLabel =
	  StringConcat(select_params.plotLabel, "-Fold CV");
    }
    show_string_label(SelectGenClone->plotLabelLabel,
		      select_params.plotLabel);
}

static void
select_run(run_flag)
     int run_flag;
{
    if(overwrite_ok(SelectGenClone->selectGenForm,
		    path_build(reports_params.exp_path,
			       select_params.scriptFname))){
	if(store_select_commands(run_flag)) return;
	if(!lnk_params.store_only)
	  select_params.pid =
	    chdir_and_execute_command(select_params.scriptFname);
    }
}

static void
select_set_common_flags()
{
    /* feat_sel_args.h */
    exp_path = reports_params.exp_path;
    data_path = data_params.data_path;
    data_fname = data_params.train_fname;
    desc_fname = data_params.desc_fname;
    npatterns = data_params.npatterns_train;
    normalize = norm_params.normalize;
    norm_fname = norm_params.norm_fname;
    /* debug and k stay at default settings */
    /* gen_search_args */
    search_direction = select_params.searchDirection;
    stop_early = select_params.stopEarly;
    search_n = select_params.genSearchN;
    if(alg_is_continuous_outputs()){
      roc_cost = select_params.rocCost;
    } else {
      roc_cost = 0;
    }
    roc_target = select_params.rocTarget;
    search_verbosity = reports_params.verbosity;
    search_param_fname = select_params.listFile;
    /* plot_select_args */
    plot_exp_path = reports_params.exp_path;
    plot_error_fname = select_params.listFile;
    select_auto_scale_flag = select_params.autoscale;
    select_plot_fname = select_params.plotFname;
    select_xmin = select_params.xmin;
    select_xmax = select_params.xmax;
    select_xstep = select_params.xstep;
    select_ymin = select_params.ymin;
    select_ymax = select_params.ymax;
    select_ystep = select_params.ystep;
    /* add double quotes here to eliminate problems with spaces */
    select_plot_label = XtMalloc(strlen(select_params.plotLabel) + 3);
    sprintf(select_plot_label, "\"%s\"", select_params.plotLabel);
    mac_search_flag = 0;
    mac_searchlines_fname = NULL;
    /* set no_graphics below */
}

static int
store_select_commands(run_flag)
     int run_flag;
{
    FILE *shell_file;
    void set_temp_exp_path(), restore_exp_path();

    select_set_common_flags();
    shell_file =
      command_file_open(SelectGenClone->selectGenForm,
			path_build(reports_params.exp_path,
				   select_params.scriptFname));
    if(shell_file == NULL) return -1;

    /* print out experiment path stuff */
    set_temp_exp_path(shell_file);

    if(run_flag != PLOT_ONLY){
	char *full_param_path;
	fprintf(shell_file, "\n#feature selection\n");
	select_store_run_command(shell_file);
	if(lnk_params.store_only)
	  fprintf(shell_file, " >& %s\n", select_params.logFname);
	else
	  fprintf(shell_file, " |& nn_tee -h %s\n", select_params.logFname);
	/* store the result in the notebook file */
	if(reports_params.notebook_on){
	  select_store_notebook(shell_file);
	}
	/* copy param file to data directory */
	/* I used to ask first, before doing the copy */
	/* this is done in two parts because path_build returns a static */
	/* if the data path is the same as the real experiment path,
	   this will not work */
	full_param_path=path_create(data_params.data_path,
				   select_params.listFile);
	fprintf(shell_file, "#check that the experiment directory is not\n");
	fprintf(shell_file, "#the same as the data directory,\n");
	fprintf(shell_file, "#then copy the search file to the data directory\n");
	fprintf(shell_file, "if (-e %s ) then \n", full_param_path);
	fprintf(shell_file, "  set src_i=`ls -i %s`\n",
		select_params.listFile);
	fprintf(shell_file, "  set dest_i=`ls -i %s`\n", full_param_path);
	fprintf(shell_file, "  if ($src_i[1] != $dest_i[1]) cp %s %s\n",
		select_params.listFile, full_param_path);
	fprintf(shell_file, "else\n");
	fprintf(shell_file, "  cp %s %s\n",
		select_params.listFile, full_param_path);
	fprintf(shell_file, "endif\n");

	XtFree(full_param_path);
    }
    /* store the plot command */
    if(select_params.genPlot || run_flag == PLOT_ONLY){
	fprintf(shell_file, "\n#feature selection plot\n");
	store_select_plot_command(shell_file);
    }
    restore_exp_path();

    command_file_close(shell_file,
		       path_build(reports_params.exp_path,
				  select_params.scriptFname),
		       "for select shell file");

    return 0;
}
static void
select_store_notebook(FILE *shell_file){

  FILE *fp;

  /* add changes to the notebook file */
  fp = fopen(reports_params.notebook_path, "a");
  if(fp == NULL){
    String entry;
    entry = XtNewString("LNKnet: cannot open notebook file ");
    entry = StringConcat(entry, reports_params.notebook_path);
    warningMessageDialog(SelectGenClone->selectGenForm, entry);
    XtFree(entry);
    return;
  }
  /* store the important things:
     search algorithm
     search direction
     search cost
  */
  fprintf(fp, "\n");
  fprintf(fp, "%s ", select_params.scriptFname);
  fprintf(fp, "%s ",
	  (!select_params.searchAlgActive || select_params.searchAlg == NN) ?
	  "NN":lnk_params.classifier_name);
  if(select_params.rocCost){
    fprintf(fp, "ROC: %d ", select_params.rocTarget);
  } else {
    fprintf(fp, "PERR");
  }
  fprintf(fp, "\n");

  fclose(fp);
  fprintf(shell_file, "echo -n \"%s \">> %s\n",
	  select_params.scriptFname, reports_params.notebook_path);
  fprintf(shell_file, "grep \"ALL\" %s >> %s\n",
	  select_params.logFname, reports_params.notebook_path);
  fprintf(shell_file, "echo -n \"%s \">> %s\n",
	  select_params.scriptFname, reports_params.notebook_path);
  fprintf(shell_file, "grep \"BEST\" %s >> %s\n",
	  select_params.logFname, reports_params.notebook_path);
}

static void
select_store_run_command(FILE *shell_file)
{
    int old_cv;
    ACTION_TYPE old_action;

    if(!select_params.searchAlgActive || select_params.searchAlg == NN){
	select_set_common_flags();
	fprintf(shell_file, "%s %s", SELECT_PROGRAM_NAME,
		nnargs_get_part_flags(argtab, TABSIZE));
	fprintf(shell_file, "%s",
		nnargs_get_part_flags(search_tab, SEARCH_TABSIZE));
    } else {
#define CV_TRAIN 4
	/* change this temporarily BEFORE set_common_flags is called */
	old_cv = lnk_params.cv_nfolds;
	old_action = lnk_params.action;

	lnk_params.cv_nfolds = select_params.nFolds;
	lnk_params.action = CROSS_VALID;

	set_common_flags(CV_TRAIN);

	lnk_params.cv_nfolds = old_cv;
	lnk_params.action = old_action;

	/* print out general args */
	fprintf(shell_file, "%s_%s %s", lnk_params.classifier_name,
		SELECT_PROGRAM_NAME, g_get_gen_class_args());
	/* print out classifier args */
	fprintf(shell_file, "%s", g_get_alg_args());
	/* if this algorithm uses clustering, store KMEANS arguments */
	if(alg_uses_clustering()){
	    ALGORITHM_TYPE temp_alg;
	    temp_alg = g_get_alg();
	    g_set_alg(KMEANS);
	    fprintf(shell_file, "%s", g_get_alg_args());
	    g_set_alg(temp_alg);
	}
	/* print out feat_search args */
	fprintf(shell_file, "%s",
		nnargs_get_part_flags(search_tab, SEARCH_TABSIZE));
    }
}

static void
store_select_plot_command(shell_file)
     FILE *shell_file;
{
    char *plot_command_line;
    select_set_common_flags();
    plot_exp_path = reports_params.exp_path;

    if(lnk_params.store_only) no_graphics = 1;
    else no_graphics = 0;

    if(lnk_params.store_only) fprintf(shell_file, "(time ");
    plot_command_line = XtNewString(PLOT_SELECT);
    plot_command_line = StringConcat(plot_command_line, " ");
    plot_command_line =
      StringConcat(plot_command_line,
		   nnargs_get_flags(select_argtab, SELECT_TABSIZE));
    /* replace the last \n with a space */
    *strrchr(plot_command_line, '\n')  = ' ';
    fprintf(shell_file, "%s", plot_command_line);
    if(lnk_params.store_only)
      fprintf(shell_file, "\\\n) >>& %s\n", select_params.logFname);
    else fprintf(shell_file, "\n");
}

static void
selectGen_update_filenames()
{
    char *temp_string, *strend;

    /* first take the .param off the name, if it is there */
    temp_string = XtNewString(select_params.listFile);
#if HAVE_STRSTR
    if(NULL != (strend = strstr(temp_string, ".param")))
      strend[0] = '\0';
#else
    if(NULL != (strend = strstr(temp_string, ".param")))
      strend[0] = '\0';
#endif
    /* now build the names */
    XtFree(select_params.scriptFname);
    select_params.scriptFname = XtNewString(temp_string);
    select_params.scriptFname = StringConcat(select_params.scriptFname,
					     ".run");
    XtFree(select_params.logFname);
    select_params.logFname = XtNewString(temp_string);
    select_params.logFname = StringConcat(select_params.logFname,
					  ".log");
    XtFree(select_params.plotFname);
    select_params.plotFname = XtNewString(temp_string);
    select_params.plotFname = StringConcat(select_params.plotFname,
					   ".plot");
    XtFree(temp_string);
}
