/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/select/select_callback.c,v 11.8 2004/01/15 19:32:12 kukolich Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            SELECT                              ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  10-Oct-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <Xm/Xm.h>
#include <nnlib.h>
#include "select.h"
/*
 * External Functions
 */
#include "select_func.h"
#include "../main/lnknet_subs.h"
#include "../lnk/lnk_func.h"
 
/*
 * Externals
 */
#include "select_params.h"
SelectParams select_params;
extern SelectWidgets *SelectClone;
#include "../data/data_params.h"
extern DataParams data_params;
#include "../norm/norm_params.h"
extern NormParams norm_params;
#include "../main/lnknet.h"
extern LnkGlyphs *LnknetClone;/* for warning glyphs */

/*
 * Statics
 */
static void select_set_defaults_variables(SelectParams *dp);
static void set_select_type(SELECT_TYPE);
static void select_error(String);
static void select_clear_error();
static void set_select_n();
static void set_feature_list();
static int g_read_search_file();
static void copy_search_feature_list();
static char *get_feature_labels();
static void set_firstN_string();
static void set_searchN_string();
static void set_featureList_string();

/******************************************************************************
 ******************************************************************************
 *
 * Module Name: select_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   select_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  10-Oct-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
select_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    SelectWidgets *ip;
    static int firstN_buttons_mapped = False;
    static int featureList_buttons_mapped = False;
    static int listFile_buttons_mapped = False;
    static int searchN_buttons_mapped = False;

    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);

    if (!strcmp(client, "selectTypeAllToggle") ||
    !strcmp(client, "selectTypeFirstToggle") ||
    !strcmp(client, "selectTypeHandToggle") ||
    !strcmp(client, "selectTypeReadToggle")) {
    /*fprintf (stderr, "select_callback: %s\n", client );*/
    XmToggleButtonCallbackStruct *state =
      (XmToggleButtonCallbackStruct *)acs;
    /*fprintf (stderr, "select_callback: %s %s\n", client,
      (state->set) ? "on", "off" );*/
    if(state->set)
      set_select_type(get_radio_value(ip->selectTypeRadioBox));

    } else if (!strcmp(client, "firstNText")) {
    /*fprintf (stderr, "select_callback: %s\n", client );*/
    firstN_buttons_mapped =
      okTextValueChanged(ip->firstNText, ip->firstNOKPBtn,
                 ip->firstNCancelPBtn,
                 ip->selectDialogShell,
                 firstN_buttons_mapped,
                 select_params.firstN_string);
    
    } else if ((!strcmp(client, "firstNTextAct")) ||
           (!strcmp(client, "firstNOKPBtn"))) {
    String firstNTextval;

    /*fprintf (stderr, "select_callback: %s\n", client );*/
    firstN_buttons_mapped =
      okTextActivate(ip->firstNText, ip->firstNOKPBtn,
             ip->firstNCancelPBtn, &firstNTextval);
    /* I won't actually be using this copy */
    XtFree (firstNTextval);
    
    /* verify the string as an integer */
    select_params.firstN =
      set_atoi_range_value(ip->firstNText, "firstN_set",
                   select_params.firstN, 0, data_params.nraw);
 
    /* do any other things needed */
    set_select_n(select_params.firstN);
    if(select_params.selectType == READ_FILE){
        copy_search_feature_list();
    } else set_feature_labels_string();

    /* the value may have changed. redisplay it */
    select_params.firstN = select_params.ninputs;

    set_firstN_string();

    } else if (!strcmp(client, "firstNCancelPBtn")) {
    /*fprintf (stderr, "select_callback: %s\n", client );*/
    firstN_buttons_mapped = 
      okTextCancel(ip->firstNText, ip->firstNOKPBtn,
               ip->firstNCancelPBtn,
               select_params.firstN_string);

    } else if (!strcmp(client, "featureListText")) {
    /*fprintf (stderr, "select_callback: %s\n", client);*/
    /*this is the value changed callback,
      make the OK/CANCEL buttons appear */
    featureList_buttons_mapped =
      okTextValueChanged(ip->featureListText, ip->featureListOKPBtn,
                 ip->featureListCancelPBtn,
                 ip->selectDialogShell,
                 featureList_buttons_mapped,
                 select_params.featureList_string);
    } else if ((!strcmp(client, "featureListTextAct")) ||
           (!strcmp(client, "featureListOKPBtn"))) {
    /*fprintf (stderr, "select_callback: %s\n", client );*/
    String featureListTextval;
    featureList_buttons_mapped =
      okTextActivate(ip->featureListText,
             ip->featureListOKPBtn,
             ip->featureListCancelPBtn,
             &featureListTextval);
    
    if(select_params.featureList != NULL)
      deallocate_ivector(select_params.featureList);
    select_params.featureList = NULL;
    /* take spaces and tabs out of the list before handing it to nnargs */
    if(featureListTextval != NULL){
        char *featureListTemp = featureListTextval;
        featureListTextval = strtok(featureListTextval, " \t");
        if(featureListTextval != NULL){
        select_params.featureList =
          nnargs_get_int_alloc_list(featureListTextval);
        XtFree(featureListTemp);
        }
    }
    
    /* make a copy of the string as it has been interpretted */
    set_featureList_string();
    
    /* do anything else necessary */
    set_feature_list();
    
    } else if (!strcmp(client, "featureListCancelPBtn")) {
    /*fprintf (stderr, "select_callback: %s\n", client );*/
    featureList_buttons_mapped = 
      okTextCancel(ip->featureListText, ip->featureListOKPBtn,
               ip->featureListCancelPBtn,
               select_params.featureList_string);
 
    } else if (!strcmp(client, "listFileText")) {
    /*fprintf (stderr, "select_callback: %s\n", client );*/
    listFile_buttons_mapped =
      okTextValueChanged(ip->listFileText, ip->listFileOKPBtn,
                 ip->listFileCancelPBtn,
                 ip->selectDialogShell,
                 listFile_buttons_mapped,
                 select_params.listFile);
    
    } else if ((!strcmp(client, "listFileTextAct")) ||
           (!strcmp(client, "listFileOKPBtn"))) {
    String listFileTextval;

    /*fprintf (stderr, "select_callback: %s\n", client );*/
    listFile_buttons_mapped =
      okTextActivate(ip->listFileText, ip->listFileOKPBtn,
             ip->listFileCancelPBtn, &listFileTextval);
    
    XtFree(select_params.listFile);
    select_params.listFile =
      set_string_not_empty(ip->listFileText, "select.listFile",
                   listFileTextval);
    
    /* do any other things needed */
    if(select_params.selectType == READ_FILE){
        g_read_search_file();
    }
    

    } else if (!strcmp(client, "listFileCancelPBtn")) {
    /*fprintf (stderr, "select_callback: %s\n", client );*/
    listFile_buttons_mapped = 
      okTextCancel(ip->listFileText, ip->listFileOKPBtn,
               ip->listFileCancelPBtn,
               select_params.listFile);
    
  } else if (!strcmp(client, "searchLimitFirstNToggle") ||
         !strcmp(client, "searchLimitBestToggle")) {
    XmToggleButtonCallbackStruct *state =
      (XmToggleButtonCallbackStruct *)acs;
    /*fprintf (stderr, "select_callback: %s %s\n", client,
      (state->set) ? "on", "off" );*/
    if(state->set){
        select_params.searchLimit =
          get_radio_value(ip->searchLimitRadioBox);
        if(select_params.searchLimit){
        gray_out_item(SelectClone->searchNForm, True);
        set_select_n(select_params.bestN);
        } else {
        gray_out_item(SelectClone->searchNForm, False);
        set_select_n(select_params.searchN);
        }
        /* re-copy the first ninputs entries in search file feature list */
        copy_search_feature_list();
    }
 
    } else if (!strcmp(client, "searchNText")) {
    /*fprintf (stderr, "select_callback: %s\n", client );*/
    searchN_buttons_mapped =
      okTextValueChanged(ip->searchNText, ip->searchNOKPBtn,
                 ip->searchNCancelPBtn,
                 ip->selectDialogShell,
                 searchN_buttons_mapped,
                 select_params.searchN_string);
    
    } else if ((!strcmp(client, "searchNTextAct")) ||
           (!strcmp(client, "searchNOKPBtn"))) {
    String searchNTextval;

    /*fprintf (stderr, "select_callback: %s\n", client );*/
    searchN_buttons_mapped =
      okTextActivate(ip->searchNText, ip->searchNOKPBtn,
             ip->searchNCancelPBtn, &searchNTextval);
    /* I won't actually be using this copy */
    XtFree (searchNTextval);
    
    /* verify the string as an integer */
    select_params.searchN =
      set_atoi_range_value(ip->searchNText, "searchN_set",
                   select_params.searchN, 0, data_params.nraw);

    /* do any other things needed */
    set_select_n(select_params.searchN);
    if(select_params.selectType == READ_FILE){
        copy_search_feature_list();
    } else set_feature_labels_string();
    
    /* the value may have changed. redisplay it */
    select_params.searchN = select_params.ninputs;

    set_searchN_string();

    } else if (!strcmp(client, "searchNCancelPBtn")) {
    /*fprintf (stderr, "select_callback: %s\n", client );*/
    searchN_buttons_mapped = 
      okTextCancel(ip->searchNText, ip->searchNOKPBtn,
               ip->searchNCancelPBtn,
               select_params.searchN_string);

    } else if (!strcmp(client, "genPBtn")) {
    /*fprintf (stderr, "select_callback: %s\n", client );*/
    selectGen_request("init");
    selectGen_request("display");
    selectGen_request("open");
 
    } else if (!strcmp(client, "closePBtn")) {
    /*fprintf (stderr, "select_callback: %s\n", client );*/
    close_popup(SelectClone->selectForm);
    
    } else {
    fprintf(stderr, "select_callback: unknown client: %s\n", (char *)client);
    }
}

void
select_request(String request)
{
    if(!strcmp(request, "open")){
    open_popup(SelectClone->selectForm);
    } else if (!strcmp(request, "init")){
    select_params.ninputs = 0;
    select_params.features = NULL;
    select_params.selectType = ALL;
    select_params.firstN = 0;
    select_params.firstN_string = NULL;
    select_params.listFile = NULL;
    select_params.featureList = NULL;
    select_params.featureList_string = NULL;
    select_params.searchN = 0;
    select_params.searchN_string = NULL;
    select_params.searchLimit = 0;
    select_params.bestN = 0;
    select_params.searchFeatures = NULL;
    /* selectGen window */
    select_params.scriptFname = NULL;
    select_params.logFname = NULL;
    select_params.plotFname = NULL;
    select_params.searchAlg = NN;
    select_params.searchAlgActive = False;
    select_params.nFolds = 4;
    select_params.nFolds_string = NULL;
    select_params.stopEarly = 0;
    select_params.genSearchN = 0;
    select_params.genSearchN_string = NULL;
    select_params.searchDirection = FORWARD;
    select_params.genPlot = 0;
    select_params.autoscale = 0;
    select_params.xmin = 0.;
    select_params.xmin_string = NULL;
    select_params.xmax = 0.;
    select_params.xmax_string = NULL;
    select_params.xstep = 0.;
    select_params.xstep_string = NULL;
    select_params.ymin = 0.;
    select_params.ymin_string = NULL;
    select_params.ymax = 0.;
    select_params.ymax_string = NULL;
    select_params.ystep = 0.;
    select_params.ystep_string = NULL;

    } else if (!strcmp(request, "display")){
    show_radio_value(SelectClone->selectTypeRadioBox,
             select_params.selectType);
    show_radio_value(SelectClone->searchLimitRadioBox,
             select_params.searchLimit);
    set_firstN_string();
    set_searchN_string();
    show_string_text(SelectClone->listFileText,
             select_params.listFile);
    set_featureList_string();
    set_select_type(select_params.selectType);
    
    } else if (!strcmp(request, "set_ninputs")){
      if(select_params.selectType == ALL){
	set_select_n(data_params.nraw);
      } else {
	set_select_n(select_params.ninputs);
      }
    } else {
    fprintf(stderr, "select_request: unknown request: %s\n", request);
    }
}

static ARG selecttab[] =
{
    /* parameters on the norm window */
    {"selectType", INT, (int *)NULL, },
    {"firstN", INT, (int *)NULL,},
    {"listFile", STRING, (int *)NULL,},
    {"featureList", INT_ALLOC_LIST, (int *)NULL,},
    {"searchLimit", INT, (int *)NULL, },
    {"searchN", INT, (int *)NULL,},
    /* parameters on the generation window */
    {"cvSearch", FLAG, (int *)NULL, "use current alg for search, else NN"},
    {"nFolds", INT, (int *)NULL, "cv folds for cvSearch"},
    {"stopEarly", FLAG, (int *)NULL, "stop searches early"},
    {"genSearchN", INT, (int *)NULL, "stop when N feature found (removed)"},
    {"searchDirection", INT, (int *)NULL, },
    {"plot", FLAG, (int *)NULL, "generate plots"},
    {"autoscale", FLAG, (int *)NULL, "autoscale norm_plot"},
    {"xmin", FLOAT, (int *)NULL,},
    {"xmax", FLOAT, (int *)NULL,},
    {"xstep", FLOAT, (int *)NULL,},
    {"ymin", FLOAT, (int *)NULL,},
    {"ymax", FLOAT, (int *)NULL,},
    {"ystep", FLOAT, (int *)NULL,},
};
#define TABSIZE (sizeof(selecttab)/sizeof(ARG))

static void
select_set_defaults_variables(dp)
     SelectParams *dp;
{
    nnargs_set_variable("selectType", (int *)&dp->selectType, selecttab, TABSIZE);
    nnargs_set_variable("firstN", (int *)&dp->firstN, selecttab, TABSIZE);
    nnargs_set_variable("listFile", (int *)&dp->listFile, selecttab, TABSIZE);
    nnargs_set_variable("featureList", (int *)&dp->featureList, selecttab, TABSIZE);
    nnargs_set_variable("searchLimit", (int *)&dp->searchLimit, selecttab, TABSIZE);
    nnargs_set_variable("searchN", (int *)&dp->searchN, selecttab, TABSIZE);
    nnargs_set_variable("cvSearch", (int *)&dp->searchAlg, selecttab, TABSIZE);
    nnargs_set_variable("nFolds", (int *)&dp->nFolds, selecttab, TABSIZE);
    nnargs_set_variable("stopEarly", (int *)&dp->stopEarly, selecttab, TABSIZE);
    nnargs_set_variable("genSearchN", (int *)&dp->genSearchN, selecttab, TABSIZE);
    nnargs_set_variable("searchDirection", (int *)&dp->searchDirection, selecttab, TABSIZE);
    nnargs_set_variable("plot", (int *)&dp->genPlot, selecttab, TABSIZE);
    nnargs_set_variable("autoscale", (int *)&dp->autoscale, selecttab, TABSIZE);
    nnargs_set_variable("xmin", (int *)&dp->xmin, selecttab, TABSIZE);
    nnargs_set_variable("xmax", (int *)&dp->xmax, selecttab, TABSIZE);
    nnargs_set_variable("xstep", (int *)&dp->xstep, selecttab, TABSIZE);
    nnargs_set_variable("ymin", (int *)&dp->ymin, selecttab, TABSIZE);
    nnargs_set_variable("ymax", (int *)&dp->ymax, selecttab, TABSIZE);
    nnargs_set_variable("ystep", (int *)&dp->ystep, selecttab, TABSIZE);
}

void
read_select_defaults(argc, argv)
     int argc;
     char **argv;
{
    /* clear flags and strings which are not printed if they are NULL */
    if (select_params.listFile != NULL){
    XtFree(select_params.listFile);
    select_params.listFile = NULL;
    }
    if (select_params.featureList != NULL){
    deallocate_ivector(select_params.featureList);
    select_params.featureList = NULL;
    }
    select_params.searchAlg = False;
    select_params.stopEarly = False;
    select_params.genPlot = False;
    select_params.autoscale = False;

    select_set_defaults_variables(&select_params);

    if(argc > 0){
    argc = nnargs(argc, argv, selecttab, TABSIZE);
    /* copy the strings into new memory */
    select_params.listFile = XtNewString(select_params.listFile);
    /* the lists are properly copied into new space by nnargs */
    } else {
    /* set any defaults */
    select_params.listFile = XtNewString("listFile");
    select_params.searchAlg = NN;
    select_params.searchDirection = FORWARD;
    select_params.genSearchN = 0;
    select_params.stopEarly = 0;
    select_params.autoscale = 1;
    select_params.xmin = 0;
    select_params.xmax = 100.;
    select_params.xstep = 10.;
    select_params.ymin = 0;
    select_params.ymax = 100.;
    select_params.ystep = 10.;
    }

    select_request("display");
}

void
write_select_defaults(default_file)
     FILE *default_file;
{
    select_set_defaults_variables(&select_params);

    if(default_file != NULL){
    fprintf(default_file, "select %s",
        nnargs_get_flags(selecttab, TABSIZE));
    }
}

void
activate_features(int onOff)
{
    if(onOff){
    gray_out_item(SelectClone->selectTypeRadioBox, False);
    } else {
    gray_out_item(SelectClone->selectTypeRadioBox, True);
    }
}

static void
set_select_type(SELECT_TYPE value)
{
    select_params.selectType = value;

    gray_out_item(SelectClone->featureListForm, TRUE);
    gray_out_item(SelectClone->firstNForm, TRUE);
    gray_out_item(SelectClone->searchNForm, TRUE);
    gray_out_item(SelectClone->listFileForm, TRUE);
    gray_out_item(SelectClone->searchLimitLabel, TRUE);
    gray_out_item(SelectClone->searchLimitRadioBox, TRUE);
    
    select_clear_error();
    switch(value){
      case ALL: /* all */
    set_select_n(data_params.nraw);
    select_params.features = NULL;
    set_feature_labels_string();
    break;
      case FIRST_N: /* first n */
    /* set ninputs to the value of what is stored in select_n */
    gray_out_item(SelectClone->firstNForm, FALSE);
    set_select_n(select_params.firstN);
    select_params.features = NULL;
    set_feature_labels_string();
    break;
      case MANUAL: /* fill in */
    gray_out_item(SelectClone->featureListForm, FALSE);
    select_params.ninputs = 0;
    set_feature_list();
    break;
      case READ_FILE: /* get from file */
    gray_out_item(SelectClone->searchLimitLabel, FALSE);
    gray_out_item(SelectClone->searchLimitRadioBox, FALSE);
    if(select_params.bestN == 0){
        gray_out_item(SelectClone->searchNForm, FALSE);
        set_select_n(select_params.searchN);
    }
    gray_out_item(SelectClone->listFileForm, FALSE);
    g_read_search_file();
    break;
      default: break;
    }
}

static void
select_error(string)
     char *string;
{
    show_string_label(SelectClone->errorLabel, string);
    show_image_label(SelectClone->warningGlyphLabel,
             LnknetClone->largeWarning);
    lnk_request("featureWarningOn");
}

static void
select_clear_error()
{
    /* clear error message if we have made it this far */
    show_string_label(SelectClone->errorLabel, "");
    show_image_label(SelectClone->warningGlyphLabel,
             LnknetClone->largeBlank);
    lnk_request("featureWarningOff");
}

static void
set_select_n(int value)
{
  select_params.ninputs = value;
  if(norm_params.normalize == LDA)
    if(select_params.ninputs > data_params.noutputs-1){
      select_params.ninputs = data_params.noutputs-1;
    }
  if(select_params.ninputs > data_params.nraw){
    select_params.ninputs = data_params.nraw;
  }
  g_update_ninputs();
}

/* called when data_params.nraw is changed */
void
update_nraw_and_ninputs()
{
    if(select_params.selectType == ALL)
      set_select_n(data_params.nraw);
    else
      set_select_n(select_params.ninputs);
    /* set all the N's to the new ninputs value and display the values*/
    select_params.searchN = select_params.firstN = select_params.ninputs;
    set_firstN_string();
    set_searchN_string();
    set_feature_labels_string();
}

static void
set_feature_list()
{
    int n;

    select_clear_error();
    select_params.features = select_params.featureList;
    if(select_params.features == NULL) return;

    /* check the values in the feature list while we're here */
    set_select_n(select_params.featureList[0]);
    if(select_params.featureList[0] > select_params.ninputs){
    /* take off the extra entries in feature list */
    select_params.featureList[0] = select_params.ninputs;
    set_featureList_string();
    }
    /* check all the features */
    for(n = 1; n <= select_params.ninputs; n++){
    if(select_params.featureList[n] < 0 ||
       select_params.featureList[n] >= data_params.nraw ||
       (norm_params.normalize == LDA &&
        select_params.featureList[n] >= data_params.noutputs)){
        select_error("Error: Bad feature in feature list");
        select_params.features = NULL;
        set_select_n(0);
        break;
    }
    }
    set_feature_labels_string();
}

static int
g_read_search_file()
{
    int rv, i;
    FILE *param_file;
    int nraw, *selected, best_n;
    float *errors;

    select_clear_error();

    /* check if the file is there, warn if not, returning -1 */
    param_file = fopen(path_build(data_params.data_path,
                  select_params.listFile), "r");
    if(param_file == NULL){
    select_error("Error: Feature list file does not exist");
    select_params.features = NULL;
    set_select_n(0);
    set_feature_labels_string();
    return -1;
    }
    rv = feat_sel_read_model(param_file, &nraw, &best_n, &selected, &errors);
    if(rv < 0){
    select_error("Error: Cannot read feature list file");
    select_params.features = NULL;
    set_select_n(0);
    set_feature_labels_string();
    return -1;
    }

    /* set ninputs */
    select_params.bestN = best_n;
    if(select_params.searchLimit){
    set_select_n(select_params.bestN);
    } else {
    set_select_n(select_params.searchN);
    }

    /* copy all of the features */
    if(select_params.searchFeatures != NULL)
      deallocate_ivector(select_params.searchFeatures);
    select_params.searchFeatures = allocate_ivector(nraw+1);
    select_params.searchFeatures[0] = nraw;
    for(i = 0; i < nraw; i++){
    select_params.searchFeatures[i+1] = selected[i];
    }
    copy_search_feature_list();
    
    /* free allocated space */
    nn_fclose(param_file,
          path_build(data_params.data_path, select_params.listFile),
          "for feature selection parameter file");
    /* free space allocated in feat_sel_read_model */
    nn_free((char *)selected, "for features");
    nn_free((char *)errors, "for errors");
    return(0);
}

static void
copy_search_feature_list()
{
    int i;

    if(select_params.featureList != NULL)
      deallocate_ivector(select_params.featureList);
    select_params.featureList = allocate_ivector(select_params.ninputs+1);
    select_params.featureList[0] = select_params.ninputs;
    if(select_params.searchFeatures != NULL){
    for(i = 0; i < select_params.ninputs &&
        i < select_params.searchFeatures[0]; i++){
        select_params.featureList[i+1] = select_params.searchFeatures[i+1];
    }
    } else {
    select_params.featureList[0] = 0;
    }
    select_params.features = select_params.featureList;

    /* display the feature list */
    set_featureList_string();
    set_feature_labels_string();
}

void
set_feature_labels_string()
{
    char *feature_labels_string;
    void region_set_input_label(), profile_set_input_label();
    static int direction = 1;
    Dimension height, width;

    /* keep the window the same size instead of automatically growing
       as soon as the labels string gets longer */
    XtVaGetValues(SelectClone->selectForm, XmNheight, &height,
          XmNwidth, &width, NULL);
    /* printf("selectForm width:%d height:%d\n", width, height);*/

    feature_labels_string = get_feature_labels();
    show_string_label(SelectClone->featureLabels,
              feature_labels_string);
    XtFree(feature_labels_string);

    /* set the window width to the same as it was */
    XtVaSetValues(SelectClone->selectForm,
          XmNheight, height,
          XmNwidth, width+direction, NULL);
    direction *= -1;

    region_set_input_label();
    profile_set_input_label();
}

static char *
get_feature_labels()
/* build a list of selected feature labels */
{
    int labels_len, total_length, i;
    char small_string[8];
    char *labels_string;

    /* we cannot use the original labels for PCA and LDA,
       let's fake the system out by pretending there are no labels */
    if(norm_params.normalize == LDA || norm_params.normalize == PCA ||
       data_params.input_labels == NULL) labels_len = 0;
    else labels_len = atoi(data_params.input_labels[0]);

    switch(select_params.selectType){
    /* put together a list of the selected features */
      case ALL:
      case FIRST_N:
    if(labels_len == 0){
        /* just use 0,1,2 */
        labels_string = XtNewString("");
        for(i = 0; i < select_params.ninputs; i++){
        sprintf(small_string, "%d", i);
        labels_string = StringConcat(labels_string, small_string);
        labels_string = StringConcat(labels_string, ",");
        }
        /* take off the comma at the end */
        total_length = strlen(labels_string);
        if(total_length > 1) labels_string[total_length-1] = '\0';
        return labels_string;
    }
    /* use the first ninputs entries in the existing list, if it is
       long enough */
    if(labels_len < select_params.ninputs){
        /* labels length isn't long enough */
        labels_string =
          XtNewString("Not enough feature labels for this data base");
        return labels_string;
    } else if(labels_len == select_params.ninputs){
        labels_string = make_labels_string(data_params.input_labels);
        return labels_string;
    } else {
        /* temporarily change the list length to only ninputs */
        /* we know that ninputs will fit because it is less than
           labels_len */
        sprintf(data_params.input_labels[0], "%d", select_params.ninputs);
        labels_string = make_labels_string(data_params.input_labels);
        sprintf(data_params.input_labels[0], "%d", labels_len);
        return labels_string;
    }
      case MANUAL:
      case READ_FILE:
    if(select_params.features == NULL)
      return XtNewString("");
    total_length = 0;

    /* add up the length of the string to be created */
    for(i = 0; i < select_params.ninputs; i++){
        /* check for bad feature numbers */
        if(select_params.features[i+1] >= labels_len){
        if(labels_len != 0) {
            labels_string =
              XtNewString("Bad feature list or not enough labels");
            return labels_string;
        }
        }
    }

    labels_string = XtNewString("");
    if(labels_len > 0){
        int j;
        for(i = 0; i < select_params.ninputs; i++){
        j = select_params.features[i+1];
        labels_string = StringConcat(labels_string,
                         data_params.input_labels[j+1]);
        labels_string = StringConcat(labels_string, ",");
        }
    } else {
        for(i = 0; i < select_params.ninputs &&
        i < select_params.features[0]; i++){
        sprintf(small_string, "%d", select_params.features[i+1]);
        labels_string = StringConcat(labels_string, small_string);
        labels_string = StringConcat(labels_string, ",");
        }
    }
    /* take off the comma at the end */
    total_length = strlen(labels_string);
    if(total_length > 1) labels_string[total_length-1] = '\0';
    return labels_string;
    default:
      return NULL;
    }
}

static void
set_firstN_string()
{
    char string[10];
    XtFree(select_params.firstN_string);
    sprintf(string, "%d", select_params.firstN);
    select_params.firstN_string = XtNewString(string);
    show_string_text(SelectClone->firstNText,
             select_params.firstN_string);
}

static void
set_searchN_string()
{
    char string[10];
    XtFree(select_params.searchN_string);
    sprintf(string, "%d", select_params.searchN);
    select_params.searchN_string = XtNewString(string);
    show_string_text(SelectClone->searchNText,
             select_params.searchN_string);
}

static void
set_featureList_string()
{
    XtFree(select_params.featureList_string);
    select_params.featureList_string =
      make_int_string(select_params.featureList);
    show_string_text(SelectClone->featureListText,
             select_params.featureList_string);
}

void
select_check_file()
/* for data window */
{
    if(select_params.selectType == READ_FILE){
    g_read_search_file();
    }
}

void
g_create_search_filename()
{
    static char *search_string[] = {".for",".back",".for_bk","float"};
    static char *norm_char[] = {".N",".S",".P",".D"};

    XtFree(select_params.listFile);
    select_params.listFile = XtNewString(data_params.data_base);
    /* I will want to include the algorithm for cv versions */
    /* I decided not to... */
    select_params.listFile =
      StringConcat(select_params.listFile,
           search_string[select_params.searchDirection]);
    select_params.listFile =
      StringConcat(select_params.listFile, norm_char[norm_params.normalize]);

    select_params.listFile = StringConcat(select_params.listFile, ".param");
    show_string_text (SelectClone->listFileText, select_params.listFile);

    /* check for parameter file and update message */
    if(select_params.selectType == READ_FILE){
    g_read_search_file();
    }
}
