/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/struct/struct_callback.c,v 11.5 2001/09/13 20:39:50 kukolich Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                            STRUCT                              ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  2-Nov-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include <nnlib.h>
#include "struct.h"
/*
 * External Functions
 */
#include "../main/lnknet_subs.h"
#include "../plots/plots_func.h"
#include "../lnk/lnk_func.h"
#include "../print/print_func.h"

/*
 * Externals
 */
extern StructWidgets *StructClone;
#include "struct_params.h"
StructParams struct_params;/* this is the real one */
#include "../lnk/lnk_params.h"
extern LnkParams lnk_params;
#include "../data/data_params.h"
extern DataParams data_params;
#include "../reports/reports_params.h"
extern ReportsParams reports_params;

/*
 * Statics
 */
#include <plot_struct.h>
static void struct_set_read_variables();
static void struct_set_write_variables();
static void struct_set_common_flags();
 
/******************************************************************************
 ******************************************************************************
 *
 * Module Name: struct_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   struct_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  2-Nov-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
struct_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    StructWidgets *ip;
    static int maxLineWidth_buttons_mapped = False;
    static int thresh_buttons_mapped = False;
 
    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
    
    if (!strcmp(client, "autoscaleToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "struct_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	struct_params.autoscale = state->set;
      
    } else if (!strcmp(client, "noTextToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "struct_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	struct_params.noText = state->set;
	
    } else if (!strcmp(client, "showWeightToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "struct_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	struct_params.showWeight = state->set;
	gray_out_item(ip->weightForm, !struct_params.showWeight);
	
    } else if (!strcmp(client, "maxLineWidthText")) {
	/*fprintf (stderr, "struct_callback: %s\n", client );*/
	maxLineWidth_buttons_mapped =
	  okTextValueChanged(ip->maxLineWidthText, ip->maxLineWidthOKPBtn,
			     ip->maxLineWidthCancelPBtn,
			     ip->structDialogShell,
			     maxLineWidth_buttons_mapped,
			     struct_params.maxLineWidth_string);
	
    } else if ((!strcmp(client, "maxLineWidthTextAct")) ||
	       (!strcmp(client, "maxLineWidthOKPBtn"))) {
	String maxLineWidthTextval;

	/*fprintf (stderr, "struct_callback: %s\n", client );*/
	maxLineWidth_buttons_mapped =
	  okTextActivate(ip->maxLineWidthText, ip->maxLineWidthOKPBtn,
			 ip->maxLineWidthCancelPBtn, &maxLineWidthTextval);
	/* I won't actually be using this copy */
	XtFree (maxLineWidthTextval);
	
	/* verify the string as an integer */
	struct_params.maxLineWidth =
	  set_atoi_value(ip->maxLineWidthText, "maxLineWidth_set",
			     struct_params.maxLineWidth);

	/* save the current text for future comparisons */
	XtFree(struct_params.maxLineWidth_string);
	struct_params.maxLineWidth_string = get_string_text(ip->maxLineWidthText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "maxLineWidthCancelPBtn")) {
	/*fprintf (stderr, "struct_callback: %s\n", client );*/
	maxLineWidth_buttons_mapped = 
	  okTextCancel(ip->maxLineWidthText, ip->maxLineWidthOKPBtn,
		       ip->maxLineWidthCancelPBtn,
		       struct_params.maxLineWidth_string);
	
    } else if (!strcmp(client, "colorToggle") ||
	       !strcmp(client, "blackAndWhiteToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "struct_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set) struct_params.blackAndWhite =
	  get_radio_value(ip->blackAndWhiteRadioBox);
	
    } else if (!strcmp(client, "showAllToggle") ||
	       !strcmp(client, "useThreshToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "struct_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	if(state->set){
	    struct_params.useThresh =
	      get_radio_value(ip->useThreshRadioBox);
	    gray_out_item(ip->threshForm, !struct_params.useThresh);
	}

    } else if (!strcmp(client, "threshText")) {
	/*fprintf (stderr, "struct_callback: %s\n", client );*/
	thresh_buttons_mapped =
	  okTextValueChanged(ip->threshText, ip->threshOKPBtn,
			     ip->threshCancelPBtn,
			     ip->structDialogShell,
			     thresh_buttons_mapped,
			     struct_params.thresh_string);
	
    } else if ((!strcmp(client, "threshTextAct")) ||
	       (!strcmp(client, "threshOKPBtn"))) {
	String threshTextval;

	/*fprintf (stderr, "struct_callback: %s\n", client );*/
	thresh_buttons_mapped =
	  okTextActivate(ip->threshText, ip->threshOKPBtn,
			 ip->threshCancelPBtn, &threshTextval);
	/* I won't actually be using this copy */
	XtFree (threshTextval);
	
	/* verify the string as a float */
	struct_params.thresh =
	  set_atof_value(ip->threshText, "thresh_set",
			 struct_params.thresh);

	/* save the current text for future comparisons */
	XtFree(struct_params.thresh_string);
	struct_params.thresh_string = get_string_text(ip->threshText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "threshCancelPBtn")) {
	/*fprintf (stderr, "struct_callback: %s\n", client );*/
	thresh_buttons_mapped = 
	  okTextCancel(ip->threshText, ip->threshOKPBtn,
		       ip->threshCancelPBtn,
		       struct_params.thresh_string);
	
    } else if (!strcmp(client, "biasToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	/*fprintf (stderr, "struct_callback: %s %s\n", client,
	  (state->set) ? "on": "off" );*/
	struct_params.bias = state->set;
	
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "struct_callback: %s\n", client );*/
	close_popup(StructClone->structForm);
	
    } else {
	fprintf(stderr, "struct_callback: unknown client: %s\n", (char *)client);
    }
}

void
struct_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(StructClone->structForm);
    } else if (!strcmp(request, "init")){
	struct_params.plotFname = NULL;
	struct_params.autoscale = 0;
	struct_params.noText = 0;
	struct_params.showWeight = 0;
	struct_params.maxLineWidth = 0;
	struct_params.maxLineWidth_string = NULL;
	struct_params.blackAndWhite = 0;
	struct_params.useThresh = 0;
	struct_params.thresh = 0.;
	struct_params.thresh_string = NULL;
	struct_params.bias = 0;
    } else if (!strcmp(request, "display")){
	char string[10];
	show_string_label(StructClone->plotFnameLabel, struct_params.plotFname);
	show_toggle_value(StructClone->autoscaleToggle, struct_params.autoscale);
	show_toggle_value(StructClone->noTextToggle, struct_params.noText);
	show_toggle_value(StructClone->showWeightToggle, struct_params.showWeight);
	gray_out_item(StructClone->weightForm, !struct_params.showWeight);
	XtFree(struct_params.maxLineWidth_string);
	sprintf(string, "%d", struct_params.maxLineWidth);
	struct_params.maxLineWidth_string = XtNewString(string);
	show_string_text(StructClone->maxLineWidthText,
			 struct_params.maxLineWidth_string);
	show_radio_value(StructClone->blackAndWhiteRadioBox, struct_params.blackAndWhite);
	show_radio_value(StructClone->useThreshRadioBox, struct_params.useThresh);
	gray_out_item(StructClone->threshForm, !struct_params.useThresh);
	XtFree(struct_params.thresh_string);
	sprintf(string, "%5.2g", struct_params.thresh);
	struct_params.thresh_string = XtNewString(string);
	show_string_text(StructClone->threshText,
			 struct_params.thresh_string);
	show_toggle_value(StructClone->biasToggle, struct_params.bias);

    } else if (!strcmp(request, "plotFname")){
	XtFree(struct_params.plotFname);
	struct_params.plotFname = XtNewString(lnk_params.exp_name);
	struct_params.plotFname = StringConcat(struct_params.plotFname,
					       ".struct.plot");
	show_string_label(StructClone->plotFnameLabel,
			  struct_params.plotFname);
	print_set_struct_status();
    } else if (!strcmp(request, "noWeights")){
	gray_out_item(StructClone->showWeightFrame, TRUE);

    } else if (!strcmp(request, "weights")){
	gray_out_item(StructClone->showWeightFrame, FALSE);

    } else if (!strcmp(request, "noBias")){
	gray_out_item(StructClone->biasToggle, TRUE);

    } else if (!strcmp(request, "bias")){
	gray_out_item(StructClone->biasToggle, FALSE);

    } else {
	fprintf(stderr, "struct_request: unknown request: %s\n", request);
    }
}

static ARG structtab[] =
{
    {"threshold", FLOAT, (int *)NULL, ""},
    {"use_threshold", FLAG, (int *)NULL, ""},
    {"show_bias", FLAG, (int *)NULL, ""},
    {"show_weight_magnitude", FLAG, (int *)NULL, ""},
    {"max_line_width", INT, (int *)NULL, ""},
    {"bw", FLAG, (int *)NULL, ""},
};
#define TABSIZE (sizeof(structtab)/sizeof(ARG))

static void
struct_set_defaults_variables(dp)
     StructParams *dp;
{
    nnargs_set_variable("threshold", (int *)&dp->thresh, structtab, TABSIZE);
    nnargs_set_variable("use_threshold", (int *)&dp->useThresh,
			structtab, TABSIZE);
    nnargs_set_variable("show_bias", (int *)&dp->bias, structtab, TABSIZE);
    nnargs_set_variable("show_weight_magnitude", (int *)&dp->showWeight,
			structtab, TABSIZE);
    nnargs_set_variable("max_line_width", (int *)&dp->maxLineWidth,
			structtab, TABSIZE);
    nnargs_set_variable("bw", (int *)&dp->blackAndWhite, structtab, TABSIZE);
}
    
void
read_struct_defaults(argc, argv)
     int argc;
     char **argv;
{
    /* clear flags and string which are not printed if they are NULL */
    struct_plot_fname = NULL;
    struct_param_fname = NULL;
    struct_defaults_fname = NULL;
    struct_no_text = 0;
    struct_autoscale = 0;
    no_graphics = 0;
    struct_params.useThresh = 0;
    struct_params.bias = 0;
    struct_params.showWeight = 0;
    struct_params.blackAndWhite = 0;
    if(argc > 0){
	struct_set_defaults_variables(&struct_params);
	argc = nnargs_part(argc, argv, structtab, TABSIZE);
	argc = nnargs(argc, argv, struct_plot_argtab, STRUCT_PLOT_TABSIZE);
	struct_set_read_variables();
    } else {
	/* set any defaults */
	struct_params.autoscale = 1;
	struct_params.maxLineWidth = 10;
	struct_params.thresh = 0.;
    }
    struct_params.plotFname = XtNewString(struct_plot_fname);

    struct_request("display");
}

void
write_struct_defaults(FILE *default_file)
{
    struct_set_write_variables();
    struct_set_defaults_variables(&struct_params);
    if(default_file != NULL){
	fprintf(default_file, "struct %s",
		nnargs_get_part_flags(structtab, TABSIZE));
	fprintf(default_file, "%s",
		nnargs_get_flags(struct_plot_argtab, STRUCT_PLOT_TABSIZE));
    }
}

static void
struct_set_read_variables()
{
    struct_params.noText = struct_no_text;
    struct_params.autoscale = struct_autoscale;
}

/* copy the variables for writing to screen file.
   zero out the uninteresting ones (derived or copied data) */
static void
struct_set_write_variables()
{
    struct_plot_fname = NULL;
    struct_param_fname = NULL;
    struct_defaults_fname = NULL;
    struct_no_text = struct_params.noText;
    struct_autoscale = struct_params.autoscale;
    no_graphics = 0;
}

void
store_struct_plot_command(FILE *shell_file)
{
    char *plot_command_line;
    char string[10];

    struct_set_common_flags();

    if(lnk_params.store_only){
	no_graphics = 1;
    } else {
	no_graphics = 0;
    }

    store_plot_start(shell_file);
    plot_command_line = XtNewString("plot_");
    plot_command_line = StringConcat(plot_command_line,
				     lnk_params.classifier_name);
    plot_command_line = StringConcat(plot_command_line, " ");
    plot_command_line = StringConcat(plot_command_line,
				     nnargs_get_flags(struct_plot_argtab,
						      STRUCT_PLOT_TABSIZE));
    *strrchr(plot_command_line, '\n') = ' ';

    switch(lnk_params.classifier_num){
      case GAUSS:
    case SVM:
	fprintf(shell_file, "%s", plot_command_line);
	break;
      case BINTREE:
	fprintf(shell_file, "%s", plot_command_line);
	g_get_test_args(shell_file);/* get extra arguments */
	break;
      default:
	plot_command_line = StringConcat(plot_command_line, "\\\n ");
	if(struct_params.useThresh){
	    plot_command_line = StringConcat(plot_command_line,
					     "-threshold ");
	    sprintf(string, "%f ", struct_params.thresh);
	    plot_command_line = StringConcat(plot_command_line, string);
	}
	if(struct_params.showWeight){
	    plot_command_line =
	      StringConcat(plot_command_line,
			   "-show_weight_magnitude -max_line_width ");
	    sprintf(string, "%d ", struct_params.maxLineWidth);
	    plot_command_line = StringConcat(plot_command_line, string);
	}
	if(lnk_params.classifier_num == MLP && struct_params.bias){
	    plot_command_line = StringConcat(plot_command_line,
					     "-show_bias ");
	}
	if(struct_params.blackAndWhite && lnk_params.classifier_num != GMIX){
	    plot_command_line = StringConcat(plot_command_line, "-bw ");
	}
	fprintf(shell_file, "%s", plot_command_line);
    }
    store_plot_end(shell_file);
    XtFree(struct_defaults_fname);
    struct_defaults_fname = NULL;
    XtFree(plot_command_line);
}

static void
struct_set_common_flags()
{
    struct_plot_fname = struct_params.plotFname;
    struct_param_fname = reports_params.param_fname;
    struct_defaults_fname = path_create(data_params.data_path,
					data_params.desc_fname);
    struct_no_text = struct_params.noText;
    struct_autoscale = struct_params.autoscale;
}
