/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
 
/* $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/svm/svm_callback.c,v 1.5 2001/11/05 19:57:16 kukolich Exp $ */

 
/******************************************************************************
 ******************************************************************************
 ****                                                                      ****
 ****                       MIT Lincoln Laboratory                         ****
 ****                             SVM                                ****
 ****                          Window Interface                            ****
 ****                                                                      ****
 ******************************************************************************
 ******************************************************************************
 *
 * File Name: $Source $
 *
 * Language: C
 *
 * File Modification History:
 *  8-Nov-1995     Linda C Kukolich   initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <Xm/Xm.h>
#include "../main/lnknet_subs.h"
#include "svm.h"
#include <nnlib.h>
#include <svm_args.h>
/*
 * External Functions
 */
#include "svm_func.h"
#include "../plots/plots_func.h"
 
/*
 * Externals
 */
extern SvmWidgets *SvmClone;
#include "svm_params.h"
SvmParams svm_params; /* this is the real one */
#include "../data/data_params.h"
extern DataParams data_params;

/*
 * Statics
 */ 
static void svm_show_kernel_choices();

/******************************************************************************
 ******************************************************************************
 *
 * Module Name: svm_callback
 *
 * Purpose: 
 * 
 * Calling Sequence:
 *   svm_callback(w, client, call)
 *
 * Inputs:
 *   Widget w            widget calling this callback
 *   XtPointer client    client data attached to widget
 *   XtPointer call      pointer to callback structure
 *
 * Outputs:
 *   none
 *
 * Return Value:
 *   none
 *
 * Side Effects:
 *   none
 *
 * Restrictions:
 *   none
 *
 * Modification History:
 *  8-Nov-1995     Linda C Kukolich  initial creation
 *
 ******************************************************************************
 ******************************************************************************
 */
 
void
svm_callback(Widget w, XtPointer client, XtPointer call)
{
    /*SUPPRESS 765*/
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    SvmWidgets *ip;
    static int kktTol_buttons_mapped = False;
    static int alphaTol_buttons_mapped = False;
    static int kernelSigma_buttons_mapped = False;
    static int kernelOrder_buttons_mapped = False;
    static int kernelPoly_buttons_mapped = False;
    static int cbound_buttons_mapped = False;

    /*
     * Get the parent structure
     */
    XtVaGetValues (w, XmNuserData, &ip, NULL);
 

    if (!strcmp(client, "kktTolText")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	kktTol_buttons_mapped =
	  okTextValueChanged(ip->kktTolText, ip->kktTolOKPBtn,
			     ip->kktTolCancelPBtn,
			     ip->svmDialogShell,
			     kktTol_buttons_mapped,
			     svm_params.kktTol_string);
	
    } else if ((!strcmp(client, "kktTolTextAct")) ||
	       (!strcmp(client, "kktTolOKPBtn"))) {
	String kktTolTextval;

	/*fprintf (stderr, "window_callback: %s\n", client );*/
	kktTol_buttons_mapped =
	  okTextActivate(ip->kktTolText, ip->kktTolOKPBtn,
			 ip->kktTolCancelPBtn, &kktTolTextval);
	/* I won't actually be using this copy */
	XtFree (kktTolTextval);
	
	/* verify the string as a float */
	svm_params.kktTol =
	  set_atof_low_value(ip->kktTolText, "kktTol_set",
			     svm_params.kktTol, 0);

	/* save the current text for future comparisons */
	XtFree(svm_params.kktTol_string);
	svm_params.kktTol_string = get_string_text(ip->kktTolText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "kktTolCancelPBtn")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	kktTol_buttons_mapped = 
	  okTextCancel(ip->kktTolText, ip->kktTolOKPBtn,
		       ip->kktTolCancelPBtn,
		       svm_params.kktTol_string);

    } else if (!strcmp(client, "alphaTolText")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	alphaTol_buttons_mapped =
	  okTextValueChanged(ip->alphaTolText, ip->alphaTolOKPBtn,
			     ip->alphaTolCancelPBtn,
			     ip->svmDialogShell,
			     alphaTol_buttons_mapped,
			     svm_params.alphaTol_string);
	
    } else if ((!strcmp(client, "alphaTolTextAct")) ||
	       (!strcmp(client, "alphaTolOKPBtn"))) {
	String alphaTolTextval;

	/*fprintf (stderr, "window_callback: %s\n", client );*/
	alphaTol_buttons_mapped =
	  okTextActivate(ip->alphaTolText, ip->alphaTolOKPBtn,
			 ip->alphaTolCancelPBtn, &alphaTolTextval);
	/* I won't actually be using this copy */
	XtFree (alphaTolTextval);
	
	/* verify the string as a float */
	svm_params.alphaTol =
	  set_atof_low_value(ip->alphaTolText, "alphaTol_set",
			     svm_params.alphaTol, 0);

	/* save the current text for future comparisons */
	XtFree(svm_params.alphaTol_string);
	svm_params.alphaTol_string = get_string_text(ip->alphaTolText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "alphaTolCancelPBtn")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	alphaTol_buttons_mapped = 
	  okTextCancel(ip->alphaTolText, ip->alphaTolOKPBtn,
		       ip->alphaTolCancelPBtn,
		       svm_params.alphaTol_string);

    } else if (!strcmp(client, "kernelLinearToggle") ||
	       !strcmp(client, "kernelGaussianToggle") ||
	       !strcmp(client, "kernelPolynomialToggle") ||
	       !strcmp(client, "kernelInhomPolyToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	if(state->set){
	  svm_params.kernelType =
	    get_radio_value(ip->kernelTypeRadioBox);
	  if(svm_params.kernelType == 0){
	    if(svm_params.kernelLinearExtra){
	      svm_params.kernelType = 4;
	    }
	  }
	  svm_show_kernel_choices();
	}

    } else if (!strcmp(client, "kernelExtraToggle")) {
      XmToggleButtonCallbackStruct *state =
	(XmToggleButtonCallbackStruct *)acs;
      if(state->set){
	svm_params.kernelLinearExtra = 1;
	svm_params.kernelType = 4; /* this can be a problem on restore! */
      } else {
	svm_params.kernelType = 0; /* linear */
	svm_params.kernelLinearExtra = 1;
      }
    } else if (!strcmp(client, "kernelSigmaText")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	kernelSigma_buttons_mapped =
	  okTextValueChanged(ip->kernelSigmaText, ip->kernelSigmaOKPBtn,
			     ip->kernelSigmaCancelPBtn,
			     ip->svmDialogShell,
			     kernelSigma_buttons_mapped,
			     svm_params.kernelSigma_string);
	
    } else if ((!strcmp(client, "kernelSigmaTextAct")) ||
	       (!strcmp(client, "kernelSigmaOKPBtn"))) {
	String kernelSigmaTextval;

	/*fprintf (stderr, "window_callback: %s\n", client );*/
	kernelSigma_buttons_mapped =
	  okTextActivate(ip->kernelSigmaText, ip->kernelSigmaOKPBtn,
			 ip->kernelSigmaCancelPBtn, &kernelSigmaTextval);
	/* I won't actually be using this copy */
	XtFree (kernelSigmaTextval);
	
	/* verify the string as a float */
	svm_params.kernelSigma =
	  set_atof_low_value(ip->kernelSigmaText, "kernelSigma_set",
			     svm_params.kernelSigma, 0.);

	/* save the current text for future comparisons */
	XtFree(svm_params.kernelSigma_string);
	svm_params.kernelSigma_string = get_string_text(ip->kernelSigmaText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "kernelSigmaCancelPBtn")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	kernelSigma_buttons_mapped = 
	  okTextCancel(ip->kernelSigmaText, ip->kernelSigmaOKPBtn,
		       ip->kernelSigmaCancelPBtn,
		       svm_params.kernelSigma_string);

    } else if (!strcmp(client, "kernelOrderText")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	kernelOrder_buttons_mapped =
	  okTextValueChanged(ip->kernelOrderText, ip->kernelOrderOKPBtn,
			     ip->kernelOrderCancelPBtn,
			     ip->svmDialogShell,
			     kernelOrder_buttons_mapped,
			     svm_params.kernelOrder_string);
	
    } else if ((!strcmp(client, "kernelOrderTextAct")) ||
	       (!strcmp(client, "kernelOrderOKPBtn"))) {
	String kernelOrderTextval;

	/*fprintf (stderr, "window_callback: %s\n", client );*/
	kernelOrder_buttons_mapped =
	  okTextActivate(ip->kernelOrderText, ip->kernelOrderOKPBtn,
			 ip->kernelOrderCancelPBtn, &kernelOrderTextval);
	/* I won't actually be using this copy */
	XtFree (kernelOrderTextval);
	
	/* verify the string as an integer */
	svm_params.kernelOrder =
	  set_atoi_low_value(ip->kernelOrderText, "kernelOrder_set",
			     svm_params.kernelOrder, 1);

	/* save the current text for future comparisons */
	XtFree(svm_params.kernelOrder_string);
	svm_params.kernelOrder_string = get_string_text(ip->kernelOrderText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "kernelOrderCancelPBtn")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	kernelOrder_buttons_mapped = 
	  okTextCancel(ip->kernelOrderText, ip->kernelOrderOKPBtn,
		       ip->kernelOrderCancelPBtn,
		       svm_params.kernelOrder_string);

    } else if (!strcmp(client, "kernelPolyText")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	kernelPoly_buttons_mapped =
	  okTextValueChanged(ip->kernelPolyText, ip->kernelPolyOKPBtn,
			     ip->kernelPolyCancelPBtn,
			     ip->svmDialogShell,
			     kernelPoly_buttons_mapped,
			     svm_params.kernelPoly_string);
	
    } else if ((!strcmp(client, "kernelPolyTextAct")) ||
	       (!strcmp(client, "kernelPolyOKPBtn"))) {
	String kernelPolyTextval;

	/*fprintf (stderr, "window_callback: %s\n", client );*/
	kernelPoly_buttons_mapped =
	  okTextActivate(ip->kernelPolyText, ip->kernelPolyOKPBtn,
			 ip->kernelPolyCancelPBtn, &kernelPolyTextval);
	/* I won't actually be using this copy */
	XtFree (kernelPolyTextval);
	
	/* verify the string as a float */
	svm_params.kernelPoly =
	  set_atof_low_value(ip->kernelPolyText, "kernelPoly_set",
			     svm_params.kernelPoly, 0.);

	/* save the current text for future comparisons */
	XtFree(svm_params.kernelPoly_string);
	svm_params.kernelPoly_string = get_string_text(ip->kernelPolyText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "kernelPolyCancelPBtn")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	kernelPoly_buttons_mapped = 
	  okTextCancel(ip->kernelPolyText, ip->kernelPolyOKPBtn,
		       ip->kernelPolyCancelPBtn,
		       svm_params.kernelPoly_string);

  } else if (!strcmp(client, "sigmoidFitToggle") ||
	     !strcmp(client, "rawOutToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	if(state->set){
	  int temp;
	  temp = get_radio_value(ip->n_sigmoid_fitRadioBox);
	  /* reverse the sense of this */
	  svm_params.sigmoidFit = (temp) ? 0 : 1;
	}

  } else if (!strcmp(client, "meVsWorldToggle") ||
	     !strcmp(client, "meVsEachToggle")) {
	XmToggleButtonCallbackStruct *state =
	  (XmToggleButtonCallbackStruct *)acs;
	if(state->set){
	  svm_params.mclassType = get_radio_value(ip->mclassTypeRadioBox);
	  svm_set_total_nclass();
	}
    } else if (!strcmp(client, "cboundText")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	cbound_buttons_mapped =
	  okTextValueChanged(ip->cboundText, ip->cboundOKPBtn,
			     ip->cboundCancelPBtn,
			     ip->svmDialogShell,
			     cbound_buttons_mapped,
			     svm_params.cbound_string);
	
    } else if ((!strcmp(client, "cboundTextAct")) ||
	       (!strcmp(client, "cboundOKPBtn"))) {
	String cboundTextval;

	/*fprintf (stderr, "window_callback: %s\n", client );*/
	cbound_buttons_mapped =
	  okTextActivate(ip->cboundText, ip->cboundOKPBtn,
			 ip->cboundCancelPBtn, &cboundTextval);
	/* I won't actually be using this copy */
	XtFree (cboundTextval);
	
	/* verify the string as a float */
	svm_params.cbound =
	  set_atof_low_value(ip->cboundText, "cbound_set",
			     svm_params.cbound, 0.);

	/* save the current text for future comparisons */
	XtFree(svm_params.cbound_string);
	svm_params.cbound_string = get_string_text(ip->cboundText);
 
	/* do any other things needed */

    } else if (!strcmp(client, "cboundCancelPBtn")) {
	/*fprintf (stderr, "window_callback: %s\n", client );*/
	cbound_buttons_mapped = 
	  okTextCancel(ip->cboundText, ip->cboundOKPBtn,
		       ip->cboundCancelPBtn,
		       svm_params.cbound_string);
	
    } else if (!strcmp(client, "closePBtn")) {
	/*fprintf (stderr, "svm_callback: %s\n", client );*/
	close_popup(SvmClone->svmForm);
	
    } else {
	fprintf(stderr, "svm_callback: unknown client: %s\n", (char *)client);
    }
}

void
svm_request(String request)
{
    if(!strcmp(request, "open")){
	open_popup(SvmClone->svmForm);
    } else if (!strcmp(request, "init")){
	svm_params.kktTol = .001;
	svm_params.kktTol_string = NULL;
	svm_params.alphaTol = .001;
	svm_params.alphaTol_string = NULL;
	svm_params.kernelSigma = .3;
	svm_params.kernelSigma_string = NULL;
	svm_params.kernelOrder = 2;
	svm_params.kernelOrder_string = NULL;
	svm_params.kernelPoly = 1.0;
	svm_params.kernelPoly_string = NULL;
	svm_params.cbound = 10.;
	svm_params.cbound_string = NULL;
	svm_params.kernelType = 0;
	svm_params.kernelLinearExtra = 0;
	svm_params.sigmoidFit = 1;
	svm_params.mclassType = 0;
    } else if (!strcmp(request, "display")){
	char string[10];
	XtFree(svm_params.kktTol_string);
	sprintf(string, "%f", svm_params.kktTol);
	svm_params.kktTol_string = XtNewString(string);
	show_string_text(SvmClone->kktTolText,
			 svm_params.kktTol_string);

	XtFree(svm_params.alphaTol_string);
	sprintf(string, "%f", svm_params.alphaTol);
	svm_params.alphaTol_string = XtNewString(string);
	show_string_text(SvmClone->alphaTolText,
			 svm_params.alphaTol_string);

	XtFree(svm_params.kernelSigma_string);
	sprintf(string, "%f", svm_params.kernelSigma);
	svm_params.kernelSigma_string = XtNewString(string);
	show_string_text(SvmClone->kernelSigmaText,
			 svm_params.kernelSigma_string);

	XtFree(svm_params.kernelOrder_string);
	sprintf(string, "%d", svm_params.kernelOrder);
	svm_params.kernelOrder_string = XtNewString(string);
	show_string_text(SvmClone->kernelOrderText,
			 svm_params.kernelOrder_string);

	XtFree(svm_params.kernelPoly_string);
	sprintf(string, "%f", svm_params.kernelPoly);
	svm_params.kernelPoly_string = XtNewString(string);
	show_string_text(SvmClone->kernelPolyText,
			 svm_params.kernelPoly_string);

	XtFree(svm_params.cbound_string);
	sprintf(string, "%f", svm_params.cbound);
	svm_params.cbound_string = XtNewString(string);
	show_string_text(SvmClone->cboundText,
			 svm_params.cbound_string);

	if(svm_params.kernelType == 0 || svm_params.kernelType == 4){
	  show_radio_value(SvmClone->kernelTypeRadioBox, 0);
	} else {
	  show_radio_value(SvmClone->kernelTypeRadioBox,
			   svm_params.kernelType);
	}
	show_toggle_value(SvmClone->kernelExtraToggle,
			  svm_params.kernelLinearExtra);
	show_radio_value(SvmClone->n_sigmoid_fitRadioBox,
			 !svm_params.sigmoidFit);
	show_radio_value(SvmClone->mclassTypeRadioBox,
			 svm_params.mclassType);
	svm_show_kernel_choices();
	svm_set_total_nclass();
    } else {
	fprintf(stderr, "svm_request: unknown request: %s\n", request);
    }
}

String
svm_get_plot_label(String string)
{
    return string;
}

static void
svm_set_read_variables()
{
    svm_params.cbound = cbound;
    svm_params.kernelType = kernel_type;
    svm_params.kernelSigma = kernel_sigma;
    svm_params.kernelOrder = kernel_order;
    svm_params.kernelPoly = kernel_poly_divisor;
    svm_params.sigmoidFit = sigmoid_fit;
    svm_params.kktTol = kkt_tolerance;
    svm_params.alphaTol = alpha_tolerance;
    svm_params.mclassType = multi_class_type;
}

static void
svm_set_write_variables()
{
  cbound = svm_params.cbound;
  kernel_type = svm_params.kernelType;
  kernel_sigma = svm_params.kernelSigma;
  kernel_order = svm_params.kernelOrder;
  kernel_poly_divisor = svm_params.kernelPoly;
  sigmoid_fit = svm_params.sigmoidFit;
  kkt_tolerance = svm_params.kktTol;
  alpha_tolerance = svm_params.alphaTol;
  multi_class_type = svm_params.mclassType;
}

#if 0
static ARG svmtab[] =
{
  /*    {"specifyBins", BOOLEAN, (int *)NULL,},*/
};
#define TABSIZE (sizeof(svmtab)/sizeof(ARG))
#endif

static void
svm_set_defaults_variables(dp)
     SvmParams *dp;
{
  /*    nnargs_set_variable("specifyBins", (int *)&dp->specifyBins, svmtab, TABSIZE);*/
}

void
read_svm_defaults(argc, argv)
     int argc;
     char **argv;
{
  /*    svm_params.specifyBins = 0;
	nBinsPerInput = NULL;*/
    svm_set_defaults_variables(&svm_params);
    if(argc > 0){
      /*	argc = nnargs_part(argc, argv, svmtab, TABSIZE);*/
	argc = nnargs(argc, argv, svm_tab, SVM_TABSIZE);
	svm_set_read_variables();

    } else {
	/* set any defaults */
      /*	svm_params.nBins = 10;*/
    }

    svm_request("display");
}

void
write_svm_defaults(FILE *default_file)
{
    svm_set_write_variables();
    svm_set_defaults_variables(&svm_params);

    if(default_file != NULL){
      fprintf(default_file, "svm ");
      /*	fprintf(default_file, "svm %s",
		nnargs_get_part_flags(svmtab, TABSIZE));*/
	fprintf(default_file, "%s",
		nnargs_get_flags(svm_tab, SVM_TABSIZE));
    }
}

String
svm_get_args ()
{
    svm_set_write_variables();
    /*    if(!svm_params.specifyBins) nBinsPerInput = NULL;*/
    return (nnargs_get_part_flags(svm_tab, SVM_TABSIZE));
}

String
svm_get_important_args ()
{
    svm_set_write_variables();
    /*    if(!svm_params.specifyBins) nBinsPerInput = NULL;*/
    return (nnargs_get_flags_one_line(svm_tab, SVM_TABSIZE));
}

void
svm_store_plot_command(FILE *shell_file)
{
  if(svm_params.sigmoidFit){
    fprintf(shell_file, "\\\n -sigmoid_fit");
  }
}

static void
svm_show_kernel_choices()
{
  switch(svm_params.kernelType){
  case 0:
  case 4:
    gray_out_item(SvmClone->kernelExtraToggle, FALSE);
    gray_out_item(SvmClone->kernelSigmaForm, TRUE);
    gray_out_item(SvmClone->kernelOrderForm, TRUE);
    gray_out_item(SvmClone->kernelPolyForm, TRUE);
    break;
  case 1:
    gray_out_item(SvmClone->kernelExtraToggle, TRUE);
    gray_out_item(SvmClone->kernelSigmaForm, FALSE);
    gray_out_item(SvmClone->kernelOrderForm, TRUE);
    gray_out_item(SvmClone->kernelPolyForm, TRUE);
    break;
  case 2:
  case 3:
    gray_out_item(SvmClone->kernelExtraToggle, TRUE);
    gray_out_item(SvmClone->kernelSigmaForm, TRUE);
    gray_out_item(SvmClone->kernelOrderForm, FALSE);
    gray_out_item(SvmClone->kernelPolyForm, FALSE);
    break;
  }
}

void
svm_set_total_nclass()
{
  int nclassifiers;
  static char total_label[64];
  if(svm_params.mclassType){
    nclassifiers = data_params.noutputs*(data_params.noutputs-1)/2;
  } else {
    nclassifiers = data_params.noutputs;
  }
  sprintf(total_label, "This will create a total of %d classifiers.",
	  nclassifiers);
  show_string_label(SvmClone->totalNclassLabel, total_label);
}
