# uil.awk
# WARNING - you must use gawk!!!
# run this awk script with the following command:
#    gawk -f uil.awk ansi=$ANSI rcs=$RCS filename.uil
#
# ansi=="yes"        produces ANSI-C function prototyping;
#                    otherwise, K&R-C function prototyping is produced
#
# rcs=="yes"         produces RCS-type source code control stuff in C-code;
#                    otherwise, SCCS-type source code control stuff is
#                    inserted

###############################################################################
# BEGIN part
###############################################################################
BEGIN {
}

###############################################################################
# main part of gawk script
###############################################################################
{
# look for "bx_app_name" and get the program name
  if (index ($0, "bx_app_name")) {
    split($4, temp, "+");
    split(temp[2], temp1, "'");
    program = temp1[1];
    program_caps = toupper(program);
    program_home = program_caps "HOME";

# Builder Xcessory 3.5 uses a different commenting scheme to declare
# the program name
  } else if (index ($0, "bx_info(\"app_name\",")) {
    split($3, temp, "\"");
    program = temp[2];
    program_caps = toupper(program);
    program_home = program_caps "HOME";

#   isolate all lines containing the word "procedure" and the "=" sign.
  } else if ((index ($0, "procedure") != 0) && \
	     (index ($0, "=") != 0)) {

#     isolate the callback procedure name and its argument
    split($NF, temp, "(");
    callback = temp[1];
    split(temp[2], temp1, ")");
    argument = temp1[1];
#     print callback " " argument;

#     sort the arguments into each callback category
    if (callback in cb_cnt) {
      cb_array[callback,cb_cnt[callback]] = argument;
      cb_cnt[callback]++;
    } else {
      cb_array[callback, 0] = argument;
      cb_cnt[callback] = 1;
    }

#   isolate the object names for building structures of widgets
  } else if ((index ($0, "TopLevelShell")) && \
	     (index ($0, "!(BX)object"))) {
    toplevelshell = $2;
    shell_cnt[toplevelshell] = 1;
    shell_array[toplevelshell, 0] = toplevelshell;
  } else if ((index ($0, "DialogShell")) && \
	     (index ($0, "!(BX)object"))) {
    toplevelshell = $2;
    shell_cnt[toplevelshell] = 1;
    shell_array[toplevelshell, 0] = toplevelshell;
  } else if ($1 == "object") {
    shell_array[toplevelshell, shell_cnt[toplevelshell]] = $2;
    shell_cnt[toplevelshell]++;
  }
}

###############################################################################
# END part
###############################################################################
END {
# save the "filename" in "filename.uil"
  split(FILENAME, temp, ".");
  dotHfilename = temp[1];

#   save the date
  "date" | getline d;
  split (d, d1);
  date = d1[3] "-" d1[2] "-" d1[6];
  

#   get the user name by looking it up in the passwd file
  "ypcat passwd | grep $USER" | getline passwd_entry;
  split (passwd_entry, passwd_field, ":");
  user = passwd_field[5];

#   create structure name array
  for (i in shell_cnt) {
#   isolate the TopLevelShell or DialogShell name and capitalize the 1st letter
    if (index (i, "TopLevelShell")) {
      split(i, temp, "TopLevelShell");
    } else if (index (i, "DialogShell")) {
      split(i, temp, "DialogShell");
    } else {
      temp[1] = i;
    }
    structureName[i] = toupper(substr(temp[1],1,1)) substr(temp[1],2);
  }

#   make the .c files
  make_c_files();

#   make the header file
  make_h_file();

#   make the Makefile
  make_Makefile();

#   make the file to rename the stubs files to "real" names
  make_rename_file();
}

###############################################################################
#   make the .c files
###############################################################################
function make_c_files () {
  for (i in cb_cnt) {
#     make filename
    filename = i ".c";
    filename_stubs = i "_stubs.c";
    
#     if this is a "create" routine, then set up the appropriate structure
#     name to be used for initializing the structure
    if (index(filename, "_create") != 0) {
      split (filename, temp, "_create");
      structname = toupper(substr(temp[1],1,1)) substr(temp[1],2);
    } else {
      structname = "";
    }
    
#     attach file and module headers
#    print "/*SUPPRESS 592*/" > filename_stubs;

    if (rcs == "yes") {
      print "static char copyright[]=" >> filename_stubs;
      print "\"**************************************************************************\\n\\" >> filename_stubs;
      print "                Copyright " d1[6] " MIT Lincoln Laboratory.\\n\\" >> filename_stubs;
      print "This software is not for release and should neither be copied\\n\\" >> filename_stubs;
      print "nor distributed without explicit permission of MIT Lincoln Laboratory\\n\\" >> filename_stubs;
      print "**************************************************************************\\n\";" >> filename_stubs;
      print " " >> filename_stubs;

    } else {
      print "static char *Sccsid = \"%W% %Y% %H% %T%\";" >> filename_stubs;
      print "static char *copyright = \"Copyright " d1[6] " by MIT, All rights reserved, Developed at Lincoln Laboratory\";" >> filename_stubs;
    }

    print " " >> filename_stubs;
    print "/******************************************************************************" >> filename_stubs;
    print " ******************************************************************************" >> filename_stubs;
    print " ****                                                                      ****" >> filename_stubs;
    print " ****                       MIT Lincoln Laboratory                         ****" >> filename_stubs;
    print " ****                            " program_caps "                              ****" >> filename_stubs;
    print " ****                          Window Interface                            ****" >> filename_stubs;
    print " ****                                                                      ****" >> filename_stubs;
    print " ******************************************************************************" >> filename_stubs;
    print " ******************************************************************************" >> filename_stubs;
    print " *" >> filename_stubs;
    print " * File Name: $Source $" >> filename_stubs;
    print " *" >> filename_stubs;
    print " * Language: C" >> filename_stubs;
    print " *" >> filename_stubs;
    print " * File Modification History:" >> filename_stubs;
    print " *  " date "     " user "   initial creation" >> filename_stubs;
    print " *" >> filename_stubs;
    print " ******************************************************************************" >> filename_stubs;
    print " ******************************************************************************" >> filename_stubs;
    print " */" >> filename_stubs;
    print " " >> filename_stubs;
    print "#include <stdio.h>" >> filename_stubs;
    print "#include <Xm/Xm.h>" >> filename_stubs;
    print "#include \"" dotHfilename ".h\"" >> filename_stubs;
    
#     declare externals
    print "/*" >> filename_stubs;
    print " * External Functions" >> filename_stubs;
    print " */" >> filename_stubs;
    print " " >> filename_stubs;
    print "/*" >> filename_stubs;
    print " * Externals" >> filename_stubs;
    print " */" >> filename_stubs;
    for (j in structureName) {
      print "extern " structureName[j] "Widgets *" structureName[j] "Clone;" >> filename_stubs;
    }
    
    print " " >> filename_stubs;
    print "/******************************************************************************" >> filename_stubs;
    print " ******************************************************************************" >> filename_stubs;
    print " *" >> filename_stubs;
    print " * Module Name: " i >> filename_stubs;
    print " *" >> filename_stubs;
    print " * Purpose: " >> filename_stubs;
    print " * " >> filename_stubs;
    print " * Calling Sequence:" >> filename_stubs;
    print " *   " i "(w, client, call)" >> filename_stubs;
    print " *" >> filename_stubs;
    print " * Inputs:" >> filename_stubs;
    print " *   Widget w            widget calling this callback" >> filename_stubs;
    print " *   XtPointer client    client data attached to widget" >> filename_stubs;
    print " *   XtPointer call      pointer to callback structure" >> filename_stubs;
    print " *" >> filename_stubs;
    print " * Outputs:" >> filename_stubs;
    print " *   none" >> filename_stubs;
    print " *" >> filename_stubs;
    print " * Return Value:" >> filename_stubs;
    print " *   none" >> filename_stubs;
    print " *" >> filename_stubs;
    print " * Side Effects:" >> filename_stubs;
    print " *   none" >> filename_stubs;
    print " *" >> filename_stubs;
    print " * Restrictions:" >> filename_stubs;
    print " *   none" >> filename_stubs;
    print " *" >> filename_stubs;
    print " * Modification History:" >> filename_stubs;
    print " *  " date "     " user "  initial creation" >> filename_stubs;
    print " *" >> filename_stubs;
    print " ******************************************************************************" >> filename_stubs;
    print " ******************************************************************************" >> filename_stubs;
    print " */" >> filename_stubs;
    print " " >> filename_stubs;
    print "void" >> filename_stubs;
    if (ansi == "yes") {
      print i "(Widget w, XtPointer client, XtPointer call)" >> filename_stubs;
    } else {
      print i "(w, client, call)" >> filename_stubs;
    print "     Widget w;" >> filename_stubs;
    print "     XtPointer client;" >> filename_stubs;
    print "     XtPointer call;" >> filename_stubs;
    }
    print "{" >> filename_stubs;
    print "  /*SUPPRESS 765*/" >> filename_stubs;
    print "  XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;" >> filename_stubs;
#     get the parent structure if this is a callback module
    if (structname == "") {
      split (filename, temp, "_callback");
      struct = toupper(substr(temp[1],1,1)) substr(temp[1],2);
      print "  " struct "Widgets *ip;" >> filename_stubs;
      print " " >> filename_stubs;
      print "  /*" >> filename_stubs;
      print "   * Get the parent structure" >> filename_stubs;
      print "   */" >> filename_stubs;
      print "  XtVaGetValues (w, XmNuserData, &ip, NULL);" >> filename_stubs;
    }
    print " " >> filename_stubs;
    
#     write the code
    print "  if (!strcmp(client, " cb_array[i,0] ")) {" >> filename_stubs;
    print "    fprintf (stderr, \"" i ": %s\\n\", client );" >> filename_stubs;

    write_create_code(i, 0);
    print " " >> filename_stubs;
    
    for (j=1; j<cb_cnt[i]; j++) {
      print "  } else if (!strcmp(client, " cb_array[i,j] ")) {" >> filename_stubs;
      print "    fprintf (stderr, \"" i ": %s\\n\", client );" >> filename_stubs;
#       initialize the structure if this is a "create" routine
      write_create_code(i, j);
      print " " >> filename_stubs;
    }

#     finish the file
    print "  } else {" >> filename_stubs;
    print "    fprintf(stderr, \"" i ": unknown client: %s\\n\", client);" >> filename_stubs;
    print "  }" >> filename_stubs;
    print "}" >> filename_stubs;
  }
}

###############################################################################
#   write "create" code
###############################################################################
function write_create_code (i, j) {
#   initialize the structure if this is a "create" routine
  if (structname != "") {
    split(cb_array[i,j], widgetname, "\""); # remove the quotes
#     check for TopLevelShell or DialogShell
    if ((index(widgetname[2], "TopLevelShell")) ||
	(index(widgetname[2], "DialogShell"))  ||
	(tolower(widgetname[2]) == tolower(structname))) {
      print "widgetname[2] = ", widgetname[2];
      print "structname = ", structname;
      print "    /*" >> filename_stubs;
      print "     * Allocate space for the structure" >> filename_stubs;
      print "     */" >> filename_stubs;
      print "    " structname "Clone = (" structname "Widgets *)XtMalloc(sizeof(" structname "Widgets));" >> filename_stubs;
    } else {
      print "    XtVaSetValues (w, XmNuserData, " structname "Clone, NULL);" >> filename_stubs;
    }
    print "    " structname "Clone->" widgetname[2] " = w;" >> filename_stubs;
  }
}

###############################################################################
#   make the header file
###############################################################################
function make_h_file () {
  filename = dotHfilename ".h";
  filename_stubs = dotHfilename "_stubs.h";
      
# attach file and module headers
#  print "/*SUPPRESS 592*/" > filename_stubs;

  print "#ifndef " dotHfilename "_H" >> filename_stubs;
  print "#define " dotHfilename "_H" >> filename_stubs;
  if (rcs == "yes") {
    print "/**************************************************************************" >> filename_stubs;
    print "                Copyright " d1[6] " MIT Lincoln Laboratory." >> filename_stubs;
    print "This software is not for release and should neither be copied" >> filename_stubs;
    print "nor distributed without explicit permission of MIT Lincoln Laboratory" >> filename_stubs;
    print "**************************************************************************/" >> filename_stubs;
    print " " >> filename_stubs;
  } else {
    print "static char *Sccsid_" program "_h = \"%W% %Y% %H% %T%\";" >> filename_stubs;
    print "static char *copyright_" program "_h = \"Copyright " d1[6] " by MIT, All rights reserved, Developed at Lincoln Laboratory\";" >> filename_stubs;
  }

  print " " >> filename_stubs;
  print "/******************************************************************************" >> filename_stubs;
  print " ******************************************************************************" >> filename_stubs;
  print " ****                                                                      ****" >> filename_stubs;
  print " ****                       MIT Lincoln Laboratory                         ****" >> filename_stubs;
  print " ****                            " program_caps "                              ****" >> filename_stubs;
  print " ****                          Window Interface                            ****" >> filename_stubs;
  print " ****                                                                      ****" >> filename_stubs;
  print " ******************************************************************************" >> filename_stubs;
  print " ******************************************************************************" >> filename_stubs;
  print " *" >> filename_stubs;
  print " * File Name: $" program_home "/src/" filename >> filename_stubs;
  print " *" >> filename_stubs;
  print " * Language: C" >> filename_stubs;
  print " *" >> filename_stubs;
  print " * Purpose: Structure definitions for " program >> filename_stubs;
  print " * " >> filename_stubs;
  print " * Calling Sequence:" >> filename_stubs;
  print " *   #include <" dotHfilename ".h>" >> filename_stubs;
  print " *" >> filename_stubs;
  print " * Modification History:" >> filename_stubs;
  print " *  " date "     " user "         initial creation" >> filename_stubs;
  print " *" >> filename_stubs;
  print " ******************************************************************************" >> filename_stubs;
  print " ******************************************************************************" >> filename_stubs;
  print " */" >> filename_stubs;
  print " " >> filename_stubs;

# write the code
  
  print "/*" >> filename_stubs;

  if (ansi == "yes") {
    ansiString = "ansi=\"yes\" ";
  } else {
    ansiString = " ";
  }

  if (rcs == "yes") {
    rcsString = "rcs=\"yes\" ";
  } else {
    rcsString = " ";
  }

  print " * Structures created by command \"gawk -f uil.awk " ansiString rcsString FILENAME "\"" >> filename_stubs;

  print " * Structures for the TopLevelShells or DialogShells" >> filename_stubs;
  print " *  and their widgets." >> filename_stubs;
  print " */" >> filename_stubs;

  for (i in shell_cnt) {
#     declare the structure
    print "typedef struct {" >> filename_stubs;

#     insert the structure elements
    for (j=0; j<shell_cnt[i]; j++) {
      print "  Widget " shell_array[i,j] ";" >> filename_stubs;
    }

#     construct the name of the structure
    print "} " structureName[i] "Widgets;" >> filename_stubs;
    print " " >> filename_stubs;
  }
  print "#endif" >> filename_stubs;
}

###############################################################################
#   make the Makefile
###############################################################################
function make_Makefile () {
  filename = "Makefile_" dotHfilename "_stubs";

  if (rcs == "yes") {
    print "#                Copyright " d1[6] " MIT Lincoln Laboratory." > filename;
    print "# This software is not for release and should neither be copied" >> filename;
    print "# nor distributed without explicit permission of MIT Lincoln Laboratory" >> filename;
    print " " >> filename;
    print "# $Header: /data/speech/kukolich/cvsroot/lnknet/src/mclass/uil.awk,v 11.2 2001/03/30 19:36:18 kukolich Exp $" >> filename;
    print " " >> filename;
    print "# " >> filename;
    
  } else {
    print "# %W% %Y% %H% %T%" > filename;
    print "# Copyright " d1[6] " by MIT, All rights reserved, Developed at Lincoln Laboratory" >> filename;
  }

  print " " >> filename;
  print "##############################################################################" >> filename;
  print "##############################################################################" >> filename;
  print "####                                                                      ####" >> filename;
  print "####                               " program_caps "                                 ####" >> filename;
  print "####                           Window Interface                           ####" >> filename;
  print "####                                                                      ####" >> filename;
  print "##############################################################################" >> filename;
  print "##############################################################################" >> filename;
  print "#" >> filename;
  print "# File Name: Makefile" >> filename;
  print "#" >> filename;
  print "# Language: make" >> filename;
  print "#" >> filename;
  print "# File Modification History:" >> filename;
  print "#  " date "     " user "  initial creation" >> filename;
  print "#" >> filename;
  print "##############################################################################" >> filename;
  print "##############################################################################" >> filename;
  print " " >> filename;
  print "PROGRAM = $(" program_home ")/bin/" program >> filename;
  print "SOURCES = \\" >> filename;
  print "        $(" program_home ")/src/" program ".c \\" >> filename;
  for (i in cb_cnt) {
    print "        $(" program_home ")/src/" i ".c \\" >> filename;
  }
  print "        $(" program_home ")/src/" program "_creation.c" >> filename;
  print " " >> filename;
  print "# Derived parameters." >> filename;
  print " " >> filename;
  print "OBJECTS = $(SOURCES:%.c=%.o)" >> filename;
  print " " >> filename;
  print "# Compiler flags." >> filename;
  print " " >> filename;
  print "CC=/usr/local/gnu/bin/gcc" >> filename;
  print "#CC=cc" >> filename;
  print " " >> filename;
  print "CFLAGS = \\" >> filename;
  print	"        -g \\" >> filename;
  print "        -D_NO_PROTO" >> filename;
  print " " >> filename;
  print "CPPFLAGS = \\" >> filename;
  print "        -I/usr/local/include \\" >> filename;
  print " " >> filename;
  print "LDLIBS = \\" >> filename;
  print "        -lXm -lXt -lX11 -lm" >> filename;
  print " " >> filename;
  print "$(PROGRAM): \\" >> filename;
  print "  $(SOURCES) \\" >> filename;
  print "  $(OBJECTS) \\" >> filename;
  print "  $(" program_home ")/src/Makefile" >> filename;
  print "	$(LINK.c) \\" >> filename;
  print "	  -o $(PROGRAM) \\" >> filename;
  print "	  $(OBJECTS) \\" >> filename;
  print "	  -L/usr/local/lib \\" >> filename;
  print "	  $(LDLIBS)" >> filename;
  print " " >> filename;
  print "saber: $(SOURCES)" >> filename
  print "	#load $(CFLAGS) $(CPPFLAGS) $(SOURCES.c) $(LDLIBS)" >> filename;
  print " " >> filename;
  
  print "#" >> filename;
  print "# Dependencies" >> filename;
  print "#" >> filename;
  print "$(" program_home ")/src/" program ".o: \\" >> filename;
  print "  $(" program_home ")/src/" program ".c \\" >> filename;
  print "  $(" program_home ")/src/" dotHfilename ".h" >> filename;
  print " " >> filename;

  for (i in cb_cnt) {
    print "$(" program_home ")/src/" i ".o: \\" >> filename;
    print "  $(" program_home ")/src/" i ".c \\" >> filename;
    print "  $(" program_home ")/src/" dotHfilename ".h" >> filename;
    print " " >> filename;
  }

  print "$(" program_home ")/src/" program "_creation.o: \\" >> filename;
  print "  $(" program_home ")/src/" program "_creation.c" >> filename;
  print " " >> filename;

  print "# Each object .o file compiled should be placed in the" >> filename;
  print "# directory file of the source .c file." >> filename;
  print " " >> filename;
  print ".c.o:" >> filename;
  print "	rm -f $@" >> filename;
  print "	$(CC) -o $@ -c $(CFLAGS) $(CPPFLAGS) $<" >> filename;
  print "	echo \" \"" >> filename;
}

###############################################################################
#   make the file to rename the stubs files to "real" names
###############################################################################
function make_rename_file () {
  filename = "stubs_to_real.csh";
  print "#!/bin/csh -f" > filename;
  print "if (-e Makefile) then" >> filename;
  print "  /bin/mv -f Makefile Makefileold" >> filename;
  print "endif" >> filename;
  print "/bin/mv -f Makefile_stubs Makefile" >> filename;
  for (i in cb_cnt) {
    print "if (-e " i ".c) then" >> filename;
    print "  /bin/mv -f " i ".c " i ".cold" >> filename;
    print "endif" >> filename;
    print "/bin/mv -f " i "_stubs.c " i ".c" >> filename;
  }
  dot_h_file = dotHfilename ".h";
  print "if (-e " dot_h_file ") then" >> filename;
  print "  /bin/mv -f " dot_h_file " " dot_h_file "old" >> filename;
  print "endif" >> filename;
  print "/bin/mv -f " program "_stubs.h " dot_h_file >> filename
}
