/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/***--------------------------------------------------------------------------
* NAME:		artmap_plot_bound.c
* PURPOSE:	MultiLayer Perceptron dependent subroutines for plot_bound
* USAGE:
*
* SEE ALSO: 	plot_bound.c artmap_pack.c
*
* DESCRIPTION:  Reads in a trained model file and plots the decision
*               region and/or scatter plot in 2 selected dimensions
*
*	read_model	Read model file describing classifier parameters
*	classify	Find class of an input vector using classifier
*	get_outputs	Get address of output vector
*	over_plot	Optional way to add classifier dependent plots
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/plot/artmap_plot_bound.c,v 11.4 2001/08/23 20:18:06 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/plot/artmap_plot_bound.c,v $
* CREATED: JAN-92lck                             MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/



/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <nnlib.h>
#include <artmap_pack.h>
#include "plot_bound.h"
#include "xlippic.h"

float *outputs;
static int model;
static int artnorm;
static int n_inputs;
static int complement_coding;
static float *comp_inputs;

int
get_extra_plot_args(int argc, char **argv)
{
    return argc;
}

void
read_model (FILE *param_file, int *total_trials, int *ninputs,
	    int *noutputs, int **features, int adjust_priors,
	    float *priors_list)
{
    float alpha_a, beta_a, ro_a, alpha_b, beta_b, ro_b;

    model = artmap_read_model (param_file, &complement_coding,
			       &ro_a, &beta_a, &alpha_a,
			       &ro_b, &beta_b, &alpha_b, &artnorm);

    if (model < 0) fatal_error ("read_model -- read error %d on model file",
	model);
				/* caution: this function returns a pointer */
    n_inputs = *ninputs = *(int *)artmap_param (model,"nfinputs");
    *noutputs = *(int *)artmap_param (model,"nclasses");
    *features = *(int **)artmap_param( model, "features");
    outputs = *(float **)artmap_param(model, "outputs");

    if(complement_coding){
	comp_inputs = allocate_vector(2 * n_inputs);
    }
}

int 
classify (float *inputs)
{
    if(complement_coding){
	artmap_complement_code_vector(model, inputs, n_inputs, comp_inputs);
	return artmap_classify(model, comp_inputs, &outputs,(float)n_inputs);
    }
    if(artnorm){
	artmap_vector_norm(inputs, n_inputs);
	return artmap_classify(model, inputs, &outputs, 1.0);
    }
    return artmap_classify(model, inputs, &outputs,
			   (float)artmap_norm(inputs, n_inputs));
}


				/* used only for profile/histogram plots */
float * 
get_outputs (void)
{
    return ( *(float **) artmap_param (model,"outputs") );
}

void
over_plot (int two_d, float internals_scale, int internals_level,
	   char **plot_labels, int x_dimen, int y_dimen, float bound_xmax,
	   float bound_xmin, float bound_ymax, float bound_ymin,
	   int ubot, int utop, int vbot, int vtop)
{
    int i;
    int  u1, v1;
    int  u2, v2;
    int ncenters;
    float **pcenters;
    int *pclasses;
    int nfinputs;
    float x1,y1,x2,y2;
    float x_mid;

    x_mid = (bound_xmax + bound_xmin)/2.;

    if (two_d) {
        gcolor("foreground");
	
	ncenters = *(int *)artmap_param(model, "num_f2a");
	nfinputs = *(int *)artmap_param(model, "nfinputs");
	pcenters = *(float ***)artmap_param(model, "exemplars_a");
	pclasses = *(int **) artmap_param(model, "wj_ab");

	for (i=0;i<ncenters;i++) {
	    x1 = artmap_unrenormalize_element(model,pcenters[i][x_dimen],
					     x_dimen);
	    y1 = artmap_unrenormalize_element(model,pcenters[i][y_dimen],
					     y_dimen);
	    x2 = artmap_unrenormalize_element(model,
					     1.0-pcenters[i][x_dimen+nfinputs],
					     x_dimen);
	    y2 = artmap_unrenormalize_element(model,
					     1.0-pcenters[i][y_dimen+nfinputs],
					     y_dimen);
	    u1 = x2u (x1);
	    u2 = x2u (x2);  
	    if(x_dimen == y_dimen){
		float xwidth = (x1-x2)/2.;
		v1 = y2v(x_mid + xwidth);
		v2 = y2v(x_mid - xwidth);
	    } else {
		v1 = y2v (y1);
		v2 = y2v (y2);
	    }

	    if (u1 >= ubot && u1 <= utop && v1 >= vbot && v1 <= vtop) {
		/* draw a box here */
		gcolor("foreground");
		uvrect(u2,v2,u1,v1);
    		if (internals_level > 2) {
		    if(plot_labels != NULL &&
		       atoi(plot_labels[0]) > pclasses[i])
		      uvprintf (u1, v1, 2, "%s", plot_labels[pclasses[i]]);
    		}
	    }
	}
    }
}
int artmap_plot_get_model(void)
{
    return model;
}
int artmap_plot_get_cc(void)
{
  return complement_coding;
}

int artmap_plot_get_artnorm(void)
{
  return artnorm;
}
