/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/***---------------------------------------------------------------------------
* NAME:		bintree_plot_bound.c
* PURPOSE:	Subroutines dependent on Condensed K Nearest Neighbor
*			Classifier for Decision Region Plot
* USAGE:	bintree_plot_bound --
*			- displays flags and default settings for plot
*
* SEE ALSO: 	bintree_pack.c plot_bound.c
*
* DESCRIPTION:  Reads in a trained model file and plots the decision
*               region and/or scatter plot in 2 dimensions
*
*	read_model	read model file for classifier parameters
*	classify	find class of an input vector
*	get_outputs	unused, need for plot_bound.c
*	over_plot	overlay centers on region plot
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/plot/bintree_plot_bound.c,v 11.4 2001/08/23 20:18:06 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/plot/bintree_plot_bound.c,v $
* CREATED: Aug-93 lck				MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/



/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#define bintree_pack_h_internal_types
#define bintree_pack_h_internal_macros
#include <bintree_pack.h>
#include "plot_bound.h"
#include "xlippic.h"
				/*** GLOBALS ***/
static int max_nodes = 0;
static int prune_tree = 0;
static int test_full_tree = 0;
static ARG argtab[] =
{
    {"max_nodes", INT, &max_nodes,
       "non-terminal nodes in tree after pruning"},
    {"prune_tree", BOOLEAN, &prune_tree, 
       "prune tree to max_nodes nodes. Default, use full tree"},
};
#define TABSIZE (2)

static int model;	/* id of model */

int
get_extra_plot_args(int argc, char **argv)
{
    argc = nnargs_part(argc, argv, argtab, TABSIZE);
    if(prune_tree) test_full_tree = 0;
    else test_full_tree = 1;
    return argc;
}

float *outputs;

void
read_model(FILE *param_file, int *total_trials, int *ninputs, int *noutputs,
	   int **features, int adjust_priors, float *priors_list)
{
    model = bintree_read_model(param_file);
    if(model < 0) fatal_error ("read_model -- read error %d on model file",
	model);

    *ninputs = *(int *)bintree_param (model, "ninputs");
    *noutputs = *(int *)bintree_param (model, "nclasses");
    *features = *(int **)bintree_param (model, "features");
    outputs = allocate_vector(*noutputs);

    bintree_test_init(model, test_full_tree, max_nodes, adjust_priors,
		      priors_list);
}

int
classify(register float *inputs)
{
    return bintree_classify(model,inputs);
} 				/* END find_class() */

float *
get_outputs(void)
{
  return outputs;
}

#define ON_SCREEN(x,y,xmax,xmin,ymax,ymin)((x) <= (xmax) && (x) >= (xmin) && (y) <= (ymax) && (y) >= (ymin))

#define BETWEEN(x,x1,x2)((((x1)<=(x))&&((x)<=(x2)))||(((x2)<=(x))&&((x)<=(x1))))
void
over_plot (int two_d, float internals_scale, int internals_level,
	   char **plot_labels, int x_dimen, int y_dimen, float bound_xmax,
	   float bound_xmin, float bound_ymax, float bound_ymin,
	   int ubot, int utop, int vbot, int vtop)
{
    int  u1, v1, u2, v2, dim1, dim2;
    float dim1_max, dim1_min, dim2_max, dim2_min;
    float x1, x2, y1, y2;
    int linear, nbins, i, nodeno;
    BTNODE **bins, *node;
    
    if (two_d) {
        gcolor("foreground");
	linear = *(int *)bintree_param(model, "linear");
	bins = *(BTNODE ***)bintree_param(model, "bins");
	nbins = *(int *)bintree_param(model, "nbins");
	if(linear){
	    float Bx, By, c, x, y;
	    float mBx, mBy, mc;
	    int first, second;

	    for(i = 0; i < nbins; i++){
		/* pick out the test for the current node */
		node = bins[i];
		/* check that it is a non-pruned non-termnial node */
		if(node->yesno == EOTREE && node->nono == EOTREE) continue;
		if(node->yeschild == NULL && node->nochild == NULL) continue;

		Bx = node->beta[x_dimen];
		By = node->beta[y_dimen];
		c = node->split_constant;

		/* check that this line is on the plotting plane */
		if(Bx == 0. && By == 0.) continue;
		
		first = second = 0;
		/* find intersection with x sides of the plot */
		if(By != 0.){
		    x = bound_xmax;
		    y = (c - Bx*x)/By;
		    if(ON_SCREEN(x, y, bound_xmax, bound_xmin,
				 bound_ymax, bound_ymin)){
			first = 1; x1 = x; y1 = y;
		    }
		    x = bound_xmin;
		    y = (c - Bx*x)/By;
		    if(ON_SCREEN(x, y, bound_xmax, bound_xmin,
				 bound_ymax, bound_ymin)){
			if(first){ second = 1; x2 = x; y2 = y;}
			else { first = 1; x1 = x; y1 = y;}
		    }
		}
		/* find intersection with y sides of the plot */
		if(Bx != 0.){
		    y = bound_ymax;
		    x = (c - By*y)/Bx;
		    if(ON_SCREEN(x, y, bound_xmax, bound_xmin,
				 bound_ymax, bound_ymin)){
			if(first){ second = 1; x2 = x; y2 = y;}
			else { first = 1; x1 = x; y1 = y;}
		    }

		    y = bound_ymin;
		    x = (c - By*y)/Bx;
		    if(ON_SCREEN(x, y, bound_xmax, bound_xmin,
				 bound_ymax, bound_ymin)){
			if(first){ second = 1; x2 = x; y2 = y;}
			else { first = 1; x1 = x; y1 = y;}
		    }
		}
		/* check that we actually found two intersections */
		if(!first || !second) continue;

		/* go through ancestors to find the endpoints of the line */
		if(node->selfno != 0) /* root */
		  do{
		      nodeno = node->selfno;
		      node = bins[node->motherno];
		      mBx = node->beta[x_dimen];
		      mBy = node->beta[y_dimen];
		      mc = node->split_constant;

		      /* check that this node shows on the plotting plane */
		      if(mBx == 0. && mBy == 0.) continue;

		      /* find the intersection of our node and this ancestor */
		      if(mBx == 0.){
			  /* check for zero slopes in x direction */
			  if(Bx == 0.) continue; /* parallel lines */
			  y = mc/mBy;
			  x = (c - By*y)/Bx;
		      }  else if(Bx == 0.){
			  y = c/By;
			  x = (mc - mBy*y)/mBx;
		      } else { /* Bx and mBx both ok */
			  float denom;

			  denom = By/Bx - mBy/mBx;
			  if(denom == 0.) continue; /* parallel lines */
			  y = (c/Bx  - mc/mBx)/denom;
			  x = (mc - mBy*y)/mBx;
		      }
		      /* check if the intersection is in our current line
			 segment */
		      if(BETWEEN(x, x1, x2) && BETWEEN(y, y1, y2)){
			  int myes;

			  /* find which side of the mother line x1,y1 is on */
			  myes = (mBx*x1 + mBy*y1 - mc) <= 0;

			  /* set one of the end points of our line */
			  if(node->yesno == nodeno){
			      if(myes){ x2 = x; y2 = y;}
			      else { x1 = x; y1 = y;}
			  } else {
			      if(myes){ x1 = x; y1 = y;}
			      else {x2 = x; y2 = y;}
			  }
		      }
		  }while(node->selfno != 0); /* root */

		/* draw the line */
		u1 = x2u(x1); u2 = x2u(x2);
		v1 = y2v(y1); v2 = y2v(y2);
		uvline(u1, v1, u2, v2);
		/* print out the node number */
		if(internals_level > 2){
		    int ix, iy;

		    if(x1 < x2){
			ix = x2u(x1); iy = y2v(y1);
			if(y1 < y2) iy += 10; else iy -= 30;
		    } else {
			ix = x2u(x2); iy = y2v(y2);
			if(y1 > y2) iy += 10; else iy -= 30;
		    }
		    uvprintf(ix + 20, iy, 1, "%d", i);
		}
		gflush();
	    }
	} else { /* univariant */
	    for(i = 0; i < nbins; i++){
		/* pick out the test for the current node */
		node = bins[i];
		/* check that it is a non-pruned non-termnial node */
		if(node->yesno == EOTREE && node->nono == EOTREE) continue;
		if(node->yeschild == NULL && node->nochild == NULL) continue;
		if(node->split_input == x_dimen){
		    dim1 = x_dimen;
		    dim2 = y_dimen;
		    dim2_max = bound_ymax;
		    dim2_min = bound_ymin;
		} else if(node->split_input == y_dimen){
		    dim1 = y_dimen;
		    dim2 = x_dimen;
		    dim2_max = bound_xmax;
		    dim2_min = bound_xmin;
		} else continue; /* this node doesn't intersect the display */
		dim1_max = dim1_min = node->split_constant;
		/* go through ancestors to find the endpoints of the line */
		if(node->selfno != 0) /* root */
		  do{
		      nodeno = node->selfno;
		      node = bins[node->motherno];
		      if(node->split_input == dim2){
			  if(node->yesno == nodeno){
			      /* this is a yes node, check the max */
			      if(node->split_constant < dim2_max)
				dim2_max = node->split_constant;
			  } else {
			      /* this is a no node, check the min */
			      if(node->split_constant > dim2_min)
				dim2_min = node->split_constant;
			  }
		      }
		  }while(node->selfno != 0); /* root */
		/* draw the line */
		if(dim1 == x_dimen){
		    x1 = dim1_max; x2 = dim1_min;
		    y1 = dim2_max; y2 = dim2_min;
		} else {
		    x1 = dim2_max; x2 = dim2_min;
		    y1 = dim1_max; y2 = dim1_min;
		}		    
		u1 = x2u(x1); u2 = x2u(x2);
		v1 = y2v(y1); v2 = y2v(y2);

		uvline(u1, v1, u2, v2);
		/* print out the node number */
		if(internals_level > 2){
		    int ix, iy;

		    if(dim1_max < x2){
			ix = x2u(x1); iy = y2v(y1);
			if(y1 < y2) iy += 10; else iy -= 30;
		    } else {
			ix = x2u(x2); iy = y2v(y2);
			if(y1 > y2) iy += 10; else iy -= 30;
		    }
		    uvprintf(ix + 20, iy, 1, "%d", i);
		}
		gflush();
	    } /* finish drawing lines */
	} /* univariant */
    } /* doing a two-d plot (not profile) */
}
