/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/***--------------------------------------------------------------------------
* NAME:		gauss_plot_bound.c
* PURPOSE:	MultiLayer Perceptron dependent subroutines for plot_bound
* USAGE:
*
* SEE ALSO: 	plot_bound.c
*
* DESCRIPTION:  Reads in a trained model file and plots the decision
*               region and/or scatter plot in 2 selected dimensions
*
*	read_model	Read model file describing classifier parameters
*	classify	Find class of an input vector
*	over_plot	overlay gauss internals on bound plot
*	get_outputs	return the pointer to the classifiers output vector
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/plot/gauss_plot_bound.c,v 11.4 2001/08/23 20:18:06 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/plot/gauss_plot_bound.c,v $
* CREATED: 4-MAR-91 dan                             MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/


/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#include <math.h>
#include <gauss_pack.h>
#include "plot_bound.h"
#include "xlippic.h"

				/*** GLOBALS ***/ 
static int model;			/* id of model */

static int noutputs;
static float *outputs;

int
get_extra_plot_args(int argc, char **argv)
{
    return argc;
}

void
read_model (FILE *param_file, int *total_trials, int *pninputs,
	    int *pnoutputs, int **features, int adjust_priors,
	    float *priors_list)
{
    model = gauss_read_model(param_file);

    gauss_test_build (model);
    gauss_test_init (model, adjust_priors, priors_list);

    *pninputs = *(int *) gauss_param(model, "ninputs", 0);
    noutputs = *pnoutputs = *(int *) gauss_param(model, "noutputs", 0);
    *total_trials = *(int *) gauss_param(model, "total_trials", 0);
    *features = *(int **) gauss_param(model, "features", 0);
    outputs = *(float **) gauss_param(model, "outputs", 0);

    if (model < 0) fatal_error ("read_model -- read error %d on model file",
	model);
}


int
classify (float *inputs)
{
    int class, i;
    class = gauss_classify (model, inputs);
    for(i = 0; i < noutputs; i++){
	outputs[i] = nn_exp10(outputs[i]);
    }

    return class;
}


				/* used only for profile/histogram plots */
float *
get_outputs (void)
{
    outputs = *(float **) gauss_param (model, "outputs", 0);
    return outputs;
}

void
over_plot (int two_d, float internals_scale, int internals_level,
	   char **plot_labels, int x_dimen, int y_dimen,
	   float bound_xmax, float bound_xmin, float bound_ymax,
	   float bound_ymin, int ubot, int utop, int vbot, int vtop)
{
    float *means, **covariance, *inv_variance;
    int i, full_covar, per_class;
    int u1, v1;
    float x, y, a, b, c;
    double theta;
    float x_mid;

    x_mid = (bound_xmax + bound_xmin)/2.;
    
    full_covar = *(int *)gauss_param(model, "full_covar", 0);
    per_class = *(int *)gauss_param(model, "per_class", 0);
    if (two_d) {
	gcolor("foreground");
	if(!per_class){ /* grand variances */
	    if(full_covar){
		/* a and b are reversed because matrix is inverted */
		covariance = *(float ***) gauss_param( model, "covariance",0);
		b = covariance[x_dimen][x_dimen];
		a = covariance[y_dimen][y_dimen];
		c = -1. * covariance[x_dimen][y_dimen];
		get_ellipse_rotation(a, b, c, &x, &y, &theta);
	    } else {
		inv_variance = *(float **)gauss_param(model, "variance", 0);
	    }
	}
	for(i = 0; i < noutputs; i++){
	    /* we should check to see if this center is close enough to
	       the plane we are drawing */
	    means = *(float **) gauss_param(model, "mean", i);
	    u1 = x2u (means[x_dimen]);
	    if(x_dimen == y_dimen) v1 = y2v(x_mid);
	    else v1 = y2v (means[y_dimen]);
	    /* if the center is on the screen */
	    if (u1 >= ubot && u1 <= utop && v1 >= vbot && v1 <= vtop) {
		if(!per_class){ /* grand */
		    if(!full_covar){ /* diagonal */
			x = sqrt(1./inv_variance[x_dimen]);
			y = sqrt(1./inv_variance[y_dimen]);
			draw_ellipse (internals_scale, u1, v1, 
				      xlength2u (x), ylength2v (y));
		    } else { /* full */
			draw_angle_ellipse (internals_scale, u1, v1, 
					    xlength2u (x), ylength2v (y),
					    theta);
		    }
		} else { /* per class variances */
		    if(!full_covar){ /* diagonal */
			inv_variance = *(float **)
			  gauss_param( model, "per_class_variance", i);
			x = sqrt(1./inv_variance[x_dimen]);
			y = sqrt(1./inv_variance[y_dimen]);
			draw_ellipse (internals_scale, u1, v1, 
				      xlength2u (x), ylength2v (y));
		    } else { /* full */
			/* a and b are reversed because matrix is inverted */
			covariance = *(float ***)
			  gauss_param( model, "per_class_covariance", i);
			b = covariance[x_dimen][x_dimen];
			a = covariance[y_dimen][y_dimen];
			c = -1. * covariance[x_dimen][y_dimen];
			get_ellipse_rotation(a, b, c, &x, &y, &theta);
			draw_angle_ellipse (internals_scale, u1, v1,
					    xlength2u (x), ylength2v (y),
					    theta);
		    }
		}
		if (internals_level > 2) {
		    if(plot_labels != NULL &&
		       atoi(plot_labels[0]) > i)
		      uvprintf (u1, v1, 2, "%s", plot_labels[i+1]);
		}
	    }
	}
    }
}
