/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/***--------------------------------------------------------------------------
* NAME:		gmix_plot_bound.c
* PURPOSE:	MultiLayer Perceptron dependent subroutines for plot_bound
* USAGE:
*
* SEE ALSO: 	plot_bound.c
*
* DESCRIPTION:  Reads in a trained model file and plots the decision
*               region and/or scatter plot in 2 selected dimensions
*
*	read_model	Read model file describing classifier parameters
*	classify	Find class of an input vector
*	get_outputs	unused, needed by plot_bound.c
*	over_plot	overlay gmix internals on bound plot
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/plot/gmix_plot_bound.c,v 11.4 2001/08/23 20:18:07 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/plot/gmix_plot_bound.c,v $
* CREATED: 4-MAR-91 dan                             MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/


/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <math.h>
#include <nnlib.h>
#include <gmix_pack.h>
#include "plot_bound.h"
#include "xlippic.h"

static int model;			/* id of model */
static int noutputs;
static float *outputs;

int get_extra_plot_args(int argc, char **argv)
{
    return argc;
}

void
read_model (FILE *param_file, int *total_trials, int *pninputs,
	    int *pnoutputs, int **features, int adjust_priors,
	    float *priors_list)
{
    model = gmix_read_model(param_file);

    *pninputs = *(int *) gmix_param(model, "ninputs", 0, 0);
    noutputs = *pnoutputs = *(int *) gmix_param(model, "noutputs", 0, 0);
    *total_trials = *(int *) gmix_param(model, "total_trials", 0, 0);
    *features = *(int **) gmix_param(model, "features", 0, 0);
    outputs = *(float **) gmix_param (model, "outputs", 0, 0);

    if (model < 0) fatal_error ("read_model -- read error %d on model file",
	model);

    gmix_test_init(model, adjust_priors, priors_list);
}


int 
classify (float *inputs)
{
    int class, i;
    class = gmix_classify (model, inputs);
    for(i = 0; i < noutputs; i++){
	outputs[i] = nn_exp10(outputs[i]);
    }
    return class;
}


				/* used only for profile/histogram plots */
float *
get_outputs (void)
{
    
    return *(float **) gmix_param (model, "outputs", 0, 0);
}

void
over_plot (int two_d, float internals_scale, int internals_level,
	   char **plot_labels, int x_dimen, int y_dimen, float bound_xmax,
	   float bound_xmin, float bound_ymax, float bound_ymin,
	   int ubot, int utop, int vbot, int vtop)
{
    int u1, v1, ulength, vlength;
    int full_flag, tied_flag, grand_flag, class, ngauss, g;
    float *vars, **covars, *means, x, y;
    double theta;
    float x_mid;

    x_mid = (bound_xmax + bound_xmin)/2.;
    
    if (two_d) {
	gcolor("foreground");
	tied_flag = *(int *) gmix_param(model, "tied_flag", 0, 0);
	grand_flag = *(int *) gmix_param(model, "grand_flag", 0, 0);
	full_flag = *(int *) gmix_param(model, "full_flag", 0, 0);
	if(tied_flag){
	    if(grand_flag){
		if(full_flag){
		    covars = *(float ***) gmix_param(model, "full_var", 0, 0);
		    get_ellipse_rotation(covars[x_dimen][x_dimen],
					 covars[y_dimen][y_dimen],
					 covars[x_dimen][y_dimen],
					 &y, &x, &theta);
		} else {
		    vars = *(float **) gmix_param(model, "diag_var", 0, 0);
		    x = sqrt(1./vars[x_dimen]);
		    y = sqrt(1./vars[y_dimen]);
		}
		ulength = xlength2u (x);
		vlength = ylength2v (y);
	    }
	    ngauss = *(int *) gmix_param(model, "ngauss", 0, 0);
	    for(g = 0; g < ngauss; g++){
		means = *(float **) gmix_param(model, "tied_means",
					       0, g);
		u1 = x2u (means[x_dimen]);
		if(x_dimen == y_dimen) v1 = y2v(x_mid);
		else v1 = y2v (means[y_dimen]);
		
		if(!grand_flag){
		    if(full_flag){
			covars = *(float ***) gmix_param(model,
							 "tied_full_var",
							 0, g);
			get_ellipse_rotation(covars[x_dimen][x_dimen],
					     covars[y_dimen][y_dimen],
					     covars[x_dimen][y_dimen],
					     &y, &x, &theta);
		    } else {
			vars = *(float **) gmix_param(model, "tied_diag_var",
						      0, g);
			x = sqrt(1./vars[x_dimen]);
			y = sqrt(1./vars[y_dimen]);
		    }
		    ulength = xlength2u (x);
		    vlength = ylength2v (y);
		}
		if (u1 >= ubot && u1 <= utop &&
		    v1 >= vbot && v1 <= vtop) {
		    if(full_flag){
			draw_angle_ellipse (internals_scale, u1, v1,
					    ulength, vlength, theta);
		    } else {
			draw_ellipse (internals_scale, u1, v1,
				      ulength, vlength);
		    }
		    if (internals_level > 2) {
			uvprintf (u1, v1, 2, "%d", g);
		    }
		}
	    }
	} else {
	    if(grand_flag == 2){
		if(full_flag){
		    covars = *(float ***) gmix_param(model, "full_var", 0, 0);
		    get_ellipse_rotation(covars[x_dimen][x_dimen],
					 covars[y_dimen][y_dimen],
					 covars[x_dimen][y_dimen],
					 &y, &x, &theta);
		} else {
		    vars = *(float **) gmix_param(model, "diag_var", 0, 0);
		    x = sqrt (1./vars[x_dimen]);
		    y = sqrt (1./vars[y_dimen]);
		}
		ulength = xlength2u (x);
		vlength = ylength2v (y);
	    }
	    for(class = 0; class < noutputs; class++){
		if(grand_flag == 1){
		    if(full_flag){
			covars = *(float ***) gmix_param(model,
							 "class_full_var",
							 class, 0);
			get_ellipse_rotation(covars[x_dimen][x_dimen],
					     covars[y_dimen][y_dimen],
					     covars[x_dimen][y_dimen],
					     &y, &x, &theta);
		    } else {
			vars = *(float **) gmix_param(model, "class_diag_var",
						      class, 0);
			x = sqrt(1./vars[x_dimen]);
			y = sqrt(1./vars[y_dimen]);
		    }
		    ulength = xlength2u (x);
		    vlength = ylength2v (y);
		}
		ngauss = *(int *) gmix_param(model, "class_ngauss",
					     class, 0);
		for(g = 0; g < ngauss; g++){
		    means = *(float **) gmix_param(model, "class_means",
						   class, g);
		    if(!grand_flag){
			if(full_flag){
			    covars =
			      *(float ***) gmix_param(model,
						      "per_class_full_var",
						      class, g);
			    get_ellipse_rotation(covars[x_dimen][x_dimen],
						 covars[y_dimen][y_dimen],
						 covars[x_dimen][y_dimen],
						 &y, &x, &theta);
			} else {
			    vars =
			      *(float **) gmix_param(model,
						     "per_class_diag_var",
						     class, g);
			    x = sqrt(1./vars[x_dimen]);
			    y = sqrt(1./vars[y_dimen]);
			}
			ulength = xlength2u (x);
			vlength = ylength2v (y);
		    }

		    u1 = x2u (means[x_dimen]);
		    if(x_dimen == y_dimen) v1 = y2v(x_mid);
		    else v1 = y2v (means[y_dimen]);
		    if (u1 >= ubot && u1 <= utop &&
			v1 >= vbot && v1 <= vtop) {
			if(full_flag){
			    draw_angle_ellipse (internals_scale, u1, v1,
					  ulength, vlength, theta);
			} else {
			    draw_ellipse (internals_scale, u1, v1,
					  ulength, vlength);
			}
			if (internals_level > 2) {
			    if(plot_labels != NULL &&
			       atoi(plot_labels[0]) > class)
			      uvprintf (u1, v1, 2, "%s", plot_labels[class+1]);
			}
		    }
		}
	    }
	}
    }
}
