/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/***--------------------------------------------------------------------------
* NAME:		histogram_plot_bound.c
* PURPOSE:	Subroutines dependent on Histogram Classifier for
*			Decision Region Plot
* USAGE:	histogram_plot_bound --
*			- displays flags and default settings for plot
*
* SEE ALSO: 	histogram_pack.c plot_bound.c
*
* DESCRIPTION:  Reads in a trained model file and plots the decision
*               region and/or scatter plot in 2 dimensions
*
*	read_model	read model file for classifier parameters
*	classify	find class of an input vector
*	get_outputs	unused, needed by plot_bound.c
*	over_plot	overlay centers on region plot
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/plot/histogram_plot_bound.c,v 11.5 2001/08/23 20:18:07 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/plot/histogram_plot_bound.c,v $
* CREATED: 28-Aug-94 lck				MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/



/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#define histogram_pack_h_internal_types
#include <histogram_pack.h>
#include "plot_bound.h"
#include "xlippic.h"
#define SQUARE (0)

static int model;			/* id of model */
static int use_gauss = 0;
static ARG argtab[] =
{
  {"use_gauss", FLAG, &use_gauss,
     "use Gaussian outputs for outlyers. Default: allow unknown class"},
};
#define TABSIZE (1)

int get_extra_plot_args(int argc, char **argv)
{
    int nargc;
    nargc = nnargs_part(argc, argv, argtab, TABSIZE);
    return nargc;
}

void read_model (FILE *param_file, int *total_trials, int *ninputs, int *noutputs, int **features, int adjust_priors, float *priors_list)
{
    int rv, nraw;

    rv = feature_read_paramfile(param_file, features, &nraw, ninputs);
    if (rv < 0)
      fatal_error("read_model -- read error %d %s", rv,
		  "on feature selection parameters in model file");

    model = histogram_read_model(param_file);
    if(model < 0) fatal_error ("read_model -- read error %d on model file",
	model);

    *ninputs = *(int *)histogram_param (model, "ninputs", 0, 0);
    *noutputs = *(int *)histogram_param (model, "nclasses", 0, 0);
    histogram_test_alloc(model);
    histogram_test_init(model, adjust_priors, priors_list, use_gauss);
}

int classify(register float *inputs)
{
    return histogram_classify(model, inputs);
}

float *get_outputs(void)
{
return (*(float **) histogram_param(model, "outputs", 0, 0));
;
}

void
over_plot (int two_d, float internals_scale, int internals_level, char **plot_labels, int x_dimen, int y_dimen, float bound_xmax, float bound_xmin, float bound_ymax, float bound_ymin, int ubot, int utop, int vbot, int vtop)
{
    int c, *nbins, b, noutputs, grand;
    int u1, u2, v1, v2;
    HISTOGRAM_BIN *bin;
    char *label, number[16];
    
    nbins = *(int **)histogram_param(model, "nbins", 0, 0);
    noutputs = *(int *) histogram_param(model, "noutputs", 0, 0);
    grand = *(int *)histogram_param(model, "grand", 0, 0);
    /* plot each edge line for the bins in each class */
    for(c = 0; c < noutputs; c++){
	if(grand && c > 0) break;
	if(grand) label = "";
	else
	  if(plot_labels != NULL && atoi(plot_labels[0]) > c)
	    label = plot_labels[c+1];
	  else {
	      sprintf(number, "%d", c);
	      label = number;
	  }

	bin = (HISTOGRAM_BIN *) histogram_param( model, "bin", c, x_dimen);
	  
	v1 = y2v(bound_ymin);
	v2 = y2v(bound_ymax);
	for(b = 0; b <= nbins[c]; b++){
	    if(bin->bin_ends[b] < bound_xmin ||
	       bin->bin_ends[b] > bound_xmax) continue;
	    u1 = u2 = x2u(bin->bin_ends[b]);
	    uvline(u1, v1, u2, v2);
	    if(internals_level > 2) uvprintf(u1, v1+10, 1, "%s", label);
	}

	if(x_dimen != y_dimen){
	  bin = (HISTOGRAM_BIN *) histogram_param( model, "bin", c, y_dimen);
	    u1 = x2u(bound_xmin);
	    u2 = x2u(bound_xmax);
	    for(b = 0; b <= nbins[c]; b++){
		if(bin->bin_ends[b] < bound_ymin ||
		   bin->bin_ends[b] > bound_ymax) continue;
		v1 = v2 = y2v(bin->bin_ends[b]);
		uvline(u1, v1, u2, v2);
		if(internals_level > 2) uvprintf(u1+10, v1, 1, "%s", label);
	    }
	}
	gflush();
    }
}
