/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/***---------------------------------------------------------------------------
* NAME:		hyper_plot_bound.c
* PURPOSE:	Subroutines dependent on Hyperspheres for Decision Region Plot
* USAGE:	hyper_plot_bound --
*			- displays flags and default settings for plot
*
* SEE ALSO: 	hyper_pack.c plot_bound.c
*
* DESCRIPTION:  Reads in a trained model file and plots the decision
*               region and/or scatter plot in 2 dimensions
*
*	read_model	read model file for classifier parameters
*	classify	find class of an input vector
*	get_outputs	unused, needed by plot_bound.c
*	over_plot	overlay hypersphere internals on bound plot
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/plot/hyper_plot_bound.c,v 11.4 2001/08/23 20:18:07 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/plot/hyper_plot_bound.c,v $
* CREATED: 4-MAR-91 dan                             MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/



/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <math.h>
#include <nnlib.h>
#include <hyper_pack.h>
#include "plot_bound.h"
#include "xlippic.h"

				/*** GLOBALS ***/ 
int model;			/* id of model */

int
get_extra_plot_args(int argc, char **argv)
{
    return argc;
}

void
read_model (FILE *param_file, int *total_trials, int *ninputs,
	    int *noutputs, int **features, int adjust_priors,
	    float *priors_list)
{
    model = hyper_read_model(param_file, total_trials);
    if(model < 0) fatal_error ("read_model -- read erro %d on model file",
	    model);

    *ninputs = *(int *)hyper_param(model, "ninputs");
    *noutputs = *(int *)hyper_param(model, "nclasses");
    *features = *(int **)hyper_param( model, "features");
}

int
classify(register float *inputs)
{
    return hyper_classify(model,inputs);
}

float *
get_outputs(void)
{
  return (float *) NULL;
}

void
over_plot (int two_d, float internals_scale, int internals_level,
		char **plot_labels, int x_dimen, int y_dimen,
		float bound_xmax, float bound_xmin, float bound_ymax,
		float bound_ymin, int ubot, int utop, int vbot, int vtop)
{
    int i;
    int  u1, v1;
    int ncenters;
    float **pcenters, *pradii;
    int *pclasses;
    float x_mid;

    x_mid = (bound_xmax + bound_xmin)/2.;

    if (two_d) {
        gcolor("foreground");
	
	ncenters = *(int *)hyper_param(model, "accum_trials");
	pcenters = *(float ***)hyper_param(model, "centroids");
	pradii = *(float **)hyper_param(model, "radii");
	pclasses = *(int **)hyper_param(model, "classes");

	for (i=0;i<ncenters;i++) {
	    u1 = x2u (pcenters[i][x_dimen]);
	    if(x_dimen == y_dimen) v1 = y2v(x_mid);
	    else v1 = y2v (pcenters[i][y_dimen]);
	    if (u1 >= ubot && u1 <= utop && v1 >= vbot && v1 <= vtop) {
	        draw_ellipse (internals_scale, u1, v1, 
	            xlength2u (sqrt (pradii[i])),
	            ylength2v (sqrt (pradii[i])));
    		if (internals_level > 2) {
		    if(plot_labels != NULL &&
		       atoi(plot_labels[0]) > pclasses[i])
		      uvprintf (u1, v1, 2, "%s", plot_labels[pclasses[i]+1]);
    		}
	    }
	}
    }
}
