/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/***---------------------------------------------------------------------------
* NAME:		irbfm_plot_bound.c
*
* SEE ALSO: 	plot_bound.c irbf_pack.c
*
* DESCRIPTION:  Reads in a trained model file and plots the decision
*               region and/or scatter plot in 2 selected dimensions
*
*	read_model	Read model file describing classifier parameters
*	classify	Find class of an input vector using classifier
*	get_outputs	Get address of output vector
*	eval		Map input to output using classifier
*	over_plot	Optional way to add classifier dependent plots
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/plot/irbfm_plot_map.c,v 11.4 2001/08/23 20:18:07 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/plot/irbfm_plot_map.c,v $
* CREATED: 19-OCT-92					MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/


/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#include <irbf_pack.h>
#include "plot_map.h"

float *outputs;
static float extra_hspread=1.0;
static ARG argtab[] =
{
    {"exhspread", FLOAT, (int *)&extra_hspread, "test factor for all spreads"},
};
#define TABSIZE (1)
static int model;			/* id of model */

int get_extra_map_args(int argc, char **argv)
{
    int nargc;
    nargc = nnargs_part(argc, argv, argtab, TABSIZE);
    return nargc;
}

void read_model (FILE *param_file, int *total_trials, int *ninputs, int *noutputs, int **features)
{
    float weight_eta = 0, mean_eta = 0, inv_var_eta = 0,
      min_var = 0;
    
    model = irbf_read_model (param_file, weight_eta, mean_eta, inv_var_eta,
			     min_var);

    *ninputs = *(int *)irbf_param(model,"ninputs");
    *noutputs = *(int *)irbf_param(model,"nclasses");
    *features = *(int **)irbf_param( model, "features");
    outputs = *(float **)irbf_param(model, "outputs");

    irbf_test_init(model, 0, NULL, extra_hspread);

    if (model < 0) fatal_error ("read_model -- read error %d on model file",
	model);
}
/* used only for profile/histogram plots */
float *get_outputs (void)
{
    return outputs;
}

void eval (float *inputs)
{
    irbf_eval(model, inputs);
}
