/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/***-------------------------------------------------------------------------
* NAME:		mlp_plot_bound.c
* PURPOSE:	MultiLayer Perceptron dependent subroutines for plot_bound
* USAGE:
*
* SEE ALSO: 	plot_bound.c mlp_pack.c
*
* DESCRIPTION:  Reads in a trained model file and plots the decision
*               region and/or scatter plot in 2 selected dimensions
*
*	read_model	Read model file describing classifier parameters
*	classify	Find class of an input vector using classifier
*	get_outputs	Get address of output vector
*	over_plot	Optional way to add classifier dependent plots
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/plot/mlp_plot_bound.c,v 11.4 2001/08/23 20:18:08 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/plot/mlp_plot_bound.c,v $
* CREATED: 4-MAR-91 dan                             MIT LINCOLN LABORATORY
------------------------------------------------------------------------***/


/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#include <mlp_pack.h>
#include "plot_bound.h"
#include "xlippic.h"

				/*** GLOBALS ***/ 
static int model;			/* id of model */

int get_extra_plot_args(int argc, char **argv)
{
    return argc;
}

void
read_model (FILE *param_file, int *total_trials, int *ninputs, int *noutputs,
	    int **features, int adjust_priors, float *priors_list)
{
    model = mlp_read_model (param_file);

    if (model < 0) fatal_error ("read_model -- read error %d on model file",
	model);
				/* caution: this function returns a pointer */
    *ninputs = *(int *)mlp_param (model,"ninputs",0);
    *noutputs = *(int *)mlp_param (model,"noutputs",0);
    *features = *(int **)mlp_param( model, "features",0);

    mlp_test_init(model, adjust_priors, priors_list);
}


int 
classify (float *inputs)
{
    return mlp_classify(model,inputs);
}


				/* used only for profile/histogram plots */
float * 
get_outputs (void)
{
    return ( *(float **) mlp_param (model,"output",0) );
}
void draw_line(int level, int hidden, double slope, double islope,
	       double xintercept, double yintercept,
	       float bound_xmax, float bound_xmin, 
	       float bound_ymax, float bound_ymin);

void
over_plot (int two_d, float internals_scale, int internals_level,
	   char **plot_labels, int x_dimen, int y_dimen,
	   float bound_xmax, float bound_xmin,
	   float bound_ymax, float bound_ymin,
	   int ubot, int utop, int vbot, int vtop)
{
    float **weights;
    int layer=1, *layers;
    double slope, islope;
    double xintercept, yintercept;
    int b_dimen, hidden;
    
    if (two_d) {
	gcolor("foreground");
	layers = *(int **)mlp_param (model,"layers",0);
	weights = *(float ***)mlp_param( model, "weights",layer);
    	b_dimen = layers[0] - 1;

    	for (hidden=0; hidden<layers[layer]-1; hidden++) {
	    if(x_dimen == y_dimen){
		slope = 1000;
		islope = 0;
	    } else {
		slope = -weights[hidden][x_dimen] / weights[hidden][y_dimen];
		if(slope != 0.) islope = 1.0/slope;
		else islope = 1000;
	    }
	    xintercept = -weights[hidden][b_dimen] / weights[hidden][x_dimen];
	    yintercept = -weights[hidden][b_dimen] / weights[hidden][y_dimen];
	    
	    draw_line (internals_level, hidden, slope, islope, xintercept, 
		       yintercept, bound_xmax, bound_xmin, bound_ymax,
		       bound_ymin);
	    gflush();
	}
    	
    }
}

void
draw_line (int level, int hidden, double slope, double islope,
	   double xintercept, double yintercept, 
	   float bound_xmax, float bound_xmin,
	   float bound_ymax, float bound_ymin)
{
    float xdisp[2];			/* values to plot */
    float ydisp[2];
    float x,y;
    int i=0;
    int ix, iy;

    /* determines how line intercepts screen and draws full length
       line across screen */

    if(islope == 0.){
	x = xintercept;
	if (x <= bound_xmax && x >= bound_xmin) {
	    y = bound_ymax;
	    xdisp[i] = x;
	    ydisp[i] = y;
	    i++;

	    y = bound_ymin;
	    xdisp[i] = x;
	    ydisp[i] = y;
	    i++;
	}
    } else if (slope == 0.){
	y = yintercept;
	if ( y <= bound_ymax && y >= bound_ymin) {
	    x = bound_xmin;
	    xdisp[i] = x;
	    ydisp[i] = y;
	    i++;

	    x = bound_xmax;
	    xdisp[i] = x;
	    ydisp[i] = y;
	    i++;
	}
    } else {
	/* check intercept on top */
	y = bound_ymax;
	x = islope*y + xintercept;
	if ( x <= bound_xmax && x >= bound_xmin) {
	    xdisp[i] = x;
	    ydisp[i] = y;
	    i++;
	}

	/* check intercept on bottom */
	y = bound_ymin;
	x = islope*y + xintercept;
	if ( x <= bound_xmax && x >= bound_xmin) {
	    xdisp[i] = x;
	    ydisp[i] = y;
	    i++;
	}

	/* check intercept on left side */
	if ( i < 2 ) {
	    x = bound_xmin;
	    y = slope*x + yintercept;
	    if ( y <= bound_ymax && y >= bound_ymin) {
		xdisp[i] = x;
		ydisp[i] = y;
		i++;
	    }
	}

	/* check intercept on right side */
	if ( i < 2 ) {
	    x = bound_xmax;
	    y = slope*x + yintercept;
	    if ( y <= bound_ymax && y >= bound_ymin) {
		xdisp[i] = x;
		ydisp[i] = y;
		i++;
	    }
	}
    }
    /* plot line across screen */
    if (i >= 2){
	uvline(x2u(xdisp[0]), y2v(ydisp[0]), x2u(xdisp[1]), y2v(ydisp[1]));
	if (level > 2) { 
	    if ( xdisp[0] < xdisp[1]) {
		ix = x2u(xdisp[0]);
		iy = y2v(ydisp[0]);
		if (ydisp[1] > ydisp[0]) iy += 10; else iy -= 30;
	    } else {
		ix = x2u(xdisp[1]);
		iy = y2v(ydisp[1]);
		if (ydisp[0] > ydisp[1]) iy += 10; else iy -= 30;
	    }
	    uvprintf (ix + 20, iy, 1, "%d", hidden);
	}
    }
}
