/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/***---------------------------------------------------------------------------
* NAME:		mlp_plot_bound.c
* PURPOSE:	MultiLayer Perceptron dependent subroutines for plot_bound
* USAGE:
*
* SEE ALSO: 	plot_bound.c mlp_pack.c
*
* DESCRIPTION:  Reads in a trained model file and plots the decision
*               region and/or scatter plot in 2 selected dimensions
*
*	read_model	Read model file describing classifier parameters
*	classify	Find class of an input vector using classifier
*	get_outputs	Get address of output vector
*	eval		Map input to output using classifier
*	over_plot	Optional way to add classifier dependent plots
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/plot/mlpm_plot_map.c,v 11.4 2001/08/23 20:18:08 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/plot/mlpm_plot_map.c,v $
* CREATED: 13-Jul-lck				MIT Lincoln Laboratory
--------------------------------------------------------------------------***/



/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include "nnlib.h"
#include <mlp_pack.h>
#include "plot_map.h"

				/*** GLOBALS ***/ 
static int model;			/* id of model */
static float *outputs;

int get_extra_map_args(int argc, char **argv)
{
    return argc;
}

void read_model (FILE *param_file, int *total_trials, int *ninputs,
		 int *noutputs, int **features)
{
    model = mlp_read_model (param_file);

    if (model < 0) fatal_error ("read_model -- read error %d on model file",
	model);
				/* caution: this function returns a pointer */
    *ninputs = *(int *)mlp_param (model,"ninputs",0);
    *noutputs = *(int *)mlp_param (model,"noutputs",0);
    *features = *(int **)mlp_param( model, "features",0);
    outputs = *(float **) mlp_param (model,"output",0);
}

float * get_outputs (void)
{
    return outputs;
}

void 
eval (float *inputs)
{
    mlp_eval(model, inputs);
}
