/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/***--------------------------------------------------------------------------
* NAME:		nbayes_plot_bound.c
* PURPOSE:	Subroutines dependent on Nbayes Classifier for
*			Decision Region Plot
* USAGE:	nbayes_plot_bound --
*			- displays flags and default settings for plot
*
* SEE ALSO: 	nbayes_pack.c plot_bound.c
*
* DESCRIPTION:  Reads in a trained model file and plots the decision
*               region and/or scatter plot in 2 dimensions
*
*	read_model	read model file for classifier parameters
*	classify	find class of an input vector
*	get_outputs	unused, needed by plot_bound.c
*	over_plot	overlay centers on region plot
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/plot/nbayes_plot_bound.c,v 11.2 2001/08/23 20:18:08 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/plot/nbayes_plot_bound.c,v $
* CREATED: 28-Aug-94 lck				MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/



/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#define nbayes_pack_h_internal_types
#include <nbayes_pack.h>
#include "plot_bound.h"
#include "xlippic.h"
#define SQUARE (0)

static int model;			/* id of model */

int get_extra_plot_args(int argc, char **argv)
{
    return argc;
}

void read_model (FILE *param_file, int *total_trials, int *ninputs, int *noutputs, int **features, int adjust_priors, float *priors_list)
{
    int rv, nraw;

    rv = feature_read_paramfile(param_file, features, &nraw, ninputs);
    if (rv < 0)
      fatal_error("read_model -- read error %d %s", rv,
		  "on feature selection parameters in model file");

    model = nbayes_read_model(param_file);
    if(model < 0) fatal_error ("read_model -- read error %d on model file",
	model);

    *ninputs = *(int *)nbayes_param (model, "ninputs", 0, 0);
    *noutputs = *(int *)nbayes_param (model, "nclasses", 0, 0);
    nbayes_test_alloc(model);
    nbayes_test_init(model, adjust_priors, priors_list);
}

int classify(register float *inputs)
{
    return nbayes_classify(model, inputs);
}

float *get_outputs(void)
{
return (*(float **) nbayes_param(model, "outputs", 0, 0));
;
}

void
over_plot (int two_d, float internals_scale, int internals_level, char **plot_labels, int x_dimen, int y_dimen, float bound_xmax, float bound_xmin, float bound_ymax, float bound_ymin, int ubot, int utop, int vbot, int vtop)
{
    int *nbins, b, ninputs;
    int u1, u2, v1, v2;
    float *bin_ends;
    
    nbins = *(int **)nbayes_param(model, "nbins", 0, 0);
    ninputs = *(int *) nbayes_param(model, "ninputs", 0, 0);
    /* plot each edge line for the bins in each class */
    bin_ends = *(float **)nbayes_param(model, "bin", x_dimen, 0);
	  
    v1 = y2v(bound_ymin);
    v2 = y2v(bound_ymax);
    for(b = 0; b <= nbins[x_dimen]; b++){
      if(bin_ends[b] < bound_xmin ||
	 bin_ends[b] > bound_xmax) continue;
      u1 = u2 = x2u(bin_ends[b]);
      uvline(u1, v1, u2, v2);
    }

    if(x_dimen != y_dimen){
      bin_ends = *(float **)nbayes_param(model, "bin", y_dimen, 0);
      u1 = x2u(bound_xmin);
      u2 = x2u(bound_xmax);
      for(b = 0; b <= nbins[y_dimen]; b++){
	if(bin_ends[b] < bound_ymin ||
	   bin_ends[b] > bound_ymax) continue;
	v1 = v2 = y2v(bin_ends[b]);
	uvline(u1, v1, u2, v2);
      }
      gflush();
    }
}
