/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/***--------------------------------------------------------------------------
* NAME:		parzen_plot_bound.c
* PURPOSE:	Subroutines dependent on Parzen Window Classifier for
*			Decision Region Plot
* USAGE:	parzen_plot_bound --
*			- displays flags and default settings for plot
*
* SEE ALSO: 	parzen_pack.c plot_bound.c
*
* DESCRIPTION:  Reads in a trained model file and plots the decision
*               region and/or scatter plot in 2 dimensions
*
*	read_model	read model file for classifier parameters
*	classify	find class of an input vector
*	get_outputs	unused, needed by plot_bound.c
*	over_plot	overlay centers on region plot
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/plot/parzen_plot_bound.c,v 11.4 2001/08/23 20:18:08 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/plot/parzen_plot_bound.c,v $
* CREATED: 14-Dec-94 lck			MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/



/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <math.h>
#include <nnlib.h>
#define parzen_pack_h_internal_types
#include <parzen_pack.h>
#include "plot_bound.h"
#include "xlippic.h"

#define SQUARE (0)

static int model;			/* id of model */
static float spread = 1.;
static int linear = 0;
static ARG argtab[] =
{
  {"spread", FLOAT, (int *)&spread,
     "factor for scaling kernal functions for testing"},
  {"linear", BOOLEAN, &linear,
     "Use Step Functions for kernals. Default: Gaussian kernals"},
};
#define TABSIZE (2)
float *outputs;
int noutputs;

int
get_extra_plot_args(int argc, char **argv)
{
    int nargc;
    nargc = nnargs_part(argc, argv, argtab, TABSIZE);
    return nargc;
}

void
read_model (FILE *param_file, int *total_trials, int *ninputs, int *pnoutputs, int **features, int adjust_priors, float *priors_list)
{
    int rv, nraw;

    rv = feature_read_paramfile(param_file, features, &nraw, ninputs);
    if (rv < 0)
      fatal_error("read_model -- read error %d %s", rv,
		  "on feature selection parameters in model file");
    model = parzen_read_model(param_file);
    if(model < 0) fatal_error ("read_model -- read error %d on model file",
	model);

    parzen_test_init (model, adjust_priors, priors_list, spread, linear);

    *total_trials = *(int *)parzen_param(model, "total_trials");
    *ninputs = *(int *)parzen_param (model, "ninputs");
    noutputs = *pnoutputs = *(int *)parzen_param (model, "nclasses");
    outputs = *(float **) parzen_param(model, "outputs");
}

int
classify(register float *inputs)
{
    int class, i;
    class = parzen_classify (model, inputs);
    if(!linear)
      for(i = 0; i < noutputs; i++){
	  outputs[i] = nn_exp10(outputs[i]);
      }

    return class;
}

float *
get_outputs(void)
{
    return (*(float **) parzen_param(model, "outputs"));
}

void
over_plot (int two_d, float internals_scale, int internals_level, char **plot_labels, int x_dimen, int y_dimen, float bound_xmax, float bound_xmin, float bound_ymax, float bound_ymin, int ubot, int utop, int vbot, int vtop)
{
    int i, c;
    int  u1, v1, u2, v2;
    int ncenters;
    float **centers;
    int *classes;
    int per_input, per_class;
    INVVAR *inv_var;
    float xvar, yvar;
    float x_mid;

    x_mid = (bound_xmax + bound_xmin)/2.;

    if (two_d) {
        gcolor("foreground");
	
	ncenters = *(int *)parzen_param(model, "total_trials");
	centers = *(float ***)parzen_param(model, "means");
	classes = *(int **) parzen_param(model, "classes");
	per_input = *(int *)parzen_param(model, "per_input");
	per_class = *(int *)parzen_param(model, "per_class");
	inv_var = (INVVAR *)parzen_param(model, "inv_var");
	

	gcolor("foreground");
	for (i=0;i<ncenters;i++) {
	    u1 = x2u (centers[i][x_dimen]);
	    if(x_dimen == y_dimen) v1 = y2v(x_mid);
	    else v1 = y2v (centers[i][y_dimen]);
	    if (u1 >= ubot && u1 <= utop && v1 >= vbot && v1 <= vtop) {
		/* draw a box here */
		/* this box should be shaped like the kernal function */
		/* the inv_var has been converted to 2*stddev */
		c = classes[i];
		if(per_class){
		    if(per_input){
			xvar = inv_var->per_class_input[c][x_dimen];
			yvar = inv_var->per_class_input[c][y_dimen];
		    } else {
			xvar = inv_var->per_class[c];
			yvar = inv_var->per_class[c];
		    }
		} else {
		    if(per_input){
			xvar = inv_var->per_input[x_dimen];
			yvar = inv_var->per_input[y_dimen];
		    } else {
			xvar = inv_var->grand;
			yvar = inv_var->grand;
		    }
		}
		if(linear){
		    u1 = x2u(centers[i][x_dimen] - xvar);
		    u2 = x2u(centers[i][x_dimen] + xvar);
		    if(x_dimen == y_dimen){
			v1 = y2v(x_mid - yvar);
			v2 = y2v(x_mid + yvar);
		    } else {
			v1 = y2v(centers[i][y_dimen] - yvar);
			v2 = y2v(centers[i][y_dimen] + yvar);
		    }
		    uvrect(u1,v1,u2,v2);
		} else {
		    draw_ellipse(internals_scale, u1, v1,
				 xlength2u(sqrt(1./xvar)),
				 ylength2v(sqrt(1./yvar)));
		}			
    		if (internals_level > 2) {
		    if(plot_labels != NULL &&
		       atoi(plot_labels[0]) > classes[i])
		      uvprintf (u1, v1, 2, "%s", plot_labels[classes[i]+1]);
    		}
	    }
	}
    }
}
