/***---------------------------------------------------------------------------
* NAME:		plot_cost.c
* PURPOSE:	Plots rms error graph
* USAGE: 
*	plot_cost -<flags>
* 
* RETURN VALUE:  NONE
*
* EXAMPLE: plot_cost --		 * shows all available flags
*
* GLOBALS: <flags> defined in plot_cost_args.h
*
* DESCRIPTION:  
*		
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/plot/plot_cost.c,v 11.4 2001/08/23 20:18:08 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/plot/plot_cost.c,v $
* CREATED: 20-Feb-91 lck                             MIT LINCOLN LABORATORY
*---------------------------------------------------------------------------
* Copyright 1991 MIT Lincoln Laboratory.
* This software is not for release and should neither be copied
* nor distributed without explicit permission of MIT Lincoln Laboratory.
--------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header:";



/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <nnlib.h>
#include <get_subs.h>
/* Needed for plot_cost_args.h */
#define PLOT_COST
#include "plot_cost_args.h"
#include "xlippic.h"

int cost_get_err_line (FILE *fptr, char *filename, ERROR_LINE *err_line_p,
		       int error_file_verbosity);

#define LINE_LEN 512

int
main(int argc, char **argv)
{
    float *x, *y;
    int symbol_mode = 0, symbol_type = 0; /* no symbols */
    char *symbol_string = NULL;
    int symbol_radius = 0;
    int cost_line_type = 1; /* 1 - solid, 2 - dotted */
    int grid_type = 1; /* full grid */
    int  auto_abscissas = 0; /* x provided by x array */
    float auto_xmin = 0.0, auto_xstep = 1.0; /* ignored */
    float height_fract = 0.9, width_fract = 1.0;
    float xoffset_fract = 0.1, yoffset_fract = 0.1;
    int xlog = 0, ylog = 0; /* linear x and y axes */
    char *xlabel = "Trials", *ylabel = "COST  ";
    char title[256];
    char *color = "black";

    char *command =
	"-geometry 600x440 -title cost_plot ";
    int quit_flag = 0;

    float cost_sum;
    int num_of_pts = 0,pts_allocated = 256,new_length;
    FILE *err_file, *mac_costlines_file;
    int line_cnt;
    int inf_flag = 0;
    ERROR_LINE err_line;
    int error_file_verbosity = 1;
    float axis_xmin,axis_xmax,axis_xstep;
    float axis_ymin,axis_ymax,axis_ystep;
    int ubot,vbot,utop,vtop;
    int total_trials;

    init_nnsubs(argc,argv,rcsid);

    /* read args */
    argc = nnargs(argc,argv,cost_argtab,COST_TABSIZE);

    /* set the data type, only one active at a time, if none, then class */
    if(class_data){
	map_data = seq_data = 0;
    } else {
	if(map_data){
	    seq_data = 0;
	} else if(!seq_data) class_data = 1;
    }

    /* open data file */
    err_file = nn_fopen(path_build(plot_exp_path,plot_error_fname),
			"r", "error file");

    if(mac_cost_flag)
      mac_costlines_file = nn_fopen(mac_costlines_fname, "w",
			       "for writing Macintosh cost plots");

    ylabel = cost_string(cost_func);
    strncpy(title, ylabel, 256);
    strncat(title, " Plot", 256);
    /* allocate space for x and y arrays */
    pts_allocated = 256;
    x = allocate_vector(pts_allocated);
    y = allocate_vector(pts_allocated);

    err_line.inputs = NULL;
    err_line.ninputs = 0;
    err_line.outputs = NULL;
    err_line.noutputs = 0;

    cost_sum = 0.0;
    total_trials = line_cnt = num_of_pts = 0;

    /* read lines, filling x and y arrays */
    while(cost_get_err_line(err_file, plot_error_fname,
			    &err_line, error_file_verbosity) != EOF){
	/* do averageing of points, to get read % errors */
	cost_sum += err_line.rmserr;
	total_trials++;
	if (++line_cnt >= plot_trial_average){
	    x[num_of_pts] = (float)total_trials;
	    y[num_of_pts] = calc_avg_cost(cost_sum, plot_trial_average,
					  cost_func);
	    line_cnt = 0;
	    cost_sum = 0.0;
	    /* check for infinities in cost, possible with gmix */
	    if(y[num_of_pts] > 1e37 || y[num_of_pts] < -1e37) inf_flag = 1;

	    if(mac_cost_flag) fprintf(mac_costlines_file, "%.3f %.3f\n",
				 x[num_of_pts], y[num_of_pts]);

	    /* expand arrays when they get full */
	    if(++num_of_pts == pts_allocated){
		new_length = pts_allocated + 256;
		x = reallocate_vector(x,pts_allocated, new_length);
		y = reallocate_vector(y,pts_allocated, new_length);
		pts_allocated = new_length;
	    }
	}
    }
    if(mac_cost_flag)
      nn_fclose(mac_costlines_file, mac_costlines_fname,
		"for writing Macintosh cost plot lines");

    /* if we got infinities, quit. We can't display them */
    if(inf_flag){
	fatal_error("Infinite Cost, cannot generate plot");
    }

    /* plot x's and y's */
    if(no_graphics){
	ginit("no graphics");
    } else {
	ginit(command);
    }
    ghardcopy(1);

    gwtitle(title);

    /* get size of plot */
    /* no symbols, no lines */
    xygraph_scale(x,y,num_of_pts,
	    2, symbol_string, symbol_type, symbol_radius,
	    0, grid_type, "",
	    auto_abscissas, auto_xmin, auto_xstep,
	    height_fract, width_fract, xoffset_fract, yoffset_fract,
	    xlog, cost_autoscale,
	    cost_xmin, cost_xmax, cost_xstep, "",
	    ylog, cost_autoscale,
	    cost_ymin, cost_ymax, cost_ystep, "","foreground",
	    &ubot,&vbot,&utop,&vtop,
	    &axis_xmin,&axis_xmax,&axis_xstep,
	    &axis_ymin,&axis_ymax,&axis_ystep);

    /* display label */
    uvprintf((utop+ubot)/2, vbot-90, 2, cost_plot_label);

    /* do real plots using scale gotten from xygraph_scale() */

    xygraph(x, y, num_of_pts, 
	symbol_mode,symbol_string,symbol_type,symbol_radius,
	cost_line_type,grid_type,path_build(plot_exp_path,plot_error_fname),
	auto_abscissas, auto_xmin, auto_xstep,
	height_fract, width_fract, xoffset_fract, yoffset_fract,
	xlog, 0, axis_xmin, axis_xmax, axis_xstep, xlabel,
	ylog, 0, axis_ymin, axis_ymax, axis_ystep, ylabel,
	color);

    gsave(path_build(plot_exp_path,cost_plot_fname),1);
    gexit(quit_flag);

    nn_fclose(err_file, path_build(plot_exp_path, plot_error_fname),
	      "error file");
    exit_nnsubs();
    exit(0);
}

int
cost_get_err_line(FILE *fptr, char *filename, ERROR_LINE *err_line_p,
		  int error_file_verbosity)
{
    int rv;
    if(class_data)
      rv = get_err_line(fptr, filename, err_line_p, error_file_verbosity);
    else if(map_data)
      rv = get_map_err_line(fptr, filename, err_line_p, error_file_verbosity);
    else /* seq_data */
      rv = get_seq_err_line(fptr, filename, err_line_p, error_file_verbosity);
    return rv;
}
