/***---------------------------------------------------------------------------
* NAME:         plot_norm.c
* PURPOSE:      Plots normalization graph
* USAGE:
*       plot_norm -<flags>
*
* RETURN VALUE:  NONE
*
* EXAMPLE: plot_norm --          * shows all available flags
*
* GLOBALS: <flags> defined in plot_norm_args.h
*
* DESCRIPTION:
*
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/plot/plot_norm.c,v 11.4 2001/08/23 20:18:09 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/plot/plot_norm.c,v $
* CREATED: 20-Feb-91 lck                             MIT LINCOLN LABORATORY
*---------------------------------------------------------------------------
* Copyright 1991 MIT Lincoln Laboratory.
* This software is not for release and should neither be copied
* nor distributed without explicit permission of MIT Lincoln Laboratory.
--------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header:";

/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <math.h>
#include <nnlib.h>
#include <get_subs.h>
#include <xlippic.h>
/* Needed for plot_norm_args.h */
#define PLOT_NORM
#include "plot_norm_args.h"
#include "xlippic.h"

#define FALSE 0
#define TRUE 1
#define SQUARE 0
#define SOLID_BLOCK 6
#define CIRCLE 7
#define FILLED_CIRCLE 8

#define SIMPLE 1        /* simple normalization (mean=0, variance=1) */
#define PCA 2           /* Principle Components Analysis */
#define LDA 3           /* Linear Discriminant Analysis */
int debug = 0;  /* needed for some subroutines */

float           axis_xmin, axis_xmax, axis_xstep;
float           axis_ymin, axis_ymax, axis_ystep;

int
main(int argc, char **argv)
{
    int norm_line_type = 1; /* 1 - solid */
    int             symbol_mode = 2, symbol_type = 0;
    char           *symbol_string = NULL;
    int             symbol_radius = 5;
    int             grid_type = 1;  /* full grid */
    int             auto_abscissas = 1; /* does auto abscissas */
    float           auto_xmin = 1.0, auto_xstep = 1.0;
    float           height_fract = 0.8, width_fract = 1.2;
    float           xoffset_fract = 0.1, yoffset_fract = 0.1;
    int             xlog = 0, ylog = 0; /* linear x and y axes */
    char           *xlabel = "Feature number";
    char       *ylabel = "EV %%  ";
    char           *command = "-geometry 600x500";
    int             quit_flag = 0;
    FILE           *maclines_file;
    int             plot_index, i;
    float   *graph_x;
    float   *eigenvalues;
    float   *eigenvalue_percent;
    int     normalization;
    float   total, tmp, max_value=0;
    char    *plot_title;

    init_nnsubs(argc, argv, rcsid);
    /* Process the command line arguments */
    argc = nnargs(argc, argv, norm_argtab, NORM_TABSIZE);

    normalization =
      read_norm_file(path_build(norm_data_path,plot_error_fname));

    if(mac_norm_flag)
      maclines_file = nn_fopen(mac_normlines_fname, "w",
                   "for writing macintosh normalization plots");
    
    if (normalization == PCA || normalization == LDA) {
        plot_index = plot_n_inputs;
        if (normalization == PCA) {
            plot_title = "Principle Components Eigenvalue Plot";
        }
        if (normalization == LDA) {
            if (plot_index > plot_n_outputs - 1) plot_index = plot_n_outputs-1;
            plot_title = "Linear Discriminant Eigenvalue Plot";
        }
        graph_x = allocate_vector(plot_index);
        eigenvalue_percent = allocate_vector(plot_index);
        eigenvalues = *(float **) norm_param("eigenvalues");

        total = 0;
        for(i = 0; i < plot_index; i++){
        total += eigenvalues[i];
        }
        for(i = 0; i < plot_index; i++){
        tmp = eigenvalues[i]/total * 100;
        eigenvalue_percent[i] = tmp;
        graph_x[i] = i;
        if (tmp > max_value) max_value = tmp;
        if(mac_norm_flag) fprintf(maclines_file, "%.3f\n",
                 eigenvalue_percent[i]);
        }
        if (norm_auto_scale_flag) {
        norm_xmin = 1;
            norm_ymin = 0;
            norm_xmax = plot_index;
            norm_ymax = ceil(max_value);
            norm_xstep = ceil(plot_index/10.);
            norm_ystep = ceil(norm_ymax/10.);
        }
    if(mac_norm_flag)
      nn_fclose(maclines_file, mac_normlines_fname,
            "for writing Macintosh normalization lines");

        if (debug>0) printf("plot_norm: %d %d %s\n",
            plot_index, norm_line_type, norm_plot_label);

    if(no_graphics){
        ginit("no graphics");
    } else {
        ginit(command);
    }
        ghardcopy(1);
        gwtitle(plot_title);

                        /* plot eigenvalues and grid */
        xygraph(graph_x, eigenvalue_percent, plot_index,
            symbol_mode, symbol_string, symbol_type, symbol_radius,
            norm_line_type+1, grid_type, norm_plot_label,
            auto_abscissas, auto_xmin, auto_xstep,
            height_fract, width_fract, xoffset_fract, yoffset_fract,
            xlog, 0, norm_xmin, norm_xmax, norm_xstep, xlabel,
            ylog, norm_auto_scale_flag, norm_ymin, norm_ymax, norm_ystep, ylabel, "");

        gsave(path_build(plot_exp_path, norm_plot_fname), 0);
        gexit(quit_flag);

        deallocate_vector(graph_x);
        deallocate_vector(eigenvalue_percent);
    } else {
        warning("plot_norm called with normalization %d not = %d or %d\n",
            normalization, PCA, LDA);
    }
    exit_nnsubs();
    exit(0);
}
