/***---------------------------------------------------------------------------
* NAME:         plot_select.c
* PURPOSE:      Plots percent error graph
* USAGE:
*       plot_select -<flags>
*
* RETURN VALUE:  NONE
*
* EXAMPLE: plot_select --          * shows all available flags
*
* GLOBALS: <flags> defined in plot_select_args.h
*
* DESCRIPTION:
*
* BUGS AND LIMITATIONS:
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/plot/plot_select.c,v 11.5 2001/08/23 20:18:10 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/plot/plot_select.c,v $
* CREATED: 20-Feb-91 lck                             MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/
/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */

static char rcsid[] = "$Header: /data/speech/kukolich/cvsroot/lnknet/src/plot/plot_select.c,v 11.5 2001/08/23 20:18:10 craig Exp $";



/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <nnlib.h>
#include <get_subs.h>
#include <xlippic.h>
/* Needed for plot_select_args.h */
#define PLOT_SELECT
#include "plot_select_args.h"
#include "xlippic.h"

#define FALSE 0
#define TRUE 1
#define SQUARE 0
#define SOLID_BLOCK 6
#define CIRCLE 7
#define FILLED_CIRCLE 8
int             debug = 0;	/* needed for some subroutines */


/* needed for x2u and y2v */
static int      ubot, vbot, utop, vtop, uwidth, vwidth;
static float    xwidth, ywidth;
float           axis_xmin, axis_xmax, axis_xstep;
float           axis_ymin, axis_ymax, axis_ystep;

int x2u (float raw_x);
int y2v (float raw_y);

int
main(int argc, char **argv)
{
    int             symbol_mode = 2, symbol_type = 0;
    char           *symbol_string = NULL;
    int             symbol_radius = 5;
    int	select_line_type = 1;/* solid */
    int             grid_type = 1;	/* full grid */
    int             auto_abscissas = 1;	/* does auto abscissas */
    float           auto_xmin = 1.0, auto_xstep = 1.0;
    float           height_fract = 0.8, width_fract = 1.2;
    float           xoffset_fract = 0.1, yoffset_fract = 0.2;
    int             xlog = 0, ylog = 0;	/* linear x and y axes */
    char           *xlabel = "Number of Features";
    char	   *ylabel = "%% Err   ";
    int             quit_flag = 0;
    char           *plot_param_fullname;
    FILE           *param_file, *maclines_file;
    int             ninputs, best, i;
    char            print_filename[128];
    float  *graph_x;
    int *selected;
    float *errors;
    int sign;
    int legend_v;

    init_nnsubs(argc, argv, rcsid);
    /* Process the command line arguments */
    argc = nnargs(argc, argv, select_argtab, SELECT_TABSIZE);

    plot_param_fullname = path_build(plot_exp_path, plot_error_fname);
    /* Open the file, check for errors */
    param_file = nn_fopen(plot_param_fullname, "r",
			  "for read of weight structure information");

    feat_sel_read_model(param_file, &ninputs, &best, &selected, &errors);

    nn_fclose(param_file, plot_param_fullname,
	      "for read of weight structure information");

    if(mac_search_flag)
      maclines_file = nn_fopen(mac_searchlines_fname, "w",
			       "for writing Macintosh feature search lines");

    graph_x = allocate_vector(ninputs);

    for(i = 0; i < ninputs; i++){
	graph_x[i] = (float)selected[i];
	if(mac_search_flag)
	  fprintf(maclines_file, "%d %.3f\n", selected[i], errors[i]);
    }
    if(mac_search_flag)
      nn_fclose(maclines_file, mac_searchlines_fname,
		"for writing Mac feature search lines");

    if (no_graphics)
      ginit("no graphics");
    else
      ginit("-geometry 610x500");
    ghardcopy(1);
    gwtitle("Feature Selection Plot");

    strcpy(print_filename, path_build(plot_exp_path, select_plot_fname));

    /* get size of plot */
    /* no symbols, no lines */
    xygraph_scale(graph_x, errors, ninputs,
		  2, symbol_string, symbol_type, symbol_radius,
		  0, grid_type, select_plot_label,
		  auto_abscissas, auto_xmin, auto_xstep,
		  height_fract, width_fract, xoffset_fract, yoffset_fract,
		  xlog, select_auto_scale_flag,
		  select_xmin, select_xmax, select_xstep, "",
		  ylog, select_auto_scale_flag,
		  select_ymin, select_ymax, select_ystep, "", "foreground",
		  &ubot, &vbot, &utop, &vtop,
		  &axis_xmin, &axis_xmax, &axis_xstep,
		  &axis_ymin, &axis_ymax, &axis_ystep);

    /* set up for x/y-u/v conversion function */
    uwidth = utop - ubot;
    vwidth = vtop - vbot;
    xwidth = axis_xmax - axis_xmin;
    ywidth = axis_ymax - axis_ymin;

    /* plot legend */
    legend_v = vbot/2;
    gcolor("foreground");
    sign = vbot/16;
    for (i = 0; i < ninputs; i++) {
	sign *= -1;
	uvprintf(x2u((float)i+1), legend_v + sign, 2, "%d",selected[i]);
    }
    uvframe(1,x2u(axis_xmin),legend_v + 5*vbot/32,
		x2u(axis_xmax) + 30, legend_v - vbot/8);
    uvprintf(512, legend_v - 7 * vbot/32, 2, "Features");

    /* plot % error and grid */
      xygraph(graph_x, errors, ninputs,
      symbol_mode, symbol_string, symbol_type, symbol_radius,
      select_line_type, grid_type, select_plot_label,
      auto_abscissas, auto_xmin, auto_xstep,
      height_fract, width_fract, xoffset_fract, yoffset_fract,
      xlog, 0, axis_xmin, axis_xmax, axis_xstep, xlabel,
      ylog, 0, axis_ymin, axis_ymax, axis_ystep, ylabel,
      "");

    gsave(print_filename, 0);
    gexit(quit_flag);

    exit_nnsubs();
    exit(0);
}
int x2u (float raw_x)
{
    return (ubot + (raw_x - axis_xmin) * uwidth / xwidth);
}

int y2v (float raw_y)
{
    return (vbot + (raw_y - axis_ymin) * vwidth / ywidth);
}

