/* (c) Copyright 1999 M.I.T. Lincoln Laboratory


Permission is hereby granted, without written agreement or royalty fee,
to use, copy, modify, and distribute this software and its
documentation for any purpose, provided that the above copyright notic
and the following three paragraphs appear in all copies of this software.

IN NO EVENT SHALL M.I.T. LINCOLN LABORATORY BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF M.I.T. LINCOLN
LABORATORY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. M.I.T. LINCOLN
LABORATORY SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT.

THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND M.I.T. LINCOLN
LABORATORY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. */
/***---------------------------------------------------------------------------
* NAME:		rbf_plot_bound.c
* PURPOSE:	Radial Basis Function Classifier main (driver) program
* USAGE:
*
* SEE ALSO: 	plot_bound.c rbf_pack.c
*
* DESCRIPTION:  Reads in a trained model file and plots the decision
*               region and/or scatter plot in 2 selected dimensions
*
*	read_model	Read model file describing classifier parameters
*	classify	Find class of an input vector using classifier
*	get_outputs	Get address of output vector
*	over_plot	Optional way to add classifier dependent plots
*
* $Header: /data/speech/kukolich/cvsroot/lnknet/src/plot/rbf_plot_bound.c,v 11.4 2001/08/23 20:18:10 craig Exp $
* $Source: /data/speech/kukolich/cvsroot/lnknet/src/plot/rbf_plot_bound.c,v $
* CREATED: 4-MAR-91 dan                             MIT LINCOLN LABORATORY
--------------------------------------------------------------------------***/


/* config.h must go first */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <math.h>
#include <nnlib.h>
#include <rbf_pack.h>
#include "plot_bound.h"
#include "xlippic.h"

float *outputs;
int noutputs;
static float extra_hspread=1.0;
static ARG argtab[] =
{
    {"exhspread", FLOAT, (int *)&extra_hspread, "test factor for all spreads"},
};
#define TABSIZE (1)

static int model;			/* id of model */

int get_extra_plot_args(int argc, char **argv)
{
    int nargc;
    nargc = nnargs_part(argc, argv, argtab, TABSIZE);
    return nargc;
}

void
read_model (FILE *param_file, int *total_trials, int *ninputs,
	    int *pnoutputs, int **features, int adjust_priors,
	    float *priors_list)
{
    model = rbf_read_model(param_file);

    *ninputs = *(int *)rbf_param(model,"ninputs");
    *pnoutputs = *(int *)rbf_param(model,"nclasses");
    *features = *(int **)rbf_param( model, "features");
    outputs = *(float **)rbf_param(model, "outputs");
    noutputs = *pnoutputs;

    if (model < 0) fatal_error ("read_model -- read error %d on model file",
	model);

    rbf_test_init(model, adjust_priors, priors_list, extra_hspread);
}


int 
classify (float *inputs)
{
    return rbf_classify(model,inputs);
}


				/* used only for profile/histogram plots */
float *
get_outputs (void)
{
    outputs = *(float **)rbf_param(model, "outputs");
    return outputs;
}

void
over_plot (int two_d, float internals_scale, int internals_level,
	   char **plot_labels, int x_dimen, int y_dimen, float bound_xmax,
	   float bound_xmin, float bound_ymax, float bound_ymin,
	   int ubot, int utop, int vbot, int vtop)
{
    float **centroids, **spreads;
    int i;
    int u1, v1;
    int ngauss_hidden;
    float hspread;
    float x_mid;

    x_mid = (bound_xmax + bound_xmin)/2.;
    
    if (two_d) {
	gcolor("foreground");
	ngauss_hidden = *(int *) rbf_param (model,"ngauss_hidden");
	hspread = *(float *) rbf_param (model,"hspread");
	centroids = *(float ***) rbf_param (model,"centroids");
	spreads = *(float ***) rbf_param (model,"spreads");
	for (i=0; i<ngauss_hidden; i++) {
	    u1 = x2u (centroids[i][x_dimen]);
	    if(x_dimen == y_dimen) v1 = y2v(x_mid);
	    else v1 = y2v (centroids[i][y_dimen]);
	    if (u1 >= ubot && u1 <= utop && v1 >= vbot && v1 <= vtop) {
	        draw_ellipse (internals_scale, u1, v1, 
	            xlength2u (sqrt (hspread * spreads[i][x_dimen])),
	            ylength2v (sqrt (hspread * spreads[i][y_dimen])));
    		if (internals_level > 2) {
		    uvprintf (u1, v1, 2, "%d", i + 1);
    		}
	    }
	}
    }
}

