#!/usr/local/bin/perl -w
# LNKnet uses GNU's automake to generate Makefiles from simpler template files
# The good news is that these files can be easily configured for a particular
# machine using a script generated by another GNU tool, autoconf. The bad news
# is that these Makefiles assume that you have two more GNU tools, gmake and
# gcc.
# The purpose of this script is to remove lines in the Makefile.in files that
# most make programs have fits over.

use strict;

my($quit) = 0;
dodir('.');

sub dodir {
  my($dir) = @_;
  my(@filenames);
  my($name, $inRule);

  opendir (DIR, $dir) || die "Can't open directory $dir";
  @filenames = readdir(DIR);
  closedir(DIR);
  
  for (@filenames) {
    if($quit){ return; }
    next if $_ eq '.';
    next if $_ eq '..';
    $name = "$dir/$_";
    if (-d $name) {
      dodir("$name");
    } else {
      if(/Makefile.in/){
	print "file: $name\n";
	open(FILE, $name);
	open(NEW, "> $name.new");
	$inRule = 0;
	while(<FILE>){
	  if (/%\.o: %\.c/){
	    $inRule = 1;
	    print NEW ".c.o:\n";
	    print NEW "\t\$(COMPILE) -c \$<\n";
	    print NEW "\n";
	  }
	  if (/%\.lo: %\.c/){
	    $inRule = 1;
	    print NEW ".c.lo:\n";
	    print NEW "\t\$(LTCOMPILE) -c \$<\n";
	    print NEW "\n";
	  }
	  if(/^DEPS_MAGIC/){
	    $_ = <FILE>;
	    $_ = <FILE>;
	    next;
	  }
	  if(/^\s$/){
	    $inRule = 0;
	  }
	  if(/^info-am/){
	    $inRule = 0;
	  }
	  if(! $inRule){
	    print NEW;
	  }
	}
	close(NEW);
	close(FILE);
	rename "$name.new", $name;
      }
    }
  }
}
