/************************************************************
 *                                                          *
 *  Permission is hereby granted  to  any  individual   or  *
 *  institution   for  use,  copying, or redistribution of  *
 *  the xgobi code and associated documentation,  provided  *
 *  that   such  code  and documentation are not sold  for  *
 *  profit and the  following copyright notice is retained  *
 *  in the code and documentation:                          *
 *     Copyright (c) 1990,1991,1992,1993,1994 Bellcore      *
 *                                                          *
 *  We welcome your questions and comments, and request     *
 *  that you share any modifications with us.               *
 *                                                          *
 *    Deborah F. Swayne            Dianne Cook              *
 *     dfs@bellcore.com         dicook@iastate.edu          *
 *      (201) 829-4263                                      *
 *                                                          *
 ************************************************************/

#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include "xincludes.h"
#include "xgobitypes.h"
#include "xgobivars.h"
#include <X11/Xatom.h>
#include <X11/Xproto.h>

#include "../bitmaps/leftarrow.xbm"
#include "../bitmaps/rightarrow.xbm"
#include "../bitmaps/uparrow.xbm"
#include "../bitmaps/downarrow.xbm"
#include "../bitmaps/plus.xbm"
#include "../bitmaps/minus.xbm"

/* Functions used in this file */
XtEventHandler expose_cback();
XtEventHandler resize_cback();
XtCallbackProc dotplot_on_cback();
XtCallbackProc xyplot_on_cback();
XtCallbackProc spin_on_cback();
XtCallbackProc grand_tour_on_cback();
XtCallbackProc corr_tour_on_cback();
XtCallbackProc scaling_cback();
XtCallbackProc brush_cback();
XtCallbackProc identify_cback();
XtCallbackProc magnify_cback();
XtCallbackProc add_axes_cback();
XtCallbackProc place_ndata_popup();
XtCallbackProc place_savedata_popup();
XtCallbackProc place_savebinary_popup();
XtCallbackProc link_brush_cback();
XtCallbackProc link_line_brush_cback();
XtCallbackProc link_ident_cback();
XtCallbackProc jump_brush_cback();
XtCallbackProc reshape_brush_cback();
XtCallbackProc carry_vars_cback();
XtCallbackProc print_panel_cback();
XtCallbackProc store_resources_cback();
XtCallbackProc help_cback();
XtCallbackProc exit_panel_cback();
XtCallbackProc exit_solo_cback();
XtCallbackProc showlines_cback(), showpoints_cback();
XtCallbackProc line_editor_cback();
void add_pb_help();
void quickplot_once();
void plot_once();
void set_wm_protocols();
/* */

static Widget form1, main_panel, sub_panel[2];
static Widget main_cmda[9], main_cmdb[5] ;
static Widget options_popup, oframe, options_mgr, options_var[11];
static Widget io_popup, aframe, io_mgr, io_var[9];

#define DOTPLOT    main_cmda[0]
#define XYPLOT     main_cmda[1]
#define ROTATE     main_cmda[2]
#define TOUR       main_cmda[3]
#define CORR       main_cmda[4]
#define SCALE      main_cmda[5]
#define BRUSH      main_cmda[6]
#define IDENTIFY   main_cmda[7]
#define EDIT_LINES main_cmda[8]
#define OPTIONS    main_cmdb[0]
#define IO         main_cmdb[1]
#define HELP       main_cmdb[2]
#define EXIT       main_cmdb[3]

#define ADDAXES        options_var[0]
#define SHOWPOINTS     options_var[1]
#define SHOWLINES      options_var[2]
#define LINKBRUSH      options_var[3]
#define LINKLINEBRUSH  options_var[4]
#define LINKIDENT      options_var[5]
#define JUMPBRUSH      options_var[6]
#define RESHAPEBRUSH   options_var[7]
#define CARRYVARS      options_var[8]
#define CLOSE_OPTIONS  options_var[8]

#define NEWDATA        io_var[0]
#define WRITEASCII     io_var[1]
#define WRITEBINARY    io_var[2]
#define PRINTPLOTWIN   io_var[3]
#define STORERESOURCES io_var[4]
#define CLOSE_IO       io_var[5]

void
init_options(xg)
  xgobidata *xg;
{
  xg->linked_brushing =      (Boolean) appdata.linkBrush;
  xg->linked_line_brushing = (Boolean) appdata.linkLineBrush;
  xg->linked_identify =      (Boolean) appdata.linkIdentify;
  xg->is_axes =              (Boolean) appdata.showAxes;
  xg->plot_the_points =      (Boolean) appdata.showPoints;
  xg->connect_the_points =   (Boolean) appdata.showLines;
  xg->carry_vars =           (Boolean) appdata.carryVars;
  xg->jump_brush =           (Boolean) appdata.jumpBrush;
  xg->reshape_brush =        (Boolean) appdata.reshapeBrush;
}

void
set_mono(w)
  Widget w;
{
  XtVaSetValues(w,
    XtNforeground, appdata.fg,
    XtNbackground, appdata.bg,
    XtNborderColor, appdata.fg,
    NULL);
}

Widget
CreateCommand(xg, label, sensitive, href, vref, parent, helpLabel)
  xgobidata *xg;
  char *label, *helpLabel;
  Boolean sensitive;
  Widget href, vref;
  Widget parent;
{
  Widget pb;

  if (mono)
    pb = XtVaCreateWidget("Command",
      commandWidgetClass, parent,
      XtNlabel, (String) label,
      XtNsensitive, (Boolean) sensitive,
      XtNfromHoriz, (Widget) href,
      XtNfromVert, (Widget) vref,
      XtNforeground, appdata.fg,
      XtNbackground, appdata.bg,
      XtNborderColor, appdata.fg,
      NULL);
  else
    pb = XtVaCreateWidget("Command",
      commandWidgetClass, parent,
      XtNlabel, (String) label,
      XtNsensitive, (Boolean) sensitive,
      XtNfromHoriz, (Widget) href,
      XtNfromVert, (Widget) vref,
      NULL);

  add_pb_help(&xg->nhelpids.pb, pb, helpLabel);
  return(pb);
}

Widget
CreateToggle(xg, label, sensitive,
href, vref, radioref,
set, type, parent, helpLabel)
  xgobidata *xg;
  char *label, *helpLabel;
  Boolean sensitive, set;
  Widget href, vref, radioref;
  int type;
  Widget parent;
{
  Widget pb;
  char name[8];

  /*
   * Using the fallback resources, we force some
   * toggle widgets to follow "one of many" behavior
   * while others are permitted to be unselected.
  */
  if (type == ONE_OF_MANY)
    strcpy(name, "Toggle");
  else
    strcpy(name, "Command");

  if (mono)
    pb = XtVaCreateWidget(name,
      toggleWidgetClass, parent,
      XtNlabel, (String) label,
      XtNsensitive, (Boolean) sensitive,
      XtNfromHoriz, (Widget) href,
      XtNfromVert, (Widget) vref,
      XtNradioGroup, (Widget) radioref,
      XtNstate, (Boolean) set,
      XtNforeground, appdata.fg,
      XtNbackground, appdata.bg,
      XtNborderColor, appdata.fg,
      NULL);
  else
    pb = XtVaCreateWidget(name,
      toggleWidgetClass, parent,
      XtNlabel, (String) label,
      XtNsensitive, (Boolean) sensitive,
      XtNfromHoriz, (Widget) href,
      XtNfromVert, (Widget) vref,
      XtNradioGroup, (Widget) radioref,
      XtNstate, (Boolean) set,
      NULL);

  add_pb_help(&xg->nhelpids.pb, pb, helpLabel);
  return(pb);
}

Boolean
current_mode(w)
  Widget w;
{
  Boolean state;
/*
 * Get the value of state for w.
*/
  XtVaGetValues(w,
    XtNstate, &state,
    NULL);

  return(!state);
}

Boolean
tour_on(xg)
  xgobidata *xg;
{
  Boolean on;

  XtVaGetValues(TOUR, XtNstate, &on, NULL);
  return(on);
}

void
set_showlines_option(lgl)
  Boolean lgl;
{
/*
 * This is needed because we're going to duplicate
 * this button on the line editing panel, and they
 * both need to show the same state, but we don't
 * want to call the callback twice.
*/
  XtVaSetValues(SHOWLINES, XtNstate, lgl, NULL);
}

void
turn_on_showlines_option(xg)
  xgobidata *xg;
{
/*
 * This is called when line editing is entered.
 * It sets xg->connect_the_points to True if it's False,
 * and calls the callback, and then makes certain the
 * toggle in the options menu is set to True.
*/
  if (!xg->connect_the_points)
  {
    XtCallCallbacks(SHOWLINES, XtNcallback, (XtPointer) xg);
    set_showlines_option(True);
  }
}

void
make_plotwindow_mouse_labels(xg)
  xgobidata *xg;
{
  xg->spin_mouse = XtVaCreateManagedWidget("MouseLabel",
    labelWidgetClass, xg->box1,
    XtNlabel, (String) "L or M: Move points ",
    NULL);
  if (mono) set_mono(xg->spin_mouse);

  xg->scale_mouse = XtVaCreateManagedWidget("MouseLabel",
    labelWidgetClass, xg->box1,
    XtNlabel, (String) "L: Move pts, M: Shape pts ",
    NULL);
  if (mono) set_mono(xg->scale_mouse);

  xg->brush_mouse = XtVaCreateManagedWidget("MouseLabel",
    labelWidgetClass, xg->box1,
    XtNlabel, (String) "Drag L: paint, M: shape ",
    NULL);
  if (mono) set_mono(xg->brush_mouse);

  xg->identify_mouse = XtVaCreateManagedWidget("MouseLabel",
    labelWidgetClass, xg->box1,
    XtNlabel, (String) "L: Toggle sticky labels ",
    NULL);
  if (mono) set_mono(xg->identify_mouse);

  xg->le_add_mouse = XtVaCreateManagedWidget("MouseLabel",
    labelWidgetClass, xg->box1,
    XtNlabel, (String) "Drag L: Add line, M: Remove last ",
    NULL);
  if (mono) set_mono(xg->le_add_mouse);

  xg->le_delete_mouse = XtVaCreateManagedWidget("MouseLabel",
    labelWidgetClass, xg->box1,
    XtNlabel, (String) "L: Delete line, M: Put it back ",
    NULL);
  if (mono) set_mono(xg->le_delete_mouse);
}

void
make_plot_window(xg)
  xgobidata *xg;
/*
 * PlotWindow: the plot window
*/
{
  xg->workspace = XtVaCreateManagedWidget("PlotWindow",
    labelWidgetClass, xg->box1,
    XtNresizable, (Boolean) True,
    XtNleft, (XtEdgeType) XtChainLeft,
    XtNtop, (XtEdgeType) XtChainTop,
    XtNright, (XtEdgeType) XtRubber,
    XtNbottom, (XtEdgeType) XtRubber,
    XtNfromVert, (Widget) xg->scale_mouse,
    XtNlabel, (String) "",
    NULL);
  if (mono) set_mono(xg->workspace);
  add_pb_help(&xg->nhelpids.pb,
    xg->workspace, "PlotWindow");

  XtAddEventHandler(xg->workspace, ExposureMask,
    FALSE, (XtEventHandler) expose_cback, (XtPointer) xg);
  XtAddEventHandler(xg->workspace, StructureNotifyMask,
    FALSE, (XtEventHandler) resize_cback, (XtPointer) xg);
}

void
turn_on_xyplotting(xg)
  xgobidata *xg;
{
  Boolean state;
/*
 * If we're not xyplotting, execute the callback to turn it on.
*/
  XtVaGetValues(XYPLOT,
     XtNstate, &state,
     NULL);
  if (!state)
  {
    XtCallCallbacks(XYPLOT, XtNcallback, (XtPointer) xg);
    XtVaSetValues(XYPLOT,
      XtNstate, (Boolean) True,
      NULL);
  }
}

void
make_arrows(xg)
  xgobidata *xg;
{
  Drawable root_window = RootWindowOfScreen(XtScreen(xg->shell));

  /*
   * Define arrows.  (depth is a global variable)
  */
  leftarr = XCreatePixmapFromBitmapData(display, root_window,
    leftarrow_bits, leftarrow_width, leftarrow_height,
    appdata.fg, appdata.bg, depth);
  rightarr = XCreatePixmapFromBitmapData(display, root_window,
    rightarrow_bits, rightarrow_width, rightarrow_height,
    appdata.fg, appdata.bg, depth);
  uparr = XCreatePixmapFromBitmapData(display, root_window,
    uparrow_bits, uparrow_width, uparrow_height,
    appdata.fg, appdata.bg, depth);
  downarr = XCreatePixmapFromBitmapData(display, root_window,
    downarrow_bits, downarrow_width, downarrow_height,
    appdata.fg, appdata.bg, depth);
  plus = XCreatePixmapFromBitmapData(display, root_window,
    plus_bits, plus_width, plus_height,
    appdata.fg, appdata.bg, depth);
  minus =  XCreatePixmapFromBitmapData(display, root_window,
    minus_bits, minus_width, minus_height,
    appdata.fg, appdata.bg, depth);
}

void
reset_3d_cmds(xg)
  xgobidata *xg;
{
  Boolean sens = True;

  if (xg->ncols_used < 3)
    sens = False;

  XtVaSetValues(ROTATE, XtNsensitive, (Boolean) sens, NULL);
  XtVaSetValues(TOUR, XtNsensitive, (Boolean) sens, NULL);
  XtVaSetValues(CORR, XtNsensitive, (Boolean) sens, NULL);
}

/* ARGSUSED */
XtCallbackProc
PopdownIOMenu(w, client_data, callback_data)
/*
 * Pop down the io menu.
*/
  Widget w;
  caddr_t client_data;
  caddr_t callback_data;
{
  XtPopdown(io_popup);
}

/* ARGSUSED */
XtCallbackProc
PopupIOMenu(w, xg, callback_data)
/*
 * Pop up the io menu.
*/
  Widget w;
  xgobidata *xg;
  caddr_t callback_data;
{
  Dimension width, height;
  Position x, y;
  static int initd = 0;

  if (!initd)
  {
    XtVaGetValues(w,
      XtNwidth, &width,
      XtNheight, &height,
      NULL);
    XtTranslateCoords(w,
      (Position) (width/2), (Position) (height/2), &x, &y);

    XtVaSetValues(io_popup,
      XtNx, (Position) x,
      XtNy, (Position) y,
      NULL);
  }

  XtPopup(io_popup, XtGrabNone);
  XRaiseWindow(display, XtWindow(io_popup));

  if (!initd)
  {
    set_wm_protocols(io_popup);
    initd = 1;
  }
}

/* ARGSUSED */
XtCallbackProc
PopdownOptionsMenu(w, client_data, callback_data)
/*
 * Pop down the options menu.
*/
  Widget w;
  caddr_t client_data;
  caddr_t callback_data;
{
  XtPopdown(options_popup);
}

void
build_options_menu(xg)
  xgobidata *xg;
{
  options_popup = XtVaCreatePopupShell("Options",
    topLevelShellWidgetClass, OPTIONS,
    XtNinput, True,
    XtNtitle, "Options",
    NULL);
  if (mono) set_mono(options_popup);

  oframe = XtVaCreateManagedWidget("Form",
    panedWidgetClass, options_popup,
    XtNorientation, (XtOrientation) XtorientVertical,
    NULL);

  options_mgr = XtVaCreateManagedWidget("Panel",
    formWidgetClass, oframe,
    NULL);
  if (mono) set_mono(options_mgr);

  ADDAXES = (Widget) CreateToggle(xg, "Add axes",
    True, (Widget) NULL, (Widget) NULL, (Widget) NULL,
    appdata.showAxes, ANY_OF_MANY, options_mgr, "AddAxes");
  XtAddCallback(ADDAXES, XtNcallback,
    (XtCallbackProc) add_axes_cback, (XtPointer) xg);

  SHOWPOINTS = (Widget) CreateToggle(xg, "Show points",
    True, (Widget) NULL, (Widget) ADDAXES, (Widget) NULL,
    appdata.showPoints, ANY_OF_MANY, options_mgr, "ShowPoints");
  XtAddCallback(SHOWPOINTS, XtNcallback,
    (XtCallbackProc) showpoints_cback, (XtPointer) xg);

  SHOWLINES = (Widget) CreateToggle(xg, "Show lines",
    True, (Widget) NULL, (Widget) SHOWPOINTS, (Widget) NULL,
    appdata.showLines, ANY_OF_MANY, options_mgr, "ShowLines");
  XtAddCallback(SHOWLINES, XtNcallback,
    (XtCallbackProc) showlines_cback, (XtPointer) xg);

  LINKBRUSH = CreateToggle(xg, "Link brushing",
    True, (Widget) NULL, (Widget) SHOWLINES, (Widget) NULL,
    xg->linked_brushing, ANY_OF_MANY, options_mgr, "LinkBrush");
  XtVaSetValues(LINKBRUSH,
    XtNvertDistance, (int) 10,
    NULL);
  XtAddCallback(LINKBRUSH, XtNcallback,
    (XtCallbackProc) link_brush_cback, (XtPointer) xg);

  LINKLINEBRUSH = CreateToggle(xg, "Link line brushing",
    True, (Widget) NULL, (Widget) LINKBRUSH, (Widget) NULL,
    xg->linked_line_brushing, ANY_OF_MANY, options_mgr, "LinkLineBrush");
  XtAddCallback(LINKLINEBRUSH, XtNcallback,
    (XtCallbackProc) link_line_brush_cback, (XtPointer) xg);

  LINKIDENT = (Widget) CreateToggle(xg, "Link identification",
    True, (Widget) NULL, (Widget) LINKLINEBRUSH, (Widget) NULL,
    xg->linked_identify, ANY_OF_MANY, options_mgr, "LinkIdent");
  XtAddCallback(LINKIDENT, XtNcallback,
    (XtCallbackProc) link_ident_cback, (XtPointer) xg);

  JUMPBRUSH = (Widget) CreateToggle(xg, "Brush jumps to pointer",
    True, (Widget) NULL, (Widget) LINKIDENT, (Widget) NULL,
    appdata.jumpBrush, ANY_OF_MANY, options_mgr, "JumpBrush");
  XtVaSetValues(JUMPBRUSH,
    XtNvertDistance, (int) 10,
    NULL);
  XtAddCallback(JUMPBRUSH, XtNcallback,
    (XtCallbackProc) jump_brush_cback, (XtPointer) xg);

  RESHAPEBRUSH = (Widget) CreateToggle(xg, "Reshape brush btwn modes",
    True, (Widget) NULL, (Widget) JUMPBRUSH, (Widget) NULL,
    appdata.reshapeBrush, ANY_OF_MANY, options_mgr, "ReshapeBrush");
  XtAddCallback(RESHAPEBRUSH, XtNcallback,
    (XtCallbackProc) reshape_brush_cback, (XtPointer) xg);

  CARRYVARS = (Widget) CreateToggle(xg, "Carry vars btwn modes",
    True, (Widget) NULL, (Widget) RESHAPEBRUSH, (Widget) NULL,
    appdata.carryVars, ANY_OF_MANY, options_mgr, "CarryVars");
  XtAddCallback(CARRYVARS, XtNcallback,
    (XtCallbackProc) carry_vars_cback, (XtPointer) xg);

  XtManageChildren(options_var, 9);

  CLOSE_OPTIONS = XtVaCreateManagedWidget("Close",
    commandWidgetClass, oframe,
    XtNshowGrip, (Boolean) False,
    XtNskipAdjust, (Boolean) True,
    XtNlabel, (String) "Click here to dismiss",
    NULL);
  if (mono) set_mono(CLOSE_OPTIONS);
  XtAddCallback(CLOSE_OPTIONS, XtNcallback,
    (XtCallbackProc) PopdownOptionsMenu, (XtPointer) NULL);
}

void
build_io_menu(xg)
  xgobidata *xg;
{
  Boolean actv;

  io_popup = XtVaCreatePopupShell("IO",
    topLevelShellWidgetClass, IO,
    XtNinput, True,
    XtNtitle, "Input/Output",
    NULL);
  if (mono) set_mono(io_popup);

  aframe = XtVaCreateManagedWidget("Form",
    panedWidgetClass, io_popup,
    XtNorientation, (XtOrientation) XtorientVertical,
    NULL);

  io_mgr = XtVaCreateManagedWidget("Panel",
    formWidgetClass, aframe,
    NULL);
  if (mono) set_mono(io_mgr);

  actv = True;
  if (xg->data_mode == Sprocess)
    actv = False;

  NEWDATA = (Widget) CreateCommand(xg, "Read data",
    (Boolean) actv, (Widget) NULL, (Widget) NULL,
    io_mgr, "ReadData");
  XtAddCallback(NEWDATA, XtNcallback,
    (XtCallbackProc) place_ndata_popup, (XtPointer) xg);

  WRITEASCII = (Widget) CreateCommand(xg, "Save transformed data (ASCII)",
    (Boolean) actv, (Widget) NULL, (Widget) NEWDATA,
    io_mgr, "SaveData");
  XtAddCallback(WRITEASCII, XtNcallback,
    (XtCallbackProc) place_savedata_popup, (XtPointer) xg);

  WRITEBINARY = (Widget) CreateCommand(xg, "Save raw data (binary)",
    (Boolean) actv, (Widget) NULL, (Widget) WRITEASCII,
    io_mgr, "SaveData");
  XtAddCallback(WRITEBINARY, XtNcallback,
    (XtCallbackProc) place_savebinary_popup, (XtPointer) xg);

  STORERESOURCES = (Widget) CreateCommand(xg, "Store resources",
    1, (Widget) NULL, (Widget) WRITEBINARY,
    io_mgr, "StoreResources");
  XtVaSetValues(STORERESOURCES,
    XtNvertDistance, (int) 10,
    NULL);
  XtAddCallback(STORERESOURCES, XtNcallback,
    (XtCallbackProc) store_resources_cback, (XtPointer) xg);

  PRINTPLOTWIN = (Widget) CreateCommand(xg, "Make postscript plot",
    1, (Widget) NULL, (Widget) STORERESOURCES,
    io_mgr, "Print");
  XtVaSetValues(PRINTPLOTWIN,
    XtNvertDistance, (int) 10,
    NULL);
  XtAddCallback(PRINTPLOTWIN, XtNcallback,
    (XtCallbackProc) print_panel_cback, (XtPointer) xg);

  XtManageChildren(io_var, 5);

  CLOSE_IO = XtVaCreateManagedWidget("Close",
    commandWidgetClass, aframe,
    XtNshowGrip, (Boolean) False,
    XtNskipAdjust, (Boolean) True,
    XtNlabel, (String) "Click here to dismiss",
    NULL);
  if (mono) set_mono(CLOSE_IO);
  XtAddCallback(CLOSE_IO, XtNcallback,
    (XtCallbackProc) PopdownIOMenu, (XtPointer) NULL);
}

/* ARGSUSED */
XtCallbackProc
PopupOptionsMenu(w, xg, callback_data)
/*
 * Pop up the options menu.
*/
  Widget w;
  xgobidata *xg;
  caddr_t callback_data;
{
  Dimension width, height;
  Position x, y;
  static int initd = 0;

  if (!initd)
  {
    XtVaGetValues(w,
      XtNwidth, &width,
      XtNheight, &height,
      NULL);
    XtTranslateCoords(w,
      (Position) (width/2), (Position) (height/2), &x, &y);

    XtVaSetValues(options_popup,
      XtNx, (Position) x,
      XtNy, (Position) y,
      NULL);
  }

  XtPopup(options_popup, XtGrabNone);
  XRaiseWindow(display, XtWindow(options_popup));

  if (!initd)
  {
    set_wm_protocols(options_popup);
    initd = 1;
  }
}

void
make_widgets(xg, parent)
  xgobidata *xg;
  Widget parent;
{
  Boolean sens;
  char accelerator[50];
  XtAccelerators parsed_accelerator;

  xg->form0 = XtVaCreateManagedWidget("Form0",
    formWidgetClass, xg->shell,
    NULL);
  if (mono) set_mono(xg->form0);

/*
 * Define main_panel, the main control panel
*/
  main_panel = XtVaCreateManagedWidget("MainPanel",
    formWidgetClass, xg->form0,
    XtNleft, (XtEdgeType) XtChainLeft,
    XtNright, (XtEdgeType) XtChainLeft,
    XtNtop, (XtEdgeType) XtChainTop,
    XtNbottom, (XtEdgeType) XtChainTop,
    NULL);
  if (mono) set_mono(main_panel);

/*
 * Panel to contain mutually exclusive commands.
*/
  sub_panel[0] = XtVaCreateWidget("SubPanel",
    formWidgetClass, main_panel,
    NULL);
  if (mono) set_mono(sub_panel[0]);
/*
 * Panel for remaining states or commands.
*/
  sub_panel[1] = XtVaCreateWidget("SubPanel",
    formWidgetClass, main_panel,
    XtNfromHoriz, (Widget) sub_panel[0],
    NULL);
  if (mono) set_mono( sub_panel[1]);

  XtManageChildren(sub_panel, 2);

/*
 * Main panel commands
*/
  DOTPLOT = (Widget) CreateToggle(xg, "DotPlot",
    1, (Widget) NULL, (Widget) NULL, (Widget) NULL, False, ONE_OF_MANY,
    sub_panel[0], "DotPlot");
  XtAddCallback(DOTPLOT, XtNcallback,
    (XtCallbackProc) dotplot_on_cback, (XtPointer) xg);

  strcpy(accelerator, "<KeyPress>D: set() notify()");
  parsed_accelerator = XtParseAcceleratorTable(accelerator);
  XtVaSetValues(DOTPLOT,
    XtNaccelerators, parsed_accelerator,
    NULL);
  XtInstallAllAccelerators(xg->form0, DOTPLOT);

  XYPLOT = (Widget) CreateToggle(xg, "XYPlot",
    1, (Widget) DOTPLOT, (Widget) NULL, DOTPLOT, True, ONE_OF_MANY,
    sub_panel[0], "XYPlot");
  XtAddCallback(XYPLOT, XtNcallback,
    (XtCallbackProc) xyplot_on_cback, (XtPointer) xg);

  strcpy(accelerator, "<KeyPress>X: set() notify()");
  parsed_accelerator = XtParseAcceleratorTable(accelerator);
  XtVaSetValues(XYPLOT,
    XtNaccelerators, parsed_accelerator,
    NULL);
  XtInstallAllAccelerators(xg->form0, XYPLOT);

/*
 * Rotation and grand tour do not work when there are fewer
 * than three columns.
*/
  sens = True;
  if (xg->ncols_used < 3)
    sens = False;

  ROTATE = (Widget) CreateToggle(xg, "Rotate",
    sens, (Widget) XYPLOT, (Widget) NULL, DOTPLOT, False, ONE_OF_MANY,
    sub_panel[0], "Rotate");
  XtAddCallback(ROTATE, XtNcallback,
    (XtCallbackProc) spin_on_cback, (XtPointer) xg);

  strcpy(accelerator, "<KeyPress>R: set() notify()");
  parsed_accelerator = XtParseAcceleratorTable(accelerator);
  XtVaSetValues(ROTATE,
    XtNaccelerators, parsed_accelerator,
    NULL);
  XtInstallAllAccelerators(xg->form0, ROTATE);

  TOUR = (Widget) CreateToggle(xg, "Tour",
    sens, (Widget) ROTATE, (Widget) NULL, DOTPLOT, False, ONE_OF_MANY,
    sub_panel[0], "Tour");
  XtAddCallback(TOUR, XtNcallback,
    (XtCallbackProc) grand_tour_on_cback, (XtPointer) xg);

  strcpy(accelerator, "<KeyPress>T: set() notify()");
  parsed_accelerator = XtParseAcceleratorTable(accelerator);
  XtVaSetValues(TOUR,
    XtNaccelerators, parsed_accelerator,
    NULL);
  XtInstallAllAccelerators(xg->form0, TOUR);

  CORR = (Widget) CreateToggle(xg, "Corr",
    sens, (Widget) TOUR, (Widget) NULL, DOTPLOT, False, ONE_OF_MANY,
    sub_panel[0], "Corr");
  XtAddCallback(CORR, XtNcallback,
    (XtCallbackProc) corr_tour_on_cback, (XtPointer) xg);

  strcpy(accelerator, "<KeyPress>C: set() notify()");
  parsed_accelerator = XtParseAcceleratorTable(accelerator);
  XtVaSetValues(CORR,
    XtNaccelerators, parsed_accelerator,
    NULL);
  XtInstallAllAccelerators(xg->form0, CORR);

  SCALE = XtVaCreateWidget("Toggle",
    toggleWidgetClass, sub_panel[0],
    XtNlabel, (String) "Scale",
    XtNradioGroup, (Widget) DOTPLOT,
    XtNfromHoriz, (Widget) CORR,
    XtNhorizDistance, (int) 6,
    NULL);
  if (mono) set_mono(SCALE);
  add_pb_help(&xg->nhelpids.pb, SCALE, "Scale" );
  XtAddCallback(SCALE, XtNcallback,
    (XtCallbackProc) scaling_cback, (XtPointer) xg);

  strcpy(accelerator, "<KeyPress>S: set() notify()");
  parsed_accelerator = XtParseAcceleratorTable(accelerator);
  XtVaSetValues(SCALE,
    XtNaccelerators, parsed_accelerator,
    NULL);
  XtInstallAllAccelerators(xg->form0, SCALE);

  BRUSH = (Widget) CreateToggle(xg, "Brush",
    1, (Widget) SCALE, (Widget) NULL, DOTPLOT, False, ONE_OF_MANY,
    sub_panel[0], "Brush");
  XtAddCallback(BRUSH, XtNcallback,
    (XtCallbackProc) brush_cback, (XtPointer) xg);

  strcpy(accelerator, "<KeyPress>B: set() notify()");
  parsed_accelerator = XtParseAcceleratorTable(accelerator);
  XtVaSetValues(BRUSH,
    XtNaccelerators, parsed_accelerator,
    NULL);
  XtInstallAllAccelerators(xg->form0, BRUSH);

  IDENTIFY = (Widget) CreateToggle(xg, "Identify",
    1, (Widget) BRUSH, (Widget) NULL, DOTPLOT, False, ONE_OF_MANY,
    sub_panel[0], "Identify");
  XtAddCallback(IDENTIFY, XtNcallback,
    (XtCallbackProc) identify_cback, (XtPointer) xg);

  strcpy(accelerator, "<KeyPress>I: set() notify()");
  parsed_accelerator = XtParseAcceleratorTable(accelerator);
  XtVaSetValues(IDENTIFY,
    XtNaccelerators, parsed_accelerator,
    NULL);
  XtInstallAllAccelerators(xg->form0, IDENTIFY);

/*
  MAGNIFY = (Widget) CreateToggle(xg, "Magnify",
    1, (Widget) IDENTIFY, (Widget) NULL, DOTPLOT, False, ONE_OF_MANY,
    sub_panel[0], "Magnify");
  XtAddCallback(MAGNIFY, XtNcallback,
    (XtCallbackProc) magnify_cback, (XtPointer) xg);
*/

  EDIT_LINES = (Widget) CreateToggle(xg, "LineEdit",
    1, (Widget) IDENTIFY, (Widget) NULL, DOTPLOT, False, ONE_OF_MANY,
    sub_panel[0], "LineEdit");
  XtAddCallback(EDIT_LINES, XtNcallback,
    (XtCallbackProc) line_editor_cback, (XtPointer) xg);

  strcpy(accelerator, "<KeyPress>L: set() notify()");
  parsed_accelerator = XtParseAcceleratorTable(accelerator);
  XtVaSetValues(EDIT_LINES,
    XtNaccelerators, parsed_accelerator,
    NULL);
  XtInstallAllAccelerators(xg->form0, EDIT_LINES);

  XtManageChildren(main_cmda, 9);

  OPTIONS = (Widget) CreateCommand(xg, "Options",
    1, (Widget) NULL, (Widget) NULL,
    sub_panel[1], "Options");
  XtAddCallback(OPTIONS, XtNcallback,
    (XtCallbackProc) PopupOptionsMenu, (XtPointer) xg);

  strcpy(accelerator, "<KeyPress>O: set() notify()");
  parsed_accelerator = XtParseAcceleratorTable(accelerator);
  XtVaSetValues(OPTIONS,
    XtNaccelerators, parsed_accelerator,
    NULL);
  XtInstallAllAccelerators(xg->form0, OPTIONS);

  IO = (Widget) CreateCommand(xg, "I/O",
    1, (Widget) OPTIONS, (Widget) NULL,
    sub_panel[1], "IO");
  XtAddCallback(IO, XtNcallback,
    (XtCallbackProc) PopupIOMenu, (XtPointer) xg);

  strcpy(accelerator, "<KeyPress>/: set() notify()");
  parsed_accelerator = XtParseAcceleratorTable(accelerator);
  XtVaSetValues(IO,
    XtNaccelerators, parsed_accelerator,
    NULL);
  XtInstallAllAccelerators(xg->form0, IO);

  HELP = (Widget) CreateCommand(xg, "Help",
    1, (Widget) IO, (Widget) NULL,
    sub_panel[1], "Help");
  XtAddCallback(HELP, XtNcallback,
    (XtCallbackProc) help_cback, (XtPointer) NULL);

  EXIT = (Widget) CreateCommand(xg, "Exit",
    1, (Widget) HELP, (Widget) NULL,
    sub_panel[1], "Exit");
  if (parent)
    XtAddCallback(EXIT, XtNcallback,
      (XtCallbackProc) exit_panel_cback, (XtPointer) xg);
  else
    XtAddCallback(EXIT, XtNcallback,
      (XtCallbackProc) exit_solo_cback, (XtPointer) xg);

  XtManageChildren(main_cmdb, 4);

/*
 * Paned widget to help users adjust the plot window and the
 * variable selection panel.
*/
  form1 = XtVaCreateManagedWidget("Form1",
    panedWidgetClass, xg->form0,
    XtNfromVert, (Widget) main_panel,
    XtNleft, (XtEdgeType) XtChainLeft,
    XtNright, (XtEdgeType) XtChainRight,
    XtNtop, (XtEdgeType) XtChainTop,
    XtNbottom, (XtEdgeType) XtRubber,
    XtNorientation, (XtOrientation) XtorientHorizontal,
    NULL);
  if (mono) set_mono(form1);

  xg->box0 = XtVaCreateManagedWidget("Box0",
    formWidgetClass, form1,
    XtNmin, 5,
    NULL);
  if (mono) set_mono(xg->box0);

  xg->box1 = XtVaCreateManagedWidget("Box1",
    formWidgetClass, form1,
    XtNmin, 5,
    NULL);
  if (mono) set_mono(xg->box1);

  xg->box2 = XtVaCreateManagedWidget("Box2",
    formWidgetClass, form1,
    XtNmin, 5,
    NULL);
  if (mono) set_mono(xg->box2);
}

#undef DOTPLOT
#undef XYPLOT
#undef ROTATE
#undef TOUR
#undef SCALE
#undef BRUSH
#undef IDENTIFY
#undef MAGNIFY
#undef EDIT_LINES
#undef OPTIONS
#undef IO
#undef HELP
#undef EXIT
#undef ADDAXES
#undef SHOWPOINTS
#undef SHOWLINES
#undef UNLINK
#undef JUMPBRUSH
#undef RESHAPEBRUSH
#undef CARRYVARS
#undef NEWDATA
#undef PRINTPLOTWIN
#undef SUBSETDATA
#undef STORERESOURCES
#undef CLOSE_OPTIONS
#undef CLOSE_IO

#undef VTOGGLE
#undef VSELECT
#undef VDESELECT
#undef CTOGGLE
#undef CSELECT
#undef CDESELECT
#undef VSELECTALL
#undef VDESELECTALL
#undef CSELECTALL
#undef CDESELECTALL
